@@ -1,35 +1,41 @@
 import config.base
 import os
 import re
+import nargs
 
 class CompilerOptions(config.base.Configure):
   def getCFlags(self, compiler, bopt):
     import config.setCompilers
 
-    if compiler.find('mpicc') >=0:
+    if compiler.endswith('mpicc') or compiler.endswith('mpiicc') :
       try:
         output   = self.executeShellCommand(compiler + ' -show', log = self.log)[0]
-        compiler = output.split(' ')[0]
         self.framework.addMakeMacro('MPICC_SHOW',output.strip().replace('\n','\\\\n'))
       except:
-        pass
+        self.framework.addMakeMacro('MPICC_SHOW',"Unavailable")
+    else:
+      self.framework.addMakeMacro('MPICC_SHOW',"Unavailable")
 
     flags = []
     # GNU gcc
     if config.setCompilers.Configure.isGNU(compiler, self.log) or config.setCompilers.Configure.isClang(compiler, self.log):
       if bopt == '':
         flags.extend(['-Wall', '-Wwrite-strings', '-Wno-strict-aliasing','-Wno-unknown-pragmas'])
+        # skip -fstack-protector for brew gcc - as this gives SEGV
+        if not (config.setCompilers.Configure.isDarwin(self.log) and config.setCompilers.Configure.isGNU(compiler, self.log)):
+          flags.extend(['-fstack-protector'])
+        flags.extend(['-mfp16-format=ieee']) #  arm for utilizing 16 bit storage of floating point
         if config.setCompilers.Configure.isClang(compiler, self.log):
           flags.extend(['-Qunused-arguments'])
         if self.argDB['with-visibility']:
           flags.extend(['-fvisibility=hidden'])
+        arg = nargs.Arg.findArgument('with-errorchecking', self.clArgs)
+        if not nargs.ArgBool('with-errorchecking', arg if arg is not None else '1', isTemporary=True).getValue():
+          flags.extend(['-Wno-unused-but-set-variable'])
       elif bopt == 'g':
         if self.argDB['with-gcov']:
           flags.extend(['-fprofile-arcs', '-ftest-coverage'])
-        if 'with-cuda' in self.argDB and self.argDB['with-cuda']:
-          flags.append('-g') #cuda 4.1 with sm_20 is buggy with -g3
-        else:
-          flags.append('-g3')
+        flags.append('-g3')
       elif bopt == 'O':
         flags.append('-g')
         if config.setCompilers.Configure.isClang(compiler, self.log):
@@ -51,7 +57,10 @@ class CompilerOptions(config.base.Configure):
       # Windows Intel
       elif compiler.find('win32fe icl') >= 0:
         if bopt == '':
-          flags.extend(['-MT'])
+          if self.argDB['with-shared-libraries']:
+            flags.extend(['-MD'])
+          else:
+            flags.extend(['-MT'])
         elif bopt == 'g':
           flags.extend(['-Z7'])
         elif bopt == 'O':
@@ -59,7 +68,11 @@ class CompilerOptions(config.base.Configure):
       # Windows Microsoft
       elif compiler.find('win32fe cl') >= 0:
         if bopt == '':
-          flags.extend(['-MT','-wd4996'])
+          dir(self)
+          if self.argDB['with-shared-libraries']:
+            flags.extend(['-MD','-wd4996'])
+          else:
+            flags.extend(['-MT','-wd4996'])
         elif bopt == 'g':
           flags.extend(['-Z7'])
         elif bopt == 'O':
@@ -81,23 +94,30 @@ class CompilerOptions(config.base.Configure):
   def getCxxFlags(self, compiler, bopt):
     import config.setCompilers
 
-    if compiler.find('mpiCC') >=0  or compiler.find('mpicxx') >=0 :
+    if compiler.endswith('mpiCC') or compiler.endswith('mpicxx') or compiler.endswith('mpiicxx'):
       try:
         output   = self.executeShellCommand(compiler+' -show', log = self.log)[0]
-        compiler = output.split(' ')[0]
         self.framework.addMakeMacro('MPICXX_SHOW',output.strip().replace('\n','\\\\n'))
       except:
-        pass
+        self.framework.addMakeMacro('MPICXX_SHOW',"Unavailable")
+    else:
+      self.framework.addMakeMacro('MPICXX_SHOW',"Unavailable")
 
     flags = []
     # GNU g++
     if config.setCompilers.Configure.isGNU(compiler, self.log) or config.setCompilers.Configure.isClang(compiler, self.log):
       if bopt == '':
         flags.extend(['-Wall', '-Wwrite-strings', '-Wno-strict-aliasing','-Wno-unknown-pragmas'])
+        # skip -fstack-protector for brew gcc - as this gives SEGV
+        if not (config.setCompilers.Configure.isDarwin(self.log) and config.setCompilers.Configure.isGNU(compiler, self.log)):
+          flags.extend(['-fstack-protector'])
         # The option below would prevent warnings about compiling C as C++ being deprecated, but it causes Clang to SEGV, http://llvm.org/bugs/show_bug.cgi?id=12924
         # flags.extend([('-x','c++')])
         if self.argDB['with-visibility']:
           flags.extend(['-fvisibility=hidden'])
+        arg = nargs.Arg.findArgument('with-errorchecking', self.clArgs)
+        if not nargs.ArgBool('with-errorchecking', arg if arg is not None else '1', isTemporary=True).getValue():
+          flags.extend(['-Wno-unused-but-set-variable'])
       elif bopt in ['g']:
         if self.argDB['with-gcov']:
           flags.extend(['-fprofile-arcs', '-ftest-coverage'])
@@ -105,7 +125,7 @@ class CompilerOptions(config.base.Configure):
         flags.append('-g')
       elif bopt in ['O']:
         flags.append('-g')
-        if os.environ.has_key('USER'):
+        if 'USER' in os.environ:
           if config.setCompilers.Configure.isClang(compiler, self.log):
             flags.append('-O3')
           else:
@@ -131,7 +151,10 @@ class CompilerOptions(config.base.Configure):
       # Windows Intel
       elif compiler.find('win32fe icl') >= 0:
         if bopt == '':
-          flags.extend(['-MT','-GX','-GR'])
+          if self.argDB['with-shared-libraries']:
+            flags.extend(['-MD','-GR','-EHsc'])
+          else:
+            flags.extend(['-MT','-GR','-EHsc']) # removing GX in favor of EHsc
         elif bopt in ['g']:
           flags.extend(['-Z7'])
         elif bopt in ['O']:
@@ -139,7 +162,10 @@ class CompilerOptions(config.base.Configure):
       # Windows Microsoft
       elif compiler.find('win32fe cl') >= 0:
         if bopt == '':
-          flags.extend(['-MT','-GR','-EHsc']) # removing GX in favor of EHsc
+          if self.argDB['with-shared-libraries']:
+            flags.extend(['-MD','-GR','-EHsc'])
+          else:
+            flags.extend(['-MT','-GR','-EHsc']) # removing GX in favor of EHsc
         elif bopt == 'g':
           flags.extend(['-Z7','-Zm200'])
         elif bopt == 'O':
@@ -160,13 +186,14 @@ class CompilerOptions(config.base.Configure):
 
   def getFortranFlags(self, compiler, bopt):
 
-    if compiler.endswith('mpif77') or compiler.endswith('mpif90'):
+    if compiler.endswith('mpif77') or compiler.endswith('mpif90') or compiler.endswith('mpifort'):
       try:
         output   = self.executeShellCommand(compiler+' -show', log = self.log)[0]
-        compiler = output.split(' ')[0]
         self.framework.addMakeMacro('MPIFC_SHOW',output.strip().replace('\n','\\\\n'))
       except:
-        pass
+        self.framework.addMakeMacro('MPIFC_SHOW',"Unavailable")
+    else:
+      self.framework.addMakeMacro('MPIFC_SHOW',"Unavailable")
 
     flags = []
     if config.setCompilers.Configure.isGNU(compiler, self.log):
@@ -188,7 +215,7 @@ class CompilerOptions(config.base.Configure):
         flags.extend(['-O'])
     else:
       # Portland Group Fortran 90
-      if compiler == 'pgf90':
+      if config.setCompilers.Configure.isPGI(compiler, self.log):
         if bopt == '':
           flags.append('-Mfree')
         elif bopt == 'O':
@@ -203,7 +230,10 @@ class CompilerOptions(config.base.Configure):
       # Windows Intel
       elif compiler.find('win32fe ifl') >= 0 or compiler.find('win32fe ifort') >= 0:
         if bopt == '':
-          flags.extend(['-MT'])
+          if self.argDB['with-shared-libraries']:
+            flags.extend(['-MD'])
+          else:
+            flags.extend(['-MT'])
         elif bopt == 'g':
           flags.extend(['-Z7'])
         elif bopt == 'O':
@@ -267,7 +297,7 @@ class CompilerOptions(config.base.Configure):
           else:
             version = output.split('\n')[0]
 
-    except RuntimeError, e:
+    except RuntimeError as e:
       self.logWrite('Could not determine compiler version: '+str(e))
     self.logWrite('getCompilerVersion: '+str(compiler)+' '+str(version)+'\n')
     return version
