@@ -39,6 +39,7 @@ class Configure(config.base.Configure):
     help.addArgument('Compilers', '-with-cxxlib-autodetect=<bool>',         nargs.ArgBool(None, 1, 'Autodetect C++ compiler libraries'))
     help.addArgument('Compilers', '-with-dependencies=<bool>',              nargs.ArgBool(None, 1, 'Compile with -MMD or equivalent flag if possible'))
     help.addArgument('Compilers', '-with-cxx-dialect=<dialect>',            nargs.Arg(None, '', 'Dialect under which to compile C++ sources (e.g., C++11)'))
+    help.addArgument('Compilers', '-with-fortran-type-initialize=<bool>',   nargs.ArgBool(None, 1, 'Initialize PETSc objects in Fortran'))
 
     return
 
@@ -102,51 +103,55 @@ class Configure(config.base.Configure):
     return
 
   # THIS SHOULD BE REWRITTEN AS checkDeclModifier()
-  # checkCStaticInline & checkCxxStaticInline are pretty much the same code right now.
+  # checkCInline & checkCxxInline are pretty much the same code right now.
   # but they could be different (later) - and they also check/set different flags - hence
   # code duplication.
-  def checkCStaticInline(self):
-    '''Check for C keyword: static inline'''
-    self.cStaticInlineKeyword = 'static'
+  def checkCInline(self):
+    '''Check for C inline keyword'''
+    self.cInlineKeyword = ' '
     self.pushLanguage('C')
-    for kw in ['static inline', 'static __inline']:
-      if self.checkCompile(kw+' int foo(int a) {return a;}','foo(1);'):
-        self.cStaticInlineKeyword = kw
-        self.logPrint('Set C StaticInline keyword to '+self.cStaticInlineKeyword , 4, 'compilers')
+    for kw in ['inline', '__inline', '__inline__']:
+      if self.checkCompile('static %s int foo(int a) {return a;}' % kw, 'foo(1);'):
+        self.cInlineKeyword = kw
+        self.logPrint('Set C Inline keyword to '+self.cInlineKeyword , 4, 'compilers')
         break
     self.popLanguage()
-    if self.cStaticInlineKeyword == 'static':
-      self.logPrint('No C StaticInline keyword. using static function', 4, 'compilers')
-    self.addDefine('C_STATIC_INLINE', self.cStaticInlineKeyword)
+    if self.cInlineKeyword == ' ':
+      self.logPrint('No C Inline keyword. Using static function', 4, 'compilers')
+    self.addDefine('C_INLINE', self.cInlineKeyword)
     return
-  def checkCxxStaticInline(self):
-    '''Check for C++ keyword: static inline'''
-    self.cxxStaticInlineKeyword = 'static'
+
+  def checkCxxInline(self):
+    '''Check for C++ inline keyword'''
+    self.cxxInlineKeyword = ' '
     self.pushLanguage('C++')
-    for kw in ['static inline', 'static __inline']:
-      if self.checkCompile(kw+' int foo(int a) {return a;}','foo(1);'):
-        self.cxxStaticInlineKeyword = kw
-        self.logPrint('Set Cxx StaticInline keyword to '+self.cxxStaticInlineKeyword , 4, 'compilers')
+    for kw in ['inline', '__inline', '__inline__']:
+      if self.checkCompile('static %s int foo(int a) {return a;}' % kw, 'foo(1);'):
+        self.cxxInlineKeyword = kw
+        self.logPrint('Set Cxx Inline keyword to '+self.cxxInlineKeyword , 4, 'compilers')
         break
     self.popLanguage()
-    if self.cxxStaticInlineKeyword == 'static':
-      self.logPrint('No Cxx StaticInline keyword. using static function', 4, 'compilers')
-    self.addDefine('CXX_STATIC_INLINE', self.cxxStaticInlineKeyword)
+    if self.cxxInlineKeyword == ' ':
+      self.logPrint('No Cxx Inline keyword. Using static function', 4, 'compilers')
+    self.addDefine('CXX_INLINE', self.cxxInlineKeyword)
     return
 
   def checkRestrict(self,language):
     '''Check for the C/CXX restrict keyword'''
-    # Try the official restrict keyword, then gcc's __restrict__, then
-    # SGI's __restrict.  __restrict has slightly different semantics than
-    # restrict (it's a bit stronger, in that __restrict pointers can't
-    # overlap even with non __restrict pointers), but I think it should be
-    # okay under the circumstances where restrict is normally used.
+    # Try keywords equivalent to C99 restrict.  Note that many C
+    # compilers require special cflags, such as -std=c99 or -restrict to
+    # recognize the "restrict" keyword and it is not always practical to
+    # expect that every user provides matching options.  Meanwhile,
+    # compilers like Intel and MSVC (reportedly) support __restrict
+    # without special options.  Glibc uses __restrict, presumably for
+    # this reason.  Note that __restrict is not standardized while
+    # "restrict" is, but implementation realities favor __restrict.
     if config.setCompilers.Configure.isPGI(self.setCompilers.CC, self.log):
       self.addDefine(language.upper()+'_RESTRICT', ' ')
       self.logPrint('PGI restrict word is broken cannot handle [restrict] '+str(language)+' restrict keyword', 4, 'compilers')
       return
     self.pushLanguage(language)
-    for kw in ['restrict', ' __restrict__', '__restrict']:
+    for kw in ['__restrict', ' __restrict__', 'restrict']:
       if self.checkCompile('', 'float * '+kw+' x;'):
         if language.lower() == 'c':
           self.cRestrict = kw
@@ -165,8 +170,71 @@ class Configure(config.base.Configure):
     self.popLanguage()
     return
 
+  def checkCrossLink(self, func1, func2, language1 = 'C', language2='FC',extraObjs = []):
+    '''Compiles C/C++ and Fortran code and tries to link them together; the C and Fortran code are independent so no name mangling is needed
+       language1 is used to compile the first code, language2 compiles the second and links them togetether'''
+    obj1 = os.path.join(self.tmpDir, 'confc.o')
+    found = 0
+    # Compile the C test object
+    self.pushLanguage(language1)
+    if not self.checkCompile(func1, None, cleanup = 0):
+      self.logPrint('Cannot compile C function: '+func1, 3, 'compilers')
+      self.popLanguage()
+      return found
+    if not os.path.isfile(self.compilerObj):
+      self.logPrint('Cannot locate object file: '+os.path.abspath(self.compilerObj), 3, 'compilers')
+      self.popLanguage()
+      return found
+    os.rename(self.compilerObj, obj1)
+    self.popLanguage()
+    # Link the test object against a Fortran driver
+    self.pushLanguage(language2)
+    oldLIBS = self.setCompilers.LIBS
+    self.setCompilers.LIBS = obj1+' '+self.setCompilers.LIBS
+    if extraObjs:
+      self.setCompilers.LIBS = ' '.join(extraObjs)+' '+' '.join([self.libraries.getLibArgument(lib) for lib in self.clibs])+' '+self.setCompilers.LIBS
+    found = self.checkLink("", func2,codeBegin = " ", codeEnd = " ")
+    self.setCompilers.LIBS = oldLIBS
+    self.popLanguage()
+    if os.path.isfile(obj1):
+      os.remove(obj1)
+    return found
+
   def checkCLibraries(self):
-    '''Determines the libraries needed to link with C'''
+    '''Determines the libraries needed to link with C compiled code'''
+    skipclibraries = 1
+    if hasattr(self.setCompilers, 'FC'):
+      self.setCompilers.saveLog()
+      try:
+        if self.checkCrossLink('#include <stdio.h>\nvoid asub(void)\n{char s[16];printf("testing %s",s);}\n',"     program main\n      print*,'testing'\n      stop\n      end\n",language1='C',language2='FC'):
+          self.logWrite(self.setCompilers.restoreLog())
+          self.logPrint('C libraries are not needed when using Fortran linker')
+        else:
+          self.logWrite(self.setCompilers.restoreLog())
+          self.logPrint('C code cannot directly be linked with Fortran linker, therefor will determine needed C libraries')
+          skipclibraries = 0
+      except RuntimeError as e:
+        self.logWrite(self.setCompilers.restoreLog())
+        self.logPrint('Error message from compiling {'+str(e)+'}', 4, 'compilers')
+        self.logPrint('C code cannot directly be linked with Fortran linker, therefor will determine needed C libraries')
+        skipclibraries = 0
+    if hasattr(self.setCompilers, 'CXX'):
+      self.setCompilers.saveLog()
+      try:
+        if self.checkCrossLink('#include <stdio.h>\nvoid asub(void)\n{char s[16];printf("testing %s",s);}\n',"int main(int argc,char **args)\n{return 0;}\n",language1='C',language2='C++'):
+          self.logWrite(self.setCompilers.restoreLog())
+          self.logPrint('C libraries are not needed when using C++ linker')
+        else:
+          self.logWrite(self.setCompilers.restoreLog())
+          self.logPrint('C code cannot directly be linked with C++ linker, therefor will determine needed C libraries')
+          skipclibraries = 0
+      except RuntimeError as e:
+        self.logWrite(self.setCompilers.restoreLog())
+        self.logPrint('Error message from compiling {'+str(e)+'}', 4, 'compilers')
+        self.logPrint('C code cannot directly be linked with C++ linker, therefor will determine needed C libraries')
+        skipclibraries = 0
+    if skipclibraries == 1: return
+
     oldFlags = self.setCompilers.LDFLAGS
     self.setCompilers.LDFLAGS += ' -v'
     self.pushLanguage('C')
@@ -196,7 +264,7 @@ class Configure(config.base.Configure):
     rpathflags = []
     try:
       while 1:
-        arg = argIter.next()
+        arg = next(argIter)
         self.logPrint( 'Checking arg '+arg, 4, 'compilers')
 
         # Intel compiler sometimes puts " " around an option like "-lsomething"
@@ -210,8 +278,12 @@ class Configure(config.base.Configure):
         if arg == '-long_double':
           continue
         # if options of type -L foobar
+        if arg == '-lto_library':
+          lib = next(argIter)
+          self.logPrint('Skipping Apple LLVM linker option -lto_library '+lib)
+          continue
         if arg == '-L':
-          lib = argIter.next()
+          lib = next(argIter)
           self.logPrint('Found -L '+lib, 4, 'compilers')
           clibs.append('-L'+lib)
           continue
@@ -236,7 +308,7 @@ class Configure(config.base.Configure):
             self.logPrint('already in lflags: '+arg, 4, 'compilers')
           continue
         # Check for system libraries
-        m = re.match(r'^-l(ang.*|crt[0-9].o|crtbegin.o|c|gcc|cygwin|crt[0-9].[0-9][0-9].[0-9].o)$', arg)
+        m = re.match(r'^-l(ang.*|crt[0-9].o|crtbegin.o|c|gcc|gcc_ext(.[0-9]+)*|System|cygwin|crt[0-9].[0-9][0-9].[0-9].o)$', arg)
         if m:
           self.logPrint('Skipping system library: '+arg, 4, 'compilers')
           continue
@@ -246,8 +318,6 @@ class Configure(config.base.Configure):
           if not arg in lflags:
             if arg == '-lkernel32':
               continue
-            elif arg == '-lm':
-              continue
             else:
               lflags.append(arg)
             self.logPrint('Found library : '+arg, 4, 'compilers')
@@ -263,7 +333,7 @@ class Configure(config.base.Configure):
           continue
         # Check for '-rpath /sharedlibpath/ or -R /sharedlibpath/'
         if arg == '-rpath' or arg == '-R':
-          lib = argIter.next()
+          lib = next(argIter)
           if lib.startswith('-'): continue # perhaps the path was striped due to quotes?
           if lib.startswith('"') and lib.endswith('"') and lib.find(' ') == -1: lib = lib[1:-1]
           lib = os.path.abspath(lib)
@@ -306,11 +376,15 @@ class Configure(config.base.Configure):
       self.setCompilers.saveLog()
       try:
         self.setCompilers.checkCompiler('FC')
-      except RuntimeError, e:
+      except RuntimeError as e:
         self.setCompilers.LIBS = oldLibs
         self.logWrite(self.setCompilers.restoreLog())
         self.logPrint('Error message from compiling {'+str(e)+'}', 4, 'compilers')
         raise RuntimeError('C libraries cannot directly be used from Fortran')
+      except OSError as e:
+        self.setCompilers.LIBS = oldLibs
+        self.logWrite(self.setCompilers.restoreLog())
+        raise e
       self.logWrite(self.setCompilers.restoreLog())
     return
 
@@ -425,6 +499,52 @@ class Configure(config.base.Configure):
 
   def checkCxxLibraries(self):
     '''Determines the libraries needed to link with C++'''
+    skipcxxlibraries = 1
+    self.setCompilers.saveLog()
+    body   = '''#include <iostream>\n#include <vector>\nvoid asub(void)\n{std::vector<int> v;\ntry  { throw 20;  }  catch (int e)  { std::cout << "An exception occurred";  }}'''
+    try:
+      if self.checkCrossLink(body,"int main(int argc,char **args)\n{return 0;}\n",language1='C++',language2='C'):
+        self.logWrite(self.setCompilers.restoreLog())
+        self.logPrint('C++ libraries are not needed when using C linker')
+      else:
+        self.logWrite(self.setCompilers.restoreLog())
+        oldLibs = self.setCompilers.LIBS
+        self.setCompilers.LIBS = '-lstdc++ '+self.setCompilers.LIBS
+        if self.checkCrossLink(body,"int main(int argc,char **args)\n{return 0;}\n",language1='C++',language2='C'):
+          self.logPrint('C++ requires -lstdc++ to link with C compiler', 3, 'compilers')
+        else:
+          self.setCompilers.LIBS = oldLibs
+          self.logPrint('C++ code cannot directly be linked with C linker, therefor will determine needed C++ libraries')
+          skipcxxlibraries = 0
+    except RuntimeError as e:
+      self.logWrite(self.setCompilers.restoreLog())
+      self.logPrint('Error message from compiling {'+str(e)+'}', 4, 'compilers')
+      self.logPrint('C++ code cannot directly be linked with C linker, therefor will determine needed C++ libraries')
+      skipcxxlibraries = 0
+    if hasattr(self.setCompilers, 'FC'):
+      self.setCompilers.saveLog()
+      try:
+        if self.checkCrossLink(body,"     program main\n      print*,'testing'\n      stop\n      end\n",language1='C++',language2='FC'):
+          self.logWrite(self.setCompilers.restoreLog())
+          self.logPrint('C++ libraries are not needed when using FC linker')
+        else:
+          self.logWrite(self.setCompilers.restoreLog())
+          oldLibs = self.setCompilers.LIBS
+          self.setCompilers.LIBS = '-lstdc++ '+self.setCompilers.LIBS
+          if self.checkCrossLink(body,"     program main\n      print*,'testing'\n      stop\n      end\n",language1='C++',language2='FC'):
+            self.logPrint('C++ requires -lstdc++ to link with FC compiler', 3, 'compilers')
+          else:
+            self.setCompilers.LIBS = oldLibs
+            self.logPrint('C++ code cannot directly be linked with FC linker, therefor will determine needed C++ libraries')
+            skipcxxlibraries = 0
+      except RuntimeError as e:
+        self.logWrite(self.setCompilers.restoreLog())
+        self.logPrint('Error message from compiling {'+str(e)+'}', 4, 'compilers')
+        self.logPrint('C++ code cannot directly be linked with FC linker, therefor will determine needed C++ libraries')
+        skipcxxlibraries = 0
+
+    if skipcxxlibraries: return
+
     oldFlags = self.setCompilers.LDFLAGS
     self.setCompilers.LDFLAGS += ' -v'
     self.pushLanguage('Cxx')
@@ -453,7 +573,7 @@ class Configure(config.base.Configure):
     rpathflags = []
     try:
       while 1:
-        arg = argIter.next()
+        arg = next(argIter)
         self.logPrint( 'Checking arg '+arg, 4, 'compilers')
 
         # Intel compiler sometimes puts " " around an option like "-lsomething"
@@ -469,10 +589,14 @@ class Configure(config.base.Configure):
 
         # if options of type -L foobar
         if arg == '-L':
-          lib = argIter.next()
+          lib = next(argIter)
           self.logPrint('Found -L '+lib, 4, 'compilers')
           cxxlibs.append('-L'+lib)
           continue
+        if arg == '-lto_library':
+          lib = next(argIter)
+          self.logPrint('Skipping Apple LLVM linker option -lto_library '+lib)
+          continue
         # Check for full library name
         m = re.match(r'^/.*\.a$', arg)
         if m:
@@ -486,7 +610,7 @@ class Configure(config.base.Configure):
         # Check for full dylib library name
         m = re.match(r'^/.*\.dylib$', arg)
         if m:
-          if not arg in lflags:
+          if not arg in lflags and not arg.endswith('LTO.dylib'):
             lflags.append(arg)
             self.logPrint('Found full library spec: '+arg, 4, 'compilers')
             cxxlibs.append(arg)
@@ -494,7 +618,7 @@ class Configure(config.base.Configure):
             self.logPrint('already in lflags: '+arg, 4, 'compilers')
           continue
         # Check for system libraries
-        m = re.match(r'^-l(ang.*|crt[0-9].o|crtbegin.o|c|gcc|cygwin|crt[0-9].[0-9][0-9].[0-9].o)$', arg)
+        m = re.match(r'^-l(ang.*|crt[0-9].o|crtbegin.o|c|gcc|gcc_ext(.[0-9]+)*|System|cygwin|crt[0-9].[0-9][0-9].[0-9].o)$', arg)
         if m:
           self.logPrint('Skipping system library: '+arg, 4, 'compilers')
           continue
@@ -504,12 +628,12 @@ class Configure(config.base.Configure):
           if not arg in lflags:
             if arg == '-lkernel32':
               continue
-            elif arg == '-lm':
-              continue
+            elif arg == '-lLTO' and self.setCompilers.isDarwin(self.log):
+              self.logPrint('Skipping -lTO')
             else:
               lflags.append(arg)
             self.logPrint('Found library: '+arg, 4, 'compilers')
-            if arg in self.clibs:
+            if (arg == '-lLTO' and self.setCompilers.isDarwin(self.log)) or arg in self.clibs:
               self.logPrint('Library already in C list so skipping in C++')
             else:
               cxxlibs.append(arg)
@@ -525,7 +649,7 @@ class Configure(config.base.Configure):
           continue
         # Check for '-rpath /sharedlibpath/ or -R /sharedlibpath/'
         if arg == '-rpath' or arg == '-R':
-          lib = argIter.next()
+          lib = next(argIter)
           if lib.startswith('-'): continue # perhaps the path was striped due to quotes?
           if lib.startswith('"') and lib.endswith('"') and lib.find(' ') == -1: lib = lib[1:-1]
           lib = os.path.abspath(lib)
@@ -566,7 +690,7 @@ class Configure(config.base.Configure):
     self.setCompilers.saveLog()
     try:
       self.setCompilers.checkCompiler('C')
-    except RuntimeError, e:
+    except RuntimeError as e:
       self.logPrint('Cxx libraries cannot directly be used from C', 4, 'compilers')
       self.logPrint('Error message from compiling {'+str(e)+'}', 4, 'compilers')
     self.setCompilers.LIBS = oldLibs
@@ -579,7 +703,7 @@ class Configure(config.base.Configure):
       self.setCompilers.saveLog()
       try:
         self.setCompilers.checkCompiler('FC')
-      except RuntimeError, e:
+      except RuntimeError as e:
         self.logPrint('Cxx libraries cannot directly be used from Fortran', 4, 'compilers')
         self.logPrint('Error message from compiling {'+str(e)+'}', 4, 'compilers')
       self.setCompilers.LIBS = oldLibs
@@ -602,6 +726,7 @@ class Configure(config.base.Configure):
     self.popLanguage()
     return
 
+
   def mangleFortranFunction(self, name):
     if self.fortranMangling == 'underscore':
       if self.fortranManglingDoubleUnderscore and name.find('_') >= 0:
@@ -680,6 +805,10 @@ class Configure(config.base.Configure):
       self.addDefine('STDCALL', '__stdcall')
       self.addDefine('HAVE_FORTRAN_CAPS', 1)
       self.addDefine('HAVE_FORTRAN_MIXED_STR_ARG', 1)
+    if config.setCompilers.Configure.isGfortran8plus(self.getCompiler('FC'), self.log):
+      self.addDefine('FORTRAN_CHARLEN_T', 'size_t')
+    else:
+      self.addDefine('FORTRAN_CHARLEN_T', 'int')
     return
 
   def checkFortranNameManglingDouble(self):
@@ -760,6 +889,46 @@ class Configure(config.base.Configure):
     for writing this extremely useful macro.'''
     if not hasattr(self.setCompilers, 'CC') or not hasattr(self.setCompilers, 'FC'):
       return
+    skipfortranlibraries = 1
+    self.setCompilers.saveLog()
+    cbody = "int main(int argc,char **args)\n{return 0;}\n";
+    self.pushLanguage('FC')
+    if self.checkLink(includes='#include <mpif.h>',body='      call MPI_Allreduce()\n'):
+      fbody = "subroutine asub()\n      print*,'testing'\n      call MPI_Allreduce()\n      return\n      end\n"
+    else:
+      fbody = "subroutine asub()\n      print*,'testing'\n      return\n      end\n"
+    self.popLanguage()
+    try:
+      if self.checkCrossLink(fbody,cbody,language1='FC',language2='C'):
+        self.logWrite(self.setCompilers.restoreLog())
+        self.logPrint('Fortran libraries are not needed when using C linker')
+      else:
+        self.logWrite(self.setCompilers.restoreLog())
+        self.logPrint('Fortran code cannot directly be linked with C linker, therefor will determine needed Fortran libraries')
+        skipfortranlibraries = 0
+    except RuntimeError as e:
+      self.logWrite(self.setCompilers.restoreLog())
+      self.logPrint('Error message from compiling {'+str(e)+'}', 4, 'compilers')
+      self.logPrint('Fortran code cannot directly be linked with C linker, therefor will determine needed Fortran libraries')
+      skipfortranlibraries = 0
+    if hasattr(self.setCompilers, 'CXX'):
+      self.setCompilers.saveLog()
+      try:
+        if self.checkCrossLink(fbody,cbody,language1='FC',language2='C++'):
+          self.logWrite(self.setCompilers.restoreLog())
+          self.logPrint('Fortran libraries are not needed when using C++ linker')
+        else:
+          self.logWrite(self.setCompilers.restoreLog())
+          self.logPrint('Fortran code cannot directly be linked with C++ linker, therefor will determine needed Fortran libraries')
+          skipfortranlibraries = 0
+      except RuntimeError as e:
+        self.logWrite(self.setCompilers.restoreLog())
+        self.logPrint('Error message from compiling {'+str(e)+'}', 4, 'compilers')
+        self.logPrint('Fortran code cannot directly be linked with CXX linker, therefor will determine needed Fortran libraries')
+        skipfortranlibraries = 0
+
+    if skipfortranlibraries == 1: return
+
     self.pushLanguage('FC')
     oldFlags = self.setCompilers.LDFLAGS
     if config.setCompilers.Configure.isNAG(self.getCompiler(), self.log):
@@ -824,7 +993,7 @@ class Configure(config.base.Configure):
     rpathflags = []
     try:
       while 1:
-        arg = argIter.next()
+        arg = next(argIter)
         self.logPrint( 'Checking arg '+arg, 4, 'compilers')
         # Intel compiler sometimes puts " " around an option like "-lsomething"
         if arg.startswith('"') and arg.endswith('"'):
@@ -834,6 +1003,10 @@ class Configure(config.base.Configure):
         if arg.endswith('"') and arg[:-1].find('"') == -1:
           arg = arg[:-1]
 
+        if arg == '-lto_library':
+          lib = next(argIter)
+          self.logPrint('Skipping Apple LLVM linker option -lto_library '+lib)
+          continue
         # Check for full library name
         m = re.match(r'^/.*\.a$', arg)
         if m:
@@ -847,7 +1020,7 @@ class Configure(config.base.Configure):
         # Check for full dylib library name
         m = re.match(r'^/.*\.dylib$', arg)
         if m:
-          if not arg in lflags:
+          if not arg.endswith('LTO.dylib') and not arg in lflags:
             lflags.append(arg)
             self.logPrint('Found full library spec: '+arg, 4, 'compilers')
             flibs.append(arg)
@@ -860,7 +1033,7 @@ class Configure(config.base.Configure):
         # AIX does this for MPI and perhaps other things
         m = re.match(r'^-I.*$', arg)
         if m:
-          inc = arg.replace('-I','')
+          inc = arg.replace('-I','',1)
           self.logPrint('Found include directory: '+inc, 4, 'compilers')
           fincs.append(inc)
           continue
@@ -877,16 +1050,17 @@ class Configure(config.base.Configure):
             self.logPrint('Already in lflags so skipping: '+arg, 4, 'compilers')
           continue
         # Check for system libraries
-        m = re.match(r'^-l(ang.*|crt[0-9].o|crtbegin.o|c|gcc|cygwin|crt[0-9].[0-9][0-9].[0-9].o)$', arg)
+        m = re.match(r'^-l(ang.*|crt[0-9].o|crtbegin.o|c|gcc|gcc_ext(.[0-9]+)*|System|cygwin|crt[0-9].[0-9][0-9].[0-9].o)$', arg)
         if m:
           self.logPrint('Found system library therefor skipping: '+arg, 4, 'compilers')
           continue
         # Check for canonical library argument
         m = re.match(r'^-[lL]$', arg)
         if m:
-          lib = arg+argIter.next()
+          lib = arg+next(argIter)
           self.logPrint('Found canonical library: '+lib, 4, 'compilers')
-          flibs.append(lib)
+          if not lib == '-LLTO' or not self.setCompilers.isDarwin(self.log):
+            flibs.append(lib)
           continue
         # intel windows compilers can use -libpath argument
         if arg.find('-libpath:')>=0:
@@ -902,22 +1076,18 @@ class Configure(config.base.Configure):
           if not arg in lflags:
             if arg == '-lkernel32':
               continue
-            elif arg == '-lm':
-              pass
             elif arg == '-lgfortranbegin':
               fmainlibs.append(arg)
               continue
             elif arg == '-lfrtbegin' and not config.setCompilers.Configure.isCygwin(self.log):
               fmainlibs.append(arg)
               continue
-            else:
+            elif not arg == '-lLTO' or not config.setCompilers.Configure.isDarwin(self.log):
               lflags.append(arg)
             self.logPrint('Found library: '+arg, 4, 'compilers')
             if arg in self.clibs:
               self.logPrint('Library already in C list so skipping in Fortran')
-            elif arg in self.cxxlibs:
-              self.logPrint('Library already in Cxx list so skipping in Fortran')
-            else:
+            elif not arg == '-lLTO' or not config.setCompilers.Configure.isDarwin(self.log):
               flibs.append(arg)
           else:
             self.logPrint('Already in lflags: '+arg, 4, 'compilers')
@@ -935,8 +1105,8 @@ class Configure(config.base.Configure):
           continue
         # Check for '-rpath /sharedlibpath/ or -R /sharedlibpath/'
         if arg == '-rpath' or arg == '-R':
-          lib = argIter.next()
-          if lib == '\\': lib = argIter.next()
+          lib = next(argIter)
+          if lib == '\\': lib = next(argIter)
           if lib.startswith('-'): continue # perhaps the path was striped due to quotes?
           if lib.startswith('"') and lib.endswith('"') and lib.find(' ') == -1: lib = lib[1:-1]
           lib = os.path.abspath(lib)
@@ -968,7 +1138,7 @@ class Configure(config.base.Configure):
         # This probably only applies to Solaris systems, and then will only
         # work with gcc...
         if arg == '-Y':
-          libs = argIter.next()
+          libs = next(argIter)
           if libs.startswith('"') and libs.endswith('"'):
             libs = libs[1:-1]
           for lib in libs.split(':'):
@@ -1040,7 +1210,7 @@ class Configure(config.base.Configure):
     self.setCompilers.saveLog()
     try:
       self.setCompilers.checkCompiler('C')
-    except RuntimeError, e:
+    except RuntimeError as e:
       self.logPrint('Fortran libraries cannot directly be used from C, try without -lcrt2.o', 4, 'compilers')
       self.logPrint('Error message from compiling {'+str(e)+'}', 4, 'compilers')
       # try removing this one
@@ -1048,7 +1218,7 @@ class Configure(config.base.Configure):
       self.setCompilers.LIBS = oldLibs+' '+' '.join([self.libraries.getLibArgument(lib) for lib in self.flibs])
       try:
         self.setCompilers.checkCompiler('C')
-      except RuntimeError, e:
+      except RuntimeError as e:
         self.logPrint('Fortran libraries still cannot directly be used from C, try without pgi.ld files', 4, 'compilers')
         self.logPrint('Error message from compiling {'+str(e)+'}', 4, 'compilers')
         tmpflibs = self.flibs
@@ -1072,14 +1242,14 @@ class Configure(config.base.Configure):
       try:
         self.setCompilers.checkCompiler('Cxx')
         self.logPrint('Fortran libraries can be used from C++', 4, 'compilers')
-      except RuntimeError, e:
+      except RuntimeError as e:
         self.logPrint(str(e), 4, 'compilers')
         # try removing this one causes grief with gnu g++ and Intel Fortran
         if '-lintrins' in self.flibs: self.flibs.remove('-lintrins')
         self.setCompilers.LIBS = oldLibs+' '+' '.join([self.libraries.getLibArgument(lib) for lib in self.flibs])
         try:
           self.setCompilers.checkCompiler('Cxx')
-        except RuntimeError, e:
+        except RuntimeError as e:
           self.logPrint(str(e), 4, 'compilers')
           if str(e).find('INTELf90_dclock') >= 0:
             self.logPrint('Intel 7.1 Fortran compiler cannot be used with g++ 3.2!', 2, 'compilers')
@@ -1125,6 +1295,37 @@ class Configure(config.base.Configure):
       raise RuntimeError('Fortran could not successfully link C++ objects')
     return
 
+  def configureFortranFlush(self):
+    self.pushLanguage('FC')
+    for baseName in ['flush','flush_']:
+      if self.checkLink(body='      call '+baseName+'(6)'):
+        self.addDefine('HAVE_FORTRAN_'+baseName.upper(), 1)
+        break
+    self.popLanguage()
+    return
+
+  def checkFortranTypeInitialize(self):
+    '''Determines if PETSc objects in Fortran are initialized by default (doesn't work with common blocks)'''
+    if self.argDB['with-fortran-type-initialize']:
+      self.addDefine('HAVE_FORTRAN_TYPE_INITIALIZE', -2)
+      self.addDefine('FORTRAN_TYPE_INITIALIZE', ' = -2')
+      self.logPrint('Initializing Fortran objects')
+    else:
+      self.addDefine('FORTRAN_TYPE_INITIALIZE', ' ')
+      self.logPrint('Not initializing Fortran objects')
+    return
+
+  def checkFortranTypeStar(self):
+    '''Determine whether the Fortran compiler handles type(*)'''
+    self.pushLanguage('FC')
+    if self.checkCompile(body = '      interface\n      subroutine a(b)\n      type(*) :: b(:)\n      end subroutine\n      end interface\n'):
+      self.addDefine('HAVE_FORTRAN_TYPE_STAR', 1)
+      self.logPrint('Fortran compiler supports type(*)')
+    else:
+      self.logPrint('Fortran compiler does not support type(*)')
+    self.popLanguage()
+    return
+
   def checkFortran90(self):
     '''Determine whether the Fortran compiler handles F90'''
     self.pushLanguage('FC')
@@ -1138,10 +1339,37 @@ class Configure(config.base.Configure):
     self.popLanguage()
     return
 
+  def checkFortran90FreeForm(self):
+    '''Determine whether the Fortran compiler handles F90FreeForm
+       We also require that the compiler handles lines longer than 132 characters'''
+    self.pushLanguage('FC')
+    if self.checkLink(body = '      INTEGER, PARAMETER ::        int = SELECTED_INT_KIND(8);              INTEGER (KIND=int) :: ierr;       ierr                            =                                                                                                               1'):
+      self.addDefine('USING_F90FREEFORM', 1)
+      self.fortranIsF90FreeForm = 1
+      self.logPrint('Fortran compiler supports F90FreeForm')
+    else:
+      self.fortranIsF90FreeForm = 0
+      self.logPrint('Fortran compiler does not support F90FreeForm')
+    self.popLanguage()
+    return
+
   def checkFortran2003(self):
     '''Determine whether the Fortran compiler handles F2003'''
     self.pushLanguage('FC')
-    if self.checkLink(body = '''
+    if self.fortranIsF90 and self.checkLink(codeBegin = '''
+      module Base_module
+        type, public :: base_type
+           integer :: A
+         contains
+           procedure, public :: Print => BasePrint
+        end type base_type
+      contains
+        subroutine BasePrint(this)
+          class(base_type) :: this
+        end subroutine BasePrint
+      end module Base_module
+
+      program main''',body = '''
       use,intrinsic :: iso_c_binding
       Type(C_Ptr),Dimension(:),Pointer :: CArray
       character(kind=c_char),pointer   :: nullc => null()
@@ -1156,6 +1384,8 @@ class Configure(config.base.Configure):
       self.fortranIsF2003 = 0
       self.logPrint('Fortran compiler does not support F2003')
     self.popLanguage()
+    for f in [os.path.abspath('base_module.mod'), os.path.abspath('BASE_MODULE.mod'), os.path.join(os.path.dirname(self.compilerObj),'base_module.mod'), os.path.join(os.path.dirname(self.compilerObj),'BASE_MODULE.mod')]:
+      if os.path.isfile(f): os.remove(f)
     return
 
   def checkFortran90Array(self):
@@ -1416,14 +1646,19 @@ class Configure(config.base.Configure):
     """
     self.setCompilers.saveLog()
     self.setCompilers.pushLanguage('C')
+    restoredlog = 0
     flags_to_try = ['','-std=c99','-std=gnu99','-std=c11''-std=gnu11','-c99']
     for flag in flags_to_try:
       if self.setCompilers.checkCompilerFlag(flag, includes, body):
         self.c99flag = flag
+        self.logWrite(self.setCompilers.restoreLog())
+        restoredlog = 1
         self.framework.logPrint('Accepted C99 compile flag: '+flag)
         break
+    if self.c99flag == '': self.addDefine('HAVE_C99', 1)
     self.setCompilers.popLanguage()
-    self.logWrite(self.setCompilers.restoreLog())
+    if not restoredlog:
+      self.logWrite(self.setCompilers.restoreLog())
     return
 
   def configure(self):
@@ -1432,7 +1667,7 @@ class Configure(config.base.Configure):
       self.isGCC = config.setCompilers.Configure.isGNU(self.setCompilers.CC, self.log)
       self.executeTest(self.checkRestrict,['C'])
       self.executeTest(self.checkCFormatting)
-      self.executeTest(self.checkCStaticInline)
+      self.executeTest(self.checkCInline)
       self.executeTest(self.checkDynamicLoadFlag)
       if self.argDB['with-clib-autodetect']:
         self.executeTest(self.checkCLibraries)
@@ -1445,7 +1680,7 @@ class Configure(config.base.Configure):
       self.executeTest(self.checkRestrict,['Cxx'])
       self.executeTest(self.checkCxxNamespace)
       self.executeTest(self.checkCxxOptionalExtensions)
-      self.executeTest(self.checkCxxStaticInline)
+      self.executeTest(self.checkCxxInline)
       if self.argDB['with-cxxlib-autodetect']:
         self.executeTest(self.checkCxxLibraries)
       self.executeTest(self.checkCxx11)
@@ -1462,10 +1697,14 @@ class Configure(config.base.Configure):
       if hasattr(self.setCompilers, 'CXX'):
         self.executeTest(self.checkFortranLinkingCxx)
       self.executeTest(self.checkFortran90)
+      self.executeTest(self.checkFortran90FreeForm)
       self.executeTest(self.checkFortran2003)
       self.executeTest(self.checkFortran90Array)
       self.executeTest(self.checkFortranModuleInclude)
       self.executeTest(self.checkFortranModuleOutput)
+      self.executeTest(self.checkFortranTypeStar)
+      self.executeTest(self.checkFortranTypeInitialize)
+      self.executeTest(self.configureFortranFlush)
     self.no_configure()
     return
 
