@@ -3,18 +3,23 @@ import config.package
 class Configure(config.package.Package):
   def __init__(self, framework):
     config.package.Package.__init__(self, framework)
-    self.gitcommit = '46cd70474923a5dde71337aa354f7423a8f2ca20'
-    self.giturls   = ['https://bitbucket.org/petsc/pkg-mumps.git']
-    self.download  = ['http://ftp.mcs.anl.gov/pub/petsc/externalpackages/MUMPS_5.0.0-p1.tar.gz']
-    self.liblist   = [['libcmumps.a','libdmumps.a','libsmumps.a','libzmumps.a','libmumps_common.a','libpord.a'],
-                     ['libcmumps.a','libdmumps.a','libsmumps.a','libzmumps.a','libmumps_common.a','libpord.a','libpthread.a'],
-                     ['libcmumps.a','libdmumps.a','libsmumps.a','libzmumps.a','libmumps_common.a','libpord.a','libmpiseq.a']]
-    self.functions = ['dmumps_c']
-    self.includes  = ['dmumps_c.h']
+    self.gitcommit        = 'v5.1.2-p2'
+    self.download         = ['git://https://bitbucket.org/petsc/pkg-mumps.git',
+                             'https://bitbucket.org/petsc/pkg-mumps/get/'+self.gitcommit+'.tar.gz']
+    self.downloaddirnames = ['petsc-pkg-mumps']
+    self.liblist          = [['libcmumps.a','libdmumps.a','libsmumps.a','libzmumps.a','libmumps_common.a','libpord.a'],
+                            ['libcmumps.a','libdmumps.a','libsmumps.a','libzmumps.a','libmumps_common.a','libpord.a','libpthread.a'],
+                            ['libcmumps.a','libdmumps.a','libsmumps.a','libzmumps.a','libmumps_common.a','libpord.a','libmpiseq.a'],
+                            ['libcmumps.a','libdmumps.a','libsmumps.a','libzmumps.a','libmumps_common.a','libpord.a','libpthread.a','libmpiseq.a']]
+    self.functions        = ['dmumps_c']
+    self.includes         = ['dmumps_c.h']
     #
     # Mumps does NOT work with 64 bit integers without a huge number of hacks we ain't making
+    self.precisions       = ['single','double']
     self.requires32bitint = 1;  # 1 means that the package will not work with 64 bit integers
     self.downloadonWindows= 1
+    self.hastests         = 1
+    self.hastestsdatafiles= 1
     return
 
   def setupHelp(self, help):
@@ -25,24 +30,29 @@ class Configure(config.package.Package):
 
   def setupDependencies(self, framework):
     config.package.Package.setupDependencies(self, framework)
+    self.flibs        = framework.require('config.packages.flibs',self)
     self.blasLapack   = framework.require('config.packages.BlasLapack',self)
     self.mpi          = framework.require('config.packages.MPI',self)
+    self.metis        = framework.require('config.packages.metis',self)
+    self.parmetis     = framework.require('config.packages.parmetis',self)
+    self.ptscotch     = framework.require('config.packages.PTScotch',self)
+    self.scalapack    = framework.require('config.packages.scalapack',self)
     if self.argDB['with-mumps-serial']:
-      self.deps       = [self.blasLapack]
+      self.deps       = [self.blasLapack,self.flibs]
+      self.odeps      = [self.metis]
     else:
-      self.scalapack  = framework.require('config.packages.scalapack',self)
-      self.deps       = [self.scalapack,self.mpi,self.blasLapack]
-      self.parmetis   = framework.require('config.packages.parmetis',self)
-      self.ptscotch   = framework.require('config.packages.PTScotch',self)
+      self.deps       = [self.scalapack,self.mpi,self.blasLapack,self.flibs]
+      self.odeps      = [self.metis,self.parmetis,self.ptscotch]
     return
 
   def consistencyChecks(self):
     config.package.Package.consistencyChecks(self)
+    if self.argDB['with-'+self.package] or self.argDB['download-'+self.package]:
+      if self.mpi.usingMPIUni and not self.argDB['with-mumps-serial']:
+        raise RuntimeError('Since you are building without MPI you must use --with-mumps-serial to install the correct MUMPS.')
     if self.argDB['with-mumps-serial']:
       if not self.mpi.usingMPIUni:
         raise RuntimeError('Serial MUMPS version is only compatible with MPIUni\nReconfigure using --with-mpi=0')
-      elif self.mpi.usingMPIUniFortranBinding:
-        raise RuntimeError('Serial MUMPS version is incompatible with the MPIUni Fortran bindings\nReconfigure using --with-mpiuni-fortran-binding=0')
     return
 
   def Install(self):
@@ -52,8 +62,6 @@ class Configure(config.package.Package):
       raise RuntimeError('Cannot install '+self.name+' without Fortran, make sure you do NOT have --with-fc=0')
     if not self.compilers.FortranDefineCompilerOption:
       raise RuntimeError('Fortran compiler cannot handle preprocessing directives from command line.')
-    if self.argDB['with-mumps-serial']:
-      raise RuntimeError('Cannot automatically install the serial version of MUMPS.')
     g = open(os.path.join(self.packageDir,'Makefile.inc'),'w')
     g.write('LPORDDIR   = $(topdir)/PORD/lib/\n')
     g.write('IPORD      = -I$(topdir)/PORD/include/\n')
@@ -64,11 +72,16 @@ class Configure(config.package.Package):
     # Disable threads on BGL
     if self.libraries.isBGL():
       orderingsc += ' -DWITHOUT_PTHREAD'
+    if self.metis.found:
+      g.write('IMETIS = '+self.headers.toString(self.metis.include)+'\n')
+      g.write('LMETIS = '+self.libraries.toString(self.metis.lib)+'\n')
+      orderingsc += ' -Dmetis'
+      orderingsf += ' '+self.compilers.FortranDefineCompilerOption+'metis'
     if self.parmetis.found:
-      g.write('IMETIS = '+self.headers.toString(self.parmetis.include)+'\n')
-      g.write('LMETIS = '+self.libraries.toString(self.parmetis.lib)+'\n')
-      orderingsc += ' -Dmetis -Dparmetis'
-      orderingsf += ' '+self.compilers.FortranDefineCompilerOption+'metis '+self.compilers.FortranDefineCompilerOption+'parmetis'
+      g.write('IPARMETIS = '+self.headers.toString(self.parmetis.include)+'\n')
+      g.write('LPARMETIS = '+self.libraries.toString(self.parmetis.lib)+'\n')
+      orderingsc += ' -Dparmetis'
+      orderingsf += ' '+self.compilers.FortranDefineCompilerOption+'parmetis'
     if self.ptscotch.found:
       g.write('ISCOTCH = '+self.headers.toString(self.ptscotch.include)+'\n')
       g.write('LSCOTCH = '+self.libraries.toString(self.ptscotch.lib)+'\n')
@@ -77,14 +90,14 @@ class Configure(config.package.Package):
 
     g.write('ORDERINGSC = '+orderingsc+'\n')
     g.write('ORDERINGSF = '+orderingsf+'\n')
-    g.write('LORDERINGS  = $(LMETIS) $(LPORD) $(LSCOTCH)\n')
-    g.write('IORDERINGSC = $(IMETIS) $(IPORD) $(ISCOTCH)\n')
+    g.write('LORDERINGS  = $(LPARMETIS) $(LMETIS) $(LPORD) $(LSCOTCH)\n')
+    g.write('IORDERINGSC = $(IPARMETIS) $(IMETIS) $(IPORD) $(ISCOTCH)\n')
     g.write('IORDERINGSF = $(ISCOTCH)\n')
 
     g.write('RM = /bin/rm -f\n')
     self.setCompilers.pushLanguage('C')
     g.write('CC = '+self.setCompilers.getCompiler()+'\n')
-    g.write('OPTC    = ' + self.setCompilers.getCompilerFlags().replace('-Wall','').replace('-Wshadow','') +'\n')
+    g.write('OPTC    = ' + self.removeWarningFlags(self.setCompilers.getCompilerFlags())+'\n')
     g.write('OUTC = -o \n')
     self.setCompilers.popLanguage()
     if not self.compilers.fortranIsF90:
@@ -109,20 +122,27 @@ class Configure(config.package.Package):
     g.write('LIBEXT  = .'+self.setCompilers.AR_LIB_SUFFIX+'\n')
     g.write('RANLIB  = '+self.setCompilers.RANLIB+'\n')
     g.write('SCALAP  = '+self.libraries.toString(self.scalapack.lib)+'\n')
-    g.write('INCPAR  = '+self.headers.toString(self.mpi.include)+'\n')
-    g.write('LIBPAR  = $(SCALAP) '+self.libraries.toString(self.mpi.lib)+'\n') #PARALLE LIBRARIES USED by MUMPS
+    if not self.argDB['with-mumps-serial']:
+      g.write('INCPAR  = '+self.headers.toString(self.mpi.include)+'\n')
+      g.write('LIBPAR  = $(SCALAP) '+self.libraries.toString(self.mpi.lib)+'\n')
+    else:
+      g.write('INCPAR  = -I../libseq\n')
     g.write('INCSEQ  = -I$(topdir)/libseq\n')
     g.write('LIBSEQ  =  $(LAPACK) -L$(topdir)/libseq -lmpiseq\n')
     g.write('LIBBLAS = '+self.libraries.toString(self.blasLapack.dlib)+'\n')
     g.write('OPTL    = -O -I.\n')
     g.write('INCS = $(INCPAR)\n')
-    g.write('LIB = $(LIBPAR)\n')
-    g.write('LIBSEQNEEDED =\n')
+    g.write('LIBS = $(LIBPAR)\n')
+    if self.argDB['with-mumps-serial']:
+      g.write('LIBSEQNEEDED = libseqneeded\n')
+      g.write('LIBS = $(LIBSEQ)\n')
+    else:
+      g.write('LIBSEQNEEDED =\n')
     g.close()
     if self.installNeeded('Makefile.inc'):
       try:
         output1,err1,ret1  = config.package.Package.executeShellCommand('cd '+self.packageDir+' && make clean', timeout=2500, log = self.log)
-      except RuntimeError, e:
+      except RuntimeError as e:
         pass
       try:
         self.logPrintBox('Compiling Mumps; this may take several minutes')
@@ -131,19 +151,11 @@ class Configure(config.package.Package):
         includeDir = os.path.join(self.installDir, self.includedir)
         self.logPrintBox('Installing Mumps; this may take several minutes')
         self.installDirProvider.printSudoPasswordMessage()
-        output,err,ret = config.package.Package.executeShellCommand(self.installSudo+'mkdir -p '+os.path.join(self.installDir,self.libdir)+' && cd '+self.packageDir+' && '+self.installSudo+'cp -f lib/*.* '+libDir+'/. && '+self.installSudo+'mkdir -p '+includeDir+' && '+self.installSudo+'cp -f include/*.* '+includeDir+'/.', timeout=2500, log = self.log)
-      except RuntimeError, e:
+        output,err,ret = config.package.Package.executeShellCommand(self.installSudo+'mkdir -p '+os.path.join(self.installDir,self.libdir)+' && cd '+self.packageDir+' && '+self.installSudo+'cp -f lib/*.* '+libDir+'/. && '+self.installSudo+'mkdir -p '+includeDir+' && '+self.installSudo+'cp -f include/*.* '+includeDir+'/.', timeout=50, log = self.log)
+        if self.argDB['with-mumps-serial']:
+          output,err,ret = config.package.Package.executeShellCommand('cd '+self.packageDir+' && '+self.installSudo+'cp -f libseq/libmpiseq.a '+libDir+'/. ', timeout=25, log = self.log)
+      except RuntimeError as e:
         raise RuntimeError('Error running make on MUMPS: '+str(e))
       self.postInstall(output1+err1+output2+err2,'Makefile.inc')
     return self.installDir
 
-  def configureLibrary(self):
-    if not self.argDB['with-mumps-serial']:
-      if self.parmetis.found:
-        self.deps.append(self.parmetis)
-      if self.ptscotch.found:
-        self.deps.append(self.ptscotch)
-    config.package.Package.configureLibrary(self)
-     # [parallem mumps] make sure either ptscotch or parmetis is enabled
-    if not self.argDB['with-mumps-serial'] and not self.ptscotch.found and not self.parmetis.found:
-       raise RuntimeError('MUMPS requires either Parmetis or PTScotch. Use either --download-parmetis or --download-ptscotch')
