@@ -3,22 +3,22 @@ import config.package
 class Configure(config.package.Package):
   def __init__(self, framework):
     config.package.Package.__init__(self, framework)
-    self.gitcommit       = '6.0.4-p1'
+    self.gitcommit       = '6.0.6-p1'
     self.download        = ['git://https://bitbucket.org/petsc/pkg-scotch.git',
-                            'http://ftp.mcs.anl.gov/pub/petsc/externalpackages/scotch_'+self.gitcommit+'.tar.gz']
-    self.downloaddirnames = ['scotch']
-    self.liblist         = [['libptesmumps.a','libptscotch.a','libptscotcherr.a','libscotch.a','libscotcherr.a']]
+                            'https://bitbucket.org/petsc/pkg-scotch/get/'+self.gitcommit+'.tar.gz']
+    self.downloaddirnames = ['scotch','petsc-pkg-scotch']
+    self.liblist         = [['libptesmumps.a','libptscotchparmetis.a','libptscotch.a','libptscotcherr.a','libesmumps.a','libscotch.a','libscotcherr.a']]
     self.functions       = ['SCOTCH_archBuild']
+    self.functionsDefine = ['SCOTCH_ParMETIS_V3_NodeND']
     self.includes        = ['ptscotch.h']
-    self.needsMath       = 1
     self.hastests        = 1
     return
 
   def setupDependencies(self, framework):
     config.package.Package.setupDependencies(self, framework)
-    self.indexTypes     = framework.require('PETSc.options.indexTypes', self)
     self.mpi            = framework.require('config.packages.MPI',self)
-    self.deps = [self.mpi]
+    self.mathlib        = framework.require('config.packages.mathlib',self)
+    self.deps           = [self.mpi, self.mathlib]
     return
 
   def Install(self):
@@ -66,13 +66,16 @@ class Configure(config.package.Package):
     # SCOTCH_PTSCOTCH is NOT defined above Mac does not know what clock_gettime() is!
     if self.setCompilers.isDarwin(self.log):
       self.cflags = self.cflags + ' -DCOMMON_TIMING_OLD'
-    if self.indexTypes.integerSize == 64:
+    if self.getDefaultIndexSize() == 64:
       self.cflags = self.cflags + ' -DINTSIZE64'
     else:
       self.cflags = self.cflags + ' -DINTSIZE32'
-    self.setCompilers.popLanguage()
+    # Prepend SCOTCH_ for the compatibility layer with ParMETIS
+    self.cflags = self.cflags + ' -DSCOTCH_METIS_PREFIX'
 
     g.write('CFLAGS	= '+self.cflags+'\n')
+    if self.argDB['with-batch']:
+      g.write('CCDFLAGS = '+self.checkNoOptFlag()+'\n')
     g.write('LDFLAGS	= '+ldflags+'\n')
     g.write('CP         = '+self.programs.cp+'\n')
     g.write('LEX	= '+self.programs.flex+'\n')
@@ -83,18 +86,13 @@ class Configure(config.package.Package):
     g.write('YACC	= '+self.programs.bison+' -y\n')
     g.close()
 
+    self.setCompilers.popLanguage()
+
     if self.installNeeded(os.path.join('src','Makefile.inc')):
       try:
         self.logPrintBox('Compiling PTScotch; this may take several minutes')
-#
-#    If desired one can have this build Scotch as well as PTScoth as indicated here
-#        output,err,ret  = config.package.Package.executeShellCommand('cd '+os.path.join(self.packageDir,'src')+' && make clean scotch ptscotch', timeout=2500, log = self.log)
-#
-        if self.mpi.found:
-          output,err,ret  = config.package.Package.executeShellCommand('cd '+os.path.join(self.packageDir,'src')+' && make clean ptesmumps', timeout=2500, log = self.log)
-        else:
-          output,err,ret  = config.package.Package.executeShellCommand('cd '+os.path.join(self.packageDir,'src')+' && make clean esmumps', timeout=2500, log = self.log)
-      except RuntimeError, e:
+        output,err,ret  = config.package.Package.executeShellCommand('cd '+os.path.join(self.packageDir,'src')+' && make clean ptesmumps esmumps', timeout=2500, log = self.log)
+      except RuntimeError as e:
         raise RuntimeError('Error running make on PTScotch: '+str(e))
 
       #Scotch has a file identical to one in ParMETIS, remove it so ParMETIS will not use it by mistake
