@@ -4,7 +4,7 @@ import os
 class Configure(config.package.CMakePackage):
   def __init__(self, framework):
     config.package.CMakePackage.__init__(self, framework)
-    self.gitcommit        = '7e10c8a' #maint/v5.2.1+ from june-16-2016
+    self.gitcommit        = '52fc55d' #master/v5.2.1+ from jul-30-2018
     self.download         = ['git://https://github.com/xiaoyeli/superlu','https://github.com/xiaoyeli/superlu/archive/'+self.gitcommit+'.tar.gz']
     self.downloaddirnames  = ['SuperLU','superlu']
     self.functions        = ['set_default_options']
@@ -18,6 +18,7 @@ class Configure(config.package.CMakePackage):
     self.downloadonWindows= 1
     self.hastests         = 1
     self.hastestsdatafiles= 1
+    self.precisions       = ['single','double']
     return
 
   def setupDependencies(self, framework):
@@ -34,8 +35,11 @@ class Configure(config.package.CMakePackage):
 
     #  Tests are broken on Apple since they depend on a shared library that is not resolved against BLAS
     args.append('-Denable_tests=0')
-    #  CMake in SuperLU should set this; but like many other packages it does not
-    args.append('-DCMAKE_INSTALL_NAME_DIR:STRING="'+os.path.join(self.installDir,self.libdir)+'"')
+    #  CMake in SuperLU should set this; but like many other packages it does not [and its different from superlu_dist]
+    args.append('-DCMAKE_INSTALL_LIBDIR:STRING="'+os.path.join(self.installDir,self.libdir)+'"')
+
+    if not hasattr(self.compilers, 'FC'):
+      args.append('-DXSDK_ENABLE_Fortran=OFF')
 
     # Add in fortran mangling flag
     if self.blasLapack.mangling == 'underscore':
