@@ -4,7 +4,7 @@ import os
 class Configure(config.package.CMakePackage):
   def __init__(self, framework):
     config.package.CMakePackage.__init__(self, framework)
-    self.gitcommit         = 'v5.1.3'
+    self.gitcommit         = 'v6.1.0'
     self.download         = ['git://https://github.com/xiaoyeli/superlu_dist','https://github.com/xiaoyeli/superlu_dist/archive/'+self.gitcommit+'.tar.gz']
     self.downloaddirnames = ['SuperLU_DIST','superlu_dist']
     self.functions        = ['set_default_options_dist']
@@ -16,6 +16,9 @@ class Configure(config.package.CMakePackage):
     self.hastests         = 1
     self.hastestsdatafiles= 1
     self.requirec99flag   = 1 # SuperLU_Dist uses C99 features
+    self.precisions       = ['double']
+    self.cxx              = 1
+    self.requirescxx11    = 1
     return
 
   def setupHelp(self, help):
@@ -25,17 +28,16 @@ class Configure(config.package.CMakePackage):
 
   def setupDependencies(self, framework):
     config.package.CMakePackage.setupDependencies(self, framework)
-    self.indexTypes     = framework.require('PETSc.options.indexTypes', self)
     self.blasLapack     = framework.require('config.packages.BlasLapack',self)
-    self.metis          = framework.require('config.packages.metis',self)
     self.parmetis       = framework.require('config.packages.parmetis',self)
     self.mpi            = framework.require('config.packages.MPI',self)
+    self.odeps          = [self.parmetis]
     if self.framework.argDB['download-superlu_dist-gpu']:
       self.cuda           = framework.require('config.packages.cuda',self)
       self.openmp         = framework.require('config.packages.openmp',self)
-      self.deps           = [self.mpi,self.blasLapack,self.parmetis,self.metis,self.cuda,self.openmp]
+      self.deps           = [self.mpi,self.blasLapack,self.cuda,self.openmp]
     else:
-      self.deps           = [self.mpi,self.blasLapack,self.parmetis,self.metis]
+      self.deps           = [self.mpi,self.blasLapack]
     return
 
   def formCMakeConfigureArgs(self):
@@ -43,13 +45,15 @@ class Configure(config.package.CMakePackage):
     if not self.framework.argDB['download-superlu_dist-gpu']:
       args.append('-DCMAKE_DISABLE_FIND_PACKAGE_OpenMP=TRUE')
     args.append('-DUSE_XSDK_DEFAULTS=YES')
-    metis_inc = self.headers.toStringNoDupes(self.metis.include)[2:]
-    parmetis_inc = self.headers.toStringNoDupes(self.parmetis.include)[2:]
     args.append('-DTPL_BLAS_LIBRARIES="'+self.libraries.toString(self.blasLapack.dlib)+'"')
-    args.append('-DTPL_PARMETIS_INCLUDE_DIRS="'+metis_inc+';'+parmetis_inc+'"')
-    args.append('-DTPL_PARMETIS_LIBRARIES="'+self.libraries.toString(self.parmetis.lib+self.metis.lib)+'"')
+    if self.parmetis.found:
+      args.append('-DTPL_PARMETIS_INCLUDE_DIRS="'+';'.join(self.parmetis.dinclude)+'"')
+      args.append('-DTPL_PARMETIS_LIBRARIES="'+self.libraries.toString(self.parmetis.dlib)+'"')
+    else:
+      args.append('-Denable_parmetislib=FALSE')
+      args.append('-DTPL_ENABLE_PARMETISLIB=FALSE')
 
-    if self.indexTypes.integerSize == 64:
+    if self.getDefaultIndexSize() == 64:
       args.append('-DXSDK_INDEX_SIZE=64')
 
     if not hasattr(self.compilers, 'FC'):
@@ -63,6 +67,7 @@ class Configure(config.package.CMakePackage):
     args.append('-DMPI_C_COMPILE_FLAGS:STRING=""')
     args.append('-DMPI_C_INCLUDE_PATH:STRING=""')
     args.append('-DMPI_C_LIBRARIES:STRING=""')
+    args.append('-DCMAKE_INSTALL_LIBDIR:STRING="'+os.path.join(self.installDir,self.libdir)+'"')
 
     # Add in fortran mangling flag
     if self.blasLapack.mangling == 'underscore':
