@@ -3,10 +3,11 @@ import config.package
 class Configure(config.package.Package):
   def __init__(self, framework):
     config.package.Package.__init__(self, framework)
-    self.download         = ['http://ftp.mcs.anl.gov/pub/petsc/externalpackages/fblaslapack-3.4.2.tar.gz']
-    self.double           = 0
-    self.downloadonWindows= 1
+    self.download               = ['http://ftp.mcs.anl.gov/pub/petsc/externalpackages/fblaslapack-3.4.2.tar.gz']
+    self.precisions             = ['single','double']
+    self.downloadonWindows      = 1
     self.skippackagewithoptions = 1
+    self.installwithbatch       = 1
 
   def setupDependencies(self, framework):
     config.package.Package.setupDependencies(self, framework)
@@ -23,7 +24,7 @@ class Configure(config.package.Package):
       raise RuntimeError('Cannot request fblaslapack without Fortran compiler, use --download-f2cblaslapack intead')
 
     self.setCompilers.pushLanguage('FC')
-    if config.setCompilers.Configure.isNAG(self.setCompilers.getLinker()):
+    if config.setCompilers.Configure.isNAG(self.setCompilers.getLinker(), self.log):
       raise RuntimeError('Cannot compile fortran blaslapack with NAG compiler')
     self.setCompilers.popLanguage()
 
@@ -78,6 +79,8 @@ class Configure(config.package.Package):
 
       if line.startswith('include'):
         line = '\n'
+      if line.find("-no-prec-div") >= 1:
+         raise RuntimeError('Some versions of the Intel compiler generate incorrect code on fblaslapack with the option -no-prec-div\nRun configure without this option')
       g.write(line) 
       line = f.readline()
     f.close()
@@ -87,18 +90,15 @@ class Configure(config.package.Package):
 
     try:
       self.logPrintBox('Compiling FBLASLAPACK; this may take several minutes')
-      output,err,ret  = config.base.Configure.executeShellCommand('cd '+blasDir+' && make -f tmpmakefile cleanblaslapck cleanlib && make -f tmpmakefile', timeout=2500, log = self.log)
-    except RuntimeError, e:
+      output1,err1,ret  = config.package.Package.executeShellCommand('cd '+blasDir+' && make -f tmpmakefile cleanblaslapck cleanlib && make -f tmpmakefile', timeout=2500, log = self.log)
+    except RuntimeError as e:
       raise RuntimeError('Error running make on '+blasDir+': '+str(e))
     try:
       self.installDirProvider.printSudoPasswordMessage()
-      output,err,ret  = config.base.Configure.executeShellCommand('cd '+blasDir+' && '+self.installSudo+'mkdir -p '+libdir+' && '+self.installSudo+'cp -f libfblas.'+self.setCompilers.AR_LIB_SUFFIX+' libflapack.'+self.setCompilers.AR_LIB_SUFFIX+' '+ libdir, timeout=300, log = self.log)
-    except RuntimeError, e:
+      output2,err2,ret  = config.package.Package.executeShellCommand('cd '+blasDir+' && '+self.installSudo+'mkdir -p '+libdir+' && '+self.installSudo+'cp -f libfblas.'+self.setCompilers.AR_LIB_SUFFIX+' libflapack.'+self.setCompilers.AR_LIB_SUFFIX+' '+ libdir, timeout=300, log = self.log)
+    except RuntimeError as e:
       raise RuntimeError('Error moving '+blasDir+' libraries: '+str(e))
-    try:
-      output,err,ret  = config.base.Configure.executeShellCommand('cd '+blasDir+' && cp -f tmpmakefile '+os.path.join(self.confDir, 'lib','petsc','conf',self.name), timeout=30, log = self.log)
-    except RuntimeError, e:
-      pass
+    self.postInstall(output1+err1+output2+err2,'tmpmakefile')
     return self.installDir
 
 
