@@ -4,12 +4,11 @@ import os
 class Configure(config.package.GNUPackage):
   def __init__(self, framework):
     config.package.GNUPackage.__init__(self, framework)
-    self.download          = ['http://ftp.gnu.org/gnu/make/make-4.1.tar.gz','http://ftp.mcs.anl.gov/pub/petsc/externalpackages/make-4.1.tar.gz']
+    self.download          = ['http://ftp.mcs.anl.gov/pub/petsc/externalpackages/make-4.2.1-6.fc28.tar.gz']
     self.complex           = 1
-    self.double            = 0
     self.downloadonWindows = 1
     self.useddirectly      = 0
-
+    self.linkedbypetsc     = 0 
     self.printdirflag      = ''
     self.noprintdirflag    = ''
     self.paroutflg         = ''
@@ -22,6 +21,8 @@ class Configure(config.package.GNUPackage):
     import nargs
     config.package.GNUPackage.setupHelp(self, help)
     help.addArgument('MAKE', '-with-make-np=<np>',                           nargs.ArgInt(None, None, min=1, help='Default number of threads to use for parallel builds'))
+    help.addArgument('MAKE', '-with-make-test-np=<np>',                      nargs.ArgInt(None, None, min=1, help='Default number of threads to use for parallel tests'))
+    help.addArgument('MAKE', '-with-make-load=<load>',                       nargs.ArgReal(None, None, min=1.0, help='max load to use for parallel builds'))
     help.addArgument('MAKE', '-download-make-cc=<prog>',                     nargs.Arg(None, None, 'C compiler for GNU make configure'))
     help.addArgument('MAKE', '-download-make-configure-options=<options>',   nargs.Arg(None, None, 'additional options for GNU make configure'))
     help.addArgument('MAKE', '-with-make-exec=<executable>',                 nargs.Arg(None, None, 'Make executable to look for'))
@@ -53,17 +54,22 @@ class Configure(config.package.GNUPackage):
     try:
       self.logPrintBox('Running configure on ' +self.PACKAGE+'; this may take several minutes')
       output1,err1,ret1  = config.base.Configure.executeShellCommand('cd '+self.packageDir+' && ./configure '+args, timeout=2000, log = self.log)
-    except RuntimeError, e:
+    except RuntimeError as e:
       raise RuntimeError('Error running configure on ' + self.PACKAGE+': '+str(e))
     try:
       self.logPrintBox('Running make on '+self.PACKAGE+'; this may take several minutes')
       output1,err1,ret  = config.package.Package.executeShellCommand('cd '+self.packageDir+' && ./build.sh && ./make install && ./make clean', timeout=2500, log = self.log)
-    except RuntimeError, e:
+    except RuntimeError as e:
       raise RuntimeError('Error building or installing make '+self.PACKAGE+': '+str(e))
     self.postInstall(output1+err1, conffile)
     return self.installDir
 
   def generateGMakeGuesses(self):
+    if 'with-make-exec' in self.argDB:
+      self.log.write('Looking for user provided Make executable '+self.argDB['with-make-exec']+'\n')
+      yield self.argDB['with-make-exec']
+      raise RuntimeError('Error! User provided with-make-exec is not GNU make : '+self.argDB['with-make-exec'])
+
     if os.path.exists('/usr/bin/cygcheck.exe'):
       if os.path.exists('/usr/bin/make'):
         yield '/usr/bin/make'
@@ -73,31 +79,18 @@ class Configure(config.package.GNUPackage):
 *** Please rerun cygwin-setup and select module "make" for install.**************''')
     yield 'gmake'
     yield 'make'
-    return
-
-  def generateMakeGuesses(self):
-    yield 'make'
-    yield 'gmake'
+    raise RuntimeError('Could not locate the GNU make utility (version greater than 3.80) on your system, specify with --with-make-exec=gmake or try --download-make')
     return
 
   def configureMake(self):
-    '''Check for user specified make - or gmake, make'''
-    if 'with-make-exec' in self.argDB:
-      self.log.write('Looking for specified Make executable '+self.argDB['with-make-exec']+'\n')
-      if not self.getExecutable(self.argDB['with-make-exec'], getFullPath=1, resultName='make'):
-        raise RuntimeError('Error! User provided make not found :'+self.argDB['with-make-exec'])
-      self.found = 1
-      return
+    '''Check Guesses for GNU make'''
     for gmake in self.generateGMakeGuesses():
-      if self.getExecutable(gmake,getFullPath = 1,resultName = 'make'):
-        if self.checkGNUMake(self.make):
-          self.found = 1
-          return
-    for make in self.generateMakeGuesses():
-      if self.getExecutable(make,getFullPath = 1,resultName = 'make'):
+      self.haveGNUMake, self.haveGNUMake4 = self.checkGNUMake(gmake)
+      if self.haveGNUMake:
+        self.getExecutable(gmake,getFullPath = 1,resultName = 'make')
         self.found = 1
         return
-    raise RuntimeError('Could not locate the make utility on your system, try --download-make')
+    raise RuntimeError('Could not locate the GNU make utility (version greater than 3.80) on your system, specify with --with-make-exec=gmake or try --download-make')
 
   def checkGNUMake(self,make):
     '''Check for GNU make'''
@@ -113,33 +106,27 @@ class Configure(config.package.GNUPackage):
         minor = int(gver.group(2))
         if ((major > 3) or (major == 3 and minor > 80)): haveGNUMake = 1
         if (major > 3): haveGNUMake4 = 1
-    except RuntimeError, e:
+    except RuntimeError as e:
       self.log.write('GNUMake check failed: '+str(e)+'\n')
     return haveGNUMake, haveGNUMake4
 
-  def configureCheckGNUMake(self):
+  def setupGNUMake(self):
     '''Setup other GNU make stuff'''
-    # make sure this gets called by --download-make or --with-make-exec etc paths.
-    self.haveGNUMake, self.haveGNUMake4 = self.checkGNUMake(self.make)
-
     if self.haveGNUMake4 and not self.setCompilers.isDarwin(self.log) and not self.setCompilers.isFreeBSD(self.log):
       self.paroutflg = "--output-sync=recurse"
 
-    if self.haveGNUMake:
-      # Setup make flags
-      self.printdirflag = ' --print-directory'
-      self.noprintdirflag = ' --no-print-directory'
-      self.addMakeMacro('MAKE_IS_GNUMAKE',1)
-      # Use rules which look inside archives
-      self.addMakeRule('libc','${LIBNAME}(${OBJSC})')
-      self.addMakeRule('libcxx','${LIBNAME}(${OBJSCXX})')
-      self.addMakeRule('libcu','${LIBNAME}(${OBJSCU})')
-    else:
-      self.logPrintBox('Warning: '+self.make+' is not GNUMake (3.81 or higher). Suggest using --download-make')
-      self.addMakeRule('libc','${OBJSC}','-${AR} ${AR_FLAGS} ${LIBNAME} ${OBJSC}')
-      self.addMakeRule('libcxx','${OBJSCXX}','-${AR} ${AR_FLAGS} ${LIBNAME} ${OBJSCXX}')
-      self.addMakeRule('libcu','${OBJSCU}','-${AR} ${AR_FLAGS} ${LIBNAME} ${OBJSCU}')
+    # Setup make flags
+    self.printdirflag = ' --print-directory'
+    self.noprintdirflag = ' --no-print-directory'
+    self.addMakeMacro('MAKE_IS_GNUMAKE',1)
+    # Use rules which look inside archives
+    self.addMakeRule('libc','${LIBNAME}(${OBJSC})')
+    self.addMakeRule('libcxx','${LIBNAME}(${OBJSCXX})')
+    self.addMakeRule('libcu','${LIBNAME}(${OBJSCU})')
     self.addMakeRule('libf','${OBJSF}','-${AR} ${AR_FLAGS} ${LIBNAME} ${OBJSF}')
+    self.addMakeMacro('OMAKE_PRINTDIR ', self.make+' '+self.printdirflag)
+    self.addMakeMacro('OMAKE', self.make+' '+self.noprintdirflag)
+    self.addMakeMacro('MAKE_PAR_OUT_FLG', self.paroutflg)
     return
 
   def compute_make_np(self,i):
@@ -155,37 +142,70 @@ class Configure(config.package.GNUPackage):
     else:         return int(4+12*f16+16*f32+32*f64+(i-64)*f99)
     return
 
+  def compute_make_test_np(self,i):
+    f32 = 0.50
+    f99 = 0.35
+    if (i<=2):    return 1
+    elif (i<=4):  return 2
+    elif (i<=32): return int(i*f32)
+    else:         return int(32*f32+(i-32)*f99)
+    return
+
+  def compute_make_load(self,i):
+    f64 = 1.5
+    f99 = 1.1
+    if (i<=64):   return i*f64
+    else:         return 64*f64+(i-64)*f99
+    return
+
   def configureMakeNP(self):
     '''check no of cores on the build machine [perhaps to do make '-j ncores']'''
     try:
       import multiprocessing # python-2.6 feature
       cores = multiprocessing.cpu_count()
       make_np = self.compute_make_np(cores)
+      make_test_np = self.compute_make_test_np(cores)
+      make_load = self.compute_make_load(cores)
       self.logPrint('module multiprocessing found %d cores: using make_np = %d' % (cores,make_np))
-    except (ImportError), e:
+    except (ImportError) as e:
       cores = 2
       make_np = 2
+      make_test_np = 1
+      make_load = 3
       self.logPrint('module multiprocessing *not* found: using default make_np = %d' % make_np)
 
     if 'with-make-np' in self.argDB and self.argDB['with-make-np']:
-        self.logPrint('using user-provided make_np = %d' % make_np)
         make_np = self.argDB['with-make-np']
+        self.logPrint('using user-provided make_np = %d' % make_np)
+
+    if not self.argDB.get('with-mpi'):
+      make_test_np = make_np
+
+    if 'with-make-test-np' in self.argDB and self.argDB['with-make-test-np']:
+        make_test_np = self.argDB['with-make-test-np']
+        self.logPrint('using user-provided make_test_np = %d' % make_test_np)
+
+    if 'with-make-load' in self.argDB and self.argDB['with-make-load']:
+        make_load = self.argDB['with-make-load']
+        self.logPrint('using user-provided make_load = %f' % make_load)
 
     self.make_np = make_np
+    self.make_test_np = make_test_np
+    self.make_load = make_load
     self.addMakeMacro('MAKE_NP',str(make_np))
+    self.addMakeMacro('MAKE_TEST_NP',str(make_test_np))
+    self.addMakeMacro('MAKE_LOAD',str(make_load))
     self.addMakeMacro('NPMAX',str(cores))
-    self.make_jnp = self.make + ' -j ' + str(self.make_np)
+    self.make_jnp = self.make + ' -j' + str(self.make_np) +' -l'+str(self.make_load)
     return
 
   def configure(self):
     if (self.argDB['download-make']):
       config.package.GNUPackage.configure(self)
       self.getExecutable('make', path=os.path.join(self.installDir,'bin'), getFullPath = 1)
+      self.haveGNUMake, self.haveGNUMake4 = self.checkGNUMake(self.make)
     else:
       self.executeTest(self.configureMake)
-    self.executeTest(self.configureCheckGNUMake)
+    self.executeTest(self.setupGNUMake)
     self.executeTest(self.configureMakeNP)
-    self.addMakeMacro('OMAKE_PRINTDIR ', self.make+' '+self.printdirflag)
-    self.addMakeMacro('OMAKE', self.make+' '+self.noprintdirflag)
-    self.addMakeMacro('MAKE_PAR_OUT_FLG', self.paroutflg)
     return
