@@ -3,12 +3,21 @@ import config.package
 class Configure(config.package.Package):
   def __init__(self, framework):
     config.package.Package.__init__(self, framework)
-    self.giturls           = ['https://bitbucket.org/petsc/petsc4py']
-    self.gitcommit         = '83a14d3' # Jun 22 2015
-    self.download          = ['http://ftp.mcs.anl.gov/pub/petsc/externalpackages/petsc4py-83a14d3.tar.gz']
-    self.functions         = []
-    self.includes          = []
+    self.gitcommit              = 'a013d13' # 3.10.0+ (maint)
+    self.download               = ['git://https://bitbucket.org/petsc/petsc4py','https://bitbucket.org/petsc/petsc4py/get/'+self.gitcommit+'.tar.gz']
+    self.functions              = []
+    self.includes               = []
     self.skippackagewithoptions = 1
+    self.useddirectly           = 0
+    self.linkedbypetsc          = 0
+    self.builtafterpetsc        = 1
+    self.downloaddirnames       = ['petsc-petsc4py','petsc4py']
+    return
+
+  def setupHelp(self, help):
+    config.package.Package.setupHelp(self,help)
+    import nargs
+    help.addArgument('PETSC4PY', '-download-petsc4py-python=<prog>',                            nargs.Arg(None, None, 'Use this python program to build petsc4py'))
     return
 
   def setupDependencies(self, framework):
@@ -22,22 +31,22 @@ class Configure(config.package.Package):
   def Install(self):
     import os
     pp = os.path.join(self.installDir,'lib','python*','site-packages')
-    if self.setCompilers.isDarwin():
+    if self.setCompilers.isDarwin(self.log):
       apple = 'You may need to\n (csh/tcsh) setenv MACOSX_DEPLOYMENT_TARGET 10.X\n (sh/bash) MACOSX_DEPLOYMENT_TARGET=10.X; export MACOSX_DEPLOYMENT_TARGET\nbefore running make on PETSc'
     else:
       apple = ''
     self.logClearRemoveDirectory()
     self.logResetRemoveDirectory()
     archflags = ""
-    if self.setCompilers.isDarwin():
-      if self.types.sizes['known-sizeof-void-p'] == 32:
+    if self.setCompilers.isDarwin(self.log):
+      if self.types.sizes['known-sizeof-void-p'] == 4:
         archflags = "ARCHFLAGS=\'-arch i386\' "
       else:
         archflags = "ARCHFLAGS=\'-arch x86_64\' "
 
     # if installing prefix location then need to set new value for PETSC_DIR/PETSC_ARCH
     if self.argDB['prefix']:
-       newdir = 'PETSC_DIR='+self.argDB['prefix']+' '+'PETSC_ARCH= MPICC=${PCC} '
+       newdir = 'PETSC_DIR='+os.path.abspath(os.path.expanduser(self.argDB['prefix']))+' '+'PETSC_ARCH= MPICC=${PCC} '
     else:
        newdir = 'MPICC=${PCC} '
 
@@ -47,19 +56,31 @@ class Configure(config.package.Package):
     else:
        newuser = ''
 
+    # determine python binary to use
+    if 'download-petsc4py-python' in self.argDB:
+      self.getExecutable(self.argDB['download-petsc4py-python'], getFullPath=1, resultName='pyexe', setMakeMacro = 0)
+    else:
+      import sys
+      self.pyexe = sys.executable
+
+    self.addDefine('PYTHON_EXE','"'+self.pyexe+'"')
+    self.addDefine('HAVE_PETSC4PY',1)
+    self.addMakeMacro('PETSC4PY','yes')
     self.addMakeRule('petsc4pybuild','', \
                        ['@echo "*** Building petsc4py ***"',\
+                          '@${RM} -f ${PETSC_ARCH}/lib/petsc/conf/petsc4py.errorflg',\
                           '@(cd '+self.packageDir+' && \\\n\
-           '+newuser+newdir+'python setup.py clean --all && \\\n\
-           '+newuser+newdir+archflags+'python setup.py build ) > ${PETSC_ARCH}/lib/petsc/conf/petsc4py.log 2>&1 || \\\n\
+           '+newuser+newdir+self.pyexe+' setup.py clean --all && \\\n\
+           '+newuser+newdir+archflags+self.pyexe+' setup.py build ) > ${PETSC_ARCH}/lib/petsc/conf/petsc4py.log 2>&1 || \\\n\
              (echo "**************************ERROR*************************************" && \\\n\
              echo "Error building petsc4py. Check ${PETSC_ARCH}/lib/petsc/conf/petsc4py.log" && \\\n\
              echo "********************************************************************" && \\\n\
+             touch ${PETSC_ARCH}/lib/petsc/conf/petsc4py.errorflg && \\\n\
              exit 1)'])
     self.addMakeRule('petsc4pyinstall','', \
                        ['@echo "*** Installing petsc4py ***"',\
                           '@(MPICC=${PCC} && export MPICC && cd '+self.packageDir+' && \\\n\
-           '+newdir+archflags+'python setup.py install --install-lib='+os.path.join(self.installDir,'lib')+') >> ${PETSC_ARCH}/lib/petsc/conf/petsc4py.log 2>&1 || \\\n\
+           '+newdir+archflags+self.pyexe+' setup.py install --install-lib='+os.path.join(self.installDir,'lib')+') >> ${PETSC_ARCH}/lib/petsc/conf/petsc4py.log 2>&1 || \\\n\
              (echo "**************************ERROR*************************************" && \\\n\
              echo "Error building petsc4py. Check ${PETSC_ARCH}/lib/petsc/conf/petsc4py.log" && \\\n\
              echo "********************************************************************" && \\\n\
@@ -78,27 +99,9 @@ class Configure(config.package.Package):
     return self.installDir
 
   def configureLibrary(self):
-    if not self.sharedLibraries.useShared:
+    if not self.sharedLibraries.useShared and not self.setCompilers.isCygwin(self.log):
         raise RuntimeError('petsc4py requires PETSc be built with shared libraries; rerun with --with-shared-libraries')
     self.checkDownload()
-    if self.setCompilers.isDarwin():
-      # The name of the Python library on Apple is Python which does not end in the expected .dylib
-      # Thus see if the python library in the standard locations points to the Python version
-      import sys
-      import os
-      prefix = sys.exec_prefix
-      if os.path.isfile(os.path.join(prefix,'Python')):
-        for i in ['/usr/lib/libpython.dylib','/usr/lib/libpython'+sys.version[:3]+'.dylib','/opt/local/lib/libpython.dylib','/opt/local/lib/libpython'+sys.version[:3]+'.dylib']:
-          if os.path.realpath(i) == os.path.join(prefix,'Python'):
-            self.addDefine('PYTHON_LIB','"'+os.path.join(i)+'"')
-            return
-        raise RuntimeError('realpath of /usr/lib/libpython.dylib ('+os.path.realpath('/usr/lib/libpython.dylib')+') does not point to the expected Python library path ('+os.path.join(prefix,'Python')+') for current Python;\n')
-      elif os.path.isfile(os.path.join(prefix,'lib','libpython.dylib')):
-        self.addDefine('PYTHON_LIB','"'+os.path.join(prefix,'lib','libpython.dylib')+'"')
-      elif os.path.isfile(os.path.join(prefix,'lib','libpython'+sys.version[:3]+'.dylib')):
-        self.addDefine('PYTHON_LIB','"'+os.path.join(prefix,'lib','libpython'+sys.version[:3]+'.dylib')+'"')
-      else:
-        raise RuntimeError('Unable to find Python dynamic library at prefix '+prefix)
 
   def alternateConfigureLibrary(self):
     self.addMakeRule('petsc4py-build','')
