@@ -1,13 +1,17 @@
 import config.package
 import os
 
+def noCheck(command, status, output, error):
+  ''' Do no check result'''
+  return
+
 class Configure(config.package.GNUPackage):
   def __init__(self, framework):
     config.package.GNUPackage.__init__(self, framework)
-    self.download          = ['git://https://bitbucket.org/petsc/pkg-sowing.git','http://ftp.mcs.anl.gov/pub/petsc/externalpackages/sowing-1.1.20-pre2.tar.gz']
-    self.gitcommit         = 'v1.1.20-pre2'
+    self.gitcommit         = 'v1.1.25-p1'
+    self.download          = ['git://https://bitbucket.org/petsc/pkg-sowing.git','https://bitbucket.org/petsc/pkg-sowing/get/'+self.gitcommit+'.tar.gz']
+    self.downloaddirnames  = ['petsc-pkg-sowing']
     self.complex           = 1
-    self.double            = 0
     self.downloadonWindows = 1
     self.publicInstall     = 0  # always install in PETSC_DIR/PETSC_ARCH (not --prefix) since this is not used by users
     self.parallelMake      = 0  # sowing does not support make -j np
@@ -25,8 +29,7 @@ class Configure(config.package.GNUPackage):
 
   def setupDependencies(self, framework):
     config.package.GNUPackage.setupDependencies(self, framework)
-    self.petscdir       = framework.require('PETSc.options.petscdir', self.setCompilers)
-    self.petscclone     = framework.require('PETSc.options.petscclone',self.setCompilers)
+    self.petscclone = framework.require('PETSc.options.petscclone', None)
     return
 
   def formGNUConfigureArgs(self):
@@ -48,21 +51,37 @@ class Configure(config.package.GNUPackage):
   def alternateConfigureLibrary(self):
     self.checkDownload()
 
+  def checkBfortVersion(self,mmajor,mminor,msubminor):
+    try:
+      import re
+      (output, error, status) = config.base.Configure.executeShellCommand(self.bfort+' -version', checkCommand=noCheck, log = self.log)
+      ver = re.compile('bfort \(sowing\) release ([0-9]+).([0-9]+).([0-9]+)').match(output)
+      major    = int(ver.group(1))
+      minor    = int(ver.group(2))
+      subminor = int(ver.group(3))
+    except RuntimeError as e:
+      self.log.write(self.bfort+' version check failed: '+str(e)+'\n')
+      return
+    if (major < mmajor) or (major == mmajor and minor < mminor) or (major == mmajor and minor == mminor and subminor < msubminor):
+      raise RuntimeError(self.bfort+' version '+str(major)+'.'+str(minor)+'.'+str(subminor)+' is older than required '+str(mmajor)+'.'+str(mminor)+'.'+str(msubminor)+'. Perhaps a stale install of sowing?')
+    return
+
   def configure(self):
-    if (self.framework.clArgDB.has_key('with-sowing') and not self.argDB['with-sowing']) or \
-          (self.framework.clArgDB.has_key('download-sowing') and not self.argDB['download-sowing']):
-      self.logPrint("Not checking sowing on user request\n")
+    if ('with-sowing' in self.framework.clArgDB and not self.argDB['with-sowing']):
+      if hasattr(self.compilers, 'FC') and self.framework.argDB['with-fortran-bindings'] and self.petscclone.isClone:
+        raise RuntimeError('Cannot use --with-sowing=0 if using Fortran (bindings) and git repository for PETSc')
+      self.logPrint("Not checking sowing on user request of --with-sowing=0\n")
       return
 
     if self.framework.batchBodies:
       self.logPrint('In --with-batch mode with outstanding batch tests to be made; hence skipping sowing for this configure')
       return
 
-    if (self.petscclone.isClone and hasattr(self.compilers, 'FC')) or self.framework.clArgDB.has_key('download-sowing'):
+    if (self.petscclone.isClone and hasattr(self.compilers, 'FC') and self.framework.argDB['with-fortran-bindings']) or ('download-sowing' in self.framework.clArgDB and self.argDB['download-sowing']):
       self.logPrint('PETSc clone, checking for Sowing \n')
       self.getExecutable('pdflatex', getFullPath = 1)
 
-      if self.framework.clArgDB.has_key('with-sowing-dir') and self.argDB['with-sowing-dir']:
+      if 'with-sowing-dir' in self.framework.clArgDB and self.argDB['with-sowing-dir']:
         installDir = os.path.join(self.argDB['with-sowing-dir'],'bin')
 
         self.getExecutable('bfort',    path=installDir, getFullPath = 1)
@@ -86,6 +105,9 @@ class Configure(config.package.GNUPackage):
         else:
           self.logPrint('Bfort not found. Installing sowing for FortranStubs')
           if (not self.argDB['download-sowing']):  self.argDB['download-sowing'] = 1
+          #check cygwin has g++
+          if os.path.exists('/usr/bin/cygcheck.exe') and not os.path.exists('/usr/bin/g++.exe'):
+            raise RuntimeError('Error! sowing on windows requires cygwin/g++. Please install it with cygwin setup.exe')
           config.package.GNUPackage.configure(self)
 
           installDir = os.path.join(self.installDir,'bin')
@@ -94,9 +116,10 @@ class Configure(config.package.GNUPackage):
           self.getExecutable('mapnames', path=installDir, getFullPath = 1)
           self.getExecutable('bib2html', path=installDir, getFullPath = 1)
 
+      self.checkBfortVersion(1,1,25)
       self.buildFortranStubs()
     else:
-      self.logPrint("Not a clone of PETSc or no Fortran, don't need Sowing\n")
+      self.logPrint("Not a clone of PETSc or no Fortran compiler or fortran-bindings disabled, don't need Sowing\n")
     return
 
   def buildFortranStubs(self):
@@ -108,13 +131,13 @@ class Configure(config.package.GNUPackage):
         self.logWrite('           Running '+self.bfort+' to generate fortran stubs\n')
         try:
           import os,sys
-          sys.path.insert(0, os.path.abspath(os.path.join('bin','maint')))
+          sys.path.insert(0, os.path.abspath(os.path.join('lib','petsc','bin','maint')))
           import generatefortranstubs
           del sys.path[0]
           generatefortranstubs.main(self.petscdir.dir, self.bfort, self.petscdir.dir,0)
           if self.compilers.fortranIsF90:
             generatefortranstubs.processf90interfaces(self.petscdir.dir,0)
           self.framework.actions.addArgument('PETSc', 'File creation', 'Generated Fortran stubs')
-        except RuntimeError, e:
+        except RuntimeError as e:
           raise RuntimeError('*******Error generating Fortran stubs: '+str(e)+'*******\n')
     return
