@@ -1,3 +1,4 @@
+from __future__ import print_function
 try:
   import readline
 except ImportError: pass
@@ -154,15 +155,15 @@ tests will cause ValueError.
     '''Returns the value. SHOULD MAKE THIS A PROPERTY'''
     if not self.isValueSet():
       checkInteractive(self.key)
-      if self.help: print self.help
+      if self.help: print(self.help)
       while 1:
         try:
           self.setValue(Arg.parseValue(raw_input(self.getEntryPrompt())))
           break
         except KeyboardInterrupt:
           raise KeyError('Could not find value for key '+str(self.key))
-        except TypeError, e:
-          print str(e)
+        except TypeError as e:
+          print(str(e))
     return self.value
 
   def checkKey(self):
@@ -250,7 +251,7 @@ class ArgFuzzyBool(Arg):
 
 class ArgInt(Arg):
   '''Arguments that represent integer numbers'''
-  def __init__(self, key, value = None, help = '', min = -2147483647L, max = 2147483648L, isTemporary = 0, deprecated = False):
+  def __init__(self, key, value = None, help = '', min = -2147483647, max = 2147483648, isTemporary = 0, deprecated = False):
     self.min = min
     self.max = max
     Arg.__init__(self, key, value, help, isTemporary, deprecated)
@@ -494,6 +495,8 @@ class ArgDownload(Arg):
       if not urlparse.urlparse(value)[0]: # how do we check if the URL is invalid?
         if os.path.isfile(value):
           value = 'file://'+os.path.abspath(value)
+        elif os.path.isdir(value):
+          value = 'dir://'+os.path.abspath(value)
         else:
           raise ValueError('Invalid download location: '+str(value)+' for key '+str(self.key))
     self.value = value
