@@ -20,7 +20,8 @@ class Configure(config.base.Configure):
 
   def setupHelp(self, help):
     import nargs
-    help.addArgument('PETSc', '-with-precision=<single,double,__float128>', nargs.Arg(None, 'double', 'Specify numerical precision'))
+    #  Dec 2016, the __fp16 type is only available with GNU compilers on ARM systems
+    help.addArgument('PETSc', '-with-precision=<__fp16,single,double,__float128>', nargs.Arg(None, 'double', 'Specify numerical precision'))
     help.addArgument('PETSc', '-with-scalar-type=<real or complex>', nargs.Arg(None, 'real', 'Specify real or complex numbers'))
     return
 
@@ -30,7 +31,6 @@ class Configure(config.base.Configure):
     self.languages = framework.require('PETSc.options.languages', self)
     self.compilers = framework.require('config.compilers', self)
     self.libraries = framework.require('config.libraries',self)
-    self.mpi       = framework.require('config.packages.MPI',self)
     return
 
 
@@ -43,10 +43,12 @@ class Configure(config.base.Configure):
         raise RuntimeError('C Compiler provided doest not support C99 complex')
       if self.languages.clanguage == 'Cxx' and not self.types.cxx_complex:
         raise RuntimeError('Cxx compiler provided does not support std::complex')
+      if self.languages.clanguage == 'Cxx':
+        self.addDefine('USE_CXXCOMPLEX',1)
     elif not self.scalartype == 'real':
       raise RuntimeError('--with-scalar-type must be real or complex')
     self.addDefine('USE_SCALAR_'+self.scalartype.upper(), '1')
-    self.framework.logPrint('Scalar type is '+str(self.scalartype))
+    self.logPrint('Scalar type is '+str(self.scalartype))
     # On apple isinf() and isnan() do not work when <complex> is included
     self.pushLanguage(self.languages.clanguage)
     if self.scalartype == 'complex' and self.languages.clanguage == 'Cxx':
@@ -56,9 +58,9 @@ class Configure(config.base.Configure):
         self.addDefine('HAVE_ISNAN',1)
       if self.checkLink('#include <math.h>\n#include <complex>\n','double b = 2.0;int a = isinf(b);\n'):
         self.addDefine('HAVE_ISINF',1)
-      if self.checkLink('#include <math.h>\n#include <complex>\n','double b = 2.0;int a = _isnan(b);\n'):
+      if self.checkLink('#include <float.h>\n#include <complex>\n','double b = 2.0;int a = _isnan(b);\n'):
         self.addDefine('HAVE__ISNAN',1)
-      if self.checkLink('#include <math.h>\n#include <complex>\n','double b = 2.0;int a = _finite(b);\n'):
+      if self.checkLink('#include <float.h>\n#include <complex>\n','double b = 2.0;int a = _finite(b);\n'):
         self.addDefine('HAVE__FINITE',1)
     else:
       if self.checkLink('#include <math.h>\n','double b = 2.0; int a = isnormal(b);\n'):
@@ -67,9 +69,9 @@ class Configure(config.base.Configure):
         self.addDefine('HAVE_ISNAN',1)
       if self.checkLink('#include <math.h>\n','double b = 2.0; int a = isinf(b);\n'):
         self.addDefine('HAVE_ISINF',1)
-      if self.checkLink('#include <math.h>\n','double b = 2.0;int a = _isnan(b);\n'):
+      if self.checkLink('#include <float.h>\n','double b = 2.0;int a = _isnan(b);\n'):
         self.addDefine('HAVE__ISNAN',1)
-      if self.checkLink('#include <math.h>\n','double b = 2.0;int a = _finite(b);\n'):
+      if self.checkLink('#include <float.h>\n','double b = 2.0;int a = _finite(b);\n'):
         self.addDefine('HAVE__FINITE',1)
     self.popLanguage()
     return
@@ -77,7 +79,14 @@ class Configure(config.base.Configure):
   def configurePrecision(self):
     '''Set the default real number precision for PETSc objects'''
     self.precision = self.framework.argDB['with-precision'].lower()
-    if self.precision == 'single':
+    if self.precision == '__fp16':  # supported by gcc trunk
+      if self.scalartype == 'complex':
+        raise RuntimeError('__fp16 can only be used with real numbers, not complex')
+      if hasattr(self.compilers, 'FC'):
+        raise RuntimeError('__fp16 can only be used with C compiler, not Fortran')
+      self.addDefine('USE_REAL___FP16', '1')
+      self.addMakeMacro('PETSC_SCALAR_SIZE', '16')
+    elif self.precision == 'single':
       self.addDefine('USE_REAL_SINGLE', '1')
       self.addMakeMacro('PETSC_SCALAR_SIZE', '32')
     elif self.precision == 'double':
@@ -91,7 +100,9 @@ class Configure(config.base.Configure):
         raise RuntimeError('quadmath support not found. --with-precision=__float128 works with gcc-4.6 and newer compilers.')
     else:
       raise RuntimeError('--with-precision must be single, double,__float128')
-    self.framework.logPrint('Precision is '+str(self.precision))
+    self.logPrint('Precision is '+str(self.precision))
+    if self.precision == '__float128' and self.scalartype == 'complex' and self.languages.clanguage == 'Cxx':
+      raise RuntimeError('Cannot use --with-precision=__float128 --with-scalar-type=complex and --with-clanguage=cxx because C++ std:complex class has no support for __float128, use --with-clanguage=c')
     return
 
   def configure(self):
