@@ -5,7 +5,7 @@
 # sources, and encoding the rules through CMake conditionals. When CMake
 # runs, it will use the conditionals written to
 #
-#     $PETSC_DIR/$PETSC_ARCH/lib/petsc/conf/PETScConfig.cmake
+#     $PETSC_DIR/$PETSC_ARCH/lib/petsc/conf/PETScBuildInternal.cmake
 #
 # by BuildSystem after a successful configure.
 #
@@ -14,6 +14,7 @@
 # This script supports one option:
 #   --verbose : Show mismatches between makefiles and the filesystem
 
+from __future__ import print_function
 import os
 from collections import deque
 
@@ -154,6 +155,8 @@ def pkgsources(pkg, mistakes):
   allconditions = defaultdict(set)
   sources = defaultdict(deque)
   for root,dirs,files in os.walk(os.path.join('src',pkg)):
+    dirs.sort()
+    files.sort()
     conditions = allconditions[os.path.dirname(root)].copy()
     makefile = os.path.join(root,'makefile')
     if not os.path.exists(makefile):
@@ -181,7 +184,7 @@ def writeRoot(f):
   f.write(r'''cmake_minimum_required (VERSION 2.6.2)
 project (PETSc C)
 
-include (${PETSC_CMAKE_ARCH}/lib/petsc/conf/PETScConfig.cmake)
+include (${PETSC_CMAKE_ARCH}/lib/petsc/conf/PETScBuildInternal.cmake)
 
 if (PETSC_HAVE_FORTRAN)
   enable_language (Fortran)
