@@ -1,8 +1,7 @@
 #!/usr/bin/env python
+from __future__ import print_function
 import os, sys
-import commands
-# to load ~/.pythonrc.py before inserting correct BuildSystem to path
-import user
+
 extraLogs = []
 petsc_arch = ''
 
@@ -10,14 +9,11 @@ petsc_arch = ''
 if 'LC_LOCAL' in os.environ and os.environ['LC_LOCAL'] != '' and os.environ['LC_LOCAL'] != 'en_US' and os.environ['LC_LOCAL']!= 'en_US.UTF-8': os.environ['LC_LOCAL'] = 'en_US.UTF-8'
 if 'LANG' in os.environ and os.environ['LANG'] != '' and os.environ['LANG'] != 'en_US' and os.environ['LANG'] != 'en_US.UTF-8': os.environ['LANG'] = 'en_US.UTF-8'
 
-if not hasattr(sys, 'version_info') or not sys.version_info[0] == 2 or not sys.version_info[1] >= 4:
-  print '*** You must have Python2 version 2.4 or higher to run ./configure        *****'
-  print '*          Python is easy to install for end users or sys-admin.              *'
-  print '*                  http://www.python.org/download/                            *'
-  print '*                                                                             *'
-  print '*           You CANNOT configure PETSc without Python                         *'
-  print '*   http://www.mcs.anl.gov/petsc/documentation/installation.html     *'
-  print '*******************************************************************************'
+if not hasattr(sys, 'version_info') or not sys.version_info[0] == 2 or not sys.version_info[1] >= 6:
+  print('*******************************************************************************')
+  print('*       Python2 version 2.6 or higher is required to run ./configure          *')
+  print('*          Try: "python2.7 ./configure" or "python2.6 ./configure"            *')
+  print('*******************************************************************************')
   sys.exit(4)
 
 def check_for_option_mistakes(opts):
@@ -25,7 +21,7 @@ def check_for_option_mistakes(opts):
     name = opt.split('=')[0]
     if name.find('_') >= 0:
       exception = False
-      for exc in ['mkl_cpardiso', 'mkl_pardiso', 'superlu_dist', 'superlu_mt', 'PETSC_ARCH', 'PETSC_DIR', 'CXX_CXXFLAGS', 'LD_SHARED', 'CC_LINKER_FLAGS', 'CXX_LINKER_FLAGS', 'FC_LINKER_FLAGS', 'AR_FLAGS', 'C_VERSION', 'CXX_VERSION', 'FC_VERSION', 'size_t', 'MPI_Comm','MPI_Fint','int64_t']:
+      for exc in ['mkl_sparse', 'mkl_sparse_optimize', 'mkl_cpardiso', 'mkl_pardiso', 'superlu_dist', 'PETSC_ARCH', 'PETSC_DIR', 'CXX_CXXFLAGS', 'LD_SHARED', 'CC_LINKER_FLAGS', 'CXX_LINKER_FLAGS', 'FC_LINKER_FLAGS', 'AR_FLAGS', 'C_VERSION', 'CXX_VERSION', 'FC_VERSION', 'size_t', 'MPI_Comm','MPI_Fint','int64_t']:
         if name.find(exc) >= 0:
           exception = True
       if not exception:
@@ -36,9 +32,13 @@ def check_for_option_mistakes(opts):
         raise ValueError('The option '+opt+' should probably be '+opt.replace('ifneeded', '1'));
   return
 
+def check_for_unsupported_combinations(opts):
+  if '--with-precision=single' in opts and '--with-clanguage=cxx' in opts and '--with-scalar-type=complex' in opts:
+    sys.exit(ValueError('PETSc does not support single precision complex with C++ clanguage, run with --with-clanguage=c'))
+
 def check_for_option_changed(opts):
 # Document changes in command line options here.
-  optMap = [('with-64bit-indices','with-64-bit-indices'),('c-blas-lapack','f2cblaslapack'),('cholmod','suitesparse'),('umfpack','suitesparse'),('f-blas-lapack','fblaslapack')]
+  optMap = [('with-64bit-indices','with-64-bit-indices'),('c-blas-lapack','f2cblaslapack'),('cholmod','suitesparse'),('umfpack','suitesparse'),('f-blas-lapack','fblaslapack'),('with-cuda-arch','CUDAFLAGS=-arch'),('with-packages-dir','with-packages-download-dir'),('with-external-packages-dir','with-packages-build-dir'),('package-dirs','with-packages-search-path'),('search-dirs','with-executables-search-path')]
   for opt in opts[1:]:
     optname = opt.split('=')[0].strip('-')
     for oldname,newname in optMap:
@@ -67,24 +67,9 @@ def check_petsc_arch(opts):
 def chkenable():
   #Replace all 'enable-'/'disable-' with 'with-'=0/1/tail
   #enable-fortran is a special case, the resulting --with-fortran is ambiguous.
-  #Would it mean --with-fc= or --with-fortran-interfaces=?
+  #Would it mean --with-fc=
   for l in range(0,len(sys.argv)):
     name = sys.argv[l]
-    if name.find('enable-fortran') >= 0:
-      if name.find('=') == -1:
-        sys.argv[l] = name.replace('enable-fortran','with-fortran-interfaces')+'=1'
-      else:
-        head, tail = name.split('=', 1)
-        sys.argv[l] = head.replace('enable-fortran','with-fortran-interfaces')+'='+tail
-      continue
-    if name.find('disable-fortran') >= 0:
-      if name.find('=') == -1:
-        sys.argv[l] = name.replace('disable-fortran','with-fortran-interfaces')+'=0'
-      else:
-        head, tail = name.split('=', 1)
-        if tail == '1': tail = '0'
-        sys.argv[l] = head.replace('disable-fortran','with-fortran-interfaces')+'='+tail
-      continue
 
     if name.find('enable-cxx') >= 0:
       if name.find('=') == -1:
@@ -129,12 +114,33 @@ def chkenable():
         if tail == '1': tail = '0'
         sys.argv[l] = head.replace('without-','with-')+'='+tail
 
+def argsAddDownload(value,deps = [],options = []):
+  # Adds --download-value to args if the command line DOES NOT already has --with-value or --download-value in it
+  # this is to prevent introducing conflicting arguments to ones that already exist
+  for opt in sys.argv[1:]:
+    optname = opt.split('=')[0].strip('-')
+    if optname in ['download-'+value,'with-'+value,'with-'+value+'-dir','with-'+value+'-include','with-'+value+'-lib']: return
+  sys.argv.append('--download-'+value)
+  for i in deps:
+    argsAddDownload(i)
+  for i in options:
+    sys.argv.append(i)
 
 def chksynonyms():
   #replace common configure options with ones that PETSc BuildSystem recognizes
+  downloadxsdk = 0
+  downloadideas = 0
   for l in range(0,len(sys.argv)):
     name = sys.argv[l]
 
+    if name.find('download-xsdk=') >= 0 or name.endswith('download-xsdk'):
+      downloadxsdk = 1
+
+    if name.find('download-ideas=') >= 0 or name.endswith('download-ideas'):
+      downloadideas = 1
+
+    if name.find('with-blas-lapack') >= 0:
+      sys.argv[l] = name.replace('with-blas-lapack','with-blaslapack')
 
     if name.find('with-debug=') >= 0 or name.endswith('with-debug'):
       if name.find('=') == -1:
@@ -164,6 +170,27 @@ def chksynonyms():
       if tail.find('quad')>=0:
         sys.argv[l]='--with-precision=__float128'
 
+  if downloadideas:
+    downloadxsdk = 1
+    argsAddDownload('alquimia')
+    # mstk currently cannot build a shared library
+    argsAddDownload('mstk',[],['--download-mstk-shared=0'])
+    argsAddDownload('ascem-io')
+    argsAddDownload('unittestcpp')
+
+  if downloadxsdk:
+    # Common external libraries
+    argsAddDownload('pflotran')
+    argsAddDownload('hdf5',['zlib'])
+    argsAddDownload('netcdf')
+    argsAddDownload('metis')
+
+    argsAddDownload('superlu_dist',['parmetis'])
+
+    argsAddDownload('hypre')
+
+    argsAddDownload('trilinos',['boost','xsdktrilinos'],['--with-cxx-dialect=C++11'])
+
 
 def chkwinf90():
   for arg in sys.argv:
@@ -173,22 +200,22 @@ def chkwinf90():
 
 def chkdosfiles():
   # cygwin - but not a hg clone - so check one of files in bin dir
-  if "\r\n" in open(os.path.join('bin','petscmpiexec'),"rb").read():
-    print '==============================================================================='
-    print ' *** Scripts are in DOS mode. Was winzip used to extract petsc sources?    ****'
-    print ' *** Please restart with a fresh tarball and use "tar -xzf petsc.tar.gz"   ****'
-    print '==============================================================================='
+  if "\r\n" in open(os.path.join('lib','petsc','bin','petscmpiexec'),"rb").read():
+    print('===============================================================================')
+    print(' *** Scripts are in DOS mode. Was winzip used to extract petsc sources?    ****')
+    print(' *** Please restart with a fresh tarball and use "tar -xzf petsc.tar.gz"   ****')
+    print('===============================================================================')
     sys.exit(3)
   return
 
 def chkcygwinlink():
   if os.path.exists('/usr/bin/cygcheck.exe') and os.path.exists('/usr/bin/link.exe') and chkwinf90():
       if '--ignore-cygwin-link' in sys.argv: return 0
-      print '==============================================================================='
-      print ' *** Cygwin /usr/bin/link detected! Compiles with CVF/Intel f90 can break!  **'
-      print ' *** To workarround do: "mv /usr/bin/link.exe /usr/bin/link-cygwin.exe"     **'
-      print ' *** Or to ignore this check, use configure option: --ignore-cygwin-link    **'
-      print '==============================================================================='
+      print('===============================================================================')
+      print(' *** Cygwin /usr/bin/link detected! Compiles with CVF/Intel f90 can break!  **')
+      print(' *** To workarround do: "mv /usr/bin/link.exe /usr/bin/link-cygwin.exe"     **')
+      print(' *** Or to ignore this check, use configure option: --ignore-cygwin-link    **')
+      print('===============================================================================')
       sys.exit(3)
   return 0
 
@@ -196,19 +223,19 @@ def chkbrokencygwin():
   if os.path.exists('/usr/bin/cygcheck.exe'):
     buf = os.popen('/usr/bin/cygcheck.exe -c cygwin').read()
     if buf.find('1.5.11-1') > -1:
-      print '==============================================================================='
-      print ' *** cygwin-1.5.11-1 detected. ./configure fails with this version ***'
-      print ' *** Please upgrade to cygwin-1.5.12-1 or newer version. This can  ***'
-      print ' *** be done by running cygwin-setup, selecting "next" all the way.***'
-      print '==============================================================================='
+      print('===============================================================================')
+      print(' *** cygwin-1.5.11-1 detected. ./configure fails with this version ***')
+      print(' *** Please upgrade to cygwin-1.5.12-1 or newer version. This can  ***')
+      print(' *** be done by running cygwin-setup, selecting "next" all the way.***')
+      print('===============================================================================')
       sys.exit(3)
   return 0
 
 def chkusingwindowspython():
   if sys.platform == 'win32':
-    print '==============================================================================='
-    print ' *** Windows python detected. Please rerun ./configure with cygwin-python. ***'
-    print '==============================================================================='
+    print('===============================================================================')
+    print(' *** Windows python detected. Please rerun ./configure with cygwin-python. ***')
+    print('===============================================================================')
     sys.exit(3)
   return 0
 
@@ -294,30 +321,33 @@ def move_configure_log(framework):
 def print_final_timestamp(framework):
   import time
   framework.log.write(('='*80)+'\n')
-  framework.log.write('Finishing Configure Run at '+time.ctime(time.time())+'\n')
+  framework.log.write('Finishing configure run at '+time.strftime('%a, %d %b %Y %H:%M:%S %z')+'\n')
   framework.log.write(('='*80)+'\n')
   return
 
 def petsc_configure(configure_options):
-  try:
+  if 'PETSC_DIR' in os.environ:
     petscdir = os.environ['PETSC_DIR']
-    sys.path.append(os.path.join(petscdir,'bin'))
-    import petscnagupgrade
-    file     = os.path.join(petscdir,'.nagged')
-    if not petscnagupgrade.naggedtoday(file):
-      petscnagupgrade.currentversion(petscdir)
-  except:
-    pass
-  print '==============================================================================='
-  print '             Configuring PETSc to compile on your system                       '
-  print '==============================================================================='
+    if petscdir.find(' ') > -1:
+      raise RuntimeError('Your PETSC_DIR '+petscdir+' has spaces in it; this is not allowed.\n Change the directory with PETSc to not have spaces in it')
+    try:
+      sys.path.append(os.path.join(petscdir,'lib','petsc','bin'))
+      import petscnagupgrade
+      file     = os.path.join(petscdir,'.nagged')
+      if not petscnagupgrade.naggedtoday(file):
+        petscnagupgrade.currentversion(petscdir)
+    except:
+      pass
+  print('===============================================================================')
+  print('             Configuring PETSc to compile on your system                       ')
+  print('===============================================================================')
 
   try:
     # Command line arguments take precedence (but don't destroy argv[0])
     sys.argv = sys.argv[:1] + configure_options + sys.argv[1:]
     check_for_option_mistakes(sys.argv)
     check_for_option_changed(sys.argv)
-  except (TypeError, ValueError), e:
+  except (TypeError, ValueError) as e:
     emsg = str(e)
     if not emsg.endswith('\n'): emsg = emsg+'\n'
     msg ='*******************************************************************************\n'\
@@ -326,6 +356,7 @@ def petsc_configure(configure_options):
     +emsg+'*******************************************************************************\n'
     sys.exit(msg)
   # check PETSC_ARCH
+  check_for_unsupported_combinations(sys.argv)
   check_petsc_arch(sys.argv)
   check_broken_configure_log_links()
 
@@ -374,7 +405,7 @@ def petsc_configure(configure_options):
       # perhaps print an error about unable to shuffle logs?
       pass
     return 0
-  except (RuntimeError, config.base.ConfigureSetupError), e:
+  except (RuntimeError, config.base.ConfigureSetupError) as e:
     emsg = str(e)
     if not emsg.endswith('\n'): emsg = emsg+'\n'
     msg ='*******************************************************************************\n'\
@@ -382,7 +413,7 @@ def petsc_configure(configure_options):
     +'-------------------------------------------------------------------------------\n'  \
     +emsg+'*******************************************************************************\n'
     se = ''
-  except (TypeError, ValueError), e:
+  except (TypeError, ValueError) as e:
     emsg = str(e)
     if not emsg.endswith('\n'): emsg = emsg+'\n'
     msg ='*******************************************************************************\n'\
@@ -390,7 +421,7 @@ def petsc_configure(configure_options):
     +'-------------------------------------------------------------------------------\n'  \
     +emsg+'*******************************************************************************\n'
     se = ''
-  except ImportError, e :
+  except ImportError as e :
     emsg = str(e)
     if not emsg.endswith('\n'): emsg = emsg+'\n'
     msg ='*******************************************************************************\n'\
@@ -398,7 +429,7 @@ def petsc_configure(configure_options):
     +'-------------------------------------------------------------------------------\n'  \
     +emsg+'*******************************************************************************\n'
     se = ''
-  except OSError, e :
+  except OSError as e :
     emsg = str(e)
     if not emsg.endswith('\n'): emsg = emsg+'\n'
     msg ='*******************************************************************************\n'\
@@ -406,20 +437,22 @@ def petsc_configure(configure_options):
     +'-------------------------------------------------------------------------------\n'  \
     +emsg+'*******************************************************************************\n'
     se = ''
-  except SystemExit, e:
+  except SystemExit as e:
     if e.code is None or e.code == 0:
       return
+    if e.code is 10:
+      sys.exit(10)
     msg ='*******************************************************************************\n'\
     +'         CONFIGURATION FAILURE  (Please send configure.log to petsc-maint@mcs.anl.gov)\n' \
     +'*******************************************************************************\n'
     se  = str(e)
-  except Exception, e:
+  except Exception as e:
     msg ='*******************************************************************************\n'\
     +'        CONFIGURATION CRASH  (Please send configure.log to petsc-maint@mcs.anl.gov)\n' \
     +'*******************************************************************************\n'
     se  = str(e)
 
-  print msg
+  print(msg)
   if not framework is None:
     framework.logClear()
     if hasattr(framework, 'log'):
@@ -430,7 +463,7 @@ def petsc_configure(configure_options):
         if hasattr(framework,'compilerFixes'):
           framework.log.write('**** C specific Configure header '+framework.compilerFixes+' ****\n')
           framework.outputCHeader(framework.log)
-      except Exception, e:
+      except Exception as e:
         framework.log.write('Problem writing headers to log: '+str(e))
       import traceback
       try:
@@ -443,7 +476,7 @@ def petsc_configure(configure_options):
         pass
       sys.exit(1)
   else:
-    print se
+    print(se)
     import traceback
     traceback.print_tb(sys.exc_info()[2])
   if hasattr(framework,'log'): framework.log.close()
