@@ -1,14 +1,12 @@
 #!/usr/bin/env python
 
 configure_options = [
-  '--with-cc=gcc46',
-  '--with-cxx=g++46', # default g++ segfaults with src/ts/examples/tutorials/ex10.c
-  '--with-fc=gfortran',
-  'LIBS=-lexecinfo', # for some reason gcc46 requires this library
+
+  '--useThreads=0', # for some reason cmake hangs when invoked from configure on bsd?
+
   '--with-clanguage=cxx',
   '--with-scalar-type=complex',
 
-  '--useThreads=0', # for some reason cmake hangs when invoked from configure on bsd?
   #'-download-fblaslapack=1',
   '--download-mpich=1',
   '--download-cmake=1',
@@ -20,7 +18,9 @@ configure_options = [
   '--download-scalapack=1',
   '--download-mumps=1',
   '--download-hdf5',
-  # no with-cxx-dialect=C++11 support '--download-elemental=1',
+  '--with-zlib=1',
+  '--with-cxx-dialect=C++11',
+  '--download-elemental=1',
   #'--download-sundials=1',
   #'--download-hypre=1',
   #'--download-suitesparse=1',
