@@ -5,7 +5,7 @@ configure_options = [
   '--with-debugging=0',
 
   '--useThreads=0', # for some reason cmake hangs when invoked from configure on bsd?
-  #'-download-f-blas-lapack=1',
+  #'-download-fblaslapack=1',
   '--download-mpich=1',
   '--download-cmake=1',
   '--download-metis=1',
@@ -15,15 +15,19 @@ configure_options = [
   '--download-superlu_dist=1',
   '--download-scalapack=1',
   '--download-mumps=1',
+  '--with-cxx-dialect=C++11',
   '--download-elemental=1',
-  #'--download-hdf5',
+  '--download-hdf5',
+  '--download-zlib=1',
   '--download-sundials=1',
   '--download-hypre=1',
-  '--download-umfpack=1',
+  '--download-suitesparse=1',
+  '--download-make=1', # required by suitesparse
   '--download-chaco=1',
   '--download-spai=1',
   '--download-netcdf=1',
   '--download-moab=1',
+  '--download-saws',
   ]
 
 if __name__ == '__main__':
