@@ -17,12 +17,16 @@ configure_options = [
   '--download-scalapack=1',
   '--download-mumps=1',
   '--download-parms=1',
-  #'--download-elemental=1',
-  #'--with-cxx-dialect=C++11', hdf5 conflicts with C++11
+  '--download-ctetgen=1',
+  '--download-elemental=1',
+  '--with-cxx-dialect=C++11',
   '--download-spai=1',
   '--download-chaco=1',
   '--download-netcdf=1',
   '--download-hdf5=1',
+  '--download-adios=1',
+  '--with-zlib=1',
+  '--download-szlib=1',
   '--download-moab=1',
   '--download-petsc4py=1',
   '--download-mpi4py=1',
