@@ -13,13 +13,15 @@ configure_options = [
   '--download-superlu=1',
   '--download-superlu_dist=1',
   '--download-scalapack=1',
+  '--download-strumpack=1',
   '--download-mumps=1',
   '--download-elemental=1',
   '--with-cxx-dialect=C++11',
-  '--download-spai=1',
+  #'--download-spai=1', valgrind leaks here will probably not get fixed in the near future
   '--download-parms=1',
   '--download-moab=1',
-  '--download-chaco=1'
+  '--download-chaco=1',
+  '--download-revolve=1',
   ]
 
 if __name__ == '__main__':
