@@ -2,7 +2,7 @@
 
 configure_options = [
   # Autodetect MPICH & Intel MKL
-  # path set to $PETSC_DIR/bin/win32fe
+  # path set to $PETSC_DIR/lib/petsc/bin/win32fe
   # Note: comment out MPI_DISPLACEMENT_CURRENT in mpif.h for MPICH2 to work with Compaq F90
   '--with-cc=win32fe cl',
   '--with-fc=win32fe f90',
@@ -12,6 +12,7 @@ configure_options = [
   'CXXFLAGS=-DMPICH_SKIP_MPICXX -MT -GR -EHsc',
   '--with-mpiexec=mpiexec --localonly',
   'DATAFILESPATH=c:/cygwin/home/sbalay/datafiles',
+  '--with-shared-libraries=0',
   ]
 
 if __name__ == '__main__':
