@@ -1,6 +1,9 @@
 #!/usr/bin/env python
 
 configure_options = [
+  '--with-cc=cc',
+  '--with-cxx=CC',
+  '--with-fc=f90',
   #'--with-debugger=/bin/true',
   '--with-debugging=0',
   'FFLAGS=-ftrap=%none',
@@ -14,14 +17,14 @@ configure_options = [
   '--download-parmetis=1',
   '--download-triangle=1',
   '--download-superlu=1',
-  '--download-superlu_dist=1',
+  #'--download-superlu_dist=1', now requires C++11
   '--download-fblaslapack=1', # -lsunperf is insufficient for scalapack
   '--download-scalapack=1',
   '--download-mumps=1',
   '--download-parms=1',
   #'--download-elemental=1', breaks with solaris compilers
   #'--download-hdf5',
-  '--download-sundials=1',
+  #'--download-sundials=1', breaks when built via ssh - but not on terminal?
   '--download-hypre=1',
   '--download-suitesparse=1',
   '--download-chaco=1',
