@@ -1,10 +1,11 @@
 #!/usr/bin/env python
 
 configure_options = [
-  '--with-cc=gcc',
-  '--with-fc=gfortran', # http://hpc.sourceforge.net/
-  '--with-cxx=g++',
+  '--with-cc=clang',
+  '--with-cxx=clang++',
+  '--with-fc=gfortran', # http://brew.sh/
 
+  'CXXFLAGS=-Wall -Wwrite-strings -Wno-strict-aliasing -Wno-unknown-pragmas -fstack-protector -fvisibility=hidden -Wno-deprecated',
   '--with-clanguage=cxx',
   '--with-scalar-type=complex',
 
@@ -16,14 +17,16 @@ configure_options = [
   '--download-ptscotch=1',
   '--download-triangle=1',
   '--download-fftw=1',
-  #'--download-superlu=1',
-  #'--download-superlu_dist=1',
+  '--download-superlu=1',
+  '--download-superlu_dist=1',
   '--download-scalapack=1',
   '--download-mumps=1',
   '--download-hdf5',
+  '--download-zlib=1',
   '--download-petsc4py=1',
   '--download-mpi4py=1',
-  # no with-cxx-dialect=C++11 support '--download-elemental=1',
+  '--with-cxx-dialect=C++11',
+  '--download-elemental=1'
   #'--download-sundials=1',
   #'--download-hypre=1',
   #'--download-suitesparse=1',
