@@ -5,13 +5,12 @@ if __name__ == '__main__':
   sys.path.insert(0, os.path.abspath('config'))
   import configure
   configure_options = [
-    '--download-f-blas-lapack=1',
+    '--download-fblaslapack=1',
     '--download-mpich=1',
     '--with-cc=gcc',
     '--with-cxx=g++',
     '--with-clanguage=C++',
     '--with-scalar-type=real',
-    '--with-dynamic-loading=1',
     '--with-shared-libraries=1',
     '--with-python=1',
     '--with-debugging=0',
