@@ -25,6 +25,7 @@ typedef struct {
   KSP           smoothu;                       /* post smoother */
   Mat           interpolate;
   Mat           restrct;                       /* restrict is a reserved word in C99 and on Cray */
+  Mat           inject;                        /* Used for moving state if provided. */
   Vec           rscale;                        /* scaling of restriction matrix */
   PetscLogEvent eventsmoothsetup;              /* if logging times for each level */
   PetscLogEvent eventsmoothsolve;
@@ -36,11 +37,11 @@ typedef struct {
     This data structure is shared by all the levels.
 */
 typedef struct {
-  PCMGType  am;                               /* Multiplicative, additive or full */
-  PetscInt  cyclesperpcapply;                 /* Number of cycles to use in each PCApply(), multiplicative only*/
-  PetscInt  maxlevels;                        /* total number of levels allocated */
-  PetscInt  galerkin;                         /* use Galerkin process to compute coarser matrices, 0=no, 1=yes, 2=yes but computed externally */
-  PetscBool usedmfornumberoflevels;           /* sets the number of levels by getting this information out of the DM */
+  PCMGType         am;                        /* Multiplicative, additive or full */
+  PetscInt         cyclesperpcapply;          /* Number of cycles to use in each PCApply(), multiplicative only*/
+  PetscInt         maxlevels;                 /* total number of levels allocated */
+  PCMGGalerkinType galerkin;                  /* use Galerkin process to compute coarser matrices */
+  PetscBool        usedmfornumberoflevels;    /* sets the number of levels by getting this information out of the DM */
 
   PetscInt     nlevels;
   PC_MG_Levels **levels;
@@ -55,7 +56,7 @@ typedef struct {
 
 PETSC_INTERN PetscErrorCode PCSetUp_MG(PC);
 PETSC_INTERN PetscErrorCode PCDestroy_MG(PC);
-PETSC_INTERN PetscErrorCode PCSetFromOptions_MG(PetscOptions *PetscOptionsObject,PC);
+PETSC_INTERN PetscErrorCode PCSetFromOptions_MG(PetscOptionItems *PetscOptionsObject,PC);
 PETSC_INTERN PetscErrorCode PCView_MG(PC,PetscViewer);
 PETSC_DEPRECATED("Use PCMGResidualDefault()") PETSC_STATIC_INLINE PetscErrorCode PCMGResidual_Default(Mat A,Vec b,Vec x,Vec r) {
   return PCMGResidualDefault(A,b,x,r);
