@@ -1,4 +1,3 @@
-
 #ifndef __SNESIMPL_H
 #define __SNESIMPL_H
 
@@ -19,7 +18,7 @@ struct _SNESOps {
   PetscErrorCode (*setup)(SNES);                                /* routine to set up the nonlinear solver */
   PetscErrorCode (*solve)(SNES);                                /* actual nonlinear solver */
   PetscErrorCode (*view)(SNES,PetscViewer);
-  PetscErrorCode (*setfromoptions)(PetscOptions*,SNES);                       /* sets options from database */
+  PetscErrorCode (*setfromoptions)(PetscOptionItems*,SNES);                       /* sets options from database */
   PetscErrorCode (*destroy)(SNES);
   PetscErrorCode (*reset)(SNES);
   PetscErrorCode (*usercompute)(SNES,void**);
@@ -39,9 +38,9 @@ struct _p_SNES {
   PETSCHEADER(struct _SNESOps);
   DM        dm;
   PetscBool dmAuto;             /* SNES created currently used DM automatically */
-  SNES      pc;
-  PCSide    pcside;
-  PetscBool usespc;
+  SNES      npc;
+  PCSide    npcside;
+  PetscBool usesnpc;            /* type can use a nonlinear preconditioner */
 
   /*  ------------------------ User-provided stuff -------------------------------*/
   void  *user;                   /* user-defined context */
@@ -92,9 +91,11 @@ struct _p_SNES {
   PetscInt    linear_its;         /* total number of linear solver iterations */
   PetscReal   norm;               /* residual norm of current iterate */
   PetscReal   rtol;               /* relative tolerance */
+  PetscReal   divtol;             /* relative divergence tolerance */
   PetscReal   abstol;             /* absolute tolerance */
   PetscReal   stol;               /* step length tolerance*/
   PetscReal   deltatol;           /* trust region convergence tolerance */
+  PetscBool   forceiteration;     /* Force SNES to take at least one iteration regardless of the initial residual norm */
   PetscInt    lagpreconditioner;  /* SNESSetLagPreconditioner() */
   PetscInt    lagjacobian;        /* SNESSetLagJacobian() */
   PetscInt    jac_iter;           /* The present iteration of the Jacobian lagging */
@@ -117,6 +118,7 @@ struct _p_SNES {
 
   /* ------------------------- Miscellaneous Information ------------------------ */
 
+  PetscInt    setfromoptionscalled;
   PetscReal   *conv_hist;         /* If !0, stores function norm (or
                                     gradient norm) at each iteration */
   PetscInt    *conv_hist_its;     /* linear iterations for each Newton step */
@@ -141,6 +143,7 @@ struct _p_SNES {
 
   /* SNESConvergedDefault context: split it off into a separate var/struct to be passed as context to SNESConvergedDefault? */
   PetscReal   ttol;              /* rtol*initial_residual_norm */
+  PetscReal   rnorm0;            /* initial residual norm (used for divergence testing) */
 
   Vec         *vwork;            /* more work vectors for Jacobian approx */
   PetscInt    nvwork;
@@ -154,6 +157,11 @@ struct _p_SNES {
   PetscInt    ntruebounds;       /* number of non-infinite bounds set for VI box constraints */
   PetscBool   usersetbounds;     /* bounds have been set via SNESVISetVariableBounds(), rather than via computevariablebounds() callback. */
 
+  PetscBool   alwayscomputesfinalresidual;  /* Does SNESSolve_XXX always compute the value of the residual at the final
+                                             * solution and put it in vec_func?  Used inside SNESSolve_FAS to determine
+                                             * if the final residual must be computed before restricting or prolonging
+                                             * it. */
+
 };
 
 typedef struct _p_DMSNES *DMSNES;
@@ -199,7 +207,6 @@ PETSC_EXTERN PetscErrorCode DMGetDMSNES(DM,DMSNES*);
 PETSC_EXTERN PetscErrorCode DMSNESView(DMSNES,PetscViewer);
 PETSC_EXTERN PetscErrorCode DMSNESLoad(DMSNES,PetscViewer);
 PETSC_EXTERN PetscErrorCode DMGetDMSNESWrite(DM,DMSNES*);
-PETSC_EXTERN PetscErrorCode DMCopyDMSNES(DM,DM);
 
 
 /* Context for Eisenstat-Walker convergence criteria for KSP solvers */
@@ -217,8 +224,6 @@ typedef struct {
   PetscReal norm_first;          /* function norm from the beginning of the first iteration. */
 } SNESKSPEW;
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESLogConvergenceHistory"
 PETSC_STATIC_INLINE PetscErrorCode SNESLogConvergenceHistory(SNES snes,PetscReal res,PetscInt its)
 {
   PetscErrorCode ierr;
@@ -239,7 +244,7 @@ PETSC_INTERN PetscErrorCode SNESVICheckLocalMin_Private(SNES,Mat,Vec,Vec,PetscRe
 PETSC_INTERN PetscErrorCode SNESReset_VI(SNES);
 PETSC_INTERN PetscErrorCode SNESDestroy_VI(SNES);
 PETSC_INTERN PetscErrorCode SNESView_VI(SNES,PetscViewer);
-PETSC_INTERN PetscErrorCode SNESSetFromOptions_VI(PetscOptions*,SNES);
+PETSC_INTERN PetscErrorCode SNESSetFromOptions_VI(PetscOptionItems*,SNES);
 PETSC_INTERN PetscErrorCode SNESSetUp_VI(SNES);
 PETSC_EXTERN_TYPEDEF typedef PetscErrorCode (*SNESVIComputeVariableBoundsFunction)(SNES,Vec,Vec);
 PETSC_INTERN PetscErrorCode SNESVISetComputeVariableBounds_VI(SNES,SNESVIComputeVariableBoundsFunction);
@@ -247,37 +252,47 @@ PETSC_INTERN PetscErrorCode SNESVISetVariableBounds_VI(SNES,Vec,Vec);
 PETSC_INTERN PetscErrorCode SNESConvergedDefault_VI(SNES,PetscInt,PetscReal,PetscReal,PetscReal,SNESConvergedReason*,void*);
 
 PetscErrorCode SNESScaleStep_Private(SNES,Vec,PetscReal*,PetscReal*,PetscReal*,PetscReal*);
-PetscErrorCode DMSNESCheckFromOptions_Internal(SNES,DM,Vec,Vec,PetscErrorCode (**)(PetscInt,const PetscReal[],PetscInt,PetscScalar*,void*),void**);
+PETSC_EXTERN PetscErrorCode DMSNESCheckFromOptions_Internal(SNES,DM,Vec,PetscErrorCode (**)(PetscInt,PetscReal,const PetscReal[],PetscInt,PetscScalar*,void*),void**);
 
-PETSC_EXTERN PetscLogEvent SNES_Solve, SNES_LineSearch, SNES_FunctionEval, SNES_JacobianEval, SNES_NGSEval, SNES_NGSFuncEval, SNES_NPCSolve;
+PETSC_EXTERN PetscLogEvent SNES_Solve;
+PETSC_EXTERN PetscLogEvent SNES_LineSearch;
+PETSC_EXTERN PetscLogEvent SNES_FunctionEval;
+PETSC_EXTERN PetscLogEvent SNES_JacobianEval;
+PETSC_EXTERN PetscLogEvent SNES_NGSEval;
+PETSC_EXTERN PetscLogEvent SNES_NGSFuncEval;
+PETSC_EXTERN PetscLogEvent SNES_NPCSolve;
+PETSC_EXTERN PetscLogEvent SNES_ObjectiveEval;
 
-extern PetscBool SNEScite;
-extern const char SNESCitation[];
+PETSC_INTERN PetscBool SNEScite;
+PETSC_INTERN const char SNESCitation[];
 
 /*
     Either generate an error or mark as diverged when a real from a SNES function norm is Nan or Inf
 */
-#define SNESCheckFunctionNorm(snes,beta)           \
-  if (PetscIsInfOrNanReal(beta)) { \
+#define SNESCheckFunctionNorm(snes,beta) \
+  if (PetscIsInfOrNanReal(beta)) {\
     if (snes->errorifnotconverged) SETERRQ(PetscObjectComm((PetscObject)snes),PETSC_ERR_NOT_CONVERGED,"SNESSolve has not converged due to Nan or Inf norm");\
-  else {\
-    PetscBool domainerror;\
-    PetscErrorCode ierr = MPI_Allreduce((int*)&snes->domainerror,(int*)&domainerror,1,MPI_INT,MPI_MAX,PetscObjectComm((PetscObject)snes));CHKERRQ(ierr); \
-    if (domainerror)  snes->reason = SNES_DIVERGED_FUNCTION_DOMAIN;\
-    else              snes->reason = SNES_DIVERGED_FNORM_NAN;                  \
-    PetscFunctionReturn(0);\
-  }\
-}
-
+    else {\
+      PetscBool domainerror;\
+      PetscErrorCode ierr = MPIU_Allreduce((int*)&snes->domainerror,(int*)&domainerror,1,MPI_INT,MPI_MAX,PetscObjectComm((PetscObject)snes));CHKERRQ(ierr);\
+      if (domainerror)  snes->reason = SNES_DIVERGED_FUNCTION_DOMAIN;\
+      else              snes->reason = SNES_DIVERGED_FNORM_NAN;\
+      PetscFunctionReturn(0);\
+    }\
+  }
 
 #define SNESCheckKSPSolve(snes)\
   {\
     KSPConvergedReason kspreason; \
-    PetscErrorCode ierr = KSPGetConvergedReason(snes->ksp,&kspreason);CHKERRQ(ierr);\
+    PetscErrorCode ierr;                                                \
+    PetscInt lits;                                                      \
+    ierr = KSPGetIterationNumber(snes->ksp,&lits);CHKERRQ(ierr);        \
+    snes->linear_its += lits;                                           \
+    ierr = KSPGetConvergedReason(snes->ksp,&kspreason);CHKERRQ(ierr);\
     if (kspreason < 0) {\
       if (kspreason == KSP_DIVERGED_NANORINF) {\
         PetscBool domainerror;\
-        ierr = MPI_Allreduce((int*)&snes->domainerror,(int*)&domainerror,1,MPI_INT,MPI_MAX,PetscObjectComm((PetscObject)snes));CHKERRQ(ierr); \
+        ierr = MPIU_Allreduce((int*)&snes->domainerror,(int*)&domainerror,1,MPI_INT,MPI_MAX,PetscObjectComm((PetscObject)snes));CHKERRQ(ierr); \
         if (domainerror)  snes->reason = SNES_DIVERGED_FUNCTION_DOMAIN;\
         else              snes->reason = SNES_DIVERGED_LINEAR_SOLVE;                  \
         PetscFunctionReturn(0);\
