@@ -24,8 +24,8 @@ PETSC_EXTERN PetscErrorCode TSAdaptRegisterAll(void);
 PETSC_EXTERN PetscErrorCode TSRKRegisterAll(void);
 PETSC_EXTERN PetscErrorCode TSARKIMEXRegisterAll(void);
 PETSC_EXTERN PetscErrorCode TSRosWRegisterAll(void);
-PETSC_EXTERN PetscErrorCode TSGLRegisterAll(void);
-PETSC_EXTERN PetscErrorCode TSGLAdaptRegisterAll(void);
+PETSC_EXTERN PetscErrorCode TSGLLERegisterAll(void);
+PETSC_EXTERN PetscErrorCode TSGLLEAdaptRegisterAll(void);
 
 typedef struct _TSOps *TSOps;
 
@@ -49,7 +49,14 @@ struct _TSOps {
   PetscErrorCode (*adjointstep)(TS);
   PetscErrorCode (*adjointsetup)(TS);
   PetscErrorCode (*adjointintegral)(TS);
+  PetscErrorCode (*forwardsetup)(TS);
+  PetscErrorCode (*forwardstep)(TS);
   PetscErrorCode (*forwardintegral)(TS);
+  PetscErrorCode (*getsolutioncomponents)(TS,PetscInt*,Vec*);
+  PetscErrorCode (*getauxsolution)(TS,Vec*);
+  PetscErrorCode (*gettimeerror)(TS,PetscInt,Vec*);
+  PetscErrorCode (*settimeerror)(TS,Vec);
+  PetscErrorCode (*startingmethod) (TS);
 };
 
 /*
@@ -61,6 +68,7 @@ typedef struct _TSTrajectoryOps *TSTrajectoryOps;
 
 struct _TSTrajectoryOps {
   PetscErrorCode (*view)(TSTrajectory,PetscViewer);
+  PetscErrorCode (*reset)(TSTrajectory);
   PetscErrorCode (*destroy)(TSTrajectory);
   PetscErrorCode (*set)(TSTrajectory,TS,PetscInt,PetscReal,Vec);
   PetscErrorCode (*get)(TSTrajectory,TS,PetscInt,PetscReal*);
@@ -70,11 +78,27 @@ struct _TSTrajectoryOps {
 
 struct _p_TSTrajectory {
   PETSCHEADER(struct _TSTrajectoryOps);
-  PetscViewer monitor;
-  PetscInt    setupcalled;             /* true if setup has been called */
-  PetscInt    recomps;                 /* counter for recomputations in the adjoint run */
-  PetscInt    diskreads,diskwrites;    /* counters for disk checkpoint reads and writes */
-  void        *data;
+  PetscViewer    monitor;
+  PetscInt       setupcalled;             /* true if setup has been called */
+  PetscInt       recomps;                 /* counter for recomputations in the adjoint run */
+  PetscInt       diskreads,diskwrites;    /* counters for disk checkpoint reads and writes */
+  char           **names;                 /* the name of each variable; each process has only the local names */
+  PetscBool      keepfiles;               /* keep the files generated during the run after the run is complete */
+  char           *dirname,*filetemplate;  /* directory name and file name template for disk checkpoints */
+  char           *dirfiletemplate;        /* complete directory and file name template for disk checkpoints */
+  PetscErrorCode (*transform)(void*,Vec,Vec*);
+  PetscErrorCode (*transformdestroy)(void*);
+  void*          transformctx;
+  void           *data;
+};
+
+typedef struct _TS_RHSSplitLink *TS_RHSSplitLink;
+struct _TS_RHSSplitLink {
+  TS              ts;
+  char            *splitname;
+  IS              is;
+  TS_RHSSplitLink next;
+  PetscLogEvent   event;
 };
 
 struct _p_TS {
@@ -86,6 +110,7 @@ struct _p_TS {
   Vec            vec_sol; /* solution vector in first and second order equations */
   Vec            vec_dot; /* time derivative vector in second order equations */
   TSAdapt        adapt;
+  TSAdaptType    default_adapt_type;
   TSEvent        event;
 
   /* ---------------- User (or PETSc) Provided stuff ---------------------*/
@@ -101,6 +126,7 @@ struct _p_TS {
   PetscErrorCode (*prestep)(TS);
   PetscErrorCode (*prestage)(TS,PetscReal);
   PetscErrorCode (*poststage)(TS,PetscReal,PetscInt,Vec*);
+  PetscErrorCode (*postevaluate)(TS);
   PetscErrorCode (*poststep)(TS);
   PetscErrorCode (*functiondomainerror)(TS,PetscReal,Vec,PetscBool*);
 
@@ -111,6 +137,7 @@ struct _p_TS {
   PetscInt  numcost;                /* number of cost functions */
   Vec       vec_costintegral;
   PetscInt  adjointsetupcalled;
+  PetscInt  adjoint_steps;
   PetscInt  adjoint_max_steps;
   PetscBool adjoint_solve;          /* immediately call TSAdjointSolve() after TSSolve() is complete */
   PetscBool costintegralfwd;        /* cost integral is evaluated in the forward run if true */
@@ -126,6 +153,16 @@ struct _p_TS {
   PetscErrorCode (*drdyfunction)(TS,PetscReal,Vec,Vec*,void*);
   PetscErrorCode (*drdpfunction)(TS,PetscReal,Vec,Vec*,void*);
 
+  /* specific to forward sensitivity analysis */
+  Mat       mat_sensip;              /* matrix storing forward sensitivities */
+  Vec       *vecs_integral_sensip;   /* one vector for each integral */
+  PetscInt  num_parameters;
+  PetscInt  num_initialvalues;
+  void      *vecsrhsjacobianpctx;
+  PetscInt  forwardsetupcalled;
+  PetscBool forward_solve;
+  PetscErrorCode (*vecsrhsjacobianp)(TS,PetscReal,Vec,Vec*,void*);
+
   /* ---------------------- IMEX support ---------------------------------*/
   /* These extra slots are only used when the user provides both Implicit and RHS */
   Mat Arhs;     /* Right hand side matrix */
@@ -136,14 +173,14 @@ struct _p_TS {
    * The present use case is that TSComputeRHSFunctionLinear() evaluates the Jacobian once and we don't want it to be immeditely re-evaluated.
    */
   struct {
-    PetscReal time;             /* The time at which the matrices were last evaluated */
-    Vec X;                      /* Solution vector at which the Jacobian was last evaluated */
-    PetscObjectState Xstate;    /* State of the solution vector */
-    MatStructure mstructure;    /* The structure returned */
+    PetscReal        time;          /* The time at which the matrices were last evaluated */
+    PetscObjectId    Xid;           /* Unique ID of solution vector at which the Jacobian was last evaluated */
+    PetscObjectState Xstate;        /* State of the solution vector */
+    MatStructure     mstructure;    /* The structure returned */
     /* Flag to unshift Jacobian before calling the IJacobian or RHSJacobian functions.  This is useful
      * if the user would like to reuse (part of) the Jacobian from the last evaluation. */
-    PetscBool reuse;
-    PetscReal scale,shift;
+    PetscBool        reuse;
+    PetscReal        scale,shift;
   } rhsjacobian;
 
   struct {
@@ -152,6 +189,8 @@ struct _p_TS {
 
   /* --------------------Nonlinear Iteration------------------------------*/
   SNES     snes;
+  PetscBool usessnes;   /* Flag set by each TSType to indicate if the type actually uses a SNES;
+                           this works around the design flaw that a SNES is ALWAYS created with TS even when it is not needed.*/
   PetscInt ksp_its;                /* total number of linear solver iterations */
   PetscInt snes_its;               /* total number of nonlinear solver iterations */
   PetscInt num_snes_failures;
@@ -170,8 +209,7 @@ struct _p_TS {
 
   PetscBool steprollback;           /* flag to indicate that the step was rolled back */
   PetscBool steprestart;            /* flag to indicate that the timestepper has to discard any history and restart */
-  PetscInt  steps;                  /* steps taken so far in latest call to TSSolve() */
-  PetscInt  total_steps;            /* steps taken in all calls to TSSolve() since the TS was created or since TSSetUp() was called */
+  PetscInt  steps;                  /* steps taken so far in all successive calls to TSSolve() */
   PetscReal ptime;                  /* time at the start of the current step (stage time is internal if it exists) */
   PetscReal time_step;              /* current time increment */
   PetscReal ptime_prev;             /* time at the start of the previous step */
@@ -187,13 +225,19 @@ struct _p_TS {
   Vec       vatol,vrtol;            /* Relative and absolute tolerance in vector form */
   PetscReal cfltime,cfltime_local;
 
+  PetscBool testjacobian;
+  PetscBool testjacobiantranspose;
   /* ------------------- Default work-area management ------------------ */
   PetscInt nwork;
   Vec      *work;
+
+  /* ---------------------- RHS splitting support ---------------------------------*/
+  PetscInt        num_rhs_splits;
+  TS_RHSSplitLink tsrhssplit;
 };
 
 struct _TSAdaptOps {
-  PetscErrorCode (*choose)(TSAdapt,TS,PetscReal,PetscInt*,PetscReal*,PetscBool*,PetscReal*);
+  PetscErrorCode (*choose)(TSAdapt,TS,PetscReal,PetscInt*,PetscReal*,PetscBool*,PetscReal*,PetscReal*,PetscReal*);
   PetscErrorCode (*destroy)(TSAdapt);
   PetscErrorCode (*reset)(TSAdapt);
   PetscErrorCode (*view)(TSAdapt,PetscViewer);
@@ -214,10 +258,15 @@ struct _p_TSAdapt {
     PetscReal  ccfl[16];         /* stability limit relative to explicit Euler */
     PetscReal  cost[16];         /* relative measure of the amount of work required for each scheme */
   } candidates;
-  PetscReal   dt_min,dt_max;
-  PetscReal   scale_solve_failed; /* Scale step by this factor if solver (linear or nonlinear) fails. */
-  PetscViewer monitor;
+  PetscBool   always_accept;
+  PetscReal   safety;             /* safety factor relative to target error/stability goal */
+  PetscReal   reject_safety;      /* extra safety factor if the last step was rejected */
+  PetscReal   clip[2];            /* admissible time step decrease/increase factors */
+  PetscReal   dt_min,dt_max;      /* admissible minimum and maximum time step */
+  PetscReal   scale_solve_failed; /* scale step by this factor if solver (linear or nonlinear) fails. */
   NormType    wnormtype;
+  PetscViewer monitor;
+  PetscInt    timestepjustincreased;
 };
 
 typedef struct _p_DMTS *DMTS;
@@ -311,6 +360,7 @@ struct _n_TSEvent {
     PetscInt  **eventidx; /* Local indices of the events in the event list */
   } recorder;
   PetscInt  recsize; /* Size of recorder stack */
+  PetscInt  refct; /* reference count */
 };
 
 PETSC_EXTERN PetscErrorCode TSEventInitialize(TSEvent,TS,PetscReal,Vec);
@@ -318,7 +368,12 @@ PETSC_EXTERN PetscErrorCode TSEventDestroy(TSEvent*);
 PETSC_EXTERN PetscErrorCode TSEventHandler(TS);
 PETSC_EXTERN PetscErrorCode TSAdjointEventHandler(TS);
 
-PETSC_EXTERN PetscLogEvent TS_AdjointStep, TS_Step, TS_PseudoComputeTimeStep, TS_FunctionEval, TS_JacobianEval;
+PETSC_EXTERN PetscLogEvent TS_AdjointStep;
+PETSC_EXTERN PetscLogEvent TS_Step;
+PETSC_EXTERN PetscLogEvent TS_PseudoComputeTimeStep;
+PETSC_EXTERN PetscLogEvent TS_FunctionEval;
+PETSC_EXTERN PetscLogEvent TS_JacobianEval;
+PETSC_EXTERN PetscLogEvent TS_ForwardStep;
 
 typedef enum {TS_STEP_INCOMPLETE, /* vec_sol, ptime, etc point to beginning of step */
               TS_STEP_PENDING,    /* vec_sol advanced, but step has not been accepted yet */
@@ -327,6 +382,7 @@ typedef enum {TS_STEP_INCOMPLETE, /* vec_sol, ptime, etc point to beginning of s
 
 struct _n_TSMonitorLGCtx {
   PetscDrawLG    lg;
+  PetscBool      semilogy;
   PetscInt       howoften;  /* when > 0 uses step % howoften, when negative only final solution plotted */
   PetscInt       ksp_its,snes_its;
   char           **names;
@@ -343,6 +399,32 @@ struct _n_TSMonitorEnvelopeCtx {
   Vec max,min;
 };
 
-PETSC_EXTERN PetscLogEvent TSTrajectory_Set, TSTrajectory_Get, TSTrajectory_DiskWrite, TSTrajectory_DiskRead;
+/*
+    Checks if the user provide a TSSetIFunction() but an explicit method is called; generate an error in that case
+*/
+PETSC_STATIC_INLINE PetscErrorCode TSCheckImplicitTerm(TS ts)
+{
+  TSIFunction      ifunction;
+  DM               dm;
+  PetscErrorCode   ierr;
 
+  PetscFunctionBegin;
+  ierr = TSGetDM(ts,&dm);CHKERRQ(ierr);
+  ierr = DMTSGetIFunction(dm,&ifunction,NULL);CHKERRQ(ierr);
+  if (ifunction) SETERRQ(PetscObjectComm((PetscObject)ts),PETSC_ERR_ARG_INCOMP,"You are attempting to use an explicit ODE integrator but provided an implicit function definition with TSSetIFunction()");
+  PetscFunctionReturn(0);
+}
+
+PETSC_EXTERN PetscLogEvent TSTrajectory_Set;
+PETSC_EXTERN PetscLogEvent TSTrajectory_Get;
+PETSC_EXTERN PetscLogEvent TSTrajectory_DiskWrite;
+PETSC_EXTERN PetscLogEvent TSTrajectory_DiskRead;
+
+struct _n_TSMonitorDrawCtx {
+  PetscViewer   viewer;
+  Vec           initialsolution;
+  PetscBool     showinitial;
+  PetscInt      howoften;  /* when > 0 uses step % howoften, when negative only final solution plotted */
+  PetscBool     showtimestepandtime;
+};
 #endif
