@@ -15,10 +15,11 @@ PETSC_EXTERN PetscErrorCode KSPInitializePackage(void);
 
   Concepts: Krylov methods
 
-        Notes: When a direct solver is used but no Krylov solver is used the KSP object is still used by with a
+        Notes:
+    When a direct solver is used but no Krylov solver is used the KSP object is still used by with a
        KSPType of KSPPREONLY (meaning application of the preconditioner is only used as the linear solver).
 
-.seealso:  KSPCreate(), KSPSetType(), KSPType, SNES, TS, PC, KSP, KSPDestroy()
+.seealso:  KSPCreate(), KSPSetType(), KSPType, SNES, TS, PC, KSP, KSPDestroy(), KSPCG, KSPGMRES
 S*/
 typedef struct _p_KSP*     KSP;
 
@@ -36,10 +37,11 @@ typedef const char* KSPType;
 #define KSPGROPPCG    "groppcg"
 #define KSPPIPECG     "pipecg"
 #define KSPPIPECGRR   "pipecgrr"
+#define KSPPIPELCG     "pipelcg"
 #define   KSPCGNE       "cgne"
-#define   KSPNASH       "nash"
-#define   KSPSTCG       "stcg"
-#define   KSPGLTR       "gltr"
+#define   KSPCGNASH     "nash"
+#define   KSPCGSTCG     "stcg"
+#define   KSPCGGLTR     "gltr"
 #define KSPFCG        "fcg"
 #define KSPPIPEFCG    "pipefcg"
 #define KSPGMRES      "gmres"
@@ -54,6 +56,7 @@ typedef const char* KSPType;
 #define   KSPFBCGS      "fbcgs"
 #define   KSPFBCGSR     "fbcgsr"
 #define   KSPBCGSL      "bcgsl"
+#define   KSPPIPEBCGS   "pipebcgs"
 #define KSPCGS        "cgs"
 #define KSPTFQMR      "tfqmr"
 #define KSPCR         "cr"
@@ -70,14 +73,16 @@ typedef const char* KSPType;
 #define KSPPIPEGCR    "pipegcr"
 #define KSPTSIRM      "tsirm"
 #define KSPCGLS       "cgls"
+#define KSPFETIDP     "fetidp"
 
 /* Logging support */
 PETSC_EXTERN PetscClassId KSP_CLASSID;
+PETSC_EXTERN PetscClassId KSPGUESS_CLASSID;
 PETSC_EXTERN PetscClassId DMKSP_CLASSID;
 
-PETSC_EXTERN PetscErrorCode KSPCreate(MPI_Comm,KSP *);
+PETSC_EXTERN PetscErrorCode KSPCreate(MPI_Comm,KSP*);
 PETSC_EXTERN PetscErrorCode KSPSetType(KSP,KSPType);
-PETSC_EXTERN PetscErrorCode KSPGetType(KSP,KSPType *);
+PETSC_EXTERN PetscErrorCode KSPGetType(KSP,KSPType*);
 PETSC_EXTERN PetscErrorCode KSPSetUp(KSP);
 PETSC_EXTERN PetscErrorCode KSPSetUpOnBlocks(KSP);
 PETSC_EXTERN PetscErrorCode KSPSolve(KSP,Vec,Vec);
@@ -88,25 +93,24 @@ PETSC_EXTERN PetscErrorCode KSPSetReusePreconditioner(KSP,PetscBool);
 PETSC_EXTERN PetscErrorCode KSPSetSkipPCSetFromOptions(KSP,PetscBool);
 
 PETSC_EXTERN PetscFunctionList KSPList;
+PETSC_EXTERN PetscFunctionList KSPGuessList;
 PETSC_EXTERN PetscErrorCode KSPRegister(const char[],PetscErrorCode (*)(KSP));
 
 PETSC_EXTERN PetscErrorCode KSPSetPCSide(KSP,PCSide);
 PETSC_EXTERN PetscErrorCode KSPGetPCSide(KSP,PCSide*);
 PETSC_EXTERN PetscErrorCode KSPSetTolerances(KSP,PetscReal,PetscReal,PetscReal,PetscInt);
 PETSC_EXTERN PetscErrorCode KSPGetTolerances(KSP,PetscReal*,PetscReal*,PetscReal*,PetscInt*);
-PETSC_EXTERN PetscErrorCode KSPSetInitialGuessNonzero(KSP,PetscBool );
-PETSC_EXTERN PetscErrorCode KSPGetInitialGuessNonzero(KSP,PetscBool  *);
-PETSC_EXTERN PetscErrorCode KSPSetInitialGuessKnoll(KSP,PetscBool );
-PETSC_EXTERN PetscErrorCode KSPGetInitialGuessKnoll(KSP,PetscBool *);
-PETSC_EXTERN PetscErrorCode KSPSetErrorIfNotConverged(KSP,PetscBool );
-PETSC_EXTERN PetscErrorCode KSPGetErrorIfNotConverged(KSP,PetscBool  *);
-PETSC_EXTERN PetscErrorCode KSPSetComputeEigenvalues(KSP,PetscBool );
-PETSC_EXTERN PetscErrorCode KSPSetComputeRitz(KSP,PetscBool );
-PETSC_EXTERN PetscErrorCode KSPGetComputeEigenvalues(KSP,PetscBool *);
-PETSC_EXTERN PetscErrorCode KSPSetComputeSingularValues(KSP,PetscBool );
-PETSC_EXTERN PetscErrorCode KSPGetComputeSingularValues(KSP,PetscBool *);
-PETSC_EXTERN PetscErrorCode KSPGetRhs(KSP,Vec *);
-PETSC_EXTERN PetscErrorCode KSPGetSolution(KSP,Vec *);
+PETSC_EXTERN PetscErrorCode KSPSetInitialGuessNonzero(KSP,PetscBool);
+PETSC_EXTERN PetscErrorCode KSPGetInitialGuessNonzero(KSP,PetscBool*);
+PETSC_EXTERN PetscErrorCode KSPSetErrorIfNotConverged(KSP,PetscBool);
+PETSC_EXTERN PetscErrorCode KSPGetErrorIfNotConverged(KSP,PetscBool*);
+PETSC_EXTERN PetscErrorCode KSPSetComputeEigenvalues(KSP,PetscBool);
+PETSC_EXTERN PetscErrorCode KSPSetComputeRitz(KSP,PetscBool);
+PETSC_EXTERN PetscErrorCode KSPGetComputeEigenvalues(KSP,PetscBool*);
+PETSC_EXTERN PetscErrorCode KSPSetComputeSingularValues(KSP,PetscBool);
+PETSC_EXTERN PetscErrorCode KSPGetComputeSingularValues(KSP,PetscBool*);
+PETSC_EXTERN PetscErrorCode KSPGetRhs(KSP,Vec*);
+PETSC_EXTERN PetscErrorCode KSPGetSolution(KSP,Vec*);
 PETSC_EXTERN PetscErrorCode KSPGetResidualNorm(KSP,PetscReal*);
 PETSC_EXTERN PetscErrorCode KSPGetIterationNumber(KSP,PetscInt*);
 PETSC_EXTERN PetscErrorCode KSPGetTotalIterations(KSP,PetscInt*);
@@ -120,42 +124,43 @@ PETSC_EXTERN PetscErrorCode KSPSetPC(KSP,PC);
 PETSC_EXTERN PetscErrorCode KSPGetPC(KSP,PC*);
 
 PETSC_EXTERN PetscErrorCode KSPMonitor(KSP,PetscInt,PetscReal);
-PETSC_EXTERN PetscErrorCode KSPMonitorSet(KSP,PetscErrorCode (*)(KSP,PetscInt,PetscReal,void*),void *,PetscErrorCode (*)(void**));
+PETSC_EXTERN PetscErrorCode KSPMonitorSet(KSP,PetscErrorCode (*)(KSP,PetscInt,PetscReal,void*),void*,PetscErrorCode (*)(void**));
 PETSC_EXTERN PetscErrorCode KSPMonitorCancel(KSP);
-PETSC_EXTERN PetscErrorCode KSPGetMonitorContext(KSP,void **);
-PETSC_EXTERN PetscErrorCode KSPGetResidualHistory(KSP,PetscReal*[],PetscInt *);
+PETSC_EXTERN PetscErrorCode KSPGetMonitorContext(KSP,void**);
+PETSC_EXTERN PetscErrorCode KSPGetResidualHistory(KSP,PetscReal*[],PetscInt*);
 PETSC_EXTERN PetscErrorCode KSPSetResidualHistory(KSP,PetscReal[],PetscInt,PetscBool );
 
 PETSC_EXTERN PetscErrorCode KSPBuildSolutionDefault(KSP,Vec,Vec*);
-PETSC_EXTERN PetscErrorCode KSPBuildResidualDefault(KSP,Vec,Vec,Vec *);
+PETSC_EXTERN PetscErrorCode KSPBuildResidualDefault(KSP,Vec,Vec,Vec*);
 PETSC_EXTERN PetscErrorCode KSPDestroyDefault(KSP);
 PETSC_EXTERN PetscErrorCode KSPSetWorkVecs(KSP,PetscInt);
 
 PETSC_EXTERN PetscErrorCode PCKSPGetKSP(PC,KSP*);
+PETSC_EXTERN PetscErrorCode PCKSPSetKSP(PC,KSP);
 PETSC_EXTERN PetscErrorCode PCBJacobiGetSubKSP(PC,PetscInt*,PetscInt*,KSP*[]);
 PETSC_EXTERN PetscErrorCode PCASMGetSubKSP(PC,PetscInt*,PetscInt*,KSP*[]);
 PETSC_EXTERN PetscErrorCode PCGASMGetSubKSP(PC,PetscInt*,PetscInt*,KSP*[]);
 PETSC_EXTERN PetscErrorCode PCFieldSplitGetSubKSP(PC,PetscInt*,KSP*[]);
+PETSC_EXTERN PetscErrorCode PCFieldSplitSchurGetSubKSP(PC,PetscInt*,KSP*[]);
 PETSC_EXTERN PetscErrorCode PCMGGetSmoother(PC,PetscInt,KSP*);
 PETSC_EXTERN PetscErrorCode PCMGGetSmootherDown(PC,PetscInt,KSP*);
 PETSC_EXTERN PetscErrorCode PCMGGetSmootherUp(PC,PetscInt,KSP*);
 PETSC_EXTERN PetscErrorCode PCMGGetCoarseSolve(PC,KSP*);
-PETSC_EXTERN PetscErrorCode PCGalerkinGetKSP(PC,KSP *);
+PETSC_EXTERN PetscErrorCode PCGalerkinGetKSP(PC,KSP*);
 
-PETSC_EXTERN PetscErrorCode KSPBuildSolution(KSP,Vec,Vec *);
-PETSC_EXTERN PetscErrorCode KSPBuildResidual(KSP,Vec,Vec,Vec *);
+PETSC_EXTERN PetscErrorCode KSPBuildSolution(KSP,Vec,Vec*);
+PETSC_EXTERN PetscErrorCode KSPBuildResidual(KSP,Vec,Vec,Vec*);
 
 PETSC_EXTERN PetscErrorCode KSPRichardsonSetScale(KSP,PetscReal);
 PETSC_EXTERN PetscErrorCode KSPRichardsonSetSelfScale(KSP,PetscBool );
 PETSC_EXTERN PetscErrorCode KSPChebyshevSetEigenvalues(KSP,PetscReal,PetscReal);
 PETSC_EXTERN PetscErrorCode KSPChebyshevEstEigSet(KSP,PetscReal,PetscReal,PetscReal,PetscReal);
-PETSC_EXTERN PetscErrorCode KSPChebyshevEstEigSetUseRandom(KSP,PetscBool);
-PETSC_EXTERN PetscErrorCode KSPChebyshevEstEigSetRandom(KSP,PetscRandom);
+PETSC_EXTERN PetscErrorCode KSPChebyshevEstEigSetUseNoisy(KSP,PetscBool);
 PETSC_EXTERN PetscErrorCode KSPChebyshevEstEigGetKSP(KSP,KSP*);
 PETSC_EXTERN PetscErrorCode KSPComputeExtremeSingularValues(KSP,PetscReal*,PetscReal*);
-PETSC_EXTERN PetscErrorCode KSPComputeEigenvalues(KSP,PetscInt,PetscReal[],PetscReal[],PetscInt *);
+PETSC_EXTERN PetscErrorCode KSPComputeEigenvalues(KSP,PetscInt,PetscReal[],PetscReal[],PetscInt*);
 PETSC_EXTERN PetscErrorCode KSPComputeEigenvaluesExplicitly(KSP,PetscInt,PetscReal[],PetscReal[]);
-PETSC_EXTERN PetscErrorCode KSPComputeRitz(KSP,PetscBool,PetscBool,PetscInt *,Vec[],PetscReal[],PetscReal[]);
+PETSC_EXTERN PetscErrorCode KSPComputeRitz(KSP,PetscBool,PetscBool,PetscInt*,Vec[],PetscReal[],PetscReal[]);
 
 /*E
 
@@ -213,6 +218,10 @@ PETSC_EXTERN PetscErrorCode KSPGCRSetRestart(KSP,PetscInt);
 PETSC_EXTERN PetscErrorCode KSPGCRGetRestart(KSP,PetscInt*);
 PETSC_EXTERN PetscErrorCode KSPGCRSetModifyPC(KSP,PetscErrorCode (*)(KSP,PetscInt,PetscReal,void*),void*,PetscErrorCode(*)(void*));
 
+PETSC_EXTERN PetscErrorCode KSPFETIDPGetInnerBDDC(KSP,PC*);
+PETSC_EXTERN PetscErrorCode KSPFETIDPSetInnerBDDC(KSP,PC);
+PETSC_EXTERN PetscErrorCode KSPFETIDPGetInnerKSP(KSP,KSP*);
+PETSC_EXTERN PetscErrorCode KSPFETIDPSetPressureOperator(KSP,Mat);
 /*E
     KSPGMRESCGSRefinementType - How the classical (unmodified) Gram-Schmidt is performed.
 
@@ -283,22 +292,23 @@ PETSC_EXTERN PetscErrorCode KSPBCGSLSetEll(KSP,PetscInt);
 PETSC_EXTERN PetscErrorCode KSPBCGSLSetUsePseudoinverse(KSP,PetscBool);
 
 PETSC_EXTERN PetscErrorCode KSPSetFromOptions(KSP);
+PETSC_EXTERN PetscErrorCode KSPResetFromOptions(KSP);
 PETSC_EXTERN PetscErrorCode KSPAddOptionsChecker(PetscErrorCode (*)(KSP));
 
-PETSC_EXTERN PetscErrorCode KSPMonitorSingularValue(KSP,PetscInt,PetscReal,PetscViewerAndFormat *);
-PETSC_EXTERN PetscErrorCode KSPMonitorDefault(KSP,PetscInt,PetscReal,PetscViewerAndFormat *);
-PETSC_EXTERN PetscErrorCode KSPLSQRMonitorDefault(KSP,PetscInt,PetscReal,PetscViewerAndFormat *);
-PETSC_EXTERN PetscErrorCode KSPMonitorRange(KSP,PetscInt,PetscReal,PetscViewerAndFormat *);
+PETSC_EXTERN PetscErrorCode KSPMonitorSingularValue(KSP,PetscInt,PetscReal,PetscViewerAndFormat*);
+PETSC_EXTERN PetscErrorCode KSPMonitorDefault(KSP,PetscInt,PetscReal,PetscViewerAndFormat*);
+PETSC_EXTERN PetscErrorCode KSPLSQRMonitorDefault(KSP,PetscInt,PetscReal,PetscViewerAndFormat*);
+PETSC_EXTERN PetscErrorCode KSPMonitorRange(KSP,PetscInt,PetscReal,PetscViewerAndFormat*);
 PETSC_EXTERN PetscErrorCode KSPMonitorDynamicTolerance(KSP ksp,PetscInt its,PetscReal fnorm,void *dummy);
 PETSC_EXTERN PetscErrorCode KSPMonitorDynamicToleranceDestroy(void **dummy);
-PETSC_EXTERN PetscErrorCode KSPMonitorTrueResidualNorm(KSP,PetscInt,PetscReal,PetscViewerAndFormat *);
-PETSC_EXTERN PetscErrorCode KSPMonitorTrueResidualMaxNorm(KSP,PetscInt,PetscReal,PetscViewerAndFormat *);
-PETSC_EXTERN PetscErrorCode KSPMonitorDefaultShort(KSP,PetscInt,PetscReal,PetscViewerAndFormat *);
-PETSC_EXTERN PetscErrorCode KSPMonitorSolution(KSP,PetscInt,PetscReal,PetscViewerAndFormat *);
+PETSC_EXTERN PetscErrorCode KSPMonitorTrueResidualNorm(KSP,PetscInt,PetscReal,PetscViewerAndFormat*);
+PETSC_EXTERN PetscErrorCode KSPMonitorTrueResidualMaxNorm(KSP,PetscInt,PetscReal,PetscViewerAndFormat*);
+PETSC_EXTERN PetscErrorCode KSPMonitorDefaultShort(KSP,PetscInt,PetscReal,PetscViewerAndFormat*);
+PETSC_EXTERN PetscErrorCode KSPMonitorSolution(KSP,PetscInt,PetscReal,PetscViewerAndFormat*);
 PETSC_EXTERN PetscErrorCode KSPMonitorSAWs(KSP,PetscInt,PetscReal,void*);
 PETSC_EXTERN PetscErrorCode KSPMonitorSAWsCreate(KSP,void**);
 PETSC_EXTERN PetscErrorCode KSPMonitorSAWsDestroy(void**);
-PETSC_EXTERN PetscErrorCode KSPGMRESMonitorKrylov(KSP,PetscInt,PetscReal,void *);
+PETSC_EXTERN PetscErrorCode KSPGMRESMonitorKrylov(KSP,PetscInt,PetscReal,void*);
 PETSC_EXTERN PetscErrorCode KSPMonitorSetFromOptions(KSP,const char[],const char[],const char [],PetscErrorCode (*)(KSP,PetscInt,PetscReal,PetscViewerAndFormat*));
 
 PETSC_EXTERN PetscErrorCode KSPUnwindPreconditioner(KSP,Vec,Vec);
@@ -306,7 +316,7 @@ PETSC_EXTERN PetscErrorCode KSPInitialResidual(KSP,Vec,Vec,Vec,Vec,Vec);
 
 PETSC_EXTERN PetscErrorCode KSPSetOperators(KSP,Mat,Mat);
 PETSC_EXTERN PetscErrorCode KSPGetOperators(KSP,Mat*,Mat*);
-PETSC_EXTERN PetscErrorCode KSPGetOperatorsSet(KSP,PetscBool *,PetscBool *);
+PETSC_EXTERN PetscErrorCode KSPGetOperatorsSet(KSP,PetscBool*,PetscBool*);
 PETSC_EXTERN PetscErrorCode KSPSetOptionsPrefix(KSP,const char[]);
 PETSC_EXTERN PetscErrorCode KSPAppendOptionsPrefix(KSP,const char[]);
 PETSC_EXTERN PetscErrorCode KSPGetOptionsPrefix(KSP,const char*[]);
@@ -314,9 +324,9 @@ PETSC_EXTERN PetscErrorCode KSPSetTabLevel(KSP,PetscInt);
 PETSC_EXTERN PetscErrorCode KSPGetTabLevel(KSP,PetscInt*);
 
 PETSC_EXTERN PetscErrorCode KSPSetDiagonalScale(KSP,PetscBool );
-PETSC_EXTERN PetscErrorCode KSPGetDiagonalScale(KSP,PetscBool *);
+PETSC_EXTERN PetscErrorCode KSPGetDiagonalScale(KSP,PetscBool*);
 PETSC_EXTERN PetscErrorCode KSPSetDiagonalScaleFix(KSP,PetscBool );
-PETSC_EXTERN PetscErrorCode KSPGetDiagonalScaleFix(KSP,PetscBool *);
+PETSC_EXTERN PetscErrorCode KSPGetDiagonalScaleFix(KSP,PetscBool*);
 
 PETSC_EXTERN PetscErrorCode KSPView(KSP,PetscViewer);
 PETSC_EXTERN PetscErrorCode KSPLoad(KSP,PetscViewer);
@@ -326,8 +336,10 @@ PETSC_EXTERN PetscErrorCode KSPReasonViewFromOptions(KSP);
 
 #define KSP_FILE_CLASSID 1211223
 
-PETSC_EXTERN PetscErrorCode KSPLSQRSetStandardErrorVec(KSP,Vec);
+PETSC_EXTERN PetscErrorCode KSPLSQRSetExactMatNorm(KSP,PetscBool);
+PETSC_EXTERN PetscErrorCode KSPLSQRSetComputeStandardErrorVec(KSP,PetscBool);
 PETSC_EXTERN PetscErrorCode KSPLSQRGetStandardErrorVec(KSP,Vec*);
+PETSC_EXTERN PetscErrorCode KSPLSQRGetNorms(KSP,PetscReal*,PetscReal*);
 
 PETSC_EXTERN PetscErrorCode PCRedundantGetKSP(PC,KSP*);
 PETSC_EXTERN PetscErrorCode PCRedistributeGetKSP(PC,KSP*);
@@ -342,7 +354,8 @@ PETSC_EXTERN PetscErrorCode PCTelescopeGetKSP(PC,KSP*);
    Each solver only supports a subset of these and some may support different ones
    depending on left or right preconditioning, see KSPSetPCSide()
 
-   Notes: this must match petsc/finclude/petscksp.h
+   Notes:
+    this must match petsc/finclude/petscksp.h
 
 .seealso: KSPSolve(), KSPGetConvergedReason(), KSPSetNormType(),
           KSPSetConvergenceTest(), KSPSetPCSide()
@@ -401,9 +414,11 @@ PETSC_EXTERN PetscErrorCode KSPSetLagNorm(KSP,PetscBool);
 
    Level: beginner
 
-   Notes: See KSPGetConvergedReason() for explanation of each value
+   Notes:
+    See KSPGetConvergedReason() for explanation of each value
 
-   Developer notes: this must match petsc/finclude/petscksp.h
+   Developer Notes:
+    this must match petsc/finclude/petscksp.h
 
       The string versions of these are KSPConvergedReasons; if you change
       any of the values here also change them that array of names.
@@ -436,7 +451,7 @@ typedef enum {/* converged */
 PETSC_EXTERN const char *const*KSPConvergedReasons;
 
 /*MC
-     KSP_CONVERGED_RTOL - norm(r) <= rtol*norm(b)
+     KSP_CONVERGED_RTOL - norm(r) <= rtol*norm(b) or rtol*norm(b - A*x_0) if KSPConvergedDefaultSetUIRNorm() was called
 
    Level: beginner
 
@@ -444,7 +459,9 @@ PETSC_EXTERN const char *const*KSPConvergedReasons;
        for left preconditioning it is the 2-norm of the preconditioned residual, and the
        2-norm of the residual for right preconditioning
 
-.seealso:  KSPSolve(), KSPGetConvergedReason(), KSPConvergedReason, KSPSetTolerances()
+   See also KSP_CONVERGED_ATOL which may apply before this tolerance.
+
+.seealso:  KSP_CONVERGED_ATOL, KSP_DIVERGED_DTOL, KSPSolve(), KSPGetConvergedReason(), KSPConvergedReason, KSPSetTolerances()
 
 M*/
 
@@ -457,9 +474,9 @@ M*/
        for left preconditioning it is the 2-norm of the preconditioned residual, and the
        2-norm of the residual for right preconditioning
 
-   Level: beginner
+   See also KSP_CONVERGED_RTOL which may apply before this tolerance.
 
-.seealso:  KSPSolve(), KSPGetConvergedReason(), KSPConvergedReason, KSPSetTolerances()
+.seealso:  KSP_CONVERGED_RTOL, KSP_DIVERGED_DTOL, KSPSolve(), KSPGetConvergedReason(), KSPConvergedReason, KSPSetTolerances()
 
 M*/
 
@@ -474,7 +491,7 @@ M*/
 
    Level: beginner
 
-.seealso:  KSPSolve(), KSPGetConvergedReason(), KSPConvergedReason, KSPSetTolerances()
+.seealso:  KSP_CONVERGED_ATOL, KSP_DIVERGED_RTOL, KSPSolve(), KSPGetConvergedReason(), KSPConvergedReason, KSPSetTolerances()
 
 M*/
 
@@ -537,7 +554,8 @@ M*/
 
    Level: beginner
 
-     Notes: This can happen with the PCICC preconditioner, use -pc_factor_shift_positive_definite to force
+     Notes:
+    This can happen with the PCICC preconditioner, use -pc_factor_shift_positive_definite to force
   the PCICC preconditioner to generate a positive definite preconditioner
 
 .seealso:  KSPSolve(), KSPGetConvergedReason(), KSPConvergedReason, KSPSetTolerances()
@@ -551,7 +569,8 @@ M*/
 
    Level: beginner
 
-    Notes: Run with -ksp_error_if_not_converged to stop the program when the error is detected and print an error message with details.
+    Notes:
+    Run with -ksp_error_if_not_converged to stop the program when the error is detected and print an error message with details.
 
 
 .seealso:  KSPSolve(), KSPGetConvergedReason(), KSPConvergedReason, KSPSetTolerances()
@@ -568,16 +587,16 @@ M*/
 
 M*/
 
-PETSC_EXTERN PetscErrorCode KSPSetConvergenceTest(KSP,PetscErrorCode (*)(KSP,PetscInt,PetscReal,KSPConvergedReason*,void*),void *,PetscErrorCode (*)(void*));
-PETSC_EXTERN PetscErrorCode KSPGetConvergenceContext(KSP,void **);
-PETSC_EXTERN PetscErrorCode KSPConvergedDefault(KSP,PetscInt,PetscReal,KSPConvergedReason*,void *);
-PETSC_EXTERN PetscErrorCode KSPConvergedLSQR(KSP,PetscInt,PetscReal,KSPConvergedReason*,void *);
-PETSC_EXTERN PetscErrorCode KSPConvergedDefaultDestroy(void *);
-PETSC_EXTERN PetscErrorCode KSPConvergedDefaultCreate(void **);
+PETSC_EXTERN PetscErrorCode KSPSetConvergenceTest(KSP,PetscErrorCode (*)(KSP,PetscInt,PetscReal,KSPConvergedReason*,void*),void*,PetscErrorCode (*)(void*));
+PETSC_EXTERN PetscErrorCode KSPGetConvergenceContext(KSP,void**);
+PETSC_EXTERN PetscErrorCode KSPConvergedDefault(KSP,PetscInt,PetscReal,KSPConvergedReason*,void*);
+PETSC_EXTERN PetscErrorCode KSPLSQRConvergedDefault(KSP,PetscInt,PetscReal,KSPConvergedReason*,void*);
+PETSC_EXTERN PetscErrorCode KSPConvergedDefaultDestroy(void*);
+PETSC_EXTERN PetscErrorCode KSPConvergedDefaultCreate(void**);
 PETSC_EXTERN PetscErrorCode KSPConvergedDefaultSetUIRNorm(KSP);
 PETSC_EXTERN PetscErrorCode KSPConvergedDefaultSetUMIRNorm(KSP);
-PETSC_EXTERN PetscErrorCode KSPConvergedSkip(KSP,PetscInt,PetscReal,KSPConvergedReason*,void *);
-PETSC_EXTERN PetscErrorCode KSPGetConvergedReason(KSP,KSPConvergedReason *);
+PETSC_EXTERN PetscErrorCode KSPConvergedSkip(KSP,PetscInt,PetscReal,KSPConvergedReason*,void*);
+PETSC_EXTERN PetscErrorCode KSPGetConvergedReason(KSP,KSPConvergedReason*);
 
 PETSC_DEPRECATED("Use KSPConvergedDefault()") PETSC_STATIC_INLINE void KSPDefaultConverged(void) { /* never called */ }
 #define KSPDefaultConverged (KSPDefaultConverged, KSPConvergedDefault)
@@ -592,7 +611,7 @@ PETSC_DEPRECATED("Use KSPConvergedDefaultSetUMIRNorm()") PETSC_STATIC_INLINE voi
 PETSC_DEPRECATED("Use KSPConvergedSkip()") PETSC_STATIC_INLINE void KSPSkipConverged(void) { /* never called */ }
 #define KSPSkipConverged (KSPSkipConverged, KSPConvergedSkip)
 
-PETSC_EXTERN PetscErrorCode KSPComputeExplicitOperator(KSP,Mat *);
+PETSC_EXTERN PetscErrorCode KSPComputeExplicitOperator(KSP,Mat*);
 
 /*E
     KSPCGType - Determines what type of CG to use
@@ -607,19 +626,12 @@ PETSC_EXTERN const char *const KSPCGTypes[];
 PETSC_EXTERN PetscErrorCode KSPCGSetType(KSP,KSPCGType);
 PETSC_EXTERN PetscErrorCode KSPCGUseSingleReduction(KSP,PetscBool );
 
-PETSC_EXTERN PetscErrorCode KSPNASHSetRadius(KSP,PetscReal);
-PETSC_EXTERN PetscErrorCode KSPNASHGetNormD(KSP,PetscReal *);
-PETSC_EXTERN PetscErrorCode KSPNASHGetObjFcn(KSP,PetscReal *);
+PETSC_EXTERN PetscErrorCode KSPCGSetRadius(KSP,PetscReal);
+PETSC_EXTERN PetscErrorCode KSPCGGetNormD(KSP,PetscReal*);
+PETSC_EXTERN PetscErrorCode KSPCGGetObjFcn(KSP,PetscReal*);
 
-PETSC_EXTERN PetscErrorCode KSPSTCGSetRadius(KSP,PetscReal);
-PETSC_EXTERN PetscErrorCode KSPSTCGGetNormD(KSP,PetscReal *);
-PETSC_EXTERN PetscErrorCode KSPSTCGGetObjFcn(KSP,PetscReal *);
-
-PETSC_EXTERN PetscErrorCode KSPGLTRSetRadius(KSP,PetscReal);
-PETSC_EXTERN PetscErrorCode KSPGLTRGetNormD(KSP,PetscReal *);
-PETSC_EXTERN PetscErrorCode KSPGLTRGetObjFcn(KSP,PetscReal *);
-PETSC_EXTERN PetscErrorCode KSPGLTRGetMinEig(KSP,PetscReal *);
-PETSC_EXTERN PetscErrorCode KSPGLTRGetLambda(KSP,PetscReal *);
+PETSC_EXTERN PetscErrorCode KSPCGGLTRGetMinEig(KSP,PetscReal*);
+PETSC_EXTERN PetscErrorCode KSPCGGLTRGetLambda(KSP,PetscReal*);
 
 PETSC_EXTERN PetscErrorCode KSPPythonSetType(KSP,const char[]);
 
@@ -636,19 +648,42 @@ PETSC_EXTERN PetscErrorCode KSPMonitorLGRange(KSP,PetscInt,PetscReal,void*);
 PETSC_EXTERN PetscErrorCode PCShellSetPreSolve(PC,PetscErrorCode (*)(PC,KSP,Vec,Vec));
 PETSC_EXTERN PetscErrorCode PCShellSetPostSolve(PC,PetscErrorCode (*)(PC,KSP,Vec,Vec));
 
-/* see src/ksp/ksp/interface/iguess.c */
-typedef struct _p_KSPFischerGuess {PetscInt method,curl,maxl,refcnt;PetscBool  monitor;Mat mat; KSP ksp;}* KSPFischerGuess;
+/*S
+     KSPGuess - Abstract PETSc object that manages all initial guess methods in Krylov methods.
 
-PETSC_EXTERN PetscErrorCode KSPFischerGuessCreate(KSP,PetscInt,PetscInt,KSPFischerGuess*);
-PETSC_EXTERN PetscErrorCode KSPFischerGuessDestroy(KSPFischerGuess*);
-PETSC_EXTERN PetscErrorCode KSPFischerGuessReset(KSPFischerGuess);
-PETSC_EXTERN PetscErrorCode KSPFischerGuessUpdate(KSPFischerGuess,Vec);
-PETSC_EXTERN PetscErrorCode KSPFischerGuessFormGuess(KSPFischerGuess,Vec,Vec);
-PETSC_EXTERN PetscErrorCode KSPFischerGuessSetFromOptions(KSPFischerGuess);
+   Level: beginner
 
+  Concepts: Krylov methods
+
+.seealso:  KSPCreate(), KSPSetGuessType(), KSPGuessType
+S*/
+typedef struct _p_KSPGuess* KSPGuess;
+/*J
+    KSPGuessType - String with the name of a PETSc initial guess for Krylov methods.
+
+   Level: beginner
+
+.seealso: KSPGuess
+J*/
+typedef const char* KSPGuessType;
+#define KSPGUESSFISCHER "fischer"
+#define KSPGUESSPOD     "pod"
+PETSC_EXTERN PetscErrorCode KSPGuessRegister(const char[],PetscErrorCode (*)(KSPGuess));
+PETSC_EXTERN PetscErrorCode KSPSetGuess(KSP,KSPGuess);
+PETSC_EXTERN PetscErrorCode KSPGetGuess(KSP,KSPGuess*);
+PETSC_EXTERN PetscErrorCode KSPGuessView(KSPGuess,PetscViewer);
+PETSC_EXTERN PetscErrorCode KSPGuessDestroy(KSPGuess*);
+PETSC_EXTERN PetscErrorCode KSPGuessCreate(MPI_Comm,KSPGuess*);
+PETSC_EXTERN PetscErrorCode KSPGuessSetType(KSPGuess,KSPGuessType);
+PETSC_EXTERN PetscErrorCode KSPGuessGetType(KSPGuess,KSPGuessType*);
+PETSC_EXTERN PetscErrorCode KSPGuessSetUp(KSPGuess);
+PETSC_EXTERN PetscErrorCode KSPGuessUpdate(KSPGuess,Vec,Vec);
+PETSC_EXTERN PetscErrorCode KSPGuessFormGuess(KSPGuess,Vec,Vec);
+PETSC_EXTERN PetscErrorCode KSPGuessSetFromOptions(KSPGuess);
+PETSC_EXTERN PetscErrorCode KSPGuessFischerSetModel(KSPGuess,PetscInt,PetscInt);
 PETSC_EXTERN PetscErrorCode KSPSetUseFischerGuess(KSP,PetscInt,PetscInt);
-PETSC_EXTERN PetscErrorCode KSPSetFischerGuess(KSP,KSPFischerGuess);
-PETSC_EXTERN PetscErrorCode KSPGetFischerGuess(KSP,KSPFischerGuess*);
+PETSC_EXTERN PetscErrorCode KSPSetInitialGuessKnoll(KSP,PetscBool);
+PETSC_EXTERN PetscErrorCode KSPGetInitialGuessKnoll(KSP,PetscBool*);
 
 /*E
     MatSchurComplementAinvType - Determines how to approximate the inverse of the (0,0) block in Schur complement preconditioning matrix assembly routines
@@ -657,7 +692,7 @@ PETSC_EXTERN PetscErrorCode KSPGetFischerGuess(KSP,KSPFischerGuess*);
 
 .seealso: MatSchurComplementGetAinvType(), MatSchurComplementSetAinvType(), MatSchurComplementGetPmat(), MatGetSchurComplement(), MatCreateSchurComplementPmat()
 E*/
-typedef enum {MAT_SCHUR_COMPLEMENT_AINV_DIAG, MAT_SCHUR_COMPLEMENT_AINV_LUMP} MatSchurComplementAinvType;
+typedef enum {MAT_SCHUR_COMPLEMENT_AINV_DIAG, MAT_SCHUR_COMPLEMENT_AINV_LUMP, MAT_SCHUR_COMPLEMENT_AINV_BLOCK_DIAG} MatSchurComplementAinvType;
 PETSC_EXTERN const char *const MatSchurComplementAinvTypes[];
 
 PETSC_EXTERN PetscErrorCode MatCreateSchurComplement(Mat,Mat,Mat,Mat,Mat,Mat*);
@@ -670,15 +705,44 @@ PETSC_EXTERN PetscErrorCode MatSchurComplementSetAinvType(Mat,MatSchurComplement
 PETSC_EXTERN PetscErrorCode MatSchurComplementGetAinvType(Mat,MatSchurComplementAinvType*);
 PETSC_EXTERN PetscErrorCode MatSchurComplementGetPmat(Mat,MatReuse,Mat*);
 PETSC_EXTERN PetscErrorCode MatSchurComplementComputeExplicitOperator(Mat,Mat*);
-PETSC_EXTERN PetscErrorCode MatGetSchurComplement(Mat,IS,IS,IS,IS,MatReuse,Mat *,MatSchurComplementAinvType,MatReuse,Mat *);
+PETSC_EXTERN PetscErrorCode MatGetSchurComplement(Mat,IS,IS,IS,IS,MatReuse,Mat*,MatSchurComplementAinvType,MatReuse,Mat*);
 PETSC_EXTERN PetscErrorCode MatCreateSchurComplementPmat(Mat,Mat,Mat,Mat,MatSchurComplementAinvType,MatReuse,Mat*);
 
+PETSC_EXTERN PetscErrorCode MatCreateLMVMDFP(MPI_Comm,PetscInt,PetscInt,Mat*);
+PETSC_EXTERN PetscErrorCode MatCreateLMVMBFGS(MPI_Comm,PetscInt,PetscInt,Mat*);
+PETSC_EXTERN PetscErrorCode MatCreateLMVMSR1(MPI_Comm,PetscInt,PetscInt,Mat*);
+PETSC_EXTERN PetscErrorCode MatCreateLMVMBrdn(MPI_Comm,PetscInt,PetscInt,Mat*);
+PETSC_EXTERN PetscErrorCode MatCreateLMVMBadBrdn(MPI_Comm,PetscInt,PetscInt,Mat*);
+PETSC_EXTERN PetscErrorCode MatCreateLMVMSymBrdn(MPI_Comm,PetscInt,PetscInt,Mat*);
+PETSC_EXTERN PetscErrorCode MatCreateLMVMSymBadBrdn(MPI_Comm,PetscInt,PetscInt,Mat*);
+PETSC_EXTERN PetscErrorCode MatCreateLMVMDiagBrdn(MPI_Comm,PetscInt,PetscInt,Mat*);
+
+PETSC_EXTERN PetscErrorCode MatLMVMUpdate(Mat, Vec, Vec);
+PETSC_EXTERN PetscErrorCode MatLMVMIsAllocated(Mat, PetscBool*);
+PETSC_EXTERN PetscErrorCode MatLMVMAllocate(Mat, Vec, Vec);
+PETSC_EXTERN PetscErrorCode MatLMVMReset(Mat, PetscBool);
+PETSC_EXTERN PetscErrorCode MatLMVMResetShift(Mat);
+PETSC_EXTERN PetscErrorCode MatLMVMClearJ0(Mat);
+PETSC_EXTERN PetscErrorCode MatLMVMSetJ0(Mat, Mat);
+PETSC_EXTERN PetscErrorCode MatLMVMSetJ0Scale(Mat, PetscReal);
+PETSC_EXTERN PetscErrorCode MatLMVMSetJ0Diag(Mat, Vec);
+PETSC_EXTERN PetscErrorCode MatLMVMSetJ0PC(Mat, PC);
+PETSC_EXTERN PetscErrorCode MatLMVMSetJ0KSP(Mat, KSP);
+PETSC_EXTERN PetscErrorCode MatLMVMApplyJ0Fwd(Mat, Vec, Vec);
+PETSC_EXTERN PetscErrorCode MatLMVMApplyJ0Inv(Mat, Vec, Vec);
+PETSC_EXTERN PetscErrorCode MatLMVMGetJ0(Mat, Mat*);
+PETSC_EXTERN PetscErrorCode MatLMVMGetJ0PC(Mat, PC*);
+PETSC_EXTERN PetscErrorCode MatLMVMGetJ0KSP(Mat, KSP*);
+PETSC_EXTERN PetscErrorCode MatLMVMGetUpdateCount(Mat, PetscInt*);
+PETSC_EXTERN PetscErrorCode MatLMVMGetRejectCount(Mat, PetscInt*);
+PETSC_EXTERN PetscErrorCode MatSymBrdnSetDelta(Mat, PetscScalar);
+
 PETSC_EXTERN PetscErrorCode KSPSetDM(KSP,DM);
 PETSC_EXTERN PetscErrorCode KSPSetDMActive(KSP,PetscBool );
 PETSC_EXTERN PetscErrorCode KSPGetDM(KSP,DM*);
 PETSC_EXTERN PetscErrorCode KSPSetApplicationContext(KSP,void*);
 PETSC_EXTERN PetscErrorCode KSPGetApplicationContext(KSP,void*);
-PETSC_EXTERN PetscErrorCode KSPSetComputeRHS(KSP,PetscErrorCode (*func)(KSP,Vec,void*),void *);
+PETSC_EXTERN PetscErrorCode KSPSetComputeRHS(KSP,PetscErrorCode (*func)(KSP,Vec,void*),void*);
 PETSC_EXTERN PetscErrorCode KSPSetComputeOperators(KSP,PetscErrorCode(*)(KSP,Mat,Mat,void*),void*);
 PETSC_EXTERN PetscErrorCode KSPSetComputeInitialGuess(KSP,PetscErrorCode(*)(KSP,Vec,void*),void*);
 PETSC_EXTERN PetscErrorCode DMKSPSetComputeOperators(DM,PetscErrorCode(*)(KSP,Mat,Mat,void*),void*);
@@ -689,9 +753,9 @@ PETSC_EXTERN PetscErrorCode DMKSPSetComputeInitialGuess(DM,PetscErrorCode(*)(KSP
 PETSC_EXTERN PetscErrorCode DMKSPGetComputeInitialGuess(DM,PetscErrorCode(**)(KSP,Vec,void*),void*);
 
 PETSC_EXTERN PetscErrorCode DMGlobalToLocalSolve(DM,Vec,Vec);
-PETSC_EXTERN PetscErrorCode DMProjectField(DM, Vec,
+PETSC_EXTERN PetscErrorCode DMProjectField(DM, PetscReal, Vec,
                                            void (**)(PetscInt, PetscInt, PetscInt,
                                                      const PetscInt[], const PetscInt[], const PetscScalar[], const PetscScalar[], const PetscScalar[],
                                                      const PetscInt[], const PetscInt[], const PetscScalar[], const PetscScalar[], const PetscScalar[],
-                                                     PetscReal, const PetscReal [], PetscScalar []), InsertMode, Vec);
+                                                     PetscReal, const PetscReal [], PetscInt, const PetscScalar[], PetscScalar []), InsertMode, Vec);
 #endif
