@@ -24,31 +24,33 @@
 #if defined(PETSC_USE_REAL_SINGLE)
 #define MPIU_REAL   MPI_FLOAT
 typedef float PetscReal;
-#define PetscSqrtReal(a)    sqrt(a)
-#define PetscExpReal(a)     exp(a)
-#define PetscLogReal(a)     log(a)
-#define PetscLog10Real(a)   log10(a)
+#define PetscRoundReal(a)   roundf(a)
+#define PetscSqrtReal(a)    sqrtf(a)
+#define PetscExpReal(a)     expf(a)
+#define PetscLogReal(a)     logf(a)
+#define PetscLog10Real(a)   log10f(a)
 #ifdef PETSC_HAVE_LOG2
-#define PetscLog2Real(a)    log2(a)
+#define PetscLog2Real(a)    log2f(a)
 #endif
-#define PetscSinReal(a)     sin(a)
-#define PetscCosReal(a)     cos(a)
-#define PetscTanReal(a)     tan(a)
-#define PetscAsinReal(a)    asin(a)
-#define PetscAcosReal(a)    acos(a)
-#define PetscAtanReal(a)    atan(a)
-#define PetscAtan2Real(a,b) atan2(a,b)
-#define PetscSinhReal(a)    sinh(a)
-#define PetscCoshReal(a)    cosh(a)
-#define PetscTanhReal(a)    tanh(a)
-#define PetscPowReal(a,b)   pow(a,b)
-#define PetscCeilReal(a)    ceil(a)
-#define PetscFloorReal(a)   floor(a)
-#define PetscFmodReal(a,b)  fmod(a,b)
+#define PetscSinReal(a)     sinf(a)
+#define PetscCosReal(a)     cosf(a)
+#define PetscTanReal(a)     tanf(a)
+#define PetscAsinReal(a)    asinf(a)
+#define PetscAcosReal(a)    acosf(a)
+#define PetscAtanReal(a)    atanf(a)
+#define PetscAtan2Real(a,b) atan2f(a,b)
+#define PetscSinhReal(a)    sinhf(a)
+#define PetscCoshReal(a)    coshf(a)
+#define PetscTanhReal(a)    tanhf(a)
+#define PetscPowReal(a,b)   powf(a,b)
+#define PetscCeilReal(a)    ceilf(a)
+#define PetscFloorReal(a)   floorf(a)
+#define PetscFmodReal(a,b)  fmodf(a,b)
 #define PetscTGamma(a)      tgammaf(a)
 #elif defined(PETSC_USE_REAL_DOUBLE)
 #define MPIU_REAL   MPI_DOUBLE
 typedef double PetscReal;
+#define PetscRoundReal(a)   round(a)
 #define PetscSqrtReal(a)    sqrt(a)
 #define PetscExpReal(a)     exp(a)
 #define PetscLogReal(a)     log(a)
@@ -82,6 +84,7 @@ extern "C" {
 PETSC_EXTERN MPI_Datatype MPIU___FLOAT128 PetscAttrMPITypeTag(__float128);
 #define MPIU_REAL MPIU___FLOAT128
 typedef __float128 PetscReal;
+#define PetscRoundReal(a)   roundq(a)
 #define PetscSqrtReal(a)    sqrtq(a)
 #define PetscExpReal(a)     expq(a)
 #define PetscLogReal(a)     logq(a)
@@ -104,6 +107,33 @@ typedef __float128 PetscReal;
 #define PetscFloorReal(a)   floorq(a)
 #define PetscFmodReal(a,b)  fmodq(a,b)
 #define PetscTGamma(a)      tgammaq(a)
+#elif defined(PETSC_USE_REAL___FP16)
+PETSC_EXTERN MPI_Datatype MPIU___FP16 PetscAttrMPITypeTag(__fp16);
+#define MPIU_REAL MPIU___FP16
+typedef __fp16 PetscReal;
+#define PetscRoundReal(a)   roundf(a)
+#define PetscSqrtReal(a)    sqrtf(a)
+#define PetscExpReal(a)     expf(a)
+#define PetscLogReal(a)     logf(a)
+#define PetscLog10Real(a)   log10f(a)
+#ifdef PETSC_HAVE_LOG2
+#define PetscLog2Real(a)    log2f(a)
+#endif
+#define PetscSinReal(a)     sinf(a)
+#define PetscCosReal(a)     cosf(a)
+#define PetscTanReal(a)     tanf(a)
+#define PetscAsinReal(a)    asinf(a)
+#define PetscAcosReal(a)    acosf(a)
+#define PetscAtanReal(a)    atanf(a)
+#define PetscAtan2Real(a,b) atan2f(a,b)
+#define PetscSinhReal(a)    sinhf(a)
+#define PetscCoshReal(a)    coshf(a)
+#define PetscTanhReal(a)    tanhf(a)
+#define PetscPowReal(a,b)   powf(a,b)
+#define PetscCeilReal(a)    ceilf(a)
+#define PetscFloorReal(a)   floorf(a)
+#define PetscFmodReal(a,b)  fmodf(a,b)
+#define PetscTGamma(a)      tgammaf(a)
 #endif /* PETSC_USE_REAL_* */
 
 /*
@@ -113,10 +143,7 @@ typedef __float128 PetscReal;
 #if !defined(PETSC_SKIP_COMPLEX)
 #define PETSC_HAVE_COMPLEX 1
 /* C++ support of complex number */
-#if defined(PETSC_HAVE_CUSP)
-#define complexlib cusp
-#include <cusp/complex.h>
-#elif defined(PETSC_HAVE_VECCUDA) && __CUDACC_VER_MAJOR__ > 6
+#if defined(PETSC_HAVE_VECCUDA) && __CUDACC_VER_MAJOR__ > 6
 /* complex headers in thrust only available in CUDA 7.0 and above */
 #define complexlib thrust
 #include <thrust/complex.h>
@@ -188,12 +215,12 @@ PETSC_EXTERN MPI_Datatype MPIU___COMPLEX128;
 #endif  /* PETSC_USE_REAL_ */
 #endif  /* ! PETSC_SKIP_COMPLEX */
 
-#elif !defined(__cplusplus) && defined(PETSC_HAVE_C99_COMPLEX)
+#elif defined(PETSC_HAVE_C99_COMPLEX) && !defined(PETSC_USE_REAL___FP16)
 #if !defined(PETSC_SKIP_COMPLEX)
 #define PETSC_HAVE_COMPLEX 1
 #include <complex.h>
 
-#if defined(PETSC_USE_REAL_SINGLE)
+#if defined(PETSC_USE_REAL_SINGLE) || defined(PETSC_USE_REAL___FP16)
 typedef float _Complex PetscComplex;
 
 #define PetscRealPartComplex(a)      crealf(a)
@@ -265,7 +292,7 @@ PETSC_EXTERN MPI_Datatype MPIU___COMPLEX128 PetscAttrMPITypeTag(__complex128);
 #define MPIU_C_DOUBLE_COMPLEX MPI_C_DOUBLE_COMPLEX
 #define MPIU_C_COMPLEX MPI_C_COMPLEX
 #else
-# if defined(__cplusplus) && defined(PETSC_HAVE_CXX_COMPLEX)
+# if defined(__cplusplus) && defined(PETSC_HAVE_CXX_COMPLEX) && !defined(PETSC_USE_REAL___FLOAT128)
   typedef complexlib::complex<double> petsc_mpiu_c_double_complex;
   typedef complexlib::complex<float> petsc_mpiu_c_complex;
 # elif !defined(__cplusplus) && defined(PETSC_HAVE_C99_COMPLEX)
@@ -281,7 +308,7 @@ PETSC_EXTERN MPI_Datatype MPIU_C_COMPLEX PetscAttrMPITypeTagLayoutCompatible(pet
 #endif /* PETSC_HAVE_COMPLEX */
 
 #if defined(PETSC_HAVE_COMPLEX)
-#  if defined(PETSC_USE_REAL_SINGLE)
+#  if defined(PETSC_USE_REAL_SINGLE) || defined(PETSC_USE_REAL___FP16)
 #    define MPIU_COMPLEX MPIU_C_COMPLEX
 #  elif defined(PETSC_USE_REAL_DOUBLE)
 #    define MPIU_COMPLEX MPIU_C_DOUBLE_COMPLEX
@@ -292,8 +319,48 @@ PETSC_EXTERN MPI_Datatype MPIU_C_COMPLEX PetscAttrMPITypeTagLayoutCompatible(pet
 
 #if (defined(PETSC_USE_COMPLEX) && !defined(PETSC_SKIP_COMPLEX))
 typedef PetscComplex PetscScalar;
+
+/*MC
+   PetscRealPart - Returns the real part of a PetscScalar
+
+   Synopsis:
+   #include <petscmath.h>
+   PetscScalar PetscRealPart(PetscScalar v)
+
+   Not Collective
+
+   Input Parameter:
+.  v - value to find the real part of
+
+   Level: beginner
+
+.seealso: PetscScalar, PetscImaginaryPart(), PetscMax(), PetscClipInterval(), PetscAbsInt(), PetscAbsReal(), PetscSqr()
+
+M*/
 #define PetscRealPart(a)      PetscRealPartComplex(a)
+
+/*MC
+   PetscImaginaryPart - Returns the imaginary part of a PetscScalar
+
+   Synopsis:
+   #include <petscmath.h>
+   PetscScalar PetscImaginaryPart(PetscScalar v)
+
+   Not Collective
+
+   Input Parameter:
+.  v - value to find the imaginary part of
+
+   Level: beginner
+
+   Notes:
+       If PETSc was configured for real numbers then this always returns the value 0
+
+.seealso: PetscScalar, PetscRealPart(), PetscMax(), PetscClipInterval(), PetscAbsInt(), PetscAbsReal(), PetscSqr()
+
+M*/
 #define PetscImaginaryPart(a) PetscImaginaryPartComplex(a)
+
 #define PetscAbsScalar(a)     PetscAbsComplex(a)
 #define PetscConj(a)          PetscConjComplex(a)
 #define PetscSqrtScalar(a)    PetscSqrtComplex(a)
@@ -321,7 +388,7 @@ typedef PetscReal PetscScalar;
 #define PetscImaginaryPart(a) ((PetscReal)0.)
 PETSC_STATIC_INLINE PetscReal PetscAbsScalar(PetscScalar a) {return a < 0.0 ? -a : a;}
 #define PetscConj(a)          (a)
-#if !defined(PETSC_USE_REAL___FLOAT128)
+#if !defined(PETSC_USE_REAL___FLOAT128)  && !defined(PETSC_USE_REAL___FP16)
 #define PetscSqrtScalar(a)    sqrt(a)
 #define PetscPowScalar(a,b)   pow(a,b)
 #define PetscExpScalar(a)     exp(a)
@@ -334,6 +401,19 @@ PETSC_STATIC_INLINE PetscReal PetscAbsScalar(PetscScalar a) {return a < 0.0 ? -a
 #define PetscSinhScalar(a)    sinh(a)
 #define PetscCoshScalar(a)    cosh(a)
 #define PetscTanhScalar(a)    tanh(a)
+#elif defined(PETSC_USE_REAL___FP16)
+#define PetscSqrtScalar(a)    sqrtf(a)
+#define PetscPowScalar(a,b)   powf(a,b)
+#define PetscExpScalar(a)     expf(a)
+#define PetscLogScalar(a)     logf(a)
+#define PetscSinScalar(a)     sinf(a)
+#define PetscCosScalar(a)     cosf(a)
+#define PetscAsinScalar(a)    asinf(a)
+#define PetscAcosScalar(a)    acosf(a)
+#define PetscTanScalar(a)     tanf(a)
+#define PetscSinhScalar(a)    sinhf(a)
+#define PetscCoshScalar(a)    coshf(a)
+#define PetscTanhScalar(a)    tanhf(a)
 #else /* PETSC_USE_REAL___FLOAT128 */
 #define PetscSqrtScalar(a)    sqrtq(a)
 #define PetscPowScalar(a,b)   powq(a,b)
@@ -353,7 +433,7 @@ PETSC_STATIC_INLINE PetscReal PetscAbsScalar(PetscScalar a) {return a < 0.0 ? -a
 
 #define PetscSign(a) (((a) >= 0) ? ((a) == 0 ? 0 : 1) : -1)
 #define PetscSignReal(a) (((a) >= 0.0) ? ((a) == 0.0 ? 0.0 : 1.0) : -1.0)
-#define PetscAbs(a)  (((a) >= 0) ? (a) : -(a))
+#define PetscAbs(a)  (((a) >= 0) ? (a) : (-(a)))
 
 /* --------------------------------------------------------------------------*/
 
@@ -361,12 +441,42 @@ PETSC_STATIC_INLINE PetscReal PetscAbsScalar(PetscScalar a) {return a < 0.0 ? -a
    Certain objects may be created using either single or double precision.
    This is currently not used.
 */
-typedef enum { PETSC_SCALAR_DOUBLE,PETSC_SCALAR_SINGLE, PETSC_SCALAR_LONG_DOUBLE } PetscScalarPrecision;
+typedef enum { PETSC_SCALAR_DOUBLE,PETSC_SCALAR_SINGLE, PETSC_SCALAR_LONG_DOUBLE, PETSC_SCALAR_HALF } PetscScalarPrecision;
 
 #if defined(PETSC_HAVE_COMPLEX)
 /* PETSC_i is the imaginary number, i */
 PETSC_EXTERN PetscComplex PETSC_i;
+
+/* Try to do the right thing for complex number construction: see
+
+  http://www.open-std.org/jtc1/sc22/wg14/www/docs/n1464.htm
+
+  for details
+*/
+PETSC_STATIC_INLINE PetscComplex PetscCMPLX(PetscReal x, PetscReal y)
+{
+#if   defined(__cplusplus) && defined(PETSC_HAVE_CXX_COMPLEX) && !defined(PETSC_USE_REAL___FLOAT128)
+  return PetscComplex(x,y);
+#elif defined(_Imaginary_I)
+  return x + y * _Imaginary_I;
+#else
+  { /* In both C99 and C11 (ISO/IEC 9899, Section 6.2.5),
+
+       "For each floating type there is a corresponding real type, which is always a real floating
+       type. For real floating types, it is the same type. For complex types, it is the type given
+       by deleting the keyword _Complex from the type name."
+
+       So type punning should be portable. */
+    union { PetscComplex z; PetscReal f[2]; } uz;
+
+    uz.f[0] = x;
+    uz.f[1] = y;
+    return uz.z;
+  }
 #endif
+}
+#endif
+
 
 /*MC
    PetscMin - Returns minimum of two numbers
@@ -381,11 +491,12 @@ PETSC_EXTERN PetscComplex PETSC_i;
 +  v1 - first value to find minimum of
 -  v2 - second value to find minimum of
 
-   Notes: type can be integer or floating point value
+   Notes:
+    type can be integer or floating point value
 
    Level: beginner
 
-.seealso: PetscMin(), PetscClipInterval(), PetscAbsInt(), PetscAbsReal(), PetscSqr()
+.seealso: PetscMax(), PetscClipInterval(), PetscAbsInt(), PetscAbsReal(), PetscSqr()
 
 M*/
 #define PetscMin(a,b)   (((a)<(b)) ?  (a) : (b))
@@ -403,7 +514,8 @@ M*/
 +  v1 - first value to find maximum of
 -  v2 - second value to find maximum of
 
-   Notes: type can be integer or floating point value
+   Notes:
+    type can be integer or floating point value
 
    Level: beginner
 
@@ -426,7 +538,8 @@ M*/
 .  a - lower end of interval
 -  b - upper end of interval
 
-   Notes: type can be integer or floating point value
+   Notes:
+    type can be integer or floating point value
 
    Level: beginner
 
@@ -452,7 +565,7 @@ M*/
 .seealso: PetscMax(), PetscMin(), PetscAbsReal(), PetscSqr()
 
 M*/
-#define PetscAbsInt(a)  (((a)<0)   ? -(a) : (a))
+#define PetscAbsInt(a)  (((a)<0)   ? (-(a)) : (a))
 
 /*MC
    PetscAbsReal - Returns the absolute value of an real number
@@ -472,7 +585,15 @@ M*/
 .seealso: PetscMax(), PetscMin(), PetscAbsInt(), PetscSqr()
 
 M*/
-#define PetscAbsReal(a) (((a)<0)   ? -(a) : (a))
+#if defined(PETSC_USE_REAL_SINGLE)
+#define PetscAbsReal(a) fabsf(a)
+#elif defined(PETSC_USE_REAL_DOUBLE)
+#define PetscAbsReal(a) fabs(a)
+#elif defined(PETSC_USE_REAL___FLOAT128)
+#define PetscAbsReal(a) fabsq(a)
+#elif defined(PETSC_USE_REAL___FP16)
+#define PetscAbsReal(a) fabsf(a)
+#endif
 
 /*MC
    PetscSqr - Returns the square of a number
@@ -486,7 +607,8 @@ M*/
    Input Parameter:
 .   v1 - the value
 
-   Notes: type can be integer or floating point value
+   Notes:
+    type can be integer or floating point value
 
    Level: beginner
 
@@ -496,16 +618,20 @@ M*/
 #define PetscSqr(a)     ((a)*(a))
 
 /* ----------------------------------------------------------------------------*/
+
+#if defined(PETSC_USE_REAL_SINGLE)
+#define PetscRealConstant(constant) constant##F
+#elif defined(PETSC_USE_REAL___FLOAT128)
+#define PetscRealConstant(constant) constant##Q
+#else
+#define PetscRealConstant(constant) constant
+#endif
+
 /*
      Basic constants
 */
-#if defined(PETSC_USE_REAL___FLOAT128)
-#define PETSC_PI                 M_PIq
-#elif defined(M_PI)
-#define PETSC_PI                 M_PI
-#else
-#define PETSC_PI                 3.14159265358979323846264338327950288419716939937510582
-#endif
+#define PETSC_PI   PetscRealConstant(3.1415926535897932384626433832795029)
+#define PETSC_PHI  PetscRealConstant(1.6180339887498948482045868343656381)
 
 #if !defined(PETSC_USE_64BIT_INDICES)
 #define PETSC_MAX_INT            2147483647
@@ -517,33 +643,44 @@ M*/
 
 #if defined(PETSC_USE_REAL_SINGLE)
 #  define PETSC_MAX_REAL                3.40282346638528860e+38F
-#  define PETSC_MIN_REAL                -PETSC_MAX_REAL
+#  define PETSC_MIN_REAL                (-PETSC_MAX_REAL)
 #  define PETSC_MACHINE_EPSILON         1.19209290e-07F
 #  define PETSC_SQRT_MACHINE_EPSILON    3.45266983e-04F
-#  define PETSC_SMALL                   1.e-5
+#  define PETSC_SMALL                   1.e-5F
 #elif defined(PETSC_USE_REAL_DOUBLE)
 #  define PETSC_MAX_REAL                1.7976931348623157e+308
-#  define PETSC_MIN_REAL                -PETSC_MAX_REAL
+#  define PETSC_MIN_REAL                (-PETSC_MAX_REAL)
 #  define PETSC_MACHINE_EPSILON         2.2204460492503131e-16
 #  define PETSC_SQRT_MACHINE_EPSILON    1.490116119384766e-08
 #  define PETSC_SMALL                   1.e-10
 #elif defined(PETSC_USE_REAL___FLOAT128)
 #  define PETSC_MAX_REAL                FLT128_MAX
-#  define PETSC_MIN_REAL                -FLT128_MAX
+#  define PETSC_MIN_REAL                (-FLT128_MAX)
 #  define PETSC_MACHINE_EPSILON         FLT128_EPSILON
-#  define PETSC_SQRT_MACHINE_EPSILON    1.38777878078e-17q
-#  define PETSC_SMALL                   1.e-20q
+#  define PETSC_SQRT_MACHINE_EPSILON    1.38777878078144567552953958511352539e-17Q
+#  define PETSC_SMALL                   1.e-20Q
+#elif defined(PETSC_USE_REAL___FP16)  /* maybe should use single precision values for these? */
+#  define PETSC_MAX_REAL                65504.
+#  define PETSC_MIN_REAL                (-PETSC_MAX_REAL)
+#  define PETSC_MACHINE_EPSILON         .00097656
+#  define PETSC_SQRT_MACHINE_EPSILON    .0312
+#  define PETSC_SMALL                   5.e-3
 #endif
 
-#define PETSC_INFINITY                PETSC_MAX_REAL/4.0
-#define PETSC_NINFINITY              -PETSC_INFINITY
+#define PETSC_INFINITY               (PETSC_MAX_REAL/4)
+#define PETSC_NINFINITY              (-PETSC_INFINITY)
 
-PETSC_EXTERN PetscErrorCode PetscIsInfOrNanReal(PetscReal);
-PETSC_EXTERN PetscErrorCode PetscIsNanReal(PetscReal);
+PETSC_EXTERN PetscBool PetscIsInfReal(PetscReal);
+PETSC_EXTERN PetscBool PetscIsNanReal(PetscReal);
 PETSC_EXTERN PetscBool PetscIsNormalReal(PetscReal);
-PETSC_STATIC_INLINE PetscErrorCode PetscIsInfOrNanScalar(PetscScalar v) {return PetscIsInfOrNanReal(PetscAbsScalar(v));}
-PETSC_STATIC_INLINE PetscErrorCode PetscIsNanScalar(PetscScalar v) {return PetscIsNanReal(PetscAbsScalar(v));}
-PETSC_STATIC_INLINE PetscErrorCode PetscIsNormalScalar(PetscScalar v) {return PetscIsNormalReal(PetscAbsScalar(v));}
+PETSC_STATIC_INLINE PetscBool PetscIsInfOrNanReal(PetscReal v) {return PetscIsInfReal(v) || PetscIsNanReal(v) ? PETSC_TRUE : PETSC_FALSE;}
+PETSC_STATIC_INLINE PetscBool PetscIsInfScalar(PetscScalar v) {return PetscIsInfReal(PetscAbsScalar(v));}
+PETSC_STATIC_INLINE PetscBool PetscIsNanScalar(PetscScalar v) {return PetscIsNanReal(PetscAbsScalar(v));}
+PETSC_STATIC_INLINE PetscBool PetscIsInfOrNanScalar(PetscScalar v) {return PetscIsInfOrNanReal(PetscAbsScalar(v));}
+PETSC_STATIC_INLINE PetscBool PetscIsNormalScalar(PetscScalar v) {return PetscIsNormalReal(PetscAbsScalar(v));}
+
+PETSC_EXTERN PetscBool PetscEqualReal(PetscReal,PetscReal);
+PETSC_EXTERN PetscBool PetscEqualScalar(PetscScalar,PetscScalar);
 
 /*
     These macros are currently hardwired to match the regular data types, so there is no support for a different
