@@ -19,7 +19,8 @@ typedef struct _p_PC* PC;
 
    Level: beginner
 
-   Notes: Click on the links below to see details on a particular solver
+   Notes:
+    Click on the links above to see details on a particular solver
 
           PCRegister() is used to register preconditioners that are then accessible via PCSetType()
 
@@ -45,6 +46,7 @@ typedef const char* PCType;
 #define PCNN              "nn"
 #define PCCHOLESKY        "cholesky"
 #define PCPBJACOBI        "pbjacobi"
+#define PCVPBJACOBI       "vpbjacobi"
 #define PCMAT             "mat"
 #define PCHYPRE           "hypre"
 #define PCPARMS           "parms"
@@ -62,13 +64,14 @@ typedef const char* PCType;
 #define PCREDISTRIBUTE    "redistribute"
 #define PCSVD             "svd"
 #define PCGAMG            "gamg"
-#define PCSACUSP          "sacusp"        /* these four run on NVIDIA GPUs using CUSP */
-#define PCSACUSPPOLY      "sacusppoly"
-#define PCBICGSTABCUSP    "bicgstabcusp"
-#define PCAINVCUSP        "ainvcusp"
+#define PCCHOWILUVIENNACL "chowiluviennacl"
+#define PCROWSCALINGVIENNACL "rowscalingviennacl"
+#define PCSAVIENNACL      "saviennacl"
 #define PCBDDC            "bddc"
 #define PCKACZMARZ        "kaczmarz"
 #define PCTELESCOPE       "telescope"
+#define PCPATCH           "patch"
+#define PCLMVM            "lmvm"
 
 /*E
     PCSide - If the preconditioner is to be applied to the left, right
@@ -87,7 +90,8 @@ PETSC_EXTERN const char *const *const PCSides;
 
    Level: advanced
 
-   Notes: this must match petsc/finclude/petscpc.h and the KSPConvergedReason values in petscksp.h
+   Notes:
+    this must match petsc/finclude/petscpc.h and the KSPConvergedReason values in petscksp.h
 
 .seealso: PCApplyRichardson()
 E*/
@@ -167,7 +171,7 @@ $  PC_COMPOSITE_ADDITIVE - results from application of all preconditioners are a
 $  PC_COMPOSITE_MULTIPLICATIVE - preconditioners are applied sequentially to the residual freshly
 $                                computed after the previous preconditioner application
 $  PC_COMPOSITE_SYMMETRIC_MULTIPLICATIVE - preconditioners are applied sequentially to the residual freshly
-$                                computed from first preconditioner to last and then back (Use only for symmetric matrices and preconditions)
+$                                computed from first preconditioner to last and then back (Use only for symmetric matrices and preconditioners)
 $  PC_COMPOSITE_SPECIAL - This is very special for a matrix of the form alpha I + R + S
 $                         where first preconditioner is built from alpha I + S and second from
 $                         alpha I + R
@@ -245,7 +249,7 @@ typedef const char *PCGAMGClassicalType;
    Level: beginner
 
    Values:
-+  PC_MG_MULTIPLICATIVE (default) - traditional V or W cycle as determined by PCMGSetCycles()
++  PC_MG_MULTIPLICATIVE (default) - traditional V or W cycle as determined by PCMGSetCycleType()
 .  PC_MG_ADDITIVE - the additive multigrid preconditioner where all levels are
                 smoothed before updating the residual. This only uses the
                 down smoother, in the preconditioner the upper smoother is ignored
@@ -257,7 +261,7 @@ typedef const char *PCGAMGClassicalType;
 -  PC_MG_KASKADE - like full multigrid except one never goes back to a coarser level
                from a finer
 
-.seealso: PCMGSetType()
+.seealso: PCMGSetType(), PCMGSetCycleType(), PCMGSetCycleTypeOnLevel()
 
 E*/
 typedef enum { PC_MG_MULTIPLICATIVE,PC_MG_ADDITIVE,PC_MG_FULL,PC_MG_KASKADE } PCMGType;
@@ -279,6 +283,25 @@ E*/
 typedef enum { PC_MG_CYCLE_V = 1,PC_MG_CYCLE_W = 2 } PCMGCycleType;
 PETSC_EXTERN const char *const PCMGCycleTypes[];
 
+/*E
+    PCMGalerkinType - Determines if the coarse grid operators are computed via the Galerkin process
+
+   Level: beginner
+
+   Values:
++  PC_MG_GALERKIN_PMAT - computes the pmat (matrix from which the preconditioner is built) via the Galerkin process from the finest grid
+.  PC_MG_GALERKIN_MAT -  computes the mat (matrix used to apply the operator) via the Galerkin process from the finest grid
+.  PC_MG_GALERKIN_BOTH - computes both the mat and pmat via the Galerkin process (if pmat == mat the construction is only done once
+-  PC_MG_GALERKIN_NONE - neither operator is computed via the Galerkin process, the user must provide the operator
+
+   Users should never set PC_MG_GALERKIN_EXTERNAL, it is used by GAMG and ML
+
+.seealso: PCMGSetCycleType()
+
+E*/
+typedef enum { PC_MG_GALERKIN_BOTH,PC_MG_GALERKIN_PMAT,PC_MG_GALERKIN_MAT, PC_MG_GALERKIN_NONE, PC_MG_GALERKIN_EXTERNAL} PCMGGalerkinType;
+PETSC_EXTERN const char *const PCMGGalerkinTypes[];
+
 /*E
     PCExoticType - Face based or wirebasket based coarse grid space
 
@@ -290,6 +313,16 @@ typedef enum { PC_EXOTIC_FACE,PC_EXOTIC_WIREBASKET } PCExoticType;
 PETSC_EXTERN const char *const PCExoticTypes[];
 PETSC_EXTERN PetscErrorCode PCExoticSetType(PC,PCExoticType);
 
+/*E
+    PCPatchConstructType - The algorithm used to construct patches for the preconditioner
+
+   Level: beginner
+
+.seealso: PCPatchSetConstructType(), PCEXOTIC
+E*/
+typedef enum {PC_PATCH_STAR, PC_PATCH_VANKA, PC_PATCH_USER, PC_PATCH_PYTHON} PCPatchConstructType;
+PETSC_EXTERN const char *const PCPatchConstructTypes[];
+
 /*E
     PCFailedReason - indicates type of PC failure
 
