@@ -12,7 +12,7 @@ PetscErrorCode MatDSFischer(Mat, Vec, Vec, Vec, Vec, PetscReal, Vec, Vec, Vec, V
 /*E
   TaoSubsetType - PetscInt representing the way TAO handles active sets
 
-+ TAO_SUBSET_SUBVEC - TAO uses PETSc's MatGetSubMatrix and VecGetSubVector
++ TAO_SUBSET_SUBVEC - TAO uses PETSc's MatCreateSubMatrix and VecGetSubVector
 . TAO_SUBSET_MASK - Matrices are zeroed out corresponding to active set entries
 - TAO_SUBSET_MATRIXFREE - Same as TAO_SUBSET_MASK, but can be applied to matrix-free operators
 
@@ -41,7 +41,7 @@ typedef struct _p_Tao*   Tao;
        Level: beginner
 
 J*/
-#define TaoType char*
+typedef const char *TaoType;
 #define TAOLMVM     "lmvm"
 #define TAONLS      "nls"
 #define TAONTR      "ntr"
@@ -51,6 +51,14 @@ J*/
 #define TAOOWLQN    "owlqn"
 #define TAOBMRM     "bmrm"
 #define TAOBLMVM    "blmvm"
+#define TAOBQNLS    "bqnls"
+#define TAOBNCG     "bncg"
+#define TAOBNLS     "bnls"
+#define TAOBNTR     "bntr"
+#define TAOBNTL     "bntl"
+#define TAOBQNKLS   "bqnkls"
+#define TAOBQNKTR   "bqnktr"
+#define TAOBQNKTL   "bqnktl"
 #define TAOBQPIP    "bqpip"
 #define TAOGPCG     "gpcg"
 #define TAONM       "nm"
@@ -61,20 +69,33 @@ J*/
 #define TAOASILS    "asils"
 #define TAOASFLS    "asfls"
 #define TAOIPM      "ipm"
-#define TAOTEST     "test"
 
 PETSC_EXTERN PetscClassId TAO_CLASSID;
 PETSC_EXTERN PetscFunctionList TaoList;
 
-/*  Convergence flags.
-    Be sure to check that these match the flags in
-    include/petsc/finclude/petsctao.h
-*/
+/*E
+    TaoConvergedReason - reason a TAO method was said to have converged or diverged
+
+   Level: beginner
+
+   The two most common reasons for divergence are
+$   1) an incorrectly coded or computed gradient or Hessian
+$   2) failure or lack of convergence in the linear system (in this case we recommend
+$      testing with -pc_type lu to eliminate the linear solver as the cause of the problem).
+
+   Developer Notes:
+    this must match petsc/finclude/petsctao.h
+
+       The string versions of these are in TAOConvergedReasons, if you change any value here you must
+     also adjust that array.
+
+.seealso: TAOSolve(), TaoGetConvergedReason(), KSPConvergedReason, SNESConvergedReason, TSConvergedReason
+E*/
 typedef enum {/* converged */
-  TAO_CONVERGED_GATOL          =  3, /* ||g(X)|| < gatol */
-  TAO_CONVERGED_GRTOL          =  4, /* ||g(X)|| / f(X)  < grtol */
-  TAO_CONVERGED_GTTOL          =  5, /* ||g(X)|| / ||g(X0)|| < gttol */
-  TAO_CONVERGED_STEPTOL        =  6, /* step size small */
+  TAO_CONVERGED_GATOL         =  3, /* ||g(X)|| < gatol */
+  TAO_CONVERGED_GRTOL         =  4, /* ||g(X)|| / f(X)  < grtol */
+  TAO_CONVERGED_GTTOL         =  5, /* ||g(X)|| / ||g(X0)|| < gttol */
+  TAO_CONVERGED_STEPTOL       =  6, /* step size small */
   TAO_CONVERGED_MINF          =  7, /* F < F_min */
   TAO_CONVERGED_USER          =  8, /* User defined */
   /* diverged */
@@ -89,14 +110,13 @@ typedef enum {/* converged */
 
 PETSC_EXTERN const char **TaoConvergedReasons;
 
-
 PETSC_EXTERN PetscErrorCode TaoInitializePackage(void);
 PETSC_EXTERN PetscErrorCode TaoFinalizePackage(void);
 PETSC_EXTERN PetscErrorCode TaoCreate(MPI_Comm,Tao*);
 PETSC_EXTERN PetscErrorCode TaoSetFromOptions(Tao);
 PETSC_EXTERN PetscErrorCode TaoSetUp(Tao);
-PETSC_EXTERN PetscErrorCode TaoSetType(Tao, const TaoType);
-PETSC_EXTERN PetscErrorCode TaoGetType(Tao, const TaoType *);
+PETSC_EXTERN PetscErrorCode TaoSetType(Tao,TaoType);
+PETSC_EXTERN PetscErrorCode TaoGetType(Tao,TaoType *);
 PETSC_EXTERN PetscErrorCode TaoSetApplicationContext(Tao, void*);
 PETSC_EXTERN PetscErrorCode TaoGetApplicationContext(Tao, void*);
 PETSC_EXTERN PetscErrorCode TaoDestroy(Tao*);
@@ -118,9 +138,11 @@ PETSC_EXTERN PetscErrorCode TaoGetSolutionVector(Tao, Vec*);
 PETSC_EXTERN PetscErrorCode TaoGetGradientVector(Tao, Vec*);
 PETSC_EXTERN PetscErrorCode TaoSetGradientNorm(Tao, Mat);
 PETSC_EXTERN PetscErrorCode TaoGetGradientNorm(Tao, Mat*);
+PETSC_EXTERN PetscErrorCode TaoGetLMVMMatrix(Tao, Mat*);
 PETSC_EXTERN PetscErrorCode TaoLMVMSetH0(Tao, Mat);
 PETSC_EXTERN PetscErrorCode TaoLMVMGetH0(Tao, Mat*);
 PETSC_EXTERN PetscErrorCode TaoLMVMGetH0KSP(Tao, KSP*);
+PETSC_EXTERN PetscErrorCode TaoLMVMRecycle(Tao, PetscBool);
 PETSC_EXTERN PetscErrorCode TaoSetObjectiveRoutine(Tao, PetscErrorCode(*)(Tao, Vec, PetscReal*,void*), void*);
 PETSC_EXTERN PetscErrorCode TaoSetGradientRoutine(Tao, PetscErrorCode(*)(Tao, Vec, Vec, void*), void*);
 PETSC_EXTERN PetscErrorCode TaoSetObjectiveAndGradientRoutine(Tao, PetscErrorCode(*)(Tao, Vec, PetscReal*, Vec, void*), void*);
@@ -140,6 +162,7 @@ PETSC_EXTERN PetscErrorCode TaoSetStateDesignIS(Tao, IS, IS);
 
 PETSC_EXTERN PetscErrorCode TaoComputeObjective(Tao, Vec, PetscReal*);
 PETSC_EXTERN PetscErrorCode TaoComputeSeparableObjective(Tao, Vec, Vec);
+PETSC_EXTERN PetscErrorCode TaoTestGradient(Tao,Vec,Vec);
 PETSC_EXTERN PetscErrorCode TaoComputeGradient(Tao, Vec, Vec);
 PETSC_EXTERN PetscErrorCode TaoComputeObjectiveAndGradient(Tao, Vec, PetscReal*, Vec);
 PETSC_EXTERN PetscErrorCode TaoComputeConstraints(Tao, Vec, Vec);
@@ -150,6 +173,7 @@ PETSC_EXTERN PetscErrorCode TaoIsObjectiveDefined(Tao,PetscBool*);
 PETSC_EXTERN PetscErrorCode TaoIsGradientDefined(Tao,PetscBool*);
 PETSC_EXTERN PetscErrorCode TaoIsObjectiveAndGradientDefined(Tao,PetscBool*);
 
+PETSC_EXTERN PetscErrorCode TaoTestHessian(Tao);
 PETSC_EXTERN PetscErrorCode TaoComputeHessian(Tao, Vec, Mat, Mat);
 PETSC_EXTERN PetscErrorCode TaoComputeJacobian(Tao, Vec, Mat, Mat);
 PETSC_EXTERN PetscErrorCode TaoComputeJacobianState(Tao, Vec, Mat, Mat, Mat);
@@ -159,7 +183,7 @@ PETSC_EXTERN PetscErrorCode TaoComputeJacobianDesign(Tao, Vec, Mat);
 
 PETSC_EXTERN PetscErrorCode TaoDefaultComputeHessian(Tao, Vec, Mat, Mat, void*);
 PETSC_EXTERN PetscErrorCode TaoDefaultComputeHessianColor(Tao, Vec, Mat, Mat, void*);
-PETSC_EXTERN PetscErrorCode TaoComputeDualVariables(Tao, Vec, Vec);
+PETSC_EXTERN PetscErrorCode TaoDefaultComputeHessianMFFD(Tao, Vec, Mat, Mat, void*);
 PETSC_EXTERN PetscErrorCode TaoComputeDualVariables(Tao, Vec, Vec);
 PETSC_EXTERN PetscErrorCode TaoSetVariableBounds(Tao, Vec, Vec);
 PETSC_EXTERN PetscErrorCode TaoGetVariableBounds(Tao, Vec*, Vec*);
@@ -187,8 +211,9 @@ PETSC_EXTERN PetscErrorCode TaoGetIterationNumber(Tao, PetscInt*);
 PETSC_EXTERN PetscErrorCode TaoSetIterationNumber(Tao, PetscInt);
 PETSC_EXTERN PetscErrorCode TaoGetTotalIterationNumber(Tao, PetscInt*);
 PETSC_EXTERN PetscErrorCode TaoSetTotalIterationNumber(Tao, PetscInt);
+PETSC_EXTERN PetscErrorCode TaoGetResidualNorm(Tao,PetscReal*);
+PETSC_EXTERN PetscErrorCode TaoGetObjective(Tao,PetscReal*);
 
-PETSC_EXTERN PetscErrorCode TaoSetOptionsPrefix(Tao, const char p[]);
 PETSC_EXTERN PetscErrorCode TaoAppendOptionsPrefix(Tao, const char p[]);
 PETSC_EXTERN PetscErrorCode TaoGetOptionsPrefix(Tao, const char *p[]);
 PETSC_EXTERN PetscErrorCode TaoResetStatistics(Tao);
@@ -204,7 +229,9 @@ PETSC_EXTERN PetscErrorCode TaoSetConvergenceHistory(Tao,PetscReal*,PetscReal*,P
 PETSC_EXTERN PetscErrorCode TaoGetConvergenceHistory(Tao,PetscReal**,PetscReal**,PetscReal**,PetscInt**,PetscInt*);
 PETSC_EXTERN PetscErrorCode TaoSetMonitor(Tao, PetscErrorCode (*)(Tao,void*),void *,PetscErrorCode (*)(void**));
 PETSC_EXTERN PetscErrorCode TaoCancelMonitors(Tao);
-PETSC_EXTERN PetscErrorCode TaoDefaultMonitor(Tao, void*);
+PETSC_EXTERN PetscErrorCode TaoMonitorDefault(Tao, void*);
+PETSC_DEPRECATED ("Use TaoMonitorDefault()") PETSC_STATIC_INLINE PetscErrorCode TaoDefaultMonitor(Tao tao, void*ctx) {return TaoMonitorDefault(tao,ctx);}
+PETSC_EXTERN PetscErrorCode TaoDefaultGMonitor(Tao, void*);
 PETSC_EXTERN PetscErrorCode TaoDefaultSMonitor(Tao, void*);
 PETSC_EXTERN PetscErrorCode TaoDefaultCMonitor(Tao, void*);
 PETSC_EXTERN PetscErrorCode TaoSolutionMonitor(Tao, void*);
@@ -221,7 +248,8 @@ PETSC_EXTERN PetscErrorCode TaoSetConvergenceTest(Tao, PetscErrorCode (*)(Tao, v
 
 PETSC_EXTERN PetscErrorCode TaoSQPCONSetStateDesignIS(Tao, IS, IS);
 PETSC_EXTERN PetscErrorCode TaoLCLSetStateDesignIS(Tao, IS, IS);
-PETSC_EXTERN PetscErrorCode TaoMonitor(Tao, PetscInt, PetscReal, PetscReal, PetscReal, PetscReal, TaoConvergedReason*);
-
-
+PETSC_EXTERN PetscErrorCode TaoMonitor(Tao, PetscInt, PetscReal, PetscReal, PetscReal, PetscReal);
+typedef struct _n_TaoMonitorDrawCtx* TaoMonitorDrawCtx;
+PETSC_EXTERN PetscErrorCode TaoMonitorDrawCtxCreate(MPI_Comm,const char[],const char[],int,int,int,int,PetscInt,TaoMonitorDrawCtx*);
+PETSC_EXTERN PetscErrorCode TaoMonitorDrawCtxDestroy(TaoMonitorDrawCtx*);
 #endif
