@@ -97,19 +97,40 @@ typedef const char* VecType;
 #define VECMPI         "mpi"
 #define VECSTANDARD    "standard"   /* seq on one process and mpi on several */
 #define VECSHARED      "shared"
-#define VECSEQCUSP     "seqcusp"
-#define VECMPICUSP     "mpicusp"
-#define VECCUSP        "cusp"       /* seqcusp on one process and mpicusp on several */
 #define VECSEQVIENNACL "seqviennacl"
 #define VECMPIVIENNACL "mpiviennacl"
 #define VECVIENNACL    "viennacl"   /* seqviennacl on one process and mpiviennacl on several */
+#define VECSEQCUDA     "seqcuda"
+#define VECMPICUDA     "mpicuda"
+#define VECCUDA        "cuda"       /* seqcuda on one process and mpicuda on several */
 #define VECNEST        "nest"
-#define VECSEQPTHREAD  "seqpthread"
-#define VECMPIPTHREAD  "mpipthread"
-#define VECPTHREAD     "pthread"    /* seqpthread on one process and mpipthread on several */
+#define VECNODE        "node"       /* use on-node shared memory */
 
+/*J
+    VecScatterType - String with the name of a PETSc vector scatter type
+
+   Level: beginner
+
+.seealso: VecScatterSetType(), VecScatter, VecScatterCreate(), VecScatterDestroy()
+J*/
+typedef const char* VecScatterType;
+#define VECSCATTERSEQ       "seq"
+#define VECSCATTERMPI1      "mpi1"
+#define VECSCATTERMPI3      "mpi3"     /* use MPI3 on-node shared memory */
+#define VECSCATTERMPI3NODE  "mpi3node" /* use MPI3 on-node shared memory for vector type VECNODE */
+
+/* Dynamic creation and loading functions */
+PETSC_EXTERN PetscFunctionList VecScatterList;
+PETSC_EXTERN PetscErrorCode VecScatterSetType(VecScatter, VecScatterType);
+PETSC_EXTERN PetscErrorCode VecScatterGetType(VecScatter, VecScatterType *);
+PETSC_EXTERN PetscErrorCode VecScatterSetFromOptions(VecScatter);
+PETSC_EXTERN PetscErrorCode VecScatterRegister(const char[],PetscErrorCode (*)(VecScatter));
+PETSC_EXTERN PetscErrorCode VecScatterCreate(Vec,IS,Vec,IS,VecScatter*);
+PETSC_EXTERN PetscErrorCode VecScatterInitializePackage(void);
+PETSC_EXTERN PetscErrorCode VecScatterFinalizePackage(void);
 
 /* Logging support */
+#define    REAL_FILE_CLASSID 1211213
 #define    VEC_FILE_CLASSID 1211214
 PETSC_EXTERN PetscClassId VEC_CLASSID;
 PETSC_EXTERN PetscClassId VEC_SCATTER_CLASSID;
@@ -124,6 +145,8 @@ PETSC_EXTERN PetscErrorCode VecCreateMPI(MPI_Comm,PetscInt,PetscInt,Vec*);
 PETSC_EXTERN PetscErrorCode VecCreateSeqWithArray(MPI_Comm,PetscInt,PetscInt,const PetscScalar[],Vec*);
 PETSC_EXTERN PetscErrorCode VecCreateMPIWithArray(MPI_Comm,PetscInt,PetscInt,PetscInt,const PetscScalar[],Vec*);
 PETSC_EXTERN PetscErrorCode VecCreateShared(MPI_Comm,PetscInt,PetscInt,Vec*);
+PETSC_EXTERN PetscErrorCode VecCreateNode(MPI_Comm,PetscInt,PetscInt,Vec*);
+
 PETSC_EXTERN PetscErrorCode VecSetFromOptions(Vec);
 PETSC_STATIC_INLINE PetscErrorCode VecViewFromOptions(Vec A,PetscObject B,const char name[]) {return PetscObjectViewFromOptions((PetscObject)A,B,name);}
 
@@ -167,7 +190,7 @@ PETSC_EXTERN const char *const NormTypes[];
 M*/
 
 /*MC
-     NORM_2 - the two norm, ||v|| = sqrt(sum_i (v_i)^2) (vectors only)
+     NORM_2 - the two norm, ||v|| = sqrt(sum_i |v_i|^2) (vectors only)
 
    Level: beginner
 
@@ -177,7 +200,7 @@ M*/
 M*/
 
 /*MC
-     NORM_FROBENIUS - ||A|| = sqrt(sum_ij (A_ij)^2), same as NORM_2 for vectors
+     NORM_FROBENIUS - ||A|| = sqrt(sum_ij |A_ij|^2), same as NORM_2 for vectors
 
    Level: beginner
 
@@ -297,9 +320,9 @@ PETSC_EXTERN PetscErrorCode VecStashGetInfo(Vec,PetscInt*,PetscInt*,PetscInt*,Pe
    many values simultaneously if possible.
 
    These values may be cached, so VecAssemblyBegin() and VecAssemblyEnd()
-   MUST be called after all calls to VecSetValues() have been completed.
+   MUST be called after all calls to VecSetValue() have been completed.
 
-   VecSetValues() uses 0-based indices in Fortran as well as in C.
+   VecSetValue() uses 0-based indices in Fortran as well as in C.
 
    Level: beginner
 
@@ -320,14 +343,13 @@ PETSC_EXTERN PetscErrorCode VecRegister(const char[],PetscErrorCode (*)(Vec));
 
 PETSC_EXTERN PetscErrorCode VecScatterCreate(Vec,IS,Vec,IS,VecScatter *);
 PETSC_EXTERN PetscErrorCode VecScatterCreateEmpty(MPI_Comm,VecScatter *);
-PETSC_EXTERN PetscErrorCode VecScatterCreateLocal(VecScatter,PetscInt,const PetscInt[],const PetscInt[],const PetscInt[],PetscInt,const PetscInt[],const PetscInt[],const PetscInt[],PetscInt);
 PETSC_EXTERN PetscErrorCode VecScatterBegin(VecScatter,Vec,Vec,InsertMode,ScatterMode);
 PETSC_EXTERN PetscErrorCode VecScatterEnd(VecScatter,Vec,Vec,InsertMode,ScatterMode);
 PETSC_EXTERN PetscErrorCode VecScatterDestroy(VecScatter*);
 PETSC_EXTERN PetscErrorCode VecScatterCopy(VecScatter,VecScatter *);
 PETSC_EXTERN PetscErrorCode VecScatterView(VecScatter,PetscViewer);
 PETSC_STATIC_INLINE PetscErrorCode VecScatterViewFromOptions(VecScatter A,PetscObject obj,const char name[]) {return PetscObjectViewFromOptions((PetscObject)A,obj,name);}
-PETSC_EXTERN PetscErrorCode VecScatterRemap(VecScatter,PetscInt *,PetscInt*);
+PETSC_EXTERN PetscErrorCode VecScatterRemap(VecScatter,PetscInt[],PetscInt[]);
 PETSC_EXTERN PetscErrorCode VecScatterGetMerged(VecScatter,PetscBool *);
 
 PETSC_EXTERN PetscErrorCode VecGetArray4d(Vec,PetscInt,PetscInt,PetscInt,PetscInt,PetscInt,PetscInt,PetscInt,PetscInt,PetscScalar****[]);
@@ -413,7 +435,7 @@ PETSC_EXTERN PetscErrorCode VecMTDotEnd(Vec,PetscInt,const Vec[],PetscScalar[]);
 PETSC_EXTERN PetscErrorCode PetscCommSplitReductionBegin(MPI_Comm);
 
 
-typedef enum {VEC_IGNORE_OFF_PROC_ENTRIES,VEC_IGNORE_NEGATIVE_INDICES} VecOption;
+typedef enum {VEC_IGNORE_OFF_PROC_ENTRIES,VEC_IGNORE_NEGATIVE_INDICES,VEC_SUBSET_OFF_PROC_ENTRIES} VecOption;
 PETSC_EXTERN PetscErrorCode VecSetOption(Vec,VecOption,PetscBool );
 
 PETSC_EXTERN PetscErrorCode VecGetArray(Vec,PetscScalar**);
@@ -424,9 +446,27 @@ PETSC_EXTERN PetscErrorCode VecGetLocalVector(Vec,Vec);
 PETSC_EXTERN PetscErrorCode VecRestoreLocalVector(Vec,Vec);
 PETSC_EXTERN PetscErrorCode VecGetLocalVectorRead(Vec,Vec);
 PETSC_EXTERN PetscErrorCode VecRestoreLocalVectorRead(Vec,Vec);
-/*
-   Accesses a pair of pointers for two vectors that may be common. When not common the first is read only
-*/
+
+/*@C
+   VecGetArrayPair - Accesses a pair of pointers for two vectors that may be common. When not common the first is read only
+
+   Logically Collective on Vec
+
+   Input Parameters:
++  x - the vector
+-  y - the second vector
+
+   Output Parameters:
++  xv - location to put pointer to the first array
+-  yv - location to put pointer to the second array
+
+   Level: developer
+
+   Not available from Fortran
+
+.seealso: VecGetArray(), VecGetArrayRead(), VecRestoreArrayPair()
+
+@*/
 PETSC_STATIC_INLINE PetscErrorCode VecGetArrayPair(Vec x,Vec y,PetscScalar **xv,PetscScalar **yv)
 {
   PetscErrorCode ierr;
@@ -440,6 +480,27 @@ PETSC_STATIC_INLINE PetscErrorCode VecGetArrayPair(Vec x,Vec y,PetscScalar **xv,
   }
   PetscFunctionReturn(0);
 }
+
+/*@C
+   VecRestoreArrayPair - Returns a pair of pointers for two vectors that may be common. When not common the first is read only
+
+   Logically Collective on Vec
+
+   Input Parameters:
++  x - the vector
+-  y - the second vector
+
+   Output Parameters:
++  xv - location to put pointer to the first array
+-  yv - location to put pointer to the second array
+
+   Level: developer
+
+   Not available from Fortran
+
+.seealso: VecGetArray(), VecGetArrayRead(), VecGetArrayPair()
+
+@*/
 PETSC_STATIC_INLINE PetscErrorCode VecRestoreArrayPair(Vec x,Vec y,PetscScalar **xv,PetscScalar **yv)
 {
   PetscErrorCode ierr;
@@ -466,13 +527,11 @@ PETSC_EXTERN PetscErrorCode VecLockPop(Vec);
 
 PETSC_EXTERN PetscErrorCode VecValidValues(Vec,PetscInt,PetscBool);
 
-PETSC_EXTERN PetscErrorCode VecContourScale(Vec,PetscReal,PetscReal);
-
 /*
     These numbers need to match the entries in
   the function table in vecimpl.h
 */
-typedef enum { VECOP_VIEW = 33, VECOP_LOAD = 41, VECOP_DUPLICATE = 0} VecOperation;
+typedef enum { VECOP_DUPLICATE = 0, VECOP_VIEW = 33, VECOP_LOAD = 41, VECOP_VIEWNATIVE = 68, VECOP_LOADNATIVE = 69 } VecOperation;
 PETSC_EXTERN PetscErrorCode VecSetOperation(Vec,VecOperation,void(*)(void));
 
 /*
@@ -498,12 +557,14 @@ PETSC_EXTERN PetscErrorCode VecScatterCreateToZero(Vec,VecScatter*,Vec*);
 PETSC_EXTERN PetscErrorCode ISComplementVec(IS,Vec,IS*);
 PETSC_EXTERN PetscErrorCode VecPow(Vec, PetscScalar);
 PETSC_EXTERN PetscErrorCode VecMedian(Vec, Vec, Vec, Vec);
+PETSC_EXTERN PetscErrorCode VecWhichInactive(Vec, Vec, Vec, Vec, PetscBool, IS *);
 PETSC_EXTERN PetscErrorCode VecWhichBetween(Vec, Vec, Vec, IS *);
 PETSC_EXTERN PetscErrorCode VecWhichBetweenOrEqual(Vec, Vec, Vec, IS *);
 PETSC_EXTERN PetscErrorCode VecWhichGreaterThan(Vec, Vec, IS * );
 PETSC_EXTERN PetscErrorCode VecWhichLessThan(Vec, Vec, IS *);
 PETSC_EXTERN PetscErrorCode VecWhichEqual(Vec, Vec, IS *);
 PETSC_EXTERN PetscErrorCode VecISAXPY(Vec, IS, PetscScalar,Vec);
+PETSC_EXTERN PetscErrorCode VecISCopy(Vec, IS, ScatterMode, Vec);
 PETSC_EXTERN PetscErrorCode VecISSet(Vec,IS, PetscScalar);
 PETSC_EXTERN PetscErrorCode VecBoundGradientProjection(Vec, Vec, Vec, Vec, Vec);
 PETSC_EXTERN PetscErrorCode VecStepBoundInfo(Vec,Vec,Vec,Vec,PetscReal*, PetscReal*,PetscReal*);
@@ -535,20 +596,6 @@ PETSC_EXTERN PetscErrorCode VecsCreateSeq(MPI_Comm,PetscInt,PetscInt,Vecs*);
 PETSC_EXTERN PetscErrorCode VecsCreateSeqWithArray(MPI_Comm,PetscInt,PetscInt,PetscScalar*,Vecs*);
 PETSC_EXTERN PetscErrorCode VecsDuplicate(Vecs,Vecs*);
 
-#if defined(PETSC_HAVE_CUSP)
-typedef struct _p_PetscCUSPIndices* PetscCUSPIndices;
-typedef struct _p_VecScatterCUSPIndices_StoS* VecScatterCUSPIndices_StoS;
-typedef struct _p_VecScatterCUSPIndices_PtoP* VecScatterCUSPIndices_PtoP;
-PETSC_EXTERN PetscErrorCode VecCUSPCopyToGPUSome_Public(Vec,PetscCUSPIndices);
-PETSC_EXTERN PetscErrorCode VecCUSPCopyFromGPUSome_Public(Vec,PetscCUSPIndices);
-PETSC_EXTERN PetscErrorCode VecScatterInitializeForGPU(VecScatter,Vec,ScatterMode);
-PETSC_EXTERN PetscErrorCode VecScatterFinalizeForGPU(VecScatter);
-PETSC_EXTERN PetscErrorCode VecCreateSeqCUSP(MPI_Comm,PetscInt,Vec*);
-PETSC_EXTERN PetscErrorCode VecCreateMPICUSP(MPI_Comm,PetscInt,PetscInt,Vec*);
-PETSC_EXTERN PetscErrorCode VecCUSPGetCUDAArray(Vec,PetscScalar**);
-PETSC_EXTERN PetscErrorCode VecCUSPRestoreCUDAArray(Vec,PetscScalar**);
-#endif
-
 #if defined(PETSC_HAVE_VIENNACL)
 typedef struct _p_PetscViennaCLIndices* PetscViennaCLIndices;
 PETSC_EXTERN PetscErrorCode PetscViennaCLIndicesCreate(PetscInt, PetscInt*,PetscInt, PetscInt*,PetscViennaCLIndices*);
@@ -558,6 +605,19 @@ PETSC_EXTERN PetscErrorCode VecViennaCLCopyFromGPUSome_Public(Vec,PetscViennaCLI
 PETSC_EXTERN PetscErrorCode VecCreateSeqViennaCL(MPI_Comm,PetscInt,Vec*);
 PETSC_EXTERN PetscErrorCode VecCreateMPIViennaCL(MPI_Comm,PetscInt,PetscInt,Vec*);
 #endif
+#if defined(PETSC_HAVE_VECCUDA)
+typedef struct _p_PetscCUDAIndices* PetscCUDAIndices;
+typedef struct _p_VecScatterCUDAIndices_StoS* VecScatterCUDAIndices_StoS;
+typedef struct _p_VecScatterCUDAIndices_PtoP* VecScatterCUDAIndices_PtoP;
+PETSC_EXTERN PetscErrorCode VecCUDACopyToGPUSome_Public(Vec,PetscCUDAIndices);
+PETSC_EXTERN PetscErrorCode VecCUDACopyFromGPUSome_Public(Vec,PetscCUDAIndices);
+PETSC_EXTERN PetscErrorCode VecScatterInitializeForGPU(VecScatter,Vec,ScatterMode);
+PETSC_EXTERN PetscErrorCode VecScatterFinalizeForGPU(VecScatter);
+PETSC_EXTERN PetscErrorCode VecCreateSeqCUDA(MPI_Comm,PetscInt,Vec*);
+PETSC_EXTERN PetscErrorCode VecCreateSeqCUDAWithArray(MPI_Comm,PetscInt,PetscInt,const PetscScalar*,Vec*);
+PETSC_EXTERN PetscErrorCode VecCreateMPICUDA(MPI_Comm,PetscInt,PetscInt,Vec*);
+PETSC_EXTERN PetscErrorCode VecCreateMPICUDAWithArray(MPI_Comm,PetscInt,PetscInt,PetscInt,const PetscScalar*,Vec*);
+#endif
 
 PETSC_EXTERN PetscErrorCode VecNestGetSubVecs(Vec,PetscInt*,Vec**);
 PETSC_EXTERN PetscErrorCode VecNestGetSubVec(Vec,PetscInt,Vec*);
@@ -566,7 +626,7 @@ PETSC_EXTERN PetscErrorCode VecNestSetSubVec(Vec,PetscInt,Vec);
 PETSC_EXTERN PetscErrorCode VecCreateNest(MPI_Comm,PetscInt,IS*,Vec*,Vec*);
 PETSC_EXTERN PetscErrorCode VecNestGetSize(Vec,PetscInt*);
 
-PETSC_EXTERN PetscErrorCode PetscOptionsGetVec(const char[],const char[],Vec,PetscBool*);
+PETSC_EXTERN PetscErrorCode PetscOptionsGetVec(PetscOptions,const char[],const char[],Vec,PetscBool*);
 PETSC_EXTERN PetscErrorCode VecChop(Vec,PetscReal);
 
 PETSC_EXTERN PetscErrorCode VecGetLayout(Vec,PetscLayout*);
@@ -577,5 +637,96 @@ PETSC_EXTERN PetscErrorCode VecGetValuesSection(Vec, PetscSection, PetscInt, Pet
 PETSC_EXTERN PetscErrorCode VecSetValuesSection(Vec, PetscSection, PetscInt, PetscScalar [], InsertMode);
 PETSC_EXTERN PetscErrorCode PetscSectionVecNorm(PetscSection, PetscSection, Vec, NormType, PetscReal []);
 
-#endif
+PETSC_EXTERN PetscErrorCode PetscSFCreateFromZero(MPI_Comm,Vec,PetscSF*);
 
+/*S
+  VecTagger - Object used to manage the tagging of a subset of indices based on the values of a vector.  The
+              motivating application is the selection of cells for refinement or coarsening based on vector containing
+              the values in an error indicator metric.
+
+  Level: advanced
+S*/
+typedef struct _p_VecTagger *VecTagger;
+
+/*J
+  VecTaggerType - String with the name of a VecTagger type
+
+  Level: advanced
+J*/
+typedef const char* VecTaggerType;
+/* tag where the vector values are in a box of explicitly defined values */
+#define VECTAGGERABSOLUTE   "absolute"
+/* tag where the vector values are in a box of values relative to the set of all values in the vector */
+#define VECTAGGERRELATIVE   "relative"
+/* tag where the vector values are in a relative range of the *cumulative distribution* of values in the vector */
+#define VECTAGGERCDF        "cdf"
+/* tag a vector as the union of other tags */
+#define VECTAGGEROR         "or"
+/* tag a vector as the intersection of other tags */
+#define VECTAGGERAND        "and"
+
+PETSC_EXTERN PetscClassId VEC_TAGGER_CLASSID;
+PETSC_EXTERN PetscFunctionList VecTaggerList;
+PETSC_EXTERN PetscErrorCode VecTaggerRegister(const char[],PetscErrorCode (*) (VecTagger));
+
+PETSC_EXTERN PetscErrorCode VecTaggerCreate(MPI_Comm,VecTagger *);
+PETSC_EXTERN PetscErrorCode VecTaggerSetBlockSize(VecTagger,PetscInt);
+PETSC_EXTERN PetscErrorCode VecTaggerGetBlockSize(VecTagger,PetscInt*);
+PETSC_EXTERN PetscErrorCode VecTaggerSetType(VecTagger,VecTaggerType);
+PETSC_EXTERN PetscErrorCode VecTaggerGetType(VecTagger,VecTaggerType *);
+PETSC_EXTERN PetscErrorCode VecTaggerSetInvert(VecTagger,PetscBool);
+PETSC_EXTERN PetscErrorCode VecTaggerGetInvert(VecTagger,PetscBool*);
+PETSC_EXTERN PetscErrorCode VecTaggerSetFromOptions(VecTagger);
+PETSC_EXTERN PetscErrorCode VecTaggerSetUp(VecTagger);
+PETSC_EXTERN PetscErrorCode VecTaggerView(VecTagger,PetscViewer);
+PETSC_EXTERN PetscErrorCode VecTaggerComputeIS(VecTagger,Vec,IS *);
+PETSC_EXTERN PetscErrorCode VecTaggerDestroy(VecTagger *);
+
+/*S
+   VecTaggerBox - A box range used to tag values.  For real scalars, this is just a closed interval; for complex scalars, the box is the closed region in the complex plane
+   such that real(min) <= real(z) <= real(max) and imag(min) <= imag(z) <= imag(max).  INF is an acceptable endpoint.
+
+   Level: beginner
+
+.seealso: VecTaggerComputeIntervals()
+S*/
+typedef struct {
+  PetscScalar min;
+  PetscScalar max;
+} VecTaggerBox;
+PETSC_EXTERN PetscErrorCode VecTaggerComputeBoxes(VecTagger,Vec,PetscInt *,VecTaggerBox **);
+
+
+PETSC_EXTERN PetscErrorCode VecTaggerAbsoluteSetBox(VecTagger,VecTaggerBox *);
+PETSC_EXTERN PetscErrorCode VecTaggerAbsoluteGetBox(VecTagger,const VecTaggerBox **);
+
+PETSC_EXTERN PetscErrorCode VecTaggerRelativeSetBox(VecTagger,VecTaggerBox *);
+PETSC_EXTERN PetscErrorCode VecTaggerRelativeGetBox(VecTagger,const VecTaggerBox **);
+
+PETSC_EXTERN PetscErrorCode VecTaggerCDFSetBox(VecTagger,VecTaggerBox *);
+PETSC_EXTERN PetscErrorCode VecTaggerCDFGetBox(VecTagger,const VecTaggerBox **);
+
+/*E
+  VecTaggerCDFMethod - Determines what method is used to compute absolute values from cumulative distribution values (e.g., what value is the preimage of .95 in the cdf).  Relevant only in parallel: in serial it is directly computed.
+
+  Level: advanced
+.seealso: VecTaggerCDFSetMethod(), VecTaggerCDFMethods
+E*/
+typedef enum {VECTAGGER_CDF_GATHER,VECTAGGER_CDF_ITERATIVE,VECTAGGER_CDF_NUM_METHODS} VecTaggerCDFMethod;
+PETSC_EXTERN const char *const VecTaggerCDFMethods[];
+
+PETSC_EXTERN PetscErrorCode VecTaggerCDFSetMethod(VecTagger,VecTaggerCDFMethod);
+PETSC_EXTERN PetscErrorCode VecTaggerCDFGetMethod(VecTagger,VecTaggerCDFMethod*);
+PETSC_EXTERN PetscErrorCode VecTaggerCDFIterativeSetTolerances(VecTagger,PetscInt,PetscReal,PetscReal);
+PETSC_EXTERN PetscErrorCode VecTaggerCDFIterativeGetTolerances(VecTagger,PetscInt*,PetscReal*,PetscReal*);
+
+PETSC_EXTERN PetscErrorCode VecTaggerOrSetSubs(VecTagger,PetscInt,VecTagger*,PetscCopyMode);
+PETSC_EXTERN PetscErrorCode VecTaggerOrGetSubs(VecTagger,PetscInt*,VecTagger**);
+
+PETSC_EXTERN PetscErrorCode VecTaggerAndSetSubs(VecTagger,PetscInt,VecTagger*,PetscCopyMode);
+PETSC_EXTERN PetscErrorCode VecTaggerAndGetSubs(VecTagger,PetscInt*,VecTagger**);
+
+PETSC_EXTERN PetscErrorCode VecTaggerInitializePackage(void);
+PETSC_EXTERN PetscErrorCode VecTaggerFinalizePackage(void);
+
+#endif
