@@ -3,26 +3,28 @@
 
 #define PETSC_VERSION_RELEASE    1
 #define PETSC_VERSION_MAJOR      3
-#define PETSC_VERSION_MINOR      7
-#define PETSC_VERSION_SUBMINOR   5
+#define PETSC_VERSION_MINOR      10
+#define PETSC_VERSION_SUBMINOR   3
 #define PETSC_VERSION_PATCH      0
-#define PETSC_RELEASE_DATE       "Apr, 25, 2016"
-#define PETSC_VERSION_DATE       "Jan, 01, 2017"
+#define PETSC_RELEASE_DATE       "Sep, 12, 2018"
+#define PETSC_VERSION_DATE       "Dec, 18, 2018"
 
 #if !defined (PETSC_VERSION_GIT)
-#define PETSC_VERSION_GIT        "v3.7.5"
+#define PETSC_VERSION_GIT        "v3.10.3"
 #endif
 
 #if !defined(PETSC_VERSION_DATE_GIT)
-#define PETSC_VERSION_DATE_GIT   "2017-01-01 09:58:23 -0600"
+#define PETSC_VERSION_DATE_GIT   "2018-12-18 11:21:49 -0600"
 #endif
 
-#define PETSC_VERSION_(MAJOR,MINOR,SUBMINOR) \
+#define PETSC_VERSION_EQ(MAJOR,MINOR,SUBMINOR) \
   ((PETSC_VERSION_MAJOR == (MAJOR)) &&       \
    (PETSC_VERSION_MINOR == (MINOR)) &&       \
    (PETSC_VERSION_SUBMINOR == (SUBMINOR)) && \
    (PETSC_VERSION_RELEASE  == 1))
 
+#define PETSC_VERSION_ PETSC_VERSION_EQ
+
 #define PETSC_VERSION_LT(MAJOR,MINOR,SUBMINOR)          \
   (PETSC_VERSION_RELEASE == 1 &&                        \
    (PETSC_VERSION_MAJOR < (MAJOR) ||                    \
@@ -33,7 +35,7 @@
 
 #define PETSC_VERSION_LE(MAJOR,MINOR,SUBMINOR) \
   (PETSC_VERSION_LT(MAJOR,MINOR,SUBMINOR) || \
-   PETSC_VERSION_(MAJOR,MINOR,SUBMINOR))
+   PETSC_VERSION_EQ(MAJOR,MINOR,SUBMINOR))
 
 #define PETSC_VERSION_GT(MAJOR,MINOR,SUBMINOR) \
   (0 == PETSC_VERSION_LE(MAJOR,MINOR,SUBMINOR))
