@@ -7,23 +7,17 @@ extern int BlastCache(void);
 extern int test1(void);
 extern int test2(void);
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   PetscErrorCode ierr;
 
-  PetscInitialize(&argc,&argv,0,0);
-
+  ierr = PetscInitialize(&argc,&argv,0,0);if (ierr) return ierr;
   ierr = test1();CHKERRQ(ierr);
   ierr = test2();CHKERRQ(ierr);
-
   ierr = PetscFinalize();
-  PetscFunctionReturn(0);
+  return ierr;
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "test1"
 int test1(void)
 {
   PetscLogDouble t1,t2;
@@ -40,8 +34,6 @@ int test1(void)
   ierr = PetscMalloc1(2000,&z);CHKERRQ(ierr);
   ierr = PetscMalloc1(2000,&zi);CHKERRQ(ierr);
 
-
-
   /* Take care of paging effects */
   ierr = PetscTime(&t1);CHKERRQ(ierr);
 
@@ -116,8 +108,6 @@ int test1(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "test2"
 int test2(void)
 {
   PetscLogDouble t1,t2;
@@ -190,8 +180,6 @@ int test2(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "BlastCache"
 int BlastCache(void)
 {
   int         i,ierr,n = 1000000;
