@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+from __future__ import print_function
 import os,sys
 sys.path.append(os.path.join(os.environ['PETSC_DIR'], 'config'))
 sys.path.append(os.getcwd())
@@ -26,9 +27,9 @@ class PETSc(object):
       return None
     return mpiexec
 
-  def example(self, num):
+  def example(self, library, num):
     '''Return the path to the executable for a given example number'''
-    return os.path.join(self.dir(), self.arch(), 'lib', 'ex'+str(num)+'-obj', 'ex'+str(num))
+    return os.path.join(self.dir(), self.arch(), 'lib', library.lower(), 'ex'+str(num))
 
   def source(self, library, num, filenametail):
     '''Return the path to the sources for a given example number'''
@@ -56,7 +57,7 @@ class PETScExample(object):
 
     Popen = subprocess.Popen
     PIPE  = subprocess.PIPE
-    if log: print 'Executing: %s\n' % (command,)
+    if log: print('Executing: %s\n' % (command,))
     pipe = Popen(command, cwd=cwd, stdin=None, stdout=PIPE, stderr=PIPE, bufsize=-1, shell=True, universal_newlines=True)
     (out, err) = pipe.communicate()
     ret = pipe.returncode
@@ -72,28 +73,50 @@ class PETScExample(object):
         a.append('-'+key+' '+str(value))
     return ' '.join(a)
 
+  def build(self, log = True):
+    sdir = os.path.join(self.petsc.dir(), 'src', self.library.lower(), 'examples', 'tutorials')
+    odir = os.getcwd()
+    os.chdir(sdir)
+    cmd = 'make ex'+str(self.num)
+    out, err, ret = self.runShellCommand(cmd, cwd = sdir, log = log)
+    if err:
+      raise RuntimeError('Unable to build example:\n'+err+out)
+    os.chdir(odir)
+    bdir = os.path.dirname(self.petsc.example(self.library, self.num))
+    try:
+      os.makedirs(bdir)
+    except OSError:
+      if not os.path.isdir(bdir):
+        raise
+    cmd = 'mv '+os.path.join(sdir, 'ex'+str(self.num))+' '+self.petsc.example(self.library, self.num)
+    out, err, ret = self.runShellCommand(cmd, log = log)
+    if ret:
+      print(err)
+      print(out)
+    return
+
   def run(self, numProcs = 1, log = True, **opts):
     cmd = ''
     if self.petsc.mpiexec() is not None:
       cmd += self.petsc.mpiexec() + ' '
       numProcs = os.environ.get('NUM_RANKS', numProcs)
       cmd += ' -n ' + str(numProcs) + ' '
-      if os.environ.has_key('PE_HOSTFILE'):
+      if 'PE_HOSTFILE' in os.environ:
         cmd += ' -hostfile hostfile '
-    cmd += ' '.join([self.petsc.example(self.num), self.optionsToString(**self.opts), self.optionsToString(**opts)])
+    cmd += ' '.join([self.petsc.example(self.library, self.num), self.optionsToString(**self.opts), self.optionsToString(**opts)])
     if 'batch' in opts and opts['batch']:
       del opts['batch']
       filename = generateBatchScript(self.num, numProcs, 120, ' '+self.optionsToString(**self.opts)+' '+self.optionsToString(**opts))
       # Submit job
       out, err, ret = self.runShellCommand('qsub -q gpu '+filename, log = log)
       if ret:
-        print err
-        print out
+        print(err)
+        print(out)
     else:
       out, err, ret = self.runShellCommand(cmd, log = log)
       if ret:
-        print err
-        print out
+        print(err)
+        print(out)
     return out
 
 def processSummary(moduleName, defaultStage, eventNames, times, events):
@@ -152,8 +175,8 @@ def plotEventTime(library, num, eventNames, sizes, times, events, filename = Non
         data.append(np.array(events[arch][bs][event])[:,0])
         data.append(color+style)
       else:
-        print 'Could not find %s in %s-%d events' % (event, arch, bs)
-  print data
+        print('Could not find %s in %s-%d events' % (event, arch, bs))
+  print(data)
   plot(*data)
   title('Performance on '+library+' Example '+str(num))
   xlabel('Number of Dof')
@@ -181,7 +204,7 @@ def plotEventFlop(library, num, eventNames, sizes, times, events, filename = Non
         data.append(1e-3*np.array(events[arch][bs][event])[:,1])
         data.append(color+style)
       else:
-        print 'Could not find %s in %s-%d events' % (event, arch, bs)
+        print('Could not find %s in %s-%d events' % (event, arch, bs))
   semilogy(*data)
   title('Performance on '+library+' Example '+str(num))
   xlabel('Number of Dof')
@@ -209,7 +232,7 @@ def plotEventScaling(library, num, eventNames, procs, events, filename = None):
         data.append(1e-3*np.array(events[arch][bs][event])[:,1])
         data.append(color+style)
       else:
-        print 'Could not find %s in %s-%d events' % (event, arch, bs)
+        print('Could not find %s in %s-%d events' % (event, arch, bs))
   plot(*data)
   title('Performance on '+library+' Example '+str(num))
   xlabel('Number of Processors')
@@ -254,8 +277,8 @@ def plotSummaryLine(library, num, eventNames, sizes, times, events):
           data.append(np.array(events[arch][bs][event])[:,0])
           data.append(color+style)
         else:
-          print 'Could not find %s in %s-%d events' % (event, arch, bs)
-    print data
+          print('Could not find %s in %s-%d events' % (event, arch, bs))
+    print(data)
     plot(*data)
     title('Performance on '+library+' Example '+str(num))
     xlabel('Number of Dof')
@@ -276,7 +299,7 @@ def plotSummaryLine(library, num, eventNames, sizes, times, events):
           data.append(np.array(events[arch][bs][event])[:,1])
           data.append(color+style)
         else:
-          print 'Could not find %s in %s-%d events' % (event, arch, bs)
+          print('Could not find %s in %s-%d events' % (event, arch, bs))
     plot(*data)
     title('Performance on '+library+' Example '+str(num))
     xlabel('Number of Dof')
@@ -418,7 +441,7 @@ if __name__ == '__main__':
 
   if args.daemon:
     import daemon
-    print 'Starting daemon'
+    print('Starting daemon')
     daemon.createDaemon('.')
 
   for run in args.runs:
