@@ -4,8 +4,6 @@ static char help[] = "Tests DMDALocalToGlocal() for dof > 1\n\n";
 #include <petscdm.h>
 #include <petscdmda.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   PetscInt       M = 6,N = 5,m = PETSC_DECIDE,n = PETSC_DECIDE,i,j,is,js,in,jen;
@@ -14,10 +12,11 @@ int main(int argc,char **argv)
   Vec            local,global;
   PetscScalar    ***l;
 
-  ierr = PetscInitialize(&argc,&argv,(char*)0,help);CHKERRQ(ierr);
-
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   /* Create distributed array and get vectors */
   ierr = DMDACreate2d(PETSC_COMM_WORLD, DM_BOUNDARY_NONE, DM_BOUNDARY_NONE,DMDA_STENCIL_BOX,M,N,m,n,3,1,NULL,NULL,&da);CHKERRQ(ierr);
+  ierr = DMSetFromOptions(da);CHKERRQ(ierr);
+  ierr = DMSetUp(da);CHKERRQ(ierr);
   ierr = DMCreateGlobalVector(da,&global);CHKERRQ(ierr);
   ierr = DMCreateLocalVector(da,&local);CHKERRQ(ierr);
 
@@ -41,6 +40,45 @@ int main(int argc,char **argv)
   ierr = VecDestroy(&global);CHKERRQ(ierr);
   ierr = DMDestroy(&da);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
+
+
+/*TEST
+
+      test:
+         filter: grep -v -i Process
+         output_file: output/ex24_1.out
+
+      test:
+         suffix: 1
+         nsize: 2
+         filter: grep -v -i Process
+         output_file: output/ex24_2.out
+
+      test:
+         suffix: 2
+         nsize: 3
+         filter: grep -v -i Process
+         output_file: output/ex24_2.out
+
+      test:
+         suffix: 3
+         nsize: 4
+         filter: grep -v -i Process
+         output_file: output/ex24_2.out
+
+      test:
+         suffix: 4
+         nsize: 5
+         filter: grep -v -i Process
+         output_file: output/ex24_2.out
+
+      test:
+         suffix: 5
+         nsize: 6
+         filter: grep -v -i Process
+         output_file: output/ex24_2.out
+
+TEST*/
