@@ -15,7 +15,7 @@ CCmplx CCmplxPow(CCmplx a,PetscReal n)
   CCmplx b;
   PetscReal r,theta;
   r      = PetscSqrtReal(a.real*a.real + a.imag*a.imag);
-  theta  = atan2(a.imag,a.real);
+  theta  = PetscAtan2Real(a.imag,a.real);
   b.real = PetscPowReal(r,n) * PetscCosReal(n*theta);
   b.imag = PetscPowReal(r,n) * PetscSinReal(n*theta);
   return b;
@@ -32,7 +32,7 @@ CCmplx CCmplxSqrt(CCmplx a)
   CCmplx b;
   PetscReal r,theta;
   r      = PetscSqrtReal(a.real*a.real + a.imag*a.imag);
-  theta  = atan2(a.imag,a.real);
+  theta  = PetscAtan2Real(a.imag,a.real);
   b.real = PetscSqrtReal(r) * PetscCosReal(0.5*theta);
   b.imag = PetscSqrtReal(r) * PetscSinReal(0.5*theta);
   return b;
@@ -53,8 +53,6 @@ PetscScalar CCmplxIm(CCmplx a)
   return (PetscScalar)a.imag;
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DAApplyConformalMapping"
 PetscErrorCode DAApplyConformalMapping(DM da,PetscInt idx)
 {
   PetscErrorCode ierr;
@@ -198,8 +196,6 @@ PetscErrorCode DAApplyConformalMapping(DM da,PetscInt idx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DAApplyTrilinearMapping"
 PetscErrorCode DAApplyTrilinearMapping(DM da)
 {
   PetscErrorCode ierr;
@@ -256,8 +252,6 @@ PetscErrorCode DAApplyTrilinearMapping(DM da)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DADefineXLinearField2D"
 PetscErrorCode DADefineXLinearField2D(DM da,Vec field)
 {
   PetscErrorCode ierr;
@@ -288,8 +282,6 @@ PetscErrorCode DADefineXLinearField2D(DM da,Vec field)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DADefineXLinearField3D"
 PetscErrorCode DADefineXLinearField3D(DM da,Vec field)
 {
   PetscErrorCode ierr;
@@ -329,8 +321,6 @@ PetscErrorCode DADefineXLinearField3D(DM da,Vec field)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "da_test_RefineCoords1D"
 PetscErrorCode da_test_RefineCoords1D(PetscInt mx)
 {
   PetscErrorCode ierr;
@@ -343,13 +333,9 @@ PetscErrorCode da_test_RefineCoords1D(PetscInt mx)
   PetscBool      output = PETSC_FALSE;
 
   PetscFunctionBeginUser;
-  ierr = DMDACreate1d(PETSC_COMM_WORLD, DM_BOUNDARY_NONE,
-                      mx+1,
-                      1, /* 1 dof */
-                      1, /* stencil = 1 */
-                      NULL,
-                      &dac);CHKERRQ(ierr);
+  ierr = DMDACreate1d(PETSC_COMM_WORLD, DM_BOUNDARY_NONE,mx+1,1, /* 1 dof */1, /* stencil = 1 */NULL,&dac);CHKERRQ(ierr);
   ierr = DMSetFromOptions(dac);CHKERRQ(ierr);
+  ierr = DMSetUp(dac);CHKERRQ(ierr);
 
   ierr = DMRefine(dac,MPI_COMM_NULL,&daf);CHKERRQ(ierr);
   ierr = DMDAGetInfo(daf,0,&Mx,0,0,0,0,0,0,0,0,0,0,0);CHKERRQ(ierr);
@@ -397,18 +383,20 @@ PetscErrorCode da_test_RefineCoords1D(PetscInt mx)
     ierr = VecDestroy(&afexact);CHKERRQ(ierr);
   }
 
-  PetscOptionsGetBool(NULL,"-output",&output,NULL);CHKERRQ(ierr);
+  PetscOptionsGetBool(NULL,NULL,"-output",&output,NULL);CHKERRQ(ierr);
   if (output) {
     ierr = PetscViewerASCIIOpen(PETSC_COMM_WORLD, "dac_1D.vtk", &vv);CHKERRQ(ierr);
-    ierr = PetscViewerSetFormat(vv, PETSC_VIEWER_ASCII_VTK);CHKERRQ(ierr);
+    ierr = PetscViewerPushFormat(vv, PETSC_VIEWER_ASCII_VTK);CHKERRQ(ierr);
     ierr = DMView(dac, vv);CHKERRQ(ierr);
     ierr = VecView(ac, vv);CHKERRQ(ierr);
+    ierr = PetscViewerPopFormat(vv);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&vv);CHKERRQ(ierr);
 
     ierr = PetscViewerASCIIOpen(PETSC_COMM_WORLD, "daf_1D.vtk", &vv);CHKERRQ(ierr);
-    ierr = PetscViewerSetFormat(vv, PETSC_VIEWER_ASCII_VTK);CHKERRQ(ierr);
+    ierr = PetscViewerPushFormat(vv, PETSC_VIEWER_ASCII_VTK);CHKERRQ(ierr);
     ierr = DMView(daf, vv);CHKERRQ(ierr);
     ierr = VecView(af, vv);CHKERRQ(ierr);
+    ierr = PetscViewerPopFormat(vv);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&vv);CHKERRQ(ierr);
   }
 
@@ -420,8 +408,6 @@ PetscErrorCode da_test_RefineCoords1D(PetscInt mx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "da_test_RefineCoords2D"
 PetscErrorCode da_test_RefineCoords2D(PetscInt mx,PetscInt my)
 {
   PetscErrorCode ierr;
@@ -434,14 +420,9 @@ PetscErrorCode da_test_RefineCoords2D(PetscInt mx,PetscInt my)
   PetscBool      output = PETSC_FALSE;
 
   PetscFunctionBeginUser;
-  ierr = DMDACreate2d(PETSC_COMM_WORLD, DM_BOUNDARY_NONE, DM_BOUNDARY_NONE, DMDA_STENCIL_BOX,
-                      mx+1, my+1,
-                      PETSC_DECIDE, PETSC_DECIDE,
-                      1, /* 1 dof */
-                      1, /* stencil = 1 */
-                      NULL, NULL,
-                      &dac);CHKERRQ(ierr);
+  ierr = DMDACreate2d(PETSC_COMM_WORLD, DM_BOUNDARY_NONE, DM_BOUNDARY_NONE, DMDA_STENCIL_BOX,mx+1,my+1,PETSC_DECIDE, PETSC_DECIDE,1, /* 1 dof */1, /* stencil = 1 */NULL, NULL,&dac);CHKERRQ(ierr);
   ierr = DMSetFromOptions(dac);CHKERRQ(ierr);
+  ierr = DMSetUp(dac);CHKERRQ(ierr);
 
   ierr = DMRefine(dac,MPI_COMM_NULL,&daf);CHKERRQ(ierr);
   ierr = DMDAGetInfo(daf,0,&Mx,&My,0,0,0,0,0,0,0,0,0,0);CHKERRQ(ierr);
@@ -452,7 +433,7 @@ PetscErrorCode da_test_RefineCoords2D(PetscInt mx,PetscInt my)
 
   /* apply conformal mappings */
   map_id = 0;
-  ierr   = PetscOptionsGetInt(NULL,"-cmap", &map_id,NULL);CHKERRQ(ierr);
+  ierr   = PetscOptionsGetInt(NULL,NULL,"-cmap", &map_id,NULL);CHKERRQ(ierr);
   if (map_id >= 1) {
     ierr = DAApplyConformalMapping(dac,map_id);CHKERRQ(ierr);
   }
@@ -497,18 +478,20 @@ PetscErrorCode da_test_RefineCoords2D(PetscInt mx,PetscInt my)
     ierr = VecDestroy(&afexact);CHKERRQ(ierr);
   }
 
-  PetscOptionsGetBool(NULL,"-output",&output,NULL);CHKERRQ(ierr);
+  PetscOptionsGetBool(NULL,NULL,"-output",&output,NULL);CHKERRQ(ierr);
   if (output) {
     ierr = PetscViewerASCIIOpen(PETSC_COMM_WORLD, "dac_2D.vtk", &vv);CHKERRQ(ierr);
-    ierr = PetscViewerSetFormat(vv, PETSC_VIEWER_ASCII_VTK);CHKERRQ(ierr);
+    ierr = PetscViewerPushFormat(vv, PETSC_VIEWER_ASCII_VTK);CHKERRQ(ierr);
     ierr = DMView(dac, vv);CHKERRQ(ierr);
     ierr = VecView(ac, vv);CHKERRQ(ierr);
+    ierr = PetscViewerPopFormat(vv);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&vv);CHKERRQ(ierr);
 
     ierr = PetscViewerASCIIOpen(PETSC_COMM_WORLD, "daf_2D.vtk", &vv);CHKERRQ(ierr);
-    ierr = PetscViewerSetFormat(vv, PETSC_VIEWER_ASCII_VTK);CHKERRQ(ierr);
+    ierr = PetscViewerPushFormat(vv, PETSC_VIEWER_ASCII_VTK);CHKERRQ(ierr);
     ierr = DMView(daf, vv);CHKERRQ(ierr);
     ierr = VecView(af, vv);CHKERRQ(ierr);
+    ierr = PetscViewerPopFormat(vv);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&vv);CHKERRQ(ierr);
   }
 
@@ -520,8 +503,6 @@ PetscErrorCode da_test_RefineCoords2D(PetscInt mx,PetscInt my)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "da_test_RefineCoords3D"
 PetscErrorCode da_test_RefineCoords3D(PetscInt mx,PetscInt my,PetscInt mz)
 {
   PetscErrorCode ierr;
@@ -534,14 +515,10 @@ PetscErrorCode da_test_RefineCoords3D(PetscInt mx,PetscInt my,PetscInt mz)
   PetscBool      output = PETSC_FALSE;
 
   PetscFunctionBeginUser;
-  ierr = DMDACreate3d(PETSC_COMM_WORLD,DM_BOUNDARY_NONE,DM_BOUNDARY_NONE,DM_BOUNDARY_NONE,DMDA_STENCIL_BOX,
-                      mx+1, my+1,mz+1,
-                      PETSC_DECIDE, PETSC_DECIDE,PETSC_DECIDE,
-                      1, /* 1 dof */
-                      1, /* stencil = 1 */
-                      NULL,NULL,NULL,
-                      &dac);CHKERRQ(ierr);
+  ierr = DMDACreate3d(PETSC_COMM_WORLD,DM_BOUNDARY_NONE,DM_BOUNDARY_NONE,DM_BOUNDARY_NONE,DMDA_STENCIL_BOX,mx+1, my+1,mz+1,PETSC_DECIDE, PETSC_DECIDE,PETSC_DECIDE,1, /* 1 dof */
+                      1, /* stencil = 1 */NULL,NULL,NULL,&dac);CHKERRQ(ierr);
   ierr = DMSetFromOptions(dac);CHKERRQ(ierr);
+  ierr = DMSetUp(dac);CHKERRQ(ierr);
 
   ierr = DMRefine(dac,MPI_COMM_NULL,&daf);CHKERRQ(ierr);
   ierr = DMDAGetInfo(daf,0,&Mx,&My,&Mz,0,0,0,0,0,0,0,0,0);CHKERRQ(ierr);
@@ -554,7 +531,7 @@ PetscErrorCode da_test_RefineCoords3D(PetscInt mx,PetscInt my,PetscInt mz)
   /*ierr = DAApplyTrilinearMapping(dac);CHKERRQ(ierr);*/
   /* apply conformal mappings */
   map_id = 0;
-  ierr   = PetscOptionsGetInt(NULL,"-cmap", &map_id,NULL);CHKERRQ(ierr);
+  ierr   = PetscOptionsGetInt(NULL,NULL,"-cmap", &map_id,NULL);CHKERRQ(ierr);
   if (map_id >= 1) {
     ierr = DAApplyConformalMapping(dac,map_id);CHKERRQ(ierr);
   }
@@ -601,18 +578,20 @@ PetscErrorCode da_test_RefineCoords3D(PetscInt mx,PetscInt my,PetscInt mz)
     ierr = VecDestroy(&afexact);CHKERRQ(ierr);
   }
 
-  PetscOptionsGetBool(NULL,"-output",&output,NULL);CHKERRQ(ierr);
+  PetscOptionsGetBool(NULL,NULL,"-output",&output,NULL);CHKERRQ(ierr);
   if (output) {
     ierr = PetscViewerASCIIOpen(PETSC_COMM_WORLD, "dac_3D.vtk", &vv);CHKERRQ(ierr);
-    ierr = PetscViewerSetFormat(vv, PETSC_VIEWER_ASCII_VTK);CHKERRQ(ierr);
+    ierr = PetscViewerPushFormat(vv, PETSC_VIEWER_ASCII_VTK);CHKERRQ(ierr);
     ierr = DMView(dac, vv);CHKERRQ(ierr);
     ierr = VecView(ac, vv);CHKERRQ(ierr);
+    ierr = PetscViewerPopFormat(vv);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&vv);CHKERRQ(ierr);
 
     ierr = PetscViewerASCIIOpen(PETSC_COMM_WORLD, "daf_3D.vtk", &vv);CHKERRQ(ierr);
-    ierr = PetscViewerSetFormat(vv, PETSC_VIEWER_ASCII_VTK);CHKERRQ(ierr);
+    ierr = PetscViewerPushFormat(vv, PETSC_VIEWER_ASCII_VTK);CHKERRQ(ierr);
     ierr = DMView(daf, vv);CHKERRQ(ierr);
     ierr = VecView(af, vv);CHKERRQ(ierr);
+    ierr = PetscViewerPopFormat(vv);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&vv);CHKERRQ(ierr);
   }
 
@@ -624,23 +603,19 @@ PetscErrorCode da_test_RefineCoords3D(PetscInt mx,PetscInt my,PetscInt mz)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   PetscErrorCode ierr;
-  PetscInt       mx,my,mz,l,nl,dim;
+  PetscInt       mx = 2,my = 2,mz = 2,l,nl,dim;
 
-  ierr = PetscInitialize(&argc,&argv,0,help);
-
-  mx = my = mz = 2;
-  PetscOptionsGetInt(NULL,"-mx", &mx, 0);
-  PetscOptionsGetInt(NULL,"-my", &my, 0);
-  PetscOptionsGetInt(NULL,"-mz", &mz, 0);
+  ierr = PetscInitialize(&argc,&argv,0,help);if (ierr) return ierr;
+  PetscOptionsGetInt(NULL,NULL,"-mx", &mx, 0);
+  PetscOptionsGetInt(NULL,NULL,"-my", &my, 0);
+  PetscOptionsGetInt(NULL,NULL,"-mz", &mz, 0);
   nl = 1;
-  PetscOptionsGetInt(NULL,"-nl", &nl, 0);
+  PetscOptionsGetInt(NULL,NULL,"-nl", &nl, 0);
   dim = 2;
-  PetscOptionsGetInt(NULL,"-dim", &dim, 0);
+  PetscOptionsGetInt(NULL,NULL,"-dim", &dim, 0);
 
   for (l=0; l<nl; l++) {
     if      (dim==1) da_test_RefineCoords1D(mx);
@@ -650,7 +625,51 @@ int main(int argc,char **argv)
     my = my * 2;
     mz = mz * 2;
   }
-
-  ierr = PetscFinalize();CHKERRQ(ierr);
-  return 0;
+  ierr = PetscFinalize();
+  return ierr;
 }
+
+
+/*TEST
+
+   test:
+      suffix: 1d
+      args: -mx 10 -nl 6 -dim 1
+
+   test:
+      suffix: 2d
+
+      test:
+         args: -mx 10 -my 10 -nl 6 -dim 2 -cmap 0
+
+      test:
+         args: -mx 10 -my 10 -nl 6 -dim 2 -cmap 1
+
+      test:
+         args: -mx 10 -my 10 -nl 6 -dim 2 -cmap 2
+
+      test:
+         args: -mx 10 -my 10 -nl 6 -dim 2 -cmap 3
+
+   test:
+      suffix: 2dp1
+      nsize: 8
+      args: -mx 10 -my 10 -nl 4 -dim 2 -cmap 3 -da_refine_x 3 -da_refine_y 4
+      timeoutfactor: 2
+
+   test:
+      suffix: 2dp2
+      nsize: 8
+      args: -mx 10 -my 10 -nl 4 -dim 2 -cmap 3 -da_refine_x 3 -da_refine_y 1
+      timeoutfactor: 2
+
+   test:
+      suffix: 3d
+      args: -mx 5 -my 5 -mz 5 -nl 4 -dim 3 -cmap 3
+
+   test:
+      suffix: 3dp1
+      nsize: 32
+      args: -mx 5 -my 5 -mz 5 -nl 3 -dim 3 -cmap 1 -da_refine_x 1 -da_refine_y 3 -da_refine_z 4
+
+TEST*/
