@@ -12,8 +12,6 @@ Use the options
 #include <petscdm.h>
 #include <petscdmda.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "FillLocalSubdomain"
 PetscErrorCode FillLocalSubdomain(DM da, Vec gvec)
 {
   DMDALocalInfo  info;
@@ -58,28 +56,21 @@ PetscErrorCode FillLocalSubdomain(DM da, Vec gvec)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   PetscErrorCode ierr;
   DM             da,*subda;
+  PetscInt       i,dim=3;
+  PetscMPIInt    size,rank;
+  Vec            v;
+  Vec            slvec,sgvec;
+  IS             *ois,*iis;
+  VecScatter     oscata;
+  VecScatter     *iscat,*oscat,*gscat;
+  DMDALocalInfo  info;
 
-  PetscInt    i,dim=3;
-  PetscMPIInt size,rank;
-
-  Vec v;
-  Vec slvec,sgvec;
-
-  IS         *ois,*iis;
-  VecScatter oscata;
-  VecScatter *iscat,*oscat,*gscat;
-
-  DMDALocalInfo info;
-
-  ierr = PetscInitialize(&argc,&argv,(char*)0,help);CHKERRQ(ierr);
-
-  ierr = PetscOptionsGetInt(NULL,"-dim",&dim,NULL);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
+  ierr = PetscOptionsGetInt(NULL,NULL,"-dim",&dim,NULL);CHKERRQ(ierr);
 
   /* Create distributed array and get vectors */
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
@@ -89,7 +80,8 @@ int main(int argc,char **argv)
   } else if (dim == 3) {
     ierr = DMDACreate3d(PETSC_COMM_WORLD, DM_BOUNDARY_NONE, DM_BOUNDARY_NONE,DM_BOUNDARY_NONE,DMDA_STENCIL_STAR,-4,-4,-4,PETSC_DECIDE,PETSC_DECIDE,PETSC_DECIDE,3,1,NULL,NULL,NULL,&da);CHKERRQ(ierr);
   }
-
+  ierr = DMSetFromOptions(da);CHKERRQ(ierr);
+  ierr = DMSetUp(da);CHKERRQ(ierr);
   ierr = DMDAGetLocalInfo(da,&info);CHKERRQ(ierr);
 
   ierr = DMCreateDomainDecomposition(da,NULL,NULL,&iis,&ois,&subda);CHKERRQ(ierr);
@@ -245,6 +237,6 @@ int main(int argc,char **argv)
 
   ierr = DMDestroy(&da);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
