@@ -1,20 +1,20 @@
 #include <petsc/private/dmdaimpl.h>    /*I   "petscdmda.h"   I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetSizes"
 /*@
-  DMDASetSizes - Sets the global sizes
+  DMDASetSizes - Sets the number of grid points in the three dimensional directions
 
   Logically Collective on DMDA
 
   Input Parameters:
 + da - the DMDA
-. M - the global X size (or PETSC_DECIDE)
-. N - the global Y size (or PETSC_DECIDE)
-- P - the global Z size (or PETSC_DECIDE)
+. M - the global X size
+. N - the global Y size
+- P - the global Z size
 
   Level: intermediate
 
+  Developer Notes: Since the dimension may not yet have been set the code cannot error check for non-positive Y and Z number of grid points
+
 .seealso: DMDAGetSize(), PetscSplitOwnership()
 @*/
 PetscErrorCode  DMDASetSizes(DM da, PetscInt M, PetscInt N, PetscInt P)
@@ -22,11 +22,14 @@ PetscErrorCode  DMDASetSizes(DM da, PetscInt M, PetscInt N, PetscInt P)
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da, DM_CLASSID, 1);
+  PetscValidHeaderSpecificType(da, DM_CLASSID, 1,DMDA);
   PetscValidLogicalCollectiveInt(da,M,2);
   PetscValidLogicalCollectiveInt(da,N,3);
   PetscValidLogicalCollectiveInt(da,P,4);
   if (da->setupcalled) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_ARG_WRONGSTATE,"This function must be called before DMSetUp()");
+  if (M < 1) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_ARG_SIZ,"Number of grid points in X direction must be positive");
+  if (N < 0) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_ARG_SIZ,"Number of grid points in Y direction must be positive");
+  if (P < 0) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_ARG_SIZ,"Number of grid points in Z direction must be positive");
 
   dd->M = M;
   dd->N = N;
@@ -34,8 +37,6 @@ PetscErrorCode  DMDASetSizes(DM da, PetscInt M, PetscInt N, PetscInt P)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetNumProcs"
 /*@
   DMDASetNumProcs - Sets the number of processes in each dimension
 
@@ -57,7 +58,7 @@ PetscErrorCode  DMDASetNumProcs(DM da, PetscInt m, PetscInt n, PetscInt p)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da, DM_CLASSID, 1);
+  PetscValidHeaderSpecificType(da, DM_CLASSID, 1,DMDA);
   PetscValidLogicalCollectiveInt(da,m,2);
   PetscValidLogicalCollectiveInt(da,n,3);
   PetscValidLogicalCollectiveInt(da,p,4);
@@ -80,8 +81,6 @@ PetscErrorCode  DMDASetNumProcs(DM da, PetscInt m, PetscInt n, PetscInt p)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetBoundaryType"
 /*@
   DMDASetBoundaryType - Sets the type of ghost nodes on domain boundaries.
 
@@ -101,7 +100,7 @@ PetscErrorCode  DMDASetBoundaryType(DM da,DMBoundaryType bx,DMBoundaryType by,DM
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidLogicalCollectiveEnum(da,bx,2);
   PetscValidLogicalCollectiveEnum(da,by,3);
   PetscValidLogicalCollectiveEnum(da,bz,4);
@@ -112,28 +111,26 @@ PetscErrorCode  DMDASetBoundaryType(DM da,DMBoundaryType bx,DMBoundaryType by,DM
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetDof"
 /*@
   DMDASetDof - Sets the number of degrees of freedom per vertex
 
   Not collective
 
-  Input Parameter:
+  Input Parameters:
 + da  - The DMDA
 - dof - Number of degrees of freedom
 
   Level: intermediate
 
 .keywords:  distributed array, degrees of freedom
-.seealso: DMDACreate(), DMDestroy(), DMDA
+.seealso: DMDAGetDof(), DMDACreate(), DMDestroy(), DMDA
 @*/
 PetscErrorCode  DMDASetDof(DM da, PetscInt dof)
 {
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidLogicalCollectiveInt(da,dof,2);
   if (da->setupcalled) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_ARG_WRONGSTATE,"This function must be called before DMSetUp()");
   dd->w  = dof;
@@ -141,8 +138,33 @@ PetscErrorCode  DMDASetDof(DM da, PetscInt dof)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetOverlap"
+/*@
+  DMDAGetDof - Gets the number of degrees of freedom per vertex
+
+  Not collective
+
+  Input Parameter:
+. da  - The DMDA
+
+  Output Parameter:
+. dof - Number of degrees of freedom
+
+  Level: intermediate
+
+.keywords:  distributed array, degrees of freedom
+.seealso: DMDASetDof(), DMDACreate(), DMDestroy(), DMDA
+@*/
+PetscErrorCode DMDAGetDof(DM da, PetscInt *dof)
+{
+  DM_DA *dd = (DM_DA *) da->data;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
+  PetscValidPointer(dof,2);
+  *dof = dd->w;
+  PetscFunctionReturn(0);
+}
+
 /*@
   DMDAGetOverlap - Gets the size of the per-processor overlap.
 
@@ -166,15 +188,13 @@ PetscErrorCode  DMDAGetOverlap(DM da,PetscInt *x,PetscInt *y,PetscInt *z)
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   if (x) *x = dd->xol;
   if (y) *y = dd->yol;
   if (z) *z = dd->zol;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetOverlap"
 /*@
   DMDASetOverlap - Sets the size of the per-processor overlap.
 
@@ -196,7 +216,7 @@ PetscErrorCode  DMDASetOverlap(DM da,PetscInt x,PetscInt y,PetscInt z)
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidLogicalCollectiveInt(da,x,2);
   PetscValidLogicalCollectiveInt(da,y,3);
   PetscValidLogicalCollectiveInt(da,z,4);
@@ -207,8 +227,6 @@ PetscErrorCode  DMDASetOverlap(DM da,PetscInt x,PetscInt y,PetscInt z)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetNumLocalSubDomains"
 /*@
   DMDAGetNumLocalSubDomains - Gets the number of local subdomains created upon decomposition.
 
@@ -230,13 +248,11 @@ PetscErrorCode  DMDAGetNumLocalSubDomains(DM da,PetscInt *Nsub)
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   if (Nsub) *Nsub = dd->Nsub;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetNumLocalSubDomains"
 /*@
   DMDASetNumLocalSubDomains - Sets the number of local subdomains created upon decomposition.
 
@@ -256,14 +272,12 @@ PetscErrorCode  DMDASetNumLocalSubDomains(DM da,PetscInt Nsub)
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidLogicalCollectiveInt(da,Nsub,2);
   dd->Nsub = Nsub;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetOffset"
 /*@
   DMDASetOffset - Sets the index offset of the DA.
 
@@ -277,7 +291,8 @@ PetscErrorCode  DMDASetNumLocalSubDomains(DM da,PetscInt Nsub)
 
   Level: intermediate
 
-  Notes: This is used primarily to overlap a computation on a local DA with that on a global DA without
+  Notes:
+    This is used primarily to overlap a computation on a local DA with that on a global DA without
   changing boundary conditions or subdomain features that depend upon the global offsets.
 
 .keywords:  distributed array, degrees of freedom
@@ -289,7 +304,7 @@ PetscErrorCode  DMDASetOffset(DM da, PetscInt xo, PetscInt yo, PetscInt zo, Pets
   DM_DA          *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidLogicalCollectiveInt(da,xo,2);
   PetscValidLogicalCollectiveInt(da,yo,3);
   PetscValidLogicalCollectiveInt(da,zo,4);
@@ -309,8 +324,6 @@ PetscErrorCode  DMDASetOffset(DM da, PetscInt xo, PetscInt yo, PetscInt zo, Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetOffset"
 /*@
   DMDAGetOffset - Gets the index offset of the DA.
 
@@ -337,7 +350,7 @@ PetscErrorCode  DMDAGetOffset(DM da,PetscInt *xo,PetscInt *yo,PetscInt *zo,Petsc
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   if (xo) *xo = dd->xo;
   if (yo) *yo = dd->yo;
   if (zo) *zo = dd->zo;
@@ -347,8 +360,6 @@ PetscErrorCode  DMDAGetOffset(DM da,PetscInt *xo,PetscInt *yo,PetscInt *zo,Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetNonOverlappingRegion"
 /*@
   DMDAGetNonOverlappingRegion - Gets the indices of the nonoverlapping region of a subdomain DM.
 
@@ -375,7 +386,7 @@ PetscErrorCode  DMDAGetNonOverlappingRegion(DM da, PetscInt *xs, PetscInt *ys, P
   DM_DA          *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   if (xs) *xs = dd->nonxs;
   if (ys) *ys = dd->nonys;
   if (zs) *zs = dd->nonzs;
@@ -386,8 +397,6 @@ PetscErrorCode  DMDAGetNonOverlappingRegion(DM da, PetscInt *xs, PetscInt *ys, P
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetNonOverlappingRegion"
 /*@
   DMDASetNonOverlappingRegion - Sets the indices of the nonoverlapping region of a subdomain DM.
 
@@ -412,7 +421,7 @@ PetscErrorCode  DMDASetNonOverlappingRegion(DM da, PetscInt xs, PetscInt ys, Pet
   DM_DA          *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidLogicalCollectiveInt(da,xs,2);
   PetscValidLogicalCollectiveInt(da,ys,3);
   PetscValidLogicalCollectiveInt(da,zs,4);
@@ -429,8 +438,6 @@ PetscErrorCode  DMDASetNonOverlappingRegion(DM da, PetscInt xs, PetscInt ys, Pet
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetStencilType"
 /*@
   DMDASetStencilType - Sets the type of the communication stencil
 
@@ -450,15 +457,40 @@ PetscErrorCode  DMDASetStencilType(DM da, DMDAStencilType stype)
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidLogicalCollectiveEnum(da,stype,2);
   if (da->setupcalled) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_ARG_WRONGSTATE,"This function must be called before DMSetUp()");
   dd->stencil_type = stype;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetStencilWidth"
+/*@
+  DMDAGetStencilType - Gets the type of the communication stencil
+
+  Not collective
+
+  Input Parameter:
+. da    - The DMDA
+
+  Output Parameter:
+. stype - The stencil type, use either DMDA_STENCIL_BOX or DMDA_STENCIL_STAR.
+
+  Level: intermediate
+
+.keywords:  distributed array, stencil
+.seealso: DMDACreate(), DMDestroy(), DMDA
+@*/
+PetscErrorCode DMDAGetStencilType(DM da, DMDAStencilType *stype)
+{
+  DM_DA *dd = (DM_DA*)da->data;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
+  PetscValidPointer(stype,2);
+  *stype = dd->stencil_type;
+  PetscFunctionReturn(0);
+}
+
 /*@
   DMDASetStencilWidth - Sets the width of the communication stencil
 
@@ -478,15 +510,40 @@ PetscErrorCode  DMDASetStencilWidth(DM da, PetscInt width)
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidLogicalCollectiveInt(da,width,2);
   if (da->setupcalled) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_ARG_WRONGSTATE,"This function must be called before DMSetUp()");
   dd->s = width;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDACheckOwnershipRanges_Private"
+/*@
+  DMDAGetStencilWidth - Gets the width of the communication stencil
+
+  Not collective
+
+  Input Parameter:
+. da    - The DMDA
+
+  Output Parameter:
+. width - The stencil width
+
+  Level: intermediate
+
+.keywords:  distributed array, stencil
+.seealso: DMDACreate(), DMDestroy(), DMDA
+@*/
+PetscErrorCode DMDAGetStencilWidth(DM da, PetscInt *width)
+{
+  DM_DA *dd = (DM_DA *) da->data;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
+  PetscValidPointer(width,2);
+  *width = dd->s;
+  PetscFunctionReturn(0);
+}
+
 static PetscErrorCode DMDACheckOwnershipRanges_Private(DM da,PetscInt M,PetscInt m,const PetscInt lx[])
 {
   PetscInt i,sum;
@@ -498,8 +555,6 @@ static PetscErrorCode DMDACheckOwnershipRanges_Private(DM da,PetscInt M,PetscInt
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetOwnershipRanges"
 /*@
   DMDASetOwnershipRanges - Sets the number of nodes in each direction on each process
 
@@ -524,7 +579,7 @@ PetscErrorCode  DMDASetOwnershipRanges(DM da, const PetscInt lx[], const PetscIn
   DM_DA          *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   if (da->setupcalled) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_ARG_WRONGSTATE,"This function must be called before DMSetUp()");
   if (lx) {
     if (dd->m < 0) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_ARG_WRONGSTATE,"Cannot set ownership ranges before setting number of procs");
@@ -553,8 +608,6 @@ PetscErrorCode  DMDASetOwnershipRanges(DM da, const PetscInt lx[], const PetscIn
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetInterpolationType"
 /*@
        DMDASetInterpolationType - Sets the type of interpolation that will be
           returned by DMCreateInterpolation()
@@ -567,7 +620,8 @@ PetscErrorCode  DMDASetOwnershipRanges(DM da, const PetscInt lx[], const PetscIn
 
    Level: intermediate
 
-   Notes: you should call this on the coarser of the two DMDAs you pass to DMCreateInterpolation()
+   Notes:
+    you should call this on the coarser of the two DMDAs you pass to DMCreateInterpolation()
 
 .keywords:  distributed array, interpolation
 
@@ -578,14 +632,12 @@ PetscErrorCode  DMDASetInterpolationType(DM da,DMDAInterpolationType ctype)
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidLogicalCollectiveEnum(da,ctype,2);
   dd->interptype = ctype;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetInterpolationType"
 /*@
        DMDAGetInterpolationType - Gets the type of interpolation that will be
           used by DMCreateInterpolation()
@@ -609,14 +661,12 @@ PetscErrorCode  DMDAGetInterpolationType(DM da,DMDAInterpolationType *ctype)
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidPointer(ctype,2);
   *ctype = dd->interptype;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetNeighbors"
 /*@C
       DMDAGetNeighbors - Gets an array containing the MPI rank of all the current
         processes neighbors.
@@ -630,11 +680,13 @@ PetscErrorCode  DMDAGetInterpolationType(DM da,DMDAInterpolationType *ctype)
 .     ranks - the neighbors ranks, stored with the x index increasing most rapidly.
               this process itself is in the list
 
-   Notes: In 2d the array is of length 9, in 3d of length 27
+   Notes:
+    In 2d the array is of length 9, in 3d of length 27
           Not supported in 1d
           Do not free the array, it is freed when the DMDA is destroyed.
 
-   Fortran Notes: In fortran you must pass in an array of the appropriate length.
+   Fortran Notes:
+    In fortran you must pass in an array of the appropriate length.
 
    Level: intermediate
 
@@ -644,13 +696,11 @@ PetscErrorCode  DMDAGetNeighbors(DM da,const PetscMPIInt *ranks[])
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   *ranks = dd->neighbors;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetOwnershipRanges"
 /*@C
       DMDAGetOwnershipRanges - Gets the ranges of indices in the x, y and z direction that are owned by each process
 
@@ -676,6 +726,8 @@ PetscErrorCode  DMDAGetNeighbors(DM da,const PetscMPIInt *ranks[])
 
     These numbers are NOT multiplied by the number of dof per node.
 
+     Not available from Fortran
+
 .seealso: DMDAGetCorners(), DMDAGetGhostCorners(), DMDACreate(), DMDACreate1d(), DMDACreate2d(), DMDACreate3d(), VecGetOwnershipRanges()
 @*/
 PetscErrorCode  DMDAGetOwnershipRanges(DM da,const PetscInt *lx[],const PetscInt *ly[],const PetscInt *lz[])
@@ -683,15 +735,13 @@ PetscErrorCode  DMDAGetOwnershipRanges(DM da,const PetscInt *lx[],const PetscInt
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   if (lx) *lx = dd->lx;
   if (ly) *ly = dd->ly;
   if (lz) *lz = dd->lz;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetRefinementFactor"
 /*@
      DMDASetRefinementFactor - Set the ratios that the DMDA grid is refined
 
@@ -710,7 +760,8 @@ PetscErrorCode  DMDAGetOwnershipRanges(DM da,const PetscInt *lx[],const PetscInt
 
   Level: intermediate
 
-    Notes: Pass PETSC_IGNORE to leave a value unchanged
+    Notes:
+    Pass PETSC_IGNORE to leave a value unchanged
 
 .seealso: DMRefine(), DMDAGetRefinementFactor()
 @*/
@@ -719,7 +770,7 @@ PetscErrorCode  DMDASetRefinementFactor(DM da, PetscInt refine_x, PetscInt refin
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidLogicalCollectiveInt(da,refine_x,2);
   PetscValidLogicalCollectiveInt(da,refine_y,3);
   PetscValidLogicalCollectiveInt(da,refine_z,4);
@@ -730,8 +781,6 @@ PetscErrorCode  DMDASetRefinementFactor(DM da, PetscInt refine_x, PetscInt refin
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetRefinementFactor"
 /*@C
      DMDAGetRefinementFactor - Gets the ratios that the DMDA grid is refined
 
@@ -747,7 +796,8 @@ PetscErrorCode  DMDASetRefinementFactor(DM da, PetscInt refine_x, PetscInt refin
 
   Level: intermediate
 
-    Notes: Pass NULL for values you do not need
+    Notes:
+    Pass NULL for values you do not need
 
 .seealso: DMRefine(), DMDASetRefinementFactor()
 @*/
@@ -756,15 +806,13 @@ PetscErrorCode  DMDAGetRefinementFactor(DM da, PetscInt *refine_x, PetscInt *ref
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   if (refine_x) *refine_x = dd->refine_x;
   if (refine_y) *refine_y = dd->refine_y;
   if (refine_z) *refine_z = dd->refine_z;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetGetMatrix"
 /*@C
      DMDASetGetMatrix - Sets the routine used by the DMDA to allocate a matrix.
 
@@ -776,21 +824,22 @@ PetscErrorCode  DMDAGetRefinementFactor(DM da, PetscInt *refine_x, PetscInt *ref
 
   Level: developer
 
-   Notes: See DMDASetBlockFills() that provides a simple way to provide the nonzero structure for
+   Notes:
+    See DMDASetBlockFills() that provides a simple way to provide the nonzero structure for
        the diagonal and off-diagonal blocks of the matrix
 
+   Not supported from Fortran
+
 .seealso: DMCreateMatrix(), DMDASetBlockFills()
 @*/
 PetscErrorCode  DMDASetGetMatrix(DM da,PetscErrorCode (*f)(DM, Mat*))
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   da->ops->creatematrix = f;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDARefineOwnershipRanges"
 /*
   Creates "balanced" ownership ranges after refinement, constrained by the need for the
   fine grid boundaries to fall within one stencil width of the coarse partition.
@@ -833,8 +882,6 @@ static PetscErrorCode DMDARefineOwnershipRanges(DM da,PetscBool periodic,PetscIn
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDACoarsenOwnershipRanges"
 /*
   Creates "balanced" ownership ranges after coarsening, constrained by the need for the
   fine grid boundaries to fall within one stencil width of the coarse partition.
@@ -876,8 +923,6 @@ static PetscErrorCode DMDACoarsenOwnershipRanges(DM da,PetscBool periodic,PetscI
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMRefine_DA"
 PetscErrorCode  DMRefine_DA(DM da,MPI_Comm comm,DM *daref)
 {
   PetscErrorCode ierr;
@@ -886,7 +931,7 @@ PetscErrorCode  DMRefine_DA(DM da,MPI_Comm comm,DM *daref)
   DM_DA          *dd = (DM_DA*)da->data,*dd2;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidPointer(daref,3);
 
   ierr = DMGetDimension(da, &dim);CHKERRQ(ierr);
@@ -966,6 +1011,41 @@ PetscErrorCode  DMRefine_DA(DM da,MPI_Comm comm,DM *daref)
   dd2->coarsen_y = dd2->refine_y = dd->refine_y;
   dd2->coarsen_z = dd2->refine_z = dd->refine_z;
 
+  if (dd->refine_z_hier) {
+    if (da->levelup - da->leveldown + 1 > -1 && da->levelup - da->leveldown + 1 < dd->refine_z_hier_n) {
+      dd2->refine_z = dd->refine_z_hier[da->levelup - da->leveldown + 1];
+    }
+    if (da->levelup - da->leveldown > -1 && da->levelup - da->leveldown < dd->refine_z_hier_n) {
+      dd2->coarsen_z = dd->refine_z_hier[da->levelup - da->leveldown];
+    }
+    dd2->refine_z_hier_n = dd->refine_z_hier_n;
+    ierr = PetscMalloc1(dd2->refine_z_hier_n,&dd2->refine_z_hier);CHKERRQ(ierr);
+    ierr = PetscMemcpy(dd2->refine_z_hier,dd->refine_z_hier,dd2->refine_z_hier_n*sizeof(PetscInt));CHKERRQ(ierr);
+  }
+  if (dd->refine_y_hier) {
+    if (da->levelup - da->leveldown + 1 > -1 && da->levelup - da->leveldown + 1 < dd->refine_y_hier_n) {
+      dd2->refine_y = dd->refine_y_hier[da->levelup - da->leveldown + 1];
+    }
+    if (da->levelup - da->leveldown > -1 && da->levelup - da->leveldown < dd->refine_y_hier_n) {
+      dd2->coarsen_y = dd->refine_y_hier[da->levelup - da->leveldown];
+    }
+    dd2->refine_y_hier_n = dd->refine_y_hier_n;
+    ierr = PetscMalloc1(dd2->refine_y_hier_n,&dd2->refine_y_hier);CHKERRQ(ierr);
+    ierr = PetscMemcpy(dd2->refine_y_hier,dd->refine_y_hier,dd2->refine_y_hier_n*sizeof(PetscInt));CHKERRQ(ierr);
+  }
+  if (dd->refine_x_hier) {
+    if (da->levelup - da->leveldown + 1 > -1 && da->levelup - da->leveldown + 1 < dd->refine_x_hier_n) {
+      dd2->refine_x = dd->refine_x_hier[da->levelup - da->leveldown + 1];
+    }
+    if (da->levelup - da->leveldown > -1 && da->levelup - da->leveldown < dd->refine_x_hier_n) {
+      dd2->coarsen_x = dd->refine_x_hier[da->levelup - da->leveldown];
+    }
+    dd2->refine_x_hier_n = dd->refine_x_hier_n;
+    ierr = PetscMalloc1(dd2->refine_x_hier_n,&dd2->refine_x_hier);CHKERRQ(ierr);
+    ierr = PetscMemcpy(dd2->refine_x_hier,dd->refine_x_hier,dd2->refine_x_hier_n*sizeof(PetscInt));CHKERRQ(ierr);
+  }
+
+
   /* copy vector type information */
   ierr = DMSetVecType(da2,da->vectype);CHKERRQ(ierr);
 
@@ -975,7 +1055,6 @@ PetscErrorCode  DMRefine_DA(DM da,MPI_Comm comm,DM *daref)
   da2->leveldown = da->leveldown;
   da2->levelup   = da->levelup + 1;
 
-  ierr = DMSetFromOptions(da2);CHKERRQ(ierr);
   ierr = DMSetUp(da2);CHKERRQ(ierr);
 
   /* interpolate coordinates if they are set on the coarse grid */
@@ -1006,8 +1085,6 @@ PetscErrorCode  DMRefine_DA(DM da,MPI_Comm comm,DM *daref)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCoarsen_DA"
 PetscErrorCode  DMCoarsen_DA(DM da, MPI_Comm comm,DM *daref)
 {
   PetscErrorCode ierr;
@@ -1016,7 +1093,7 @@ PetscErrorCode  DMCoarsen_DA(DM da, MPI_Comm comm,DM *daref)
   DM_DA          *dd = (DM_DA*)da->data,*dd2;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidPointer(daref,3);
 
   ierr = DMGetDimension(da, &dim);CHKERRQ(ierr);
@@ -1096,6 +1173,40 @@ PetscErrorCode  DMCoarsen_DA(DM da, MPI_Comm comm,DM *daref)
   dd2->coarsen_y = dd2->refine_y = dd->coarsen_y;
   dd2->coarsen_z = dd2->refine_z = dd->coarsen_z;
 
+  if (dd->refine_z_hier) {
+    if (da->levelup - da->leveldown -1 > -1 && da->levelup - da->leveldown - 1< dd->refine_z_hier_n) {
+      dd2->refine_z = dd->refine_z_hier[da->levelup - da->leveldown - 1];
+    }
+    if (da->levelup - da->leveldown - 2 > -1 && da->levelup - da->leveldown - 2 < dd->refine_z_hier_n) {
+      dd2->coarsen_z = dd->refine_z_hier[da->levelup - da->leveldown - 2];
+    }
+    dd2->refine_z_hier_n = dd->refine_z_hier_n;
+    ierr = PetscMalloc1(dd2->refine_z_hier_n,&dd2->refine_z_hier);CHKERRQ(ierr);
+    ierr = PetscMemcpy(dd2->refine_z_hier,dd->refine_z_hier,dd2->refine_z_hier_n*sizeof(PetscInt));CHKERRQ(ierr);
+  }
+  if (dd->refine_y_hier) {
+    if (da->levelup - da->leveldown - 1 > -1 && da->levelup - da->leveldown - 1< dd->refine_y_hier_n) {
+      dd2->refine_y = dd->refine_y_hier[da->levelup - da->leveldown - 1];
+    }
+    if (da->levelup - da->leveldown - 2 > -1 && da->levelup - da->leveldown - 2 < dd->refine_y_hier_n) {
+      dd2->coarsen_y = dd->refine_y_hier[da->levelup - da->leveldown - 2];
+    }
+    dd2->refine_y_hier_n = dd->refine_y_hier_n;
+    ierr = PetscMalloc1(dd2->refine_y_hier_n,&dd2->refine_y_hier);CHKERRQ(ierr);
+    ierr = PetscMemcpy(dd2->refine_y_hier,dd->refine_y_hier,dd2->refine_y_hier_n*sizeof(PetscInt));CHKERRQ(ierr);
+  }
+  if (dd->refine_x_hier) {
+    if (da->levelup - da->leveldown - 1 > -1 && da->levelup - da->leveldown - 1 < dd->refine_x_hier_n) {
+      dd2->refine_x = dd->refine_x_hier[da->levelup - da->leveldown - 1];
+    }
+    if (da->levelup - da->leveldown - 2 > -1 && da->levelup - da->leveldown - 2 < dd->refine_x_hier_n) {
+      dd2->coarsen_x = dd->refine_x_hier[da->levelup - da->leveldown - 2];
+    }
+    dd2->refine_x_hier_n = dd->refine_x_hier_n;
+    ierr = PetscMalloc1(dd2->refine_x_hier_n,&dd2->refine_x_hier);CHKERRQ(ierr);
+    ierr = PetscMemcpy(dd2->refine_x_hier,dd->refine_x_hier,dd2->refine_x_hier_n*sizeof(PetscInt));CHKERRQ(ierr);
+  }
+
   /* copy vector type information */
   ierr = DMSetVecType(da2,da->vectype);CHKERRQ(ierr);
 
@@ -1105,7 +1216,6 @@ PetscErrorCode  DMCoarsen_DA(DM da, MPI_Comm comm,DM *daref)
   da2->leveldown = da->leveldown + 1;
   da2->levelup   = da->levelup;
 
-  ierr = DMSetFromOptions(da2);CHKERRQ(ierr);
   ierr = DMSetUp(da2);CHKERRQ(ierr);
 
   /* inject coordinates if they are set on the fine grid */
@@ -1139,8 +1249,6 @@ PetscErrorCode  DMCoarsen_DA(DM da, MPI_Comm comm,DM *daref)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMRefineHierarchy_DA"
 PetscErrorCode  DMRefineHierarchy_DA(DM da,PetscInt nlevels,DM daf[])
 {
   PetscErrorCode ierr;
@@ -1158,11 +1266,11 @@ PetscErrorCode  DMRefineHierarchy_DA(DM da,PetscInt nlevels,DM daf[])
     ierr = DMDAGetRefinementFactor(da,&refx[i],&refy[i],&refz[i]);CHKERRQ(ierr);
   }
   n    = nlevels;
-  ierr = PetscOptionsGetIntArray(((PetscObject)da)->prefix,"-da_refine_hierarchy_x",refx,&n,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetIntArray(((PetscObject)da)->options,((PetscObject)da)->prefix,"-da_refine_hierarchy_x",refx,&n,NULL);CHKERRQ(ierr);
   n    = nlevels;
-  ierr = PetscOptionsGetIntArray(((PetscObject)da)->prefix,"-da_refine_hierarchy_y",refy,&n,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetIntArray(((PetscObject)da)->options,((PetscObject)da)->prefix,"-da_refine_hierarchy_y",refy,&n,NULL);CHKERRQ(ierr);
   n    = nlevels;
-  ierr = PetscOptionsGetIntArray(((PetscObject)da)->prefix,"-da_refine_hierarchy_z",refz,&n,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetIntArray(((PetscObject)da)->options,((PetscObject)da)->prefix,"-da_refine_hierarchy_z",refz,&n,NULL);CHKERRQ(ierr);
 
   ierr = DMDASetRefinementFactor(da,refx[0],refy[0],refz[0]);CHKERRQ(ierr);
   ierr = DMRefine(da,PetscObjectComm((PetscObject)da),&daf[0]);CHKERRQ(ierr);
@@ -1174,8 +1282,6 @@ PetscErrorCode  DMRefineHierarchy_DA(DM da,PetscInt nlevels,DM daf[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCoarsenHierarchy_DA"
 PetscErrorCode  DMCoarsenHierarchy_DA(DM da,PetscInt nlevels,DM dac[])
 {
   PetscErrorCode ierr;
@@ -1192,3 +1298,117 @@ PetscErrorCode  DMCoarsenHierarchy_DA(DM da,PetscInt nlevels,DM dac[])
   }
   PetscFunctionReturn(0);
 }
+
+#include <petscgll.h>
+
+PetscErrorCode DMDASetGLLCoordinates_1d(DM dm,PetscGLL *gll)
+{
+  PetscErrorCode ierr;
+  PetscInt       i,j,n = gll->n,xs,xn,q;
+  PetscScalar    *xx;
+  PetscReal      h;
+  Vec            x;
+  DM_DA          *da = (DM_DA*)dm->data;
+
+  PetscFunctionBegin;
+  if (da->bx != DM_BOUNDARY_PERIODIC) {
+    ierr = DMDAGetInfo(dm,NULL,&q,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
+    q    = (q-1)/(n-1);  /* number of spectral elements */
+    h    = 2.0/q;
+    ierr = DMDAGetCorners(dm,&xs,NULL,NULL,&xn,NULL,NULL);CHKERRQ(ierr);
+    xs   = xs/(n-1);
+    xn   = xn/(n-1);
+    ierr = DMDASetUniformCoordinates(dm,-1.,1.,0.,0.,0.,0.);CHKERRQ(ierr);
+    ierr = DMGetCoordinates(dm,&x);CHKERRQ(ierr);
+    ierr = DMDAVecGetArray(dm,x,&xx);CHKERRQ(ierr);
+
+    /* loop over local spectral elements */
+    for (j=xs; j<xs+xn; j++) {
+      /*
+       Except for the first process, each process starts on the second GLL point of the first element on that process
+       */
+      for (i= (j == xs && xs > 0)? 1 : 0; i<n; i++) {
+        xx[j*(n-1) + i] = -1.0 + h*j + h*(gll->nodes[i]+1.0)/2.;
+      }
+    }
+    ierr = DMDAVecRestoreArray(dm,x,&xx);CHKERRQ(ierr);
+  } else SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_SUP,"Not yet implemented for periodic");
+  PetscFunctionReturn(0);
+}
+
+/*@
+
+     DMDASetGLLCoordinates - Sets the global coordinates from -1 to 1 to the GLL points of as many GLL elements that fit the number of grid points
+
+   Collective on DM
+
+   Input Parameters:
++   da - the DMDA object
+-   gll - the GLL object
+
+   Notes:
+    the parallel decomposition of grid points must correspond to the degree of the GLL. That is, the number of grid points
+          on each process much be divisible by the number of GLL elements needed per process. This depends on whether the DM is
+          periodic or not.
+
+   Level: advanced
+
+.seealso:   DMDACreate(), PetscGLLCreate(), DMGetCoordinates()
+@*/
+PetscErrorCode DMDASetGLLCoordinates(DM da,PetscGLL *gll)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (da->dim == 1) {
+    ierr = DMDASetGLLCoordinates_1d(da,gll);CHKERRQ(ierr);
+  } else SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_SUP,"Not yet implemented for 2 or 3d");
+  PetscFunctionReturn(0);
+}
+
+PETSC_INTERN PetscErrorCode DMGetCompatibility_DA(DM da1,DM dm2,PetscBool *compatible,PetscBool *set)
+{
+  PetscErrorCode ierr;
+  DM_DA          *dd1 = (DM_DA*)da1->data,*dd2;
+  DM             da2;
+  DMType         dmtype2;
+  PetscBool      isda,compatibleLocal;
+  PetscInt       i;
+
+  PetscFunctionBegin;
+  if (!da1->setupcalled) SETERRQ(PetscObjectComm((PetscObject)da1),PETSC_ERR_ARG_WRONGSTATE,"DMSetUp() must be called on first DM before DMGetCompatibility()");
+  ierr = DMGetType(dm2,&dmtype2);CHKERRQ(ierr);
+  ierr = PetscStrcmp(dmtype2,DMDA,&isda);CHKERRQ(ierr);
+  if (isda) {
+    da2 = dm2;
+    dd2 = (DM_DA*)da2->data;
+    if (!da2->setupcalled) SETERRQ(PetscObjectComm((PetscObject)da2),PETSC_ERR_ARG_WRONGSTATE,"DMSetUp() must be called on second DM before DMGetCompatibility()");
+    compatibleLocal = (PetscBool)(da1->dim == da2->dim);
+    if (compatibleLocal) compatibleLocal = (PetscBool)(compatibleLocal && (dd1->s == dd2->s)); /* Stencil width */
+    /*                                                                           Global size              ranks               Boundary type */
+    if (compatibleLocal)                 compatibleLocal = (PetscBool)(compatibleLocal && (dd1->M == dd2->M) && (dd1->m == dd2->m) && (dd1->bx == dd2->bx));
+    if (compatibleLocal && da1->dim > 1) compatibleLocal = (PetscBool)(compatibleLocal && (dd1->N == dd2->N) && (dd1->n == dd2->n) && (dd1->by == dd2->by));
+    if (compatibleLocal && da1->dim > 2) compatibleLocal = (PetscBool)(compatibleLocal && (dd1->P == dd2->P) && (dd1->p == dd2->p) && (dd1->bz == dd2->bz));
+    if (compatibleLocal) {
+      for (i=0; i<dd1->m; ++i) {
+        compatibleLocal = (PetscBool)(compatibleLocal && (dd1->lx[i] == dd2->lx[i]));           /* Local size     */
+      }
+    }
+    if (compatibleLocal && da1->dim > 1) {
+      for (i=0; i<dd1->n; ++i) {
+        compatibleLocal = (PetscBool)(compatibleLocal && (dd1->ly[i] == dd2->ly[i]));
+      }
+    }
+    if (compatibleLocal && da1->dim > 2) {
+      for (i=0; i<dd1->p; ++i) {
+        compatibleLocal = (PetscBool)(compatibleLocal && (dd1->lz[i] == dd2->lz[i]));
+      }
+    }
+    *compatible = compatibleLocal;
+    *set = PETSC_TRUE;
+  } else {
+    /* Decline to determine compatibility with other DM types */
+    *set = PETSC_FALSE;
+  }
+  PetscFunctionReturn(0);
+}
