@@ -2,13 +2,11 @@
 #include <petsc/private/dmdaimpl.h>    /*I   "petscdmda.h"   I*/
 #include <petscdraw.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "DMView_DA_2d"
-PetscErrorCode DMView_DA_2d(DM da,PetscViewer viewer)
+static PetscErrorCode DMView_DA_2d(DM da,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   PetscMPIInt    rank;
-  PetscBool      iascii,isdraw,isbinary;
+  PetscBool      iascii,isdraw,isglvis,isbinary;
   DM_DA          *dd = (DM_DA*)da->data;
 #if defined(PETSC_HAVE_MATLAB_ENGINE)
   PetscBool ismatlab;
@@ -19,6 +17,7 @@ PetscErrorCode DMView_DA_2d(DM da,PetscViewer viewer)
 
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERGLVIS,&isglvis);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERBINARY,&isbinary);CHKERRQ(ierr);
 #if defined(PETSC_HAVE_MATLAB_ENGINE)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERMATLAB,&ismatlab);CHKERRQ(ierr);
@@ -27,14 +26,35 @@ PetscErrorCode DMView_DA_2d(DM da,PetscViewer viewer)
     PetscViewerFormat format;
 
     ierr = PetscViewerGetFormat(viewer, &format);CHKERRQ(ierr);
-    if (format != PETSC_VIEWER_ASCII_VTK && format != PETSC_VIEWER_ASCII_VTK_CELL) {
+    if (format == PETSC_VIEWER_LOAD_BALANCE) {
+      PetscInt      i,nmax = 0,nmin = PETSC_MAX_INT,navg = 0,*nz,nzlocal;
+      DMDALocalInfo info;
+      PetscMPIInt   size;
+      ierr = MPI_Comm_size(PetscObjectComm((PetscObject)da),&size);CHKERRQ(ierr);
+      ierr = DMDAGetLocalInfo(da,&info);CHKERRQ(ierr);
+      nzlocal = info.xm*info.ym;
+      ierr = PetscMalloc1(size,&nz);CHKERRQ(ierr);
+      ierr = MPI_Allgather(&nzlocal,1,MPIU_INT,nz,1,MPIU_INT,PetscObjectComm((PetscObject)da));CHKERRQ(ierr);
+      for (i=0; i<(PetscInt)size; i++) {
+        nmax = PetscMax(nmax,nz[i]);
+        nmin = PetscMin(nmin,nz[i]);
+        navg += nz[i];
+      }
+      ierr = PetscFree(nz);CHKERRQ(ierr);
+      navg = navg/size;
+      ierr = PetscViewerASCIIPrintf(viewer,"  Load Balance - Grid Points: Min %D  avg %D  max %D\n",nmin,navg,nmax);CHKERRQ(ierr);
+      PetscFunctionReturn(0);
+    }
+    if (format != PETSC_VIEWER_ASCII_VTK && format != PETSC_VIEWER_ASCII_VTK_CELL && format != PETSC_VIEWER_ASCII_GLVIS) {
       DMDALocalInfo info;
       ierr = DMDAGetLocalInfo(da,&info);CHKERRQ(ierr);
-      ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_TRUE);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
       ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Processor [%d] M %D N %D m %D n %D w %D s %D\n",rank,dd->M,dd->N,dd->m,dd->n,dd->w,dd->s);CHKERRQ(ierr);
       ierr = PetscViewerASCIISynchronizedPrintf(viewer,"X range of indices: %D %D, Y range of indices: %D %D\n",info.xs,info.xs+info.xm,info.ys,info.ys+info.ym);CHKERRQ(ierr);
       ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
-      ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_FALSE);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPopSynchronized(viewer);CHKERRQ(ierr);
+    } else if (format == PETSC_VIEWER_ASCII_GLVIS) {
+      ierr = DMView_DA_GLVis(da,viewer);CHKERRQ(ierr);
     } else {
       ierr = DMView_DA_VTK(da,viewer);CHKERRQ(ierr);
     }
@@ -49,12 +69,14 @@ PetscErrorCode DMView_DA_2d(DM da,PetscViewer viewer)
     PetscBool      isnull;
 
     ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
-    ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr); if (isnull) PetscFunctionReturn(0);
-    if (!da->coordinates) {
-      ierr = PetscDrawSetCoordinates(draw,xmin,ymin,xmax,ymax);CHKERRQ(ierr);
-    }
-    ierr = PetscDrawSynchronizedClear(draw);CHKERRQ(ierr);
+    ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr);
+    if (isnull) PetscFunctionReturn(0);
 
+    ierr = PetscDrawCheckResizedWindow(draw);CHKERRQ(ierr);
+    ierr = PetscDrawClear(draw);CHKERRQ(ierr);
+    ierr = PetscDrawSetCoordinates(draw,xmin,ymin,xmax,ymax);CHKERRQ(ierr);
+
+    ierr = PetscDrawCollectiveBegin(draw);CHKERRQ(ierr);
     /* first processor draw all node lines */
     if (!rank) {
       ymin = 0.0; ymax = dd->N - 1;
@@ -66,46 +88,49 @@ PetscErrorCode DMView_DA_2d(DM da,PetscViewer viewer)
         ierr = PetscDrawLine(draw,xmin,ymin,xmax,ymin,PETSC_DRAW_BLACK);CHKERRQ(ierr);
       }
     }
-    ierr = PetscDrawSynchronizedFlush(draw);CHKERRQ(ierr);
+    ierr = PetscDrawCollectiveEnd(draw);CHKERRQ(ierr);
+    ierr = PetscDrawFlush(draw);CHKERRQ(ierr);
     ierr = PetscDrawPause(draw);CHKERRQ(ierr);
 
+    ierr = PetscDrawCollectiveBegin(draw);CHKERRQ(ierr);
     /* draw my box */
-    ymin = dd->ys; ymax = dd->ye - 1; xmin = dd->xs/dd->w;
-    xmax =(dd->xe-1)/dd->w;
+    xmin = dd->xs/dd->w; xmax =(dd->xe-1)/dd->w; ymin = dd->ys; ymax = dd->ye - 1;
     ierr = PetscDrawLine(draw,xmin,ymin,xmax,ymin,PETSC_DRAW_RED);CHKERRQ(ierr);
     ierr = PetscDrawLine(draw,xmin,ymin,xmin,ymax,PETSC_DRAW_RED);CHKERRQ(ierr);
     ierr = PetscDrawLine(draw,xmin,ymax,xmax,ymax,PETSC_DRAW_RED);CHKERRQ(ierr);
     ierr = PetscDrawLine(draw,xmax,ymin,xmax,ymax,PETSC_DRAW_RED);CHKERRQ(ierr);
-
     /* put in numbers */
     base = (dd->base)/dd->w;
     for (y=ymin; y<=ymax; y++) {
       for (x=xmin; x<=xmax; x++) {
-        sprintf(node,"%d",(int)base++);
+        ierr = PetscSNPrintf(node,sizeof(node),"%d",(int)base++);CHKERRQ(ierr);
         ierr = PetscDrawString(draw,x,y,PETSC_DRAW_BLACK,node);CHKERRQ(ierr);
       }
     }
-
-    ierr = PetscDrawSynchronizedFlush(draw);CHKERRQ(ierr);
+    ierr = PetscDrawCollectiveEnd(draw);CHKERRQ(ierr);
+    ierr = PetscDrawFlush(draw);CHKERRQ(ierr);
     ierr = PetscDrawPause(draw);CHKERRQ(ierr);
-    /* overlay ghost numbers, useful for error checking */
-    /* put in numbers */
 
-    base = 0;
+    ierr = PetscDrawCollectiveBegin(draw);CHKERRQ(ierr);
+    /* overlay ghost numbers, useful for error checking */
     ierr = ISLocalToGlobalMappingGetBlockIndices(da->ltogmap,&idx);CHKERRQ(ierr);
-    ymin = dd->Ys; ymax = dd->Ye; xmin = dd->Xs; xmax = dd->Xe;
+    base = 0; xmin = dd->Xs; xmax = dd->Xe; ymin = dd->Ys; ymax = dd->Ye;
     for (y=ymin; y<ymax; y++) {
       for (x=xmin; x<xmax; x++) {
         if ((base % dd->w) == 0) {
-          sprintf(node,"%d",(int)(idx[base/dd->w]));
+          ierr = PetscSNPrintf(node,sizeof(node),"%d",(int)(idx[base/dd->w]));CHKERRQ(ierr);
           ierr = PetscDrawString(draw,x/dd->w,y,PETSC_DRAW_BLUE,node);CHKERRQ(ierr);
         }
         base++;
       }
     }
     ierr = ISLocalToGlobalMappingRestoreBlockIndices(da->ltogmap,&idx);CHKERRQ(ierr);
-    ierr = PetscDrawSynchronizedFlush(draw);CHKERRQ(ierr);
+    ierr = PetscDrawCollectiveEnd(draw);CHKERRQ(ierr);
+    ierr = PetscDrawFlush(draw);CHKERRQ(ierr);
     ierr = PetscDrawPause(draw);CHKERRQ(ierr);
+    ierr = PetscDrawSave(draw);CHKERRQ(ierr);
+  } else if (isglvis) {
+    ierr = DMView_DA_GLVis(da,viewer);CHKERRQ(ierr);
   } else if (isbinary) {
     ierr = DMView_DA_Binary(da,viewer);CHKERRQ(ierr);
 #if defined(PETSC_HAVE_MATLAB_ENGINE)
@@ -116,64 +141,8 @@ PetscErrorCode DMView_DA_2d(DM da,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-/*
-      M is number of grid points
-      m is number of processors
-
-*/
-#undef __FUNCT__
-#define __FUNCT__ "DMDASplitComm2d"
-PetscErrorCode  DMDASplitComm2d(MPI_Comm comm,PetscInt M,PetscInt N,PetscInt sw,MPI_Comm *outcomm)
-{
-  PetscErrorCode ierr;
-  PetscInt       m,n = 0,x = 0,y = 0;
-  PetscMPIInt    size,csize,rank;
-
-  PetscFunctionBegin;
-  ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
-  ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
-
-  csize = 4*size;
-  do {
-    if (csize % 4) SETERRQ4(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Cannot split communicator of size %d tried %d %D %D",size,csize,x,y);
-    csize = csize/4;
-
-    m = (PetscInt)(0.5 + PetscSqrtReal(((PetscReal)M)*((PetscReal)csize)/((PetscReal)N)));
-    if (!m) m = 1;
-    while (m > 0) {
-      n = csize/m;
-      if (m*n == csize) break;
-      m--;
-    }
-    if (M > N && m < n) {PetscInt _m = m; m = n; n = _m;}
-
-    x = M/m + ((M % m) > ((csize-1) % m));
-    y = (N + (csize-1)/m)/n;
-  } while ((x < 4 || y < 4) && csize > 1);
-  if (size != csize) {
-    MPI_Group   entire_group,sub_group;
-    PetscMPIInt i,*groupies;
-
-    ierr = MPI_Comm_group(comm,&entire_group);CHKERRQ(ierr);
-    ierr = PetscMalloc1(csize,&groupies);CHKERRQ(ierr);
-    for (i=0; i<csize; i++) {
-      groupies[i] = (rank/csize)*csize + i;
-    }
-    ierr = MPI_Group_incl(entire_group,csize,groupies,&sub_group);CHKERRQ(ierr);
-    ierr = PetscFree(groupies);CHKERRQ(ierr);
-    ierr = MPI_Comm_create(comm,sub_group,outcomm);CHKERRQ(ierr);
-    ierr = MPI_Group_free(&entire_group);CHKERRQ(ierr);
-    ierr = MPI_Group_free(&sub_group);CHKERRQ(ierr);
-    ierr = PetscInfo1(0,"DMDASplitComm2d:Creating redundant coarse problems of size %d\n",csize);CHKERRQ(ierr);
-  } else {
-    *outcomm = comm;
-  }
-  PetscFunctionReturn(0);
-}
 
 #if defined(new)
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetDiagonal_MFFD"
 /*
   DMDAGetDiagonal_MFFD - Gets the diagonal for a matrix free matrix where local
     function lives on a DMDA
@@ -218,8 +187,6 @@ PetscErrorCode DMDAGetDiagonal_MFFD(DM da,Vec U,Vec a)
 }
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSetUp_DA_2D"
 PetscErrorCode  DMSetUp_DA_2D(DM da)
 {
   DM_DA            *dd = (DM_DA*)da->data;
@@ -250,7 +217,7 @@ PetscErrorCode  DMSetUp_DA_2D(DM da)
   if (stencil_type == DMDA_STENCIL_BOX && (bx == DM_BOUNDARY_MIRROR || by == DM_BOUNDARY_MIRROR)) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_SUP,"Mirror boundary and box stencil");
   ierr = PetscObjectGetComm((PetscObject)da,&comm);CHKERRQ(ierr);
 #if !defined(PETSC_USE_64BIT_INDICES)
-  if (((Petsc64bitInt) M)*((Petsc64bitInt) N)*((Petsc64bitInt) dof) > (Petsc64bitInt) PETSC_MPI_INT_MAX) SETERRQ3(comm,PETSC_ERR_INT_OVERFLOW,"Mesh of %D by %D by %D (dof) is too large for 32 bit indices",M,N,dof);
+  if (((PetscInt64) M)*((PetscInt64) N)*((PetscInt64) dof) > (PetscInt64) PETSC_MPI_INT_MAX) SETERRQ3(comm,PETSC_ERR_INT_OVERFLOW,"Mesh of %D by %D by %D (dof) is too large for 32 bit indices",M,N,dof);
 #endif
 
   if (dof < 1) SETERRQ1(comm,PETSC_ERR_ARG_OUTOFRANGE,"Must have 1 or more degrees of freedom per node: %D",dof);
@@ -428,20 +395,11 @@ PetscErrorCode  DMSetUp_DA_2D(DM da)
   dd->nlocal = (Xe-Xs)*(Ye-Ys)*dof;
   ierr       = VecCreateSeqWithArray(PETSC_COMM_SELF,dof,dd->nlocal,NULL,&local);CHKERRQ(ierr);
 
-  /* generate appropriate vector scatters */
-  /* local to global inserts non-ghost point region into global */
-  ierr  = PetscMalloc1((IXe-IXs)*(IYe-IYs),&idx);CHKERRQ(ierr);
-  left  = xs - Xs; right = left + x;
-  down  = ys - Ys; up = down + y;
-  count = 0;
-  for (i=down; i<up; i++) {
-    for (j=left; j<right; j++) {
-      idx[count++] = i*(Xe-Xs) + j;
-    }
-  }
+  /* generate global to local vector scatter and local to global mapping*/
 
   /* global to local must include ghost points within the domain,
      but not ghost points outside the domain that aren't periodic */
+  ierr  = PetscMalloc1((IXe-IXs)*(IYe-IYs),&idx);CHKERRQ(ierr);
   if (stencil_type == DMDA_STENCIL_BOX) {
     left  = IXs - Xs; right = left + (IXe-IXs);
     down  = IYs - Ys; up = down + (IYe-IYs);
@@ -659,9 +617,7 @@ PetscErrorCode  DMSetUp_DA_2D(DM da)
     n0 = sn0; n2 = sn2; n6 = sn6; n8 = sn8;
   }
 
-  if (((stencil_type == DMDA_STENCIL_STAR)  ||
-       (bx && bx != DM_BOUNDARY_PERIODIC) ||
-       (by && by != DM_BOUNDARY_PERIODIC))) {
+  if (((stencil_type == DMDA_STENCIL_STAR)  || (bx && bx != DM_BOUNDARY_PERIODIC) || (by && by != DM_BOUNDARY_PERIODIC))) {
     /*
         Recompute the local to global mappings, this time keeping the
       information about the cross corner processor numbers and any ghosted
@@ -785,8 +741,6 @@ PetscErrorCode  DMSetUp_DA_2D(DM da)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDACreate2d"
 /*@C
    DMDACreate2d -  Creates an object that will manage the communication of  two-dimensional
    regular array data that is distributed across some processors.
@@ -798,8 +752,7 @@ PetscErrorCode  DMSetUp_DA_2D(DM da)
 .  bx,by - type of ghost nodes the array have.
          Use one of DM_BOUNDARY_NONE, DM_BOUNDARY_GHOSTED, DM_BOUNDARY_PERIODIC.
 .  stencil_type - stencil type.  Use either DMDA_STENCIL_BOX or DMDA_STENCIL_STAR.
-.  M,N - global dimension in each direction of the array (use -M and or -N to indicate that it may be set to a different value
-            from the command line with -da_grid_x <M> -da_grid_y <N>)
+.  M,N - global dimension in each direction of the array
 .  m,n - corresponding number of processors in each dimension
          (or PETSC_DECIDE to have calculated)
 .  dof - number of degrees of freedom per node
@@ -815,13 +768,13 @@ PetscErrorCode  DMSetUp_DA_2D(DM da)
 
    Options Database Key:
 +  -dm_view - Calls DMView() at the conclusion of DMDACreate2d()
-.  -da_grid_x <nx> - number of grid points in x direction, if M < 0
-.  -da_grid_y <ny> - number of grid points in y direction, if N < 0
+.  -da_grid_x <nx> - number of grid points in x direction
+.  -da_grid_y <ny> - number of grid points in y direction
 .  -da_processors_x <nx> - number of processors in x direction
 .  -da_processors_y <ny> - number of processors in y direction
 .  -da_refine_x <rx> - refinement ratio in x direction
 .  -da_refine_y <ry> - refinement ratio in y direction
--  -da_refine <n> - refine the DMDA n times before creating, if M or N < 0
+-  -da_refine <n> - refine the DMDA n times before creating
 
 
    Level: beginner
@@ -835,6 +788,11 @@ PetscErrorCode  DMSetUp_DA_2D(DM da)
    The appropriate vector objects can be obtained with calls to DMCreateGlobalVector()
    and DMCreateLocalVector() and calls to VecDuplicate() if more are needed.
 
+   You must call DMSetUp() after this call before using this DM.
+
+   If you wish to use the options database to change values in the DMDA call DMSetFromOptions() after this call
+   but before DMSetUp().
+
 .keywords: distributed array, create, two-dimensional
 
 .seealso: DMDestroy(), DMView(), DMDACreate1d(), DMDACreate3d(), DMGlobalToLocalBegin(), DMDAGetRefinementFactor(),
@@ -858,8 +816,5 @@ PetscErrorCode  DMDACreate2d(MPI_Comm comm,DMBoundaryType bx,DMBoundaryType by,D
   ierr = DMDASetStencilType(*da, stencil_type);CHKERRQ(ierr);
   ierr = DMDASetStencilWidth(*da, s);CHKERRQ(ierr);
   ierr = DMDASetOwnershipRanges(*da, lx, ly, NULL);CHKERRQ(ierr);
-  /* This violates the behavior for other classes, but right now users expect negative dimensions to be handled this way */
-  ierr = DMSetFromOptions(*da);CHKERRQ(ierr);
-  ierr = DMSetUp(*da);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
