@@ -4,19 +4,40 @@
 */
 
 #include <petsc/private/dmdaimpl.h>    /*I   "petscdmda.h"   I*/
+#include <petscdmfield.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCreateCoordinateDM_DA"
 PetscErrorCode DMCreateCoordinateDM_DA(DM dm, DM *cdm)
 {
   PetscErrorCode ierr;
   PetscFunctionBegin;
-  ierr = DMDAGetReducedDMDA(dm,dm->dim,cdm);CHKERRQ(ierr);
+  ierr = DMDACreateCompatibleDMDA(dm,dm->dim,cdm);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode DMCreateCoordinateField_DA(DM dm, DMField *field)
+{
+  PetscReal      gmin[3], gmax[3];
+  PetscScalar    corners[24];
+  PetscInt       dim;
+  PetscInt       i, j;
+  DM             cdm;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = DMGetDimension(dm,&dim);CHKERRQ(ierr);
+  /* TODO: this is wrong if coordinates are not rectilinear */
+  ierr = DMDAGetBoundingBox(dm,gmin,gmax);CHKERRQ(ierr);
+  for (i = 0; i < (1 << dim); i++) {
+    for (j = 0; j < dim; j++) {
+      corners[i*dim + j] = (i & (1 << j)) ? gmax[j] : gmin[j];
+    }
+  }
+  ierr = DMClone(dm,&cdm);CHKERRQ(ierr);
+  ierr = DMFieldCreateDA(cdm,dim,corners,field);CHKERRQ(ierr);
+  ierr = DMDestroy(&cdm);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetFieldName"
 /*@C
    DMDASetFieldName - Sets the names of individual field components in multicomponent
    vectors associated with a DMDA.
@@ -29,11 +50,14 @@ PetscErrorCode DMCreateCoordinateDM_DA(DM dm, DM *cdm)
         number of degrees of freedom per node within the DMDA
 -  names - the name of the field (component)
 
+  Notes:
+    It must be called after having called DMSetUp().
+
   Level: intermediate
 
 .keywords: distributed array, get, component name
 
-.seealso: DMDAGetFieldName(), DMDASetCoordinateName(), DMDAGetCoordinateName(), DMDASetFieldNames()
+.seealso: DMDAGetFieldName(), DMDASetCoordinateName(), DMDAGetCoordinateName(), DMDASetFieldNames(), DMSetUp()
 @*/
 PetscErrorCode  DMDASetFieldName(DM da,PetscInt nf,const char name[])
 {
@@ -41,15 +65,14 @@ PetscErrorCode  DMDASetFieldName(DM da,PetscInt nf,const char name[])
   DM_DA          *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   if (nf < 0 || nf >= dd->w) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Invalid field number: %D",nf);
+  if (!dd->fieldname) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_ORDER,"You should call DMSetUp() first");
   ierr = PetscFree(dd->fieldname[nf]);CHKERRQ(ierr);
   ierr = PetscStrallocpy(name,&dd->fieldname[nf]);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetFieldNames"
 /*@C
    DMDAGetFieldNames - Gets the name of each component in the vector associated with the DMDA
 
@@ -63,6 +86,8 @@ PetscErrorCode  DMDASetFieldName(DM da,PetscInt nf,const char name[])
 
    Level: intermediate
 
+   Not supported from Fortran, use DMDAGetFieldName()
+
 .keywords: distributed array, get, component name
 
 .seealso: DMDAGetFieldName(), DMDASetCoordinateName(), DMDAGetCoordinateName(), DMDASetFieldName(), DMDASetFieldNames()
@@ -76,8 +101,6 @@ PetscErrorCode  DMDAGetFieldNames(DM da,const char * const **names)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetFieldNames"
 /*@C
    DMDASetFieldNames - Sets the name of each component in the vector associated with the DMDA
 
@@ -87,25 +110,34 @@ PetscErrorCode  DMDAGetFieldNames(DM da,const char * const **names)
 +  dm - the DMDA object
 -  names - the names of the components, final string must be NULL, must have the same number of entries as the dof used in creating the DMDA
 
+   Notes:
+    It must be called after having called DMSetUp().
+
    Level: intermediate
 
+   Not supported from Fortran, use DMDASetFieldName()
+
 .keywords: distributed array, get, component name
 
-.seealso: DMDAGetFieldName(), DMDASetCoordinateName(), DMDAGetCoordinateName(), DMDASetFieldName()
+.seealso: DMDAGetFieldName(), DMDASetCoordinateName(), DMDAGetCoordinateName(), DMDASetFieldName(), DMSetUp()
 @*/
 PetscErrorCode  DMDASetFieldNames(DM da,const char * const *names)
 {
-  PetscErrorCode    ierr;
-  DM_DA             *dd = (DM_DA*)da->data;
+  PetscErrorCode ierr;
+  DM_DA          *dd = (DM_DA*)da->data;
+  char           **fieldname;
+  PetscInt       nf = 0;
 
   PetscFunctionBegin;
+  if (!dd->fieldname) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_ORDER,"You should call DMSetUp() first");
+  while (names[nf++]) {};
+  if (nf != dd->w+1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Invalid number of fields %D",nf-1);
+  ierr = PetscStrArrayallocpy(names,&fieldname);CHKERRQ(ierr);
   ierr = PetscStrArrayDestroy(&dd->fieldname);CHKERRQ(ierr);
-  ierr = PetscStrArrayallocpy(names,&dd->fieldname);CHKERRQ(ierr);
+  dd->fieldname = fieldname;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetFieldName"
 /*@C
    DMDAGetFieldName - Gets the names of individual field components in multicomponent
    vectors associated with a DMDA.
@@ -120,26 +152,28 @@ PetscErrorCode  DMDASetFieldNames(DM da,const char * const *names)
    Output Parameter:
 .  names - the name of the field (component)
 
+  Notes:
+    It must be called after having called DMSetUp().
+
   Level: intermediate
 
 .keywords: distributed array, get, component name
 
-.seealso: DMDASetFieldName(), DMDASetCoordinateName(), DMDAGetCoordinateName()
+.seealso: DMDASetFieldName(), DMDASetCoordinateName(), DMDAGetCoordinateName(), DMSetUp()
 @*/
 PetscErrorCode  DMDAGetFieldName(DM da,PetscInt nf,const char **name)
 {
   DM_DA *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidPointer(name,3);
   if (nf < 0 || nf >= dd->w) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Invalid field number: %D",nf);
+  if (!dd->fieldname) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_ORDER,"You should call DMSetUp() first");
   *name = dd->fieldname[nf];
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetCoordinateName"
 /*@C
    DMDASetCoordinateName - Sets the name of the coordinate directions associated with a DMDA, for example "x" or "y"
 
@@ -150,11 +184,16 @@ PetscErrorCode  DMDAGetFieldName(DM da,PetscInt nf,const char **name)
 .  nf - coordinate number for the DMDA (0, 1, ... dim-1),
 -  name - the name of the coordinate
 
+  Notes:
+    It must be called after having called DMSetUp().
+
   Level: intermediate
 
+  Not supported from Fortran
+
 .keywords: distributed array, get, component name
 
-.seealso: DMDAGetCoordinateName(), DMDASetFieldName(), DMDAGetFieldName()
+.seealso: DMDAGetCoordinateName(), DMDASetFieldName(), DMDAGetFieldName(), DMSetUp()
 @*/
 PetscErrorCode DMDASetCoordinateName(DM dm,PetscInt nf,const char name[])
 {
@@ -162,15 +201,14 @@ PetscErrorCode DMDASetCoordinateName(DM dm,PetscInt nf,const char name[])
   DM_DA          *dd = (DM_DA*)dm->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(dm,DM_CLASSID,1,DMDA);
   if (nf < 0 || nf >= dm->dim) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Invalid coordinate number: %D",nf);
+  if (!dd->coordinatename) SETERRQ(PetscObjectComm((PetscObject)dm),PETSC_ERR_ORDER,"You should call DMSetUp() first");
   ierr = PetscFree(dd->coordinatename[nf]);CHKERRQ(ierr);
   ierr = PetscStrallocpy(name,&dd->coordinatename[nf]);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetCoordinateName"
 /*@C
    DMDAGetCoordinateName - Gets the name of a coodinate direction associated with a DMDA.
 
@@ -183,29 +221,33 @@ PetscErrorCode DMDASetCoordinateName(DM dm,PetscInt nf,const char name[])
    Output Parameter:
 .  names - the name of the coordinate direction
 
+  Notes:
+    It must be called after having called DMSetUp().
+
   Level: intermediate
 
+  Not supported from Fortran
+
 .keywords: distributed array, get, component name
 
-.seealso: DMDASetCoordinateName(), DMDASetFieldName(), DMDAGetFieldName()
+.seealso: DMDASetCoordinateName(), DMDASetFieldName(), DMDAGetFieldName(), DMSetUp()
 @*/
 PetscErrorCode DMDAGetCoordinateName(DM dm,PetscInt nf,const char **name)
 {
   DM_DA *dd = (DM_DA*)dm->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(dm,DM_CLASSID,1,DMDA);
   PetscValidPointer(name,3);
   if (nf < 0 || nf >= dm->dim) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Invalid coordinate number: %D",nf);
+  if (!dd->coordinatename) SETERRQ(PetscObjectComm((PetscObject)dm),PETSC_ERR_ORDER,"You should call DMSetUp() first");
   *name = dd->coordinatename[nf];
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetCorners"
-/*@
+/*@C
    DMDAGetCorners - Returns the global (x,y,z) indices of the lower left
-   corner of the local region, excluding ghost points.
+   corner and size of the local region, excluding ghost points.
 
    Not Collective
 
@@ -237,19 +279,20 @@ PetscErrorCode  DMDAGetCorners(DM da,PetscInt *x,PetscInt *y,PetscInt *z,PetscIn
   DM_DA    *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   /* since the xs, xe ... have all been multiplied by the number of degrees
      of freedom per cell, w = dd->w, we divide that out before returning.*/
   w = dd->w;
-  if (x) *x = dd->xs/w + dd->xo; if (m) *m = (dd->xe - dd->xs)/w;
+  if (x) *x = dd->xs/w + dd->xo;
   /* the y and z have NOT been multiplied by w */
-  if (y) *y = dd->ys + dd->yo; if (n) *n = (dd->ye - dd->ys);
-  if (z) *z = dd->zs + dd->zo; if (p) *p = (dd->ze - dd->zs);
+  if (y) *y = dd->ys + dd->yo;
+  if (z) *z = dd->zs + dd->zo;
+  if (m) *m = (dd->xe - dd->xs)/w;
+  if (n) *n = (dd->ye - dd->ys);
+  if (p) *p = (dd->ze - dd->zs);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetLocalBoundingBox"
 /*@
    DMDAGetLocalBoundingBox - Returns the local bounding box for the DMDA.
 
@@ -264,6 +307,8 @@ PetscErrorCode  DMDAGetCorners(DM da,PetscInt *x,PetscInt *y,PetscInt *z,PetscIn
 
   Level: beginner
 
+  Not supported from Fortran
+
 .keywords: distributed array, get, coordinates
 
 .seealso: DMDAGetCoordinateDA(), DMGetCoordinates(), DMDAGetBoundingBox()
@@ -278,7 +323,7 @@ PetscErrorCode DMDAGetLocalBoundingBox(DM dm,PetscReal lmin[],PetscReal lmax[])
   PetscInt          N,Ni;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(dm,DM_CLASSID,1,DMDA);
   dim  = dm->dim;
   ierr = DMGetCoordinates(dm,&coords);CHKERRQ(ierr);
   if (coords) {
@@ -307,8 +352,6 @@ PetscErrorCode DMDAGetLocalBoundingBox(DM dm,PetscReal lmin[],PetscReal lmax[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetBoundingBox"
 /*@
    DMDAGetBoundingBox - Returns the global bounding box for the DMDA.
 
@@ -337,15 +380,27 @@ PetscErrorCode DMDAGetBoundingBox(DM dm,PetscReal gmin[],PetscReal gmax[])
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
   ierr = PetscMPIIntCast(dm->dim,&count);CHKERRQ(ierr);
   ierr = DMDAGetLocalBoundingBox(dm,lmin,lmax);CHKERRQ(ierr);
-  if (gmin) {ierr = MPI_Allreduce(lmin,gmin,count,MPIU_REAL,MPIU_MIN,PetscObjectComm((PetscObject)dm));CHKERRQ(ierr);}
-  if (gmax) {ierr = MPI_Allreduce(lmax,gmax,count,MPIU_REAL,MPIU_MAX,PetscObjectComm((PetscObject)dm));CHKERRQ(ierr);}
+  if (gmin) {ierr = MPIU_Allreduce(lmin,gmin,count,MPIU_REAL,MPIU_MIN,PetscObjectComm((PetscObject)dm));CHKERRQ(ierr);}
+  if (gmax) {ierr = MPIU_Allreduce(lmax,gmax,count,MPIU_REAL,MPIU_MAX,PetscObjectComm((PetscObject)dm));CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetReducedDMDA"
 /*@
-   DMDAGetReducedDMDA - Gets the DMDA with the same layout but with fewer or more fields
+   DMDAGetReducedDMDA - Deprecated; use DMDACreateCompatibleDMDA()
+
+   Level: deprecated
+@*/
+PetscErrorCode DMDAGetReducedDMDA(DM da,PetscInt nfields,DM *nda)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = DMDACreateCompatibleDMDA(da,nfields,nda);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+   DMDACreateCompatibleDMDA - Creates a DMDA with the same layout but with fewer or more fields
 
    Collective on DMDA
 
@@ -362,7 +417,7 @@ PetscErrorCode DMDAGetBoundingBox(DM dm,PetscReal gmin[],PetscReal gmax[])
 
 .seealso: DMDAGetGhostCorners(), DMSetCoordinates(), DMDASetUniformCoordinates(), DMGetCoordinates(), DMDAGetGhostedCoordinates()
 @*/
-PetscErrorCode  DMDAGetReducedDMDA(DM da,PetscInt nfields,DM *nda)
+PetscErrorCode  DMDACreateCompatibleDMDA(DM da,PetscInt nfields,DM *nda)
 {
   PetscErrorCode   ierr;
   DM_DA            *dd = (DM_DA*)da->data;
@@ -396,9 +451,9 @@ PetscErrorCode  DMDAGetReducedDMDA(DM da,PetscInt nfields,DM *nda)
   } else if (dim == 3) {
     ierr = DMDACreate3d(PetscObjectComm((PetscObject)da),bx,by,bz,stencil_type,M,N,P,m,n,p,nfields,s,lx,ly,lz,nda);CHKERRQ(ierr);
   }
+  ierr = DMSetUp(*nda);CHKERRQ(ierr);
   if (da->coordinates) {
     ierr = PetscObjectReference((PetscObject)da->coordinates);CHKERRQ(ierr);
-
     (*nda)->coordinates = da->coordinates;
   }
 
@@ -415,8 +470,6 @@ PetscErrorCode  DMDAGetReducedDMDA(DM da,PetscInt nfields,DM *nda)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetCoordinateArray"
 /*@C
    DMDAGetCoordinateArray - Gets an array containing the coordinates of the DMDA
 
@@ -430,6 +483,8 @@ PetscErrorCode  DMDAGetReducedDMDA(DM da,PetscInt nfields,DM *nda)
 
   Level: intermediate
 
+  Not supported from Fortran
+
 .keywords: distributed array, get, component name
 
 .seealso: DMDASetCoordinateName(), DMDASetFieldName(), DMDAGetFieldName(), DMDARestoreCoordinateArray()
@@ -448,8 +503,6 @@ PetscErrorCode DMDAGetCoordinateArray(DM dm,void *xc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDARestoreCoordinateArray"
 /*@C
    DMDARestoreCoordinateArray - Sets an array containing the coordinates of the DMDA
 
@@ -461,6 +514,8 @@ PetscErrorCode DMDAGetCoordinateArray(DM dm,void *xc)
 
   Level: intermediate
 
+  Not supported from Fortran
+
 .keywords: distributed array, get, component name
 
 .seealso: DMDASetCoordinateName(), DMDASetFieldName(), DMDAGetFieldName(), DMDAGetCoordinateArray()
