@@ -5,8 +5,6 @@
 
 #include <petsc/private/dmdaimpl.h>    /*I   "petscdmda.h"   I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "VecDuplicate_MPI_DA"
 PetscErrorCode  VecDuplicate_MPI_DA(Vec g,Vec *gg)
 {
   PetscErrorCode ierr;
@@ -22,8 +20,6 @@ PetscErrorCode  VecDuplicate_MPI_DA(Vec g,Vec *gg)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCreateGlobalVector_DA"
 PetscErrorCode  DMCreateGlobalVector_DA(DM da,Vec *g)
 {
   PetscErrorCode ierr;
@@ -50,8 +46,6 @@ PetscErrorCode  DMCreateGlobalVector_DA(DM da,Vec *g)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDACreateNaturalVector"
 /*@
    DMDACreateNaturalVector - Creates a parallel PETSc vector that
    will hold vector values in the natural numbering, rather than in
@@ -87,7 +81,7 @@ PetscErrorCode  DMDACreateNaturalVector(DM da,Vec *g)
   DM_DA          *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidPointer(g,2);
   if (dd->natural) {
     ierr = PetscObjectGetReference((PetscObject)dd->natural,&cnt);CHKERRQ(ierr);
@@ -101,7 +95,7 @@ PetscErrorCode  DMDACreateNaturalVector(DM da,Vec *g)
     ierr = VecCreate(PetscObjectComm((PetscObject)da),g);CHKERRQ(ierr);
     ierr = VecSetSizes(*g,dd->Nlocal,PETSC_DETERMINE);CHKERRQ(ierr);
     ierr = VecSetBlockSize(*g, dd->w);CHKERRQ(ierr);
-    ierr = VecSetType(*g,VECMPI);CHKERRQ(ierr);
+    ierr = VecSetType(*g,da->vectype);CHKERRQ(ierr);
     ierr = PetscObjectReference((PetscObject)*g);CHKERRQ(ierr);
 
     dd->natural = *g;
