@@ -1,7 +1,6 @@
+#include <petscsf.h>
 #include <petsc/private/dmdaimpl.h>     /*I  "petscdmda.h"   I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "FillClosureArray_Static"
 PETSC_STATIC_INLINE PetscErrorCode FillClosureArray_Static(DM dm, PetscSection section, PetscInt nP, const PetscInt points[], PetscScalar *vArray, PetscInt *csize, PetscScalar **array)
 {
   PetscScalar    *a;
@@ -19,7 +18,7 @@ PETSC_STATIC_INLINE PetscErrorCode FillClosureArray_Static(DM dm, PetscSection s
   }
   if (csize) *csize = size;
   if (array) {
-    ierr = DMGetWorkArray(dm, size, PETSC_SCALAR, &a);CHKERRQ(ierr);
+    ierr = DMGetWorkArray(dm, size, MPIU_SCALAR, &a);CHKERRQ(ierr);
     for (i = 0, k = 0; i < nP; ++i) {
       const PetscInt p = points[i];
 
@@ -34,8 +33,6 @@ PETSC_STATIC_INLINE PetscErrorCode FillClosureArray_Static(DM dm, PetscSection s
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FillClosureVec_Private"
 PETSC_STATIC_INLINE PetscErrorCode FillClosureVec_Private(DM dm, PetscSection section, PetscInt nP, const PetscInt points[], PetscScalar *vArray, const PetscScalar *array, InsertMode mode)
 {
   PetscInt       dof, off, d, k, i;
@@ -60,8 +57,6 @@ PETSC_STATIC_INLINE PetscErrorCode FillClosureVec_Private(DM dm, PetscSection se
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "GetPointArray_Private"
 PETSC_STATIC_INLINE PetscErrorCode GetPointArray_Private(DM dm,PetscInt n,PetscInt *points,PetscInt *rn,const PetscInt **rpoints)
 {
   PetscErrorCode ierr;
@@ -70,15 +65,13 @@ PETSC_STATIC_INLINE PetscErrorCode GetPointArray_Private(DM dm,PetscInt n,PetscI
   PetscFunctionBegin;
   if (rn) *rn = n;
   if (rpoints) {
-    ierr     = DMGetWorkArray(dm,n,PETSC_INT,&work);CHKERRQ(ierr);
+    ierr     = DMGetWorkArray(dm,n,MPIU_INT,&work);CHKERRQ(ierr);
     ierr     = PetscMemcpy(work,points,n*sizeof(PetscInt));CHKERRQ(ierr);
     *rpoints = work;
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RestorePointArray_Private"
 PETSC_STATIC_INLINE PetscErrorCode RestorePointArray_Private(DM dm,PetscInt *rn,const PetscInt **rpoints)
 {
   PetscErrorCode ierr;
@@ -86,13 +79,12 @@ PETSC_STATIC_INLINE PetscErrorCode RestorePointArray_Private(DM dm,PetscInt *rn,
   PetscFunctionBegin;
   if (rn) *rn = 0;
   if (rpoints) {
-    ierr = DMRestoreWorkArray(dm,*rn, PETSC_INT, (void*) rpoints);CHKERRQ(ierr);
+    /* note the second argument to DMRestoreWorkArray() is always ignored */
+    ierr = DMRestoreWorkArray(dm,0, MPIU_INT, (void*) rpoints);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetTransitiveClosure"
 PetscErrorCode DMDAGetTransitiveClosure(DM dm, PetscInt p, PetscBool useClosure, PetscInt *closureSize, PetscInt **closure)
 {
   PetscInt       dim = dm->dim;
@@ -103,6 +95,7 @@ PetscErrorCode DMDAGetTransitiveClosure(DM dm, PetscInt p, PetscBool useClosure,
   PetscFunctionBegin;
   if (!useClosure) SETERRQ(PetscObjectComm((PetscObject) dm), PETSC_ERR_SUP, "Star operation is not yet supported");
   PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
+  PetscValidIntPointer(closureSize,4);
   ierr = DMDAGetHeightStratum(dm, -1,  &pStart, &pEnd);CHKERRQ(ierr);
   ierr = DMDAGetHeightStratum(dm,  0,  &cStart, &cEnd);CHKERRQ(ierr);
   ierr = DMDAGetHeightStratum(dm,  1,  &fStart, &fEnd);CHKERRQ(ierr);
@@ -133,8 +126,8 @@ PetscErrorCode DMDAGetTransitiveClosure(DM dm, PetscInt p, PetscBool useClosure,
       PetscInt xf = cy*nxF + cx + xfStart;
       PetscInt yf = c + yfStart;
 
-      if (closureSize) {PetscValidPointer(closureSize, 4); *closureSize = 9;}
-      if (!(*closure)) {ierr = DMGetWorkArray(dm, *closureSize, PETSC_INT, closure);CHKERRQ(ierr);}
+      *closureSize = 9;
+      if (!(*closure)) {ierr = DMGetWorkArray(dm, *closureSize, MPIU_INT, closure);CHKERRQ(ierr);}
       (*closure)[0] = p; (*closure)[1] = yf; (*closure)[2] = xf+1; (*closure)[3] = yf+nyF; (*closure)[4] = xf+0; (*closure)[5] = v+0; (*closure)[6]= v+1; (*closure)[7] = v+nVx+1; (*closure)[8] = v+nVx+0;
     } else {
       /* 6 faces, 12 edges, 8 vertices
@@ -155,15 +148,15 @@ PetscErrorCode DMDAGetTransitiveClosure(DM dm, PetscInt p, PetscBool useClosure,
       PetscInt xf = cy*nxF + cx + xfStart;
       PetscInt yf = c + yfStart;
 
-      if (closureSize) {PetscValidPointer(closureSize, 4); *closureSize = 26;}
-      if (!(*closure)) {ierr = DMGetWorkArray(dm, *closureSize, PETSC_INT, closure);CHKERRQ(ierr);}
+      *closureSize = 26;
+      if (!(*closure)) {ierr = DMGetWorkArray(dm, *closureSize, MPIU_INT, closure);CHKERRQ(ierr);}
 #endif
       SETERRQ(PetscObjectComm((PetscObject)dm), PETSC_ERR_SUP, "Not implemented");
     }
   } else if ((p >= vStart) || (p < vEnd)) {
     /* Vertex */
-    if (closureSize) {PetscValidPointer(closureSize, 4); *closureSize = 1;}
-    if (!(*closure)) {ierr = DMGetWorkArray(dm, *closureSize, PETSC_INT, closure);CHKERRQ(ierr);}
+    *closureSize = 1;
+    if (!(*closure)) {ierr = DMGetWorkArray(dm, *closureSize, MPIU_INT, closure);CHKERRQ(ierr);}
     (*closure)[0] = p;
   } else if ((p >= fStart) || (p < fStart + nXF)) {
     /* X Face */
@@ -172,8 +165,8 @@ PetscErrorCode DMDAGetTransitiveClosure(DM dm, PetscInt p, PetscBool useClosure,
       /* 2 vertices: The bottom vertex has the same numbering as the face */
       PetscInt f = p - xfStart;
 
-      if (closureSize) {PetscValidPointer(closureSize, 4); *closureSize = 3;}
-      if (!(*closure)) {ierr = DMGetWorkArray(dm, *closureSize, PETSC_INT, closure);CHKERRQ(ierr);}
+      *closureSize = 3;
+      if (!(*closure)) {ierr = DMGetWorkArray(dm, *closureSize, MPIU_INT, closure);CHKERRQ(ierr);}
       (*closure)[0] = p; (*closure)[1] = f; (*closure)[2] = f+nVx;
     } else if (dim == 3) {
       /* 4 vertices */
@@ -186,8 +179,8 @@ PetscErrorCode DMDAGetTransitiveClosure(DM dm, PetscInt p, PetscBool useClosure,
       /* 2 vertices: The left vertex has the same numbering as the face */
       PetscInt f = p - yfStart;
 
-      if (closureSize) {PetscValidPointer(closureSize, 4); *closureSize = 3;}
-      if (!(*closure)) {ierr = DMGetWorkArray(dm, *closureSize, PETSC_INT, closure);CHKERRQ(ierr);}
+      *closureSize = 3;
+      if (!(*closure)) {ierr = DMGetWorkArray(dm, *closureSize, MPIU_INT, closure);CHKERRQ(ierr);}
       (*closure)[0] = p; (*closure)[1] = f; (*closure)[2]= f+1;
     } else if (dim == 3) {
       /* 4 vertices */
@@ -205,19 +198,15 @@ PetscErrorCode DMDAGetTransitiveClosure(DM dm, PetscInt p, PetscBool useClosure,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDARestoreTransitiveClosure"
 PetscErrorCode DMDARestoreTransitiveClosure(DM dm, PetscInt p, PetscBool useClosure, PetscInt *closureSize, PetscInt **closure)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = DMRestoreWorkArray(dm, 0, PETSC_INT, closure);CHKERRQ(ierr);
+  ierr = DMRestoreWorkArray(dm, 0, MPIU_INT, closure);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetClosure"
 PetscErrorCode DMDAGetClosure(DM dm, PetscSection section, PetscInt p,PetscInt *n,const PetscInt **closure)
 {
   PetscInt       dim = dm->dim;
@@ -229,7 +218,7 @@ PetscErrorCode DMDAGetClosure(DM dm, PetscSection section, PetscInt p,PetscInt *
   PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
   if (n) PetscValidIntPointer(n,4);
   if (closure) PetscValidPointer(closure, 5);
-  if (!section) {ierr = DMGetDefaultSection(dm, &section);CHKERRQ(ierr);}
+  if (!section) {ierr = DMGetSection(dm, &section);CHKERRQ(ierr);}
   if (!section) SETERRQ(PetscObjectComm((PetscObject)dm), PETSC_ERR_ARG_WRONG, "This DM has not default PetscSection");
   ierr    = DMDAGetHeightStratum(dm, -1,  &pStart, &pEnd);CHKERRQ(ierr);
   ierr    = DMDAGetHeightStratum(dm, 0,   &cStart, &cEnd);CHKERRQ(ierr);
@@ -312,8 +301,6 @@ PetscErrorCode DMDAGetClosure(DM dm, PetscSection section, PetscInt p,PetscInt *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDARestoreClosure"
 /* Zeros n and closure. */
 PetscErrorCode DMDARestoreClosure(DM dm, PetscSection section, PetscInt p,PetscInt *n,const PetscInt **closure)
 {
@@ -327,8 +314,6 @@ PetscErrorCode DMDARestoreClosure(DM dm, PetscSection section, PetscInt p,PetscI
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetClosureScalar"
 PetscErrorCode DMDAGetClosureScalar(DM dm, PetscSection section, PetscInt p, PetscScalar *vArray, PetscInt *csize, PetscScalar **values)
 {
   PetscInt       dim = dm->dim;
@@ -340,7 +325,7 @@ PetscErrorCode DMDAGetClosureScalar(DM dm, PetscSection section, PetscInt p, Pet
   PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
   PetscValidScalarPointer(vArray, 4);
   if (values) PetscValidPointer(values, 6);
-  if (!section) {ierr = DMGetDefaultSection(dm, &section);CHKERRQ(ierr);}
+  if (!section) {ierr = DMGetSection(dm, &section);CHKERRQ(ierr);}
   if (!section) SETERRQ(PetscObjectComm((PetscObject)dm), PETSC_ERR_ARG_WRONG, "This DM has not default PetscSection");
   ierr    = DMDAGetHeightStratum(dm, -1,  &pStart, &pEnd);CHKERRQ(ierr);
   ierr    = DMDAGetHeightStratum(dm, 0,   &cStart, &cEnd);CHKERRQ(ierr);
@@ -442,8 +427,6 @@ PetscErrorCode DMDAGetClosureScalar(DM dm, PetscSection section, PetscInt p, Pet
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAVecGetClosure"
 PetscErrorCode DMDAVecGetClosure(DM dm, PetscSection section, Vec v, PetscInt p, PetscInt *csize, PetscScalar **values)
 {
   PetscScalar    *vArray;
@@ -459,8 +442,6 @@ PetscErrorCode DMDAVecGetClosure(DM dm, PetscSection section, Vec v, PetscInt p,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDARestoreClosureScalar"
 PetscErrorCode DMDARestoreClosureScalar(DM dm, PetscSection section, PetscInt p, PetscScalar *vArray, PetscInt *csize, PetscScalar **values)
 {
   PetscErrorCode ierr;
@@ -468,12 +449,10 @@ PetscErrorCode DMDARestoreClosureScalar(DM dm, PetscSection section, PetscInt p,
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
   PetscValidPointer(values, 6);
-  ierr  = DMRestoreWorkArray(dm, csize ? *csize : 0, PETSC_SCALAR, (void*) values);CHKERRQ(ierr);
+  ierr  = DMRestoreWorkArray(dm, csize ? *csize : 0, MPIU_SCALAR, (void*) values);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAVecRestoreClosure"
 PetscErrorCode DMDAVecRestoreClosure(DM dm, PetscSection section, Vec v, PetscInt p, PetscInt *csize, PetscScalar **values)
 {
   PetscErrorCode ierr;
@@ -486,8 +465,6 @@ PetscErrorCode DMDAVecRestoreClosure(DM dm, PetscSection section, Vec v, PetscIn
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetClosureScalar"
 PetscErrorCode DMDASetClosureScalar(DM dm, PetscSection section, PetscInt p,PetscScalar *vArray, const PetscScalar *values, InsertMode mode)
 {
   PetscInt       dim = dm->dim;
@@ -496,10 +473,10 @@ PetscErrorCode DMDASetClosureScalar(DM dm, PetscSection section, PetscInt p,Pets
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
+  PetscValidHeaderSpecificType(dm, DM_CLASSID, 1,DMDA);
   PetscValidScalarPointer(values, 4);
   PetscValidPointer(values, 5);
-  if (!section) {ierr = DMGetDefaultSection(dm, &section);CHKERRQ(ierr);}
+  if (!section) {ierr = DMGetSection(dm, &section);CHKERRQ(ierr);}
   if (!section) SETERRQ(PetscObjectComm((PetscObject)dm), PETSC_ERR_ARG_WRONG, "This DM has not default PetscSection");
   ierr    = DMDAGetHeightStratum(dm, -1,  &pStart, &pEnd);CHKERRQ(ierr);
   ierr    = DMDAGetHeightStratum(dm, 0,   &cStart, &cEnd);CHKERRQ(ierr);
@@ -598,8 +575,6 @@ PetscErrorCode DMDASetClosureScalar(DM dm, PetscSection section, PetscInt p,Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAVecSetClosure"
 PetscErrorCode DMDAVecSetClosure(DM dm, PetscSection section, Vec v, PetscInt p, const PetscScalar *values, InsertMode mode)
 {
   PetscScalar    *vArray;
@@ -615,8 +590,6 @@ PetscErrorCode DMDAVecSetClosure(DM dm, PetscSection section, Vec v, PetscInt p,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAConvertToCell"
 /*@
   DMDAConvertToCell - Convert (i,j,k) to local cell number
 
@@ -649,8 +622,6 @@ PetscErrorCode DMDAConvertToCell(DM dm, MatStencil s, PetscInt *cell)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAComputeCellGeometry_2D"
 PetscErrorCode DMDAComputeCellGeometry_2D(DM dm, const PetscScalar vertices[], const PetscReal refPoint[], PetscReal J[], PetscReal invJ[], PetscReal *detJ)
 {
   const PetscScalar x0   = vertices[0];
@@ -686,15 +657,13 @@ PetscErrorCode DMDAComputeCellGeometry_2D(DM dm, const PetscScalar vertices[], c
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAComputeCellGeometryFEM"
 PetscErrorCode DMDAComputeCellGeometryFEM(DM dm, PetscInt cell, PetscQuadrature quad, PetscReal v0[], PetscReal J[], PetscReal invJ[], PetscReal detJ[])
 {
   DM               cdm;
   Vec              coordinates;
   const PetscReal *quadPoints;
   PetscScalar     *vertices = NULL;
-  PetscInt         numQuadPoints, csize, dim, d, q;
+  PetscInt         Nq, csize, dim, d, q;
   PetscErrorCode   ierr;
 
   PetscFunctionBegin;
@@ -706,8 +675,8 @@ PetscErrorCode DMDAComputeCellGeometryFEM(DM dm, PetscInt cell, PetscQuadrature
   for (d = 0; d < dim; ++d) v0[d] = PetscRealPart(vertices[d]);
   switch (dim) {
   case 2:
-    ierr = PetscQuadratureGetData(quad, NULL, &numQuadPoints, &quadPoints, NULL);CHKERRQ(ierr);
-    for (q = 0; q < numQuadPoints; ++q) {
+    ierr = PetscQuadratureGetData(quad, NULL, NULL, &Nq, &quadPoints, NULL);CHKERRQ(ierr);
+    for (q = 0; q < Nq; ++q) {
       ierr = DMDAComputeCellGeometry_2D(dm, vertices, &quadPoints[q*dim], J, invJ, detJ);CHKERRQ(ierr);
     }
     break;
@@ -717,3 +686,220 @@ PetscErrorCode DMDAComputeCellGeometryFEM(DM dm, PetscInt cell, PetscQuadrature
   ierr = DMDAVecRestoreClosure(cdm, NULL, coordinates, cell, &csize, &vertices);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
+PetscErrorCode private_DMDALocatePointsIS_2D_Regular(DM dmregular,Vec pos,IS *iscell)
+{
+  PetscInt          n,bs,p,npoints;
+  PetscInt          xs,xe,Xs,Xe,mxlocal;
+  PetscInt          ys,ye,Ys,Ye,mylocal;
+  PetscInt          d,c0,c1;
+  PetscReal         gmin_l[2],gmax_l[2],dx[2];
+  PetscReal         gmin[2],gmax[2];
+  PetscInt          *cellidx;
+  Vec               coor;
+  const PetscScalar *_coor;
+  PetscErrorCode    ierr;
+
+  PetscFunctionBegin;
+  ierr = DMDAGetCorners(dmregular,&xs,&ys,0,&xe,&ye,0);CHKERRQ(ierr);
+  ierr = DMDAGetGhostCorners(dmregular,&Xs,&Ys,0,&Xe,&Ye,0);CHKERRQ(ierr);
+  xe += xs; Xe += Xs; if (xs != Xs) xs -= 1;
+  ye += ys; Ye += Ys; if (ys != Ys) ys -= 1;
+  
+  mxlocal = xe - xs - 1;
+  mylocal = ye - ys - 1;
+  
+  ierr = DMGetCoordinatesLocal(dmregular,&coor);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(coor,&_coor);CHKERRQ(ierr);
+  c0 = (xs-Xs) + (ys-Ys)*(Xe-Xs);
+  c1 = (xe-2-Xs+1) + (ye-2-Ys+1)*(Xe-Xs);
+  
+  gmin_l[0] = PetscRealPart(_coor[2*c0+0]);
+  gmin_l[1] = PetscRealPart(_coor[2*c0+1]);
+  
+  gmax_l[0] = PetscRealPart(_coor[2*c1+0]);
+  gmax_l[1] = PetscRealPart(_coor[2*c1+1]);
+  
+  dx[0] = (gmax_l[0]-gmin_l[0])/((PetscReal)mxlocal);
+  dx[1] = (gmax_l[1]-gmin_l[1])/((PetscReal)mylocal);
+  
+  ierr = VecRestoreArrayRead(coor,&_coor);CHKERRQ(ierr);
+  
+  ierr = DMDAGetBoundingBox(dmregular,gmin,gmax);CHKERRQ(ierr);
+  
+  ierr = VecGetLocalSize(pos,&n);CHKERRQ(ierr);
+  ierr = VecGetBlockSize(pos,&bs);CHKERRQ(ierr);
+  npoints = n/bs;
+  
+  ierr = PetscMalloc1(npoints,&cellidx);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(pos,&_coor);CHKERRQ(ierr);
+  for (p=0; p<npoints; p++) {
+    PetscReal coor_p[2];
+    PetscInt  mi[2];
+    
+    coor_p[0] = PetscRealPart(_coor[2*p]);
+    coor_p[1] = PetscRealPart(_coor[2*p+1]);
+    
+    cellidx[p] = DMLOCATEPOINT_POINT_NOT_FOUND;
+    
+    if (coor_p[0] < gmin_l[0]) { continue; }
+    if (coor_p[0] > gmax_l[0]) { continue; }
+    if (coor_p[1] < gmin_l[1]) { continue; }
+    if (coor_p[1] > gmax_l[1]) { continue; }
+    
+    for (d=0; d<2; d++) {
+      mi[d] = (PetscInt)( (coor_p[d] - gmin[d])/dx[d] );
+    }
+    
+    if (mi[0] < xs)     { continue; }
+    if (mi[0] > (xe-1)) { continue; }
+    if (mi[1] < ys)     { continue; }
+    if (mi[1] > (ye-1)) { continue; }
+    
+    if (mi[0] == (xe-1)) { mi[0]--; }
+    if (mi[1] == (ye-1)) { mi[1]--; }
+    
+    cellidx[p] = (mi[0]-xs) + (mi[1]-ys) * mxlocal;
+  }
+  ierr = VecRestoreArrayRead(pos,&_coor);CHKERRQ(ierr);
+  ierr = ISCreateGeneral(PETSC_COMM_SELF,npoints,cellidx,PETSC_OWN_POINTER,iscell);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode private_DMDALocatePointsIS_3D_Regular(DM dmregular,Vec pos,IS *iscell)
+{
+  PetscInt          n,bs,p,npoints;
+  PetscInt          xs,xe,Xs,Xe,mxlocal;
+  PetscInt          ys,ye,Ys,Ye,mylocal;
+  PetscInt          zs,ze,Zs,Ze,mzlocal;
+  PetscInt          d,c0,c1;
+  PetscReal         gmin_l[3],gmax_l[3],dx[3];
+  PetscReal         gmin[3],gmax[3];
+  PetscInt          *cellidx;
+  Vec               coor;
+  const PetscScalar *_coor;
+  PetscErrorCode    ierr;
+
+  PetscFunctionBegin;
+  ierr = DMDAGetCorners(dmregular,&xs,&ys,&zs,&xe,&ye,&ze);CHKERRQ(ierr);
+  ierr = DMDAGetGhostCorners(dmregular,&Xs,&Ys,&Zs,&Xe,&Ye,&Ze);CHKERRQ(ierr);
+  xe += xs; Xe += Xs; if (xs != Xs) xs -= 1;
+  ye += ys; Ye += Ys; if (ys != Ys) ys -= 1;
+  ze += zs; Ze += Zs; if (zs != Zs) zs -= 1;
+  
+  mxlocal = xe - xs - 1;
+  mylocal = ye - ys - 1;
+  mzlocal = ze - zs - 1;
+  
+  ierr = DMGetCoordinatesLocal(dmregular,&coor);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(coor,&_coor);CHKERRQ(ierr);
+  c0 = (xs-Xs)     + (ys-Ys)    *(Xe-Xs) + (zs-Zs)    *(Xe-Xs)*(Ye-Ys);
+  c1 = (xe-2-Xs+1) + (ye-2-Ys+1)*(Xe-Xs) + (ze-2-Zs+1)*(Xe-Xs)*(Ye-Ys);
+  
+  gmin_l[0] = PetscRealPart(_coor[3*c0+0]);
+  gmin_l[1] = PetscRealPart(_coor[3*c0+1]);
+  gmin_l[2] = PetscRealPart(_coor[3*c0+2]);
+  
+  gmax_l[0] = PetscRealPart(_coor[3*c1+0]);
+  gmax_l[1] = PetscRealPart(_coor[3*c1+1]);
+  gmax_l[2] = PetscRealPart(_coor[3*c1+2]);
+  
+  dx[0] = (gmax_l[0]-gmin_l[0])/((PetscReal)mxlocal);
+  dx[1] = (gmax_l[1]-gmin_l[1])/((PetscReal)mylocal);
+  dx[2] = (gmax_l[2]-gmin_l[2])/((PetscReal)mzlocal);
+  
+  ierr = VecRestoreArrayRead(coor,&_coor);CHKERRQ(ierr);
+  
+  ierr = DMDAGetBoundingBox(dmregular,gmin,gmax);CHKERRQ(ierr);
+  
+  ierr = VecGetLocalSize(pos,&n);CHKERRQ(ierr);
+  ierr = VecGetBlockSize(pos,&bs);CHKERRQ(ierr);
+  npoints = n/bs;
+  
+  ierr = PetscMalloc1(npoints,&cellidx);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(pos,&_coor);CHKERRQ(ierr);
+  for (p=0; p<npoints; p++) {
+    PetscReal coor_p[3];
+    PetscInt  mi[3];
+    
+    coor_p[0] = PetscRealPart(_coor[3*p]);
+    coor_p[1] = PetscRealPart(_coor[3*p+1]);
+    coor_p[2] = PetscRealPart(_coor[3*p+2]);
+    
+    cellidx[p] = DMLOCATEPOINT_POINT_NOT_FOUND;
+    
+    if (coor_p[0] < gmin_l[0]) { continue; }
+    if (coor_p[0] > gmax_l[0]) { continue; }
+    if (coor_p[1] < gmin_l[1]) { continue; }
+    if (coor_p[1] > gmax_l[1]) { continue; }
+    if (coor_p[2] < gmin_l[2]) { continue; }
+    if (coor_p[2] > gmax_l[2]) { continue; }
+    
+    for (d=0; d<3; d++) {
+      mi[d] = (PetscInt)( (coor_p[d] - gmin[d])/dx[d] );
+    }
+    
+    if (mi[0] < xs)     { continue; }
+    if (mi[0] > (xe-1)) { continue; }
+    if (mi[1] < ys)     { continue; }
+    if (mi[1] > (ye-1)) { continue; }
+    if (mi[2] < zs)     { continue; }
+    if (mi[2] > (ze-1)) { continue; }
+    
+    if (mi[0] == (xe-1)) { mi[0]--; }
+    if (mi[1] == (ye-1)) { mi[1]--; }
+    if (mi[2] == (ze-1)) { mi[2]--; }
+    
+    cellidx[p] = (mi[0]-xs) + (mi[1]-ys) * mxlocal + (mi[2]-zs) * mxlocal * mylocal;
+  }
+  ierr = VecRestoreArrayRead(pos,&_coor);CHKERRQ(ierr);
+  ierr = ISCreateGeneral(PETSC_COMM_SELF,npoints,cellidx,PETSC_OWN_POINTER,iscell);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode DMLocatePoints_DA_Regular(DM dm,Vec pos,DMPointLocationType ltype,PetscSF cellSF)
+{
+  IS iscell;
+  PetscSFNode *cells;
+  PetscInt p,bs,dim,npoints,nfound;
+  const PetscInt *boxCells;
+  PetscErrorCode ierr;
+  
+  PetscFunctionBegin;
+  ierr = VecGetBlockSize(pos,&dim);CHKERRQ(ierr);
+  switch (dim) {
+    case 1:
+      SETERRQ(PetscObjectComm((PetscObject)dm),PETSC_ERR_SUP,"Support not provided for 1D");
+      break;
+    case 2:
+      ierr = private_DMDALocatePointsIS_2D_Regular(dm,pos,&iscell);CHKERRQ(ierr);
+      break;
+    case 3:
+      ierr = private_DMDALocatePointsIS_3D_Regular(dm,pos,&iscell);CHKERRQ(ierr);
+      break;
+    default:
+      SETERRQ(PetscObjectComm((PetscObject)dm),PETSC_ERR_SUP,"Unsupport spatial dimension");
+      break;
+  }
+  
+  ierr = VecGetLocalSize(pos,&npoints);CHKERRQ(ierr);
+  ierr = VecGetBlockSize(pos,&bs);CHKERRQ(ierr);
+  npoints = npoints / bs;
+  
+  ierr = PetscMalloc1(npoints, &cells);CHKERRQ(ierr);
+  ierr = ISGetIndices(iscell, &boxCells);CHKERRQ(ierr);
+  
+  for (p=0; p<npoints; p++) {
+    cells[p].rank  = 0;
+    cells[p].index = DMLOCATEPOINT_POINT_NOT_FOUND;
+    
+    cells[p].index = boxCells[p];
+  }
+  ierr = ISRestoreIndices(iscell, &boxCells);CHKERRQ(ierr);
+  
+  nfound = npoints;
+  ierr = PetscSFSetGraph(cellSF, npoints, nfound, NULL, PETSC_OWN_POINTER, cells, PETSC_OWN_POINTER);CHKERRQ(ierr);
+  ierr = ISDestroy(&iscell);CHKERRQ(ierr);
+  
+  PetscFunctionReturn(0);
+}
