@@ -5,11 +5,9 @@
 
 #include <petsc/private/dmdaimpl.h>    /*I   "petscdmda.h"   I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetGhostCorners"
-/*@
+/*@C
    DMDAGetGhostCorners - Returns the global (x,y,z) indices of the lower left
-   corner of the local region, including ghost points.
+   corner and size of the local region, including ghost points.
 
    Not Collective
 
@@ -42,13 +40,16 @@ PetscErrorCode  DMDAGetGhostCorners(DM da,PetscInt *x,PetscInt *y,PetscInt *z,Pe
   DM_DA    *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   /* since the xs, xe ... have all been multiplied by the number of degrees
      of freedom per cell, w = dd->w, we divide that out before returning.*/
   w = dd->w;
-  if (x) *x = dd->Xs/w + dd->xo; if (m) *m = (dd->Xe - dd->Xs)/w;
-  if (y) *y = dd->Ys   + dd->yo; if (n) *n = (dd->Ye - dd->Ys);
-  if (z) *z = dd->Zs   + dd->zo; if (p) *p = (dd->Ze - dd->Zs);
+  if (x) *x = dd->Xs/w + dd->xo;
+  if (y) *y = dd->Ys   + dd->yo;
+  if (z) *z = dd->Zs   + dd->zo;
+  if (m) *m = (dd->Xe - dd->Xs)/w;
+  if (n) *n = (dd->Ye - dd->Ys);
+  if (p) *p = (dd->Ze - dd->Zs);
   PetscFunctionReturn(0);
 }
 
