@@ -5,8 +5,6 @@
 
 #include <petsc/private/dmdaimpl.h>    /*I   "petscdmda.h"   I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetNatural_Private"
 /*
    Gets the natural number for each global number on the process.
 
@@ -51,9 +49,7 @@ PetscErrorCode DMDAGetNatural_Private(DM da,PetscInt *outNlocal,IS *isnatural)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDASetAOType"
-/*@
+/*@C
    DMDASetAOType - Sets the type of application ordering for a distributed array.
 
    Collective on DMDA
@@ -82,7 +78,7 @@ PetscErrorCode  DMDASetAOType(DM da,AOType aotype)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   ierr = PetscObjectTypeCompare((PetscObject)da,DMDA,&isdmda);CHKERRQ(ierr);
   if (!isdmda) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_SUP,"Requires a DMDA as input");
   /* now we can safely dereference */
@@ -98,8 +94,6 @@ PetscErrorCode  DMDASetAOType(DM da,AOType aotype)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetAO"
 /*@
    DMDAGetAO - Gets the application ordering context for a distributed array.
 
@@ -120,6 +114,8 @@ PetscErrorCode  DMDASetAOType(DM da,AOType aotype)
    for each node (rather than 1 component for the whole grid, then the next
    component, etc.)
 
+   Do NOT call AODestroy() on the ao returned by this function.
+
 .keywords: distributed array, get, global, indices, local-to-global
 
 .seealso: DMDACreate2d(), DMDASetAOType(), DMDAGetGhostCorners(), DMDAGetCorners(), DMDALocalToGlocal()
@@ -133,7 +129,7 @@ PetscErrorCode  DMDAGetAO(DM da,AO *ao)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidPointer(ao,2);
   ierr = PetscObjectTypeCompare((PetscObject)da,DMDA,&isdmda);CHKERRQ(ierr);
   if (!isdmda) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_SUP,"Requires a DMDA as input");
