@@ -1,9 +1,7 @@
 
-#include <petsc-private/dmdaimpl.h>    /*I   "petscdmda.h"   I*/
+#include <petsc/private/dmdaimpl.h>    /*I   "petscdmda.h"   I*/
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDACreatePF"
 /*@C
    DMDACreatePF - Creates an appropriately dimensioned PF mathematical function object
       from a DMDA.
@@ -18,6 +16,7 @@
 
    Level: advanced
 
+   Not supported from Fortran
 
 .keywords:  distributed array, grid function
 
@@ -29,9 +28,9 @@ PetscErrorCode  DMDACreatePF(DM da,PF *pf)
   DM_DA          *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   PetscValidPointer(pf,2);
-  ierr = PFCreate(PetscObjectComm((PetscObject)da),dd->dim,dd->w,pf);CHKERRQ(ierr);
+  ierr = PFCreate(PetscObjectComm((PetscObject)da),da->dim,dd->w,pf);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
