@@ -3,13 +3,11 @@
   Code for manipulating distributed regular arrays in parallel.
 */
 
-#include <petsc-private/dmdaimpl.h>    /*I   "petscdmda.h"   I*/
-extern PetscErrorCode DMDALocalToLocalCreate(DM);
+#include <petsc/private/dmdaimpl.h>    /*I   "petscdmda.h"   I*/
+extern PetscErrorCode DMLocalToLocalCreate_DA(DM);
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAGetScatter"
 /*@C
-   DMDAGetScatter - Gets the local-to-global, local-to-global, and
+   DMDAGetScatter - Gets the global-to-local, and
    local-to-local vector scatter contexts for a distributed array.
 
    Collective on DMDA
@@ -18,8 +16,7 @@ extern PetscErrorCode DMDALocalToLocalCreate(DM);
 .  da - the distributed array
 
    Output Parameters:
-+  ltog - local-to-global scatter context (may be NULL)
-.  gtol - global-to-local scatter context (may be NULL)
++  gtol - global-to-local scatter context (may be NULL)
 -  ltol - local-to-local scatter context (may be NULL)
 
    Level: developer
@@ -33,18 +30,17 @@ extern PetscErrorCode DMDALocalToLocalCreate(DM);
 
 .seealso: DMGlobalToLocalBegin(), DMGlobalToLocalEnd(), DMLocalToGlobalBegin()
 @*/
-PetscErrorCode  DMDAGetScatter(DM da,VecScatter *ltog,VecScatter *gtol,VecScatter *ltol)
+PetscErrorCode  DMDAGetScatter(DM da,VecScatter *gtol,VecScatter *ltol)
 {
   PetscErrorCode ierr;
   DM_DA          *dd = (DM_DA*)da->data;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(da,DM_CLASSID,1);
-  if (ltog) *ltog = dd->ltog;
+  PetscValidHeaderSpecificType(da,DM_CLASSID,1,DMDA);
   if (gtol) *gtol = dd->gtol;
   if (ltol) {
     if (!dd->ltol) {
-      ierr = DMDALocalToLocalCreate(da);CHKERRQ(ierr);
+      ierr = DMLocalToLocalCreate_DA(da);CHKERRQ(ierr);
     }
     *ltol = dd->ltol;
   }
