@@ -1,22 +1,24 @@
-#include <petsc-private/fortranimpl.h>
-#include <petsc-private/dmdaimpl.h>
+#include <petsc/private/fortranimpl.h>
+#include <petsc/private/dmdaimpl.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define dmdagetownershipranges_        DMDAGETOWNERSHIPRANGES
 #define dmdagetneighbors_              DMDAGETNEIGHBORS
+#define dmdagetrefinementfactor_       DMDAGETREFINEMENTFACTOR
 #elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
 #define dmdagetownershipranges_        dmdagetownershipranges
 #define dmdagetneighbors_              dmdagetneighbors
+#define dmdagetrefinementfactor_       dmdagetrefinementfactor
 #endif
 
 PETSC_EXTERN void PETSC_STDCALL dmdagetneighbors_(DM *da,PetscMPIInt *ranks,PetscErrorCode *ierr)
 {
   const PetscMPIInt *r;
-  PetscInt          n;
-  DM_DA             *dd = (DM_DA*)(*da)->data;
+  PetscInt          n,dim;
 
   *ierr = DMDAGetNeighbors(*da,&r);if (*ierr) return;
-  if (dd->dim == 2) n = 9;
+  *ierr = DMGetDimension(*da,&dim);if (*ierr) return;
+  if (dim == 2) n = 9;
   else n = 27;
   *ierr = PetscMemcpy(ranks,r,n*sizeof(PetscMPIInt));
 }
@@ -42,3 +44,11 @@ PETSC_EXTERN void PETSC_STDCALL dmdagetownershipranges_(DM *da,PetscInt lx[],Pet
   }
 }
 
+PETSC_EXTERN void PETSC_STDCALL dmdagetrefinementfactor_(DM *da,PetscInt *lx,PetscInt *ly,PetscInt *lz,PetscErrorCode *ierr)
+{
+  CHKFORTRANNULLINTEGER(lx);
+  CHKFORTRANNULLINTEGER(ly);
+  CHKFORTRANNULLINTEGER(lz);
+  *ierr = DMDAGetRefinementFactor(*da,lx,ly,lz);
+}
+
