@@ -4,7 +4,7 @@
 */
 
 #include <petsc/private/dmdaimpl.h>      /*I  "petscdmda.h"   I*/
-#include <petsc/private/vecimpl.h>
+#include <petsc/private/glvisvecimpl.h>
 #include <petscdraw.h>
 #include <petscviewerhdf5.h>
 
@@ -12,10 +12,11 @@
         The data that is passed into the graphics callback
 */
 typedef struct {
-  PetscInt          m,n,step,k;
-  PetscReal         min,max,scale;
+  PetscMPIInt       rank;
+  PetscInt          m,n,dof,k;
+  PetscReal         xmin,xmax,ymin,ymax,min,max;
   const PetscScalar *xy,*v;
-  PetscBool         showgrid;
+  PetscBool         showaxis,showgrid;
   const char        *name0,*name1;
 } ZoomCtx;
 
@@ -24,68 +25,47 @@ typedef struct {
     in one particular set of coordinates. It is a callback
     called from PetscDrawZoom()
 */
-#undef __FUNCT__
-#define __FUNCT__ "VecView_MPI_Draw_DA2d_Zoom"
 PetscErrorCode VecView_MPI_Draw_DA2d_Zoom(PetscDraw draw,void *ctx)
 {
   ZoomCtx           *zctx = (ZoomCtx*)ctx;
   PetscErrorCode    ierr;
-  PetscInt          m,n,i,j,k,step,id,c1,c2,c3,c4;
-  PetscReal         s,min,max,x1,x2,x3,x4,y_1,y2,y3,y4,xmin = PETSC_MAX_REAL,xmax = PETSC_MIN_REAL,ymin = PETSC_MAX_REAL,ymax = PETSC_MIN_REAL;
-  PetscReal         xminf,xmaxf,yminf,ymaxf,w;
-  const PetscScalar *v,*xy;
-  char              value[16];
-  size_t            len;
+  PetscInt          m,n,i,j,k,dof,id,c1,c2,c3,c4;
+  PetscReal         min,max,x1,x2,x3,x4,y_1,y2,y3,y4;
+  const PetscScalar *xy,*v;
 
   PetscFunctionBegin;
   m    = zctx->m;
   n    = zctx->n;
-  step = zctx->step;
+  dof  = zctx->dof;
   k    = zctx->k;
-  v    = zctx->v;
   xy   = zctx->xy;
-  s    = zctx->scale;
+  v    = zctx->v;
   min  = zctx->min;
   max  = zctx->max;
 
   /* PetscDraw the contour plot patch */
+  ierr = PetscDrawCollectiveBegin(draw);CHKERRQ(ierr);
   for (j=0; j<n-1; j++) {
     for (i=0; i<m-1; i++) {
       id   = i+j*m;
       x1   = PetscRealPart(xy[2*id]);
       y_1  = PetscRealPart(xy[2*id+1]);
-      c1   = (int)(PETSC_DRAW_BASIC_COLORS+s*(PetscClipInterval(PetscRealPart(v[k+step*id]),min,max)-min));
-      xmin = PetscMin(xmin,x1);
-      ymin = PetscMin(ymin,y_1);
-      xmax = PetscMax(xmax,x1);
-      ymax = PetscMax(ymax,y_1);
+      c1   = PetscDrawRealToColor(PetscRealPart(v[k+dof*id]),min,max);
 
       id   = i+j*m+1;
       x2   = PetscRealPart(xy[2*id]);
       y2   = PetscRealPart(xy[2*id+1]);
-      c2   = (int)(PETSC_DRAW_BASIC_COLORS+s*(PetscClipInterval(PetscRealPart(v[k+step*id]),min,max)-min));
-      xmin = PetscMin(xmin,x2);
-      ymin = PetscMin(ymin,y2);
-      xmax = PetscMax(xmax,x2);
-      ymax = PetscMax(ymax,y2);
+      c2   = PetscDrawRealToColor(PetscRealPart(v[k+dof*id]),min,max);
 
       id   = i+j*m+1+m;
       x3   = PetscRealPart(xy[2*id]);
       y3   = PetscRealPart(xy[2*id+1]);
-      c3   = (int)(PETSC_DRAW_BASIC_COLORS+s*(PetscClipInterval(PetscRealPart(v[k+step*id]),min,max)-min));
-      xmin = PetscMin(xmin,x3);
-      ymin = PetscMin(ymin,y3);
-      xmax = PetscMax(xmax,x3);
-      ymax = PetscMax(ymax,y3);
+      c3   = PetscDrawRealToColor(PetscRealPart(v[k+dof*id]),min,max);
 
-      id = i+j*m+m;
-      x4 = PetscRealPart(xy[2*id]);
-      y4 = PetscRealPart(xy[2*id+1]);
-      c4 = (int)(PETSC_DRAW_BASIC_COLORS+s*(PetscClipInterval(PetscRealPart(v[k+step*id]),min,max)-min));
-      xmin = PetscMin(xmin,x4);
-      ymin = PetscMin(ymin,y4);
-      xmax = PetscMax(xmax,x4);
-      ymax = PetscMax(ymax,y4);
+      id   = i+j*m+m;
+      x4   = PetscRealPart(xy[2*id]);
+      y4   = PetscRealPart(xy[2*id+1]);
+      c4   = PetscDrawRealToColor(PetscRealPart(v[k+dof*id]),min,max);
 
       ierr = PetscDrawTriangle(draw,x1,y_1,x2,y2,x3,y3,c1,c2,c3);CHKERRQ(ierr);
       ierr = PetscDrawTriangle(draw,x1,y_1,x3,y3,x4,y4,c1,c3,c4);CHKERRQ(ierr);
@@ -97,47 +77,48 @@ PetscErrorCode VecView_MPI_Draw_DA2d_Zoom(PetscDraw draw,void *ctx)
       }
     }
   }
-  if (zctx->name0) {
-    PetscReal xl,yl,xr,yr,x,y;
-    ierr = PetscDrawGetCoordinates(draw,&xl,&yl,&xr,&yr);CHKERRQ(ierr);
-    x    = xl + .3*(xr - xl);
-    xl   = xl + .01*(xr - xl);
-    y    = yr - .3*(yr - yl);
-    yl   = yl + .01*(yr - yl);
-    ierr = PetscDrawString(draw,x,yl,PETSC_DRAW_BLACK,zctx->name0);CHKERRQ(ierr);
-    ierr = PetscDrawStringVertical(draw,xl,y,PETSC_DRAW_BLACK,zctx->name1);CHKERRQ(ierr);
+  if (zctx->showaxis && !zctx->rank) {
+    if (zctx->name0 || zctx->name1) {
+      PetscReal xl,yl,xr,yr,x,y;
+      ierr = PetscDrawGetCoordinates(draw,&xl,&yl,&xr,&yr);CHKERRQ(ierr);
+      x  = xl + .30*(xr - xl);
+      xl = xl + .01*(xr - xl);
+      y  = yr - .30*(yr - yl);
+      yl = yl + .01*(yr - yl);
+      if (zctx->name0) {ierr = PetscDrawString(draw,x,yl,PETSC_DRAW_BLACK,zctx->name0);CHKERRQ(ierr);}
+      if (zctx->name1) {ierr = PetscDrawStringVertical(draw,xl,y,PETSC_DRAW_BLACK,zctx->name1);CHKERRQ(ierr);}
+    }
+    /*
+       Ideally we would use the PetscDrawAxis object to manage displaying the coordinate limits
+       but that may require some refactoring.
+    */
+    {
+      double xmin = (double)zctx->xmin, ymin = (double)zctx->ymin;
+      double xmax = (double)zctx->xmax, ymax = (double)zctx->ymax;
+      char   value[16]; size_t len; PetscReal w;
+      ierr = PetscSNPrintf(value,16,"%0.2e",xmin);CHKERRQ(ierr);
+      ierr = PetscDrawString(draw,xmin,ymin - .05*(ymax - ymin),PETSC_DRAW_BLACK,value);CHKERRQ(ierr);
+      ierr = PetscSNPrintf(value,16,"%0.2e",xmax);CHKERRQ(ierr);
+      ierr = PetscStrlen(value,&len);CHKERRQ(ierr);
+      ierr = PetscDrawStringGetSize(draw,&w,NULL);CHKERRQ(ierr);
+      ierr = PetscDrawString(draw,xmax - len*w,ymin - .05*(ymax - ymin),PETSC_DRAW_BLACK,value);CHKERRQ(ierr);
+      ierr = PetscSNPrintf(value,16,"%0.2e",ymin);CHKERRQ(ierr);
+      ierr = PetscDrawString(draw,xmin - .05*(xmax - xmin),ymin,PETSC_DRAW_BLACK,value);CHKERRQ(ierr);
+      ierr = PetscSNPrintf(value,16,"%0.2e",ymax);CHKERRQ(ierr);
+      ierr = PetscDrawString(draw,xmin - .05*(xmax - xmin),ymax,PETSC_DRAW_BLACK,value);CHKERRQ(ierr);
+    }
   }
-  /*
-     Ideally we would use the PetscDrawAxis object to manage displaying the coordinate limits 
-     but that may require some refactoring.
-  */
-  ierr = MPI_Allreduce(&xmin,&xminf,1,MPIU_REAL,MPIU_MIN,PetscObjectComm((PetscObject)draw));CHKERRQ(ierr);
-  ierr = MPI_Allreduce(&xmax,&xmaxf,1,MPIU_REAL,MPIU_MAX,PetscObjectComm((PetscObject)draw));CHKERRQ(ierr);
-  ierr = MPI_Allreduce(&ymin,&yminf,1,MPIU_REAL,MPIU_MIN,PetscObjectComm((PetscObject)draw));CHKERRQ(ierr);
-  ierr = MPI_Allreduce(&ymax,&ymaxf,1,MPIU_REAL,MPIU_MAX,PetscObjectComm((PetscObject)draw));CHKERRQ(ierr);
-  ierr = PetscSNPrintf(value,16,"%f",xminf);CHKERRQ(ierr);
-  ierr = PetscDrawString(draw,xminf,yminf - .05*(ymaxf - yminf),PETSC_DRAW_BLACK,value);CHKERRQ(ierr);
-  ierr = PetscSNPrintf(value,16,"%f",xmaxf);CHKERRQ(ierr);
-  ierr = PetscStrlen(value,&len);CHKERRQ(ierr);
-  ierr = PetscDrawStringGetSize(draw,&w,NULL);CHKERRQ(ierr);
-  ierr = PetscDrawString(draw,xmaxf - len*w,yminf - .05*(ymaxf - yminf),PETSC_DRAW_BLACK,value);CHKERRQ(ierr);
-  ierr = PetscSNPrintf(value,16,"%f",yminf);CHKERRQ(ierr);
-  ierr = PetscDrawString(draw,xminf - .05*(xmaxf - xminf),yminf,PETSC_DRAW_BLACK,value);CHKERRQ(ierr);
-  ierr = PetscSNPrintf(value,16,"%f",ymaxf);CHKERRQ(ierr);
-  ierr = PetscDrawString(draw,xminf - .05*(xmaxf - xminf),ymaxf,PETSC_DRAW_BLACK,value);CHKERRQ(ierr);
+  ierr = PetscDrawCollectiveEnd(draw);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecView_MPI_Draw_DA2d"
 PetscErrorCode VecView_MPI_Draw_DA2d(Vec xin,PetscViewer viewer)
 {
   DM                 da,dac,dag;
   PetscErrorCode     ierr;
-  PetscMPIInt        rank;
   PetscInt           N,s,M,w,ncoors = 4;
   const PetscInt     *lx,*ly;
-  PetscReal          coors[4],ymin,ymax,xmin,xmax;
+  PetscReal          coors[4];
   PetscDraw          draw,popup;
   PetscBool          isnull,useports = PETSC_FALSE;
   MPI_Comm           comm;
@@ -153,22 +134,25 @@ PetscErrorCode VecView_MPI_Draw_DA2d(Vec xin,PetscViewer viewer)
 
   PetscFunctionBegin;
   zctx.showgrid = PETSC_FALSE;
+  zctx.showaxis = PETSC_TRUE;
 
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
-  ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr); if (isnull) PetscFunctionReturn(0);
+  ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr);
+  if (isnull) PetscFunctionReturn(0);
+
   ierr = PetscViewerDrawGetBounds(viewer,&nbounds,&bounds);CHKERRQ(ierr);
 
   ierr = VecGetDM(xin,&da);CHKERRQ(ierr);
   if (!da) SETERRQ(PetscObjectComm((PetscObject)xin),PETSC_ERR_ARG_WRONG,"Vector not generated from a DMDA");
 
   ierr = PetscObjectGetComm((PetscObject)xin,&comm);CHKERRQ(ierr);
-  ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(comm,&zctx.rank);CHKERRQ(ierr);
 
   ierr = DMDAGetInfo(da,0,&M,&N,0,&zctx.m,&zctx.n,0,&w,&s,&bx,&by,0,&st);CHKERRQ(ierr);
   ierr = DMDAGetOwnershipRanges(da,&lx,&ly,NULL);CHKERRQ(ierr);
 
   /*
-        Obtain a sequential vector that is going to contain the local values plus ONE layer of
+     Obtain a sequential vector that is going to contain the local values plus ONE layer of
      ghosted values to draw the graphics from. We also need its corresponding DMDA (dac) that will
      update the local values pluse ONE layer of ghost values.
   */
@@ -180,6 +164,7 @@ PetscErrorCode VecView_MPI_Draw_DA2d(Vec xin,PetscViewer viewer)
          create a special DMDA to handle one level of ghost points for graphics
       */
       ierr = DMDACreate2d(comm,DM_BOUNDARY_NONE,DM_BOUNDARY_NONE,DMDA_STENCIL_BOX,M,N,zctx.m,zctx.n,w,1,lx,ly,&dac);CHKERRQ(ierr);
+      ierr = DMSetUp(dac);CHKERRQ(ierr);
       ierr = PetscInfo(da,"Creating auxilary DMDA for managing graphics ghost points\n");CHKERRQ(ierr);
     } else {
       /* otherwise we can use the da we already have */
@@ -213,7 +198,9 @@ PetscErrorCode VecView_MPI_Draw_DA2d(Vec xin,PetscViewer viewer)
   ierr = DMGlobalToLocalEnd(dac,xin,INSERT_VALUES,xlocal);CHKERRQ(ierr);
   ierr = VecGetArrayRead(xlocal,&zctx.v);CHKERRQ(ierr);
 
-  /* get coordinates of nodes */
+  /*
+      Get coordinates of nodes
+  */
   ierr = DMGetCoordinates(da,&xcoor);CHKERRQ(ierr);
   if (!xcoor) {
     ierr = DMDASetUniformCoordinates(da,0.0,1.0,0.0,1.0,0.0,0.0);CHKERRQ(ierr);
@@ -221,25 +208,30 @@ PetscErrorCode VecView_MPI_Draw_DA2d(Vec xin,PetscViewer viewer)
   }
 
   /*
-      Determine the min and max  coordinates in plot
+      Determine the min and max coordinates in plot
   */
-  ierr     = VecStrideMin(xcoor,0,NULL,&xmin);CHKERRQ(ierr);
-  ierr     = VecStrideMax(xcoor,0,NULL,&xmax);CHKERRQ(ierr);
-  ierr     = VecStrideMin(xcoor,1,NULL,&ymin);CHKERRQ(ierr);
-  ierr     = VecStrideMax(xcoor,1,NULL,&ymax);CHKERRQ(ierr);
-  coors[0] = xmin - .05*(xmax- xmin); coors[2] = xmax + .05*(xmax - xmin);
-  coors[1] = ymin - .05*(ymax- ymin); coors[3] = ymax + .05*(ymax - ymin);
-  ierr     = PetscInfo4(da,"Preparing DMDA 2d contour plot coordinates %g %g %g %g\n",(double)coors[0],(double)coors[1],(double)coors[2],(double)coors[3]);CHKERRQ(ierr);
-
-  ierr = PetscOptionsGetRealArray(NULL,"-draw_coordinates",coors,&ncoors,NULL);CHKERRQ(ierr);
+  ierr = VecStrideMin(xcoor,0,NULL,&zctx.xmin);CHKERRQ(ierr);
+  ierr = VecStrideMax(xcoor,0,NULL,&zctx.xmax);CHKERRQ(ierr);
+  ierr = VecStrideMin(xcoor,1,NULL,&zctx.ymin);CHKERRQ(ierr);
+  ierr = VecStrideMax(xcoor,1,NULL,&zctx.ymax);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-draw_contour_axis",&zctx.showaxis,NULL);CHKERRQ(ierr);
+  if (zctx.showaxis) {
+    coors[0] = zctx.xmin - .05*(zctx.xmax - zctx.xmin); coors[1] = zctx.ymin - .05*(zctx.ymax - zctx.ymin);
+    coors[2] = zctx.xmax + .05*(zctx.xmax - zctx.xmin); coors[3] = zctx.ymax + .05*(zctx.ymax - zctx.ymin);
+  } else {
+    coors[0] = zctx.xmin; coors[1] = zctx.ymin; coors[2] = zctx.xmax; coors[3] = zctx.ymax;
+  }
+  ierr = PetscOptionsGetRealArray(NULL,NULL,"-draw_coordinates",coors,&ncoors,NULL);CHKERRQ(ierr);
+  ierr = PetscInfo4(da,"Preparing DMDA 2d contour plot coordinates %g %g %g %g\n",(double)coors[0],(double)coors[1],(double)coors[2],(double)coors[3]);CHKERRQ(ierr);
 
   /*
-       get local ghosted version of coordinates
+      Get local ghosted version of coordinates
   */
   ierr = PetscObjectQuery((PetscObject)da,"GraphicsCoordinateGhosted",(PetscObject*)&xcoorl);CHKERRQ(ierr);
   if (!xcoorl) {
     /* create DMDA to get local version of graphics */
     ierr = DMDACreate2d(comm,DM_BOUNDARY_NONE,DM_BOUNDARY_NONE,DMDA_STENCIL_BOX,M,N,zctx.m,zctx.n,2,1,lx,ly,&dag);CHKERRQ(ierr);
+    ierr = DMSetUp(dag);CHKERRQ(ierr);
     ierr = PetscInfo(dag,"Creating auxilary DMDA for managing graphics coordinates ghost points\n");CHKERRQ(ierr);
     ierr = DMCreateLocalVector(dag,&xcoorl);CHKERRQ(ierr);
     ierr = PetscObjectCompose((PetscObject)da,"GraphicsCoordinateGhosted",(PetscObject)xcoorl);CHKERRQ(ierr);
@@ -251,43 +243,34 @@ PetscErrorCode VecView_MPI_Draw_DA2d(Vec xin,PetscViewer viewer)
   ierr = DMGlobalToLocalBegin(dag,xcoor,INSERT_VALUES,xcoorl);CHKERRQ(ierr);
   ierr = DMGlobalToLocalEnd(dag,xcoor,INSERT_VALUES,xcoorl);CHKERRQ(ierr);
   ierr = VecGetArrayRead(xcoorl,&zctx.xy);CHKERRQ(ierr);
+  ierr = DMDAGetCoordinateName(da,0,&zctx.name0);CHKERRQ(ierr);
+  ierr = DMDAGetCoordinateName(da,1,&zctx.name1);CHKERRQ(ierr);
 
   /*
-        Get information about size of area each processor must do graphics for
+      Get information about size of area each processor must do graphics for
   */
-  ierr = DMDAGetInfo(dac,0,&M,&N,0,0,0,0,&zctx.step,0,&bx,&by,0,0);CHKERRQ(ierr);
-  ierr = DMDAGetGhostCorners(dac,0,0,0,&zctx.m,&zctx.n,0);CHKERRQ(ierr);
-
-  ierr = PetscOptionsGetBool(NULL,"-draw_contour_grid",&zctx.showgrid,NULL);CHKERRQ(ierr);
+  ierr = DMDAGetInfo(dac,NULL,&M,&N,NULL,NULL,NULL,NULL,&zctx.dof,NULL,&bx,&by,NULL,NULL);CHKERRQ(ierr);
+  ierr = DMDAGetGhostCorners(dac,NULL,NULL,NULL,&zctx.m,&zctx.n,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-draw_contour_grid",&zctx.showgrid,NULL);CHKERRQ(ierr);
 
   ierr = DMDASelectFields(da,&ndisplayfields,&displayfields);CHKERRQ(ierr);
-
   ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
-  ierr = PetscOptionsGetBool(NULL,"-draw_ports",&useports,NULL);CHKERRQ(ierr);
-  if (useports || format == PETSC_VIEWER_DRAW_PORTS) {
-    ierr       = PetscDrawSynchronizedClear(draw);CHKERRQ(ierr);
-    ierr       = PetscDrawViewPortsCreate(draw,ndisplayfields,&ports);CHKERRQ(ierr);
-    zctx.name0 = 0;
-    zctx.name1 = 0;
-  } else {
-    ierr = DMDAGetCoordinateName(da,0,&zctx.name0);CHKERRQ(ierr);
-    ierr = DMDAGetCoordinateName(da,1,&zctx.name1);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-draw_ports",&useports,NULL);CHKERRQ(ierr);
+  if (format == PETSC_VIEWER_DRAW_PORTS) useports = PETSC_TRUE;
+  if (useports) {
+    ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
+    ierr = PetscDrawCheckResizedWindow(draw);CHKERRQ(ierr);
+    ierr = PetscDrawClear(draw);CHKERRQ(ierr);
+    ierr = PetscDrawViewPortsCreate(draw,ndisplayfields,&ports);CHKERRQ(ierr);
   }
 
   /*
-     Loop over each field; drawing each in a different window
+      Loop over each field; drawing each in a different window
   */
   for (i=0; i<ndisplayfields; i++) {
     zctx.k = displayfields[i];
-    if (useports) {
-      ierr = PetscDrawViewPortsSet(ports,i);CHKERRQ(ierr);
-    } else {
-      ierr = PetscViewerDrawGetDraw(viewer,i,&draw);CHKERRQ(ierr);
-    }
 
-    /*
-        Determine the min and max color in plot
-    */
+    /* determine the min and max value in plot */
     ierr = VecStrideMin(xin,zctx.k,NULL,&zctx.min);CHKERRQ(ierr);
     ierr = VecStrideMax(xin,zctx.k,NULL,&zctx.max);CHKERRQ(ierr);
     if (zctx.k < nbounds) {
@@ -298,36 +281,36 @@ PetscErrorCode VecView_MPI_Draw_DA2d(Vec xin,PetscViewer viewer)
       zctx.min -= 1.e-12;
       zctx.max += 1.e-12;
     }
-
-    if (!rank) {
-      const char *title;
-
-      ierr = DMDAGetFieldName(da,zctx.k,&title);CHKERRQ(ierr);
-      if (title) {
-        ierr = PetscDrawSetTitle(draw,title);CHKERRQ(ierr);
-      }
-    }
-    ierr = PetscDrawSetCoordinates(draw,coors[0],coors[1],coors[2],coors[3]);CHKERRQ(ierr);
     ierr = PetscInfo2(da,"DMDA 2d contour plot min %g max %g\n",(double)zctx.min,(double)zctx.max);CHKERRQ(ierr);
 
+    if (useports) {
+      ierr = PetscDrawViewPortsSet(ports,i);CHKERRQ(ierr);
+    } else {
+      const char *title;
+      ierr = PetscViewerDrawGetDraw(viewer,i,&draw);CHKERRQ(ierr);
+      ierr = DMDAGetFieldName(da,zctx.k,&title);CHKERRQ(ierr);
+      if (title) {ierr = PetscDrawSetTitle(draw,title);CHKERRQ(ierr);}
+    }
+
     ierr = PetscDrawGetPopup(draw,&popup);CHKERRQ(ierr);
-    if (popup) {ierr = PetscDrawScalePopup(popup,zctx.min,zctx.max);CHKERRQ(ierr);}
-
-    zctx.scale = (245.0 - PETSC_DRAW_BASIC_COLORS)/(zctx.max - zctx.min);
-
+    ierr = PetscDrawScalePopup(popup,zctx.min,zctx.max);CHKERRQ(ierr);
+    ierr = PetscDrawSetCoordinates(draw,coors[0],coors[1],coors[2],coors[3]);CHKERRQ(ierr);
     ierr = PetscDrawZoom(draw,VecView_MPI_Draw_DA2d_Zoom,&zctx);CHKERRQ(ierr);
+    if (!useports) {ierr = PetscDrawSave(draw);CHKERRQ(ierr);}
+  }
+  if (useports) {
+    ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
+    ierr = PetscDrawSave(draw);CHKERRQ(ierr);
   }
-  ierr = PetscFree(displayfields);CHKERRQ(ierr);
-  ierr = PetscDrawViewPortsDestroy(ports);CHKERRQ(ierr);
 
+  ierr = PetscDrawViewPortsDestroy(ports);CHKERRQ(ierr);
+  ierr = PetscFree(displayfields);CHKERRQ(ierr);
   ierr = VecRestoreArrayRead(xcoorl,&zctx.xy);CHKERRQ(ierr);
   ierr = VecRestoreArrayRead(xlocal,&zctx.v);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #if defined(PETSC_HAVE_HDF5)
-#undef __FUNCT__
-#define __FUNCT__ "VecGetHDF5ChunkSize"
 static PetscErrorCode VecGetHDF5ChunkSize(DM_DA *da, Vec xin, PetscInt dimension, PetscInt timestep, hsize_t *chunkDims)
 {
   PetscMPIInt    comm_size;
@@ -427,8 +410,6 @@ static PetscErrorCode VecGetHDF5ChunkSize(DM_DA *da, Vec xin, PetscInt dimension
 #endif
 
 #if defined(PETSC_HAVE_HDF5)
-#undef __FUNCT__
-#define __FUNCT__ "VecView_MPI_HDF5_DA"
 PetscErrorCode VecView_MPI_HDF5_DA(Vec xin,PetscViewer viewer)
 {
   DM                dm;
@@ -518,6 +499,8 @@ PetscErrorCode VecView_MPI_HDF5_DA(Vec xin,PetscViewer viewer)
   filescalartype = H5T_NATIVE_FLOAT;
 #elif defined(PETSC_USE_REAL___FLOAT128)
 #error "HDF5 output with 128 bit floats not supported."
+#elif defined(PETSC_USE_REAL___FP16)
+#error "HDF5 output with 16 bit floats not supported."
 #else
   memscalartype = H5T_NATIVE_DOUBLE;
   if (spoutput == PETSC_TRUE) filescalartype = H5T_NATIVE_FLOAT;
@@ -599,8 +582,6 @@ PetscErrorCode VecView_MPI_HDF5_DA(Vec xin,PetscViewer viewer)
 extern PetscErrorCode VecView_MPI_Draw_DA1d(Vec,PetscViewer);
 
 #if defined(PETSC_HAVE_MPIIO)
-#undef __FUNCT__
-#define __FUNCT__ "DMDAArrayMPIIO"
 static PetscErrorCode DMDAArrayMPIIO(DM da,PetscViewer viewer,Vec xin,PetscBool write)
 {
   PetscErrorCode    ierr;
@@ -668,15 +649,13 @@ static PetscErrorCode DMDAArrayMPIIO(DM da,PetscViewer viewer,Vec xin,PetscBool
 }
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "VecView_MPI_DA"
 PetscErrorCode  VecView_MPI_DA(Vec xin,PetscViewer viewer)
 {
   DM                da;
   PetscErrorCode    ierr;
   PetscInt          dim;
   Vec               natural;
-  PetscBool         isdraw,isvtk;
+  PetscBool         isdraw,isvtk,isglvis;
 #if defined(PETSC_HAVE_HDF5)
   PetscBool         ishdf5;
 #endif
@@ -691,6 +670,7 @@ PetscErrorCode  VecView_MPI_DA(Vec xin,PetscViewer viewer)
 #if defined(PETSC_HAVE_HDF5)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERHDF5,&ishdf5);CHKERRQ(ierr);
 #endif
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERGLVIS,&isglvis);CHKERRQ(ierr);
   if (isdraw) {
     ierr = DMDAGetInfo(da,&dim,0,0,0,0,0,0,0,0,0,0,0,0);CHKERRQ(ierr);
     if (dim == 1) {
@@ -698,20 +678,31 @@ PetscErrorCode  VecView_MPI_DA(Vec xin,PetscViewer viewer)
     } else if (dim == 2) {
       ierr = VecView_MPI_Draw_DA2d(xin,viewer);CHKERRQ(ierr);
     } else SETERRQ1(PetscObjectComm((PetscObject)da),PETSC_ERR_SUP,"Cannot graphically view vector associated with this dimensional DMDA %D",dim);
-  } else if (isvtk) {           /* Duplicate the Vec and hold a reference to the DM */
+  } else if (isvtk) {           /* Duplicate the Vec */
     Vec Y;
-    ierr = PetscObjectReference((PetscObject)da);CHKERRQ(ierr);
     ierr = VecDuplicate(xin,&Y);CHKERRQ(ierr);
     if (((PetscObject)xin)->name) {
       /* If xin was named, copy the name over to Y. The duplicate names are safe because nobody else will ever see Y. */
       ierr = PetscObjectSetName((PetscObject)Y,((PetscObject)xin)->name);CHKERRQ(ierr);
     }
     ierr = VecCopy(xin,Y);CHKERRQ(ierr);
-    ierr = PetscViewerVTKAddField(viewer,(PetscObject)da,DMDAVTKWriteAll,PETSC_VTK_POINT_FIELD,(PetscObject)Y);CHKERRQ(ierr);
+    {
+      PetscObject dmvtk;
+      PetscBool   compatible,compatibleSet;
+      ierr = PetscViewerVTKGetDM(viewer,&dmvtk);CHKERRQ(ierr);
+      if (dmvtk) {
+        PetscValidHeaderSpecific((DM)dmvtk,DM_CLASSID,-1);
+        ierr = DMGetCompatibility(da,(DM)dmvtk,&compatible,&compatibleSet);CHKERRQ(ierr);
+        if (!compatibleSet || !compatible) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_ARG_INCOMP,"Cannot confirm compatibility of DMs associated with Vecs viewed in the same VTK file. Check that grids are the same.");
+      }
+      ierr = PetscViewerVTKAddField(viewer,(PetscObject)da,DMDAVTKWriteAll,PETSC_VTK_POINT_FIELD,PETSC_FALSE,(PetscObject)Y);CHKERRQ(ierr);
+    }
 #if defined(PETSC_HAVE_HDF5)
   } else if (ishdf5) {
     ierr = VecView_MPI_HDF5_DA(xin,viewer);CHKERRQ(ierr);
 #endif
+  } else if (isglvis) {
+    ierr = VecView_GLVis(xin,viewer);CHKERRQ(ierr);
   } else {
 #if defined(PETSC_HAVE_MPIIO)
     PetscBool isbinary,isMPIIO;
@@ -738,10 +729,12 @@ PetscErrorCode  VecView_MPI_DA(Vec xin,PetscViewer viewer)
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     if (format == PETSC_VIEWER_BINARY_MATLAB) {
       /* temporarily remove viewer format so it won't trigger in the VecView() */
-      ierr = PetscViewerSetFormat(viewer,PETSC_VIEWER_DEFAULT);CHKERRQ(ierr);
+      ierr = PetscViewerPushFormat(viewer,PETSC_VIEWER_DEFAULT);CHKERRQ(ierr);
     }
 
+    ((PetscObject)natural)->donotPetscObjectPrintClassNamePrefixType = PETSC_TRUE;
     ierr = VecView(natural,viewer);CHKERRQ(ierr);
+    ((PetscObject)natural)->donotPetscObjectPrintClassNamePrefixType = PETSC_FALSE;
 
     if (format == PETSC_VIEWER_BINARY_MATLAB) {
       MPI_Comm    comm;
@@ -751,7 +744,7 @@ PetscErrorCode  VecView_MPI_DA(Vec xin,PetscViewer viewer)
       PetscInt    dim,ni,nj,nk,pi,pj,pk,dof,n;
 
       /* set the viewer format back into the viewer */
-      ierr = PetscViewerSetFormat(viewer,format);CHKERRQ(ierr);
+      ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
       ierr = PetscObjectGetComm((PetscObject)viewer,&comm);CHKERRQ(ierr);
       ierr = PetscViewerBinaryGetInfoPointer(viewer,&info);CHKERRQ(ierr);
       ierr = DMDAGetInfo(da,&dim,&ni,&nj,&nk,&pi,&pj,&pk,&dof,0,0,0,0,0);CHKERRQ(ierr);
@@ -781,8 +774,6 @@ PetscErrorCode  VecView_MPI_DA(Vec xin,PetscViewer viewer)
 }
 
 #if defined(PETSC_HAVE_HDF5)
-#undef __FUNCT__
-#define __FUNCT__ "VecLoad_HDF5_DA"
 PetscErrorCode VecLoad_HDF5_DA(Vec xin, PetscViewer viewer)
 {
   DM             da;
@@ -806,6 +797,8 @@ PetscErrorCode VecLoad_HDF5_DA(Vec xin, PetscViewer viewer)
   scalartype = H5T_NATIVE_FLOAT;
 #elif defined(PETSC_USE_REAL___FLOAT128)
 #error "HDF5 output with 128 bit floats not supported."
+#elif defined(PETSC_USE_REAL___FP16)
+#error "HDF5 output with 16 bit floats not supported."
 #else
   scalartype = H5T_NATIVE_DOUBLE;
 #endif
@@ -852,9 +845,7 @@ PetscErrorCode VecLoad_HDF5_DA(Vec xin, PetscViewer viewer)
     else if (dd->w != (PetscInt) dims[dofInd]) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_FILE_UNEXPECTED, "Number of dofs in file is %D, not %D as expected",(PetscInt)dims[dofInd],dd->w);
 
   /* Other cases where rdim != dim cannot be handled currently */
-  } else if (rdim != dim) {
-    SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_FILE_UNEXPECTED, "Dimension of array in file is %d, not %d as expected with dof = %D",rdim,dim,dd->w);
-  }
+  } else if (rdim != dim) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_FILE_UNEXPECTED, "Dimension of array in file is %d, not %d as expected with dof = %D",rdim,dim,dd->w);
 
   /* Set up the hyperslab size */
   dim = 0;
@@ -914,8 +905,6 @@ PetscErrorCode VecLoad_HDF5_DA(Vec xin, PetscViewer viewer)
 }
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "VecLoad_Binary_DA"
 PetscErrorCode VecLoad_Binary_DA(Vec xin, PetscViewer viewer)
 {
   DM             da;
@@ -949,15 +938,13 @@ PetscErrorCode VecLoad_Binary_DA(Vec xin, PetscViewer viewer)
   ierr = DMDANaturalToGlobalEnd(da,natural,INSERT_VALUES,xin);CHKERRQ(ierr);
   ierr = VecDestroy(&natural);CHKERRQ(ierr);
   ierr = PetscInfo(xin,"Loading vector from natural ordering into DMDA\n");CHKERRQ(ierr);
-  ierr = PetscOptionsGetInt(((PetscObject)xin)->prefix,"-vecload_block_size",&bs,&flag);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,((PetscObject)xin)->prefix,"-vecload_block_size",&bs,&flag);CHKERRQ(ierr);
   if (flag && bs != dd->w) {
     ierr = PetscInfo2(xin,"Block size in file %D not equal to DMDA's dof %D\n",bs,dd->w);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecLoad_Default_DA"
 PetscErrorCode  VecLoad_Default_DA(Vec xin, PetscViewer viewer)
 {
   PetscErrorCode ierr;
