@@ -7,283 +7,6 @@
 #include <petscdmda.h>                /*I "petscdmda.h" I*/
 #include <../src/dm/impls/da/hypre/mhyp.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "MatHYPRE_IJMatrixPreallocate"
-PetscErrorCode MatHYPRE_IJMatrixPreallocate(Mat A_d, Mat A_o,HYPRE_IJMatrix ij)
-{
-  PetscErrorCode ierr;
-  PetscInt       i,n_d,n_o;
-  const PetscInt *ia_d,*ia_o;
-  PetscBool      done_d=PETSC_FALSE,done_o=PETSC_FALSE;
-  PetscInt       *nnz_d=NULL,*nnz_o=NULL;
-
-  PetscFunctionBegin;
-  if (A_d) { /* determine number of nonzero entries in local diagonal part */
-    ierr = MatGetRowIJ(A_d,0,PETSC_FALSE,PETSC_FALSE,&n_d,&ia_d,NULL,&done_d);CHKERRQ(ierr);
-    if (done_d) {
-      ierr = PetscMalloc1(n_d,&nnz_d);CHKERRQ(ierr);
-      for (i=0; i<n_d; i++) {
-        nnz_d[i] = ia_d[i+1] - ia_d[i];
-      }
-    }
-    ierr = MatRestoreRowIJ(A_d,0,PETSC_FALSE,PETSC_FALSE,NULL,&ia_d,NULL,&done_d);CHKERRQ(ierr);
-  }
-  if (A_o) { /* determine number of nonzero entries in local off-diagonal part */
-    ierr = MatGetRowIJ(A_o,0,PETSC_FALSE,PETSC_FALSE,&n_o,&ia_o,NULL,&done_o);CHKERRQ(ierr);
-    if (done_o) {
-      ierr = PetscMalloc1(n_o,&nnz_o);CHKERRQ(ierr);
-      for (i=0; i<n_o; i++) {
-        nnz_o[i] = ia_o[i+1] - ia_o[i];
-      }
-    }
-    ierr = MatRestoreRowIJ(A_o,0,PETSC_FALSE,PETSC_FALSE,&n_o,&ia_o,NULL,&done_o);CHKERRQ(ierr);
-  }
-  if (done_d) {    /* set number of nonzeros in HYPRE IJ matrix */
-    if (!done_o) { /* only diagonal part */
-      ierr = PetscMalloc1(n_d,&nnz_o);CHKERRQ(ierr);
-      for (i=0; i<n_d; i++) {
-        nnz_o[i] = 0;
-      }
-    }
-    PetscStackCallStandard(HYPRE_IJMatrixSetDiagOffdSizes,(ij,(HYPRE_Int *)nnz_d,(HYPRE_Int *)nnz_o));
-    ierr = PetscFree(nnz_d);CHKERRQ(ierr);
-    ierr = PetscFree(nnz_o);CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "MatHYPRE_IJMatrixCreate"
-PetscErrorCode MatHYPRE_IJMatrixCreate(Mat A,HYPRE_IJMatrix *ij)
-{
-  PetscErrorCode ierr;
-  PetscInt       rstart,rend,cstart,cend;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(A,MAT_CLASSID,1);
-  PetscValidType(A,1);
-  PetscValidPointer(ij,2);
-  ierr   = MatSetUp(A);CHKERRQ(ierr);
-  rstart = A->rmap->rstart;
-  rend   = A->rmap->rend;
-  cstart = A->cmap->rstart;
-  cend   = A->cmap->rend;
-  PetscStackCallStandard(HYPRE_IJMatrixCreate,(PetscObjectComm((PetscObject)A),rstart,rend-1,cstart,cend-1,ij));
-  PetscStackCallStandard(HYPRE_IJMatrixSetObjectType,(*ij,HYPRE_PARCSR));
-  {
-    PetscBool      same;
-    Mat            A_d,A_o;
-    const PetscInt *colmap;
-    ierr = PetscObjectTypeCompare((PetscObject)A,MATMPIAIJ,&same);CHKERRQ(ierr);
-    if (same) {
-      ierr = MatMPIAIJGetSeqAIJ(A,&A_d,&A_o,&colmap);CHKERRQ(ierr);
-      ierr = MatHYPRE_IJMatrixPreallocate(A_d,A_o,*ij);CHKERRQ(ierr);
-      PetscFunctionReturn(0);
-    }
-    ierr = PetscObjectTypeCompare((PetscObject)A,MATMPIBAIJ,&same);CHKERRQ(ierr);
-    if (same) {
-      ierr = MatMPIBAIJGetSeqBAIJ(A,&A_d,&A_o,&colmap);CHKERRQ(ierr);
-      ierr = MatHYPRE_IJMatrixPreallocate(A_d,A_o,*ij);CHKERRQ(ierr);
-      PetscFunctionReturn(0);
-    }
-    ierr = PetscObjectTypeCompare((PetscObject)A,MATSEQAIJ,&same);CHKERRQ(ierr);
-    if (same) {
-      ierr = MatHYPRE_IJMatrixPreallocate(A,NULL,*ij);CHKERRQ(ierr);
-      PetscFunctionReturn(0);
-    }
-    ierr = PetscObjectTypeCompare((PetscObject)A,MATSEQBAIJ,&same);CHKERRQ(ierr);
-    if (same) {
-      ierr = MatHYPRE_IJMatrixPreallocate(A,NULL,*ij);CHKERRQ(ierr);
-      PetscFunctionReturn(0);
-    }
-  }
-  PetscFunctionReturn(0);
-}
-
-extern PetscErrorCode MatHYPRE_IJMatrixFastCopy_MPIAIJ(Mat,HYPRE_IJMatrix);
-extern PetscErrorCode MatHYPRE_IJMatrixFastCopy_SeqAIJ(Mat,HYPRE_IJMatrix);
-/*
-    Copies the data over (column indices, numerical values) to hypre matrix
-*/
-
-#undef __FUNCT__
-#define __FUNCT__ "MatHYPRE_IJMatrixCopy"
-PetscErrorCode MatHYPRE_IJMatrixCopy(Mat A,HYPRE_IJMatrix ij)
-{
-  PetscErrorCode    ierr;
-  PetscInt          i,rstart,rend,ncols,nr,nc;
-  const PetscScalar *values;
-  const PetscInt    *cols;
-  PetscBool         flg;
-
-  PetscFunctionBegin;
-  ierr = PetscObjectTypeCompare((PetscObject)A,MATMPIAIJ,&flg);CHKERRQ(ierr);
-  ierr = MatGetSize(A,&nr,&nc);CHKERRQ(ierr);
-  if (flg && nr == nc) {
-    ierr = MatHYPRE_IJMatrixFastCopy_MPIAIJ(A,ij);CHKERRQ(ierr);
-    PetscFunctionReturn(0);
-  }
-  ierr = PetscObjectTypeCompare((PetscObject)A,MATSEQAIJ,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = MatHYPRE_IJMatrixFastCopy_SeqAIJ(A,ij);CHKERRQ(ierr);
-    PetscFunctionReturn(0);
-  }
-
-  ierr = PetscLogEventBegin(MAT_Convert,A,0,0,0);CHKERRQ(ierr);
-  PetscStackCallStandard(HYPRE_IJMatrixInitialize,(ij));
-  ierr = MatGetOwnershipRange(A,&rstart,&rend);CHKERRQ(ierr);
-  for (i=rstart; i<rend; i++) {
-    ierr = MatGetRow(A,i,&ncols,&cols,&values);CHKERRQ(ierr);
-    PetscStackCallStandard(HYPRE_IJMatrixSetValues,(ij,1,(HYPRE_Int *)&ncols,(HYPRE_Int *)&i,(HYPRE_Int *)cols,values));
-    ierr = MatRestoreRow(A,i,&ncols,&cols,&values);CHKERRQ(ierr);
-  }
-  PetscStackCallStandard(HYPRE_IJMatrixAssemble,(ij));
-  ierr = PetscLogEventEnd(MAT_Convert,A,0,0,0);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-/*
-    This copies the CSR format directly from the PETSc data structure to the hypre
-    data structure without calls to MatGetRow() or hypre's set values.
-
-*/
-#include <_hypre_IJ_mv.h>
-#include <HYPRE_IJ_mv.h>
-#include <../src/mat/impls/aij/mpi/mpiaij.h>
-
-#undef __FUNCT__
-#define __FUNCT__ "MatHYPRE_IJMatrixFastCopy_SeqAIJ"
-PetscErrorCode MatHYPRE_IJMatrixFastCopy_SeqAIJ(Mat A,HYPRE_IJMatrix ij)
-{
-  PetscErrorCode ierr;
-  Mat_SeqAIJ     *pdiag = (Mat_SeqAIJ*)A->data;
-
-  hypre_ParCSRMatrix    *par_matrix;
-  hypre_AuxParCSRMatrix *aux_matrix;
-  hypre_CSRMatrix       *hdiag;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(A,MAT_CLASSID,1);
-  PetscValidType(A,1);
-  PetscValidPointer(ij,2);
-
-  ierr = PetscLogEventBegin(MAT_Convert,A,0,0,0);CHKERRQ(ierr);
-  PetscStackCallStandard(HYPRE_IJMatrixInitialize,(ij));
-  par_matrix = (hypre_ParCSRMatrix*)hypre_IJMatrixObject(ij);
-  aux_matrix = (hypre_AuxParCSRMatrix*)hypre_IJMatrixTranslator(ij);
-  hdiag      = hypre_ParCSRMatrixDiag(par_matrix);
-
-  /*
-       this is the Hack part where we monkey directly with the hypre datastructures
-  */
-  ierr = PetscMemcpy(hdiag->i,pdiag->i,(A->rmap->n + 1)*sizeof(PetscInt));
-  ierr = PetscMemcpy(hdiag->j,pdiag->j,pdiag->nz*sizeof(PetscInt));
-  ierr = PetscMemcpy(hdiag->data,pdiag->a,pdiag->nz*sizeof(PetscScalar));
-
-  hypre_AuxParCSRMatrixNeedAux(aux_matrix) = 0;
-  PetscStackCallStandard(HYPRE_IJMatrixAssemble,(ij));
-  ierr = PetscLogEventEnd(MAT_Convert,A,0,0,0);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "MatHYPRE_IJMatrixFastCopy_MPIAIJ"
-PetscErrorCode MatHYPRE_IJMatrixFastCopy_MPIAIJ(Mat A,HYPRE_IJMatrix ij)
-{
-  PetscErrorCode ierr;
-  Mat_MPIAIJ     *pA = (Mat_MPIAIJ*)A->data;
-  Mat_SeqAIJ     *pdiag,*poffd;
-  PetscInt       i,*garray = pA->garray,*jj,cstart,*pjj;
-
-  hypre_ParCSRMatrix    *par_matrix;
-  hypre_AuxParCSRMatrix *aux_matrix;
-  hypre_CSRMatrix       *hdiag,*hoffd;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(A,MAT_CLASSID,1);
-  PetscValidType(A,1);
-  PetscValidPointer(ij,2);
-  pdiag = (Mat_SeqAIJ*) pA->A->data;
-  poffd = (Mat_SeqAIJ*) pA->B->data;
-  /* cstart is only valid for square MPIAIJ layed out in the usual way */
-  ierr = MatGetOwnershipRange(A,&cstart,NULL);CHKERRQ(ierr);
-
-  ierr = PetscLogEventBegin(MAT_Convert,A,0,0,0);CHKERRQ(ierr);
-  PetscStackCallStandard(HYPRE_IJMatrixInitialize,(ij));
-  par_matrix = (hypre_ParCSRMatrix*)hypre_IJMatrixObject(ij);
-  aux_matrix = (hypre_AuxParCSRMatrix*)hypre_IJMatrixTranslator(ij);
-  hdiag      = hypre_ParCSRMatrixDiag(par_matrix);
-  hoffd      = hypre_ParCSRMatrixOffd(par_matrix);
-
-  /*
-       this is the Hack part where we monkey directly with the hypre datastructures
-  */
-  ierr = PetscMemcpy(hdiag->i,pdiag->i,(pA->A->rmap->n + 1)*sizeof(PetscInt));
-  /* need to shift the diag column indices (hdiag->j) back to global numbering since hypre is expecting this */
-  jj  = (PetscInt*)hdiag->j;
-  pjj = (PetscInt*)pdiag->j;
-  for (i=0; i<pdiag->nz; i++) jj[i] = cstart + pjj[i];
-  ierr = PetscMemcpy(hdiag->data,pdiag->a,pdiag->nz*sizeof(PetscScalar));
-  ierr = PetscMemcpy(hoffd->i,poffd->i,(pA->A->rmap->n + 1)*sizeof(PetscInt));
-  /* need to move the offd column indices (hoffd->j) back to global numbering since hypre is expecting this
-     If we hacked a hypre a bit more we might be able to avoid this step */
-  jj  = (PetscInt*) hoffd->j;
-  pjj = (PetscInt*) poffd->j;
-  for (i=0; i<poffd->nz; i++) jj[i] = garray[pjj[i]];
-  ierr = PetscMemcpy(hoffd->data,poffd->a,poffd->nz*sizeof(PetscScalar));
-
-  hypre_AuxParCSRMatrixNeedAux(aux_matrix) = 0;
-  PetscStackCallStandard(HYPRE_IJMatrixAssemble,(ij));
-  ierr = PetscLogEventEnd(MAT_Convert,A,0,0,0);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-/*
-    Does NOT copy the data over, instead uses DIRECTLY the pointers from the PETSc MPIAIJ format
-
-    This is UNFINISHED and does NOT work! The problem is that hypre puts the diagonal entry first
-    which will corrupt the PETSc data structure if we did this. Need a work around to this problem.
-*/
-#include <_hypre_IJ_mv.h>
-#include <HYPRE_IJ_mv.h>
-
-#undef __FUNCT__
-#define __FUNCT__ "MatHYPRE_IJMatrixLink"
-PetscErrorCode MatHYPRE_IJMatrixLink(Mat A,HYPRE_IJMatrix *ij)
-{
-  PetscErrorCode        ierr;
-  PetscInt              rstart,rend,cstart,cend;
-  PetscBool             flg;
-  hypre_AuxParCSRMatrix *aux_matrix;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(A,MAT_CLASSID,1);
-  PetscValidType(A,1);
-  PetscValidPointer(ij,2);
-  ierr = PetscObjectTypeCompare((PetscObject)A,MATMPIAIJ,&flg);CHKERRQ(ierr);
-  if (!flg) SETERRQ(PetscObjectComm((PetscObject)A),PETSC_ERR_SUP,"Can only use with PETSc MPIAIJ matrices");
-  ierr = MatSetUp(A);CHKERRQ(ierr);
-
-  ierr   = PetscLogEventBegin(MAT_Convert,A,0,0,0);CHKERRQ(ierr);
-  rstart = A->rmap->rstart;
-  rend   = A->rmap->rend;
-  cstart = A->cmap->rstart;
-  cend   = A->cmap->rend;
-  PetscStackCallStandard(HYPRE_IJMatrixCreate,(PetscObjectComm((PetscObject)A),rstart,rend-1,cstart,cend-1,ij));
-  PetscStackCallStandard(HYPRE_IJMatrixSetObjectType,(*ij,HYPRE_PARCSR));
-
-  PetscStackCallStandard(HYPRE_IJMatrixInitialize,(*ij));
-  PetscStackCall("hypre_IJMatrixTranslator",aux_matrix = (hypre_AuxParCSRMatrix*)hypre_IJMatrixTranslator(*ij));
-
-  hypre_AuxParCSRMatrixNeedAux(aux_matrix) = 0;
-
-  /* this is the Hack part where we monkey directly with the hypre datastructures */
-
-  PetscStackCallStandard(HYPRE_IJMatrixAssemble,(*ij));
-  ierr = PetscLogEventEnd(MAT_Convert,A,0,0,0);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
 /* -----------------------------------------------------------------------------------------------------------------*/
 
 /*MC
@@ -292,17 +15,16 @@ PetscErrorCode MatHYPRE_IJMatrixLink(Mat A,HYPRE_IJMatrix *ij)
 
    Level: intermediate
 
-   Notes: Unlike the more general support for blocks in hypre this allows only one block per process and requires the block
+   Notes:
+    Unlike the more general support for blocks in hypre this allows only one block per process and requires the block
           be defined by a DMDA.
 
-          The matrix needs a DMDA associated with it by either a call to MatSetupDM() or if the matrix is obtained from DMCreateMatrix()
+          The matrix needs a DMDA associated with it by either a call to MatSetDM() or if the matrix is obtained from DMCreateMatrix()
 
-.seealso: MatCreate(), PCPFMG, MatSetupDM(), DMCreateMatrix()
+.seealso: MatCreate(), PCPFMG, MatSetDM(), DMCreateMatrix()
 M*/
 
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSetValuesLocal_HYPREStruct_3d"
 PetscErrorCode  MatSetValuesLocal_HYPREStruct_3d(Mat mat,PetscInt nrow,const PetscInt irow[],PetscInt ncol,const PetscInt icol[],const PetscScalar y[],InsertMode addv)
 {
   PetscErrorCode    ierr;
@@ -311,7 +33,7 @@ PetscErrorCode  MatSetValuesLocal_HYPREStruct_3d(Mat mat,PetscInt nrow,const Pet
   Mat_HYPREStruct   *ex     = (Mat_HYPREStruct*) mat->data;
 
   PetscFunctionBegin;
-  if (PetscUnlikely(ncol >= 7)) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"ncol %D >= 7 too large",ncol);
+  if (PetscUnlikely(ncol > 7)) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"ncol %D > 7 too large",ncol);
   for (i=0; i<nrow; i++) {
     for (j=0; j<ncol; j++) {
       stencil = icol[j] - irow[i];
@@ -345,8 +67,6 @@ PetscErrorCode  MatSetValuesLocal_HYPREStruct_3d(Mat mat,PetscInt nrow,const Pet
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatZeroRowsLocal_HYPREStruct_3d"
 PetscErrorCode  MatZeroRowsLocal_HYPREStruct_3d(Mat mat,PetscInt nrow,const PetscInt irow[],PetscScalar d,Vec x,Vec b)
 {
   PetscErrorCode  ierr;
@@ -369,8 +89,6 @@ PetscErrorCode  MatZeroRowsLocal_HYPREStruct_3d(Mat mat,PetscInt nrow,const Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatZeroEntries_HYPREStruct_3d"
 PetscErrorCode MatZeroEntries_HYPREStruct_3d(Mat mat)
 {
   PetscErrorCode  ierr;
@@ -384,18 +102,18 @@ PetscErrorCode MatZeroEntries_HYPREStruct_3d(Mat mat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSetupDM_HYPREStruct"
-static PetscErrorCode  MatSetupDM_HYPREStruct(Mat mat,DM da)
+static PetscErrorCode  MatSetUp_HYPREStruct(Mat mat)
 {
   PetscErrorCode         ierr;
   Mat_HYPREStruct        *ex = (Mat_HYPREStruct*) mat->data;
-  PetscInt               dim,dof,sw[3],nx,ny,nz,ilower[3],iupper[3],ssize,i;
+  PetscInt               dim,dof,sw[6],nx,ny,nz,ilower[3],iupper[3],ssize,i;
   DMBoundaryType         px,py,pz;
   DMDAStencilType        st;
   ISLocalToGlobalMapping ltog;
+  DM                     da;
 
   PetscFunctionBegin;
+  ierr   = MatGetDM(mat,(DM*)&da);CHKERRQ(ierr);
   ex->da = da;
   ierr   = PetscObjectReference((PetscObject)da);CHKERRQ(ierr);
 
@@ -420,8 +138,7 @@ static PetscErrorCode  MatSetupDM_HYPREStruct(Mat mat,DM da)
   PetscStackCallStandard(HYPRE_StructGridSetExtents,(ex->hgrid,(HYPRE_Int *)ilower,(HYPRE_Int *)iupper));
   PetscStackCallStandard(HYPRE_StructGridAssemble,(ex->hgrid));
 
-  sw[1] = sw[0];
-  sw[2] = sw[1];
+  sw[5] = sw[4] = sw[3] = sw[2] = sw[1] = sw[0];
   PetscStackCallStandard(HYPRE_StructGridSetNumGhost,(ex->hgrid,(HYPRE_Int *)sw));
 
   /* create the hypre stencil object and set its information */
@@ -474,13 +191,14 @@ static PetscErrorCode  MatSetupDM_HYPREStruct(Mat mat,DM da)
   ierr = PetscLayoutSetBlockSize(mat->cmap,1);CHKERRQ(ierr);
   ierr = PetscLayoutSetUp(mat->rmap);CHKERRQ(ierr);
   ierr = PetscLayoutSetUp(mat->cmap);CHKERRQ(ierr);
+  mat->preallocated = PETSC_TRUE;
 
   if (dim == 3) {
     mat->ops->setvalueslocal = MatSetValuesLocal_HYPREStruct_3d;
     mat->ops->zerorowslocal  = MatZeroRowsLocal_HYPREStruct_3d;
     mat->ops->zeroentries    = MatZeroEntries_HYPREStruct_3d;
 
-    ierr = MatZeroEntries_HYPREStruct_3d(mat);CHKERRQ(ierr);
+    /*        ierr = MatZeroEntries_HYPREStruct_3d(mat);CHKERRQ(ierr); */
   } else SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_SUP,"Only support for 3d DMDA currently");
 
   /* get values that will be used repeatedly in MatSetValuesLocal() and MatZeroRowsLocal() repeatedly */
@@ -494,14 +212,13 @@ static PetscErrorCode  MatSetupDM_HYPREStruct(Mat mat,DM da)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_HYPREStruct"
 PetscErrorCode MatMult_HYPREStruct(Mat A,Vec x,Vec y)
 {
-  PetscErrorCode  ierr;
-  PetscScalar     *xx,*yy;
-  PetscInt        ilower[3],iupper[3];
-  Mat_HYPREStruct *mx = (Mat_HYPREStruct*)(A->data);
+  PetscErrorCode    ierr;
+  const PetscScalar *xx;
+  PetscScalar       *yy;
+  PetscInt          ilower[3],iupper[3];
+  Mat_HYPREStruct   *mx = (Mat_HYPREStruct*)(A->data);
 
   PetscFunctionBegin;
   ierr = DMDAGetCorners(mx->da,&ilower[0],&ilower[1],&ilower[2],&iupper[0],&iupper[1],&iupper[2]);CHKERRQ(ierr);
@@ -512,9 +229,9 @@ PetscErrorCode MatMult_HYPREStruct(Mat A,Vec x,Vec y)
 
   /* copy x values over to hypre */
   PetscStackCallStandard(HYPRE_StructVectorSetConstantValues,(mx->hb,0.0));
-  ierr = VecGetArray(x,&xx);CHKERRQ(ierr);
-  PetscStackCallStandard(HYPRE_StructVectorSetBoxValues,(mx->hb,(HYPRE_Int *)ilower,(HYPRE_Int *)iupper,xx));
-  ierr = VecRestoreArray(x,&xx);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(x,&xx);CHKERRQ(ierr);
+  PetscStackCallStandard(HYPRE_StructVectorSetBoxValues,(mx->hb,(HYPRE_Int *)ilower,(HYPRE_Int *)iupper,(PetscScalar*)xx));
+  ierr = VecRestoreArrayRead(x,&xx);CHKERRQ(ierr);
   PetscStackCallStandard(HYPRE_StructVectorAssemble,(mx->hb));
   PetscStackCallStandard(HYPRE_StructMatrixMatvec,(1.0,mx->hmat,mx->hb,0.0,mx->hx));
 
@@ -525,8 +242,6 @@ PetscErrorCode MatMult_HYPREStruct(Mat A,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatAssemblyEnd_HYPREStruct"
 PetscErrorCode MatAssemblyEnd_HYPREStruct(Mat mat,MatAssemblyType mode)
 {
   Mat_HYPREStruct *ex = (Mat_HYPREStruct*) mat->data;
@@ -538,8 +253,6 @@ PetscErrorCode MatAssemblyEnd_HYPREStruct(Mat mat,MatAssemblyType mode)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatZeroEntries_HYPREStruct"
 PetscErrorCode MatZeroEntries_HYPREStruct(Mat mat)
 {
   PetscFunctionBegin;
@@ -547,9 +260,6 @@ PetscErrorCode MatZeroEntries_HYPREStruct(Mat mat)
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_HYPREStruct"
 PetscErrorCode MatDestroy_HYPREStruct(Mat mat)
 {
   Mat_HYPREStruct *ex = (Mat_HYPREStruct*) mat->data;
@@ -559,12 +269,12 @@ PetscErrorCode MatDestroy_HYPREStruct(Mat mat)
   PetscStackCallStandard(HYPRE_StructMatrixDestroy,(ex->hmat));
   PetscStackCallStandard(HYPRE_StructVectorDestroy,(ex->hx));
   PetscStackCallStandard(HYPRE_StructVectorDestroy,(ex->hb));
+  ierr = PetscObjectDereference((PetscObject)ex->da);CHKERRQ(ierr);
+  ierr = MPI_Comm_free(&(ex->hcomm));CHKERRQ(ierr);
+  ierr = PetscFree(ex);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "MatCreate_HYPREStruct"
 PETSC_EXTERN PetscErrorCode MatCreate_HYPREStruct(Mat B)
 {
   Mat_HYPREStruct *ex;
@@ -582,11 +292,11 @@ PETSC_EXTERN PetscErrorCode MatCreate_HYPREStruct(Mat B)
   B->ops->mult        = MatMult_HYPREStruct;
   B->ops->zeroentries = MatZeroEntries_HYPREStruct;
   B->ops->destroy     = MatDestroy_HYPREStruct;
+  B->ops->setup       = MatSetUp_HYPREStruct;
 
   ex->needsinitialization = PETSC_TRUE;
 
   ierr = MPI_Comm_dup(PetscObjectComm((PetscObject)B),&(ex->hcomm));CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)B,"MatSetupDM_C",MatSetupDM_HYPREStruct);CHKERRQ(ierr);
   ierr = PetscObjectChangeTypeName((PetscObject)B,MATHYPRESTRUCT);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -598,18 +308,17 @@ PETSC_EXTERN PetscErrorCode MatCreate_HYPREStruct(Mat B)
 
    Level: intermediate
 
-   Notes: Unlike hypre's general semi-struct object consisting of a collection of structured-grid objects and unstructured
-          grid objects, we will restrict the semi-struct objects to consist of only structured-grid components.
+   Notes:
+    Unlike hypre's general semi-struct object consisting of a collection of structured-grid objects and unstructured
+          grid objects, we restrict the semi-struct objects to consist of only structured-grid components.
 
           Unlike the more general support for parts and blocks in hypre this allows only one part, and one block per process and requires the block
           be defined by a DMDA.
 
-          The matrix needs a DMDA associated with it by either a call to MatSetupDM() or if the matrix is obtained from DMCreateMatrix()
+          The matrix needs a DMDA associated with it by either a call to MatSetDM() or if the matrix is obtained from DMCreateMatrix()
 
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSetValuesLocal_HYPRESStruct_3d"
 PetscErrorCode  MatSetValuesLocal_HYPRESStruct_3d(Mat mat,PetscInt nrow,const PetscInt irow[],PetscInt ncol,const PetscInt icol[],const PetscScalar y[],InsertMode addv)
 {
   PetscErrorCode    ierr;
@@ -617,21 +326,19 @@ PetscErrorCode  MatSetValuesLocal_HYPRESStruct_3d(Mat mat,PetscInt nrow,const Pe
   const PetscScalar *values = y;
   Mat_HYPRESStruct  *ex     = (Mat_HYPRESStruct*) mat->data;
 
-  PetscInt part = 0;          /* Petsc sstruct interface only allows 1 part */
-  PetscInt ordering;
-  PetscInt grid_rank, to_grid_rank;
-  PetscInt var_type, to_var_type;
-  PetscInt to_var_entry = 0;
-
-  PetscInt nvars= ex->nvars;
-  PetscInt row,*entries;
+  PetscInt          part = 0;          /* Petsc sstruct interface only allows 1 part */
+  PetscInt          ordering;
+  PetscInt          grid_rank, to_grid_rank;
+  PetscInt          var_type, to_var_type;
+  PetscInt          to_var_entry = 0;
+  PetscInt          nvars= ex->nvars;
+  PetscInt          row,*entries;
 
   PetscFunctionBegin;
-  ierr    = PetscMalloc1(7*nvars,&entries);CHKERRQ(ierr);
-  ordering= ex->dofs_order;  /* ordering= 0   nodal ordering
-                                          1   variable ordering */
-  /* stencil entries are orderer by variables: var0_stencil0, var0_stencil1, ..., var0_stencil6, var1_stencil0, var1_stencil1, ...  */
-
+  ierr     = PetscMalloc1(7*nvars,&entries);CHKERRQ(ierr);
+  ordering = ex->dofs_order;  /* ordering= 0   nodal ordering
+                                           1   variable ordering */
+  /* stencil entries are ordered by variables: var0_stencil0, var0_stencil1, ..., var0_stencil6, var1_stencil0, var1_stencil1, ...  */
   if (!ordering) {
     for (i=0; i<nrow; i++) {
       grid_rank= irow[i]/nvars;
@@ -721,8 +428,6 @@ PetscErrorCode  MatSetValuesLocal_HYPRESStruct_3d(Mat mat,PetscInt nrow,const Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatZeroRowsLocal_HYPRESStruct_3d"
 PetscErrorCode  MatZeroRowsLocal_HYPRESStruct_3d(Mat mat,PetscInt nrow,const PetscInt irow[],PetscScalar d,Vec x,Vec b)
 {
   PetscErrorCode   ierr;
@@ -730,12 +435,12 @@ PetscErrorCode  MatZeroRowsLocal_HYPRESStruct_3d(Mat mat,PetscInt nrow,const Pet
   PetscScalar      **values;
   Mat_HYPRESStruct *ex = (Mat_HYPRESStruct*) mat->data;
 
-  PetscInt part     = 0;      /* Petsc sstruct interface only allows 1 part */
-  PetscInt ordering = ex->dofs_order;
-  PetscInt grid_rank;
-  PetscInt var_type;
-  PetscInt nvars= ex->nvars;
-  PetscInt row,*entries;
+  PetscInt         part     = 0;      /* Petsc sstruct interface only allows 1 part */
+  PetscInt         ordering = ex->dofs_order;
+  PetscInt         grid_rank;
+  PetscInt         var_type;
+  PetscInt         nvars= ex->nvars;
+  PetscInt         row,*entries;
 
   PetscFunctionBegin;
   if (x && b) SETERRQ(PetscObjectComm((PetscObject)mat),PETSC_ERR_SUP,"No support");
@@ -784,8 +489,6 @@ PetscErrorCode  MatZeroRowsLocal_HYPRESStruct_3d(Mat mat,PetscInt nrow,const Pet
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatZeroEntries_HYPRESStruct_3d"
 PetscErrorCode MatZeroEntries_HYPRESStruct_3d(Mat mat)
 {
   PetscErrorCode   ierr;
@@ -820,9 +523,7 @@ PetscErrorCode MatZeroEntries_HYPRESStruct_3d(Mat mat)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSetupDM_HYPRESStruct"
-static PetscErrorCode  MatSetupDM_HYPRESStruct(Mat mat,DM da)
+static PetscErrorCode  MatSetUp_HYPRESStruct(Mat mat)
 {
   PetscErrorCode         ierr;
   Mat_HYPRESStruct       *ex = (Mat_HYPRESStruct*) mat->data;
@@ -833,7 +534,10 @@ static PetscErrorCode  MatSetupDM_HYPRESStruct(Mat mat,DM da)
   PetscInt               nparts= 1;  /* assuming only one part */
   PetscInt               part  = 0;
   ISLocalToGlobalMapping ltog;
+  DM                     da;
+
   PetscFunctionBegin;
+  ierr   = MatGetDM(mat,(DM*)&da);CHKERRQ(ierr);
   ex->da = da;
   ierr   = PetscObjectReference((PetscObject)da);CHKERRQ(ierr);
 
@@ -858,9 +562,7 @@ static PetscErrorCode  MatSetupDM_HYPRESStruct(Mat mat,DM da)
   /* create the hypre grid object and set its information */
   if (px || py || pz) SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_SUP,"Ask us to add periodic support by calling HYPRE_SStructGridSetPeriodic()");
   PetscStackCallStandard(HYPRE_SStructGridCreate,(ex->hcomm,dim,nparts,&ex->ss_grid));
-
   PetscStackCallStandard(HYPRE_SStructGridSetExtents,(ex->ss_grid,part,ex->hbox.imin,ex->hbox.imax));
-
   {
     HYPRE_SStructVariable *vartypes;
     ierr = PetscMalloc1(ex->nvars,&vartypes);CHKERRQ(ierr);
@@ -949,17 +651,18 @@ static PetscErrorCode  MatSetupDM_HYPRESStruct(Mat mat,DM da)
   /* set the global and local sizes of the matrix */
   ierr = DMDAGetCorners(da,0,0,0,&nx,&ny,&nz);CHKERRQ(ierr);
   ierr = MatSetSizes(mat,dof*nx*ny*nz,dof*nx*ny*nz,PETSC_DECIDE,PETSC_DECIDE);CHKERRQ(ierr);
-  ierr = PetscLayoutSetBlockSize(mat->rmap,1);CHKERRQ(ierr);
-  ierr = PetscLayoutSetBlockSize(mat->cmap,1);CHKERRQ(ierr);
+  ierr = PetscLayoutSetBlockSize(mat->rmap,dof);CHKERRQ(ierr);
+  ierr = PetscLayoutSetBlockSize(mat->cmap,dof);CHKERRQ(ierr);
   ierr = PetscLayoutSetUp(mat->rmap);CHKERRQ(ierr);
   ierr = PetscLayoutSetUp(mat->cmap);CHKERRQ(ierr);
+  mat->preallocated = PETSC_TRUE;
 
   if (dim == 3) {
     mat->ops->setvalueslocal = MatSetValuesLocal_HYPRESStruct_3d;
     mat->ops->zerorowslocal  = MatZeroRowsLocal_HYPRESStruct_3d;
     mat->ops->zeroentries    = MatZeroEntries_HYPRESStruct_3d;
 
-    ierr = MatZeroEntries_HYPRESStruct_3d(mat);CHKERRQ(ierr);
+    /* ierr = MatZeroEntries_HYPRESStruct_3d(mat);CHKERRQ(ierr); */
   } else SETERRQ(PetscObjectComm((PetscObject)da),PETSC_ERR_SUP,"Only support for 3d DMDA currently");
 
   /* get values that will be used repeatedly in MatSetValuesLocal() and MatZeroRowsLocal() repeatedly */
@@ -978,19 +681,18 @@ static PetscErrorCode  MatSetupDM_HYPRESStruct(Mat mat,DM da)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_HYPRESStruct"
 PetscErrorCode MatMult_HYPRESStruct(Mat A,Vec x,Vec y)
 {
-  PetscErrorCode   ierr;
-  PetscScalar      *xx,*yy;
-  PetscInt         ilower[3],iupper[3];
-  Mat_HYPRESStruct *mx     = (Mat_HYPRESStruct*)(A->data);
-  PetscInt         ordering= mx->dofs_order;
-  PetscInt         nvars   = mx->nvars;
-  PetscInt         part    = 0;
-  PetscInt         size;
-  PetscInt         i;
+  PetscErrorCode    ierr;
+  const PetscScalar *xx;
+  PetscScalar       *yy;
+  PetscInt          ilower[3],iupper[3];
+  Mat_HYPRESStruct  *mx     = (Mat_HYPRESStruct*)(A->data);
+  PetscInt          ordering= mx->dofs_order;
+  PetscInt          nvars   = mx->nvars;
+  PetscInt          part    = 0;
+  PetscInt          size;
+  PetscInt          i;
 
   PetscFunctionBegin;
   ierr       = DMDAGetCorners(mx->da,&ilower[0],&ilower[1],&ilower[2],&iupper[0],&iupper[1],&iupper[2]);CHKERRQ(ierr);
@@ -1004,11 +706,11 @@ PetscErrorCode MatMult_HYPRESStruct(Mat A,Vec x,Vec y)
   /* copy x values over to hypre for variable ordering */
   if (ordering) {
     PetscStackCallStandard(HYPRE_SStructVectorSetConstantValues,(mx->ss_b,0.0));
-    ierr = VecGetArray(x,&xx);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(x,&xx);CHKERRQ(ierr);
     for (i= 0; i< nvars; i++) {
-      PetscStackCallStandard(HYPRE_SStructVectorSetBoxValues,(mx->ss_b,part,(HYPRE_Int *)ilower,(HYPRE_Int *)iupper,i,xx+(size*i)));
+      PetscStackCallStandard(HYPRE_SStructVectorSetBoxValues,(mx->ss_b,part,(HYPRE_Int *)ilower,(HYPRE_Int *)iupper,i,(PetscScalar*)xx+(size*i)));
     }
-    ierr = VecRestoreArray(x,&xx);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(x,&xx);CHKERRQ(ierr);
     PetscStackCallStandard(HYPRE_SStructVectorAssemble,(mx->ss_b));
     PetscStackCallStandard(HYPRE_SStructMatrixMatvec,(1.0,mx->ss_mat,mx->ss_b,0.0,mx->ss_x));
 
@@ -1024,7 +726,7 @@ PetscErrorCode MatMult_HYPRESStruct(Mat A,Vec x,Vec y)
 
     ierr = PetscMalloc1(nvars*size,&z);CHKERRQ(ierr);
     PetscStackCallStandard(HYPRE_SStructVectorSetConstantValues,(mx->ss_b,0.0));
-    ierr = VecGetArray(x,&xx);CHKERRQ(ierr);
+    ierr = VecGetArrayRead(x,&xx);CHKERRQ(ierr);
 
     /* transform nodal to hypre's variable ordering for sys_pfmg */
     for (i= 0; i< size; i++) {
@@ -1034,7 +736,7 @@ PetscErrorCode MatMult_HYPRESStruct(Mat A,Vec x,Vec y)
     for (i= 0; i< nvars; i++) {
       PetscStackCallStandard(HYPRE_SStructVectorSetBoxValues,(mx->ss_b,part,(HYPRE_Int *)ilower,(HYPRE_Int *)iupper,i,z+(size*i)));
     }
-    ierr = VecRestoreArray(x,&xx);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(x,&xx);CHKERRQ(ierr);
     PetscStackCallStandard(HYPRE_SStructVectorAssemble,(mx->ss_b));
     PetscStackCallStandard(HYPRE_SStructMatrixMatvec,(1.0,mx->ss_mat,mx->ss_b,0.0,mx->ss_x));
 
@@ -1054,8 +756,6 @@ PetscErrorCode MatMult_HYPRESStruct(Mat A,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatAssemblyEnd_HYPRESStruct"
 PetscErrorCode MatAssemblyEnd_HYPRESStruct(Mat mat,MatAssemblyType mode)
 {
   Mat_HYPRESStruct *ex = (Mat_HYPRESStruct*) mat->data;
@@ -1066,8 +766,6 @@ PetscErrorCode MatAssemblyEnd_HYPRESStruct(Mat mat,MatAssemblyType mode)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatZeroEntries_HYPRESStruct"
 PetscErrorCode MatZeroEntries_HYPRESStruct(Mat mat)
 {
   PetscFunctionBegin;
@@ -1075,24 +773,25 @@ PetscErrorCode MatZeroEntries_HYPRESStruct(Mat mat)
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_HYPRESStruct"
 PetscErrorCode MatDestroy_HYPRESStruct(Mat mat)
 {
-  Mat_HYPRESStruct *ex = (Mat_HYPRESStruct*) mat->data;
-  PetscErrorCode   ierr;
+  Mat_HYPRESStruct       *ex = (Mat_HYPRESStruct*) mat->data;
+  PetscErrorCode         ierr;
+  ISLocalToGlobalMapping ltog;
 
   PetscFunctionBegin;
+  ierr = DMGetLocalToGlobalMapping(ex->da,&ltog);CHKERRQ(ierr);
+  ierr = ISLocalToGlobalMappingRestoreIndices(ltog, (const PetscInt **) &ex->gindices);CHKERRQ(ierr);
   PetscStackCallStandard(HYPRE_SStructGraphDestroy,(ex->ss_graph));
   PetscStackCallStandard(HYPRE_SStructMatrixDestroy,(ex->ss_mat));
   PetscStackCallStandard(HYPRE_SStructVectorDestroy,(ex->ss_x));
   PetscStackCallStandard(HYPRE_SStructVectorDestroy,(ex->ss_b));
+  ierr = PetscObjectDereference((PetscObject)ex->da);CHKERRQ(ierr);
+  ierr = MPI_Comm_free(&(ex->hcomm));CHKERRQ(ierr);
+  ierr = PetscFree(ex);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCreate_HYPRESStruct"
 PETSC_EXTERN PetscErrorCode MatCreate_HYPRESStruct(Mat B)
 {
   Mat_HYPRESStruct *ex;
@@ -1110,11 +809,11 @@ PETSC_EXTERN PetscErrorCode MatCreate_HYPRESStruct(Mat B)
   B->ops->mult        = MatMult_HYPRESStruct;
   B->ops->zeroentries = MatZeroEntries_HYPRESStruct;
   B->ops->destroy     = MatDestroy_HYPRESStruct;
+  B->ops->setup       = MatSetUp_HYPRESStruct;
 
   ex->needsinitialization = PETSC_TRUE;
 
   ierr = MPI_Comm_dup(PetscObjectComm((PetscObject)B),&(ex->hcomm));CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)B,"MatSetupDM_C",MatSetupDM_HYPRESStruct);CHKERRQ(ierr);
   ierr = PetscObjectChangeTypeName((PetscObject)B,MATHYPRESSTRUCT);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
