@@ -2,8 +2,6 @@
 
 #include <petscdmmoab.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "DMMoabSetFieldVector"
 /*@C
   DMMoabSetFieldVector - Sets the vector reference that represents the solution associated
   with a particular field component.
@@ -24,7 +22,7 @@
 PetscErrorCode DMMoabSetFieldVector(DM dm, PetscInt ifield, Vec fvec)
 {
   DM_Moab        *dmmoab;
-  moab::Tag     vtag,ntag;
+  moab::Tag     vtag, ntag;
   const PetscScalar *varray;
   PetscScalar *farray;
   moab::ErrorCode merr;
@@ -32,43 +30,43 @@ PetscErrorCode DMMoabSetFieldVector(DM dm, PetscInt ifield, Vec fvec)
   std::string tag_name;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
   dmmoab = (DM_Moab*)(dm)->data;
 
   if ((ifield < 0) || (ifield >= dmmoab->numFields)) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "The field %d should be positive and less than %d.", ifield, dmmoab->numFields);
 
   /* Create a tag in MOAB mesh to index and keep track of number of Petsc vec tags */
-  merr = dmmoab->mbiface->tag_get_handle(dmmoab->fieldNames[ifield],1,moab::MB_TYPE_DOUBLE,ntag,
-                                          moab::MB_TAG_DENSE|moab::MB_TAG_CREAT);MBERRNM(merr);
+  merr = dmmoab->mbiface->tag_get_handle(dmmoab->fieldNames[ifield], 1, moab::MB_TYPE_DOUBLE, ntag,
+                                         moab::MB_TAG_DENSE | moab::MB_TAG_CREAT); MBERRNM(merr);
 
-  ierr = DMMoabGetVecTag(fvec,&vtag);CHKERRQ(ierr);
+  ierr = DMMoabGetVecTag(fvec, &vtag);CHKERRQ(ierr);
 
   merr = dmmoab->mbiface->tag_get_name(vtag, tag_name);
-  if (!tag_name.length() && merr !=moab::MB_SUCCESS) {
-    ierr = VecGetArrayRead(fvec,&varray);CHKERRQ(ierr);
+  if (!tag_name.length() && merr != moab::MB_SUCCESS) {
+    ierr = VecGetArrayRead(fvec, &varray);CHKERRQ(ierr);
     /* use the entity handle and the Dof index to set the right value */
-    merr = dmmoab->mbiface->tag_set_data(ntag, *dmmoab->vowned, (const void*)varray);MBERRNM(merr);
-    ierr = VecRestoreArrayRead(fvec,&varray);CHKERRQ(ierr);
+    merr = dmmoab->mbiface->tag_set_data(ntag, *dmmoab->vowned, (const void*)varray); MBERRNM(merr);
+    ierr = VecRestoreArrayRead(fvec, &varray);CHKERRQ(ierr);
   }
   else {
-    ierr = PetscMalloc1(dmmoab->nloc,&farray);CHKERRQ(ierr);
+    ierr = PetscMalloc1(dmmoab->nloc, &farray);CHKERRQ(ierr);
     /* we are using a MOAB Vec - directly copy the tag data to new one */
-    merr = dmmoab->mbiface->tag_get_data(vtag, *dmmoab->vowned, (void*)farray);MBERRNM(merr);
-    merr = dmmoab->mbiface->tag_set_data(ntag, *dmmoab->vowned, (const void*)farray);MBERRNM(merr);
+    merr = dmmoab->mbiface->tag_get_data(vtag, *dmmoab->vowned, (void*)farray); MBERRNM(merr);
+    merr = dmmoab->mbiface->tag_set_data(ntag, *dmmoab->vowned, (const void*)farray); MBERRNM(merr);
     /* make sure the parallel exchange for ghosts are done appropriately */
     ierr = PetscFree(farray);CHKERRQ(ierr);
   }
-  merr = dmmoab->pcomm->exchange_tags(ntag, *dmmoab->vowned);MBERRNM(merr);
+#ifdef MOAB_HAVE_MPI
+  merr = dmmoab->pcomm->exchange_tags(ntag, *dmmoab->vowned); MBERRNM(merr);
+#endif
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMMoabSetGlobalFieldVector"
 /*@C
   DMMoabSetGlobalFieldVector - Sets the vector reference that represents the global solution associated
-  with all fields (components) managed by DM. 
-  A useful utility when updating the DM solution after a solve, to be serialized with the mesh for 
+  with all fields (components) managed by DM.
+  A useful utility when updating the DM solution after a solve, to be serialized with the mesh for
   checkpointing purposes.
 
   Not Collective
@@ -86,67 +84,63 @@ PetscErrorCode DMMoabSetFieldVector(DM dm, PetscInt ifield, Vec fvec)
 PetscErrorCode DMMoabSetGlobalFieldVector(DM dm, Vec fvec)
 {
   DM_Moab        *dmmoab;
-  moab::Tag     vtag,ntag;
+  moab::Tag     vtag, ntag;
   const PetscScalar   *rarray;
-  PetscScalar   *varray,*farray;
+  PetscScalar   *varray, *farray;
   moab::ErrorCode merr;
   PetscErrorCode  ierr;
-  PetscInt i,ifield;
+  PetscInt i, ifield;
   std::string tag_name;
   moab::Range::iterator iter;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
   dmmoab = (DM_Moab*)(dm)->data;
 
   /* get the Tag corresponding to the global vector - possible that there is no tag associated.. */
-  ierr = DMMoabGetVecTag(fvec,&vtag);CHKERRQ(ierr);
+  ierr = DMMoabGetVecTag(fvec, &vtag);CHKERRQ(ierr);
   merr = dmmoab->mbiface->tag_get_name(vtag, tag_name);
-  ierr = PetscMalloc1(dmmoab->nloc,&farray);CHKERRQ(ierr);
-  if (!tag_name.length() && merr !=moab::MB_SUCCESS) {
+  ierr = PetscMalloc1(dmmoab->nloc, &farray);CHKERRQ(ierr);
+  if (!tag_name.length() && merr != moab::MB_SUCCESS) {
     /* not a MOAB vector - use VecGetSubVector to get the parts as needed */
-
-    ierr = VecGetArrayRead(fvec,&rarray);CHKERRQ(ierr);
-    for (ifield=0; ifield<dmmoab->numFields; ++ifield) {
+    ierr = VecGetArrayRead(fvec, &rarray);CHKERRQ(ierr);
+    for (ifield = 0; ifield < dmmoab->numFields; ++ifield) {
 
       /* Create a tag in MOAB mesh to index and keep track of number of Petsc vec tags */
-      merr = dmmoab->mbiface->tag_get_handle(dmmoab->fieldNames[ifield],1,moab::MB_TYPE_DOUBLE,ntag,
-                                            moab::MB_TAG_DENSE|moab::MB_TAG_CREAT);MBERRNM(merr);
+      merr = dmmoab->mbiface->tag_get_handle(dmmoab->fieldNames[ifield], 1, moab::MB_TYPE_DOUBLE, ntag,
+                                             moab::MB_TAG_DENSE | moab::MB_TAG_CREAT); MBERRNM(merr);
 
-      for(i=0;i<dmmoab->nloc;i++) {
-        if (dmmoab->bs == 1)
-          farray[i]=rarray[ifield*dmmoab->nloc+i];
-        else
-          farray[i]=rarray[i*dmmoab->numFields+ifield];
+      for (i = 0; i < dmmoab->nloc; i++) {
+        farray[i] = (dmmoab->bs == 1 ? rarray[ifield * dmmoab->nloc + i] : rarray[i * dmmoab->numFields + ifield]);
       }
 
       /* use the entity handle and the Dof index to set the right value */
-      merr = dmmoab->mbiface->tag_set_data(ntag, *dmmoab->vowned, (const void*)farray);MBERRNM(merr);
+      merr = dmmoab->mbiface->tag_set_data(ntag, *dmmoab->vowned, (const void*)farray); MBERRNM(merr);
     }
-    ierr = VecRestoreArrayRead(fvec,&rarray);CHKERRQ(ierr);
+    ierr = VecRestoreArrayRead(fvec, &rarray);CHKERRQ(ierr);
   }
   else {
-    ierr = PetscMalloc1(dmmoab->nloc*dmmoab->numFields,&varray);CHKERRQ(ierr);
+    ierr = PetscMalloc1(dmmoab->nloc * dmmoab->numFields, &varray);CHKERRQ(ierr);
 
     /* we are using a MOAB Vec - directly copy the tag data to new one */
-    merr = dmmoab->mbiface->tag_get_data(vtag, *dmmoab->vowned, (void*)varray);MBERRNM(merr);
-    for (ifield=0; ifield<dmmoab->numFields; ++ifield) {
+    merr = dmmoab->mbiface->tag_get_data(vtag, *dmmoab->vowned, (void*)varray); MBERRNM(merr);
+    for (ifield = 0; ifield < dmmoab->numFields; ++ifield) {
 
       /* Create a tag in MOAB mesh to index and keep track of number of Petsc vec tags */
-      merr = dmmoab->mbiface->tag_get_handle(dmmoab->fieldNames[ifield],1,moab::MB_TYPE_DOUBLE,ntag,
-                                            moab::MB_TAG_DENSE|moab::MB_TAG_CREAT);MBERRNM(merr);
+      merr = dmmoab->mbiface->tag_get_handle(dmmoab->fieldNames[ifield], 1, moab::MB_TYPE_DOUBLE, ntag,
+                                             moab::MB_TAG_DENSE | moab::MB_TAG_CREAT); MBERRNM(merr);
 
       /* we are using a MOAB Vec - directly copy the tag data to new one */
-      for(i=0; i < dmmoab->nloc; i++) {
-        if (dmmoab->bs == 1)
-          farray[i]=varray[ifield*dmmoab->nloc+i];
-        else
-          farray[i]=varray[i*dmmoab->numFields+ifield];
+      for (i = 0; i < dmmoab->nloc; i++) {
+        farray[i] = (dmmoab->bs == 1 ? varray[ifield * dmmoab->nloc + i] : varray[i * dmmoab->numFields + ifield]);
       }
 
-      merr = dmmoab->mbiface->tag_set_data(ntag, *dmmoab->vowned, (const void*)farray);MBERRNM(merr);
+      merr = dmmoab->mbiface->tag_set_data(ntag, *dmmoab->vowned, (const void*)farray); MBERRNM(merr);
+
+#ifdef MOAB_HAVE_MPI
       /* make sure the parallel exchange for ghosts are done appropriately */
-      merr = dmmoab->pcomm->exchange_tags(ntag, *dmmoab->vlocal);MBERRNM(merr);
+      merr = dmmoab->pcomm->exchange_tags(ntag, *dmmoab->vlocal); MBERRNM(merr);
+#endif
     }
     ierr = PetscFree(varray);CHKERRQ(ierr);
   }
@@ -155,8 +149,6 @@ PetscErrorCode DMMoabSetGlobalFieldVector(DM dm, Vec fvec)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMMoabSetFieldNames"
 /*@C
   DMMoabSetFieldNames - Sets the number of fields and their names to be managed by the DM
 
@@ -170,22 +162,22 @@ PetscErrorCode DMMoabSetGlobalFieldVector(DM dm, Vec fvec)
   Level: intermediate
 
 .keywords: discretization manager, set, component name
-  
+
 .seealso: DMMoabGetFieldName(), DMMoabSetFieldName()
 @*/
-PetscErrorCode DMMoabSetFieldNames(DM dm,PetscInt numFields,const char* fields[])
+PetscErrorCode DMMoabSetFieldNames(DM dm, PetscInt numFields, const char* fields[])
 {
   PetscErrorCode ierr;
   PetscInt       i;
   DM_Moab        *dmmoab;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
   dmmoab = (DM_Moab*)(dm)->data;
 
   /* first deallocate the existing field structure */
   if (dmmoab->fieldNames) {
-    for(i=0; i<dmmoab->numFields; i++) {
+    for (i = 0; i < dmmoab->numFields; i++) {
       ierr = PetscFree(dmmoab->fieldNames[i]);CHKERRQ(ierr);
     }
     ierr = PetscFree(dmmoab->fieldNames);CHKERRQ(ierr);
@@ -193,18 +185,17 @@ PetscErrorCode DMMoabSetFieldNames(DM dm,PetscInt numFields,const char* fields[]
 
   /* now re-allocate and assign field names  */
   dmmoab->numFields = numFields;
-  ierr = PetscMalloc(numFields*sizeof(char*),&dmmoab->fieldNames);CHKERRQ(ierr);
+  ierr = PetscMalloc1(numFields, &dmmoab->fieldNames);CHKERRQ(ierr);
   if (fields) {
-    for(i=0; i<dmmoab->numFields; i++) {
+    for (i = 0; i < dmmoab->numFields; i++) {
       ierr = PetscStrallocpy(fields[i], (char**) &dmmoab->fieldNames[i]);CHKERRQ(ierr);
     }
   }
+  ierr = DMSetNumFields(dm, numFields);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMMoabGetFieldName"
 /*@C
   DMMoabGetFieldName - Gets the names of individual field components in multicomponent
   vectors associated with a DMDA.
@@ -230,7 +221,7 @@ PetscErrorCode DMMoabGetFieldName(DM dm, PetscInt field, const char **fieldName)
   DM_Moab        *dmmoab;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
   dmmoab = (DM_Moab*)(dm)->data;
   if ((field < 0) || (field >= dmmoab->numFields)) SETERRQ3(PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "DM field %d should be in [%d, %d)", field, 0, dmmoab->numFields);
 
@@ -239,8 +230,6 @@ PetscErrorCode DMMoabGetFieldName(DM dm, PetscInt field, const char **fieldName)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMMoabSetFieldName"
 /*@C
   DMMoabSetFieldName - Sets the name of a field (component) managed by the DM
 
@@ -252,10 +241,11 @@ PetscErrorCode DMMoabGetFieldName(DM dm, PetscInt field, const char **fieldName)
 . fieldName - the field (component) name
 
   Level: intermediate
-  Notes: Can only be called after DMMoabSetFields supplied with correct numFields
+  Notes:
+    Can only be called after DMMoabSetFields supplied with correct numFields
 
 .keywords: discretization manager, set, component name
-  
+
 .seealso: DMMoabGetFieldName(), DMMoabSetFields()
 @*/
 PetscErrorCode DMMoabSetFieldName(DM dm, PetscInt field, const char *fieldName)
@@ -264,8 +254,8 @@ PetscErrorCode DMMoabSetFieldName(DM dm, PetscInt field, const char *fieldName)
   DM_Moab        *dmmoab;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
-  PetscValidCharPointer(fieldName,3);
+  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
+  PetscValidCharPointer(fieldName, 3);
 
   dmmoab = (DM_Moab*)(dm)->data;
   if ((field < 0) || (field >= dmmoab->numFields)) SETERRQ3(PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "DM field %d should be in [%d, %d)", field, 0, dmmoab->numFields);
@@ -278,8 +268,6 @@ PetscErrorCode DMMoabSetFieldName(DM dm, PetscInt field, const char *fieldName)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMMoabGetFieldDof"
 /*@C
   DMMoabGetFieldDof - Gets the global degree-of-freedom of a field (component) defined on a
   particular MOAB EntityHandle.
@@ -300,21 +288,20 @@ PetscErrorCode DMMoabSetFieldName(DM dm, PetscInt field, const char *fieldName)
 
 .seealso: DMMoabGetFieldDofs(), DMMoabGetFieldDofsLocal()
 @*/
-PetscErrorCode DMMoabGetFieldDof(DM dm,moab::EntityHandle point,PetscInt field,PetscInt* dof)
+PetscErrorCode DMMoabGetFieldDof(DM dm, moab::EntityHandle point, PetscInt field, PetscInt* dof)
 {
   DM_Moab        *dmmoab;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
   dmmoab = (DM_Moab*)(dm)->data;
 
-  *dof=dmmoab->gidmap[(PetscInt)point];
+  *dof = (dmmoab->bs == 1 ? dmmoab->gidmap[dmmoab->mbiface->id_from_handle(point) - dmmoab->seqstart] + field * dmmoab->n :
+              dmmoab->gidmap[dmmoab->mbiface->id_from_handle(point) - dmmoab->seqstart] * dmmoab->numFields + field);
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMMoabGetFieldDofs"
 /*@C
   DMMoabGetFieldDofs - Gets the global degree-of-freedom of a field (component) defined on an
   array of MOAB EntityHandles.
@@ -333,40 +320,34 @@ PetscErrorCode DMMoabGetFieldDof(DM dm,moab::EntityHandle point,PetscInt field,P
   Level: intermediate
 
 .keywords: discretization manager, get, global degrees of freedom
-  
+
 .seealso: DMMoabGetFieldDof(), DMMoabGetFieldDofsLocal()
 @*/
-PetscErrorCode DMMoabGetFieldDofs(DM dm,PetscInt npoints,const moab::EntityHandle* points,PetscInt field,PetscInt* dof)
+PetscErrorCode DMMoabGetFieldDofs(DM dm, PetscInt npoints, const moab::EntityHandle* points, PetscInt field, PetscInt* dof)
 {
   PetscInt        i;
   PetscErrorCode  ierr;
   DM_Moab        *dmmoab;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
-  PetscValidPointer(points,3);
+  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
+  PetscValidPointer(points, 3);
   dmmoab = (DM_Moab*)(dm)->data;
 
   if (!dof) {
-    ierr = PetscMalloc1(npoints,&dof);CHKERRQ(ierr);
+    ierr = PetscMalloc1(npoints, &dof);CHKERRQ(ierr);
   }
 
-  if (dmmoab->bs > 1) { /* compute the DOF based on local blocking in the fields */
-    for (i=0; i<npoints; ++i)
-      dof[i] = dmmoab->gidmap[(PetscInt)points[i]]*dmmoab->numFields+field;
-  }
-  else {
-    /* assume all fields have equal distribution; i.e., all fields are either defined on vertices or elements and not on a mixture */
-    /* TODO: eliminate the limitation using PetscSection to manage DOFs */
-    for (i=0; i<npoints; ++i)
-      dof[i] = dmmoab->gidmap[(PetscInt)points[i]]+field*dmmoab->n;
-  }
+  /* compute the DOF based on local blocking in the fields */
+  /* We also assume all fields have equal distribution; i.e., all fields are either defined on vertices or elements and not on a mixture */
+  /* TODO: eliminate the limitation using PetscSection to manage DOFs */
+  for (i = 0; i < npoints; ++i)
+    dof[i] = (dmmoab->bs == 1 ? dmmoab->gidmap[dmmoab->mbiface->id_from_handle(points[i]) - dmmoab->seqstart] + field * dmmoab->n :
+              dmmoab->gidmap[dmmoab->mbiface->id_from_handle(points[i]) - dmmoab->seqstart] * dmmoab->numFields + field);
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMMoabGetFieldDofsLocal"
 /*@C
   DMMoabGetFieldDofsLocal - Gets the local degrees-of-freedom of a field (component) defined on an
   array of MOAB EntityHandles.
@@ -385,41 +366,35 @@ PetscErrorCode DMMoabGetFieldDofs(DM dm,PetscInt npoints,const moab::EntityHandl
   Level: intermediate
 
 .keywords: discretization manager, get, local degrees of freedom
-  
+
 .seealso: DMMoabGetFieldDof(), DMMoabGetFieldDofs()
 @*/
-PetscErrorCode DMMoabGetFieldDofsLocal(DM dm,PetscInt npoints,const moab::EntityHandle* points,PetscInt field,PetscInt* dof)
+PetscErrorCode DMMoabGetFieldDofsLocal(DM dm, PetscInt npoints, const moab::EntityHandle* points, PetscInt field, PetscInt* dof)
 {
-  PetscInt i,offset;
+  PetscInt i;
   PetscErrorCode  ierr;
   DM_Moab        *dmmoab;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
-  PetscValidPointer(points,3);
+  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
+  PetscValidPointer(points, 3);
   dmmoab = (DM_Moab*)(dm)->data;
 
   if (!dof) {
-    ierr = PetscMalloc1(npoints,&dof);CHKERRQ(ierr);
+    ierr = PetscMalloc1(npoints, &dof);CHKERRQ(ierr);
   }
 
-  if (dmmoab->bs > 1) { /* compute the DOF based on local blocking in the fields */
-    for (i=0; i<npoints; ++i)
-      dof[i] = dmmoab->lidmap[(PetscInt)points[i]]*dmmoab->numFields+field;
-  }
-  else {
-    /* assume all fields have equal distribution; i.e., all fields are either defined on vertices or elements and not on a mixture */
-    /* TODO: eliminate the limitation using PetscSection to manage DOFs */
-    offset = field*dmmoab->n;
-    for (i=0; i<npoints; ++i)
-      dof[i] = dmmoab->lidmap[(PetscInt)points[i]]+offset;
+  /* compute the DOF based on local blocking in the fields */
+  /* assume all fields have equal distribution; i.e., all fields are either defined on vertices or elements and not on a mixture */
+  /* TODO: eliminate the limitation using PetscSection to manage DOFs */
+  for (i = 0; i < npoints; ++i) {
+    dof[i] = (dmmoab->bs > 1 ? dmmoab->lidmap[dmmoab->mbiface->id_from_handle(points[i]) - dmmoab->seqstart] * dmmoab->numFields + field :
+              dmmoab->lidmap[dmmoab->mbiface->id_from_handle(points[i]) - dmmoab->seqstart] + field * dmmoab->n);
   }
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMMoabGetDofs"
 /*@C
   DMMoabGetDofs - Gets the global degree-of-freedom for all fields (components) defined on an
   array of MOAB EntityHandles.
@@ -437,45 +412,36 @@ PetscErrorCode DMMoabGetFieldDofsLocal(DM dm,PetscInt npoints,const moab::Entity
   Level: intermediate
 
 .keywords: discretization manager, get, global degrees of freedom
-  
+
 .seealso: DMMoabGetFieldDofs(), DMMoabGetDofsLocal(), DMMoabGetDofsBlocked()
 @*/
-PetscErrorCode DMMoabGetDofs(DM dm,PetscInt npoints,const moab::EntityHandle* points,PetscInt* dof)
+PetscErrorCode DMMoabGetDofs(DM dm, PetscInt npoints, const moab::EntityHandle* points, PetscInt* dof)
 {
-  PetscInt        i,field,offset;  
+  PetscInt        i, field, offset;
   PetscErrorCode  ierr;
   DM_Moab        *dmmoab;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
-  PetscValidPointer(points,3);
+  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
+  PetscValidPointer(points, 3);
   dmmoab = (DM_Moab*)(dm)->data;
 
   if (!dof) {
-    ierr = PetscMalloc1(dmmoab->numFields*npoints,&dof);CHKERRQ(ierr);
+    ierr = PetscMalloc1(dmmoab->numFields * npoints, &dof);CHKERRQ(ierr);
   }
-  
-  if (dmmoab->bs > 1) { /* compute the DOF based on local blocking in the fields */
-    for (field=0; field<dmmoab->numFields; ++field) {
-      for (i=0; i<npoints; ++i)
-        dof[i*dmmoab->numFields+field] = dmmoab->gidmap[(PetscInt)points[i]]*dmmoab->numFields+field;
-    }
-  }
-  else {
-    /* assume all fields have equal distribution; i.e., all fields are either defined on vertices or elements and not on a mixture */
-    /* TODO: eliminate the limitation using PetscSection to manage DOFs */
-    for (field=0; field<dmmoab->numFields; ++field) {
-      offset = field*dmmoab->n;
-      for (i=0; i<npoints; ++i)
-        dof[i*dmmoab->numFields+field] = dmmoab->gidmap[(PetscInt)points[i]]+offset;
-    }
+
+  /* compute the DOF based on local blocking in the fields */
+  /* assume all fields have equal distribution; i.e., all fields are either defined on vertices or elements and not on a mixture */
+  /* TODO: eliminate the limitation using PetscSection to manage DOFs */
+  for (field = 0; field < dmmoab->numFields; ++field) {
+    offset = field * dmmoab->n;
+    for (i = 0; i < npoints; ++i)
+      dof[i * dmmoab->numFields + field] = (dmmoab->bs > 1 ? dmmoab->gidmap[dmmoab->mbiface->id_from_handle(points[i]) - dmmoab->seqstart] * dmmoab->numFields + field :
+                                            dmmoab->gidmap[dmmoab->mbiface->id_from_handle(points[i]) - dmmoab->seqstart] + offset);
   }
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "DMMoabGetDofsLocal"
 /*@C
   DMMoabGetDofsLocal - Gets the local degree-of-freedom for all fields (components) defined on an
   array of MOAB EntityHandles.
@@ -493,45 +459,37 @@ PetscErrorCode DMMoabGetDofs(DM dm,PetscInt npoints,const moab::EntityHandle* po
   Level: intermediate
 
 .keywords: discretization manager, get, global degrees of freedom
-  
+
 .seealso: DMMoabGetFieldDofs(), DMMoabGetDofs(), DMMoabGetDofsBlocked()
 @*/
-PetscErrorCode DMMoabGetDofsLocal(DM dm,PetscInt npoints,const moab::EntityHandle* points,PetscInt* dof)
+PetscErrorCode DMMoabGetDofsLocal(DM dm, PetscInt npoints, const moab::EntityHandle* points, PetscInt* dof)
 {
-  PetscInt        i,field,offset;
+  PetscInt        i, field, offset;
   PetscErrorCode  ierr;
   DM_Moab        *dmmoab;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
-  PetscValidPointer(points,3);
+  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
+  PetscValidPointer(points, 3);
   dmmoab = (DM_Moab*)(dm)->data;
 
   if (!dof) {
-    ierr = PetscMalloc1(dmmoab->numFields*npoints,&dof);CHKERRQ(ierr);
+    ierr = PetscMalloc1(dmmoab->numFields * npoints, &dof);CHKERRQ(ierr);
   }
 
-  if (dmmoab->bs > 1) { /* compute the DOF based on local blocking in the fields */
-    for (field=0; field<dmmoab->numFields; ++field) {
-      for (i=0; i<npoints; ++i)
-        dof[i*dmmoab->numFields+field] = dmmoab->lidmap[(PetscInt)points[i]]*dmmoab->numFields+field;
-    }
-  }
-  else {
-    /* assume all fields have equal distribution; i.e., all fields are either defined on vertices or elements and not on a mixture */
-    /* TODO: eliminate the limitation using PetscSection to manage DOFs */
-    for (field=0; field<dmmoab->numFields; ++field) {
-      offset = field*dmmoab->n;
-      for (i=0; i<npoints; ++i)
-        dof[i*dmmoab->numFields+field] = dmmoab->lidmap[(PetscInt)points[i]]+offset;
-    }
+  /* compute the DOF based on local blocking in the fields */
+  /* assume all fields have equal distribution; i.e., all fields are either defined on vertices or elements and not on a mixture */
+  /* TODO: eliminate the limitation using PetscSection to manage DOFs */
+  for (field = 0; field < dmmoab->numFields; ++field) {
+    offset = field * dmmoab->n;
+    for (i = 0; i < npoints; ++i)
+      dof[i * dmmoab->numFields + field] = (dmmoab->bs > 1 ? dmmoab->lidmap[dmmoab->mbiface->id_from_handle(points[i]) - dmmoab->seqstart] * dmmoab->numFields + field :
+                                            dmmoab->lidmap[dmmoab->mbiface->id_from_handle(points[i]) - dmmoab->seqstart] + offset);
   }
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMMoabGetDofsBlocked"
 /*@C
   DMMoabGetDofsBlocked - Gets the global degree-of-freedom for the first field (component) defined on an
   array of MOAB EntityHandles. It is useful when performing Blocked(Get/Set) methods in computation
@@ -550,33 +508,31 @@ PetscErrorCode DMMoabGetDofsLocal(DM dm,PetscInt npoints,const moab::EntityHandl
   Level: intermediate
 
 .keywords: discretization manager, get, global degrees of freedom
-  
+
 .seealso: DMMoabGetDofsLocal(), DMMoabGetDofs(), DMMoabGetDofsBlockedLocal()
 @*/
-PetscErrorCode DMMoabGetDofsBlocked(DM dm,PetscInt npoints,const moab::EntityHandle* points,PetscInt* dof)
+PetscErrorCode DMMoabGetDofsBlocked(DM dm, PetscInt npoints, const moab::EntityHandle* points, PetscInt* dof)
 {
   PetscInt        i;
   DM_Moab        *dmmoab;
   PetscErrorCode  ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
-  PetscValidPointer(points,2);
+  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
+  PetscValidPointer(points, 3);
   dmmoab = (DM_Moab*)(dm)->data;
 
   if (!dof) {
-    ierr = PetscMalloc1(npoints,&dof);CHKERRQ(ierr);
+    ierr = PetscMalloc1(npoints, &dof);CHKERRQ(ierr);
   }
 
-  for (i=0; i<npoints; ++i) {
-    dof[i]=dmmoab->gidmap[(PetscInt)points[i]];
+  for (i = 0; i < npoints; ++i) {
+    dof[i] = dmmoab->gidmap[(PetscInt)points[i] - dmmoab->seqstart];
   }
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMMoabGetDofsBlockedLocal"
 /*@C
   DMMoabGetDofsBlockedLocal - Gets the local degree-of-freedom for the first field (component) defined on an
   array of MOAB EntityHandles. It is useful when performing local Blocked(Get/Set) methods in computation
@@ -595,32 +551,30 @@ PetscErrorCode DMMoabGetDofsBlocked(DM dm,PetscInt npoints,const moab::EntityHan
   Level: intermediate
 
 .keywords: discretization manager, get, global degrees of freedom
-  
+
 .seealso: DMMoabGetDofsLocal(), DMMoabGetDofs(), DMMoabGetDofsBlockedLocal()
 @*/
-PetscErrorCode DMMoabGetDofsBlockedLocal(DM dm,PetscInt npoints,const moab::EntityHandle* points,PetscInt* dof)
+PetscErrorCode DMMoabGetDofsBlockedLocal(DM dm, PetscInt npoints, const moab::EntityHandle* points, PetscInt* dof)
 {
   PetscInt        i;
   DM_Moab        *dmmoab;
   PetscErrorCode  ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
-  PetscValidPointer(points,2);
+  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
+  PetscValidPointer(points, 3);
   dmmoab = (DM_Moab*)(dm)->data;
 
   if (!dof) {
-    ierr = PetscMalloc1(npoints,&dof);CHKERRQ(ierr);
+    ierr = PetscMalloc1(npoints, &dof);CHKERRQ(ierr);
   }
 
-  for (i=0; i<npoints; ++i)
-    dof[i] = dmmoab->lidmap[(PetscInt)points[i]];
+  for (i = 0; i < npoints; ++i)
+    dof[i] = dmmoab->lidmap[(PetscInt)points[i] - dmmoab->seqstart];
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMMoabGetVertexDofsBlocked"
 /*@C
   DMMoabGetVertexDofsBlocked - Gets the global degree-of-freedom for the first field (component) defined on an
   array of locally owned MOAB mesh vertices. It's utility is when performing Finite-Difference type calculations
@@ -637,15 +591,15 @@ PetscErrorCode DMMoabGetDofsBlockedLocal(DM dm,PetscInt npoints,const moab::Enti
   Level: intermediate
 
 .keywords: discretization manager, get, blocked degrees of freedom
-  
+
 .seealso: DMMoabGetVertexDofsBlockedLocal(), DMMoabGetDofsBlocked(), DMMoabGetDofsBlockedLocal()
 @*/
-PetscErrorCode DMMoabGetVertexDofsBlocked(DM dm,PetscInt** dof)
+PetscErrorCode DMMoabGetVertexDofsBlocked(DM dm, PetscInt** dof)
 {
   DM_Moab        *dmmoab;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
   dmmoab = (DM_Moab*)(dm)->data;
 
   *dof = dmmoab->gidmap;
@@ -653,8 +607,6 @@ PetscErrorCode DMMoabGetVertexDofsBlocked(DM dm,PetscInt** dof)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMMoabGetVertexDofsBlockedLocal"
 /*@C
   DMMoabGetVertexDofsBlockedLocal - Gets the local degree-of-freedom for the first field (component) defined on an
   array of locally owned MOAB mesh vertices. It's utility is when performing Finite-Difference type calculations
@@ -671,16 +623,16 @@ PetscErrorCode DMMoabGetVertexDofsBlocked(DM dm,PetscInt** dof)
   Level: intermediate
 
 .keywords: discretization manager, get, blocked degrees of freedom
-  
+
 .seealso: DMMoabGetVertexDofsBlocked(), DMMoabGetDofsBlocked(), DMMoabGetDofsBlockedLocal()
 @*/
-PetscErrorCode DMMoabGetVertexDofsBlockedLocal(DM dm,PetscInt** dof)
+PetscErrorCode DMMoabGetVertexDofsBlockedLocal(DM dm, PetscInt** dof)
 {
   DM_Moab        *dmmoab;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
-  PetscValidPointer(dof,2);
+  PetscValidHeaderSpecific(dm, DM_CLASSID, 1);
+  PetscValidPointer(dof, 2);
   dmmoab = (DM_Moab*)(dm)->data;
 
   *dof = dmmoab->lidmap;
