@@ -12,8 +12,6 @@ typedef struct {
   char tagname[PETSC_MAX_PATH_LEN];
 } AppCtx;
 
-#undef __FUNCT__
-#define __FUNCT__ "ProcessOptions"
 PetscErrorCode ProcessOptions(MPI_Comm comm, AppCtx *options)
 {
   PetscErrorCode ierr;
@@ -25,21 +23,18 @@ PetscErrorCode ProcessOptions(MPI_Comm comm, AppCtx *options)
   options->dim = -1;
 
   ierr = PetscOptionsBegin(comm, "", "MOAB example options", "DMMOAB");CHKERRQ(ierr);
-  ierr = PetscOptionsInt("-dim", "The topological mesh dimension", "ex1.c", options->dim, &options->dim, NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsString("-filename", "The file containing the mesh", "ex1.c", options->filename, options->filename, sizeof(options->filename), NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsString("-tagname", "The tag name from which to create a vector", "ex1.c", options->tagname, options->tagname, sizeof(options->tagname), &flg);CHKERRQ(ierr);
+  ierr = PetscOptionsInt("-dim", "The topological mesh dimension", "ex1.cxx", options->dim, &options->dim, NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsString("-filename", "The file containing the mesh", "ex1.cxx", options->filename, options->filename, sizeof(options->filename), NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsString("-tagname", "The tag name from which to create a vector", "ex1.cxx", options->tagname, options->tagname, sizeof(options->tagname), &flg);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();
 
   ierr = PetscLogEventRegister("CreateMesh",          DM_CLASSID,   &options->createMeshEvent);CHKERRQ(ierr);
   PetscFunctionReturn(0);
-};
+}
 
-#undef __FUNCT__
-#define __FUNCT__ "CreateMesh"
 PetscErrorCode CreateMesh(MPI_Comm comm, AppCtx *user, DM *dm)
 {
   moab::Interface *iface=NULL;
-  moab::ParallelComm *pcomm=NULL;
   moab::Tag tag=NULL;
   moab::Tag ltog_tag=NULL;
   moab::Range range;
@@ -49,7 +44,7 @@ PetscErrorCode CreateMesh(MPI_Comm comm, AppCtx *user, DM *dm)
 
   PetscFunctionBegin;
   ierr = PetscLogEventBegin(user->createMeshEvent,0,0,0,0);CHKERRQ(ierr);
-  ierr = DMMoabCreateMoab(comm, iface, pcomm, &ltog_tag, &range, dm);CHKERRQ(ierr);
+  ierr = DMMoabCreateMoab(comm, iface, &ltog_tag, &range, dm);CHKERRQ(ierr);
   std::cout << "Created DMMoab using DMMoabCreateMoab." << std::endl;
   ierr = DMMoabGetInterface(*dm, &iface);CHKERRQ(ierr);
 
@@ -102,26 +97,23 @@ PetscErrorCode CreateMesh(MPI_Comm comm, AppCtx *user, DM *dm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc, char **argv)
 {
-  AppCtx         user;                 /* user-defined work context */
-  moab::ErrorCode merr;
-  PetscErrorCode ierr;
-  Vec            vec;
+  AppCtx            user;                 /* user-defined work context */
+  moab::ErrorCode   merr;
+  PetscErrorCode    ierr;
+  Vec               vec;
   moab::Interface*  mbImpl=NULL;
   moab::Tag         datatag=NULL;
 
-  ierr = PetscInitialize(&argc, &argv, NULL, help);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc, &argv, NULL,help);if (ierr) return ierr;
   ierr = ProcessOptions(PETSC_COMM_WORLD, &user);CHKERRQ(ierr);
 
   ierr = CreateMesh(PETSC_COMM_WORLD, &user, &user.dm);CHKERRQ(ierr); /* create the MOAB dm and the mesh */
 
   ierr = DMMoabGetInterface(user.dm, &mbImpl);CHKERRQ(ierr);
   merr = mbImpl->tag_get_handle(user.tagname, datatag);MBERRNM(merr);
-  ierr = DMMoabCreateVector(user.dm, datatag, NULL, PETSC_TRUE, PETSC_FALSE,
-                              &vec);CHKERRQ(ierr); /* create a vec from user-input tag */
+  ierr = DMMoabCreateVector(user.dm, datatag, NULL, PETSC_TRUE, PETSC_FALSE,&vec);CHKERRQ(ierr); /* create a vec from user-input tag */
 
   std::cout << "Created VecMoab from existing tag." << std::endl;
   ierr = VecDestroy(&vec);CHKERRQ(ierr);
@@ -129,6 +121,6 @@ int main(int argc, char **argv)
   ierr = DMDestroy(&user.dm);CHKERRQ(ierr);
   std::cout << "Destroyed DMMoab." << std::endl;
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
