@@ -8,12 +8,13 @@ typedef struct {
   PetscInt  numRefinements; /* The number of refinement steps */
   PetscBool cellHybrid;     /* Use a hybrid mesh */
   PetscBool cellSimplex;    /* Use simplices or hexes */
+  PetscBool testPartition;  /* Use a fixed partitioning for testing */
   PetscInt  testNum;        /* The particular mesh to test */
+  PetscBool simplex2tensor; /* Refine simplicials in hexes */
   PetscBool uninterpolate;  /* Uninterpolate the mesh at the end */
+  PetscBool reinterpolate;  /* Reinterpolate the mesh at the end */
 } AppCtx;
 
-#undef __FUNCT__
-#define __FUNCT__ "ProcessOptions"
 PetscErrorCode ProcessOptions(MPI_Comm comm, AppCtx *options)
 {
   PetscErrorCode ierr;
@@ -24,23 +25,27 @@ PetscErrorCode ProcessOptions(MPI_Comm comm, AppCtx *options)
   options->numRefinements = 0;
   options->cellHybrid     = PETSC_TRUE;
   options->cellSimplex    = PETSC_TRUE;
+  options->testPartition  = PETSC_TRUE;
   options->testNum        = 0;
+  options->simplex2tensor = PETSC_FALSE;
   options->uninterpolate  = PETSC_FALSE;
+  options->reinterpolate  = PETSC_FALSE;
 
   ierr = PetscOptionsBegin(comm, "", "Meshing Problem Options", "DMPLEX");CHKERRQ(ierr);
   ierr = PetscOptionsInt("-debug", "The debugging level", "ex4.c", options->debug, &options->debug, NULL);CHKERRQ(ierr);
   ierr = PetscOptionsInt("-dim", "The topological mesh dimension", "ex4.c", options->dim, &options->dim, NULL);CHKERRQ(ierr);
   ierr = PetscOptionsInt("-num_refinements", "The number of refinement steps", "ex4.c", options->numRefinements, &options->numRefinements, NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsBool("-cell_hybrid", "Use a hyrbid mesh", "ex4.c", options->cellHybrid, &options->cellHybrid, NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-cell_hybrid", "Use a hybrid mesh", "ex4.c", options->cellHybrid, &options->cellHybrid, NULL);CHKERRQ(ierr);
   ierr = PetscOptionsBool("-cell_simplex", "Use simplices if true, otherwise hexes", "ex4.c", options->cellSimplex, &options->cellSimplex, NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-test_partition", "Use a fixed partition for testing", "ex4.c", options->testPartition, &options->testPartition, NULL);CHKERRQ(ierr);
   ierr = PetscOptionsInt("-test_num", "The particular mesh to test", "ex4.c", options->testNum, &options->testNum, NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-simplex2tensor", "Refine simplicial cells in tensor product cells", "ex4.c", options->simplex2tensor, &options->simplex2tensor, NULL);CHKERRQ(ierr);
   ierr = PetscOptionsBool("-uninterpolate", "Uninterpolate the mesh at the end", "ex4.c", options->uninterpolate, &options->uninterpolate, NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-reinterpolate", "Reinterpolate the mesh at the end", "ex4.c", options->reinterpolate, &options->reinterpolate, NULL);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();
   PetscFunctionReturn(0);
-};
+}
 
-#undef __FUNCT__
-#define __FUNCT__ "CreateSimplex_1D"
 /* Two segments
 
   2-------0-------3-------1-------4
@@ -67,7 +72,7 @@ PetscErrorCode CreateSimplex_1D(MPI_Comm comm, DM *dm)
 
     ierr = DMPlexCreateFromDAG(*dm, depth, numPoints, coneSize, cones, coneOrientations, vertexCoords);CHKERRQ(ierr);
   } else {
-    PetscInt numPoints[2] = {0, 0, 0};
+    PetscInt numPoints[2] = {0, 0};
 
     ierr = DMPlexCreateFromDAG(*dm, depth, numPoints, NULL, NULL, NULL, NULL);CHKERRQ(ierr);
   }
@@ -75,8 +80,6 @@ PetscErrorCode CreateSimplex_1D(MPI_Comm comm, DM *dm)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "CreateSimplex_2D"
 /* Two triangles
         4
       / | \
@@ -131,8 +134,6 @@ PetscErrorCode CreateSimplex_2D(MPI_Comm comm, DM *dm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CreateSimplexHybrid_2D"
 /* Two triangles separated by a zero-volume cell with 4 vertices/2 edges
         5--16--8
       / |      | \
@@ -146,7 +147,7 @@ PetscErrorCode CreateSimplex_2D(MPI_Comm comm, DM *dm)
 */
 PetscErrorCode CreateSimplexHybrid_2D(MPI_Comm comm, PetscInt testNum, DM *dm)
 {
-  DM             idm = NULL, hdm = NULL;
+  DM             idm, hdm = NULL;
   DMLabel        faultLabel, hybridLabel;
   PetscInt       p;
   PetscMPIInt    rank;
@@ -166,7 +167,7 @@ PetscErrorCode CreateSimplexHybrid_2D(MPI_Comm comm, PetscInt testNum, DM *dm)
       PetscInt    faultPoints[2]      = {3, 4};
 
       ierr = DMPlexCreateFromDAG(*dm, 1, numPoints, coneSize, cones, coneOrientations, vertexCoords);CHKERRQ(ierr);
-      for(p = 0; p < 2; ++p) {ierr = DMPlexSetLabelValue(*dm, "fault", faultPoints[p], 1);CHKERRQ(ierr);}
+      for (p = 0; p < 2; ++p) {ierr = DMSetLabelValue(*dm, "fault", faultPoints[p], 1);CHKERRQ(ierr);}
     }
     break;
     case 1:
@@ -179,18 +180,17 @@ PetscErrorCode CreateSimplexHybrid_2D(MPI_Comm comm, PetscInt testNum, DM *dm)
       PetscInt    faultPoints[3]       = {5, 6, 7};
 
       ierr = DMPlexCreateFromDAG(*dm, 1, numPoints, coneSize, cones, coneOrientations, vertexCoords);CHKERRQ(ierr);
-      for(p = 0; p < 3; ++p) {ierr = DMPlexSetLabelValue(*dm, "fault", faultPoints[p], 1);CHKERRQ(ierr);}
+      for (p = 0; p < 3; ++p) {ierr = DMSetLabelValue(*dm, "fault", faultPoints[p], 1);CHKERRQ(ierr);}
     }
     break;
     default: SETERRQ1(comm, PETSC_ERR_ARG_OUTOFRANGE, "No test mesh %d", testNum);
     }
     ierr = DMPlexCheckSymmetry(*dm);CHKERRQ(ierr);
     ierr = DMPlexInterpolate(*dm, &idm);CHKERRQ(ierr);
-    ierr = DMPlexCopyCoordinates(*dm, idm);CHKERRQ(ierr);
     ierr = DMViewFromOptions(idm, NULL, "-in_dm_view");CHKERRQ(ierr);
     ierr = DMPlexCheckSymmetry(idm);CHKERRQ(ierr);
-    ierr = DMPlexGetLabel(*dm, "fault", &faultLabel);CHKERRQ(ierr);
-    ierr = DMPlexCreateHybridMesh(idm, faultLabel, &hybridLabel, &hdm);CHKERRQ(ierr);
+    ierr = DMGetLabel(*dm, "fault", &faultLabel);CHKERRQ(ierr);
+    ierr = DMPlexCreateHybridMesh(idm, faultLabel, NULL, &hybridLabel, NULL, NULL, &hdm);CHKERRQ(ierr);
     ierr = DMLabelDestroy(&hybridLabel);CHKERRQ(ierr);
     ierr = DMDestroy(&idm);CHKERRQ(ierr);
     ierr = DMDestroy(dm);CHKERRQ(ierr);
@@ -200,9 +200,8 @@ PetscErrorCode CreateSimplexHybrid_2D(MPI_Comm comm, PetscInt testNum, DM *dm)
 
     ierr = DMPlexCreateFromDAG(*dm, 1, numPoints, NULL, NULL, NULL, NULL);CHKERRQ(ierr);
     ierr = DMPlexInterpolate(*dm, &idm);CHKERRQ(ierr);
-    ierr = DMPlexCopyCoordinates(*dm, idm);CHKERRQ(ierr);
     ierr = DMViewFromOptions(idm, NULL, "-in_dm_view");CHKERRQ(ierr);
-    ierr = DMPlexCreateHybridMesh(idm, NULL, NULL, &hdm);CHKERRQ(ierr);
+    ierr = DMPlexCreateHybridMesh(idm, NULL, NULL, NULL, NULL, NULL, &hdm);CHKERRQ(ierr);
     ierr = DMDestroy(&idm);CHKERRQ(ierr);
     ierr = DMDestroy(dm);CHKERRQ(ierr);
     *dm  = hdm;
@@ -210,8 +209,6 @@ PetscErrorCode CreateSimplexHybrid_2D(MPI_Comm comm, PetscInt testNum, DM *dm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CreateTensorProduct_2D"
 /* Two quadrilaterals
 
   5----10-----4----14-----7
@@ -248,11 +245,9 @@ PetscErrorCode CreateTensorProduct_2D(MPI_Comm comm, PetscInt testNum, DM *dm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CreateTensorProductHybrid_2D"
 PetscErrorCode CreateTensorProductHybrid_2D(MPI_Comm comm, PetscInt testNum, DM *dm)
 {
-  DM             idm = NULL, hdm = NULL;
+  DM             idm, hdm = NULL;
   DMLabel        faultLabel, hybridLabel;
   PetscInt       p;
   PetscMPIInt    rank;
@@ -269,23 +264,21 @@ PetscErrorCode CreateTensorProductHybrid_2D(MPI_Comm comm, PetscInt testNum, DM
     PetscInt    faultPoints[2]      = {3, 4};
 
     ierr = DMPlexCreateFromDAG(*dm, 1, numPoints, coneSize, cones, coneOrientations, vertexCoords);CHKERRQ(ierr);
-    for(p = 0; p < 2; ++p) {ierr = DMPlexSetLabelValue(*dm, "fault", faultPoints[p], 1);CHKERRQ(ierr);}
+    for(p = 0; p < 2; ++p) {ierr = DMSetLabelValue(*dm, "fault", faultPoints[p], 1);CHKERRQ(ierr);}
     ierr = DMPlexCheckSymmetry(*dm);CHKERRQ(ierr);
     ierr = DMPlexInterpolate(*dm, &idm);CHKERRQ(ierr);
-    ierr = DMPlexCopyCoordinates(*dm, idm);CHKERRQ(ierr);
     ierr = DMViewFromOptions(idm, NULL, "-in_dm_view");CHKERRQ(ierr);
     ierr = DMPlexCheckSymmetry(idm);CHKERRQ(ierr);
-    ierr = DMPlexGetLabel(*dm, "fault", &faultLabel);CHKERRQ(ierr);
-    ierr = DMPlexCreateHybridMesh(idm, faultLabel, &hybridLabel, &hdm);CHKERRQ(ierr);
+    ierr = DMGetLabel(*dm, "fault", &faultLabel);CHKERRQ(ierr);
+    ierr = DMPlexCreateHybridMesh(idm, faultLabel, NULL, &hybridLabel, NULL, NULL, &hdm);CHKERRQ(ierr);
     ierr = DMLabelDestroy(&hybridLabel);CHKERRQ(ierr);
   } else {
     PetscInt numPoints[3] = {0, 0, 0};
 
     ierr = DMPlexCreateFromDAG(*dm, 1, numPoints, NULL, NULL, NULL, NULL);CHKERRQ(ierr);
     ierr = DMPlexInterpolate(*dm, &idm);CHKERRQ(ierr);
-    ierr = DMPlexCopyCoordinates(*dm, idm);CHKERRQ(ierr);
     ierr = DMViewFromOptions(idm, NULL, "-in_dm_view");CHKERRQ(ierr);
-    ierr = DMPlexCreateHybridMesh(idm, NULL, NULL, &hdm);CHKERRQ(ierr);
+    ierr = DMPlexCreateHybridMesh(idm, NULL, NULL, NULL, NULL, NULL, &hdm);CHKERRQ(ierr);
   }
   ierr = DMDestroy(&idm);CHKERRQ(ierr);
   ierr = DMDestroy(dm);CHKERRQ(ierr);
@@ -293,8 +286,6 @@ PetscErrorCode CreateTensorProductHybrid_2D(MPI_Comm comm, PetscInt testNum, DM
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CreateSimplex_3D"
 /* Two tetrahedrons
 
  cell   5          5______    cell
@@ -309,7 +300,7 @@ PetscErrorCode CreateTensorProductHybrid_2D(MPI_Comm comm, PetscInt testNum, DM
 */
 PetscErrorCode CreateSimplex_3D(MPI_Comm comm, PetscInt testNum, DM *dm)
 {
-  DM             idm = NULL;
+  DM             idm;
   PetscInt       depth = 3;
   PetscMPIInt    rank;
   PetscErrorCode ierr;
@@ -340,7 +331,6 @@ PetscErrorCode CreateSimplex_3D(MPI_Comm comm, PetscInt testNum, DM *dm)
       depth = 1;
       ierr = DMPlexCreateFromDAG(*dm, depth, numPoints, coneSize, cones, coneOrientations, vertexCoords);CHKERRQ(ierr);
       ierr = DMPlexInterpolate(*dm, &idm);CHKERRQ(ierr);
-      ierr = DMPlexCopyCoordinates(*dm, idm);CHKERRQ(ierr);
       ierr = DMViewFromOptions(idm, NULL, "-in_dm_view");CHKERRQ(ierr);
       ierr = DMDestroy(dm);CHKERRQ(ierr);
       *dm  = idm;
@@ -357,7 +347,6 @@ PetscErrorCode CreateSimplex_3D(MPI_Comm comm, PetscInt testNum, DM *dm)
       depth = 1;
       ierr = DMPlexCreateFromDAG(*dm, depth, numPoints, coneSize, cones, coneOrientations, vertexCoords);CHKERRQ(ierr);
       ierr = DMPlexInterpolate(*dm, &idm);CHKERRQ(ierr);
-      ierr = DMPlexCopyCoordinates(*dm, idm);CHKERRQ(ierr);
       ierr = DMViewFromOptions(idm, NULL, "-in_dm_view");CHKERRQ(ierr);
       ierr = DMDestroy(dm);CHKERRQ(ierr);
       *dm  = idm;
@@ -372,7 +361,6 @@ PetscErrorCode CreateSimplex_3D(MPI_Comm comm, PetscInt testNum, DM *dm)
     switch (testNum) {
     case 1:
       ierr = DMPlexInterpolate(*dm, &idm);CHKERRQ(ierr);
-      ierr = DMPlexCopyCoordinates(*dm, idm);CHKERRQ(ierr);
       ierr = DMViewFromOptions(idm, NULL, "-in_dm_view");CHKERRQ(ierr);
       ierr = DMDestroy(dm);CHKERRQ(ierr);
       *dm  = idm;
@@ -382,8 +370,6 @@ PetscErrorCode CreateSimplex_3D(MPI_Comm comm, PetscInt testNum, DM *dm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CreateSimplexHybrid_3D"
 /* Two tetrahedrons separated by a zero-volume cell with 6 vertices
 
  cell   6 ___33___10______    cell
@@ -399,7 +385,7 @@ PetscErrorCode CreateSimplex_3D(MPI_Comm comm, PetscInt testNum, DM *dm)
 */
 PetscErrorCode CreateSimplexHybrid_3D(MPI_Comm comm, PetscInt testNum, DM *dm)
 {
-  DM             idm = NULL, hdm = NULL;
+  DM             idm, hdm = NULL;
   DMLabel        faultLabel, hybridLabel;
   PetscInt       p;
   PetscMPIInt    rank;
@@ -419,7 +405,7 @@ PetscErrorCode CreateSimplexHybrid_3D(MPI_Comm comm, PetscInt testNum, DM *dm)
       PetscInt    faultPoints[3]      = {3, 4, 5};
 
       ierr = DMPlexCreateFromDAG(*dm, 1, numPoints, coneSize, cones, coneOrientations, vertexCoords);CHKERRQ(ierr);
-      for(p = 0; p < 3; ++p) {ierr = DMPlexSetLabelValue(*dm, "fault", faultPoints[p], 1);CHKERRQ(ierr);}
+      for(p = 0; p < 3; ++p) {ierr = DMSetLabelValue(*dm, "fault", faultPoints[p], 1);CHKERRQ(ierr);}
     }
     break;
     case 1:
@@ -437,18 +423,17 @@ PetscErrorCode CreateSimplexHybrid_3D(MPI_Comm comm, PetscInt testNum, DM *dm)
       PetscInt    faultPoints[3]       = {9, 10, 11};
 
       ierr = DMPlexCreateFromDAG(*dm, 1, numPoints, coneSize, cones, coneOrientations, vertexCoords);CHKERRQ(ierr);
-      for(p = 0; p < 3; ++p) {ierr = DMPlexSetLabelValue(*dm, "fault", faultPoints[p], 1);CHKERRQ(ierr);}
+      for(p = 0; p < 3; ++p) {ierr = DMSetLabelValue(*dm, "fault", faultPoints[p], 1);CHKERRQ(ierr);}
     }
     break;
     default: SETERRQ1(comm, PETSC_ERR_ARG_OUTOFRANGE, "No test mesh %d", testNum);
     }
     ierr = DMPlexCheckSymmetry(*dm);CHKERRQ(ierr);
     ierr = DMPlexInterpolate(*dm, &idm);CHKERRQ(ierr);
-    ierr = DMPlexCopyCoordinates(*dm, idm);CHKERRQ(ierr);
     ierr = DMViewFromOptions(idm, NULL, "-in_dm_view");CHKERRQ(ierr);
     ierr = DMPlexCheckSymmetry(idm);CHKERRQ(ierr);
-    ierr = DMPlexGetLabel(*dm, "fault", &faultLabel);CHKERRQ(ierr);
-    ierr = DMPlexCreateHybridMesh(idm, faultLabel, &hybridLabel, &hdm);CHKERRQ(ierr);
+    ierr = DMGetLabel(*dm, "fault", &faultLabel);CHKERRQ(ierr);
+    ierr = DMPlexCreateHybridMesh(idm, faultLabel, NULL, &hybridLabel, NULL, NULL, &hdm);CHKERRQ(ierr);
     ierr = DMLabelDestroy(&hybridLabel);CHKERRQ(ierr);
     ierr = DMDestroy(&idm);CHKERRQ(ierr);
     ierr = DMDestroy(dm);CHKERRQ(ierr);
@@ -458,9 +443,8 @@ PetscErrorCode CreateSimplexHybrid_3D(MPI_Comm comm, PetscInt testNum, DM *dm)
 
     ierr = DMPlexCreateFromDAG(*dm, 1, numPoints, NULL, NULL, NULL, NULL);CHKERRQ(ierr);
     ierr = DMPlexInterpolate(*dm, &idm);CHKERRQ(ierr);
-    ierr = DMPlexCopyCoordinates(*dm, idm);CHKERRQ(ierr);
     ierr = DMViewFromOptions(idm, NULL, "-in_dm_view");CHKERRQ(ierr);
-    ierr = DMPlexCreateHybridMesh(idm, NULL, NULL, &hdm);CHKERRQ(ierr);
+    ierr = DMPlexCreateHybridMesh(idm, NULL, NULL, NULL, NULL, NULL, &hdm);CHKERRQ(ierr);
     ierr = DMDestroy(&idm);CHKERRQ(ierr);
     ierr = DMDestroy(dm);CHKERRQ(ierr);
     *dm  = hdm;
@@ -468,11 +452,9 @@ PetscErrorCode CreateSimplexHybrid_3D(MPI_Comm comm, PetscInt testNum, DM *dm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CreateTensorProduct_3D"
 PetscErrorCode CreateTensorProduct_3D(MPI_Comm comm, PetscInt testNum, DM *dm)
 {
-  DM             idm = NULL;
+  DM             idm;
   PetscMPIInt    rank;
   PetscErrorCode ierr;
 
@@ -513,18 +495,15 @@ PetscErrorCode CreateTensorProduct_3D(MPI_Comm comm, PetscInt testNum, DM *dm)
     ierr = DMPlexCreateFromDAG(*dm, 1, numPoints, NULL, NULL, NULL, NULL);CHKERRQ(ierr);
   }
   ierr = DMPlexInterpolate(*dm, &idm);CHKERRQ(ierr);
-  ierr = DMPlexCopyCoordinates(*dm, idm);CHKERRQ(ierr);
   ierr = DMViewFromOptions(idm, NULL, "-in_dm_view");CHKERRQ(ierr);
   ierr = DMDestroy(dm);CHKERRQ(ierr);
   *dm  = idm;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CreateTensorProductHybrid_3D"
 PetscErrorCode CreateTensorProductHybrid_3D(MPI_Comm comm, PetscInt testNum, DM *dm)
 {
-  DM             idm = NULL, hdm = NULL;
+  DM             idm, hdm = NULL;
   DMLabel        faultLabel;
   PetscInt       p;
   PetscMPIInt    rank;
@@ -546,7 +525,7 @@ PetscErrorCode CreateTensorProductHybrid_3D(MPI_Comm comm, PetscInt testNum, DM
       PetscInt    faultPoints[4]       = {2, 3, 5, 6};
 
       ierr = DMPlexCreateFromDAG(*dm, 1, numPoints, coneSize, cones, coneOrientations, vertexCoords);CHKERRQ(ierr);
-      for(p = 0; p < 4; ++p) {ierr = DMPlexSetLabelValue(*dm, "fault", faultPoints[p], 1);CHKERRQ(ierr);}
+      for(p = 0; p < 4; ++p) {ierr = DMSetLabelValue(*dm, "fault", faultPoints[p], 1);CHKERRQ(ierr);}
     }
     break;
     case 1:
@@ -569,18 +548,17 @@ PetscErrorCode CreateTensorProductHybrid_3D(MPI_Comm comm, PetscInt testNum, DM
       PetscInt    faultPoints[6]       = {20, 21, 22, 23, 24, 25};
 
       ierr = DMPlexCreateFromDAG(*dm, 1, numPoints, coneSize, cones, coneOrientations, vertexCoords);CHKERRQ(ierr);
-      for(p = 0; p < 6; ++p) {ierr = DMPlexSetLabelValue(*dm, "fault", faultPoints[p], 1);CHKERRQ(ierr);}
+      for(p = 0; p < 6; ++p) {ierr = DMSetLabelValue(*dm, "fault", faultPoints[p], 1);CHKERRQ(ierr);}
     }
     break;
     default: SETERRQ1(comm, PETSC_ERR_ARG_OUTOFRANGE, "No test mesh %d", testNum);
     }
     ierr = DMPlexCheckSymmetry(*dm);CHKERRQ(ierr);
     ierr = DMPlexInterpolate(*dm, &idm);CHKERRQ(ierr);
-    ierr = DMPlexCopyCoordinates(*dm, idm);CHKERRQ(ierr);
     ierr = DMViewFromOptions(idm, NULL, "-in_dm_view");CHKERRQ(ierr);
     ierr = DMPlexCheckSymmetry(idm);CHKERRQ(ierr);
-    ierr = DMPlexGetLabel(*dm, "fault", &faultLabel);CHKERRQ(ierr);
-    ierr = DMPlexCreateHybridMesh(idm, faultLabel, NULL, &hdm);CHKERRQ(ierr);
+    ierr = DMGetLabel(*dm, "fault", &faultLabel);CHKERRQ(ierr);
+    ierr = DMPlexCreateHybridMesh(idm, faultLabel, NULL, NULL, NULL, NULL, &hdm);CHKERRQ(ierr);
     ierr = DMDestroy(&idm);CHKERRQ(ierr);
     ierr = DMDestroy(dm);CHKERRQ(ierr);
     *dm  = hdm;
@@ -589,9 +567,8 @@ PetscErrorCode CreateTensorProductHybrid_3D(MPI_Comm comm, PetscInt testNum, DM
 
     ierr = DMPlexCreateFromDAG(*dm, 1, numPoints, NULL, NULL, NULL, NULL);CHKERRQ(ierr);
     ierr = DMPlexInterpolate(*dm, &idm);CHKERRQ(ierr);
-    ierr = DMPlexCopyCoordinates(*dm, idm);CHKERRQ(ierr);
     ierr = DMViewFromOptions(idm, NULL, "-in_dm_view");CHKERRQ(ierr);
-    ierr = DMPlexCreateHybridMesh(idm, NULL, NULL, &hdm);CHKERRQ(ierr);
+    ierr = DMPlexCreateHybridMesh(idm, NULL, NULL, NULL, NULL, NULL, &hdm);CHKERRQ(ierr);
     ierr = DMDestroy(&idm);CHKERRQ(ierr);
     ierr = DMDestroy(dm);CHKERRQ(ierr);
     *dm  = hdm;
@@ -599,19 +576,18 @@ PetscErrorCode CreateTensorProductHybrid_3D(MPI_Comm comm, PetscInt testNum, DM
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CreateMesh"
 PetscErrorCode CreateMesh(MPI_Comm comm, AppCtx *user, DM *dm)
 {
   PetscInt       dim            = user->dim;
   PetscInt       numRefinements = user->numRefinements;
   PetscBool      cellHybrid     = user->cellHybrid;
   PetscBool      cellSimplex    = user->cellSimplex;
-  PetscMPIInt    rank;
+  PetscMPIInt    rank, size;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   ierr = MPI_Comm_rank(comm, &rank);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(comm, &size);CHKERRQ(ierr);
   ierr = DMCreate(comm, dm);CHKERRQ(ierr);
   ierr = DMSetType(*dm, DMPLEX);CHKERRQ(ierr);
   ierr = DMSetDimension(*dm, dim);CHKERRQ(ierr);
@@ -653,6 +629,142 @@ PetscErrorCode CreateMesh(MPI_Comm comm, AppCtx *user, DM *dm)
   default:
     SETERRQ1(comm, PETSC_ERR_ARG_OUTOFRANGE, "Cannot make meshes for dimension %d", dim);
   }
+  if (user->testPartition && size > 1) {
+    PetscPartitioner part;
+    PetscInt  *sizes  = NULL;
+    PetscInt  *points = NULL;
+
+    if (!rank) {
+      if (dim == 2 && cellSimplex && !cellHybrid && size == 2) {
+        switch (user->testNum) {
+        case 0: {
+          PetscInt triSizes_p2[2]  = {1, 1};
+          PetscInt triPoints_p2[2] = {0, 1};
+
+          ierr = PetscMalloc2(2, &sizes, 2, &points);CHKERRQ(ierr);
+          ierr = PetscMemcpy(sizes,  triSizes_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);
+          ierr = PetscMemcpy(points, triPoints_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);break;}
+        default:
+          SETERRQ1(PETSC_COMM_WORLD, PETSC_ERR_ARG_WRONG, "Could not find matching test number %d for triangular mesh on 2 procs", user->testNum);
+        }
+      } else if (dim == 2 && cellSimplex && cellHybrid && size == 2) {
+        switch (user->testNum) {
+        case 0: {
+          PetscInt triSizes_p2[2]  = {1, 2};
+          PetscInt triPoints_p2[3] = {0, 1, 2};
+
+          ierr = PetscMalloc2(2, &sizes, 3, &points);CHKERRQ(ierr);
+          ierr = PetscMemcpy(sizes,  triSizes_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);
+          ierr = PetscMemcpy(points, triPoints_p2, 3 * sizeof(PetscInt));CHKERRQ(ierr);break;}
+        default:
+          SETERRQ1(PETSC_COMM_WORLD, PETSC_ERR_ARG_WRONG, "Could not find matching test number %d for triangular hybrid mesh on 2 procs", user->testNum);
+        }
+      } else if (dim == 2 && !cellSimplex && !cellHybrid && size == 2) {
+        switch (user->testNum) {
+        case 0: {
+          PetscInt quadSizes_p2[2]  = {1, 1};
+          PetscInt quadPoints_p2[2] = {0, 1};
+
+          ierr = PetscMalloc2(2, &sizes, 2, &points);CHKERRQ(ierr);
+          ierr = PetscMemcpy(sizes,  quadSizes_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);
+          ierr = PetscMemcpy(points, quadPoints_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);break;}
+        default:
+          SETERRQ1(PETSC_COMM_WORLD, PETSC_ERR_ARG_WRONG, "Could not find matching test number %d for quadrilateral mesh on 2 procs", user->testNum);
+        }
+      } else if (dim == 2 && !cellSimplex && cellHybrid && size == 2) {
+        switch (user->testNum) {
+        case 0: {
+          PetscInt quadSizes_p2[2]  = {1, 2};
+          PetscInt quadPoints_p2[3] = {0, 1, 2};
+
+          ierr = PetscMalloc2(2, &sizes, 3, &points);CHKERRQ(ierr);
+          ierr = PetscMemcpy(sizes,  quadSizes_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);
+          ierr = PetscMemcpy(points, quadPoints_p2, 3 * sizeof(PetscInt));CHKERRQ(ierr);break;}
+        default:
+          SETERRQ1(PETSC_COMM_WORLD, PETSC_ERR_ARG_WRONG, "Could not find matching test number %d for quadrilateral hybrid mesh on 2 procs", user->testNum);
+        }
+      } else if (dim == 3 && cellSimplex && !cellHybrid && size == 2) {
+        switch (user->testNum) {
+        case 0: {
+          PetscInt tetSizes_p2[2]  = {1, 1};
+          PetscInt tetPoints_p2[2] = {0, 1};
+
+          ierr = PetscMalloc2(2, &sizes, 2, &points);CHKERRQ(ierr);
+          ierr = PetscMemcpy(sizes,  tetSizes_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);
+          ierr = PetscMemcpy(points, tetPoints_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);break;}
+        case 1: {
+          PetscInt tetSizes_p2[2]  = {1, 1};
+          PetscInt tetPoints_p2[2] = {0, 1};
+
+          ierr = PetscMalloc2(2, &sizes, 2, &points);CHKERRQ(ierr);
+          ierr = PetscMemcpy(sizes,  tetSizes_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);
+          ierr = PetscMemcpy(points, tetPoints_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);break;}
+        default:
+          SETERRQ1(PETSC_COMM_WORLD, PETSC_ERR_ARG_WRONG, "Could not find matching test number %d for tetrahedral mesh on 2 procs", user->testNum);
+        }
+      } else if (dim == 3 && cellSimplex && cellHybrid && size == 2) {
+        switch (user->testNum) {
+        case 0: {
+          PetscInt tetSizes_p2[2]  = {1, 2};
+          PetscInt tetPoints_p2[3] = {0, 1, 2};
+
+          ierr = PetscMalloc2(2, &sizes, 3, &points);CHKERRQ(ierr);
+          ierr = PetscMemcpy(sizes,  tetSizes_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);
+          ierr = PetscMemcpy(points, tetPoints_p2, 3 * sizeof(PetscInt));CHKERRQ(ierr);break;}
+        case 1: {
+          PetscInt tetSizes_p2[2]  = {3, 4};
+          PetscInt tetPoints_p2[7] = {0, 3, 5, 1, 2, 4, 6};
+
+          ierr = PetscMalloc2(2, &sizes, 7, &points);CHKERRQ(ierr);
+          ierr = PetscMemcpy(sizes,  tetSizes_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);
+          ierr = PetscMemcpy(points, tetPoints_p2, 7 * sizeof(PetscInt));CHKERRQ(ierr);break;}
+        default:
+          SETERRQ1(PETSC_COMM_WORLD, PETSC_ERR_ARG_WRONG, "Could not find matching test number %d for tetrahedral hybrid mesh on 2 procs", user->testNum);
+        }
+      } else if (dim == 3 && !cellSimplex && !cellHybrid && size == 2) {
+        switch (user->testNum) {
+        case 0: {
+          PetscInt hexSizes_p2[2]  = {1, 1};
+          PetscInt hexPoints_p2[2] = {0, 1};
+
+          ierr = PetscMalloc2(2, &sizes, 2, &points);CHKERRQ(ierr);
+          ierr = PetscMemcpy(sizes,  hexSizes_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);
+          ierr = PetscMemcpy(points, hexPoints_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);break;}
+        default:
+          SETERRQ1(PETSC_COMM_WORLD, PETSC_ERR_ARG_WRONG, "Could not find matching test number %d for hexahedral mesh on 2 procs", user->testNum);
+        }
+      } else if (dim == 3 && !cellSimplex && cellHybrid && size == 2) {
+        switch (user->testNum) {
+        case 0: {
+          PetscInt hexSizes_p2[2]  = {1, 1};
+          PetscInt hexPoints_p2[2] = {0, 1};
+
+          ierr = PetscMalloc2(2, &sizes, 2, &points);CHKERRQ(ierr);
+          ierr = PetscMemcpy(sizes,  hexSizes_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);
+          ierr = PetscMemcpy(points, hexPoints_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);break;}
+        case 1: {
+          PetscInt hexSizes_p2[2]  = {5, 4};
+          PetscInt hexPoints_p2[9] = {3, 4, 5, 7, 8, 0, 1, 2, 6};
+
+          ierr = PetscMalloc2(2, &sizes, 9, &points);CHKERRQ(ierr);
+          ierr = PetscMemcpy(sizes,  hexSizes_p2, 2 * sizeof(PetscInt));CHKERRQ(ierr);
+          ierr = PetscMemcpy(points, hexPoints_p2, 9 * sizeof(PetscInt));CHKERRQ(ierr);break;}
+        default:
+          SETERRQ1(PETSC_COMM_WORLD, PETSC_ERR_ARG_WRONG, "Could not find matching test number %d for hexahedral hybrid mesh on 2 procs", user->testNum);
+        }
+      } else SETERRQ(PETSC_COMM_WORLD, PETSC_ERR_ARG_WRONG, "Could not find matching test partition");
+    }
+    ierr = DMPlexGetPartitioner(*dm, &part);CHKERRQ(ierr);
+    ierr = PetscPartitionerSetType(part, PETSCPARTITIONERSHELL);CHKERRQ(ierr);
+    ierr = PetscPartitionerShellSetPartition(part, size, sizes, points);CHKERRQ(ierr);
+    ierr = PetscFree2(sizes, points);CHKERRQ(ierr);
+  } else {
+    PetscPartitioner part;
+
+    ierr = DMPlexGetPartitioner(*dm,&part);CHKERRQ(ierr);
+    ierr = PetscPartitionerSetFromOptions(part);CHKERRQ(ierr);
+  }
+  ierr = DMSetFromOptions(*dm);CHKERRQ(ierr);
   {
     DM refinedMesh     = NULL;
     DM distributedMesh = NULL;
@@ -678,27 +790,48 @@ PetscErrorCode CreateMesh(MPI_Comm comm, AppCtx *user, DM *dm)
       }
     }
   }
-  if (user->uninterpolate) {
+  if (user->simplex2tensor) {
+    DM rdm = NULL;
+    ierr = DMPlexSetRefinementUniform(*dm, PETSC_TRUE);CHKERRQ(ierr);
+    ierr = DMPlexRefineSimplexToTensor(*dm, &rdm);CHKERRQ(ierr);
+    if (rdm) {
+      ierr = DMDestroy(dm);CHKERRQ(ierr);
+      *dm  = rdm;
+    }
+    user->cellSimplex = PETSC_FALSE;
+  }
+  if (user->uninterpolate || user->reinterpolate) {
     DM udm = NULL;
 
     ierr = DMPlexUninterpolate(*dm, &udm);CHKERRQ(ierr);
+    ierr = DMPlexCopyCoordinates(*dm, udm);CHKERRQ(ierr);
     ierr = DMDestroy(dm);CHKERRQ(ierr);
     *dm  = udm;
   }
+
+  if (user->reinterpolate) {
+    DM idm = NULL;
+
+    ierr = DMPlexInterpolate(*dm, &idm);CHKERRQ(ierr);
+    ierr = DMPlexCopyCoordinates(*dm, idm);CHKERRQ(ierr);
+    ierr = DMDestroy(dm);CHKERRQ(ierr);
+    *dm  = idm;
+    ierr = DMPlexCheckSymmetry(*dm);CHKERRQ(ierr);
+    ierr = DMPlexCheckSkeleton(*dm, user->cellSimplex, 0);CHKERRQ(ierr);
+    ierr = DMPlexCheckFaces(*dm, user->cellSimplex, 0);CHKERRQ(ierr);
+  }
   ierr = PetscObjectSetName((PetscObject) *dm, "Hybrid Mesh");CHKERRQ(ierr);
   ierr = DMViewFromOptions(*dm, NULL, "-dm_view");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc, char **argv)
 {
   DM             dm;
   AppCtx         user;                 /* user-defined work context */
   PetscErrorCode ierr;
 
-  ierr = PetscInitialize(&argc, &argv, NULL, help);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc, &argv, NULL, help);if (ierr) return ierr;
   ierr = ProcessOptions(PETSC_COMM_WORLD, &user);CHKERRQ(ierr);
   ierr = CreateMesh(PETSC_COMM_WORLD, &user, &dm);CHKERRQ(ierr);
   ierr = DMPlexCheckSymmetry(dm);CHKERRQ(ierr);
@@ -706,5 +839,182 @@ int main(int argc, char **argv)
   if (!user.uninterpolate) {ierr = DMPlexCheckFaces(dm, user.cellSimplex, 0);CHKERRQ(ierr);}
   ierr = DMDestroy(&dm);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
+
+/*TEST
+
+  # 2D Simplex 0-3
+  test:
+    suffix: 0
+    args: -dim 2 -cell_hybrid 0 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 1
+    args: -dim 2 -cell_hybrid 0 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 2
+    nsize: 2
+    args: -dim 2 -cell_hybrid 0 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 3
+    nsize: 2
+    args: -dim 2 -cell_hybrid 0 -num_refinements 1 -dm_view ascii::ascii_info_detail
+
+  # 2D Hybrid Simplex 4-7
+  test:
+    suffix: 4
+    args: -dim 2 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 5
+    args: -dim 2 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 6
+    nsize: 2
+    args: -dim 2 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 7
+    nsize: 2
+    args: -dim 2 -num_refinements 1 -dm_view ascii::ascii_info_detail
+
+  # 3D Simplex 8-11
+  test:
+    suffix: 8
+    args: -dim 3 -cell_hybrid 0 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 9
+    nsize: 2
+    args: -dim 3 -cell_hybrid 0 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 10
+    args: -dim 3 -cell_hybrid 0 -test_num 1 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 11
+    nsize: 2
+    args: -dim 3 -cell_hybrid 0 -test_num 1 -num_refinements 1 -dm_view ascii::ascii_info_detail
+
+  # 2D Quad 12-13
+  test:
+    suffix: 12
+    args: -dim 2 -cell_simplex 0 -cell_hybrid 0 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 13
+    nsize: 2
+    args: -dim 2 -cell_simplex 0 -cell_hybrid 0 -num_refinements 1 -dm_view ascii::ascii_info_detail
+
+  # 3D Hex 14-15
+  test:
+    suffix: 14
+    args: -dim 3 -cell_simplex 0 -cell_hybrid 0 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 15
+    nsize: 2
+    args: -dim 3 -cell_simplex 0 -cell_hybrid 0 -num_refinements 1 -dm_view ascii::ascii_info_detail
+
+  # 3D Hybrid Simplex 16-19
+  test:
+    suffix: 16
+    args: -dim 3 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 17
+    nsize: 2
+    args: -dim 3 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 18
+    args: -dim 3 -test_num 1 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 19
+    nsize: 2
+    args: -dim 3 -test_num 1 -num_refinements 1 -dm_view ascii::ascii_info_detail
+
+  # 3D Hybrid Hex 20-23
+  test:
+    suffix: 20
+    args: -dim 3 -cell_simplex 0 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 21
+    nsize: 2
+    args: -dim 3 -cell_simplex 0 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 22
+    args: -dim 3 -cell_simplex 0 -test_num 1 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 23
+    nsize: 2
+    args: -dim 3 -cell_simplex 0 -test_num 1 -num_refinements 1 -dm_view ascii::ascii_info_detail
+
+  # 2D Hybrid Simplex 24-24
+  test:
+    suffix: 24
+    args: -dim 2 -test_num 1 -num_refinements 1 -dm_view ascii::ascii_info_detail
+
+  # 3D Multiple Refinement 25-26
+  test:
+    suffix: 25
+    args: -dim 3 -cell_hybrid 0 -test_num 2 -num_refinements 2 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 26
+    args: -dim 3 -cell_simplex 0 -cell_hybrid 0 -test_num 1 -num_refinements 2 -dm_view ascii::ascii_info_detail
+
+  # 2D Hybrid Quad 27-28
+  test:
+    suffix: 27
+    args: -dim 2 -cell_simplex 0 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 28
+    nsize: 2
+    args: -dim 2 -cell_simplex 0 -num_refinements 1 -dm_view ascii::ascii_info_detail
+
+  # 1D Simplex 29-31
+  test:
+    suffix: 29
+    args: -dim 1 -cell_hybrid 0 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 30
+    args: -dim 1 -cell_hybrid 0 -num_refinements 1 -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 31
+    args: -dim 1 -cell_hybrid 0 -num_refinements 5 -dm_view ascii::ascii_info_detail
+
+  # 2D Simplex 32-34
+  test:
+    suffix: 32
+    args: -dim 2 -cell_hybrid 0 -num_refinements 1 -uninterpolate -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 33
+    nsize: 2
+    args: -dim 2 -cell_hybrid 0 -num_refinements 1 -uninterpolate -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 34
+    nsize: 2
+    args: -dim 2 -cell_hybrid 0 -num_refinements 3 -uninterpolate -dm_view ascii::ascii_info_detail
+
+  # Hybrid interpolation
+  testset:
+    nsize: 2
+    args: -test_partition 0 -petscpartitioner_type simple -dm_view -reinterpolate
+    test:
+      suffix: hybint_2d_0
+      args: -dim 2 -num_refinements 2
+    test:
+      suffix: hybint_2d_s2t_0
+      args: -dim 2 -num_refinements 2 -simplex2tensor
+    test:
+      suffix: hybint_2d_1
+      args: -dim 2 -num_refinements 2 -test_num 1
+    test:
+      suffix: hybint_2d_s2t_1
+      args: -dim 2 -num_refinements 2 -simplex2tensor -test_num 1
+    test:
+      suffix: hybint_3d_0
+      args: -dim 3 -num_refinements 1
+    test:
+      suffix: hybint_3d_s2t_0
+      args: -dim 3 -num_refinements 1 -simplex2tensor
+    test:
+      suffix: hybint_3d_1
+      args: -dim 3 -num_refinements 1 -test_num 1
+    test:
+      suffix: hybint_3d_s2t_1
+      args: -dim 3 -num_refinements 1  -simplex2tensor -test_num 1
+
+TEST*/
