@@ -15,8 +15,6 @@ typedef struct {
   PetscReal *centroid, *normal, *vol;     /* FVM data */
 } AppCtx;
 
-#undef __FUNCT__
-#define __FUNCT__ "ReadMesh"
 PetscErrorCode ReadMesh(MPI_Comm comm, const char *filename, AppCtx *user, DM *dm)
 {
   PetscMPIInt    rank;
@@ -26,20 +24,17 @@ PetscErrorCode ReadMesh(MPI_Comm comm, const char *filename, AppCtx *user, DM *d
   ierr = MPI_Comm_rank(comm, &rank);CHKERRQ(ierr);
   ierr = DMPlexCreateFromFile(comm, filename, PETSC_FALSE, dm);CHKERRQ(ierr);
   if (user->interpolate) {
-    DM interpolatedMesh = NULL;
+    DM idm;
 
-    ierr = DMPlexInterpolate(*dm, &interpolatedMesh);CHKERRQ(ierr);
-    ierr = DMPlexCopyCoordinates(*dm, interpolatedMesh);CHKERRQ(ierr);
+    ierr = DMPlexInterpolate(*dm, &idm);CHKERRQ(ierr);
     ierr = DMDestroy(dm);CHKERRQ(ierr);
-    *dm  = interpolatedMesh;
+    *dm  = idm;
   }
   ierr = PetscObjectSetName((PetscObject) *dm, "Input Mesh");CHKERRQ(ierr);
   ierr = DMViewFromOptions(*dm, NULL, "-dm_view");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ProcessOptions"
 PetscErrorCode ProcessOptions(MPI_Comm comm, AppCtx *options)
 {
   const char    *runTypes[2] = {"reference", "file"};
@@ -72,34 +67,32 @@ PetscErrorCode ProcessOptions(MPI_Comm comm, AppCtx *options)
                         numCells*dim,&options->centroid,numCells*dim,&options->normal,numCells,&options->vol);CHKERRQ(ierr);
     n    = numCells*dim;
     ierr = PetscOptionsRealArray("-v0", "Input v0 for each cell", "ex8.c", options->v0, &n, &flag);CHKERRQ(ierr);
-    if (flag && n != numCells*dim) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Invalid size of v0 %d should be %d", n, numCells*dim);
+    if (flag && n != numCells*dim) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Invalid size of v0 %D should be %D", n, numCells*dim);
     n    = numCells*dim*dim;
     ierr = PetscOptionsRealArray("-J", "Input Jacobian for each cell", "ex8.c", options->J, &n, &flag);CHKERRQ(ierr);
-    if (flag && n != numCells*dim*dim) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Invalid size of J %d should be %d", n, numCells*dim*dim);
+    if (flag && n != numCells*dim*dim) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Invalid size of J %D should be %D", n, numCells*dim*dim);
     n    = numCells*dim*dim;
     ierr = PetscOptionsRealArray("-invJ", "Input inverse Jacobian for each cell", "ex8.c", options->invJ, &n, &flag);CHKERRQ(ierr);
-    if (flag && n != numCells*dim*dim) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Invalid size of invJ %d should be %d", n, numCells*dim*dim);
+    if (flag && n != numCells*dim*dim) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Invalid size of invJ %D should be %D", n, numCells*dim*dim);
     n    = numCells;
     ierr = PetscOptionsRealArray("-detJ", "Input Jacobian determinant for each cell", "ex8.c", options->detJ, &n, &flag);CHKERRQ(ierr);
-    if (flag && n != numCells) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Invalid size of detJ %d should be %d", n, numCells);
+    if (flag && n != numCells) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Invalid size of detJ %D should be %D", n, numCells);
     n    = numCells*dim;
     ierr = PetscOptionsRealArray("-centroid", "Input centroid for each cell", "ex8.c", options->centroid, &n, &flag);CHKERRQ(ierr);
-    if (flag && n != numCells*dim) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Invalid size of centroid %d should be %d", n, numCells*dim);
+    if (flag && n != numCells*dim) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Invalid size of centroid %D should be %D", n, numCells*dim);
     n    = numCells*dim;
     ierr = PetscOptionsRealArray("-normal", "Input normal for each cell", "ex8.c", options->normal, &n, &flag);CHKERRQ(ierr);
-    if (flag && n != numCells*dim) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Invalid size of normal %d should be %d", n, numCells*dim);
+    if (flag && n != numCells*dim) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Invalid size of normal %D should be %D", n, numCells*dim);
     n    = numCells;
     ierr = PetscOptionsRealArray("-vol", "Input volume for each cell", "ex8.c", options->vol, &n, &flag);CHKERRQ(ierr);
-    if (flag && n != numCells) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Invalid size of vol %d should be %d", n, numCells);
+    if (flag && n != numCells) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_ARG_SIZ, "Invalid size of vol %D should be %D", n, numCells);
   }
   ierr = PetscOptionsEnd();
 
   if (options->transform) {ierr = PetscPrintf(comm, "Using random transforms");CHKERRQ(ierr);}
   PetscFunctionReturn(0);
-};
+}
 
-#undef __FUNCT__
-#define __FUNCT__ "ChangeCoordinates"
 PetscErrorCode ChangeCoordinates(DM dm, PetscInt spaceDim, PetscScalar vertexCoords[])
 {
   PetscSection   coordSection;
@@ -120,7 +113,7 @@ PetscErrorCode ChangeCoordinates(DM dm, PetscInt spaceDim, PetscScalar vertexCoo
   }
   ierr = PetscSectionSetUp(coordSection);CHKERRQ(ierr);
   ierr = PetscSectionGetStorageSize(coordSection, &coordSize);CHKERRQ(ierr);
-  ierr = VecCreate(PetscObjectComm((PetscObject) dm), &coordinates);CHKERRQ(ierr);
+  ierr = VecCreate(PETSC_COMM_SELF, &coordinates);CHKERRQ(ierr);
   ierr = PetscObjectSetName((PetscObject) coordinates, "coordinates");CHKERRQ(ierr);
   ierr = VecSetSizes(coordinates, coordSize, PETSC_DETERMINE);CHKERRQ(ierr);
   ierr = VecSetFromOptions(coordinates);CHKERRQ(ierr);
@@ -141,8 +134,8 @@ PetscErrorCode ChangeCoordinates(DM dm, PetscInt spaceDim, PetscScalar vertexCoo
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CheckFEMGeometry"
+#define RelativeError(a,b) PetscAbs(a-b)/(1.0+PetscMax(PetscAbs(a),PetscAbs(b)))
+
 PetscErrorCode CheckFEMGeometry(DM dm, PetscInt cell, PetscInt spaceDim, PetscReal v0Ex[], PetscReal JEx[], PetscReal invJEx[], PetscReal detJEx)
 {
   PetscReal      v0[3], J[9], invJ[9], detJ;
@@ -154,24 +147,22 @@ PetscErrorCode CheckFEMGeometry(DM dm, PetscInt cell, PetscInt spaceDim, PetscRe
   for (d = 0; d < spaceDim; ++d) {
     if (v0[d] != v0Ex[d]) {
       switch (spaceDim) {
-      case 2: SETERRQ4(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid v0 (%g, %g) != (%g, %g)", v0[0], v0[1], v0Ex[0], v0Ex[1]);break;
-      case 3: SETERRQ6(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid v0 (%g, %g, %g) != (%g, %g, %g)", v0[0], v0[1], v0[2], v0Ex[0], v0Ex[1], v0Ex[2]);break;
-      default: SETERRQ1(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid space dimension %d", spaceDim);
+      case 2: SETERRQ4(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid v0 (%g, %g) != (%g, %g)", (double)v0[0], (double)v0[1], (double)v0Ex[0], (double)v0Ex[1]);break;
+      case 3: SETERRQ6(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid v0 (%g, %g, %g) != (%g, %g, %g)", (double)v0[0], (double)v0[1], (double)v0[2], (double)v0Ex[0], (double)v0Ex[1], (double)v0Ex[2]);break;
+      default: SETERRQ1(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid space dimension %D", spaceDim);
       }
     }
   }
   for (i = 0; i < spaceDim; ++i) {
     for (j = 0; j < spaceDim; ++j) {
-      if (fabs(J[i*spaceDim+j]    - JEx[i*spaceDim+j])    > 1.0e-9) SETERRQ4(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid J[%d,%d]: %g != %g", i, j, J[i*spaceDim+j], JEx[i*spaceDim+j]);
-      if (fabs(invJ[i*spaceDim+j] - invJEx[i*spaceDim+j]) > 1.0e-9) SETERRQ4(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid invJ[%d,%d]: %g != %g", i, j, invJ[i*spaceDim+j], invJEx[i*spaceDim+j]);
+      if (RelativeError(J[i*spaceDim+j],JEx[i*spaceDim+j])    > 10*PETSC_SMALL) SETERRQ4(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid J[%D,%D]: %g != %g", i, j, (double)J[i*spaceDim+j], (double)JEx[i*spaceDim+j]);
+      if (RelativeError(invJ[i*spaceDim+j],invJEx[i*spaceDim+j]) > 10*PETSC_SMALL) SETERRQ4(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid invJ[%D,%D]: %g != %g", i, j, (double)invJ[i*spaceDim+j], (double)invJEx[i*spaceDim+j]);
     }
   }
-  if (fabs(detJ - detJEx) > 1.0e-9) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid |J| = %g != %g", detJ, detJEx);
+  if (RelativeError(detJ,detJEx) > 10*PETSC_SMALL) SETERRQ3(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid |J| = %g != %g diff %g", (double)detJ, (double)detJEx,(double)(detJ - detJEx));
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CheckFVMGeometry"
 PetscErrorCode CheckFVMGeometry(DM dm, PetscInt cell, PetscInt spaceDim, PetscReal centroidEx[], PetscReal normalEx[], PetscReal volEx)
 {
   PetscReal      centroid[3], normal[3], vol;
@@ -181,15 +172,13 @@ PetscErrorCode CheckFVMGeometry(DM dm, PetscInt cell, PetscInt spaceDim, PetscRe
   PetscFunctionBegin;
   ierr = DMPlexComputeCellGeometryFVM(dm, cell, &vol, centroid, normal);CHKERRQ(ierr);
   for (d = 0; d < spaceDim; ++d) {
-    if (fabs(centroid[d] - centroidEx[d]) > 1.0e-9) SETERRQ3(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid centroid[%d]: %g != %g", d, centroid[d], centroidEx[d]);
-    if (fabs(normal[d] - normalEx[d]) > 1.0e-9) SETERRQ3(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid normal[%d]: %g != %g", d, normal[d], normalEx[d]);
+    if (RelativeError(centroid[d],centroidEx[d]) > 10*PETSC_SMALL) SETERRQ4(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid centroid[%D]: %g != %g diff %g", d, (double)centroid[d], (double)centroidEx[d],(double)(centroid[d]-centroidEx[d]));
+    if (RelativeError(normal[d],normalEx[d]) > 10*PETSC_SMALL) SETERRQ3(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid normal[%D]: %g != %g", d, (double)normal[d], (double)normalEx[d]);
   }
-  if (fabs(volEx - vol) > 1.0e-9) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid volume = %g != %g", vol, volEx);
+  if (RelativeError(volEx,vol) > 10*PETSC_SMALL) SETERRQ3(PETSC_COMM_SELF, PETSC_ERR_PLIB, "Invalid volume = %g != %g diff %g", (double)vol, (double)volEx,(double)(vol - volEx));
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TestTriangle"
 PetscErrorCode TestTriangle(MPI_Comm comm, PetscBool interpolate, PetscBool transform)
 {
   DM             dm;
@@ -213,11 +202,9 @@ PetscErrorCode TestTriangle(MPI_Comm comm, PetscBool interpolate, PetscBool tran
 
     ierr = DMPlexCreateFromDAG(dm, 1, numPoints, coneSize, cones, coneOrientations, vertexCoords);CHKERRQ(ierr);
     if (interpolate) {
-      DM idm = NULL;
+      DM idm;
 
       ierr = DMPlexInterpolate(dm, &idm);CHKERRQ(ierr);
-      ierr = PetscObjectSetName((PetscObject) idm, "triangle");CHKERRQ(ierr);
-      ierr = DMPlexCopyCoordinates(dm, idm);CHKERRQ(ierr);
       ierr = DMDestroy(&dm);CHKERRQ(ierr);
       dm   = idm;
     }
@@ -229,7 +216,7 @@ PetscErrorCode TestTriangle(MPI_Comm comm, PetscBool interpolate, PetscBool tran
     PetscReal JEx[4]        = {1.0, 0.0, 0.0, 1.0};
     PetscReal invJEx[4]     = {1.0, 0.0, 0.0, 1.0};
     PetscReal detJEx        = 1.0;
-    PetscReal centroidEx[2] = {-0.333333333333, -0.333333333333};
+    PetscReal centroidEx[2] = {-((PetscReal)1.)/((PetscReal)3.), -((PetscReal)1.)/((PetscReal)3.)};
     PetscReal normalEx[2]   = {0.0, 0.0};
     PetscReal volEx         = 2.0;
 
@@ -250,7 +237,7 @@ PetscErrorCode TestTriangle(MPI_Comm comm, PetscBool interpolate, PetscBool tran
       PetscReal   JEx[4]          = {1.0, 0.0, 0.0, 1.0}, R[4], rot[2], rotM[4];
       PetscReal   invJEx[4]       = {1.0, 0.0, 0.0, 1.0};
       PetscReal   detJEx          = 1.0, scale, phi;
-      PetscReal   centroidEx[2]   = {-0.333333333333, -0.333333333333};
+      PetscReal   centroidEx[2]   = {-((PetscReal)1.)/((PetscReal)3.), -((PetscReal)1.)/((PetscReal)3.)};
       PetscReal   normalEx[2]     = {0.0, 0.0};
       PetscReal   volEx           = 2.0;
       PetscInt    d, e, f, p;
@@ -262,7 +249,7 @@ PetscErrorCode TestTriangle(MPI_Comm comm, PetscBool interpolate, PetscBool tran
       for (p = 0; p < 3; ++p) {
         for (d = 0; d < dim; ++d) {
           for (e = 0, rot[d] = 0.0; e < dim; ++e) {
-            rot[d] += R[d*dim+e] * vertexCoords[p*dim+e];
+            rot[d] += R[d*dim+e] * PetscRealPart(vertexCoords[p*dim+e]);
           }
         }
         for (d = 0; d < dim; ++d) vertexCoords[p*dim+d] = rot[d];
@@ -298,19 +285,19 @@ PetscErrorCode TestTriangle(MPI_Comm comm, PetscBool interpolate, PetscBool tran
         }
       }
       for (d = 0; d < dim; ++d) {
-        ierr = PetscRandomGetValueReal(r, &trans[d]);CHKERRQ(ierr);
+        ierr = PetscRandomGetValue(r, &trans[d]);CHKERRQ(ierr);
         for (p = 0; p < 3; ++p) {
           vertexCoords[p*dim+d] *= scale;
           vertexCoords[p*dim+d] += trans[d];
         }
-        v0Ex[d] = vertexCoords[d];
+        v0Ex[d] = PetscRealPart(vertexCoords[d]);
         for (e = 0; e < dim; ++e) {
           JEx[d*dim+e]    *= scale;
           invJEx[d*dim+e] /= scale;
         }
         detJEx *= scale;
         centroidEx[d] *= scale;
-        centroidEx[d] += trans[d];
+        centroidEx[d] += PetscRealPart(trans[d]);
         volEx *= scale;
       }
       ierr = ChangeCoordinates(dm, dim, vertexCoords);CHKERRQ(ierr);
@@ -328,7 +315,7 @@ PetscErrorCode TestTriangle(MPI_Comm comm, PetscBool interpolate, PetscBool tran
     PetscReal JEx[9]        = {1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0};
     PetscReal invJEx[9]     = {1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0};
     PetscReal detJEx        = 1.0;
-    PetscReal centroidEx[3] = {-0.333333333333, -0.333333333333, 0.0};
+    PetscReal centroidEx[3] = {-((PetscReal)1.)/((PetscReal)3.), -((PetscReal)1.)/((PetscReal)3.), 0.0};
     PetscReal normalEx[3]   = {0.0, 0.0, 1.0};
     PetscReal volEx         = 2.0;
 
@@ -343,7 +330,7 @@ PetscErrorCode TestTriangle(MPI_Comm comm, PetscBool interpolate, PetscBool tran
     PetscReal   JEx[9]          = {0.0, 0.0, 1.0, 1.0, 0.0, 0.0, 0.0, 1.0, 0.0};
     PetscReal   invJEx[9]       = {0.0, 1.0, 0.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0};
     PetscReal   detJEx          = 1.0;
-    PetscReal   centroidEx[3]   = {0.0, -0.333333333333, -0.333333333333};
+    PetscReal   centroidEx[3]   = {0.0, -((PetscReal)1.)/((PetscReal)3.), -((PetscReal)1.)/((PetscReal)3.)};
     PetscReal   normalEx[3]     = {1.0, 0.0, 0.0};
     PetscReal   volEx           = 2.0;
 
@@ -368,7 +355,7 @@ PetscErrorCode TestTriangle(MPI_Comm comm, PetscBool interpolate, PetscBool tran
       PetscReal   JEx[9]          = {1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0}, R[9], rot[3], rotM[9];
       PetscReal   invJEx[9]       = {1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0};
       PetscReal   detJEx          = 1.0, scale, phi, theta, psi = 0.0;
-      PetscReal   centroidEx[3]   = {-0.333333333333, -0.333333333333, 0.0};
+      PetscReal   centroidEx[3]   = {-((PetscReal)1.)/((PetscReal)3.), -((PetscReal)1.)/((PetscReal)3.), 0.0};
       PetscReal   normalEx[3]     = {0.0, 0.0, 1.0};
       PetscReal   volEx           = 2.0;
       PetscInt    d, e, f, p;
@@ -377,13 +364,13 @@ PetscErrorCode TestTriangle(MPI_Comm comm, PetscBool interpolate, PetscBool tran
       ierr = PetscRandomGetValueReal(ang, &phi);CHKERRQ(ierr);
       ierr = PetscRandomGetValueReal(ang2, &theta);CHKERRQ(ierr);
       for (d = 0; d < dim; ++d) {
-        ierr = PetscRandomGetValueReal(r, &trans[d]);CHKERRQ(ierr);
+        ierr = PetscRandomGetValue(r, &trans[d]);CHKERRQ(ierr);
         for (p = 0; p < 3; ++p) {
           vertexCoords[p*dim+d] *= scale;
           vertexCoords[p*dim+d] += trans[d];
         }
         centroidEx[d] *= scale;
-        centroidEx[d] += trans[d];
+        centroidEx[d] += PetscRealPart(trans[d]);
         for (e = 0; e < dim-1; ++e) {
           JEx[d*dim+e]    *= scale;
           invJEx[d*dim+e] /= scale;
@@ -399,7 +386,7 @@ PetscErrorCode TestTriangle(MPI_Comm comm, PetscBool interpolate, PetscBool tran
       for (p = 0; p < 3; ++p) {
         for (d = 0; d < dim; ++d) {
           for (e = 0, rot[d] = 0.0; e < dim; ++e) {
-            rot[d] += R[d*dim+e] * vertexCoords[p*dim+e];
+            rot[d] += R[d*dim+e] * PetscRealPart(vertexCoords[p*dim+e]);
           }
         }
         for (d = 0; d < dim; ++d) vertexCoords[p*dim+d] = rot[d];
@@ -417,7 +404,7 @@ PetscErrorCode TestTriangle(MPI_Comm comm, PetscBool interpolate, PetscBool tran
       }
       for (d = 0; d < dim; ++d) normalEx[d] = rot[d];
       for (d = 0; d < dim; ++d) {
-        v0Ex[d] = vertexCoords[d];
+        v0Ex[d] = PetscRealPart(vertexCoords[d]);
         for (e = 0; e < dim; ++e) {
           for (f = 0, rotM[d*dim+e] = 0.0; f < dim; ++f) {
             rotM[d*dim+e] += R[d*dim+f] * JEx[f*dim+e];
@@ -454,8 +441,6 @@ PetscErrorCode TestTriangle(MPI_Comm comm, PetscBool interpolate, PetscBool tran
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TestQuadrilateral"
 PetscErrorCode TestQuadrilateral(MPI_Comm comm, PetscBool interpolate, PetscBool transform)
 {
   DM             dm;
@@ -479,11 +464,9 @@ PetscErrorCode TestQuadrilateral(MPI_Comm comm, PetscBool interpolate, PetscBool
 
     ierr = DMPlexCreateFromDAG(dm, 1, numPoints, coneSize, cones, coneOrientations, vertexCoords);CHKERRQ(ierr);
     if (interpolate) {
-      DM idm = NULL;
+      DM idm;
 
       ierr = DMPlexInterpolate(dm, &idm);CHKERRQ(ierr);
-      ierr = PetscObjectSetName((PetscObject) idm, "quadrilateral");CHKERRQ(ierr);
-      ierr = DMPlexCopyCoordinates(dm, idm);CHKERRQ(ierr);
       ierr = DMDestroy(&dm);CHKERRQ(ierr);
       dm   = idm;
     }
@@ -528,7 +511,7 @@ PetscErrorCode TestQuadrilateral(MPI_Comm comm, PetscBool interpolate, PetscBool
       for (p = 0; p < 4; ++p) {
         for (d = 0; d < dim; ++d) {
           for (e = 0, rot[d] = 0.0; e < dim; ++e) {
-            rot[d] += R[d*dim+e] * vertexCoords[p*dim+e];
+            rot[d] += R[d*dim+e] * PetscRealPart(vertexCoords[p*dim+e]);
           }
         }
         for (d = 0; d < dim; ++d) vertexCoords[p*dim+d] = rot[d];
@@ -564,19 +547,19 @@ PetscErrorCode TestQuadrilateral(MPI_Comm comm, PetscBool interpolate, PetscBool
         }
       }
       for (d = 0; d < dim; ++d) {
-        ierr = PetscRandomGetValueReal(r, &trans[d]);CHKERRQ(ierr);
+        ierr = PetscRandomGetValue(r, &trans[d]);CHKERRQ(ierr);
         for (p = 0; p < 4; ++p) {
           vertexCoords[p*dim+d] *= scale;
           vertexCoords[p*dim+d] += trans[d];
         }
-        v0Ex[d] = vertexCoords[d];
+        v0Ex[d] = PetscRealPart(vertexCoords[d]);
         for (e = 0; e < dim; ++e) {
           JEx[d*dim+e]    *= scale;
           invJEx[d*dim+e] /= scale;
         }
         detJEx *= scale;
         centroidEx[d] *= scale;
-        centroidEx[d] += trans[d];
+        centroidEx[d] += PetscRealPart(trans[d]);
         volEx *= scale;
       }
       ierr = ChangeCoordinates(dm, dim, vertexCoords);CHKERRQ(ierr);
@@ -628,13 +611,13 @@ PetscErrorCode TestQuadrilateral(MPI_Comm comm, PetscBool interpolate, PetscBool
       ierr = PetscRandomGetValueReal(ang, &phi);CHKERRQ(ierr);
       ierr = PetscRandomGetValueReal(ang2, &theta);CHKERRQ(ierr);
       for (d = 0; d < dim; ++d) {
-        ierr = PetscRandomGetValueReal(r, &trans[d]);CHKERRQ(ierr);
+        ierr = PetscRandomGetValue(r, &trans[d]);CHKERRQ(ierr);
         for (p = 0; p < 4; ++p) {
           vertexCoords[p*dim+d] *= scale;
           vertexCoords[p*dim+d] += trans[d];
         }
         centroidEx[d] *= scale;
-        centroidEx[d] += trans[d];
+        centroidEx[d] += PetscRealPart(trans[d]);
         for (e = 0; e < dim-1; ++e) {
           JEx[d*dim+e]    *= scale;
           invJEx[d*dim+e] /= scale;
@@ -650,7 +633,7 @@ PetscErrorCode TestQuadrilateral(MPI_Comm comm, PetscBool interpolate, PetscBool
       for (p = 0; p < 4; ++p) {
         for (d = 0; d < dim; ++d) {
           for (e = 0, rot[d] = 0.0; e < dim; ++e) {
-            rot[d] += R[d*dim+e] * vertexCoords[p*dim+e];
+            rot[d] += R[d*dim+e] * PetscRealPart(vertexCoords[p*dim+e]);
           }
         }
         for (d = 0; d < dim; ++d) vertexCoords[p*dim+d] = rot[d];
@@ -668,7 +651,7 @@ PetscErrorCode TestQuadrilateral(MPI_Comm comm, PetscBool interpolate, PetscBool
       }
       for (d = 0; d < dim; ++d) normalEx[d] = rot[d];
       for (d = 0; d < dim; ++d) {
-        v0Ex[d] = vertexCoords[d];
+        v0Ex[d] = PetscRealPart(vertexCoords[d]);
         for (e = 0; e < dim; ++e) {
           for (f = 0, rotM[d*dim+e] = 0.0; f < dim; ++f) {
             rotM[d*dim+e] += R[d*dim+f] * JEx[f*dim+e];
@@ -705,8 +688,6 @@ PetscErrorCode TestQuadrilateral(MPI_Comm comm, PetscBool interpolate, PetscBool
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TestTetrahedron"
 PetscErrorCode TestTetrahedron(MPI_Comm comm, PetscBool interpolate, PetscBool transform)
 {
   DM             dm;
@@ -730,11 +711,9 @@ PetscErrorCode TestTetrahedron(MPI_Comm comm, PetscBool interpolate, PetscBool t
 
     ierr = DMPlexCreateFromDAG(dm, 1, numPoints, coneSize, cones, coneOrientations, vertexCoords);CHKERRQ(ierr);
     if (interpolate) {
-      DM idm = NULL;
+      DM idm;
 
       ierr = DMPlexInterpolate(dm, &idm);CHKERRQ(ierr);
-      ierr = PetscObjectSetName((PetscObject) idm, "tetrahedron");CHKERRQ(ierr);
-      ierr = DMPlexCopyCoordinates(dm, idm);CHKERRQ(ierr);
       ierr = DMDestroy(&dm);CHKERRQ(ierr);
       dm   = idm;
     }
@@ -748,7 +727,7 @@ PetscErrorCode TestTetrahedron(MPI_Comm comm, PetscBool interpolate, PetscBool t
     PetscReal detJEx        = 1.0;
     PetscReal centroidEx[3] = {-0.5, -0.5, -0.5};
     PetscReal normalEx[3]   = {0.0, 0.0, 0.0};
-    PetscReal volEx         = 4.0/3.0;
+    PetscReal volEx         = (PetscReal)4.0/(PetscReal)3.0;
 
     ierr = CheckFEMGeometry(dm, 0, dim, v0Ex, JEx, invJEx, detJEx);CHKERRQ(ierr);
     if (interpolate) {ierr = CheckFVMGeometry(dm, 0, dim, centroidEx, normalEx, volEx);CHKERRQ(ierr);}
@@ -772,20 +751,20 @@ PetscErrorCode TestTetrahedron(MPI_Comm comm, PetscBool interpolate, PetscBool t
       PetscReal   detJEx           = 1.0, scale, phi, theta, psi = 0.0;
       PetscReal   centroidEx[3]    = {-0.5, -0.5, -0.5};
       PetscReal   normalEx[3]      = {0.0, 0.0, 0.0};
-      PetscReal   volEx            = 4.0/3.0;
+      PetscReal   volEx            = (PetscReal)4.0/(PetscReal)3.0;
       PetscInt    d, e, f, p;
 
       ierr = PetscRandomGetValueReal(r, &scale);CHKERRQ(ierr);
       ierr = PetscRandomGetValueReal(ang, &phi);CHKERRQ(ierr);
       ierr = PetscRandomGetValueReal(ang2, &theta);CHKERRQ(ierr);
       for (d = 0; d < dim; ++d) {
-        ierr = PetscRandomGetValueReal(r, &trans[d]);CHKERRQ(ierr);
+        ierr = PetscRandomGetValue(r, &trans[d]);CHKERRQ(ierr);
         for (p = 0; p < 4; ++p) {
           vertexCoords[p*dim+d] *= scale;
           vertexCoords[p*dim+d] += trans[d];
         }
         centroidEx[d] *= scale;
-        centroidEx[d] += trans[d];
+        centroidEx[d] += PetscRealPart(trans[d]);
         for (e = 0; e < dim; ++e) {
           JEx[d*dim+e]    *= scale;
           invJEx[d*dim+e] /= scale;
@@ -799,7 +778,7 @@ PetscErrorCode TestTetrahedron(MPI_Comm comm, PetscBool interpolate, PetscBool t
       for (p = 0; p < 4; ++p) {
         for (d = 0; d < dim; ++d) {
           for (e = 0, rot[d] = 0.0; e < dim; ++e) {
-            rot[d] += R[d*dim+e] * vertexCoords[p*dim+e];
+            rot[d] += R[d*dim+e] * PetscRealPart(vertexCoords[p*dim+e]);
           }
         }
         for (d = 0; d < dim; ++d) vertexCoords[p*dim+d] = rot[d];
@@ -811,7 +790,7 @@ PetscErrorCode TestTetrahedron(MPI_Comm comm, PetscBool interpolate, PetscBool t
       }
       for (d = 0; d < dim; ++d) centroidEx[d] = rot[d];
       for (d = 0; d < dim; ++d) {
-        v0Ex[d] = vertexCoords[d];
+        v0Ex[d] = PetscRealPart(vertexCoords[d]);
         for (e = 0; e < dim; ++e) {
           for (f = 0, rotM[d*dim+e] = 0.0; f < dim; ++f) {
             rotM[d*dim+e] += R[d*dim+f] * JEx[f*dim+e];
@@ -848,8 +827,6 @@ PetscErrorCode TestTetrahedron(MPI_Comm comm, PetscBool interpolate, PetscBool t
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TestHexahedron"
 PetscErrorCode TestHexahedron(MPI_Comm comm, PetscBool interpolate, PetscBool transform)
 {
   DM             dm;
@@ -874,11 +851,9 @@ PetscErrorCode TestHexahedron(MPI_Comm comm, PetscBool interpolate, PetscBool tr
 
     ierr = DMPlexCreateFromDAG(dm, 1, numPoints, coneSize, cones, coneOrientations, vertexCoords);CHKERRQ(ierr);
     if (interpolate) {
-      DM idm = NULL;
+      DM idm;
 
       ierr = DMPlexInterpolate(dm, &idm);CHKERRQ(ierr);
-      ierr = PetscObjectSetName((PetscObject) idm, "hexahedron");CHKERRQ(ierr);
-      ierr = DMPlexCopyCoordinates(dm, idm);CHKERRQ(ierr);
       ierr = DMDestroy(&dm);CHKERRQ(ierr);
       dm   = idm;
     }
@@ -924,13 +899,13 @@ PetscErrorCode TestHexahedron(MPI_Comm comm, PetscBool interpolate, PetscBool tr
       ierr = PetscRandomGetValueReal(ang, &phi);CHKERRQ(ierr);
       ierr = PetscRandomGetValueReal(ang2, &theta);CHKERRQ(ierr);
       for (d = 0; d < dim; ++d) {
-        ierr = PetscRandomGetValueReal(r, &trans[d]);CHKERRQ(ierr);
+        ierr = PetscRandomGetValue(r, &trans[d]);CHKERRQ(ierr);
         for (p = 0; p < 8; ++p) {
           vertexCoords[p*dim+d] *= scale;
           vertexCoords[p*dim+d] += trans[d];
         }
         centroidEx[d] *= scale;
-        centroidEx[d] += trans[d];
+        centroidEx[d] += PetscRealPart(trans[d]);
         for (e = 0; e < dim; ++e) {
           JEx[d*dim+e]    *= scale;
           invJEx[d*dim+e] /= scale;
@@ -944,7 +919,7 @@ PetscErrorCode TestHexahedron(MPI_Comm comm, PetscBool interpolate, PetscBool tr
       for (p = 0; p < 8; ++p) {
         for (d = 0; d < dim; ++d) {
           for (e = 0, rot[d] = 0.0; e < dim; ++e) {
-            rot[d] += R[d*dim+e] * vertexCoords[p*dim+e];
+            rot[d] += R[d*dim+e] * PetscRealPart(vertexCoords[p*dim+e]);
           }
         }
         for (d = 0; d < dim; ++d) vertexCoords[p*dim+d] = rot[d];
@@ -956,7 +931,7 @@ PetscErrorCode TestHexahedron(MPI_Comm comm, PetscBool interpolate, PetscBool tr
       }
       for (d = 0; d < dim; ++d) centroidEx[d] = rot[d];
       for (d = 0; d < dim; ++d) {
-        v0Ex[d] = vertexCoords[d];
+        v0Ex[d] = PetscRealPart(vertexCoords[d]);
         for (e = 0; e < dim; ++e) {
           for (f = 0, rotM[d*dim+e] = 0.0; f < dim; ++f) {
             rotM[d*dim+e] += R[d*dim+f] * JEx[f*dim+e];
@@ -993,14 +968,12 @@ PetscErrorCode TestHexahedron(MPI_Comm comm, PetscBool interpolate, PetscBool tr
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc, char **argv)
 {
   AppCtx         user;
   PetscErrorCode ierr;
 
-  ierr = PetscInitialize(&argc, &argv, NULL, help);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc, &argv, NULL,help);if (ierr) return ierr;
   ierr = ProcessOptions(PETSC_COMM_WORLD, &user);CHKERRQ(ierr);
   if (user.runType == RUN_REFERENCE) {
     ierr = TestTriangle(PETSC_COMM_SELF, user.interpolate, user.transform);CHKERRQ(ierr);
@@ -1020,5 +993,30 @@ int main(int argc, char **argv)
     ierr = DMDestroy(&user.dm);CHKERRQ(ierr);
   }
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
+
+/*TEST
+
+  test:
+    suffix: 0
+    args: -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 1
+    args: -interpolate -dm_view ascii::ascii_info_detail
+  test:
+    suffix: 2
+    args: -transform
+  test:
+    suffix: 3
+    args: -interpolate -transform
+  test:
+    suffix: 4
+    requires: exodusii
+    args: -run_type file -filename ${wPETSC_DIR}/share/petsc/datafiles/meshes/simpleblock-100.exo -dm_view ascii::ascii_info_detail -v0 -1.5,-0.5,0.5,-0.5,-0.5,0.5,0.5,-0.5,0.5 -J 0.0,0.0,0.5,0.0,0.5,0.0,-0.5,0.0,0.0,0.0,0.0,0.5,0.0,0.5,0.0,-0.5,0.0,0.0,0.0,0.0,0.5,0.0,0.5,0.0,-0.5,0.0,0.0 -invJ 0.0,0.0,-2.0,0.0,2.0,0.0,2.0,0.0,0.0,0.0,0.0,-2.0,0.0,2.0,0.0,2.0,0.0,0.0,0.0,0.0,-2.0,0.0,2.0,0.0,2.0,0.0,0.0 -detJ 0.125,0.125,0.125
+  test:
+    suffix: 5
+    requires: exodusii
+    args: -interpolate -run_type file -filename ${wPETSC_DIR}/share/petsc/datafiles/meshes/simpleblock-100.exo -dm_view ascii::ascii_info_detail -v0 -1.5,-0.5,0.5,-0.5,-0.5,0.5,0.5,-0.5,0.5 -J 0.0,0.0,0.5,0.0,0.5,0.0,-0.5,0.0,0.0,0.0,0.0,0.5,0.0,0.5,0.0,-0.5,0.0,0.0,0.0,0.0,0.5,0.0,0.5,0.0,-0.5,0.0,0.0 -invJ 0.0,0.0,-2.0,0.0,2.0,0.0,2.0,0.0,0.0,0.0,0.0,-2.0,0.0,2.0,0.0,2.0,0.0,0.0,0.0,0.0,-2.0,0.0,2.0,0.0,2.0,0.0,0.0 -detJ 0.125,0.125,0.125 -centroid -1.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0 -normal 0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0 -vol 1.0,1.0,1.0
+
+TEST*/
