@@ -3,12 +3,9 @@
 #include <petsc/private/vecimpl.h>
 #include <petscviewerhdf5.h>
 
-PETSC_EXTERN PetscErrorCode VecView_Seq(Vec, PetscViewer);
 PETSC_EXTERN PetscErrorCode VecView_MPI(Vec, PetscViewer);
 
 #if defined(PETSC_HAVE_HDF5)
-#undef __FUNCT__
-#define __FUNCT__ "DMSequenceView_HDF5"
 static PetscErrorCode DMSequenceView_HDF5(DM dm, const char *seqname, PetscInt seqnum, PetscScalar value, PetscViewer viewer)
 {
   Vec            stamp;
@@ -39,9 +36,7 @@ static PetscErrorCode DMSequenceView_HDF5(DM dm, const char *seqname, PetscInt s
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSequenceLoad_HDF5"
-PetscErrorCode DMSequenceLoad_HDF5(DM dm, const char *seqname, PetscInt seqnum, PetscScalar *value, PetscViewer viewer)
+PetscErrorCode DMSequenceLoad_HDF5_Internal(DM dm, const char *seqname, PetscInt seqnum, PetscScalar *value, PetscViewer viewer)
 {
   Vec            stamp;
   PetscMPIInt    rank;
@@ -72,9 +67,7 @@ PetscErrorCode DMSequenceLoad_HDF5(DM dm, const char *seqname, PetscInt seqnum,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecView_Plex_Local_HDF5"
-PetscErrorCode VecView_Plex_Local_HDF5(Vec v, PetscViewer viewer)
+PetscErrorCode VecView_Plex_Local_HDF5_Internal(Vec v, PetscViewer viewer)
 {
   DM                      dm;
   DM                      dmBC;
@@ -91,13 +84,13 @@ PetscErrorCode VecView_Plex_Local_HDF5(Vec v, PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject) v, VECSEQ, &isseq);CHKERRQ(ierr);
   ierr = VecGetDM(v, &dm);CHKERRQ(ierr);
-  ierr = DMGetDefaultSection(dm, &section);CHKERRQ(ierr);
+  ierr = DMGetSection(dm, &section);CHKERRQ(ierr);
   ierr = DMGetOutputSequenceNumber(dm, &seqnum, &seqval);CHKERRQ(ierr);
   ierr = PetscViewerHDF5SetTimestep(viewer, seqnum);CHKERRQ(ierr);
   ierr = DMSequenceView_HDF5(dm, "time", seqnum, (PetscScalar) seqval, viewer);CHKERRQ(ierr);
   ierr = PetscViewerGetFormat(viewer, &format);CHKERRQ(ierr);
   ierr = DMGetOutputDM(dm, &dmBC);CHKERRQ(ierr);
-  ierr = DMGetDefaultGlobalSection(dmBC, &sectionGlobal);CHKERRQ(ierr);
+  ierr = DMGetGlobalSection(dmBC, &sectionGlobal);CHKERRQ(ierr);
   ierr = DMGetGlobalVector(dmBC, &gv);CHKERRQ(ierr);
   ierr = PetscObjectGetName((PetscObject) v, &name);CHKERRQ(ierr);
   ierr = PetscObjectSetName((PetscObject) gv, name);CHKERRQ(ierr);
@@ -109,8 +102,10 @@ PetscErrorCode VecView_Plex_Local_HDF5(Vec v, PetscViewer viewer)
   if (format == PETSC_VIEWER_HDF5_VIZ) {
     /* Output visualization representation */
     PetscInt numFields, f;
+    DMLabel  cutLabel;
 
     ierr = PetscSectionGetNumFields(section, &numFields);CHKERRQ(ierr);
+    ierr = DMGetLabel(dm, "periodic_cut", &cutLabel);CHKERRQ(ierr);
     for (f = 0; f < numFields; ++f) {
       Vec         subv;
       IS          is;
@@ -125,14 +120,65 @@ PetscErrorCode VecView_Plex_Local_HDF5(Vec v, PetscViewer viewer)
       ierr = PetscSectionGetFieldName(section, f, &fname);CHKERRQ(ierr);
       if (!fname) continue;
       ierr = PetscViewerHDF5PushGroup(viewer, fgroup);CHKERRQ(ierr);
-      ierr = PetscSectionGetField_Internal(section, sectionGlobal, gv, f, pStart, pEnd, &is, &subv);CHKERRQ(ierr);
-      ierr = PetscStrcpy(subname, name);CHKERRQ(ierr);
-      ierr = PetscStrcat(subname, "_");CHKERRQ(ierr);
-      ierr = PetscStrcat(subname, fname);CHKERRQ(ierr);
+      if (cutLabel) {
+        const PetscScalar *ga;
+        PetscScalar       *suba;
+        PetscInt           Nc, gstart, subSize = 0, extSize = 0, subOff = 0, newOff = 0, p;
+
+        ierr = PetscSectionGetFieldComponents(section, f, &Nc);CHKERRQ(ierr);
+        for (p = pStart; p < pEnd; ++p) {
+          PetscInt gdof, fdof = 0, val;
+
+          ierr = PetscSectionGetDof(sectionGlobal, p, &gdof);CHKERRQ(ierr);
+          if (gdof > 0) {ierr = PetscSectionGetFieldDof(section, p, f, &fdof);CHKERRQ(ierr);}
+          subSize += fdof;
+          ierr = DMLabelGetValue(cutLabel, p, &val);CHKERRQ(ierr);
+          if (val == 1) extSize += fdof;
+        }
+        ierr = VecCreate(PetscObjectComm((PetscObject) gv), &subv);CHKERRQ(ierr);
+        ierr = VecSetSizes(subv, subSize+extSize, PETSC_DETERMINE);CHKERRQ(ierr);
+        ierr = VecSetBlockSize(subv, Nc);CHKERRQ(ierr);
+        ierr = VecSetType(subv, VECSTANDARD);CHKERRQ(ierr);
+        ierr = VecGetOwnershipRange(gv, &gstart, NULL);CHKERRQ(ierr);
+        ierr = VecGetArrayRead(gv, &ga);CHKERRQ(ierr);
+        ierr = VecGetArray(subv, &suba);CHKERRQ(ierr);
+        for (p = pStart; p < pEnd; ++p) {
+          PetscInt gdof, goff, val;
+
+          ierr = PetscSectionGetDof(sectionGlobal, p, &gdof);CHKERRQ(ierr);
+          if (gdof > 0) {
+            PetscInt fdof, fc, f2, poff = 0;
+
+            ierr = PetscSectionGetOffset(sectionGlobal, p, &goff);CHKERRQ(ierr);
+            /* Can get rid of this loop by storing field information in the global section */
+            for (f2 = 0; f2 < f; ++f2) {
+              ierr  = PetscSectionGetFieldDof(section, p, f2, &fdof);CHKERRQ(ierr);
+              poff += fdof;
+            }
+            ierr = PetscSectionGetFieldDof(section, p, f, &fdof);CHKERRQ(ierr);
+            for (fc = 0; fc < fdof; ++fc, ++subOff) suba[subOff] = ga[goff+poff+fc - gstart];
+            ierr = DMLabelGetValue(cutLabel, p, &val);CHKERRQ(ierr);
+            if (val == 1) {
+              for (fc = 0; fc < fdof; ++fc, ++newOff) suba[subSize+newOff] = ga[goff+poff+fc - gstart];
+            }
+          }
+        }
+        ierr = VecRestoreArrayRead(gv, &ga);CHKERRQ(ierr);
+        ierr = VecRestoreArray(subv, &suba);CHKERRQ(ierr);
+      } else {
+        ierr = PetscSectionGetField_Internal(section, sectionGlobal, gv, f, pStart, pEnd, &is, &subv);CHKERRQ(ierr);
+      }
+      ierr = PetscStrncpy(subname, name,sizeof(subname));CHKERRQ(ierr);
+      ierr = PetscStrlcat(subname, "_",sizeof(subname));CHKERRQ(ierr);
+      ierr = PetscStrlcat(subname, fname,sizeof(subname));CHKERRQ(ierr);
       ierr = PetscObjectSetName((PetscObject) subv, subname);CHKERRQ(ierr);
       if (isseq) {ierr = VecView_Seq(subv, viewer);CHKERRQ(ierr);}
       else       {ierr = VecView_MPI(subv, viewer);CHKERRQ(ierr);}
-      ierr = PetscSectionRestoreField_Internal(section, sectionGlobal, gv, f, pStart, pEnd, &is, &subv);CHKERRQ(ierr);
+      if (cutLabel) {
+        ierr = VecDestroy(&subv);CHKERRQ(ierr);
+      } else {
+        ierr = PetscSectionRestoreField_Internal(section, sectionGlobal, gv, f, pStart, pEnd, &is, &subv);CHKERRQ(ierr);
+      }
       ierr = PetscViewerHDF5PopGroup(viewer);CHKERRQ(ierr);
       ierr = PetscSNPrintf(group, PETSC_MAX_PATH_LEN, "%s/%s", fgroup, subname);CHKERRQ(ierr);
       ierr = PetscViewerHDF5HasAttribute(viewer, group, "vector_field_type", &flag);CHKERRQ(ierr);
@@ -149,13 +195,12 @@ PetscErrorCode VecView_Plex_Local_HDF5(Vec v, PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecView_Plex_HDF5"
-PetscErrorCode VecView_Plex_HDF5(Vec v, PetscViewer viewer)
+PetscErrorCode VecView_Plex_HDF5_Internal(Vec v, PetscViewer viewer)
 {
   DM             dm;
   Vec            locv;
   const char    *name;
+  PetscReal      time;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -165,18 +210,32 @@ PetscErrorCode VecView_Plex_HDF5(Vec v, PetscViewer viewer)
   ierr = PetscObjectSetName((PetscObject) locv, name);CHKERRQ(ierr);
   ierr = DMGlobalToLocalBegin(dm, v, INSERT_VALUES, locv);CHKERRQ(ierr);
   ierr = DMGlobalToLocalEnd(dm, v, INSERT_VALUES, locv);CHKERRQ(ierr);
+  ierr = DMGetOutputSequenceNumber(dm, NULL, &time);CHKERRQ(ierr);
+  ierr = DMPlexInsertBoundaryValues(dm, PETSC_TRUE, locv, time, NULL, NULL, NULL);CHKERRQ(ierr);
   ierr = PetscViewerHDF5PushGroup(viewer, "/fields");CHKERRQ(ierr);
   ierr = PetscViewerPushFormat(viewer, PETSC_VIEWER_HDF5_VIZ);CHKERRQ(ierr);
-  ierr = VecView_Plex_Local(locv, viewer);CHKERRQ(ierr);
+  ierr = VecView_Plex_Local_HDF5_Internal(locv, viewer);CHKERRQ(ierr);
   ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
   ierr = PetscViewerHDF5PopGroup(viewer);CHKERRQ(ierr);
   ierr = DMRestoreLocalVector(dm, &locv);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecLoad_Plex_HDF5"
-PetscErrorCode VecLoad_Plex_HDF5(Vec v, PetscViewer viewer)
+PetscErrorCode VecView_Plex_HDF5_Native_Internal(Vec v, PetscViewer viewer)
+{
+  PetscBool      isseq;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectTypeCompare((PetscObject) v, VECSEQ, &isseq);CHKERRQ(ierr);
+  ierr = PetscViewerHDF5PushGroup(viewer, "/fields");CHKERRQ(ierr);
+  if (isseq) {ierr = VecView_Seq(v, viewer);CHKERRQ(ierr);}
+  else       {ierr = VecView_MPI(v, viewer);CHKERRQ(ierr);}
+  ierr = PetscViewerHDF5PopGroup(viewer);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode VecLoad_Plex_HDF5_Internal(Vec v, PetscViewer viewer)
 {
   DM             dm;
   Vec            locv;
@@ -200,8 +259,22 @@ PetscErrorCode VecLoad_Plex_HDF5(Vec v, PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMPlexWriteTopology_HDF5_Static"
+PetscErrorCode VecLoad_Plex_HDF5_Native_Internal(Vec v, PetscViewer viewer)
+{
+  DM             dm;
+  PetscInt       seqnum;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = VecGetDM(v, &dm);CHKERRQ(ierr);
+  ierr = DMGetOutputSequenceNumber(dm, &seqnum, NULL);CHKERRQ(ierr);
+  ierr = PetscViewerHDF5SetTimestep(viewer, seqnum);CHKERRQ(ierr);
+  ierr = PetscViewerHDF5PushGroup(viewer, "/fields");CHKERRQ(ierr);
+  ierr = VecLoad_Default(v, viewer);CHKERRQ(ierr);
+  ierr = PetscViewerHDF5PopGroup(viewer);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 static PetscErrorCode DMPlexWriteTopology_HDF5_Static(DM dm, IS globalPointNumbers, PetscViewer viewer)
 {
   IS              orderIS, conesIS, cellsIS, orntsIS;
@@ -266,14 +339,16 @@ static PetscErrorCode DMPlexWriteTopology_HDF5_Static(DM dm, IS globalPointNumbe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMPlexWriteTopology_Vertices_HDF5_Static"
-static PetscErrorCode DMPlexWriteTopology_Vertices_HDF5_Static(DM dm, DMLabel label, PetscInt labelId, PetscViewer viewer)
+static PetscErrorCode DMPlexWriteTopology_Vertices_HDF5_Static(DM dm, PetscViewer viewer)
 {
+  DMLabel         cutLabel;
+  PetscSection    cSection;
   IS              cellIS, globalVertexNumbers;
   const PetscInt *gvertex;
   PetscInt       *vertices;
-  PetscInt        dim, depth, vStart, vEnd, v, cellHeight, cStart, cMax, cEnd, cell, conesSize = 0, numCornersLocal = 0, numCorners;
+  IS              cutvertices;
+  const PetscInt *cutverts;
+  PetscInt        dim, depth, vStart, vEnd, vExtra = 0, v, cellHeight, cStart, cMax, cEnd, cell, conesSize = 0, numCornersLocal = 0, numCorners;
   hid_t           fileId, groupId;
   PetscErrorCode  ierr;
 
@@ -281,20 +356,15 @@ static PetscErrorCode DMPlexWriteTopology_Vertices_HDF5_Static(DM dm, DMLabel la
   ierr = DMGetDimension(dm, &dim);CHKERRQ(ierr);
   ierr = DMPlexGetDepth(dm, &depth);CHKERRQ(ierr);
   ierr = DMPlexGetDepthStratum(dm, 0, &vStart, &vEnd);CHKERRQ(ierr);
+  ierr = DMGetCoordinateSection(dm, &cSection);CHKERRQ(ierr);
   ierr = DMPlexGetVTKCellHeight(dm, &cellHeight);CHKERRQ(ierr);
   ierr = DMPlexGetHeightStratum(dm, cellHeight, &cStart, &cEnd);CHKERRQ(ierr);
-  ierr = DMPlexGetHybridBounds(dm, &cMax, PETSC_NULL, PETSC_NULL, PETSC_NULL);CHKERRQ(ierr);
+  ierr = DMPlexGetHybridBounds(dm, &cMax, NULL, NULL, NULL);CHKERRQ(ierr);
   if (cMax >= 0) cEnd = PetscMin(cEnd, cMax);
-  if (depth == 1) PetscFunctionReturn(0);
   for (cell = cStart; cell < cEnd; ++cell) {
     PetscInt *closure = NULL;
     PetscInt  closureSize, v, Nc = 0;
 
-    if (label) {
-      PetscInt value;
-      ierr = DMLabelGetValue(label, cell, &value);CHKERRQ(ierr);
-      if (value == labelId) continue;
-    }
     ierr = DMPlexGetTransitiveClosure(dm, cell, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
     for (v = 0; v < closureSize*2; v += 2) {
       if ((closure[v] >= vStart) && (closure[v] < vEnd)) ++Nc;
@@ -304,7 +374,7 @@ static PetscErrorCode DMPlexWriteTopology_Vertices_HDF5_Static(DM dm, DMLabel la
     if (!numCornersLocal)           numCornersLocal = Nc;
     else if (numCornersLocal != Nc) numCornersLocal = 1;
   }
-  ierr = MPI_Allreduce(&numCornersLocal, &numCorners, 1, MPIU_INT, MPI_MAX, PetscObjectComm((PetscObject) dm));CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(&numCornersLocal, &numCorners, 1, MPIU_INT, MPI_MAX, PetscObjectComm((PetscObject) dm));CHKERRQ(ierr);
   if (numCornersLocal && (numCornersLocal != numCorners || numCorners == 1)) SETERRQ(PETSC_COMM_SELF, PETSC_ERR_SUP, "Visualization topology currently only supports identical cell shapes");
 
   ierr = PetscViewerHDF5PushGroup(viewer, "/viz");CHKERRQ(ierr);
@@ -312,17 +382,24 @@ static PetscErrorCode DMPlexWriteTopology_Vertices_HDF5_Static(DM dm, DMLabel la
   PetscStackCallHDF5(H5Gclose,(groupId));
   ierr = PetscViewerHDF5PopGroup(viewer);CHKERRQ(ierr);
 
+  ierr = DMGetLabel(dm, "periodic_cut", &cutLabel);CHKERRQ(ierr);
+  if (cutLabel) {
+    ierr = DMLabelGetStratumIS(cutLabel, 1, &cutvertices);CHKERRQ(ierr);
+    ierr = ISGetIndices(cutvertices, &cutverts);CHKERRQ(ierr);
+    ierr = ISGetLocalSize(cutvertices, &vExtra);CHKERRQ(ierr);
+  }
   ierr = DMPlexGetVertexNumbering(dm, &globalVertexNumbers);CHKERRQ(ierr);
   ierr = ISGetIndices(globalVertexNumbers, &gvertex);CHKERRQ(ierr);
   ierr = PetscMalloc1(conesSize, &vertices);CHKERRQ(ierr);
   for (cell = cStart, v = 0; cell < cEnd; ++cell) {
     PetscInt *closure = NULL;
     PetscInt  closureSize, Nc = 0, p;
+    PetscBool replace = PETSC_FALSE;
 
-    if (label) {
+    if (cutLabel) {
       PetscInt value;
-      ierr = DMLabelGetValue(label, cell, &value);CHKERRQ(ierr);
-      if (value == labelId) continue;
+      ierr = DMLabelGetValue(cutLabel, cell, &value);CHKERRQ(ierr);
+      if (value == 2) replace = PETSC_TRUE;
     }
     ierr = DMPlexGetTransitiveClosure(dm, cell, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
     for (p = 0; p < closureSize*2; p += 2) {
@@ -334,33 +411,31 @@ static PetscErrorCode DMPlexWriteTopology_Vertices_HDF5_Static(DM dm, DMLabel la
     for (p = 0; p < Nc; ++p) {
       const PetscInt gv = gvertex[closure[p] - vStart];
       vertices[v++] = gv < 0 ? -(gv+1) : gv;
+      if (replace) {
+        PetscInt newv;
+        ierr = PetscFindInt(closure[p], vExtra, cutverts, &newv);CHKERRQ(ierr);
+        if (newv >= 0) vertices[v-1] = vEnd - vStart + newv;
+      }
     }
     ierr = DMPlexRestoreTransitiveClosure(dm, cell, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
   }
+  if (cutLabel) {
+    ierr = ISRestoreIndices(cutvertices, &cutverts);CHKERRQ(ierr);
+    ierr = ISDestroy(&cutvertices);CHKERRQ(ierr);
+  }
   if (v != conesSize) SETERRQ2(PETSC_COMM_SELF, PETSC_ERR_LIB, "Total number of cell vertices %d != %d", v, conesSize);
   ierr = ISCreateGeneral(PetscObjectComm((PetscObject) dm), conesSize, vertices, PETSC_OWN_POINTER, &cellIS);CHKERRQ(ierr);
   ierr = PetscLayoutSetBlockSize(cellIS->map, numCorners);CHKERRQ(ierr);
   ierr = PetscObjectSetName((PetscObject) cellIS, "cells");CHKERRQ(ierr);
   ierr = PetscViewerHDF5PushGroup(viewer, "/viz/topology");CHKERRQ(ierr);
   ierr = ISView(cellIS, viewer);CHKERRQ(ierr);
-#if 0
-  if (numCorners == 1) {
-    ierr = ISView(coneIS, viewer);CHKERRQ(ierr);
-  } else {
-    ierr = PetscViewerHDF5WriteAttribute(viewer, "/viz/topology/cells", "cell_corners", PETSC_INT, (void *) &numCorners);CHKERRQ(ierr);
-  }
-  ierr = ISDestroy(&coneIS);CHKERRQ(ierr);
-#else
   ierr = PetscViewerHDF5WriteAttribute(viewer, "/viz/topology/cells", "cell_corners", PETSC_INT, (void *) &numCorners);CHKERRQ(ierr);
-#endif
   ierr = ISDestroy(&cellIS);CHKERRQ(ierr);
 
   ierr = PetscViewerHDF5WriteAttribute(viewer, "/viz/topology/cells", "cell_dim", PETSC_INT, (void *) &dim);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMPlexWriteCoordinates_HDF5_Static"
 static PetscErrorCode DMPlexWriteCoordinates_HDF5_Static(DM dm, PetscViewer viewer)
 {
   DM             cdm;
@@ -393,17 +468,17 @@ static PetscErrorCode DMPlexWriteCoordinates_HDF5_Static(DM dm, PetscViewer view
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMPlexWriteCoordinates_Vertices_HDF5_Static"
 static PetscErrorCode DMPlexWriteCoordinates_Vertices_HDF5_Static(DM dm, PetscViewer viewer)
 {
   Vec              coordinates, newcoords;
   PetscSection     cSection;
   PetscScalar     *coords, *ncoords;
+  DMLabel          cutLabel;
   const PetscReal *L;
   const DMBoundaryType *bd;
   PetscReal        lengthScale;
-  PetscInt         vStart, vEnd, v, bs, coordSize, dof, off, d;
+  PetscInt         vStart, vEnd, vExtra = 0, v, bs, coordSize, dof, off, d;
+  PetscBool        localized, embedded;
   hid_t            fileId, groupId;
   PetscErrorCode   ierr;
 
@@ -413,44 +488,110 @@ static PetscErrorCode DMPlexWriteCoordinates_Vertices_HDF5_Static(DM dm, PetscVi
   ierr = DMGetCoordinatesLocal(dm, &coordinates);CHKERRQ(ierr);
   ierr = VecGetBlockSize(coordinates, &bs);CHKERRQ(ierr);
   ierr = VecGetLocalSize(coordinates, &coordSize);CHKERRQ(ierr);
-  if (coordSize == (vEnd - vStart)*bs) PetscFunctionReturn(0);
-  ierr = DMGetPeriodicity(dm, NULL, &L, &bd);CHKERRQ(ierr);
+  ierr = DMGetCoordinatesLocalized(dm,&localized);CHKERRQ(ierr);
+  if (localized == PETSC_FALSE) PetscFunctionReturn(0);
+  ierr = DMGetLabel(dm, "periodic_cut", &cutLabel);CHKERRQ(ierr);
+  if (cutLabel) {
+    IS              vertices;
+    const PetscInt *verts;
+    PetscInt        n;
+
+    ierr = DMLabelGetStratumIS(cutLabel, 1, &vertices);CHKERRQ(ierr);
+    ierr = ISGetIndices(vertices, &verts);CHKERRQ(ierr);
+    ierr = ISGetLocalSize(vertices, &n);CHKERRQ(ierr);
+    for (v = 0; v < n; ++v) {
+      if ((verts[v] >= vStart) && (verts[v] < vEnd)) ++vExtra;
+    }
+    ierr = ISRestoreIndices(vertices, &verts);CHKERRQ(ierr);
+    ierr = ISDestroy(&vertices);CHKERRQ(ierr);
+  }
+  ierr = DMGetPeriodicity(dm, NULL, NULL, &L, &bd);CHKERRQ(ierr);
   ierr = DMGetCoordinateSection(dm, &cSection);CHKERRQ(ierr);
   ierr = VecCreate(PetscObjectComm((PetscObject) coordinates), &newcoords);CHKERRQ(ierr);
+  ierr = PetscSectionGetDof(cSection, vStart, &dof);CHKERRQ(ierr);
+  embedded  = (PetscBool) (L && dof == 2 && !cutLabel);
   coordSize = 0;
+  coordSize += dof*vExtra;
   for (v = vStart; v < vEnd; ++v) {
     ierr = PetscSectionGetDof(cSection, v, &dof);CHKERRQ(ierr);
-    if (L && dof == 2) coordSize += dof+1;
-    else               coordSize += dof;
+    if (embedded) coordSize += dof+1;
+    else          coordSize += dof;
   }
-  if (L && bs == 2) {ierr = VecSetBlockSize(newcoords, bs+1);CHKERRQ(ierr);}
-  else              {ierr = VecSetBlockSize(newcoords, bs);CHKERRQ(ierr);}
+  if (embedded) {ierr = VecSetBlockSize(newcoords, bs+1);CHKERRQ(ierr);}
+  else          {ierr = VecSetBlockSize(newcoords, bs);CHKERRQ(ierr);}
   ierr = VecSetSizes(newcoords, coordSize, PETSC_DETERMINE);CHKERRQ(ierr);
-  ierr = VecSetType(newcoords,VECSTANDARD);CHKERRQ(ierr);
+  ierr = VecSetType(newcoords, VECSTANDARD);CHKERRQ(ierr);
   ierr = VecGetArray(coordinates, &coords);CHKERRQ(ierr);
   ierr = VecGetArray(newcoords,   &ncoords);CHKERRQ(ierr);
   coordSize = 0;
   for (v = vStart; v < vEnd; ++v) {
     ierr = PetscSectionGetDof(cSection, v, &dof);CHKERRQ(ierr);
     ierr = PetscSectionGetOffset(cSection, v, &off);CHKERRQ(ierr);
-    if (L && dof == 2) {
-      /* Need to do torus */
-      if ((bd[0] == DM_BOUNDARY_PERIODIC) && (bd[1] == DM_BOUNDARY_PERIODIC)) {SETERRQ(PetscObjectComm((PetscObject) dm), PETSC_ERR_SUP, "Cannot embed doubly periodic domain yet");}
-      else if ((bd[0] == DM_BOUNDARY_PERIODIC)) {
+    if (embedded) {
+      if ((bd[0] == DM_BOUNDARY_PERIODIC) && (bd[1] == DM_BOUNDARY_PERIODIC)) {
+        PetscReal theta, phi, r, R;
+        /* XY-periodic */
+        /* Suppose its an y-z circle, then
+             \hat r = (0, cos(th), sin(th)) \hat x = (1, 0, 0)
+           and the circle in that plane is
+             \hat r cos(phi) + \hat x sin(phi) */
+        theta = 2.0*PETSC_PI*PetscRealPart(coords[off+1])/L[1];
+        phi   = 2.0*PETSC_PI*PetscRealPart(coords[off+0])/L[0];
+        r     = L[0]/(2.0*PETSC_PI * 2.0*L[1]);
+        R     = L[1]/(2.0*PETSC_PI);
+        ncoords[coordSize++] =  PetscSinReal(phi) * r;
+        ncoords[coordSize++] = -PetscCosReal(theta) * (R + r * PetscCosReal(phi));
+        ncoords[coordSize++] =  PetscSinReal(theta) * (R + r * PetscCosReal(phi));
+      } else if ((bd[0] == DM_BOUNDARY_PERIODIC)) {
         /* X-periodic */
-        ncoords[coordSize++] = -cos(2.0*PETSC_PI*coords[off+0]/L[0])*(L[0]/(2.0*PETSC_PI));
+        ncoords[coordSize++] = -PetscCosReal(2.0*PETSC_PI*PetscRealPart(coords[off+0])/L[0])*(L[0]/(2.0*PETSC_PI));
         ncoords[coordSize++] = coords[off+1];
-        ncoords[coordSize++] = sin(2.0*PETSC_PI*coords[off+0]/L[0])*(L[0]/(2.0*PETSC_PI));
+        ncoords[coordSize++] = PetscSinReal(2.0*PETSC_PI*PetscRealPart(coords[off+0])/L[0])*(L[0]/(2.0*PETSC_PI));
       } else if ((bd[1] == DM_BOUNDARY_PERIODIC)) {
         /* Y-periodic */
         ncoords[coordSize++] = coords[off+0];
-        ncoords[coordSize++] = sin(2.0*PETSC_PI*coords[off+1]/L[1])*(L[1]/(2.0*PETSC_PI));
-        ncoords[coordSize++] = -cos(2.0*PETSC_PI*coords[off+1]/L[1])*(L[1]/(2.0*PETSC_PI));
+        ncoords[coordSize++] = PetscSinReal(2.0*PETSC_PI*PetscRealPart(coords[off+1])/L[1])*(L[1]/(2.0*PETSC_PI));
+        ncoords[coordSize++] = -PetscCosReal(2.0*PETSC_PI*PetscRealPart(coords[off+1])/L[1])*(L[1]/(2.0*PETSC_PI));
+      } else if ((bd[0] == DM_BOUNDARY_TWIST)) {
+        PetscReal phi, r, R;
+        /* Mobius strip */
+        /* Suppose its an x-z circle, then
+             \hat r = (-cos(phi), 0, sin(phi)) \hat y = (0, 1, 0)
+           and in that plane we rotate by pi as we go around the circle
+             \hat r cos(phi/2) + \hat y sin(phi/2) */
+        phi   = 2.0*PETSC_PI*PetscRealPart(coords[off+0])/L[0];
+        R     = L[0];
+        r     = PetscRealPart(coords[off+1]) - L[1]/2.0;
+        ncoords[coordSize++] = -PetscCosReal(phi) * (R + r * PetscCosReal(phi/2.0));
+        ncoords[coordSize++] =  PetscSinReal(phi/2.0) * r;
+        ncoords[coordSize++] =  PetscSinReal(phi) * (R + r * PetscCosReal(phi/2.0));
       } else SETERRQ(PetscObjectComm((PetscObject) dm), PETSC_ERR_SUP, "Cannot handle periodicity in this domain");
     } else {
-      for (d = 0; d < dof; ++d, ++coordSize) ncoords[coordSize] = coords[off+d];
+      if (cutLabel) {
+        ierr = DMLocalizeCoordinate(dm, &coords[off], PETSC_TRUE, &ncoords[coordSize]);CHKERRQ(ierr);
+        coordSize += dof;
+      } else {
+        for (d = 0; d < dof; ++d, ++coordSize) ncoords[coordSize] = coords[off+d];
+      }
     }
   }
+  if (cutLabel) {
+    IS              vertices;
+    const PetscInt *verts;
+    PetscInt        n;
+
+    ierr = DMLabelGetStratumIS(cutLabel, 1, &vertices);CHKERRQ(ierr);
+    ierr = ISGetIndices(vertices, &verts);CHKERRQ(ierr);
+    ierr = ISGetLocalSize(vertices, &n);CHKERRQ(ierr);
+    for (v = 0; v < n; ++v) {
+      if ((verts[v] < vStart) || (verts[v] >= vEnd)) continue;
+      ierr = PetscSectionGetDof(cSection, verts[v], &dof);CHKERRQ(ierr);
+      ierr = PetscSectionGetOffset(cSection, verts[v], &off);CHKERRQ(ierr);
+      for (d = 0; d < dof; ++d, ++coordSize) ncoords[coordSize] = coords[off+d] + ((bd[d] == DM_BOUNDARY_PERIODIC) ? L[d] : 0.0);
+    }
+    ierr = ISRestoreIndices(vertices, &verts);CHKERRQ(ierr);
+    ierr = ISDestroy(&vertices);CHKERRQ(ierr);
+  }
   ierr = VecRestoreArray(coordinates, &coords);CHKERRQ(ierr);
   ierr = VecRestoreArray(newcoords,   &ncoords);CHKERRQ(ierr);
   ierr = PetscObjectSetName((PetscObject) newcoords, "vertices");CHKERRQ(ierr);
@@ -466,75 +607,68 @@ static PetscErrorCode DMPlexWriteCoordinates_Vertices_HDF5_Static(DM dm, PetscVi
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMPlexView_HDF5"
-/* We only write cells and vertices. Does this screw up parallel reading? */
-PetscErrorCode DMPlexView_HDF5(DM dm, PetscViewer viewer)
+
+static PetscErrorCode DMPlexWriteLabels_HDF5_Static(DM dm, IS globalPointNumbers, PetscViewer viewer)
 {
-  DMLabel           label   = NULL;
-  PetscInt          labelId = 0;
-  IS                globalPointNumbers;
   const PetscInt   *gpoint;
   PetscInt          numLabels, l;
   hid_t             fileId, groupId;
-  PetscViewerFormat format;
   PetscErrorCode    ierr;
 
   PetscFunctionBegin;
-  ierr = DMPlexCreatePointNumbering(dm, &globalPointNumbers);CHKERRQ(ierr);
-  ierr = PetscViewerGetFormat(viewer, &format);CHKERRQ(ierr);
-  ierr = DMPlexWriteCoordinates_HDF5_Static(dm, viewer);CHKERRQ(ierr);
-  if (format == PETSC_VIEWER_HDF5_VIZ) {ierr = DMPlexWriteCoordinates_Vertices_HDF5_Static(dm, viewer);CHKERRQ(ierr);}
-  ierr = DMPlexWriteTopology_HDF5_Static(dm, globalPointNumbers, viewer);CHKERRQ(ierr);
-  if (format == PETSC_VIEWER_HDF5_VIZ) {ierr = DMPlexWriteTopology_Vertices_HDF5_Static(dm, label, labelId, viewer);CHKERRQ(ierr);}
-  /* Write Labels*/
   ierr = ISGetIndices(globalPointNumbers, &gpoint);CHKERRQ(ierr);
   ierr = PetscViewerHDF5PushGroup(viewer, "/labels");CHKERRQ(ierr);
   ierr = PetscViewerHDF5OpenGroup(viewer, &fileId, &groupId);CHKERRQ(ierr);
   if (groupId != fileId) PetscStackCallHDF5(H5Gclose,(groupId));
   ierr = PetscViewerHDF5PopGroup(viewer);CHKERRQ(ierr);
-  ierr = DMPlexGetNumLabels(dm, &numLabels);CHKERRQ(ierr);
+  ierr = DMGetNumLabels(dm, &numLabels);CHKERRQ(ierr);
   for (l = 0; l < numLabels; ++l) {
     DMLabel         label;
     const char     *name;
-    IS              valueIS, globalValueIS;
+    IS              valueIS, pvalueIS, globalValueIS;
     const PetscInt *values;
     PetscInt        numValues, v;
     PetscBool       isDepth, output;
     char            group[PETSC_MAX_PATH_LEN];
 
-    ierr = DMPlexGetLabelName(dm, l, &name);CHKERRQ(ierr);
-    ierr = DMPlexGetLabelOutput(dm, name, &output);CHKERRQ(ierr);
+    ierr = DMGetLabelName(dm, l, &name);CHKERRQ(ierr);
+    ierr = DMGetLabelOutput(dm, name, &output);CHKERRQ(ierr);
     ierr = PetscStrncmp(name, "depth", 10, &isDepth);CHKERRQ(ierr);
     if (isDepth || !output) continue;
-    ierr = DMPlexGetLabel(dm, name, &label);CHKERRQ(ierr);
+    ierr = DMGetLabel(dm, name, &label);CHKERRQ(ierr);
     ierr = PetscSNPrintf(group, PETSC_MAX_PATH_LEN, "/labels/%s", name);CHKERRQ(ierr);
     ierr = PetscViewerHDF5PushGroup(viewer, group);CHKERRQ(ierr);
     ierr = PetscViewerHDF5OpenGroup(viewer, &fileId, &groupId);CHKERRQ(ierr);
     if (groupId != fileId) PetscStackCallHDF5(H5Gclose,(groupId));
     ierr = PetscViewerHDF5PopGroup(viewer);CHKERRQ(ierr);
     ierr = DMLabelGetValueIS(label, &valueIS);CHKERRQ(ierr);
-    ierr = ISAllGather(valueIS, &globalValueIS);CHKERRQ(ierr);
+    /* Must copy to a new IS on the global comm */
+    ierr = ISGetLocalSize(valueIS, &numValues);CHKERRQ(ierr);
+    ierr = ISGetIndices(valueIS, &values);CHKERRQ(ierr);
+    ierr = ISCreateGeneral(PetscObjectComm((PetscObject) dm), numValues, values, PETSC_COPY_VALUES, &pvalueIS);CHKERRQ(ierr);
+    ierr = ISRestoreIndices(valueIS, &values);CHKERRQ(ierr);
+    ierr = ISAllGather(pvalueIS, &globalValueIS);CHKERRQ(ierr);
+    ierr = ISDestroy(&pvalueIS);CHKERRQ(ierr);
     ierr = ISSortRemoveDups(globalValueIS);CHKERRQ(ierr);
     ierr = ISGetLocalSize(globalValueIS, &numValues);CHKERRQ(ierr);
     ierr = ISGetIndices(globalValueIS, &values);CHKERRQ(ierr);
     for (v = 0; v < numValues; ++v) {
       IS              stratumIS, globalStratumIS;
-      const PetscInt *spoints;
-      PetscInt       *gspoints, n, gn, p;
-      const char     *iname;
+      const PetscInt *spoints = NULL;
+      PetscInt       *gspoints, n = 0, gn, p;
+      const char     *iname = "indices";
 
       ierr = PetscSNPrintf(group, PETSC_MAX_PATH_LEN, "/labels/%s/%d", name, values[v]);CHKERRQ(ierr);
       ierr = DMLabelGetStratumIS(label, values[v], &stratumIS);CHKERRQ(ierr);
 
-      ierr = ISGetLocalSize(stratumIS, &n);CHKERRQ(ierr);
-      ierr = ISGetIndices(stratumIS, &spoints);CHKERRQ(ierr);
+      if (stratumIS) {ierr = ISGetLocalSize(stratumIS, &n);CHKERRQ(ierr);}
+      if (stratumIS) {ierr = ISGetIndices(stratumIS, &spoints);CHKERRQ(ierr);}
       for (gn = 0, p = 0; p < n; ++p) if (gpoint[spoints[p]] >= 0) ++gn;
       ierr = PetscMalloc1(gn,&gspoints);CHKERRQ(ierr);
       for (gn = 0, p = 0; p < n; ++p) if (gpoint[spoints[p]] >= 0) gspoints[gn++] = gpoint[spoints[p]];
-      ierr = ISRestoreIndices(stratumIS, &spoints);CHKERRQ(ierr);
+      if (stratumIS) {ierr = ISRestoreIndices(stratumIS, &spoints);CHKERRQ(ierr);}
       ierr = ISCreateGeneral(PetscObjectComm((PetscObject) dm), gn, gspoints, PETSC_OWN_POINTER, &globalStratumIS);CHKERRQ(ierr);
-      ierr = PetscObjectGetName((PetscObject) stratumIS, &iname);CHKERRQ(ierr);
+      if (stratumIS) {ierr = PetscObjectGetName((PetscObject) stratumIS, &iname);CHKERRQ(ierr);}
       ierr = PetscObjectSetName((PetscObject) globalStratumIS, iname);CHKERRQ(ierr);
 
       ierr = PetscViewerHDF5PushGroup(viewer, group);CHKERRQ(ierr);
@@ -548,6 +682,48 @@ PetscErrorCode DMPlexView_HDF5(DM dm, PetscViewer viewer)
     ierr = ISDestroy(&valueIS);CHKERRQ(ierr);
   }
   ierr = ISRestoreIndices(globalPointNumbers, &gpoint);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/* We only write cells and vertices. Does this screw up parallel reading? */
+PetscErrorCode DMPlexView_HDF5_Internal(DM dm, PetscViewer viewer)
+{
+  IS                globalPointNumbers;
+  PetscViewerFormat format;
+  PetscBool         viz_geom=PETSC_FALSE, xdmf_topo=PETSC_FALSE, petsc_topo=PETSC_FALSE;
+  PetscErrorCode    ierr;
+
+  PetscFunctionBegin;
+  ierr = DMPlexCreatePointNumbering(dm, &globalPointNumbers);CHKERRQ(ierr);
+  ierr = DMPlexWriteCoordinates_HDF5_Static(dm, viewer);CHKERRQ(ierr);
+  ierr = DMPlexWriteLabels_HDF5_Static(dm, globalPointNumbers, viewer);CHKERRQ(ierr);
+
+  ierr = PetscViewerGetFormat(viewer, &format);CHKERRQ(ierr);
+  switch (format) {
+    case PETSC_VIEWER_HDF5_VIZ:
+      viz_geom    = PETSC_TRUE;
+      xdmf_topo   = PETSC_TRUE;
+      break;
+    case PETSC_VIEWER_HDF5_XDMF:
+      xdmf_topo   = PETSC_TRUE;
+      break;
+    case PETSC_VIEWER_HDF5_PETSC: 
+      petsc_topo  = PETSC_TRUE;
+      break;
+    case PETSC_VIEWER_DEFAULT:
+    case PETSC_VIEWER_NATIVE:
+      viz_geom    = PETSC_TRUE;
+      xdmf_topo   = PETSC_TRUE;
+      petsc_topo  = PETSC_TRUE;
+      break;
+    default:
+      SETERRQ1(PetscObjectComm((PetscObject)dm), PETSC_ERR_SUP, "PetscViewerFormat %s not supported for HDF5 output.", PetscViewerFormats[format]);
+  }
+
+  if (viz_geom)   {ierr = DMPlexWriteCoordinates_Vertices_HDF5_Static(dm, viewer);CHKERRQ(ierr);}
+  if (xdmf_topo)  {ierr = DMPlexWriteTopology_Vertices_HDF5_Static(dm, viewer);CHKERRQ(ierr);}
+  if (petsc_topo) {ierr = DMPlexWriteTopology_HDF5_Static(dm, globalPointNumbers, viewer);CHKERRQ(ierr);}
+
   ierr = ISDestroy(&globalPointNumbers);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -599,20 +775,36 @@ static herr_t ReadLabelHDF5_Static(hid_t g_id, const char *name, const H5L_info_
   PetscErrorCode ierr;
   herr_t         err;
 
-  ierr = DMPlexCreateLabel(dm, name); if (ierr) return (herr_t) ierr;
-  ierr = DMPlexGetLabel(dm, name, &((LabelCtx *) op_data)->label); if (ierr) return (herr_t) ierr;
+  ierr = DMCreateLabel(dm, name); if (ierr) return (herr_t) ierr;
+  ierr = DMGetLabel(dm, name, &((LabelCtx *) op_data)->label); if (ierr) return (herr_t) ierr;
   PetscStackCall("H5Literate_by_name",err = H5Literate_by_name(g_id, name, H5_INDEX_NAME, H5_ITER_NATIVE, &idx, ReadLabelStratumHDF5_Static, op_data, 0));
   return err;
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMPlexLoad_HDF5"
+PetscErrorCode DMPlexLoadLabels_HDF5_Internal(DM dm, PetscViewer viewer)
+{
+  LabelCtx        ctx;
+  hid_t           fileId, groupId;
+  hsize_t         idx = 0;
+  PetscErrorCode  ierr;
+
+  PetscFunctionBegin;
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject) dm), &ctx.rank);CHKERRQ(ierr);
+  ctx.dm     = dm;
+  ctx.viewer = viewer;
+  ierr = PetscViewerHDF5PushGroup(viewer, "/labels");CHKERRQ(ierr);
+  ierr = PetscViewerHDF5OpenGroup(viewer, &fileId, &groupId);CHKERRQ(ierr);
+  PetscStackCallHDF5(H5Literate,(groupId, H5_INDEX_NAME, H5_ITER_NATIVE, &idx, ReadLabelHDF5_Static, &ctx));
+  PetscStackCallHDF5(H5Gclose,(groupId));
+  ierr = PetscViewerHDF5PopGroup(viewer);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /* The first version will read everything onto proc 0, letting the user distribute
    The next will create a naive partition, and then rebalance after reading
 */
-PetscErrorCode DMPlexLoad_HDF5(DM dm, PetscViewer viewer)
+PetscErrorCode DMPlexLoad_HDF5_Internal(DM dm, PetscViewer viewer)
 {
-  LabelCtx        ctx;
   PetscSection    coordSection;
   Vec             coordinates;
   IS              orderIS, conesIS, cellsIS, orntsIS;
@@ -620,8 +812,6 @@ PetscErrorCode DMPlexLoad_HDF5(DM dm, PetscViewer viewer)
   PetscReal       lengthScale;
   PetscInt       *cone, *ornt;
   PetscInt        dim, spatialDim, N, numVertices, vStart, vEnd, v, pEnd, p, q, maxConeSize = 0, c;
-  hid_t           fileId, groupId;
-  hsize_t         idx = 0;
   PetscMPIInt     rank;
   PetscErrorCode  ierr;
 
@@ -720,15 +910,8 @@ PetscErrorCode DMPlexLoad_HDF5(DM dm, PetscViewer viewer)
   ierr = PetscSectionSetUp(coordSection);CHKERRQ(ierr);
   ierr = DMSetCoordinates(dm, coordinates);CHKERRQ(ierr);
   ierr = VecDestroy(&coordinates);CHKERRQ(ierr);
-  /* Read Labels*/
-  ctx.rank   = rank;
-  ctx.dm     = dm;
-  ctx.viewer = viewer;
-  ierr = PetscViewerHDF5PushGroup(viewer, "/labels");CHKERRQ(ierr);
-  ierr = PetscViewerHDF5OpenGroup(viewer, &fileId, &groupId);CHKERRQ(ierr);
-  PetscStackCallHDF5(H5Literate,(groupId, H5_INDEX_NAME, H5_ITER_NATIVE, &idx, ReadLabelHDF5_Static, &ctx));
-  PetscStackCallHDF5(H5Gclose,(groupId));
-  ierr = PetscViewerHDF5PopGroup(viewer);CHKERRQ(ierr);
+  /* Read Labels */
+  ierr = DMPlexLoadLabels_HDF5_Internal(dm, viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 #endif
