@@ -1,4 +1,4 @@
-#include <petsc-private/dmpleximpl.h>
+#include <petsc/private/dmpleximpl.h>
 #include <../src/sys/classes/viewer/impls/vtk/vtkvimpl.h>
 
 typedef struct {
@@ -15,19 +15,15 @@ static const char precision[] = "Float64";
 static const char precision[] = "UnknownPrecision";
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "TransferWrite"
-static PetscErrorCode TransferWrite(PetscViewer viewer,FILE *fp,PetscMPIInt srank,PetscMPIInt root,const void *send,void *recv,PetscMPIInt count,PetscDataType datatype,PetscMPIInt tag)
+static PetscErrorCode TransferWrite(PetscViewer viewer,FILE *fp,PetscMPIInt srank,PetscMPIInt root,const void *send,void *recv,PetscMPIInt count,MPI_Datatype mpidatatype,PetscMPIInt tag)
 {
   PetscMPIInt    rank;
   PetscErrorCode ierr;
   MPI_Comm       comm;
-  MPI_Datatype   mpidatatype;
 
   PetscFunctionBegin;
   ierr = PetscObjectGetComm((PetscObject)viewer,&comm);CHKERRQ(ierr);
   ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
-  ierr = PetscDataTypeToMPIDataType(datatype,&mpidatatype);CHKERRQ(ierr);
 
   if (rank == srank && rank != root) {
     ierr = MPI_Send((void*)send,count,mpidatatype,root,tag,comm);CHKERRQ(ierr);
@@ -43,58 +39,67 @@ static PetscErrorCode TransferWrite(PetscViewer viewer,FILE *fp,PetscMPIInt sran
       if (count != nrecv) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Array size mismatch");
       buffer = recv;
     }
-    ierr = PetscViewerVTKFWrite(viewer,fp,buffer,count,datatype);CHKERRQ(ierr);
+    ierr = PetscViewerVTKFWrite(viewer,fp,buffer,count,mpidatatype);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMPlexGetVTKConnectivity"
-static PetscErrorCode DMPlexGetVTKConnectivity(DM dm,PieceInfo *piece,PetscVTKInt **oconn,PetscVTKInt **ooffsets,PetscVTKType **otypes)
+static PetscErrorCode DMPlexGetVTKConnectivity(DM dm, PetscBool localized, PieceInfo *piece,PetscVTKInt **oconn,PetscVTKInt **ooffsets,PetscVTKType **otypes)
 {
   PetscErrorCode ierr;
+  PetscSection   coordSection;
   PetscVTKInt    *conn,*offsets;
   PetscVTKType   *types;
-  PetscInt       dim,vStart,vEnd,cStart,cEnd,pStart,pEnd,cellHeight,cMax,numLabelCells,hasLabel,c,v,countcell,countconn;
+  PetscInt       dim,vStart,vEnd,cStart,cEnd,pStart,pEnd,cellHeight,numLabelCells,hasLabel,c,v,countcell,countconn;
 
   PetscFunctionBegin;
-  ierr = PetscMalloc3(piece->nconn,PetscVTKInt,&conn,piece->ncells,PetscVTKInt,&offsets,piece->ncells,PetscVTKType,&types);CHKERRQ(ierr);
-
-  ierr = DMPlexGetDimension(dm,&dim);CHKERRQ(ierr);
+  ierr = PetscMalloc3(piece->nconn,&conn,piece->ncells,&offsets,piece->ncells,&types);CHKERRQ(ierr);
+  ierr = DMGetCoordinateSection(dm, &coordSection);CHKERRQ(ierr);
+  ierr = DMGetDimension(dm,&dim);CHKERRQ(ierr);
   ierr = DMPlexGetChart(dm,&pStart,&pEnd);CHKERRQ(ierr);
   ierr = DMPlexGetVTKCellHeight(dm, &cellHeight);CHKERRQ(ierr);
   ierr = DMPlexGetHeightStratum(dm, cellHeight, &cStart, &cEnd);CHKERRQ(ierr);
   ierr = DMPlexGetDepthStratum(dm, 0, &vStart, &vEnd);CHKERRQ(ierr);
-  ierr = DMPlexGetHybridBounds(dm, &cMax, NULL, NULL, NULL);CHKERRQ(ierr);
-  if (cMax >= 0) cEnd = PetscMin(cEnd, cMax);
-  ierr     = DMPlexGetStratumSize(dm, "vtk", 1, &numLabelCells);CHKERRQ(ierr);
+  ierr     = DMGetStratumSize(dm, "vtk", 1, &numLabelCells);CHKERRQ(ierr);
   hasLabel = numLabelCells > 0 ? PETSC_TRUE : PETSC_FALSE;
 
   countcell = 0;
   countconn = 0;
   for (c = cStart; c < cEnd; ++c) {
-    PetscInt *closure = NULL;
-    PetscInt  closureSize,nverts,celltype,startoffset;
+    PetscInt nverts,dof = 0,celltype,startoffset,nC=0;
 
     if (hasLabel) {
       PetscInt value;
 
-      ierr = DMPlexGetLabelValue(dm, "vtk", c, &value);CHKERRQ(ierr);
+      ierr = DMGetLabelValue(dm, "vtk", c, &value);CHKERRQ(ierr);
       if (value != 1) continue;
     }
     startoffset = countconn;
-    ierr        = DMPlexGetTransitiveClosure(dm, c, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
-    for (v = 0; v < closureSize*2; v += 2) {
-      if ((closure[v] >= vStart) && (closure[v] < vEnd)) {
-        conn[countconn++] = closure[v] - vStart;
-      }
+    if (localized) {
+      ierr = PetscSectionGetDof(coordSection, c, &dof);CHKERRQ(ierr);
     }
-    ierr = DMPlexRestoreTransitiveClosure(dm, c, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
+    if (!dof) {
+      PetscInt *closure = NULL;
+      PetscInt  closureSize;
+
+      ierr = DMPlexGetTransitiveClosure(dm, c, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
+      for (v = 0; v < closureSize*2; v += 2) {
+        if ((closure[v] >= vStart) && (closure[v] < vEnd)) {
+          if (!localized) conn[countconn++] = closure[v] - vStart;
+          else conn[countconn++] = startoffset + nC;
+          ++nC;
+        }
+      }
+      ierr = DMPlexRestoreTransitiveClosure(dm, c, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
+    } else {
+      for (nC = 0; nC < dof/dim; nC++) conn[countconn++] = startoffset + nC;
+    }
+    ierr = DMPlexInvertCell(dim, nC, &conn[countconn-nC]);CHKERRQ(ierr);
 
     offsets[countcell] = countconn;
 
     nverts = countconn - startoffset;
-    ierr   = DMPlexVTKGetCellType(dm,dim,nverts,&celltype);CHKERRQ(ierr);
+    ierr   = DMPlexVTKGetCellType_Internal(dm,dim,nverts,&celltype);CHKERRQ(ierr);
 
     types[countcell] = celltype;
     countcell++;
@@ -107,8 +112,6 @@ static PetscErrorCode DMPlexGetVTKConnectivity(DM dm,PieceInfo *piece,PetscVTKIn
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMPlexVTKWriteAll_VTU"
 /*
   Write all fields that have been provided to the viewer
   Multi-block XML format with binary appended data.
@@ -116,12 +119,14 @@ static PetscErrorCode DMPlexGetVTKConnectivity(DM dm,PieceInfo *piece,PetscVTKIn
 PetscErrorCode DMPlexVTKWriteAll_VTU(DM dm,PetscViewer viewer)
 {
   MPI_Comm                 comm;
+  PetscSection             coordSection;
   PetscViewer_VTK          *vtk = (PetscViewer_VTK*)viewer->data;
   PetscViewerVTKObjectLink link;
   FILE                     *fp;
   PetscMPIInt              rank,size,tag;
   PetscErrorCode           ierr;
-  PetscInt                 dim,cellHeight,cStart,cEnd,vStart,vEnd,cMax,numLabelCells,hasLabel,c,v,r,i;
+  PetscInt                 dimEmbed,cellHeight,cStart,cEnd,vStart,vEnd,numLabelCells,hasLabel,c,v,r,i;
+  PetscBool                localized;
   PieceInfo                piece,*gpiece = NULL;
   void                     *buffer = NULL;
 
@@ -143,36 +148,49 @@ PetscErrorCode DMPlexVTKWriteAll_VTU(DM dm,PetscViewer viewer)
 #endif
   ierr = PetscFPrintf(comm,fp,"  <UnstructuredGrid>\n");CHKERRQ(ierr);
 
-  ierr = DMPlexGetDimension(dm, &dim);CHKERRQ(ierr);
+  ierr = DMGetCoordinateDim(dm, &dimEmbed);CHKERRQ(ierr);
   ierr = DMPlexGetVTKCellHeight(dm, &cellHeight);CHKERRQ(ierr);
   ierr = DMPlexGetHeightStratum(dm, cellHeight, &cStart, &cEnd);CHKERRQ(ierr);
   ierr = DMPlexGetDepthStratum(dm, 0, &vStart, &vEnd);CHKERRQ(ierr);
-  ierr = DMPlexGetHybridBounds(dm, &cMax, NULL, NULL, NULL);CHKERRQ(ierr);
-  if (cMax >= 0) cEnd = PetscMin(cEnd, cMax);
-  ierr = DMPlexGetStratumSize(dm, "vtk", 1, &numLabelCells);CHKERRQ(ierr);
+  ierr = DMGetStratumSize(dm, "vtk", 1, &numLabelCells);CHKERRQ(ierr);
+  ierr = DMGetCoordinatesLocalized(dm, &localized);CHKERRQ(ierr);
+  ierr = DMGetCoordinateSection(dm, &coordSection);CHKERRQ(ierr);
 
   hasLabel        = numLabelCells > 0 ? PETSC_TRUE : PETSC_FALSE;
-  piece.nvertices = vEnd - vStart;
+  piece.nvertices = 0;
   piece.ncells    = 0;
   piece.nconn     = 0;
+  if (!localized) piece.nvertices = vEnd - vStart;
   for (c = cStart; c < cEnd; ++c) {
-    PetscInt *closure = NULL;
-    PetscInt closureSize;
-
+    PetscInt dof = 0;
     if (hasLabel) {
       PetscInt value;
 
-      ierr = DMPlexGetLabelValue(dm, "vtk", c, &value);CHKERRQ(ierr);
+      ierr = DMGetLabelValue(dm, "vtk", c, &value);CHKERRQ(ierr);
       if (value != 1) continue;
     }
-    ierr = DMPlexGetTransitiveClosure(dm, c, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
-    for (v = 0; v < closureSize*2; v += 2) {
-      if ((closure[v] >= vStart) && (closure[v] < vEnd)) piece.nconn++;
+    if (localized) {
+      ierr = PetscSectionGetDof(coordSection, c, &dof);CHKERRQ(ierr);
+    }
+    if (!dof) {
+      PetscInt *closure = NULL;
+      PetscInt closureSize;
+
+      ierr = DMPlexGetTransitiveClosure(dm, c, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
+      for (v = 0; v < closureSize*2; v += 2) {
+        if ((closure[v] >= vStart) && (closure[v] < vEnd)) {
+          piece.nconn++;
+          if (localized) piece.nvertices++;
+        }
+      }
+      ierr = DMPlexRestoreTransitiveClosure(dm, c, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
+    } else {
+      piece.nvertices += dof/dimEmbed;
+      piece.nconn     += dof/dimEmbed;
     }
-    ierr = DMPlexRestoreTransitiveClosure(dm, c, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
     piece.ncells++;
   }
-  if (!rank) {ierr = PetscMalloc(size*sizeof(piece),&gpiece);CHKERRQ(ierr);}
+  if (!rank) {ierr = PetscMalloc1(size,&gpiece);CHKERRQ(ierr);}
   ierr = MPI_Gather((PetscInt*)&piece,sizeof(piece)/sizeof(PetscInt),MPIU_INT,(PetscInt*)gpiece,sizeof(piece)/sizeof(PetscInt),MPIU_INT,0,comm);CHKERRQ(ierr);
 
   /*
@@ -216,19 +234,36 @@ PetscErrorCode DMPlexVTKWriteAll_VTU(DM dm,PetscViewer viewer)
         ierr = PetscSectionGetDof(dm->defaultSection,cStart,&bs);CHKERRQ(ierr);
         ierr = PetscSectionGetNumFields(dm->defaultSection,&nfields);CHKERRQ(ierr);
         for (field=0,i=0; field<(nfields?nfields:1); field++) {
-          PetscInt   fbs,j;
+          PetscInt     fbs,j;
+          PetscFV      fv = NULL;
+          PetscObject  f;
+          PetscClassId fClass;
           const char *fieldname = NULL;
           char       buf[256];
           if (nfields) {        /* We have user-defined fields/components */
             ierr = PetscSectionGetFieldDof(dm->defaultSection,cStart,field,&fbs);CHKERRQ(ierr);
             ierr = PetscSectionGetFieldName(dm->defaultSection,field,&fieldname);CHKERRQ(ierr);
           } else fbs = bs;      /* Say we have one field with 'bs' components */
+          ierr = DMGetField(dm,field,&f);CHKERRQ(ierr);
+          ierr = PetscObjectGetClassId(f,&fClass);CHKERRQ(ierr);
+          if (fClass == PETSCFV_CLASSID) {
+            fv = (PetscFV) f;
+          }
           if (!fieldname) {
             ierr = PetscSNPrintf(buf,sizeof(buf),"CellField%D",field);CHKERRQ(ierr);
             fieldname = buf;
           }
           for (j=0; j<fbs; j++) {
-            ierr = PetscFPrintf(comm,fp,"        <DataArray type=\"%s\" Name=\"%s%s.%D\" NumberOfComponents=\"1\" format=\"appended\" offset=\"%D\" />\n",precision,vecname,fieldname,j,boffset);CHKERRQ(ierr);
+            const char *compName = NULL;
+            if (fv) {
+              ierr = PetscFVGetComponentName(fv,j,&compName);CHKERRQ(ierr);
+            }
+            if (compName) {
+              ierr = PetscFPrintf(comm,fp,"        <DataArray type=\"%s\" Name=\"%s%s.%s\" NumberOfComponents=\"1\" format=\"appended\" offset=\"%D\" />\n",precision,vecname,fieldname,compName,boffset);CHKERRQ(ierr);
+            }
+            else {
+              ierr = PetscFPrintf(comm,fp,"        <DataArray type=\"%s\" Name=\"%s%s.%D\" NumberOfComponents=\"1\" format=\"appended\" offset=\"%D\" />\n",precision,vecname,fieldname,j,boffset);CHKERRQ(ierr);
+            }
             boffset += gpiece[r].ncells*sizeof(PetscScalar) + sizeof(int);
             i++;
           }
@@ -295,35 +330,83 @@ PetscErrorCode DMPlexVTKWriteAll_VTU(DM dm,PetscViewer viewer)
         const PetscScalar *x;
         PetscScalar       *y = NULL;
         Vec               coords;
-        nsend = piece.nvertices*3;
+
         ierr  = DMGetCoordinatesLocal(dm,&coords);CHKERRQ(ierr);
         ierr  = VecGetArrayRead(coords,&x);CHKERRQ(ierr);
-        if (dim != 3) {
-          ierr = PetscMalloc(piece.nvertices*3*sizeof(PetscScalar),&y);CHKERRQ(ierr);
-          for (i=0; i<piece.nvertices; i++) {
-            y[i*3+0] = x[i*dim+0];
-            y[i*3+1] = (dim > 1) ? x[i*dim+1] : 0;
-            y[i*3+2] = 0;
+        if (dimEmbed != 3 || localized) {
+          ierr = PetscMalloc1(piece.nvertices*3,&y);CHKERRQ(ierr);
+          if (localized) {
+            PetscInt cnt;
+            for (c=cStart,cnt=0; c<cEnd; c++) {
+              PetscInt off, dof;
+
+              ierr = PetscSectionGetDof(coordSection, c, &dof);CHKERRQ(ierr);
+              if (!dof) {
+                PetscInt *closure = NULL;
+                PetscInt closureSize;
+
+                ierr = DMPlexGetTransitiveClosure(dm, c, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
+                for (v = 0; v < closureSize*2; v += 2) {
+                  if ((closure[v] >= vStart) && (closure[v] < vEnd)) {
+                    ierr = PetscSectionGetOffset(coordSection, closure[v], &off);CHKERRQ(ierr);
+                    if (dimEmbed != 3) {
+                      y[cnt*3+0] = x[off+0];
+                      y[cnt*3+1] = (dimEmbed > 1) ? x[off+1] : 0.0;
+                      y[cnt*3+2] = 0.0;
+                    } else {
+                      y[cnt*3+0] = x[off+0];
+                      y[cnt*3+1] = x[off+1];
+                      y[cnt*3+2] = x[off+2];
+                    }
+                    cnt++;
+                  }
+                }
+                ierr = DMPlexRestoreTransitiveClosure(dm, c, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
+              } else {
+                ierr = PetscSectionGetOffset(coordSection, c, &off);CHKERRQ(ierr);
+                if (dimEmbed != 3) {
+                  for (i=0; i<dof/dimEmbed; i++) {
+                    y[cnt*3+0] = x[off + i*dimEmbed + 0];
+                    y[cnt*3+1] = (dimEmbed > 1) ? x[off + i*dimEmbed + 1] : 0.0;
+                    y[cnt*3+2] = 0.0;
+                    cnt++;
+                  }
+                } else {
+                  for (i=0; i<dof; i ++) {
+		    y[cnt*3+i] = x[off + i];
+                  }
+                  cnt += dof/dimEmbed;
+                }
+              }
+            }
+            if (cnt != piece.nvertices) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Count does not match");
+          } else {
+            for (i=0; i<piece.nvertices; i++) {
+              y[i*3+0] = x[i*dimEmbed+0];
+              y[i*3+1] = (dimEmbed > 1) ? x[i*dimEmbed+1] : 0;
+              y[i*3+2] = 0.0;
+            }
           }
         }
-        ierr = TransferWrite(viewer,fp,r,0,y ? y : x,buffer,nsend,PETSC_SCALAR,tag);CHKERRQ(ierr);
-        ierr = PetscFree(y);CHKERRQ(ierr);
-        ierr = VecRestoreArrayRead(coords,&x);CHKERRQ(ierr);
+        nsend = piece.nvertices*3;
+        ierr  = TransferWrite(viewer,fp,r,0,y ? y : x,buffer,nsend,MPIU_SCALAR,tag);CHKERRQ(ierr);
+        ierr  = PetscFree(y);CHKERRQ(ierr);
+        ierr  = VecRestoreArrayRead(coords,&x);CHKERRQ(ierr);
       }
       {                           /* Connectivity, offsets, types */
-        PetscVTKInt  *connectivity = NULL,*offsets;
-        PetscVTKType *types;
-        ierr = DMPlexGetVTKConnectivity(dm,&piece,&connectivity,&offsets,&types);CHKERRQ(ierr);
-        ierr = TransferWrite(viewer,fp,r,0,connectivity,buffer,piece.nconn,PETSC_INT32,tag);CHKERRQ(ierr);
-        ierr = TransferWrite(viewer,fp,r,0,offsets,buffer,piece.ncells,PETSC_INT32,tag);CHKERRQ(ierr);
-        ierr = TransferWrite(viewer,fp,r,0,types,buffer,piece.ncells,PETSC_UINT8,tag);CHKERRQ(ierr);
+        PetscVTKInt  *connectivity = NULL, *offsets = NULL;
+        PetscVTKType *types = NULL;
+        ierr = DMPlexGetVTKConnectivity(dm,localized,&piece,&connectivity,&offsets,&types);CHKERRQ(ierr);
+        ierr = TransferWrite(viewer,fp,r,0,connectivity,buffer,piece.nconn,MPI_INT,tag);CHKERRQ(ierr);
+        ierr = TransferWrite(viewer,fp,r,0,offsets,buffer,piece.ncells,MPI_INT,tag);CHKERRQ(ierr);
+        ierr = TransferWrite(viewer,fp,r,0,types,buffer,piece.ncells,MPI_CHAR,tag);CHKERRQ(ierr);
         ierr = PetscFree3(connectivity,offsets,types);CHKERRQ(ierr);
       }
       {                         /* Owners (cell data) */
         PetscVTKInt *owners;
-        ierr = PetscMalloc(piece.ncells*sizeof(PetscVTKInt),&owners);CHKERRQ(ierr);
+        ierr = PetscMalloc1(piece.ncells,&owners);CHKERRQ(ierr);
         for (i=0; i<piece.ncells; i++) owners[i] = rank;
-        ierr = TransferWrite(viewer,fp,r,0,owners,buffer,piece.ncells,PETSC_INT32,tag);CHKERRQ(ierr);
+        ierr = TransferWrite(viewer,fp,r,0,owners,buffer,piece.ncells,MPI_INT,tag);CHKERRQ(ierr);
         ierr = PetscFree(owners);CHKERRQ(ierr);
       }
       /* Cell data */
@@ -335,21 +418,21 @@ PetscErrorCode DMPlexVTKWriteAll_VTU(DM dm,PetscViewer viewer)
         if ((link->ft != PETSC_VTK_CELL_FIELD) && (link->ft != PETSC_VTK_CELL_VECTOR_FIELD)) continue;
         ierr = PetscSectionGetDof(dm->defaultSection,cStart,&bs);CHKERRQ(ierr);
         ierr = VecGetArrayRead(X,&x);CHKERRQ(ierr);
-        ierr = PetscMalloc(piece.ncells*sizeof(PetscScalar),&y);CHKERRQ(ierr);
+        ierr = PetscMalloc1(piece.ncells,&y);CHKERRQ(ierr);
         for (i=0; i<bs; i++) {
           PetscInt cnt;
           for (c=cStart,cnt=0; c<cEnd; c++) {
-            const PetscScalar *xpoint;
+            PetscScalar *xpoint;
             if (hasLabel) {     /* Ignore some cells */
               PetscInt value;
-              ierr = DMPlexGetLabelValue(dm, "vtk", c, &value);CHKERRQ(ierr);
+              ierr = DMGetLabelValue(dm, "vtk", c, &value);CHKERRQ(ierr);
               if (value != 1) continue;
             }
             ierr     = DMPlexPointLocalRead(dm,c,x,&xpoint);CHKERRQ(ierr);
             y[cnt++] = xpoint[i];
           }
           if (cnt != piece.ncells) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Count does not match");
-          ierr = TransferWrite(viewer,fp,r,0,y,buffer,piece.ncells,PETSC_SCALAR,tag);CHKERRQ(ierr);
+          ierr = TransferWrite(viewer,fp,r,0,y,buffer,piece.ncells,MPIU_SCALAR,tag);CHKERRQ(ierr);
         }
         ierr = PetscFree(y);CHKERRQ(ierr);
         ierr = VecRestoreArrayRead(X,&x);CHKERRQ(ierr);
@@ -363,33 +446,52 @@ PetscErrorCode DMPlexVTKWriteAll_VTU(DM dm,PetscViewer viewer)
         if ((link->ft != PETSC_VTK_POINT_FIELD) && (link->ft != PETSC_VTK_POINT_VECTOR_FIELD)) continue;
         ierr = PetscSectionGetDof(dm->defaultSection,vStart,&bs);CHKERRQ(ierr);
         ierr = VecGetArrayRead(X,&x);CHKERRQ(ierr);
-        ierr = PetscMalloc(piece.nvertices*sizeof(PetscScalar),&y);CHKERRQ(ierr);
+        ierr = PetscMalloc1(piece.nvertices,&y);CHKERRQ(ierr);
         for (i=0; i<bs; i++) {
           PetscInt cnt;
-          for (v=vStart,cnt=0; v<vEnd; v++) {
-            const PetscScalar *xpoint;
-            ierr     = DMPlexPointLocalRead(dm,v,x,&xpoint);CHKERRQ(ierr);
-            y[cnt++] = xpoint[i];
+          if (!localized) {
+            for (v=vStart,cnt=0; v<vEnd; v++) {
+              PetscScalar *xpoint;
+              ierr     = DMPlexPointLocalRead(dm,v,x,&xpoint);CHKERRQ(ierr);
+              y[cnt++] = xpoint[i];
+            }
+          } else {
+            for (c=cStart,cnt=0; c<cEnd; c++) {
+              PetscInt *closure = NULL;
+              PetscInt  closureSize, off;
+
+              ierr = DMPlexGetTransitiveClosure(dm, c, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
+              for (v = 0, off = 0; v < closureSize*2; v += 2) {
+                if ((closure[v] >= vStart) && (closure[v] < vEnd)) {
+                  PetscScalar *xpoint;
+
+                  ierr = DMPlexPointLocalRead(dm,closure[v],x,&xpoint);CHKERRQ(ierr);
+                  y[cnt + off++] = xpoint[i];
+                }
+              }
+              cnt += off;
+              ierr = DMPlexRestoreTransitiveClosure(dm, c, PETSC_TRUE, &closureSize, &closure);CHKERRQ(ierr);
+            }
           }
           if (cnt != piece.nvertices) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Count does not match");
-          ierr = TransferWrite(viewer,fp,r,0,y,buffer,piece.nvertices,PETSC_SCALAR,tag);CHKERRQ(ierr);
+          ierr = TransferWrite(viewer,fp,r,0,y,buffer,piece.nvertices,MPIU_SCALAR,tag);CHKERRQ(ierr);
         }
         ierr = PetscFree(y);CHKERRQ(ierr);
         ierr = VecRestoreArrayRead(X,&x);CHKERRQ(ierr);
       }
     } else if (!rank) {
-      ierr = TransferWrite(viewer,fp,r,0,NULL,buffer,gpiece[r].nvertices*3,PETSC_SCALAR,tag);CHKERRQ(ierr); /* positions */
-      ierr = TransferWrite(viewer,fp,r,0,NULL,buffer,gpiece[r].nconn,PETSC_INT32,tag);CHKERRQ(ierr); /* connectivity */
-      ierr = TransferWrite(viewer,fp,r,0,NULL,buffer,gpiece[r].ncells,PETSC_INT32,tag);CHKERRQ(ierr); /* offsets */
-      ierr = TransferWrite(viewer,fp,r,0,NULL,buffer,gpiece[r].ncells,PETSC_UINT8,tag);CHKERRQ(ierr); /* types */
-      ierr = TransferWrite(viewer,fp,r,0,NULL,buffer,gpiece[r].ncells,PETSC_INT32,tag);CHKERRQ(ierr); /* owner rank (cells) */
+      ierr = TransferWrite(viewer,fp,r,0,NULL,buffer,gpiece[r].nvertices*3,MPIU_SCALAR,tag);CHKERRQ(ierr); /* positions */
+      ierr = TransferWrite(viewer,fp,r,0,NULL,buffer,gpiece[r].nconn,MPI_INT,tag);CHKERRQ(ierr); /* connectivity */
+      ierr = TransferWrite(viewer,fp,r,0,NULL,buffer,gpiece[r].ncells,MPI_INT,tag);CHKERRQ(ierr); /* offsets */
+      ierr = TransferWrite(viewer,fp,r,0,NULL,buffer,gpiece[r].ncells,MPI_CHAR,tag);CHKERRQ(ierr); /* types */
+      ierr = TransferWrite(viewer,fp,r,0,NULL,buffer,gpiece[r].ncells,MPI_INT,tag);CHKERRQ(ierr); /* owner rank (cells) */
       /* all cell data */
       for (link=vtk->link; link; link=link->next) {
         PetscInt bs;
         if ((link->ft != PETSC_VTK_CELL_FIELD) && (link->ft != PETSC_VTK_CELL_VECTOR_FIELD)) continue;
         ierr = PetscSectionGetDof(dm->defaultSection,cStart,&bs);CHKERRQ(ierr);
         for (i=0; i<bs; i++) {
-          ierr = TransferWrite(viewer,fp,r,0,NULL,buffer,gpiece[r].ncells,PETSC_SCALAR,tag);CHKERRQ(ierr);
+          ierr = TransferWrite(viewer,fp,r,0,NULL,buffer,gpiece[r].ncells,MPIU_SCALAR,tag);CHKERRQ(ierr);
         }
       }
       /* all point data */
@@ -398,7 +500,7 @@ PetscErrorCode DMPlexVTKWriteAll_VTU(DM dm,PetscViewer viewer)
         if ((link->ft != PETSC_VTK_POINT_FIELD) && (link->ft != PETSC_VTK_POINT_VECTOR_FIELD)) continue;
         ierr = PetscSectionGetDof(dm->defaultSection,vStart,&bs);CHKERRQ(ierr);
         for (i=0; i<bs; i++) {
-          ierr = TransferWrite(viewer,fp,r,0,NULL,buffer,gpiece[r].nvertices,PETSC_SCALAR,tag);CHKERRQ(ierr);
+          ierr = TransferWrite(viewer,fp,r,0,NULL,buffer,gpiece[r].nvertices,MPIU_SCALAR,tag);CHKERRQ(ierr);
         }
       }
     }
@@ -407,5 +509,6 @@ PetscErrorCode DMPlexVTKWriteAll_VTU(DM dm,PetscViewer viewer)
   ierr = PetscFree(buffer);CHKERRQ(ierr);
   ierr = PetscFPrintf(comm,fp,"\n  </AppendedData>\n");CHKERRQ(ierr);
   ierr = PetscFPrintf(comm,fp,"</VTKFile>\n");CHKERRQ(ierr);
+  ierr = PetscFClose(comm,fp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
