@@ -9,10 +9,9 @@ typedef struct  {
   VecScatter gtol;
   VecScatter ltog;
   VecScatter ltol;
+  void       *ctx;
 } DM_Shell;
 
-#undef __FUNCT__
-#define __FUNCT__ "DMGlobalToLocalBeginDefaultShell"
 /*@
    DMGlobalToLocalBeginDefaultShell - Uses the GlobalToLocal VecScatter context set by the user to begin a global to local scatter
    Collective
@@ -25,7 +24,7 @@ typedef struct  {
 
    Level: advanced
 
-   Note:  This is not normally called directly by user code, generally user code calls DMGlobalToLocalBegin() and DMGlobalToLocalEnd(). If the user provides their own custom routines to DMShellSetLocalToGlobal() then those routines might have reason to call this function. 
+   Note:  This is not normally called directly by user code, generally user code calls DMGlobalToLocalBegin() and DMGlobalToLocalEnd(). If the user provides their own custom routines to DMShellSetLocalToGlobal() then those routines might have reason to call this function.
 
 .seealso: DMGlobalToLocalEndDefaultShell()
 @*/
@@ -40,8 +39,6 @@ PetscErrorCode DMGlobalToLocalBeginDefaultShell(DM dm,Vec g,InsertMode mode,Vec
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMGlobalToLocalEndDefaultShell"
 /*@
    DMGlobalToLocalEndDefaultShell - Uses the GlobalToLocal VecScatter context set by the user to end a global to local scatter
    Collective
@@ -67,8 +64,6 @@ PetscErrorCode DMGlobalToLocalEndDefaultShell(DM dm,Vec g,InsertMode mode,Vec l)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMLocalToGlobalBeginDefaultShell"
 /*@
    DMLocalToGlobalBeginDefaultShell - Uses the LocalToGlobal VecScatter context set by the user to begin a local to global scatter
    Collective
@@ -81,7 +76,7 @@ PetscErrorCode DMGlobalToLocalEndDefaultShell(DM dm,Vec g,InsertMode mode,Vec l)
 
    Level: advanced
 
-   Note:  This is not normally called directly by user code, generally user code calls DMLocalToGlobalBegin() and DMLocalToGlobalEnd(). If the user provides their own custom routines to DMShellSetLocalToGlobal() then those routines might have reason to call this function. 
+   Note:  This is not normally called directly by user code, generally user code calls DMLocalToGlobalBegin() and DMLocalToGlobalEnd(). If the user provides their own custom routines to DMShellSetLocalToGlobal() then those routines might have reason to call this function.
 
 .seealso: DMLocalToGlobalEndDefaultShell()
 @*/
@@ -96,8 +91,6 @@ PetscErrorCode DMLocalToGlobalBeginDefaultShell(DM dm,Vec l,InsertMode mode,Vec
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMLocalToGlobalEndDefaultShell"
 /*@
    DMLocalToGlobalEndDefaultShell - Uses the LocalToGlobal VecScatter context set by the user to end a local to global scatter
    Collective
@@ -123,8 +116,6 @@ PetscErrorCode DMLocalToGlobalEndDefaultShell(DM dm,Vec l,InsertMode mode,Vec g)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMLocalToLocalBeginDefaultShell"
 /*@
    DMLocalToLocalBeginDefaultShell - Uses the LocalToLocal VecScatter context set by the user to begin a local to local scatter
    Collective
@@ -139,7 +130,7 @@ PetscErrorCode DMLocalToGlobalEndDefaultShell(DM dm,Vec l,InsertMode mode,Vec g)
 
    Level: advanced
 
-   Note:  This is not normally called directly by user code, generally user code calls DMLocalToLocalBegin() and DMLocalToLocalEnd(). If the user provides their own custom routines to DMShellSetLocalToLocal() then those routines might have reason to call this function. 
+   Note:  This is not normally called directly by user code, generally user code calls DMLocalToLocalBegin() and DMLocalToLocalEnd(). If the user provides their own custom routines to DMShellSetLocalToLocal() then those routines might have reason to call this function.
 
 .seealso: DMLocalToLocalEndDefaultShell()
 @*/
@@ -154,8 +145,6 @@ PetscErrorCode DMLocalToLocalBeginDefaultShell(DM dm,Vec g,InsertMode mode,Vec l
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMLocalToLocalEndDefaultShell"
 /*@
    DMLocalToLocalEndDefaultShell - Uses the LocalToLocal VecScatter context set by the user to end a local to local scatter
    Collective
@@ -183,8 +172,6 @@ PetscErrorCode DMLocalToLocalEndDefaultShell(DM dm,Vec g,InsertMode mode,Vec l)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCreateMatrix_Shell"
 static PetscErrorCode DMCreateMatrix_Shell(DM dm,Mat *J)
 {
   PetscErrorCode ierr;
@@ -229,8 +216,6 @@ static PetscErrorCode DMCreateMatrix_Shell(DM dm,Mat *J)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCreateGlobalVector_Shell"
 PetscErrorCode DMCreateGlobalVector_Shell(DM dm,Vec *gvec)
 {
   PetscErrorCode ierr;
@@ -255,8 +240,6 @@ PetscErrorCode DMCreateGlobalVector_Shell(DM dm,Vec *gvec)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCreateLocalVector_Shell"
 PetscErrorCode DMCreateLocalVector_Shell(DM dm,Vec *gvec)
 {
   PetscErrorCode ierr;
@@ -281,8 +264,62 @@ PetscErrorCode DMCreateLocalVector_Shell(DM dm,Vec *gvec)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetMatrix"
+/*@
+   DMShellSetContext - set some data to be usable by this DM
+
+   Collective
+
+   Input Arguments:
++  dm - shell DM
+-  ctx - the context
+
+   Level: advanced
+
+.seealso: DMCreateMatrix(), DMShellGetContext()
+@*/
+PetscErrorCode DMShellSetContext(DM dm,void *ctx)
+{
+  DM_Shell       *shell = (DM_Shell*)dm->data;
+  PetscErrorCode ierr;
+  PetscBool      isshell;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  ierr = PetscObjectTypeCompare((PetscObject)dm,DMSHELL,&isshell);CHKERRQ(ierr);
+  if (!isshell) PetscFunctionReturn(0);
+  shell->ctx = ctx;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   DMShellGetContext - set some data to be usable by this DM
+
+   Collective
+
+   Input Argument:
+.  dm - shell DM
+
+   Output Argument:
+.  ctx - the context
+
+   Level: advanced
+
+.seealso: DMCreateMatrix(), DMShellSetContext()
+@*/
+PetscErrorCode DMShellGetContext(DM dm,void **ctx)
+{
+  DM_Shell       *shell = (DM_Shell*)dm->data;
+  PetscErrorCode ierr;
+  PetscBool      isshell;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  ierr = PetscObjectTypeCompare((PetscObject)dm,DMSHELL,&isshell);CHKERRQ(ierr);
+  if (!isshell) PetscFunctionReturn(0);
+  *ctx = shell->ctx;
+  PetscFunctionReturn(0);
+}
+
 /*@
    DMShellSetMatrix - sets a template matrix associated with the DMShell
 
@@ -294,7 +331,7 @@ PetscErrorCode DMCreateLocalVector_Shell(DM dm,Vec *gvec)
 
    Level: advanced
 
-.seealso: DMCreateMatrix(), DMShellSetCreateMatrix()
+.seealso: DMCreateMatrix(), DMShellSetCreateMatrix(), DMShellSetContext(), DMShellGetContext()
 @*/
 PetscErrorCode DMShellSetMatrix(DM dm,Mat J)
 {
@@ -313,8 +350,6 @@ PetscErrorCode DMShellSetMatrix(DM dm,Mat J)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetCreateMatrix"
 /*@C
    DMShellSetCreateMatrix - sets the routine to create a matrix associated with the shell DM
 
@@ -326,7 +361,7 @@ PetscErrorCode DMShellSetMatrix(DM dm,Mat J)
 
    Level: advanced
 
-.seealso: DMCreateMatrix(), DMShellSetMatrix()
+.seealso: DMCreateMatrix(), DMShellSetMatrix(), DMShellSetContext(), DMShellGetContext()
 @*/
 PetscErrorCode DMShellSetCreateMatrix(DM dm,PetscErrorCode (*func)(DM,Mat*))
 {
@@ -337,8 +372,6 @@ PetscErrorCode DMShellSetCreateMatrix(DM dm,PetscErrorCode (*func)(DM,Mat*))
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetGlobalVector"
 /*@
    DMShellSetGlobalVector - sets a template global vector associated with the DMShell
 
@@ -357,20 +390,31 @@ PetscErrorCode DMShellSetGlobalVector(DM dm,Vec X)
   DM_Shell       *shell = (DM_Shell*)dm->data;
   PetscErrorCode ierr;
   PetscBool      isshell;
+  DM             vdm;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
   PetscValidHeaderSpecific(X,VEC_CLASSID,2);
   ierr = PetscObjectTypeCompare((PetscObject)dm,DMSHELL,&isshell);CHKERRQ(ierr);
   if (!isshell) PetscFunctionReturn(0);
+  ierr = VecGetDM(X,&vdm);CHKERRQ(ierr);
+  /*
+      if the vector proposed as the new base global vector for the DM is a DM vector associated
+      with the same DM then the current base global vector for the DM is ok and if we replace it with the new one
+      we get a circular dependency that prevents the DM from being destroy when it should be.
+      This occurs when SNESSet/GetNPC() is used with a SNES that does not have a user provided
+      DM attached to it since the inner SNES (which shares the DM with the outer SNES) tries
+      to set its input vector (which is associated with the DM) as the base global vector.
+      Thanks to Juan P. Mendez Granado Re: [petsc-maint] Nonlinear conjugate gradien
+      for pointing out the problem.
+   */
+  if (vdm == dm) PetscFunctionReturn(0);
   ierr           = PetscObjectReference((PetscObject)X);CHKERRQ(ierr);
   ierr           = VecDestroy(&shell->Xglobal);CHKERRQ(ierr);
   shell->Xglobal = X;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetCreateGlobalVector"
 /*@C
    DMShellSetCreateGlobalVector - sets the routine to create a global vector associated with the shell DM
 
@@ -382,7 +426,7 @@ PetscErrorCode DMShellSetGlobalVector(DM dm,Vec X)
 
    Level: advanced
 
-.seealso: DMShellSetGlobalVector(), DMShellSetCreateMatrix()
+.seealso: DMShellSetGlobalVector(), DMShellSetCreateMatrix(), DMShellSetContext(), DMShellGetContext()
 @*/
 PetscErrorCode DMShellSetCreateGlobalVector(DM dm,PetscErrorCode (*func)(DM,Vec*))
 {
@@ -393,8 +437,6 @@ PetscErrorCode DMShellSetCreateGlobalVector(DM dm,PetscErrorCode (*func)(DM,Vec*
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetLocalVector"
 /*@
    DMShellSetLocalVector - sets a template local vector associated with the DMShell
 
@@ -413,20 +455,31 @@ PetscErrorCode DMShellSetLocalVector(DM dm,Vec X)
   DM_Shell       *shell = (DM_Shell*)dm->data;
   PetscErrorCode ierr;
   PetscBool      isshell;
+  DM             vdm;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
   PetscValidHeaderSpecific(X,VEC_CLASSID,2);
   ierr = PetscObjectTypeCompare((PetscObject)dm,DMSHELL,&isshell);CHKERRQ(ierr);
   if (!isshell) PetscFunctionReturn(0);
+  ierr           = VecGetDM(X,&vdm);CHKERRQ(ierr);
+  /*
+      if the vector proposed as the new base global vector for the DM is a DM vector associated
+      with the same DM then the current base global vector for the DM is ok and if we replace it with the new one
+      we get a circular dependency that prevents the DM from being destroy when it should be.
+      This occurs when SNESSet/GetNPC() is used with a SNES that does not have a user provided
+      DM attached to it since the inner SNES (which shares the DM with the outer SNES) tries
+      to set its input vector (which is associated with the DM) as the base global vector.
+      Thanks to Juan P. Mendez Granado Re: [petsc-maint] Nonlinear conjugate gradien
+      for pointing out the problem.
+   */
+  if (vdm == dm) PetscFunctionReturn(0);
   ierr = PetscObjectReference((PetscObject)X);CHKERRQ(ierr);
   ierr = VecDestroy(&shell->Xlocal);CHKERRQ(ierr);
   shell->Xlocal = X;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetCreateLocalVector"
 /*@C
    DMShellSetCreateLocalVector - sets the routine to create a local vector associated with the shell DM
 
@@ -438,7 +491,7 @@ PetscErrorCode DMShellSetLocalVector(DM dm,Vec X)
 
    Level: advanced
 
-.seealso: DMShellSetLocalVector(), DMShellSetCreateMatrix()
+.seealso: DMShellSetLocalVector(), DMShellSetCreateMatrix(), DMShellSetContext(), DMShellGetContext()
 @*/
 PetscErrorCode DMShellSetCreateLocalVector(DM dm,PetscErrorCode (*func)(DM,Vec*))
 {
@@ -449,8 +502,6 @@ PetscErrorCode DMShellSetCreateLocalVector(DM dm,PetscErrorCode (*func)(DM,Vec*)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetGlobalToLocal"
 /*@C
    DMShellSetGlobalToLocal - Sets the routines used to perform a global to local scatter
 
@@ -461,8 +512,9 @@ PetscErrorCode DMShellSetCreateLocalVector(DM dm,PetscErrorCode (*func)(DM,Vec*)
 .  begin - the routine that begins the global to local scatter
 -  end - the routine that ends the global to local scatter
 
-   Notes: If these functions are not provided but DMShellSetGlobalToLocalVecScatter() is called then
-   DMGlobalToLocalBeginDefaultShell()/DMGlobalToLocalEndDefaultShell() are used to to perform the transfers 
+   Notes:
+    If these functions are not provided but DMShellSetGlobalToLocalVecScatter() is called then
+   DMGlobalToLocalBeginDefaultShell()/DMGlobalToLocalEndDefaultShell() are used to to perform the transfers
 
    Level: advanced
 
@@ -475,8 +527,6 @@ PetscErrorCode DMShellSetGlobalToLocal(DM dm,PetscErrorCode (*begin)(DM,Vec,Inse
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetLocalToGlobal"
 /*@C
    DMShellSetLocalToGlobal - Sets the routines used to perform a local to global scatter
 
@@ -487,8 +537,9 @@ PetscErrorCode DMShellSetGlobalToLocal(DM dm,PetscErrorCode (*begin)(DM,Vec,Inse
 .  begin - the routine that begins the local to global scatter
 -  end - the routine that ends the local to global scatter
 
-   Notes: If these functions are not provided but DMShellSetLocalToGlobalVecScatter() is called then
-   DMLocalToGlobalBeginDefaultShell()/DMLocalToGlobalEndDefaultShell() are used to to perform the transfers 
+   Notes:
+    If these functions are not provided but DMShellSetLocalToGlobalVecScatter() is called then
+   DMLocalToGlobalBeginDefaultShell()/DMLocalToGlobalEndDefaultShell() are used to to perform the transfers
 
    Level: advanced
 
@@ -501,8 +552,6 @@ PetscErrorCode DMShellSetLocalToGlobal(DM dm,PetscErrorCode (*begin)(DM,Vec,Inse
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetLocalToLocal"
 /*@C
    DMShellSetLocalToLocal - Sets the routines used to perform a local to local scatter
 
@@ -513,8 +562,9 @@ PetscErrorCode DMShellSetLocalToGlobal(DM dm,PetscErrorCode (*begin)(DM,Vec,Inse
 .  begin - the routine that begins the local to local scatter
 -  end - the routine that ends the local to local scatter
 
-   Notes: If these functions are not provided but DMShellSetLocalToLocalVecScatter() is called then
-   DMLocalToLocalBeginDefaultShell()/DMLocalToLocalEndDefaultShell() are used to to perform the transfers 
+   Notes:
+    If these functions are not provided but DMShellSetLocalToLocalVecScatter() is called then
+   DMLocalToLocalBeginDefaultShell()/DMLocalToLocalEndDefaultShell() are used to to perform the transfers
 
    Level: advanced
 
@@ -527,8 +577,6 @@ PetscErrorCode DMShellSetLocalToLocal(DM dm,PetscErrorCode (*begin)(DM,Vec,Inser
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetGlobalToLocalVecScatter"
 /*@
    DMShellSetGlobalToLocalVecScatter - Sets a VecScatter context for global to local communication
 
@@ -555,8 +603,6 @@ PetscErrorCode DMShellSetGlobalToLocalVecScatter(DM dm, VecScatter gtol)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetLocalToGlobalVecScatter"
 /*@
    DMShellSetLocalToGlobalVecScatter - Sets a VecScatter context for local to global communication
 
@@ -583,8 +629,6 @@ PetscErrorCode DMShellSetLocalToGlobalVecScatter(DM dm, VecScatter ltog)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetLocalToLocalVecScatter"
 /*@
    DMShellSetLocalToLocalVecScatter - Sets a VecScatter context for local to local communication
 
@@ -611,8 +655,6 @@ PetscErrorCode DMShellSetLocalToLocalVecScatter(DM dm, VecScatter ltol)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetCoarsen"
 /*@C
    DMShellSetCoarsen - Set the routine used to coarsen the shell DM
 
@@ -624,7 +666,7 @@ PetscErrorCode DMShellSetLocalToLocalVecScatter(DM dm, VecScatter ltol)
 
    Level: advanced
 
-.seealso: DMShellSetRefine(), DMCoarsen()
+.seealso: DMShellSetRefine(), DMCoarsen(), DMShellSetContext(), DMShellGetContext()
 @*/
 PetscErrorCode DMShellSetCoarsen(DM dm, PetscErrorCode (*coarsen)(DM,MPI_Comm,DM*))
 {
@@ -639,8 +681,6 @@ PetscErrorCode DMShellSetCoarsen(DM dm, PetscErrorCode (*coarsen)(DM,MPI_Comm,DM
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetRefine"
 /*@C
    DMShellSetRefine - Set the routine used to refine the shell DM
 
@@ -652,7 +692,7 @@ PetscErrorCode DMShellSetCoarsen(DM dm, PetscErrorCode (*coarsen)(DM,MPI_Comm,DM
 
    Level: advanced
 
-.seealso: DMShellSetCoarsen(), DMRefine()
+.seealso: DMShellSetCoarsen(), DMRefine(), DMShellSetContext(), DMShellGetContext()
 @*/
 PetscErrorCode DMShellSetRefine(DM dm, PetscErrorCode (*refine)(DM,MPI_Comm,DM*))
 {
@@ -667,8 +707,6 @@ PetscErrorCode DMShellSetRefine(DM dm, PetscErrorCode (*refine)(DM,MPI_Comm,DM*)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetCreateInterpolation"
 /*@C
    DMShellSetCreateInterpolation - Set the routine used to create the interpolation operator
 
@@ -680,7 +718,7 @@ PetscErrorCode DMShellSetRefine(DM dm, PetscErrorCode (*refine)(DM,MPI_Comm,DM*)
 
    Level: advanced
 
-.seealso: DMShellSetCreateInjection(), DMCreateInterpolation()
+.seealso: DMShellSetCreateInjection(), DMCreateInterpolation(), DMShellSetCreateRestriction(), DMShellSetContext(), DMShellGetContext()
 @*/
 PetscErrorCode DMShellSetCreateInterpolation(DM dm, PetscErrorCode (*interp)(DM,DM,Mat*,Vec*))
 {
@@ -695,8 +733,32 @@ PetscErrorCode DMShellSetCreateInterpolation(DM dm, PetscErrorCode (*interp)(DM,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetCreateInjection"
+/*@C
+   DMShellSetCreateRestriction - Set the routine used to create the restriction operator
+
+   Logically Collective on DM
+
+   Input Arguments
++  dm - the shell DM
+-  striction- the routine to create the restriction
+
+   Level: advanced
+
+.seealso: DMShellSetCreateInjection(), DMCreateInterpolation(), DMShellSetContext(), DMShellGetContext()
+@*/
+PetscErrorCode DMShellSetCreateRestriction(DM dm, PetscErrorCode (*restriction)(DM,DM,Mat*))
+{
+  PetscErrorCode ierr;
+  PetscBool      isshell;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  ierr = PetscObjectTypeCompare((PetscObject)dm,DMSHELL,&isshell);CHKERRQ(ierr);
+  if (!isshell) PetscFunctionReturn(0);
+  dm->ops->createrestriction = restriction;
+  PetscFunctionReturn(0);
+}
+
 /*@C
    DMShellSetCreateInjection - Set the routine used to create the injection operator
 
@@ -708,7 +770,7 @@ PetscErrorCode DMShellSetCreateInterpolation(DM dm, PetscErrorCode (*interp)(DM,
 
    Level: advanced
 
-.seealso: DMShellSetCreateInterpolation(), DMCreateInjection()
+.seealso: DMShellSetCreateInterpolation(), DMCreateInjection(), DMShellSetContext(), DMShellGetContext()
 @*/
 PetscErrorCode DMShellSetCreateInjection(DM dm, PetscErrorCode (*inject)(DM,DM,Mat*))
 {
@@ -723,8 +785,22 @@ PetscErrorCode DMShellSetCreateInjection(DM dm, PetscErrorCode (*inject)(DM,DM,M
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellSetCreateFieldDecomposition"
+static PetscErrorCode DMHasCreateInjection_Shell(DM dm, PetscBool *flg)
+{
+  PetscErrorCode ierr;
+  PetscBool      isshell;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  PetscValidPointer(flg,2);
+  ierr = PetscObjectTypeCompare((PetscObject)dm,DMSHELL,&isshell);CHKERRQ(ierr);
+  if (!isshell) {
+    *flg = PETSC_FALSE;
+  } else {
+    *flg = dm->ops->getinjection ? PETSC_TRUE : PETSC_FALSE;
+  }
+  PetscFunctionReturn(0);
+}
 /*@C
    DMShellSetCreateFieldDecomposition - Set the routine used to create a decomposition of fields for the shell DM
 
@@ -736,7 +812,7 @@ PetscErrorCode DMShellSetCreateInjection(DM dm, PetscErrorCode (*inject)(DM,DM,M
 
    Level: advanced
 
-.seealso: DMCreateFieldDecomposition()
+.seealso: DMCreateFieldDecomposition(), DMShellSetContext(), DMShellGetContext()
 @*/
 PetscErrorCode DMShellSetCreateFieldDecomposition(DM dm, PetscErrorCode (*decomp)(DM,PetscInt*,char***, IS**,DM**))
 {
@@ -751,8 +827,84 @@ PetscErrorCode DMShellSetCreateFieldDecomposition(DM dm, PetscErrorCode (*decomp
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDestroy_Shell"
+/*@C
+   DMShellSetCreateDomainDecomposition - Set the routine used to create a domain decomposition for the shell DM
+
+   Logically Collective on DM
+
+   Input Arguments
++  dm - the shell DM
+-  decomp - the routine to create the decomposition
+
+   Level: advanced
+
+.seealso: DMCreateDomainDecomposition(), DMShellSetContext(), DMShellGetContext()
+@*/
+PetscErrorCode DMShellSetCreateDomainDecomposition(DM dm, PetscErrorCode (*decomp)(DM,PetscInt*,char***, IS**,IS**,DM**))
+{
+  PetscErrorCode ierr;
+  PetscBool      isshell;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  ierr = PetscObjectTypeCompare((PetscObject)dm,DMSHELL,&isshell);CHKERRQ(ierr);
+  if (!isshell) PetscFunctionReturn(0);
+  dm->ops->createdomaindecomposition = decomp;
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   DMShellSetCreateDomainDecompositionScatters - Set the routine used to create the scatter contexts for domain decomposition with a shell DM
+
+   Logically Collective on DM
+
+   Input Arguments
++  dm - the shell DM
+-  scatter - the routine to create the scatters
+
+   Level: advanced
+
+.seealso: DMCreateDomainDecompositionScatters(), DMShellSetContext(), DMShellGetContext()
+@*/
+PetscErrorCode DMShellSetCreateDomainDecompositionScatters(DM dm, PetscErrorCode (*scatter)(DM,PetscInt,DM*,VecScatter**,VecScatter**,VecScatter**))
+{
+  PetscErrorCode ierr;
+  PetscBool      isshell;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  ierr = PetscObjectTypeCompare((PetscObject)dm,DMSHELL,&isshell);CHKERRQ(ierr);
+  if (!isshell) PetscFunctionReturn(0);
+  dm->ops->createddscatters = scatter;
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   DMShellSetCreateSubDM - Set the routine used to create a sub DM from the shell DM
+
+   Logically Collective on DM
+
+   Input Arguments
++  dm - the shell DM
+-  subdm - the routine to create the decomposition
+
+   Level: advanced
+
+.seealso: DMCreateSubDM(), DMShellSetContext(), DMShellGetContext()
+@*/
+PetscErrorCode DMShellSetCreateSubDM(DM dm, PetscErrorCode (*subdm)(DM,PetscInt,const PetscInt[],IS*,DM*))
+{
+  PetscErrorCode ierr;
+  PetscBool      isshell;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  ierr = PetscObjectTypeCompare((PetscObject)dm,DMSHELL,&isshell);CHKERRQ(ierr);
+  if (!isshell) PetscFunctionReturn(0);
+  dm->ops->createsubdm = subdm;
+  PetscFunctionReturn(0);
+}
+
 static PetscErrorCode DMDestroy_Shell(DM dm)
 {
   PetscErrorCode ierr;
@@ -770,8 +922,6 @@ static PetscErrorCode DMDestroy_Shell(DM dm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMView_Shell"
 static PetscErrorCode DMView_Shell(DM dm,PetscViewer v)
 {
   PetscErrorCode ierr;
@@ -782,8 +932,6 @@ static PetscErrorCode DMView_Shell(DM dm,PetscViewer v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMLoad_Shell"
 static PetscErrorCode DMLoad_Shell(DM dm,PetscViewer v)
 {
   PetscErrorCode ierr;
@@ -795,9 +943,7 @@ static PetscErrorCode DMLoad_Shell(DM dm,PetscViewer v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCreateSubDM_Shell"
-PetscErrorCode DMCreateSubDM_Shell(DM dm, PetscInt numFields, PetscInt fields[], IS *is, DM *subdm)
+PetscErrorCode DMCreateSubDM_Shell(DM dm, PetscInt numFields, const PetscInt fields[], IS *is, DM *subdm)
 {
   PetscErrorCode ierr;
 
@@ -807,8 +953,6 @@ PetscErrorCode DMCreateSubDM_Shell(DM dm, PetscInt numFields, PetscInt fields[],
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCreate_Shell"
 PETSC_EXTERN PetscErrorCode DMCreate_Shell(DM dm)
 {
   PetscErrorCode ierr;
@@ -833,11 +977,10 @@ PETSC_EXTERN PetscErrorCode DMCreate_Shell(DM dm)
   dm->ops->localtolocalbegin  = DMLocalToLocalBeginDefaultShell;
   dm->ops->localtolocalend    = DMLocalToLocalEndDefaultShell;
   dm->ops->createsubdm        = DMCreateSubDM_Shell;
+  dm->ops->hascreateinjection = DMHasCreateInjection_Shell;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMShellCreate"
 /*@
     DMShellCreate - Creates a shell DM object, used to manage user-defined problem data
 
@@ -851,7 +994,7 @@ PETSC_EXTERN PetscErrorCode DMCreate_Shell(DM dm)
 
     Level: advanced
 
-.seealso DMDestroy(), DMCreateGlobalVector(), DMCreateLocalVector()
+.seealso DMDestroy(), DMCreateGlobalVector(), DMCreateLocalVector(), DMShellSetContext(), DMShellGetContext()
 @*/
 PetscErrorCode  DMShellCreate(MPI_Comm comm,DM *dm)
 {
