@@ -1,7 +1,5 @@
 #include <petsc/private/dmimpl.h> /*I "petscdm.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "DMGetLocalVector"
 /*@
    DMGetLocalVector - Gets a Seq PETSc vector that
    may be used with the DMXXX routines. This vector has spaces for the ghost values.
@@ -62,8 +60,6 @@ alldone:
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMRestoreLocalVector"
 /*@
    DMRestoreLocalVector - Returns a Seq PETSc vector that
      obtained from DMGetLocalVector(). Do not use with vector obtained via
@@ -108,8 +104,6 @@ alldone:
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMGetGlobalVector"
 /*@
    DMGetGlobalVector - Gets a MPI PETSc vector that
    may be used with the DMXXX routines.
@@ -172,8 +166,6 @@ alldone:
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMClearGlobalVectors"
 /*@
    DMClearGlobalVectors - Destroys all the global vectors that have been stashed in this DM
 
@@ -209,8 +201,41 @@ PetscErrorCode  DMClearGlobalVectors(DM dm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMRestoreGlobalVector"
+/*@
+   DMClearLocalVectors - Destroys all the local vectors that have been stashed in this DM
+
+   Collective on DM
+
+   Input Parameter:
+.  dm - the distributed array
+
+   Level: developer
+
+.keywords: distributed array, create, Local, vector
+
+.seealso: DMCreateLocalVector(), VecDuplicate(), VecDuplicateVecs(),
+          DMDACreate1d(), DMDACreate2d(), DMDACreate3d(), DMLocalToLocalBegin(),
+          DMLocalToLocalEnd(), DMLocalToLocalBegin(), DMCreateLocalVector(), DMRestoreLocalVector()
+          VecStrideMax(), VecStrideMin(), VecStrideNorm()
+
+@*/
+PetscErrorCode  DMClearLocalVectors(DM dm)
+{
+  PetscErrorCode ierr;
+  PetscInt       i;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  for (i=0; i<DM_MAX_WORK_VECTORS; i++) {
+    Vec g;
+    if (dm->localout[i]) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Clearing DM of local vectors that has a local vector obtained with DMGetLocalVector()");
+    g = dm->localin[i];
+    dm->localin[i] = NULL;
+    ierr = VecDestroy(&g);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 /*@
    DMRestoreGlobalVector - Returns a Seq PETSc vector that
      obtained from DMGetGlobalVector(). Do not use with vector obtained via
@@ -238,6 +263,7 @@ PetscErrorCode  DMRestoreGlobalVector(DM dm,Vec *g)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
   PetscValidPointer(g,2);
+  VecLocked(*g, 2);
   for (j=0; j<DM_MAX_WORK_VECTORS; j++) {
     if (*g == dm->globalout[j]) {
       dm->globalout[j] = NULL;
@@ -255,8 +281,45 @@ alldone:
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMGetNamedGlobalVector"
+/*@C
+   DMHasNamedGlobalVector - check for a named, persistent global vector
+
+   Not Collective
+
+   Input Arguments:
++  dm - DM to hold named vectors
+-  name - unique name for Vec
+
+   Output Arguments:
+.  exists - true if the vector was previously created
+
+   Level: developer
+
+   Note: If a Vec with the given name does not exist, it is created.
+
+.seealso: DMGetNamedGlobalVector(),DMRestoreNamedLocalVector()
+@*/
+PetscErrorCode DMHasNamedGlobalVector(DM dm,const char *name,PetscBool *exists)
+{
+  PetscErrorCode ierr;
+  DMNamedVecLink link;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  PetscValidCharPointer(name,2);
+  PetscValidPointer(exists,3);
+  *exists = PETSC_FALSE;
+  for (link=dm->namedglobal; link; link=link->next) {
+    PetscBool match;
+    ierr = PetscStrcmp(name,link->name,&match);CHKERRQ(ierr);
+    if (match) {
+      *exists = PETSC_TRUE;
+      break;
+    }
+  }
+  PetscFunctionReturn(0);
+}
+
 /*@C
    DMGetNamedGlobalVector - get access to a named, persistent global vector
 
@@ -306,8 +369,6 @@ found:
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMRestoreNamedGlobalVector"
 /*@C
    DMRestoreNamedGlobalVector - restore access to a named, persistent global vector
 
@@ -349,8 +410,45 @@ PetscErrorCode DMRestoreNamedGlobalVector(DM dm,const char *name,Vec *X)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMGetNamedLocalVector"
+/*@C
+   DMHasNamedLocalVector - check for a named, persistent local vector
+
+   Not Collective
+
+   Input Arguments:
++  dm - DM to hold named vectors
+-  name - unique name for Vec
+
+   Output Arguments:
+.  exists - true if the vector was previously created
+
+   Level: developer
+
+   Note: If a Vec with the given name does not exist, it is created.
+
+.seealso: DMGetNamedGlobalVector(),DMRestoreNamedLocalVector()
+@*/
+PetscErrorCode DMHasNamedLocalVector(DM dm,const char *name,PetscBool *exists)
+{
+  PetscErrorCode ierr;
+  DMNamedVecLink link;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  PetscValidCharPointer(name,2);
+  PetscValidPointer(exists,3);
+  *exists = PETSC_FALSE;
+  for (link=dm->namedlocal; link; link=link->next) {
+    PetscBool match;
+    ierr = PetscStrcmp(name,link->name,&match);CHKERRQ(ierr);
+    if (match) {
+      *exists = PETSC_TRUE;
+      break;
+    }
+  }
+  PetscFunctionReturn(0);
+}
+
 /*@C
    DMGetNamedLocalVector - get access to a named, persistent local vector
 
@@ -400,8 +498,6 @@ found:
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMRestoreNamedLocalVector"
 /*@C
    DMRestoreNamedLocalVector - restore access to a named, persistent local vector
 
