@@ -2,8 +2,6 @@ static const char help[] = "Test MatNest solving a linear system\n\n";
 
 #include <petscksp.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "test_solve"
 PetscErrorCode test_solve(void)
 {
   Mat            A11, A12,A21,A22, A, tmp[2][2];
@@ -99,11 +97,11 @@ PetscErrorCode test_solve(void)
   x2 = _tmp_x[1];
 
   PetscPrintf(PETSC_COMM_WORLD, "x1 \n");
-  PetscViewerSetFormat(PETSC_VIEWER_STDOUT_WORLD, PETSC_VIEWER_ASCII_INFO_DETAIL);
+  PetscViewerPushFormat(PETSC_VIEWER_STDOUT_WORLD, PETSC_VIEWER_ASCII_INFO_DETAIL);CHKERRQ(ierr);
   ierr = VecView(x1, PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
   PetscPrintf(PETSC_COMM_WORLD, "x2 \n");
-  PetscViewerSetFormat(PETSC_VIEWER_STDOUT_WORLD, PETSC_VIEWER_ASCII_INFO_DETAIL);
   ierr = VecView(x2, PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
+  PetscViewerPopFormat(PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
 
   ierr = KSPDestroy(&ksp);CHKERRQ(ierr);
   ierr = VecDestroy(&x);CHKERRQ(ierr);
@@ -119,8 +117,6 @@ PetscErrorCode test_solve(void)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "test_solve_matgetvecs"
 PetscErrorCode test_solve_matgetvecs(void)
 {
   Mat            A11, A12,A21, A;
@@ -206,11 +202,11 @@ PetscErrorCode test_solve_matgetvecs(void)
   x2   = tmp_x[1];
 
   ierr = PetscPrintf(PETSC_COMM_WORLD, "x1 \n");CHKERRQ(ierr);
-  ierr = PetscViewerSetFormat(PETSC_VIEWER_STDOUT_WORLD, PETSC_VIEWER_ASCII_INFO_DETAIL);CHKERRQ(ierr);
+  ierr = PetscViewerPushFormat(PETSC_VIEWER_STDOUT_WORLD, PETSC_VIEWER_ASCII_INFO_DETAIL);CHKERRQ(ierr);
   ierr = VecView(x1, PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_WORLD, "x2 \n");CHKERRQ(ierr);
-  ierr = PetscViewerSetFormat(PETSC_VIEWER_STDOUT_WORLD, PETSC_VIEWER_ASCII_INFO_DETAIL);CHKERRQ(ierr);
   ierr = VecView(x2, PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
+  ierr = PetscViewerPopFormat(PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
 
   ierr = KSPDestroy(&ksp);CHKERRQ(ierr);
   ierr = VecDestroy(&x);CHKERRQ(ierr);
@@ -223,15 +219,29 @@ PetscErrorCode test_solve_matgetvecs(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc, char **args)
 {
   PetscErrorCode ierr;
 
-  PetscInitialize(&argc, &args,(char*)0, help);
+  ierr = PetscInitialize(&argc, &args,(char*)0, help);if (ierr) return ierr;
   ierr = test_solve();CHKERRQ(ierr);
   ierr = test_solve_matgetvecs();CHKERRQ(ierr);
-  PetscFinalize();
-  return 0;
+  ierr = PetscFinalize();
+  return ierr;
 }
+
+/*TEST
+
+    test:
+
+    test:
+      suffix: 2
+      nsize: 2
+
+    test:
+      suffix: 3
+      nsize: 2
+      args: -ksp_monitor_short -ksp_type bicg
+      requires: !single
+
+TEST*/
