@@ -11,13 +11,13 @@ It is copied and intended to move dirty codes from ksp/examples/tutorials/ex10.c
    ex30 -f0 <datafile> -ksp_type preonly  \n\
         -help -ksp_view                  \n\
         -num_numfac <num_numfac> -num_rhs <num_rhs> \n\
-        -ksp_type preonly -pc_type lu -pc_factor_mat_solver_package superlu or superlu_dist or mumps \n\
-        -ksp_type preonly -pc_type cholesky -pc_factor_mat_solver_package mumps \n\
+        -ksp_type preonly -pc_type lu -pc_factor_mat_solver_type superlu or superlu_dist or mumps \n\
+        -ksp_type preonly -pc_type cholesky -pc_factor_mat_solver_type mumps \n\
    mpiexec -n <np> ex30 -f0 <datafile> -ksp_type cg -pc_type asm -pc_asm_type basic -sub_pc_type icc -mat_type sbaij
 
-   ./ex30 -f0 $D/small -mat_sigma -3.999999999999999 -ksp_type fgmres -pc_type lu -pc_factor_mat_solver_package superlu -mat_superlu_conditionnumber -ckerror -mat_superlu_diagpivotthresh 0
+   ./ex30 -f0 $D/small -mat_sigma -3.999999999999999 -ksp_type fgmres -pc_type lu -pc_factor_mat_solver_type superlu -mat_superlu_conditionnumber -ckerror -mat_superlu_diagpivotthresh 0
    ./ex30 -f0 $D/small -mat_sigma -3.999999999999999 -ksp_type fgmres -pc_type hypre -pc_hypre_type boomeramg -ksp_type fgmres -ckError
-   ./ex30 -f0 $D/small -mat_sigma -3.999999999999999 -ksp_type fgmres -pc_type lu -pc_factor_mat_solver_package petsc -pc_factor_shift_type NONZERO -pc_factor_shift_amount 1.e-5 -ckerror
+   ./ex30 -f0 $D/small -mat_sigma -3.999999999999999 -ksp_type fgmres -pc_type lu -pc_factor_mat_solver_type petsc -pc_factor_shift_type NONZERO -pc_factor_shift_amount 1.e-5 -ckerror
  \n\n";
 */
 /*T
@@ -27,8 +27,6 @@ T*/
 
 #include <petscksp.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
   KSP            ksp;
@@ -39,35 +37,35 @@ int main(int argc,char **args)
   PetscBool      table     = PETSC_FALSE,flg,flgB=PETSC_FALSE,trans=PETSC_FALSE,partition=PETSC_FALSE,initialguess = PETSC_FALSE;
   PetscBool      outputSoln=PETSC_FALSE;
   PetscErrorCode ierr;
-  PetscInt       its,num_numfac,n,M;
+  PetscInt       its,num_numfac;
   PetscReal      rnorm,enorm;
   PetscBool      preload=PETSC_TRUE,diagonalscale,isSymmetric,ckrnorm=PETSC_TRUE,Test_MatDuplicate=PETSC_FALSE,ckerror=PETSC_FALSE;
   PetscMPIInt    rank;
   PetscScalar    sigma;
   PetscInt       m;
 
-  PetscInitialize(&argc,&args,(char*)0,help);
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
   ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
-  ierr = PetscOptionsGetBool(NULL,"-table",&table,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetBool(NULL,"-trans",&trans,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetBool(NULL,"-partition",&partition,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetBool(NULL,"-initialguess",&initialguess,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetBool(NULL,"-output_solution",&outputSoln,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetBool(NULL,"-ckrnorm",&ckrnorm,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetBool(NULL,"-ckerror",&ckerror,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-table",&table,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-trans",&trans,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-partition",&partition,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-initialguess",&initialguess,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-output_solution",&outputSoln,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-ckrnorm",&ckrnorm,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-ckerror",&ckerror,NULL);CHKERRQ(ierr);
 
   /*
      Determine files from which we read the two linear systems
      (matrix and right-hand-side vector).
   */
-  ierr = PetscOptionsGetString(NULL,"-f",file[0],PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-f",file[0],PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr    = PetscStrcpy(file[1],file[0]);CHKERRQ(ierr);
     preload = PETSC_FALSE;
   } else {
-    ierr = PetscOptionsGetString(NULL,"-f0",file[0],PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsGetString(NULL,NULL,"-f0",file[0],PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
     if (!flg) SETERRQ(PETSC_COMM_WORLD,1,"Must indicate binary file with the -f0 or -f option");
-    ierr = PetscOptionsGetString(NULL,"-f1",file[1],PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsGetString(NULL,NULL,"-f1",file[1],PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
     if (!flg) preload = PETSC_FALSE;   /* don't bother with second system */
   }
 
@@ -80,7 +78,7 @@ int main(int argc,char **args)
         then load another (larger) system and solve it as well.
         This process preloads the instructions with the smaller
         system so that more accurate performance monitoring (via
-        -log_summary) can be done with the larger one (that actually
+        -log_view) can be done with the larger one (that actually
         is the system of interest).
   */
   PetscPreLoadBegin(preload,"Load system");
@@ -102,22 +100,22 @@ int main(int argc,char **args)
   ierr = MatSetFromOptions(A);CHKERRQ(ierr);
   ierr = MatLoad(A,fd);CHKERRQ(ierr);
 
-  if (!preload) {
-    flg  = PETSC_FALSE;
-    ierr = PetscOptionsGetString(NULL,"-rhs",file[2],PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
-    if (flg) {   /* rhs is stored in a separate file */
-      ierr = PetscViewerDestroy(&fd);CHKERRQ(ierr);
-      ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file[2],FILE_MODE_READ,&fd);CHKERRQ(ierr);
-    } else {
-      /* if file contains no RHS, then use a vector of all ones */
-      ierr = PetscInfo(0,"Using vector of ones for RHS\n");CHKERRQ(ierr);
-      ierr = MatGetLocalSize(A,&m,NULL);CHKERRQ(ierr);
-      ierr = VecCreate(PETSC_COMM_WORLD,&b);CHKERRQ(ierr);
-      ierr = VecSetSizes(b,m,PETSC_DECIDE);CHKERRQ(ierr);
-      ierr = VecSetFromOptions(b);CHKERRQ(ierr);
-      ierr = VecSet(b,1.0);CHKERRQ(ierr);
-      ierr = PetscObjectSetName((PetscObject)b, "Rhs vector");CHKERRQ(ierr);
-    }
+  flg  = PETSC_FALSE;
+  ierr = PetscOptionsGetString(NULL,NULL,"-rhs",file[2],PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+  if (flg) {   /* rhs is stored in a separate file */
+    ierr = PetscViewerDestroy(&fd);CHKERRQ(ierr);
+    ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file[2],FILE_MODE_READ,&fd);CHKERRQ(ierr);
+    ierr = VecCreate(PETSC_COMM_WORLD,&b);CHKERRQ(ierr);
+    ierr = VecLoad(b,fd);CHKERRQ(ierr);
+  } else {
+    /* if file contains no RHS, then use a vector of all ones */
+    ierr = PetscInfo(0,"Using vector of ones for RHS\n");CHKERRQ(ierr);
+    ierr = MatGetLocalSize(A,&m,NULL);CHKERRQ(ierr);
+    ierr = VecCreate(PETSC_COMM_WORLD,&b);CHKERRQ(ierr);
+    ierr = VecSetSizes(b,m,PETSC_DECIDE);CHKERRQ(ierr);
+    ierr = VecSetFromOptions(b);CHKERRQ(ierr);
+    ierr = VecSet(b,1.0);CHKERRQ(ierr);
+    ierr = PetscObjectSetName((PetscObject)b, "Rhs vector");CHKERRQ(ierr);
   }
   ierr = PetscViewerDestroy(&fd);CHKERRQ(ierr);
 
@@ -126,15 +124,15 @@ int main(int argc,char **args)
     ierr = MatDuplicate(A,MAT_COPY_VALUES,&B);CHKERRQ(ierr);
     ierr = MatEqual(A,B,&flg);CHKERRQ(ierr);
     if (!flg) {
-      PetscPrintf(PETSC_COMM_WORLD,"  A != B \n");CHKERRQ(ierr);
+      ierr = PetscPrintf(PETSC_COMM_WORLD,"  A != B \n");CHKERRQ(ierr);
     }
     ierr = MatDestroy(&B);CHKERRQ(ierr);
   }
 
   /* Add a shift to A */
-  ierr = PetscOptionsGetScalar(NULL,"-mat_sigma",&sigma,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetScalar(NULL,NULL,"-mat_sigma",&sigma,&flg);CHKERRQ(ierr);
   if (flg) {
-    ierr = PetscOptionsGetString(NULL,"-fB",file[2],PETSC_MAX_PATH_LEN,&flgB);CHKERRQ(ierr);
+    ierr = PetscOptionsGetString(NULL,NULL,"-fB",file[2],PETSC_MAX_PATH_LEN,&flgB);CHKERRQ(ierr);
     if (flgB) {
       /* load B to get A = A + sigma*B */
       ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file[2],FILE_MODE_READ,&fd);CHKERRQ(ierr);
@@ -151,7 +149,7 @@ int main(int argc,char **args)
   /* Make A singular for testing zero-pivot of ilu factorization        */
   /* Example: ./ex30 -f0 <datafile> -test_zeropivot -set_row_zero -pc_factor_shift_nonzero */
   flg  = PETSC_FALSE;
-  ierr = PetscOptionsGetBool(NULL, "-test_zeropivot", &flg,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL, "-test_zeropivot", &flg,NULL);CHKERRQ(ierr);
   if (flg) {
     PetscInt          row,ncols;
     const PetscInt    *cols;
@@ -160,9 +158,9 @@ int main(int argc,char **args)
     PetscScalar       *zeros;
     row  = 0;
     ierr = MatGetRow(A,row,&ncols,&cols,&vals);CHKERRQ(ierr);
-    ierr = PetscCalloc1(ncols+1,&zeros);
+    ierr = PetscCalloc1(ncols+1,&zeros);CHKERRQ(ierr);
     flg1 = PETSC_FALSE;
-    ierr = PetscOptionsGetBool(NULL, "-set_row_zero", &flg1,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsGetBool(NULL,NULL, "-set_row_zero", &flg1,NULL);CHKERRQ(ierr);
     if (flg1) {   /* set entire row as zero */
       ierr = MatSetValues(A,1,&row,ncols,cols,zeros,INSERT_VALUES);CHKERRQ(ierr);
     } else {   /* only set (row,row) entry as zero */
@@ -174,49 +172,19 @@ int main(int argc,char **args)
 
   /* Check whether A is symmetric */
   flg  = PETSC_FALSE;
-  ierr = PetscOptionsGetBool(NULL, "-check_symmetry", &flg,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL, "-check_symmetry", &flg,NULL);CHKERRQ(ierr);
   if (flg) {
     Mat Atrans;
-    ierr = MatTranspose(A, MAT_INITIAL_MATRIX,&Atrans);
-    ierr = MatEqual(A, Atrans, &isSymmetric);
+    ierr = MatTranspose(A, MAT_INITIAL_MATRIX,&Atrans);CHKERRQ(ierr);
+    ierr = MatEqual(A, Atrans, &isSymmetric);CHKERRQ(ierr);
     if (isSymmetric) {
-      PetscPrintf(PETSC_COMM_WORLD,"A is symmetric \n");CHKERRQ(ierr);
+      ierr = PetscPrintf(PETSC_COMM_WORLD,"A is symmetric \n");CHKERRQ(ierr);
     } else {
-      PetscPrintf(PETSC_COMM_WORLD,"A is non-symmetric \n");CHKERRQ(ierr);
+      ierr = PetscPrintf(PETSC_COMM_WORLD,"A is non-symmetric \n");CHKERRQ(ierr);
     }
     ierr = MatDestroy(&Atrans);CHKERRQ(ierr);
   }
 
-  /*
-     If the loaded matrix is larger than the vector (due to being padded
-     to match the block size of the system), then create a new padded vector.
-  */
-
-  ierr = MatGetLocalSize(A,&m,&n);CHKERRQ(ierr);
-  if (m != n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ, "This example is not intended for rectangular matrices (%d, %d)", m, n);
-  ierr = MatGetSize(A,&M,NULL);CHKERRQ(ierr);
-  ierr = VecGetSize(b,&m);CHKERRQ(ierr);
-  if (M != m) {   /* Create a new vector b by padding the old one */
-    PetscInt    j,mvec,start,end,indx;
-    Vec         tmp;
-    PetscScalar *bold;
-
-    ierr = VecCreate(PETSC_COMM_WORLD,&tmp);CHKERRQ(ierr);
-    ierr = VecSetSizes(tmp,n,PETSC_DECIDE);CHKERRQ(ierr);
-    ierr = VecSetFromOptions(tmp);CHKERRQ(ierr);
-    ierr = VecGetOwnershipRange(b,&start,&end);CHKERRQ(ierr);
-    ierr = VecGetLocalSize(b,&mvec);CHKERRQ(ierr);
-    ierr = VecGetArray(b,&bold);CHKERRQ(ierr);
-    for (j=0; j<mvec; j++) {
-      indx = start+j;
-      ierr = VecSetValues(tmp,1,&indx,bold+j,INSERT_VALUES);CHKERRQ(ierr);
-    }
-    ierr = VecRestoreArray(b,&bold);CHKERRQ(ierr);
-    ierr = VecDestroy(&b);CHKERRQ(ierr);
-    ierr = VecAssemblyBegin(tmp);CHKERRQ(ierr);
-    ierr = VecAssemblyEnd(tmp);CHKERRQ(ierr);
-    b    = tmp;
-  }
   ierr = VecDuplicate(b,&b2);CHKERRQ(ierr);
   ierr = VecDuplicate(b,&x);CHKERRQ(ierr);
   ierr = PetscObjectSetName((PetscObject)x, "Solution vector");CHKERRQ(ierr);
@@ -255,7 +223,7 @@ int main(int argc,char **args)
     ierr = PetscFree(count);CHKERRQ(ierr);
     ierr = ISDestroy(&nis);CHKERRQ(ierr);
     ierr = ISSort(is);CHKERRQ(ierr);
-    ierr = MatGetSubMatrix(A,is,is,MAT_INITIAL_MATRIX,&BB);CHKERRQ(ierr);
+    ierr = MatCreateSubMatrix(A,is,is,MAT_INITIAL_MATRIX,&BB);CHKERRQ(ierr);
 
     /* need to move the vector also */
     ierr = ISDestroy(&is);CHKERRQ(ierr);
@@ -269,7 +237,7 @@ int main(int argc,char **args)
   ierr       = KSPCreate(PETSC_COMM_WORLD,&ksp);CHKERRQ(ierr);
   ierr       = KSPSetInitialGuessNonzero(ksp,initialguess);CHKERRQ(ierr);
   num_numfac = 1;
-  ierr       = PetscOptionsGetInt(NULL,"-num_numfac",&num_numfac,NULL);CHKERRQ(ierr);
+  ierr       = PetscOptionsGetInt(NULL,NULL,"-num_numfac",&num_numfac,NULL);CHKERRQ(ierr);
   while (num_numfac--) {
 
     ierr = KSPSetOperators(ksp,A,A);CHKERRQ(ierr);
@@ -290,7 +258,7 @@ int main(int argc,char **args)
      than diagonally scaling the matrix before computing the preconditioner
     */
     diagonalscale = PETSC_FALSE;
-    ierr          = PetscOptionsGetBool(NULL,"-diagonal_scale",&diagonalscale,NULL);CHKERRQ(ierr);
+    ierr          = PetscOptionsGetBool(NULL,NULL,"-diagonal_scale",&diagonalscale,NULL);CHKERRQ(ierr);
     if (diagonalscale) {
       PC       pc;
       PetscInt j,start,end,n;
@@ -320,7 +288,7 @@ int main(int argc,char **args)
       ierr = KSPGetIterationNumber(ksp,&its);CHKERRQ(ierr);
     } else {
       PetscInt num_rhs=1;
-      ierr = PetscOptionsGetInt(NULL,"-num_rhs",&num_rhs,NULL);CHKERRQ(ierr);
+      ierr = PetscOptionsGetInt(NULL,NULL,"-num_rhs",&num_rhs,NULL);CHKERRQ(ierr);
 
       while (num_rhs--) {
         ierr = KSPSolve(ksp,b,x);CHKERRQ(ierr);
@@ -371,7 +339,7 @@ int main(int argc,char **args)
       ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
     }
 
-    ierr = PetscOptionsGetString(NULL,"-solution",file[3],PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+    ierr = PetscOptionsGetString(NULL,NULL,"-solution",file[3],PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
     if (flg) {
       PetscViewer viewer;
       Vec         xstar;
@@ -394,11 +362,11 @@ int main(int argc,char **args)
     }
 
     flg  = PETSC_FALSE;
-    ierr = PetscOptionsGetBool(NULL, "-ksp_reason", &flg,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsGetBool(NULL,NULL, "-ksp_reason", &flg,NULL);CHKERRQ(ierr);
     if (flg) {
       KSPConvergedReason reason;
       ierr = KSPGetConvergedReason(ksp,&reason);CHKERRQ(ierr);
-      PetscPrintf(PETSC_COMM_WORLD,"KSPConvergedReason: %D\n", reason);
+      ierr = PetscPrintf(PETSC_COMM_WORLD,"KSPConvergedReason: %D\n", reason);CHKERRQ(ierr);
     }
 
   }   /* while (num_numfac--) */
@@ -418,6 +386,69 @@ int main(int argc,char **args)
      ----------------------------------------------------------- */
 
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
+/*TEST
+
+    test:
+      args: -f ${DATAFILESPATH}/matrices/small -ksp_type preonly -pc_type ilu -pc_factor_mat_ordering_type natural -num_numfac 2 -pc_factor_reuse_fill
+      requires: datafilespath !complex double !define(PETSC_USE_64BIT_INDICES)
+      output_file: output/ex30.out
+
+    test:
+      suffix: 2
+      args: -f ${DATAFILESPATH}/matrices/arco1 -mat_type baij -matload_block_size 3 -ksp_type preonly -pc_type ilu -pc_factor_mat_ordering_type natural -num_numfac 2
+      requires: datafilespath !complex double !define(PETSC_USE_64BIT_INDICES)
+
+    test:
+      suffix: shiftnz
+      args: -f0 ${DATAFILESPATH}/matrices/small -mat_sigma -4.0 -ksp_type preonly -pc_type lu -pc_factor_shift_type NONZERO -pc_factor_shift_amount 1.e-5
+      requires: datafilespath !complex double !define(PETSC_USE_64BIT_INDICES)
+
+    test:
+      suffix: shiftpd
+      args: -f0 ${DATAFILESPATH}/matrices/small -mat_sigma -4.0 -ksp_type preonly -pc_type lu -pc_factor_shift_type POSITIVE_DEFINITE
+      requires: datafilespath !complex double !define(PETSC_USE_64BIT_INDICES)
+
+    test:
+      suffix: shift_cholesky_aij
+      args: -f0 ${DATAFILESPATH}/matrices/small -mat_sigma -4.0 -ksp_type preonly -pc_type cholesky -pc_factor_shift_type NONZERO -pc_factor_shift_amount 1.e-5 
+      requires: datafilespath !complex double !define(PETSC_USE_64BIT_INDICES)
+      output_file: output/ex30_shiftnz.out
+
+    test:
+      suffix: shiftpd_2
+      args: -f0 ${DATAFILESPATH}/matrices/small -mat_sigma -4.0 -ksp_type preonly -pc_type cholesky -pc_factor_shift_type POSITIVE_DEFINITE
+      requires: datafilespath !complex double !define(PETSC_USE_64BIT_INDICES)
+
+    test:
+      suffix: shift_cholesky_sbaij
+      args: -f0 ${DATAFILESPATH}/matrices/small -mat_sigma -4.0 -ksp_type preonly -pc_type cholesky -pc_factor_shift_type NONZERO -pc_factor_shift_amount 1.e-5 -mat_type sbaij
+      requires: datafilespath !complex double !define(PETSC_USE_64BIT_INDICES)
+      output_file: output/ex30_shiftnz.out
+
+    test:
+      suffix: shiftpd_2_sbaij
+      args: -f0 ${DATAFILESPATH}/matrices/small -mat_sigma -4.0 -ksp_type preonly -pc_type cholesky -pc_factor_shift_type POSITIVE_DEFINITE -mat_type sbaij
+      requires: datafilespath !complex double !define(PETSC_USE_64BIT_INDICES)
+      output_file: output/ex30_shiftpd_2.out
+
+    test:
+      suffix: shiftinblocks
+      args:  -f0 ${DATAFILESPATH}/matrices/small -mat_sigma -4.0 -ksp_type preonly -pc_type lu -pc_factor_shift_type INBLOCKS
+      requires: datafilespath !complex double !define(PETSC_USE_64BIT_INDICES)
+
+    test:
+      suffix: shiftinblocks2
+      args: -f0 ${DATAFILESPATH}/matrices/small -mat_sigma -4.0 -ksp_type preonly -pc_type cholesky -pc_factor_shift_type INBLOCKS
+      requires: datafilespath !complex double !define(PETSC_USE_64BIT_INDICES)
+      output_file: output/ex30_shiftinblocks.out
+
+    test:
+      suffix: shiftinblockssbaij
+      args: -f0 ${DATAFILESPATH}/matrices/small -mat_sigma -4.0 -ksp_type preonly -pc_type cholesky -pc_factor_shift_type INBLOCKS -mat_type sbaij
+      requires: datafilespath !complex double !define(PETSC_USE_64BIT_INDICES)
+      output_file: output/ex30_shiftinblocks.out
+
+TEST*/
