@@ -6,11 +6,8 @@ Input arguments are:\n\
 #include <petscksp.h>
 #include <petsclog.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
-#if !defined(PETSC_USE_COMPLEX)
   PetscErrorCode ierr;
   PetscInt       its;
   PetscLogStage  stage1,stage2;
@@ -21,18 +18,12 @@ int main(int argc,char **args)
   PetscViewer    fd;
   PetscBool      table = PETSC_FALSE,flg;
   KSP            ksp;
-#endif
-
-  PetscInitialize(&argc,&args,(char*)0,help);
-
-#if defined(PETSC_USE_COMPLEX)
-  SETERRQ(PETSC_COMM_WORLD,1,"This example does not work with complex numbers");
-#else
-  ierr = PetscOptionsGetBool(NULL,"-table",&table,NULL);CHKERRQ(ierr);
 
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
+  ierr = PetscOptionsGetBool(NULL,NULL,"-table",&table,NULL);CHKERRQ(ierr);
 
   /* Read matrix and RHS */
-  ierr = PetscOptionsGetString(NULL,"-f",file,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-f",file,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
   if (!flg) SETERRQ(PETSC_COMM_WORLD,1,"Must indicate binary file with the -f option");
   ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file,FILE_MODE_READ,&fd);CHKERRQ(ierr);
   ierr = MatCreate(PETSC_COMM_WORLD,&A);CHKERRQ(ierr);
@@ -89,7 +80,7 @@ int main(int argc,char **args)
   PetscLogStagePop();
 
   /* Show result */
-  ierr = MatMult(A,x,u);
+  ierr = MatMult(A,x,u);CHKERRQ(ierr);
   ierr = VecAXPY(u,-1.0,b);CHKERRQ(ierr);
   ierr = VecNorm(u,NORM_2,&norm);CHKERRQ(ierr);
   ierr = KSPGetIterationNumber(ksp,&its);CHKERRQ(ierr);
@@ -113,9 +104,24 @@ int main(int argc,char **args)
   ierr = VecDestroy(&b);CHKERRQ(ierr);
   ierr = VecDestroy(&u);CHKERRQ(ierr);
   ierr = MatDestroy(&A);CHKERRQ(ierr);
-
   ierr = PetscFinalize();
-#endif
-  return 0;
+  return ierr;
 }
 
+/*TEST
+
+    test:
+      args: -ksp_type preonly  -pc_type lu -options_left no  -f ${DATAFILESPATH}/matrices/arco1 
+      requires: datafilespath double !complex !define(PETSC_USE_64BIT_INDICES)
+
+    test:
+      suffix: 2
+      args: -sub_pc_type ilu -options_left no  -f ${DATAFILESPATH}/matrices/arco1 -ksp_gmres_restart 100 -ksp_gmres_cgs_refinement_type refine_always -sub_ksp_type preonly -pc_type bjacobi -pc_bjacobi_blocks 8 -sub_pc_factor_in_place -ksp_monitor_short
+      requires: datafilespath double  !complex !define(PETSC_USE_64BIT_INDICES)
+
+    test:
+      suffix: 7
+      args: -ksp_gmres_cgs_refinement_type refine_always -pc_type asm -pc_asm_blocks 6 -f ${DATAFILESPATH}/matrices/small -matload_block_size 6  -ksp_monitor_short  
+      requires: datafilespath double  !complex !define(PETSC_USE_64BIT_INDICES)
+
+TEST*/
