@@ -1,17 +1,17 @@
 
 static char help[] = "Solves a linear system in parallel with KSP.  Also\n\
 illustrates setting a user-defined shell preconditioner and using the\n\
-macro __FUNCT__ to define routine names for use in error handling.\n\
 Input parameters include:\n\
   -user_defined_pc : Activate a user-defined preconditioner\n\n";
 
 /*T
    Concepts: KSP^basic parallel example
    Concepts: PC^setting a user-defined shell preconditioner
-   Concepts: error handling^Using the macro __FUNCT__ to define routine names;
    Processors: n
 T*/
 
+
+
 /*
   Include "petscksp.h" so that we can use KSP solvers.  Note that this file
   automatically includes:
@@ -35,17 +35,13 @@ extern PetscErrorCode SampleShellPCDestroy(PC);
 
 /*
    User-defined routines.  Note that immediately before each routine below,
-   we define the macro __FUNCT__ to be a string containing the routine name.
    If defined, this macro is used in the PETSc error handlers to provide a
    complete traceback of routine names.  All PETSc library routines use this
    macro, and users can optionally employ it as well in their application
    codes.  Note that users can get a traceback of PETSc errors regardless of
-   whether they define __FUNCT__ in application codes; this macro merely
    provides the added traceback detail of the application routine names.
 */
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
   Vec            x,b,u;   /* approx solution, RHS, exact solution */
@@ -59,9 +55,9 @@ int main(int argc,char **args)
   PetscErrorCode ierr;
   PetscBool      user_defined_pc = PETSC_FALSE;
 
-  PetscInitialize(&argc,&args,(char*)0,help);
-  ierr = PetscOptionsGetInt(NULL,"-m",&m,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetInt(NULL,"-n",&n,NULL);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
+  ierr = PetscOptionsGetInt(NULL,NULL,"-m",&m,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-n",&n,NULL);CHKERRQ(ierr);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
          Compute the matrix and right-hand-side vector that define
@@ -157,7 +153,7 @@ int main(int argc,char **args)
   /*
      Set a user-defined "shell" preconditioner if desired
   */
-  ierr = PetscOptionsGetBool(NULL,"-user_defined_pc",&user_defined_pc,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-user_defined_pc",&user_defined_pc,NULL);CHKERRQ(ierr);
   if (user_defined_pc) {
     /* (Required) Indicate to PETSc that we're using a "shell" preconditioner */
     ierr = PCSetType(pc,PCSHELL);CHKERRQ(ierr);
@@ -220,7 +216,7 @@ int main(int argc,char **args)
   ierr = VecDestroy(&b);CHKERRQ(ierr);  ierr = MatDestroy(&A);CHKERRQ(ierr);
 
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 
 }
 
@@ -228,8 +224,6 @@ int main(int argc,char **args)
 /*          Routines for a user-defined shell preconditioner           */
 /***********************************************************************/
 
-#undef __FUNCT__
-#define __FUNCT__ "SampleShellPCCreate"
 /*
    SampleShellPCCreate - This routine creates a user-defined
    preconditioner context.
@@ -248,8 +242,6 @@ PetscErrorCode SampleShellPCCreate(SampleShellPC **shell)
   return 0;
 }
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "SampleShellPCSetUp"
 /*
    SampleShellPCSetUp - This routine sets up a user-defined
    preconditioner context.
@@ -283,8 +275,6 @@ PetscErrorCode SampleShellPCSetUp(PC pc,Mat pmat,Vec x)
   return 0;
 }
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "SampleShellPCApply"
 /*
    SampleShellPCApply - This routine demonstrates the use of a
    user-provided preconditioner.
@@ -312,8 +302,6 @@ PetscErrorCode SampleShellPCApply(PC pc,Vec x,Vec y)
   return 0;
 }
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "SampleShellPCDestroy"
 /*
    SampleShellPCDestroy - This routine destroys a user-defined
    preconditioner context.
@@ -332,3 +320,20 @@ PetscErrorCode SampleShellPCDestroy(PC pc)
 
   return 0;
 }
+
+
+/*TEST
+
+   build:
+      requires: !complex !single
+
+   test:
+      nsize: 2
+      args: -ksp_view -user_defined_pc -ksp_gmres_cgs_refinement_type refine_always
+
+   test:
+      suffix: tsirm
+      args: -m 60 -n 60 -ksp_type tsirm -pc_type ksp -ksp_monitor_short -ksp_ksp_type fgmres -ksp_ksp_rtol 1e-10 -ksp_pc_type mg -ksp_ksp_max_it 30
+      timeoutfactor: 4
+
+TEST*/
