@@ -13,6 +13,8 @@ Input parameters include:\n\
    Processors: n
 T*/
 
+
+
 /*
   Include "petscksp.h" so that we can use KSP solvers.  Note that this file
   automatically includes:
@@ -23,13 +25,11 @@ T*/
 */
 #include <petscksp.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
-  Vec            x,b,u;  /* approx solution, RHS, exact solution */
+  Vec            x,b,u;    /* approx solution, RHS, exact solution */
   Mat            A;        /* linear system matrix */
-  KSP            ksp;     /* linear solver context */
+  KSP            ksp;      /* linear solver context */
   PetscRandom    rctx;     /* random number generator context */
   PetscReal      norm;     /* norm of solution error */
   PetscInt       i,j,Ii,J,Istart,Iend,m = 8,n = 7,its;
@@ -40,9 +40,9 @@ int main(int argc,char **args)
   PetscLogStage stage;
 #endif
 
-  PetscInitialize(&argc,&args,(char*)0,help);
-  ierr = PetscOptionsGetInt(NULL,"-m",&m,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetInt(NULL,"-n",&n,NULL);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
+  ierr = PetscOptionsGetInt(NULL,NULL,"-m",&m,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-n",&n,NULL);CHKERRQ(ierr);
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
          Compute the matrix and right-hand-side vector that define
          the linear system, Ax = b.
@@ -62,6 +62,10 @@ int main(int argc,char **args)
   ierr = MatSetFromOptions(A);CHKERRQ(ierr);
   ierr = MatMPIAIJSetPreallocation(A,5,NULL,5,NULL);CHKERRQ(ierr);
   ierr = MatSeqAIJSetPreallocation(A,5,NULL);CHKERRQ(ierr);
+  ierr = MatSeqSBAIJSetPreallocation(A,1,5,NULL);CHKERRQ(ierr);
+  ierr = MatMPISBAIJSetPreallocation(A,1,5,NULL,5,NULL);CHKERRQ(ierr);
+  ierr = MatMPISELLSetPreallocation(A,5,NULL,5,NULL);CHKERRQ(ierr);
+  ierr = MatSeqSELLSetPreallocation(A,5,NULL);CHKERRQ(ierr);
 
   /*
      Currently, all PETSc parallel matrix formats are partitioned by
@@ -135,7 +139,7 @@ int main(int argc,char **args)
      elements of 1.0;  Alternatively, using the runtime option
      -random_sol forms a solution vector with random components.
   */
-  ierr = PetscOptionsGetBool(NULL,"-random_exact_sol",&flg,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-random_exact_sol",&flg,NULL);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscRandomCreate(PETSC_COMM_WORLD,&rctx);CHKERRQ(ierr);
     ierr = PetscRandomSetFromOptions(rctx);CHKERRQ(ierr);
@@ -150,7 +154,7 @@ int main(int argc,char **args)
      View the exact solution vector if desired
   */
   flg  = PETSC_FALSE;
-  ierr = PetscOptionsGetBool(NULL,"-view_exact_sol",&flg,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-view_exact_sol",&flg,NULL);CHKERRQ(ierr);
   if (flg) {ierr = VecView(u,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);}
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -226,8 +230,127 @@ int main(int argc,char **args)
      Always call PetscFinalize() before exiting a program.  This routine
        - finalizes the PETSc libraries as well as MPI
        - provides summary and diagnostic information if certain runtime
-         options are chosen (e.g., -log_summary).
+         options are chosen (e.g., -log_view).
   */
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
+
+
+/*TEST
+
+   build:
+      requires: !complex !single
+
+   test:
+      suffix: chebyest_1
+      args: -m 80 -n 80 -ksp_pc_side right -pc_type ksp -ksp_ksp_type chebyshev -ksp_ksp_max_it 5 -ksp_ksp_chebyshev_esteig 0.9,0,0,1.1 -ksp_monitor_short
+
+   test:
+      suffix: chebyest_2
+      args: -m 80 -n 80 -ksp_pc_side right -pc_type ksp -ksp_ksp_type chebyshev -ksp_ksp_max_it 5 -ksp_ksp_chebyshev_esteig 0.9,0,0,1.1 -ksp_esteig_ksp_type cg -ksp_monitor_short 
+
+   test:
+      args: -ksp_monitor_short -m 5 -n 5 -ksp_gmres_cgs_refinement_type refine_always
+
+   test:
+      suffix: 2
+      nsize: 2
+      args: -ksp_monitor_short -m 5 -n 5 -ksp_gmres_cgs_refinement_type refine_always
+
+   test:
+      suffix: 3
+      args: -pc_type sor -pc_sor_symmetric -ksp_monitor_short -ksp_gmres_cgs_refinement_type refine_always
+
+   test:
+      suffix: 4
+      args: -pc_type eisenstat -ksp_monitor_short -ksp_gmres_cgs_refinement_type refine_always
+
+   test:
+      suffix: 5
+      nsize: 2
+      args: -ksp_monitor_short -m 5 -n 5 -mat_view draw -ksp_gmres_cgs_refinement_type refine_always -nox
+      output_file: output/ex2_2.out
+
+   test:
+      suffix: bjacobi
+      nsize: 4
+      args: -pc_type bjacobi -pc_bjacobi_blocks 1 -ksp_monitor_short -sub_pc_type jacobi -sub_ksp_type gmres
+
+   test:
+      suffix: bjacobi_2
+      nsize: 4
+      args: -pc_type bjacobi -pc_bjacobi_blocks 2 -ksp_monitor_short -sub_pc_type jacobi -sub_ksp_type gmres -ksp_view
+
+   test:
+      suffix: bjacobi_3
+      nsize: 4
+      args: -pc_type bjacobi -pc_bjacobi_blocks 4 -ksp_monitor_short -sub_pc_type jacobi -sub_ksp_type gmres
+
+   test:
+      suffix: fbcgs
+      args: -ksp_type fbcgs -pc_type ilu
+
+   test:
+      suffix: fbcgs_2
+      nsize: 3
+      args: -ksp_type fbcgsr -pc_type bjacobi
+
+   test:
+      suffix: groppcg
+      args: -ksp_monitor_short -ksp_type groppcg -m 9 -n 9
+
+   test:
+      suffix: mkl_pardiso_cholesky
+      requires: mkl_pardiso
+      args: -ksp_type preonly -pc_type cholesky -mat_type sbaij -pc_factor_mat_solver_type mkl_pardiso
+
+   test:
+      suffix: mkl_pardiso_lu
+      requires: mkl_pardiso
+      args: -ksp_type preonly -pc_type lu -pc_factor_mat_solver_type mkl_pardiso
+
+   test:
+      suffix: pipebcgs
+      args: -ksp_monitor_short -ksp_type pipebcgs -m 9 -n 9
+
+   test:
+      suffix: pipecg
+      args: -ksp_monitor_short -ksp_type pipecg -m 9 -n 9
+
+   test:
+      suffix: pipecgrr
+      args: -ksp_monitor_short -ksp_type pipecgrr -m 9 -n 9
+
+   test:
+      suffix: pipecr
+      args: -ksp_monitor_short -ksp_type pipecr -m 9 -n 9
+
+   test:
+      suffix: pipelcg
+      args: -ksp_monitor_short -ksp_type pipelcg -m 9 -n 9 -pc_type none -ksp_pipelcg_pipel 2 -ksp_pipelcg_lmax 2
+      filter: grep -v "sqrt breakdown in iteration"
+
+   test:
+      suffix: sell
+      args: -ksp_monitor_short -ksp_gmres_cgs_refinement_type refine_always -m 9 -n 9 -mat_type sell
+
+   test:
+      requires: mumps
+      suffix: sell_mumps
+      args: -ksp_type preonly -m 9 -n 12 -mat_type sell -pc_type lu -pc_factor_mat_solver_type mumps -pc_factor_mat_ordering_type natural
+
+   test:
+      suffix: telescope
+      nsize: 4
+      args: -m 100 -n 100 -ksp_converged_reason -pc_type telescope -pc_telescope_reduction_factor 4 -telescope_pc_type bjacobi
+
+   test:
+      suffix: umfpack
+      requires: suitesparse
+      args: -ksp_type preonly -pc_type lu -pc_factor_mat_solver_type umfpack
+
+   test:
+     suffix: pc_symmetric
+     args: -m 10 -n 9 -ksp_converged_reason -ksp_type gmres -ksp_pc_side symmetric -pc_type cholesky
+TEST*/
