@@ -4,6 +4,8 @@
    Processors: n
 T*/
 
+
+
 /*
 Added at the request of Marc Garbey.
 
@@ -41,8 +43,6 @@ typedef struct {
   BCType    bcType;
 } UserContext;
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   KSP            ksp;
@@ -53,14 +53,15 @@ int main(int argc,char **argv)
   PetscInt       bc;
   Vec            b,x;
 
-  PetscInitialize(&argc,&argv,(char*)0,help);
-
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   ierr = KSPCreate(PETSC_COMM_WORLD,&ksp);CHKERRQ(ierr);
-  ierr = DMDACreate2d(PETSC_COMM_WORLD, DM_BOUNDARY_NONE, DM_BOUNDARY_NONE,DMDA_STENCIL_STAR,-3,-3,PETSC_DECIDE,PETSC_DECIDE,1,1,0,0,&da);CHKERRQ(ierr);
+  ierr = DMDACreate2d(PETSC_COMM_WORLD, DM_BOUNDARY_NONE, DM_BOUNDARY_NONE,DMDA_STENCIL_STAR,3,3,PETSC_DECIDE,PETSC_DECIDE,1,1,0,0,&da);CHKERRQ(ierr);
+  ierr = DMSetFromOptions(da);CHKERRQ(ierr);
+  ierr = DMSetUp(da);CHKERRQ(ierr);
   ierr = DMDASetUniformCoordinates(da,0,1,0,1,0,0);CHKERRQ(ierr);
   ierr = DMDASetFieldName(da,0,"Pressure");CHKERRQ(ierr);
 
-  ierr        = PetscOptionsBegin(PETSC_COMM_WORLD, "", "Options for the inhomogeneous Poisson equation", "DMqq");
+  ierr        = PetscOptionsBegin(PETSC_COMM_WORLD, "", "Options for the inhomogeneous Poisson equation", "DMqq");CHKERRQ(ierr);
   user.rho    = 1.0;
   ierr        = PetscOptionsReal("-rho", "The conductivity", "ex29.c", user.rho, &user.rho, NULL);CHKERRQ(ierr);
   user.nu     = 0.1;
@@ -68,7 +69,7 @@ int main(int argc,char **argv)
   bc          = (PetscInt)DIRICHLET;
   ierr        = PetscOptionsEList("-bc_type","Type of boundary condition","ex29.c",bcTypes,2,bcTypes[0],&bc,NULL);CHKERRQ(ierr);
   user.bcType = (BCType)bc;
-  ierr        = PetscOptionsEnd();
+  ierr        = PetscOptionsEnd();CHKERRQ(ierr);
 
   ierr = KSPSetComputeRHS(ksp,ComputeRHS,&user);CHKERRQ(ierr);
   ierr = KSPSetComputeOperators(ksp,ComputeMatrix,&user);CHKERRQ(ierr);
@@ -82,12 +83,9 @@ int main(int argc,char **argv)
   ierr = DMDestroy(&da);CHKERRQ(ierr);
   ierr = KSPDestroy(&ksp);CHKERRQ(ierr);
   ierr = PetscFinalize();
-
-  return 0;
+  return ierr;
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ComputeRHS"
 PetscErrorCode ComputeRHS(KSP ksp,Vec b,void *ctx)
 {
   UserContext    *user = (UserContext*)ctx;
@@ -126,8 +124,6 @@ PetscErrorCode ComputeRHS(KSP ksp,Vec b,void *ctx)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "ComputeRho"
 PetscErrorCode ComputeRho(PetscInt i, PetscInt j, PetscInt mx, PetscInt my, PetscReal centerRho, PetscReal *rho)
 {
   PetscFunctionBeginUser;
@@ -139,8 +135,6 @@ PetscErrorCode ComputeRho(PetscInt i, PetscInt j, PetscInt mx, PetscInt my, Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ComputeMatrix"
 PetscErrorCode ComputeMatrix(KSP ksp,Mat J,Mat jac,void *ctx)
 {
   UserContext    *user = (UserContext*)ctx;
@@ -151,6 +145,7 @@ PetscErrorCode ComputeMatrix(KSP ksp,Mat J,Mat jac,void *ctx)
   PetscReal      Hx,Hy,HydHx,HxdHy,rho;
   MatStencil     row, col[5];
   DM             da;
+  PetscBool      check_matis = PETSC_FALSE;
 
   PetscFunctionBeginUser;
   ierr      = KSPGetDM(ksp,&da);CHKERRQ(ierr);
@@ -203,6 +198,28 @@ PetscErrorCode ComputeMatrix(KSP ksp,Mat J,Mat jac,void *ctx)
   }
   ierr = MatAssemblyBegin(jac,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(jac,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+  ierr = MatViewFromOptions(jac,NULL,"-view_mat");CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-check_matis",&check_matis,NULL);CHKERRQ(ierr);
+  if (check_matis) {
+    void      (*f)(void);
+    Mat       J2;
+    MatType   jtype;
+    PetscReal nrm;
+
+    ierr = MatGetType(jac,&jtype);CHKERRQ(ierr);
+    ierr = MatConvert(jac,MATIS,MAT_INITIAL_MATRIX,&J2);CHKERRQ(ierr);
+    ierr = MatViewFromOptions(J2,NULL,"-view_conv");CHKERRQ(ierr);
+    ierr = MatConvert(J2,jtype,MAT_INPLACE_MATRIX,&J2);CHKERRQ(ierr);
+    ierr = MatGetOperation(jac,MATOP_VIEW,&f);CHKERRQ(ierr);
+    ierr = MatSetOperation(J2,MATOP_VIEW,f);CHKERRQ(ierr);
+    ierr = MatSetDM(J2,da);CHKERRQ(ierr);
+    ierr = MatViewFromOptions(J2,NULL,"-view_conv_assembled");CHKERRQ(ierr);
+    ierr = MatAXPY(J2,-1.,jac,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
+    ierr = MatNorm(J2,NORM_FROBENIUS,&nrm);CHKERRQ(ierr);
+    ierr = PetscPrintf(PETSC_COMM_WORLD,"Error MATIS %g\n",(double)nrm);CHKERRQ(ierr);
+    ierr = MatViewFromOptions(J2,NULL,"-view_conv_err");CHKERRQ(ierr);
+    ierr = MatDestroy(&J2);CHKERRQ(ierr);
+  }
   if (user->bcType == NEUMANN) {
     MatNullSpace nullspace;
 
@@ -212,3 +229,29 @@ PetscErrorCode ComputeMatrix(KSP ksp,Mat J,Mat jac,void *ctx)
   }
   PetscFunctionReturn(0);
 }
+
+
+/*TEST
+
+   test:
+      args: -pc_type mg -pc_mg_type full -ksp_type fgmres -ksp_monitor_short -da_refine 8 -ksp_rtol 1.e-3
+
+   test:
+      suffix: 2
+      args: -bc_type neumann -pc_type mg -pc_mg_type full -ksp_type fgmres -ksp_monitor_short -da_refine 8 -mg_coarse_pc_factor_shift_type nonzero
+      requires: !single
+
+   test:
+      suffix: telescope
+      nsize: 4
+      args: -ksp_monitor_short -da_grid_x 257 -da_grid_y 257 -pc_type mg -pc_mg_galerkin pmat -pc_mg_levels 4 -ksp_type richardson -mg_levels_ksp_type chebyshev -mg_levels_pc_type jacobi -mg_coarse_pc_type telescope -mg_coarse_pc_telescope_ignore_kspcomputeoperators -mg_coarse_telescope_pc_type mg -mg_coarse_telescope_pc_mg_galerkin pmat -mg_coarse_telescope_pc_mg_levels 3 -mg_coarse_telescope_mg_levels_ksp_type chebyshev -mg_coarse_telescope_mg_levels_pc_type jacobi -mg_coarse_pc_telescope_reduction_factor 4
+
+   test:
+      suffix: 3
+      args: -ksp_view -da_refine 2 -pc_type mg -pc_mg_distinct_smoothup -mg_levels_up_pc_type jacobi
+
+   test:
+      suffix: 4
+      args: -ksp_view -da_refine 2 -pc_type mg -pc_mg_distinct_smoothup -mg_levels_up_ksp_max_it 3 -mg_levels_ksp_max_it 4
+
+TEST*/
