@@ -7,8 +7,6 @@ Load of 1.0 in x direction on all nodes (not a true uniform load).\n\
 
 #include <petscksp.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
   Mat            Amat,Pmat;
@@ -21,7 +19,6 @@ int main(int argc,char **args)
   MPI_Comm       comm;
   PetscBool      use_coords = PETSC_FALSE;
   PetscMPIInt    npe,mype;
-  PC             pc;
   PetscScalar    DD[8][8],DD2[8][8];
 #if defined(PETSC_USE_LOG)
   PetscLogStage stage[2];
@@ -36,15 +33,15 @@ int main(int argc,char **args)
                              {0.0000E-00, -3.333333333333333E-01,  2.0000E-01, -2.666666666666667E-01, 0.0000E-00,  6.666666666666667E-02, -2.0000E-01,  5.333333333333333E-01 } };
 
 
-  PetscInitialize(&argc,&args,(char*)0,help);
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
   comm = PETSC_COMM_WORLD;
   ierr  = MPI_Comm_rank(comm, &mype);CHKERRQ(ierr);
   ierr  = MPI_Comm_size(comm, &npe);CHKERRQ(ierr);
-  ierr  = PetscOptionsGetInt(NULL,"-ne",&ne,NULL);CHKERRQ(ierr);
+  ierr  = PetscOptionsGetInt(NULL,NULL,"-ne",&ne,NULL);CHKERRQ(ierr);
   h     = 1./ne;
   /* ne*ne; number of global elements */
-  ierr = PetscOptionsGetReal(NULL,"-alpha",&soft_alpha,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetBool(NULL,"-use_coordinates",&use_coords,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-alpha",&soft_alpha,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-use_coordinates",&use_coords,NULL);CHKERRQ(ierr);
   M    = 2*(ne+1)*(ne+1); /* global number of equations */
   m    = (ne+1)*(ne+1)/npe;
   if (mype==npe-1) m = (ne+1)*(ne+1) - (npe-1)*m;
@@ -54,6 +51,7 @@ int main(int argc,char **args)
   ierr = MatSetSizes(Amat,m,m,M,M);CHKERRQ(ierr);
   ierr = MatSetBlockSize(Amat,2);CHKERRQ(ierr);
   ierr = MatSetType(Amat,MATAIJ);CHKERRQ(ierr);
+  ierr = MatSetFromOptions(Amat);CHKERRQ(ierr);
   ierr = MatSeqAIJSetPreallocation(Amat,18,NULL);CHKERRQ(ierr);
   ierr = MatMPIAIJSetPreallocation(Amat,18,NULL,18,NULL);CHKERRQ(ierr);
 
@@ -61,6 +59,7 @@ int main(int argc,char **args)
   ierr = MatSetSizes(Pmat,m,m,M,M);CHKERRQ(ierr);
   ierr = MatSetBlockSize(Pmat,2);CHKERRQ(ierr);
   ierr = MatSetType(Pmat,MATAIJ);CHKERRQ(ierr);
+  ierr = MatSetFromOptions(Pmat);CHKERRQ(ierr);
   ierr = MatSeqAIJSetPreallocation(Pmat,18,NULL);CHKERRQ(ierr);
   ierr = MatMPIAIJSetPreallocation(Pmat,18,NULL,12,NULL);CHKERRQ(ierr);
 
@@ -138,7 +137,7 @@ int main(int argc,char **args)
       DD[7][5] =  6.66666666666666519E-002;
       DD[7][6] = -0.20000000000000001;
       DD[7][7] =  0.53333333333333321;
-    
+
     /* BC version of element */
     for (i=0; i<8; i++) {
       for (j=0; j<8; j++) {
@@ -151,7 +150,7 @@ int main(int argc,char **args)
   }
   {
     PetscReal *coords;
-    ierr = PetscMalloc1(m,&coords);
+    ierr = PetscMalloc1(m,&coords);CHKERRQ(ierr);
     /* forms the element stiffness and coordinates */
     for (Ii = Istart/2, ix = 0; Ii < Iend/2; Ii++, ix++) {
       j = Ii/(ne+1); i = Ii%(ne+1);
@@ -159,12 +158,13 @@ int main(int argc,char **args)
       x            = h*(Ii % (ne+1)); y = h*(Ii/(ne+1));
       coords[2*ix] = x; coords[2*ix+1] = y;
       if (i<ne && j<ne) {
-        PetscInt jj,ii,idx[4] = {Ii, Ii+1, Ii + (ne+1) + 1, Ii + (ne+1)};
+        PetscInt jj,ii,idx[4];
         /* radius */
         PetscReal radius = PetscSqrtReal((x-.5+h/2)*(x-.5+h/2) + (y-.5+h/2)*(y-.5+h/2));
         PetscReal alpha  = 1.0;
         if (radius < 0.25) alpha = soft_alpha;
 
+        idx[0] = Ii; idx[1] = Ii+1; idx[2] = Ii + (ne+1) + 1;  idx[3] = Ii + (ne+1);
         for (ii=0; ii<8; ii++) {
           for (jj=0;jj<8;jj++) DD[ii][jj] = alpha*DD1[ii][jj];
         }
@@ -194,14 +194,13 @@ int main(int argc,char **args)
 
     /* Setup solver */
     ierr = KSPCreate(PETSC_COMM_WORLD,&ksp);CHKERRQ(ierr);
-    ierr = KSPSetType(ksp, KSPCG);CHKERRQ(ierr);
-    ierr = KSPGetPC(ksp, &pc);CHKERRQ(ierr);
-    ierr = PCSetType(pc, PCGAMG);CHKERRQ(ierr);
     ierr = KSPSetFromOptions(ksp);CHKERRQ(ierr);
 
     /* finish KSP/PC setup */
     ierr = KSPSetOperators(ksp, Amat, Amat);CHKERRQ(ierr);
     if (use_coords) {
+      PC             pc;
+      ierr = KSPGetPC(ksp, &pc);CHKERRQ(ierr);
       ierr = PCSetCoordinates(pc, 2, m/2, coords);CHKERRQ(ierr);
     }
     ierr = PetscFree(coords);CHKERRQ(ierr);
@@ -210,8 +209,9 @@ int main(int argc,char **args)
   if (!PETSC_TRUE) {
     PetscViewer viewer;
     ierr = PetscViewerASCIIOpen(comm, "Amat.m", &viewer);CHKERRQ(ierr);
-    ierr = PetscViewerSetFormat(viewer, PETSC_VIEWER_ASCII_MATLAB);CHKERRQ(ierr);
+    ierr = PetscViewerPushFormat(viewer, PETSC_VIEWER_ASCII_MATLAB);CHKERRQ(ierr);
     ierr = MatView(Amat,viewer);CHKERRQ(ierr);
+    ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&viewer);
   }
 
@@ -251,24 +251,12 @@ int main(int argc,char **args)
     ierr = VecAXPY(bb, -1.0, res);CHKERRQ(ierr);
     ierr = VecDestroy(&res);CHKERRQ(ierr);
     ierr = VecNorm(bb, NORM_2, &norm);CHKERRQ(ierr);
-    PetscPrintf(PETSC_COMM_WORLD,"[%d]%s |b-Ax|/|b|=%e, |b|=%e\n",0,__FUNCT__,norm/norm2,norm2);
+    ierr = PetscPrintf(PETSC_COMM_WORLD,"[%d]%s |b-Ax|/|b|=%e, |b|=%e\n",0,PETSC_FUNCTION_NAME,norm/norm2,norm2);CHKERRQ(ierr);
     ierr = PetscViewerASCIIOpen(comm, "residual.m", &viewer);CHKERRQ(ierr);
-    ierr = PetscViewerSetFormat(viewer, PETSC_VIEWER_ASCII_MATLAB);CHKERRQ(ierr);
+    ierr = PetscViewerPushFormat(viewer, PETSC_VIEWER_ASCII_MATLAB);CHKERRQ(ierr);
     ierr = VecView(bb,viewer);CHKERRQ(ierr);
+    ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&viewer);
-
-
-    /* ierr = PetscViewerASCIIOpen(comm, "rhs.m", &viewer);CHKERRQ(ierr); */
-    /* ierr = PetscViewerSetFormat(viewer, PETSC_VIEWER_ASCII_MATLAB); */
-    /* CHKERRQ(ierr); */
-    /* ierr = VecView(bb,viewer);CHKERRQ(ierr); */
-    /* ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr); */
-
-    /* ierr = PetscViewerASCIIOpen(comm, "solution.m", &viewer);CHKERRQ(ierr); */
-    /* ierr = PetscViewerSetFormat(viewer, PETSC_VIEWER_ASCII_MATLAB); */
-    /* CHKERRQ(ierr); */
-    /* ierr = VecView(xx, viewer);CHKERRQ(ierr); */
-    /* ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr); */
   }
 
   /* Free work space */
@@ -279,6 +267,40 @@ int main(int argc,char **args)
   ierr = MatDestroy(&Pmat);CHKERRQ(ierr);
 
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
+
+
+/*TEST
+
+   test:
+      nsize: 4
+      args: -ne 29 -alpha 1.e-3 -ksp_type cg -pc_type gamg -pc_gamg_type agg -pc_gamg_agg_nsmooths 1 -use_coordinates -ksp_converged_reason -mg_levels_esteig_ksp_type cg -ksp_rtol 1.e-3 -ksp_monitor_short
+      output_file: output/ex55_sa.out
+
+   test:
+      suffix: Classical
+      nsize: 4
+      args: -ne 29 -alpha 1.e-3 -ksp_type cg -pc_type gamg -pc_gamg_type classical -mg_levels_ksp_max_it 5 -ksp_converged_reason -mg_levels_esteig_ksp_type cg
+      output_file: output/ex55_classical.out
+
+   test:
+      suffix: NC
+      nsize: 4
+      args: -ne 29 -alpha 1.e-3 -ksp_type cg -pc_type gamg -pc_gamg_type agg -pc_gamg_agg_nsmooths 1 -ksp_converged_reason -mg_levels_esteig_ksp_type cg
+
+   test:
+      suffix: geo
+      nsize: 4
+      args: -ne 29 -alpha 1.e-3 -ksp_type cg -pc_type gamg -pc_gamg_type geo -use_coordinates -ksp_monitor_short -mg_levels_esteig_ksp_type cg -ksp_type cg -ksp_norm_type unpreconditioned
+      output_file: output/ex55_0.out
+      requires: triangle
+
+   test:
+      suffix: hypre
+      nsize: 4
+      requires: hypre
+      args: -ne 29 -alpha 1.e-3 -ksp_type cg -pc_type hypre -pc_hypre_type boomeramg -ksp_monitor_short
+
+TEST*/
