@@ -12,26 +12,22 @@ static PetscErrorCode MaybeLogStagePush(PetscLogStage stage) { return log_stages
 static PetscErrorCode MaybeLogStagePop() { return log_stages ? PetscLogStagePop() : 0; }
 PetscErrorCode elem_3d_elast_v_25(PetscScalar *);
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
   Mat            Amat;
   PetscErrorCode ierr;
   PetscInt       m,nn,M,Istart,Iend,i,j,k,ii,jj,kk,ic,ne=4,id;
   PetscReal      x,y,z,h,*coords,soft_alpha=1.e-3;
-  PetscBool      two_solves=PETSC_FALSE,test_nonzero_cols=PETSC_FALSE,use_nearnullspace=PETSC_FALSE;
+  PetscBool      two_solves=PETSC_FALSE,test_nonzero_cols=PETSC_FALSE,use_nearnullspace=PETSC_FALSE,test_late_bs=PETSC_FALSE;
   Vec            xx,bb;
   KSP            ksp;
   MPI_Comm       comm;
   PetscMPIInt    npe,mype;
-  PC             pc;
   PetscScalar    DD[24][24],DD2[24][24];
   PetscLogStage  stage[6];
   PetscScalar    DD1[24][24];
-  PCType         type;
 
-  PetscInitialize(&argc,&args,(char*)0,help);
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
   comm = PETSC_COMM_WORLD;
   ierr  = MPI_Comm_rank(comm, &mype);CHKERRQ(ierr);
   ierr  = MPI_Comm_size(comm, &npe);CHKERRQ(ierr);
@@ -46,6 +42,7 @@ int main(int argc,char **args)
     ierr = PetscOptionsBool("-two_solves","solve additional variant of the problem","",two_solves,&two_solves,NULL);CHKERRQ(ierr);
     ierr = PetscOptionsBool("-test_nonzero_cols","nonzero test","",test_nonzero_cols,&test_nonzero_cols,NULL);CHKERRQ(ierr);
     ierr = PetscOptionsBool("-use_mat_nearnullspace","MatNearNullSpace API test","",use_nearnullspace,&use_nearnullspace,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsBool("-test_late_bs","","",test_late_bs,&test_late_bs,NULL);CHKERRQ(ierr);
   }
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
 
@@ -72,17 +69,12 @@ int main(int argc,char **args)
     if (mype==npe-1) m = nn*nn*nn - (npe-1)*m;
   }
   m *= 3; /* number of equations local*/
-  /* Setup solver, get PC type and pc */
+  /* Setup solver */
   ierr = KSPCreate(PETSC_COMM_WORLD,&ksp);CHKERRQ(ierr);
-  ierr = KSPSetType(ksp, KSPCG);CHKERRQ(ierr);
   ierr = KSPSetComputeSingularValues(ksp, PETSC_TRUE);CHKERRQ(ierr);
-  ierr = KSPGetPC(ksp, &pc);CHKERRQ(ierr);
-  ierr = PCSetType(pc, PCGAMG);CHKERRQ(ierr); /* default */
   ierr = KSPSetFromOptions(ksp);CHKERRQ(ierr);
-  ierr = PCGetType(pc, &type);CHKERRQ(ierr);
-
   {
-    /* configureation */
+    /* configuration */
     const PetscInt NP = (PetscInt)(PetscPowReal((PetscReal)npe,1./3.) + .5);
     const PetscInt ipx = mype%NP, ipy = (mype%(NP*NP))/NP, ipz = mype/(NP*NP);
     const PetscInt Ni0 = ipx*(nn/NP), Nj0 = ipy*(nn/NP), Nk0 = ipz*(nn/NP);
@@ -115,24 +107,21 @@ int main(int argc,char **args)
     /* create stiffness matrix */
     ierr = MatCreate(comm,&Amat);CHKERRQ(ierr);
     ierr = MatSetSizes(Amat,m,m,M,M);CHKERRQ(ierr);
-    ierr = MatSetBlockSize(Amat,3);CHKERRQ(ierr);
+    if (!test_late_bs) {
+      ierr = MatSetBlockSize(Amat,3);CHKERRQ(ierr);
+    }
     ierr = MatSetType(Amat,MATAIJ);CHKERRQ(ierr);
+    ierr = MatSetFromOptions(Amat);CHKERRQ(ierr);
     ierr = MatSeqAIJSetPreallocation(Amat,0,d_nnz);CHKERRQ(ierr);
     ierr = MatMPIAIJSetPreallocation(Amat,0,d_nnz,0,o_nnz);CHKERRQ(ierr);
 
     ierr = PetscFree(d_nnz);CHKERRQ(ierr);
     ierr = PetscFree(o_nnz);CHKERRQ(ierr);
+    ierr = MatCreateVecs(Amat,&bb,&xx);CHKERRQ(ierr);
 
     ierr = MatGetOwnershipRange(Amat,&Istart,&Iend);CHKERRQ(ierr);
 
     if (m != Iend - Istart) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_PLIB,"m %D does not equal Iend %D - Istart %D",m,Iend,Istart);
-    /* Generate vectors */
-    ierr = VecCreate(comm,&xx);CHKERRQ(ierr);
-    ierr = VecSetSizes(xx,m,M);CHKERRQ(ierr);
-    ierr = VecSetBlockSize(xx,3);CHKERRQ(ierr);
-    ierr = VecSetFromOptions(xx);CHKERRQ(ierr);
-    ierr = VecDuplicate(xx,&bb);CHKERRQ(ierr);
-    ierr = VecSet(bb,.0);CHKERRQ(ierr);
     /* generate element matrices */
     {
       PetscBool hasData = PETSC_TRUE;
@@ -148,7 +137,7 @@ int main(int argc,char **args)
         /* Get array data */
         ierr = elem_3d_elast_v_25((PetscScalar*)DD1);CHKERRQ(ierr);
       }
-    
+
       /* BC version of element */
       for (i=0; i<24; i++) {
         for (j=0; j<24; j++) {
@@ -178,21 +167,25 @@ int main(int argc,char **args)
     for (i=Ni0,ic=0,ii=0; i<Ni1; i++,ii++) {
       for (j=Nj0,jj=0; j<Nj1; j++,jj++) {
         for (k=Nk0,kk=0; k<Nk1; k++,kk++,ic++) {
-
           /* coords */
           x = coords[3*ic] = h*(PetscReal)i;
           y = coords[3*ic+1] = h*(PetscReal)j;
           z = coords[3*ic+2] = h*(PetscReal)k;
           /* matrix */
           id = id0 + ii + NN*jj + NN*NN*kk;
-
           if (i<ne && j<ne && k<ne) {
             /* radius */
             PetscReal radius = PetscSqrtReal((x-.5+h/2)*(x-.5+h/2)+(y-.5+h/2)*(y-.5+h/2)+(z-.5+h/2)*(z-.5+h/2));
             PetscReal alpha = 1.0;
-            PetscInt  jx,ix,idx[8] = { id, id+1, id+NN+1, id+NN,
-                                       id        + NN*NN, id+1    + NN*NN,
-                                       id+NN+1 + NN*NN, id+NN + NN*NN };
+            PetscInt  jx,ix,idx[8],idx3[24];
+            idx[0] = id;
+            idx[1] = id+1;
+            idx[2] = id+NN+1;
+            idx[3] = id+NN;
+            idx[4] = id + NN*NN;
+            idx[5] = id+1 + NN*NN;
+            idx[6] = id+NN+1 + NN*NN;
+            idx[7] = id+NN + NN*NN;
 
             /* correct indices */
             if (i==Ni1-1 && Ni1!=nn) {
@@ -220,15 +213,27 @@ int main(int argc,char **args)
               for (jx=0;jx<24;jx++) DD[ix][jx] = alpha*DD1[ix][jx];
             }
             if (k>0) {
-              ierr = MatSetValuesBlocked(Amat,8,idx,8,idx,(const PetscScalar*)DD,ADD_VALUES);CHKERRQ(ierr);
-              ierr = VecSetValuesBlocked(bb,8,idx,(const PetscScalar*)vv,ADD_VALUES);CHKERRQ(ierr);
+              if (!test_late_bs) {
+                ierr = MatSetValuesBlocked(Amat,8,idx,8,idx,(const PetscScalar*)DD,ADD_VALUES);CHKERRQ(ierr);
+                ierr = VecSetValuesBlocked(bb,8,idx,(const PetscScalar*)vv,ADD_VALUES);CHKERRQ(ierr);
+              } else {
+                for (ix=0; ix<8; ix++) { idx3[3*ix] = 3*idx[ix]; idx3[3*ix+1] = 3*idx[ix]+1; idx3[3*ix+2] = 3*idx[ix]+2; }
+                ierr = MatSetValues(Amat,24,idx3,24,idx3,(const PetscScalar*)DD,ADD_VALUES);CHKERRQ(ierr);
+                ierr = VecSetValues(bb,24,idx3,(const PetscScalar*)vv,ADD_VALUES);CHKERRQ(ierr);
+              }
             } else {
               /* a BC */
               for (ix=0;ix<24;ix++) {
                 for (jx=0;jx<24;jx++) DD[ix][jx] = alpha*DD2[ix][jx];
               }
-              ierr = MatSetValuesBlocked(Amat,8,idx,8,idx,(const PetscScalar*)DD,ADD_VALUES);CHKERRQ(ierr);
-              ierr = VecSetValuesBlocked(bb,8,idx,(const PetscScalar*)v2,ADD_VALUES);CHKERRQ(ierr);
+              if (!test_late_bs) {
+                ierr = MatSetValuesBlocked(Amat,8,idx,8,idx,(const PetscScalar*)DD,ADD_VALUES);CHKERRQ(ierr);
+                ierr = VecSetValuesBlocked(bb,8,idx,(const PetscScalar*)v2,ADD_VALUES);CHKERRQ(ierr);
+              } else {
+                for (ix=0; ix<8; ix++) { idx3[3*ix] = 3*idx[ix]; idx3[3*ix+1] = 3*idx[ix]+1; idx3[3*ix+2] = 3*idx[ix]+2; }
+                ierr = MatSetValues(Amat,24,idx3,24,idx3,(const PetscScalar*)DD,ADD_VALUES);CHKERRQ(ierr);
+                ierr = VecSetValues(bb,24,idx3,(const PetscScalar*)v2,ADD_VALUES);CHKERRQ(ierr);
+              }
             }
           }
         }
@@ -240,12 +245,22 @@ int main(int argc,char **args)
     ierr = VecAssemblyBegin(bb);CHKERRQ(ierr);
     ierr = VecAssemblyEnd(bb);CHKERRQ(ierr);
   }
+  ierr = MatAssemblyBegin(Amat,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+  ierr = MatAssemblyEnd(Amat,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+  ierr = VecAssemblyBegin(bb);CHKERRQ(ierr);
+  ierr = VecAssemblyEnd(bb);CHKERRQ(ierr);
+  if (test_late_bs) {
+    ierr = VecSetBlockSize(xx,3);CHKERRQ(ierr);
+    ierr = VecSetBlockSize(bb,3);CHKERRQ(ierr);
+    ierr = MatSetBlockSize(Amat,3);CHKERRQ(ierr);
+  }
 
   if (!PETSC_TRUE) {
     PetscViewer viewer;
     ierr = PetscViewerASCIIOpen(comm, "Amat.m", &viewer);CHKERRQ(ierr);
-    ierr = PetscViewerSetFormat(viewer, PETSC_VIEWER_ASCII_MATLAB);CHKERRQ(ierr);
+    ierr = PetscViewerPushFormat(viewer, PETSC_VIEWER_ASCII_MATLAB);CHKERRQ(ierr);
     ierr = MatView(Amat,viewer);CHKERRQ(ierr);
+    ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&viewer);
   }
 
@@ -268,6 +283,8 @@ int main(int argc,char **args)
     ierr = MatNullSpaceDestroy(&matnull);CHKERRQ(ierr);
     ierr = VecDestroy(&vec_coords);CHKERRQ(ierr);
   } else {
+    PC             pc;
+    ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
     ierr = PCSetCoordinates(pc, 3, m/3, coords);CHKERRQ(ierr);
   }
 
@@ -313,14 +330,10 @@ int main(int argc,char **args)
     ierr = MaybeLogStagePop();CHKERRQ(ierr);
     ierr = MaybeLogStagePush(stage[4]);CHKERRQ(ierr);
 
-    /* 3rd solve */
-    ierr = MatScale(Amat, 100000.0);CHKERRQ(ierr);
-    ierr = KSPSetOperators(ksp, Amat, Amat);CHKERRQ(ierr);
-    ierr = KSPSetUp(ksp);CHKERRQ(ierr);
-
     ierr = MaybeLogStagePop();CHKERRQ(ierr);
     ierr = MaybeLogStagePush(stage[5]);CHKERRQ(ierr);
 
+    /* 3rd solve */
     ierr = KSPSolve(ksp, bb, xx);CHKERRQ(ierr);
 
     ierr = MaybeLogStagePop();CHKERRQ(ierr);
@@ -333,24 +346,7 @@ int main(int argc,char **args)
     ierr = VecAXPY(bb, -1.0, res);CHKERRQ(ierr);
     ierr = VecDestroy(&res);CHKERRQ(ierr);
     ierr = VecNorm(bb, NORM_2, &norm);CHKERRQ(ierr);
-    PetscPrintf(PETSC_COMM_WORLD,"[%d]%s |b-Ax|/|b|=%e, |b|=%e, emax=%e\n",0,__FUNCT__,(double)(norm/norm2),(double)norm2,(double)emax);
-    /*ierr = PetscViewerASCIIOpen(comm, "residual.m", &viewer);CHKERRQ(ierr);
-     ierr = PetscViewerSetFormat(viewer, PETSC_VIEWER_ASCII_MATLAB);CHKERRQ(ierr);
-     ierr = VecView(bb,viewer);CHKERRQ(ierr);
-     ierr = PetscViewerDestroy(&viewer);*/
-
-
-    /* ierr = PetscViewerASCIIOpen(comm, "rhs.m", &viewer);CHKERRQ(ierr); */
-    /* ierr = PetscViewerSetFormat(viewer, PETSC_VIEWER_ASCII_MATLAB); */
-    /* CHKERRQ(ierr); */
-    /* ierr = VecView(bb,viewer);CHKERRQ(ierr); */
-    /* ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr); */
-
-    /* ierr = PetscViewerASCIIOpen(comm, "solution.m", &viewer);CHKERRQ(ierr); */
-    /* ierr = PetscViewerSetFormat(viewer, PETSC_VIEWER_ASCII_MATLAB); */
-    /* CHKERRQ(ierr); */
-    /* ierr = VecView(xx, viewer);CHKERRQ(ierr); */
-    /* ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr); */
+    ierr = PetscPrintf(PETSC_COMM_WORLD,"[%d]%s |b-Ax|/|b|=%e, |b|=%e, emax=%e\n",0,PETSC_FUNCTION_NAME,(double)(norm/norm2),(double)norm2,(double)emax);CHKERRQ(ierr);
   }
 
   /* Free work space */
@@ -361,12 +357,10 @@ int main(int argc,char **args)
   ierr = PetscFree(coords);CHKERRQ(ierr);
 
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
 /* Data was previously provided in the file data/elem_3d_elast_v_25.tx */
-#undef __FUNCT__
-#define __FUNCT__ "elem_3d_elast_v_25"
 PetscErrorCode elem_3d_elast_v_25(PetscScalar *dd)
 {
   PetscErrorCode ierr;
@@ -953,3 +947,53 @@ PetscErrorCode elem_3d_elast_v_25(PetscScalar *dd)
   ierr = PetscMemcpy(dd,DD,sizeof(PetscScalar)*576);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
+
+/*TEST
+
+   test:
+      nsize: 8
+      args: -ne 13 -alpha 1.e-3 -ksp_type cg -pc_type gamg -pc_gamg_agg_nsmooths 1 -pc_gamg_reuse_interpolation true -two_solves -ksp_converged_reason -ksp_view -use_mat_nearnullspace -mg_levels_esteig_ksp_type cg -mg_levels_esteig_ksp_max_it 10 -pc_gamg_square_graph 1 -mg_levels_ksp_max_it 1 -mg_levels_ksp_type chebyshev -mg_levels_ksp_chebyshev_esteig 0,0.2,0,1.05 -gamg_est_ksp_type cg -gamg_est_ksp_max_it 10 -pc_gamg_asm_use_agg true -mg_levels_sub_pc_type lu -mg_levels_pc_asm_overlap 0 -pc_gamg_threshold -0.01 -pc_gamg_coarse_eq_limit 200 -pc_gamg_process_eq_limit 30 -pc_gamg_repartition false -pc_mg_cycle_type v -pc_gamg_use_parallel_coarse_grid_solver -mg_coarse_pc_type jacobi -mg_coarse_ksp_type cg -ksp_monitor_short
+      filter: grep -v variant
+
+   test:
+      suffix: 2
+      nsize: 8
+      args: -ne 31 -alpha 1.e-3 -ksp_type cg -pc_type gamg -pc_gamg_agg_nsmooths 1 -pc_gamg_reuse_interpolation true -two_solves -ksp_converged_reason -use_mat_nearnullspace -mg_levels_esteig_ksp_type cg -mg_levels_esteig_ksp_max_it 10 -pc_gamg_square_graph 1 -mg_levels_ksp_max_it 1 -mg_levels_ksp_type chebyshev -mg_levels_ksp_chebyshev_esteig 0,0.2,0,1.05 -gamg_est_ksp_type cg -gamg_est_ksp_max_it 10 -pc_gamg_asm_use_agg true -mg_levels_sub_pc_type lu -mg_levels_pc_asm_overlap 0 -pc_gamg_threshold -0.01 -pc_gamg_coarse_eq_limit 200 -pc_gamg_process_eq_limit 30 -pc_gamg_repartition false -pc_mg_cycle_type v -pc_gamg_use_parallel_coarse_grid_solver -mg_coarse_pc_type jacobi -mg_coarse_ksp_type cg
+      filter: grep -v variant
+      TODO: this has a memory leak but the previous test does not
+
+   test:
+      suffix: latebs
+      filter: grep -v variant
+      nsize: 8
+      args: -test_late_bs 0 -ne 9 -alpha 1.e-3 -ksp_type cg -pc_type gamg -pc_gamg_agg_nsmooths 1 -pc_gamg_reuse_interpolation true -two_solves -ksp_converged_reason -ksp_view -use_mat_nearnullspace -mg_levels_esteig_ksp_type cg -mg_levels_esteig_ksp_max_it 10 -pc_gamg_square_graph 1 -mg_levels_ksp_max_it 1 -mg_levels_ksp_type chebyshev -mg_levels_ksp_chebyshev_esteig 0,0.2,0,1.05 -gamg_est_ksp_type cg -gamg_est_ksp_max_it 10 -pc_gamg_asm_use_agg true -mg_levels_sub_pc_type lu -mg_levels_pc_asm_overlap 0 -pc_gamg_threshold -0.01 -pc_gamg_coarse_eq_limit 200 -pc_gamg_process_eq_limit 30 -pc_gamg_repartition false -pc_mg_cycle_type v -pc_gamg_use_parallel_coarse_grid_solver -mg_coarse_pc_type jacobi -mg_coarse_ksp_type cg -ksp_monitor_short -ksp_view
+
+   test:
+      suffix: latebs-2
+      filter: grep -v variant
+      nsize: 8
+      args: -test_late_bs -ne 9 -alpha 1.e-3 -ksp_type cg -pc_type gamg -pc_gamg_agg_nsmooths 1 -pc_gamg_reuse_interpolation true -two_solves -ksp_converged_reason -ksp_view -use_mat_nearnullspace -mg_levels_esteig_ksp_type cg -mg_levels_esteig_ksp_max_it 10 -pc_gamg_square_graph 1 -mg_levels_ksp_max_it 1 -mg_levels_ksp_type chebyshev -mg_levels_ksp_chebyshev_esteig 0,0.2,0,1.05 -gamg_est_ksp_type cg -gamg_est_ksp_max_it 10 -pc_gamg_asm_use_agg true -mg_levels_sub_pc_type lu -mg_levels_pc_asm_overlap 0 -pc_gamg_threshold -0.01 -pc_gamg_coarse_eq_limit 200 -pc_gamg_process_eq_limit 30 -pc_gamg_repartition false -pc_mg_cycle_type v -pc_gamg_use_parallel_coarse_grid_solver -mg_coarse_pc_type jacobi -mg_coarse_ksp_type cg -ksp_monitor_short -ksp_view
+
+   test:
+      suffix: ml
+      nsize: 8
+      args: -ne 9 -alpha 1.e-3 -ksp_type cg -pc_type ml -mg_levels_ksp_type chebyshev -mg_levels_ksp_chebyshev_esteig 0,0.05,0,1.05 -mg_levels_pc_type sor -ksp_monitor_short -mg_levels_esteig_ksp_type cg
+      requires: ml
+
+   test:
+      suffix: nns
+      args: -ne 9 -alpha 1.e-3 -ksp_converged_reason -ksp_type cg -ksp_max_it 50 -pc_type gamg -pc_gamg_type agg -pc_gamg_agg_nsmooths 1 -pc_gamg_coarse_eq_limit 1000 -mg_levels_ksp_type chebyshev -mg_levels_pc_type sor -pc_gamg_reuse_interpolation true -two_solves -use_mat_nearnullspace -mg_levels_esteig_ksp_type cg
+
+   test:
+      suffix: nns_telescope
+      nsize: 2
+      args: -use_mat_nearnullspace -ksp_monitor_short -pc_type telescope -pc_telescope_reduction_factor 2 -telescope_pc_type gamg
+
+   test:
+      suffix: seqaijmkl
+      nsize: 8
+      requires: mkl_sparse
+      args: -ne 9 -alpha 1.e-3 -ksp_type cg -pc_type gamg -pc_gamg_agg_nsmooths 1 -pc_gamg_reuse_interpolation true -two_solves -ksp_converged_reason -use_mat_nearnullspace -mg_levels_esteig_ksp_type cg -mg_levels_esteig_ksp_max_it 10 -pc_gamg_square_graph 1 -mg_levels_ksp_max_it 1 -mg_levels_ksp_type chebyshev -mg_levels_pc_type jacobi -mg_levels_ksp_chebyshev_esteig 0,0.2,0,1.05 -gamg_est_ksp_type cg -gamg_est_ksp_max_it 10 -pc_gamg_threshold 0.01 -pc_gamg_coarse_eq_limit 2000 -pc_gamg_process_eq_limit 200 -pc_gamg_repartition false -pc_mg_cycle_type v -ksp_monitor_short -mat_seqaij_type seqaijmkl
+
+TEST*/
