@@ -9,9 +9,7 @@
 #include <../src/ksp/ksp/impls/bcgs/bcgsimpl.h>       /*I  "petscksp.h"  I*/
 #include <petsc/private/vecimpl.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_FBCGSR"
-PetscErrorCode KSPSetUp_FBCGSR(KSP ksp)
+static PetscErrorCode KSPSetUp_FBCGSR(KSP ksp)
 {
   PetscErrorCode ierr;
 
@@ -20,10 +18,7 @@ PetscErrorCode KSPSetUp_FBCGSR(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#include <petsc/private/pcimpl.h>            /*I "petscksp.h" I*/
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_FBCGSR"
-PetscErrorCode  KSPSolve_FBCGSR(KSP ksp)
+static PetscErrorCode  KSPSolve_FBCGSR(KSP ksp)
 {
   PetscErrorCode    ierr;
   PetscInt          i,j,N;
@@ -36,7 +31,8 @@ PetscErrorCode  KSPSolve_FBCGSR(KSP ksp)
   PetscScalar       insums[4],outsums[4];
   KSP_BCGS          *bcgs = (KSP_BCGS*)ksp->data;
   PC                pc;
-
+  Mat               mat;
+  
   PetscFunctionBegin;
   if (!ksp->vec_rhs->petscnative) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP,"Only coded for PETSc vectors");
   ierr = VecGetLocalSize(ksp->vec_sol,&N);CHKERRQ(ierr);
@@ -68,8 +64,9 @@ PetscErrorCode  KSPSolve_FBCGSR(KSP ksp)
   /* Compute initial residual */
   ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
   ierr = PCSetUp(pc);CHKERRQ(ierr);
+  ierr = PCGetOperators(pc,&mat,NULL);CHKERRQ(ierr);
   if (!ksp->guess_zero) {
-    ierr = KSP_MatMult(ksp,pc->mat,X,P2);CHKERRQ(ierr); /* P2 is used as temporary storage */
+    ierr = KSP_MatMult(ksp,mat,X,P2);CHKERRQ(ierr); /* P2 is used as temporary storage */
     ierr = VecCopy(B,R);CHKERRQ(ierr);
     ierr = VecAXPY(R,-1.0,P2);CHKERRQ(ierr);
   } else {
@@ -77,9 +74,7 @@ PetscErrorCode  KSPSolve_FBCGSR(KSP ksp)
   }
 
   /* Test for nothing to do */
-  if (ksp->normtype != KSP_NORM_NONE) {
-    ierr = VecNorm(R,NORM_2,&rho);CHKERRQ(ierr);
-  }
+  ierr = VecNorm(R,NORM_2,&rho);CHKERRQ(ierr);
   ierr       = PetscObjectSAWsTakeAccess((PetscObject)ksp);CHKERRQ(ierr);
   ksp->its   = 0;
   ksp->rnorm = rho;
@@ -98,7 +93,7 @@ PetscErrorCode  KSPSolve_FBCGSR(KSP ksp)
 
     /* matmult and pc */
     ierr = KSP_PCApply(ksp,P,P2);CHKERRQ(ierr); /* p2 <- K p */
-    ierr = KSP_MatMult(ksp,pc->mat,P2,V);CHKERRQ(ierr); /* v <- A p2 */
+    ierr = KSP_MatMult(ksp,mat,P2,V);CHKERRQ(ierr); /* v <- A p2 */
 
     /* inner prodcuts */
     if (i==0) {
@@ -111,13 +106,13 @@ PetscErrorCode  KSPSolve_FBCGSR(KSP ksp)
         tau   += r[j]*rp[j]; /* tau <- (r,rp) */
         sigma += v[j]*rp[j]; /* sigma <- (v,rp) */
       }
-      PetscLogFlops(4.0*N);
+      ierr = PetscLogFlops(4.0*N);CHKERRQ(ierr);
       ierr      = PetscLogEventEnd(VEC_ReduceArithmetic,0,0,0,0);CHKERRQ(ierr);
       insums[0] = tau;
       insums[1] = sigma;
-      ierr      = PetscLogEventBarrierBegin(VEC_ReduceBarrier,0,0,0,0,PetscObjectComm((PetscObject)ksp));CHKERRQ(ierr);
-      ierr      = MPI_Allreduce(insums,outsums,2,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)ksp));CHKERRQ(ierr);
-      ierr      = PetscLogEventBarrierEnd(VEC_ReduceBarrier,0,0,0,0,PetscObjectComm((PetscObject)ksp));CHKERRQ(ierr);
+      ierr      = PetscLogEventBegin(VEC_ReduceCommunication,0,0,0,0);CHKERRQ(ierr);
+      ierr      = MPIU_Allreduce(insums,outsums,2,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)ksp));CHKERRQ(ierr);
+      ierr      = PetscLogEventEnd(VEC_ReduceCommunication,0,0,0,0);CHKERRQ(ierr);
       tau       = outsums[0];
       sigma     = outsums[1];
     }
@@ -130,7 +125,7 @@ PetscErrorCode  KSPSolve_FBCGSR(KSP ksp)
 
     /* matmult and pc */
     ierr = KSP_PCApply(ksp,S,S2);CHKERRQ(ierr); /* s2 <- K s */
-    ierr = KSP_MatMult(ksp,pc->mat,S2,T);CHKERRQ(ierr); /* t <- A s2 */
+    ierr = KSP_MatMult(ksp,mat,S2,T);CHKERRQ(ierr); /* t <- A s2 */
 
     /* inner prodcuts */
     ierr = PetscLogEventBegin(VEC_ReduceArithmetic,0,0,0,0);CHKERRQ(ierr);
@@ -141,7 +136,7 @@ PetscErrorCode  KSPSolve_FBCGSR(KSP ksp)
       xi3 += t[j]*t[j]; /* xi3 <- (t,t) */
       xi4 += t[j]*rp[j]; /* xi4 <- (t,rp) */
     }
-    PetscLogFlops(8.0*N);
+    ierr = PetscLogFlops(8.0*N);CHKERRQ(ierr);
     ierr = PetscLogEventEnd(VEC_ReduceArithmetic,0,0,0,0);CHKERRQ(ierr);
 
     insums[0] = xi1;
@@ -149,9 +144,9 @@ PetscErrorCode  KSPSolve_FBCGSR(KSP ksp)
     insums[2] = xi3;
     insums[3] = xi4;
 
-    ierr = PetscLogEventBarrierBegin(VEC_ReduceBarrier,0,0,0,0,PetscObjectComm((PetscObject)ksp));CHKERRQ(ierr);
-    ierr = MPI_Allreduce(insums,outsums,4,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)ksp));CHKERRQ(ierr);
-    ierr = PetscLogEventBarrierEnd(VEC_ReduceBarrier,0,0,0,0,PetscObjectComm((PetscObject)ksp));CHKERRQ(ierr);
+    ierr = PetscLogEventBegin(VEC_ReduceCommunication,0,0,0,0);CHKERRQ(ierr);
+    ierr = MPIU_Allreduce(insums,outsums,4,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)ksp));CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(VEC_ReduceCommunication,0,0,0,0);CHKERRQ(ierr);
     xi1  = outsums[0];
     xi2  = outsums[1];
     xi3  = outsums[2];
@@ -185,7 +180,7 @@ PetscErrorCode  KSPSolve_FBCGSR(KSP ksp)
       r[j] = s[j] - omega * t[j]; /* r <- s - omega t */
       p[j] = r[j] + beta * (p[j] - omega * v[j]); /* p <- r + beta * (p - omega v) */
     }
-    PetscLogFlops(6.0*N);
+    ierr = PetscLogFlops(6.0*N);CHKERRQ(ierr);
     ierr = PetscLogEventEnd(VEC_Ops,0,0,0,0);CHKERRQ(ierr);
 
   }
@@ -202,12 +197,11 @@ PetscErrorCode  KSPSolve_FBCGSR(KSP ksp)
 
    Level: beginner
 
-   Notes: Only allow right preconditioning
+   Notes:
+    Only allow right preconditioning
 
 .seealso:  KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP, KSPBICG, KSPFBCGSL, KSPSetPCSide()
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_FBCGSR"
 PETSC_EXTERN PetscErrorCode KSPCreate_FBCGSR(KSP ksp)
 {
   PetscErrorCode ierr;
