@@ -1,9 +1,7 @@
 
 #include <petsc/private/kspimpl.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_BiCG"
-PetscErrorCode KSPSetUp_BiCG(KSP ksp)
+static PetscErrorCode KSPSetUp_BiCG(KSP ksp)
 {
   PetscErrorCode ierr;
 
@@ -15,9 +13,7 @@ PetscErrorCode KSPSetUp_BiCG(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_BiCG"
-PetscErrorCode  KSPSolve_BiCG(KSP ksp)
+static PetscErrorCode  KSPSolve_BiCG(KSP ksp)
 {
   PetscErrorCode ierr;
   PetscInt       i;
@@ -137,7 +133,8 @@ PetscErrorCode  KSPSolve_BiCG(KSP ksp)
 
    Level: beginner
 
-   Notes: this method requires that one be apply to apply the transpose of the preconditioner and operator
+   Notes:
+    this method requires that one be apply to apply the transpose of the preconditioner and operator
          as well as the operator and preconditioner.
          Supports only left preconditioning
 
@@ -147,8 +144,6 @@ PetscErrorCode  KSPSolve_BiCG(KSP ksp)
 .seealso:  KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP, KSPBCGS, KSPCGNE
 
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_BiCG"
 PETSC_EXTERN PetscErrorCode KSPCreate_BiCG(KSP ksp)
 {
   PetscErrorCode ierr;
