@@ -12,15 +12,15 @@
                                    memory it needed
     Here the "_XXX" denotes a particular implementation, in this case
     we use _CG (e.g. KSPCreate_CG, KSPDestroy_CG). These routines are
-    are actually called vai the common user interface routines
+    are actually called via the common user interface routines
     KSPSetType(), KSPSetFromOptions(), KSPSolve(), and KSPDestroy() so the
     application code interface remains identical for all preconditioners.
 
     Other basic routines for the KSP objects include
         KSPSetUp_XXX()
-        KSPView_XXX()             - Prints details of solver being used.
+        KSPView_XXX()            - Prints details of solver being used.
 
-    Detailed notes:
+    Detailed Notes:
     By default, this code implements the CG (Conjugate Gradient) method,
     which is valid for real symmetric (and complex Hermitian) positive
     definite matrices. Note that for the complex Hermitian case, the
@@ -37,7 +37,7 @@
     all such matrices ... and thus we don't recommend using this method.
 */
 /*
-       cgimpl.h defines the simple data structured used to store information
+    cgimpl.h defines the simple data structured used to store information
     related to the type of matrix (e.g. complex symmetric) being solved and
     data used during the optional Lanczo process used to compute eigenvalues
 */
@@ -51,9 +51,7 @@ extern PetscErrorCode KSPComputeEigenvalues_CG(KSP,PetscInt,PetscReal*,PetscReal
       This is called once, usually automatically by KSPSolve() or KSPSetUp()
      but can be called directly by KSPSetUp()
 */
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_CG"
-PetscErrorCode KSPSetUp_CG(KSP ksp)
+static PetscErrorCode KSPSetUp_CG(KSP ksp)
 {
   KSP_CG         *cgP = (KSP_CG*)ksp->data;
   PetscErrorCode ierr;
@@ -80,23 +78,27 @@ PetscErrorCode KSPSetUp_CG(KSP ksp)
 }
 
 /*
-       KSPSolve_CG - This routine actually applies the conjugate gradient  method
+     A macro used in the following KSPSolve_CG and KSPSolve_CG_SingleReduction routines
+*/
+#define VecXDot(x,y,a) (((cg->type) == (KSP_CG_HERMITIAN)) ? VecDot(x,y,a) : VecTDot(x,y,a))
 
-   This routine is MUCH too messy. I has too many options (norm type and single reduction) embedded making the code confusing and likely to be buggy.
+/*
+     KSPSolve_CG - This routine actually applies the conjugate gradient method
+
+     Note : this routine can be replaced with another one (see below) which implements
+            another variant of CG.
 
    Input Parameter:
 .     ksp - the Krylov space object that was set to use conjugate gradient, by, for
             example, KSPCreate(MPI_Comm,KSP *ksp); KSPSetType(ksp,KSPCG);
 */
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_CG"
-PetscErrorCode  KSPSolve_CG(KSP ksp)
+static PetscErrorCode KSPSolve_CG(KSP ksp)
 {
   PetscErrorCode ierr;
   PetscInt       i,stored_max_it,eigs;
-  PetscScalar    dpi = 0.0,a = 1.0,beta,betaold = 1.0,b = 0,*e = 0,*d = 0,delta,dpiold;
+  PetscScalar    dpi = 0.0,a = 1.0,beta,betaold = 1.0,b = 0,*e = 0,*d = 0,dpiold;
   PetscReal      dp  = 0.0;
-  Vec            X,B,Z,R,P,S,W;
+  Vec            X,B,Z,R,P,W;
   KSP_CG         *cg;
   Mat            Amat,Pmat;
   PetscBool      diagonalscale;
@@ -113,66 +115,50 @@ PetscErrorCode  KSPSolve_CG(KSP ksp)
   R             = ksp->work[0];
   Z             = ksp->work[1];
   P             = ksp->work[2];
-  if (cg->singlereduction) {
-    S = ksp->work[3];
-    W = ksp->work[4];
-  } else {
-    S = 0;                      /* unused */
-    W = Z;
-  }
-
-#define VecXDot(x,y,a) (((cg->type) == (KSP_CG_HERMITIAN)) ? VecDot(x,y,a) : VecTDot(x,y,a))
+  W             = Z;
 
   if (eigs) {e = cg->e; d = cg->d; e[0] = 0.0; }
   ierr = PCGetOperators(ksp->pc,&Amat,&Pmat);CHKERRQ(ierr);
 
   ksp->its = 0;
   if (!ksp->guess_zero) {
-    ierr = KSP_MatMult(ksp,Amat,X,R);CHKERRQ(ierr);            /*     r <- b - Ax     */
+    ierr = KSP_MatMult(ksp,Amat,X,R);CHKERRQ(ierr);            /*    r <- b - Ax                       */
     ierr = VecAYPX(R,-1.0,B);CHKERRQ(ierr);
   } else {
-    ierr = VecCopy(B,R);CHKERRQ(ierr);                         /*     r <- b (x is 0) */
+    ierr = VecCopy(B,R);CHKERRQ(ierr);                         /*    r <- b (x is 0)                   */
   }
 
   switch (ksp->normtype) {
-  case KSP_NORM_PRECONDITIONED:
-    ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);                   /*     z <- Br         */
-    ierr = VecNorm(Z,NORM_2,&dp);CHKERRQ(ierr);                /*    dp <- z'*z = e'*A'*B'*B*A'*e'     */
-    break;
-  case KSP_NORM_UNPRECONDITIONED:
-    ierr = VecNorm(R,NORM_2,&dp);CHKERRQ(ierr);                /*    dp <- r'*r = e'*A'*A*e            */
-    break;
-  case KSP_NORM_NATURAL:
-    ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);                   /*     z <- Br         */
-    if (cg->singlereduction) {
-      ierr = KSP_MatMult(ksp,Amat,Z,S);CHKERRQ(ierr);
-      ierr = VecXDot(Z,S,&delta);CHKERRQ(ierr);
-    }
-    ierr = VecXDot(Z,R,&beta);CHKERRQ(ierr);                     /*  beta <- z'*r       */
-    KSPCheckDot(ksp,beta);
-    dp = PetscSqrtReal(PetscAbsScalar(beta));                           /*    dp <- r'*z = r'*B*r = e'*A'*B*A*e */
-    break;
-  case KSP_NORM_NONE:
-    dp = 0.0;
-    break;
-  default: SETERRQ1(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP,"%s",KSPNormTypes[ksp->normtype]);
+    case KSP_NORM_PRECONDITIONED:
+      ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*    z <- Br                           */
+      ierr = VecNorm(Z,NORM_2,&dp);CHKERRQ(ierr);              /*    dp <- z'*z = e'*A'*B'*B*A'*e'     */
+      break;
+    case KSP_NORM_UNPRECONDITIONED:
+      ierr = VecNorm(R,NORM_2,&dp);CHKERRQ(ierr);              /*    dp <- r'*r = e'*A'*A*e            */
+      break;
+    case KSP_NORM_NATURAL:
+      ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*    z <- Br                           */
+      ierr = VecXDot(Z,R,&beta);CHKERRQ(ierr);                 /*    beta <- z'*r                      */
+      KSPCheckDot(ksp,beta);
+      dp = PetscSqrtReal(PetscAbsScalar(beta));                /*    dp <- r'*z = r'*B*r = e'*A'*B*A*e */
+      break;
+    case KSP_NORM_NONE:
+      dp = 0.0;
+      break;
+    default: SETERRQ1(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP,"%s",KSPNormTypes[ksp->normtype]);
   }
   ierr       = KSPLogResidualHistory(ksp,dp);CHKERRQ(ierr);
   ierr       = KSPMonitor(ksp,0,dp);CHKERRQ(ierr);
   ksp->rnorm = dp;
 
-  ierr = (*ksp->converged)(ksp,0,dp,&ksp->reason,ksp->cnvP);CHKERRQ(ierr);      /* test for convergence */
+  ierr = (*ksp->converged)(ksp,0,dp,&ksp->reason,ksp->cnvP);CHKERRQ(ierr);     /* test for convergence */
   if (ksp->reason) PetscFunctionReturn(0);
 
   if (ksp->normtype != KSP_NORM_PRECONDITIONED && (ksp->normtype != KSP_NORM_NATURAL)) {
-    ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);                   /*     z <- Br         */
+    ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);                /*     z <- Br                           */
   }
   if (ksp->normtype != KSP_NORM_NATURAL) {
-    if (cg->singlereduction) {
-      ierr = KSP_MatMult(ksp,Amat,Z,S);CHKERRQ(ierr);
-      ierr = VecXDot(Z,S,&delta);CHKERRQ(ierr);
-    }
-    ierr = VecXDot(Z,R,&beta);CHKERRQ(ierr);         /*  beta <- z'*r       */
+    ierr = VecXDot(Z,R,&beta);CHKERRQ(ierr);                  /*     beta <- z'*r                      */
     KSPCheckDot(ksp,beta);
   }
 
@@ -185,13 +171,14 @@ PetscErrorCode  KSPSolve_CG(KSP ksp)
       break;
 #if !defined(PETSC_USE_COMPLEX)
     } else if ((i > 0) && (beta*betaold < 0.0)) {
+      if (ksp->errorifnotconverged) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_NOT_CONVERGED,"Diverged due to indefinite preconditioner");
       ksp->reason = KSP_DIVERGED_INDEFINITE_PC;
       ierr        = PetscInfo(ksp,"diverging due to indefinite preconditioner\n");CHKERRQ(ierr);
       break;
 #endif
     }
     if (!i) {
-      ierr = VecCopy(Z,P);CHKERRQ(ierr);         /*     p <- z          */
+      ierr = VecCopy(Z,P);CHKERRQ(ierr);                       /*     p <- z                           */
       b    = 0.0;
     } else {
       b = beta/betaold;
@@ -199,87 +186,237 @@ PetscErrorCode  KSPSolve_CG(KSP ksp)
         if (ksp->max_it != stored_max_it) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP,"Can not change maxit AND calculate eigenvalues");
         e[i] = PetscSqrtReal(PetscAbsScalar(b))/a;
       }
-      ierr = VecAYPX(P,b,Z);CHKERRQ(ierr);    /*     p <- z + b* p   */
+      ierr = VecAYPX(P,b,Z);CHKERRQ(ierr);                     /*     p <- z + b* p                    */
     }
     dpiold = dpi;
-    if (!cg->singlereduction || !i) {
-      ierr = KSP_MatMult(ksp,Amat,P,W);CHKERRQ(ierr);          /*     w <- Ap         */
-      ierr = VecXDot(P,W,&dpi);CHKERRQ(ierr);                  /*     dpi <- p'w     */
-    } else {
-      ierr = VecAYPX(W,beta/betaold,S);CHKERRQ(ierr);                  /*     w <- Ap         */
-      dpi  = delta - beta*beta*dpiold/(betaold*betaold);             /*     dpi <- p'w     */
-    }
+    ierr = KSP_MatMult(ksp,Amat,P,W);CHKERRQ(ierr);            /*     w <- Ap                          */
+    ierr = VecXDot(P,W,&dpi);CHKERRQ(ierr);                    /*     dpi <- p'w                       */
+    KSPCheckDot(ksp,dpi);
     betaold = beta;
-    KSPCheckDot(ksp,beta);
 
     if ((dpi == 0.0) || ((i > 0) && (PetscRealPart(dpi*dpiold) <= 0.0))) {
+      if (ksp->errorifnotconverged) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_NOT_CONVERGED,"Diverged due to indefinite matrix");
       ksp->reason = KSP_DIVERGED_INDEFINITE_MAT;
       ierr        = PetscInfo(ksp,"diverging due to indefinite or negative definite matrix\n");CHKERRQ(ierr);
       break;
     }
-    a = beta/dpi;                                 /*     a = beta/p'w   */
+    a = beta/dpi;                                              /*     a = beta/p'w                     */
     if (eigs) d[i] = PetscSqrtReal(PetscAbsScalar(b))*e[i] + 1.0/a;
-    ierr = VecAXPY(X,a,P);CHKERRQ(ierr);          /*     x <- x + ap     */
-    ierr = VecAXPY(R,-a,W);CHKERRQ(ierr);                      /*     r <- r - aw    */
+    ierr = VecAXPY(X,a,P);CHKERRQ(ierr);                       /*     x <- x + ap                      */
+    ierr = VecAXPY(R,-a,W);CHKERRQ(ierr);                      /*     r <- r - aw                      */
     if (ksp->normtype == KSP_NORM_PRECONDITIONED && ksp->chknorm < i+2) {
-      ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*     z <- Br         */
-      if (cg->singlereduction) {
-        ierr = KSP_MatMult(ksp,Amat,Z,S);CHKERRQ(ierr);
-      }
-      ierr = VecNorm(Z,NORM_2,&dp);CHKERRQ(ierr);              /*    dp <- z'*z       */
+      ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*     z <- Br                          */
+      ierr = VecNorm(Z,NORM_2,&dp);CHKERRQ(ierr);              /*     dp <- z'*z                       */
     } else if (ksp->normtype == KSP_NORM_UNPRECONDITIONED && ksp->chknorm < i+2) {
-      ierr = VecNorm(R,NORM_2,&dp);CHKERRQ(ierr);              /*    dp <- r'*r       */
+      ierr = VecNorm(R,NORM_2,&dp);CHKERRQ(ierr);              /*     dp <- r'*r                       */
     } else if (ksp->normtype == KSP_NORM_NATURAL) {
-      ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*     z <- Br         */
-      if (cg->singlereduction) {
-        PetscScalar tmp[2];
-        Vec         vecs[2];
-        vecs[0] = S; vecs[1] = R;
-        ierr    = KSP_MatMult(ksp,Amat,Z,S);CHKERRQ(ierr);
-        ierr  = VecMDot(Z,2,vecs,tmp);CHKERRQ(ierr);
-        delta = tmp[0]; beta = tmp[1];
-      } else {
-        ierr = VecXDot(Z,R,&beta);CHKERRQ(ierr);     /*  beta <- r'*z       */
-      }
+      ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*     z <- Br                          */
+      ierr = VecXDot(Z,R,&beta);CHKERRQ(ierr);                 /*     beta <- r'*z                     */
       KSPCheckDot(ksp,beta);
       dp = PetscSqrtReal(PetscAbsScalar(beta));
     } else {
       dp = 0.0;
     }
     ksp->rnorm = dp;
-    CHKERRQ(ierr);KSPLogResidualHistory(ksp,dp);CHKERRQ(ierr);
+    ierr = KSPLogResidualHistory(ksp,dp);CHKERRQ(ierr);
+    if (eigs) cg->ned = ksp->its;
     ierr = KSPMonitor(ksp,i+1,dp);CHKERRQ(ierr);
     ierr = (*ksp->converged)(ksp,i+1,dp,&ksp->reason,ksp->cnvP);CHKERRQ(ierr);
     if (ksp->reason) break;
 
     if ((ksp->normtype != KSP_NORM_PRECONDITIONED && (ksp->normtype != KSP_NORM_NATURAL)) || (ksp->chknorm >= i+2)) {
-      ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);                   /*     z <- Br         */
-      if (cg->singlereduction) {
-        ierr = KSP_MatMult(ksp,Amat,Z,S);CHKERRQ(ierr);
-      }
+      ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*     z <- Br                          */
     }
     if ((ksp->normtype != KSP_NORM_NATURAL) || (ksp->chknorm >= i+2)) {
-      if (cg->singlereduction) {
-        PetscScalar tmp[2];
-        Vec         vecs[2];
-        vecs[0] = S; vecs[1] = R;
-        ierr  = VecMDot(Z,2,vecs,tmp);CHKERRQ(ierr);
-        delta = tmp[0]; beta = tmp[1];
-      } else {
-        ierr = VecXDot(Z,R,&beta);CHKERRQ(ierr);        /*  beta <- z'*r       */
-      }
+      ierr = VecXDot(Z,R,&beta);CHKERRQ(ierr);                 /*     beta <- z'*r                     */
       KSPCheckDot(ksp,beta);
     }
 
     i++;
   } while (i<ksp->max_it);
   if (i >= ksp->max_it) ksp->reason = KSP_DIVERGED_ITS;
-  if (eigs) cg->ned = ksp->its;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDestroy_CG"
+/*    
+       KSPSolve_CG_SingleReduction
+
+       This variant of CG is identical in exact arithmetic to the standard algorithm, 
+       but is rearranged to use only a single reduction stage per iteration, using additional
+       intermediate vectors.
+
+       See KSPCGUseSingleReduction_CG()
+
+*/
+static PetscErrorCode KSPSolve_CG_SingleReduction(KSP ksp)
+{
+  PetscErrorCode ierr;
+  PetscInt       i,stored_max_it,eigs;
+  PetscScalar    dpi = 0.0,a = 1.0,beta,betaold = 1.0,b = 0,*e = 0,*d = 0,delta,dpiold,tmp[2];
+  PetscReal      dp  = 0.0;
+  Vec            X,B,Z,R,P,S,W,tmpvecs[2];
+  KSP_CG         *cg;
+  Mat            Amat,Pmat;
+  PetscBool      diagonalscale;
+
+  PetscFunctionBegin;
+  ierr = PCGetDiagonalScale(ksp->pc,&diagonalscale);CHKERRQ(ierr);
+  if (diagonalscale) SETERRQ1(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP,"Krylov method %s does not support diagonal scaling",((PetscObject)ksp)->type_name);
+
+  cg            = (KSP_CG*)ksp->data;
+  eigs          = ksp->calc_sings;
+  stored_max_it = ksp->max_it;
+  X             = ksp->vec_sol;
+  B             = ksp->vec_rhs;
+  R             = ksp->work[0];
+  Z             = ksp->work[1];
+  P             = ksp->work[2];
+  S             = ksp->work[3];
+  W             = ksp->work[4];
+
+  if (eigs) {e = cg->e; d = cg->d; e[0] = 0.0; }
+  ierr = PCGetOperators(ksp->pc,&Amat,&Pmat);CHKERRQ(ierr);
+
+  ksp->its = 0;
+  if (!ksp->guess_zero) {
+    ierr = KSP_MatMult(ksp,Amat,X,R);CHKERRQ(ierr);            /*    r <- b - Ax                       */
+    ierr = VecAYPX(R,-1.0,B);CHKERRQ(ierr);
+  } else {
+    ierr = VecCopy(B,R);CHKERRQ(ierr);                         /*    r <- b (x is 0)                   */
+  }
+
+  switch (ksp->normtype) {
+    case KSP_NORM_PRECONDITIONED:
+      ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*    z <- Br                           */
+      ierr = VecNorm(Z,NORM_2,&dp);CHKERRQ(ierr);              /*    dp <- z'*z = e'*A'*B'*B*A'*e'     */
+      break;
+    case KSP_NORM_UNPRECONDITIONED:
+      ierr = VecNorm(R,NORM_2,&dp);CHKERRQ(ierr);              /*    dp <- r'*r = e'*A'*A*e            */
+      break;
+    case KSP_NORM_NATURAL:
+      ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*    z <- Br                           */
+      ierr = KSP_MatMult(ksp,Amat,Z,S);CHKERRQ(ierr);
+      ierr = VecXDot(Z,S,&delta);CHKERRQ(ierr);                /*    delta <- z'*A*z = r'*B*A*B*r      */
+      ierr = VecXDot(Z,R,&beta);CHKERRQ(ierr);                 /*    beta <- z'*r                      */
+      KSPCheckDot(ksp,beta);
+      dp = PetscSqrtReal(PetscAbsScalar(beta));                /*    dp <- r'*z = r'*B*r = e'*A'*B*A*e */
+      break;
+    case KSP_NORM_NONE:
+      dp = 0.0;
+      break;
+    default: SETERRQ1(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP,"%s",KSPNormTypes[ksp->normtype]);
+  }
+  ierr       = KSPLogResidualHistory(ksp,dp);CHKERRQ(ierr);
+  ierr       = KSPMonitor(ksp,0,dp);CHKERRQ(ierr);
+  ksp->rnorm = dp;
+
+  ierr = (*ksp->converged)(ksp,0,dp,&ksp->reason,ksp->cnvP);CHKERRQ(ierr);     /* test for convergence */
+  if (ksp->reason) PetscFunctionReturn(0);
+
+  if (ksp->normtype != KSP_NORM_PRECONDITIONED && (ksp->normtype != KSP_NORM_NATURAL)) {
+    ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);                 /*    z <- Br                           */
+  }
+  if (ksp->normtype != KSP_NORM_NATURAL) {
+    ierr = KSP_MatMult(ksp,Amat,Z,S);CHKERRQ(ierr);
+    ierr = VecXDot(Z,S,&delta);CHKERRQ(ierr);                  /*    delta <- z'*A*z = r'*B*A*B*r      */
+    ierr = VecXDot(Z,R,&beta);CHKERRQ(ierr);                   /*    beta <- z'*r                      */
+    KSPCheckDot(ksp,beta);
+  }
+
+  i = 0;
+  do {
+    ksp->its = i+1;
+    if (beta == 0.0) {
+      ksp->reason = KSP_CONVERGED_ATOL;
+      ierr        = PetscInfo(ksp,"converged due to beta = 0\n");CHKERRQ(ierr);
+      break;
+#if !defined(PETSC_USE_COMPLEX)
+    } else if ((i > 0) && (beta*betaold < 0.0)) {
+      if (ksp->errorifnotconverged) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_NOT_CONVERGED,"Diverged due to indefinite preconditioner");
+      ksp->reason = KSP_DIVERGED_INDEFINITE_PC;
+      ierr        = PetscInfo(ksp,"diverging due to indefinite preconditioner\n");CHKERRQ(ierr);
+      break;
+#endif
+    }
+    if (!i) {
+      ierr = VecCopy(Z,P);CHKERRQ(ierr);                       /*    p <- z                           */
+      b    = 0.0;
+    } else {
+      b = beta/betaold;
+      if (eigs) {
+        if (ksp->max_it != stored_max_it) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP,"Can not change maxit AND calculate eigenvalues");
+        e[i] = PetscSqrtReal(PetscAbsScalar(b))/a;
+      }
+      ierr = VecAYPX(P,b,Z);CHKERRQ(ierr);                     /*    p <- z + b* p                     */
+    }
+    dpiold = dpi;
+    if (!i) {
+      ierr = KSP_MatMult(ksp,Amat,P,W);CHKERRQ(ierr);          /*    w <- Ap                           */
+      ierr = VecXDot(P,W,&dpi);CHKERRQ(ierr);                  /*    dpi <- p'w                        */
+    } else {
+      ierr = VecAYPX(W,beta/betaold,S);CHKERRQ(ierr);          /*    w <- Ap                           */
+      dpi  = delta - beta*beta*dpiold/(betaold*betaold);       /*    dpi <- p'w                        */
+    }
+    betaold = beta;
+    KSPCheckDot(ksp,beta);
+
+    if ((dpi == 0.0) || ((i > 0) && (PetscRealPart(dpi*dpiold) <= 0.0))) {
+      if (ksp->errorifnotconverged) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_NOT_CONVERGED,"Diverged due to indefinite matrix");
+      ksp->reason = KSP_DIVERGED_INDEFINITE_MAT;
+      ierr        = PetscInfo(ksp,"diverging due to indefinite or negative definite matrix\n");CHKERRQ(ierr);
+      break;
+    }
+    a = beta/dpi;                                              /*    a = beta/p'w                      */
+    if (eigs) d[i] = PetscSqrtReal(PetscAbsScalar(b))*e[i] + 1.0/a;
+    ierr = VecAXPY(X,a,P);CHKERRQ(ierr);                       /*    x <- x + ap                       */
+    ierr = VecAXPY(R,-a,W);CHKERRQ(ierr);                      /*    r <- r - aw                       */
+    if (ksp->normtype == KSP_NORM_PRECONDITIONED && ksp->chknorm < i+2) {
+      ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*    z <- Br                           */
+      ierr = KSP_MatMult(ksp,Amat,Z,S);CHKERRQ(ierr);
+      ierr = VecNorm(Z,NORM_2,&dp);CHKERRQ(ierr);              /*    dp <- z'*z                        */
+    } else if (ksp->normtype == KSP_NORM_UNPRECONDITIONED && ksp->chknorm < i+2) {
+      ierr = VecNorm(R,NORM_2,&dp);CHKERRQ(ierr);              /*    dp <- r'*r                        */
+    } else if (ksp->normtype == KSP_NORM_NATURAL) {
+      ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*    z <- Br                           */
+      tmpvecs[0] = S; tmpvecs[1] = R;
+      ierr  = KSP_MatMult(ksp,Amat,Z,S);CHKERRQ(ierr);       
+      ierr  = VecMDot(Z,2,tmpvecs,tmp);CHKERRQ(ierr);          /*    delta <- z'*A*z = r'*B*A*B*r      */
+      delta = tmp[0]; beta = tmp[1];                           /*    beta <- z'*r                      */
+      KSPCheckDot(ksp,beta);
+      dp = PetscSqrtReal(PetscAbsScalar(beta));                /*    dp <- r'*z = r'*B*r = e'*A'*B*A*e */
+    } else {
+      dp = 0.0;
+    }
+    ksp->rnorm = dp;
+    CHKERRQ(ierr);KSPLogResidualHistory(ksp,dp);CHKERRQ(ierr);
+    if (eigs) cg->ned = ksp->its;
+    ierr = KSPMonitor(ksp,i+1,dp);CHKERRQ(ierr);
+    ierr = (*ksp->converged)(ksp,i+1,dp,&ksp->reason,ksp->cnvP);CHKERRQ(ierr);
+    if (ksp->reason) break;
+
+    if ((ksp->normtype != KSP_NORM_PRECONDITIONED && (ksp->normtype != KSP_NORM_NATURAL)) || (ksp->chknorm >= i+2)) {
+      ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr);               /*    z <- Br                           */
+      ierr = KSP_MatMult(ksp,Amat,Z,S);CHKERRQ(ierr);
+    }
+    if ((ksp->normtype != KSP_NORM_NATURAL) || (ksp->chknorm >= i+2)) {
+      tmpvecs[0] = S; tmpvecs[1] = R;
+      ierr  = VecMDot(Z,2,tmpvecs,tmp);CHKERRQ(ierr);
+      delta = tmp[0]; beta = tmp[1];                           /*    delta <- z'*A*z = r'*B'*A*B*r     */
+      KSPCheckDot(ksp,beta);                                   /*    beta <- z'*r                      */
+    }
+
+    i++;
+  } while (i<ksp->max_it);
+  if (i >= ksp->max_it) ksp->reason = KSP_DIVERGED_ITS;
+  PetscFunctionReturn(0);
+}
+
+/*
+     KSPDestroy_CG - Frees resources allocated in KSPSetup_CG and clears function
+                     compositions from KSPCreate_CG. If adding your own KSP implementation,
+                     you must be sure to free all allocated resources here to prevent
+                     leaks.
+*/
 PetscErrorCode KSPDestroy_CG(KSP ksp)
 {
   KSP_CG         *cg = (KSP_CG*)ksp->data;
@@ -297,18 +434,12 @@ PetscErrorCode KSPDestroy_CG(KSP ksp)
 }
 
 /*
-     KSPView_CG - Prints information about the current Krylov method being used
-
-      Currently this only prints information to a file (or stdout) about the
-      symmetry of the problem. If your Krylov method has special options or
-      flags that information should be printed here.
-
+     KSPView_CG - Prints information about the current Krylov method being used.
+                  If your Krylov method has special options or flags that information 
+                  should be printed here.
 */
-#undef __FUNCT__
-#define __FUNCT__ "KSPView_CG"
 PetscErrorCode KSPView_CG(KSP ksp,PetscViewer viewer)
 {
-#if defined(PETSC_USE_COMPLEX)
   KSP_CG         *cg = (KSP_CG*)ksp->data;
   PetscErrorCode ierr;
   PetscBool      iascii;
@@ -316,9 +447,13 @@ PetscErrorCode KSPView_CG(KSP ksp,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  CG or CGNE: variant %s\n",KSPCGTypes[cg->type]);CHKERRQ(ierr);
-  }
+#if defined(PETSC_USE_COMPLEX)
+    ierr = PetscViewerASCIIPrintf(viewer,"  variant %s\n",KSPCGTypes[cg->type]);CHKERRQ(ierr);
 #endif
+    if (cg->singlereduction) {
+      ierr = PetscViewerASCIIPrintf(viewer,"  using single-reduction variant\n");CHKERRQ(ierr);
+    }
+  }
   PetscFunctionReturn(0);
 }
 
@@ -326,9 +461,7 @@ PetscErrorCode KSPView_CG(KSP ksp,PetscViewer viewer)
     KSPSetFromOptions_CG - Checks the options database for options related to the
                            conjugate gradient method.
 */
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetFromOptions_CG"
-PetscErrorCode KSPSetFromOptions_CG(PetscOptions *PetscOptionsObject,KSP ksp)
+PetscErrorCode KSPSetFromOptions_CG(PetscOptionItems *PetscOptionsObject,KSP ksp)
 {
   PetscErrorCode ierr;
   KSP_CG         *cg = (KSP_CG*)ksp->data;
@@ -339,7 +472,7 @@ PetscErrorCode KSPSetFromOptions_CG(PetscOptions *PetscOptionsObject,KSP ksp)
   ierr = PetscOptionsEnum("-ksp_cg_type","Matrix is Hermitian or complex symmetric","KSPCGSetType",KSPCGTypes,(PetscEnum)cg->type,
                           (PetscEnum*)&cg->type,NULL);CHKERRQ(ierr);
 #endif
-  ierr = PetscOptionsBool("-ksp_cg_single_reduction","Merge inner products into single MPI_Allreduce()","KSPCGUseSingleReduction",cg->singlereduction,&cg->singlereduction,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-ksp_cg_single_reduction","Merge inner products into single MPIU_Allreduce()","KSPCGUseSingleReduction",cg->singlereduction,&cg->singlereduction,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -349,9 +482,7 @@ PetscErrorCode KSPSetFromOptions_CG(PetscOptions *PetscOptionsObject,KSP ksp)
                       This routine is registered below in KSPCreate_CG() and called from the
                       routine KSPCGSetType() (see the file cgtype.c).
 */
-#undef __FUNCT__
-#define __FUNCT__ "KSPCGSetType_CG"
-static PetscErrorCode  KSPCGSetType_CG(KSP ksp,KSPCGType type)
+PetscErrorCode  KSPCGSetType_CG(KSP ksp,KSPCGType type)
 {
   KSP_CG *cg = (KSP_CG*)ksp->data;
 
@@ -360,14 +491,24 @@ static PetscErrorCode  KSPCGSetType_CG(KSP ksp,KSPCGType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPCGUseSingleReduction_CG"
+/*
+    KSPCGUseSingleReduction_CG
+
+    This routine sets a flag to use a variant of CG. Note that (in somewhat
+    atypical fashion) it also swaps out the routine called when KSPSolve()
+    is invoked.
+*/
 static PetscErrorCode  KSPCGUseSingleReduction_CG(KSP ksp,PetscBool flg)
 {
   KSP_CG *cg = (KSP_CG*)ksp->data;
 
   PetscFunctionBegin;
   cg->singlereduction = flg;
+  if (cg->singlereduction) {
+    ksp->ops->solve = KSPSolve_CG_SingleReduction;
+  } else {
+    ksp->ops->solve = KSPSolve_CG;
+  }
   PetscFunctionReturn(0);
 }
 
@@ -378,35 +519,43 @@ static PetscErrorCode  KSPCGUseSingleReduction_CG(KSP ksp,PetscBool flg)
     It must be labeled as PETSC_EXTERN to be dynamically linkable in C++
 */
 /*MC
-     KSPCG - The preconditioned conjugate gradient (PCG) iterative method
+     KSPCG - The Preconditioned Conjugate Gradient (PCG) iterative method
 
    Options Database Keys:
 +   -ksp_cg_type Hermitian - (for complex matrices only) indicates the matrix is Hermitian, see KSPCGSetType()
 .   -ksp_cg_type symmetric - (for complex matrices only) indicates the matrix is symmetric
--   -ksp_cg_single_reduction - performs both inner products needed in the algorithm with a single MPI_Allreduce() call, see KSPCGUseSingleReduction()
+-   -ksp_cg_single_reduction - performs both inner products needed in the algorithm with a single MPIU_Allreduce() call, see KSPCGUseSingleReduction()
 
    Level: beginner
 
-   Notes: The PCG method requires both the matrix and preconditioner to be symmetric positive (or negative) (semi) definite
-          Only left preconditioning is supported.
+   Notes:
+    The PCG method requires both the matrix and preconditioner to be symmetric positive (or negative) (semi) definite.  
+   
+   Only left preconditioning is supported; there are several ways to motivate preconditioned CG, but they all produce the same algorithm. 
+   One can interpret preconditioning A with B to mean any of the following\:
+.n  (1) Solve a left-preconditioned system BAx = Bb, using inv(B) to define an inner product in the algorithm.
+.n  (2) Solve a right-preconditioned system ABy = b, x = By, using B to define an inner product in the algorithm.
+.n  (3) Solve a symmetrically-preconditioned system, E^TAEy = E^Tb, x = Ey, where B = EE^T.
+.n  (4) Solve Ax=b with CG, but use the inner product defined by B to define the method [2].
+.n  In all cases, the resulting algorithm only requires application of B to vectors.
 
-   For complex numbers there are two different CG methods. One for Hermitian symmetric matrices and one for non-Hermitian symmetric matrices. Use
+   For complex numbers there are two different CG methods, one for Hermitian symmetric matrices and one for non-Hermitian symmetric matrices. Use
    KSPCGSetType() to indicate which type you are using.
 
-   Developer Notes: KSPSolve_CG() should actually query the matrix to determine if it is Hermitian symmetric or not and NOT require the user to
+   Developer Notes:
+    KSPSolve_CG() should actually query the matrix to determine if it is Hermitian symmetric or not and NOT require the user to
    indicate it to the KSP object.
 
    References:
-   Methods of Conjugate Gradients for Solving Linear Systems, Magnus R. Hestenes and Eduard Stiefel,
+.   1. - Magnus R. Hestenes and Eduard Stiefel, Methods of Conjugate Gradients for Solving Linear Systems,
    Journal of Research of the National Bureau of Standards Vol. 49, No. 6, December 1952 Research Paper 2379
-   pp. 409--436.
+.   2. - Josef Malek and Zdenek Strakos, Preconditioning and the Conjugate Gradient Method in the Context of Solving PDEs, 
+    SIAM, 2014.
 
 .seealso:  KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP,
            KSPCGSetType(), KSPCGUseSingleReduction(), KSPPIPECG, KSPGROPPCG
 
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_CG"
 PETSC_EXTERN PetscErrorCode KSPCreate_CG(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -424,7 +573,7 @@ PETSC_EXTERN PetscErrorCode KSPCreate_CG(KSP ksp)
   ierr = KSPSetSupportedNorm(ksp,KSP_NORM_PRECONDITIONED,PC_LEFT,3);CHKERRQ(ierr);
   ierr = KSPSetSupportedNorm(ksp,KSP_NORM_UNPRECONDITIONED,PC_LEFT,2);CHKERRQ(ierr);
   ierr = KSPSetSupportedNorm(ksp,KSP_NORM_NATURAL,PC_LEFT,2);CHKERRQ(ierr);
-  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_NONE,PC_LEFT,2);CHKERRQ(ierr);
+  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_NONE,PC_LEFT,1);CHKERRQ(ierr);
 
   /*
        Sets the functions that are associated with this data structure
@@ -447,7 +596,3 @@ PETSC_EXTERN PetscErrorCode KSPCreate_CG(KSP ksp)
   ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPCGUseSingleReduction_C",KSPCGUseSingleReduction_CG);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-
-
-
-
