@@ -1,6 +1,5 @@
 
-#include <petsc/private/kspimpl.h>             /*I "petscksp.h" I*/
-#include <../src/ksp/ksp/impls/cg/nash/nashimpl.h>
+#include <../src/ksp/ksp/impls/cg/nash/nashimpl.h>    /*I "petscksp.h" I*/
 
 #define NASH_PRECONDITIONED_DIRECTION   0
 #define NASH_UNPRECONDITIONED_DIRECTION 1
@@ -8,95 +7,12 @@
 
 static const char *DType_Table[64] = {  "preconditioned", "unpreconditioned"};
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPNASHSetRadius"
-/*@
-    KSPNASHSetRadius - Sets the radius of the trust region.
-
-    Logically Collective on KSP
-
-    Input Parameters:
-+   ksp    - the iterative context
--   radius - the trust region radius (Infinity is the default)
-
-    Options Database Key:
-.   -ksp_nash_radius <r>
-
-    Level: advanced
-
-.keywords: KSP, NASH, set, trust region radius
-@*/
-PetscErrorCode  KSPNASHSetRadius(KSP ksp, PetscReal radius)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(ksp, KSP_CLASSID, 1);
-  if (radius < 0.0) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_ARG_OUTOFRANGE, "Radius negative");
-  PetscValidLogicalCollectiveReal(ksp,radius,2);
-  ierr = PetscTryMethod(ksp,"KSPNASHSetRadius_C",(KSP,PetscReal),(ksp,radius));CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "KSPNASHGetNormD"
-/*@
-    KSPNASHGetNormD - Got norm of the direction.
-
-    Collective on KSP
-
-    Input Parameters:
-+   ksp    - the iterative context
--   norm_d - the norm of the direction
-
-    Level: advanced
-
-.keywords: KSP, NASH, get, norm direction
-@*/
-PetscErrorCode  KSPNASHGetNormD(KSP ksp, PetscReal *norm_d)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(ksp, KSP_CLASSID, 1);
-  ierr = PetscUseMethod(ksp,"KSPNASHGetNormD_C",(KSP,PetscReal*),(ksp,norm_d));CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "KSPNASHGetObjFcn"
-/*@
-    KSPNASHGetObjFcn - Get objective function value.
-
-    Collective on KSP
-
-    Input Parameters:
-+   ksp   - the iterative context
--   o_fcn - the objective function value
-
-    Level: advanced
-
-.keywords: KSP, NASH, get, objective function
-@*/
-PetscErrorCode  KSPNASHGetObjFcn(KSP ksp, PetscReal *o_fcn)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(ksp, KSP_CLASSID, 1);
-  ierr = PetscUseMethod(ksp,"KSPNASHGetObjFcn_C",(KSP,PetscReal*),(ksp,o_fcn));CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_NASH"
-
-PetscErrorCode KSPSolve_NASH(KSP ksp)
+static PetscErrorCode KSPCGSolve_NASH(KSP ksp)
 {
 #if defined(PETSC_USE_COMPLEX)
   SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP, "NASH is not available for complex systems");
 #else
-  KSP_NASH       *cg = (KSP_NASH*)ksp->data;
+  KSPCG_NASH     *cg = (KSPCG_NASH*)ksp->data;
   PetscErrorCode ierr;
   Mat            Qmat, Mmat;
   Vec            r, z, p, d;
@@ -170,7 +86,7 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
     /*************************************************************************/
 
     ksp->reason = KSP_DIVERGED_NANORINF;
-    ierr        = PetscInfo1(ksp, "KSPSolve_NASH: bad preconditioner: rz=%g\n", (double)rz);CHKERRQ(ierr);
+    ierr        = PetscInfo1(ksp, "KSPCGSolve_NASH: bad preconditioner: rz=%g\n", (double)rz);CHKERRQ(ierr);
 
     if (cg->radius) {
       if (r2 >= rr) {
@@ -205,7 +121,7 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
     /*************************************************************************/
 
     ksp->reason = KSP_DIVERGED_INDEFINITE_PC;
-    ierr        = PetscInfo1(ksp, "KSPSolve_NASH: indefinite preconditioner: rz=%g\n", (double)rz);CHKERRQ(ierr);
+    ierr        = PetscInfo1(ksp, "KSPCGSolve_NASH: indefinite preconditioner: rz=%g\n", (double)rz);CHKERRQ(ierr);
 
     if (cg->radius) {
       if (r2 >= rr) {
@@ -283,7 +199,7 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
     /*************************************************************************/
 
     ksp->reason = KSP_DIVERGED_NANORINF;
-    ierr        = PetscInfo1(ksp, "KSPSolve_NASH: bad matrix: kappa=%g\n", (double)kappa);CHKERRQ(ierr);
+    ierr        = PetscInfo1(ksp, "KSPCGSolve_NASH: bad matrix: kappa=%g\n", (double)kappa);CHKERRQ(ierr);
 
     if (cg->radius) {
       if (r2 >= rr) {
@@ -337,7 +253,7 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
     /*************************************************************************/
 
     ksp->reason = KSP_CONVERGED_CG_NEG_CURVE;
-    ierr        = PetscInfo1(ksp, "KSPSolve_NASH: negative curvature: kappa=%g\n", (double)kappa);CHKERRQ(ierr);
+    ierr        = PetscInfo1(ksp, "KSPCGSolve_NASH: negative curvature: kappa=%g\n", (double)kappa);CHKERRQ(ierr);
 
     if (cg->radius && norm_p > 0.0) {
       /***********************************************************************/
@@ -411,7 +327,7 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
       /***********************************************************************/
 
       ksp->reason = KSP_CONVERGED_CG_CONSTRAINED;
-      ierr        = PetscInfo1(ksp, "KSPSolve_NASH: constrained step: radius=%g\n", (double)cg->radius);CHKERRQ(ierr);
+      ierr        = PetscInfo1(ksp, "KSPCGSolve_NASH: constrained step: radius=%g\n", (double)cg->radius);CHKERRQ(ierr);
 
       if (norm_p > 0.0) {
         /*********************************************************************/
@@ -473,7 +389,7 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
       /***********************************************************************/
 
       ksp->reason = KSP_DIVERGED_INDEFINITE_PC;
-      ierr        = PetscInfo1(ksp, "KSPSolve_NASH: cg indefinite preconditioner: rz=%g\n", (double)rz);CHKERRQ(ierr);
+      ierr        = PetscInfo1(ksp, "KSPCGSolve_NASH: cg indefinite preconditioner: rz=%g\n", (double)rz);CHKERRQ(ierr);
       break;
     }
 
@@ -510,7 +426,7 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
       /* The method has converged.                                           */
       /***********************************************************************/
 
-      ierr = PetscInfo2(ksp, "KSPSolve_NASH: truncated step: rnorm=%g, radius=%g\n", (double)norm_r, (double)cg->radius);CHKERRQ(ierr);
+      ierr = PetscInfo2(ksp, "KSPCGSolve_NASH: truncated step: rnorm=%g, radius=%g\n", (double)norm_r, (double)cg->radius);CHKERRQ(ierr);
       break;
     }
 
@@ -525,7 +441,7 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
       /***********************************************************************/
 
       ksp->reason = KSP_DIVERGED_BREAKDOWN;
-      ierr        = PetscInfo1(ksp, "KSPSolve_NASH: breakdown: beta=%g\n", (double)beta);CHKERRQ(ierr);
+      ierr        = PetscInfo1(ksp, "KSPCGSolve_NASH: breakdown: beta=%g\n", (double)beta);CHKERRQ(ierr);
       break;
     }
 
@@ -535,7 +451,7 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
 
     if (ksp->its >= max_cg_its) {
       ksp->reason = KSP_DIVERGED_ITS;
-      ierr        = PetscInfo1(ksp, "KSPSolve_NASH: iterlim: its=%D\n", ksp->its);CHKERRQ(ierr);
+      ierr        = PetscInfo1(ksp, "KSPCGSolve_NASH: iterlim: its=%D\n", ksp->its);CHKERRQ(ierr);
       break;
     }
 
@@ -576,7 +492,7 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
       /***********************************************************************/
 
       ksp->reason = KSP_CONVERGED_CG_NEG_CURVE;
-      ierr        = PetscInfo1(ksp, "KSPSolve_NASH: negative curvature: kappa=%g\n", (double)kappa);CHKERRQ(ierr);
+      ierr        = PetscInfo1(ksp, "KSPCGSolve_NASH: negative curvature: kappa=%g\n", (double)kappa);CHKERRQ(ierr);
       break;
     }
   }
@@ -584,24 +500,20 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_NASH"
-PetscErrorCode KSPSetUp_NASH(KSP ksp)
+static PetscErrorCode KSPCGSetUp_NASH(KSP ksp)
 {
   PetscErrorCode ierr;
 
-  PetscFunctionBegin;
   /***************************************************************************/
   /* Set work vectors needed by conjugate gradient method and allocate       */
   /***************************************************************************/
 
-  ierr = KSPSetWorkVecs(ksp, 3);CHKERRQ(ierr);
+  PetscFunctionBegin;
+  ierr = KSPSetWorkVecs(ksp,3);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDestroy_NASH"
-PetscErrorCode KSPDestroy_NASH(KSP ksp)
+static PetscErrorCode KSPCGDestroy_NASH(KSP ksp)
 {
   PetscErrorCode ierr;
 
@@ -610,9 +522,9 @@ PetscErrorCode KSPDestroy_NASH(KSP ksp)
   /* Clear composed functions                                                */
   /***************************************************************************/
 
-  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPNASHSetRadius_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPNASHGetNormD_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPNASHGetObjFcn_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPCGSetRadius_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPCGGetNormD_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPCGGetObjFcn_C",NULL);CHKERRQ(ierr);
 
   /***************************************************************************/
   /* Destroy KSP object.                                                     */
@@ -622,66 +534,59 @@ PetscErrorCode KSPDestroy_NASH(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPNASHSetRadius_NASH"
-static PetscErrorCode  KSPNASHSetRadius_NASH(KSP ksp, PetscReal radius)
+static PetscErrorCode  KSPCGSetRadius_NASH(KSP ksp, PetscReal radius)
 {
-  KSP_NASH *cg = (KSP_NASH*)ksp->data;
+  KSPCG_NASH *cg = (KSPCG_NASH*)ksp->data;
 
   PetscFunctionBegin;
   cg->radius = radius;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPNASHGetNormD_NASH"
-static PetscErrorCode  KSPNASHGetNormD_NASH(KSP ksp, PetscReal *norm_d)
+static PetscErrorCode  KSPCGGetNormD_NASH(KSP ksp, PetscReal *norm_d)
 {
-  KSP_NASH *cg = (KSP_NASH*)ksp->data;
+  KSPCG_NASH *cg = (KSPCG_NASH*)ksp->data;
 
   PetscFunctionBegin;
   *norm_d = cg->norm_d;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPNASHGetObjFcn_NASH"
-static PetscErrorCode  KSPNASHGetObjFcn_NASH(KSP ksp, PetscReal *o_fcn)
+static PetscErrorCode  KSPCGGetObjFcn_NASH(KSP ksp, PetscReal *o_fcn)
 {
-  KSP_NASH *cg = (KSP_NASH*)ksp->data;
+  KSPCG_NASH *cg = (KSPCG_NASH*)ksp->data;
 
   PetscFunctionBegin;
   *o_fcn = cg->o_fcn;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetFromOptions_NASH"
-PetscErrorCode KSPSetFromOptions_NASH(PetscOptions *PetscOptionsObject,KSP ksp)
+static PetscErrorCode KSPCGSetFromOptions_NASH(PetscOptionItems *PetscOptionsObject,KSP ksp)
 {
   PetscErrorCode ierr;
-  KSP_NASH       *cg = (KSP_NASH*)ksp->data;
+  KSPCG_NASH     *cg = (KSPCG_NASH*)ksp->data;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsHead(PetscOptionsObject,"KSP NASH options");CHKERRQ(ierr);
+  ierr = PetscOptionsHead(PetscOptionsObject,"KSPCG NASH options");CHKERRQ(ierr);
 
-  ierr = PetscOptionsReal("-ksp_nash_radius", "Trust Region Radius", "KSPNASHSetRadius", cg->radius, &cg->radius, NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsReal("-ksp_cg_radius", "Trust Region Radius", "KSPCGSetRadius", cg->radius, &cg->radius, NULL);CHKERRQ(ierr);
 
-  ierr = PetscOptionsEList("-ksp_nash_dtype", "Norm used for direction", "", DType_Table, NASH_DIRECTION_TYPES, DType_Table[cg->dtype], &cg->dtype, NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsEList("-ksp_cg_dtype", "Norm used for direction", "", DType_Table, NASH_DIRECTION_TYPES, DType_Table[cg->dtype], &cg->dtype, NULL);CHKERRQ(ierr);
 
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 /*MC
-     KSPNASH -   Code to run conjugate gradient method subject to a constraint
+     KSPCGNASH -   Code to run conjugate gradient method subject to a constraint
          on the solution norm. This is used in Trust Region methods for
          nonlinear equations, SNESNEWTONTR
 
    Options Database Keys:
-.      -ksp_nash_radius <r> - Trust Region Radius
+.      -ksp_cg_radius <r> - Trust Region Radius
 
-   Notes: This is rarely used directly
+   Notes:
+    This is rarely used directly
 
    Level: developer
 
@@ -709,15 +614,13 @@ $  other KSP converged/diverged reasons
   Notes:
   The preconditioner supplied should be symmetric and positive definite.
 
-.seealso:  KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP, KSPNASHSetRadius(), KSPNASHGetNormD(), KSPNASHGetObjFcn()
+.seealso:  KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP, KSPCGSetRadius(), KSPCGGetNormD(), KSPCGGetObjFcn()
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_NASH"
-PETSC_EXTERN PetscErrorCode KSPCreate_NASH(KSP ksp)
+PETSC_EXTERN PetscErrorCode KSPCreate_CGNASH(KSP ksp)
 {
   PetscErrorCode ierr;
-  KSP_NASH       *cg;
+  KSPCG_NASH     *cg;
 
   PetscFunctionBegin;
   ierr       = PetscNewLog(ksp,&cg);CHKERRQ(ierr);
@@ -734,16 +637,16 @@ PETSC_EXTERN PetscErrorCode KSPCreate_NASH(KSP ksp)
   /* (in C++ this is the same as defining virtual functions).                */
   /***************************************************************************/
 
-  ksp->ops->setup          = KSPSetUp_NASH;
-  ksp->ops->solve          = KSPSolve_NASH;
-  ksp->ops->destroy        = KSPDestroy_NASH;
-  ksp->ops->setfromoptions = KSPSetFromOptions_NASH;
+  ksp->ops->setup          = KSPCGSetUp_NASH;
+  ksp->ops->solve          = KSPCGSolve_NASH;
+  ksp->ops->destroy        = KSPCGDestroy_NASH;
+  ksp->ops->setfromoptions = KSPCGSetFromOptions_NASH;
   ksp->ops->buildsolution  = KSPBuildSolutionDefault;
   ksp->ops->buildresidual  = KSPBuildResidualDefault;
   ksp->ops->view           = 0;
 
-  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPNASHSetRadius_C",KSPNASHSetRadius_NASH);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPNASHGetNormD_C",KSPNASHGetNormD_NASH);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPNASHGetObjFcn_C",KSPNASHGetObjFcn_NASH);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPCGSetRadius_C",KSPCGSetRadius_NASH);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPCGGetNormD_C",KSPCGGetNormD_NASH);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPCGGetObjFcn_C",KSPCGGetObjFcn_NASH);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
