@@ -12,15 +12,17 @@
 /*     Journal on Numerical Analysis, 21, pages 553-572, 1984.               */
 /*****************************************************************************/
 
-#if !defined(__NASH)
-#define __NASH
+#if !defined(__CG_NASH)
+#define __CG_NASH
+
+#include <petsc/private/kspimpl.h>
 
 typedef struct {
   PetscReal radius;
   PetscReal norm_d;
   PetscReal o_fcn;
   PetscInt  dtype;
-} KSP_NASH;
+} KSPCG_NASH;
 
 #endif
 
