@@ -7,8 +7,6 @@
 */
 #include <petsc/private/kspimpl.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_CGS"
 static PetscErrorCode KSPSetUp_CGS(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -18,8 +16,6 @@ static PetscErrorCode KSPSetUp_CGS(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_CGS"
 static PetscErrorCode  KSPSolve_CGS(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -136,18 +132,19 @@ static PetscErrorCode  KSPSolve_CGS(KSP ksp)
 
    Level: beginner
 
-   References: Sonneveld, 1989.
+   References:
+.     1. - Sonneveld, 1989.
 
-   Notes: Does not require a symmetric matrix. Does not apply transpose of the matrix.
+   Notes:
+    Does not require a symmetric matrix. Does not apply transpose of the matrix.
         Supports left and right preconditioning, but not symmetric.
 
-   Developer Notes: Has this weird support for doing the convergence test with the natural norm, I assume this works only with
+   Developer Notes:
+    Has this weird support for doing the convergence test with the natural norm, I assume this works only with
       no preconditioning and symmetric positive definite operator.
 
 .seealso: KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP, KSPBCGS, KSPSetPCSide()
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_CGS"
 PETSC_EXTERN PetscErrorCode KSPCreate_CGS(KSP ksp)
 {
   PetscErrorCode ierr;
