@@ -1,9 +1,6 @@
 
-#include <petsc/private/kspimpl.h>                    /*I "petscksp.h" I*/
-#include <../src/ksp/ksp/impls/cheby/chebyshevimpl.h>
+#include <../src/ksp/ksp/impls/cheby/chebyshevimpl.h>    /*I "petscksp.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPReset_Chebyshev"
 static PetscErrorCode KSPReset_Chebyshev(KSP ksp)
 {
   KSP_Chebyshev  *cheb = (KSP_Chebyshev*)ksp->data;
@@ -14,8 +11,6 @@ static PetscErrorCode KSPReset_Chebyshev(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_Chebyshev"
 static PetscErrorCode KSPSetUp_Chebyshev(KSP ksp)
 {
   KSP_Chebyshev  *cheb = (KSP_Chebyshev*)ksp->data;
@@ -29,8 +24,6 @@ static PetscErrorCode KSPSetUp_Chebyshev(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPChebyshevSetEigenvalues_Chebyshev"
 static PetscErrorCode KSPChebyshevSetEigenvalues_Chebyshev(KSP ksp,PetscReal emax,PetscReal emin)
 {
   KSP_Chebyshev  *chebyshevP = (KSP_Chebyshev*)ksp->data;
@@ -46,8 +39,6 @@ static PetscErrorCode KSPChebyshevSetEigenvalues_Chebyshev(KSP ksp,PetscReal ema
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPChebyshevEstEigSet_Chebyshev"
 static PetscErrorCode KSPChebyshevEstEigSet_Chebyshev(KSP ksp,PetscReal a,PetscReal b,PetscReal c,PetscReal d)
 {
   KSP_Chebyshev  *cheb = (KSP_Chebyshev*)ksp->data;
@@ -58,11 +49,11 @@ static PetscErrorCode KSPChebyshevEstEigSet_Chebyshev(KSP ksp,PetscReal a,PetscR
     if (!cheb->kspest) { /* should this block of code be moved to KSPSetUp_Chebyshev()? */
       ierr = KSPCreate(PetscObjectComm((PetscObject)ksp),&cheb->kspest);CHKERRQ(ierr);
       ierr = PetscObjectIncrementTabLevel((PetscObject)cheb->kspest,(PetscObject)ksp,1);CHKERRQ(ierr);
-      ierr = KSPSetOptionsPrefix(cheb->kspest,((PetscObject)ksp)->prefix);CHKERRQ(ierr);
-      ierr = KSPAppendOptionsPrefix(cheb->kspest,"esteig_");CHKERRQ(ierr);
-      ierr = KSPSetSkipPCSetFromOptions(cheb->kspest,PETSC_TRUE);CHKERRQ(ierr);
-
       ierr = KSPSetPC(cheb->kspest,ksp->pc);CHKERRQ(ierr);
+      /* use PetscObjectSet/AppendOptionsPrefix() instead of KSPSet/AppendOptionsPrefix() so that the PC prefix is not changed */
+      ierr = PetscObjectSetOptionsPrefix((PetscObject)cheb->kspest,((PetscObject)ksp)->prefix);CHKERRQ(ierr);
+      ierr = PetscObjectAppendOptionsPrefix((PetscObject)cheb->kspest,"esteig_");CHKERRQ(ierr);
+      ierr = KSPSetSkipPCSetFromOptions(cheb->kspest,PETSC_TRUE);CHKERRQ(ierr);
 
       ierr = KSPSetComputeEigenvalues(cheb->kspest,PETSC_TRUE);CHKERRQ(ierr);
 
@@ -83,33 +74,15 @@ static PetscErrorCode KSPChebyshevEstEigSet_Chebyshev(KSP ksp,PetscReal a,PetscR
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPChebyshevEstEigSetRandom_Chebyshev"
-static PetscErrorCode KSPChebyshevEstEigSetRandom_Chebyshev(KSP ksp,PetscRandom random)
-{
-  KSP_Chebyshev  *cheb = (KSP_Chebyshev*)ksp->data;
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  if (random) {ierr = PetscObjectReference((PetscObject)random);CHKERRQ(ierr);}
-  ierr = PetscRandomDestroy(&cheb->random);CHKERRQ(ierr);
-  cheb->random = random;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "KSPChebyshevEstEigSetUseRandom_Chebyshev"
-static PetscErrorCode KSPChebyshevEstEigSetUseRandom_Chebyshev(KSP ksp,PetscBool use)
+static PetscErrorCode KSPChebyshevEstEigSetUseNoisy_Chebyshev(KSP ksp,PetscBool use)
 {
   KSP_Chebyshev  *cheb = (KSP_Chebyshev*)ksp->data;
 
   PetscFunctionBegin;
-  cheb->userandom = use;
+  cheb->usenoisy = use;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPChebyshevSetEigenvalues"
 /*@
    KSPChebyshevSetEigenvalues - Sets estimates for the extreme eigenvalues
    of the preconditioned problem.
@@ -142,8 +115,6 @@ PetscErrorCode  KSPChebyshevSetEigenvalues(KSP ksp,PetscReal emax,PetscReal emin
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPChebyshevEstEigSet"
 /*@
    KSPChebyshevEstEigSet - Automatically estimate the eigenvalues to use for Chebyshev
 
@@ -172,7 +143,7 @@ PetscErrorCode  KSPChebyshevSetEigenvalues(KSP ksp,PetscReal emax,PetscReal emin
 
    The default transform is (0,0.1; 0,1.1) which targets the "upper" part of the spectrum, as desirable for use with multigrid.
 
-   The eigenvalues are estimated using the Lanczo (KSPCG) or Arnoldi (KSPGMRES) process using a random right hand side vector.
+   The eigenvalues are estimated using the Lanczo (KSPCG) or Arnoldi (KSPGMRES) process using a noisy right hand side vector.
 
    Level: intermediate
 
@@ -192,66 +163,35 @@ PetscErrorCode KSPChebyshevEstEigSet(KSP ksp,PetscReal a,PetscReal b,PetscReal c
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPChebyshevEstEigSetUseRandom"
 /*@
-   KSPChebyshevEstEigSetUseRandom - use a random right hand side in order to do the estimate instead of the given right hand side
+   KSPChebyshevEstEigSetUseNoisy - use a noisy right hand side in order to do the estimate instead of the given right hand side
 
    Logically Collective
 
    Input Arguments:
 +  ksp - linear solver context
--  use - PETSC_TRUE to use random
+-  use - PETSC_TRUE to use noisy
 
    Options Database:
-+  -ksp_chebyshev_esteig_random <true,false>
++  -ksp_chebyshev_esteig_noisy <true,false>
 
-  Notes: This alledgely works better for multigrid smoothers
-
-  Use KSPChebyshevEstEigSetRandom() to provide the random number generator to be used. Otherwise it creates a default
+  Notes:
+    This alledgely works better for multigrid smoothers
 
   Level: intermediate
 
-.seealso: KSPChebyshevEstEigSet(), PetscRandomCreate(), KSPChebyshevEstEigSetRandom()
+.seealso: KSPChebyshevEstEigSet()
 @*/
-PetscErrorCode KSPChebyshevEstEigSetUseRandom(KSP ksp,PetscBool use)
+PetscErrorCode KSPChebyshevEstEigSetUseNoisy(KSP ksp,PetscBool use)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
-  ierr = PetscTryMethod(ksp,"KSPChebyshevEstEigSetUseRandom_C",(KSP,PetscBool),(ksp,use));CHKERRQ(ierr);
+  ierr = PetscTryMethod(ksp,"KSPChebyshevEstEigSetUseNoisy_C",(KSP,PetscBool),(ksp,use));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPChebyshevEstEigSetRandom"
-/*@
-   KSPChebyshevEstEigSetRandom - set random context for estimating eigenvalues
-
-   Logically Collective
-
-   Input Arguments:
-+  ksp - linear solver context
--  random - random number context or NULL to use default
-
-  Level: intermediate
-
-.seealso: KSPChebyshevEstEigSet(), PetscRandomCreate()
-@*/
-PetscErrorCode KSPChebyshevEstEigSetRandom(KSP ksp,PetscRandom random)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
-  if (random) PetscValidHeaderSpecific(random,PETSC_RANDOM_CLASSID,2);
-  ierr = PetscTryMethod(ksp,"KSPChebyshevEstEigSetRandom_C",(KSP,PetscRandom),(ksp,random));CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "KSPChebyshevEstEigGetKSP"
 /*@
   KSPChebyshevEstEigGetKSP - Get the Krylov method context used to estimate eigenvalues for the Chebyshev method.  If
   a Krylov method is not being used for this purpose, NULL is returned.  The reference count of the returned KSP is
@@ -278,8 +218,6 @@ PetscErrorCode KSPChebyshevEstEigGetKSP(KSP ksp, KSP *kspest)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPChebyshevEstEigGetKSP_Chebyshev"
 static PetscErrorCode KSPChebyshevEstEigGetKSP_Chebyshev(KSP ksp, KSP *kspest)
 {
   KSP_Chebyshev *cheb = (KSP_Chebyshev*)ksp->data;
@@ -289,8 +227,6 @@ static PetscErrorCode KSPChebyshevEstEigGetKSP_Chebyshev(KSP ksp, KSP *kspest)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetFromOptions_Chebyshev"
 static PetscErrorCode KSPSetFromOptions_Chebyshev(PetscOptionItems *PetscOptionsObject,KSP ksp)
 {
   KSP_Chebyshev  *cheb = (KSP_Chebyshev*)ksp->data;
@@ -330,24 +266,13 @@ static PetscErrorCode KSPSetFromOptions_Chebyshev(PetscOptionItems *PetscOptions
   }
 
   if (cheb->kspest) {
-    ierr = PetscOptionsBool("-ksp_chebyshev_esteig_random","Use random right hand side for estimate","KSPChebyshevEstEigSetUseRandom",cheb->userandom,&cheb->userandom,NULL);CHKERRQ(ierr);
-    if (cheb->userandom) {
-      const char  *ksppre;
-      if (!cheb->random) {
-        ierr = PetscRandomCreate(PetscObjectComm((PetscObject)ksp),&cheb->random);CHKERRQ(ierr);
-      }
-      ierr = KSPGetOptionsPrefix(cheb->kspest, &ksppre);CHKERRQ(ierr);
-      ierr = PetscObjectSetOptionsPrefix((PetscObject)cheb->random,ksppre);CHKERRQ(ierr);
-      ierr = PetscRandomSetFromOptions(cheb->random);CHKERRQ(ierr);
-    }
+    ierr = PetscOptionsBool("-ksp_chebyshev_esteig_noisy","Use noisy right hand side for estimate","KSPChebyshevEstEigSetUseNoisy",cheb->usenoisy,&cheb->usenoisy,NULL);CHKERRQ(ierr);
     ierr = KSPSetFromOptions(cheb->kspest);CHKERRQ(ierr);
   }
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPChebyshevComputeExtremeEigenvalues_Private"
 /*
  * Must be passed a KSP solver that has "converged", with KSPSetComputeEigenvalues() called before the solve
  */
@@ -373,8 +298,15 @@ static PetscErrorCode KSPChebyshevComputeExtremeEigenvalues_Private(KSP kspest,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_Chebyshev"
+PETSC_STATIC_INLINE PetscScalar chebyhash(PetscInt xx)
+{
+  unsigned int x = xx;
+  x = ((x >> 16) ^ x) * 0x45d9f3b;
+  x = ((x >> 16) ^ x) * 0x45d9f3b;
+  x = ((x >> 16) ^ x);
+  return (PetscScalar)((PetscInt64)x-2147483648)*5.e-10; /* center around zero, scaled about -1. to 1.*/
+}
+
 static PetscErrorCode KSPSolve_Chebyshev(KSP ksp)
 {
   KSP_Chebyshev  *cheb = (KSP_Chebyshev*)ksp->data;
@@ -404,12 +336,21 @@ static PetscErrorCode KSPSolve_Chebyshev(KSP ksp)
       Vec                B;
       KSPConvergedReason reason;
 
-      if (cheb->userandom) {
+      if (cheb->usenoisy) {
         B  = ksp->work[1];
-        if (!cheb->random) {
-          ierr = PetscRandomCreate(PetscObjectComm((PetscObject)B),&cheb->random);CHKERRQ(ierr);
+        {
+          PetscErrorCode ierr;
+          PetscInt       n,i,istart;
+          PetscScalar    *xx;
+          ierr = VecGetOwnershipRange(B,&istart,NULL);CHKERRQ(ierr);
+          ierr = VecGetLocalSize(B,&n);CHKERRQ(ierr);
+          ierr = VecGetArray(B,&xx);CHKERRQ(ierr);
+          for (i=0; i<n; i++) {
+            PetscScalar v = chebyhash(i+istart);
+            xx[i] = v;
+          }
+          ierr = VecRestoreArray(B,&xx);CHKERRQ(ierr);
         }
-        ierr = VecSetRandom(B,cheb->random);CHKERRQ(ierr);
       } else {
         PC        pc;
         PetscBool change;
@@ -448,6 +389,8 @@ static PetscErrorCode KSPSolve_Chebyshev(KSP ksp)
 
       ierr = KSPChebyshevComputeExtremeEigenvalues_Private(cheb->kspest,&min,&max);CHKERRQ(ierr);
 
+      cheb->emin_computed = min;
+      cheb->emax_computed = max;
       cheb->emin = cheb->tform[0]*min + cheb->tform[1]*max;
       cheb->emax = cheb->tform[2]*min + cheb->tform[3]*max;
 
@@ -563,8 +506,6 @@ static PetscErrorCode KSPSolve_Chebyshev(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPView_Chebyshev"
 static  PetscErrorCode KSPView_Chebyshev(KSP ksp,PetscViewer viewer)
 {
   KSP_Chebyshev  *cheb = (KSP_Chebyshev*)ksp->data;
@@ -574,25 +515,21 @@ static  PetscErrorCode KSPView_Chebyshev(KSP ksp,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  Chebyshev: eigenvalue estimates:  min = %g, max = %g\n",(double)cheb->emin,(double)cheb->emax);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  eigenvalue estimates used:  min = %g, max = %g\n",(double)cheb->emin,(double)cheb->emax);CHKERRQ(ierr);
     if (cheb->kspest) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  Chebyshev: eigenvalues estimated using %s with translations  [%g %g; %g %g]\n",((PetscObject) cheb->kspest)->type_name,(double)cheb->tform[0],(double)cheb->tform[1],(double)cheb->tform[2],(double)cheb->tform[3]);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  eigenvalues estimate via %s min %g, max %g\n",((PetscObject)(cheb->kspest))->type_name,(double)cheb->emin_computed,(double)cheb->emax_computed);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  eigenvalues estimated using %s with translations  [%g %g; %g %g]\n",((PetscObject) cheb->kspest)->type_name,(double)cheb->tform[0],(double)cheb->tform[1],(double)cheb->tform[2],(double)cheb->tform[3]);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
       ierr = KSPView(cheb->kspest,viewer);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
-      if (cheb->userandom) {
-        ierr = PetscViewerASCIIPrintf(viewer,"  Chebyshev: estimating eigenvalues using random right hand side\n");CHKERRQ(ierr);
-        ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
-        ierr = PetscRandomView(cheb->random,viewer);CHKERRQ(ierr);
-        ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
+      if (cheb->usenoisy) {
+        ierr = PetscViewerASCIIPrintf(viewer,"  estimating eigenvalues using noisy right hand side\n");CHKERRQ(ierr);
       }
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDestroy_Chebyshev"
 static PetscErrorCode KSPDestroy_Chebyshev(KSP ksp)
 {
   KSP_Chebyshev  *cheb = (KSP_Chebyshev*)ksp->data;
@@ -600,10 +537,9 @@ static PetscErrorCode KSPDestroy_Chebyshev(KSP ksp)
 
   PetscFunctionBegin;
   ierr = KSPDestroy(&cheb->kspest);CHKERRQ(ierr);
-  ierr = PetscRandomDestroy(&cheb->random);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPChebyshevSetEigenvalues_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPChebyshevEstEigSet_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPChebyshevEstEigSetRandom_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPChebyshevEstEigSetUseNoisy_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPChebyshevEstEigGetKSP_C",NULL);CHKERRQ(ierr);
   ierr = KSPDestroyDefault(ksp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -618,11 +554,12 @@ static PetscErrorCode KSPDestroy_Chebyshev(KSP ksp)
 .   -ksp_chebyshev_esteig <a,b,c,d> - estimate eigenvalues using a Krylov method, then use this
                          transform for Chebyshev eigenvalue bounds (KSPChebyshevEstEigSet())
 .   -ksp_chebyshev_esteig_steps - number of estimation steps
--   -ksp_chebyshev_esteig_random - use random number generator to create right hand side for eigenvalue estimator
+-   -ksp_chebyshev_esteig_noisy - use noisy number generator to create right hand side for eigenvalue estimator
 
    Level: beginner
 
-   Notes: The Chebyshev method requires both the matrix and preconditioner to
+   Notes:
+    The Chebyshev method requires both the matrix and preconditioner to
           be symmetric positive (semi) definite.
           Only support for left preconditioning.
 
@@ -630,13 +567,11 @@ static PetscErrorCode KSPDestroy_Chebyshev(KSP ksp)
           The user should call KSPChebyshevSetEigenvalues() if they have eigenvalue estimates.
 
 .seealso:  KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP,
-           KSPChebyshevSetEigenvalues(), KSPChebyshevEstEigSet(), KSPChebyshevEstEigSetUseRandom(), KSPChebyshevEstEigSetRandom(),
+           KSPChebyshevSetEigenvalues(), KSPChebyshevEstEigSet(), KSPChebyshevEstEigSetUseNoisy()
            KSPRICHARDSON, KSPCG, PCMG
 
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_Chebyshev"
 PETSC_EXTERN PetscErrorCode KSPCreate_Chebyshev(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -648,6 +583,8 @@ PETSC_EXTERN PetscErrorCode KSPCreate_Chebyshev(KSP ksp)
   ksp->data = (void*)chebyshevP;
   ierr      = KSPSetSupportedNorm(ksp,KSP_NORM_PRECONDITIONED,PC_LEFT,3);CHKERRQ(ierr);
   ierr      = KSPSetSupportedNorm(ksp,KSP_NORM_UNPRECONDITIONED,PC_LEFT,2);CHKERRQ(ierr);
+  ierr      = KSPSetSupportedNorm(ksp,KSP_NORM_NONE,PC_LEFT,1);CHKERRQ(ierr);
+  ierr      = KSPSetSupportedNorm(ksp,KSP_NORM_NONE,PC_RIGHT,1);CHKERRQ(ierr);
 
   chebyshevP->emin = 0.;
   chebyshevP->emax = 0.;
@@ -657,7 +594,7 @@ PETSC_EXTERN PetscErrorCode KSPCreate_Chebyshev(KSP ksp)
   chebyshevP->tform[2] = 0;
   chebyshevP->tform[3] = 1.1;
   chebyshevP->eststeps = 10;
-  chebyshevP->userandom = PETSC_FALSE;
+  chebyshevP->usenoisy = PETSC_TRUE;
 
   ksp->ops->setup          = KSPSetUp_Chebyshev;
   ksp->ops->solve          = KSPSolve_Chebyshev;
@@ -670,8 +607,7 @@ PETSC_EXTERN PetscErrorCode KSPCreate_Chebyshev(KSP ksp)
 
   ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPChebyshevSetEigenvalues_C",KSPChebyshevSetEigenvalues_Chebyshev);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPChebyshevEstEigSet_C",KSPChebyshevEstEigSet_Chebyshev);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPChebyshevEstEigSetRandom_C",KSPChebyshevEstEigSetRandom_Chebyshev);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPChebyshevEstEigSetUseRandom_C",KSPChebyshevEstEigSetUseRandom_Chebyshev);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPChebyshevEstEigSetUseNoisy_C",KSPChebyshevEstEigSetUseNoisy_Chebyshev);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPChebyshevEstEigGetKSP_C",KSPChebyshevEstEigGetKSP_Chebyshev);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
