@@ -7,15 +7,11 @@
 extern PetscErrorCode KSPComputeExtremeSingularValues_CG(KSP,PetscReal*,PetscReal*);
 extern PetscErrorCode KSPComputeEigenvalues_CG(KSP,PetscInt,PetscReal*,PetscReal*,PetscInt*);
 
-const char *const KSPFCGTruncationTypes[]     = {"STANDARD","NOTAY","KSPFCGTrunctionTypes","KSP_FCG_TRUNC_TYPE_",0};
-
 #define KSPFCG_DEFAULT_MMAX 30          /* maximum number of search directions to keep */
 #define KSPFCG_DEFAULT_NPREALLOC 10     /* number of search directions to preallocate */
 #define KSPFCG_DEFAULT_VECB 5           /* number of search directions to allocate each time new direction vectors are needed */
-#define KSPFCG_DEFAULT_TRUNCSTRAT KSP_FCG_TRUNC_TYPE_NOTAY
+#define KSPFCG_DEFAULT_TRUNCSTRAT KSP_FCD_TRUNC_TYPE_NOTAY
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPAllocateVectors_FCG"
 static PetscErrorCode KSPAllocateVectors_FCG(KSP ksp, PetscInt nvecsneeded, PetscInt chunksize)
 {
   PetscErrorCode  ierr;
@@ -43,9 +39,7 @@ static PetscErrorCode KSPAllocateVectors_FCG(KSP ksp, PetscInt nvecsneeded, Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_FCG"
-PetscErrorCode    KSPSetUp_FCG(KSP ksp)
+static PetscErrorCode    KSPSetUp_FCG(KSP ksp)
 {
   PetscErrorCode ierr;
   KSP_FCG        *fcg = (KSP_FCG*)ksp->data;
@@ -63,6 +57,11 @@ PetscErrorCode    KSPSetUp_FCG(KSP ksp)
   ierr = PetscMalloc5(fcg->mmax+1,&fcg->Pvecs,fcg->mmax+1,&fcg->Cvecs,fcg->mmax+1,&fcg->pPvecs,fcg->mmax+1,&fcg->pCvecs,fcg->mmax+2,&fcg->chunksizes);CHKERRQ(ierr);
   ierr = PetscLogObjectMemory((PetscObject)ksp,2*(fcg->mmax+1)*sizeof(Vec*) + 2*(fcg->mmax + 1)*sizeof(Vec**) + (fcg->mmax + 2)*sizeof(PetscInt));CHKERRQ(ierr);
 
+  /* If the requested number of preallocated vectors is greater than mmax reduce nprealloc */
+  if(fcg->nprealloc > fcg->mmax+1){
+    ierr = PetscInfo2(NULL,"Requested nprealloc=%d is greater than m_max+1=%d. Resetting nprealloc = m_max+1.\n",fcg->nprealloc, fcg->mmax+1);CHKERRQ(ierr);
+  }
+
   /* Preallocate additional work vectors */
   ierr = KSPAllocateVectors_FCG(ksp,fcg->nprealloc,fcg->nprealloc);CHKERRQ(ierr);
   /*
@@ -80,9 +79,7 @@ PetscErrorCode    KSPSetUp_FCG(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_FCG"
-PetscErrorCode KSPSolve_FCG(KSP ksp)
+static PetscErrorCode KSPSolve_FCG(KSP ksp)
 {
   PetscErrorCode ierr;
   PetscInt       i,k,idx,mi;
@@ -162,17 +159,19 @@ PetscErrorCode KSPSolve_FCG(KSP ksp)
     Pcurr = fcg->Pvecs[idx];
     Ccurr = fcg->Cvecs[idx];
 
-    /* Compute a new column of P (Currently does not support modified G-S or iterative refinement)*/
+    /* number of old directions to orthogonalize against */
     switch(fcg->truncstrat){
-      case KSP_FCG_TRUNC_TYPE_NOTAY :
-        mi = PetscMax(1,i%(fcg->mmax+1));
-        break;
-      case KSP_FCG_TRUNC_TYPE_STANDARD :
+      case KSP_FCD_TRUNC_TYPE_STANDARD:
         mi = fcg->mmax;
         break;
+      case KSP_FCD_TRUNC_TYPE_NOTAY:
+        mi = ((i-1) % fcg->mmax)+1;
+        break;
       default:
-        SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Unrecognized FCG Truncation Strategy");CHKERRQ(ierr);
+        SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Unrecognized Truncation Strategy");
     }
+
+    /* Compute a new column of P (Currently does not support modified G-S or iterative refinement)*/
     ierr = VecCopy(Z,Pcurr);CHKERRQ(ierr);
 
     {
@@ -253,17 +252,15 @@ PetscErrorCode KSPSolve_FCG(KSP ksp)
       } else {
         d[i] = PetscSqrtReal(PetscAbsScalar(beta))*e[i] + 1.0/alpha;
       }
+      fcg->ned = ksp->its-1;
     }
     ++i;
   } while (i<ksp->max_it);
   if (i >= ksp->max_it) ksp->reason = KSP_DIVERGED_ITS;
-  if (eigs) fcg->ned = ksp->its-1;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDestroy_FCG"
-PetscErrorCode KSPDestroy_FCG(KSP ksp)
+static PetscErrorCode KSPDestroy_FCG(KSP ksp)
 {
   PetscErrorCode ierr;
   PetscInt       i;
@@ -290,9 +287,7 @@ PetscErrorCode KSPDestroy_FCG(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPView_FCG"
-PetscErrorCode KSPView_FCG(KSP ksp,PetscViewer viewer)
+static PetscErrorCode KSPView_FCG(KSP ksp,PetscViewer viewer)
 {
   KSP_FCG        *fcg = (KSP_FCG*)ksp->data;
   PetscErrorCode ierr;
@@ -303,22 +298,20 @@ PetscErrorCode KSPView_FCG(KSP ksp,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERSTRING,&isstring);CHKERRQ(ierr);
 
-  if (fcg->truncstrat == KSP_FCG_TRUNC_TYPE_STANDARD) truncstr = "Using standard truncation strategy";
-  else if (fcg->truncstrat == KSP_FCG_TRUNC_TYPE_NOTAY) truncstr = "Using Notay's truncation strategy";
+  if (fcg->truncstrat == KSP_FCD_TRUNC_TYPE_STANDARD) truncstr = "Using standard truncation strategy";
+  else if (fcg->truncstrat == KSP_FCD_TRUNC_TYPE_NOTAY) truncstr = "Using Notay's truncation strategy";
   else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Undefined FCG truncation strategy");
 
   if (iascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  FCG: m_max=%D\n",fcg->mmax);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  FCG: preallocated %D directions\n",PetscMin(fcg->nprealloc,fcg->mmax+1));CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  FCG: %s\n",truncstr);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  m_max=%D\n",fcg->mmax);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  preallocated %D directions\n",PetscMin(fcg->nprealloc,fcg->mmax+1));CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  %s\n",truncstr);CHKERRQ(ierr);
   } else if (isstring) {
     ierr = PetscViewerStringSPrintf(viewer,"m_max %D nprealloc %D %s",fcg->mmax,fcg->nprealloc,truncstr);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPFCGSetMmax"
 /*@
   KSPFCGSetMmax - set the maximum number of previous directions FCG will store for orthogonalization
 
@@ -350,8 +343,6 @@ PetscErrorCode KSPFCGSetMmax(KSP ksp,PetscInt mmax)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPFCGGetMmax"
 /*@
   KSPFCGGetMmax - get the maximum number of previous directions FCG will store
 
@@ -385,8 +376,6 @@ PetscErrorCode KSPFCGGetMmax(KSP ksp,PetscInt *mmax)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPFCGSetNprealloc"
 /*@
   KSPFCGSetNprealloc - set the number of directions to preallocate with FCG
 
@@ -399,7 +388,7 @@ PetscErrorCode KSPFCGGetMmax(KSP ksp,PetscInt *mmax)
   Level: advanced
 
   Options Database:
-. -ksp_fcg_nprealloc <N>
+. -ksp_fcg_nprealloc <N> - number of directions to preallocate
 
 .seealso: KSPFCG, KSPFCGGetTruncationType(), KSPFCGGetNprealloc()
 @*/
@@ -415,8 +404,6 @@ PetscErrorCode KSPFCGSetNprealloc(KSP ksp,PetscInt nprealloc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPFCGGetNprealloc"
 /*@
   KSPFCGGetNprealloc - get the number of directions preallocate by FCG
 
@@ -428,9 +415,6 @@ PetscErrorCode KSPFCGSetNprealloc(KSP ksp,PetscInt nprealloc)
    Output Parameter:
 .  nprealloc - the number of directions preallocated
 
-  Options Database:
-. -ksp_fcg_nprealloc <N>
-
    Level: advanced
 
 .keywords: KSP, FCG, truncation
@@ -447,15 +431,13 @@ PetscErrorCode KSPFCGGetNprealloc(KSP ksp,PetscInt *nprealloc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPFCGSetTruncationType"
 /*@
   KSPFCGSetTruncationType - specify how many of its stored previous directions FCG uses during orthoganalization
 
   Logically Collective on KSP
 
-  KSP_FCG_TRUNC_TYPE_STANDARD uses all (up to mmax) stored directions
-  KSP_FCG_TRUNC_TYPE_NOTAY uses the last max(1,mod(i,mmax)) stored directions at iteration i=0,1..
+  KSP_FCD_TRUNC_TYPE_STANDARD uses all (up to mmax) stored directions
+  KSP_FCD_TRUNC_TYPE_NOTAY uses the last max(1,mod(i,mmax)) stored directions at iteration i=0,1,..
 
   Input Parameters:
 +  ksp - the Krylov space context
@@ -464,11 +446,11 @@ PetscErrorCode KSPFCGGetNprealloc(KSP ksp,PetscInt *nprealloc)
   Level: intermediate
 
   Options Database:
-. -ksp_fcg_truncation_type <standard, notay>
+. -ksp_fcg_truncation_type <standard, notay> - specify how many of its stored previous directions FCG uses during orthoganalization
 
-  .seealso: KSPFCGTruncationType, KSPFCGGetTruncationType
+  .seealso: KSPFCDTruncationType, KSPFCGGetTruncationType
 @*/
-PetscErrorCode KSPFCGSetTruncationType(KSP ksp,KSPFCGTruncationType truncstrat)
+PetscErrorCode KSPFCGSetTruncationType(KSP ksp,KSPFCDTruncationType truncstrat)
 {
   KSP_FCG *fcg=(KSP_FCG*)ksp->data;
 
@@ -479,8 +461,6 @@ PetscErrorCode KSPFCGSetTruncationType(KSP ksp,KSPFCGTruncationType truncstrat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPFCGGetTruncationType"
 /*@
   KSPFCGGetTruncationType - get the truncation strategy employed by FCG
 
@@ -492,16 +472,13 @@ PetscErrorCode KSPFCGSetTruncationType(KSP ksp,KSPFCGTruncationType truncstrat)
    Output Parameter:
 .  truncstrat - the strategy type
 
-  Options Database:
-. -ksp_fcg_truncation_type <standard, notay>
-
    Level: intermediate
 
 .keywords: KSP, FCG, truncation
 
-.seealso: KSPFCG, KSPFCGSetTruncationType, KSPFCGTruncationType
+.seealso: KSPFCG, KSPFCGSetTruncationType, KSPFCDTruncationType
 @*/
-PetscErrorCode KSPFCGGetTruncationType(KSP ksp,KSPFCGTruncationType *truncstrat)
+PetscErrorCode KSPFCGGetTruncationType(KSP ksp,KSPFCDTruncationType *truncstrat)
 {
   KSP_FCG *fcg=(KSP_FCG*)ksp->data;
 
@@ -511,9 +488,7 @@ PetscErrorCode KSPFCGGetTruncationType(KSP ksp,KSPFCGTruncationType *truncstrat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetFromOptions_FCG"
-PetscErrorCode KSPSetFromOptions_FCG(PetscOptions *PetscOptionsObject,KSP ksp)
+static PetscErrorCode KSPSetFromOptions_FCG(PetscOptionItems *PetscOptionsObject,KSP ksp)
 {
   PetscErrorCode ierr;
   KSP_FCG        *fcg=(KSP_FCG*)ksp->data;
@@ -530,7 +505,7 @@ PetscErrorCode KSPSetFromOptions_FCG(PetscOptions *PetscOptionsObject,KSP ksp)
   if (flg) { 
     ierr = KSPFCGSetNprealloc(ksp,nprealloc);CHKERRQ(ierr); 
   }
-  ierr = PetscOptionsEnum("-ksp_fcg_truncation_type","Truncation approach for directions","KSPFCGSetTruncationType",KSPFCGTruncationTypes,(PetscEnum)fcg->truncstrat,(PetscEnum*)&fcg->truncstrat,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsEnum("-ksp_fcg_truncation_type","Truncation approach for directions","KSPFCGSetTruncationType",KSPFCDTruncationTypes,(PetscEnum)fcg->truncstrat,(PetscEnum*)&fcg->truncstrat,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -539,9 +514,9 @@ PetscErrorCode KSPSetFromOptions_FCG(PetscOptions *PetscOptionsObject,KSP ksp)
       KSPFCG - Implements the Flexible Conjugate Gradient method (FCG)
 
   Options Database Keys:
-+   -ksp_fcg_mmax <N>
-.   -ksp_fcg_nprealloc <N>
--   -ksp_fcg_truncation_type <standard,notay>
++   -ksp_fcg_mmax <N>  - maximum number of search directions
+.   -ksp_fcg_nprealloc <N> - number of directions to preallocate
+-   -ksp_fcg_truncation_type <standard,notay> - truncation approach for directions
 
     Contributed by Patrick Sanan
 
@@ -551,16 +526,13 @@ PetscErrorCode KSPSetFromOptions_FCG(PetscOptions *PetscOptionsObject,KSP ksp)
    Level: beginner
 
   References:
-    1) Notay, Y."Flexible Conjugate Gradients", SIAM J. Sci. Comput. 22:4, pp 1444-1460, 2000
-
-    2) Axelsson, O. and Vassilevski, P. S. "A Black Box Generalized Conjugate Gradient Solver with Inner Iterations and Variable-Step Preconditioning",
-    SIAM J. Matrix Anal. Appl. 12:4, pp 625-44, 1991
++    1. - Notay, Y."Flexible Conjugate Gradients", SIAM J. Sci. Comput. 22:4, 2000
+-    2. - Axelsson, O. and Vassilevski, P. S. "A Black Box Generalized Conjugate Gradient Solver with Inner Iterations and Variable step Preconditioning",
+    SIAM J. Matrix Anal. Appl. 12:4, 1991
 
  .seealso : KSPGCR, KSPFGMRES, KSPCG, KSPFCGSetMmax(), KSPFCGGetMmax(), KSPFCGSetNprealloc(), KSPFCGGetNprealloc(), KSPFCGSetTruncationType(), KSPFCGGetTruncationType()
 
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_FCG"
 PETSC_EXTERN PetscErrorCode KSPCreate_FCG(KSP ksp)
 {
   PetscErrorCode ierr;
