@@ -1,5 +1,4 @@
 
-#include <petscksp.h>
 #include <petsc/private/kspimpl.h>
 
 typedef struct {
@@ -15,9 +14,7 @@ typedef struct {
   void *modifypc_ctx;                                        /* user defined data for the modifypc function */
 } KSP_GCR;
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_GCR_cycle"
-PetscErrorCode KSPSolve_GCR_cycle(KSP ksp)
+static PetscErrorCode KSPSolve_GCR_cycle(KSP ksp)
 {
   KSP_GCR        *ctx = (KSP_GCR*)ksp->data;
   PetscErrorCode ierr;
@@ -25,10 +22,12 @@ PetscErrorCode KSPSolve_GCR_cycle(KSP ksp)
   Mat            A, B;
   PC             pc;
   Vec            s,v,r;
-  PetscReal      norm_r,nrm;
+  /*
+     The residual norm will not be computed when ksp->its > ksp->chknorm hence need to initialize norm_r with some dummy value
+  */
+  PetscReal      norm_r = 0.0,nrm;
   PetscInt       k, i, restart;
   Vec            x;
-  PetscReal      res;
 
   PetscFunctionBegin;
   restart = ctx->restart;
@@ -65,14 +64,13 @@ PetscErrorCode KSPSolve_GCR_cycle(KSP ksp)
     }
     /* update the local counter and the global counter */
     ksp->its++;
-    res        = norm_r;
-    ksp->rnorm = res;
+    ksp->rnorm = norm_r;
 
-    ierr = KSPLogResidualHistory(ksp,res);CHKERRQ(ierr);
-    ierr = KSPMonitor(ksp,ksp->its,res);CHKERRQ(ierr);
+    ierr = KSPLogResidualHistory(ksp,norm_r);CHKERRQ(ierr);
+    ierr = KSPMonitor(ksp,ksp->its,norm_r);CHKERRQ(ierr);
 
-    if (ksp->its > ksp->chknorm) {
-      ierr = (*ksp->converged)(ksp,ksp->its,res,&ksp->reason,ksp->cnvP);CHKERRQ(ierr);
+    if (ksp->its-1 > ksp->chknorm) {
+      ierr = (*ksp->converged)(ksp,ksp->its,norm_r,&ksp->reason,ksp->cnvP);CHKERRQ(ierr);
       if (ksp->reason) break;
     }
 
@@ -85,9 +83,7 @@ PetscErrorCode KSPSolve_GCR_cycle(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_GCR"
-PetscErrorCode KSPSolve_GCR(KSP ksp)
+static PetscErrorCode KSPSolve_GCR(KSP ksp)
 {
   KSP_GCR        *ctx = (KSP_GCR*)ksp->data;
   PetscErrorCode ierr;
@@ -123,9 +119,7 @@ PetscErrorCode KSPSolve_GCR(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPView_GCR"
-PetscErrorCode KSPView_GCR(KSP ksp, PetscViewer viewer)
+static PetscErrorCode KSPView_GCR(KSP ksp, PetscViewer viewer)
 {
   KSP_GCR        *ctx = (KSP_GCR*)ksp->data;
   PetscErrorCode ierr;
@@ -134,16 +128,14 @@ PetscErrorCode KSPView_GCR(KSP ksp, PetscViewer viewer)
   PetscFunctionBegin;
   PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);
   if (iascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  GCR: restart = %D \n", ctx->restart);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  GCR: restarts performed = %D \n", ctx->n_restarts);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  restart = %D \n", ctx->restart);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  restarts performed = %D \n", ctx->n_restarts);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_GCR"
-PetscErrorCode KSPSetUp_GCR(KSP ksp)
+static PetscErrorCode KSPSetUp_GCR(KSP ksp)
 {
   KSP_GCR        *ctx = (KSP_GCR*)ksp->data;
   PetscErrorCode ierr;
@@ -163,9 +155,7 @@ PetscErrorCode KSPSetUp_GCR(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPReset_GCR"
-PetscErrorCode KSPReset_GCR(KSP ksp)
+static PetscErrorCode KSPReset_GCR(KSP ksp)
 {
   PetscErrorCode ierr;
   KSP_GCR        *ctx = (KSP_GCR*)ksp->data;
@@ -181,9 +171,7 @@ PetscErrorCode KSPReset_GCR(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDestroy_GCR"
-PetscErrorCode KSPDestroy_GCR(KSP ksp)
+static PetscErrorCode KSPDestroy_GCR(KSP ksp)
 {
   PetscErrorCode ierr;
 
@@ -193,9 +181,7 @@ PetscErrorCode KSPDestroy_GCR(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetFromOptions_GCR"
-PetscErrorCode KSPSetFromOptions_GCR(PetscOptions *PetscOptionsObject,KSP ksp)
+static PetscErrorCode KSPSetFromOptions_GCR(PetscOptionItems *PetscOptionsObject,KSP ksp)
 {
   PetscErrorCode ierr;
   KSP_GCR        *ctx = (KSP_GCR*)ksp->data;
@@ -214,8 +200,6 @@ PetscErrorCode KSPSetFromOptions_GCR(PetscOptions *PetscOptionsObject,KSP ksp)
 typedef PetscErrorCode (*KSPGCRModifyPCFunction)(KSP,PetscInt,PetscReal,void*);
 typedef PetscErrorCode (*KSPGCRDestroyFunction)(void*);
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGCRSetModifyPC_GCR"
 static PetscErrorCode  KSPGCRSetModifyPC_GCR(KSP ksp,KSPGCRModifyPCFunction function,void *data,KSPGCRDestroyFunction destroy)
 {
   KSP_GCR *ctx = (KSP_GCR*)ksp->data;
@@ -228,8 +212,6 @@ static PetscErrorCode  KSPGCRSetModifyPC_GCR(KSP ksp,KSPGCRModifyPCFunction func
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGCRSetModifyPC"
 /*@C
  KSPGCRSetModifyPC - Sets the routine used by GCR to modify the preconditioner.
 
@@ -266,8 +248,6 @@ PetscErrorCode  KSPGCRSetModifyPC(KSP ksp,PetscErrorCode (*function)(KSP,PetscIn
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGCRSetRestart_GCR"
 static PetscErrorCode KSPGCRSetRestart_GCR(KSP ksp,PetscInt restart)
 {
   KSP_GCR *ctx;
@@ -278,8 +258,6 @@ static PetscErrorCode KSPGCRSetRestart_GCR(KSP ksp,PetscInt restart)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGCRSetRestart"
 PetscErrorCode  KSPGCRSetRestart(KSP ksp, PetscInt restart)
 {
   PetscErrorCode ierr;
@@ -289,9 +267,7 @@ PetscErrorCode  KSPGCRSetRestart(KSP ksp, PetscInt restart)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPBuildSolution_GCR"
-PetscErrorCode  KSPBuildSolution_GCR(KSP ksp, Vec v, Vec *V)
+static PetscErrorCode  KSPBuildSolution_GCR(KSP ksp, Vec v, Vec *V)
 {
   PetscErrorCode ierr;
   Vec            x;
@@ -307,9 +283,7 @@ PetscErrorCode  KSPBuildSolution_GCR(KSP ksp, Vec v, Vec *V)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPBuildResidual_GCR"
-PetscErrorCode  KSPBuildResidual_GCR(KSP ksp, Vec t, Vec v, Vec *V)
+static PetscErrorCode  KSPBuildResidual_GCR(KSP ksp, Vec t, Vec v, Vec *V)
 {
   PetscErrorCode ierr;
   KSP_GCR        *ctx;
@@ -328,39 +302,42 @@ PetscErrorCode  KSPBuildResidual_GCR(KSP ksp, Vec t, Vec v, Vec *V)
 /*MC
      KSPGCR - Implements the preconditioned Generalized Conjugate Residual method.
 
-
    Options Database Keys:
-+   -ksp_gcr_restart <restart> - the number of stored vectors to orthogonalize against
+.   -ksp_gcr_restart <restart> - the number of stored vectors to orthogonalize against
 
    Level: beginner
 
-    Notes: The GCR Krylov method supports non-symmetric matrices and permits the use of a preconditioner
+    Notes:
+    The GCR Krylov method supports non-symmetric matrices and permits the use of a preconditioner
            which may vary from one iteration to the next. Users can can define a method to vary the
            preconditioner between iterates via KSPGCRSetModifyPC().
+
            Restarts are solves with x0 not equal to zero. When a restart occurs, the initial starting
            solution is given by the current estimate for x which was obtained by the last restart
            iterations of the GCR algorithm.
+
            Unlike GMRES and FGMRES, when using GCR, the solution and residual vector can be directly accessed at any iterate,
            with zero computational cost, via a call to KSPBuildSolution() and KSPBuildResidual() respectively.
+
            This implementation of GCR will only apply the stopping condition test whenever ksp->its > ksp->chknorm,
            where ksp->chknorm is specified via the command line argument -ksp_check_norm_iteration or via
-           the function KSPSetCheckNormIteration().
+           the function KSPSetCheckNormIteration(). Hence the residual norm reported by the monitor and stored
+           in the residual history will be listed as 0.0 before this iteration. It is actually not 0.0; just not calculated.
+
            The method implemented requires the storage of 2 x restart + 1 vectors, twice as much as GMRES.
            Support only for right preconditioning.
 
     Contributed by Dave May
 
     References:
-           S. C. Eisenstat, H. C. Elman, and H. C. Schultz. Variational iterative methods for
-           non-symmetric systems of linear equations. SIAM J. Numer. Anal., 20, 345-357, 1983
+.          1. - S. C. Eisenstat, H. C. Elman, and H. C. Schultz. Variational iterative methods for
+           nonsymmetric systems of linear equations. SIAM J. Numer. Anal., 20, 1983
 
 
 .seealso:  KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP,
            KSPGCRSetRestart(), KSPGCRSetModifyPC(), KSPGMRES, KSPFGMRES
 
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_GCR"
 PETSC_EXTERN PetscErrorCode KSPCreate_GCR(KSP ksp)
 {
   PetscErrorCode ierr;
