@@ -1,53 +1,43 @@
 
 #define PETSCKSP_DLL
 
-/*
- * This file implements the GMRES method with the Krylov basis built from Newton polynomials [1, 2, 3]. The basis thus generated is called the Newton basis. To reduce the negative effects of restarting, this basis is augmented with harmonic Ritz vectors that approximate the smallest or the largest eigenvalues [4].
- * References :
- * [1] Bai, Zhaojun and  Hu, D. and Reichel, L. A Newton basis GMRES implementation. IMA J. Numer. Anal. 14 (1994), no. 4, 563-581.
- *  [2] Sidje, Roger B. Alternatives for parallel Krylov subspace basis computation. Numer. Linear Algebra Appl. 4 (1997), no. 4, 305-331
- *  [3] Erhel, Jocelyne. A parallel GMRES version for general sparse matrices. Electron. Trans. Numer. Anal. 3 (1995), Dec., 160-176
- *
- *
- * This KSP is fully described in this paper together with many numerical experiments
- *  [4] D. Nuentsa Wakam and J. Erhel, Parallelism and robustness in GMRES with the Newton basis and the deflation of eigenvalues. Research report INRIA RR-7787, November 2011.
- *
- *  NOTE: In this implementation, although the deflation is done either by augmenting the basis (which is the primary goal), the preconditioning deflation defined in KSP DGMRES can be used as well with this KSP (database option -ksp_agmres_DeflPrecond, this is an  experimental option).
- *
- */
-
 #include <../src/ksp/ksp/impls/gmres/agmres/agmresimpl.h>
 
 #define AGMRES_DEFAULT_MAXK 30
 #define AGMRES_DELTA_DIRECTIONS 10
 static PetscErrorCode KSPAGMRESBuildSoln(KSP,PetscInt);
-static PetscErrorCode KSPComputeShifts_GMRES(KSP);
 static PetscErrorCode KSPAGMRESBuildBasis(KSP);
 static PetscErrorCode KSPAGMRESBuildHessenberg(KSP);
 
 PetscLogEvent KSP_AGMRESComputeDeflationData, KSP_AGMRESBuildBasis, KSP_AGMRESComputeShifts, KSP_AGMRESRoddec;
 
+extern PetscErrorCode KSPSetUp_DGMRES(KSP);
+extern PetscErrorCode KSPBuildSolution_DGMRES(KSP,Vec,Vec*);
+extern PetscErrorCode KSPSolve_DGMRES(KSP);
+extern PetscErrorCode KSPDGMRESComputeDeflationData_DGMRES(KSP,PetscInt*);
+extern PetscErrorCode KSPDGMRESComputeSchurForm_DGMRES(KSP,PetscInt*);
+extern PetscErrorCode KSPDGMRESApplyDeflation_DGMRES(KSP,Vec,Vec);
+extern PetscErrorCode KSPDestroy_DGMRES(KSP);
+extern PetscErrorCode KSPSetFromOptions_DGMRES(PetscOptionItems *,KSP);
+extern PetscErrorCode KSPDGMRESSetEigen_DGMRES(KSP,PetscInt);
 /*
  * This function allocates  data for the Newton basis GMRES implementation.
  * Note that most data are allocated in KSPSetUp_DGMRES and KSPSetUp_GMRES, including the space for the basis vectors, the various Hessenberg matrices and the Givens rotations coefficients
  *
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_AGMRES"
-PetscErrorCode    KSPSetUp_AGMRES(KSP ksp)
+static PetscErrorCode    KSPSetUp_AGMRES(KSP ksp)
 {
-  PetscErrorCode ierr;
-  PetscInt       hes;
-  PetscInt       nloc;
-  KSP_AGMRES     *agmres = (KSP_AGMRES*)ksp->data;
-  PetscInt       neig    = agmres->neig;
-  PetscInt       max_k   = agmres->max_k;
-  PetscInt       N       = MAXKSPSIZE;
-  PetscInt       lwork   = PetscMax(8 * N + 16, 4 * neig * (N - neig));
+  PetscErrorCode  ierr;
+  PetscInt        hes;
+  PetscInt        nloc;
+  KSP_AGMRES      *agmres = (KSP_AGMRES*)ksp->data;
+  PetscInt        neig    = agmres->neig;
+  const PetscInt  max_k   = agmres->max_k;
+  PetscInt        N       = MAXKSPSIZE;
+  PetscInt        lwork   = PetscMax(8 * N + 16, 4 * neig * (N - neig));
 
   PetscFunctionBegin;
   if (ksp->pc_side == PC_SYMMETRIC) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP,"no symmetric preconditioning for KSPAGMRES");
-  max_k = agmres->max_k;
   N     = MAXKSPSIZE;
   /* Preallocate space during the call to KSPSetup_GMRES for the Krylov basis */
   agmres->q_preallocate = PETSC_TRUE; /* No allocation on the fly */
@@ -89,9 +79,7 @@ PetscErrorCode    KSPSetUp_AGMRES(KSP ksp)
  * This function is provided to be compliant with the KSP GMRES  scheme.
  *
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPBuildSolution_AGMRES"
-PetscErrorCode KSPBuildSolution_AGMRES(KSP ksp,Vec ptr, Vec *result)
+static PetscErrorCode KSPBuildSolution_AGMRES(KSP ksp,Vec ptr, Vec *result)
 {
   KSP_AGMRES     *agmres = (KSP_AGMRES*)ksp->data;
   PetscErrorCode ierr;
@@ -116,20 +104,17 @@ PetscErrorCode KSPBuildSolution_AGMRES(KSP ksp,Vec ptr, Vec *result)
  * One cycle of GMRES with the Arnoldi process is performed and the eigenvalues of the induced Hessenberg matrix (the Ritz values) are computed.
  * NOTE: This function is not currently used; the next function is rather used when  the eigenvectors are needed next to augment the basis
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPComputeShifts_GMRES"
-static PetscErrorCode KSPComputeShifts_GMRES(KSP ksp)
+PetscErrorCode KSPComputeShifts_GMRES(KSP ksp)
 {
-  PetscErrorCode ierr;
-  KSP_AGMRES     *agmres = (KSP_AGMRES*)(ksp->data);
-  KSP            kspgmres;
-  Mat            Amat, Pmat;
-  MatStructure   flag;
-  PetscInt       max_k = agmres->max_k;
-  PC             pc;
-  PetscInt       m;
-  PetscScalar    *Rshift, *Ishift;
-
+  PetscErrorCode  ierr;
+  KSP_AGMRES      *agmres = (KSP_AGMRES*)(ksp->data);
+  KSP             kspgmres;
+  Mat             Amat, Pmat;
+  const PetscInt  max_k = agmres->max_k;
+  PC              pc;
+  PetscInt        m;
+  PetscScalar     *Rshift, *Ishift;
+  PetscBool       flg;
 
   PetscFunctionBegin;
   /* Perform one cycle of classical GMRES (with the Arnoldi process) to get the Hessenberg matrix
@@ -140,8 +125,8 @@ static PetscErrorCode KSPComputeShifts_GMRES(KSP ksp)
   ierr = PCGetOperators(ksp->pc, &Amat, &Pmat);CHKERRQ(ierr);
   ierr = KSPSetOperators(kspgmres, Amat, Pmat);CHKERRQ(ierr);
   ierr = KSPSetFromOptions(kspgmres);CHKERRQ(ierr);
-  ierr = PetscOptionsHasName(NULL, "-ksp_view", &flg);CHKERRQ(ierr);
-  if (flag) { ierr = PetscOptionsClearValue("-ksp_view");CHKERRQ(ierr); }
+  ierr = PetscOptionsHasName(NULL,((PetscObject)ksp)->prefix, "-ksp_view", &flg);CHKERRQ(ierr);
+  if (flg) { ierr = PetscOptionsClearValue(NULL,"-ksp_view");CHKERRQ(ierr); }
   ierr = KSPSetType(kspgmres, KSPGMRES);CHKERRQ(ierr);
   ierr = KSPGMRESSetRestart(kspgmres, max_k);CHKERRQ(ierr);
   ierr = KSPGetPC(ksp, &pc);CHKERRQ(ierr);
@@ -174,7 +159,7 @@ static PetscErrorCode KSPComputeShifts_GMRES(KSP ksp)
     agmres->HasShifts = PETSC_TRUE;
   }
   /* Restore KSP view options */
-  if (flg) { ierr = PetscOptionsSetValue("-ksp_view", "");CHKERRQ(ierr); }
+  if (flg) { ierr = PetscOptionsSetValue(NULL,"-ksp_view", "");CHKERRQ(ierr); }
   PetscFunctionReturn(0);
 }
 
@@ -189,15 +174,14 @@ static PetscErrorCode KSPComputeShifts_GMRES(KSP ksp)
  *  - The shifts as complex pair of arrays in wr and wi (size max_k).
  *  - The harmonic Ritz vectors (agmres->U) if deflation is needed.
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPComputeShifts_DGMRES"
-PetscErrorCode KSPComputeShifts_DGMRES(KSP ksp)
+static PetscErrorCode KSPComputeShifts_DGMRES(KSP ksp)
 {
   PetscErrorCode ierr;
   KSP_AGMRES     *agmres = (KSP_AGMRES*)(ksp->data);
   PetscInt       max_k   = agmres->max_k; /* size of the (non augmented) Krylov subspace */
   PetscInt       Neig    = 0;
-  PetscInt       max_it  = ksp->max_it;
+  const PetscInt max_it  = ksp->max_it;
+  PetscBool      flg;
 
   /* Perform one cycle of dgmres to find the eigenvalues and compute the first approximations of the eigenvectors */
 
@@ -206,7 +190,7 @@ PetscErrorCode KSPComputeShifts_DGMRES(KSP ksp)
   /* Send the size of the augmented basis to DGMRES */
   ksp->max_it             = max_k; /* set this to have DGMRES performing only one cycle */
   ksp->ops->buildsolution = KSPBuildSolution_DGMRES;
-  ierr                    = KSPSolve_DGMRES(ksp);
+  ierr                    = KSPSolve_DGMRES(ksp);CHKERRQ(ierr);
   ksp->guess_zero         = PETSC_FALSE;
   if (ksp->reason == KSP_CONVERGED_RTOL) {
     ierr = PetscLogEventEnd(KSP_AGMRESComputeShifts, ksp, 0,0,0);CHKERRQ(ierr);
@@ -222,8 +206,7 @@ PetscErrorCode KSPComputeShifts_DGMRES(KSP ksp)
   }
 
   /* It may happen that the Ritz values from one cycle of GMRES are not accurate enough to provide a good stability. In this case, another cycle of GMRES is performed.  The two sets of values thus generated are sorted and the most accurate are kept as shifts */
-  PetscBool flg;
-  ierr = PetscOptionsHasName(NULL, "-ksp_agmres_ImproveShifts", &flg);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL, "-ksp_agmres_ImproveShifts", &flg);CHKERRQ(ierr);
   if (!flg) {
     ierr = KSPAGMRESLejaOrdering(agmres->wr, agmres->wi, agmres->Rshift, agmres->Ishift, max_k);CHKERRQ(ierr);
   } else { /* Perform another cycle of DGMRES to find another set of eigenvalues */
@@ -235,7 +218,7 @@ PetscErrorCode KSPComputeShifts_DGMRES(KSP ksp)
       wi[i] = agmres->wi[i];
     }
 
-    ierr = KSPSolve_DGMRES(ksp);
+    ierr = KSPSolve_DGMRES(ksp);CHKERRQ(ierr);
 
     ksp->guess_zero = PETSC_FALSE;
     if (ksp->reason == KSP_CONVERGED_RTOL) PetscFunctionReturn(0);
@@ -279,8 +262,6 @@ PetscErrorCode KSPComputeShifts_DGMRES(KSP ksp)
  *  - agmres->vecs or VEC_V : basis vectors
  *  - agmres->Scale : Scaling factors (equal to 1 if no scaling is done)
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPAGMRESBuildBasis"
 static PetscErrorCode KSPAGMRESBuildBasis(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -288,7 +269,7 @@ static PetscErrorCode KSPAGMRESBuildBasis(KSP ksp)
   PetscReal      *Rshift = agmres->Rshift;
   PetscReal      *Ishift = agmres->Ishift;
   PetscReal      *Scale  = agmres->Scale;
-  PetscInt       max_k   = agmres->max_k;
+  const PetscInt max_k   = agmres->max_k;
   PetscInt       KspSize = KSPSIZE;  /* if max_k == KspSizen then the basis should not be augmented */
   PetscInt       j       = 1;
 
@@ -391,8 +372,6 @@ static PetscErrorCode KSPAGMRESBuildBasis(KSP ksp)
  * NOTE: Note that the computed Hessenberg matrix is not mathematically equivalent to that in the real Arnoldi process (in KSP GMRES). If it is needed, it can be explicitly  formed as H <-- H * RLoc^-1.
  *
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPAGMRESBuildHessenberg"
 static PetscErrorCode KSPAGMRESBuildHessenberg(KSP ksp)
 {
   KSP_AGMRES     *agmres = (KSP_AGMRES*)ksp->data;
@@ -401,7 +380,7 @@ static PetscErrorCode KSPAGMRESBuildHessenberg(KSP ksp)
   PetscScalar    *Scale  = agmres->Scale;
   PetscErrorCode ierr;
   PetscInt       i       = 0, j = 0;
-  PetscInt       max_k   = agmres->max_k;
+  const PetscInt max_k   = agmres->max_k;
   PetscInt       KspSize = KSPSIZE;
   PetscInt       N       = MAXKSPSIZE+1;
 
@@ -440,19 +419,17 @@ static PetscErrorCode KSPAGMRESBuildHessenberg(KSP ksp)
  *
  */
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPAGMRESBuildSoln"
 static PetscErrorCode KSPAGMRESBuildSoln(KSP ksp,PetscInt it)
 {
   KSP_AGMRES     *agmres = (KSP_AGMRES*)ksp->data;
   PetscErrorCode ierr;
-  PetscInt       max_k = agmres->max_k;       /* Size of the non-augmented Krylov basis */
+  const PetscInt max_k = agmres->max_k;       /* Size of the non-augmented Krylov basis */
   PetscInt       i, j;
   PetscInt       r = agmres->r;           /* current number of augmented eigenvectors */
   PetscBLASInt   KspSize;
   PetscBLASInt   lC;
   PetscBLASInt   N;
-  PetscBLASInt   ldH = N + 1;
+  PetscBLASInt   ldH = it + 1;
   PetscBLASInt   lwork;
   PetscBLASInt   info, nrhs = 1;
 
@@ -521,8 +498,6 @@ static PetscErrorCode KSPAGMRESBuildSoln(KSP ksp,PetscInt it)
  .
  NOTE: Unlike GMRES where the residual norm is available at each (inner) iteration,  here it is available at the end of the cycle.
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPAGMRESCycle"
 static PetscErrorCode KSPAGMRESCycle(PetscInt *itcount,KSP ksp)
 {
   KSP_AGMRES     *agmres = (KSP_AGMRES*)(ksp->data);
@@ -563,9 +538,7 @@ static PetscErrorCode KSPAGMRESCycle(PetscInt *itcount,KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_AGMRES"
-PetscErrorCode KSPSolve_AGMRES(KSP ksp)
+static PetscErrorCode KSPSolve_AGMRES(KSP ksp)
 {
   PetscErrorCode ierr;
   PetscInt       its;
@@ -594,6 +567,7 @@ PetscErrorCode KSPSolve_AGMRES(KSP ksp)
       agmres->matvecs += 1;
     }
     ierr    = VecNormalize(VEC_V(0),&(ksp->rnorm));CHKERRQ(ierr);
+    KSPCheckNorm(ksp,ksp->rnorm);
     res_old = ksp->rnorm; /* Record the residual norm to test if deflation is needed */
 
     ksp->ops->buildsolution = KSPBuildSolution_AGMRES;
@@ -620,9 +594,7 @@ PetscErrorCode KSPSolve_AGMRES(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDestroy_AGMRES"
-PetscErrorCode KSPDestroy_AGMRES(KSP ksp)
+static PetscErrorCode KSPDestroy_AGMRES(KSP ksp)
 {
   PetscErrorCode ierr;
   KSP_AGMRES     *agmres = (KSP_AGMRES*)ksp->data;
@@ -653,14 +625,12 @@ PetscErrorCode KSPDestroy_AGMRES(KSP ksp)
     ierr = PetscFree(agmres->Z);CHKERRQ(ierr);
     ierr = PetscFree(agmres->beta);CHKERRQ(ierr);
   }
-  ierr = KSPDestroy_DGMRES(ksp);
+  ierr = KSPDestroy_DGMRES(ksp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPView_AGMRES"
-PetscErrorCode KSPView_AGMRES(KSP ksp,PetscViewer viewer)
+static PetscErrorCode KSPView_AGMRES(KSP ksp,PetscViewer viewer)
 {
   KSP_AGMRES     *agmres = (KSP_AGMRES*)ksp->data;
   const char     *cstr   = "RODDEC ORTHOGONOLIZATION";
@@ -678,33 +648,31 @@ PetscErrorCode KSPView_AGMRES(KSP ksp,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERSTRING,&isstring);CHKERRQ(ierr);
 
   if (iascii) {
-    ierr = PetscViewerASCIIPrintf(viewer, " AGMRES : restart=%d using %s\n", agmres->max_k, cstr);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer, " AGMRES : %s\n", Nstr);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer, " AGMRES: Number of matvecs : %D\n", agmres->matvecs);CHKERRQ(ierr);
-    if (agmres->force) PetscViewerASCIIPrintf (viewer, " AGMRES: Adaptive strategy is used: FALSE\n");
-    else PetscViewerASCIIPrintf(viewer, " AGMRES: Adaptive strategy is used: TRUE\n");
+    ierr = PetscViewerASCIIPrintf(viewer, " restart=%d using %s\n", agmres->max_k, cstr);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer, " %s\n", Nstr);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer, " Number of matvecs : %D\n", agmres->matvecs);CHKERRQ(ierr);
+    if (agmres->force) {ierr = PetscViewerASCIIPrintf (viewer, " Adaptive strategy is used: FALSE\n");CHKERRQ(ierr);}
+    else PetscViewerASCIIPrintf(viewer, " Adaptive strategy is used: TRUE\n");
     if (agmres->DeflPrecond) {
-      ierr=PetscViewerASCIIPrintf(viewer, " AGMRES: STRATEGY OF DEFLATION: PRECONDITIONER \n");CHKERRQ(ierr);
-      ierr=PetscViewerASCIIPrintf(viewer, "  AGMRES: Frequency of extracted eigenvalues = %D\n", agmres->neig);CHKERRQ(ierr);
-      ierr=PetscViewerASCIIPrintf(viewer, "  AGMRES: Total number of extracted eigenvalues = %D\n", agmres->r);CHKERRQ(ierr);
-      ierr=PetscViewerASCIIPrintf(viewer, "  AGMRES: Maximum number of eigenvalues set to be extracted = %D\n", agmres->max_neig);CHKERRQ(ierr);
+      ierr=PetscViewerASCIIPrintf(viewer, " STRATEGY OF DEFLATION: PRECONDITIONER \n");CHKERRQ(ierr);
+      ierr=PetscViewerASCIIPrintf(viewer, "  Frequency of extracted eigenvalues = %D\n", agmres->neig);CHKERRQ(ierr);
+      ierr=PetscViewerASCIIPrintf(viewer, "  Total number of extracted eigenvalues = %D\n", agmres->r);CHKERRQ(ierr);
+      ierr=PetscViewerASCIIPrintf(viewer, "  Maximum number of eigenvalues set to be extracted = %D\n", agmres->max_neig);CHKERRQ(ierr);
     } else {
       if (agmres->ritz) sprintf(ritzvec, "Ritz vectors");
       else sprintf(ritzvec, "Harmonic Ritz vectors");
-      ierr = PetscViewerASCIIPrintf(viewer, " AGMRES: STRATEGY OF DEFLATION: AUGMENT\n");CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPrintf(viewer," AGMRES: augmented vectors  %d at frequency %d with %s\n", agmres->r, agmres->neig, ritzvec);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer, " STRATEGY OF DEFLATION: AUGMENT\n");CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer," augmented vectors  %d at frequency %d with %s\n", agmres->r, agmres->neig, ritzvec);CHKERRQ(ierr);
     }
-    ierr=PetscViewerASCIIPrintf(viewer, " AGMRES: Minimum relaxation parameter for the adaptive strategy(smv)  = %g\n", agmres->smv);
-    ierr=PetscViewerASCIIPrintf(viewer, " AGMRES: Maximum relaxation parameter for the adaptive strategy(bgv)  = %g\n", agmres->bgv);
+    ierr=PetscViewerASCIIPrintf(viewer, " Minimum relaxation parameter for the adaptive strategy(smv)  = %g\n", agmres->smv);CHKERRQ(ierr);
+    ierr=PetscViewerASCIIPrintf(viewer, " Maximum relaxation parameter for the adaptive strategy(bgv)  = %g\n", agmres->bgv);CHKERRQ(ierr);
   } else if (isstring) {
     ierr = PetscViewerStringSPrintf(viewer,"%s restart %D",cstr,agmres->max_k);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetFromOptions_AGMRES"
-PetscErrorCode KSPSetFromOptions_AGMRES(PetscOptions *PetscOptionsObject,KSP ksp)
+static PetscErrorCode KSPSetFromOptions_AGMRES(PetscOptionItems *PetscOptionsObject,KSP ksp)
 {
   PetscErrorCode ierr;
   PetscInt       neig;
@@ -712,7 +680,7 @@ PetscErrorCode KSPSetFromOptions_AGMRES(PetscOptions *PetscOptionsObject,KSP ksp
   PetscBool      flg;
 
   PetscFunctionBegin;
-  ierr = KSPSetFromOptions_DGMRES(PetscOptionsObject,ksp);  /* Set common options from DGMRES and GMRES */
+  ierr = KSPSetFromOptions_DGMRES(PetscOptionsObject,ksp);CHKERRQ(ierr);  /* Set common options from DGMRES and GMRES */
   ierr = PetscOptionsHead(PetscOptionsObject,"KSP AGMRES Options");CHKERRQ(ierr);
   ierr = PetscOptionsInt("-ksp_agmres_eigen", "Number of eigenvalues to deflate", "KSPDGMRESSetEigen", agmres->neig, &neig, &flg);CHKERRQ(ierr);
   if (flg) {
@@ -724,7 +692,7 @@ PetscErrorCode KSPSetFromOptions_AGMRES(PetscOptions *PetscOptionsObject,KSP ksp
   else agmres->max_neig = agmres->neig+EIG_OFFSET;
   ierr                = PetscOptionsBool("-ksp_agmres_DeflPrecond", "Determine if the deflation should be applied as a preconditioner -- similar to KSP DGMRES", "KSPGMRESDeflPrecond",agmres->DeflPrecond,&agmres->DeflPrecond,NULL);CHKERRQ(ierr);
   ierr                = PetscOptionsBool("-ksp_agmres_ritz", "Compute the Ritz vectors instead of the Harmonic Ritz vectors ", "KSPGMRESHarmonic",agmres->ritz,&agmres->ritz ,&flg);CHKERRQ(ierr);
-  ierr                = PetscOptionsReal("-ksp_agmres_MinRatio", "Relaxation parameter in the adaptive strategy; smallest multiple of the remaining number of steps allowed", "KSPGMRESSetMinRatio", agrmes->smv, &agmres->smv, NULL);CHKERRQ(ierr);
+  ierr                = PetscOptionsReal("-ksp_agmres_MinRatio", "Relaxation parameter in the adaptive strategy; smallest multiple of the remaining number of steps allowed", "KSPGMRESSetMinRatio", agmres->smv, &agmres->smv, NULL);CHKERRQ(ierr);
   ierr = PetscOptionsReal("-ksp_agmres_MaxRatio", "Relaxation parameter in the adaptive strategy; Largest multiple of the remaining number of steps allowed", "KSPGMRESSetMaxRatio",agmres->bgv,&agmres->bgv, &flg);CHKERRQ(ierr);
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -751,9 +719,11 @@ There are  many ongoing work that aim at avoiding (or minimizing) the communicat
 
  Level: beginner
 
- Notes : Left and right preconditioning are supported, but not symmetric preconditioning. Complex arithmetic is not supported
+ Notes:
+    Left and right preconditioning are supported, but not symmetric preconditioning. Complex arithmetic is not supported
 
- Developer Notes: This object is subclassed off of KSPDGMRES
+ Developer Notes:
+    This object is subclassed off of KSPDGMRES
 
  Contributed by Desire NUENTSA WAKAM, INRIA <desire.nuentsa_wakam@inria.fr>
  Inputs from Guy Atenekeng <atenekeng@yahoo.com> and R.B. Sidje <roger.b.sidje@ua.edu>
@@ -773,8 +743,6 @@ Mathematics, 62(9), pp. 1171-1186, 2012
  KSPGMRESCGSRefinementType, KSPGMRESSetCGSRefinementType(), KSPGMRESGetCGSRefinementType(), KSPGMRESMonitorKrylov(), KSPSetPCSide()
  M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_AGMRES"
 PETSC_EXTERN PetscErrorCode KSPCreate_AGMRES(KSP ksp)
 {
   KSP_AGMRES     *agmres;
@@ -784,10 +752,8 @@ PETSC_EXTERN PetscErrorCode KSPCreate_AGMRES(KSP ksp)
   ierr      = PetscNewLog(ksp,&agmres);CHKERRQ(ierr);
   ksp->data = (void*)agmres;
 
-  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_PRECONDITIONED,PC_LEFT,3);
-  CHKERRQ(ierr);
-  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_UNPRECONDITIONED,PC_RIGHT,2);
-  CHKERRQ(ierr);
+  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_PRECONDITIONED,PC_LEFT,3);CHKERRQ(ierr);
+  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_UNPRECONDITIONED,PC_RIGHT,2);CHKERRQ(ierr);
   ksp->ops->buildsolution                = KSPBuildSolution_AGMRES;
   ksp->ops->setup                        = KSPSetUp_AGMRES;
   ksp->ops->solve                        = KSPSolve_AGMRES;
@@ -810,11 +776,10 @@ PETSC_EXTERN PetscErrorCode KSPCreate_AGMRES(KSP ksp)
   ierr=PetscObjectComposeFunction((PetscObject) ksp, "KSPDGMRESComputeDeflationData_C",KSPDGMRESComputeDeflationData_DGMRES);CHKERRQ(ierr);
   ierr=PetscObjectComposeFunction((PetscObject) ksp, "KSPDGMRESApplyDeflation_C",KSPDGMRESApplyDeflation_DGMRES);CHKERRQ(ierr);
 
-  PetscInt KSP_CLASSID = 1;
-  ierr = PetscLogEventRegister("AGMRESComputeDefl", KSP_CLASSID, &KSP_AGMRESComputeDeflationData);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("AGMRESCompDefl",   KSP_CLASSID, &KSP_AGMRESComputeDeflationData);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("AGMRESBuildBasis", KSP_CLASSID, &KSP_AGMRESBuildBasis);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("AGMRESCompShifts", KSP_CLASSID, &KSP_AGMRESComputeShifts);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("AGMRESOrthog", KSP_CLASSID, &KSP_AGMRESRoddec);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("AGMRESOrthog",     KSP_CLASSID, &KSP_AGMRESRoddec);CHKERRQ(ierr);
 
   agmres->haptol         = 1.0e-30;
   agmres->q_preallocate  = 0;
@@ -834,12 +799,16 @@ PETSC_EXTERN PetscErrorCode KSPCreate_AGMRES(KSP ksp)
   agmres->lambdaN     = 0.0;
   agmres->smv         = SMV;
   agmres->bgv         = 1;
-  agmres->force       = 0;
+  agmres->force       = PETSC_FALSE;
   agmres->matvecs     = 0;
-  agmres->improve     = 0;
+  agmres->improve     = PETSC_FALSE;
   agmres->HasShifts   = PETSC_FALSE;
   agmres->r           = 0;
   agmres->HasSchur    = PETSC_FALSE;
   agmres->DeflPrecond = PETSC_FALSE;
+  ierr = PetscObjectGetNewTag((PetscObject)ksp,&agmres->tag);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
+/*  LocalWords:  iascii
+ */
