@@ -1,6 +1,6 @@
 #define PETSCKSP_DLL
 
-#include <agmresimpl.h>
+#include <../src/ksp/ksp/impls/gmres/agmres/agmresimpl.h>
 /*
  *  This file implements the RODDEC algorithm : its purpose is to orthogonalize a set of vectors distributed across several processes. These processes are organized in a virtual ring.
  * References : [1] Sidje, Roger B. Alternatives for parallel Krylov subspace basis computation. Numer. Linear Algebra Appl. 4 (1997), no. 4, 305-331
@@ -18,8 +18,6 @@
  */
 static PetscErrorCode  KSPAGMRESRoddecGivens(PetscReal*, PetscReal*, PetscReal*, PetscInt);
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPAGMRESRoddecInitNeighboor"
 PetscErrorCode KSPAGMRESRoddecInitNeighboor(KSP ksp)
 {
   MPI_Comm       comm;
@@ -31,8 +29,8 @@ PetscErrorCode KSPAGMRESRoddecInitNeighboor(KSP ksp)
   ierr = PetscObjectGetComm((PetscObject)agmres->vecs[0], &comm);CHKERRQ(ierr);
   ierr = MPI_Comm_rank(comm, &rank);CHKERRQ(ierr);
   ierr = MPI_Comm_size(comm, &size);CHKERRQ(ierr);
-  ierr = MPI_Allreduce(&rank, &First, 1, MPIU_INT, MPI_MIN, comm);CHKERRQ(ierr);
-  ierr = MPI_Allreduce(&rank, &Last, 1, MPIU_INT, MPI_MAX, comm);CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(&rank, &First, 1, MPIU_INT, MPI_MIN, comm);CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(&rank, &Last, 1, MPIU_INT, MPI_MAX, comm);CHKERRQ(ierr);
 
   if ((rank != Last) && (!rank)) {
     agmres->Ileft  = rank - 1;
@@ -55,8 +53,6 @@ PetscErrorCode KSPAGMRESRoddecInitNeighboor(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPAGMRESRoddecGivens"
 static PetscErrorCode  KSPAGMRESRoddecGivens(PetscReal * c, PetscReal * s, PetscReal * r, PetscInt make_r)
 {
   PetscReal a, b, t;
@@ -117,8 +113,6 @@ static PetscErrorCode  KSPAGMRESRoddecGivens(PetscReal * c, PetscReal * s, Petsc
  *  - agmres->tloc : scalar factors of the elementary reflectors.
 
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPAGMRESRoddec"
 PetscErrorCode KSPAGMRESRoddec(KSP ksp, PetscInt nvec)
 {
   KSP_AGMRES     *agmres = (KSP_AGMRES*) ksp->data;
@@ -131,9 +125,9 @@ PetscErrorCode KSPAGMRESRoddec(KSP ksp, PetscInt nvec)
   PetscMPIInt    rank     = agmres->rank;
   PetscMPIInt    First    = agmres->First;
   PetscMPIInt    Last     = agmres->Last;
-  PetscBLASInt   nloc,pas,len;
-  PetscInt       d, i, j, k;
-  PetscInt       pos,tag;
+  PetscBLASInt   pas,len,bnloc,bpos;
+  PetscInt       nloc,d, i, j, k;
+  PetscInt       pos;
   PetscReal      c, s, rho, Ajj, val, tt, old;
   PetscScalar    *col;
   MPI_Status     status;
@@ -142,19 +136,18 @@ PetscErrorCode KSPAGMRESRoddec(KSP ksp, PetscInt nvec)
 
   PetscFunctionBegin;
   ierr = PetscObjectGetComm((PetscObject)ksp,&comm);CHKERRQ(ierr);
-  tag  = 0x666;
   ierr = PetscLogEventBegin(KSP_AGMRESRoddec,ksp,0,0,0);CHKERRQ(ierr);
   ierr = PetscMemzero(agmres->Rloc, N*N*sizeof(PetscScalar));CHKERRQ(ierr);
   /* check input arguments */
   if (nvec < 1) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_ARG_OUTOFRANGE, "The number of input vectors shoud be positive");
   ierr = VecGetLocalSize(VEC_V(0), &nloc);CHKERRQ(ierr);
+  ierr = PetscBLASIntCast(nloc,&bnloc);CHKERRQ(ierr);
   if (nvec > nloc) SETERRQ(PetscObjectComm((PetscObject)ksp), PETSC_ERR_ARG_WRONG, "In QR factorization, the number of local rows should be greater or equal to the number of columns");
   pas = 1;
-  k   = 0;
   /* Copy the vectors of the basis */
   for (j = 0; j < nvec; j++) {
     ierr = VecGetArray(VEC_V(j), &col);CHKERRQ(ierr);
-    PetscStackCallBLAS("BLAScopy",BLAScopy_(&nloc, col, &pas, &Qloc[j*nloc], &pas));
+    PetscStackCallBLAS("BLAScopy",BLAScopy_(&bnloc, col, &pas, &Qloc[j*nloc], &pas));
     ierr = VecRestoreArray(VEC_V(j), &col);CHKERRQ(ierr);
   }
   /* Each process performs a local QR on its own block */
@@ -177,19 +170,19 @@ PetscErrorCode KSPAGMRESRoddec(KSP ksp, PetscInt nvec)
       Qloc[j*nloc+j] = rho;
     }
   }
-  /*annihilate undesirable Rloc, diagonal by diagonal*/
+  /* annihilate undesirable Rloc, diagonal by diagonal*/
   for (d = 0; d < nvec; d++) {
     len = nvec - d;
     if (rank == First) {
-      PetscStackCallBLAS("BLAScopy",BLAScopy_(&len, &(Qloc[d*nloc+d]), &nloc, &(wbufptr[d]), &pas));
-      ierr = MPI_Send(&(wbufptr[d]), len, MPIU_SCALAR, rank + 1, tag, comm);CHKERRQ(ierr);
+      PetscStackCallBLAS("BLAScopy",BLAScopy_(&len, &(Qloc[d*nloc+d]), &bnloc, &(wbufptr[d]), &pas));
+      ierr = MPI_Send(&(wbufptr[d]), len, MPIU_SCALAR, rank + 1, agmres->tag, comm);CHKERRQ(ierr);
     } else {
-      ierr = MPI_Recv(&(wbufptr[d]), len, MPIU_SCALAR, rank - 1, tag, comm, &status);CHKERRQ(ierr);
-      /*Elimination of Rloc(1,d)*/
+      ierr = MPI_Recv(&(wbufptr[d]), len, MPIU_SCALAR, rank - 1, agmres->tag, comm, &status);CHKERRQ(ierr);
+      /* Elimination of Rloc(1,d)*/
       c    = wbufptr[d];
       s    = Qloc[d*nloc];
-      ierr = KSPAGMRESRoddecGivens(&c, &s, &rho, 1);
-      /*Apply Givens Rotation*/
+      ierr = KSPAGMRESRoddecGivens(&c, &s, &rho, 1);CHKERRQ(ierr);
+      /* Apply Givens Rotation*/
       for (k = d; k < nvec; k++) {
         old          = wbufptr[k];
         wbufptr[k]   =  c * old - s * Qloc[k*nloc];
@@ -197,7 +190,7 @@ PetscErrorCode KSPAGMRESRoddec(KSP ksp, PetscInt nvec)
       }
       Qloc[d*nloc] = rho;
       if (rank != Last) {
-        ierr = MPI_Send(& (wbufptr[d]), len, MPIU_SCALAR, rank + 1, tag, comm);CHKERRQ(ierr);
+        ierr = MPI_Send(& (wbufptr[d]), len, MPIU_SCALAR, rank + 1, agmres->tag, comm);CHKERRQ(ierr);
       }
       /* zero-out the d-th diagonal of Rloc ...*/
       for (j = d + 1; j < nvec; j++) {
@@ -223,11 +216,12 @@ PetscErrorCode KSPAGMRESRoddec(KSP ksp, PetscInt nvec)
   if (rank == Last) {
     for (d = 0; d < nvec; d++) {
       pos    = nvec - d;
+      ierr = PetscBLASIntCast(pos,&bpos);CHKERRQ(ierr);
       sgn[d] = PetscSign(*RLOC(d,d));
-      PetscStackCallBLAS("BLASscal",BLASscal_(&pos, &(sgn[d]), RLOC(d,d), &N));
+      PetscStackCallBLAS("BLASscal",BLASscal_(&bpos, &(sgn[d]), RLOC(d,d), &N));
     }
   }
-  /*BroadCast Rloc to all other processes
+  /* BroadCast Rloc to all other processes
    * NWD : should not be needed
    */
   ierr = MPI_Bcast(agmres->Rloc,N*N,MPIU_SCALAR,Last,comm);CHKERRQ(ierr);
@@ -244,8 +238,6 @@ PetscErrorCode KSPAGMRESRoddec(KSP ksp, PetscInt nvec)
  *  Output :
  *   - Out : Petsc vector (distributed as the basis vectors)
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPAGMRESRodvec"
 PetscErrorCode KSPAGMRESRodvec(KSP ksp, PetscInt nvec, PetscScalar *In, Vec Out)
 {
   KSP_AGMRES     *agmres  = (KSP_AGMRES*) ksp->data;
@@ -258,14 +250,15 @@ PetscErrorCode KSPAGMRESRodvec(KSP ksp, PetscInt nvec, PetscScalar *In, Vec Out)
   PetscMPIInt    Iright   = agmres->Iright, Ileft = agmres->Ileft;
   PetscScalar    *y, *zloc;
   PetscErrorCode ierr;
-  PetscInt       nloc,tag,d, len, i, j;
-  PetscInt       dpt,pas;
-  PetscReal      c, s, rho, zp, zq, yd, tt;
+  PetscInt       nloc,d, len, i, j;
+  PetscBLASInt   bnvec,pas,blen;
+  PetscInt       dpt;
+  PetscReal      c, s, rho, zp, zq, yd=0.0, tt;
   MPI_Status     status;
 
   PetscFunctionBegin;
+  ierr = PetscBLASIntCast(nvec,&bnvec);CHKERRQ(ierr);
   ierr = PetscObjectGetComm((PetscObject)ksp,&comm);CHKERRQ(ierr);
-  tag  = 0x666;
   pas  = 1;
   ierr = VecGetLocalSize(VEC_V(0), &nloc);CHKERRQ(ierr);
   ierr = PetscMalloc1(nvec, &y);CHKERRQ(ierr);
@@ -276,34 +269,34 @@ PetscErrorCode KSPAGMRESRodvec(KSP ksp, PetscInt nvec, PetscScalar *In, Vec Out)
     for (i = 0; i < nvec; i++) y[i] = sgn[i] * y[i];
   }
   for (i = 0; i < nloc; i++) zloc[i] = 0.0;
-  if (agmres->size == 1) PetscStackCallBLAS("BLAScopy",BLAScopy_(&nvec, y, &pas, &(zloc[0]), &pas));
+  if (agmres->size == 1) PetscStackCallBLAS("BLAScopy",BLAScopy_(&bnvec, y, &pas, &(zloc[0]), &pas));
   else {
     for (d = nvec - 1; d >= 0; d--) {
       if (rank == First) {
-        ierr = MPI_Recv(&(zloc[d]), 1, MPIU_SCALAR, Iright, tag, comm, &status);CHKERRQ(ierr);
+        ierr = MPI_Recv(&(zloc[d]), 1, MPIU_SCALAR, Iright, agmres->tag, comm, &status);CHKERRQ(ierr);
       } else {
         for (j = nvec - 1; j >= d + 1; j--) {
           i         = j - d;
-          ierr      = KSPAGMRESRoddecGivens(&c, &s, &(Qloc[j * nloc + i]), 0);
+          ierr      = KSPAGMRESRoddecGivens(&c, &s, &(Qloc[j * nloc + i]), 0);CHKERRQ(ierr);
           zp        = zloc[i-1];
           zq        = zloc[i];
           zloc[i-1] =     c * zp + s * zq;
           zloc[i]   =     -s * zp + c * zq;
         }
-        ierr = KSPAGMRESRoddecGivens(&c, &s, &(Qloc[d * nloc]), 0);
+        ierr = KSPAGMRESRoddecGivens(&c, &s, &(Qloc[d * nloc]), 0);CHKERRQ(ierr);
         if (rank == Last) {
           zp      = y[d];
           zq      = zloc[0];
           y[d]    =      c * zp + s * zq;
           zloc[0] =   -s * zp + c * zq;
-          ierr    = MPI_Send(&(y[d]), 1, MPIU_SCALAR, Ileft, tag, comm);CHKERRQ(ierr);
+          ierr    = MPI_Send(&(y[d]), 1, MPIU_SCALAR, Ileft, agmres->tag, comm);CHKERRQ(ierr);
         } else {
-          ierr    = MPI_Recv(&yd, 1, MPIU_SCALAR, Iright, tag, comm, &status);CHKERRQ(ierr);
+          ierr    = MPI_Recv(&yd, 1, MPIU_SCALAR, Iright, agmres->tag, comm, &status);CHKERRQ(ierr);
           zp      = yd;
           zq      = zloc[0];
           yd      =      c * zp + s * zq;
           zloc[0] =   -s * zp + c * zq;
-          ierr    = MPI_Send(&yd, 1, MPIU_SCALAR, Ileft, tag, comm);CHKERRQ(ierr);
+          ierr    = MPI_Send(&yd, 1, MPIU_SCALAR, Ileft, agmres->tag, comm);CHKERRQ(ierr);
         }
       }
     }
@@ -312,10 +305,11 @@ PetscErrorCode KSPAGMRESRodvec(KSP ksp, PetscInt nvec, PetscScalar *In, Vec Out)
     dpt = j * nloc + j;
     if (tloc[j] != 0.0) {
       len       = nloc - j;
+      ierr      = PetscBLASIntCast(len,&blen);CHKERRQ(ierr); 
       rho       = Qloc[dpt];
       Qloc[dpt] = 1.0;
-      tt        = tloc[j] * (BLASdot_(&len, &(Qloc[dpt]), &pas, &(zloc[j]), &pas));
-      PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&len, &tt, &(Qloc[dpt]), &pas, &(zloc[j]), &pas));
+      tt        = tloc[j] * (BLASdot_(&blen, &(Qloc[dpt]), &pas, &(zloc[j]), &pas));
+      PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&blen, &tt, &(Qloc[dpt]), &pas, &(zloc[j]), &pas));
       Qloc[dpt] = rho;
     }
   }
