@@ -13,8 +13,6 @@ static PetscErrorCode    KSPDGMRESGetNewVectors(KSP,PetscInt);
 static PetscErrorCode    KSPDGMRESUpdateHessenberg(KSP,PetscInt,PetscBool,PetscReal*);
 static PetscErrorCode    KSPDGMRESBuildSoln(PetscScalar*,Vec,Vec,KSP,PetscInt);
 
-#undef __FUNCT__
-#define __FUNCT__  "KSPDGMRESSetEigen"
 PetscErrorCode  KSPDGMRESSetEigen(KSP ksp,PetscInt nb_eig)
 {
   PetscErrorCode ierr;
@@ -23,8 +21,6 @@ PetscErrorCode  KSPDGMRESSetEigen(KSP ksp,PetscInt nb_eig)
   ierr = PetscTryMethod((ksp),"KSPDGMRESSetEigen_C",(KSP,PetscInt),(ksp,nb_eig));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__  "KSPDGMRESSetMaxEigen"
 PetscErrorCode  KSPDGMRESSetMaxEigen(KSP ksp,PetscInt max_neig)
 {
   PetscErrorCode ierr;
@@ -33,8 +29,6 @@ PetscErrorCode  KSPDGMRESSetMaxEigen(KSP ksp,PetscInt max_neig)
   ierr = PetscTryMethod((ksp),"KSPDGMRESSetMaxEigen_C",(KSP,PetscInt),(ksp,max_neig));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__  "KSPDGMRESForce"
 PetscErrorCode  KSPDGMRESForce(KSP ksp,PetscBool force)
 {
   PetscErrorCode ierr;
@@ -43,8 +37,6 @@ PetscErrorCode  KSPDGMRESForce(KSP ksp,PetscBool force)
   ierr = PetscTryMethod((ksp),"KSPDGMRESForce_C",(KSP,PetscBool),(ksp,force));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__  "KSPDGMRESSetMaxEigen"
 PetscErrorCode  KSPDGMRESSetRatio(KSP ksp,PetscReal ratio)
 {
   PetscErrorCode ierr;
@@ -53,8 +45,6 @@ PetscErrorCode  KSPDGMRESSetRatio(KSP ksp,PetscReal ratio)
   ierr = PetscTryMethod((ksp),"KSPDGMRESSetRatio_C",(KSP,PetscReal),(ksp,ratio));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__  "KSPDGMRESComputeSchurForm"
 PetscErrorCode  KSPDGMRESComputeSchurForm(KSP ksp,PetscInt *neig)
 {
   PetscErrorCode ierr;
@@ -63,18 +53,14 @@ PetscErrorCode  KSPDGMRESComputeSchurForm(KSP ksp,PetscInt *neig)
   ierr = PetscUseMethod((ksp),"KSPDGMRESComputeSchurForm_C",(KSP, PetscInt*),(ksp, neig));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__  "KSPDGMRESComputeDeflationData"
-PetscErrorCode  KSPDGMRESComputeDeflationData(KSP ksp)
+PetscErrorCode  KSPDGMRESComputeDeflationData(KSP ksp,PetscInt *curneigh)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscUseMethod((ksp),"KSPDGMRESComputeDeflationData_C",(KSP),(ksp));CHKERRQ(ierr);
+  ierr = PetscUseMethod((ksp),"KSPDGMRESComputeDeflationData_C",(KSP,PetscInt*),(ksp,curneigh));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__  "KSPDGMRESApplyDeflation"
 PetscErrorCode  KSPDGMRESApplyDeflation(KSP ksp, Vec x, Vec y)
 {
   PetscErrorCode ierr;
@@ -84,8 +70,6 @@ PetscErrorCode  KSPDGMRESApplyDeflation(KSP ksp, Vec x, Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDGMRESImproveEig"
 PetscErrorCode  KSPDGMRESImproveEig(KSP ksp, PetscInt neig)
 {
   PetscErrorCode ierr;
@@ -95,8 +79,6 @@ PetscErrorCode  KSPDGMRESImproveEig(KSP ksp, PetscInt neig)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_DGMRES"
 PetscErrorCode  KSPSetUp_DGMRES(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -151,8 +133,6 @@ PetscErrorCode  KSPSetUp_DGMRES(KSP ksp)
  On entry, the value in vector VEC_VV(0) should be the initial residual
  (this allows shortcuts where the initial preconditioned residual is 0).
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPDGMRESCycle"
 PetscErrorCode KSPDGMRESCycle(PetscInt *itcount,KSP ksp)
 {
   KSP_DGMRES     *dgmres = (KSP_DGMRES*)(ksp->data);
@@ -268,14 +248,12 @@ PetscErrorCode KSPDGMRESCycle(PetscInt *itcount,KSP ksp)
     test = max_k *PetscLogReal(ksp->rtol/res) /PetscLogReal(res/res_old);
     /* Compute data for the deflation if the residual rtol will not be reached in the remaining number of steps allowed  */
     if ((test > dgmres->smv*(ksp->max_it-ksp->its)) || dgmres->force) {
-      ierr =  KSPDGMRESComputeDeflationData(ksp);CHKERRQ(ierr);
+      ierr =  KSPDGMRESComputeDeflationData(ksp,NULL);CHKERRQ(ierr);
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_DGMRES"
 PetscErrorCode KSPSolve_DGMRES(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -319,8 +297,6 @@ PetscErrorCode KSPSolve_DGMRES(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDestroy_DGMRES"
 PetscErrorCode KSPDestroy_DGMRES(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -373,8 +349,6 @@ PetscErrorCode KSPDestroy_DGMRES(KSP ksp)
 
  This is an internal routine that knows about the GMRES internals.
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPDGMRESBuildSoln"
 static PetscErrorCode KSPDGMRESBuildSoln(PetscScalar *nrs,Vec vs,Vec vdest,KSP ksp,PetscInt it)
 {
   PetscScalar    tt;
@@ -423,8 +397,6 @@ static PetscErrorCode KSPDGMRESBuildSoln(PetscScalar *nrs,Vec vs,Vec vdest,KSP k
 /*
  Do the scalar work for the orthogonalization.  Return new residual norm.
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPDGMRESUpdateHessenberg"
 static PetscErrorCode KSPDGMRESUpdateHessenberg(KSP ksp,PetscInt it,PetscBool hapend,PetscReal *res)
 {
   PetscScalar *hh,*cc,*ss,tt;
@@ -478,8 +450,6 @@ static PetscErrorCode KSPDGMRESUpdateHessenberg(KSP ksp,PetscInt it,PetscBool ha
 /*
  This routine allocates more work vectors, starting from VEC_VV(it).
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPDGMRESGetNewVectors"
 static PetscErrorCode KSPDGMRESGetNewVectors(KSP ksp,PetscInt it)
 {
   KSP_DGMRES     *dgmres = (KSP_DGMRES*) ksp->data;
@@ -508,8 +478,6 @@ static PetscErrorCode KSPDGMRESGetNewVectors(KSP ksp,PetscInt it)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPBuildSolution_DGMRES"
 PetscErrorCode KSPBuildSolution_DGMRES(KSP ksp,Vec ptr,Vec *result)
 {
   KSP_DGMRES     *dgmres = (KSP_DGMRES*) ksp->data;
@@ -534,8 +502,6 @@ PetscErrorCode KSPBuildSolution_DGMRES(KSP ksp,Vec ptr,Vec *result)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPView_DGMRES"
 PetscErrorCode KSPView_DGMRES(KSP ksp,PetscViewer viewer)
 {
   KSP_DGMRES     *dgmres = (KSP_DGMRES*) ksp->data;
@@ -546,27 +512,25 @@ PetscErrorCode KSPView_DGMRES(KSP ksp,PetscViewer viewer)
   ierr = KSPView_GMRES(ksp,viewer);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject) viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
-    if (dgmres->force) PetscViewerASCIIPrintf(viewer, "   DGMRES: Adaptive strategy is used: FALSE\n");
-    else PetscViewerASCIIPrintf(viewer, "   DGMRES: Adaptive strategy is used: TRUE\n");
+    if (dgmres->force) PetscViewerASCIIPrintf(viewer, "    Adaptive strategy is used: FALSE\n");
+    else PetscViewerASCIIPrintf(viewer, "    Adaptive strategy is used: TRUE\n");
     ierr = PetscOptionsHasName(((PetscObject)ksp)->options,((PetscObject)ksp)->prefix, "-ksp_dgmres_harmonic_ritz", &isharmonic);CHKERRQ(ierr);
     if (isharmonic) {
-      ierr = PetscViewerASCIIPrintf(viewer, "  DGMRES: Frequency of extracted eigenvalues = %D using Harmonic Ritz values \n", dgmres->neig);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer, "   Frequency of extracted eigenvalues = %D using Harmonic Ritz values \n", dgmres->neig);CHKERRQ(ierr);
     } else {
-      ierr = PetscViewerASCIIPrintf(viewer, "  DGMRES: Frequency of extracted eigenvalues = %D using Ritz values \n", dgmres->neig);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer, "   Frequency of extracted eigenvalues = %D using Ritz values \n", dgmres->neig);CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIIPrintf(viewer, "  DGMRES: Total number of extracted eigenvalues = %D\n", dgmres->r);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer, "  DGMRES: Maximum number of eigenvalues set to be extracted = %D\n", dgmres->max_neig);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer, "  DGMRES: relaxation parameter for the adaptive strategy(smv)  = %g\n", dgmres->smv);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer, "  DGMRES: Number of matvecs : %D\n", dgmres->matvecs);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer, "   Total number of extracted eigenvalues = %D\n", dgmres->r);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer, "   Maximum number of eigenvalues set to be extracted = %D\n", dgmres->max_neig);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer, "   relaxation parameter for the adaptive strategy(smv)  = %g\n", dgmres->smv);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer, "   Number of matvecs : %D\n", dgmres->matvecs);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
 /* New DGMRES functions */
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDGMRESSetEigen_DGMRES"
-static PetscErrorCode  KSPDGMRESSetEigen_DGMRES(KSP ksp,PetscInt neig)
+PetscErrorCode  KSPDGMRESSetEigen_DGMRES(KSP ksp,PetscInt neig)
 {
   KSP_DGMRES *dgmres = (KSP_DGMRES*) ksp->data;
 
@@ -576,8 +540,6 @@ static PetscErrorCode  KSPDGMRESSetEigen_DGMRES(KSP ksp,PetscInt neig)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDGMRESSetMaxEigen_DGMRES"
 static PetscErrorCode  KSPDGMRESSetMaxEigen_DGMRES(KSP ksp,PetscInt max_neig)
 {
   KSP_DGMRES *dgmres = (KSP_DGMRES*) ksp->data;
@@ -588,8 +550,6 @@ static PetscErrorCode  KSPDGMRESSetMaxEigen_DGMRES(KSP ksp,PetscInt max_neig)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDGMRESSetRatio_DGMRES"
 static PetscErrorCode  KSPDGMRESSetRatio_DGMRES(KSP ksp,PetscReal ratio)
 {
   KSP_DGMRES *dgmres = (KSP_DGMRES*) ksp->data;
@@ -600,8 +560,6 @@ static PetscErrorCode  KSPDGMRESSetRatio_DGMRES(KSP ksp,PetscReal ratio)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDGMRESForce_DGMRES"
 static PetscErrorCode  KSPDGMRESForce_DGMRES(KSP ksp,PetscBool force)
 {
   KSP_DGMRES *dgmres = (KSP_DGMRES*) ksp->data;
@@ -611,10 +569,6 @@ static PetscErrorCode  KSPDGMRESForce_DGMRES(KSP ksp,PetscBool force)
   PetscFunctionReturn(0);
 }
 
-extern PetscErrorCode KSPSetFromOptions_GMRES(PetscOptionItems *PetscOptionsObject,KSP);
-
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetFromOptions_DGMRES"
 PetscErrorCode KSPSetFromOptions_DGMRES(PetscOptionItems *PetscOptionsObject,KSP ksp)
 {
   PetscErrorCode ierr;
@@ -641,9 +595,7 @@ PetscErrorCode KSPSetFromOptions_DGMRES(PetscOptionItems *PetscOptionsObject,KSP
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDGMRESComputeDeflationData_DGMRES"
-static PetscErrorCode  KSPDGMRESComputeDeflationData_DGMRES(KSP ksp, PetscInt *ExtrNeig)
+PetscErrorCode  KSPDGMRESComputeDeflationData_DGMRES(KSP ksp, PetscInt *ExtrNeig)
 {
   KSP_DGMRES     *dgmres = (KSP_DGMRES*) ksp->data;
   PetscErrorCode ierr;
@@ -786,9 +738,7 @@ static PetscErrorCode  KSPDGMRESComputeDeflationData_DGMRES(KSP ksp, PetscInt *E
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDGMRESComputeSchurForm_DGMRES"
-static PetscErrorCode  KSPDGMRESComputeSchurForm_DGMRES(KSP ksp, PetscInt *neig)
+PetscErrorCode  KSPDGMRESComputeSchurForm_DGMRES(KSP ksp, PetscInt *neig)
 {
   KSP_DGMRES     *dgmres = (KSP_DGMRES*) ksp->data;
   PetscErrorCode ierr;
@@ -936,9 +886,7 @@ static PetscErrorCode  KSPDGMRESComputeSchurForm_DGMRES(KSP ksp, PetscInt *neig)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDGMRESApplyDeflation_DGMRES"
-static PetscErrorCode  KSPDGMRESApplyDeflation_DGMRES(KSP ksp, Vec x, Vec y)
+PetscErrorCode  KSPDGMRESApplyDeflation_DGMRES(KSP ksp, Vec x, Vec y)
 {
   KSP_DGMRES     *dgmres = (KSP_DGMRES*) ksp->data;
   PetscInt       i, r     = dgmres->r;
@@ -1003,8 +951,6 @@ static PetscErrorCode  KSPDGMRESApplyDeflation_DGMRES(KSP ksp, Vec x, Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDGMRESImproveEig_DGMRES"
 static PetscErrorCode  KSPDGMRESImproveEig_DGMRES(KSP ksp, PetscInt neig)
 {
   KSP_DGMRES   *dgmres = (KSP_DGMRES*) ksp->data;
@@ -1225,7 +1171,8 @@ static PetscErrorCode  KSPDGMRESImproveEig_DGMRES(KSP ksp, PetscInt neig)
 
  Level: beginner
 
- Notes: Left and right preconditioning are supported, but not symmetric preconditioning. Complex arithmetic is not yet supported
+ Notes:
+    Left and right preconditioning are supported, but not symmetric preconditioning. Complex arithmetic is not yet supported
 
  References:
 +  1. - J. Erhel, K. Burrage and B. Pohl,  Restarted GMRES preconditioned by deflation,J. Computational and Applied Mathematics, 69(1996).
@@ -1241,8 +1188,6 @@ static PetscErrorCode  KSPDGMRESImproveEig_DGMRES(KSP ksp, PetscInt neig)
 
  M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_DGMRES"
 PETSC_EXTERN PetscErrorCode KSPCreate_DGMRES(KSP ksp)
 {
   KSP_DGMRES     *dgmres;
@@ -1279,7 +1224,7 @@ PETSC_EXTERN PetscErrorCode KSPCreate_DGMRES(KSP ksp)
   ierr = PetscObjectComposeFunction((PetscObject)ksp, "KSPDGMRESApplyDeflation_C",KSPDGMRESApplyDeflation_DGMRES);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)ksp, "KSPDGMRESImproveEig_C", KSPDGMRESImproveEig_DGMRES);CHKERRQ(ierr);
 
-  ierr = PetscLogEventRegister("DGMRESComputeDefl", KSP_CLASSID, &KSP_DGMRESComputeDeflationData);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("DGMRESCompDefl",  KSP_CLASSID, &KSP_DGMRESComputeDeflationData);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("DGMRESApplyDefl", KSP_CLASSID, &KSP_DGMRESApplyDeflation);CHKERRQ(ierr);
 
   dgmres->haptol         = 1.0e-30;
