@@ -4,6 +4,21 @@
 
 #include <../src/ksp/ksp/impls/gmres/pipefgmres/pipefgmresimpl.h>       /*I  "petscksp.h"  I*/
 
+static PetscBool  cited = PETSC_FALSE;
+static const char citation[] =
+  "@article{SSM2016,\n"
+  "  author = {P. Sanan and S.M. Schnepp and D.A. May},\n"
+  "  title = {Pipelined, Flexible Krylov Subspace Methods},\n"
+  "  journal = {SIAM Journal on Scientific Computing},\n"
+  "  volume = {38},\n"
+  "  number = {5},\n"
+  "  pages = {C441-C470},\n"
+  "  year = {2016},\n"
+  "  doi = {10.1137/15M1049130},\n"
+  "  URL = {http://dx.doi.org/10.1137/15M1049130},\n"
+  "  eprint = {http://dx.doi.org/10.1137/15M1049130}\n"
+  "}\n";
+
 #define PIPEFGMRES_DELTA_DIRECTIONS 10
 #define PIPEFGMRES_DEFAULT_MAXK     30
 
@@ -20,8 +35,6 @@ extern PetscErrorCode KSPReset_PIPEFGMRES(KSP);
     but can be called directly by KSPSetUp().
 
 */
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_PIPEFGMRES"
 static PetscErrorCode KSPSetUp_PIPEFGMRES(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -77,8 +90,6 @@ static PetscErrorCode KSPSetUp_PIPEFGMRES(KSP ksp)
 
  */
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPPIPEFGMRESCycle"
 static PetscErrorCode KSPPIPEFGMRESCycle(PetscInt *itcount,KSP ksp)
 {
   KSP_PIPEFGMRES *pipefgmres = (KSP_PIPEFGMRES*)(ksp->data);
@@ -220,6 +231,8 @@ static PetscErrorCode KSPPIPEFGMRESCycle(PetscInt *itcount,KSP ksp)
     /* Compute the norm of the un-normalized new direction using the rearranged formula
        Note that these are shifted ("barred") quantities */
     for(k=0;k<=loc_it;k++) tt -= ((PetscReal)(PetscAbsScalar(lhh[k]) * PetscAbsScalar(lhh[k])));
+    /* On AVX512 this is accumulating roundoff errors for eg: tt=-2.22045e-16 */
+    if ((tt < 0.0) && tt > -PETSC_SMALL) tt = 0.0 ;
     if (tt < 0.0) {
       /* If we detect square root breakdown in the norm, we must restart the algorithm.
          Here this means we simply break the current loop and reconstruct the solution
@@ -227,7 +240,7 @@ static PetscErrorCode KSPPIPEFGMRESCycle(PetscInt *itcount,KSP ksp)
          we do not update the iteration count, so computation done in this iteration
          should be disregarded.
          */
-      ierr = PetscInfo1(ksp,"Restart due to square root breakdown at it = \n",ksp->its);CHKERRQ(ierr);
+      ierr = PetscInfo2(ksp,"Restart due to square root breakdown at it = %D, tt=%g\n",ksp->its,(double)tt);CHKERRQ(ierr);
       break;
     } else {
       tt = PetscSqrtReal(tt);
@@ -346,8 +359,6 @@ static PetscErrorCode KSPPIPEFGMRESCycle(PetscInt *itcount,KSP ksp)
 .     outits - number of iterations used
 
 */
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_PIPEFGMRES"
 static PetscErrorCode KSPSolve_PIPEFGMRES(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -363,6 +374,8 @@ static PetscErrorCode KSPSolve_PIPEFGMRES(KSP ksp)
   SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_SUP,"PIPEFGMRES has not been implemented for use with complex scalars");
 #endif
 
+  ierr = PetscCitationsRegister(citation,&cited);CHKERRQ(ierr);
+
   if (ksp->calc_sings && !pipefgmres->Rsvd) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_ORDER,"Must call KSPSetComputeSingularValues() before KSPSetUp() is called");
   ierr     = PetscObjectSAWsTakeAccess((PetscObject)ksp);CHKERRQ(ierr);
   ksp->its = 0;
@@ -384,8 +397,6 @@ static PetscErrorCode KSPSolve_PIPEFGMRES(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDestroy_PIPEFGMRES"
 static PetscErrorCode KSPDestroy_PIPEFGMRES(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -409,8 +420,6 @@ static PetscErrorCode KSPDestroy_PIPEFGMRES(KSP ksp)
 
      This is an internal routine that knows about the PIPEFGMRES internals.
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPPIPEFGMRESBuildSoln"
 static PetscErrorCode KSPPIPEFGMRESBuildSoln(PetscScalar *nrs,Vec vguess,Vec vdest,KSP ksp,PetscInt it)
 {
   PetscScalar    tt;
@@ -466,8 +475,6 @@ static PetscErrorCode KSPPIPEFGMRESBuildSoln(PetscScalar *nrs,Vec vguess,Vec vde
 .        res - the new residual
 
  */
-#undef __FUNCT__
-#define __FUNCT__ "KSPPIPEFGMRESUpdateHessenberg"
 /*
 .  it - column of the Hessenberg that is complete, PIPEFGMRES is actually computing two columns ahead of this
  */
@@ -558,8 +565,6 @@ static PetscErrorCode KSPPIPEFGMRESUpdateHessenberg(KSP ksp,PetscInt it,PetscBoo
    calls directly.
 
 */
-#undef __FUNCT__
-#define __FUNCT__ "KSPBuildSolution_PIPEFGMRES"
 PetscErrorCode KSPBuildSolution_PIPEFGMRES(KSP ksp,Vec ptr,Vec *result)
 {
   KSP_PIPEFGMRES *pipefgmres = (KSP_PIPEFGMRES*)ksp->data;
@@ -584,8 +589,6 @@ PetscErrorCode KSPBuildSolution_PIPEFGMRES(KSP ksp,Vec ptr,Vec *result)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetFromOptions_PIPEFGMRES"
 PetscErrorCode KSPSetFromOptions_PIPEFGMRES(PetscOptionItems *PetscOptionsObject,KSP ksp)
 {
   PetscErrorCode ierr;
@@ -602,8 +605,6 @@ PetscErrorCode KSPSetFromOptions_PIPEFGMRES(PetscOptionItems *PetscOptionsObject
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPView_PIPEFGMRES"
 PetscErrorCode KSPView_PIPEFGMRES(KSP ksp,PetscViewer viewer)
 {
   KSP_PIPEFGMRES *pipefgmres = (KSP_PIPEFGMRES*)ksp->data;
@@ -615,26 +616,24 @@ PetscErrorCode KSPView_PIPEFGMRES(KSP ksp,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERSTRING,&isstring);CHKERRQ(ierr);
 
   if (iascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  GMRES: restart=%D\n",pipefgmres->max_k);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  GMRES: happy breakdown tolerance %g\n",(double)pipefgmres->haptol);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  restart=%D\n",pipefgmres->max_k);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  happy breakdown tolerance %g\n",(double)pipefgmres->haptol);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
-    ierr = PetscViewerASCIIPrintf(viewer," PIPEFGMRES: shift=%g+%gi\n",PetscRealPart(pipefgmres->shift),PetscImaginaryPart(pipefgmres->shift));CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  shift=%g+%gi\n",PetscRealPart(pipefgmres->shift),PetscImaginaryPart(pipefgmres->shift));CHKERRQ(ierr);
 #else
-    ierr = PetscViewerASCIIPrintf(viewer," PIPEFGMRES: shift=%g\n",pipefgmres->shift);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  shift=%g\n",pipefgmres->shift);CHKERRQ(ierr);
 #endif
   } else if (isstring) {
     ierr = PetscViewerStringSPrintf(viewer,"restart %D",pipefgmres->max_k);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
-    ierr = PetscViewerStringSPrintf(viewer,"   PIPEFGMRES: shift=%g+%gi\n",PetscRealPart(pipefgmres->shift),PetscImaginaryPart(pipefgmres->shift));CHKERRQ(ierr);
+    ierr = PetscViewerStringSPrintf(viewer,"   shift=%g+%gi\n",PetscRealPart(pipefgmres->shift),PetscImaginaryPart(pipefgmres->shift));CHKERRQ(ierr);
 #else
-    ierr = PetscViewerStringSPrintf(viewer,"   PIPEFGMRES: shift=%g\n",pipefgmres->shift);CHKERRQ(ierr);
+    ierr = PetscViewerStringSPrintf(viewer,"   shift=%g\n",pipefgmres->shift);CHKERRQ(ierr);
 #endif
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPReset_PIPEFGMRES"
 PetscErrorCode KSPReset_PIPEFGMRES(KSP ksp)
 {
   KSP_PIPEFGMRES *pipefgmres = (KSP_PIPEFGMRES*)ksp->data;
@@ -658,13 +657,7 @@ PetscErrorCode KSPReset_PIPEFGMRES(KSP ksp)
 /*MC
    KSPPIPEFGMRES - Implements the Pipelined Generalized Minimal Residual method.
 
-   A Flexible, 1-stage pipelined variant of GMRES
-
-   This variant is not "explicitly normalized" like PGMRES, and requires a shift parameter.
-
-   A heuristic for choosing the shift parameter is the largest eigenvalue of the preconditioned operator.
-
-   Only right preconditioning is supported (but this preconditioner may be nonlinear, as with FGMRES)
+   A flexible, 1-stage pipelined variant of GMRES.
 
    Options Database Keys:
 +   -ksp_gmres_restart <restart> - the number of Krylov directions to orthogonalize against
@@ -675,20 +668,31 @@ PetscErrorCode KSPReset_PIPEFGMRES(KSP ksp)
 -   -ksp_gmres_krylov_monitor - plot the Krylov space generated
 
 
-   Level: beginner
+   Level: intermediate
 
    Notes:
+
+   This variant is not "explicitly normalized" like KSPPGMRES, and requires a shift parameter.
+
+   A heuristic for choosing the shift parameter is the largest eigenvalue of the preconditioned operator.
+
+   Only right preconditioning is supported (but this preconditioner may be nonlinear/variable/inexact, as with KSPFGMRES).
    MPI configuration may be necessary for reductions to make asynchronous progress, which is important for performance of pipelined methods.
    See the FAQ on the PETSc website for details.
 
-   Developer Notes: This object is subclassed off of KSPGMRES
+   Developer Notes:
+    This class is subclassed off of KSPGMRES.
+
+   Reference:
+    P. Sanan, S.M. Schnepp, and D.A. May,
+    "Pipelined, Flexible Krylov Subspace Methods,"
+    SIAM Journal on Scientific Computing 2016 38:5, C441-C470,
+    DOI: 10.1137/15M1049130
 
 .seealso:  KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP, KSPLGMRES, KSPPIPECG, KSPPIPECR, KSPPGMRES, KSPFGMRES
-           KSPGMRESSetRestart(), KSPGMRESSetHapTol(), KSPGMRESSetPreAllocateVectors(), KSPGMRESMonitorKrylov(), KSPPIPEGMRESSetShift()
+           KSPGMRESSetRestart(), KSPGMRESSetHapTol(), KSPGMRESSetPreAllocateVectors(), KSPGMRESMonitorKrylov(), KSPPIPEFGMRESSetShift()
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_PIPEFGMRES"
 PETSC_EXTERN PetscErrorCode KSPCreate_PIPEFGMRES(KSP ksp)
 {
   KSP_PIPEFGMRES *pipefgmres;
@@ -729,8 +733,6 @@ PETSC_EXTERN PetscErrorCode KSPCreate_PIPEFGMRES(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPPIPEFGMRESGetNewVectors"
 static PetscErrorCode KSPPIPEFGMRESGetNewVectors(KSP ksp,PetscInt it)
 {
   KSP_PIPEFGMRES *pipefgmres = (KSP_PIPEFGMRES*)ksp->data;
@@ -778,8 +780,6 @@ static PetscErrorCode KSPPIPEFGMRESGetNewVectors(KSP ksp,PetscInt it)
   pipefgmres->nwork_alloc++;
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__ "KSPPIPEFGMRESSetShift"
 /*@
   KSPPIPEFGMRESSetShift - Set the shift parameter for the flexible, pipelined GMRES solver.
 
