@@ -1,7 +1,5 @@
 
 #include <../src/ksp/ksp/impls/lcd/lcdimpl.h>
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_LCD"
 
 PetscErrorCode KSPSetUp_LCD(KSP ksp)
 {
@@ -30,8 +28,6 @@ PetscErrorCode KSPSetUp_LCD(KSP ksp)
 .     its - number of iterations used
 
 */
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_LCD"
 PetscErrorCode  KSPSolve_LCD(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -75,7 +71,6 @@ PetscErrorCode  KSPSolve_LCD(KSP ksp)
   ierr = (*ksp->converged)(ksp,0,rnorm,&ksp->reason,ksp->cnvP);CHKERRQ(ierr);
   if (ksp->reason) PetscFunctionReturn(0);
 
-  it = 0;
   VecCopy(R,lcd->P[0]);
 
   while (!ksp->reason && ksp->its < ksp->max_it) {
@@ -122,8 +117,6 @@ PetscErrorCode  KSPSolve_LCD(KSP ksp)
        KSPDestroy_LCD - Frees all memory space used by the Krylov method
 
 */
-#undef __FUNCT__
-#define __FUNCT__ "KSPReset_LCD"
 PetscErrorCode KSPReset_LCD(KSP ksp)
 {
   KSP_LCD        *lcd = (KSP_LCD*)ksp->data;
@@ -136,8 +129,6 @@ PetscErrorCode KSPReset_LCD(KSP ksp)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDestroy_LCD"
 PetscErrorCode KSPDestroy_LCD(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -156,8 +147,6 @@ PetscErrorCode KSPDestroy_LCD(KSP ksp)
       flags that information should be printed here.
 
 */
-#undef __FUNCT__
-#define __FUNCT__ "KSPView_LCD"
 PetscErrorCode KSPView_LCD(KSP ksp,PetscViewer viewer)
 {
 
@@ -168,8 +157,8 @@ PetscErrorCode KSPView_LCD(KSP ksp,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  LCD: restart=%d\n",lcd->restart);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  LCD: happy breakdown tolerance %g\n",lcd->haptol);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  restart=%d\n",lcd->restart);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  happy breakdown tolerance %g\n",lcd->haptol);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -178,9 +167,7 @@ PetscErrorCode KSPView_LCD(KSP ksp,PetscViewer viewer)
     KSPSetFromOptions_LCD - Checks the options database for options related to the
                             LCD method.
 */
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetFromOptions_LCD"
-PetscErrorCode KSPSetFromOptions_LCD(PetscOptions *PetscOptionsObject,KSP ksp)
+PetscErrorCode KSPSetFromOptions_LCD(PetscOptionItems *PetscOptionsObject,KSP ksp)
 {
   PetscErrorCode ierr;
   PetscBool      flg;
@@ -204,22 +191,23 @@ PetscErrorCode KSPSetFromOptions_LCD(PetscOptions *PetscOptionsObject,KSP ksp)
 
    Level: beginner
 
-    Notes: Support only for left preconditioning
+    Notes:
+    Support only for left preconditioning
 
     References:
-   - J.Y. Yuan, G.H.Golub, R.J. Plemmons, and W.A.G. Cecilio. Semiconjugate
++    1. - J.Y. Yuan, G.H.Golub, R.J. Plemmons, and W.A.G. Cecilio. Semiconjugate
      direction methods for real positive definite system. BIT Numerical
-     Mathematics, 44(1):189-207,2004.
-   - Y. Dai and J.Y. Yuan. Study on semi-conjugate direction methods for
-     non-symmetric systems. International Journal for Numerical Methods in
-     Engineering, 60:1383-1399,2004.
-   - L. Catabriga, A.L.G.A. Coutinho, and L.P.Franca. Evaluating the LCD
+     Mathematics, 44(1),2004.
+.    2. - Y. Dai and J.Y. Yuan. Study on semiconjugate direction methods for
+     nonsymmetric systems. International Journal for Numerical Methods in
+     Engineering, 60, 2004.
+.    3. - L. Catabriga, A.L.G.A. Coutinho, and L.P.Franca. Evaluating the LCD
      algorithm for solving linear systems of equations arising from implicit
      SUPG formulation of compressible flows. International Journal for
-     Numerical Methods in Engineering, 60:1513-1534,2004
-   - L. Catabriga, A. M. P. Valli, B. Z. Melotti, L. M. Pessoa,
+     Numerical Methods in Engineering, 60, 2004
+-    4. - L. Catabriga, A. M. P. Valli, B. Z. Melotti, L. M. Pessoa,
      A. L. G. A. Coutinho, Performance of LCD iterative method in the finite
-     element and finite difference solution of convection-diffusion
+     element and finite difference solution of convection diffusion
      equations,  Communications in Numerical Methods in Engineering, (Early
      View).
 
@@ -231,8 +219,6 @@ PetscErrorCode KSPSetFromOptions_LCD(PetscOptions *PetscOptionsObject,KSP ksp)
 
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_LCD"
 PETSC_EXTERN PetscErrorCode KSPCreate_LCD(KSP ksp)
 {
   PetscErrorCode ierr;
