@@ -1,7 +1,5 @@
 #include <petsc/private/kspimpl.h>          /*I "petscksp.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPPythonSetType"
 /*@C
    KSPPythonSetType - Initalize a KSP object implemented in Python.
 
