@@ -1,11 +1,8 @@
 
-#include <petsc/private/kspimpl.h>             /*I "petscksp.h" I*/
-#include <../src/ksp/ksp/impls/qcg/qcgimpl.h>
+#include <../src/ksp/ksp/impls/qcg/qcgimpl.h>     /*I "petscksp.h" I*/
 
 static PetscErrorCode KSPQCGQuadraticRoots(Vec,Vec,PetscReal,PetscReal*,PetscReal*);
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPQCGSetTrustRegionRadius"
 /*@
     KSPQCGSetTrustRegionRadius - Sets the radius of the trust region.
 
@@ -33,8 +30,6 @@ PetscErrorCode  KSPQCGSetTrustRegionRadius(KSP ksp,PetscReal delta)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPQCGGetTrialStepNorm"
 /*@
     KSPQCGGetTrialStepNorm - Gets the norm of a trial step vector.  The WCG step may be
     constrained, so this is not necessarily the length of the ultimate step taken in QCG.
@@ -59,8 +54,6 @@ PetscErrorCode  KSPQCGGetTrialStepNorm(KSP ksp,PetscReal *tsnorm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPQCGGetQuadratic"
 /*@
     KSPQCGGetQuadratic - Gets the value of the quadratic function, evaluated at the new iterate:
 
@@ -97,8 +90,6 @@ PetscErrorCode  KSPQCGGetQuadratic(KSP ksp,PetscReal *quadratic)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_QCG"
 
 PetscErrorCode KSPSolve_QCG(KSP ksp)
 {
@@ -279,8 +270,6 @@ PetscErrorCode KSPSolve_QCG(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_QCG"
 PetscErrorCode KSPSetUp_QCG(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -291,8 +280,6 @@ PetscErrorCode KSPSetUp_QCG(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDestroy_QCG"
 PetscErrorCode KSPDestroy_QCG(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -305,8 +292,6 @@ PetscErrorCode KSPDestroy_QCG(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPQCGSetTrustRegionRadius_QCG"
 static PetscErrorCode  KSPQCGSetTrustRegionRadius_QCG(KSP ksp,PetscReal delta)
 {
   KSP_QCG *cgP = (KSP_QCG*)ksp->data;
@@ -316,8 +301,6 @@ static PetscErrorCode  KSPQCGSetTrustRegionRadius_QCG(KSP ksp,PetscReal delta)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPQCGGetTrialStepNorm_QCG"
 static PetscErrorCode  KSPQCGGetTrialStepNorm_QCG(KSP ksp,PetscReal *ltsnrm)
 {
   KSP_QCG *cgP = (KSP_QCG*)ksp->data;
@@ -327,8 +310,6 @@ static PetscErrorCode  KSPQCGGetTrialStepNorm_QCG(KSP ksp,PetscReal *ltsnrm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPQCGGetQuadratic_QCG"
 static PetscErrorCode  KSPQCGGetQuadratic_QCG(KSP ksp,PetscReal *quadratic)
 {
   KSP_QCG *cgP = (KSP_QCG*)ksp->data;
@@ -338,9 +319,7 @@ static PetscErrorCode  KSPQCGGetQuadratic_QCG(KSP ksp,PetscReal *quadratic)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetFromOptions_QCG"
-PetscErrorCode KSPSetFromOptions_QCG(PetscOptions *PetscOptionsObject,KSP ksp)
+PetscErrorCode KSPSetFromOptions_QCG(PetscOptionItems *PetscOptionsObject,KSP ksp)
 {
   PetscErrorCode ierr;
   PetscReal      delta;
@@ -362,11 +341,13 @@ PetscErrorCode KSPSetFromOptions_QCG(PetscOptions *PetscOptionsObject,KSP ksp)
    Options Database Keys:
 .      -ksp_qcg_trustregionradius <r> - Trust Region Radius
 
-   Notes: This is rarely used directly
+   Notes:
+    This is rarely used directly
 
    Level: developer
 
-  Notes:  Use preconditioned conjugate gradient to compute
+  Notes:
+    Use preconditioned conjugate gradient to compute
       an approximate minimizer of the quadratic function
 
             q(s) = g^T * s + .5 * s^T * H * s
@@ -396,15 +377,13 @@ $  other KSP converged/diverged reasons
                 Here L is an incomplete Cholesky factor of H.
 
   References:
-   The Conjugate Gradient Method and Trust Regions in Large Scale Optimization, Trond Steihaug
-   SIAM Journal on Numerical Analysis, Vol. 20, No. 3 (Jun., 1983), pp. 626-637
+.  1. - Trond Steihaug, The Conjugate Gradient Method and Trust Regions in Large Scale Optimization,
+   SIAM Journal on Numerical Analysis, Vol. 20, No. 3 (Jun., 1983).
 
 .seealso:  KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP, KSPQCGSetTrustRegionRadius()
            KSPQCGGetTrialStepNorm(), KSPQCGGetQuadratic()
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_QCG"
 PETSC_EXTERN PetscErrorCode KSPCreate_QCG(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -432,8 +411,6 @@ PETSC_EXTERN PetscErrorCode KSPCreate_QCG(KSP ksp)
 }
 
 /* ---------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "KSPQCGQuadraticRoots"
 /*
   KSPQCGQuadraticRoots - Computes the roots of the quadratic,
          ||s + step*p|| - delta = 0
