@@ -2,11 +2,8 @@
 /*
             This implements Richardson Iteration.
 */
-#include <petsc/private/kspimpl.h>              /*I "petscksp.h" I*/
-#include <../src/ksp/ksp/impls/rich/richardsonimpl.h>
+#include <../src/ksp/ksp/impls/rich/richardsonimpl.h>     /*I "petscksp.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_Richardson"
 PetscErrorCode KSPSetUp_Richardson(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -21,8 +18,6 @@ PetscErrorCode KSPSetUp_Richardson(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_Richardson"
 PetscErrorCode  KSPSolve_Richardson(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -65,17 +60,15 @@ PetscErrorCode  KSPSolve_Richardson(KSP ksp)
   /* if user has provided fast Richardson code use that */
   ierr = PCApplyRichardsonExists(ksp->pc,&exists);CHKERRQ(ierr);
   ierr = MatGetNullSpace(Pmat,&nullsp);CHKERRQ(ierr);
-  if (exists && richardsonP->scale == 1.0 && !ksp->numbermonitors && !ksp->transpose_solve & !nullsp) {
+  if (exists && maxit > 0 && richardsonP->scale == 1.0 && !ksp->numbermonitors && !ksp->transpose_solve && !nullsp) {
     PCRichardsonConvergedReason reason;
     ierr        = PCApplyRichardson(ksp->pc,b,x,r,ksp->rtol,ksp->abstol,ksp->divtol,maxit,ksp->guess_zero,&ksp->its,&reason);CHKERRQ(ierr);
     ksp->reason = (KSPConvergedReason)reason;
     PetscFunctionReturn(0);
-  } else if (exists && !ksp->numbermonitors && !ksp->transpose_solve & !nullsp) {
+  } else if (exists && maxit > 0 && !ksp->numbermonitors && !ksp->transpose_solve && !nullsp) {
     ierr  = PetscInfo(ksp,"KSPSolve_Richardson: Warning, skipping optimized PCApplyRichardson() because scale factor is not 1.0\n");CHKERRQ(ierr);
   }
 
-  scale = richardsonP->scale;
-
   if (!ksp->guess_zero) {                          /*   r <- b - A x     */
     ierr = KSP_MatMult(ksp,Amat,x,r);CHKERRQ(ierr);
     ierr = VecAYPX(r,-1.0,b);CHKERRQ(ierr);
@@ -135,7 +128,7 @@ PetscErrorCode  KSPSolve_Richardson(KSP ksp)
         if (ksp->reason) break;
       }
 
-      ierr = VecAXPY(x,scale,z);CHKERRQ(ierr);    /*   x  <- x + scale z */
+      ierr = VecAXPY(x,richardsonP->scale,z);CHKERRQ(ierr);    /*   x  <- x + scale z */
       ksp->its++;
 
       if (i+1 < maxit || ksp->normtype != KSP_NORM_NONE) {
@@ -168,8 +161,6 @@ PetscErrorCode  KSPSolve_Richardson(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPView_Richardson"
 PetscErrorCode KSPView_Richardson(KSP ksp,PetscViewer viewer)
 {
   KSP_Richardson *richardsonP = (KSP_Richardson*)ksp->data;
@@ -180,17 +171,15 @@ PetscErrorCode KSPView_Richardson(KSP ksp,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
     if (richardsonP->selfscale) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  Richardson: using self-scale best computed damping factor\n");CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  using self-scale best computed damping factor\n");CHKERRQ(ierr);
     } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"  Richardson: damping factor=%g\n",(double)richardsonP->scale);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  damping factor=%g\n",(double)richardsonP->scale);CHKERRQ(ierr);
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetFromOptions_Richardson"
-PetscErrorCode KSPSetFromOptions_Richardson(PetscOptions *PetscOptionsObject,KSP ksp)
+PetscErrorCode KSPSetFromOptions_Richardson(PetscOptionItems *PetscOptionsObject,KSP ksp)
 {
   KSP_Richardson *rich = (KSP_Richardson*)ksp->data;
   PetscErrorCode ierr;
@@ -207,8 +196,6 @@ PetscErrorCode KSPSetFromOptions_Richardson(PetscOptions *PetscOptionsObject,KSP
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDestroy_Richardson"
 PetscErrorCode KSPDestroy_Richardson(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -219,8 +206,6 @@ PetscErrorCode KSPDestroy_Richardson(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPRichardsonSetScale_Richardson"
 static PetscErrorCode  KSPRichardsonSetScale_Richardson(KSP ksp,PetscReal scale)
 {
   KSP_Richardson *richardsonP;
@@ -231,8 +216,6 @@ static PetscErrorCode  KSPRichardsonSetScale_Richardson(KSP ksp,PetscReal scale)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPRichardsonSetSelfScale_Richardson"
 static PetscErrorCode  KSPRichardsonSetSelfScale_Richardson(KSP ksp,PetscBool selfscale)
 {
   KSP_Richardson *richardsonP;
@@ -251,13 +234,15 @@ static PetscErrorCode  KSPRichardsonSetSelfScale_Richardson(KSP ksp,PetscBool se
 
    Level: beginner
 
-   Notes: x^{n+1} = x^{n} + scale*B(b - A x^{n})
+   Notes:
+    x^{n+1} = x^{n} + scale*B(b - A x^{n})
 
           Here B is the application of the preconditioner
 
           This method often (usually) will not converge unless scale is very small.
 
-   Notes: For some preconditioners, currently SOR, the convergence test is skipped to improve speed,
+   Notes:
+    For some preconditioners, currently SOR, the convergence test is skipped to improve speed,
     thus it always iterates the maximum number of iterations you've selected. When -ksp_monitor
     (or any other monitor) is turned on, the norm is computed at each iteration and so the convergence test is run unless
     you specifically call KSPSetNormType(ksp,KSP_NORM_NONE);
@@ -269,18 +254,16 @@ static PetscErrorCode  KSPRichardsonSetSelfScale_Richardson(KSP ksp,PetscBool se
     Supports only left preconditioning
 
   References:
-  "The Approximate Arithmetical Solution by Finite Differences of Physical Problems Involving
+.  1. - L. F. Richardson, "The Approximate Arithmetical Solution by Finite Differences of Physical Problems Involving
    Differential Equations, with an Application to the Stresses in a Masonry Dam",
-  L. F. Richardson, Philosophical Transactions of the Royal Society of London. Series A,
-  Containing Papers of a Mathematical or Physical Character, Vol. 210, 1911 (1911), pp. 307-357.
+  Philosophical Transactions of the Royal Society of London. Series A,
+  Containing Papers of a Mathematical or Physical Character, Vol. 210, 1911 (1911).
 
 .seealso:  KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP,
            KSPRichardsonSetScale()
 
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_Richardson"
 PETSC_EXTERN PetscErrorCode KSPCreate_Richardson(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -292,6 +275,7 @@ PETSC_EXTERN PetscErrorCode KSPCreate_Richardson(KSP ksp)
 
   ierr = KSPSetSupportedNorm(ksp,KSP_NORM_PRECONDITIONED,PC_LEFT,3);CHKERRQ(ierr);
   ierr = KSPSetSupportedNorm(ksp,KSP_NORM_UNPRECONDITIONED,PC_LEFT,2);CHKERRQ(ierr);
+  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_NONE,PC_LEFT,1);CHKERRQ(ierr);  
 
   ksp->ops->setup          = KSPSetUp_Richardson;
   ksp->ops->solve          = KSPSolve_Richardson;
