@@ -1,12 +1,10 @@
 
-#include <petsc-private/kspimpl.h>
+#include <petsc/private/kspimpl.h>
 
 typedef struct {
   PetscReal haptol;
 } KSP_SYMMLQ;
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_SYMMLQ"
 PetscErrorCode KSPSetUp_SYMMLQ(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -16,8 +14,6 @@ PetscErrorCode KSPSetUp_SYMMLQ(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_SYMMLQ"
 PetscErrorCode  KSPSolve_SYMMLQ(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -28,7 +24,6 @@ PetscErrorCode  KSPSolve_SYMMLQ(KSP ksp)
   PetscReal      np,s_prod;
   Vec            X,B,R,Z,U,V,W,UOLD,VOLD,Wbar;
   Mat            Amat,Pmat;
-  MatStructure   pflag;
   KSP_SYMMLQ     *symmlq = (KSP_SYMMLQ*)ksp->data;
   PetscBool      diagonalscale;
 
@@ -47,7 +42,7 @@ PetscErrorCode  KSPSolve_SYMMLQ(KSP ksp)
   VOLD = ksp->work[6];
   Wbar = ksp->work[7];
 
-  ierr = PCGetOperators(ksp->pc,&Amat,&Pmat,&pflag);CHKERRQ(ierr);
+  ierr = PCGetOperators(ksp->pc,&Amat,&Pmat);CHKERRQ(ierr);
 
   ksp->its = 0;
 
@@ -65,7 +60,7 @@ PetscErrorCode  KSPSolve_SYMMLQ(KSP ksp)
   ierr = KSP_PCApply(ksp,R,Z);CHKERRQ(ierr); /* z  <- B*r       */
   ierr = VecDot(R,Z,&dp);CHKERRQ(ierr);             /* dp = r'*z;      */
   if (PetscAbsScalar(dp) < symmlq->haptol) {
-    ierr        = PetscInfo2(ksp,"Detected happy breakdown %G tolerance %G\n",PetscAbsScalar(dp),symmlq->haptol);CHKERRQ(ierr);
+    ierr        = PetscInfo2(ksp,"Detected happy breakdown %g tolerance %g\n",(double)PetscAbsScalar(dp),(double)symmlq->haptol);CHKERRQ(ierr);
     ksp->rnorm  = 0.0;  /* what should we really put here? */
     ksp->reason = KSP_CONVERGED_HAPPY_BREAKDOWN;  /* bugfix proposed by Lourens (lourens.vanzanen@shell.com) */
     PetscFunctionReturn(0);
@@ -132,7 +127,7 @@ PetscErrorCode  KSPSolve_SYMMLQ(KSP ksp)
     betaold = beta;                                /* beta_k                  */
     ierr    = VecDot(R,Z,&dp);CHKERRQ(ierr);       /* dp <- r'*z;             */
     if (PetscAbsScalar(dp) < symmlq->haptol) {
-      ierr = PetscInfo2(ksp,"Detected happy breakdown %G tolerance %G\n",PetscAbsScalar(dp),symmlq->haptol);CHKERRQ(ierr);
+      ierr = PetscInfo2(ksp,"Detected happy breakdown %g tolerance %g\n",(double)PetscAbsScalar(dp),(double)symmlq->haptol);CHKERRQ(ierr);
       dp   = 0.0;
     }
 
@@ -187,7 +182,8 @@ PetscErrorCode  KSPSolve_SYMMLQ(KSP ksp)
 
    Level: beginner
 
-   Notes: The operator and the preconditioner must be symmetric for this method. The
+   Notes:
+    The operator and the preconditioner must be symmetric for this method. The
           preconditioner must be POSITIVE-DEFINITE.
 
           Supports only left preconditioning.
@@ -196,17 +192,15 @@ PetscErrorCode  KSPSolve_SYMMLQ(KSP ksp)
 
 .seealso: KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_SYMMLQ"
 PETSC_EXTERN PetscErrorCode KSPCreate_SYMMLQ(KSP ksp)
 {
   KSP_SYMMLQ     *symmlq;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_PRECONDITIONED,PC_LEFT,2);CHKERRQ(ierr);
+  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_PRECONDITIONED,PC_LEFT,3);CHKERRQ(ierr);
 
-  ierr           = PetscNewLog(ksp,KSP_SYMMLQ,&symmlq);CHKERRQ(ierr);
+  ierr           = PetscNewLog(ksp,&symmlq);CHKERRQ(ierr);
   symmlq->haptol = 1.e-18;
   ksp->data      = (void*)symmlq;
 
