@@ -6,11 +6,8 @@
     complex numbers version, so most probably some are incorrect.
 */
 
-#include <petsc/private/kspimpl.h>
 #include <../src/ksp/ksp/impls/tcqmr/tcqmrimpl.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSolve_TCQMR"
 static PetscErrorCode KSPSolve_TCQMR(KSP ksp)
 {
   PetscReal      rnorm0,rnorm,dp1,Gamma;
@@ -140,8 +137,6 @@ static PetscErrorCode KSPSolve_TCQMR(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUp_TCQMR"
 static PetscErrorCode KSPSetUp_TCQMR(KSP ksp)
 {
   PetscErrorCode ierr;
@@ -160,7 +155,8 @@ static PetscErrorCode KSPSetUp_TCQMR(KSP ksp)
 
    Level: beginner
 
-  Notes: Supports either left or right preconditioning, but not symmetric
+  Notes:
+    Supports either left or right preconditioning, but not symmetric
 
           The "residual norm" computed in this algorithm is actually just an upper bound on the actual residual norm.
           That is for left preconditioning it is a bound on the preconditioned residual and for right preconditioning
@@ -174,8 +170,6 @@ static PetscErrorCode KSPSetUp_TCQMR(KSP ksp)
 
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate_TCQMR"
 PETSC_EXTERN PetscErrorCode KSPCreate_TCQMR(KSP ksp)
 {
   PetscErrorCode ierr;
