@@ -4,11 +4,23 @@
 */
 
 #include <petsc/private/kspimpl.h>  /*I "petscksp.h" I*/
+#include <petscdraw.h>
 
-extern PetscBool KSPRegisterAllCalled;
+static PetscErrorCode KSPSetupMonitor_Private(KSP ksp, PetscViewer viewer, PetscViewerFormat format, PetscErrorCode (*monitor)(KSP,PetscInt,PetscReal,void*), PetscBool useMonitor)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (useMonitor) {
+    PetscViewerAndFormat *vf;
+
+    ierr = PetscViewerAndFormatCreate(viewer, format, &vf);CHKERRQ(ierr);
+    ierr = PetscObjectDereference((PetscObject) viewer);CHKERRQ(ierr);
+    ierr = KSPMonitorSet(ksp, monitor, vf, (PetscErrorCode (*)(void**)) PetscViewerAndFormatDestroy);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetOptionsPrefix"
 /*@C
    KSPSetOptionsPrefix - Sets the prefix used for searching for all
    KSP options in the database.
@@ -55,8 +67,6 @@ PetscErrorCode  KSPSetOptionsPrefix(KSP ksp,const char prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPAppendOptionsPrefix"
 /*@C
    KSPAppendOptionsPrefix - Appends to the prefix used for searching for all
    KSP options in the database.
@@ -89,8 +99,6 @@ PetscErrorCode  KSPAppendOptionsPrefix(KSP ksp,const char prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGetTabLevel"
 /*@
    KSPGetTabLevel - Gets the number of tabs that ASCII output used by ksp.
 
@@ -104,7 +112,8 @@ PetscErrorCode  KSPAppendOptionsPrefix(KSP ksp,const char prefix[])
 
    Level: developer
 
-    Notes: this is used in conjunction with KSPSetTabLevel() to manage the output from the KSP and its PC coherently.
+    Notes:
+    this is used in conjunction with KSPSetTabLevel() to manage the output from the KSP and its PC coherently.
 
 
 .seealso:  KSPSetTabLevel()
@@ -120,8 +129,6 @@ PetscErrorCode  KSPGetTabLevel(KSP ksp,PetscInt *tab)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetTabLevel"
 /*@
    KSPSetTabLevel - Sets the number of tabs that ASCII output for the ksp andn its pc will use.
 
@@ -133,7 +140,8 @@ PetscErrorCode  KSPGetTabLevel(KSP ksp,PetscInt *tab)
 
    Level: developer
 
-    Notes: this is used to manage the output from KSP and PC objects that are imbedded in other objects,
+    Notes:
+    this is used to manage the output from KSP and PC objects that are imbedded in other objects,
            for example, the KSP object inside a SNES object. By indenting each lower level further the heirarchy
            of objects is very clear.  By setting the KSP object's tab level with KSPSetTabLevel() its PC object
            automatically receives the same tab level, so that whatever objects the pc might create are tabbed
@@ -154,16 +162,14 @@ PetscErrorCode  KSPSetTabLevel(KSP ksp, PetscInt tab)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUseFischerGuess"
-/*@C
-   KSPSetUseFischerGuess - Use the Paul Fischer algorithm, see KSPFischerGuessCreate()
+/*@
+   KSPSetUseFischerGuess - Use the Paul Fischer algorithm
 
    Logically Collective on KSP
 
    Input Parameters:
 +  ksp - the Krylov context
-.  model - use model 1, model 2 or 0 to turn it off
+.  model - use model 1, model 2 or any other number to turn it off
 -  size - size of subspace used to generate initial guess
 
     Options Database:
@@ -173,63 +179,61 @@ PetscErrorCode  KSPSetTabLevel(KSP ksp, PetscInt tab)
 
 .keywords: KSP, set, options, prefix, database
 
-.seealso: KSPSetOptionsPrefix(), KSPAppendOptionsPrefix(), KSPSetUseFischerGuess(), KSPSetFischerGuess(), KSPGetFischerInitialGuess()
+.seealso: KSPSetOptionsPrefix(), KSPAppendOptionsPrefix(), KSPSetUseFischerGuess(), KSPSetGuess(), KSPGetGuess()
 @*/
 PetscErrorCode  KSPSetUseFischerGuess(KSP ksp,PetscInt model,PetscInt size)
 {
+  KSPGuess       guess;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
   PetscValidLogicalCollectiveInt(ksp,model,2);
-  PetscValidLogicalCollectiveInt(ksp,model,3);
-  ierr = KSPFischerGuessDestroy(&ksp->guess);CHKERRQ(ierr);
-  if (model == 1 || model == 2) {
-    ierr = KSPFischerGuessCreate(ksp,model,size,&ksp->guess);CHKERRQ(ierr);
-    ierr = KSPFischerGuessSetFromOptions(ksp->guess);CHKERRQ(ierr);
-  } else if (model != 0) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_ARG_OUTOFRANGE,"Model must be 1 or 2 (or 0 to turn off guess generation)");
+  PetscValidLogicalCollectiveInt(ksp,size,3);
+  ierr = KSPGetGuess(ksp,&guess);CHKERRQ(ierr);
+  ierr = KSPGuessSetType(guess,KSPGUESSFISCHER);CHKERRQ(ierr);
+  ierr = KSPGuessFischerSetModel(guess,model,size);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetFischerGuess"
-/*@C
-   KSPSetFischerGuess - Use the Paul Fischer algorithm created by KSPFischerGuessCreate()
+/*@
+   KSPSetGuess - Set the initial guess object
 
    Logically Collective on KSP
 
    Input Parameters:
 +  ksp - the Krylov context
--  guess - the object created with KSPFischerGuessCreate()
+-  guess - the object created with KSPGuessCreate()
 
    Level: advanced
 
-   Notes: this allows a single KSP to be used with several different initial guess generators (likely for different linear
+   Notes:
+    this allows a single KSP to be used with several different initial guess generators (likely for different linear
           solvers, see KSPSetPC()).
 
-          This increases the reference count of the guess object, you must destroy the object with KSPFischerGuessDestroy()
+          This increases the reference count of the guess object, you must destroy the object with KSPGuessDestroy()
           before the end of the program.
 
 .keywords: KSP, set, options, prefix, database
 
-.seealso: KSPSetOptionsPrefix(), KSPAppendOptionsPrefix(), KSPSetUseFischerGuess(), KSPSetFischerGuess(), KSPGetFischerGuess()
+.seealso: KSPSetOptionsPrefix(), KSPAppendOptionsPrefix(), KSPSetUseFischerGuess(), KSPGetGuess()
 @*/
-PetscErrorCode  KSPSetFischerGuess(KSP ksp,KSPFischerGuess guess)
+PetscErrorCode  KSPSetGuess(KSP ksp,KSPGuess guess)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
-  ierr       = KSPFischerGuessDestroy(&ksp->guess);CHKERRQ(ierr);
+  PetscValidHeaderSpecific(guess,KSPGUESS_CLASSID,2);
+  ierr = PetscObjectReference((PetscObject)guess);CHKERRQ(ierr);
+  ierr = KSPGuessDestroy(&ksp->guess);CHKERRQ(ierr);
   ksp->guess = guess;
-  if (guess) guess->refcnt++;
+  ksp->guess->ksp = ksp;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGetFischerGuess"
-/*@C
-   KSPGetFischerGuess - Gets the initial guess generator set with either KSPSetFischerGuess() or KSPCreateFischerGuess()/KSPSetFischerGuess()
+/*@
+   KSPGetGuess - Gets the initial guess generator for the KSP.
 
    Not Collective
 
@@ -243,17 +247,29 @@ PetscErrorCode  KSPSetFischerGuess(KSP ksp,KSPFischerGuess guess)
 
 .keywords: KSP, set, options, prefix, database
 
-.seealso: KSPSetOptionsPrefix(), KSPAppendOptionsPrefix(), KSPSetUseFischerGuess(), KSPSetFischerGuess()
+.seealso: KSPSetOptionsPrefix(), KSPAppendOptionsPrefix(), KSPSetUseFischerGuess(), KSPSetGuess()
 @*/
-PetscErrorCode  KSPGetFischerGuess(KSP ksp,KSPFischerGuess *guess)
+PetscErrorCode  KSPGetGuess(KSP ksp,KSPGuess *guess)
 {
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
+  PetscValidPointer(guess,2);
+  if (!ksp->guess) {
+    const char* prefix;
+
+    ierr = KSPGuessCreate(PetscObjectComm((PetscObject)ksp),&ksp->guess);CHKERRQ(ierr);
+    ierr = PetscObjectGetOptionsPrefix((PetscObject)ksp,&prefix);CHKERRQ(ierr);
+    if (prefix) {
+      ierr = PetscObjectSetOptionsPrefix((PetscObject)ksp->guess,prefix);CHKERRQ(ierr);
+    }
+    ksp->guess->ksp = ksp;
+  }
   *guess = ksp->guess;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGetOptionsPrefix"
 /*@C
    KSPGetOptionsPrefix - Gets the prefix used for searching for all
    KSP options in the database.
@@ -266,7 +282,8 @@ PetscErrorCode  KSPGetFischerGuess(KSP ksp,KSPFischerGuess *guess)
    Output Parameters:
 .  prefix - pointer to the prefix string used is returned
 
-   Notes: On the fortran side, the user should pass in a string 'prifix' of
+   Notes:
+    On the fortran side, the user should pass in a string 'prefix' of
    sufficient length to hold the prefix.
 
    Level: advanced
@@ -285,8 +302,41 @@ PetscErrorCode  KSPGetOptionsPrefix(KSP ksp,const char *prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetFromOptions"
+/*@C
+   KSPMonitorSetFromOptions - Sets a monitor function and viewer appropriate for the type indicated by the user
+
+   Collective on KSP
+
+   Input Parameters:
++  ksp - KSP object you wish to monitor
+.  name - the monitor type one is seeking
+.  help - message indicating what monitoring is done
+.  manual - manual page for the monitor
+-  monitor - the monitor function, the context for this object is a PetscViewerAndFormat
+
+   Level: developer
+
+.seealso: PetscOptionsGetViewer(), PetscOptionsGetReal(), PetscOptionsHasName(), PetscOptionsGetString(),
+          PetscOptionsGetIntArray(), PetscOptionsGetRealArray(), PetscOptionsBool()
+          PetscOptionsInt(), PetscOptionsString(), PetscOptionsReal(), PetscOptionsBool(),
+          PetscOptionsName(), PetscOptionsBegin(), PetscOptionsEnd(), PetscOptionsHead(),
+          PetscOptionsStringArray(),PetscOptionsRealArray(), PetscOptionsScalar(),
+          PetscOptionsBoolGroupBegin(), PetscOptionsBoolGroup(), PetscOptionsBoolGroupEnd(),
+          PetscOptionsFList(), PetscOptionsEList()
+@*/
+PetscErrorCode  KSPMonitorSetFromOptions(KSP ksp,const char name[],const char help[], const char manual[],PetscErrorCode (*monitor)(KSP,PetscInt,PetscReal,PetscViewerAndFormat*))
+{
+  PetscErrorCode       ierr;
+  PetscBool            flg;
+  PetscViewer          viewer;
+  PetscViewerFormat    format;
+
+  PetscFunctionBegin;
+  ierr = PetscOptionsGetViewer(PetscObjectComm((PetscObject)ksp),((PetscObject)ksp)->prefix,name,&viewer,&format,&flg);CHKERRQ(ierr);
+  ierr = KSPSetupMonitor_Private(ksp, viewer, format, (PetscErrorCode (*)(KSP,PetscInt,PetscReal,void*)) monitor, flg);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    KSPSetFromOptions - Sets KSP options from the options database.
    This routine must be called before KSPSetUp() if the user is to be
@@ -317,6 +367,7 @@ PetscErrorCode  KSPGetOptionsPrefix(KSP ksp,const char *prefix[])
 .   -ksp_lag_norm - compute the norm of the residual for the ith iteration on the i+1 iteration; this means that one can use
        the norm of the residual for convergence test WITHOUT an extra MPI_Allreduce() limiting global synchronizations.
        This will require 1 more iteration of the solver than usual.
+.   -ksp_guess_type - Type of initial guess generator for repeated linear solves
 .   -ksp_fischer_guess <model,size> - uses the Fischer initial guess generator for repeated linear solves
 .   -ksp_constant_null_space - assume the operator (matrix) has the constant vector in its null space
 .   -ksp_test_null_space - tests the null space set with MatSetNullSpace() to see if it truly is a null space
@@ -324,7 +375,7 @@ PetscErrorCode  KSPGetOptionsPrefix(KSP ksp,const char *prefix[])
 .   -ksp_monitor_cancel - cancel all previous convergene monitor routines set
 .   -ksp_monitor <optional filename> - print residual norm at each iteration
 .   -ksp_monitor_lg_residualnorm - plot residual norm at each iteration
-.   -ksp_monitor_solution - plot solution at each iteration
+.   -ksp_monitor_solution [ascii binary or draw][:filename][:format option] - plot solution at each iteration
 -   -ksp_monitor_singular_value - monitor extreme singular values at each iteration
 
    Notes:
@@ -335,24 +386,27 @@ PetscErrorCode  KSPGetOptionsPrefix(KSP ksp,const char *prefix[])
 
 .keywords: KSP, set, from, options, database
 
-.seealso: KSPSetUseFischerGuess()
+.seealso: KSPSetOptionsPrefix(), KSPResetFromOptions(), KSPSetUseFischerGuess()
 
 @*/
 PetscErrorCode  KSPSetFromOptions(KSP ksp)
 {
-  PetscErrorCode ierr;
   PetscInt       indx;
-  const char     *convtests[] = {"default","skip"};
-  char           type[256], monfilename[PETSC_MAX_PATH_LEN];
-  PetscViewer    monviewer;
+  const char     *convtests[] = {"default","skip","lsqr"};
+  char           type[256], guesstype[256], monfilename[PETSC_MAX_PATH_LEN];
   PetscBool      flg,flag,reuse,set;
   PetscInt       model[2]={0,0},nmax;
   KSPNormType    normtype;
   PCSide         pcside;
   void           *ctx;
+  MPI_Comm       comm;
+  const char    *prefix;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
+  ierr = PetscObjectGetComm((PetscObject) ksp, &comm);CHKERRQ(ierr);
+  ierr = PetscObjectGetOptionsPrefix((PetscObject) ksp, &prefix);CHKERRQ(ierr);
   if (!ksp->skippcsetfromoptions) {
     if (!ksp->pc) {ierr = KSPGetPC(ksp,&ksp->pc);CHKERRQ(ierr);}
     ierr = PCSetFromOptions(ksp->pc);CHKERRQ(ierr);
@@ -372,14 +426,17 @@ PetscErrorCode  KSPSetFromOptions(KSP ksp)
   }
 
   ierr = PetscObjectTypeCompare((PetscObject)ksp,KSPPREONLY,&flg);CHKERRQ(ierr);
-  if (flg) goto skipoptions;
+  if (flg) {
+    ierr = PetscOptionsBool("-ksp_error_if_not_converged","Generate error if solver does not converge","KSPSetErrorIfNotConverged",ksp->errorifnotconverged,&ksp->errorifnotconverged,NULL);CHKERRQ(ierr);
+    goto skipoptions;
+  }
 
   ierr = PetscOptionsInt("-ksp_max_it","Maximum number of iterations","KSPSetTolerances",ksp->max_it,&ksp->max_it,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsReal("-ksp_rtol","Relative decrease in residual norm","KSPSetTolerances",ksp->rtol,&ksp->rtol,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsReal("-ksp_atol","Absolute value of residual norm","KSPSetTolerances",ksp->abstol,&ksp->abstol,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsReal("-ksp_divtol","Residual norm increase cause divergence","KSPSetTolerances",ksp->divtol,&ksp->divtol,NULL);CHKERRQ(ierr);
 
-  ierr = PetscOptionsBool("-ksp_converged_use_initial_residual_norm","Use initial residual residual norm for computing relative convergence","KSPConvergedDefaultSetUIRNorm",PETSC_FALSE,&flag,&set);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-ksp_converged_use_initial_residual_norm","Use initial residual norm for computing relative convergence","KSPConvergedDefaultSetUIRNorm",PETSC_FALSE,&flag,&set);CHKERRQ(ierr);
   if (set && flag) {ierr = KSPConvergedDefaultSetUIRNorm(ksp);CHKERRQ(ierr);}
   ierr = PetscOptionsBool("-ksp_converged_use_min_initial_residual_norm","Use minimum of initial residual norm and b for computing relative convergence","KSPConvergedDefaultSetUMIRNorm",PETSC_FALSE,&flag,&set);CHKERRQ(ierr);
   if (set && flag) {ierr = KSPConvergedDefaultSetUMIRNorm(ksp);CHKERRQ(ierr);}
@@ -393,14 +450,21 @@ PetscErrorCode  KSPSetFromOptions(KSP ksp)
 
   ierr = PetscOptionsBool("-ksp_knoll","Use preconditioner applied to b for initial guess","KSPSetInitialGuessKnoll",ksp->guess_knoll,&ksp->guess_knoll,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsBool("-ksp_error_if_not_converged","Generate error if solver does not converge","KSPSetErrorIfNotConverged",ksp->errorifnotconverged,&ksp->errorifnotconverged,NULL);CHKERRQ(ierr);
-  nmax = 2;
-  ierr = PetscOptionsIntArray("-ksp_fischer_guess","Use Paul Fischer's algorithm for initial guess","KSPSetUseFischerGuess",model,&nmax,&flag);CHKERRQ(ierr);
-  if (flag) {
-    if (nmax != 2) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_ARG_OUTOFRANGE,"Must pass in model,size as arguments");
-    ierr = KSPSetUseFischerGuess(ksp,model[0],model[1]);CHKERRQ(ierr);
+  ierr = PetscOptionsFList("-ksp_guess_type","Initial guess in Krylov method",NULL,KSPGuessList,NULL,guesstype,256,&flg);CHKERRQ(ierr);
+  if (flg) {
+    ierr = KSPGetGuess(ksp,&ksp->guess);CHKERRQ(ierr);
+    ierr = KSPGuessSetType(ksp->guess,guesstype);CHKERRQ(ierr);
+    ierr = KSPGuessSetFromOptions(ksp->guess);CHKERRQ(ierr);
+  } else { /* old option for KSP */
+    nmax = 2;
+    ierr = PetscOptionsIntArray("-ksp_fischer_guess","Use Paul Fischer's algorithm for initial guess","KSPSetUseFischerGuess",model,&nmax,&flag);CHKERRQ(ierr);
+    if (flag) {
+      if (nmax != 2) SETERRQ(comm,PETSC_ERR_ARG_OUTOFRANGE,"Must pass in model,size as arguments");
+      ierr = KSPSetUseFischerGuess(ksp,model[0],model[1]);CHKERRQ(ierr);
+    }
   }
 
-  ierr = PetscOptionsEList("-ksp_convergence_test","Convergence test","KSPSetConvergenceTest",convtests,2,"default",&indx,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsEList("-ksp_convergence_test","Convergence test","KSPSetConvergenceTest",convtests,3,"default",&indx,&flg);CHKERRQ(ierr);
   if (flg) {
     switch (indx) {
     case 0:
@@ -408,10 +472,14 @@ PetscErrorCode  KSPSetFromOptions(KSP ksp)
       ierr = KSPSetConvergenceTest(ksp,KSPConvergedDefault,ctx,KSPConvergedDefaultDestroy);CHKERRQ(ierr);
       break;
     case 1: ierr = KSPSetConvergenceTest(ksp,KSPConvergedSkip,NULL,NULL);CHKERRQ(ierr);    break;
+    case 2:
+      ierr = KSPConvergedDefaultCreate(&ctx);CHKERRQ(ierr);
+      ierr = KSPSetConvergenceTest(ksp,KSPLSQRConvergedDefault,ctx,KSPConvergedDefaultDestroy);CHKERRQ(ierr);
+      break;
     }
   }
 
-  ierr = KSPSetUpNorms_Private(ksp,&normtype,&pcside);CHKERRQ(ierr);
+  ierr = KSPSetUpNorms_Private(ksp,PETSC_FALSE,&normtype,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsEnum("-ksp_norm_type","KSP Norm type","KSPSetNormType",KSPNormTypes,(PetscEnum)normtype,(PetscEnum*)&normtype,&flg);CHKERRQ(ierr);
   if (flg) { ierr = KSPSetNormType(ksp,normtype);CHKERRQ(ierr); }
 
@@ -438,12 +506,12 @@ PetscErrorCode  KSPSetFromOptions(KSP ksp)
     MatNullSpace nsp;
     Mat          Amat;
 
-    ierr = MatNullSpaceCreate(PetscObjectComm((PetscObject)ksp),PETSC_TRUE,0,NULL,&nsp);CHKERRQ(ierr);
+    ierr = MatNullSpaceCreate(comm,PETSC_TRUE,0,NULL,&nsp);CHKERRQ(ierr);
     ierr = PCGetOperators(ksp->pc,&Amat,NULL);CHKERRQ(ierr);
     if (Amat) {
       ierr = MatSetNullSpace(Amat,nsp);CHKERRQ(ierr);
       ierr = MatNullSpaceDestroy(&nsp);CHKERRQ(ierr);
-    } else SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_ARG_WRONGSTATE,"Cannot set nullspace, matrix has not yet been provided");
+    } else SETERRQ(comm,PETSC_ERR_ARG_WRONGSTATE,"Cannot set nullspace, matrix has not yet been provided");
   }
 
   ierr = PetscOptionsBool("-ksp_monitor_cancel","Remove any hardwired monitor routines","KSPMonitorCancel",PETSC_FALSE,&flg,&set);CHKERRQ(ierr);
@@ -454,27 +522,20 @@ PetscErrorCode  KSPSetFromOptions(KSP ksp)
   if (set && flg) {
     ierr = KSPMonitorCancel(ksp);CHKERRQ(ierr);
   }
-  /*
-    Prints preconditioned residual norm at each iteration
-  */
-  ierr = PetscOptionsString("-ksp_monitor","Monitor preconditioned residual norm","KSPMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+  ierr = KSPMonitorSetFromOptions(ksp,"-ksp_monitor","Monitor the (preconditioned) residual norm","KSPMonitorDefault",KSPMonitorDefault);CHKERRQ(ierr);
+  ierr = KSPMonitorSetFromOptions(ksp,"-ksp_monitor_range","Monitor the percentage of large entries in the residual","KSPMonitorRange",KSPMonitorRange);CHKERRQ(ierr);
+  ierr = KSPMonitorSetFromOptions(ksp,"-ksp_monitor_true_residual","Monitor the unprecondiitoned residual norm","KSPMOnitorTrueResidual",KSPMonitorTrueResidualNorm);CHKERRQ(ierr);
+  ierr = KSPMonitorSetFromOptions(ksp,"-ksp_monitor_max","Monitor the maximum norm of the residual","KSPMonitorTrueResidualMaxNorm",KSPMonitorTrueResidualMaxNorm);CHKERRQ(ierr);
+  ierr = KSPMonitorSetFromOptions(ksp,"-ksp_monitor_short","Monitor preconditioned residual norm with fewer digits","KSPMonitorDefaultShort",KSPMonitorDefaultShort);CHKERRQ(ierr);
+  ierr = KSPMonitorSetFromOptions(ksp,"-ksp_monitor_solution","Monitor the solution","KSPMonitorSolution",KSPMonitorSolution);CHKERRQ(ierr);
+  ierr = KSPMonitorSetFromOptions(ksp,"-ksp_monitor_singular_value","Monitor singular values","KSPMonitorSingularValue",KSPMonitorSingularValue);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,((PetscObject)ksp)->prefix,"-ksp_monitor_singular_value",&flg);CHKERRQ(ierr);
   if (flg) {
-#if defined(PETSC_HAVE_THREADSAFETY)
-#pragma omp critical
-#endif
-    ierr = PetscViewerASCIIOpen(PetscObjectComm((PetscObject)ksp),monfilename,&monviewer);CHKERRQ(ierr);
-    ierr = KSPMonitorSet(ksp,KSPMonitorDefault,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
-  }
-  /*
-    Prints preconditioned residual norm at each iteration
-  */
-  ierr = PetscOptionsString("-ksp_monitor_range","Monitor percent of residual entries more than 10 percent of max","KSPMonitorRange","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = PetscViewerASCIIOpen(PetscObjectComm((PetscObject)ksp),monfilename,&monviewer);CHKERRQ(ierr);
-    ierr = KSPMonitorSet(ksp,KSPMonitorRange,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
+    ierr = KSPSetComputeSingularValues(ksp,PETSC_TRUE);CHKERRQ(ierr);
   }
   ierr = PetscObjectTypeCompare((PetscObject)ksp->pc,PCKSP,&flg);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)ksp->pc,PCBJACOBI,&flag);CHKERRQ(ierr);
+
   if (flg || flag) {
     /* A hack for using dynamic tolerance in preconditioner */
     ierr = PetscOptionsString("-sub_ksp_dynamic_tolerance","Use dynamic tolerance for PC if PC is a KSP","KSPMonitorDynamicTolerance","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
@@ -487,46 +548,8 @@ PetscErrorCode  KSPSetFromOptions(KSP ksp)
       ierr        = KSPMonitorSet(ksp,KSPMonitorDynamicTolerance,scale,KSPMonitorDynamicToleranceDestroy);CHKERRQ(ierr);
     }
   }
-  /*
-    Plots the vector solution
-  */
-  ierr = PetscOptionsBool("-ksp_monitor_solution","Monitor solution graphically","KSPMonitorSet",PETSC_FALSE,&flg,&set);CHKERRQ(ierr);
-  if (set && flg) {
-    ierr = KSPMonitorSet(ksp,KSPMonitorSolution,NULL,NULL);CHKERRQ(ierr);
-  }
-  /*
-    Prints preconditioned and true residual norm at each iteration
-  */
-  ierr = PetscOptionsString("-ksp_monitor_true_residual","Monitor true residual norm","KSPMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = PetscViewerASCIIOpen(PetscObjectComm((PetscObject)ksp),monfilename,&monviewer);CHKERRQ(ierr);
-    ierr = KSPMonitorSet(ksp,KSPMonitorTrueResidualNorm,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
-  }
-  /*
-    Prints with max norm at each iteration
-  */
-  ierr = PetscOptionsString("-ksp_monitor_max","Monitor true residual max norm","KSPMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = PetscViewerASCIIOpen(PetscObjectComm((PetscObject)ksp),monfilename,&monviewer);CHKERRQ(ierr);
-    ierr = KSPMonitorSet(ksp,KSPMonitorTrueResidualMaxNorm,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
-  }
-  /*
-    Prints extreme eigenvalue estimates at each iteration
-  */
-  ierr = PetscOptionsString("-ksp_monitor_singular_value","Monitor singular values","KSPMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = KSPSetComputeSingularValues(ksp,PETSC_TRUE);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIOpen(PetscObjectComm((PetscObject)ksp),monfilename,&monviewer);CHKERRQ(ierr);
-    ierr = KSPMonitorSet(ksp,KSPMonitorSingularValue,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
-  }
-  /*
-    Prints preconditioned residual norm with fewer digits
-  */
-  ierr = PetscOptionsString("-ksp_monitor_short","Monitor preconditioned residual norm with fewer digits","KSPMonitorSet","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = PetscViewerASCIIOpen(PetscObjectComm((PetscObject)ksp),monfilename,&monviewer);CHKERRQ(ierr);
-    ierr = KSPMonitorSet(ksp,KSPMonitorDefaultShort,monviewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
-  }
+
+
   /*
    Calls Python function
   */
@@ -537,20 +560,20 @@ PetscErrorCode  KSPSetFromOptions(KSP ksp)
   */
   ierr = PetscOptionsBool("-ksp_monitor_lg_residualnorm","Monitor graphically preconditioned residual norm","KSPMonitorSet",PETSC_FALSE,&flg,&set);CHKERRQ(ierr);
   if (set && flg) {
-    PetscObject *ctx;
+    PetscDrawLG ctx;
 
-    ierr = KSPMonitorLGResidualNormCreate(0,0,PETSC_DECIDE,PETSC_DECIDE,300,300,&ctx);CHKERRQ(ierr);
-    ierr = KSPMonitorSet(ksp,(PetscErrorCode (*)(KSP,PetscInt,PetscReal,void*))KSPMonitorLGResidualNorm,ctx,(PetscErrorCode (*)(void**))KSPMonitorLGResidualNormDestroy);CHKERRQ(ierr);
+    ierr = KSPMonitorLGResidualNormCreate(comm,NULL,NULL,PETSC_DECIDE,PETSC_DECIDE,400,300,&ctx);CHKERRQ(ierr);
+    ierr = KSPMonitorSet(ksp,KSPMonitorLGResidualNorm,ctx,(PetscErrorCode (*)(void**))PetscDrawLGDestroy);CHKERRQ(ierr);
   }
   /*
     Graphically plots preconditioned and true residual norm
   */
   ierr = PetscOptionsBool("-ksp_monitor_lg_true_residualnorm","Monitor graphically true residual norm","KSPMonitorSet",PETSC_FALSE,&flg,&set);CHKERRQ(ierr);
   if (set && flg) {
-    PetscObject *ctx;
+    PetscDrawLG ctx;
 
-    ierr = KSPMonitorLGTrueResidualNormCreate(0,0,PETSC_DECIDE,PETSC_DECIDE,300,300,&ctx);CHKERRQ(ierr);
-    ierr = KSPMonitorSet(ksp,(PetscErrorCode (*)(KSP,PetscInt,PetscReal,void*))KSPMonitorLGTrueResidualNorm,ctx,(PetscErrorCode (*)(void**))KSPMonitorLGTrueResidualNormDestroy);CHKERRQ(ierr);
+    ierr = KSPMonitorLGTrueResidualNormCreate(comm,NULL,NULL,PETSC_DECIDE,PETSC_DECIDE,400,300,&ctx);CHKERRQ(ierr);
+    ierr = KSPMonitorSet(ksp,KSPMonitorLGTrueResidualNorm,ctx,(PetscErrorCode (*)(void**))PetscDrawLGDestroy);CHKERRQ(ierr);
   }
   /*
     Graphically plots preconditioned residual norm and range of residual element values
@@ -559,9 +582,53 @@ PetscErrorCode  KSPSetFromOptions(KSP ksp)
   if (set && flg) {
     PetscViewer ctx;
 
-    ierr = PetscViewerDrawOpen(PetscObjectComm((PetscObject)ksp),0,0,PETSC_DECIDE,PETSC_DECIDE,300,300,&ctx);CHKERRQ(ierr);
+    ierr = PetscViewerDrawOpen(comm,NULL,NULL,PETSC_DECIDE,PETSC_DECIDE,400,300,&ctx);CHKERRQ(ierr);
     ierr = KSPMonitorSet(ksp,KSPMonitorLGRange,ctx,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
   }
+  /* TODO Do these show up in help? */
+  ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_view",                        &ksp->viewer,         &ksp->format,         &ksp->view);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_view_pre",                    &ksp->viewerPre,      &ksp->formatPre,      &ksp->viewPre);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_converged_reason",            &ksp->viewerReason,   &ksp->formatReason,   &ksp->viewReason);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_view_mat",                    &ksp->viewerMat,      &ksp->formatMat,      &ksp->viewMat);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_view_pmat",                   &ksp->viewerPMat,     &ksp->formatPMat,     &ksp->viewPMat);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_view_rhs",                    &ksp->viewerRhs,      &ksp->formatRhs,      &ksp->viewRhs);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_view_solution",               &ksp->viewerSol,      &ksp->formatSol,      &ksp->viewSol);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_view_mat_explicit",           &ksp->viewerMatExp,   &ksp->formatMatExp,   &ksp->viewMatExp);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_view_eigenvalues",            &ksp->viewerEV,       &ksp->formatEV,       &ksp->viewEV);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_view_singularvalues",         &ksp->viewerSV,       &ksp->formatSV,       &ksp->viewSV);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_view_eigenvalues_explicit",   &ksp->viewerEVExp,    &ksp->formatEVExp,    &ksp->viewEVExp);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_view_final_residual",         &ksp->viewerFinalRes, &ksp->formatFinalRes, &ksp->viewFinalRes);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_view_preconditioned_operator_explicit", &ksp->viewerPOpExp, &ksp->formatPOpExp, &ksp->viewPOpExp);CHKERRQ(ierr);
+
+  /* Deprecated options */
+  if (!ksp->viewEV)       {ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_compute_eigenvalues",              &ksp->viewerEV,       &ksp->formatEV,       &ksp->viewEV);CHKERRQ(ierr);}
+  if (!ksp->viewEV)       {
+    ierr = PetscOptionsName("-ksp_plot_eigenvalues", "[deprecated since PETSc 3.9; use -ksp_view_eigenvalues draw]", "KSPView", &ksp->viewEV);CHKERRQ(ierr);
+    if (ksp->viewEV) {
+      ksp->formatEV = PETSC_VIEWER_DEFAULT;
+      ksp->viewerEV = PETSC_VIEWER_DRAW_(comm);
+      ierr = PetscObjectReference((PetscObject) ksp->viewerEV);CHKERRQ(ierr);
+    }
+  }
+  if (!ksp->viewEV)       {
+    ierr = PetscOptionsName("-ksp_plot_eigencontours", "[deprecated since PETSc 3.9; use -ksp_view_eigenvalues draw::draw_contour]", "KSPView", &ksp->viewEV);CHKERRQ(ierr);
+    if (ksp->viewEV) {
+      ksp->formatEV = PETSC_VIEWER_DRAW_CONTOUR;
+      ksp->viewerEV = PETSC_VIEWER_DRAW_(comm);
+      ierr = PetscObjectReference((PetscObject) ksp->viewerEV);CHKERRQ(ierr);
+    }
+  }
+  if (!ksp->viewEVExp)    {ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_compute_eigenvalues_explicitly",   &ksp->viewerEVExp,    &ksp->formatEVExp,    &ksp->viewEVExp);CHKERRQ(ierr);}
+  if (!ksp->viewEVExp)    {
+    ierr = PetscOptionsName("-ksp_plot_eigenvalues_explicitly", "[deprecated since PETSc 3.9; use -ksp_view_eigenvalues_explicit draw]", "KSPView", &ksp->viewEVExp);CHKERRQ(ierr);
+    if (ksp->viewEVExp) {
+      ksp->formatEVExp = PETSC_VIEWER_DEFAULT;
+      ksp->viewerEVExp = PETSC_VIEWER_DRAW_(comm);
+      ierr = PetscObjectReference((PetscObject) ksp->viewerEVExp);CHKERRQ(ierr);
+    }
+  }
+  if (!ksp->viewSV)       {ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_compute_singularvalues",           &ksp->viewerSV,       &ksp->formatSV,       &ksp->viewSV);CHKERRQ(ierr);}
+  if (!ksp->viewFinalRes) {ierr = PetscOptionsGetViewer(comm, prefix, "-ksp_final_residual",                   &ksp->viewerFinalRes, &ksp->formatFinalRes, &ksp->viewFinalRes);CHKERRQ(ierr);}
 
 #if defined(PETSC_HAVE_SAWS)
   /*
@@ -577,7 +644,7 @@ PetscErrorCode  KSPSetFromOptions(KSP ksp)
 #endif
 
   /* -----------------------------------------------------------------------*/
-  ierr = KSPSetUpNorms_Private(ksp,&normtype,&pcside);CHKERRQ(ierr);
+  ierr = KSPSetUpNorms_Private(ksp,PETSC_FALSE,NULL,&pcside);CHKERRQ(ierr);
   ierr = PetscOptionsEnum("-ksp_pc_side","KSP preconditioner side","KSPSetPCSide",PCSides,(PetscEnum)pcside,(PetscEnum*)&pcside,&flg);CHKERRQ(ierr);
   if (flg) {ierr = KSPSetPCSide(ksp,pcside);CHKERRQ(ierr);}
 
@@ -604,7 +671,31 @@ PetscErrorCode  KSPSetFromOptions(KSP ksp)
   }
   skipoptions:
   /* process any options handlers added with PetscObjectAddOptionsHandler() */
-  ierr = PetscObjectProcessOptionsHandlers((PetscObject)ksp);CHKERRQ(ierr);
+  ierr = PetscObjectProcessOptionsHandlers(PetscOptionsObject,(PetscObject)ksp);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
+  ksp->setfromoptionscalled++;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   KSPResetFromOptions - Sets various KSP parameters from user options ONLY if the KSP was previously set from options
+
+   Collective on KSP
+
+   Input Parameter:
+.  ksp - the KSP context
+
+   Level: beginner
+
+.keywords: KSP, linear, set, options, database
+
+.seealso: KSPSetFromOptions(), KSPSetOptionsPrefix()
+@*/
+PetscErrorCode KSPResetFromOptions(KSP ksp)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (ksp->setfromoptionscalled) {ierr = KSPSetFromOptions(ksp);CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
