@@ -7,6 +7,7 @@
 /* Logging support */
 PetscClassId  KSP_CLASSID;
 PetscClassId  DMKSP_CLASSID;
+PetscClassId  KSPGUESS_CLASSID;
 PetscLogEvent KSP_GMRESOrthogonalization, KSP_SetUp, KSP_Solve;
 
 /*
@@ -15,8 +16,6 @@ PetscLogEvent KSP_GMRESOrthogonalization, KSP_SetUp, KSP_Solve;
 PetscFunctionList KSPList              = 0;
 PetscBool         KSPRegisterAllCalled = PETSC_FALSE;
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPLoad"
 /*@C
   KSPLoad - Loads a KSP that has been stored in binary  with KSPView().
 
@@ -73,8 +72,6 @@ PetscErrorCode  KSPLoad(KSP newdm, PetscViewer viewer)
 #if defined(PETSC_HAVE_SAWS)
 #include <petscviewersaws.h>
 #endif
-#undef __FUNCT__
-#define __FUNCT__ "KSPView"
 /*@C
    KSPView - Prints the KSP data structure.
 
@@ -114,7 +111,9 @@ PetscErrorCode  KSPView(KSP ksp,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)ksp));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)ksp),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(ksp,1,viewer,2);
 
@@ -134,7 +133,7 @@ PetscErrorCode  KSPView(KSP ksp,PetscViewer viewer)
     if (ksp->guess_zero) {
       ierr = PetscViewerASCIIPrintf(viewer,"  maximum iterations=%D, initial guess is zero\n",ksp->max_it);CHKERRQ(ierr);
     } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"  maximum iterations=%D\n", ksp->max_it);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  maximum iterations=%D, nonzero initial guess\n", ksp->max_it);CHKERRQ(ierr);
     }
     if (ksp->guess_knoll) {ierr = PetscViewerASCIIPrintf(viewer,"  using preconditioner applied to right hand side for initial guess\n");CHKERRQ(ierr);}
     ierr = PetscViewerASCIIPrintf(viewer,"  tolerances:  relative=%g, absolute=%g, divergence=%g\n",(double)ksp->rtol,(double)ksp->abstol,(double)ksp->divtol);CHKERRQ(ierr);
@@ -145,9 +144,12 @@ PetscErrorCode  KSPView(KSP ksp,PetscViewer viewer)
     } else {
       ierr = PetscViewerASCIIPrintf(viewer,"  left preconditioning\n");CHKERRQ(ierr);
     }
-    if (ksp->guess) {ierr = PetscViewerASCIIPrintf(viewer,"  using Fischers initial guess method %D with size %D\n",ksp->guess->method,ksp->guess->maxl);CHKERRQ(ierr);}
+    if (ksp->guess) {
+      ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
+      ierr = KSPGuessView(ksp->guess,viewer);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
+    }
     if (ksp->dscale) {ierr = PetscViewerASCIIPrintf(viewer,"  diagonally scaled system\n");CHKERRQ(ierr);}
-    if (!ksp->guess_zero) {ierr = PetscViewerASCIIPrintf(viewer,"  using nonzero initial guess\n");CHKERRQ(ierr);}
     ierr = PetscViewerASCIIPrintf(viewer,"  using %s norm type for convergence test\n",KSPNormTypes[ksp->normtype]);CHKERRQ(ierr);
   } else if (isbinary) {
     PetscInt    classid = KSP_FILE_CLASSID;
@@ -175,8 +177,8 @@ PetscErrorCode  KSPView(KSP ksp,PetscViewer viewer)
     ierr = PetscDrawGetCurrentPoint(draw,&x,&y);CHKERRQ(ierr);
     ierr = PetscObjectTypeCompare((PetscObject)ksp,KSPPREONLY,&flg);CHKERRQ(ierr);
     if (!flg) {
-      ierr   = PetscStrcpy(str,"KSP: ");CHKERRQ(ierr);
-      ierr   = PetscStrcat(str,((PetscObject)ksp)->type_name);CHKERRQ(ierr);
+      ierr   = PetscStrncpy(str,"KSP: ",sizeof(str));CHKERRQ(ierr);
+      ierr   = PetscStrlcat(str,((PetscObject)ksp)->type_name,sizeof(str));CHKERRQ(ierr);
       ierr   = PetscDrawStringBoxed(draw,x,y,PETSC_DRAW_RED,PETSC_DRAW_BLACK,str,NULL,&h);CHKERRQ(ierr);
       bottom = y - h;
     } else {
@@ -219,8 +221,6 @@ PetscErrorCode  KSPView(KSP ksp,PetscViewer viewer)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetNormType"
 /*@
    KSPSetNormType - Sets the norm that is used for convergence testing.
 
@@ -249,31 +249,21 @@ $   KSP_NORM_NATURAL - supported  by KSPCG, KSPCR, KSPCGNE, KSPCGS
    Developer Notes:
    Supported combinations of norm and preconditioner side are set using KSPSetSupportedNorm().
 
-
    Level: advanced
 
 .keywords: KSP, create, context, norms
 
-.seealso: KSPSetUp(), KSPSolve(), KSPDestroy(), KSPConvergedSkip(), KSPSetCheckNormIteration(), KSPSetPCSide(), KSPGetPCSide()
+.seealso: KSPSetUp(), KSPSolve(), KSPDestroy(), KSPConvergedSkip(), KSPSetCheckNormIteration(), KSPSetPCSide(), KSPGetPCSide(), KSPNormType
 @*/
 PetscErrorCode  KSPSetNormType(KSP ksp,KSPNormType normtype)
 {
-  PetscErrorCode ierr;
-
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
   PetscValidLogicalCollectiveEnum(ksp,normtype,2);
   ksp->normtype = ksp->normtype_set = normtype;
-  if (normtype == KSP_NORM_NONE) {
-    ierr = KSPSetConvergenceTest(ksp,KSPConvergedSkip,0,0);CHKERRQ(ierr);
-    ierr = PetscInfo(ksp,"Warning: setting KSPNormType to skip computing the norm\n\
- KSP convergence test is implicitly set to KSPConvergedSkip\n");CHKERRQ(ierr);
-  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetCheckNormIteration"
 /*@
    KSPSetCheckNormIteration - Sets the first iteration at which the norm of the residual will be
      computed and used in the convergence test.
@@ -306,8 +296,6 @@ PetscErrorCode  KSPSetCheckNormIteration(KSP ksp,PetscInt it)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetLagNorm"
 /*@
    KSPSetLagNorm - Lags the residual norm calculation so that it is computed as part of the MPI_Allreduce() for
    computing the inner products for the next iteration.  This can reduce communication costs at the expense of doing
@@ -344,8 +332,6 @@ PetscErrorCode  KSPSetLagNorm(KSP ksp,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetSupportedNorm"
 /*@
    KSPSetSupportedNorm - Sets a norm and preconditioner side supported by a KSP
 
@@ -355,7 +341,7 @@ PetscErrorCode  KSPSetLagNorm(KSP ksp,PetscBool flg)
 +  ksp - Krylov method
 .  normtype - supported norm type
 .  pcside - preconditioner side that can be used with this norm
--  preference - integer preference for this combination, larger values have higher priority
+-  priority - positive integer preference for this combination; larger values have higher priority
 
    Level: developer
 
@@ -364,10 +350,6 @@ PetscErrorCode  KSPSetLagNorm(KSP ksp,PetscBool flg)
    which norms and preconditioner sides are supported. Users should not need to call this
    function.
 
-   KSP_NORM_NONE is supported by default with all KSP methods and any PC side at priority 1.  If a KSP explicitly does
-   not support KSP_NORM_NONE, it should set this by setting priority=0.  Since defaulting to KSP_NORM_NONE is usually
-   undesirable, more desirable norms should usually have priority 2 or higher.
-
 .seealso: KSPSetNormType(), KSPSetPCSide()
 @*/
 PetscErrorCode KSPSetSupportedNorm(KSP ksp,KSPNormType normtype,PCSide pcside,PetscInt priority)
@@ -379,24 +361,18 @@ PetscErrorCode KSPSetSupportedNorm(KSP ksp,KSPNormType normtype,PCSide pcside,Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPNormSupportTableReset_Private"
 PetscErrorCode KSPNormSupportTableReset_Private(KSP ksp)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   ierr = PetscMemzero(ksp->normsupporttable,sizeof(ksp->normsupporttable));CHKERRQ(ierr);
-  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_NONE,PC_LEFT,1);CHKERRQ(ierr);
-  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_NONE,PC_RIGHT,1);CHKERRQ(ierr);
   ksp->pc_side  = ksp->pc_side_set;
   ksp->normtype = ksp->normtype_set;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetUpNorms_Private"
-PetscErrorCode KSPSetUpNorms_Private(KSP ksp,KSPNormType *normtype,PCSide *pcside)
+PetscErrorCode KSPSetUpNorms_Private(KSP ksp,PetscBool errorifnotsupported,KSPNormType *normtype,PCSide *pcside)
 {
   PetscInt i,j,best,ibest = 0,jbest = 0;
 
@@ -404,28 +380,24 @@ PetscErrorCode KSPSetUpNorms_Private(KSP ksp,KSPNormType *normtype,PCSide *pcsid
   best = 0;
   for (i=0; i<KSP_NORM_MAX; i++) {
     for (j=0; j<PC_SIDE_MAX; j++) {
-      if ((ksp->normtype == KSP_NORM_DEFAULT || ksp->normtype == i)
-          && (ksp->pc_side == PC_SIDE_DEFAULT || ksp->pc_side == j)
-          && (ksp->normsupporttable[i][j] > best)) {
+      if ((ksp->normtype == KSP_NORM_DEFAULT || ksp->normtype == i) && (ksp->pc_side == PC_SIDE_DEFAULT || ksp->pc_side == j) && (ksp->normsupporttable[i][j] > best)) {
         best  = ksp->normsupporttable[i][j];
         ibest = i;
         jbest = j;
       }
     }
   }
-  if (best < 1) {
+  if (best < 1 && errorifnotsupported) {
     if (ksp->normtype == KSP_NORM_DEFAULT && ksp->pc_side == PC_SIDE_DEFAULT) SETERRQ1(PetscObjectComm((PetscObject)ksp),PETSC_ERR_PLIB,"The %s KSP implementation did not call KSPSetSupportedNorm()",((PetscObject)ksp)->type_name);
     if (ksp->normtype == KSP_NORM_DEFAULT) SETERRQ2(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP,"KSP %s does not support %s",((PetscObject)ksp)->type_name,PCSides[ksp->pc_side]);
     if (ksp->pc_side == PC_SIDE_DEFAULT) SETERRQ2(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP,"KSP %s does not support %s",((PetscObject)ksp)->type_name,KSPNormTypes[ksp->normtype]);
     SETERRQ3(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP,"KSP %s does not support %s with %s",((PetscObject)ksp)->type_name,KSPNormTypes[ksp->normtype],PCSides[ksp->pc_side]);
   }
-  *normtype = (KSPNormType)ibest;
-  *pcside   = (PCSide)jbest;
+  if (normtype) *normtype = (KSPNormType)ibest;
+  if (pcside)   *pcside   = (PCSide)jbest;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGetNormType"
 /*@
    KSPGetNormType - Gets the norm that is used for convergence testing.
 
@@ -450,7 +422,7 @@ PetscErrorCode  KSPGetNormType(KSP ksp, KSPNormType *normtype)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
   PetscValidPointer(normtype,2);
-  ierr      = KSPSetUpNorms_Private(ksp,&ksp->normtype,&ksp->pc_side);CHKERRQ(ierr);
+  ierr      = KSPSetUpNorms_Private(ksp,PETSC_TRUE,&ksp->normtype,&ksp->pc_side);CHKERRQ(ierr);
   *normtype = ksp->normtype;
   PetscFunctionReturn(0);
 }
@@ -459,8 +431,6 @@ PetscErrorCode  KSPGetNormType(KSP ksp, KSPNormType *normtype)
 #include <petscviewersaws.h>
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetOperators"
 /*@
    KSPSetOperators - Sets the matrix associated with the linear system
    and a (possibly) different one associated with the preconditioner.
@@ -541,14 +511,9 @@ PetscErrorCode  KSPSetOperators(KSP ksp,Mat Amat,Mat Pmat)
   if (!ksp->pc) {ierr = KSPGetPC(ksp,&ksp->pc);CHKERRQ(ierr);}
   ierr = PCSetOperators(ksp->pc,Amat,Pmat);CHKERRQ(ierr);
   if (ksp->setupstage == KSP_SETUP_NEWRHS) ksp->setupstage = KSP_SETUP_NEWMATRIX;  /* so that next solve call will call PCSetUp() on new matrix */
-  if (ksp->guess) {
-    ierr = KSPFischerGuessReset(ksp->guess);CHKERRQ(ierr);
-  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGetOperators"
 /*@
    KSPGetOperators - Gets the matrix associated with the linear system
    and a (possibly) different one associated with the preconditioner.
@@ -564,7 +529,8 @@ PetscErrorCode  KSPSetOperators(KSP ksp,Mat Amat,Mat Pmat)
 
     Level: intermediate
 
-   Notes: DOES NOT increase the reference counts of the matrix, so you should NOT destroy them.
+   Notes:
+    DOES NOT increase the reference counts of the matrix, so you should NOT destroy them.
 
 .keywords: KSP, set, get, operators, matrix, preconditioner, linear system
 
@@ -581,8 +547,6 @@ PetscErrorCode  KSPGetOperators(KSP ksp,Mat *Amat,Mat *Pmat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGetOperatorsSet"
 /*@C
    KSPGetOperatorsSet - Determines if the matrix associated with the linear system and
    possibly a different one associated with the preconditioner have been set in the KSP.
@@ -613,8 +577,6 @@ PetscErrorCode  KSPGetOperatorsSet(KSP ksp,PetscBool  *mat,PetscBool  *pmat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetPreSolve"
 /*@C
    KSPSetPreSolve - Sets a function that is called before every KSPSolve() is started
 
@@ -640,8 +602,6 @@ PetscErrorCode  KSPSetPreSolve(KSP ksp,PetscErrorCode (*presolve)(KSP,Vec,Vec,vo
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetPostSolve"
 /*@C
    KSPSetPostSolve - Sets a function that is called after every KSPSolve() completes (whether it converges or not)
 
@@ -667,8 +627,6 @@ PetscErrorCode  KSPSetPostSolve(KSP ksp,PetscErrorCode (*postsolve)(KSP,Vec,Vec,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreate"
 /*@
    KSPCreate - Creates the default KSP context.
 
@@ -725,6 +683,7 @@ PetscErrorCode  KSPCreate(MPI_Comm comm,KSP *inksp)
   ksp->res_hist_max   = 0;
   ksp->res_hist_reset = PETSC_TRUE;
   ksp->numbermonitors = 0;
+  ksp->setfromoptionscalled = 0;
 
   ierr                    = KSPConvergedDefaultCreate(&ctx);CHKERRQ(ierr);
   ierr                    = KSPSetConvergenceTest(ksp,KSPConvergedDefault,ctx,KSPConvergedDefaultDestroy);CHKERRQ(ierr);
@@ -746,8 +705,6 @@ PetscErrorCode  KSPCreate(MPI_Comm comm,KSP *inksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetType"
 /*@C
    KSPSetType - Builds KSP for a particular solver.
 
@@ -791,6 +748,7 @@ PetscErrorCode  KSPSetType(KSP ksp, KSPType type)
 {
   PetscErrorCode ierr,(*r)(KSP);
   PetscBool      match;
+  void           *ctx;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
@@ -808,6 +766,8 @@ PetscErrorCode  KSPSetType(KSP ksp, KSPType type)
   }
   /* Reinitialize function pointers in KSPOps structure */
   ierr                    = PetscMemzero(ksp->ops,sizeof(struct _KSPOps));CHKERRQ(ierr);
+  ierr                    = KSPConvergedDefaultCreate(&ctx);CHKERRQ(ierr);
+  ierr                    = KSPSetConvergenceTest(ksp,KSPConvergedDefault,ctx,KSPConvergedDefaultDestroy);CHKERRQ(ierr);
   ksp->ops->buildsolution = KSPBuildSolutionDefault;
   ksp->ops->buildresidual = KSPBuildResidualDefault;
   ierr                    = KSPNormSupportTableReset_Private(ksp);CHKERRQ(ierr);
@@ -818,8 +778,6 @@ PetscErrorCode  KSPSetType(KSP ksp, KSPType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGetType"
 /*@C
    KSPGetType - Gets the KSP type as a string from the KSP object.
 
@@ -846,8 +804,6 @@ PetscErrorCode  KSPGetType(KSP ksp,KSPType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPRegister"
 /*@C
   KSPRegister -  Adds a method to the Krylov subspace solver package.
 
@@ -882,6 +838,7 @@ PetscErrorCode  KSPRegister(const char sname[],PetscErrorCode (*function)(KSP))
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = KSPInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&KSPList,sname,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
