@@ -8,8 +8,6 @@
 #include <petsc/private/kspimpl.h>   /*I "petscksp.h" I*/
 #include <petscdmshell.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGetResidualNorm"
 /*@
    KSPGetResidualNorm - Gets the last (approximate preconditioned)
    residual norm that has been computed.
@@ -37,8 +35,6 @@ PetscErrorCode  KSPGetResidualNorm(KSP ksp,PetscReal *rnorm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGetIterationNumber"
 /*@
    KSPGetIterationNumber - Gets the current iteration number; if the
          KSPSolve() is complete, returns the number of iterations
@@ -69,8 +65,6 @@ PetscErrorCode  KSPGetIterationNumber(KSP ksp,PetscInt *its)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGetTotalIterations"
 /*@
    KSPGetTotalIterations - Gets the total number of iterations this KSP object has performed since was created, counted over all linear solves
 
@@ -84,7 +78,8 @@ PetscErrorCode  KSPGetIterationNumber(KSP ksp,PetscInt *its)
 
    Level: intermediate
 
-   Notes: Use KSPGetIterationNumber() to get the count for the most recent solve only
+   Notes:
+    Use KSPGetIterationNumber() to get the count for the most recent solve only
    If this is called within a linear solve (such as in a KSPMonitor routine) then it does not include iterations within that current solve
 
 .keywords: KSP, get, residual norm
@@ -100,8 +95,6 @@ PetscErrorCode  KSPGetTotalIterations(KSP ksp,PetscInt *its)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorSingularValue"
 /*@C
     KSPMonitorSingularValue - Prints the two norm of the true residual and
     estimation of the extreme singular values of the preconditioned problem
@@ -128,17 +121,16 @@ PetscErrorCode  KSPGetTotalIterations(KSP ksp,PetscInt *its)
 
 .seealso: KSPComputeExtremeSingularValues()
 @*/
-PetscErrorCode  KSPMonitorSingularValue(KSP ksp,PetscInt n,PetscReal rnorm,void *dummy)
+PetscErrorCode  KSPMonitorSingularValue(KSP ksp,PetscInt n,PetscReal rnorm,PetscViewerAndFormat *dummy)
 {
   PetscReal      emin,emax,c;
   PetscErrorCode ierr;
-  PetscViewer    viewer = (PetscViewer) dummy;
+  PetscViewer    viewer = dummy->viewer;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
-  if (!viewer) {
-    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)ksp),&viewer);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,4);
+  ierr = PetscViewerPushFormat(viewer,dummy->format);CHKERRQ(ierr);
   ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)ksp)->tablevel);CHKERRQ(ierr);
   if (!ksp->calc_sings) {
     ierr = PetscViewerASCIIPrintf(viewer,"%3D KSP Residual norm %14.12e \n",n,(double)rnorm);CHKERRQ(ierr);
@@ -148,11 +140,10 @@ PetscErrorCode  KSPMonitorSingularValue(KSP ksp,PetscInt n,PetscReal rnorm,void
     ierr = PetscViewerASCIIPrintf(viewer,"%3D KSP Residual norm %14.12e %% max %14.12e min %14.12e max/min %14.12e\n",n,(double)rnorm,(double)emax,(double)emin,(double)c);CHKERRQ(ierr);
   }
   ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)ksp)->tablevel);CHKERRQ(ierr);
+  ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorSolution"
 /*@C
    KSPMonitorSolution - Monitors progress of the KSP solvers by calling
    VecView() for the approximate solution at each iteration.
@@ -163,7 +154,7 @@ PetscErrorCode  KSPMonitorSingularValue(KSP ksp,PetscInt n,PetscReal rnorm,void
 +  ksp - the KSP context
 .  its - iteration number
 .  fgnorm - 2-norm of residual (or gradient)
--  dummy - either a viewer or NULL
+-  dummy - a viewer
 
    Level: intermediate
 
@@ -175,29 +166,21 @@ PetscErrorCode  KSPMonitorSingularValue(KSP ksp,PetscInt n,PetscReal rnorm,void
 
 .seealso: KSPMonitorSet(), KSPMonitorDefault(), VecView()
 @*/
-PetscErrorCode  KSPMonitorSolution(KSP ksp,PetscInt its,PetscReal fgnorm,void *dummy)
+PetscErrorCode  KSPMonitorSolution(KSP ksp,PetscInt its,PetscReal fgnorm,PetscViewerAndFormat *dummy)
 {
   PetscErrorCode ierr;
   Vec            x;
-  PetscViewer    viewer = (PetscViewer) dummy;
+  PetscViewer    viewer = dummy->viewer;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   ierr = KSPBuildSolution(ksp,NULL,&x);CHKERRQ(ierr);
-  if (!viewer) {
-    MPI_Comm comm;
-    ierr   = PetscObjectGetComm((PetscObject)ksp,&comm);CHKERRQ(ierr);
-    viewer = PETSC_VIEWER_DRAW_(comm);
-  }
+  ierr = PetscViewerPushFormat(viewer,dummy->format);CHKERRQ(ierr);
   ierr = VecView(x,viewer);CHKERRQ(ierr);
+  ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#if defined(PETSC_HAVE_THREADSAFETY)
-#define KSPMonitorDefault KSPMonitorDefaultUnsafe
-#endif
-
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorDefault"
 /*@C
    KSPMonitorDefault - Print the residual norm at each iteration of an
    iterative solver.
@@ -208,7 +191,7 @@ PetscErrorCode  KSPMonitorSolution(KSP ksp,PetscInt its,PetscReal fgnorm,void *d
 +  ksp   - iterative context
 .  n     - iteration number
 .  rnorm - 2-norm (preconditioned) residual value (may be estimated).
--  dummy - unused monitor context
+-  dummy - an ASCII PetscViewer
 
    Level: intermediate
 
@@ -216,37 +199,24 @@ PetscErrorCode  KSPMonitorSolution(KSP ksp,PetscInt its,PetscReal fgnorm,void *d
 
 .seealso: KSPMonitorSet(), KSPMonitorTrueResidualNorm(), KSPMonitorLGResidualNormCreate()
 @*/
-PetscErrorCode  KSPMonitorDefault(KSP ksp,PetscInt n,PetscReal rnorm,void *dummy)
+PetscErrorCode  KSPMonitorDefault(KSP ksp,PetscInt n,PetscReal rnorm,PetscViewerAndFormat *dummy)
 {
   PetscErrorCode ierr;
-  PetscViewer    viewer = (PetscViewer) dummy;
+  PetscViewer    viewer =  dummy->viewer;
 
   PetscFunctionBegin;
-  if (!viewer) {
-    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)ksp),&viewer);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,4);
+  ierr = PetscViewerPushFormat(viewer,dummy->format);CHKERRQ(ierr);
   ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)ksp)->tablevel);CHKERRQ(ierr);
   if (n == 0 && ((PetscObject)ksp)->prefix) {
     ierr = PetscViewerASCIIPrintf(viewer,"  Residual norms for %s solve.\n",((PetscObject)ksp)->prefix);CHKERRQ(ierr);
   }
   ierr = PetscViewerASCIIPrintf(viewer,"%3D KSP Residual norm %14.12e \n",n,(double)rnorm);CHKERRQ(ierr);
   ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)ksp)->tablevel);CHKERRQ(ierr);
+  ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#if defined(PETSC_HAVE_THREADSAFETY)
-#undef KSPMonitorDefault
-PetscErrorCode KSPMonitorDefault(KSP ksp,PetscInt n,PetscReal rnorm,void *dummy)
-{
-  PetscErrorCode ierr;
-#pragma omp critical
-  ierr = KSPMonitorDefaultUnsafe(ksp,n,rnorm,dummy);
-  return ierr;
-}
-#endif
-
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorTrueResidualNorm"
 /*@C
    KSPMonitorTrueResidualNorm - Prints the true residual norm as well as the preconditioned
    residual norm at each iteration of an iterative solver.
@@ -257,7 +227,7 @@ PetscErrorCode KSPMonitorDefault(KSP ksp,PetscInt n,PetscReal rnorm,void *dummy)
 +  ksp   - iterative context
 .  n     - iteration number
 .  rnorm - 2-norm (preconditioned) residual value (may be estimated).
--  dummy - unused monitor context
+-  dummy - an ASCII PetscViewer
 
    Options Database Key:
 .  -ksp_monitor_true_residual - Activates KSPMonitorTrueResidualNorm()
@@ -271,18 +241,17 @@ PetscErrorCode KSPMonitorDefault(KSP ksp,PetscInt n,PetscReal rnorm,void *dummy)
 
 .seealso: KSPMonitorSet(), KSPMonitorDefault(), KSPMonitorLGResidualNormCreate(),KSPMonitorTrueResidualMaxNorm()
 @*/
-PetscErrorCode  KSPMonitorTrueResidualNorm(KSP ksp,PetscInt n,PetscReal rnorm,void *dummy)
+PetscErrorCode  KSPMonitorTrueResidualNorm(KSP ksp,PetscInt n,PetscReal rnorm,PetscViewerAndFormat *dummy)
 {
   PetscErrorCode ierr;
   Vec            resid;
   PetscReal      truenorm,bnorm;
-  PetscViewer    viewer = (PetscViewer)dummy;
+  PetscViewer    viewer = dummy->viewer;
   char           normtype[256];
 
   PetscFunctionBegin;
-  if (!viewer) {
-    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)ksp),&viewer);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,4);
+  ierr = PetscViewerPushFormat(viewer,dummy->format);CHKERRQ(ierr);
   ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)ksp)->tablevel);CHKERRQ(ierr);
   if (n == 0 && ((PetscObject)ksp)->prefix) {
     ierr = PetscViewerASCIIPrintf(viewer,"  Residual norms for %s solve.\n",((PetscObject)ksp)->prefix);CHKERRQ(ierr);
@@ -295,14 +264,12 @@ PetscErrorCode  KSPMonitorTrueResidualNorm(KSP ksp,PetscInt n,PetscReal rnorm,vo
   ierr = PetscStrtolower(normtype);CHKERRQ(ierr);
   ierr = PetscViewerASCIIPrintf(viewer,"%3D KSP %s resid norm %14.12e true resid norm %14.12e ||r(i)||/||b|| %14.12e\n",n,normtype,(double)rnorm,(double)truenorm,(double)(truenorm/bnorm));CHKERRQ(ierr);
   ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)ksp)->tablevel);CHKERRQ(ierr);
+  ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorTrueResidualMaxNorm"
 /*@C
-   KSPMonitorTrueResidualMaxNorm - Prints the true residual max norm as well as the preconditioned
-   residual norm at each iteration of an iterative solver.
+   KSPMonitorTrueResidualMaxNorm - Prints the true residual max norm each iteration of an iterative solver.
 
    Collective on KSP
 
@@ -310,7 +277,7 @@ PetscErrorCode  KSPMonitorTrueResidualNorm(KSP ksp,PetscInt n,PetscReal rnorm,vo
 +  ksp   - iterative context
 .  n     - iteration number
 .  rnorm - norm (preconditioned) residual value (may be estimated).
--  dummy - unused monitor context
+-  dummy - an ASCII viewer
 
    Options Database Key:
 .  -ksp_monitor_max - Activates KSPMonitorTrueResidualMaxNorm()
@@ -324,18 +291,17 @@ PetscErrorCode  KSPMonitorTrueResidualNorm(KSP ksp,PetscInt n,PetscReal rnorm,vo
 
 .seealso: KSPMonitorSet(), KSPMonitorDefault(), KSPMonitorLGResidualNormCreate(),KSPMonitorTrueResidualNorm()
 @*/
-PetscErrorCode  KSPMonitorTrueResidualMaxNorm(KSP ksp,PetscInt n,PetscReal rnorm,void *dummy)
+PetscErrorCode  KSPMonitorTrueResidualMaxNorm(KSP ksp,PetscInt n,PetscReal rnorm,PetscViewerAndFormat *dummy)
 {
   PetscErrorCode ierr;
   Vec            resid;
   PetscReal      truenorm,bnorm;
-  PetscViewer    viewer = (PetscViewer)dummy;
+  PetscViewer    viewer = dummy->viewer;
   char           normtype[256];
 
   PetscFunctionBegin;
-  if (!viewer) {
-    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)ksp),&viewer);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,4);
+  ierr = PetscViewerPushFormat(viewer,dummy->format);CHKERRQ(ierr);
   ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)ksp)->tablevel);CHKERRQ(ierr);
   if (n == 0 && ((PetscObject)ksp)->prefix) {
     ierr = PetscViewerASCIIPrintf(viewer,"  Residual norms (max) for %s solve.\n",((PetscObject)ksp)->prefix);CHKERRQ(ierr);
@@ -346,14 +312,12 @@ PetscErrorCode  KSPMonitorTrueResidualMaxNorm(KSP ksp,PetscInt n,PetscReal rnorm
   ierr = VecNorm(ksp->vec_rhs,NORM_INFINITY,&bnorm);CHKERRQ(ierr);
   ierr = PetscStrncpy(normtype,KSPNormTypes[ksp->normtype],sizeof(normtype));CHKERRQ(ierr);
   ierr = PetscStrtolower(normtype);CHKERRQ(ierr);
-  /* ierr = PetscViewerASCIIPrintf(viewer,"%3D KSP %s resid norm %14.12e true resid norm %14.12e ||r(i)||_inf/||b||_inf %14.12e\n",n,normtype,(double)rnorm,(double)truenorm,(double)(truenorm/bnorm));CHKERRQ(ierr); */
   ierr = PetscViewerASCIIPrintf(viewer,"%3D KSP true resid max norm %14.12e ||r(i)||/||b|| %14.12e\n",n,(double)truenorm,(double)(truenorm/bnorm));CHKERRQ(ierr);
   ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)ksp)->tablevel);CHKERRQ(ierr);
+  ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorRange_Private"
 PetscErrorCode  KSPMonitorRange_Private(KSP ksp,PetscInt it,PetscReal *per)
 {
   PetscErrorCode ierr;
@@ -372,13 +336,11 @@ PetscErrorCode  KSPMonitorRange_Private(KSP ksp,PetscInt it,PetscReal *per)
   for (i=0; i<n; i++) pwork += (PetscAbsScalar(r[i]) > .20*rmax);
   ierr = VecRestoreArrayRead(resid,&r);CHKERRQ(ierr);
   ierr = VecDestroy(&resid);CHKERRQ(ierr);
-  ierr = MPI_Allreduce(&pwork,per,1,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)ksp));CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(&pwork,per,1,MPIU_REAL,MPIU_SUM,PetscObjectComm((PetscObject)ksp));CHKERRQ(ierr);
   *per = *per/N;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorRange"
 /*@C
    KSPMonitorRange - Prints the percentage of residual elements that are more then 10 percent of the maximum value.
 
@@ -388,7 +350,7 @@ PetscErrorCode  KSPMonitorRange_Private(KSP ksp,PetscInt it,PetscReal *per)
 +  ksp   - iterative context
 .  it    - iteration number
 .  rnorm - 2-norm (preconditioned) residual value (may be estimated).
--  dummy - unused monitor context
+-  dummy - an ASCII viewer
 
    Options Database Key:
 .  -ksp_monitor_range - Activates KSPMonitorRange()
@@ -399,18 +361,17 @@ PetscErrorCode  KSPMonitorRange_Private(KSP ksp,PetscInt it,PetscReal *per)
 
 .seealso: KSPMonitorSet(), KSPMonitorDefault(), KSPMonitorLGResidualNormCreate()
 @*/
-PetscErrorCode  KSPMonitorRange(KSP ksp,PetscInt it,PetscReal rnorm,void *dummy)
+PetscErrorCode  KSPMonitorRange(KSP ksp,PetscInt it,PetscReal rnorm,PetscViewerAndFormat *dummy)
 {
   PetscErrorCode   ierr;
   PetscReal        perc,rel;
-  PetscViewer      viewer = (PetscViewer)dummy;
+  PetscViewer      viewer = dummy->viewer;
   /* should be in a MonitorRangeContext */
   static PetscReal prev;
 
   PetscFunctionBegin;
-  if (!viewer) {
-    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)ksp),&viewer);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,4);
+  ierr = PetscViewerPushFormat(viewer,dummy->format);CHKERRQ(ierr);
   ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)ksp)->tablevel);CHKERRQ(ierr);
   if (!it) prev = rnorm;
   if (it == 0 && ((PetscObject)ksp)->prefix) {
@@ -422,11 +383,10 @@ PetscErrorCode  KSPMonitorRange(KSP ksp,PetscInt it,PetscReal rnorm,void *dummy)
   prev = rnorm;
   ierr = PetscViewerASCIIPrintf(viewer,"%3D KSP preconditioned resid norm %14.12e Percent values above 20 percent of maximum %5.2f relative decrease %5.2e ratio %5.2e \n",it,(double)rnorm,(double)(100.0*perc),(double)rel,(double)(rel/perc));CHKERRQ(ierr);
   ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)ksp)->tablevel);CHKERRQ(ierr);
+  ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorDynamicTolerance"
 /*@C
    KSPMonitorDynamicTolerance - Recompute the inner tolerance in every
    outer iteration in an adaptive way.
@@ -460,7 +420,8 @@ PetscErrorCode KSPMonitorDynamicTolerance(KSP ksp,PetscInt its,PetscReal fnorm,v
   PetscReal      outer_rtol, outer_abstol, outer_dtol, inner_rtol;
   PetscInt       outer_maxits,nksp,first,i;
   KSPDynTolCtx   *scale   = (KSPDynTolCtx*)dummy;
-  KSP            kspinner = NULL, *subksp = NULL;
+  KSP            *subksp = NULL;
+  PetscBool      isksp;
 
   PetscFunctionBegin;
   ierr = KSPGetPC(ksp, &pc);CHKERRQ(ierr);
@@ -476,8 +437,11 @@ PetscErrorCode KSPMonitorDynamicTolerance(KSP ksp,PetscInt its,PetscReal fnorm,v
   /*ierr = PetscPrintf(PETSC_COMM_WORLD, "        Inner rtol = %g\n", (double)inner_rtol);CHKERRQ(ierr);*/
 
   /* if pc is ksp */
-  ierr = PCKSPGetKSP(pc, &kspinner);CHKERRQ(ierr);
-  if (kspinner) {
+  ierr = PetscObjectTypeCompare((PetscObject)pc,PCKSP,&isksp);CHKERRQ(ierr);
+  if (isksp) {
+    KSP kspinner;
+
+    ierr = PCKSPGetKSP(pc, &kspinner);CHKERRQ(ierr);
     ierr = KSPSetTolerances(kspinner, inner_rtol, outer_abstol, outer_dtol, outer_maxits);CHKERRQ(ierr);
     PetscFunctionReturn(0);
   }
@@ -495,8 +459,6 @@ PetscErrorCode KSPMonitorDynamicTolerance(KSP ksp,PetscInt its,PetscReal fnorm,v
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorDynamicToleranceDestroy"
 /*
   Destroy the dummy context used in KSPMonitorDynamicTolerance()
 */
@@ -509,24 +471,23 @@ PetscErrorCode KSPMonitorDynamicToleranceDestroy(void **dummy)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorDefaultShort"
 /*
   Default (short) KSP Monitor, same as KSPMonitorDefault() except
   it prints fewer digits of the residual as the residual gets smaller.
   This is because the later digits are meaningless and are often
   different on different machines; by using this routine different
   machines will usually generate the same output.
+
+  Deprecated: Intentionally has no manual page
 */
-PetscErrorCode  KSPMonitorDefaultShort(KSP ksp,PetscInt its,PetscReal fnorm,void *dummy)
+PetscErrorCode  KSPMonitorDefaultShort(KSP ksp,PetscInt its,PetscReal fnorm,PetscViewerAndFormat *dummy)
 {
   PetscErrorCode ierr;
-  PetscViewer    viewer = (PetscViewer)dummy;
+  PetscViewer    viewer = dummy->viewer;
 
   PetscFunctionBegin;
-  if (!viewer) {
-    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)ksp),&viewer);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,4);
+  ierr = PetscViewerPushFormat(viewer,dummy->format);CHKERRQ(ierr);
   ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)ksp)->tablevel);CHKERRQ(ierr);
   if (its == 0 && ((PetscObject)ksp)->prefix) {
     ierr = PetscViewerASCIIPrintf(viewer,"  Residual norms for %s solve.\n",((PetscObject)ksp)->prefix);CHKERRQ(ierr);
@@ -540,11 +501,10 @@ PetscErrorCode  KSPMonitorDefaultShort(KSP ksp,PetscInt its,PetscReal fnorm,void
     ierr = PetscViewerASCIIPrintf(viewer,"%3D KSP Residual norm < 1.e-11\n",its);CHKERRQ(ierr);
   }
   ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)ksp)->tablevel);CHKERRQ(ierr);
+  ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPConvergedSkip"
 /*@C
    KSPConvergedSkip - Convergence test that do not return as converged
    until the maximum number of iterations is reached.
@@ -584,8 +544,6 @@ PetscErrorCode  KSPConvergedSkip(KSP ksp,PetscInt n,PetscReal rnorm,KSPConverged
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPConvergedDefaultCreate"
 /*@C
    KSPConvergedDefaultCreate - Creates and initializes the space used by the KSPConvergedDefault() function context
 
@@ -612,8 +570,6 @@ PetscErrorCode  KSPConvergedDefaultCreate(void **ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPConvergedDefaultSetUIRNorm"
 /*@
    KSPConvergedDefaultSetUIRNorm - makes the default convergence test use || B*(b - A*(initial guess))||
       instead of || B*b ||. In the case of right preconditioner or if KSPSetNormType(ksp,KSP_NORM_UNPRECONDIITONED)
@@ -656,8 +612,6 @@ PetscErrorCode  KSPConvergedDefaultSetUIRNorm(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPConvergedDefaultSetUMIRNorm"
 /*@
    KSPConvergedDefaultSetUMIRNorm - makes the default convergence test use min(|| B*(b - A*(initial guess))||,|| B*b ||)
       In the case of right preconditioner or if KSPSetNormType(ksp,KSP_NORM_UNPRECONDIITONED)
@@ -694,8 +648,6 @@ PetscErrorCode  KSPConvergedDefaultSetUMIRNorm(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPConvergedDefault"
 /*@C
    KSPConvergedDefault - Determines convergence of the linear iterative solvers by default
 
@@ -720,7 +672,8 @@ PetscErrorCode  KSPConvergedDefaultSetUMIRNorm(KSP ksp)
 +     rtol = relative tolerance,
 .     abstol = absolute tolerance.
 .     dtol = divergence tolerance,
--     rnorm_0 is the two norm of the right hand side. When initial guess is non-zero you
+-     rnorm_0 is the two norm of the right hand side (or the preconditioned norm, depending on what was set with
+          KSPSetNormType(). When initial guess is non-zero you
           can call KSPConvergedDefaultSetUIRNorm() to use the norm of (b - A*(initial guess))
           as the starting point for relative norm convergence testing, that is as rnorm_0
 
@@ -753,13 +706,13 @@ PetscErrorCode  KSPConvergedDefault(KSP ksp,PetscInt n,PetscReal rnorm,KSPConver
   *reason = KSP_CONVERGED_ITERATING;
 
   ierr = KSPGetNormType(ksp,&normtype);CHKERRQ(ierr);
-  if (normtype == KSP_NORM_NONE) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_ARG_WRONGSTATE,"Use KSPConvergedSkip() with KSPNormType of KSP_NORM_NONE");
+  if (normtype == KSP_NORM_NONE) PetscFunctionReturn(0);
 
   if (!cctx) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_ARG_NULL,"Convergence context must have been created with KSPConvergedDefaultCreate()");
   if (!n) {
     /* if user gives initial guess need to compute norm of b */
     if (!ksp->guess_zero && !cctx->initialrtol) {
-      PetscReal snorm;
+      PetscReal snorm = 0.0;
       if (ksp->normtype == KSP_NORM_UNPRECONDITIONED || ksp->pc_side == PC_RIGHT) {
         ierr = PetscInfo(ksp,"user has provided nonzero initial guess, computing 2-norm of RHS\n");CHKERRQ(ierr);
         ierr = VecNorm(ksp->vec_rhs,NORM_2,&snorm);CHKERRQ(ierr);        /*     <- b'*b */
@@ -795,8 +748,19 @@ PetscErrorCode  KSPConvergedDefault(KSP ksp,PetscInt n,PetscReal rnorm,KSPConver
   if (n <= ksp->chknorm) PetscFunctionReturn(0);
 
   if (PetscIsInfOrNanReal(rnorm)) {
-    ierr    = PetscInfo(ksp,"Linear solver has created a not a number (NaN) as the residual norm, declaring divergence \n");CHKERRQ(ierr);
-    *reason = KSP_DIVERGED_NANORINF;
+    PCFailedReason pcreason;
+    PetscInt       sendbuf,pcreason_max;
+    ierr = PCGetSetUpFailedReason(ksp->pc,&pcreason);CHKERRQ(ierr);
+    sendbuf = (PetscInt)pcreason;
+    ierr = MPI_Allreduce(&sendbuf,&pcreason_max,1,MPIU_INT,MPIU_MAX,PetscObjectComm((PetscObject)ksp));CHKERRQ(ierr);
+    if (pcreason_max) {
+      *reason = KSP_DIVERGED_PCSETUP_FAILED;
+      ierr    = VecSetInf(ksp->vec_sol);CHKERRQ(ierr);
+      ierr    = PetscInfo(ksp,"Linear solver pcsetup fails, declaring divergence \n");CHKERRQ(ierr);
+    } else {
+      *reason = KSP_DIVERGED_NANORINF;
+      ierr    = PetscInfo(ksp,"Linear solver has created a not a number (NaN) as the residual norm, declaring divergence \n");CHKERRQ(ierr);
+    }
   } else if (rnorm <= ksp->ttol) {
     if (rnorm < ksp->abstol) {
       ierr    = PetscInfo3(ksp,"Linear solver has converged. Residual norm %14.12e is less than absolute tolerance %14.12e at iteration %D\n",(double)rnorm,(double)ksp->abstol,n);CHKERRQ(ierr);
@@ -816,8 +780,6 @@ PetscErrorCode  KSPConvergedDefault(KSP ksp,PetscInt n,PetscReal rnorm,KSPConver
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPConvergedDefaultDestroy"
 /*@C
    KSPConvergedDefaultDestroy - Frees the space used by the KSPConvergedDefault() function context
 
@@ -844,8 +806,6 @@ PetscErrorCode  KSPConvergedDefaultDestroy(void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPBuildSolutionDefault"
 /*
    KSPBuildSolutionDefault - Default code to create/move the solution.
 
@@ -899,8 +859,6 @@ PetscErrorCode KSPBuildSolutionDefault(KSP ksp,Vec v,Vec *V)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPBuildResidualDefault"
 /*
    KSPBuildResidualDefault - Default code to compute the residual.
 
@@ -935,8 +893,6 @@ PetscErrorCode KSPBuildResidualDefault(KSP ksp,Vec t,Vec v,Vec *V)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPCreateVecs"
 /*@C
   KSPCreateVecs - Gets a number of work vectors.
 
@@ -1033,18 +989,20 @@ PetscErrorCode KSPCreateVecs(KSP ksp,PetscInt rightn, Vec **right,PetscInt leftn
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetWorkVecs"
-/*
+/*@C
   KSPSetWorkVecs - Sets a number of work vectors into a KSP object
 
+  Collective on KSP
+
   Input Parameters:
-. ksp  - iterative context
-. nw   - number of work vectors to allocate
++ ksp  - iterative context
+- nw   - number of work vectors to allocate
 
-   Developers Note: This is PETSC_EXTERN because it may be used by user written plugin KSP implementations
+  Level: developer
 
- */
+  Developers Note: This is PETSC_EXTERN because it may be used by user written plugin KSP implementations
+
+@*/
 PetscErrorCode KSPSetWorkVecs(KSP ksp,PetscInt nw)
 {
   PetscErrorCode ierr;
@@ -1057,8 +1015,6 @@ PetscErrorCode KSPSetWorkVecs(KSP ksp,PetscInt nw)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPDestroyDefault"
 /*
   KSPDestroyDefault - Destroys a iterative context variable for methods with
   no separate context.  Preferred calling sequence KSPDestroy().
@@ -1079,8 +1035,6 @@ PetscErrorCode KSPDestroyDefault(KSP ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGetConvergedReason"
 /*@
    KSPGetConvergedReason - Gets the reason the KSP iteration was stopped.
 
@@ -1092,26 +1046,28 @@ PetscErrorCode KSPDestroyDefault(KSP ksp)
    Output Parameter:
 .  reason - negative value indicates diverged, positive value converged, see KSPConvergedReason
 
-   Possible values for reason:
-+  KSP_CONVERGED_RTOL (residual 2-norm decreased by a factor of rtol, from 2-norm of right hand side)
-.  KSP_CONVERGED_ATOL (residual 2-norm less than abstol)
-.  KSP_CONVERGED_ITS (used by the preonly preconditioner that always uses ONE iteration, or when the KSPConvergedSkip() convergence
-           test routine is set.
-.  KSP_CONVERGED_CG_NEG_CURVE
-.  KSP_CONVERGED_CG_CONSTRAINED
-.  KSP_CONVERGED_STEP_LENGTH
-.  KSP_DIVERGED_ITS  (required more than its to reach convergence)
-.  KSP_DIVERGED_DTOL (residual norm increased by a factor of divtol)
-.  KSP_DIVERGED_NANORINF (residual norm became Not-a-number or Inf likely due to 0/0)
-.  KSP_DIVERGED_BREAKDOWN (generic breakdown in method)
--  KSP_DIVERGED_BREAKDOWN_BICG (Initial residual is orthogonal to preconditioned initial
-                                residual. Try a different preconditioner, or a different initial Level.)
+   Possible values for reason: See also manual page for each reason
+$  KSP_CONVERGED_RTOL (residual 2-norm decreased by a factor of rtol, from 2-norm of right hand side)
+$  KSP_CONVERGED_ATOL (residual 2-norm less than abstol)
+$  KSP_CONVERGED_ITS (used by the preonly preconditioner that always uses ONE iteration, or when the KSPConvergedSkip() convergence test routine is set.
+$  KSP_CONVERGED_CG_NEG_CURVE (see note below)
+$  KSP_CONVERGED_CG_CONSTRAINED (see note below)
+$  KSP_CONVERGED_STEP_LENGTH (see note below)
+$  KSP_CONVERGED_ITERATING (returned if the solver is not yet finished)
+$  KSP_DIVERGED_ITS  (required more than its to reach convergence)
+$  KSP_DIVERGED_DTOL (residual norm increased by a factor of divtol)
+$  KSP_DIVERGED_NANORINF (residual norm became Not-a-number or Inf likely due to 0/0)
+$  KSP_DIVERGED_BREAKDOWN (generic breakdown in method)
+$  KSP_DIVERGED_BREAKDOWN_BICG (Initial residual is orthogonal to preconditioned initial residual. Try a different preconditioner, or a different initial Level.)
 
-   See also manual page for each reason.
+   Options Database:
+.   -ksp_converged_reason - prints the reason to standard out
 
-   guess: beginner
+   Notes:
+    If this routine is called before or doing the KSPSolve() the value of KSP_CONVERGED_ITERATING is returned
 
-   Notes: Can only be called after the call the KSPSolve() is complete.
+   The values  KSP_CONVERGED_CG_NEG_CURVE, KSP_CONVERGED_CG_CONSTRAINED, and KSP_CONVERGED_STEP_LENGTH are returned only by the special KSPCGNASH, KSPCGSTCG, and KSPCGGLTR
+   solvers which are used by the SNESNEWTONTR (trust region) solver.
 
    Level: intermediate
 
@@ -1129,8 +1085,6 @@ PetscErrorCode  KSPGetConvergedReason(KSP ksp,KSPConvergedReason *reason)
 }
 
 #include <petsc/private/dmimpl.h>
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetDM"
 /*@
    KSPSetDM - Sets the DM that may be used by some preconditioners
 
@@ -1138,11 +1092,16 @@ PetscErrorCode  KSPGetConvergedReason(KSP ksp,KSPConvergedReason *reason)
 
    Input Parameters:
 +  ksp - the preconditioner context
--  dm - the dm
+-  dm - the dm, cannot be NULL
 
-   Notes: If this is used then the KSP will attempt to use the DM to create the matrix and use the routine
-          set with DMKSPSetComputeOperators(). Use KSPSetDMActive(ksp,PETSC_FALSE) to instead use the matrix
-          you've provided with KSPSetOperators().
+   Notes:
+   If this is used then the KSP will attempt to use the DM to create the matrix and use the routine set with
+   DMKSPSetComputeOperators(). Use KSPSetDMActive(ksp,PETSC_FALSE) to instead use the matrix you've provided with
+   KSPSetOperators().
+
+   A DM can only be used for solving one problem at a time because information about the problem is stored on the DM,
+   even when not using interfaces like DMKSPSetComputeOperators().  Use DMClone() to get a distinct DM when solving
+   different problems using the same function space.
 
    Level: intermediate
 
@@ -1155,9 +1114,10 @@ PetscErrorCode  KSPSetDM(KSP ksp,DM dm)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ksp,KSP_CLASSID,1);
-  if (dm) {ierr = PetscObjectReference((PetscObject)dm);CHKERRQ(ierr);}
+  PetscValidHeaderSpecific(dm,DM_CLASSID,2);
+  ierr = PetscObjectReference((PetscObject)dm);CHKERRQ(ierr);
   if (ksp->dm) {                /* Move the DMSNES context over to the new DM unless the new DM already has one */
-    if (ksp->dm->dmksp && ksp->dmAuto && !dm->dmksp) {
+    if (ksp->dm->dmksp && !dm->dmksp) {
       DMKSP kdm;
       ierr = DMCopyDMKSP(ksp->dm,dm);CHKERRQ(ierr);
       ierr = DMGetDMKSP(ksp->dm,&kdm);CHKERRQ(ierr);
@@ -1173,8 +1133,6 @@ PetscErrorCode  KSPSetDM(KSP ksp,DM dm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetDMActive"
 /*@
    KSPSetDMActive - Indicates the DM should be used to generate the linear system matrix and right hand side
 
@@ -1200,8 +1158,6 @@ PetscErrorCode  KSPSetDMActive(KSP ksp,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGetDM"
 /*@
    KSPGetDM - Gets the DM that may be used by some preconditioners
 
@@ -1232,8 +1188,6 @@ PetscErrorCode  KSPGetDM(KSP ksp,DM *dm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPSetApplicationContext"
 /*@
    KSPSetApplicationContext - Sets the optional user-defined context for the linear solver.
 
@@ -1243,6 +1197,10 @@ PetscErrorCode  KSPGetDM(KSP ksp,DM *dm)
 +  ksp - the KSP context
 -  usrP - optional user context
 
+   Fortran Notes:
+    To use this from Fortran you must write a Fortran interface definition for this
+    function that tells Fortran the Fortran derived data type that you are passing in as the ctx argument.
+
    Level: intermediate
 
 .keywords: KSP, set, application, context
@@ -1262,8 +1220,6 @@ PetscErrorCode  KSPSetApplicationContext(KSP ksp,void *usrP)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPGetApplicationContext"
 /*@
    KSPGetApplicationContext - Gets the user-defined context for the linear solver.
 
@@ -1275,6 +1231,10 @@ PetscErrorCode  KSPSetApplicationContext(KSP ksp,void *usrP)
    Output Parameter:
 .  usrP - user context
 
+   Fortran Notes:
+    To use this from Fortran you must write a Fortran interface definition for this
+    function that tells Fortran the Fortran derived data type that you are passing in as the ctx argument.
+
    Level: intermediate
 
 .keywords: KSP, get, application, context
