@@ -1,8 +1,6 @@
 
 #include <petsc/private/kspimpl.h>   /*I "petscksp.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPInitialResidual"
 /*@
    KSPInitialResidual - Computes the residual. Either b - A*C*u = b - A*x with right
      preconditioning or C*(b - A*x) with left preconditioning; that later
@@ -53,8 +51,14 @@ PetscErrorCode  KSPInitialResidual(KSP ksp,Vec vsoln,Vec vt1,Vec vt2,Vec vres,Ve
     ierr = KSP_MatMult(ksp,Amat,vsoln,vt1);CHKERRQ(ierr);
     ierr = VecCopy(vb,vt2);CHKERRQ(ierr);
     ierr = VecAXPY(vt2,-1.0,vt1);CHKERRQ(ierr);
-    ierr = (ksp->pc_side == PC_RIGHT) ? (VecCopy(vt2,vres)) : (KSP_PCApply(ksp,vt2,vres));CHKERRQ(ierr);
-    ierr = PCDiagonalScaleLeft(ksp->pc,vres,vres);CHKERRQ(ierr);
+    if (ksp->pc_side == PC_RIGHT) {
+      ierr = PCDiagonalScaleLeft(ksp->pc,vt2,vres);CHKERRQ(ierr);
+    } else if (ksp->pc_side == PC_LEFT) {
+        ierr = KSP_PCApply(ksp,vt2,vres);CHKERRQ(ierr);
+        ierr = PCDiagonalScaleLeft(ksp->pc,vres,vres);CHKERRQ(ierr);
+    } else if (ksp->pc_side == PC_SYMMETRIC) {
+      ierr = PCApplySymmetricLeft(ksp->pc,vt2,vres);CHKERRQ(ierr);
+    } else SETERRQ1(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP, "Invalid preconditioning side %d", (int)ksp->pc_side);
   } else {
     ierr = VecCopy(vb,vt2);CHKERRQ(ierr);
     if (ksp->pc_side == PC_RIGHT) {
@@ -69,8 +73,6 @@ PetscErrorCode  KSPInitialResidual(KSP ksp,Vec vsoln,Vec vt1,Vec vt2,Vec vres,Ve
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPUnwindPreconditioner"
 /*@
    KSPUnwindPreconditioner - Unwinds the preconditioning in the solution. That is,
      takes solution to the preconditioned problem and gets the solution to the
