@@ -1,18 +1,20 @@
 
 #include <../src/ksp/pc/impls/factor/icc/icc.h>   /*I "petscpc.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetup_ICC"
-static PetscErrorCode PCSetup_ICC(PC pc)
+static PetscErrorCode PCSetUp_ICC(PC pc)
 {
-  PC_ICC         *icc = (PC_ICC*)pc->data;
-  IS             perm,cperm;
-  PetscErrorCode ierr;
-  MatInfo        info;
+  PC_ICC                 *icc = (PC_ICC*)pc->data;
+  IS                     perm,cperm;
+  PetscErrorCode         ierr;
+  MatInfo                info;
+  MatSolverType          stype;
+  MatFactorError         err;
 
   PetscFunctionBegin;
+  pc->failedreason = PC_NOERROR;
   ierr = MatGetOrdering(pc->pmat, ((PC_Factor*)icc)->ordering,&perm,&cperm);CHKERRQ(ierr);
 
+  ierr = MatSetErrorIfFailure(pc->pmat,pc->erroriffailure);CHKERRQ(ierr);
   if (!pc->setupcalled) {
     if (!((PC_Factor*)icc)->fact) {
       ierr = MatGetFactor(pc->pmat,((PC_Factor*)icc)->solvertype,MAT_FACTOR_ICC,&((PC_Factor*)icc)->fact);CHKERRQ(ierr);
@@ -23,17 +25,33 @@ static PetscErrorCode PCSetup_ICC(PC pc)
     ierr = MatGetFactor(pc->pmat,((PC_Factor*)icc)->solvertype,MAT_FACTOR_ICC,&((PC_Factor*)icc)->fact);CHKERRQ(ierr);
     ierr = MatICCFactorSymbolic(((PC_Factor*)icc)->fact,pc->pmat,perm,&((PC_Factor*)icc)->info);CHKERRQ(ierr);
   }
-  ierr            = MatGetInfo(((PC_Factor*)icc)->fact,MAT_LOCAL,&info);CHKERRQ(ierr);
-  icc->actualfill = info.fill_ratio_needed;
+  ierr                = MatGetInfo(((PC_Factor*)icc)->fact,MAT_LOCAL,&info);CHKERRQ(ierr);
+  icc->hdr.actualfill = info.fill_ratio_needed;
 
   ierr = ISDestroy(&cperm);CHKERRQ(ierr);
   ierr = ISDestroy(&perm);CHKERRQ(ierr);
+
+  ierr = MatFactorGetError(((PC_Factor*)icc)->fact,&err);CHKERRQ(ierr);
+  if (err) { /* FactorSymbolic() fails */
+    pc->failedreason = (PCFailedReason)err;
+    PetscFunctionReturn(0);
+  }
+ 
   ierr = MatCholeskyFactorNumeric(((PC_Factor*)icc)->fact,pc->pmat,&((PC_Factor*)icc)->info);CHKERRQ(ierr);
+  ierr = MatFactorGetError(((PC_Factor*)icc)->fact,&err);CHKERRQ(ierr);
+  if (err) { /* FactorNumeric() fails */
+    pc->failedreason = (PCFailedReason)err;
+  }
+
+  ierr = PCFactorGetMatSolverType(pc,&stype);CHKERRQ(ierr);
+  if (!stype) {
+    MatSolverType solverpackage;
+    ierr = MatFactorGetSolverType(((PC_Factor*)icc)->fact,&solverpackage);CHKERRQ(ierr);
+    ierr = PCFactorSetMatSolverType(pc,solverpackage);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCReset_ICC"
 static PetscErrorCode PCReset_ICC(PC pc)
 {
   PC_ICC         *icc = (PC_ICC*)pc->data;
@@ -44,8 +62,6 @@ static PetscErrorCode PCReset_ICC(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCDestroy_ICC"
 static PetscErrorCode PCDestroy_ICC(PC pc)
 {
   PC_ICC         *icc = (PC_ICC*)pc->data;
@@ -59,8 +75,6 @@ static PetscErrorCode PCDestroy_ICC(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_ICC"
 static PetscErrorCode PCApply_ICC(PC pc,Vec x,Vec y)
 {
   PC_ICC         *icc = (PC_ICC*)pc->data;
@@ -71,8 +85,6 @@ static PetscErrorCode PCApply_ICC(PC pc,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplySymmetricLeft_ICC"
 static PetscErrorCode PCApplySymmetricLeft_ICC(PC pc,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -83,8 +95,6 @@ static PetscErrorCode PCApplySymmetricLeft_ICC(PC pc,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplySymmetricRight_ICC"
 static PetscErrorCode PCApplySymmetricRight_ICC(PC pc,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -95,9 +105,7 @@ static PetscErrorCode PCApplySymmetricRight_ICC(PC pc,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetFromOptions_ICC"
-static PetscErrorCode PCSetFromOptions_ICC(PetscOptions *PetscOptionsObject,PC pc)
+static PetscErrorCode PCSetFromOptions_ICC(PetscOptionItems *PetscOptionsObject,PC pc)
 {
   PC_ICC         *icc = (PC_ICC*)pc->data;
   PetscBool      flg;
@@ -122,8 +130,6 @@ static PetscErrorCode PCSetFromOptions_ICC(PetscOptions *PetscOptionsObject,PC p
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCView_ICC"
 static PetscErrorCode PCView_ICC(PC pc,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -135,15 +141,6 @@ static PetscErrorCode PCView_ICC(PC pc,PetscViewer viewer)
 
 extern PetscErrorCode  PCFactorSetDropTolerance_ILU(PC,PetscReal,PetscReal,PetscInt);
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorGetUseInPlace_ICC"
-PetscErrorCode  PCFactorGetUseInPlace_ICC(PC pc,PetscBool *flg)
-{
-  PetscFunctionBegin;
-  *flg = PETSC_FALSE;
-  PetscFunctionReturn(0);
-}
-
 /*MC
      PCICC - Incomplete Cholesky factorization preconditioners.
 
@@ -158,7 +155,8 @@ PetscErrorCode  PCFactorGetUseInPlace_ICC(PC pc,PetscBool *flg)
 
   Concepts: incomplete Cholesky factorization
 
-   Notes: Only implemented for some matrix formats. Not implemented in parallel.
+   Notes:
+    Only implemented for some matrix formats. Not implemented in parallel.
 
           For BAIJ matrices this implements a point block ICC.
 
@@ -168,10 +166,9 @@ PetscErrorCode  PCFactorGetUseInPlace_ICC(PC pc,PetscBool *flg)
           to turn off the shift.
 
    References:
-   Review article: APPROXIMATE AND INCOMPLETE FACTORIZATIONS, TONY F. CHAN AND HENK A. VAN DER VORST
-      http://igitur-archive.library.uu.nl/math/2001-0621-115821/proc.pdf chapter in Parallel Numerical
-      Algorithms, edited by D. Keyes, A. Semah, V. Venkatakrishnan, ICASE/LaRC Interdisciplinary Series in
-      Science and Engineering, Kluwer, pp. 167--202.
+.  1. - TONY F. CHAN AND HENK A. VAN DER VORST, Review article: APPROXIMATE AND INCOMPLETE FACTORIZATIONS, 
+      Chapter in Parallel Numerical Algorithms, edited by D. Keyes, A. Semah, V. Venkatakrishnan, ICASE/LaRC Interdisciplinary Series in
+      Science and Engineering, Kluwer.
 
 
 .seealso:  PCCreate(), PCSetType(), PCType (for list of available types), PC, PCSOR, MatOrderingType,
@@ -181,56 +178,31 @@ PetscErrorCode  PCFactorGetUseInPlace_ICC(PC pc,PetscBool *flg)
 
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCCreate_ICC"
 PETSC_EXTERN PetscErrorCode PCCreate_ICC(PC pc)
 {
   PetscErrorCode ierr;
   PC_ICC         *icc;
 
   PetscFunctionBegin;
-  ierr = PetscNewLog(pc,&icc);CHKERRQ(ierr);
+  ierr     = PetscNewLog(pc,&icc);CHKERRQ(ierr);
+  pc->data = (void*)icc;
+  ierr     = PCFactorInitialize(pc);CHKERRQ(ierr);
+  ierr     = PetscStrallocpy(MATORDERINGNATURAL,(char**)&((PC_Factor*)icc)->ordering);CHKERRQ(ierr);
 
-  ((PC_Factor*)icc)->fact = 0;
+  ((PC_Factor*)icc)->factortype     = MAT_FACTOR_ICC;
+  ((PC_Factor*)icc)->info.fill      = 1.0;
+  ((PC_Factor*)icc)->info.dtcol     = PETSC_DEFAULT;
+  ((PC_Factor*)icc)->info.shifttype = (PetscReal) MAT_SHIFT_POSITIVE_DEFINITE;
 
-  ierr = PetscStrallocpy(MATORDERINGNATURAL,(char**)&((PC_Factor*)icc)->ordering);CHKERRQ(ierr);
-  ierr = PetscStrallocpy(MATSOLVERPETSC,&((PC_Factor*)icc)->solvertype);CHKERRQ(ierr);
-  ierr = MatFactorInfoInitialize(&((PC_Factor*)icc)->info);CHKERRQ(ierr);
-
-  ((PC_Factor*)icc)->factortype  = MAT_FACTOR_ICC;
-  ((PC_Factor*)icc)->info.levels = 0.;
-  ((PC_Factor*)icc)->info.fill   = 1.0;
-  icc->implctx                   = 0;
-
-  ((PC_Factor*)icc)->info.dtcol       = PETSC_DEFAULT;
-  ((PC_Factor*)icc)->info.shifttype   = (PetscReal) MAT_SHIFT_POSITIVE_DEFINITE;
-  ((PC_Factor*)icc)->info.shiftamount = 100.0*PETSC_MACHINE_EPSILON;
-  ((PC_Factor*)icc)->info.zeropivot   = 100.0*PETSC_MACHINE_EPSILON;
-
-  pc->data                     = (void*)icc;
   pc->ops->apply               = PCApply_ICC;
   pc->ops->applytranspose      = PCApply_ICC;
-  pc->ops->setup               = PCSetup_ICC;
+  pc->ops->setup               = PCSetUp_ICC;
   pc->ops->reset               = PCReset_ICC;
   pc->ops->destroy             = PCDestroy_ICC;
   pc->ops->setfromoptions      = PCSetFromOptions_ICC;
   pc->ops->view                = PCView_ICC;
-  pc->ops->getfactoredmatrix   = PCFactorGetMatrix_Factor;
   pc->ops->applysymmetricleft  = PCApplySymmetricLeft_ICC;
   pc->ops->applysymmetricright = PCApplySymmetricRight_ICC;
-
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetUpMatSolverPackage_C",PCFactorSetUpMatSolverPackage_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorGetMatSolverPackage_C",PCFactorGetMatSolverPackage_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetZeroPivot_C",PCFactorSetZeroPivot_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetShiftType_C",PCFactorSetShiftType_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetShiftAmount_C",PCFactorSetShiftAmount_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetLevels_C",PCFactorSetLevels_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorGetLevels_C",PCFactorGetLevels_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetFill_C",PCFactorSetFill_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetMatOrderingType_C",PCFactorSetMatOrderingType_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetMatSolverPackage_C",PCFactorSetMatSolverPackage_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetDropTolerance_C",PCFactorSetDropTolerance_ILU);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorGetUseInPlace_C",PCFactorGetUseInPlace_ICC);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
