@@ -4,20 +4,6 @@
 */
 #include <../src/ksp/pc/impls/factor/ilu/ilu.h>     /*I "petscpc.h"  I*/
 
-/* ------------------------------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetReuseFill_ILU"
-PetscErrorCode  PCFactorSetReuseFill_ILU(PC pc,PetscBool flag)
-{
-  PC_ILU *lu = (PC_ILU*)pc->data;
-
-  PetscFunctionBegin;
-  lu->reusefill = flag;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorReorderForNonzeroDiagonal_ILU"
 PetscErrorCode  PCFactorReorderForNonzeroDiagonal_ILU(PC pc,PetscReal z)
 {
   PC_ILU *ilu = (PC_ILU*)pc->data;
@@ -29,22 +15,18 @@ PetscErrorCode  PCFactorReorderForNonzeroDiagonal_ILU(PC pc,PetscReal z)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCReset_ILU"
 PetscErrorCode PCReset_ILU(PC pc)
 {
   PC_ILU         *ilu = (PC_ILU*)pc->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (!ilu->inplace) {ierr = MatDestroy(&((PC_Factor*)ilu)->fact);CHKERRQ(ierr);}
+  if (!ilu->hdr.inplace) {ierr = MatDestroy(&((PC_Factor*)ilu)->fact);CHKERRQ(ierr);}
   if (ilu->row && ilu->col && ilu->row != ilu->col) {ierr = ISDestroy(&ilu->row);CHKERRQ(ierr);}
   ierr = ISDestroy(&ilu->col);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetDropTolerance_ILU"
 PetscErrorCode  PCFactorSetDropTolerance_ILU(PC pc,PetscReal dt,PetscReal dtcol,PetscInt dtcount)
 {
   PC_ILU *ilu = (PC_ILU*)pc->data;
@@ -60,42 +42,7 @@ PetscErrorCode  PCFactorSetDropTolerance_ILU(PC pc,PetscReal dt,PetscReal dtcol,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetReuseOrdering_ILU"
-PetscErrorCode  PCFactorSetReuseOrdering_ILU(PC pc,PetscBool flag)
-{
-  PC_ILU *ilu = (PC_ILU*)pc->data;
-
-  PetscFunctionBegin;
-  ilu->reuseordering = flag;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetUseInPlace_ILU"
-PetscErrorCode  PCFactorSetUseInPlace_ILU(PC pc,PetscBool flg)
-{
-  PC_ILU *dir = (PC_ILU*)pc->data;
-
-  PetscFunctionBegin;
-  dir->inplace = flg;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorGetUseInPlace_ILU"
-PetscErrorCode  PCFactorGetUseInPlace_ILU(PC pc,PetscBool *flg)
-{
-  PC_ILU *dir = (PC_ILU*)pc->data;
-
-  PetscFunctionBegin;
-  *flg = dir->inplace;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PCSetFromOptions_ILU"
-static PetscErrorCode PCSetFromOptions_ILU(PetscOptions *PetscOptionsObject,PC pc)
+static PetscErrorCode PCSetFromOptions_ILU(PetscOptionItems *PetscOptionsObject,PC pc)
 {
   PetscErrorCode ierr;
   PetscInt       itmp;
@@ -123,40 +70,26 @@ static PetscErrorCode PCSetFromOptions_ILU(PetscOptions *PetscOptionsObject,PC p
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCView_ILU"
 static PetscErrorCode PCView_ILU(PC pc,PetscViewer viewer)
 {
-  PC_ILU         *ilu = (PC_ILU*)pc->data;
   PetscErrorCode ierr;
-  PetscBool      iascii;
 
   PetscFunctionBegin;
-  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
-  if (iascii) {
-    if (ilu->inplace) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  ILU: in-place factorization\n");CHKERRQ(ierr);
-    } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"  ILU: out-of-place factorization\n");CHKERRQ(ierr);
-    }
-
-    if (ilu->reusefill)     {ierr = PetscViewerASCIIPrintf(viewer,"  ILU: Reusing fill from past factorization\n");CHKERRQ(ierr);}
-    if (ilu->reuseordering) {ierr = PetscViewerASCIIPrintf(viewer,"  ILU: Reusing reordering from past factorization\n");CHKERRQ(ierr);}
-  }
   ierr = PCView_Factor(pc,viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUp_ILU"
 static PetscErrorCode PCSetUp_ILU(PC pc)
 {
-  PetscErrorCode ierr;
-  PC_ILU         *ilu = (PC_ILU*)pc->data;
-  MatInfo        info;
-  PetscBool      flg;
+  PetscErrorCode         ierr;
+  PC_ILU                 *ilu = (PC_ILU*)pc->data;
+  MatInfo                info;
+  PetscBool              flg;
+  MatSolverType          stype;
+  MatFactorError         err;
 
   PetscFunctionBegin;
+  pc->failedreason = PC_NOERROR;
   /* ugly hack to change default, since it is not support by some matrix types */
   if (((PC_Factor*)ilu)->info.shifttype == (PetscReal)MAT_SHIFT_NONZERO) {
     ierr = PetscObjectTypeCompare((PetscObject)pc->pmat,MATSEQAIJ,&flg);CHKERRQ(ierr);
@@ -164,12 +97,13 @@ static PetscErrorCode PCSetUp_ILU(PC pc)
       ierr = PetscObjectTypeCompare((PetscObject)pc->pmat,MATMPIAIJ,&flg);CHKERRQ(ierr);
       if (!flg) {
         ((PC_Factor*)ilu)->info.shifttype = (PetscReal)MAT_SHIFT_INBLOCKS;
-        PetscInfo(pc,"Changing shift type from NONZERO to INBLOCKS because block matrices do not support NONZERO\n");CHKERRQ(ierr);
+        ierr = PetscInfo(pc,"Changing shift type from NONZERO to INBLOCKS because block matrices do not support NONZERO\n");CHKERRQ(ierr);
       }
     }
   }
 
-  if (ilu->inplace) {
+  ierr = MatSetErrorIfFailure(pc->pmat,pc->erroriffailure);CHKERRQ(ierr);
+  if (ilu->hdr.inplace) {
     if (!pc->setupcalled) {
 
       /* In-place factorization only makes sense with the natural ordering,
@@ -185,6 +119,12 @@ static PetscErrorCode PCSetUp_ILU(PC pc)
     ((PC_Factor*)ilu)->info.diagonal_fill = 0.0;
 
     ierr = MatILUFactor(pc->pmat,ilu->row,ilu->col,&((PC_Factor*)ilu)->info);CHKERRQ(ierr);CHKERRQ(ierr);
+    ierr = MatFactorGetError(pc->pmat,&err);CHKERRQ(ierr);
+    if (err) { /* Factor() fails */
+      pc->failedreason = (PCFailedReason)err;
+      PetscFunctionReturn(0);
+    }
+
     ((PC_Factor*)ilu)->fact = pc->pmat;
     /* must update the pc record of the matrix state or the PC will attempt to run PCSetUp() yet again */
     ierr = PetscObjectStateGet((PetscObject)pc->pmat,&pc->matstate);CHKERRQ(ierr);
@@ -203,12 +143,11 @@ static PetscErrorCode PCSetUp_ILU(PC pc)
       }
       ierr = MatILUFactorSymbolic(((PC_Factor*)ilu)->fact,pc->pmat,ilu->row,ilu->col,&((PC_Factor*)ilu)->info);CHKERRQ(ierr);
       ierr = MatGetInfo(((PC_Factor*)ilu)->fact,MAT_LOCAL,&info);CHKERRQ(ierr);
-
-      ilu->actualfill = info.fill_ratio_needed;
+      ilu->hdr.actualfill = info.fill_ratio_needed;
 
       ierr = PetscLogObjectParent((PetscObject)pc,(PetscObject)((PC_Factor*)ilu)->fact);CHKERRQ(ierr);
     } else if (pc->flag != SAME_NONZERO_PATTERN) {
-      if (!ilu->reuseordering) {
+      if (!ilu->hdr.reuseordering) {
         /* compute a new ordering for the ILU */
         ierr = ISDestroy(&ilu->row);CHKERRQ(ierr);
         ierr = ISDestroy(&ilu->col);CHKERRQ(ierr);
@@ -224,19 +163,32 @@ static PetscErrorCode PCSetUp_ILU(PC pc)
       ierr = MatGetFactor(pc->pmat,((PC_Factor*)ilu)->solvertype,MAT_FACTOR_ILU,&((PC_Factor*)ilu)->fact);CHKERRQ(ierr);
       ierr = MatILUFactorSymbolic(((PC_Factor*)ilu)->fact,pc->pmat,ilu->row,ilu->col,&((PC_Factor*)ilu)->info);CHKERRQ(ierr);
       ierr = MatGetInfo(((PC_Factor*)ilu)->fact,MAT_LOCAL,&info);CHKERRQ(ierr);
-
-      ilu->actualfill = info.fill_ratio_needed;
+      ilu->hdr.actualfill = info.fill_ratio_needed;
 
       ierr = PetscLogObjectParent((PetscObject)pc,(PetscObject)((PC_Factor*)ilu)->fact);CHKERRQ(ierr);
     }
-    ierr = MatSetErrorIfFPE(pc->pmat,pc->erroriffailure);CHKERRQ(ierr);
+    ierr = MatFactorGetError(((PC_Factor*)ilu)->fact,&err);CHKERRQ(ierr);
+    if (err) { /* FactorSymbolic() fails */
+      pc->failedreason = (PCFailedReason)err;
+      PetscFunctionReturn(0);
+    }
+
     ierr = MatLUFactorNumeric(((PC_Factor*)ilu)->fact,pc->pmat,&((PC_Factor*)ilu)->info);CHKERRQ(ierr);
+    ierr = MatFactorGetError(((PC_Factor*)ilu)->fact,&err);CHKERRQ(ierr);
+    if (err) { /* FactorNumeric() fails */
+      pc->failedreason = (PCFailedReason)err;
+    }
+  }
+
+  ierr = PCFactorGetMatSolverType(pc,&stype);CHKERRQ(ierr);
+  if (!stype) {
+    MatSolverType solverpackage;
+    ierr = MatFactorGetSolverType(((PC_Factor*)ilu)->fact,&solverpackage);CHKERRQ(ierr);
+    ierr = PCFactorSetMatSolverType(pc,solverpackage);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCDestroy_ILU"
 static PetscErrorCode PCDestroy_ILU(PC pc)
 {
   PC_ILU         *ilu = (PC_ILU*)pc->data;
@@ -250,8 +202,6 @@ static PetscErrorCode PCDestroy_ILU(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_ILU"
 static PetscErrorCode PCApply_ILU(PC pc,Vec x,Vec y)
 {
   PC_ILU         *ilu = (PC_ILU*)pc->data;
@@ -262,8 +212,6 @@ static PetscErrorCode PCApply_ILU(PC pc,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyTranspose_ILU"
 static PetscErrorCode PCApplyTranspose_ILU(PC pc,Vec x,Vec y)
 {
   PC_ILU         *ilu = (PC_ILU*)pc->data;
@@ -274,8 +222,6 @@ static PetscErrorCode PCApplyTranspose_ILU(PC pc,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplySymmetricLeft_ILU"
 static PetscErrorCode PCApplySymmetricLeft_ILU(PC pc,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -286,8 +232,6 @@ static PetscErrorCode PCApplySymmetricLeft_ILU(PC pc,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplySymmetricRight_ILU"
 static PetscErrorCode PCApplySymmetricRight_ILU(PC pc,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -319,7 +263,8 @@ static PetscErrorCode PCApplySymmetricRight_ILU(PC pc,Vec x,Vec y)
 
   Concepts: incomplete factorization
 
-   Notes: Only implemented for some matrix formats. (for parallel see PCHYPRE for hypre's ILU)
+   Notes:
+    Only implemented for some matrix formats. (for parallel see PCHYPRE for hypre's ILU)
 
           For BAIJ matrices this implements a point block ILU
 
@@ -330,17 +275,13 @@ static PetscErrorCode PCApplySymmetricRight_ILU(PC pc,Vec x,Vec y)
           is never done on the GPU).
 
    References:
-   T. Dupont, R. Kendall, and H. Rachford. An approximate factorization procedure for solving
-   self-adjoint elliptic difference equations. SIAM J. Numer. Anal., 5:559--573, 1968.
-
-   T.A. Oliphant. An implicit numerical method for solving two-dimensional time-dependent dif-
-   fusion problems. Quart. Appl. Math., 19:221--229, 1961.
-
-   Review article: 
-    APPROXIMATE AND INCOMPLETE FACTORIZATIONS, TONY F. CHAN AND HENK A. VAN DER VORST
-      http://igitur-archive.library.uu.nl/math/2001-0621-115821/proc.pdf chapter in Parallel Numerical
++  1. - T. Dupont, R. Kendall, and H. Rachford. An approximate factorization procedure for solving
+   self adjoint elliptic difference equations. SIAM J. Numer. Anal., 5, 1968.
+.  2. -  T.A. Oliphant. An implicit numerical method for solving two dimensional timedependent diffusion problems. Quart. Appl. Math., 19, 1961.
+-  3. -  TONY F. CHAN AND HENK A. VAN DER VORST, APPROXIMATE AND INCOMPLETE FACTORIZATIONS, 
+      Chapter in Parallel Numerical
       Algorithms, edited by D. Keyes, A. Semah, V. Venkatakrishnan, ICASE/LaRC Interdisciplinary Series in
-      Science and Engineering, Kluwer, pp. 167--202.
+      Science and Engineering, Kluwer.
 
 
 .seealso:  PCCreate(), PCSetType(), PCType (for list of available types), PC, PCSOR, MatOrderingType,
@@ -351,37 +292,25 @@ static PetscErrorCode PCApplySymmetricRight_ILU(PC pc,Vec x,Vec y)
 
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCCreate_ILU"
 PETSC_EXTERN PetscErrorCode PCCreate_ILU(PC pc)
 {
   PetscErrorCode ierr;
   PC_ILU         *ilu;
 
   PetscFunctionBegin;
-  ierr = PetscNewLog(pc,&ilu);CHKERRQ(ierr);
+  ierr     = PetscNewLog(pc,&ilu);CHKERRQ(ierr);
+  pc->data = (void*)ilu;
+  ierr     = PCFactorInitialize(pc);CHKERRQ(ierr);
 
-  ((PC_Factor*)ilu)->fact               = 0;
-  ierr                                  = MatFactorInfoInitialize(&((PC_Factor*)ilu)->info);CHKERRQ(ierr);
   ((PC_Factor*)ilu)->factortype         = MAT_FACTOR_ILU;
   ((PC_Factor*)ilu)->info.levels        = 0.;
   ((PC_Factor*)ilu)->info.fill          = 1.0;
   ilu->col                              = 0;
   ilu->row                              = 0;
-  ilu->inplace                          = PETSC_FALSE;
-  ierr                                  = PetscStrallocpy(MATSOLVERPETSC,&((PC_Factor*)ilu)->solvertype);CHKERRQ(ierr);
   ierr                                  = PetscStrallocpy(MATORDERINGNATURAL,(char**)&((PC_Factor*)ilu)->ordering);CHKERRQ(ierr);
-  ilu->reuseordering                    = PETSC_FALSE;
   ((PC_Factor*)ilu)->info.dt            = PETSC_DEFAULT;
   ((PC_Factor*)ilu)->info.dtcount       = PETSC_DEFAULT;
   ((PC_Factor*)ilu)->info.dtcol         = PETSC_DEFAULT;
-  ((PC_Factor*)ilu)->info.shifttype     = (PetscReal)MAT_SHIFT_NONE;
-  ((PC_Factor*)ilu)->info.shiftamount   = 100.0*PETSC_MACHINE_EPSILON;
-  ((PC_Factor*)ilu)->info.zeropivot     = 100.0*PETSC_MACHINE_EPSILON;
-  ((PC_Factor*)ilu)->info.pivotinblocks = 1.0;
-  ilu->reusefill                        = PETSC_FALSE;
-  ((PC_Factor*)ilu)->info.diagonal_fill = 0.0;
-  pc->data                              = (void*)ilu;
 
   pc->ops->reset               = PCReset_ILU;
   pc->ops->destroy             = PCDestroy_ILU;
@@ -389,30 +318,11 @@ PETSC_EXTERN PetscErrorCode PCCreate_ILU(PC pc)
   pc->ops->applytranspose      = PCApplyTranspose_ILU;
   pc->ops->setup               = PCSetUp_ILU;
   pc->ops->setfromoptions      = PCSetFromOptions_ILU;
-  pc->ops->getfactoredmatrix   = PCFactorGetMatrix_Factor;
   pc->ops->view                = PCView_ILU;
   pc->ops->applysymmetricleft  = PCApplySymmetricLeft_ILU;
   pc->ops->applysymmetricright = PCApplySymmetricRight_ILU;
   pc->ops->applyrichardson     = 0;
-
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetZeroPivot_C",PCFactorSetZeroPivot_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetShiftType_C",PCFactorSetShiftType_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetShiftAmount_C",PCFactorSetShiftAmount_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorGetMatSolverPackage_C",PCFactorGetMatSolverPackage_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetMatSolverPackage_C",PCFactorSetMatSolverPackage_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetUpMatSolverPackage_C",PCFactorSetUpMatSolverPackage_Factor);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetDropTolerance_C",PCFactorSetDropTolerance_ILU);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetFill_C",PCFactorSetFill_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetMatOrderingType_C",PCFactorSetMatOrderingType_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetReuseOrdering_C",PCFactorSetReuseOrdering_ILU);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetReuseFill_C",PCFactorSetReuseFill_ILU);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetLevels_C",PCFactorSetLevels_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorGetLevels_C",PCFactorGetLevels_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetUseInPlace_C",PCFactorSetUseInPlace_ILU);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorGetUseInPlace_C",PCFactorGetUseInPlace_ILU);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetAllowDiagonalFill_C",PCFactorSetAllowDiagonalFill_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorGetAllowDiagonalFill_C",PCFactorGetAllowDiagonalFill_Factor);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorSetPivotInBlocks_C",PCFactorSetPivotInBlocks_Factor);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCFactorReorderForNonzeroDiagonal_C",PCFactorReorderForNonzeroDiagonal_ILU);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
