@@ -1,8 +1,6 @@
 
-#include "../src/ksp/pc/impls/is/pcis.h" /*I "petscpc.h" I*/
+#include <../src/ksp/pc/impls/is/pcis.h> /*I "petscpc.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCISSetUseStiffnessScaling_IS"
 static PetscErrorCode PCISSetUseStiffnessScaling_IS(PC pc, PetscBool use)
 {
   PC_IS *pcis = (PC_IS*)pc->data;
@@ -12,8 +10,6 @@ static PetscErrorCode PCISSetUseStiffnessScaling_IS(PC pc, PetscBool use)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCISSetUseStiffnessScaling"
 /*@
  PCISSetUseStiffnessScaling - Tells PCIS to construct partition of unity using
                               local matrices' diagonal.
@@ -40,22 +36,18 @@ PetscErrorCode PCISSetUseStiffnessScaling(PC pc, PetscBool use)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCISSetSubdomainDiagonalScaling_IS"
 static PetscErrorCode PCISSetSubdomainDiagonalScaling_IS(PC pc, Vec scaling_factors)
 {
   PetscErrorCode ierr;
   PC_IS          *pcis = (PC_IS*)pc->data;
 
   PetscFunctionBegin;
-  ierr    = VecDestroy(&pcis->D);CHKERRQ(ierr);
   ierr    = PetscObjectReference((PetscObject)scaling_factors);CHKERRQ(ierr);
+  ierr    = VecDestroy(&pcis->D);CHKERRQ(ierr);
   pcis->D = scaling_factors;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCISSetSubdomainDiagonalScaling"
 /*@
  PCISSetSubdomainDiagonalScaling - Set diagonal scaling for PCIS.
 
@@ -82,8 +74,6 @@ PetscErrorCode PCISSetSubdomainDiagonalScaling(PC pc, Vec scaling_factors)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCISSetSubdomainScalingFactor_IS"
 static PetscErrorCode PCISSetSubdomainScalingFactor_IS(PC pc, PetscScalar scal)
 {
   PC_IS *pcis = (PC_IS*)pc->data;
@@ -93,8 +83,6 @@ static PetscErrorCode PCISSetSubdomainScalingFactor_IS(PC pc, PetscScalar scal)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCISSetSubdomainScalingFactor"
 /*@
  PCISSetSubdomainScalingFactor - Set scaling factor for PCIS.
 
@@ -126,53 +114,45 @@ PetscErrorCode PCISSetSubdomainScalingFactor(PC pc, PetscScalar scal)
 /*
    PCISSetUp -
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCISSetUp"
-PetscErrorCode  PCISSetUp(PC pc)
+PetscErrorCode  PCISSetUp(PC pc, PetscBool computematrices, PetscBool computesolvers)
 {
   PC_IS          *pcis  = (PC_IS*)(pc->data);
-  Mat_IS         *matis = (Mat_IS*)pc->mat->data;
-  PetscInt       i;
+  Mat_IS         *matis;
+  MatReuse       reuse;
   PetscErrorCode ierr;
-  PetscBool      flg;
-  Vec            counter;
+  PetscBool      flg,issbaij;
 
   PetscFunctionBegin;
-  ierr = PetscObjectTypeCompare((PetscObject)pc->mat,MATIS,&flg);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)pc->pmat,MATIS,&flg);CHKERRQ(ierr);
   if (!flg) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONG,"Preconditioner type of Neumann Neumman requires matrix of type MATIS");
+  matis = (Mat_IS*)pc->pmat->data;
 
-  pcis->pure_neumann = matis->pure_neumann;
-
-  /*
-    Creating the local vector vec1_N, containing the inverse of the number
-    of subdomains to which each local node (either owned or ghost)
-    pertains. To accomplish that, we scatter local vectors of 1's to
-    a global vector (adding the values); scatter the result back to
-    local vectors and finally invert the result.
-  */
-  ierr = VecDuplicate(matis->x,&pcis->vec1_N);CHKERRQ(ierr);
-  ierr = MatGetVecs(pc->pmat,&counter,0);CHKERRQ(ierr); /* temporary auxiliar vector */
-  ierr = VecSet(counter,0.0);CHKERRQ(ierr);
-  ierr = VecSet(pcis->vec1_N,1.0);CHKERRQ(ierr);
-  ierr = VecScatterBegin(matis->ctx,pcis->vec1_N,counter,ADD_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
-  ierr = VecScatterEnd  (matis->ctx,pcis->vec1_N,counter,ADD_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
-  ierr = VecScatterBegin(matis->ctx,counter,pcis->vec1_N,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
-  ierr = VecScatterEnd  (matis->ctx,counter,pcis->vec1_N,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
-  /*
-    Creating local and global index sets for interior and
-    inteface nodes. Notice that interior nodes have D[i]==1.0.
-  */
-  {
+  /* first time creation, get info on substructuring */
+  if (!pc->setupcalled) {
     PetscInt    n_I;
     PetscInt    *idx_I_local,*idx_B_local,*idx_I_global,*idx_B_global;
-    PetscScalar *array;
+    PetscBT     bt;
+    PetscInt    i,j;
+
+    /* get info on mapping */
+    ierr = PetscObjectReference((PetscObject)pc->pmat->rmap->mapping);CHKERRQ(ierr);
+    ierr = ISLocalToGlobalMappingDestroy(&pcis->mapping);CHKERRQ(ierr);
+    pcis->mapping = pc->pmat->rmap->mapping;
+    ierr = ISLocalToGlobalMappingGetSize(pcis->mapping,&pcis->n);CHKERRQ(ierr);
+    ierr = ISLocalToGlobalMappingGetInfo(pcis->mapping,&(pcis->n_neigh),&(pcis->neigh),&(pcis->n_shared),&(pcis->shared));CHKERRQ(ierr);
+
     /* Identifying interior and interface nodes, in local numbering */
-    ierr = VecGetSize(pcis->vec1_N,&pcis->n);CHKERRQ(ierr);
-    ierr = VecGetArray(pcis->vec1_N,&array);CHKERRQ(ierr);
-    ierr = PetscMalloc(pcis->n*sizeof(PetscInt),&idx_I_local);CHKERRQ(ierr);
-    ierr = PetscMalloc(pcis->n*sizeof(PetscInt),&idx_B_local);CHKERRQ(ierr);
+    ierr = PetscBTCreate(pcis->n,&bt);CHKERRQ(ierr);
+    for (i=0;i<pcis->n_neigh;i++)
+      for (j=0;j<pcis->n_shared[i];j++) {
+          ierr = PetscBTSet(bt,pcis->shared[i][j]);CHKERRQ(ierr);
+      }
+
+    /* Creating local and global index sets for interior and inteface nodes. */
+    ierr = PetscMalloc1(pcis->n,&idx_I_local);CHKERRQ(ierr);
+    ierr = PetscMalloc1(pcis->n,&idx_B_local);CHKERRQ(ierr);
     for (i=0, pcis->n_B=0, n_I=0; i<pcis->n; i++) {
-      if (array[i] == 1.0) {
+      if (!PetscBTLookup(bt,i)) {
         idx_I_local[n_I] = i;
         n_I++;
       } else {
@@ -180,90 +160,134 @@ PetscErrorCode  PCISSetUp(PC pc)
         pcis->n_B++;
       }
     }
+
     /* Getting the global numbering */
     idx_B_global = idx_I_local + n_I; /* Just avoiding allocating extra memory, since we have vacant space */
     idx_I_global = idx_B_local + pcis->n_B;
-    ierr         = ISLocalToGlobalMappingApply(matis->mapping,pcis->n_B,idx_B_local,idx_B_global);CHKERRQ(ierr);
-    ierr         = ISLocalToGlobalMappingApply(matis->mapping,n_I,      idx_I_local,idx_I_global);CHKERRQ(ierr);
+    ierr         = ISLocalToGlobalMappingApply(pcis->mapping,pcis->n_B,idx_B_local,idx_B_global);CHKERRQ(ierr);
+    ierr         = ISLocalToGlobalMappingApply(pcis->mapping,n_I,idx_I_local,idx_I_global);CHKERRQ(ierr);
 
-    /* Creating the index sets. */
-    ierr = ISCreateGeneral(MPI_COMM_SELF,pcis->n_B,idx_B_local,PETSC_COPY_VALUES, &pcis->is_B_local);CHKERRQ(ierr);
-    ierr = ISCreateGeneral(MPI_COMM_SELF,pcis->n_B,idx_B_global,PETSC_COPY_VALUES,&pcis->is_B_global);CHKERRQ(ierr);
-    ierr = ISCreateGeneral(MPI_COMM_SELF,n_I,idx_I_local,PETSC_COPY_VALUES, &pcis->is_I_local);CHKERRQ(ierr);
-    ierr = ISCreateGeneral(MPI_COMM_SELF,n_I,idx_I_global,PETSC_COPY_VALUES,&pcis->is_I_global);CHKERRQ(ierr);
+    /* Creating the index sets */
+    ierr = ISCreateGeneral(PETSC_COMM_SELF,pcis->n_B,idx_B_local,PETSC_COPY_VALUES, &pcis->is_B_local);CHKERRQ(ierr);
+    ierr = ISCreateGeneral(PetscObjectComm((PetscObject)pc),pcis->n_B,idx_B_global,PETSC_COPY_VALUES,&pcis->is_B_global);CHKERRQ(ierr);
+    ierr = ISCreateGeneral(PETSC_COMM_SELF,n_I,idx_I_local,PETSC_COPY_VALUES, &pcis->is_I_local);CHKERRQ(ierr);
+    ierr = ISCreateGeneral(PetscObjectComm((PetscObject)pc),n_I,idx_I_global,PETSC_COPY_VALUES,&pcis->is_I_global);CHKERRQ(ierr);
 
-    /* Freeing memory and restoring arrays */
+    /* Freeing memory */
     ierr = PetscFree(idx_B_local);CHKERRQ(ierr);
     ierr = PetscFree(idx_I_local);CHKERRQ(ierr);
-    ierr = VecRestoreArray(pcis->vec1_N,&array);CHKERRQ(ierr);
+    ierr = PetscBTDestroy(&bt);CHKERRQ(ierr);
+
+    /* Creating work vectors and arrays */
+    ierr = VecDuplicate(matis->x,&pcis->vec1_N);CHKERRQ(ierr);
+    ierr = VecDuplicate(pcis->vec1_N,&pcis->vec2_N);CHKERRQ(ierr);
+    ierr = VecCreate(PETSC_COMM_SELF,&pcis->vec1_D);CHKERRQ(ierr);
+    ierr = VecSetSizes(pcis->vec1_D,pcis->n-pcis->n_B,PETSC_DECIDE);CHKERRQ(ierr);
+    ierr = VecSetType(pcis->vec1_D,((PetscObject)pcis->vec1_N)->type_name);CHKERRQ(ierr);
+    ierr = VecDuplicate(pcis->vec1_D,&pcis->vec2_D);CHKERRQ(ierr);
+    ierr = VecDuplicate(pcis->vec1_D,&pcis->vec3_D);CHKERRQ(ierr);
+    ierr = VecDuplicate(pcis->vec1_D,&pcis->vec4_D);CHKERRQ(ierr);
+    ierr = VecCreate(PETSC_COMM_SELF,&pcis->vec1_B);CHKERRQ(ierr);
+    ierr = VecSetSizes(pcis->vec1_B,pcis->n_B,PETSC_DECIDE);CHKERRQ(ierr);
+    ierr = VecSetType(pcis->vec1_B,((PetscObject)pcis->vec1_N)->type_name);CHKERRQ(ierr);
+    ierr = VecDuplicate(pcis->vec1_B,&pcis->vec2_B);CHKERRQ(ierr);
+    ierr = VecDuplicate(pcis->vec1_B,&pcis->vec3_B);CHKERRQ(ierr);
+    ierr = MatCreateVecs(pc->pmat,&pcis->vec1_global,0);CHKERRQ(ierr);
+    ierr = PetscMalloc1(pcis->n,&pcis->work_N);CHKERRQ(ierr);
+    /* scaling vector */
+    if (!pcis->D) { /* it can happen that the user passed in a scaling vector via PCISSetSubdomainDiagonalScaling */
+      ierr = VecDuplicate(pcis->vec1_B,&pcis->D);CHKERRQ(ierr);
+      ierr = VecSet(pcis->D,pcis->scaling_factor);CHKERRQ(ierr);
+    }
+
+    /* Creating the scatter contexts */
+    ierr = VecScatterCreate(pcis->vec1_N,pcis->is_I_local,pcis->vec1_D,(IS)0,&pcis->N_to_D);CHKERRQ(ierr);
+    ierr = VecScatterCreate(pcis->vec1_global,pcis->is_I_global,pcis->vec1_D,(IS)0,&pcis->global_to_D);CHKERRQ(ierr);
+    ierr = VecScatterCreate(pcis->vec1_N,pcis->is_B_local,pcis->vec1_B,(IS)0,&pcis->N_to_B);CHKERRQ(ierr);
+    ierr = VecScatterCreate(pcis->vec1_global,pcis->is_B_global,pcis->vec1_B,(IS)0,&pcis->global_to_B);CHKERRQ(ierr);
+
+    /* map from boundary to local */
+    ierr = ISLocalToGlobalMappingCreateIS(pcis->is_B_local,&pcis->BtoNmap);CHKERRQ(ierr);
   }
 
   /*
     Extracting the blocks A_II, A_BI, A_IB and A_BB from A. If the numbering
     is such that interior nodes come first than the interface ones, we have
 
-    [           |      ]
-    [    A_II   | A_IB ]
-    A = [           |      ]
-    [-----------+------]
-    [    A_BI   | A_BB ]
+        [ A_II | A_IB ]
+    A = [------+------]
+        [ A_BI | A_BB ]
   */
+  if (computematrices) {
+    reuse = MAT_INITIAL_MATRIX;
+    if (pcis->reusesubmatrices && pc->setupcalled) {
+      if (pc->flag == SAME_NONZERO_PATTERN) {
+        reuse = MAT_REUSE_MATRIX;
+      } else {
+        reuse = MAT_INITIAL_MATRIX;
+      }
+    }
+    if (reuse == MAT_INITIAL_MATRIX) {
+      ierr = MatDestroy(&pcis->A_II);CHKERRQ(ierr);
+      ierr = MatDestroy(&pcis->A_IB);CHKERRQ(ierr);
+      ierr = MatDestroy(&pcis->A_BI);CHKERRQ(ierr);
+      ierr = MatDestroy(&pcis->A_BB);CHKERRQ(ierr);
+    }
 
-  ierr = MatGetSubMatrix(matis->A,pcis->is_I_local,pcis->is_I_local,MAT_INITIAL_MATRIX,&pcis->A_II);CHKERRQ(ierr);
-  ierr = MatGetSubMatrix(matis->A,pcis->is_I_local,pcis->is_B_local,MAT_INITIAL_MATRIX,&pcis->A_IB);CHKERRQ(ierr);
-  ierr = MatGetSubMatrix(matis->A,pcis->is_B_local,pcis->is_I_local,MAT_INITIAL_MATRIX,&pcis->A_BI);CHKERRQ(ierr);
-  ierr = MatGetSubMatrix(matis->A,pcis->is_B_local,pcis->is_B_local,MAT_INITIAL_MATRIX,&pcis->A_BB);CHKERRQ(ierr);
+    ierr = MatCreateSubMatrix(matis->A,pcis->is_I_local,pcis->is_I_local,reuse,&pcis->A_II);CHKERRQ(ierr);
+    ierr = MatCreateSubMatrix(matis->A,pcis->is_B_local,pcis->is_B_local,reuse,&pcis->A_BB);CHKERRQ(ierr);
+    ierr = PetscObjectTypeCompare((PetscObject)matis->A,MATSEQSBAIJ,&issbaij);CHKERRQ(ierr);
+    if (!issbaij) {
+      ierr = MatCreateSubMatrix(matis->A,pcis->is_I_local,pcis->is_B_local,reuse,&pcis->A_IB);CHKERRQ(ierr);
+      ierr = MatCreateSubMatrix(matis->A,pcis->is_B_local,pcis->is_I_local,reuse,&pcis->A_BI);CHKERRQ(ierr);
+    } else {
+      Mat newmat;
 
-  /*
-    Creating work vectors and arrays
-  */
-  /* pcis->vec1_N has already been created */
-  ierr = VecDuplicate(pcis->vec1_N,&pcis->vec2_N);CHKERRQ(ierr);
-  ierr = VecCreateSeq(PETSC_COMM_SELF,pcis->n-pcis->n_B,&pcis->vec1_D);CHKERRQ(ierr);
-  ierr = VecDuplicate(pcis->vec1_D,&pcis->vec2_D);CHKERRQ(ierr);
-  ierr = VecDuplicate(pcis->vec1_D,&pcis->vec3_D);CHKERRQ(ierr);
-  ierr = VecCreateSeq(PETSC_COMM_SELF,pcis->n_B,&pcis->vec1_B);CHKERRQ(ierr);
-  ierr = VecDuplicate(pcis->vec1_B,&pcis->vec2_B);CHKERRQ(ierr);
-  ierr = VecDuplicate(pcis->vec1_B,&pcis->vec3_B);CHKERRQ(ierr);
-  ierr = MatGetVecs(pc->pmat,&pcis->vec1_global,0);CHKERRQ(ierr);
-  ierr = PetscMalloc((pcis->n)*sizeof(PetscScalar),&pcis->work_N);CHKERRQ(ierr);
+      ierr = MatConvert(matis->A,MATSEQBAIJ,MAT_INITIAL_MATRIX,&newmat);CHKERRQ(ierr);
+      ierr = MatCreateSubMatrix(newmat,pcis->is_I_local,pcis->is_B_local,reuse,&pcis->A_IB);CHKERRQ(ierr);
+      ierr = MatCreateSubMatrix(newmat,pcis->is_B_local,pcis->is_I_local,reuse,&pcis->A_BI);CHKERRQ(ierr);
+      ierr = MatDestroy(&newmat);CHKERRQ(ierr);
+    }
+  }
 
-  /* Creating the scatter contexts */
-  ierr = VecScatterCreate(pcis->vec1_global,pcis->is_I_global,pcis->vec1_D,(IS)0,&pcis->global_to_D);CHKERRQ(ierr);
-  ierr = VecScatterCreate(pcis->vec1_N,pcis->is_B_local,pcis->vec1_B,(IS)0,&pcis->N_to_B);CHKERRQ(ierr);
-  ierr = VecScatterCreate(pcis->vec1_global,pcis->is_B_global,pcis->vec1_B,(IS)0,&pcis->global_to_B);CHKERRQ(ierr);
+  /* Creating scaling vector D */
+  ierr = PetscOptionsGetBool(((PetscObject)pc)->options,((PetscObject)pc)->prefix,"-pc_is_use_stiffness_scaling",&pcis->use_stiffness_scaling,NULL);CHKERRQ(ierr);
+  if (pcis->use_stiffness_scaling) {
+    PetscScalar *a;
+    PetscInt    i,n;
 
-  /* Creating scaling "matrix" D */
-  ierr = PetscOptionsGetBool(((PetscObject)pc)->prefix,"-pc_is_use_stiffness_scaling",&pcis->use_stiffness_scaling,NULL);CHKERRQ(ierr);
-  if (!pcis->D) {
-    ierr = VecDuplicate(pcis->vec1_B,&pcis->D);CHKERRQ(ierr);
-    if (!pcis->use_stiffness_scaling) {
-      ierr = VecSet(pcis->D,pcis->scaling_factor);CHKERRQ(ierr);
+    if (pcis->A_BB) {
+      ierr = MatGetDiagonal(pcis->A_BB,pcis->D);CHKERRQ(ierr);
     } else {
       ierr = MatGetDiagonal(matis->A,pcis->vec1_N);CHKERRQ(ierr);
       ierr = VecScatterBegin(pcis->N_to_B,pcis->vec1_N,pcis->D,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
-      ierr = VecScatterEnd  (pcis->N_to_B,pcis->vec1_N,pcis->D,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
+      ierr = VecScatterEnd(pcis->N_to_B,pcis->vec1_N,pcis->D,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
     }
+    ierr = VecAbs(pcis->D);CHKERRQ(ierr);
+    ierr = VecGetLocalSize(pcis->D,&n);CHKERRQ(ierr);
+    ierr = VecGetArray(pcis->D,&a);CHKERRQ(ierr);
+    for (i=0;i<n;i++) if (PetscAbsScalar(a[i])<PETSC_SMALL) a[i] = 1.0;
+    ierr = VecRestoreArray(pcis->D,&a);CHKERRQ(ierr);
   }
-  ierr = VecCopy(pcis->D,pcis->vec1_B);CHKERRQ(ierr);
-  ierr = VecSet(counter,0.0);CHKERRQ(ierr);
-  ierr = VecScatterBegin(pcis->global_to_B,pcis->vec1_B,counter,ADD_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
-  ierr = VecScatterEnd  (pcis->global_to_B,pcis->vec1_B,counter,ADD_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
-  ierr = VecScatterBegin(pcis->global_to_B,counter,pcis->vec1_B,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
-  ierr = VecScatterEnd  (pcis->global_to_B,counter,pcis->vec1_B,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
+  ierr = VecSet(pcis->vec1_global,0.0);CHKERRQ(ierr);
+  ierr = VecScatterBegin(pcis->global_to_B,pcis->D,pcis->vec1_global,ADD_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
+  ierr = VecScatterEnd(pcis->global_to_B,pcis->D,pcis->vec1_global,ADD_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
+  ierr = VecScatterBegin(pcis->global_to_B,pcis->vec1_global,pcis->vec1_B,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
+  ierr = VecScatterEnd(pcis->global_to_B,pcis->vec1_global,pcis->vec1_B,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
   ierr = VecPointwiseDivide(pcis->D,pcis->D,pcis->vec1_B);CHKERRQ(ierr);
 
   /* See historical note 01, at the bottom of this file. */
 
-  /*
-    Creating the KSP contexts for the local Dirichlet and Neumann problems.
-  */
-  {
+  /* Creating the KSP contexts for the local Dirichlet and Neumann problems */
+  if (computesolvers) {
     PC pc_ctx;
+
+    pcis->pure_neumann = matis->pure_neumann;
     /* Dirichlet */
     ierr = KSPCreate(PETSC_COMM_SELF,&pcis->ksp_D);CHKERRQ(ierr);
+    ierr = KSPSetErrorIfNotConverged(pcis->ksp_D,pc->erroriffailure);CHKERRQ(ierr);
     ierr = PetscObjectIncrementTabLevel((PetscObject)pcis->ksp_D,(PetscObject)pc,1);CHKERRQ(ierr);
-    ierr = KSPSetOperators(pcis->ksp_D,pcis->A_II,pcis->A_II,SAME_PRECONDITIONER);CHKERRQ(ierr);
+    ierr = KSPSetOperators(pcis->ksp_D,pcis->A_II,pcis->A_II);CHKERRQ(ierr);
     ierr = KSPSetOptionsPrefix(pcis->ksp_D,"is_localD_");CHKERRQ(ierr);
     ierr = KSPGetPC(pcis->ksp_D,&pc_ctx);CHKERRQ(ierr);
     ierr = PCSetType(pc_ctx,PCLU);CHKERRQ(ierr);
@@ -273,8 +297,9 @@ PetscErrorCode  PCISSetUp(PC pc)
     ierr = KSPSetUp(pcis->ksp_D);CHKERRQ(ierr);
     /* Neumann */
     ierr = KSPCreate(PETSC_COMM_SELF,&pcis->ksp_N);CHKERRQ(ierr);
+    ierr = KSPSetErrorIfNotConverged(pcis->ksp_N,pc->erroriffailure);CHKERRQ(ierr);
     ierr = PetscObjectIncrementTabLevel((PetscObject)pcis->ksp_N,(PetscObject)pc,1);CHKERRQ(ierr);
-    ierr = KSPSetOperators(pcis->ksp_N,matis->A,matis->A,SAME_PRECONDITIONER);CHKERRQ(ierr);
+    ierr = KSPSetOperators(pcis->ksp_N,matis->A,matis->A);CHKERRQ(ierr);
     ierr = KSPSetOptionsPrefix(pcis->ksp_N,"is_localN_");CHKERRQ(ierr);
     ierr = KSPGetPC(pcis->ksp_N,&pc_ctx);CHKERRQ(ierr);
     ierr = PCSetType(pc_ctx,PCLU);CHKERRQ(ierr);
@@ -289,21 +314,21 @@ PetscErrorCode  PCISSetUp(PC pc)
       PetscReal fixed_factor,
                 floating_factor;
 
-      ierr = PetscOptionsGetReal(((PetscObject)pc_ctx)->prefix,"-pc_is_damp_fixed",&fixed_factor,&damp_fixed);CHKERRQ(ierr);
+      ierr = PetscOptionsGetReal(((PetscObject)pc_ctx)->options,((PetscObject)pc_ctx)->prefix,"-pc_is_damp_fixed",&fixed_factor,&damp_fixed);CHKERRQ(ierr);
       if (!damp_fixed) fixed_factor = 0.0;
-      ierr = PetscOptionsGetBool(((PetscObject)pc_ctx)->prefix,"-pc_is_damp_fixed",&damp_fixed,NULL);CHKERRQ(ierr);
+      ierr = PetscOptionsGetBool(((PetscObject)pc_ctx)->options,((PetscObject)pc_ctx)->prefix,"-pc_is_damp_fixed",&damp_fixed,NULL);CHKERRQ(ierr);
 
-      ierr = PetscOptionsGetBool(((PetscObject)pc_ctx)->prefix,"-pc_is_remove_nullspace_fixed",&remove_nullspace_fixed,NULL);CHKERRQ(ierr);
+      ierr = PetscOptionsGetBool(((PetscObject)pc_ctx)->options,((PetscObject)pc_ctx)->prefix,"-pc_is_remove_nullspace_fixed",&remove_nullspace_fixed,NULL);CHKERRQ(ierr);
 
-      ierr = PetscOptionsGetReal(((PetscObject)pc_ctx)->prefix,"-pc_is_set_damping_factor_floating",
+      ierr = PetscOptionsGetReal(((PetscObject)pc_ctx)->options,((PetscObject)pc_ctx)->prefix,"-pc_is_set_damping_factor_floating",
                               &floating_factor,&set_damping_factor_floating);CHKERRQ(ierr);
       if (!set_damping_factor_floating) floating_factor = 0.0;
-      ierr = PetscOptionsGetBool(((PetscObject)pc_ctx)->prefix,"-pc_is_set_damping_factor_floating",&set_damping_factor_floating,NULL);CHKERRQ(ierr);
+      ierr = PetscOptionsGetBool(((PetscObject)pc_ctx)->options,((PetscObject)pc_ctx)->prefix,"-pc_is_set_damping_factor_floating",&set_damping_factor_floating,NULL);CHKERRQ(ierr);
       if (!set_damping_factor_floating) floating_factor = 1.e-12;
 
-      ierr = PetscOptionsGetBool(((PetscObject)pc_ctx)->prefix,"-pc_is_not_damp_floating",&not_damp_floating,NULL);CHKERRQ(ierr);
+      ierr = PetscOptionsGetBool(((PetscObject)pc_ctx)->options,((PetscObject)pc_ctx)->prefix,"-pc_is_not_damp_floating",&not_damp_floating,NULL);CHKERRQ(ierr);
 
-      ierr = PetscOptionsGetBool(((PetscObject)pc_ctx)->prefix,"-pc_is_not_remove_nullspace_floating",&not_remove_nullspace_floating,NULL);CHKERRQ(ierr);
+      ierr = PetscOptionsGetBool(((PetscObject)pc_ctx)->options,((PetscObject)pc_ctx)->prefix,"-pc_is_not_remove_nullspace_floating",&not_remove_nullspace_floating,NULL);CHKERRQ(ierr);
 
       if (pcis->pure_neumann) {  /* floating subdomain */
         if (!(not_damp_floating)) {
@@ -313,7 +338,7 @@ PetscErrorCode  PCISSetUp(PC pc)
         if (!(not_remove_nullspace_floating)) {
           MatNullSpace nullsp;
           ierr = MatNullSpaceCreate(PETSC_COMM_SELF,PETSC_TRUE,0,NULL,&nullsp);CHKERRQ(ierr);
-          ierr = KSPSetNullSpace(pcis->ksp_N,nullsp);CHKERRQ(ierr);
+          ierr = MatSetNullSpace(matis->A,nullsp);CHKERRQ(ierr);
           ierr = MatNullSpaceDestroy(&nullsp);CHKERRQ(ierr);
         }
       } else {  /* fixed subdomain */
@@ -324,7 +349,7 @@ PetscErrorCode  PCISSetUp(PC pc)
         if (remove_nullspace_fixed) {
           MatNullSpace nullsp;
           ierr = MatNullSpaceCreate(PETSC_COMM_SELF,PETSC_TRUE,0,NULL,&nullsp);CHKERRQ(ierr);
-          ierr = KSPSetNullSpace(pcis->ksp_N,nullsp);CHKERRQ(ierr);
+          ierr = MatSetNullSpace(matis->A,nullsp);CHKERRQ(ierr);
           ierr = MatNullSpaceDestroy(&nullsp);CHKERRQ(ierr);
         }
       }
@@ -332,12 +357,6 @@ PetscErrorCode  PCISSetUp(PC pc)
     /* the vectors in the following line are dummy arguments, just telling the KSP the vector size. Values are not used */
     ierr = KSPSetUp(pcis->ksp_N);CHKERRQ(ierr);
   }
-
-  ierr = ISLocalToGlobalMappingGetInfo(((Mat_IS*)(pc->mat->data))->mapping,&(pcis->n_neigh),&(pcis->neigh),&(pcis->n_shared),&(pcis->shared));CHKERRQ(ierr);
-
-  pcis->ISLocalToGlobalMappingGetInfoWasCalled = PETSC_TRUE;
-
-  ierr = VecDestroy(&counter);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -345,8 +364,6 @@ PetscErrorCode  PCISSetUp(PC pc)
 /*
    PCISDestroy -
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCISDestroy"
 PetscErrorCode  PCISDestroy(PC pc)
 {
   PC_IS          *pcis = (PC_IS*)(pc->data);
@@ -369,17 +386,21 @@ PetscErrorCode  PCISDestroy(PC pc)
   ierr = VecDestroy(&pcis->vec1_D);CHKERRQ(ierr);
   ierr = VecDestroy(&pcis->vec2_D);CHKERRQ(ierr);
   ierr = VecDestroy(&pcis->vec3_D);CHKERRQ(ierr);
+  ierr = VecDestroy(&pcis->vec4_D);CHKERRQ(ierr);
   ierr = VecDestroy(&pcis->vec1_B);CHKERRQ(ierr);
   ierr = VecDestroy(&pcis->vec2_B);CHKERRQ(ierr);
   ierr = VecDestroy(&pcis->vec3_B);CHKERRQ(ierr);
   ierr = VecDestroy(&pcis->vec1_global);CHKERRQ(ierr);
   ierr = VecScatterDestroy(&pcis->global_to_D);CHKERRQ(ierr);
   ierr = VecScatterDestroy(&pcis->N_to_B);CHKERRQ(ierr);
+  ierr = VecScatterDestroy(&pcis->N_to_D);CHKERRQ(ierr);
   ierr = VecScatterDestroy(&pcis->global_to_B);CHKERRQ(ierr);
   ierr = PetscFree(pcis->work_N);CHKERRQ(ierr);
-  if (pcis->ISLocalToGlobalMappingGetInfoWasCalled) {
-    ierr = ISLocalToGlobalMappingRestoreInfo((ISLocalToGlobalMapping)0,&(pcis->n_neigh),&(pcis->neigh),&(pcis->n_shared),&(pcis->shared));CHKERRQ(ierr);
+  if (pcis->n_neigh > -1) {
+    ierr = ISLocalToGlobalMappingRestoreInfo(pcis->mapping,&(pcis->n_neigh),&(pcis->neigh),&(pcis->n_shared),&(pcis->shared));CHKERRQ(ierr);
   }
+  ierr = ISLocalToGlobalMappingDestroy(&pcis->mapping);CHKERRQ(ierr);
+  ierr = ISLocalToGlobalMappingDestroy(&pcis->BtoNmap);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCISSetUseStiffnessScaling_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCISSetSubdomainScalingFactor_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCISSetSubdomainDiagonalScaling_C",NULL);CHKERRQ(ierr);
@@ -390,42 +411,15 @@ PetscErrorCode  PCISDestroy(PC pc)
 /*
    PCISCreate -
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCISCreate"
 PetscErrorCode  PCISCreate(PC pc)
 {
   PC_IS          *pcis = (PC_IS*)(pc->data);
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  pcis->is_B_local  = 0;
-  pcis->is_I_local  = 0;
-  pcis->is_B_global = 0;
-  pcis->is_I_global = 0;
-  pcis->A_II        = 0;
-  pcis->A_IB        = 0;
-  pcis->A_BI        = 0;
-  pcis->A_BB        = 0;
-  pcis->D           = 0;
-  pcis->ksp_N       = 0;
-  pcis->ksp_D      = 0;
-  pcis->vec1_N      = 0;
-  pcis->vec2_N      = 0;
-  pcis->vec1_D      = 0;
-  pcis->vec2_D      = 0;
-  pcis->vec3_D      = 0;
-  pcis->vec1_B      = 0;
-  pcis->vec2_B      = 0;
-  pcis->vec3_B      = 0;
-  pcis->vec1_global = 0;
-  pcis->work_N      = 0;
-  pcis->global_to_D = 0;
-  pcis->N_to_B      = 0;
-  pcis->global_to_B = 0;
-
-  pcis->ISLocalToGlobalMappingGetInfoWasCalled = PETSC_FALSE;
-
-  pcis->scaling_factor = 1.0;
+  pcis->n_neigh          = -1;
+  pcis->scaling_factor   = 1.0;
+  pcis->reusesubmatrices = PETSC_TRUE;
   /* composing functions */
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCISSetUseStiffnessScaling_C",PCISSetUseStiffnessScaling_IS);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCISSetSubdomainScalingFactor_C",PCISSetSubdomainScalingFactor_IS);CHKERRQ(ierr);
@@ -448,8 +442,6 @@ PetscErrorCode  PCISCreate(PC pc)
 .  vec2_D - garbage (used as work space)
 
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCISApplySchur"
 PetscErrorCode  PCISApplySchur(PC pc, Vec v, Vec vec1_B, Vec vec2_B, Vec vec1_D, Vec vec2_D)
 {
   PetscErrorCode ierr;
@@ -484,8 +476,6 @@ PetscErrorCode  PCISApplySchur(PC pc, Vec v, Vec vec1_B, Vec vec2_B, Vec vec1_D,
    Notes:
    The entries in the array that do not correspond to interface nodes remain unaltered.
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCISScatterArrayNToVecB"
 PetscErrorCode  PCISScatterArrayNToVecB(PetscScalar *array_N, Vec v_B, InsertMode imode, ScatterMode smode, PC pc)
 {
   PetscInt       i;
@@ -535,8 +525,6 @@ PetscErrorCode  PCISScatterArrayNToVecB(PetscScalar *array_N, Vec v_B, InsertMod
 .  vec2_N - vector of local nodes (interior and interface, including ghosts); returns garbage (used as work space)
 
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCISApplyInvSchur"
 PetscErrorCode  PCISApplyInvSchur(PC pc, Vec b, Vec x, Vec vec1_N, Vec vec2_N)
 {
   PetscErrorCode ierr;
@@ -557,7 +545,7 @@ PetscErrorCode  PCISApplyInvSchur(PC pc, Vec b, Vec x, Vec vec1_N, Vec vec2_N)
   /* Checking for consistency of the RHS */
   {
     PetscBool flg = PETSC_FALSE;
-    ierr = PetscOptionsGetBool(NULL,"-pc_is_check_consistency",&flg,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsGetBool(NULL,NULL,"-pc_is_check_consistency",&flg,NULL);CHKERRQ(ierr);
     if (flg) {
       PetscScalar average;
       PetscViewer viewer;
@@ -565,14 +553,14 @@ PetscErrorCode  PCISApplyInvSchur(PC pc, Vec b, Vec x, Vec vec1_N, Vec vec2_N)
 
       ierr    = VecSum(vec1_N,&average);CHKERRQ(ierr);
       average = average / ((PetscReal)pcis->n);
-      ierr    = PetscViewerASCIISynchronizedAllow(viewer,PETSC_TRUE);CHKERRQ(ierr);
+      ierr    = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
       if (pcis->pure_neumann) {
         ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Subdomain %04d is floating. Average = % 1.14e\n",PetscGlobalRank,PetscAbsScalar(average));CHKERRQ(ierr);
       } else {
         ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Subdomain %04d is fixed.    Average = % 1.14e\n",PetscGlobalRank,PetscAbsScalar(average));CHKERRQ(ierr);
       }
       ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
-      ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_FALSE);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPopSynchronized(viewer);CHKERRQ(ierr);
     }
   }
   /* Solving the system for vec2_N */
@@ -582,12 +570,3 @@ PetscErrorCode  PCISApplyInvSchur(PC pc, Vec b, Vec x, Vec vec1_N, Vec vec2_N)
   ierr = VecScatterEnd  (pcis->N_to_B,vec2_N,x,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-
-
-
-
-
-
-
-
-
