@@ -1,15 +1,11 @@
-
 #include <petsc/private/pcimpl.h>
 #include <petscksp.h>            /*I "petscksp.h" I*/
 
 typedef struct {
-  KSP       ksp;
-  PetscInt  its;                    /* total number of iterations KSP uses */
+  KSP      ksp;
+  PetscInt its; /* total number of iterations KSP uses */
 } PC_KSP;
 
-
-#undef __FUNCT__
-#define __FUNCT__ "PCKSPCreateKSP_KSP"
 static PetscErrorCode  PCKSPCreateKSP_KSP(PC pc)
 {
   PetscErrorCode ierr;
@@ -26,39 +22,42 @@ static PetscErrorCode  PCKSPCreateKSP_KSP(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_KSP"
 static PetscErrorCode PCApply_KSP(PC pc,Vec x,Vec y)
 {
-  PetscErrorCode ierr;
-  PetscInt       its;
-  PC_KSP         *jac = (PC_KSP*)pc->data;
+  PetscErrorCode     ierr;
+  PetscInt           its;
+  PC_KSP             *jac = (PC_KSP*)pc->data;
+  KSPConvergedReason reason;
 
   PetscFunctionBegin;
-  ierr      = KSPSetInitialGuessNonzero(jac->ksp,pc->nonzero_guess);CHKERRQ(ierr);
-  ierr      = KSPSolve(jac->ksp,x,y);CHKERRQ(ierr);
+  ierr = KSPSolve(jac->ksp,x,y);CHKERRQ(ierr);
+  ierr = KSPGetConvergedReason(jac->ksp,&reason);CHKERRQ(ierr);
+  if (reason == KSP_DIVERGED_PCSETUP_FAILED) {
+    pc->failedreason = PC_SUBPC_ERROR;
+  }
   ierr      = KSPGetIterationNumber(jac->ksp,&its);CHKERRQ(ierr);
   jac->its += its;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyTranspose_KSP"
 static PetscErrorCode PCApplyTranspose_KSP(PC pc,Vec x,Vec y)
 {
-  PetscErrorCode ierr;
-  PetscInt       its;
-  PC_KSP         *jac = (PC_KSP*)pc->data;
+  PetscErrorCode     ierr;
+  PetscInt           its;
+  PC_KSP             *jac = (PC_KSP*)pc->data;
+  KSPConvergedReason reason;
 
   PetscFunctionBegin;
-  ierr      = KSPSolveTranspose(jac->ksp,x,y);CHKERRQ(ierr);
+  ierr = KSPSolveTranspose(jac->ksp,x,y);CHKERRQ(ierr);
+  ierr = KSPGetConvergedReason(jac->ksp,&reason);CHKERRQ(ierr);
+  if (reason == KSP_DIVERGED_PCSETUP_FAILED) {
+    pc->failedreason = PC_SUBPC_ERROR;
+  }
   ierr      = KSPGetIterationNumber(jac->ksp,&its);CHKERRQ(ierr);
   jac->its += its;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUp_KSP"
 static PetscErrorCode PCSetUp_KSP(PC pc)
 {
   PetscErrorCode ierr;
@@ -66,45 +65,41 @@ static PetscErrorCode PCSetUp_KSP(PC pc)
   Mat            mat;
 
   PetscFunctionBegin;
-  if (!jac->ksp) {ierr = PCKSPCreateKSP_KSP(pc);CHKERRQ(ierr);}
-  ierr = KSPSetFromOptions(jac->ksp);CHKERRQ(ierr);
+  if (!jac->ksp) {
+    ierr = PCKSPCreateKSP_KSP(pc);CHKERRQ(ierr);
+    ierr = KSPSetFromOptions(jac->ksp);CHKERRQ(ierr);
+  }
   if (pc->useAmat) mat = pc->mat;
   else             mat = pc->pmat;
-
   ierr = KSPSetOperators(jac->ksp,mat,pc->pmat);CHKERRQ(ierr);
   ierr = KSPSetUp(jac->ksp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 /* Default destroy, if it has never been setup */
-#undef __FUNCT__
-#define __FUNCT__ "PCReset_KSP"
 static PetscErrorCode PCReset_KSP(PC pc)
 {
   PC_KSP         *jac = (PC_KSP*)pc->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (jac->ksp) {ierr = KSPReset(jac->ksp);CHKERRQ(ierr);}
+  ierr = KSPDestroy(&jac->ksp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCDestroy_KSP"
 static PetscErrorCode PCDestroy_KSP(PC pc)
 {
   PC_KSP         *jac = (PC_KSP*)pc->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PCReset_KSP(pc);CHKERRQ(ierr);
   ierr = KSPDestroy(&jac->ksp);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCKSPGetKSP_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCKSPSetKSP_C",NULL);CHKERRQ(ierr);
   ierr = PetscFree(pc->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCView_KSP"
 static PetscErrorCode PCView_KSP(PC pc,PetscViewer viewer)
 {
   PC_KSP         *jac = (PC_KSP*)pc->data;
@@ -116,22 +111,60 @@ static PetscErrorCode PCView_KSP(PC pc,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
     if (pc->useAmat) {
-      ierr = PetscViewerASCIIPrintf(viewer,"Using Amat (not Pmat) as operator on inner solve\n");CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  Using Amat (not Pmat) as operator on inner solve\n");CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIIPrintf(viewer,"KSP and PC on KSP preconditioner follow\n");CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"---------------------------------\n");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  KSP and PC on KSP preconditioner follow\n");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  ---------------------------------\n");CHKERRQ(ierr);
   }
   ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
   ierr = KSPView(jac->ksp,viewer);CHKERRQ(ierr);
   ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
   if (iascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"---------------------------------\n");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  ---------------------------------\n");CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCKSPGetKSP_KSP"
+static PetscErrorCode  PCKSPSetKSP_KSP(PC pc,KSP ksp)
+{
+  PC_KSP         *jac = (PC_KSP*)pc->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectReference((PetscObject)ksp);CHKERRQ(ierr);
+  ierr = KSPDestroy(&jac->ksp);CHKERRQ(ierr);
+  jac->ksp = ksp;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   PCKSPSetKSP - Sets the KSP context for a KSP PC.
+
+   Collective on PC
+
+   Input Parameter:
++  pc - the preconditioner context
+-  ksp - the KSP solver
+
+   Notes:
+   The PC and the KSP must have the same communicator
+
+   Level: advanced
+
+.keywords:  PC, KSP, set, context
+@*/
+PetscErrorCode  PCKSPSetKSP(PC pc,KSP ksp)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  PetscValidHeaderSpecific(ksp,KSP_CLASSID,2);
+  PetscCheckSameComm(pc,1,ksp,2);
+  ierr = PetscTryMethod(pc,"PCKSPSetKSP_C",(PC,KSP),(pc,ksp));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 static PetscErrorCode  PCKSPGetKSP_KSP(PC pc,KSP *ksp)
 {
   PC_KSP         *jac = (PC_KSP*)pc->data;
@@ -143,8 +176,6 @@ static PetscErrorCode  PCKSPGetKSP_KSP(PC pc,KSP *ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCKSPGetKSP"
 /*@
    PCKSPGetKSP - Gets the KSP context for a KSP PC.
 
@@ -154,11 +185,13 @@ static PetscErrorCode  PCKSPGetKSP_KSP(PC pc,KSP *ksp)
 .  pc - the preconditioner context
 
    Output Parameters:
-.  ksp - the PC solver
+.  ksp - the KSP solver
 
    Notes:
    You must call KSPSetUp() before calling PCKSPGetKSP().
 
+   If the PC is not a PCKSP object it raises an error
+
    Level: advanced
 
 .keywords:  PC, KSP, get, context
@@ -170,8 +203,21 @@ PetscErrorCode  PCKSPGetKSP(PC pc,KSP *ksp)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
   PetscValidPointer(ksp,2);
-  *ksp = NULL;
-  ierr = PetscTryMethod(pc,"PCKSPGetKSP_C",(PC,KSP*),(pc,ksp));CHKERRQ(ierr);
+  ierr = PetscUseMethod(pc,"PCKSPGetKSP_C",(PC,KSP*),(pc,ksp));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode PCSetFromOptions_KSP(PetscOptionItems *PetscOptionsObject,PC pc)
+{
+  PC_KSP         *jac = (PC_KSP*)pc->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscOptionsHead(PetscOptionsObject,"PC KSP options");CHKERRQ(ierr);
+  if (jac->ksp) {
+    ierr = KSPSetFromOptions(jac->ksp);CHKERRQ(ierr);
+   }
+  ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -190,20 +236,25 @@ PetscErrorCode  PCKSPGetKSP(PC pc,KSP *ksp)
 
    Concepts: inner iteration
 
-   Notes: Using a Krylov method inside another Krylov method can be dangerous (you get divergence or
+   Notes:
+    Using a Krylov method inside another Krylov method can be dangerous (you get divergence or
           the incorrect answer) unless you use KSPFGMRES as the other Krylov method
 
-   Developer Notes: PCApply_KSP() uses the flag set by PCSetInitialGuessNonzero(), I think this is totally wrong, because it is then not
-     using this inner KSP as a preconditioner (that is a linear operator applied to some vector), it is actually just using
-     the inner KSP just like the outer KSP.
+   Developer Notes:
+    If the outer Krylov method has a nonzero initial guess it will compute a new residual based on that initial guess
+    and pass that as the right hand side into this KSP (and hence this KSP will always have a zero initial guess). For all outer Krylov methods
+    except Richardson this is neccessary since Krylov methods, even the flexible ones, need to "see" the result of the action of the preconditioner on the
+    input (current residual) vector, the action of the preconditioner cannot depend also on some other vector (the "initial guess"). For
+    KSPRICHARDSON it is possible to provide a PCApplyRichardson_PCKSP() that short circuits returning to the KSP object at each iteration to compute the
+    residual, see for example PCApplyRichardson_SOR(). We do not implement a PCApplyRichardson_PCKSP()  because (1) using a KSP directly inside a Richardson
+    is not an efficient algorithm anyways and (2) implementing it for its > 1 would essentially require that we implement Richardson (reimplementing the
+    Richardson code) inside the PCApplyRichardson_PCKSP() leading to duplicate code.
 
 .seealso:  PCCreate(), PCSetType(), PCType (for list of available types), PC,
            PCSHELL, PCCOMPOSITE, PCSetUseAmat(), PCKSPGetKSP()
 
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCCreate_KSP"
 PETSC_EXTERN PetscErrorCode PCCreate_KSP(PC pc)
 {
   PetscErrorCode ierr;
@@ -211,21 +262,18 @@ PETSC_EXTERN PetscErrorCode PCCreate_KSP(PC pc)
 
   PetscFunctionBegin;
   ierr = PetscNewLog(pc,&jac);CHKERRQ(ierr);
+  pc->data = (void*)jac;
 
+  ierr = PetscMemzero(pc->ops,sizeof(struct _PCOps));CHKERRQ(ierr);
   pc->ops->apply           = PCApply_KSP;
   pc->ops->applytranspose  = PCApplyTranspose_KSP;
   pc->ops->setup           = PCSetUp_KSP;
   pc->ops->reset           = PCReset_KSP;
   pc->ops->destroy         = PCDestroy_KSP;
-  pc->ops->setfromoptions  = 0;
+  pc->ops->setfromoptions  = PCSetFromOptions_KSP;
   pc->ops->view            = PCView_KSP;
-  pc->ops->applyrichardson = 0;
 
-  pc->data = (void*)jac;
-
-
-  jac->its             = 0;
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCKSPGetKSP_C",PCKSPGetKSP_KSP);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCKSPSetKSP_C",PCKSPSetKSP_KSP);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-
