@@ -1,10 +1,9 @@
 
-#include <../src/ksp/pc/impls/mg/mgimpl.h>       /*I "petscksp.h" I*/
+#include <petsc/private/pcmgimpl.h>       /*I "petscksp.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGResidual_Default"
+/* ---------------------------------------------------------------------------*/
 /*@C
-   PCMGResidual_Default - Default routine to calculate the residual.
+   PCMGResidualDefault - Default routine to calculate the residual.
 
    Collective on Mat and Vec
 
@@ -22,20 +21,15 @@
 
 .seealso: PCMGSetResidual()
 @*/
-PetscErrorCode  PCMGResidual_Default(Mat mat,Vec b,Vec x,Vec r)
+PetscErrorCode  PCMGResidualDefault(Mat mat,Vec b,Vec x,Vec r)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = MatMult(mat,x,r);CHKERRQ(ierr);
-  ierr = VecAYPX(r,-1.0,b);CHKERRQ(ierr);
+  ierr = MatResidual(mat,b,x,r);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-/* ---------------------------------------------------------------------------*/
-
-#undef __FUNCT__
-#define __FUNCT__ "PCMGGetCoarseSolve"
 /*@
    PCMGGetCoarseSolve - Gets the solver context to be used on the coarse grid.
 
@@ -50,6 +44,8 @@ PetscErrorCode  PCMGResidual_Default(Mat mat,Vec b,Vec x,Vec r)
    Level: advanced
 
 .keywords: MG, multigrid, get, coarse grid
+
+.seealso: PCMGGetSmootherUp(), PCMGGetSmootherDown(), PCMGGetSmoother()
 @*/
 PetscErrorCode  PCMGGetCoarseSolve(PC pc,KSP *ksp)
 {
@@ -62,8 +58,6 @@ PetscErrorCode  PCMGGetCoarseSolve(PC pc,KSP *ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGSetResidual"
 /*@C
    PCMGSetResidual - Sets the function to be used to calculate the residual
    on the lth level.
@@ -81,7 +75,7 @@ PetscErrorCode  PCMGGetCoarseSolve(PC pc,KSP *ksp)
 
 .keywords:  MG, set, multigrid, residual, level
 
-.seealso: PCMGResidual_Default()
+.seealso: PCMGResidualDefault()
 @*/
 PetscErrorCode  PCMGSetResidual(PC pc,PetscInt l,PetscErrorCode (*residual)(Mat,Vec,Vec,Vec),Mat mat)
 {
@@ -93,16 +87,13 @@ PetscErrorCode  PCMGSetResidual(PC pc,PetscInt l,PetscErrorCode (*residual)(Mat,
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
   if (!mglevels) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"Must set MG levels before calling");
   if (residual) mglevels[l]->residual = residual;
-  if (!mglevels[l]->residual) mglevels[l]->residual = PCMGResidual_Default;
+  if (!mglevels[l]->residual) mglevels[l]->residual = PCMGResidualDefault;
   if (mat) {ierr = PetscObjectReference((PetscObject)mat);CHKERRQ(ierr);}
   ierr = MatDestroy(&mglevels[l]->A);CHKERRQ(ierr);
-
   mglevels[l]->A = mat;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGSetInterpolation"
 /*@
    PCMGSetInterpolation - Sets the function to be used to calculate the
    interpolation from l-1 to the lth level
@@ -144,8 +135,6 @@ PetscErrorCode  PCMGSetInterpolation(PC pc,PetscInt l,Mat mat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGGetInterpolation"
 /*@
    PCMGGetInterpolation - Gets the function to be used to calculate the
    interpolation from l-1 to the lth level
@@ -157,7 +146,7 @@ PetscErrorCode  PCMGSetInterpolation(PC pc,PetscInt l,Mat mat)
 -  l - the level (0 is coarsest) to supply [Do not supply 0]
 
    Output Parameter:
-.  mat - the interpolation matrix
+.  mat - the interpolation matrix, can be NULL
 
    Level: advanced
 
@@ -173,21 +162,19 @@ PetscErrorCode  PCMGGetInterpolation(PC pc,PetscInt l,Mat *mat)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
-  PetscValidPointer(mat,3);
+  if (mat) PetscValidPointer(mat,3);
   if (!mglevels) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"Must set MG levels before calling");
   if (l <= 0 || mg->nlevels <= l) SETERRQ2(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_OUTOFRANGE,"Level %D must be in range {1,...,%D}",l,mg->nlevels-1);
   if (!mglevels[l]->interpolate) {
-    if (!mglevels[l]->restrct) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"Must call PCMGSetInterpolation() or PCMGSetInterpolation()");
+    if (!mglevels[l]->restrct) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"Must call PCMGSetInterpolation() or PCMGSetRestriction()");
     ierr = PCMGSetInterpolation(pc,l,mglevels[l]->restrct);CHKERRQ(ierr);
   }
-  *mat = mglevels[l]->interpolate;
+  if (mat) *mat = mglevels[l]->interpolate;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGSetRestriction"
 /*@
-   PCMGSetRestriction - Sets the function to be used to restrict vector
+   PCMGSetRestriction - Sets the function to be used to restrict dual vectors
    from level l to l-1.
 
    Logically Collective on PC and Mat
@@ -211,7 +198,7 @@ PetscErrorCode  PCMGGetInterpolation(PC pc,PetscInt l,Mat *mat)
 
 .keywords: MG, set, multigrid, restriction, level
 
-.seealso: PCMGSetInterpolation()
+.seealso: PCMGSetInterpolation(), PCMGetSetInjection()
 @*/
 PetscErrorCode  PCMGSetRestriction(PC pc,PetscInt l,Mat mat)
 {
@@ -231,10 +218,8 @@ PetscErrorCode  PCMGSetRestriction(PC pc,PetscInt l,Mat mat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGGetRestriction"
 /*@
-   PCMGGetRestriction - Gets the function to be used to restrict vector
+   PCMGGetRestriction - Gets the function to be used to restrict dual vectors
    from level l to l-1.
 
    Logically Collective on PC and Mat
@@ -250,7 +235,7 @@ PetscErrorCode  PCMGSetRestriction(PC pc,PetscInt l,Mat mat)
 
 .keywords: MG, get, multigrid, restriction, level
 
-.seealso: PCMGGetInterpolation(), PCMGSetRestriction(), PCMGGetRScale()
+.seealso: PCMGGetInterpolation(), PCMGSetRestriction(), PCMGGetRScale(), PCMGGetInjection()
 @*/
 PetscErrorCode  PCMGGetRestriction(PC pc,PetscInt l,Mat *mat)
 {
@@ -260,19 +245,17 @@ PetscErrorCode  PCMGGetRestriction(PC pc,PetscInt l,Mat *mat)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
-  PetscValidPointer(mat,3);
+  if (mat) PetscValidPointer(mat,3);
   if (!mglevels) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"Must set MG levels before calling");
   if (l <= 0 || mg->nlevels <= l) SETERRQ2(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_OUTOFRANGE,"Level %D must be in range {1,...,%D}",l,mg->nlevels-1);
   if (!mglevels[l]->restrct) {
     if (!mglevels[l]->interpolate) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"Must call PCMGSetRestriction() or PCMGSetInterpolation()");
     ierr = PCMGSetRestriction(pc,l,mglevels[l]->interpolate);CHKERRQ(ierr);
   }
-  *mat = mglevels[l]->restrct;
+  if (mat) *mat = mglevels[l]->restrct;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGSetRScale"
 /*@
    PCMGSetRScale - Sets the pointwise scaling for the restriction operator from level l to l-1.
 
@@ -286,11 +269,11 @@ PetscErrorCode  PCMGGetRestriction(PC pc,PetscInt l,Mat *mat)
    Level: advanced
 
    Notes:
-       When evaluating a function on a coarse level one does not want to do F(R * x) one does F(rscale * R * x) where rscale is 1 over the row sums of R.
+       When evaluating a function on a coarse level one does not want to do F(R * x) one does F(rscale * R * x) where rscale is 1 over the row sums of R.  It is preferable to use PCMGSetInjection() to control moving primal vectors.
 
 .keywords: MG, set, multigrid, restriction, level
 
-.seealso: PCMGSetInterpolation(), PCMGSetRestriction(), PCMGGetRScale()
+.seealso: PCMGSetInterpolation(), PCMGSetRestriction(), PCMGGetRScale(), PCMGSetInjection()
 @*/
 PetscErrorCode  PCMGSetRScale(PC pc,PetscInt l,Vec rscale)
 {
@@ -309,8 +292,6 @@ PetscErrorCode  PCMGSetRScale(PC pc,PetscInt l,Vec rscale)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGGetRScale"
 /*@
    PCMGGetRScale - Gets the pointwise scaling for the restriction operator from level l to l-1.
 
@@ -324,11 +305,11 @@ PetscErrorCode  PCMGSetRScale(PC pc,PetscInt l,Vec rscale)
    Level: advanced
 
    Notes:
-       When evaluating a function on a coarse level one does not want to do F(R * x) one does F(rscale * R * x) where rscale is 1 over the row sums of R.
+       When evaluating a function on a coarse level one does not want to do F(R * x) one does F(rscale * R * x) where rscale is 1 over the row sums of R.  It is preferable to use PCMGGetInjection() to control moving primal vectors.
 
 .keywords: MG, set, multigrid, restriction, level
 
-.seealso: PCMGSetInterpolation(), PCMGGetRestriction()
+.seealso: PCMGSetInterpolation(), PCMGGetRestriction(), PCMGGetInjection()
 @*/
 PetscErrorCode PCMGGetRScale(PC pc,PetscInt l,Vec *rscale)
 {
@@ -345,7 +326,7 @@ PetscErrorCode PCMGGetRScale(PC pc,PetscInt l,Vec *rscale)
     Vec      X,Y,coarse,fine;
     PetscInt M,N;
     ierr = PCMGGetRestriction(pc,l,&R);CHKERRQ(ierr);
-    ierr = MatGetVecs(R,&X,&Y);CHKERRQ(ierr);
+    ierr = MatCreateVecs(R,&X,&Y);CHKERRQ(ierr);
     ierr = MatGetSize(R,&M,&N);CHKERRQ(ierr);
     if (M < N) {
       fine = X;
@@ -363,8 +344,74 @@ PetscErrorCode PCMGGetRScale(PC pc,PetscInt l,Vec *rscale)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGGetSmoother"
+/*@
+   PCMGSetInjection - Sets the function to be used to inject primal vectors
+   from level l to l-1.
+
+   Logically Collective on PC and Mat
+
+   Input Parameters:
++  pc - the multigrid context
+.  l - the level (0 is coarsest) to supply [Do not supply 0]
+-  mat - the injection matrix
+
+   Level: advanced
+
+.keywords: MG, set, multigrid, restriction, injection, level
+
+.seealso: PCMGSetRestriction()
+@*/
+PetscErrorCode  PCMGSetInjection(PC pc,PetscInt l,Mat mat)
+{
+  PetscErrorCode ierr;
+  PC_MG          *mg        = (PC_MG*)pc->data;
+  PC_MG_Levels   **mglevels = mg->levels;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,3);
+  if (!mglevels) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"Must set MG levels before calling");
+  if (!l) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_OUTOFRANGE,"Do not set restriction routine for coarsest level");
+  ierr = PetscObjectReference((PetscObject)mat);CHKERRQ(ierr);
+  ierr = MatDestroy(&mglevels[l]->inject);CHKERRQ(ierr);
+
+  mglevels[l]->inject = mat;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   PCMGGetInjection - Gets the function to be used to inject primal vectors
+   from level l to l-1.
+
+   Logically Collective on PC and Mat
+
+   Input Parameters:
++  pc - the multigrid context
+-  l - the level (0 is coarsest) to supply [Do not supply 0]
+
+   Output Parameter:
+.  mat - the restriction matrix (may be NULL if no injection is available).
+
+   Level: advanced
+
+.keywords: MG, get, multigrid, restriction, injection, level
+
+.seealso: PCMGSetInjection(), PCMGetGetRestriction()
+@*/
+PetscErrorCode  PCMGGetInjection(PC pc,PetscInt l,Mat *mat)
+{
+  PC_MG          *mg        = (PC_MG*)pc->data;
+  PC_MG_Levels   **mglevels = mg->levels;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  if (mat) PetscValidPointer(mat,3);
+  if (!mglevels) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"Must set MG levels before calling");
+  if (l <= 0 || mg->nlevels <= l) SETERRQ2(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_OUTOFRANGE,"Level %D must be in range {1,...,%D}",l,mg->nlevels-1);
+  if (mat) *mat = mglevels[l]->inject;
+  PetscFunctionReturn(0);
+}
+
 /*@
    PCMGGetSmoother - Gets the KSP context to be used as smoother for
    both pre- and post-smoothing.  Call both PCMGGetSmootherUp() and
@@ -380,11 +427,16 @@ PetscErrorCode PCMGGetRScale(PC pc,PetscInt l,Vec *rscale)
    Ouput Parameters:
 .  ksp - the smoother
 
+   Notes:
+   Once you have called this routine, you can call KSPSetOperators(ksp,...) on the resulting ksp to provide the operators for the smoother for this level.
+   You can also modify smoother options by calling the various KSPSetXXX() options on this ksp. In addition you can call KSPGetPC(ksp,&pc)
+   and modify PC options for the smoother; for example PCSetType(pc,PCSOR); to use SOR smoothing.
+
    Level: advanced
 
 .keywords: MG, get, multigrid, level, smoother, pre-smoother, post-smoother
 
-.seealso: PCMGGetSmootherUp(), PCMGGetSmootherDown()
+.seealso: PCMGGetSmootherUp(), PCMGGetSmootherDown(), PCMGGetCoarseSolve()
 @*/
 PetscErrorCode  PCMGGetSmoother(PC pc,PetscInt l,KSP *ksp)
 {
@@ -397,8 +449,6 @@ PetscErrorCode  PCMGGetSmoother(PC pc,PetscInt l,KSP *ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGGetSmootherUp"
 /*@
    PCMGGetSmootherUp - Gets the KSP context to be used as smoother after
    coarse grid correction (post-smoother).
@@ -414,7 +464,8 @@ PetscErrorCode  PCMGGetSmoother(PC pc,PetscInt l,KSP *ksp)
 
    Level: advanced
 
-   Notes: calling this will result in a different pre and post smoother so you may need to
+   Notes:
+    calling this will result in a different pre and post smoother so you may need to
          set options on the pre smoother also
 
 .keywords: MG, multigrid, get, smoother, up, post-smoother, level
@@ -453,22 +504,22 @@ PetscErrorCode  PCMGGetSmootherUp(PC pc,PetscInt l,KSP *ksp)
     ierr = PCGetType(ipc,&pctype);CHKERRQ(ierr);
 
     ierr = KSPCreate(comm,&mglevels[l]->smoothu);CHKERRQ(ierr);
+    ierr = KSPSetErrorIfNotConverged(mglevels[l]->smoothu,pc->erroriffailure);CHKERRQ(ierr);
     ierr = PetscObjectIncrementTabLevel((PetscObject)mglevels[l]->smoothu,(PetscObject)pc,mglevels[0]->levels-l);CHKERRQ(ierr);
     ierr = KSPSetOptionsPrefix(mglevels[l]->smoothu,prefix);CHKERRQ(ierr);
     ierr = KSPSetTolerances(mglevels[l]->smoothu,rtol,abstol,dtol,maxits);CHKERRQ(ierr);
     ierr = KSPSetType(mglevels[l]->smoothu,ksptype);CHKERRQ(ierr);
     ierr = KSPSetNormType(mglevels[l]->smoothu,normtype);CHKERRQ(ierr);
-    ierr = KSPSetConvergenceTest(mglevels[l]->smoothu,KSPSkipConverged,NULL,NULL);CHKERRQ(ierr);
+    ierr = KSPSetConvergenceTest(mglevels[l]->smoothu,KSPConvergedSkip,NULL,NULL);CHKERRQ(ierr);
     ierr = KSPGetPC(mglevels[l]->smoothu,&ipc);CHKERRQ(ierr);
     ierr = PCSetType(ipc,pctype);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent(pc,mglevels[l]->smoothu);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)pc,(PetscObject)mglevels[l]->smoothu);CHKERRQ(ierr);
+    ierr = PetscObjectComposedDataSetInt((PetscObject) mglevels[l]->smoothu, PetscMGLevelId, mglevels[l]->level);CHKERRQ(ierr);
   }
   if (ksp) *ksp = mglevels[l]->smoothu;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGGetSmootherDown"
 /*@
    PCMGGetSmootherDown - Gets the KSP context to be used as smoother before
    coarse grid correction (pre-smoother).
@@ -484,7 +535,8 @@ PetscErrorCode  PCMGGetSmootherUp(PC pc,PetscInt l,KSP *ksp)
 
    Level: advanced
 
-   Notes: calling this will result in a different pre and post smoother so you may need to
+   Notes:
+    calling this will result in a different pre and post smoother so you may need to
          set options on the post smoother also
 
 .keywords: MG, multigrid, get, smoother, down, pre-smoother, level
@@ -507,25 +559,23 @@ PetscErrorCode  PCMGGetSmootherDown(PC pc,PetscInt l,KSP *ksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGSetCyclesOnLevel"
 /*@
-   PCMGSetCyclesOnLevel - Sets the number of cycles to run on this level.
+   PCMGSetCycleTypeOnLevel - Sets the type of cycle (aka cycle index) to run on the specified level.
 
    Logically Collective on PC
 
    Input Parameters:
 +  pc - the multigrid context
-.  l  - the level (0 is coarsest) this is to be used for
--  n  - the number of cycles
+.  l  - the level (0 is coarsest)
+-  c  - either PC_MG_CYCLE_V or PC_MG_CYCLE_W
 
    Level: advanced
 
 .keywords: MG, multigrid, set, cycles, V-cycle, W-cycle, level
 
-.seealso: PCMGSetCycles()
+.seealso: PCMGSetCycleType()
 @*/
-PetscErrorCode  PCMGSetCyclesOnLevel(PC pc,PetscInt l,PetscInt c)
+PetscErrorCode  PCMGSetCycleTypeOnLevel(PC pc,PetscInt l,PCMGCycleType c)
 {
   PC_MG        *mg        = (PC_MG*)pc->data;
   PC_MG_Levels **mglevels = mg->levels;
@@ -534,13 +584,11 @@ PetscErrorCode  PCMGSetCyclesOnLevel(PC pc,PetscInt l,PetscInt c)
   PetscValidHeaderSpecific(pc,PC_CLASSID,1);
   if (!mglevels) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_ARG_WRONGSTATE,"Must set MG levels before calling");
   PetscValidLogicalCollectiveInt(pc,l,2);
-  PetscValidLogicalCollectiveInt(pc,c,3);
+  PetscValidLogicalCollectiveEnum(pc,c,3);
   mglevels[l]->cycles = c;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGSetRhs"
 /*@
    PCMGSetRhs - Sets the vector space to be used to store the right-hand side
    on a particular level.
@@ -554,7 +602,8 @@ PetscErrorCode  PCMGSetCyclesOnLevel(PC pc,PetscInt l,PetscInt c)
 
    Level: advanced
 
-   Notes: If this is not provided PETSc will automatically generate one.
+   Notes:
+    If this is not provided PETSc will automatically generate one.
 
           You do not need to keep a reference to this vector if you do
           not need it PCDestroy() will properly free it.
@@ -580,8 +629,6 @@ PetscErrorCode  PCMGSetRhs(PC pc,PetscInt l,Vec c)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGSetX"
 /*@
    PCMGSetX - Sets the vector space to be used to store the solution on a
    particular level.
@@ -595,7 +642,8 @@ PetscErrorCode  PCMGSetRhs(PC pc,PetscInt l,Vec c)
 
    Level: advanced
 
-   Notes: If this is not provided PETSc will automatically generate one.
+   Notes:
+    If this is not provided PETSc will automatically generate one.
 
           You do not need to keep a reference to this vector if you do
           not need it PCDestroy() will properly free it.
@@ -621,8 +669,6 @@ PetscErrorCode  PCMGSetX(PC pc,PetscInt l,Vec c)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCMGSetR"
 /*@
    PCMGSetR - Sets the vector space to be used to store the residual on a
    particular level.
@@ -636,7 +682,8 @@ PetscErrorCode  PCMGSetX(PC pc,PetscInt l,Vec c)
 
    Level: advanced
 
-   Notes: If this is not provided PETSc will automatically generate one.
+   Notes:
+    If this is not provided PETSc will automatically generate one.
 
           You do not need to keep a reference to this vector if you do
           not need it PCDestroy() will properly free it.
