@@ -63,8 +63,6 @@ typedef struct {
   PetscReal         *coords; /* ML has a grid object for each level: the finest grid will point into coords */
 } PC_ML;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscML_getrow"
 static int PetscML_getrow(ML_Operator *ML_data, int N_requested_rows, int requested_rows[],int allocated_space, int columns[], double values[], int row_lengths[])
 {
   PetscErrorCode ierr;
@@ -90,8 +88,6 @@ static int PetscML_getrow(ML_Operator *ML_data, int N_requested_rows, int reques
   return(1);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscML_comm"
 static PetscErrorCode PetscML_comm(double p[],void *ML_data)
 {
   PetscErrorCode    ierr;
@@ -116,8 +112,6 @@ static PetscErrorCode PetscML_comm(double p[],void *ML_data)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscML_matvec"
 static int PetscML_matvec(ML_Operator *ML_data,int in_length,double p[],int out_length,double ap[])
 {
   PetscErrorCode ierr;
@@ -143,8 +137,6 @@ static int PetscML_matvec(ML_Operator *ML_data,int in_length,double p[],int out_
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_ML"
 static PetscErrorCode MatMult_ML(Mat A,Vec x,Vec y)
 {
   PetscErrorCode    ierr;
@@ -165,8 +157,6 @@ static PetscErrorCode MatMult_ML(Mat A,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMultAdd_ML"
 /* Computes y = w + A * x
    It is possible that w == y, but not x == y
 */
@@ -206,8 +196,6 @@ static PetscErrorCode MatMultAdd_ML(Mat A,Vec x,Vec w,Vec y)
 }
 
 /* newtype is ignored since only handles one case */
-#undef __FUNCT__
-#define __FUNCT__ "MatConvert_MPIAIJ_ML"
 static PetscErrorCode MatConvert_MPIAIJ_ML(Mat A,MatType newtype,MatReuse scall,Mat *Aloc)
 {
   PetscErrorCode ierr;
@@ -271,9 +259,6 @@ static PetscErrorCode MatConvert_MPIAIJ_ML(Mat A,MatType newtype,MatReuse scall,
   PetscFunctionReturn(0);
 }
 
-extern PetscErrorCode MatDestroy_Shell(Mat);
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_ML"
 static PetscErrorCode MatDestroy_ML(Mat A)
 {
   PetscErrorCode ierr;
@@ -284,13 +269,9 @@ static PetscErrorCode MatDestroy_ML(Mat A)
   ierr = VecDestroy(&shell->y);CHKERRQ(ierr);
   if (shell->work) {ierr = VecDestroy(&shell->work);CHKERRQ(ierr);}
   ierr = PetscFree(shell);CHKERRQ(ierr);
-  ierr = MatDestroy_Shell(A);CHKERRQ(ierr);
-  ierr = PetscObjectChangeTypeName((PetscObject)A,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatWrapML_SeqAIJ"
 static PetscErrorCode MatWrapML_SeqAIJ(ML_Operator *mlmat,MatReuse reuse,Mat *newmat)
 {
   struct ML_CSR_MSRdata *matdata = (struct ML_CSR_MSRdata*)mlmat->data;
@@ -351,8 +332,6 @@ static PetscErrorCode MatWrapML_SeqAIJ(ML_Operator *mlmat,MatReuse reuse,Mat *ne
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatWrapML_SHELL"
 static PetscErrorCode MatWrapML_SHELL(ML_Operator *mlmat,MatReuse reuse,Mat *newmat)
 {
   PetscErrorCode ierr;
@@ -376,6 +355,7 @@ static PetscErrorCode MatWrapML_SHELL(ML_Operator *mlmat,MatReuse reuse,Mat *new
   ierr = MatCreateShell(MLcomm->USR_comm,m,n,PETSC_DETERMINE,PETSC_DETERMINE,shellctx,newmat);CHKERRQ(ierr);
   ierr = MatShellSetOperation(*newmat,MATOP_MULT,(void(*)(void))MatMult_ML);CHKERRQ(ierr);
   ierr = MatShellSetOperation(*newmat,MATOP_MULT_ADD,(void(*)(void))MatMultAdd_ML);CHKERRQ(ierr);
+  ierr = MatShellSetOperation(*newmat,MATOP_DESTROY,(void(*)(void))MatDestroy_ML);CHKERRQ(ierr);
 
   shellctx->A         = *newmat;
   shellctx->mlmat     = mlmat;
@@ -384,13 +364,9 @@ static PetscErrorCode MatWrapML_SHELL(ML_Operator *mlmat,MatReuse reuse,Mat *new
   ierr = VecCreate(MLcomm->USR_comm,&shellctx->y);CHKERRQ(ierr);
   ierr = VecSetSizes(shellctx->y,m,PETSC_DECIDE);CHKERRQ(ierr);
   ierr = VecSetType(shellctx->y,VECSTANDARD);CHKERRQ(ierr);
-
-  (*newmat)->ops->destroy = MatDestroy_ML;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatWrapML_MPIAIJ"
 static PetscErrorCode MatWrapML_MPIAIJ(ML_Operator *mlmat,MatReuse reuse,Mat *newmat)
 {
   PetscInt       *aj;
@@ -460,8 +436,6 @@ static PetscErrorCode MatWrapML_MPIAIJ(ML_Operator *mlmat,MatReuse reuse,Mat *ne
    Input Parameter:
    .  pc - the preconditioner context
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCSetCoordinates_ML"
 static PetscErrorCode PCSetCoordinates_ML(PC pc, PetscInt ndm, PetscInt a_nloc, PetscReal *coords)
 {
   PC_MG          *mg    = (PC_MG*)pc->data;
@@ -503,8 +477,6 @@ static PetscErrorCode PCSetCoordinates_ML(PC pc, PetscInt ndm, PetscInt a_nloc,
 
 /* -----------------------------------------------------------------------------*/
 extern PetscErrorCode PCReset_MG(PC);
-#undef __FUNCT__
-#define __FUNCT__ "PCReset_ML"
 PetscErrorCode PCReset_ML(PC pc)
 {
   PetscErrorCode ierr;
@@ -569,11 +541,9 @@ PetscErrorCode PCReset_ML(PC pc)
    The interface routine PCSetUp() is not usually called directly by
    the user, but instead is called by PCApply() if necessary.
 */
-extern PetscErrorCode PCSetFromOptions_MG(PetscOptions *PetscOptionsObject,PC);
+extern PetscErrorCode PCSetFromOptions_MG(PetscOptionItems *PetscOptionsObject,PC);
 extern PetscErrorCode PCReset_MG(PC);
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUp_ML"
 PetscErrorCode PCSetUp_ML(PC pc)
 {
   PetscErrorCode   ierr;
@@ -1019,8 +989,6 @@ PetscErrorCode PCSetUp_ML(PC pc)
 
    Application Interface Routine: PCDestroy()
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCDestroy_ML"
 PetscErrorCode PCDestroy_ML(PC pc)
 {
   PetscErrorCode ierr;
@@ -1035,16 +1003,13 @@ PetscErrorCode PCDestroy_ML(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetFromOptions_ML"
-PetscErrorCode PCSetFromOptions_ML(PetscOptions *PetscOptionsObject,PC pc)
+PetscErrorCode PCSetFromOptions_ML(PetscOptionItems *PetscOptionsObject,PC pc)
 {
   PetscErrorCode ierr;
   PetscInt       indx,PrintLevel,partindx;
   const char     *scheme[] = {"Uncoupled","Coupled","MIS","METIS"};
   const char     *part[]   = {"Zoltan","ParMETIS"};
 #if defined(HAVE_ML_ZOLTAN)
-  PetscInt   zidx;
   const char *zscheme[] = {"RCB","hypergraph","fast_hypergraph"};
 #endif
   PC_MG       *mg    = (PC_MG*)pc->data;
@@ -1159,8 +1124,7 @@ PetscErrorCode PCSetFromOptions_ML(PetscOptions *PetscOptionsObject,PC pc)
    Options Database Key:
    Multigrid options(inherited):
 +  -pc_mg_cycles <1>: 1 for V cycle, 2 for W-cycle (MGSetCycles)
-.  -pc_mg_smoothup <1>: Number of post-smoothing steps (MGSetNumberSmoothUp)
-.  -pc_mg_smoothdown <1>: Number of pre-smoothing steps (MGSetNumberSmoothDown)
+.  -pc_mg_distinct_smoothup: Should one configure the up and down smoothers separately (PCMGSetDistinctSmoothUp)
 -  -pc_mg_type <multiplicative>: (one of) additive multiplicative full kascade
    ML options:
 +  -pc_ml_PrintLevel <0>: Print level (ML_Set_PrintLevel)
@@ -1184,14 +1148,12 @@ PetscErrorCode PCSetFromOptions_ML(PetscOptions *PetscOptionsObject,PC pc)
   Concepts: multigrid
 
 .seealso:  PCCreate(), PCSetType(), PCType (for list of available types), PC, PCMGType,
-           PCMGSetLevels(), PCMGGetLevels(), PCMGSetType(), MPSetCycles(), PCMGSetNumberSmoothDown(),
-           PCMGSetNumberSmoothUp(), PCMGGetCoarseSolve(), PCMGSetResidual(), PCMGSetInterpolation(),
+           PCMGSetLevels(), PCMGGetLevels(), PCMGSetType(), MPSetCycles(), PCMGSetDistinctSmoothUp(),
+           PCMGGetCoarseSolve(), PCMGSetResidual(), PCMGSetInterpolation(),
            PCMGSetRestriction(), PCMGGetSmoother(), PCMGGetSmootherUp(), PCMGGetSmootherDown(),
            PCMGSetCycleTypeOnLevel(), PCMGSetRhs(), PCMGSetX(), PCMGSetR()
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCCreate_ML"
 PETSC_EXTERN PetscErrorCode PCCreate_ML(PC pc)
 {
   PetscErrorCode ierr;
@@ -1203,9 +1165,9 @@ PETSC_EXTERN PetscErrorCode PCCreate_ML(PC pc)
   ierr = PCSetType(pc,PCMG);CHKERRQ(ierr); /* calls PCCreate_MG() and MGCreate_Private() */
   ierr = PetscObjectChangeTypeName((PetscObject)pc,PCML);CHKERRQ(ierr);
   /* Since PCMG tries to use DM assocated with PC must delete it */
-  ierr         = DMDestroy(&pc->dm);CHKERRQ(ierr);
-  mg           = (PC_MG*)pc->data;
-  mg->galerkin = 2;             /* Use Galerkin, but it is computed externally */
+  ierr = DMDestroy(&pc->dm);CHKERRQ(ierr);
+  ierr = PCMGSetGalerkin(pc,PC_MG_GALERKIN_EXTERNAL);CHKERRQ(ierr);
+  mg   = (PC_MG*)pc->data;
 
   /* create a supporting struct and attach it to pc */
   ierr         = PetscNewLog(pc,&pc_ml);CHKERRQ(ierr);
