@@ -2,10 +2,8 @@
 /*
     Identity preconditioner, simply copies vector x to y.
 */
-#include <petsc-private/pcimpl.h>          /*I "petscpc.h" I*/
+#include <petsc/private/pcimpl.h>          /*I "petscpc.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_None"
 PetscErrorCode PCApply_None(PC pc,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -23,13 +21,12 @@ PetscErrorCode PCApply_None(PC pc,Vec x,Vec y)
 
   Concepts: preconditioners
 
-  Notes: This is implemented by a VecCopy()
+  Notes:
+    This is implemented by a VecCopy()
 
 .seealso:  PCCreate(), PCSetType(), PCType (for list of available types), PC
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCCreate_None"
 PETSC_EXTERN PetscErrorCode PCCreate_None(PC pc)
 {
   PetscFunctionBegin;
