@@ -19,8 +19,6 @@ typedef struct {
   MatFactorShiftType shifttype;
 } PC_Redundant;
 
-#undef __FUNCT__
-#define __FUNCT__ "PCFactorSetShiftType_Redundant"
 PetscErrorCode  PCFactorSetShiftType_Redundant(PC pc,MatFactorShiftType shifttype)
 {
   PC_Redundant   *red = (PC_Redundant*)pc->data;
@@ -38,8 +36,6 @@ PetscErrorCode  PCFactorSetShiftType_Redundant(PC pc,MatFactorShiftType shifttyp
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCView_Redundant"
 static PetscErrorCode PCView_Redundant(PC pc,PetscViewer viewer)
 {
   PC_Redundant   *red = (PC_Redundant*)pc->data;
@@ -52,9 +48,9 @@ static PetscErrorCode PCView_Redundant(PC pc,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERSTRING,&isstring);CHKERRQ(ierr);
   if (iascii) {
     if (!red->psubcomm) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  Redundant preconditioner: Not yet setup\n");CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  Not yet setup\n");CHKERRQ(ierr);
     } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"  Redundant preconditioner: First (color=0) of %D PCs follows\n",red->nsubcomm);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  First (color=0) of %D PCs follows\n",red->nsubcomm);CHKERRQ(ierr);
       ierr = PetscViewerGetSubViewer(viewer,((PetscObject)red->pc)->comm,&subviewer);CHKERRQ(ierr);
       if (!red->psubcomm->color) { /* only view first redundant pc */
         ierr = PetscViewerASCIIPushTab(subviewer);CHKERRQ(ierr);
@@ -69,8 +65,7 @@ static PetscErrorCode PCView_Redundant(PC pc,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUp_Redundant"
+#include <../src/mat/impls/aij/mpi/mpiaij.h>
 static PetscErrorCode PCSetUp_Redundant(PC pc)
 {
   PC_Redundant   *red = (PC_Redundant*)pc->data;
@@ -82,7 +77,7 @@ static PetscErrorCode PCSetUp_Redundant(PC pc)
 
   PetscFunctionBegin;
   ierr = PetscObjectGetComm((PetscObject)pc,&comm);CHKERRQ(ierr);
-  
+
   /* if pmatrix set by user is sequential then we do not need to gather the parallel matrix */
   ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
   if (size == 1) red->useparallelmat = PETSC_FALSE;
@@ -92,7 +87,7 @@ static PetscErrorCode PCSetUp_Redundant(PC pc)
     if (!red->psubcomm) { /* create red->psubcomm, new ksp and pc over subcomm */
       KSP ksp;
       ierr = PCRedundantGetKSP(pc,&ksp);CHKERRQ(ierr);
-    } 
+    }
     subcomm = PetscSubcommChild(red->psubcomm);
 
     if (red->useparallelmat) {
@@ -107,12 +102,12 @@ static PetscErrorCode PCSetUp_Redundant(PC pc)
           ierr = KSPSetType(red->ksp,KSPGMRES);CHKERRQ(ierr);
           ierr = PCSetType(red->pc,PCBJACOBI);CHKERRQ(ierr);
         } else {
-          ierr = PCFactorSetMatSolverPackage(red->pc,NULL);CHKERRQ(ierr);
+          ierr = PCFactorSetMatSolverType(red->pc,NULL);CHKERRQ(ierr);
         }
       }
 
       ierr = KSPSetOperators(red->ksp,red->pmats,red->pmats);CHKERRQ(ierr);
-       
+
       /* get working vectors xsub and ysub */
       ierr = MatCreateVecs(red->pmats,&red->xsub,&red->ysub);CHKERRQ(ierr);
 
@@ -185,8 +180,6 @@ static PetscErrorCode PCSetUp_Redundant(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_Redundant"
 static PetscErrorCode PCApply_Redundant(PC pc,Vec x,Vec y)
 {
   PC_Redundant   *red = (PC_Redundant*)pc->data;
@@ -224,8 +217,6 @@ static PetscErrorCode PCApply_Redundant(PC pc,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyTranspose_Redundant"
 static PetscErrorCode PCApplyTranspose_Redundant(PC pc,Vec x,Vec y)
 {
   PC_Redundant   *red = (PC_Redundant*)pc->data;
@@ -263,8 +254,6 @@ static PetscErrorCode PCApplyTranspose_Redundant(PC pc,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCReset_Redundant"
 static PetscErrorCode PCReset_Redundant(PC pc)
 {
   PC_Redundant   *red = (PC_Redundant*)pc->data;
@@ -284,8 +273,6 @@ static PetscErrorCode PCReset_Redundant(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCDestroy_Redundant"
 static PetscErrorCode PCDestroy_Redundant(PC pc)
 {
   PC_Redundant   *red = (PC_Redundant*)pc->data;
@@ -299,8 +286,6 @@ static PetscErrorCode PCDestroy_Redundant(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetFromOptions_Redundant"
 static PetscErrorCode PCSetFromOptions_Redundant(PetscOptionItems *PetscOptionsObject,PC pc)
 {
   PetscErrorCode ierr;
@@ -313,8 +298,6 @@ static PetscErrorCode PCSetFromOptions_Redundant(PetscOptionItems *PetscOptionsO
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCRedundantSetNumber_Redundant"
 static PetscErrorCode PCRedundantSetNumber_Redundant(PC pc,PetscInt nreds)
 {
   PC_Redundant *red = (PC_Redundant*)pc->data;
@@ -324,8 +307,6 @@ static PetscErrorCode PCRedundantSetNumber_Redundant(PC pc,PetscInt nreds)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCRedundantSetNumber"
 /*@
    PCRedundantSetNumber - Sets the number of redundant preconditioner contexts.
 
@@ -351,8 +332,6 @@ PetscErrorCode PCRedundantSetNumber(PC pc,PetscInt nredundant)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCRedundantSetScatter_Redundant"
 static PetscErrorCode PCRedundantSetScatter_Redundant(PC pc,VecScatter in,VecScatter out)
 {
   PC_Redundant   *red = (PC_Redundant*)pc->data;
@@ -370,8 +349,6 @@ static PetscErrorCode PCRedundantSetScatter_Redundant(PC pc,VecScatter in,VecSca
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCRedundantSetScatter"
 /*@
    PCRedundantSetScatter - Sets the scatter used to copy values into the
      redundant local solve and the scatter to move them back into the global
@@ -400,8 +377,6 @@ PetscErrorCode PCRedundantSetScatter(PC pc,VecScatter in,VecScatter out)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCRedundantGetKSP_Redundant"
 static PetscErrorCode PCRedundantGetKSP_Redundant(PC pc,KSP *innerksp)
 {
   PetscErrorCode ierr;
@@ -443,8 +418,6 @@ static PetscErrorCode PCRedundantGetKSP_Redundant(PC pc,KSP *innerksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCRedundantGetKSP"
 /*@
    PCRedundantGetKSP - Gets the less parallel KSP created by the redundant PC.
 
@@ -471,8 +444,6 @@ PetscErrorCode PCRedundantGetKSP(PC pc,KSP *innerksp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCRedundantGetOperators_Redundant"
 static PetscErrorCode PCRedundantGetOperators_Redundant(PC pc,Mat *mat,Mat *pmat)
 {
   PC_Redundant *red = (PC_Redundant*)pc->data;
@@ -483,8 +454,6 @@ static PetscErrorCode PCRedundantGetOperators_Redundant(PC pc,Mat *mat,Mat *pmat
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCRedundantGetOperators"
 /*@
    PCRedundantGetOperators - gets the sequential matrix and preconditioner matrix
 
@@ -525,18 +494,18 @@ PetscErrorCode PCRedundantGetOperators(PC pc,Mat *mat,Mat *pmat)
 
    Level: intermediate
 
-   Notes: The default KSP is preonly and the default PC is LU.
+   Notes:
+    The default KSP is preonly and the default PC is LU.
 
    PCFactorSetShiftType() applied to this PC will convey they shift type into the inner PC if it is factorization based.
 
-   Developer Notes: Note that PCSetInitialGuessNonzero()  is not used by this class but likely should be.
+   Developer Notes:
+    Note that PCSetInitialGuessNonzero()  is not used by this class but likely should be.
 
 .seealso:  PCCreate(), PCSetType(), PCType (for list of available types), PCRedundantSetScatter(),
            PCRedundantGetKSP(), PCRedundantGetOperators(), PCRedundantSetNumber()
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCCreate_Redundant"
 PETSC_EXTERN PetscErrorCode PCCreate_Redundant(PC pc)
 {
   PetscErrorCode ierr;
