@@ -238,20 +238,20 @@ PETSC_EXTERN void PETSC_STDCALL pcshellsetview_(PC *pc,void (PETSC_STDCALL *view
   *ierr = PCShellSetView(*pc,ourshellview);
 }
 
-PETSC_EXTERN void PETSC_STDCALL pcshellsetname_(PC *pc,CHAR name PETSC_MIXED_LEN(len), PetscErrorCode *ierr PETSC_END_LEN(len))
+PETSC_EXTERN void PETSC_STDCALL pcshellsetname_(PC *pc,char* name PETSC_MIXED_LEN(len), PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   char *c;
   FIXCHAR(name,len,c);
-  *ierr = PCShellSetName(*pc,c);
+  *ierr = PCShellSetName(*pc,c);if (*ierr) return;
   FREECHAR(name,c);
 }
 
-PETSC_EXTERN void PETSC_STDCALL pcshellgetname_(PC *pc,CHAR name PETSC_MIXED_LEN(len), PetscErrorCode *ierr PETSC_END_LEN(len))
+PETSC_EXTERN void PETSC_STDCALL pcshellgetname_(PC *pc,char* name PETSC_MIXED_LEN(len), PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   const char *c;
 
   *ierr = PCShellGetName(*pc,&c);if (*ierr) return;
-  *ierr = PetscStrncpy(name,c,len);
+  *ierr = PetscStrncpy(name,c,len);if (*ierr) return;
   FIXRETURNCHAR(PETSC_TRUE,name,len);
 }
 
