@@ -5,7 +5,6 @@
 */
 
 #include <petsc/private/pcimpl.h>        /*I "petscpc.h" I*/
-#include <petsc/private/vecimpl.h>
 
 typedef struct {
   void *ctx;                     /* user provided contexts for preconditioner */
@@ -13,6 +12,8 @@ typedef struct {
   PetscErrorCode (*destroy)(PC);
   PetscErrorCode (*setup)(PC);
   PetscErrorCode (*apply)(PC,Vec,Vec);
+  PetscErrorCode (*applysymmetricleft)(PC,Vec,Vec);
+  PetscErrorCode (*applysymmetricright)(PC,Vec,Vec);
   PetscErrorCode (*applyBA)(PC,PCSide,Vec,Vec,Vec);
   PetscErrorCode (*presolve)(PC,KSP,Vec,Vec);
   PetscErrorCode (*postsolve)(PC,KSP,Vec,Vec);
@@ -23,8 +24,6 @@ typedef struct {
   char *name;
 } PC_Shell;
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellGetContext"
 /*@C
     PCShellGetContext - Returns the user-provided context associated with a shell PC
 
@@ -41,6 +40,10 @@ typedef struct {
     Notes:
     This routine is intended for use within various shell routines
 
+   Fortran Notes:
+    To use this from Fortran you must write a Fortran interface definition for this
+    function that tells Fortran the Fortran derived data type that you are passing in as the ctx argument.
+
 .keywords: PC, shell, get, context
 
 .seealso: PCShellSetContext()
@@ -59,8 +62,6 @@ PetscErrorCode  PCShellGetContext(PC pc,void **ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetContext"
 /*@
     PCShellSetContext - sets the context for a shell PC
 
@@ -72,8 +73,10 @@ PetscErrorCode  PCShellGetContext(PC pc,void **ctx)
 
    Level: advanced
 
-   Fortran Notes: The context can only be an integer or a PetscObject
-      unfortunately it cannot be a Fortran array or derived type.
+   Fortran Notes:
+    To use this from Fortran you must write a Fortran interface definition for this
+    function that tells Fortran the Fortran derived data type that you are passing in as the ctx argument.
+
 
 
 .seealso: PCShellGetContext(), PCSHELL
@@ -91,8 +94,6 @@ PetscErrorCode  PCShellSetContext(PC pc,void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUp_Shell"
 static PetscErrorCode PCSetUp_Shell(PC pc)
 {
   PC_Shell       *shell = (PC_Shell*)pc->data;
@@ -104,34 +105,71 @@ static PetscErrorCode PCSetUp_Shell(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_Shell"
 static PetscErrorCode PCApply_Shell(PC pc,Vec x,Vec y)
 {
-  PC_Shell       *shell = (PC_Shell*)pc->data;
-  PetscErrorCode ierr;
+  PC_Shell         *shell = (PC_Shell*)pc->data;
+  PetscErrorCode   ierr;
+  PetscObjectState instate,outstate;
 
   PetscFunctionBegin;
   if (!shell->apply) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_USER,"No apply() routine provided to Shell PC");
+  ierr = PetscObjectStateGet((PetscObject)y, &instate);CHKERRQ(ierr);
   PetscStackCall("PCSHELL user function apply()",ierr = (*shell->apply)(pc,x,y);CHKERRQ(ierr));
+  ierr = PetscObjectStateGet((PetscObject)y, &outstate);CHKERRQ(ierr);
+  if (instate == outstate) {
+    /* increase the state of the output vector since the user did not update its state themselve as should have been done */
+    ierr = PetscObjectStateIncrease((PetscObject)y);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyBA_Shell"
-static PetscErrorCode PCApplyBA_Shell(PC pc,PCSide side,Vec x,Vec y,Vec w)
+static PetscErrorCode PCApplySymmetricLeft_Shell(PC pc,Vec x,Vec y)
 {
   PC_Shell       *shell = (PC_Shell*)pc->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  if (!shell->applysymmetricleft) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_USER,"No apply() routine provided to Shell PC");
+  PetscStackCall("PCSHELL user function apply()",ierr = (*shell->applysymmetricleft)(pc,x,y);CHKERRQ(ierr));
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode PCApplySymmetricRight_Shell(PC pc,Vec x,Vec y)
+{
+  PC_Shell       *shell = (PC_Shell*)pc->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (!shell->applysymmetricright) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_USER,"No apply() routine provided to Shell PC");
+  PetscStackCall("PCSHELL user function apply()",ierr = (*shell->applysymmetricright)(pc,x,y);CHKERRQ(ierr));
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode PCApplyBA_Shell(PC pc,PCSide side,Vec x,Vec y,Vec w)
+{
+  PC_Shell         *shell = (PC_Shell*)pc->data;
+  PetscErrorCode   ierr;
+  PetscObjectState instate,outstate;
+
+  PetscFunctionBegin;
   if (!shell->applyBA) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_USER,"No applyBA() routine provided to Shell PC");
+  ierr = PetscObjectStateGet((PetscObject)w, &instate);CHKERRQ(ierr);
   PetscStackCall("PCSHELL user function applyBA()",ierr = (*shell->applyBA)(pc,side,x,y,w);CHKERRQ(ierr));
+  ierr = PetscObjectStateGet((PetscObject)w, &outstate);CHKERRQ(ierr);
+  if (instate == outstate) {
+    /* increase the state of the output vector since the user did not update its state themselve as should have been done */
+    ierr = PetscObjectStateIncrease((PetscObject)w);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode PCPreSolveChangeRHS_Shell(PC pc,PetscBool* change)
+{
+  PetscFunctionBegin;
+  *change = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPreSolve_Shell"
 static PetscErrorCode PCPreSolve_Shell(PC pc,KSP ksp,Vec b,Vec x)
 {
   PC_Shell       *shell = (PC_Shell*)pc->data;
@@ -143,8 +181,6 @@ static PetscErrorCode PCPreSolve_Shell(PC pc,KSP ksp,Vec b,Vec x)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCPostSolve_Shell"
 static PetscErrorCode PCPostSolve_Shell(PC pc,KSP ksp,Vec b,Vec x)
 {
   PC_Shell       *shell = (PC_Shell*)pc->data;
@@ -156,34 +192,42 @@ static PetscErrorCode PCPostSolve_Shell(PC pc,KSP ksp,Vec b,Vec x)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyTranspose_Shell"
 static PetscErrorCode PCApplyTranspose_Shell(PC pc,Vec x,Vec y)
 {
-  PC_Shell       *shell = (PC_Shell*)pc->data;
-  PetscErrorCode ierr;
+  PC_Shell         *shell = (PC_Shell*)pc->data;
+  PetscErrorCode   ierr;
+  PetscObjectState instate,outstate;
 
   PetscFunctionBegin;
   if (!shell->applytranspose) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_USER,"No applytranspose() routine provided to Shell PC");
+  ierr = PetscObjectStateGet((PetscObject)y, &instate);CHKERRQ(ierr);
   PetscStackCall("PCSHELL user function applytranspose()",ierr = (*shell->applytranspose)(pc,x,y);CHKERRQ(ierr));
+  ierr = PetscObjectStateGet((PetscObject)y, &outstate);CHKERRQ(ierr);
+  if (instate == outstate) {
+    /* increase the state of the output vector since the user did not update its state themself as should have been done */
+    ierr = PetscObjectStateIncrease((PetscObject)y);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyRichardson_Shell"
 static PetscErrorCode PCApplyRichardson_Shell(PC pc,Vec x,Vec y,Vec w,PetscReal rtol,PetscReal abstol, PetscReal dtol,PetscInt it,PetscBool guesszero,PetscInt *outits,PCRichardsonConvergedReason *reason)
 {
-  PetscErrorCode ierr;
-  PC_Shell       *shell = (PC_Shell*)pc->data;
+  PetscErrorCode   ierr;
+  PC_Shell         *shell = (PC_Shell*)pc->data;
+  PetscObjectState instate,outstate;
 
   PetscFunctionBegin;
   if (!shell->applyrich) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_USER,"No applyrichardson() routine provided to Shell PC");
+  ierr = PetscObjectStateGet((PetscObject)y, &instate);CHKERRQ(ierr);
   PetscStackCall("PCSHELL user function applyrichardson()",ierr = (*shell->applyrich)(pc,x,y,w,rtol,abstol,dtol,it,guesszero,outits,reason);CHKERRQ(ierr));
+  ierr = PetscObjectStateGet((PetscObject)y, &outstate);CHKERRQ(ierr);
+  if (instate == outstate) {
+    /* increase the state of the output vector since the user did not update its state themself as should have been done */
+    ierr = PetscObjectStateIncrease((PetscObject)y);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCDestroy_Shell"
 static PetscErrorCode PCDestroy_Shell(PC pc)
 {
   PC_Shell       *shell = (PC_Shell*)pc->data;
@@ -192,12 +236,24 @@ static PetscErrorCode PCDestroy_Shell(PC pc)
   PetscFunctionBegin;
   ierr = PetscFree(shell->name);CHKERRQ(ierr);
   if (shell->destroy) PetscStackCall("PCSHELL user function destroy()",ierr = (*shell->destroy)(pc);CHKERRQ(ierr));
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetDestroy_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetSetUp_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetApply_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetApplySymmetricLeft_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetApplySymmetricRight_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetApplyBA_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetPreSolve_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetPostSolve_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetView_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetApplyTranspose_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetName_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellGetName_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetApplyRichardson_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCPreSolveChangeRHS_C",NULL);CHKERRQ(ierr);
   ierr = PetscFree(pc->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCView_Shell"
 static PetscErrorCode PCView_Shell(PC pc,PetscViewer viewer)
 {
   PC_Shell       *shell = (PC_Shell*)pc->data;
@@ -208,9 +264,9 @@ static PetscErrorCode PCView_Shell(PC pc,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
     if (shell->name) {
-      ierr = PetscViewerASCIIPrintf(viewer,"  Shell: %s\n",shell->name);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  %s\n",shell->name);CHKERRQ(ierr);
     } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"  Shell: no name\n");CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  no name\n");CHKERRQ(ierr);
     }
   }
   if (shell->view) {
@@ -222,8 +278,6 @@ static PetscErrorCode PCView_Shell(PC pc,PetscViewer viewer)
 }
 
 /* ------------------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetDestroy_Shell"
 static PetscErrorCode  PCShellSetDestroy_Shell(PC pc, PetscErrorCode (*destroy)(PC))
 {
   PC_Shell *shell= (PC_Shell*)pc->data;
@@ -233,8 +287,6 @@ static PetscErrorCode  PCShellSetDestroy_Shell(PC pc, PetscErrorCode (*destroy)(
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetSetUp_Shell"
 static PetscErrorCode  PCShellSetSetUp_Shell(PC pc, PetscErrorCode (*setup)(PC))
 {
   PC_Shell *shell = (PC_Shell*)pc->data;;
@@ -246,8 +298,6 @@ static PetscErrorCode  PCShellSetSetUp_Shell(PC pc, PetscErrorCode (*setup)(PC))
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetApply_Shell"
 static PetscErrorCode  PCShellSetApply_Shell(PC pc,PetscErrorCode (*apply)(PC,Vec,Vec))
 {
   PC_Shell *shell = (PC_Shell*)pc->data;
@@ -257,8 +307,24 @@ static PetscErrorCode  PCShellSetApply_Shell(PC pc,PetscErrorCode (*apply)(PC,Ve
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetApplyBA_Shell"
+static PetscErrorCode  PCShellSetApplySymmetricLeft_Shell(PC pc,PetscErrorCode (*apply)(PC,Vec,Vec))
+{
+  PC_Shell *shell = (PC_Shell*)pc->data;
+
+  PetscFunctionBegin;
+  shell->applysymmetricleft = apply;
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode  PCShellSetApplySymmetricRight_Shell(PC pc,PetscErrorCode (*apply)(PC,Vec,Vec))
+{
+  PC_Shell *shell = (PC_Shell*)pc->data;
+
+  PetscFunctionBegin;
+  shell->applysymmetricright = apply;
+  PetscFunctionReturn(0);
+}
+
 static PetscErrorCode  PCShellSetApplyBA_Shell(PC pc,PetscErrorCode (*applyBA)(PC,PCSide,Vec,Vec,Vec))
 {
   PC_Shell *shell = (PC_Shell*)pc->data;
@@ -270,21 +336,23 @@ static PetscErrorCode  PCShellSetApplyBA_Shell(PC pc,PetscErrorCode (*applyBA)(P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetPreSolve_Shell"
 static PetscErrorCode  PCShellSetPreSolve_Shell(PC pc,PetscErrorCode (*presolve)(PC,KSP,Vec,Vec))
 {
-  PC_Shell *shell = (PC_Shell*)pc->data;
+  PC_Shell       *shell = (PC_Shell*)pc->data;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   shell->presolve = presolve;
-  if (presolve) pc->ops->presolve = PCPreSolve_Shell;
-  else          pc->ops->presolve = 0;
+  if (presolve) {
+    pc->ops->presolve = PCPreSolve_Shell;
+    ierr = PetscObjectComposeFunction((PetscObject)pc,"PCPreSolveChangeRHS_C",PCPreSolveChangeRHS_Shell);CHKERRQ(ierr);
+  } else {
+    pc->ops->presolve = 0;
+    ierr = PetscObjectComposeFunction((PetscObject)pc,"PCPreSolveChangeRHS_C",NULL);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetPostSolve_Shell"
 static PetscErrorCode  PCShellSetPostSolve_Shell(PC pc,PetscErrorCode (*postsolve)(PC,KSP,Vec,Vec))
 {
   PC_Shell *shell = (PC_Shell*)pc->data;
@@ -296,8 +364,6 @@ static PetscErrorCode  PCShellSetPostSolve_Shell(PC pc,PetscErrorCode (*postsolv
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetView_Shell"
 static PetscErrorCode  PCShellSetView_Shell(PC pc,PetscErrorCode (*view)(PC,PetscViewer))
 {
   PC_Shell *shell = (PC_Shell*)pc->data;
@@ -307,8 +373,6 @@ static PetscErrorCode  PCShellSetView_Shell(PC pc,PetscErrorCode (*view)(PC,Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetApplyTranspose_Shell"
 static PetscErrorCode  PCShellSetApplyTranspose_Shell(PC pc,PetscErrorCode (*applytranspose)(PC,Vec,Vec))
 {
   PC_Shell *shell = (PC_Shell*)pc->data;
@@ -320,8 +384,6 @@ static PetscErrorCode  PCShellSetApplyTranspose_Shell(PC pc,PetscErrorCode (*app
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetApplyRichardson_Shell"
 static PetscErrorCode  PCShellSetApplyRichardson_Shell(PC pc,PetscErrorCode (*applyrich)(PC,Vec,Vec,Vec,PetscReal,PetscReal,PetscReal,PetscInt,PetscBool ,PetscInt*,PCRichardsonConvergedReason*))
 {
   PC_Shell *shell = (PC_Shell*)pc->data;
@@ -333,8 +395,6 @@ static PetscErrorCode  PCShellSetApplyRichardson_Shell(PC pc,PetscErrorCode (*ap
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetName_Shell"
 static PetscErrorCode  PCShellSetName_Shell(PC pc,const char name[])
 {
   PC_Shell       *shell = (PC_Shell*)pc->data;
@@ -346,8 +406,6 @@ static PetscErrorCode  PCShellSetName_Shell(PC pc,const char name[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellGetName_Shell"
 static PetscErrorCode  PCShellGetName_Shell(PC pc,const char *name[])
 {
   PC_Shell *shell = (PC_Shell*)pc->data;
@@ -359,8 +417,6 @@ static PetscErrorCode  PCShellGetName_Shell(PC pc,const char *name[])
 
 /* -------------------------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetDestroy"
 /*@C
    PCShellSetDestroy - Sets routine to use to destroy the user-provided
    application context.
@@ -378,7 +434,8 @@ static PetscErrorCode  PCShellGetName_Shell(PC pc,const char *name[])
 
 .  ptr - the application context
 
-   Notes: the function MUST return an error code of 0 on success and nonzero on failure.
+   Notes:
+    the function MUST return an error code of 0 on success and nonzero on failure.
 
    Level: developer
 
@@ -397,8 +454,6 @@ PetscErrorCode  PCShellSetDestroy(PC pc,PetscErrorCode (*destroy)(PC))
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetSetUp"
 /*@C
    PCShellSetSetUp - Sets routine to use to "setup" the preconditioner whenever the
    matrix operator is changed.
@@ -416,7 +471,8 @@ PetscErrorCode  PCShellSetDestroy(PC pc,PetscErrorCode (*destroy)(PC))
 
 .  pc - the preconditioner, get the application context with PCShellGetContext()
 
-   Notes: the function MUST return an error code of 0 on success and nonzero on failure.
+   Notes:
+    the function MUST return an error code of 0 on success and nonzero on failure.
 
    Level: developer
 
@@ -435,8 +491,6 @@ PetscErrorCode  PCShellSetSetUp(PC pc,PetscErrorCode (*setup)(PC))
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetView"
 /*@C
    PCShellSetView - Sets routine to use as viewer of shell preconditioner
 
@@ -454,7 +508,8 @@ PetscErrorCode  PCShellSetSetUp(PC pc,PetscErrorCode (*setup)(PC))
 +  pc - the preconditioner, get the application context with PCShellGetContext()
 -  v   - viewer
 
-   Notes: the function MUST return an error code of 0 on success and nonzero on failure.
+   Notes:
+    the function MUST return an error code of 0 on success and nonzero on failure.
 
    Level: developer
 
@@ -472,8 +527,6 @@ PetscErrorCode  PCShellSetView(PC pc,PetscErrorCode (*view)(PC,PetscViewer))
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetApply"
 /*@C
    PCShellSetApply - Sets routine to use as preconditioner.
 
@@ -492,15 +545,14 @@ PetscErrorCode  PCShellSetView(PC pc,PetscErrorCode (*view)(PC,PetscViewer))
 .  xin - input vector
 -  xout - output vector
 
-   Notes: the function MUST return an error code of 0 on success and nonzero on failure.
-
-   Developer Notes: There should also be a PCShellSetApplySymmetricRight() and PCShellSetApplySymmetricLeft().
+   Notes:
+    the function MUST return an error code of 0 on success and nonzero on failure.
 
    Level: developer
 
 .keywords: PC, shell, set, apply, user-provided
 
-.seealso: PCShellSetApplyRichardson(), PCShellSetSetUp(), PCShellSetApplyTranspose(), PCShellSetContext(), PCShellSetApplyBA()
+.seealso: PCShellSetApplyRichardson(), PCShellSetSetUp(), PCShellSetApplyTranspose(), PCShellSetContext(), PCShellSetApplyBA(), PCShellSetApplySymmetricRight(),PCShellSetApplySymmetricLeft()
 @*/
 PetscErrorCode  PCShellSetApply(PC pc,PetscErrorCode (*apply)(PC,Vec,Vec))
 {
@@ -512,8 +564,80 @@ PetscErrorCode  PCShellSetApply(PC pc,PetscErrorCode (*apply)(PC,Vec,Vec))
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetApplyBA"
+/*@C
+   PCShellSetApplySymmetricLeft - Sets routine to use as left preconditioner (when the PC_SYMMETRIC is used).
+
+   Logically Collective on PC
+
+   Input Parameters:
++  pc - the preconditioner context
+-  apply - the application-provided left preconditioning routine
+
+   Calling sequence of apply:
+.vb
+   PetscErrorCode apply (PC pc,Vec xin,Vec xout)
+.ve
+
++  pc - the preconditioner, get the application context with PCShellGetContext()
+.  xin - input vector
+-  xout - output vector
+
+   Notes:
+    the function MUST return an error code of 0 on success and nonzero on failure.
+
+   Level: developer
+
+.keywords: PC, shell, set, apply, user-provided
+
+.seealso: PCShellSetApply(), PCShellSetApplySymmetricLeft(), PCShellSetSetUp(), PCShellSetApplyTranspose(), PCShellSetContext()
+@*/
+PetscErrorCode  PCShellSetApplySymmetricLeft(PC pc,PetscErrorCode (*apply)(PC,Vec,Vec))
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  ierr = PetscTryMethod(pc,"PCShellSetApplySymmetricLeft_C",(PC,PetscErrorCode (*)(PC,Vec,Vec)),(pc,apply));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   PCShellSetApplySymmetricRight - Sets routine to use as right preconditioner (when the PC_SYMMETRIC is used).
+
+   Logically Collective on PC
+
+   Input Parameters:
++  pc - the preconditioner context
+-  apply - the application-provided right preconditioning routine
+
+   Calling sequence of apply:
+.vb
+   PetscErrorCode apply (PC pc,Vec xin,Vec xout)
+.ve
+
++  pc - the preconditioner, get the application context with PCShellGetContext()
+.  xin - input vector
+-  xout - output vector
+
+   Notes:
+    the function MUST return an error code of 0 on success and nonzero on failure.
+
+   Level: developer
+
+.keywords: PC, shell, set, apply, user-provided
+
+.seealso: PCShellSetApply(), PCShellSetApplySymmetricLeft(), PCShellSetSetUp(), PCShellSetApplyTranspose(), PCShellSetContext()
+@*/
+PetscErrorCode  PCShellSetApplySymmetricRight(PC pc,PetscErrorCode (*apply)(PC,Vec,Vec))
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
+  ierr = PetscTryMethod(pc,"PCShellSetApplySymmetricRight_C",(PC,PetscErrorCode (*)(PC,Vec,Vec)),(pc,apply));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@C
    PCShellSetApplyBA - Sets routine to use as preconditioner times operator.
 
@@ -532,7 +656,8 @@ PetscErrorCode  PCShellSetApply(PC pc,PetscErrorCode (*apply)(PC,Vec,Vec))
 .  xin - input vector
 -  xout - output vector
 
-   Notes: the function MUST return an error code of 0 on success and nonzero on failure.
+   Notes:
+    the function MUST return an error code of 0 on success and nonzero on failure.
 
    Level: developer
 
@@ -550,8 +675,6 @@ PetscErrorCode  PCShellSetApplyBA(PC pc,PetscErrorCode (*applyBA)(PC,PCSide,Vec,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetApplyTranspose"
 /*@C
    PCShellSetApplyTranspose - Sets routine to use as preconditioner transpose.
 
@@ -570,7 +693,8 @@ PetscErrorCode  PCShellSetApplyBA(PC pc,PetscErrorCode (*applyBA)(PC,PCSide,Vec,
 .  xin - input vector
 -  xout - output vector
 
-   Notes: the function MUST return an error code of 0 on success and nonzero on failure.
+   Notes:
+    the function MUST return an error code of 0 on success and nonzero on failure.
 
    Level: developer
 
@@ -591,8 +715,6 @@ PetscErrorCode  PCShellSetApplyTranspose(PC pc,PetscErrorCode (*applytranspose)(
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetPreSolve"
 /*@C
    PCShellSetPreSolve - Sets routine to apply to the operators/vectors before a KSPSolve() is
       applied. This usually does something like scale the linear system in some application
@@ -613,7 +735,8 @@ PetscErrorCode  PCShellSetApplyTranspose(PC pc,PetscErrorCode (*applytranspose)(
 .  xin - input vector
 -  xout - output vector
 
-   Notes: the function MUST return an error code of 0 on success and nonzero on failure.
+   Notes:
+    the function MUST return an error code of 0 on success and nonzero on failure.
 
    Level: developer
 
@@ -631,8 +754,6 @@ PetscErrorCode  PCShellSetPreSolve(PC pc,PetscErrorCode (*presolve)(PC,KSP,Vec,V
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetPostSolve"
 /*@C
    PCShellSetPostSolve - Sets routine to apply to the operators/vectors before a KSPSolve() is
       applied. This usually does something like scale the linear system in some application
@@ -653,7 +774,8 @@ PetscErrorCode  PCShellSetPreSolve(PC pc,PetscErrorCode (*presolve)(PC,KSP,Vec,V
 .  xin - input vector
 -  xout - output vector
 
-   Notes: the function MUST return an error code of 0 on success and nonzero on failure.
+   Notes:
+    the function MUST return an error code of 0 on success and nonzero on failure.
 
    Level: developer
 
@@ -671,8 +793,6 @@ PetscErrorCode  PCShellSetPostSolve(PC pc,PetscErrorCode (*postsolve)(PC,KSP,Vec
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetName"
 /*@C
    PCShellSetName - Sets an optional name to associate with a shell
    preconditioner.
@@ -699,8 +819,6 @@ PetscErrorCode  PCShellSetName(PC pc,const char name[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellGetName"
 /*@C
    PCShellGetName - Gets an optional name that the user has set for a shell
    preconditioner.
@@ -730,8 +848,6 @@ PetscErrorCode  PCShellGetName(PC pc,const char *name[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCShellSetApplyRichardson"
 /*@C
    PCShellSetApplyRichardson - Sets routine to use as preconditioner
    in Richardson iteration.
@@ -756,7 +872,8 @@ PetscErrorCode  PCShellGetName(PC pc,const char *name[])
 .  dtol - if residual norm increases by this factor than return
 -  maxits - number of iterations to run
 
-   Notes: the function MUST return an error code of 0 on success and nonzero on failure.
+   Notes:
+    the function MUST return an error code of 0 on success and nonzero on failure.
 
    Level: developer
 
@@ -804,8 +921,6 @@ $             PCShellSetDestroy(pc,destroy);               (optional)
            PCShellGetName(), PCShellSetContext(), PCShellGetContext(), PCShellSetApplyBA()
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCCreate_Shell"
 PETSC_EXTERN PetscErrorCode PCCreate_Shell(PC pc)
 {
   PetscErrorCode ierr;
@@ -818,6 +933,8 @@ PETSC_EXTERN PetscErrorCode PCCreate_Shell(PC pc)
   pc->ops->destroy         = PCDestroy_Shell;
   pc->ops->view            = PCView_Shell;
   pc->ops->apply           = PCApply_Shell;
+  pc->ops->applysymmetricleft  = PCApplySymmetricLeft_Shell;
+  pc->ops->applysymmetricright = PCApplySymmetricRight_Shell;
   pc->ops->applytranspose  = 0;
   pc->ops->applyrichardson = 0;
   pc->ops->setup           = 0;
@@ -834,10 +951,14 @@ PETSC_EXTERN PetscErrorCode PCCreate_Shell(PC pc)
   shell->setup          = 0;
   shell->view           = 0;
   shell->destroy        = 0;
+  shell->applysymmetricleft  = 0;
+  shell->applysymmetricright = 0;
 
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetDestroy_C",PCShellSetDestroy_Shell);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetSetUp_C",PCShellSetSetUp_Shell);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetApply_C",PCShellSetApply_Shell);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetApplySymmetricLeft_C",PCShellSetApplySymmetricLeft_Shell);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetApplySymmetricRight_C",PCShellSetApplySymmetricRight_Shell);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetApplyBA_C",PCShellSetApplyBA_Shell);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetPreSolve_C",PCShellSetPreSolve_Shell);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)pc,"PCShellSetPostSolve_C",PCShellSetPostSolve_Shell);CHKERRQ(ierr);
