@@ -12,8 +12,6 @@ typedef struct {
   PetscReal  fshift;
 } PC_SOR;
 
-#undef __FUNCT__
-#define __FUNCT__ "PCDestroy_SOR"
 static PetscErrorCode PCDestroy_SOR(PC pc)
 {
   PetscErrorCode ierr;
@@ -23,43 +21,50 @@ static PetscErrorCode PCDestroy_SOR(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_SOR"
 static PetscErrorCode PCApply_SOR(PC pc,Vec x,Vec y)
 {
   PC_SOR         *jac = (PC_SOR*)pc->data;
   PetscErrorCode ierr;
   PetscInt       flag = jac->sym | SOR_ZERO_INITIAL_GUESS;
-  PetscReal      fshift;
-  
+
   PetscFunctionBegin;
-  fshift = (jac->fshift ? jac->fshift : pc->erroriffailure ? 0.0 : -1.0);
-  ierr = MatSOR(pc->pmat,x,jac->omega,(MatSORType)flag,fshift,jac->its,jac->lits,y);CHKERRQ(ierr);
+  ierr = MatSOR(pc->pmat,x,jac->omega,(MatSORType)flag,jac->fshift,jac->its,jac->lits,y);CHKERRQ(ierr);
+  ierr = MatFactorGetError(pc->pmat,(MatFactorError*)&pc->failedreason);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode PCApplyTranspose_SOR(PC pc,Vec x,Vec y)
+{
+  PC_SOR         *jac = (PC_SOR*)pc->data;
+  PetscErrorCode ierr;
+  PetscInt       flag = jac->sym | SOR_ZERO_INITIAL_GUESS;
+  PetscBool      set,sym;
+
+  PetscFunctionBegin;
+  ierr = MatIsSymmetricKnown(pc->pmat,&set,&sym);CHKERRQ(ierr);
+  if (!set || !sym || (jac->sym != SOR_SYMMETRIC_SWEEP && jac->sym != SOR_LOCAL_SYMMETRIC_SWEEP)) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_SUP,"Can only apply transpose of SOR if matrix is symmetric and sweep is symmetric");
+  ierr = MatSOR(pc->pmat,x,jac->omega,(MatSORType)flag,jac->fshift,jac->its,jac->lits,y);CHKERRQ(ierr);
+  ierr = MatFactorGetError(pc->pmat,(MatFactorError*)&pc->failedreason);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyRichardson_SOR"
 static PetscErrorCode PCApplyRichardson_SOR(PC pc,Vec b,Vec y,Vec w,PetscReal rtol,PetscReal abstol, PetscReal dtol,PetscInt its,PetscBool guesszero,PetscInt *outits,PCRichardsonConvergedReason *reason)
 {
   PC_SOR         *jac = (PC_SOR*)pc->data;
   PetscErrorCode ierr;
   MatSORType     stype = jac->sym;
-  PetscReal      fshift;
 
   PetscFunctionBegin;
   ierr = PetscInfo1(pc,"Warning, convergence critera ignored, using %D iterations\n",its);CHKERRQ(ierr);
   if (guesszero) stype = (MatSORType) (stype | SOR_ZERO_INITIAL_GUESS);
-  fshift = (jac->fshift ? jac->fshift : pc->erroriffailure ? 0.0 : -1.0);
-  ierr = MatSOR(pc->pmat,b,jac->omega,stype,fshift,its*jac->its,jac->lits,y);CHKERRQ(ierr);
+  ierr = MatSOR(pc->pmat,b,jac->omega,stype,jac->fshift,its*jac->its,jac->lits,y);CHKERRQ(ierr);
+  ierr = MatFactorGetError(pc->pmat,(MatFactorError*)&pc->failedreason);CHKERRQ(ierr); 
   *outits = its;
   *reason = PCRICHARDSON_CONVERGED_ITS;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetFromOptions_SOR"
-PetscErrorCode PCSetFromOptions_SOR(PetscOptions *PetscOptionsObject,PC pc)
+PetscErrorCode PCSetFromOptions_SOR(PetscOptionItems *PetscOptionsObject,PC pc)
 {
   PC_SOR         *jac = (PC_SOR*)pc->data;
   PetscErrorCode ierr;
@@ -87,8 +92,6 @@ PetscErrorCode PCSetFromOptions_SOR(PetscOptions *PetscOptionsObject,PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCView_SOR"
 PetscErrorCode PCView_SOR(PC pc,PetscViewer viewer)
 {
   PC_SOR         *jac = (PC_SOR*)pc->data;
@@ -100,7 +103,7 @@ PetscErrorCode PCView_SOR(PC pc,PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
-    if (sym & SOR_ZERO_INITIAL_GUESS) {ierr = PetscViewerASCIIPrintf(viewer,"  SOR:  zero initial guess\n");CHKERRQ(ierr);}
+    if (sym & SOR_ZERO_INITIAL_GUESS) {ierr = PetscViewerASCIIPrintf(viewer,"  zero initial guess\n");CHKERRQ(ierr);}
     if (sym == SOR_APPLY_UPPER)                                              sortype = "apply_upper";
     else if (sym == SOR_APPLY_LOWER)                                         sortype = "apply_lower";
     else if (sym & SOR_EISENSTAT)                                            sortype = "Eisenstat";
@@ -111,15 +114,13 @@ PetscErrorCode PCView_SOR(PC pc,PetscViewer viewer)
     else if (sym & SOR_LOCAL_FORWARD_SWEEP)                                  sortype = "local_forward";
     else if (sym & SOR_LOCAL_BACKWARD_SWEEP)                                 sortype = "local_backward";
     else                                                                     sortype = "unknown";
-    ierr = PetscViewerASCIIPrintf(viewer,"  SOR: type = %s, iterations = %D, local iterations = %D, omega = %g\n",sortype,jac->its,jac->lits,(double)jac->omega);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  type = %s, iterations = %D, local iterations = %D, omega = %g\n",sortype,jac->its,jac->lits,(double)jac->omega);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
 
 /* ------------------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "PCSORSetSymmetric_SOR"
 static PetscErrorCode  PCSORSetSymmetric_SOR(PC pc,MatSORType flag)
 {
   PC_SOR *jac = (PC_SOR*)pc->data;
@@ -129,8 +130,6 @@ static PetscErrorCode  PCSORSetSymmetric_SOR(PC pc,MatSORType flag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSORSetOmega_SOR"
 static PetscErrorCode  PCSORSetOmega_SOR(PC pc,PetscReal omega)
 {
   PC_SOR *jac = (PC_SOR*)pc->data;
@@ -141,8 +140,6 @@ static PetscErrorCode  PCSORSetOmega_SOR(PC pc,PetscReal omega)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSORSetIterations_SOR"
 static PetscErrorCode  PCSORSetIterations_SOR(PC pc,PetscInt its,PetscInt lits)
 {
   PC_SOR *jac = (PC_SOR*)pc->data;
@@ -153,8 +150,6 @@ static PetscErrorCode  PCSORSetIterations_SOR(PC pc,PetscInt its,PetscInt lits)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSORGetSymmetric_SOR"
 static PetscErrorCode  PCSORGetSymmetric_SOR(PC pc,MatSORType *flag)
 {
   PC_SOR *jac = (PC_SOR*)pc->data;
@@ -164,8 +159,6 @@ static PetscErrorCode  PCSORGetSymmetric_SOR(PC pc,MatSORType *flag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSORGetOmega_SOR"
 static PetscErrorCode  PCSORGetOmega_SOR(PC pc,PetscReal *omega)
 {
   PC_SOR *jac = (PC_SOR*)pc->data;
@@ -175,8 +168,6 @@ static PetscErrorCode  PCSORGetOmega_SOR(PC pc,PetscReal *omega)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSORGetIterations_SOR"
 static PetscErrorCode  PCSORGetIterations_SOR(PC pc,PetscInt *its,PetscInt *lits)
 {
   PC_SOR *jac = (PC_SOR*)pc->data;
@@ -188,8 +179,6 @@ static PetscErrorCode  PCSORGetIterations_SOR(PC pc,PetscInt *its,PetscInt *lits
 }
 
 /* ------------------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "PCSORGetSymmetric"
 /*@
    PCSORGetSymmetric - Gets the form the SOR preconditioner is using;   backward, or forward relaxation.  The local variants perform SOR on
    each processor.  By default forward relaxation is used.
@@ -238,8 +227,6 @@ PetscErrorCode  PCSORGetSymmetric(PC pc,MatSORType *flag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSORGetOmega"
 /*@
    PCSORGetOmega - Gets the SOR relaxation coefficient, omega
    (where omega = 1.0 by default).
@@ -271,8 +258,6 @@ PetscErrorCode  PCSORGetOmega(PC pc,PetscReal *omega)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSORGetIterations"
 /*@
    PCSORGetIterations - Gets the number of inner iterations to
    be used by the SOR preconditioner. The default is 1.
@@ -292,7 +277,8 @@ PetscErrorCode  PCSORGetOmega(PC pc,PetscReal *omega)
 
    Level: intermediate
 
-   Notes: When run on one processor the number of smoothings is lits*its
+   Notes:
+    When run on one processor the number of smoothings is lits*its
 
 .keywords: PC, SOR, SSOR, set, iterations
 
@@ -308,8 +294,6 @@ PetscErrorCode  PCSORGetIterations(PC pc,PetscInt *its,PetscInt *lits)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSORSetSymmetric"
 /*@
    PCSORSetSymmetric - Sets the SOR preconditioner to use symmetric (SSOR),
    backward, or forward relaxation.  The local variants perform SOR on
@@ -358,8 +342,6 @@ PetscErrorCode  PCSORSetSymmetric(PC pc,MatSORType flag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSORSetOmega"
 /*@
    PCSORSetOmega - Sets the SOR relaxation coefficient, omega
    (where omega = 1.0 by default).
@@ -390,8 +372,6 @@ PetscErrorCode  PCSORSetOmega(PC pc,PetscReal omega)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSORSetIterations"
 /*@
    PCSORSetIterations - Sets the number of inner iterations to
    be used by the SOR preconditioner. The default is 1.
@@ -409,7 +389,8 @@ PetscErrorCode  PCSORSetOmega(PC pc,PetscReal omega)
 
    Level: intermediate
 
-   Notes: When run on one processor the number of smoothings is lits*its
+   Notes:
+    When run on one processor the number of smoothings is lits*its
 
 .keywords: PC, SOR, SSOR, set, iterations
 
@@ -445,7 +426,8 @@ PetscErrorCode  PCSORSetIterations(PC pc,PetscInt its,PetscInt lits)
 
   Concepts: SOR, preconditioners, Gauss-Seidel
 
-   Notes: Only implemented for the AIJ  and SeqBAIJ matrix formats.
+   Notes:
+    Only implemented for the AIJ  and SeqBAIJ matrix formats.
           Not a true parallel SOR, in parallel this implementation corresponds to block
           Jacobi with SOR on each block.
 
@@ -459,16 +441,13 @@ PetscErrorCode  PCSORSetIterations(PC pc,PetscInt its,PetscInt lits)
           For SeqBAIJ the diagonal blocks are inverted using dense LU with partial pivoting. If a zero pivot is detected 
           the computation is stopped with an error
 
-  Developer Notes: We should add support for diagonal blocks that are singular to generate a Inf and thus cause KSPSolve()
-           to terminate with KSP_DIVERGED_NANORIF instead of stopping the program allowing
-           a nonlinear solver/ODE integrator to recover without stopping the program as currently happens.
+          If used with KSPRICHARDSON and no monitors the convergence test is skipped to improve speed, thus it always iterates 
+          the maximum number of iterations you've selected for KSP. It is usually used in this mode as a smoother for multigrid.
 
 .seealso:  PCCreate(), PCSetType(), PCType (for list of available types), PC,
            PCSORSetIterations(), PCSORSetSymmetric(), PCSORSetOmega(), PCEISENSTAT
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCCreate_SOR"
 PETSC_EXTERN PetscErrorCode PCCreate_SOR(PC pc)
 {
   PetscErrorCode ierr;
@@ -478,6 +457,7 @@ PETSC_EXTERN PetscErrorCode PCCreate_SOR(PC pc)
   ierr = PetscNewLog(pc,&jac);CHKERRQ(ierr);
 
   pc->ops->apply           = PCApply_SOR;
+  pc->ops->applytranspose  = PCApplyTranspose_SOR;
   pc->ops->applyrichardson = PCApplyRichardson_SOR;
   pc->ops->setfromoptions  = PCSetFromOptions_SOR;
   pc->ops->setup           = 0;
