@@ -32,8 +32,6 @@ typedef enum {READ=1, WRITE=2, READ_WRITE=3} AccessMode;
    The interface routine PCSetUp() is not usually called directly by
    the user, but instead is called by PCApply() if necessary.
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUp_SVD"
 static PetscErrorCode PCSetUp_SVD(PC pc)
 {
 #if defined(PETSC_MISSING_LAPACK_GESVD)
@@ -64,7 +62,11 @@ static PetscErrorCode PCSetUp_SVD(PC pc)
     ierr = MatDuplicate(jac->A,MAT_DO_NOT_COPY_VALUES,&jac->U);CHKERRQ(ierr);
     ierr = MatDuplicate(jac->A,MAT_DO_NOT_COPY_VALUES,&jac->Vt);CHKERRQ(ierr);
   }
-  ierr  = MatGetSize(pc->pmat,&n,NULL);CHKERRQ(ierr);
+  ierr  = MatGetSize(jac->A,&n,NULL);CHKERRQ(ierr);
+  if (!n) {
+    ierr = PetscInfo(pc,"Matrix has zero rows, skipping svd\n");CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  }
   ierr  = PetscBLASIntCast(n,&nb);CHKERRQ(ierr);
   lwork = 5*nb;
   ierr  = PetscMalloc1(lwork,&work);CHKERRQ(ierr);
@@ -90,7 +92,7 @@ static PetscErrorCode PCSetUp_SVD(PC pc)
     PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("A","A",&nb,&nb,a,&nb,dd,u,&nb,v,&nb,work,&lwork,rwork,&lierr));
     if (lierr) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"gesv() error %d",lierr);
     ierr = PetscFree(rwork);CHKERRQ(ierr);
-    for (i=0; i<nb; i++) d[i] = dd[i];
+    for (i=0; i<n; i++) d[i] = dd[i];
     ierr = PetscFree(dd);CHKERRQ(ierr);
     ierr = PetscFPTrapPop();CHKERRQ(ierr);
   }
@@ -145,8 +147,6 @@ static PetscErrorCode PCSetUp_SVD(PC pc)
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSVDGetVec"
 static PetscErrorCode PCSVDGetVec(PC pc,PCSide side,AccessMode amode,Vec x,Vec *xred)
 {
   PC_SVD         *jac = (PC_SVD*)pc->data;
@@ -184,8 +184,6 @@ static PetscErrorCode PCSVDGetVec(PC pc,PCSide side,AccessMode amode,Vec x,Vec *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSVDRestoreVec"
 static PetscErrorCode PCSVDRestoreVec(PC pc,PCSide side,AccessMode amode,Vec x,Vec *xred)
 {
   PC_SVD         *jac = (PC_SVD*)pc->data;
@@ -226,8 +224,6 @@ static PetscErrorCode PCSVDRestoreVec(PC pc,PCSide side,AccessMode amode,Vec x,V
 
    Application Interface Routine: PCApply()
  */
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_SVD"
 static PetscErrorCode PCApply_SVD(PC pc,Vec x,Vec y)
 {
   PC_SVD         *jac = (PC_SVD*)pc->data;
@@ -253,8 +249,6 @@ static PetscErrorCode PCApply_SVD(PC pc,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApplyTranspose_SVD"
 static PetscErrorCode PCApplyTranspose_SVD(PC pc,Vec x,Vec y)
 {
   PC_SVD         *jac = (PC_SVD*)pc->data;
@@ -264,24 +258,14 @@ static PetscErrorCode PCApplyTranspose_SVD(PC pc,Vec x,Vec y)
   PetscFunctionBegin;
   ierr = PCSVDGetVec(pc,PC_LEFT,READ,x,&xred);CHKERRQ(ierr);
   ierr = PCSVDGetVec(pc,PC_RIGHT,WRITE,y,&yred);CHKERRQ(ierr);
-#if !defined(PETSC_USE_COMPLEX)
-  ierr = MatMultTranspose(jac->Vt,work,yred);CHKERRQ(ierr);
-#else
-  ierr = MatMultHermitianTranspose(jac->Vt,work,yred);CHKERRQ(ierr);
-#endif
+  ierr = MatMult(jac->Vt,xred,work);CHKERRQ(ierr);
   ierr = VecPointwiseMult(work,work,jac->diag);CHKERRQ(ierr);
-#if !defined(PETSC_USE_COMPLEX)
-  ierr = MatMultTranspose(jac->U,xred,work);CHKERRQ(ierr);
-#else
-  ierr = MatMultHermitianTranspose(jac->U,xred,work);CHKERRQ(ierr);
-#endif
+  ierr = MatMult(jac->U,work,yred);CHKERRQ(ierr);
   ierr = PCSVDRestoreVec(pc,PC_LEFT,READ,x,&xred);CHKERRQ(ierr);
   ierr = PCSVDRestoreVec(pc,PC_RIGHT,WRITE,y,&yred);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCReset_SVD"
 static PetscErrorCode PCReset_SVD(PC pc)
 {
   PC_SVD         *jac = (PC_SVD*)pc->data;
@@ -310,8 +294,6 @@ static PetscErrorCode PCReset_SVD(PC pc)
 
    Application Interface Routine: PCDestroy()
 */
-#undef __FUNCT__
-#define __FUNCT__ "PCDestroy_SVD"
 static PetscErrorCode PCDestroy_SVD(PC pc)
 {
   PC_SVD         *jac = (PC_SVD*)pc->data;
@@ -324,9 +306,7 @@ static PetscErrorCode PCDestroy_SVD(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetFromOptions_SVD"
-static PetscErrorCode PCSetFromOptions_SVD(PetscOptions *PetscOptionsObject,PC pc)
+static PetscErrorCode PCSetFromOptions_SVD(PetscOptionItems *PetscOptionsObject,PC pc)
 {
   PetscErrorCode ierr;
   PC_SVD         *jac = (PC_SVD*)pc->data;
@@ -348,6 +328,20 @@ static PetscErrorCode PCSetFromOptions_SVD(PetscOptions *PetscOptionsObject,PC p
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode PCView_SVD(PC pc,PetscViewer viewer)
+{
+  PC_SVD         *svd = (PC_SVD*)pc->data;
+  PetscErrorCode ierr;
+  PetscBool      iascii;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
+  if (iascii) {
+    ierr = PetscViewerASCIIPrintf(viewer,"  All singular values smaller than %g treated as zero\n",(double)svd->zerosing);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  Provided essential rank of the matrix %D (all other eigenvalues are zeroed)\n",svd->essrank);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
 /* -------------------------------------------------------------------------- */
 /*
    PCCreate_SVD - Creates a SVD preconditioner context, PC_SVD,
@@ -378,8 +372,6 @@ static PetscErrorCode PCSetFromOptions_SVD(PetscOptions *PetscOptionsObject,PC p
 .seealso:  PCCreate(), PCSetType(), PCType (for list of available types), PC
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PCCreate_SVD"
 PETSC_EXTERN PetscErrorCode PCCreate_SVD(PC pc)
 {
   PC_SVD         *jac;
@@ -407,7 +399,7 @@ PETSC_EXTERN PetscErrorCode PCCreate_SVD(PC pc)
   pc->ops->reset           = PCReset_SVD;
   pc->ops->destroy         = PCDestroy_SVD;
   pc->ops->setfromoptions  = PCSetFromOptions_SVD;
-  pc->ops->view            = 0;
+  pc->ops->view            = PCView_SVD;
   pc->ops->applyrichardson = 0;
   PetscFunctionReturn(0);
 }
