@@ -208,6 +208,7 @@ PCTFS_gs_id *PCTFS_gs_init(PetscInt *elms, PetscInt nel, PetscInt level)
 
   ierr = MPI_Comm_group(MPI_COMM_WORLD,&PCTFS_gs_group);CHKERRABORT(PETSC_COMM_WORLD,ierr);
   ierr = MPI_Comm_create(MPI_COMM_WORLD,PCTFS_gs_group,&PCTFS_gs_comm);CHKERRABORT(PETSC_COMM_WORLD,ierr);
+  ierr = MPI_Group_free(&PCTFS_gs_group);CHKERRABORT(PETSC_COMM_WORLD,ierr);
 
   gs->PCTFS_gs_comm=PCTFS_gs_comm;
 
@@ -413,11 +414,8 @@ static PetscErrorCode gsi_via_bit_mask(PCTFS_gs_id *gs)
     /* ok find intersection */
     map    = gs->companion;
     reduce = gs->local_reduce;
-    for (i=0, t1=0; i<gs->num_local; i++, reduce++)
-    {
-      if ((PCTFS_ivec_binary_search(**reduce,gs->pw_elm_list,gs->len_pw_list)>=0)
-          ||
-          PCTFS_ivec_binary_search(**reduce,gs->tree_map_in,gs->tree_map_sz)>=0) {
+    for (i=0, t1=0; i<gs->num_local; i++, reduce++) {
+      if ((PCTFS_ivec_binary_search(**reduce,gs->pw_elm_list,gs->len_pw_list)>=0) || PCTFS_ivec_binary_search(**reduce,gs->tree_map_in,gs->tree_map_sz)>=0) {
         t1++;
         if (gs->num_local_reduce[i]<=0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"nobody in list?");
         gs->num_local_reduce[i] *= -1;
@@ -905,9 +903,11 @@ static PetscErrorCode PCTFS_gs_gop_local_in_plus(PCTFS_gs_id *gs,  PetscScalar *
 /******************************************************************************/
 PetscErrorCode PCTFS_gs_free(PCTFS_gs_id *gs)
 {
-  PetscInt i;
+  PetscInt       i;
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = MPI_Comm_free(&gs->PCTFS_gs_comm);CHKERRABORT(PETSC_COMM_WORLD,ierr);
   if (gs->nghs) free((void*) gs->nghs);
   if (gs->pw_nghs) free((void*) gs->pw_nghs);
 
@@ -950,7 +950,6 @@ PetscErrorCode PCTFS_gs_free(PCTFS_gs_id *gs)
 
   /* local info */
   if (gs->num_local_total>=0) {
-    /*      for (i=0;i<gs->num_local_total;i++) */
     for (i=0;i<gs->num_local_total+1;i++) {
       if (gs->num_gop_local_reduce[i]) free((void*) gs->gop_local_reduce[i]);
     }
