@@ -2,7 +2,7 @@
         Provides an interface to the Tufo-Fischer parallel direct solver
 */
 
-#include <petsc-private/pcimpl.h>   /*I "petscpc.h" I*/
+#include <petsc/private/pcimpl.h>   /*I "petscpc.h" I*/
 #include <../src/mat/impls/aij/mpi/mpiaij.h>
 #include <../src/ksp/pc/impls/tfs/tfs.h>
 
@@ -13,8 +13,6 @@ typedef struct {
   PetscInt nd;
 } PC_TFS;
 
-#undef __FUNCT__
-#define __FUNCT__ "PCDestroy_TFS"
 PetscErrorCode PCDestroy_TFS(PC pc)
 {
   PC_TFS         *tfs = (PC_TFS*)pc->data;
@@ -35,42 +33,38 @@ PetscErrorCode PCDestroy_TFS(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_TFS_XXT"
 static PetscErrorCode PCApply_TFS_XXT(PC pc,Vec x,Vec y)
 {
-  PC_TFS         *tfs = (PC_TFS*)pc->data;
-  PetscScalar    *xx,*yy;
-  PetscErrorCode ierr;
+  PC_TFS            *tfs = (PC_TFS*)pc->data;
+  PetscScalar       *yy;
+  const PetscScalar *xx;
+  PetscErrorCode    ierr;
 
   PetscFunctionBegin;
-  ierr = VecGetArray(x,&xx);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(x,&xx);CHKERRQ(ierr);
   ierr = VecGetArray(y,&yy);CHKERRQ(ierr);
-  ierr = XXT_solve(tfs->xxt,yy,xx);CHKERRQ(ierr);
-  ierr = VecRestoreArray(x,&xx);CHKERRQ(ierr);
+  ierr = XXT_solve(tfs->xxt,yy,(PetscScalar*)xx);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(x,&xx);CHKERRQ(ierr);
   ierr = VecRestoreArray(y,&yy);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_TFS_XYT"
 static PetscErrorCode PCApply_TFS_XYT(PC pc,Vec x,Vec y)
 {
-  PC_TFS         *tfs = (PC_TFS*)pc->data;
-  PetscScalar    *xx,*yy;
-  PetscErrorCode ierr;
+  PC_TFS            *tfs = (PC_TFS*)pc->data;
+  PetscScalar       *yy;
+  const PetscScalar *xx;
+  PetscErrorCode    ierr;
 
   PetscFunctionBegin;
-  ierr = VecGetArray(x,&xx);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(x,&xx);CHKERRQ(ierr);
   ierr = VecGetArray(y,&yy);CHKERRQ(ierr);
-  ierr = XYT_solve(tfs->xyt,yy,xx);CHKERRQ(ierr);
-  ierr = VecRestoreArray(x,&xx);CHKERRQ(ierr);
+  ierr = XYT_solve(tfs->xyt,yy,(PetscScalar*)xx);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(x,&xx);CHKERRQ(ierr);
   ierr = VecRestoreArray(y,&yy);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCTFSLocalMult_TFS"
 static PetscErrorCode PCTFSLocalMult_TFS(PC pc,PetscScalar *xin,PetscScalar *xout)
 {
   PC_TFS         *tfs = (PC_TFS*)pc->data;
@@ -90,8 +84,6 @@ static PetscErrorCode PCTFSLocalMult_TFS(PC pc,PetscScalar *xin,PetscScalar *xou
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetUp_TFS"
 static PetscErrorCode PCSetUp_TFS(PC pc)
 {
   PC_TFS         *tfs = (PC_TFS*)pc->data;
@@ -113,7 +105,7 @@ static PetscErrorCode PCSetUp_TFS(PC pc)
 
   /* generate the local to global mapping */
   ncol = a->A->cmap->n + a->B->cmap->n;
-  ierr = PetscMalloc((ncol)*sizeof(PetscInt),&localtoglobal);CHKERRQ(ierr);
+  ierr = PetscMalloc1(ncol,&localtoglobal);CHKERRQ(ierr);
   for (i=0; i<a->A->cmap->n; i++) localtoglobal[i] = A->cmap->rstart + i + 1;
   for (i=0; i<a->B->cmap->n; i++) localtoglobal[i+a->A->cmap->n] = a->garray[i] + 1;
 
@@ -141,34 +133,32 @@ static PetscErrorCode PCSetUp_TFS(PC pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetFromOptions_TFS"
-static PetscErrorCode PCSetFromOptions_TFS(PC pc)
+static PetscErrorCode PCSetFromOptions_TFS(PetscOptionItems *PetscOptionsObject,PC pc)
 {
   PetscFunctionBegin;
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__ "PCView_TFS"
 static PetscErrorCode PCView_TFS(PC pc,PetscViewer viewer)
 {
   PetscFunctionBegin;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCCreate_TFS"
 /*MC
      PCTFS - A parallel direct solver intended for problems with very few unknowns (like the
-         coarse grid in multigrid).
+         coarse grid in multigrid). Performs a Cholesky or LU factorization of a matrix defined by
+         its local matrix vector product.
 
-   Implemented by  Henry M. Tufo III and Paul Fischer
+   Implemented by  Henry M. Tufo III and Paul Fischer originally for Nek5000 and called XXT or XYT
 
    Level: beginner
 
-   Notes: Only implemented for the MPIAIJ matrices
+   Notes:
+    Only implemented for the MPIAIJ matrices
 
-          Only works on a solver object that lives on all of PETSC_COMM_WORLD!
+    Only works on a solver object that lives on all of PETSC_COMM_WORLD!
+
+    Only works for real numbers (is not built if PetscScalar is complex)
 
 .seealso:  PCCreate(), PCSetType(), PCType (for list of available types), PC
 M*/
@@ -181,7 +171,7 @@ PETSC_EXTERN PetscErrorCode PCCreate_TFS(PC pc)
   PetscFunctionBegin;
   ierr = MPI_Comm_compare(PETSC_COMM_WORLD,PetscObjectComm((PetscObject)pc),&cmp);CHKERRQ(ierr);
   if (cmp != MPI_IDENT && cmp != MPI_CONGRUENT) SETERRQ(PetscObjectComm((PetscObject)pc),PETSC_ERR_SUP,"TFS only works with PETSC_COMM_WORLD objects");
-  ierr = PetscNewLog(pc,PC_TFS,&tfs);CHKERRQ(ierr);
+  ierr = PetscNewLog(pc,&tfs);CHKERRQ(ierr);
 
   tfs->xxt = 0;
   tfs->xyt = 0;
