@@ -114,8 +114,8 @@ Last Modification:
 ***********************************types.h************************************/
 
 typedef PetscErrorCode (*vfp)(void*,void*,PetscInt,...);
-typedef PetscErrorCode (*rbfp)(PetscScalar*, PetscScalar*, PetscInt len);
-typedef PetscInt (*bfp)(void*, void*, PetscInt *len, MPI_Datatype *dt);
+typedef PetscErrorCode (*rbfp)(PetscScalar*, PetscScalar*, PetscInt);
+typedef PetscInt (*bfp)(void*, void*, PetscInt*, MPI_Datatype*);
 
 /***********************************comm.h*************************************
 
@@ -136,12 +136,12 @@ PETSC_INTERN PetscMPIInt PCTFS_num_nodes;
 PETSC_INTERN PetscMPIInt PCTFS_floor_num_nodes;
 PETSC_INTERN PetscMPIInt PCTFS_i_log2_num_nodes;
 
-PETSC_INTERN PetscErrorCode PCTFS_giop(PetscInt *vals, PetscInt *work, PetscInt n, PetscInt *oprs);
-PETSC_INTERN PetscErrorCode PCTFS_grop(PetscScalar *vals, PetscScalar *work, PetscInt n, PetscInt *oprs);
+PETSC_INTERN PetscErrorCode PCTFS_giop(PetscInt*,PetscInt*,PetscInt,PetscInt*);
+PETSC_INTERN PetscErrorCode PCTFS_grop(PetscScalar*,PetscScalar*,PetscInt,PetscInt*);
 PETSC_INTERN PetscErrorCode PCTFS_comm_init(void);
-PETSC_INTERN PetscErrorCode PCTFS_giop_hc(PetscInt *vals, PetscInt *work, PetscInt n, PetscInt *oprs, PetscInt dim);
-PETSC_INTERN PetscErrorCode PCTFS_grop_hc(PetscScalar *vals, PetscScalar *work, PetscInt n, PetscInt *oprs, PetscInt dim);
-PETSC_INTERN PetscErrorCode PCTFS_ssgl_radd(PetscScalar *vals, PetscScalar *work, PetscInt level, PetscInt *segs);
+PETSC_INTERN PetscErrorCode PCTFS_giop_hc(PetscInt*,PetscInt*,PetscInt,PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_grop_hc(PetscScalar*,PetscScalar*,PetscInt,PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_ssgl_radd(PetscScalar*,PetscScalar*,PetscInt,PetscInt*);
 
 #define MSGTAG0 101
 #define MSGTAG1 1001
@@ -166,55 +166,53 @@ PETSC_INTERN PetscErrorCode PCTFS_ssgl_radd(PetscScalar *vals, PetscScalar *work
 #define GL_MIN_ABS      12
 #define GL_EXISTS       13
 
-PETSC_INTERN PetscInt *PCTFS_ivec_copy(PetscInt *arg1, PetscInt *arg2, PetscInt n);
+PETSC_INTERN PetscInt       *PCTFS_ivec_copy(PetscInt*,PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_zero(PetscInt*, PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_set(PetscInt*,PetscInt,PetscInt);
 
-PETSC_INTERN PetscErrorCode PCTFS_ivec_zero(PetscInt *arg1, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_ivec_set(PetscInt *arg1, PetscInt arg2, PetscInt n);
+PETSC_INTERN PetscInt PCTFS_ivec_lb(PetscInt*,PetscInt);
+PETSC_INTERN PetscInt PCTFS_ivec_ub(PetscInt*,PetscInt);
+PETSC_INTERN PetscInt PCTFS_ivec_sum(PetscInt*,PetscInt);
+PETSC_INTERN vfp      PCTFS_ivec_fct_addr(PetscInt);
 
-PETSC_INTERN PetscInt PCTFS_ivec_lb(PetscInt *work, PetscInt n);
-PETSC_INTERN PetscInt PCTFS_ivec_ub(PetscInt *work, PetscInt n);
-PETSC_INTERN PetscInt PCTFS_ivec_sum(PetscInt *arg1, PetscInt n);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_non_uniform(PetscInt*,PetscInt*,PetscInt,PetscInt*);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_max(PetscInt*,PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_min(PetscInt*,PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_mult(PetscInt*,PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_add(PetscInt*,PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_xor(PetscInt*,PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_or(PetscInt*,PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_and(PetscInt*,PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_lxor(PetscInt*,PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_lor(PetscInt*,PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_land(PetscInt*,PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_and3(PetscInt*,PetscInt*,PetscInt*,PetscInt);
 
-PETSC_INTERN vfp PCTFS_ivec_fct_addr(PetscInt type);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_sort_companion(PetscInt*,PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_sort(PetscInt*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_SMI_sort(void*,void*,PetscInt,PetscInt);
+PETSC_INTERN PetscInt       PCTFS_ivec_binary_search(PetscInt,PetscInt*,PetscInt);
+PETSC_INTERN PetscInt       PCTFS_ivec_linear_search(PetscInt,PetscInt*,PetscInt);
 
-PETSC_INTERN PetscErrorCode PCTFS_ivec_non_uniform(PetscInt *arg1, PetscInt *arg2, PetscInt n, PetscInt *arg3);
-PETSC_INTERN PetscErrorCode PCTFS_ivec_max(PetscInt *arg1, PetscInt *arg2, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_ivec_min(PetscInt *arg1, PetscInt *arg2, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_ivec_mult(PetscInt *arg1, PetscInt *arg2, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_ivec_add(PetscInt *arg1, PetscInt *arg2, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_ivec_xor(PetscInt *arg1, PetscInt *arg2, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_ivec_or(PetscInt *arg1, PetscInt *arg2, PetscInt len);
-PETSC_INTERN PetscErrorCode PCTFS_ivec_and(PetscInt *arg1, PetscInt *arg2, PetscInt len);
-PETSC_INTERN PetscErrorCode PCTFS_ivec_lxor(PetscInt *arg1, PetscInt *arg2, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_ivec_lor(PetscInt *arg1, PetscInt *arg2, PetscInt len);
-PETSC_INTERN PetscErrorCode PCTFS_ivec_land(PetscInt *arg1, PetscInt *arg2, PetscInt len);
-PETSC_INTERN PetscErrorCode PCTFS_ivec_and3(PetscInt *arg1,  PetscInt *arg2,  PetscInt *arg3, PetscInt n);
-
-PETSC_INTERN PetscErrorCode PCTFS_ivec_sort_companion(PetscInt *ar, PetscInt *ar2, PetscInt size);
-PETSC_INTERN PetscErrorCode PCTFS_ivec_sort(PetscInt *ar, PetscInt size);
-PETSC_INTERN PetscErrorCode PCTFS_SMI_sort(void *ar1, void *ar2, PetscInt size, PetscInt type);
-PETSC_INTERN PetscInt PCTFS_ivec_binary_search(PetscInt item, PetscInt *list, PetscInt n);
-PETSC_INTERN PetscInt PCTFS_ivec_linear_search(PetscInt item, PetscInt *list, PetscInt n);
-
-PETSC_INTERN PetscErrorCode PCTFS_ivec_sort_companion_hack(PetscInt *ar, PetscInt **ar2, PetscInt size);
+PETSC_INTERN PetscErrorCode PCTFS_ivec_sort_companion_hack(PetscInt*,PetscInt**,PetscInt);
 
 #define SORT_INTEGER 1
 #define SORT_INT_PTR 2
 
-PETSC_INTERN PetscErrorCode PCTFS_rvec_zero(PetscScalar *arg1, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_rvec_one(PetscScalar *arg1, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_rvec_set(PetscScalar *arg1, PetscScalar arg2, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_rvec_copy(PetscScalar *arg1, PetscScalar *arg2, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_rvec_scale(PetscScalar *arg1, PetscScalar arg2, PetscInt n);
+PETSC_INTERN PetscErrorCode PCTFS_rvec_zero(PetscScalar*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_rvec_one(PetscScalar*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_rvec_set(PetscScalar*,PetscScalar,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_rvec_copy(PetscScalar*,PetscScalar*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_rvec_scale(PetscScalar*,PetscScalar,PetscInt);
 
-PETSC_INTERN vfp PCTFS_rvec_fct_addr(PetscInt type);
-PETSC_INTERN PetscErrorCode PCTFS_rvec_add(PetscScalar *arg1, PetscScalar *arg2, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_rvec_mult(PetscScalar *arg1, PetscScalar *arg2, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_rvec_max(PetscScalar *arg1, PetscScalar *arg2, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_rvec_max_abs(PetscScalar *arg1, PetscScalar *arg2, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_rvec_min(PetscScalar *arg1, PetscScalar *arg2, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_rvec_min_abs(PetscScalar *arg1, PetscScalar *arg2, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_vec_exists(PetscScalar *arg1, PetscScalar *arg2, PetscInt n);
+PETSC_INTERN vfp            PCTFS_rvec_fct_addr(PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_rvec_add(PetscScalar*,PetscScalar*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_rvec_mult(PetscScalar*,PetscScalar*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_rvec_max(PetscScalar*,PetscScalar*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_rvec_max_abs(PetscScalar*,PetscScalar*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_rvec_min(PetscScalar*,PetscScalar*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_rvec_min_abs(PetscScalar*,PetscScalar*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_vec_exists(PetscScalar*,PetscScalar*,PetscInt);
 
 /***********************************gs.h***************************************
 
@@ -232,15 +230,13 @@ Last Modification:
 ************************************gs.h**************************************/
 
 typedef struct gather_scatter_id *PCTFS_gs_ADT;
-/*typedef PetscErrorCode (*Rbfp)(PetscScalar *, PetscScalar *, PetscInt len);*/
 
-PETSC_INTERN PCTFS_gs_ADT PCTFS_gs_init(PetscInt *elms, PetscInt nel, PetscInt level);
-PETSC_INTERN PetscErrorCode   PCTFS_gs_gop_vec(PCTFS_gs_ADT PCTFS_gs_handle, PetscScalar *vals, const char *op, PetscInt step);
-/* PETSC_INTERN PetscErrorCode   PCTFS_gs_gop_binary(PCTFS_gs_ADT gs, PetscScalar *vals, Rbfp fct); */
-PETSC_INTERN PetscErrorCode   PCTFS_gs_gop_hc(PCTFS_gs_ADT PCTFS_gs_handle, PetscScalar *vals, const char *op, PetscInt dim);
-PETSC_INTERN PetscErrorCode   PCTFS_gs_free(PCTFS_gs_ADT PCTFS_gs_handle);
-PETSC_INTERN PetscErrorCode   PCTFS_gs_init_msg_buf_sz(PetscInt buf_size);
-PETSC_INTERN PetscErrorCode   PCTFS_gs_init_vec_sz(PetscInt size);
+PETSC_INTERN PCTFS_gs_ADT   PCTFS_gs_init(PetscInt*,PetscInt,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_gs_gop_vec(PCTFS_gs_ADT,PetscScalar*,const char*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_gs_gop_hc(PCTFS_gs_ADT,PetscScalar*,const char*,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_gs_free(PCTFS_gs_ADT);
+PETSC_INTERN PetscErrorCode PCTFS_gs_init_msg_buf_sz(PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_gs_init_vec_sz(PetscInt);
 
 /*************************************xxt.h************************************
 Module Name: xxt
@@ -281,7 +277,7 @@ Input : pointer to ADT.
 Description: This function frees the storage associated with an xxt handle
 Usage: XXT_free(xxt_handle);
 **************************************xxt.h***********************************/
-PETSC_INTERN PetscInt XXT_free(xxt_ADT xxt_handle);
+PETSC_INTERN PetscInt XXT_free(xxt_ADT);
 
 
 /*************************************xxt.h************************************
@@ -313,12 +309,12 @@ ML beliefs/usage: move this to to ML_XXT_factor routine
 
 Usage:
 **************************************xxt.h***********************************/
-PETSC_INTERN PetscInt XXT_factor(xxt_ADT xxt_handle,     /* prev. allocated xxt  handle */
-                           PetscInt *local2global, /* global column mapping       */
-                           PetscInt n,             /* local num rows              */
-                           PetscInt m,             /* local num cols              */
-                           PetscErrorCode (*matvec)(void*,PetscScalar*,PetscScalar*),    /* b_loc=A_local.x_loc         */
-                           void *grid_data);       /* grid data for matvec        */
+PETSC_INTERN PetscInt XXT_factor(xxt_ADT,     /* prev. allocated xxt  handle */
+                           PetscInt*, /* global column mapping       */
+                           PetscInt,             /* local num rows              */
+                           PetscInt,             /* local num cols              */
+                           PetscErrorCode (*)(void*,PetscScalar*,PetscScalar*),    /* b_loc=A_local.x_loc         */
+                           void*);       /* grid data for matvec        */
 
 
 /*************************************xxt.h************************************
@@ -333,14 +329,14 @@ XXT_solve(xxt_handle, double *x, double *b)
 XXT_solve(xxt_handle, double *x, NULL)
 assumes x has been initialized to be b
 **************************************xxt.h***********************************/
-PETSC_INTERN PetscInt XXT_solve(xxt_ADT xxt_handle, PetscScalar *x, PetscScalar *b);
+PETSC_INTERN PetscInt XXT_solve(xxt_ADT,PetscScalar*,PetscScalar*);
 
 /*************************************xxt.h************************************
 Function: XXT_stats
 
 Input : handle
 **************************************xxt.h***********************************/
-PETSC_INTERN PetscInt XXT_stats(xxt_ADT xxt_handle);
+PETSC_INTERN PetscInt XXT_stats(xxt_ADT);
 
 
 /*************************************xxt.h************************************
@@ -394,7 +390,7 @@ Input : pointer to ADT.
 Description: This function frees the storage associated with an xyt handle
 Usage: XYT_free(xyt_handle);
 **************************************xyt.h***********************************/
-PETSC_INTERN PetscInt XYT_free(xyt_ADT xyt_handle);
+PETSC_INTERN PetscInt XYT_free(xyt_ADT);
 
 
 /*************************************xyt.h************************************
@@ -427,12 +423,12 @@ ML beliefs/usage: move this to to ML_XYT_factor routine
 
 Usage:
 **************************************xyt.h***********************************/
-PETSC_INTERN PetscInt XYT_factor(xyt_ADT xyt_handle,     /* prev. allocated xyt  handle */
-                           PetscInt *local2global, /* global column mapping       */
-                           PetscInt n,             /* local num rows              */
-                           PetscInt m,             /* local num cols              */
-                           PetscErrorCode (*matvec)(void*,PetscScalar*,PetscScalar*), /* b_loc=A_local.x_loc         */
-                           void *grid_data);       /* grid data for matvec        */
+PETSC_INTERN PetscInt XYT_factor(xyt_ADT,     /* prev. allocated xyt  handle */
+                           PetscInt*, /* global column mapping       */
+                           PetscInt,             /* local num rows              */
+                           PetscInt,             /* local num cols              */
+                           PetscErrorCode (*)(void*,PetscScalar*,PetscScalar*), /* b_loc=A_local.x_loc         */
+                           void*);       /* grid data for matvec        */
 
 
 /*************************************xyt.h************************************
@@ -444,7 +440,7 @@ Return:
 Description: This function performs x = E^-1.b
 Usage: XYT_solve(xyt_handle, double *x, double *b)
 **************************************xyt.h***********************************/
-PETSC_INTERN PetscInt XYT_solve(xyt_ADT xyt_handle, PetscScalar *x, PetscScalar *b);
+PETSC_INTERN PetscInt XYT_solve(xyt_ADT,PetscScalar*,PetscScalar*);
 
 
 /*************************************xyt.h************************************
@@ -452,7 +448,7 @@ Function: XYT_stats
 
 Input : handle
 **************************************xyt.h***********************************/
-PETSC_INTERN PetscInt XYT_stats(xyt_ADT xyt_handle);
+PETSC_INTERN PetscInt XYT_stats(xyt_ADT);
 
 
 /********************************bit_mask.h************************************
@@ -469,12 +465,12 @@ Providence, RI 02912
 Last Modification:
 11.21.97
 *********************************bit_mask.h***********************************/
-PETSC_INTERN PetscInt PCTFS_div_ceil(PetscInt numin, PetscInt denom);
-PETSC_INTERN PetscErrorCode PCTFS_set_bit_mask(PetscInt *bm, PetscInt len, PetscInt val);
-PETSC_INTERN PetscInt PCTFS_len_bit_mask(PetscInt num_items);
-PETSC_INTERN PetscInt PCTFS_ct_bits(char *ptr, PetscInt n);
-PETSC_INTERN PetscErrorCode PCTFS_bm_to_proc(char *ptr, PetscInt p_mask, PetscInt *msg_list);
-PETSC_INTERN PetscInt PCTFS_len_buf(PetscInt item_size, PetscInt num_items);
+PETSC_INTERN PetscInt       PCTFS_div_ceil(PetscInt,PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_set_bit_mask(PetscInt*,PetscInt,PetscInt);
+PETSC_INTERN PetscInt       PCTFS_len_bit_mask(PetscInt);
+PETSC_INTERN PetscInt       PCTFS_ct_bits(char*, PetscInt);
+PETSC_INTERN PetscErrorCode PCTFS_bm_to_proc(char*,PetscInt,PetscInt*);
+PETSC_INTERN PetscInt       PCTFS_len_buf(PetscInt, PetscInt);
 
 #endif
 
