@@ -4,6 +4,7 @@
 PETSC_EXTERN PetscErrorCode PCCreate_Jacobi(PC);
 PETSC_EXTERN PetscErrorCode PCCreate_BJacobi(PC);
 PETSC_EXTERN PetscErrorCode PCCreate_PBJacobi(PC);
+PETSC_EXTERN PetscErrorCode PCCreate_VPBJacobi(PC);
 PETSC_EXTERN PetscErrorCode PCCreate_ILU(PC);
 PETSC_EXTERN PetscErrorCode PCCreate_None(PC);
 PETSC_EXTERN PetscErrorCode PCCreate_LU(PC);
@@ -28,6 +29,9 @@ PETSC_EXTERN PetscErrorCode PCCreate_Redistribute(PC);
 PETSC_EXTERN PetscErrorCode PCCreate_SVD(PC);
 PETSC_EXTERN PetscErrorCode PCCreate_GAMG(PC);
 PETSC_EXTERN PetscErrorCode PCCreate_Kaczmarz(PC);
+PETSC_EXTERN PetscErrorCode PCCreate_Telescope(PC);
+PETSC_EXTERN PetscErrorCode PCCreate_Patch(PC);
+PETSC_EXTERN PetscErrorCode PCCreate_LMVM(PC);
 
 #if defined(PETSC_HAVE_ML)
 PETSC_EXTERN PetscErrorCode PCCreate_ML(PC);
@@ -44,19 +48,16 @@ PETSC_EXTERN PetscErrorCode PCCreate_SysPFMG(PC);
 #if !defined(PETSC_USE_COMPLEX)
 PETSC_EXTERN PetscErrorCode PCCreate_TFS(PC);
 #endif
-#if defined(PETSC_HAVE_CUSP)
-PETSC_EXTERN PetscErrorCode PCCreate_SACUSP(PC);
-PETSC_EXTERN PetscErrorCode PCCreate_SACUSPPoly(PC);
-PETSC_EXTERN PetscErrorCode PCCreate_BiCGStabCUSP(PC);
-PETSC_EXTERN PetscErrorCode PCCreate_AINVCUSP(PC);
+#if defined(PETSC_HAVE_VIENNACL)
+PETSC_EXTERN PetscErrorCode PCCreate_CHOWILUVIENNACL(PC);
+PETSC_EXTERN PetscErrorCode PCCreate_ROWSCALINGVIENNACL(PC);
+PETSC_EXTERN PetscErrorCode PCCreate_SAVIENNACL(PC);
 #endif
 #if defined(PETSC_HAVE_PARMS)
 PETSC_EXTERN PetscErrorCode PCCreate_PARMS(PC);
 #endif
 PETSC_EXTERN PetscErrorCode PCCreate_BDDC(PC);
 
-#undef __FUNCT__
-#define __FUNCT__ "PCRegisterAll"
 /*@C
    PCRegisterAll - Registers all of the preconditioners in the PC package.
 
@@ -82,6 +83,7 @@ PetscErrorCode  PCRegisterAll(void)
   ierr = PCRegister(PCNONE         ,PCCreate_None);CHKERRQ(ierr);
   ierr = PCRegister(PCJACOBI       ,PCCreate_Jacobi);CHKERRQ(ierr);
   ierr = PCRegister(PCPBJACOBI     ,PCCreate_PBJacobi);CHKERRQ(ierr);
+  ierr = PCRegister(PCVPBJACOBI    ,PCCreate_VPBJacobi);CHKERRQ(ierr);
   ierr = PCRegister(PCBJACOBI      ,PCCreate_BJacobi);CHKERRQ(ierr);
   ierr = PCRegister(PCSOR          ,PCCreate_SOR);CHKERRQ(ierr);
   ierr = PCRegister(PCLU           ,PCCreate_LU);CHKERRQ(ierr);
@@ -107,6 +109,8 @@ PetscErrorCode  PCRegisterAll(void)
   ierr = PCRegister(PCSVD          ,PCCreate_SVD);CHKERRQ(ierr);
   ierr = PCRegister(PCGAMG         ,PCCreate_GAMG);CHKERRQ(ierr);
   ierr = PCRegister(PCKACZMARZ     ,PCCreate_Kaczmarz);CHKERRQ(ierr);
+  ierr = PCRegister(PCTELESCOPE    ,PCCreate_Telescope);CHKERRQ(ierr);
+  ierr = PCRegister(PCPATCH        ,PCCreate_Patch);CHKERRQ(ierr);
 #if defined(PETSC_HAVE_ML)
   ierr = PCRegister(PCML           ,PCCreate_ML);CHKERRQ(ierr);
 #endif
@@ -121,15 +125,15 @@ PetscErrorCode  PCRegisterAll(void)
 #if !defined(PETSC_USE_COMPLEX)
   ierr = PCRegister(PCTFS          ,PCCreate_TFS);CHKERRQ(ierr);
 #endif
-#if defined(PETSC_HAVE_CUSP)
-  ierr = PCRegister(PCSACUSP       ,PCCreate_SACUSP);CHKERRQ(ierr);
-  ierr = PCRegister(PCAINVCUSP     ,PCCreate_AINVCUSP);CHKERRQ(ierr);
-  ierr = PCRegister(PCBICGSTABCUSP ,PCCreate_BiCGStabCUSP);CHKERRQ(ierr);
-  ierr = PCRegister(PCSACUSPPOLY   ,PCCreate_SACUSPPoly);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_VIENNACL)
+  ierr = PCRegister(PCCHOWILUVIENNACL,PCCreate_CHOWILUVIENNACL);CHKERRQ(ierr);
+  ierr = PCRegister(PCROWSCALINGVIENNACL,PCCreate_ROWSCALINGVIENNACL);CHKERRQ(ierr);
+  ierr = PCRegister(PCSAVIENNACL   ,PCCreate_SAVIENNACL);CHKERRQ(ierr);
 #endif
 #if defined(PETSC_HAVE_PARMS)
   ierr = PCRegister(PCPARMS        ,PCCreate_PARMS);CHKERRQ(ierr);
 #endif
   ierr = PCRegister(PCBDDC         ,PCCreate_BDDC);CHKERRQ(ierr);
+  ierr = PCRegister(PCLMVM         ,PCCreate_LMVM);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
