@@ -12,8 +12,6 @@ PetscBool PCRegisterAllCalled = PETSC_FALSE;
 */
 PetscFunctionList PCList = 0;
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetType"
 /*@C
    PCSetType - Builds PC for a particular preconditioner type
 
@@ -89,8 +87,6 @@ PetscErrorCode  PCSetType(PC pc,PCType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGetType"
 /*@C
    PCGetType - Gets the PC method type and name (as a string) from the PC
    context.
@@ -121,8 +117,6 @@ PetscErrorCode  PCGetType(PC pc,PCType *type)
 
 extern PetscErrorCode PCGetDefaultType_Private(PC,const char*[]);
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetFromOptions"
 /*@
    PCSetFromOptions - Sets PC options from the options database.
    This routine must be called before PCSetUp() if the user is to be
@@ -179,14 +173,12 @@ PetscErrorCode  PCSetFromOptions(PC pc)
 
   skipoptions:
   /* process any options handlers added with PetscObjectAddOptionsHandler() */
-  ierr = PetscObjectProcessOptionsHandlers((PetscObject)pc);CHKERRQ(ierr);
+  ierr = PetscObjectProcessOptionsHandlers(PetscOptionsObject,(PetscObject)pc);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
   pc->setfromoptionscalled++;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetDM"
 /*@
    PCSetDM - Sets the DM that may be used by some preconditioners
 
@@ -194,10 +186,13 @@ PetscErrorCode  PCSetFromOptions(PC pc)
 
    Input Parameters:
 +  pc - the preconditioner context
--  dm - the dm
+-  dm - the dm, can be NULL
 
    Level: intermediate
 
+   Developer Notes:
+    The routines KSP/SNES/TSSetDM() require the dm to be non-NULL, but this one can be NULL since all it does is 
+    replace the current DM
 
 .seealso: PCGetDM(), KSPSetDM(), KSPGetDM()
 @*/
@@ -213,8 +208,6 @@ PetscErrorCode  PCSetDM(PC pc,DM dm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGetDM"
 /*@
    PCGetDM - Gets the DM that may be used by some preconditioners
 
@@ -239,8 +232,6 @@ PetscErrorCode  PCGetDM(PC pc,DM *dm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCSetApplicationContext"
 /*@
    PCSetApplicationContext - Sets the optional user-defined context for the linear solver.
 
@@ -264,8 +255,6 @@ PetscErrorCode  PCSetApplicationContext(PC pc,void *usrP)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCGetApplicationContext"
 /*@
    PCGetApplicationContext - Gets the user-defined context for the linear solver.
 
