@@ -7,13 +7,10 @@
  *
  *  PetscCDCreate
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDCreate"
 PetscErrorCode PetscCDCreate(PetscInt a_size, PetscCoarsenData **a_out)
 {
   PetscErrorCode   ierr;
   PetscCoarsenData *ail;
-  PetscInt         ii;
 
   PetscFunctionBegin;
   /* alocate pool, partially */
@@ -24,18 +21,14 @@ PetscErrorCode PetscCDCreate(PetscInt a_size, PetscCoarsenData **a_out)
   ail->chk_sz          = 0;
   /* allocate array */
   ail->size = a_size;
-  ierr      = PetscMalloc1(a_size, &ail->array);CHKERRQ(ierr);
-  for (ii=0; ii<a_size; ii++) ail->array[ii] = NULL;
+  ierr      = PetscCalloc1(a_size, &ail->array);CHKERRQ(ierr);
   ail->extra_nodes = NULL;
   ail->mat         = NULL;
-  /* ail->removedIS = NULL; */
   PetscFunctionReturn(0);
 }
 
 /* NPDestroy
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDDestroy"
 PetscErrorCode PetscCDDestroy(PetscCoarsenData *ail)
 {
   PetscErrorCode ierr;
@@ -51,34 +44,29 @@ PetscErrorCode PetscCDDestroy(PetscCoarsenData *ail)
   if (ail->pool_list.array) {
     ierr = PetscFree(ail->pool_list.array);CHKERRQ(ierr);
   }
-  /* if (ail->removedIS) { */
-  /*   ierr = ISDestroy(&ail->removedIS);CHKERRQ(ierr); */
-  /* } */
-  /* delete this (+array) */
   ierr = PetscFree(ail->array);CHKERRQ(ierr);
   /* delete this (+agg+pool array) */
   ierr = PetscFree(ail);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
 /* PetscCDSetChuckSize
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDSetChuckSize"
 PetscErrorCode PetscCDSetChuckSize(PetscCoarsenData *ail, PetscInt a_sz)
 {
   PetscFunctionBegin;
   ail->chk_sz = a_sz;
   PetscFunctionReturn(0);
 }
+
 /*  PetscCDGetNewNode
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDGetNewNode"
 PetscErrorCode PetscCDGetNewNode(PetscCoarsenData *ail, PetscCDIntNd **a_out, PetscInt a_id)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  *a_out = NULL;                /* squelch -Wmaybe-uninitialized */
   if (ail->extra_nodes) {
     PetscCDIntNd *node = ail->extra_nodes;
     ail->extra_nodes = node->next;
@@ -108,53 +96,46 @@ PetscErrorCode PetscCDGetNewNode(PetscCoarsenData *ail, PetscCDIntNd **a_out, Pe
   PetscFunctionReturn(0);
 }
 
-/* PetscLLNSetID
+/* PetscCDIntNdSetID
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscLLNSetID"
-PetscErrorCode PetscLLNSetID(PetscCDIntNd *a_this, PetscInt a_id)
+PetscErrorCode PetscCDIntNdSetID(PetscCDIntNd *a_this, PetscInt a_id)
 {
   PetscFunctionBegin;
   a_this->gid = a_id;
   PetscFunctionReturn(0);
 }
-/* PetscLLNGetID
+
+/* PetscCDIntNdGetID
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscLLNGetID"
-PetscErrorCode PetscLLNGetID(const PetscCDIntNd *a_this, PetscInt *a_gid)
+PetscErrorCode PetscCDIntNdGetID(const PetscCDIntNd *a_this, PetscInt *a_gid)
 {
   PetscFunctionBegin;
   *a_gid = a_this->gid;
   PetscFunctionReturn(0);
 }
+
 /* PetscCDGetHeadPos
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDGetHeadPos"
-PetscErrorCode PetscCDGetHeadPos(const PetscCoarsenData *ail, PetscInt a_idx, PetscCDPos *pos)
+PetscErrorCode PetscCDGetHeadPos(const PetscCoarsenData *ail, PetscInt a_idx, PetscCDIntNd **pos)
 {
   PetscFunctionBegin;
-  if (a_idx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER,"a_idx >= ail->size: a_idx=%d.",a_idx);
+  if (a_idx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_PLIB,"a_idx >= ail->size: a_idx=%D.",a_idx);
   *pos = ail->array[a_idx];
   PetscFunctionReturn(0);
 }
+
 /* PetscCDGetNextPos
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDGetNextPos"
-PetscErrorCode PetscCDGetNextPos(const PetscCoarsenData *ail, PetscInt l_idx, PetscCDPos *pos)
+PetscErrorCode PetscCDGetNextPos(const PetscCoarsenData *ail, PetscInt l_idx, PetscCDIntNd **pos)
 {
   PetscFunctionBegin;
-  if (!(*pos)) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_USER,"NULL input position.");
+  if (!(*pos)) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"NULL input position.");
   *pos = (*pos)->next;
   PetscFunctionReturn(0);
 }
 
 /* PetscCDAppendID
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDAppendID"
 PetscErrorCode PetscCDAppendID(PetscCoarsenData *ail, PetscInt a_idx, PetscInt a_id)
 {
   PetscErrorCode ierr;
@@ -162,7 +143,7 @@ PetscErrorCode PetscCDAppendID(PetscCoarsenData *ail, PetscInt a_idx, PetscInt a
 
   PetscFunctionBegin;
   ierr = PetscCDGetNewNode(ail, &n, a_id);CHKERRQ(ierr);
-  if (a_idx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER,"Index %d out of range.",a_idx);
+  if (a_idx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Index %D out of range.",a_idx);
   if (!(n2=ail->array[a_idx])) ail->array[a_idx] = n;
   else {
     do {
@@ -177,16 +158,15 @@ PetscErrorCode PetscCDAppendID(PetscCoarsenData *ail, PetscInt a_idx, PetscInt a
   }
   PetscFunctionReturn(0);
 }
+
 /* PetscCDAppendNode
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDAppendNode"
 PetscErrorCode PetscCDAppendNode(PetscCoarsenData *ail, PetscInt a_idx,  PetscCDIntNd *a_n)
 {
   PetscCDIntNd *n2;
 
   PetscFunctionBegin;
-  if (a_idx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER,"Index %d out of range.",a_idx);
+  if (a_idx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Index %D out of range.",a_idx);
   if (!(n2=ail->array[a_idx])) ail->array[a_idx] = a_n;
   else {
     do {
@@ -204,14 +184,12 @@ PetscErrorCode PetscCDAppendNode(PetscCoarsenData *ail, PetscInt a_idx,  PetscCD
 
 /* PetscCDRemoveNextNode: a_last->next, this exposes single linked list structure to API
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDRemoveNextNode"
 PetscErrorCode PetscCDRemoveNextNode(PetscCoarsenData *ail, PetscInt a_idx,  PetscCDIntNd *a_last)
 {
   PetscCDIntNd *del;
 
   PetscFunctionBegin;
-  if (a_idx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER,"Index %d out of range.",a_idx);
+  if (a_idx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Index %D out of range.",a_idx);
   if (!a_last->next) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"a_last should have a next");
   del          = a_last->next;
   a_last->next = del->next;
@@ -222,8 +200,6 @@ PetscErrorCode PetscCDRemoveNextNode(PetscCoarsenData *ail, PetscInt a_idx,  Pet
 
 /* PetscCDPrint
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDPrint"
 PetscErrorCode PetscCDPrint(const PetscCoarsenData *ail, MPI_Comm comm)
 {
   PetscErrorCode ierr;
@@ -236,26 +212,25 @@ PetscErrorCode PetscCDPrint(const PetscCoarsenData *ail, MPI_Comm comm)
   for (ii=0; ii<ail->size; ii++) {
     kk = 0;
     n  = ail->array[ii];
-    if (n) PetscPrintf(comm,"[%d]%s list %d:\n",rank,__FUNCT__,ii);
+    if (n) {ierr = PetscPrintf(comm,"[%d]%s list %d:\n",rank,PETSC_FUNCTION_NAME,ii);CHKERRQ(ierr);}
     while (n) {
-      PetscPrintf(comm,"\t[%d] %d) id %d\n",rank,++kk,n->gid);
+      PetscPrintf(comm,"\t[%d] %D) id %D\n",rank,++kk,n->gid);
       n = n->next;
     }
   }
   PetscFunctionReturn(0);
 }
+
 /* PetscCDAppendRemove
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDAppendRemove"
 PetscErrorCode PetscCDAppendRemove(PetscCoarsenData *ail, PetscInt a_destidx, PetscInt a_srcidx)
 {
   PetscCDIntNd *n;
 
   PetscFunctionBegin;
-  if (a_srcidx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER,"Index %d out of range.",a_srcidx);
-  if (a_destidx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER,"Index %d out of range.",a_destidx);
-  if (a_destidx==a_srcidx) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER,"a_destidx==a_srcidx %d.",a_destidx);
+  if (a_srcidx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Index %D out of range.",a_srcidx);
+  if (a_destidx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Index %D out of range.",a_destidx);
+  if (a_destidx==a_srcidx) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_PLIB,"a_destidx==a_srcidx %D.",a_destidx);
   n = ail->array[a_destidx];
   if (!n) ail->array[a_destidx] = ail->array[a_srcidx];
   else {
@@ -273,14 +248,12 @@ PetscErrorCode PetscCDAppendRemove(PetscCoarsenData *ail, PetscInt a_destidx, Pe
 
 /* PetscCDRemoveAll
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDRemoveAll"
 PetscErrorCode PetscCDRemoveAll(PetscCoarsenData *ail, PetscInt a_idx)
 {
   PetscCDIntNd *rem,*n1;
 
   PetscFunctionBegin;
-  if (a_idx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER,"Index %d out of range.",a_idx);
+  if (a_idx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Index %D out of range.",a_idx);
   rem               = ail->array[a_idx];
   ail->array[a_idx] = NULL;
   if (!(n1=ail->extra_nodes)) ail->extra_nodes = rem;
@@ -293,15 +266,13 @@ PetscErrorCode PetscCDRemoveAll(PetscCoarsenData *ail, PetscInt a_idx)
 
 /* PetscCDSizeAt
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDSizeAt"
 PetscErrorCode PetscCDSizeAt(const PetscCoarsenData *ail, PetscInt a_idx, PetscInt *a_sz)
 {
   PetscCDIntNd *n1;
   PetscInt     sz = 0;
 
   PetscFunctionBegin;
-  if (a_idx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER,"Index %d out of range.",a_idx);
+  if (a_idx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Index %D out of range.",a_idx);
   n1 = ail->array[a_idx];
   while (n1) {
     n1 = n1->next;
@@ -313,20 +284,16 @@ PetscErrorCode PetscCDSizeAt(const PetscCoarsenData *ail, PetscInt a_idx, PetscI
 
 /* PetscCDEmptyAt
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDEmptyAt"
 PetscErrorCode PetscCDEmptyAt(const PetscCoarsenData *ail, PetscInt a_idx, PetscBool *a_e)
 {
   PetscFunctionBegin;
-  if (a_idx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_USER,"Index %d out of range.",a_idx);
+  if (a_idx>=ail->size) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Index %D out of range.",a_idx);
   *a_e = (PetscBool)(ail->array[a_idx]==NULL);
   PetscFunctionReturn(0);
 }
 
 /* PetscCDGetMIS
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDGetMIS"
 PetscErrorCode PetscCDGetMIS(PetscCoarsenData *ail, IS *a_mis)
 {
   PetscErrorCode ierr;
@@ -347,10 +314,9 @@ PetscErrorCode PetscCDGetMIS(PetscCoarsenData *ail, IS *a_mis)
   ierr = ISCreateGeneral(PETSC_COMM_SELF, kk, permute, PETSC_OWN_POINTER, a_mis);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
 /* PetscCDGetMat
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDGetMat"
 PetscErrorCode PetscCDGetMat(const PetscCoarsenData *ail, Mat *a_mat)
 {
   PetscFunctionBegin;
@@ -360,8 +326,6 @@ PetscErrorCode PetscCDGetMat(const PetscCoarsenData *ail, Mat *a_mat)
 
 /* PetscCDSetMat
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDSetMat"
 PetscErrorCode PetscCDSetMat(PetscCoarsenData *ail, Mat a_mat)
 {
   PetscFunctionBegin;
@@ -369,66 +333,63 @@ PetscErrorCode PetscCDSetMat(PetscCoarsenData *ail, Mat a_mat)
   PetscFunctionReturn(0);
 }
 
-
 /* PetscCDGetASMBlocks
  */
-#undef __FUNCT__
-#define __FUNCT__ "PetscCDGetASMBlocks"
-PetscErrorCode PetscCDGetASMBlocks(const PetscCoarsenData *ail, const PetscInt a_bs, PetscInt *a_sz, IS **a_local_is)
+PetscErrorCode PetscCDGetASMBlocks(const PetscCoarsenData *ail, const PetscInt a_bs, Mat mat, PetscInt *a_sz, IS **a_local_is)
 {
   PetscErrorCode ierr;
   PetscCDIntNd   *n;
-  PetscInt       lsz,ii,kk,*idxs,jj;
-  IS             *is_loc;
+  PetscInt       lsz,ii,kk,*idxs,jj,s,e,gid;
+  IS             *is_loc,is_bcs;
 
   PetscFunctionBegin;
   for (ii=kk=0; ii<ail->size; ii++) {
     if (ail->array[ii]) kk++;
   }
-  *a_sz = kk; /* out */
-
-  ierr        = PetscMalloc1(kk, &is_loc);CHKERRQ(ierr);
-  *a_local_is = is_loc; /* out */
+  /* count BCs */
+  ierr = MatGetOwnershipRange(mat, &s, &e);CHKERRQ(ierr);
+  for (gid=s,lsz=0; gid<e; gid++) {
+    ierr = MatGetRow(mat,gid,&jj,0,0);CHKERRQ(ierr);
+    if (jj<2) lsz++;
+    ierr = MatRestoreRow(mat,gid,&jj,0,0);CHKERRQ(ierr);
+  }
+  if (lsz) {
+    ierr = PetscMalloc1(a_bs*lsz, &idxs);CHKERRQ(ierr);
+    for (gid=s,lsz=0; gid<e; gid++) {
+      ierr = MatGetRow(mat,gid,&jj,0,0);CHKERRQ(ierr);
+      if (jj<2) {
+        for (jj=0; jj<a_bs; lsz++,jj++) idxs[lsz] = a_bs*gid + jj;
+      }
+      ierr = MatRestoreRow(mat,gid,&jj,0,0);CHKERRQ(ierr);
+    }
+    ierr = ISCreateGeneral(PETSC_COMM_SELF, lsz, idxs, PETSC_OWN_POINTER, &is_bcs);CHKERRQ(ierr);
+    *a_sz = kk + 1; /* out */
+  } else {
+    is_bcs=0;
+    *a_sz = kk; /* out */
+  }
+  ierr = PetscMalloc1(*a_sz, &is_loc);CHKERRQ(ierr);
 
   for (ii=kk=0; ii<ail->size; ii++) {
     for (lsz=0, n=ail->array[ii]; n; lsz++, n=n->next) /* void */;
     if (lsz) {
       ierr = PetscMalloc1(a_bs*lsz, &idxs);CHKERRQ(ierr);
       for (lsz = 0, n=ail->array[ii]; n; n = n->next) {
-        PetscInt gid;
-        ierr = PetscLLNGetID(n, &gid);CHKERRQ(ierr);
+        ierr = PetscCDIntNdGetID(n, &gid);CHKERRQ(ierr);
         for (jj=0; jj<a_bs; lsz++,jj++) idxs[lsz] = a_bs*gid + jj;
       }
       ierr = ISCreateGeneral(PETSC_COMM_SELF, lsz, idxs, PETSC_OWN_POINTER, &is_loc[kk++]);CHKERRQ(ierr);
     }
   }
+  if (is_bcs) {
+    is_loc[kk++] = is_bcs;
+  }
   if (*a_sz != kk) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"*a_sz %D != kk %D",*a_sz,kk);
+  *a_local_is = is_loc; /* out */
+
   PetscFunctionReturn(0);
 }
 
-
-/* PetscCDSetRemovedIS
- */
-/* PetscErrorCode PetscCDSetRemovedIS(PetscCoarsenData *ail, MPI_Comm comm, const PetscInt a_sz, PetscInt a_ids[]) */
-/* { */
-/*   PetscErrorCode ierr; */
-
-/*   if (ail->removedIS) { */
-/*     ierr = ISDestroy(&ail->removedIS);CHKERRQ(ierr); */
-/*   } */
-/*   ierr = ISCreateGeneral(comm, a_sz, a_ids, PETSC_COPY_VALUES, &ail->removedIS);CHKERRQ(ierr); */
-/*   PetscFunctionReturn(0); */
-/* } */
-
-/* PetscCDGetRemovedIS
- */
-/* PetscErrorCode PetscCDGetRemovedIS(PetscCoarsenData *ail, IS *a_is) */
-/* { */
-/*   *a_is = ail->removedIS; */
-/*   ail->removedIS = NULL; /\* hack to relinquish ownership *\/ */
-/*   PetscFunctionReturn(0); */
-/* } */
-
 /* ********************************************************************** */
 /* edge for priority queue */
 typedef struct edge_tag {
@@ -453,8 +414,6 @@ static int gamg_hem_compare(const void *a, const void *b)
    Output Parameter:
    . a_locals_llist - array of list of local nodes rooted at local node
 */
-#undef __FUNCT__
-#define __FUNCT__ "heavyEdgeMatchAgg"
 static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_locals_llist)
 {
   PetscErrorCode   ierr;
@@ -515,7 +474,10 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
       mpimat = (Mat_MPIAIJ*)cMat->data;
       matA   = (Mat_SeqAIJ*)mpimat->A->data;
       matB   = (Mat_SeqAIJ*)mpimat->B->data;
-      if (!matB->compressedrow.use) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"matB must have compressed row usage");
+      if (!matB->compressedrow.use) {
+        /* force construction of compressed row data structure since code below requires it */
+        ierr = MatCheckCompressedRow(mpimat->B,matB->nonzerorowcnt,&matB->compressedrow,matB->i,mpimat->B->rmap->n,-1.0);CHKERRQ(ierr);
+      }
     } else {
       matA = (Mat_SeqAIJ*)cMat->data;
     }
@@ -567,37 +529,18 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
     for (kk=0; kk<nloc; kk++) lid_cprowID[kk] = -1;
     /* set index into compressed row 'lid_cprowID' */
     if (matB) {
-      ii = matB->compressedrow.i;
       for (ix=0; ix<matB->compressedrow.nrows; ix++) {
         lid_cprowID[matB->compressedrow.rindex[ix]] = ix;
       }
     }
 
-    /* get removed IS, use '' */
-    /* if (iter==1) { */
-    /*   PetscInt *lid_rem,idx; */
-    /*   ierr = PetscMalloc1(nloc, &lid_rem);CHKERRQ(ierr); */
-    /*   for (kk=idx=0;kk<nloc;kk++) { */
-    /*     PetscInt nn,lid=kk; */
-    /*     ii = matA->i; nn = ii[lid+1] - ii[lid]; */
-    /*     if ((ix=lid_cprowID[lid]) != -1) { /\* if I have any ghost neighbors *\/ */
-    /*       ii = matB->compressedrow.i; */
-    /*       nn += ii[ix+1] - ii[ix]; */
-    /*     } */
-    /*     if (nn < 2) { */
-    /*       lid_rem[idx++] = kk + my0; */
-    /*     } */
-    /*   } */
-    /*   ierr = PetscCDSetRemovedIS(agg_llists, comm, idx, lid_rem);CHKERRQ(ierr); */
-    /*   ierr = PetscFree(lid_rem);CHKERRQ(ierr); */
-    /* } */
-
     /* compute 'locMaxEdge' & 'locMaxPE', and create list of edges, count edges' */
     for (nEdges=0,kk=0,gid=my0; kk<nloc; kk++,gid++) {
       PetscReal   max_e = 0., tt;
       PetscScalar vval;
       PetscInt    lid   = kk;
       PetscMPIInt max_pe=rank,pe;
+
       ii = matA->i; n = ii[lid+1] - ii[lid]; idx = matA->j + ii[lid];
       ap = matA->a + ii[lid];
       for (jj=0; jj<n; jj++) {
@@ -635,7 +578,7 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
 
       ierr = VecDuplicate(mpimat->lvec, &ghostMaxPE);CHKERRQ(ierr);
       ierr = VecScatterBegin(mpimat->Mvctx,locMaxPE,ghostMaxPE,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
-      ierr =   VecScatterEnd(mpimat->Mvctx,locMaxPE,ghostMaxPE,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
+      ierr = VecScatterEnd(mpimat->Mvctx,locMaxPE,ghostMaxPE,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
       ierr = VecGetArray(ghostMaxPE, &cpcol_max_pe);CHKERRQ(ierr);
     }
 
@@ -703,15 +646,15 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
 
         nactive_edges++;
         /* skip if I have a bigger edge someplace (lid_max_ew gets updated) */
-        if (PetscRealPart(lid_max_ew[lid0]) > e->weight + 1.e-12) continue;
+        if (PetscRealPart(lid_max_ew[lid0]) > e->weight + PETSC_SMALL) continue;
 
         if (cpid1 == -1) {
-          if (PetscRealPart(lid_max_ew[lid1]) > e->weight + 1.e-12) continue;
+          if (PetscRealPart(lid_max_ew[lid1]) > e->weight + PETSC_SMALL) continue;
         } else {
           /* see if edge might get matched on other proc */
           PetscReal g_max_e = PetscRealPart(cpcol_max_ew[cpid1]);
-          if (g_max_e > e->weight + 1.e-12) continue;
-          else if (e->weight > g_max_e - 1.e-12 && (PetscMPIInt)PetscRealPart(cpcol_max_pe[cpid1]) > rank) {
+          if (g_max_e > e->weight + PETSC_SMALL) continue;
+          else if (e->weight > g_max_e - PETSC_SMALL && (PetscMPIInt)PetscRealPart(cpcol_max_pe[cpid1]) > rank) {
             /* check for max_e == to this edge and larger processor that will deal with this */
             continue;
           }
@@ -729,7 +672,7 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
               if (cpcol_matched[lidj]) continue;
               ew = PetscRealPart(ap[jj]); max_e = PetscRealPart(cpcol_max_ew[lidj]);
               /* check for max_e == to this edge and larger processor that will deal with this */
-              if (ew > max_e - 1.e-12 && ew > PetscRealPart(lid_max_ew[lid0]) - 1.e-12 && (PetscMPIInt)PetscRealPart(cpcol_max_pe[lidj]) > rank) {
+              if (ew > max_e - PETSC_SMALL && ew > PetscRealPart(lid_max_ew[lid0]) - PETSC_SMALL && (PetscMPIInt)PetscRealPart(cpcol_max_pe[lidj]) > rank) {
                 isOK = PETSC_FALSE;
               }
             }
@@ -746,7 +689,7 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
                 if (cpcol_matched[lidj]) continue;
                 ew = PetscRealPart(ap[jj]); max_e = PetscRealPart(cpcol_max_ew[lidj]);
                 /* check for max_e == to this edge and larger processor that will deal with this */
-                if (ew > max_e - 1.e-12 && ew > PetscRealPart(lid_max_ew[lid1]) - 1.e-12 && (PetscMPIInt)PetscRealPart(cpcol_max_pe[lidj]) > rank) {
+                if (ew > max_e - PETSC_SMALL && ew > PetscRealPart(lid_max_ew[lid1]) - PETSC_SMALL && (PetscMPIInt)PetscRealPart(cpcol_max_pe[lidj]) > rank) {
                   isOK = PETSC_FALSE;
                 }
               }
@@ -778,11 +721,12 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
 
       /* deal with deleted ghost on first pass */
       if (size>1 && sub_it != n_sub_its-1) {
-        PetscCDPos pos;  PetscBool ise = PETSC_FALSE;
-        PetscInt   nSend1, **sbuffs1,nSend2;
 #define REQ_BF_SIZE 100
-        MPI_Request *sreqs2[REQ_BF_SIZE],*rreqs2[REQ_BF_SIZE];
-        MPI_Status status;
+        PetscCDIntNd *pos;
+        PetscBool    ise = PETSC_FALSE;
+        PetscInt     nSend1, **sbuffs1,nSend2;
+        MPI_Request  *sreqs2[REQ_BF_SIZE],*rreqs2[REQ_BF_SIZE];
+        MPI_Status   status;
 
         /* send request */
         for (proc=0,nSend1=0; proc<size; proc++) {
@@ -790,8 +734,6 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
           if (!ise) nSend1++;
         }
         ierr = PetscMalloc1(nSend1, &sbuffs1);CHKERRQ(ierr);
-        /* ierr = PetscMalloc4(nSend1, sbuffs1, nSend1, rbuffs1, nSend1, sreqs1, nSend1, rreqs1);CHKERRQ(ierr); */
-        /* PetscFree4(sbuffs1,rbuffs1,sreqs1,rreqs1); */
         for (proc=0,nSend1=0; proc<size; proc++) {
           /* count ghosts */
           ierr = PetscCDSizeAt(deleted_list,proc,&n);CHKERRQ(ierr);
@@ -801,7 +743,6 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
             MPI_Request *request;
             n   /= 2;
             ierr = PetscMalloc1(2 + 2*n + n*CHUNCK_SIZE + 2, &sbuff);CHKERRQ(ierr);
-            /* PetscMalloc4(2+2*n,sbuffs1[nSend1],n*CHUNCK_SIZE,rbuffs1[nSend1],1,rreqs2[nSend1],1,sreqs2[nSend1]); */
             /* save requests */
             sbuffs1[nSend1] = sbuff;
             request         = (MPI_Request*)sbuff;
@@ -811,10 +752,10 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
             ierr = PetscCDGetHeadPos(deleted_list,proc,&pos);CHKERRQ(ierr);
             while (pos) {
               PetscInt lid0, cpid, gid;
-              ierr  = PetscLLNGetID(pos, &cpid);CHKERRQ(ierr);
+              ierr  = PetscCDIntNdGetID(pos, &cpid);CHKERRQ(ierr);
               gid   = (PetscInt)PetscRealPart(cpcol_gid[cpid]);
               ierr  = PetscCDGetNextPos(deleted_list,proc,&pos);CHKERRQ(ierr);
-              ierr  = PetscLLNGetID(pos, &lid0);CHKERRQ(ierr);
+              ierr  = PetscCDIntNdGetID(pos, &lid0);CHKERRQ(ierr);
               ierr  = PetscCDGetNextPos(deleted_list,proc,&pos);CHKERRQ(ierr);
               *pt++ = gid; *pt++ = lid0;
             }
@@ -832,13 +773,14 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
         }
         /* recieve requests, send response, clear lists */
         kk     = nactive_edges;
-        ierr   = MPI_Allreduce(&kk,&nactive_edges,1,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr); /* not correct syncronization and global */
+        ierr   = MPIU_Allreduce(&kk,&nactive_edges,1,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr); /* not correct syncronization and global */
         nSend2 = 0;
         while (1) {
 #define BF_SZ 10000
           PetscMPIInt flag,count;
-          PetscInt rbuff[BF_SZ],*pt,*pt2,*pt3,count2,*sbuff,count3;
+          PetscInt    rbuff[BF_SZ],*pt,*pt2,*pt3,count2,*sbuff,count3;
           MPI_Request *request;
+
           ierr = MPI_Iprobe(MPI_ANY_SOURCE, tag1, comm, &flag, &status);CHKERRQ(ierr);
           if (!flag) break;
           ierr = MPI_Get_count(&status, MPIU_INT, &count);CHKERRQ(ierr);
@@ -851,21 +793,18 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
           n  = *pt++; kk = *pt++;
           while (n--) {
             PetscInt gid1=*pt++, lid1=gid1-my0; kk=*pt++;
-            if (lid_matched[lid1]) {
-              PetscPrintf(PETSC_COMM_SELF,"\t *** [%d]%s %d) ERROR recieved deleted gid %d, deleted by (lid) %d from proc %d\n",rank,__FUNCT__,sub_it,gid1,kk);
-              PetscSleep(1);
-            }
+            if (lid_matched[lid1]) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Recieved deleted gid %D, deleted by (lid) %D from proc %D\n",sub_it,gid1,kk);
             lid_matched[lid1] = PETSC_TRUE; /* keep track of what we've done this round */
             ierr              = PetscCDSizeAt(agg_llists, lid1, &kk);CHKERRQ(ierr);
             count2           += kk + 2;
             count3++; /* number of verts requested (n) */
           }
-          if (count2 > count3*CHUNCK_SIZE) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Irecv will be too small: %d",count2);
+          if (count2 > count3*CHUNCK_SIZE) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Irecv will be too small: %D",count2);
           /* send tag2 *[lid0, n, n*[gid] ] */
           ierr             = PetscMalloc(count2*sizeof(PetscInt) + sizeof(MPI_Request), &sbuff);CHKERRQ(ierr);
           request          = (MPI_Request*)sbuff;
           sreqs2[nSend2++] = request; /* cache request */
-          if (nSend2==REQ_BF_SIZE) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"buffer too small for requests: %d",nSend2);
+          if (nSend2==REQ_BF_SIZE) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"buffer too small for requests: %D",nSend2);
           pt2 = sbuff = (PetscInt*)(request+1);
           pt  = rbuff;
           n   = *pt++; kk = *pt++;
@@ -875,11 +814,10 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
             /* write [lid0, n, n*[gid] ] */
             *pt2++ = lid0;
             pt3    = pt2++; /* save pointer for later */
-            /* for (pos=PetscCDGetHeadPos(agg_llists,lid1) ; pos ; pos=PetscCDGetNextPos(agg_llists,lid1,pos)) { */
             ierr = PetscCDGetHeadPos(agg_llists,lid1,&pos);CHKERRQ(ierr);
             while (pos) {
               PetscInt gid;
-              ierr   = PetscLLNGetID(pos, &gid);CHKERRQ(ierr);
+              ierr   = PetscCDIntNdGetID(pos, &gid);CHKERRQ(ierr);
               ierr   = PetscCDGetNextPos(agg_llists,lid1,&pos);CHKERRQ(ierr);
               *pt2++ = gid;
             }
@@ -896,6 +834,7 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
           PetscMPIInt count;
           MPI_Request *request;
           PetscInt    *pt, *pt2;
+
           request = rreqs2[kk]; /* no need to free -- buffer is in 'sbuffs1' */
           ierr    = MPI_Wait(request, &status);CHKERRQ(ierr);
           ierr    = MPI_Get_count(&status, MPIU_INT, &count);CHKERRQ(ierr);
@@ -936,13 +875,12 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
         ierr = VecAssemblyBegin(locMaxPE);CHKERRQ(ierr);
         ierr = VecAssemblyEnd(locMaxPE);CHKERRQ(ierr);
         ierr = VecScatterBegin(mpimat->Mvctx,locMaxPE,ghostMaxEdge,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
-        ierr =   VecScatterEnd(mpimat->Mvctx,locMaxPE,ghostMaxEdge,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
+        ierr = VecScatterEnd(mpimat->Mvctx,locMaxPE,ghostMaxEdge,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
         ierr = VecGetArray(ghostMaxEdge, &cpcol_max_ew);CHKERRQ(ierr);
         ierr = VecGetLocalSize(mpimat->lvec, &n);CHKERRQ(ierr);
         for (kk=0; kk<n; kk++) {
           cpcol_matched[kk] = (PetscBool)(PetscRealPart(cpcol_max_ew[kk]) != 0.0);
         }
-
         ierr = VecRestoreArray(ghostMaxEdge, &cpcol_max_ew);CHKERRQ(ierr);
       } /* size > 1 */
 
@@ -952,6 +890,7 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
         PetscReal max_e = 0.,tt;
         PetscScalar vval;
         PetscInt lid = kk;
+
         if (lid_matched[lid]) vval = 0.;
         else {
           ii = matA->i; n = ii[lid+1] - ii[lid]; idx = matA->j + ii[lid];
@@ -981,7 +920,7 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
       if (size>1 && sub_it != n_sub_its-1) {
         /* compute 'cpcol_max_ew' */
         ierr = VecScatterBegin(mpimat->Mvctx,locMaxEdge,ghostMaxEdge,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
-        ierr =   VecScatterEnd(mpimat->Mvctx,locMaxEdge,ghostMaxEdge,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
+        ierr = VecScatterEnd(mpimat->Mvctx,locMaxEdge,ghostMaxEdge,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
         ierr = VecGetArray(ghostMaxEdge, &cpcol_max_ew);CHKERRQ(ierr);
         ierr = VecGetArray(locMaxEdge, &lid_max_ew);CHKERRQ(ierr);
 
@@ -991,6 +930,7 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
           PetscReal ew,v1_max_e,v0_max_e=PetscRealPart(lid_max_ew[lid]);
           PetscScalar vval;
           PetscMPIInt max_pe=rank,pe;
+
           if (lid_matched[lid]) vval = (PetscScalar)rank;
           else if ((ix=lid_cprowID[lid]) != -1) { /* if I have any ghost neighbors */
             ii  = matB->compressedrow.i; n = ii[ix+1] - ii[ix];
@@ -1001,7 +941,7 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
               if (cpcol_matched[lidj]) continue;
               ew = PetscRealPart(ap[jj]); v1_max_e = PetscRealPart(cpcol_max_ew[lidj]);
               /* get max pe that has a max_e == to this edge w */
-              if ((pe=cpcol_pe[idx[jj]]) > max_pe && ew > v1_max_e - 1.e-12 && ew > v0_max_e - 1.e-12) max_pe = pe;
+              if ((pe=cpcol_pe[idx[jj]]) > max_pe && ew > v1_max_e - PETSC_SMALL && ew > v0_max_e - PETSC_SMALL) max_pe = pe;
             }
             vval = (PetscScalar)max_pe;
           }
@@ -1011,7 +951,7 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
         ierr = VecAssemblyEnd(locMaxPE);CHKERRQ(ierr);
 
         ierr = VecScatterBegin(mpimat->Mvctx,locMaxPE,ghostMaxPE,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
-        ierr =   VecScatterEnd(mpimat->Mvctx,locMaxPE,ghostMaxPE,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
+        ierr = VecScatterEnd(mpimat->Mvctx,locMaxPE,ghostMaxPE,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
         ierr = VecGetArray(ghostMaxPE, &cpcol_max_pe);CHKERRQ(ierr);
         ierr = VecRestoreArray(locMaxEdge, &lid_max_ew);CHKERRQ(ierr);
       } /* deal with deleted ghost */
@@ -1074,9 +1014,9 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
 
   /* make fake matrix */
   if (size>1) {
-    Mat mat;
-    PetscCDPos pos;
-    PetscInt gid, NN, MM, jj = 0, mxsz = 0;
+    Mat          mat;
+    PetscCDIntNd *pos;
+    PetscInt     gid, NN, MM, jj = 0, mxsz = 0;
 
     for (kk=0; kk<nloc; kk++) {
       ierr = PetscCDSizeAt(agg_llists, kk, &jj);CHKERRQ(ierr);
@@ -1087,13 +1027,12 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
 
     ierr = MatCreateAIJ(comm, nloc, nloc,PETSC_DETERMINE, PETSC_DETERMINE,0, 0, mxsz, 0, &mat);CHKERRQ(ierr);
 
-    /* */
     for (kk=0,gid=my0; kk<nloc; kk++,gid++) {
       /* for (pos=PetscCDGetHeadPos(agg_llists,kk) ; pos ; pos=PetscCDGetNextPos(agg_llists,kk,pos)) { */
       ierr = PetscCDGetHeadPos(agg_llists,kk,&pos);CHKERRQ(ierr);
       while (pos) {
         PetscInt gid1;
-        ierr = PetscLLNGetID(pos, &gid1);CHKERRQ(ierr);
+        ierr = PetscCDIntNdGetID(pos, &gid1);CHKERRQ(ierr);
         ierr = PetscCDGetNextPos(agg_llists,kk,&pos);CHKERRQ(ierr);
 
         if (gid1 < my0 || gid1 >= my0+nloc) {
@@ -1103,7 +1042,6 @@ static PetscErrorCode heavyEdgeMatchAgg(IS perm,Mat a_Gmat,PetscCoarsenData **a_
     }
     ierr = MatAssemblyBegin(mat,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
     ierr = MatAssemblyEnd(mat,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-
     ierr = PetscCDSetMat(agg_llists, mat);CHKERRQ(ierr);
   }
 
@@ -1120,8 +1058,6 @@ typedef struct {
 /*
    HEM coarsen, simple greedy.
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatCoarsenApply_HEM"
 static PetscErrorCode MatCoarsenApply_HEM(MatCoarsen coarse)
 {
   /* MatCoarsen_HEM *HEM = (MatCoarsen_HEM*)coarse->subctx; */
@@ -1133,7 +1069,7 @@ static PetscErrorCode MatCoarsenApply_HEM(MatCoarsen coarse)
   if (!coarse->perm) {
     IS       perm;
     PetscInt n,m;
-    
+
     ierr = MatGetLocalSize(mat, &m, &n);CHKERRQ(ierr);
     ierr = ISCreateStride(PetscObjectComm((PetscObject)mat), m, 0, 1, &perm);CHKERRQ(ierr);
     ierr = heavyEdgeMatchAgg(perm, mat, &coarse->agg_lists);CHKERRQ(ierr);
@@ -1144,8 +1080,6 @@ static PetscErrorCode MatCoarsenApply_HEM(MatCoarsen coarse)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCoarsenView_HEM"
 static PetscErrorCode MatCoarsenView_HEM(MatCoarsen coarse,PetscViewer viewer)
 {
   /* MatCoarsen_HEM *HEM = (MatCoarsen_HEM*)coarse->subctx; */
@@ -1158,15 +1092,14 @@ static PetscErrorCode MatCoarsenView_HEM(MatCoarsen coarse,PetscViewer viewer)
   ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)coarse),&rank);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
+    ierr = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
     ierr = PetscViewerASCIISynchronizedPrintf(viewer,"  [%d] HEM aggregator\n",rank);CHKERRQ(ierr);
     ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPopSynchronized(viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCoarsenDestroy_HEM"
 static PetscErrorCode MatCoarsenDestroy_HEM(MatCoarsen coarse)
 {
   MatCoarsen_HEM *HEM = (MatCoarsen_HEM*)coarse->subctx;
@@ -1189,8 +1122,6 @@ static PetscErrorCode MatCoarsenDestroy_HEM(MatCoarsen coarse)
 
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCoarsenCreate_HEM"
 PETSC_EXTERN PetscErrorCode MatCoarsenCreate_HEM(MatCoarsen coarse)
 {
   PetscErrorCode ierr;
@@ -1202,6 +1133,5 @@ PETSC_EXTERN PetscErrorCode MatCoarsenCreate_HEM(MatCoarsen coarse)
   coarse->ops->apply   = MatCoarsenApply_HEM;
   coarse->ops->view    = MatCoarsenView_HEM;
   coarse->ops->destroy = MatCoarsenDestroy_HEM;
-  /* coarse->ops->setfromoptions = MatCoarsenSetFromOptions_HEM; */
   PetscFunctionReturn(0);
 }
