@@ -7,11 +7,11 @@
 #define matcoloringsettype_ matcoloringsettype
 #endif
 
-PETSC_EXTERN void PETSC_STDCALL matcoloringsettype_(MatColoring *mc,CHAR type PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len))
+PETSC_EXTERN void PETSC_STDCALL matcoloringsettype_(MatColoring *mc,char* type PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   char *t;
 
   FIXCHAR(type,len,t);
-  *ierr = MatColoringSetType(*mc,t);
+  *ierr = MatColoringSetType(*mc,t);if (*ierr) return;
   FREECHAR(type,t);
 }
