@@ -19,8 +19,6 @@ extern PetscErrorCode ComputeMatrix(DM,Mat);
 extern PetscErrorCode ComputeRHS(DM,Vec);
 extern PetscErrorCode ComputeRHSMatrix(PetscInt,PetscInt,Mat*);
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
   PetscErrorCode ierr;
@@ -30,16 +28,16 @@ int main(int argc,char **args)
   Mat            A,F,RHS,X,C1;
   MatFactorInfo  info;
   IS             perm,iperm;
-  PetscInt       dof =1,M=-8,m,n,nrhs;
+  PetscInt       dof =1,M=8,m,n,nrhs;
   PetscScalar    one = 1.0;
-  PetscReal      norm,tol=1.e-13;
+  PetscReal      norm,tol = 1000*PETSC_MACHINE_EPSILON;
   PetscBool      InplaceLU=PETSC_FALSE;
 
-  PetscInitialize(&argc,&args,(char*)0,help);
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
   if (size != 1) SETERRQ(PETSC_COMM_WORLD,1,"This is a uniprocessor example only\n");
-  ierr = PetscOptionsGetInt(NULL,"-dof",&dof,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetInt(NULL,"-M",&M,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-dof",&dof,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-M",&M,NULL);CHKERRQ(ierr);
 
   ierr = DMDACreate(PETSC_COMM_WORLD,&da);CHKERRQ(ierr);
   ierr = DMSetDimension(da,3);CHKERRQ(ierr);
@@ -50,6 +48,7 @@ int main(int argc,char **args)
   ierr = DMDASetDof(da,dof);CHKERRQ(ierr);
   ierr = DMDASetStencilWidth(da,1);CHKERRQ(ierr);
   ierr = DMDASetOwnershipRanges(da,NULL,NULL,NULL);CHKERRQ(ierr);
+  ierr = DMSetMatType(da,MATBAIJ);CHKERRQ(ierr);
   ierr = DMSetFromOptions(da);CHKERRQ(ierr);
   ierr = DMSetUp(da);CHKERRQ(ierr);
 
@@ -58,19 +57,17 @@ int main(int argc,char **args)
   ierr = VecDuplicate(b,&y);CHKERRQ(ierr);
   ierr = ComputeRHS(da,b);CHKERRQ(ierr);
   ierr = VecSet(y,one);CHKERRQ(ierr);
-  ierr = DMSetMatType(da,MATBAIJ);CHKERRQ(ierr);
   ierr = DMCreateMatrix(da,&A);CHKERRQ(ierr);
   ierr = ComputeMatrix(da,A);CHKERRQ(ierr);
   ierr = MatGetSize(A,&m,&n);CHKERRQ(ierr);
   nrhs = 2;
-  ierr = PetscOptionsGetInt(NULL,"-nrhs",&nrhs,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-nrhs",&nrhs,NULL);CHKERRQ(ierr);
   ierr = ComputeRHSMatrix(m,nrhs,&RHS);CHKERRQ(ierr);
   ierr = MatDuplicate(RHS,MAT_DO_NOT_COPY_VALUES,&X);CHKERRQ(ierr);
 
   ierr = MatGetOrdering(A,MATORDERINGND,&perm,&iperm);CHKERRQ(ierr);
 
-
-  ierr = PetscOptionsGetBool(NULL,"-inplacelu",&InplaceLU,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-inplacelu",&InplaceLU,NULL);CHKERRQ(ierr);
   ierr = MatFactorInfoInitialize(&info);CHKERRQ(ierr);
   if (!InplaceLU) {
     ierr      = MatGetFactor(A,MATSOLVERPETSC,MAT_FACTOR_LU,&F);CHKERRQ(ierr);
@@ -79,24 +76,6 @@ int main(int argc,char **args)
     ierr      = MatLUFactorNumeric(F,A,&info);CHKERRQ(ierr);
   } else { /* Test inplace factorization */
     ierr = MatDuplicate(A,MAT_COPY_VALUES,&F);CHKERRQ(ierr);
-    /* or create F without DMDA
-    MatType     type;
-    PetscInt          i,ncols;
-    const PetscInt    *cols;
-    const PetscScalar *vals;
-    ierr = MatGetSize(A,&m,&n);CHKERRQ(ierr);
-    ierr = MatGetType(A,&type);CHKERRQ(ierr);
-    ierr = MatCreate(PetscObjectComm((PetscObject)A),&F);CHKERRQ(ierr);
-    ierr = MatSetSizes(F,PETSC_DECIDE,PETSC_DECIDE,m,n);CHKERRQ(ierr);
-    ierr = MatSetType(F,type);CHKERRQ(ierr);
-    ierr = MatSetFromOptions(F);CHKERRQ(ierr);
-    for (i=0; i<m; i++) {
-      ierr = MatGetRow(A,i,&ncols,&cols,&vals);CHKERRQ(ierr);
-      ierr = MatSetValues(F,1,&i,ncols,cols,vals,INSERT_VALUES);CHKERRQ(ierr);
-    }
-    ierr = MatAssemblyBegin(F,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-    ierr = MatAssemblyEnd(F,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-    */
     ierr = MatLUFactor(F,perm,iperm,&info);CHKERRQ(ierr);
   }
 
@@ -124,7 +103,7 @@ int main(int argc,char **args)
   ierr = MatSolveAdd(F,b,y,x);CHKERRQ(ierr);
   ierr = MatMult(A,y,b1);CHKERRQ(ierr);
   ierr = VecScale(b1,-1.0);CHKERRQ(ierr);
-  ierr = MatMultAdd(A,x,b1,b1);
+  ierr = MatMultAdd(A,x,b1,b1);CHKERRQ(ierr);
   ierr = VecAXPY(b1,-1.0,b);CHKERRQ(ierr);
   ierr = VecNorm(b1,NORM_2,&norm);CHKERRQ(ierr);
   if (norm > tol) {
@@ -135,7 +114,7 @@ int main(int argc,char **args)
   ierr = MatSolveTransposeAdd(F,b,y,x);CHKERRQ(ierr);
   ierr = MatMultTranspose(A,y,b1);CHKERRQ(ierr);
   ierr = VecScale(b1,-1.0);CHKERRQ(ierr);
-  ierr = MatMultTransposeAdd(A,x,b1,b1);
+  ierr = MatMultTransposeAdd(A,x,b1,b1);CHKERRQ(ierr);
   ierr = VecAXPY(b1,-1.0,b);CHKERRQ(ierr);
   ierr = VecNorm(b1,NORM_2,&norm);CHKERRQ(ierr);
   if (norm > tol) {
@@ -164,11 +143,9 @@ int main(int argc,char **args)
   ierr = ISDestroy(&iperm);CHKERRQ(ierr);
   ierr = DMDestroy(&da);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ComputeRHS"
 PetscErrorCode ComputeRHS(DM da,Vec b)
 {
   PetscErrorCode ierr;
@@ -182,8 +159,6 @@ PetscErrorCode ComputeRHS(DM da,Vec b)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ComputeRHSMatrix"
 PetscErrorCode ComputeRHSMatrix(PetscInt m,PetscInt nrhs,Mat *C)
 {
   PetscErrorCode ierr;
@@ -221,8 +196,6 @@ PetscErrorCode ComputeRHSMatrix(PetscInt m,PetscInt nrhs,Mat *C)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "ComputeMatrix"
 PetscErrorCode ComputeMatrix(DM da,Mat B)
 {
   PetscErrorCode ierr;
@@ -233,7 +206,6 @@ PetscErrorCode ComputeMatrix(DM da,Mat B)
 
   PetscFunctionBegin;
   ierr = PetscRandomCreate(PETSC_COMM_WORLD,&rand);CHKERRQ(ierr);
-  ierr = PetscRandomSetType(rand,PETSCRAND);CHKERRQ(ierr);
   ierr = PetscRandomSetSeed(rand,1);CHKERRQ(ierr);
   ierr = PetscRandomSetInterval(rand,-.001,.001);CHKERRQ(ierr);
   ierr = PetscRandomSetFromOptions(rand);CHKERRQ(ierr);
@@ -305,3 +277,17 @@ PetscErrorCode ComputeMatrix(DM da,Mat B)
   PetscFunctionReturn(0);
 }
 
+
+
+/*TEST
+
+   test:
+      args: -dm_mat_type aij -dof 1
+      output_file: output/ex129.out
+
+   test:
+      suffix: 2
+      args: -dm_mat_type aij -dof 1 -inplacelu
+      output_file: output/ex129.out
+
+TEST*/
