@@ -8,8 +8,6 @@ static char help[] = "Test sequential r2c/c2r FFTW without PETSc interface \n\n"
 #include <petscmat.h>
 #include <fftw3.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
   typedef enum {RANDOM, CONSTANT, TANH, NUM_FUNCS} FuncType;
@@ -27,7 +25,7 @@ int main(int argc,char **args)
   PetscScalar     *x_array,*y_array,*z_array;
   fftw_plan       fplan,bplan;
 
-  ierr = PetscInitialize(&argc,&args,(char*)0,help);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
 #if defined(PETSC_USE_COMPLEX)
   SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_SUP, "This example requires real numbers");
 #endif
@@ -128,7 +126,7 @@ int main(int argc,char **args)
     for (i=0; i<4; i++) {
       /* FFTW_FORWARD */
       fftw_execute(fplan);
-      
+
       /* FFTW_BACKWARD: destroys its input array 'y_array' even for out-of-place transforms! */
       fftw_execute(bplan);
     }
@@ -157,5 +155,16 @@ int main(int argc,char **args)
   }
   ierr = PetscRandomDestroy(&rdm);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
+
+
+/*TEST
+
+   build:
+     requires: fftw !complex
+
+   test:
+     output_file: output/ex142.out
+
+TEST*/
