@@ -2,8 +2,6 @@ static char help[]="This program illustrates the use of PETSc-fftw interface for
 #include <petscmat.h>
 #include <fftw3-mpi.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 extern PetscErrorCode InputTransformFFT(Mat,Vec,Vec);
 extern PetscErrorCode OutputTransformFFT(Mat,Vec,Vec);
 PetscInt main(PetscInt argc,char **args)
@@ -19,7 +17,7 @@ PetscInt main(PetscInt argc,char **args)
   PetscReal      fac;
   PetscScalar    one=1;
 
-  ierr = PetscInitialize(&argc,&args,(char*)0,help);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
 #if defined(PETSC_USE_COMPLEX)
   SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_SUP, "This example requires real numbers");
 #endif
@@ -95,7 +93,7 @@ PetscInt main(PetscInt argc,char **args)
   ierr = MatDestroy(&A);CHKERRQ(ierr);
   ierr = PetscRandomDestroy(&rdm);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
 
