@@ -1,7 +1,5 @@
-
 static char help[] = "Test MatrixMarket outputing.\n\n";
 
-
 /*
   Include "petscmat.h" so that we can use matrices.
   automatically includes:
@@ -9,10 +7,12 @@ static char help[] = "Test MatrixMarket outputing.\n\n";
      petscmat.h    - matrices
      petscis.h     - index sets            petscviewer.h - viewers
 */
+/*T
+   TODO: Need to determine if deprecated
+T*/
+
 #include <petscmat.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
   Mat            A;
@@ -20,19 +20,27 @@ int main(int argc,char **args)
   char           file[PETSC_MAX_PATH_LEN];  /* input file name */
   PetscErrorCode ierr;
   PetscBool      flg;
- 
-  PetscInitialize(&argc,&args,(char*)0,help);
 
-  ierr = PetscOptionsGetString(NULL,"-f0",file,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
+  ierr = PetscOptionsGetString(NULL,NULL,"-f0",file,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
   if (!flg) SETERRQ(PETSC_COMM_WORLD,1,"Must indicate binary file with the -f0 option");
   ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file,FILE_MODE_READ,&fd);CHKERRQ(ierr);
   ierr = MatCreate(PETSC_COMM_WORLD,&A);CHKERRQ(ierr);
   ierr = MatLoad(A,fd);CHKERRQ(ierr);
   ierr = PetscViewerDestroy(&fd);CHKERRQ(ierr);
 
-  ierr = PetscViewerSetFormat(PETSC_VIEWER_STDOUT_WORLD,PETSC_VIEWER_ASCII_MATRIXMARKET);CHKERRQ(ierr);
+  ierr = PetscViewerPushFormat(PETSC_VIEWER_STDOUT_WORLD,PETSC_VIEWER_ASCII_MATRIXMARKET);CHKERRQ(ierr);
   ierr = MatView(A,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
+  ierr = PetscViewerPopFormat(PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
   ierr = MatDestroy(&A);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
+
+/*TEST
+
+   test:
+      args: -f0 ${wPETSC_DIR}/share/petsc/datafiles/matrices/ns-real-int32-float64
+      requires: !complex double !define(PETSC_USE_64BIT_INDICES)
+
+TEST*/
