@@ -3,8 +3,6 @@ static char help[] = "Tests MatTranspose(), MatNorm(), MatAXPY() and MatAYPX().\
 
 #include <petscmat.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   Mat            mat,tmat = 0;
@@ -15,15 +13,15 @@ int main(int argc,char **argv)
   PetscScalar    v, alpha;
   PetscReal      normf,normi,norm1;
 
-  ierr = PetscInitialize(&argc,&argv,(char*)0,help);CHKERRQ(ierr);
-  ierr = PetscViewerSetFormat(PETSC_VIEWER_STDOUT_WORLD,PETSC_VIEWER_ASCII_COMMON);CHKERRQ(ierr);
-  ierr = PetscOptionsGetInt(NULL,"-m",&m,NULL);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
+  ierr = PetscViewerPushFormat(PETSC_VIEWER_STDOUT_WORLD,PETSC_VIEWER_ASCII_COMMON);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-m",&m,NULL);CHKERRQ(ierr);
   ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
   n    = m;
-  ierr = PetscOptionsHasName(NULL,"-rectA",&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-rectA",&flg);CHKERRQ(ierr);
   if (flg) {n += 2; rect = 1;}
-  ierr = PetscOptionsHasName(NULL,"-rectB",&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-rectB",&flg);CHKERRQ(ierr);
   if (flg) {n -= 2; rect = 1;}
 
   /* ------- Assemble matrix, test MatValid() --------- */
@@ -49,7 +47,7 @@ int main(int argc,char **argv)
   ierr = MatView(mat,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
 
   /* --------------- Test MatTranspose()  -------------- */
-  ierr = PetscOptionsHasName(NULL,"-in_place",&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-in_place",&flg);CHKERRQ(ierr);
   if (!rect && flg) {
     ierr = MatTranspose(mat,MAT_REUSE_MATRIX,&mat);CHKERRQ(ierr);   /* in-place transpose */
     tmat = mat; mat = 0;
@@ -68,7 +66,7 @@ int main(int argc,char **argv)
   /* ----------------- Test MatAXPY(), MatAYPX()  ----------------- */
   if (mat && !rect) {
     alpha = 1.0;
-    ierr  = PetscOptionsGetScalar(NULL,"-alpha",&alpha,NULL);CHKERRQ(ierr);
+    ierr  = PetscOptionsGetScalar(NULL,NULL,"-alpha",&alpha,NULL);CHKERRQ(ierr);
     ierr  = PetscPrintf(PETSC_COMM_WORLD,"MatAXPY:  B = B + alpha * A\n");CHKERRQ(ierr);
     ierr  = MatAXPY(tmat,alpha,mat,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
     ierr  = MatView(tmat,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
@@ -115,13 +113,102 @@ int main(int argc,char **argv)
     ierr = MatDestroy(&matB);CHKERRQ(ierr);
   }
 
+  ierr = PetscViewerPopFormat(PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
   /* Free data structures */
-  if (mat)  {ierr = MatDestroy(&mat);CHKERRQ(ierr);}
-  if (tmat) {ierr = MatDestroy(&tmat);CHKERRQ(ierr);}
-
+  ierr = MatDestroy(&mat);CHKERRQ(ierr);
+  ierr = MatDestroy(&tmat);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
 
 
+
+
+/*TEST
+
+   test:
+      suffix: 11_A
+      args: -mat_type seqaij -rectA
+      filter: grep -v "Mat Object"
+
+   test:
+      suffix: 12_A
+      args: -mat_type seqdense -rectA
+      filter: grep -v "Mat Object"
+
+   test:
+      suffix: 11_B
+      args: -mat_type seqaij -rectB
+      filter: grep -v "Mat Object"
+
+   test:
+      suffix: 12_B
+      args: -mat_type seqdense -rectB
+      filter: grep -v "Mat Object"
+
+   test:
+      suffix: 21
+      args: -mat_type mpiaij
+
+   test:
+      suffix: 22
+      args: -mat_type mpidense
+
+   test:
+      suffix: 23
+      nsize: 3
+      args: -mat_type mpiaij
+      filter: grep -v type | grep -v "MPI processes"
+
+   test:
+      suffix: 24
+      nsize: 3
+      args: -mat_type mpidense
+
+   test:
+      suffix: 2_aijcusparse_1
+      args: -mat_type mpiaijcusparse
+      output_file: output/ex2_23.out
+      requires: veccuda
+      filter: grep -v type | grep -v "MPI processes"
+
+   test:
+      suffix: 2_aijcusparse_2
+      nsize: 3
+      args: -mat_type mpiaijcusparse
+      output_file: output/ex2_23.out
+      requires: veccuda
+      filter: grep -v type | grep -v "MPI processes"
+
+   test:
+      suffix: 3
+      nsize: 2
+      args: -mat_type mpiaij -rectA
+
+   test:
+      suffix: 3_aijcusparse
+      nsize: 2
+      args: -mat_type mpiaijcusparse -rectA
+      requires: veccuda
+
+   test:
+      suffix: 4
+      nsize: 2
+      args: -mat_type mpidense -rectA
+
+   test:
+      suffix: aijcusparse_1
+      args: -mat_type seqaijcusparse -rectA
+      filter: grep -v "Mat Object"
+      output_file: output/ex2_11_A_aijcusparse.out
+      requires: veccuda
+
+   test:
+      suffix: aijcusparse_2
+      args: -mat_type seqaijcusparse -rectB
+      filter: grep -v "Mat Object"
+      output_file: output/ex2_11_B_aijcusparse.out
+      requires: veccuda
+
+TEST*/
