@@ -3,8 +3,6 @@ static char help[] = "Tests the vatious routines in MatSeqBAIJ format.\n";
 
 #include <petscmat.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
   Mat            A,B,Fact;
@@ -18,11 +16,10 @@ int main(int argc,char **args)
   PetscBool      flg;
   MatFactorInfo  info;
 
-  PetscInitialize(&argc,&args,(char*)0,help);
-
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
   /* Test MatSetValues() and MatGetValues() */
-  ierr = PetscOptionsGetInt(NULL,"-mat_block_size",&bs,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetInt(NULL,"-mat_size",&m,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-mat_block_size",&bs,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-mat_size",&m,NULL);CHKERRQ(ierr);
   M    = m*bs;
   ierr = MatCreateSeqBAIJ(PETSC_COMM_SELF,bs,M,M,1,NULL,&A);CHKERRQ(ierr);
   ierr = MatSetOption(A,MAT_NEW_NONZERO_ALLOCATION_ERR,PETSC_FALSE);CHKERRQ(ierr);
@@ -39,7 +36,7 @@ int main(int argc,char **args)
   for (row=0; row<M; row++) {
     for (i=0; i<25*bs; i++) {
       ierr = PetscRandomGetValue(rdm,&rval);CHKERRQ(ierr);
-      col  = (PetscInt)(PetscRealPart(rval)*M);
+      col  = PetscMin(M-1,(PetscInt)(PetscRealPart(rval)*M));
       ierr = MatSetValues(A,1,&row,1,&col,&rval,INSERT_VALUES);CHKERRQ(ierr);
       ierr = MatSetValues(B,1,&row,1,&col,&rval,INSERT_VALUES);CHKERRQ(ierr);
     }
@@ -48,16 +45,16 @@ int main(int argc,char **args)
   /* Now set blocks of values */
   for (i=0; i<20*bs; i++) {
     ierr     = PetscRandomGetValue(rdm,&rval);CHKERRQ(ierr);
-    cols[0]  = (PetscInt)(PetscRealPart(rval)*M);
+    cols[0]  = PetscMin(M-1,(PetscInt)(PetscRealPart(rval)*M));
     vals1[0] = rval;
     ierr     = PetscRandomGetValue(rdm,&rval);CHKERRQ(ierr);
-    cols[1]  = (PetscInt)(PetscRealPart(rval)*M);
+    cols[1]  = PetscMin(M-1,(PetscInt)(PetscRealPart(rval)*M));
     vals1[1] = rval;
     ierr     = PetscRandomGetValue(rdm,&rval);CHKERRQ(ierr);
-    rows[0]  = (PetscInt)(PetscRealPart(rval)*M);
+    rows[0]  = PetscMin(M-1,(PetscInt)(PetscRealPart(rval)*M));
     vals1[2] = rval;
     ierr     = PetscRandomGetValue(rdm,&rval);CHKERRQ(ierr);
-    rows[1]  = (PetscInt)(PetscRealPart(rval)*M);
+    rows[1]  = PetscMin(M-1,(PetscInt)(PetscRealPart(rval)*M));
     vals1[3] = rval;
     ierr     = MatSetValues(A,2,rows,2,cols,vals1,INSERT_VALUES);CHKERRQ(ierr);
     ierr     = MatSetValues(B,2,rows,2,cols,vals1,INSERT_VALUES);CHKERRQ(ierr);
@@ -71,19 +68,19 @@ int main(int argc,char **args)
   /* Test MatNorm() */
   ierr  = MatNorm(A,NORM_FROBENIUS,&s1norm);CHKERRQ(ierr);
   ierr  = MatNorm(B,NORM_FROBENIUS,&s2norm);CHKERRQ(ierr);
-  rnorm = PetscAbsScalar(s2norm-s1norm)/s2norm;
+  rnorm = PetscAbsReal(s2norm-s1norm)/s2norm;
   if (rnorm>tol) {
     ierr = PetscPrintf(PETSC_COMM_SELF,"Error: MatNorm_FROBENIUS()- NormA=%16.14e NormB=%16.14e bs = %D\n",s1norm,s2norm,bs);CHKERRQ(ierr);
   }
   ierr  = MatNorm(A,NORM_INFINITY,&s1norm);CHKERRQ(ierr);
   ierr  = MatNorm(B,NORM_INFINITY,&s2norm);CHKERRQ(ierr);
-  rnorm = PetscAbsScalar(s2norm-s1norm)/s2norm;
+  rnorm = PetscAbsReal(s2norm-s1norm)/s2norm;
   if (rnorm>tol) {
     ierr = PetscPrintf(PETSC_COMM_SELF,"Error: MatNorm_INFINITY()- NormA=%16.14e NormB=%16.14e bs = %D\n",s1norm,s2norm,bs);CHKERRQ(ierr);
   }
   ierr  = MatNorm(A,NORM_1,&s1norm);CHKERRQ(ierr);
   ierr  = MatNorm(B,NORM_1,&s2norm);CHKERRQ(ierr);
-  rnorm = PetscAbsScalar(s2norm-s1norm)/s2norm;
+  rnorm = PetscAbsReal(s2norm-s1norm)/s2norm;
   if (rnorm>tol) {
     ierr = PetscPrintf(PETSC_COMM_SELF,"Error: MatNorm_NORM_1()- NormA=%16.14e NormB=%16.14e bs = %D\n",s1norm,s2norm,bs);CHKERRQ(ierr);
   }
@@ -94,19 +91,19 @@ int main(int argc,char **args)
   ierr = MatShift(B,rval);CHKERRQ(ierr);
 
   /* Test MatTranspose() */
-  ierr = MatTranspose(A,MAT_REUSE_MATRIX,&A);CHKERRQ(ierr);
-  ierr = MatTranspose(B,MAT_REUSE_MATRIX,&B);CHKERRQ(ierr);
+  ierr = MatTranspose(A,MAT_INPLACE_MATRIX,&A);CHKERRQ(ierr);
+  ierr = MatTranspose(B,MAT_INPLACE_MATRIX,&B);CHKERRQ(ierr);
 
   /* Now do MatGetValues()  */
   for (i=0; i<30; i++) {
     ierr    = PetscRandomGetValue(rdm,&rval);CHKERRQ(ierr);
-    cols[0] = (PetscInt)(PetscRealPart(rval)*M);
+    cols[0] = PetscMin(M-1,(PetscInt)(PetscRealPart(rval)*M));
     ierr    = PetscRandomGetValue(rdm,&rval);CHKERRQ(ierr);
-    cols[1] = (PetscInt)(PetscRealPart(rval)*M);
+    cols[1] = PetscMin(M-1,(PetscInt)(PetscRealPart(rval)*M));
     ierr    = PetscRandomGetValue(rdm,&rval);CHKERRQ(ierr);
-    rows[0] = (PetscInt)(PetscRealPart(rval)*M);
+    rows[0] = PetscMin(M-1,(PetscInt)(PetscRealPart(rval)*M));
     ierr    = PetscRandomGetValue(rdm,&rval);CHKERRQ(ierr);
-    rows[1] = (PetscInt)(PetscRealPart(rval)*M);
+    rows[1] = PetscMin(M-1,(PetscInt)(PetscRealPart(rval)*M));
     ierr    = MatGetValues(A,2,rows,2,cols,vals1);CHKERRQ(ierr);
     ierr    = MatGetValues(B,2,rows,2,cols,vals2);CHKERRQ(ierr);
     ierr    = PetscMemcmp(vals1,vals2,4*sizeof(PetscScalar),&flg);CHKERRQ(ierr);
@@ -255,5 +252,13 @@ int main(int argc,char **args)
   ierr = ISDestroy(&is2);CHKERRQ(ierr);
   ierr = PetscRandomDestroy(&rdm);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
+
+
+/*TEST
+
+   test:
+      args: -mat_block_size {{1 2 3 4 5 6 7 8}}
+
+TEST*/
