@@ -1,4 +1,8 @@
 
+/*T
+   requires: !complex
+T*/
+
 #include <petscmat.h>
 
 #if !defined(PETSC_USE_64BIT_INDICES)
@@ -9,8 +13,6 @@ them using the PETSc sparse format. Input parameters are:\n\
 #endif
 
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
 #if !defined(PETSC_USE_64BIT_INDICES)
@@ -24,17 +26,16 @@ int main(int argc,char **args)
   FILE           *file;
   PetscViewer    view;
 
-  PetscInitialize(&argc,&args,(char*)0,help);
-
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
   /* Read in matrix and RHS */
-  ierr = PetscOptionsGetString(NULL,"-fin",filein,256,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-fin",filein,256,&flg);CHKERRQ(ierr);
   if (!flg) SETERRQ(PETSC_COMM_WORLD,1,"Must indicate file for reading");
-  ierr = PetscOptionsGetString(NULL,"-fout",fileout,256,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-fout",fileout,256,&flg);CHKERRQ(ierr);
   if (!flg) SETERRQ(PETSC_COMM_WORLD,1,"Must indicate file for writing");
 
   ierr = PetscFixFilename(filein,finname);CHKERRQ(ierr);
-  if (!(file = fopen(finname,"r"))) SETERRQ(PETSC_COMM_SELF,1,"cannot open input file\n");
-  fscanf(file,"%d\n",&n);
+  if (!(file = fopen(finname,"r"))) SETERRQ(PETSC_COMM_SELF,1,"Cannot open input file\n");
+  if (fscanf(file,"%d\n",&n) != 1) SETERRQ(PETSC_COMM_SELF,1,"Badly formatted input file\n");
 
   ierr = MatCreate(PETSC_COMM_WORLD,&A);CHKERRQ(ierr);
   ierr = MatSetSizes(A,PETSC_DECIDE,PETSC_DECIDE,n,n);CHKERRQ(ierr);
@@ -44,7 +45,7 @@ int main(int argc,char **args)
   ierr = VecSetFromOptions(b);CHKERRQ(ierr);
 
   for (row=0; row<n; row++) {
-    fscanf(file,"row %d:",&rowin);
+    if (fscanf(file,"row %d:",&rowin) != 1) SETERRQ(PETSC_COMM_SELF,1,"Badly formatted input file\n");
     if (rowin != row) SETERRQ(PETSC_COMM_SELF,1,"Bad file");
     while (fscanf(file," %d %le",&col,(double*)&val)) {
       ierr = MatSetValues(A,1,&row,1,&col,&val,INSERT_VALUES);CHKERRQ(ierr);
@@ -54,7 +55,7 @@ int main(int argc,char **args)
   ierr = MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = VecGetArray(b,&array);CHKERRQ(ierr);
   for (row=0; row<n; row++) {
-    fscanf(file," ii= %d %le",&col,(double*)(array+row));
+    if (fscanf(file," ii= %d %le",&col,(double*)(array+row)) != 2)  SETERRQ(PETSC_COMM_SELF,1,"Badly formatted input file\n");
   }
   ierr = VecRestoreArray(b,&array);CHKERRQ(ierr);
 
@@ -68,9 +69,18 @@ int main(int argc,char **args)
 
   ierr = VecDestroy(&b);CHKERRQ(ierr);
   ierr = MatDestroy(&A);CHKERRQ(ierr);
-
   ierr = PetscFinalize();
-#endif
+  return ierr;
+#else
   return 0;
+#endif
 }
 
+
+
+/*TEST
+
+   test:
+     TODO: need to provide ASCII matrix to read in
+
+TEST*/
