@@ -1,8 +1,6 @@
 static char help[] = "Testing MatCreateMPIMatConcatenateSeqMat().\n\n";
 
 #include <petscmat.h>
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   PetscErrorCode ierr;
@@ -11,9 +9,9 @@ int main(int argc,char **argv)
   PetscScalar    value[3],*vals;
   PetscInt       i,col[3],n=5,bs=1;
   
-  PetscInitialize(&argc,&argv,(char*)0,help);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
-  ierr = PetscOptionsGetInt(NULL,"-bs",&bs,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-bs",&bs,NULL);CHKERRQ(ierr);
 
   /* Create seqaij matrices of size (n+rank) by n */
   ierr = MatCreate(PETSC_COMM_SELF,&seqmat);CHKERRQ(ierr);
@@ -53,8 +51,8 @@ int main(int argc,char **argv)
   }
   ierr = MatAssemblyBegin(seqmat,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(seqmat,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-  if (rank == 1) {
-    printf("[%d] seqmat:\n",rank);
+  if (!rank) {
+    ierr = PetscPrintf(PETSC_COMM_SELF,"[%d] seqmat:\n",rank);CHKERRQ(ierr);
     ierr = MatView(seqmat,PETSC_VIEWER_STDOUT_SELF);CHKERRQ(ierr);
   }
 
@@ -65,6 +63,24 @@ int main(int argc,char **argv)
   ierr = MatDestroy(&seqmat);CHKERRQ(ierr);
   ierr = MatDestroy(&mpimat);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
+
+
+/*TEST
+
+   test:
+      nsize: 3
+
+   test:
+      suffix: 2
+      nsize: 3
+      args: -mat_type baij
+
+   test:
+      suffix: 3
+      nsize: 3
+      args: -mat_type baij -bs 2
+
+TEST*/
