@@ -3,8 +3,6 @@ static char help[] = "Tests MPI parallel matrix creation. Test MatCreateRedundan
 
 #include <petscmat.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
   Mat            C,Credundant;
@@ -18,8 +16,8 @@ int main(int argc,char **args)
   Vec            x,y;
   MPI_Comm       subcomm;
 
-  PetscInitialize(&argc,&args,(char*)0,help);
-  ierr = PetscOptionsGetInt(NULL,"-m",&m,NULL);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
+  ierr = PetscOptionsGetInt(NULL,NULL,"-m",&m,NULL);CHKERRQ(ierr);
   ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
   n    = 2*size;
@@ -52,7 +50,7 @@ int main(int argc,char **args)
 
   /* Form vectors */
   ierr = MatCreateVecs(C,&x,&y);CHKERRQ(ierr);
-  ierr = VecGetLocalSize(x,&ldim);
+  ierr = VecGetLocalSize(x,&ldim);CHKERRQ(ierr);
   ierr = VecGetOwnershipRange(x,&low,&high);CHKERRQ(ierr);
   for (i=0; i<ldim; i++) {
     iglobal = i + low;
@@ -63,40 +61,35 @@ int main(int argc,char **args)
   ierr = VecAssemblyEnd(x);CHKERRQ(ierr);
 
   ierr = MatMult(C,x,y);CHKERRQ(ierr);
-  /*
-  ierr = VecView(x,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
-  ierr = VecView(y,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
-   */
-  ierr = PetscOptionsHasName(NULL,"-view_info",&flg_info);CHKERRQ(ierr);
+
+  ierr = PetscOptionsHasName(NULL,NULL,"-view_info",&flg_info);CHKERRQ(ierr);
   if (flg_info)  {
-    ierr = PetscViewerSetFormat(PETSC_VIEWER_STDOUT_WORLD,PETSC_VIEWER_ASCII_INFO);CHKERRQ(ierr);
+    ierr = PetscViewerPushFormat(PETSC_VIEWER_STDOUT_WORLD,PETSC_VIEWER_ASCII_INFO);CHKERRQ(ierr);
     ierr = MatView(C,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
   
     ierr = MatGetInfo(C,MAT_GLOBAL_SUM,&info);CHKERRQ(ierr);
-    ierr = PetscPrintf(PETSC_COMM_WORLD,"matrix information (global sums):\n\
-     nonzeros = %D, allocated nonzeros = %D\n",(PetscInt)info.nz_used,(PetscInt)info.nz_allocated);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(PETSC_VIEWER_STDOUT_WORLD,"matrix information (global sums):\nnonzeros = %D, allocated nonzeros = %D\n",(PetscInt)info.nz_used,(PetscInt)info.nz_allocated);CHKERRQ(ierr);
     ierr = MatGetInfo (C,MAT_GLOBAL_MAX,&info);CHKERRQ(ierr);
-    ierr = PetscPrintf(PETSC_COMM_WORLD,"matrix information (global max):\n\
-     nonzeros = %D, allocated nonzeros = %D\n",(PetscInt)info.nz_used,(PetscInt)info.nz_allocated);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(PETSC_VIEWER_STDOUT_WORLD,"matrix information (global max):\nnonzeros = %D, allocated nonzeros = %D\n",(PetscInt)info.nz_used,(PetscInt)info.nz_allocated);CHKERRQ(ierr);
   }
   
-  ierr = PetscOptionsHasName(NULL,"-view_mat",&flg_mat);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-view_mat",&flg_mat);CHKERRQ(ierr);
   if (flg_mat) {
     ierr = MatView(C,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
   }
 
   /* Test MatCreateRedundantMatrix() */
   nsubcomms = size;
-  ierr = PetscOptionsGetInt(NULL,"-nsubcomms",&nsubcomms,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-nsubcomms",&nsubcomms,NULL);CHKERRQ(ierr);
   ierr = MatCreateRedundantMatrix(C,nsubcomms,MPI_COMM_NULL,MAT_INITIAL_MATRIX,&Credundant);CHKERRQ(ierr);
   ierr = MatCreateRedundantMatrix(C,nsubcomms,MPI_COMM_NULL,MAT_REUSE_MATRIX,&Credundant);CHKERRQ(ierr);
 
   ierr = PetscObjectGetComm((PetscObject)Credundant,&subcomm);CHKERRQ(ierr);
   ierr = MPI_Comm_size(subcomm,&subsize);CHKERRQ(ierr);
     
-  if (subsize==1 && flg_mat) {
-    printf("\n[%d] Credundant:\n",rank);
-    ierr = MatView(Credundant,PETSC_VIEWER_STDOUT_SELF);CHKERRQ(ierr);
+  if (subsize==2 && flg_mat) {
+    ierr = PetscViewerASCIIPrintf(PETSC_VIEWER_STDOUT_(subcomm),"\n[%d] Credundant:\n",rank);CHKERRQ(ierr);
+    ierr = MatView(Credundant,PETSC_VIEWER_STDOUT_(subcomm));CHKERRQ(ierr);
   }
   ierr = MatDestroy(&Credundant);CHKERRQ(ierr);
    
@@ -121,5 +114,54 @@ int main(int argc,char **args)
   ierr = VecDestroy(&y);CHKERRQ(ierr);
   ierr = MatDestroy(&C);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
+
+
+/*TEST
+
+   test:
+      nsize: 3
+      args: -view_info
+
+   test:
+      suffix: 2
+      nsize: 3
+      args: -nsubcomms 2 -view_mat -psubcomm_type interlaced
+
+   test:
+      suffix: 3
+      nsize: 3
+      args: -nsubcomms 2 -view_mat -psubcomm_type contiguous
+
+   test:
+      suffix: 3_baij
+      nsize: 3
+      args: -mat_type baij -nsubcomms 2 -view_mat
+
+   test:
+      suffix: 3_sbaij
+      nsize: 3
+      args: -mat_type sbaij -nsubcomms 2 -view_mat
+
+   test:
+      suffix: 3_dense
+      nsize: 3
+      args: -mat_type dense -nsubcomms 2 -view_mat
+
+   test:
+      suffix: 4_baij
+      nsize: 3
+      args: -mat_type baij -nsubcomms 2 -view_mat -psubcomm_type interlaced
+
+   test:
+      suffix: 4_sbaij
+      nsize: 3
+      args: -mat_type sbaij -nsubcomms 2 -view_mat -psubcomm_type interlaced
+
+   test:
+      suffix: 4_dense
+      nsize: 3
+      args: -mat_type dense -nsubcomms 2 -view_mat -psubcomm_type interlaced
+
+TEST*/
