@@ -1,9 +1,7 @@
-static const char help[] = "Tests MatGetSubMatrix with MatSubMatrix versus MatAIJ, non-square\n";
+static const char help[] = "Tests MatCreateSubMatrix with MatSubMatrix versus MatAIJ, non-square\n";
 
 #include <petscmat.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "AssembleMatrix"
 static PetscErrorCode AssembleMatrix(MPI_Comm comm,Mat *A)
 {
   PetscErrorCode ierr;
@@ -26,8 +24,6 @@ static PetscErrorCode AssembleMatrix(MPI_Comm comm,Mat *A)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "Compare2"
 static PetscErrorCode Compare2(Vec *X,const char *test)
 {
   PetscErrorCode ierr;
@@ -41,9 +37,9 @@ static PetscErrorCode Compare2(Vec *X,const char *test)
   ierr = VecAYPX(Y,-1.0,X[1]);CHKERRQ(ierr);
   ierr = VecNorm(Y,NORM_INFINITY,&norm);CHKERRQ(ierr);
 
-  ierr = PetscOptionsGetInt(NULL,"-verbose",&verbose,NULL);CHKERRQ(ierr);
-  if (norm < 1.e-12 && verbose < 1) {
-    ierr = PetscPrintf(PETSC_COMM_WORLD,"%30s: norm difference < 1e-12\n",test);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-verbose",&verbose,NULL);CHKERRQ(ierr);
+  if (norm < PETSC_SQRT_MACHINE_EPSILON && verbose < 1) {
+    ierr = PetscPrintf(PETSC_COMM_WORLD,"%30s: norm difference < sqrt(eps_machine)\n",test);CHKERRQ(ierr);
   } else {
     ierr = PetscPrintf(PETSC_COMM_WORLD,"%30s: norm difference %g\n",test,(double)norm);CHKERRQ(ierr);
   }
@@ -56,8 +52,6 @@ static PetscErrorCode Compare2(Vec *X,const char *test)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CheckMatrices"
 static PetscErrorCode CheckMatrices(Mat A,Mat B,Vec left,Vec right,Vec X,Vec Y,Vec X1,Vec Y1)
 {
   PetscErrorCode ierr;
@@ -104,8 +98,6 @@ static PetscErrorCode CheckMatrices(Mat A,Mat B,Vec left,Vec right,Vec X,Vec Y,V
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc, char *argv[])
 {
   PetscErrorCode ierr;
@@ -118,8 +110,8 @@ int main(int argc, char *argv[])
   ierr = PetscInitialize(&argc,&argv,NULL,help);CHKERRQ(ierr);
   ierr = AssembleMatrix(PETSC_COMM_WORLD,&A);CHKERRQ(ierr);
   ierr = AssembleMatrix(PETSC_COMM_WORLD,&B);CHKERRQ(ierr);
-  ierr = MatShellSetOperation(B,MATOP_GET_SUBMATRIX,NULL);CHKERRQ(ierr);
-  ierr = MatShellSetOperation(B,MATOP_GET_SUBMATRICES,NULL);CHKERRQ(ierr);
+  ierr = MatSetOperation(B,MATOP_CREATE_SUBMATRIX,NULL);CHKERRQ(ierr);
+  ierr = MatSetOperation(B,MATOP_CREATE_SUBMATRICES,NULL);CHKERRQ(ierr);
   ierr = MatGetOwnershipRange(A,&ms,NULL);CHKERRQ(ierr);
 
   idxrow[0] = ms+1;
@@ -133,8 +125,8 @@ int main(int argc, char *argv[])
   idxcol[3] = ms+5;
   ierr      = ISCreateGeneral(PETSC_COMM_WORLD,4,idxcol,PETSC_USE_POINTER,&iscol);CHKERRQ(ierr);
 
-  ierr = MatGetSubMatrix(A,isrow,iscol,MAT_INITIAL_MATRIX,&Asub);CHKERRQ(ierr);
-  ierr = MatGetSubMatrix(B,isrow,iscol,MAT_INITIAL_MATRIX,&Bsub);CHKERRQ(ierr);
+  ierr = MatCreateSubMatrix(A,isrow,iscol,MAT_INITIAL_MATRIX,&Asub);CHKERRQ(ierr);
+  ierr = MatCreateSubMatrix(B,isrow,iscol,MAT_INITIAL_MATRIX,&Bsub);CHKERRQ(ierr);
 
   ierr = MatCreateVecs(Asub,&right,&left);CHKERRQ(ierr);
   ierr = VecDuplicate(right,&X);CHKERRQ(ierr);
@@ -142,7 +134,7 @@ int main(int argc, char *argv[])
   ierr = VecDuplicate(left,&Y);CHKERRQ(ierr);
   ierr = VecDuplicate(left,&Y1);CHKERRQ(ierr);
 
-  ierr = PetscOptionsGetBool(NULL,"-random",&random,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-random",&random,NULL);CHKERRQ(ierr);
   if (random) {
     ierr = VecSetRandom(right,NULL);CHKERRQ(ierr);
     ierr = VecSetRandom(left,NULL);CHKERRQ(ierr);
@@ -172,6 +164,14 @@ int main(int argc, char *argv[])
   ierr = VecDestroy(&X1);CHKERRQ(ierr);
   ierr = VecDestroy(&Y1);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
+
+
+/*TEST
+
+   test:
+      nsize: 3
+
+TEST*/
