@@ -20,8 +20,6 @@ T*/
 */
 #include <petscmat.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
   Mat             A;                      /* matrix */
@@ -32,9 +30,7 @@ int main(int argc,char **args)
   MatOrderingType rtype = MATORDERINGRCM;
   PetscBool       flg,PetscPreLoad = PETSC_FALSE;
 
-  PetscInitialize(&argc,&args,(char*)0,help);
-
-
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
   /*
      Determine files from which we read the two linear systems
      (matrix and right-hand-side vector).
@@ -53,7 +49,7 @@ int main(int argc,char **args)
         then load another (larger) system and solve it as well.
         This process preloads the instructions with the smaller
         system so that more accurate performance monitoring (via
-        -log_summary) can be done with the larger one (that actually
+        -log_view) can be done with the larger one (that actually
         is the system of interest).
   */
   PetscPreLoadBegin(PetscPreLoad,"Load");
@@ -93,6 +89,16 @@ int main(int argc,char **args)
   PetscPreLoadEnd();
 
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
+
+
+/*TEST
+
+   test:
+      requires: datafilespath double !complex !define(PETSC_USE_64BIT_INDICES)
+      args: -f0 ${DATAFILESPATH}/matrices/medium -viewer_binary_skip_info
+      TODO: Need to develop comparison test
+
+TEST*/
