@@ -16,8 +16,6 @@ T*/
 #include <petscmat.h>
 
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
   Mat            A;                       /* matrix */
@@ -28,14 +26,12 @@ int main(int argc,char **args)
   PetscInt       n,cstart,cend;
   PetscBool      flg;
 
-  PetscInitialize(&argc,&args,(char*)0,help);
-
-
+  ierr = PetscInitialize(&argc,&args,(char*)0,help);if (ierr) return ierr;
   /*
      Determine files from which we read the two linear systems
      (matrix and right-hand-side vector).
   */
-  ierr = PetscOptionsGetString(NULL,"-f",file,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-f",file,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
   if (!flg) SETERRQ(PETSC_COMM_WORLD,1,"Must indicate binary file with the -f option");
 
   /*
@@ -62,6 +58,33 @@ int main(int argc,char **args)
 
   ierr = MatDestroy(&A);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
+
+
+/*TEST
+
+   test:
+      suffix: mpiaij
+      nsize: 2
+      requires: datafilespath double !complex !define(PETSC_USE_64BIT_INDICES)
+      args: -f ${DATAFILESPATH}/matrices/small -a_mat_type mpiaij
+
+   test:
+      suffix: mpidense
+      nsize: 2
+      requires: datafilespath double !complex !define(PETSC_USE_64BIT_INDICES)
+      args: -f ${DATAFILESPATH}/matrices/small -a_mat_type mpidense
+
+   test:
+      suffix: seqaij
+      requires: datafilespath double !complex !define(PETSC_USE_64BIT_INDICES)
+      args: -f ${DATAFILESPATH}/matrices/small -a_mat_type seqaij
+
+   test:
+      suffix: seqdense
+      requires: datafilespath double !complex !define(PETSC_USE_64BIT_INDICES)
+      args: -f ${DATAFILESPATH}/matrices/small -a_mat_type seqdense
+
+TEST*/
