@@ -6,10 +6,10 @@ static char help[] = "Example of using graph partitioning with a matrix in which
    Processors: n
 T*/
 
+
+
 #include <petscmat.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc, char **args)
 {
   Mat             A;
@@ -67,5 +67,18 @@ int main(int argc, char **args)
   ierr = MatDestroy(&A);CHKERRQ(ierr);
   ierr = PetscFree2(emptyranks,bigranks);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
+
+
+/*TEST
+
+   build:
+      requires: c99
+
+   test:
+      nsize: 8
+      args: -emptyranks 0,2,4 -bigranks 1,3,7
+      TODO: Need to determine if deprecated
+
+TEST*/
