@@ -1,18 +1,9 @@
 
 static char help[] = "Each process opens the file and reads its part. Not scalable do not copy\n";
 
-#define USE_FAST_MAT_SET_VALUES
-
 #include <petscsys.h>
 #include <petscviewer.h>
-
-#if defined(USE_FAST_MAT_SET_VALUES)
-#include <../src/mat/impls/aij/mpi/mpiaij.h>
-#define MatSetValues MatSetValues_MPIAIJ
-#else
 #include <petscmat.h>
-#endif
-
 
 /*
    Opens a separate file for each process and reads in ITS portion
@@ -21,8 +12,6 @@ static char help[] = "Each process opens the file and reads its part. Not scalab
 
     petsc-maint@mcs.anl.gov
 */
-#undef __FUNCT__
-#define __FUNCT__ "Mat_Parallel_Load"
 int Mat_Parallel_Load(MPI_Comm comm,const char *name,Mat *newmat)
 {
   Mat            A;
@@ -152,11 +141,22 @@ int main(int argc,char **args)
   char           name[1024];
   PetscBool      flg;
 
-  PetscInitialize(&argc,&args,0,help);
-  ierr = PetscOptionsGetString(NULL,"-f",name,1024,&flg);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&args,0,help);if (ierr) return ierr;
+  ierr = PetscOptionsGetString(NULL,NULL,"-f",name,1024,&flg);CHKERRQ(ierr);
   if (!flg) SETERRQ(PETSC_COMM_SELF,1,"Must pass in filename with -f option");
   ierr = Mat_Parallel_Load(PETSC_COMM_WORLD,name,&A);CHKERRQ(ierr);
   ierr = MatDestroy(&A);CHKERRQ(ierr);
-  PetscFinalize();
-  return 0;
+  ierr = PetscFinalize();
+  return ierr;
 }
+
+
+/*TEST
+
+   test:
+      nsize: 2
+      requires: datafilespath double !complex !define(PETSC_USE_64BIT_INDICES)
+      args: -f ${DATAFILESPATH}/matrices/medium -viewer_binary_skip_info
+      TODO: Need to develop comparison test
+
+TEST*/
