@@ -3,10 +3,233 @@
     Defines the basic matrix operations for the ADJ adjacency list matrix data-structure.
 */
 #include <../src/mat/impls/adj/mpi/mpiadj.h>    /*I "petscmat.h" I*/
+#include <petscsf.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "MatView_MPIAdj_ASCII"
-PetscErrorCode MatView_MPIAdj_ASCII(Mat A,PetscViewer viewer)
+/*
+ * The interface should be easy to use for both MatCreateSubMatrix (parallel sub-matrix) and MatCreateSubMatrices (sequential sub-matrices)
+ * */
+static PetscErrorCode MatCreateSubMatrix_MPIAdj_data(Mat adj,IS irows, IS icols, PetscInt **sadj_xadj,PetscInt **sadj_adjncy,PetscInt **sadj_values)
+{
+  PetscInt           nlrows_is,icols_n,i,j,nroots,nleaves,owner,rlocalindex,*ncols_send,*ncols_recv;
+  PetscInt           nlrows_mat,*adjncy_recv,Ncols_recv,Ncols_send,*xadj_recv,*values_recv;
+  PetscInt          *ncols_recv_offsets,loc,rnclos,*sadjncy,*sxadj,*svalues,isvalue;
+  const PetscInt    *irows_indices,*icols_indices,*xadj, *adjncy;
+  Mat_MPIAdj        *a = (Mat_MPIAdj*)adj->data;
+  PetscLayout        rmap;
+  MPI_Comm           comm;
+  PetscSF            sf;
+  PetscSFNode       *iremote;
+  PetscBool          done;
+  PetscErrorCode     ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectGetComm((PetscObject)adj,&comm);CHKERRQ(ierr);
+  ierr = MatGetLayouts(adj,&rmap,NULL);CHKERRQ(ierr);
+  ierr = ISGetLocalSize(irows,&nlrows_is);CHKERRQ(ierr);
+  ierr = ISGetIndices(irows,&irows_indices);CHKERRQ(ierr);
+  ierr = PetscCalloc1(nlrows_is,&iremote);CHKERRQ(ierr);
+  /* construct sf graph*/
+  nleaves = nlrows_is;
+  for (i=0; i<nlrows_is; i++){
+    owner = -1;
+    rlocalindex = -1;
+    ierr = PetscLayoutFindOwnerIndex(rmap,irows_indices[i],&owner,&rlocalindex);CHKERRQ(ierr);
+    iremote[i].rank  = owner;
+    iremote[i].index = rlocalindex;
+  }
+  ierr = MatGetRowIJ(adj,0,PETSC_FALSE,PETSC_FALSE,&nlrows_mat,&xadj,&adjncy,&done);CHKERRQ(ierr);
+  ierr = PetscCalloc4(nlrows_mat,&ncols_send,nlrows_is,&xadj_recv,nlrows_is+1,&ncols_recv_offsets,nlrows_is,&ncols_recv);CHKERRQ(ierr);
+  nroots = nlrows_mat;
+  for (i=0; i<nlrows_mat; i++){
+    ncols_send[i] = xadj[i+1]-xadj[i];
+  }
+  ierr = PetscSFCreate(comm,&sf);CHKERRQ(ierr);
+  ierr = PetscSFSetGraph(sf,nroots,nleaves,NULL,PETSC_OWN_POINTER,iremote,PETSC_OWN_POINTER);CHKERRQ(ierr);
+  ierr = PetscSFSetType(sf,PETSCSFBASIC);CHKERRQ(ierr);
+  ierr = PetscSFSetFromOptions(sf);CHKERRQ(ierr);
+  ierr = PetscSFBcastBegin(sf,MPIU_INT,ncols_send,ncols_recv);CHKERRQ(ierr);
+  ierr = PetscSFBcastEnd(sf,MPIU_INT,ncols_send,ncols_recv);CHKERRQ(ierr);
+  ierr = PetscSFBcastBegin(sf,MPIU_INT,xadj,xadj_recv);CHKERRQ(ierr);
+  ierr = PetscSFBcastEnd(sf,MPIU_INT,xadj,xadj_recv);CHKERRQ(ierr);
+  ierr = PetscSFDestroy(&sf);CHKERRQ(ierr);
+  Ncols_recv =0;
+  for (i=0; i<nlrows_is; i++){
+    Ncols_recv             += ncols_recv[i];
+    ncols_recv_offsets[i+1] = ncols_recv[i]+ncols_recv_offsets[i];
+  }
+  Ncols_send = 0;
+  for(i=0; i<nlrows_mat; i++){
+    Ncols_send += ncols_send[i];
+  }
+  ierr = PetscCalloc1(Ncols_recv,&iremote);CHKERRQ(ierr);
+  ierr = PetscCalloc1(Ncols_recv,&adjncy_recv);CHKERRQ(ierr);
+  nleaves = Ncols_recv;
+  Ncols_recv = 0;
+  for (i=0; i<nlrows_is; i++){
+    ierr = PetscLayoutFindOwner(rmap,irows_indices[i],&owner);CHKERRQ(ierr);
+    for (j=0; j<ncols_recv[i]; j++){
+      iremote[Ncols_recv].rank    = owner;
+      iremote[Ncols_recv++].index = xadj_recv[i]+j;
+    }
+  }
+  ierr = ISRestoreIndices(irows,&irows_indices);CHKERRQ(ierr);
+  /*if we need to deal with edge weights ???*/
+  if (a->values){isvalue=1;} else {isvalue=0;}
+  /*involve a global communication */
+  /*ierr = MPIU_Allreduce(&isvalue,&isvalue,1,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);*/
+  if (isvalue){ierr = PetscCalloc1(Ncols_recv,&values_recv);CHKERRQ(ierr);}
+  nroots = Ncols_send;
+  ierr = PetscSFCreate(comm,&sf);CHKERRQ(ierr);
+  ierr = PetscSFSetGraph(sf,nroots,nleaves,NULL,PETSC_OWN_POINTER,iremote,PETSC_OWN_POINTER);CHKERRQ(ierr);
+  ierr = PetscSFSetType(sf,PETSCSFBASIC);CHKERRQ(ierr);
+  ierr = PetscSFSetFromOptions(sf);CHKERRQ(ierr);
+  ierr = PetscSFBcastBegin(sf,MPIU_INT,adjncy,adjncy_recv);CHKERRQ(ierr);
+  ierr = PetscSFBcastEnd(sf,MPIU_INT,adjncy,adjncy_recv);CHKERRQ(ierr);
+  if (isvalue){
+    ierr = PetscSFBcastBegin(sf,MPIU_INT,a->values,values_recv);CHKERRQ(ierr);
+    ierr = PetscSFBcastEnd(sf,MPIU_INT,a->values,values_recv);CHKERRQ(ierr);
+  }
+  ierr = PetscSFDestroy(&sf);CHKERRQ(ierr);
+  ierr = MatRestoreRowIJ(adj,0,PETSC_FALSE,PETSC_FALSE,&nlrows_mat,&xadj,&adjncy,&done);CHKERRQ(ierr);
+  ierr = ISGetLocalSize(icols,&icols_n);CHKERRQ(ierr);
+  ierr = ISGetIndices(icols,&icols_indices);CHKERRQ(ierr);
+  rnclos = 0;
+  for (i=0; i<nlrows_is; i++){
+    for (j=ncols_recv_offsets[i]; j<ncols_recv_offsets[i+1]; j++){
+      ierr = PetscFindInt(adjncy_recv[j], icols_n, icols_indices, &loc);CHKERRQ(ierr);
+      if (loc<0){
+        adjncy_recv[j] = -1;
+        if (isvalue) values_recv[j] = -1;
+        ncols_recv[i]--;
+      } else {
+        rnclos++;
+      }
+    }
+  }
+  ierr = ISRestoreIndices(icols,&icols_indices);CHKERRQ(ierr);
+  ierr = PetscCalloc1(rnclos,&sadjncy);CHKERRQ(ierr);
+  if (isvalue) {ierr = PetscCalloc1(rnclos,&svalues);CHKERRQ(ierr);}
+  ierr = PetscCalloc1(nlrows_is+1,&sxadj);CHKERRQ(ierr);
+  rnclos = 0;
+  for(i=0; i<nlrows_is; i++){
+    for(j=ncols_recv_offsets[i]; j<ncols_recv_offsets[i+1]; j++){
+      if (adjncy_recv[j]<0) continue;
+      sadjncy[rnclos] = adjncy_recv[j];
+      if (isvalue) svalues[rnclos] = values_recv[j];
+      rnclos++;
+    }
+  }
+  for(i=0; i<nlrows_is; i++){
+    sxadj[i+1] = sxadj[i]+ncols_recv[i];
+  }
+  if (sadj_xadj)  { *sadj_xadj = sxadj;} else    { ierr = PetscFree(sxadj);CHKERRQ(ierr);}
+  if (sadj_adjncy){ *sadj_adjncy = sadjncy;} else { ierr = PetscFree(sadjncy);CHKERRQ(ierr);}
+  if (sadj_values){
+    if (isvalue) *sadj_values = svalues; else *sadj_values=0;
+  } else {
+    if (isvalue) {ierr = PetscFree(svalues);CHKERRQ(ierr);}
+  }
+  ierr = PetscFree4(ncols_send,xadj_recv,ncols_recv_offsets,ncols_recv);CHKERRQ(ierr);
+  ierr = PetscFree(adjncy_recv);CHKERRQ(ierr);
+  if (isvalue) {ierr = PetscFree(values_recv);CHKERRQ(ierr);}
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode MatCreateSubMatrices_MPIAdj_Private(Mat mat,PetscInt n,const IS irow[],const IS icol[],PetscBool subcomm,MatReuse scall,Mat *submat[])
+{
+  PetscInt           i,irow_n,icol_n,*sxadj,*sadjncy,*svalues;
+  PetscInt          *indices,nindx,j,k,loc;
+  PetscMPIInt        issame;
+  const PetscInt    *irow_indices,*icol_indices;
+  MPI_Comm           scomm_row,scomm_col,scomm_mat;
+  PetscErrorCode     ierr;
+
+  PetscFunctionBegin;
+  nindx = 0;
+  /*
+   * Estimate a maximum number for allocating memory
+   */
+  for(i=0; i<n; i++){
+    ierr = ISGetLocalSize(irow[i],&irow_n);CHKERRQ(ierr);
+    ierr = ISGetLocalSize(icol[i],&icol_n);CHKERRQ(ierr);
+    nindx = nindx>(irow_n+icol_n)? nindx:(irow_n+icol_n);
+  }
+  ierr = PetscCalloc1(nindx,&indices);CHKERRQ(ierr);
+  /* construct a submat */
+  for(i=0; i<n; i++){
+    if (subcomm){
+      ierr = PetscObjectGetComm((PetscObject)irow[i],&scomm_row);CHKERRQ(ierr);
+      ierr = PetscObjectGetComm((PetscObject)icol[i],&scomm_col);CHKERRQ(ierr);
+      ierr = MPI_Comm_compare(scomm_row,scomm_col,&issame);CHKERRQ(ierr);
+      if (issame != MPI_IDENT) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"row index set must have the same comm as the col index set\n");
+      ierr = MPI_Comm_compare(scomm_row,PETSC_COMM_SELF,&issame);CHKERRQ(ierr);
+      if (issame == MPI_IDENT) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP," can not use PETSC_COMM_SELF as comm when extracting a parallel submatrix\n");
+    } else {
+      scomm_row = PETSC_COMM_SELF;
+    }
+    /*get sub-matrix data*/
+    sxadj=0; sadjncy=0; svalues=0;
+    ierr = MatCreateSubMatrix_MPIAdj_data(mat,irow[i],icol[i],&sxadj,&sadjncy,&svalues);CHKERRQ(ierr);
+    ierr = ISGetLocalSize(irow[i],&irow_n);CHKERRQ(ierr);
+    ierr = ISGetLocalSize(icol[i],&icol_n);CHKERRQ(ierr);
+    ierr = ISGetIndices(irow[i],&irow_indices);CHKERRQ(ierr);
+    ierr = PetscMemcpy(indices,irow_indices,sizeof(PetscInt)*irow_n);CHKERRQ(ierr);
+    ierr = ISRestoreIndices(irow[i],&irow_indices);CHKERRQ(ierr);
+    ierr = ISGetIndices(icol[i],&icol_indices);CHKERRQ(ierr);
+    ierr = PetscMemcpy(indices+irow_n,icol_indices,sizeof(PetscInt)*icol_n);CHKERRQ(ierr);
+    ierr = ISRestoreIndices(icol[i],&icol_indices);CHKERRQ(ierr);
+    nindx = irow_n+icol_n;
+    ierr = PetscSortRemoveDupsInt(&nindx,indices);CHKERRQ(ierr);
+    /* renumber columns */
+    for(j=0; j<irow_n; j++){
+      for(k=sxadj[j]; k<sxadj[j+1]; k++){
+        ierr = PetscFindInt(sadjncy[k],nindx,indices,&loc);CHKERRQ(ierr);
+#if defined(PETSC_USE_DEBUG)
+        if (loc<0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"can not find col %D",sadjncy[k]);
+#endif
+        sadjncy[k] = loc;
+      }
+    }
+    if (scall==MAT_INITIAL_MATRIX){
+      ierr = MatCreateMPIAdj(scomm_row,irow_n,icol_n,sxadj,sadjncy,svalues,submat[i]);CHKERRQ(ierr);
+    } else {
+       Mat                sadj = *(submat[i]);
+       Mat_MPIAdj         *sa  = (Mat_MPIAdj*)((sadj)->data);
+       ierr = PetscObjectGetComm((PetscObject)sadj,&scomm_mat);CHKERRQ(ierr);
+       ierr = MPI_Comm_compare(scomm_row,scomm_mat,&issame);CHKERRQ(ierr);
+       if (issame != MPI_IDENT) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"submatrix  must have the same comm as the col index set\n");
+       ierr = PetscMemcpy(sa->i,sxadj,sizeof(PetscInt)*(irow_n+1));CHKERRQ(ierr);
+       ierr = PetscMemcpy(sa->j,sadjncy,sizeof(PetscInt)*sxadj[irow_n]);CHKERRQ(ierr);
+       if (svalues){ierr = PetscMemcpy(sa->values,svalues,sizeof(PetscInt)*sxadj[irow_n]);CHKERRQ(ierr);}
+       ierr = PetscFree(sxadj);CHKERRQ(ierr);
+       ierr = PetscFree(sadjncy);CHKERRQ(ierr);
+       if (svalues) {ierr = PetscFree(svalues);CHKERRQ(ierr);}
+    }
+  }
+  ierr = PetscFree(indices);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode MatCreateSubMatricesMPI_MPIAdj(Mat mat,PetscInt n, const IS irow[],const IS icol[],MatReuse scall,Mat *submat[])
+{
+  PetscErrorCode     ierr;
+  /*get sub-matrices across a sub communicator */
+  PetscFunctionBegin;
+  ierr = MatCreateSubMatrices_MPIAdj_Private(mat,n,irow,icol,PETSC_TRUE,scall,submat);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode MatCreateSubMatrices_MPIAdj(Mat mat,PetscInt n,const IS irow[],const IS icol[],MatReuse scall,Mat *submat[])
+{
+  PetscErrorCode     ierr;
+
+  PetscFunctionBegin;
+  /*get sub-matrices based on PETSC_COMM_SELF */
+  ierr = MatCreateSubMatrices_MPIAdj_Private(mat,n,irow,icol,PETSC_FALSE,scall,submat);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode MatView_MPIAdj_ASCII(Mat A,PetscViewer viewer)
 {
   Mat_MPIAdj        *a = (Mat_MPIAdj*)A->data;
   PetscErrorCode    ierr;
@@ -19,11 +242,10 @@ PetscErrorCode MatView_MPIAdj_ASCII(Mat A,PetscViewer viewer)
   ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
   if (format == PETSC_VIEWER_ASCII_INFO) {
     PetscFunctionReturn(0);
-  } else if (format == PETSC_VIEWER_ASCII_MATLAB) {
-    SETERRQ(PetscObjectComm((PetscObject)A),PETSC_ERR_SUP,"MATLAB format not supported");
-  } else {
+  } else if (format == PETSC_VIEWER_ASCII_MATLAB) SETERRQ(PetscObjectComm((PetscObject)A),PETSC_ERR_SUP,"MATLAB format not supported");
+  else {
     ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_TRUE);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
     for (i=0; i<m; i++) {
       ierr = PetscViewerASCIISynchronizedPrintf(viewer,"row %D:",i+A->rmap->rstart);CHKERRQ(ierr);
       for (j=a->i[i]; j<a->i[i+1]; j++) {
@@ -33,14 +255,12 @@ PetscErrorCode MatView_MPIAdj_ASCII(Mat A,PetscViewer viewer)
     }
     ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
     ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPopSynchronized(viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatView_MPIAdj"
-PetscErrorCode MatView_MPIAdj(Mat A,PetscViewer viewer)
+static PetscErrorCode MatView_MPIAdj(Mat A,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   PetscBool      iascii;
@@ -53,9 +273,7 @@ PetscErrorCode MatView_MPIAdj(Mat A,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_MPIAdj"
-PetscErrorCode MatDestroy_MPIAdj(Mat mat)
+static PetscErrorCode MatDestroy_MPIAdj(Mat mat)
 {
   Mat_MPIAdj     *a = (Mat_MPIAdj*)mat->data;
   PetscErrorCode ierr;
@@ -83,9 +301,7 @@ PetscErrorCode MatDestroy_MPIAdj(Mat mat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSetOption_MPIAdj"
-PetscErrorCode MatSetOption_MPIAdj(Mat A,MatOption op,PetscBool flg)
+static PetscErrorCode MatSetOption_MPIAdj(Mat A,MatOption op,PetscBool flg)
 {
   Mat_MPIAdj     *a = (Mat_MPIAdj*)A->data;
   PetscErrorCode ierr;
@@ -106,45 +322,14 @@ PetscErrorCode MatSetOption_MPIAdj(Mat A,MatOption op,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-
-/*
-     Adds diagonal pointers to sparse matrix structure.
-*/
-
-#undef __FUNCT__
-#define __FUNCT__ "MatMarkDiagonal_MPIAdj"
-PetscErrorCode MatMarkDiagonal_MPIAdj(Mat A)
-{
-  Mat_MPIAdj     *a = (Mat_MPIAdj*)A->data;
-  PetscErrorCode ierr;
-  PetscInt       i,j,m = A->rmap->n;
-
-  PetscFunctionBegin;
-  ierr = PetscMalloc1(m,&a->diag);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory((PetscObject)A,m*sizeof(PetscInt));CHKERRQ(ierr);
-  for (i=0; i<A->rmap->n; i++) {
-    for (j=a->i[i]; j<a->i[i+1]; j++) {
-      if (a->j[j] == i) {
-        a->diag[i] = j;
-        break;
-      }
-    }
-  }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "MatGetRow_MPIAdj"
-PetscErrorCode MatGetRow_MPIAdj(Mat A,PetscInt row,PetscInt *nz,PetscInt **idx,PetscScalar **v)
+static PetscErrorCode MatGetRow_MPIAdj(Mat A,PetscInt row,PetscInt *nz,PetscInt **idx,PetscScalar **v)
 {
   Mat_MPIAdj *a = (Mat_MPIAdj*)A->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   row -= A->rmap->rstart;
-
   if (row < 0 || row >= A->rmap->n) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Row out of range");
-
   *nz = a->i[row+1] - a->i[row];
   if (v) {
     PetscInt j;
@@ -153,24 +338,22 @@ PetscErrorCode MatGetRow_MPIAdj(Mat A,PetscInt row,PetscInt *nz,PetscInt **idx,P
       a->rowvalues_alloc = PetscMax(a->rowvalues_alloc*2, *nz);
       ierr = PetscMalloc1(a->rowvalues_alloc,&a->rowvalues);CHKERRQ(ierr);
     }
-    for (j=0; j<*nz; j++) a->rowvalues[j] = a->values[a->i[row]+j];
+    for (j=0; j<*nz; j++){
+      a->rowvalues[j] = a->values ? a->values[a->i[row]+j]:1.0;
+    }
     *v = (*nz) ? a->rowvalues : NULL;
   }
   if (idx) *idx = (*nz) ? a->j + a->i[row] : NULL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatRestoreRow_MPIAdj"
-PetscErrorCode MatRestoreRow_MPIAdj(Mat A,PetscInt row,PetscInt *nz,PetscInt **idx,PetscScalar **v)
+static PetscErrorCode MatRestoreRow_MPIAdj(Mat A,PetscInt row,PetscInt *nz,PetscInt **idx,PetscScalar **v)
 {
   PetscFunctionBegin;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatEqual_MPIAdj"
-PetscErrorCode MatEqual_MPIAdj(Mat A,Mat B,PetscBool * flg)
+static PetscErrorCode MatEqual_MPIAdj(Mat A,Mat B,PetscBool * flg)
 {
   Mat_MPIAdj     *a = (Mat_MPIAdj*)A->data,*b = (Mat_MPIAdj*)B->data;
   PetscErrorCode ierr;
@@ -188,13 +371,11 @@ PetscErrorCode MatEqual_MPIAdj(Mat A,Mat B,PetscBool * flg)
   /* if a->j are the same */
   ierr = PetscMemcmp(a->j,b->j,(a->nz)*sizeof(PetscInt),&flag);CHKERRQ(ierr);
 
-  ierr = MPI_Allreduce(&flag,flg,1,MPIU_BOOL,MPI_LAND,PetscObjectComm((PetscObject)A));CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(&flag,flg,1,MPIU_BOOL,MPI_LAND,PetscObjectComm((PetscObject)A));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatGetRowIJ_MPIAdj"
-PetscErrorCode MatGetRowIJ_MPIAdj(Mat A,PetscInt oshift,PetscBool symmetric,PetscBool blockcompressed,PetscInt *m,const PetscInt *inia[],const PetscInt *inja[],PetscBool  *done)
+static PetscErrorCode MatGetRowIJ_MPIAdj(Mat A,PetscInt oshift,PetscBool symmetric,PetscBool blockcompressed,PetscInt *m,const PetscInt *inia[],const PetscInt *inja[],PetscBool  *done)
 {
   PetscInt   i;
   Mat_MPIAdj *a   = (Mat_MPIAdj*)A->data;
@@ -214,18 +395,18 @@ PetscErrorCode MatGetRowIJ_MPIAdj(Mat A,PetscInt oshift,PetscBool symmetric,Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatRestoreRowIJ_MPIAdj"
-PetscErrorCode MatRestoreRowIJ_MPIAdj(Mat A,PetscInt oshift,PetscBool symmetric,PetscBool blockcompressed,PetscInt *m,const PetscInt *inia[],const PetscInt *inja[],PetscBool  *done)
+static PetscErrorCode MatRestoreRowIJ_MPIAdj(Mat A,PetscInt oshift,PetscBool symmetric,PetscBool blockcompressed,PetscInt *m,const PetscInt *inia[],const PetscInt *inja[],PetscBool  *done)
 {
   PetscInt   i;
   Mat_MPIAdj *a   = (Mat_MPIAdj*)A->data;
   PetscInt   **ia = (PetscInt**)inia,**ja = (PetscInt**)inja;
 
   PetscFunctionBegin;
-  if (ia && a->i != *ia) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"ia passed back is not one obtained with MatGetRowIJ()");
-  if (ja && a->j != *ja) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"ja passed back is not one obtained with MatGetRowIJ()");
+  if (ia && a->i != *ia) SETERRQ(PetscObjectComm((PetscObject)A),PETSC_ERR_ARG_WRONGSTATE,"ia passed back is not one obtained with MatGetRowIJ()");
+  if (ja && a->j != *ja) SETERRQ(PetscObjectComm((PetscObject)A),PETSC_ERR_ARG_WRONGSTATE,"ja passed back is not one obtained with MatGetRowIJ()");
   if (oshift) {
+    if (!ia) SETERRQ(PetscObjectComm((PetscObject)A),PETSC_ERR_ARG_WRONGSTATE,"If oshift then you must passed in inia[] argument");
+    if (!ja) SETERRQ(PetscObjectComm((PetscObject)A),PETSC_ERR_ARG_WRONGSTATE,"If oshift then you must passed in inja[] argument");
     for (i=0; i<=(*m); i++) (*ia)[i]--;
     for (i=0; i<(*ia)[*m]; i++) {
       (*ja)[i]--;
@@ -234,8 +415,6 @@ PetscErrorCode MatRestoreRowIJ_MPIAdj(Mat A,PetscInt oshift,PetscBool symmetric,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatConvertFrom_MPIAdj"
 PetscErrorCode  MatConvertFrom_MPIAdj(Mat A,MatType type,MatReuse reuse,Mat *newmat)
 {
   Mat               B;
@@ -287,8 +466,8 @@ PetscErrorCode  MatConvertFrom_MPIAdj(Mat A,MatType type,MatReuse reuse,Mat *new
   ierr = MatSetType(B,type);CHKERRQ(ierr);
   ierr = MatMPIAdjSetPreallocation(B,ia,ja,a);CHKERRQ(ierr);
 
-  if (reuse == MAT_REUSE_MATRIX) {
-    ierr = MatHeaderReplace(A,B);CHKERRQ(ierr);
+  if (reuse == MAT_INPLACE_MATRIX) {
+    ierr = MatHeaderReplace(A,&B);CHKERRQ(ierr);
   } else {
     *newmat = B;
   }
@@ -336,7 +515,7 @@ static struct _MatOps MatOps_Values = {0,
                                        0,
                                        0,
                                 /*39*/ 0,
-                                       0,
+                                       MatCreateSubMatrices_MPIAdj,
                                        0,
                                        0,
                                        0,
@@ -424,7 +603,7 @@ static struct _MatOps MatOps_Values = {0,
                                        0,
                                        0,
                                        0,
-                                       0,
+                                       MatCreateSubMatricesMPI_MPIAdj,
                                /*129*/ 0,
                                        0,
                                        0,
@@ -440,8 +619,6 @@ static struct _MatOps MatOps_Values = {0,
                                        0
 };
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMPIAdjSetPreallocation_MPIAdj"
 static PetscErrorCode  MatMPIAdjSetPreallocation_MPIAdj(Mat B,PetscInt *i,PetscInt *j,PetscInt *values)
 {
   Mat_MPIAdj     *b = (Mat_MPIAdj*)B->data;
@@ -479,8 +656,6 @@ static PetscErrorCode  MatMPIAdjSetPreallocation_MPIAdj(Mat B,PetscInt *i,PetscI
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMPIAdjCreateNonemptySubcommMat_MPIAdj"
 static PetscErrorCode MatMPIAdjCreateNonemptySubcommMat_MPIAdj(Mat A,Mat *B)
 {
   Mat_MPIAdj     *a = (Mat_MPIAdj*)A->data;
@@ -528,8 +703,51 @@ static PetscErrorCode MatMPIAdjCreateNonemptySubcommMat_MPIAdj(Mat A,Mat *B)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMPIAdjCreateNonemptySubcommMat"
+PetscErrorCode  MatMPIAdjToSeq_MPIAdj(Mat A,Mat *B)
+{
+  PetscErrorCode ierr;
+  PetscInt       M,N,*II,*J,NZ,nz,m,nzstart,i;
+  PetscInt       *Values = NULL;
+  Mat_MPIAdj     *adj = (Mat_MPIAdj*)A->data;
+  PetscMPIInt    mnz,mm,*allnz,*allm,size,*dispnz,*dispm;
+
+  PetscFunctionBegin;
+  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)A),&size);CHKERRQ(ierr);
+  ierr = MatGetSize(A,&M,&N);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(A,&m,NULL);CHKERRQ(ierr);
+  nz   = adj->nz;
+  if (adj->i[m] != nz) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"nz %D not correct i[m] %d",nz,adj->i[m]);
+  ierr = MPI_Allreduce(&nz,&NZ,1,MPIU_INT,MPI_SUM,PetscObjectComm((PetscObject)A));CHKERRQ(ierr);
+
+  ierr = PetscMPIIntCast(nz,&mnz);CHKERRQ(ierr);
+  ierr = PetscCalloc2(size,&allnz,size,&dispnz);CHKERRQ(ierr);
+  ierr = MPI_Allgather(&mnz,1,MPI_INT,allnz,1,MPI_INT,PetscObjectComm((PetscObject)A));CHKERRQ(ierr);
+  dispnz[0] = 0; for (i=1; i<size; i++) dispnz[i] = dispnz[i-1]+ allnz[i-1];
+  if (adj->values) {
+    ierr = PetscMalloc1(NZ,&Values);CHKERRQ(ierr);
+    ierr = MPI_Allgatherv(adj->values,mnz,MPIU_INT,Values,allnz,dispnz,MPIU_INT,PetscObjectComm((PetscObject)A));CHKERRQ(ierr);
+  }
+  ierr = PetscMalloc1(NZ,&J);CHKERRQ(ierr);
+  ierr = MPI_Allgatherv(adj->j,mnz,MPIU_INT,J,allnz,dispnz,MPIU_INT,PetscObjectComm((PetscObject)A));CHKERRQ(ierr);
+  ierr = PetscFree2(allnz,dispnz);CHKERRQ(ierr);
+  ierr = MPI_Scan(&nz,&nzstart,1,MPIU_INT,MPI_SUM,PetscObjectComm((PetscObject)A));CHKERRQ(ierr);
+  nzstart -= nz;
+  /* shift the i[] values so they will be correct after being received */
+  for (i=0; i<m; i++) adj->i[i] += nzstart;
+  ierr = PetscMalloc1(M+1,&II);CHKERRQ(ierr);
+  ierr = PetscMPIIntCast(m,&mm);CHKERRQ(ierr);
+  ierr = PetscMalloc2(size,&allm,size,&dispm);CHKERRQ(ierr);
+  ierr = MPI_Allgather(&mm,1,MPI_INT,allm,1,MPI_INT,PetscObjectComm((PetscObject)A));CHKERRQ(ierr);
+  dispm[0] = 0; for (i=1; i<size; i++) dispm[i] = dispm[i-1]+ allm[i-1];
+  ierr = MPI_Allgatherv(adj->i,mm,MPIU_INT,II,allm,dispm,MPIU_INT,PetscObjectComm((PetscObject)A));CHKERRQ(ierr);
+  ierr = PetscFree2(allm,dispm);CHKERRQ(ierr);
+  II[M] = NZ;
+  /* shift the i[] values back */
+  for (i=0; i<m; i++) adj->i[i] -= nzstart;
+  ierr = MatCreateMPIAdj(PETSC_COMM_SELF,M,N,II,J,Values,B);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    MatMPIAdjCreateNonemptySubcommMat - create the same MPIAdj matrix on a subcommunicator containing only processes owning a positive number of rows
 
@@ -569,8 +787,6 @@ PetscErrorCode MatMPIAdjCreateNonemptySubcommMat(Mat A,Mat *B)
 .seealso: MatCreateMPIAdj
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCreate_MPIAdj"
 PETSC_EXTERN PetscErrorCode MatCreate_MPIAdj(Mat B)
 {
   Mat_MPIAdj     *b;
@@ -584,12 +800,35 @@ PETSC_EXTERN PetscErrorCode MatCreate_MPIAdj(Mat B)
 
   ierr = PetscObjectComposeFunction((PetscObject)B,"MatMPIAdjSetPreallocation_C",MatMPIAdjSetPreallocation_MPIAdj);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)B,"MatMPIAdjCreateNonemptySubcommMat_C",MatMPIAdjCreateNonemptySubcommMat_MPIAdj);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)B,"MatMPIAdjToSeq_C",MatMPIAdjToSeq_MPIAdj);CHKERRQ(ierr);
   ierr = PetscObjectChangeTypeName((PetscObject)B,MATMPIADJ);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMPIAdjSetPreallocation"
+/*@C
+   MatMPIAdjToSeq - Converts an parallel MPIAdj matrix to complete MPIAdj on each process (needed by sequential preconditioners)
+
+   Logically Collective on MPI_Comm
+
+   Input Parameter:
+.  A - the matrix
+
+   Output Parameter:
+.  B - the same matrix on all processes
+
+   Level: intermediate
+
+.seealso: MatCreate(), MatCreateMPIAdj(), MatSetValues()
+@*/
+PetscErrorCode  MatMPIAdjToSeq(Mat A,Mat *B)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscUseMethod(A,"MatMPIAdjToSeq_C",(Mat,Mat*),(A,B));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@C
    MatMPIAdjSetPreallocation - Sets the array used for storing the matrix elements
 
@@ -615,8 +854,6 @@ PetscErrorCode  MatMPIAdjSetPreallocation(Mat B,PetscInt *i,PetscInt *j,PetscInt
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCreateMPIAdj"
 /*@C
    MatCreateMPIAdj - Creates a sparse matrix representing an adjacency list.
    The matrix does not have numerical values associated with it, but is
@@ -638,7 +875,8 @@ PetscErrorCode  MatMPIAdjSetPreallocation(Mat B,PetscInt *i,PetscInt *j,PetscInt
 
    Level: intermediate
 
-   Notes: This matrix object does not support most matrix operations, include
+   Notes:
+    This matrix object does not support most matrix operations, include
    MatSetValues().
    You must NOT free the ii, values and jj arrays yourself. PETSc will free them
    when the matrix is destroyed; you must allocate them with PetscMalloc(). If you
