@@ -1,13 +1,11 @@
-
+#include <../src/mat/impls/sell/mpi/mpisell.h>
 #include <../src/mat/impls/aij/mpi/mpiaij.h>
 #include <../src/mat/impls/baij/mpi/mpibaij.h>
 #include <petsc/private/isimpl.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringApply_BAIJ"
-PetscErrorCode  MatFDColoringApply_BAIJ(Mat J,MatFDColoring coloring,Vec x1,void *sctx)
+PetscErrorCode MatFDColoringApply_BAIJ(Mat J,MatFDColoring coloring,Vec x1,void *sctx)
 {
-  PetscErrorCode    (*f)(void*,Vec,Vec,void*) = (PetscErrorCode (*)(void*,Vec,Vec,void*))coloring->f;
+  PetscErrorCode    (*f)(void*,Vec,Vec,void*)=(PetscErrorCode (*)(void*,Vec,Vec,void*))coloring->f;
   PetscErrorCode    ierr;
   PetscInt          k,cstart,cend,l,row,col,nz,spidx,i,j;
   PetscScalar       dx=0.0,*w3_array,*dy_i,*dy=coloring->dy;
@@ -26,9 +24,9 @@ PetscErrorCode  MatFDColoringApply_BAIJ(Mat J,MatFDColoring coloring,Vec x1,void
   PetscFunctionBegin;
   /* (1) Set w1 = F(x1) */
   if (!coloring->fset) {
-    ierr = PetscLogEventBegin(MAT_FDColoringFunction,0,0,0,0);CHKERRQ(ierr);
+    ierr = PetscLogEventBegin(MAT_FDColoringFunction,coloring,0,0,0);CHKERRQ(ierr);
     ierr = (*f)(sctx,x1,w1,fctx);CHKERRQ(ierr);
-    ierr = PetscLogEventEnd(MAT_FDColoringFunction,0,0,0,0);CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(MAT_FDColoringFunction,coloring,0,0,0);CHKERRQ(ierr);
   } else {
     coloring->fset = PETSC_FALSE;
   }
@@ -157,8 +155,7 @@ PetscErrorCode  MatFDColoringApply_BAIJ(Mat J,MatFDColoring coloring,Vec x1,void
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringApply_AIJ"
+/* this is declared PETSC_EXTERN because it is used by MatFDColoringUseDM() which is in the DM library */
 PetscErrorCode  MatFDColoringApply_AIJ(Mat J,MatFDColoring coloring,Vec x1,void *sctx)
 {
   PetscErrorCode    (*f)(void*,Vec,Vec,void*) = (PetscErrorCode (*)(void*,Vec,Vec,void*))coloring->f;
@@ -170,12 +167,14 @@ PetscErrorCode  MatFDColoringApply_AIJ(Mat J,MatFDColoring coloring,Vec x1,void
   PetscReal         epsilon=coloring->error_rel,umin=coloring->umin,unorm;
   Vec               w1=coloring->w1,w2=coloring->w2,w3,vscale=coloring->vscale;
   void              *fctx=coloring->fctx;
-  PetscInt          ctype=coloring->ctype,nxloc,nrows_k;
+  ISColoringType    ctype=coloring->ctype;
+  PetscInt          nxloc,nrows_k;
   MatEntry          *Jentry=coloring->matentry;
   MatEntry2         *Jentry2=coloring->matentry2;
   const PetscInt    ncolors=coloring->ncolors,*ncolumns=coloring->ncolumns,*nrows=coloring->nrows;
 
   PetscFunctionBegin;
+  if ((ctype == IS_COLORING_LOCAL) && (J->ops->fdcoloringapply == MatFDColoringApply_AIJ)) SETERRQ(PetscObjectComm((PetscObject)J),PETSC_ERR_SUP,"Must call MatColoringUseDM() with IS_COLORING_LOCAL");
   /* (1) Set w1 = F(x1) */
   if (!coloring->fset) {
     ierr = PetscLogEventBegin(MAT_FDColoringFunction,0,0,0,0);CHKERRQ(ierr);
@@ -230,7 +229,6 @@ PetscErrorCode  MatFDColoringApply_AIJ(Mat J,MatFDColoring coloring,Vec x1,void
 
     nbcols = 0;
     for (k=0; k<ncolors; k+=bcols) {
-      coloring->currentcolor = k;
 
       /*
        (3-1) Loop over each column associated with color
@@ -240,6 +238,7 @@ PetscErrorCode  MatFDColoringApply_AIJ(Mat J,MatFDColoring coloring,Vec x1,void
       dy_k = dy;
       if (k + bcols > ncolors) bcols = ncolors - k;
       for (i=0; i<bcols; i++) {
+        coloring->currentcolor = k+i;
 
         ierr = VecCopy(x1,w3);CHKERRQ(ierr);
         ierr = VecGetArray(w3,&w3_array);CHKERRQ(ierr);
@@ -359,8 +358,6 @@ PetscErrorCode  MatFDColoringApply_AIJ(Mat J,MatFDColoring coloring,Vec x1,void
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringSetUp_MPIXAIJ"
 PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDColoring c)
 {
   PetscErrorCode         ierr;
@@ -375,7 +372,7 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
   PetscScalar            *A_val,*B_val,**valaddrhit;
   MatEntry               *Jentry;
   MatEntry2              *Jentry2;
-  PetscBool              isBAIJ;
+  PetscBool              isBAIJ,isSELL;
   PetscInt               bcols=c->bcols;
 #if defined(PETSC_USE_CTABLE)
   PetscTable             colmap=NULL;
@@ -384,13 +381,14 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
 #endif
 
   PetscFunctionBegin;
-  if (ctype == IS_COLORING_GHOSTED) {
+  if (ctype == IS_COLORING_LOCAL) {
     if (!map) SETERRQ(PetscObjectComm((PetscObject)mat),PETSC_ERR_ARG_INCOMP,"When using ghosted differencing matrix must have local to global mapping provided with MatSetLocalToGlobalMapping");
     ierr = ISLocalToGlobalMappingGetIndices(map,&ltog);CHKERRQ(ierr);
   }
 
   ierr = MatGetBlockSize(mat,&bs);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)mat,MATMPIBAIJ,&isBAIJ);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)mat,MATMPISELL,&isSELL);CHKERRQ(ierr);
   if (isBAIJ) {
     Mat_MPIBAIJ *baij=(Mat_MPIBAIJ*)mat->data;
     Mat_SeqBAIJ *spA,*spB;
@@ -399,8 +397,8 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
     nz = spA->nz + spB->nz; /* total nonzero entries of mat */
     if (!baij->colmap) {
       ierr = MatCreateColmap_MPIBAIJ_Private(mat);CHKERRQ(ierr);
-      colmap = baij->colmap;
     }
+    colmap = baij->colmap;
     ierr = MatGetColumnIJ_SeqBAIJ_Color(A,0,PETSC_FALSE,PETSC_FALSE,&ncols,&A_ci,&A_cj,&spidxA,NULL);CHKERRQ(ierr);
     ierr = MatGetColumnIJ_SeqBAIJ_Color(B,0,PETSC_FALSE,PETSC_FALSE,&ncols,&B_ci,&B_cj,&spidxB,NULL);CHKERRQ(ierr);
 
@@ -415,6 +413,26 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
       ierr = VecCreateGhost(PetscObjectComm((PetscObject)mat),mat->cmap->n,PETSC_DETERMINE,B->cmap->n,garray,&c->vscale);CHKERRQ(ierr);
       ierr = PetscFree(garray);CHKERRQ(ierr);
     }
+  } else if (isSELL) {
+    Mat_MPISELL *sell=(Mat_MPISELL*)mat->data;
+    Mat_SeqSELL *spA,*spB;
+    A = sell->A;  spA = (Mat_SeqSELL*)A->data; A_val = spA->val;
+    B = sell->B;  spB = (Mat_SeqSELL*)B->data; B_val = spB->val;
+    nz = spA->nz + spB->nz; /* total nonzero entries of mat */
+    if (!sell->colmap) {
+      /* Allow access to data structures of local part of matrix
+       - creates aij->colmap which maps global column number to local number in part B */
+      ierr = MatCreateColmap_MPISELL_Private(mat);CHKERRQ(ierr);
+    }
+    colmap = sell->colmap;
+    ierr = MatGetColumnIJ_SeqSELL_Color(A,0,PETSC_FALSE,PETSC_FALSE,&ncols,&A_ci,&A_cj,&spidxA,NULL);CHKERRQ(ierr);
+    ierr = MatGetColumnIJ_SeqSELL_Color(B,0,PETSC_FALSE,PETSC_FALSE,&ncols,&B_ci,&B_cj,&spidxB,NULL);CHKERRQ(ierr);
+
+    bs = 1; /* only bs=1 is supported for non MPIBAIJ matrix */
+
+    if (ctype == IS_COLORING_GLOBAL && c->htype[0] == 'd') { /* create vscale for storing dx */
+      ierr = VecCreateGhost(PetscObjectComm((PetscObject)mat),mat->cmap->n,PETSC_DETERMINE,B->cmap->n,sell->garray,&c->vscale);CHKERRQ(ierr);
+    }
   } else {
     Mat_MPIAIJ *aij=(Mat_MPIAIJ*)mat->data;
     Mat_SeqAIJ *spA,*spB;
@@ -425,8 +443,8 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
       /* Allow access to data structures of local part of matrix
        - creates aij->colmap which maps global column number to local number in part B */
       ierr = MatCreateColmap_MPIAIJ_Private(mat);CHKERRQ(ierr);
-      colmap = aij->colmap;
     }
+    colmap = aij->colmap;
     ierr = MatGetColumnIJ_SeqAIJ_Color(A,0,PETSC_FALSE,PETSC_FALSE,&ncols,&A_ci,&A_cj,&spidxA,NULL);CHKERRQ(ierr);
     ierr = MatGetColumnIJ_SeqAIJ_Color(B,0,PETSC_FALSE,PETSC_FALSE,&ncols,&B_ci,&B_cj,&spidxB,NULL);CHKERRQ(ierr);
 
@@ -443,7 +461,7 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
 
   ierr       = PetscMalloc1(nis,&c->ncolumns);CHKERRQ(ierr);
   ierr       = PetscMalloc1(nis,&c->columns);CHKERRQ(ierr);
-  ierr       = PetscMalloc1(nis,&c->nrows);CHKERRQ(ierr);
+  ierr       = PetscCalloc1(nis,&c->nrows);CHKERRQ(ierr);
   ierr       = PetscLogObjectMemory((PetscObject)c,3*nis*sizeof(PetscInt));CHKERRQ(ierr);
 
   if (c->htype[0] == 'd') {
@@ -494,7 +512,7 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
       ierr = PetscMalloc1(nctot+1,&cols);CHKERRQ(ierr);
       ierr = MPI_Allgatherv((void*)is,n,MPIU_INT,cols,ncolsonproc,disp,MPIU_INT,PetscObjectComm((PetscObject)mat));CHKERRQ(ierr);
       ierr = PetscFree2(ncolsonproc,disp);CHKERRQ(ierr);
-    } else if (ctype == IS_COLORING_GHOSTED) {
+    } else if (ctype == IS_COLORING_LOCAL) {
       /* Determine local number of columns of this color on this process, including ghost points */
       nctot = n;
       ierr  = PetscMalloc1(nctot+1,&cols);CHKERRQ(ierr);
@@ -506,7 +524,7 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
     bs2     = bs*bs;
     nrows_i = 0;
     for (j=0; j<nctot; j++) { /* loop over columns*/
-      if (ctype == IS_COLORING_GHOSTED) {
+      if (ctype == IS_COLORING_LOCAL) {
         col = ltog[cols[j]];
       } else {
         col = cols[j];
@@ -577,7 +595,10 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
     ierr = MatRestoreColumnIJ_SeqBAIJ_Color(A,0,PETSC_FALSE,PETSC_FALSE,&ncols,&A_ci,&A_cj,&spidxA,NULL);CHKERRQ(ierr);
     ierr = MatRestoreColumnIJ_SeqBAIJ_Color(B,0,PETSC_FALSE,PETSC_FALSE,&ncols,&B_ci,&B_cj,&spidxB,NULL);CHKERRQ(ierr);
     ierr = PetscMalloc1(bs*mat->rmap->n,&c->dy);CHKERRQ(ierr);
-  } else {
+  } else if (isSELL) {
+    ierr = MatRestoreColumnIJ_SeqSELL_Color(A,0,PETSC_FALSE,PETSC_FALSE,&ncols,&A_ci,&A_cj,&spidxA,NULL);CHKERRQ(ierr);
+    ierr = MatRestoreColumnIJ_SeqSELL_Color(B,0,PETSC_FALSE,PETSC_FALSE,&ncols,&B_ci,&B_cj,&spidxB,NULL);CHKERRQ(ierr);
+  }else {
     ierr = MatRestoreColumnIJ_SeqAIJ_Color(A,0,PETSC_FALSE,PETSC_FALSE,&ncols,&A_ci,&A_cj,&spidxA,NULL);CHKERRQ(ierr);
     ierr = MatRestoreColumnIJ_SeqAIJ_Color(B,0,PETSC_FALSE,PETSC_FALSE,&ncols,&B_ci,&B_cj,&spidxB,NULL);CHKERRQ(ierr);
   }
@@ -585,29 +606,48 @@ PetscErrorCode MatFDColoringSetUp_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDCol
   ierr = ISColoringRestoreIS(iscoloring,&isa);CHKERRQ(ierr);
   ierr = PetscFree2(rowhit,valaddrhit);CHKERRQ(ierr);
 
-  if (ctype == IS_COLORING_GHOSTED) {
+  if (ctype == IS_COLORING_LOCAL) {
     ierr = ISLocalToGlobalMappingRestoreIndices(map,&ltog);CHKERRQ(ierr);
   }
   ierr = PetscInfo3(c,"ncolors %D, brows %D and bcols %D are used.\n",c->ncolors,c->brows,c->bcols);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringCreate_MPIXAIJ"
 PetscErrorCode MatFDColoringCreate_MPIXAIJ(Mat mat,ISColoring iscoloring,MatFDColoring c)
 {
   PetscErrorCode ierr;
   PetscInt       bs,nis=iscoloring->n,m=mat->rmap->n;
-  PetscBool      isBAIJ;
+  PetscBool      isBAIJ,isSELL;
 
   PetscFunctionBegin;
   /* set default brows and bcols for speedup inserting the dense matrix into sparse Jacobian;
    bcols is chosen s.t. dy-array takes 50% of memory space as mat */
   ierr = MatGetBlockSize(mat,&bs);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)mat,MATMPIBAIJ,&isBAIJ);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)mat,MATMPISELL,&isSELL);CHKERRQ(ierr);
   if (isBAIJ || m == 0) {
     c->brows = m;
     c->bcols = 1;
+  } else if (isSELL) {
+    /* bcols is chosen s.t. dy-array takes 50% of local memory space as mat */
+    Mat_MPISELL *sell=(Mat_MPISELL*)mat->data;
+    Mat_SeqSELL *spA,*spB;
+    Mat        A,B;
+    PetscInt   nz,brows,bcols;
+    PetscReal  mem;
+
+    bs    = 1; /* only bs=1 is supported for MPISELL matrix */
+
+    A = sell->A;  spA = (Mat_SeqSELL*)A->data;
+    B = sell->B;  spB = (Mat_SeqSELL*)B->data;
+    nz = spA->nz + spB->nz; /* total local nonzero entries of mat */
+    mem = nz*(sizeof(PetscScalar) + sizeof(PetscInt)) + 3*m*sizeof(PetscInt);
+    bcols = (PetscInt)(0.5*mem /(m*sizeof(PetscScalar)));
+    brows = 1000/bcols;
+    if (bcols > nis) bcols = nis;
+    if (brows == 0 || brows > m) brows = m;
+    c->brows = brows;
+    c->bcols = bcols;
   } else { /* mpiaij matrix */
     /* bcols is chosen s.t. dy-array takes 50% of local memory space as mat */
     Mat_MPIAIJ *aij=(Mat_MPIAIJ*)mat->data;
