@@ -3,10 +3,9 @@
    Support for the parallel AIJ matrix vector multiply
 */
 #include <../src/mat/impls/aij/mpi/mpiaij.h>
-#include <petsc-private/isimpl.h>    /* needed because accesses data structure of ISLocalToGlobalMapping directly */
+#include <petsc/private/vecimpl.h>
+#include <petsc/private/isimpl.h>    /* needed because accesses data structure of ISLocalToGlobalMapping directly */
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSetUpMultiply_MPIAIJ"
 PetscErrorCode MatSetUpMultiply_MPIAIJ(Mat mat)
 {
   Mat_MPIAIJ     *aij = (Mat_MPIAIJ*)mat->data;
@@ -24,83 +23,93 @@ PetscErrorCode MatSetUpMultiply_MPIAIJ(Mat mat)
 #endif
 
   PetscFunctionBegin;
+  if (!aij->garray) {
 #if defined(PETSC_USE_CTABLE)
-  /* use a table */
-  ierr = PetscTableCreate(aij->B->rmap->n,mat->cmap->N+1,&gid1_lid1);CHKERRQ(ierr);
-  for (i=0; i<aij->B->rmap->n; i++) {
-    for (j=0; j<B->ilen[i]; j++) {
-      PetscInt data,gid1 = aj[B->i[i] + j] + 1;
-      ierr = PetscTableFind(gid1_lid1,gid1,&data);CHKERRQ(ierr);
-      if (!data) {
-        /* one based table */
-        ierr = PetscTableAdd(gid1_lid1,gid1,++ec,INSERT_VALUES);CHKERRQ(ierr);
+    /* use a table */
+    ierr = PetscTableCreate(aij->B->rmap->n,mat->cmap->N+1,&gid1_lid1);CHKERRQ(ierr);
+    for (i=0; i<aij->B->rmap->n; i++) {
+      for (j=0; j<B->ilen[i]; j++) {
+        PetscInt data,gid1 = aj[B->i[i] + j] + 1;
+        ierr = PetscTableFind(gid1_lid1,gid1,&data);CHKERRQ(ierr);
+        if (!data) {
+          /* one based table */
+          ierr = PetscTableAdd(gid1_lid1,gid1,++ec,INSERT_VALUES);CHKERRQ(ierr);
+        }
       }
     }
-  }
-  /* form array of columns we need */
-  ierr = PetscMalloc((ec+1)*sizeof(PetscInt),&garray);CHKERRQ(ierr);
-  ierr = PetscTableGetHeadPosition(gid1_lid1,&tpos);CHKERRQ(ierr);
-  while (tpos) {
-    ierr = PetscTableGetNext(gid1_lid1,&tpos,&gid,&lid);CHKERRQ(ierr);
-    gid--;
-    lid--;
-    garray[lid] = gid;
-  }
-  ierr = PetscSortInt(ec,garray);CHKERRQ(ierr); /* sort, and rebuild */
-  ierr = PetscTableRemoveAll(gid1_lid1);CHKERRQ(ierr);
-  for (i=0; i<ec; i++) {
-    ierr = PetscTableAdd(gid1_lid1,garray[i]+1,i+1,INSERT_VALUES);CHKERRQ(ierr);
-  }
-  /* compact out the extra columns in B */
-  for (i=0; i<aij->B->rmap->n; i++) {
-    for (j=0; j<B->ilen[i]; j++) {
-      PetscInt gid1 = aj[B->i[i] + j] + 1;
-      ierr = PetscTableFind(gid1_lid1,gid1,&lid);CHKERRQ(ierr);
+    /* form array of columns we need */
+    ierr = PetscMalloc1(ec+1,&garray);CHKERRQ(ierr);
+    ierr = PetscTableGetHeadPosition(gid1_lid1,&tpos);CHKERRQ(ierr);
+    while (tpos) {
+      ierr = PetscTableGetNext(gid1_lid1,&tpos,&gid,&lid);CHKERRQ(ierr);
+      gid--;
       lid--;
-      aj[B->i[i] + j] = lid;
+      garray[lid] = gid;
     }
-  }
-  aij->B->cmap->n = aij->B->cmap->N = ec;
+    ierr = PetscSortInt(ec,garray);CHKERRQ(ierr); /* sort, and rebuild */
+    ierr = PetscTableRemoveAll(gid1_lid1);CHKERRQ(ierr);
+    for (i=0; i<ec; i++) {
+      ierr = PetscTableAdd(gid1_lid1,garray[i]+1,i+1,INSERT_VALUES);CHKERRQ(ierr);
+    }
+    /* compact out the extra columns in B */
+    for (i=0; i<aij->B->rmap->n; i++) {
+      for (j=0; j<B->ilen[i]; j++) {
+        PetscInt gid1 = aj[B->i[i] + j] + 1;
+        ierr = PetscTableFind(gid1_lid1,gid1,&lid);CHKERRQ(ierr);
+        lid--;
+        aj[B->i[i] + j] = lid;
+      }
+    }
+    aij->B->cmap->n = aij->B->cmap->N = ec;
+    aij->B->cmap->bs = 1;
 
-  ierr = PetscLayoutSetUp((aij->B->cmap));CHKERRQ(ierr);
-  ierr = PetscTableDestroy(&gid1_lid1);CHKERRQ(ierr);
+    ierr = PetscLayoutSetUp((aij->B->cmap));CHKERRQ(ierr);
+    ierr = PetscTableDestroy(&gid1_lid1);CHKERRQ(ierr);
 #else
-  /* Make an array as long as the number of columns */
-  /* mark those columns that are in aij->B */
-  ierr = PetscMalloc((N+1)*sizeof(PetscInt),&indices);CHKERRQ(ierr);
-  ierr = PetscMemzero(indices,N*sizeof(PetscInt));CHKERRQ(ierr);
-  for (i=0; i<aij->B->rmap->n; i++) {
-    for (j=0; j<B->ilen[i]; j++) {
-      if (!indices[aj[B->i[i] + j]]) ec++;
-      indices[aj[B->i[i] + j]] = 1;
+    /* Make an array as long as the number of columns */
+    /* mark those columns that are in aij->B */
+    ierr = PetscCalloc1(N+1,&indices);CHKERRQ(ierr);
+    for (i=0; i<aij->B->rmap->n; i++) {
+      for (j=0; j<B->ilen[i]; j++) {
+        if (!indices[aj[B->i[i] + j]]) ec++;
+        indices[aj[B->i[i] + j]] = 1;
+      }
     }
-  }
 
-  /* form array of columns we need */
-  ierr = PetscMalloc((ec+1)*sizeof(PetscInt),&garray);CHKERRQ(ierr);
-  ec   = 0;
-  for (i=0; i<N; i++) {
-    if (indices[i]) garray[ec++] = i;
-  }
-
-  /* make indices now point into garray */
-  for (i=0; i<ec; i++) {
-    indices[garray[i]] = i;
-  }
-
-  /* compact out the extra columns in B */
-  for (i=0; i<aij->B->rmap->n; i++) {
-    for (j=0; j<B->ilen[i]; j++) {
-      aj[B->i[i] + j] = indices[aj[B->i[i] + j]];
+    /* form array of columns we need */
+    ierr = PetscMalloc1(ec+1,&garray);CHKERRQ(ierr);
+    ec   = 0;
+    for (i=0; i<N; i++) {
+      if (indices[i]) garray[ec++] = i;
     }
-  }
-  aij->B->cmap->n = aij->B->cmap->N = ec;
 
-  ierr = PetscLayoutSetUp((aij->B->cmap));CHKERRQ(ierr);
-  ierr = PetscFree(indices);CHKERRQ(ierr);
+    /* make indices now point into garray */
+    for (i=0; i<ec; i++) {
+      indices[garray[i]] = i;
+    }
+
+    /* compact out the extra columns in B */
+    for (i=0; i<aij->B->rmap->n; i++) {
+      for (j=0; j<B->ilen[i]; j++) {
+        aj[B->i[i] + j] = indices[aj[B->i[i] + j]];
+      }
+    }
+    aij->B->cmap->n = aij->B->cmap->N = ec;
+    aij->B->cmap->bs = 1;
+
+    ierr = PetscLayoutSetUp((aij->B->cmap));CHKERRQ(ierr);
+    ierr = PetscFree(indices);CHKERRQ(ierr);
 #endif
-  /* create local vector that is used to scatter into */
-  ierr = VecCreateSeq(PETSC_COMM_SELF,ec,&aij->lvec);CHKERRQ(ierr);
+  } else {
+    garray = aij->garray;
+  }
+
+  if (!aij->lvec) {
+    /* create local vector that is used to scatter into */
+    ierr = VecCreateSeq(PETSC_COMM_SELF,ec,&aij->lvec);CHKERRQ(ierr);
+  } else {
+    ierr = VecGetSize(aij->lvec,&ec);CHKERRQ(ierr);
+  }
 
   /* create two temporary Index sets for build scatter gather */
   ierr = ISCreateGeneral(((PetscObject)mat)->comm,ec,garray,PETSC_COPY_VALUES,&from);CHKERRQ(ierr);
@@ -112,24 +121,29 @@ PetscErrorCode MatSetUpMultiply_MPIAIJ(Mat mat)
   ierr = VecCreateMPIWithArray(PetscObjectComm((PetscObject)mat),1,mat->cmap->n,mat->cmap->N,NULL,&gvec);CHKERRQ(ierr);
 
   /* generate the scatter context */
-  ierr = VecScatterCreate(gvec,from,aij->lvec,to,&aij->Mvctx);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent(mat,aij->Mvctx);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent(mat,aij->lvec);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent(mat,from);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent(mat,to);CHKERRQ(ierr);
-
+  if (aij->Mvctx_mpi1_flg) {
+    ierr = VecScatterDestroy(&aij->Mvctx_mpi1);CHKERRQ(ierr);
+    ierr = VecScatterCreate(gvec,from,aij->lvec,to,&aij->Mvctx_mpi1);CHKERRQ(ierr);
+    ierr = VecScatterSetType(aij->Mvctx_mpi1,VECSCATTERMPI1);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)mat,(PetscObject)aij->Mvctx_mpi1);CHKERRQ(ierr);
+  } else {
+    ierr = VecScatterDestroy(&aij->Mvctx);CHKERRQ(ierr);
+    ierr = VecScatterCreate(gvec,from,aij->lvec,to,&aij->Mvctx);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)mat,(PetscObject)aij->Mvctx);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)mat,(PetscObject)aij->lvec);CHKERRQ(ierr);
+    ierr = PetscLogObjectMemory((PetscObject)mat,(ec+1)*sizeof(PetscInt));CHKERRQ(ierr);
+  }
   aij->garray = garray;
 
-  ierr = PetscLogObjectMemory(mat,(ec+1)*sizeof(PetscInt));CHKERRQ(ierr);
+  ierr = PetscLogObjectParent((PetscObject)mat,(PetscObject)from);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent((PetscObject)mat,(PetscObject)to);CHKERRQ(ierr);
+
   ierr = ISDestroy(&from);CHKERRQ(ierr);
   ierr = ISDestroy(&to);CHKERRQ(ierr);
   ierr = VecDestroy(&gvec);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "MatDisAssemble_MPIAIJ"
 /*
      Takes the local part of an already assembled MPIAIJ matrix
    and disassembles it. This is to allow new nonzeros into the matrix
@@ -152,13 +166,12 @@ PetscErrorCode MatDisAssemble_MPIAIJ(Mat A)
   /* free stuff related to matrix-vec multiply */
   ierr = VecGetSize(aij->lvec,&ec);CHKERRQ(ierr); /* needed for PetscLogObjectMemory below */
   ierr = VecDestroy(&aij->lvec);CHKERRQ(ierr);
-  ierr = VecScatterDestroy(&aij->Mvctx);CHKERRQ(ierr);
   if (aij->colmap) {
 #if defined(PETSC_USE_CTABLE)
     ierr = PetscTableDestroy(&aij->colmap);CHKERRQ(ierr);
 #else
     ierr = PetscFree(aij->colmap);CHKERRQ(ierr);
-    ierr = PetscLogObjectMemory(A,-aij->B->cmap->n*sizeof(PetscInt));CHKERRQ(ierr);
+    ierr = PetscLogObjectMemory((PetscObject)A,-aij->B->cmap->n*sizeof(PetscInt));CHKERRQ(ierr);
 #endif
   }
 
@@ -167,17 +180,25 @@ PetscErrorCode MatDisAssemble_MPIAIJ(Mat A)
   ierr = MatAssemblyEnd(B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
 
   /* invent new B and copy stuff over */
-  ierr = PetscMalloc((m+1)*sizeof(PetscInt),&nz);CHKERRQ(ierr);
+  ierr = PetscMalloc1(m+1,&nz);CHKERRQ(ierr);
   for (i=0; i<m; i++) {
     nz[i] = Baij->i[i+1] - Baij->i[i];
   }
   ierr = MatCreate(PETSC_COMM_SELF,&Bnew);CHKERRQ(ierr);
   ierr = MatSetSizes(Bnew,m,n,m,n);CHKERRQ(ierr);
-  ierr = MatSetBlockSizes(Bnew,A->rmap->bs,A->cmap->bs);CHKERRQ(ierr);
+  ierr = MatSetBlockSizesFromMats(Bnew,A,A);CHKERRQ(ierr);
   ierr = MatSetType(Bnew,((PetscObject)B)->type_name);CHKERRQ(ierr);
   ierr = MatSeqAIJSetPreallocation(Bnew,0,nz);CHKERRQ(ierr);
 
-  ((Mat_SeqAIJ*)Bnew->data)->nonew = Baij->nonew; /* Inherit insertion error options. */
+  if (Baij->nonew >= 0) { /* Inherit insertion error options (if positive). */
+    ((Mat_SeqAIJ*)Bnew->data)->nonew = Baij->nonew;
+  }
+
+  /*
+   Ensure that B's nonzerostate is monotonically increasing.
+   Or should this follow the MatSetValues() loop to preserve B's nonzerstate across a MatDisAssemble() call?
+   */
+  Bnew->nonzerostate = B->nonzerostate;
 
   ierr = PetscFree(nz);CHKERRQ(ierr);
   for (i=0; i<m; i++) {
@@ -188,9 +209,9 @@ PetscErrorCode MatDisAssemble_MPIAIJ(Mat A)
     }
   }
   ierr = PetscFree(aij->garray);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory(A,-ec*sizeof(PetscInt));CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)A,-ec*sizeof(PetscInt));CHKERRQ(ierr);
   ierr = MatDestroy(&B);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent(A,Bnew);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent((PetscObject)A,(PetscObject)Bnew);CHKERRQ(ierr);
 
   aij->B           = Bnew;
   A->was_assembled = PETSC_FALSE;
@@ -203,8 +224,6 @@ static PetscInt *auglyrmapd = 0,*auglyrmapo = 0; /* mapping from the local order
 static Vec auglydd          = 0,auglyoo     = 0; /* work vectors used to scale the two parts of the local matrix */
 
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMPIAIJDiagonalScaleLocalSetUp"
 PetscErrorCode MatMPIAIJDiagonalScaleLocalSetUp(Mat inA,Vec scale)
 {
   Mat_MPIAIJ     *ina = (Mat_MPIAIJ*) inA->data; /*access private part of matrix */
@@ -215,8 +234,7 @@ PetscErrorCode MatMPIAIJDiagonalScaleLocalSetUp(Mat inA,Vec scale)
   PetscFunctionBegin;
   ierr = MatGetOwnershipRange(inA,&cstart,&cend);CHKERRQ(ierr);
   ierr = MatGetSize(ina->A,NULL,&n);CHKERRQ(ierr);
-  ierr = PetscMalloc((inA->rmap->mapping->n+1)*sizeof(PetscInt),&r_rmapd);CHKERRQ(ierr);
-  ierr = PetscMemzero(r_rmapd,inA->rmap->mapping->n*sizeof(PetscInt));CHKERRQ(ierr);
+  ierr = PetscCalloc1(inA->rmap->mapping->n+1,&r_rmapd);CHKERRQ(ierr);
   nt   = 0;
   for (i=0; i<inA->rmap->mapping->n; i++) {
     if (inA->rmap->mapping->indices[i] >= cstart && inA->rmap->mapping->indices[i] < cend) {
@@ -225,7 +243,7 @@ PetscErrorCode MatMPIAIJDiagonalScaleLocalSetUp(Mat inA,Vec scale)
     }
   }
   if (nt != n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Hmm nt %D n %D",nt,n);
-  ierr = PetscMalloc((n+1)*sizeof(PetscInt),&auglyrmapd);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n+1,&auglyrmapd);CHKERRQ(ierr);
   for (i=0; i<inA->rmap->mapping->n; i++) {
     if (r_rmapd[i]) {
       auglyrmapd[(r_rmapd[i]-1)-cstart] = i;
@@ -234,14 +252,12 @@ PetscErrorCode MatMPIAIJDiagonalScaleLocalSetUp(Mat inA,Vec scale)
   ierr = PetscFree(r_rmapd);CHKERRQ(ierr);
   ierr = VecCreateSeq(PETSC_COMM_SELF,n,&auglydd);CHKERRQ(ierr);
 
-  ierr = PetscMalloc((inA->cmap->N+1)*sizeof(PetscInt),&lindices);CHKERRQ(ierr);
-  ierr = PetscMemzero(lindices,inA->cmap->N*sizeof(PetscInt));CHKERRQ(ierr);
+  ierr = PetscCalloc1(inA->cmap->N+1,&lindices);CHKERRQ(ierr);
   for (i=0; i<ina->B->cmap->n; i++) {
     lindices[garray[i]] = i+1;
   }
   no   = inA->rmap->mapping->n - nt;
-  ierr = PetscMalloc((inA->rmap->mapping->n+1)*sizeof(PetscInt),&r_rmapo);CHKERRQ(ierr);
-  ierr = PetscMemzero(r_rmapo,inA->rmap->mapping->n*sizeof(PetscInt));CHKERRQ(ierr);
+  ierr = PetscCalloc1(inA->rmap->mapping->n+1,&r_rmapo);CHKERRQ(ierr);
   nt   = 0;
   for (i=0; i<inA->rmap->mapping->n; i++) {
     if (lindices[inA->rmap->mapping->indices[i]]) {
@@ -251,7 +267,7 @@ PetscErrorCode MatMPIAIJDiagonalScaleLocalSetUp(Mat inA,Vec scale)
   }
   if (nt > no) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Hmm nt %D no %D",nt,n);
   ierr = PetscFree(lindices);CHKERRQ(ierr);
-  ierr = PetscMalloc((nt+1)*sizeof(PetscInt),&auglyrmapo);CHKERRQ(ierr);
+  ierr = PetscMalloc1(nt+1,&auglyrmapo);CHKERRQ(ierr);
   for (i=0; i<inA->rmap->mapping->n; i++) {
     if (r_rmapo[i]) {
       auglyrmapo[(r_rmapo[i]-1)] = i;
@@ -262,8 +278,6 @@ PetscErrorCode MatMPIAIJDiagonalScaleLocalSetUp(Mat inA,Vec scale)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMPIAIJDiagonalScaleLocal"
 PetscErrorCode MatMPIAIJDiagonalScaleLocal(Mat A,Vec scale)
 {
   /* This routine should really be abandoned as it duplicates MatDiagonalScaleLocal */
@@ -274,21 +288,20 @@ PetscErrorCode MatMPIAIJDiagonalScaleLocal(Mat A,Vec scale)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatDiagonalScaleLocal_MPIAIJ"
 PetscErrorCode  MatDiagonalScaleLocal_MPIAIJ(Mat A,Vec scale)
 {
-  Mat_MPIAIJ     *a = (Mat_MPIAIJ*) A->data; /*access private part of matrix */
-  PetscErrorCode ierr;
-  PetscInt       n,i;
-  PetscScalar    *d,*o,*s;
+  Mat_MPIAIJ        *a = (Mat_MPIAIJ*) A->data; /*access private part of matrix */
+  PetscErrorCode    ierr;
+  PetscInt          n,i;
+  PetscScalar       *d,*o;
+  const PetscScalar *s;
 
   PetscFunctionBegin;
   if (!auglyrmapd) {
     ierr = MatMPIAIJDiagonalScaleLocalSetUp(A,scale);CHKERRQ(ierr);
   }
 
-  ierr = VecGetArray(scale,&s);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(scale,&s);CHKERRQ(ierr);
 
   ierr = VecGetLocalSize(auglydd,&n);CHKERRQ(ierr);
   ierr = VecGetArray(auglydd,&d);CHKERRQ(ierr);
@@ -304,11 +317,9 @@ PetscErrorCode  MatDiagonalScaleLocal_MPIAIJ(Mat A,Vec scale)
   for (i=0; i<n; i++) {
     o[i] = s[auglyrmapo[i]]; /* copy "off-diagonal" portion of scale into oo vector */
   }
-  ierr = VecRestoreArray(scale,&s);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(scale,&s);CHKERRQ(ierr);
   ierr = VecRestoreArray(auglyoo,&o);CHKERRQ(ierr);
   /* column scale "off-diagonal" portion of local matrix */
   ierr = MatDiagonalScale(a->B,NULL,auglyoo);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-
-
