@@ -1,6 +1,6 @@
 
 /*
-        Provides an interface to the SuperLU_DIST_2.2 sparse solver
+        Provides an interface to the SuperLU_DIST sparse solver
 */
 
 #include <../src/mat/impls/aij/seq/aij.h>
@@ -9,11 +9,6 @@
 #include <stdlib.h>
 #endif
 
-#if defined(PETSC_USE_64BIT_INDICES)
-/* ugly SuperLU_Dist variable telling it to use long long int */
-#define _LONGINT
-#endif
-
 EXTERN_C_BEGIN
 #if defined(PETSC_USE_COMPLEX)
 #include <superlu_zdefs.h>
@@ -50,33 +45,37 @@ typedef struct {
   PetscBool              CleanUpSuperLU_Dist;  /* Flag to clean up (non-global) SuperLU objects during Destroy */
 } Mat_SuperLU_DIST;
 
-extern PetscErrorCode MatFactorInfo_SuperLU_DIST(Mat,PetscViewer);
-extern PetscErrorCode MatLUFactorNumeric_SuperLU_DIST(Mat,Mat,const MatFactorInfo*);
-extern PetscErrorCode MatDestroy_SuperLU_DIST(Mat);
-extern PetscErrorCode MatView_SuperLU_DIST(Mat,PetscViewer);
-extern PetscErrorCode MatSolve_SuperLU_DIST(Mat,Vec,Vec);
-extern PetscErrorCode MatLUFactorSymbolic_SuperLU_DIST(Mat,Mat,IS,IS,const MatFactorInfo*);
-extern PetscErrorCode MatDestroy_MPIAIJ(Mat);
 
-#undef __FUNCT__
-#define __FUNCT__ "MatGetDiagonal_SuperLU_DIST"
-PetscErrorCode MatGetDiagonal_SuperLU_DIST(Mat A,Vec v)
+PetscErrorCode MatSuperluDistGetDiagU_SuperLU_DIST(Mat F,PetscScalar *diagU)
 {
+  Mat_SuperLU_DIST  *lu= (Mat_SuperLU_DIST*)F->data;
+
   PetscFunctionBegin;
-  SETERRQ(PetscObjectComm((PetscObject)A),PETSC_ERR_SUP,"Mat type: SuperLU_DIST factor");
+#if defined(PETSC_USE_COMPLEX)
+  PetscStackCall("SuperLU_DIST:pzGetDiagU",pzGetDiagU(F->rmap->N,&lu->LUstruct,&lu->grid,(doublecomplex*)diagU));
+#else
+  PetscStackCall("SuperLU_DIST:pdGetDiagU",pdGetDiagU(F->rmap->N,&lu->LUstruct,&lu->grid,diagU));
+#endif
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_SuperLU_DIST"
-PetscErrorCode MatDestroy_SuperLU_DIST(Mat A)
+PetscErrorCode MatSuperluDistGetDiagU(Mat F,PetscScalar *diagU)
 {
-  PetscErrorCode   ierr;
-  Mat_SuperLU_DIST *lu = (Mat_SuperLU_DIST*)A->spptr;
-  PetscBool        flg;
+  PetscErrorCode    ierr;
 
   PetscFunctionBegin;
-  if (lu && lu->CleanUpSuperLU_Dist) {
+  PetscValidHeaderSpecific(F,MAT_CLASSID,1);
+  ierr = PetscTryMethod(F,"MatSuperluDistGetDiagU_C",(Mat,PetscScalar*),(F,diagU));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode MatDestroy_SuperLU_DIST(Mat A)
+{
+  PetscErrorCode   ierr;
+  Mat_SuperLU_DIST *lu = (Mat_SuperLU_DIST*)A->data;
+
+  PetscFunctionBegin;
+  if (lu->CleanUpSuperLU_Dist) {
     /* Deallocate SuperLU_DIST storage */
     if (lu->MatInputMode == GLOBAL) {
       PetscStackCall("SuperLU_DIST:Destroy_CompCol_Matrix_dist",Destroy_CompCol_Matrix_dist(&lu->A_sup));
@@ -98,28 +97,23 @@ PetscErrorCode MatDestroy_SuperLU_DIST(Mat A)
     PetscStackCall("SuperLU_DIST:superlu_gridexit",superlu_gridexit(&lu->grid));
     ierr = MPI_Comm_free(&(lu->comm_superlu));CHKERRQ(ierr);
   }
-  ierr = PetscFree(A->spptr);CHKERRQ(ierr);
+  ierr = PetscFree(A->data);CHKERRQ(ierr);
+  /* clear composed functions */
+  ierr = PetscObjectComposeFunction((PetscObject)A,"MatFactorGetSolverType_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)A,"MatSuperluDistGetDiagU_C",NULL);CHKERRQ(ierr);
 
-  ierr = PetscObjectTypeCompare((PetscObject)A,MATSEQAIJ,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = MatDestroy_SeqAIJ(A);CHKERRQ(ierr);
-  } else {
-    ierr = MatDestroy_MPIAIJ(A);CHKERRQ(ierr);
-  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SuperLU_DIST"
-PetscErrorCode MatSolve_SuperLU_DIST(Mat A,Vec b_mpi,Vec x)
+static PetscErrorCode MatSolve_SuperLU_DIST(Mat A,Vec b_mpi,Vec x)
 {
-  Mat_SuperLU_DIST *lu = (Mat_SuperLU_DIST*)A->spptr;
+  Mat_SuperLU_DIST *lu = (Mat_SuperLU_DIST*)A->data;
   PetscErrorCode   ierr;
   PetscMPIInt      size;
   PetscInt         m=A->rmap->n,M=A->rmap->N,N=A->cmap->N;
   SuperLUStat_t    stat;
   double           berr[1];
-  PetscScalar      *bptr;
+  PetscScalar      *bptr=NULL;
   PetscInt         nrhs=1;
   Vec              x_seq;
   IS               iden;
@@ -128,7 +122,9 @@ PetscErrorCode MatSolve_SuperLU_DIST(Mat A,Vec b_mpi,Vec x)
   static PetscBool cite = PETSC_FALSE;
 
   PetscFunctionBegin;
+  if (lu->options.Fact != FACTORED) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"SuperLU_DIST options.Fact mush equal FACTORED");
   ierr = PetscCitationsRegister("@article{lidemmel03,\n  author = {Xiaoye S. Li and James W. Demmel},\n  title = {{SuperLU_DIST}: A Scalable Distributed-Memory Sparse Direct\n           Solver for Unsymmetric Linear Systems},\n  journal = {ACM Trans. Mathematical Software},\n  volume = {29},\n  number = {2},\n  pages = {110-140},\n  year = 2003\n}\n",&cite);CHKERRQ(ierr);
+
   ierr = MPI_Comm_size(PetscObjectComm((PetscObject)A),&size);CHKERRQ(ierr);
   if (size > 1 && lu->MatInputMode == GLOBAL) {
     /* global mat input, convert b to x_seq */
@@ -154,8 +150,6 @@ PetscErrorCode MatSolve_SuperLU_DIST(Mat A,Vec b_mpi,Vec x)
     ierr = VecGetArray(x,&bptr);CHKERRQ(ierr);
   }
 
-  if (lu->options.Fact != FACTORED) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"SuperLU_DIST options.Fact mush equal FACTORED");
-
   PetscStackCall("SuperLU_DIST:PStatInit",PStatInit(&stat));        /* Initialize the statistics variables. */
   if (lu->MatInputMode == GLOBAL) {
 #if defined(PETSC_USE_COMPLEX)
@@ -191,11 +185,9 @@ PetscErrorCode MatSolve_SuperLU_DIST(Mat A,Vec b_mpi,Vec x)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatSolve_SuperLU_DIST"
-PetscErrorCode MatMatSolve_SuperLU_DIST(Mat A,Mat B_mpi,Mat X)
+static PetscErrorCode MatMatSolve_SuperLU_DIST(Mat A,Mat B_mpi,Mat X)
 {
-  Mat_SuperLU_DIST *lu = (Mat_SuperLU_DIST*)A->spptr;
+  Mat_SuperLU_DIST *lu = (Mat_SuperLU_DIST*)A->data;
   PetscErrorCode   ierr;
   PetscMPIInt      size;
   PetscInt         M=A->rmap->N,m=A->rmap->n,nrhs;
@@ -256,23 +248,63 @@ PetscErrorCode MatMatSolve_SuperLU_DIST(Mat A,Mat B_mpi,Mat X)
   PetscFunctionReturn(0);
 }
 
+/*
+  input:
+   F:        numeric Cholesky factor
+  output:
+   nneg:     total number of negative pivots
+   nzero:    total number of zero pivots
+   npos:     (global dimension of F) - nneg - nzero
+*/
+static PetscErrorCode MatGetInertia_SuperLU_DIST(Mat F,PetscInt *nneg,PetscInt *nzero,PetscInt *npos)
+{
+  PetscErrorCode   ierr;
+  Mat_SuperLU_DIST *lu = (Mat_SuperLU_DIST*)F->data;
+  PetscScalar      *diagU=NULL;
+  PetscInt         M,i,neg=0,zero=0,pos=0;
+  PetscReal        r;
 
-#undef __FUNCT__
-#define __FUNCT__ "MatLUFactorNumeric_SuperLU_DIST"
-PetscErrorCode MatLUFactorNumeric_SuperLU_DIST(Mat F,Mat A,const MatFactorInfo *info)
+  PetscFunctionBegin;
+  if (!F->assembled) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Matrix factor F is not assembled");
+  if (lu->options.RowPerm != NOROWPERM) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Must set NOROWPERM");
+  ierr = MatGetSize(F,&M,NULL);CHKERRQ(ierr);
+  ierr = PetscMalloc1(M,&diagU);CHKERRQ(ierr);
+  ierr = MatSuperluDistGetDiagU(F,diagU);CHKERRQ(ierr);
+  for (i=0; i<M; i++) {
+#if defined(PETSC_USE_COMPLEX)
+    r = PetscImaginaryPart(diagU[i])/10.0;
+    if (r< -PETSC_MACHINE_EPSILON || r>PETSC_MACHINE_EPSILON) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"diagU[%d]=%g + i %g is non-real",i,PetscRealPart(diagU[i]),r*10.0);
+    r = PetscRealPart(diagU[i]);
+#else
+    r = diagU[i];
+#endif
+    if (r > 0) {
+      pos++;
+    } else if (r < 0) {
+      neg++;
+    } else zero++;
+  }
+
+  ierr = PetscFree(diagU);CHKERRQ(ierr);
+  if (nneg)  *nneg  = neg;
+  if (nzero) *nzero = zero;
+  if (npos)  *npos  = pos;
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode MatLUFactorNumeric_SuperLU_DIST(Mat F,Mat A,const MatFactorInfo *info)
 {
   Mat              *tseq,A_seq = NULL;
   Mat_SeqAIJ       *aa,*bb;
-  Mat_SuperLU_DIST *lu = (Mat_SuperLU_DIST*)(F)->spptr;
+  Mat_SuperLU_DIST *lu = (Mat_SuperLU_DIST*)F->data;
   PetscErrorCode   ierr;
   PetscInt         M=A->rmap->N,N=A->cmap->N,i,*ai,*aj,*bi,*bj,nz,rstart,*garray,
-                   m=A->rmap->n, colA_start,j,jcol,jB,countA,countB,*bjj,*ajj;
+                   m=A->rmap->n, colA_start,j,jcol,jB,countA,countB,*bjj,*ajj=NULL;
   int              sinfo;   /* SuperLU_Dist info flag is always an int even with long long indices */
   PetscMPIInt      size;
   SuperLUStat_t    stat;
   double           *berr=0;
   IS               isrow;
-  Mat              F_diag=NULL;
 #if defined(PETSC_USE_COMPLEX)
   doublecomplex    *av, *bv;
 #else
@@ -285,7 +317,7 @@ PetscErrorCode MatLUFactorNumeric_SuperLU_DIST(Mat F,Mat A,const MatFactorInfo *
   if (lu->MatInputMode == GLOBAL) { /* global mat input */
     if (size > 1) { /* convert mpi A to seq mat A */
       ierr = ISCreateStride(PETSC_COMM_SELF,M,0,1,&isrow);CHKERRQ(ierr);
-      ierr = MatGetSubMatrices(A,1,&isrow,&isrow,MAT_INITIAL_MATRIX,&tseq);CHKERRQ(ierr);
+      ierr = MatCreateSubMatrices(A,1,&isrow,&isrow,MAT_INITIAL_MATRIX,&tseq);CHKERRQ(ierr);
       ierr = ISDestroy(&isrow);CHKERRQ(ierr);
 
       A_seq = *tseq;
@@ -351,10 +383,20 @@ PetscErrorCode MatLUFactorNumeric_SuperLU_DIST(Mat F,Mat A,const MatFactorInfo *
       if (lu->FactPattern == SamePattern_SameRowPerm) {
         lu->options.Fact = SamePattern_SameRowPerm; /* matrix has similar numerical values */
       } else if (lu->FactPattern == SamePattern) {
-        PetscStackCall("SuperLU_DIST:Destroy_LU",Destroy_LU(N, &lu->grid, &lu->LUstruct)); /* Deallocate storage associated with the L and U matrices. */
+        PetscStackCall("SuperLU_DIST:Destroy_LU",Destroy_LU(N, &lu->grid, &lu->LUstruct)); /* Deallocate L and U matrices. */
         lu->options.Fact = SamePattern;
+      } else if (lu->FactPattern == DOFACT) {
+        PetscStackCall("SuperLU_DIST:Destroy_CompRowLoc_Matrix_dist",Destroy_CompRowLoc_Matrix_dist(&lu->A_sup));
+        PetscStackCall("SuperLU_DIST:Destroy_LU",Destroy_LU(N, &lu->grid, &lu->LUstruct));
+        lu->options.Fact = DOFACT;
+
+#if defined(PETSC_USE_COMPLEX)
+      PetscStackCall("SuperLU_DIST:zallocateA_dist",zallocateA_dist(m, nz, &lu->val, &lu->col, &lu->row));
+#else
+      PetscStackCall("SuperLU_DIST:dallocateA_dist",dallocateA_dist(m, nz, &lu->val, &lu->col, &lu->row));
+#endif
       } else {
-        SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"options.Fact must be one of SamePattern SamePattern_SameRowPerm");
+        SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"options.Fact must be one of SamePattern SamePattern_SameRowPerm DOFACT");
       }
     }
     nz = 0;
@@ -431,14 +473,14 @@ PetscErrorCode MatLUFactorNumeric_SuperLU_DIST(Mat F,Mat A,const MatFactorInfo *
       SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_MAT_LU_ZRPVT,"Zero pivot in row %D",sinfo);
     } else {
       if (sinfo <= lu->A_sup.ncol) {
-        F->errortype = MAT_FACTOR_NUMERIC_ZEROPIVOT;
+        F->factorerrortype = MAT_FACTOR_NUMERIC_ZEROPIVOT;
         ierr = PetscInfo1(F,"U(i,i) is exactly zero, i= %D\n",sinfo);CHKERRQ(ierr);
       } else if (sinfo > lu->A_sup.ncol) {
         /* 
          number of bytes allocated when memory allocation
          failure occurred, plus A->ncol.
          */
-        F->errortype = MAT_FACTOR_OUTMEMORY;
+        F->factorerrortype = MAT_FACTOR_OUTMEMORY;
         ierr = PetscInfo1(F,"Number of bytes allocated when memory allocation fails %D\n",sinfo);CHKERRQ(ierr);
       }
     }
@@ -454,22 +496,16 @@ PetscErrorCode MatLUFactorNumeric_SuperLU_DIST(Mat F,Mat A,const MatFactorInfo *
     PStatPrint(&lu->options, &stat, &lu->grid);  /* Print the statistics. */
   }
   PetscStackCall("SuperLU_DIST:PStatFree",PStatFree(&stat));
-  if (size > 1) {
-    F_diag            = ((Mat_MPIAIJ*)(F)->data)->A;
-    F_diag->assembled = PETSC_TRUE;
-  }
-  (F)->assembled    = PETSC_TRUE;
-  (F)->preallocated = PETSC_TRUE;
+  F->assembled    = PETSC_TRUE;
+  F->preallocated = PETSC_TRUE;
   lu->options.Fact  = FACTORED; /* The factored form of A is supplied. Local option used by this func. only */
   PetscFunctionReturn(0);
 }
 
 /* Note the Petsc r and c permutations are ignored */
-#undef __FUNCT__
-#define __FUNCT__ "MatLUFactorSymbolic_SuperLU_DIST"
-PetscErrorCode MatLUFactorSymbolic_SuperLU_DIST(Mat F,Mat A,IS r,IS c,const MatFactorInfo *info)
+static PetscErrorCode MatLUFactorSymbolic_SuperLU_DIST(Mat F,Mat A,IS r,IS c,const MatFactorInfo *info)
 {
-  Mat_SuperLU_DIST *lu = (Mat_SuperLU_DIST*)F->spptr;
+  Mat_SuperLU_DIST *lu = (Mat_SuperLU_DIST*)F->data;
   PetscInt         M   = A->rmap->N,N=A->cmap->N;
 
   PetscFunctionBegin;
@@ -482,21 +518,120 @@ PetscErrorCode MatLUFactorSymbolic_SuperLU_DIST(Mat F,Mat A,IS r,IS c,const MatF
   F->ops->lufactornumeric = MatLUFactorNumeric_SuperLU_DIST;
   F->ops->solve           = MatSolve_SuperLU_DIST;
   F->ops->matsolve        = MatMatSolve_SuperLU_DIST;
+  F->ops->getinertia      = NULL;
+
+  if (A->symmetric || A->hermitian) {
+    F->ops->getinertia = MatGetInertia_SuperLU_DIST;
+  }
   lu->CleanUpSuperLU_Dist = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFactorGetSolverPackage_aij_superlu_dist"
-static PetscErrorCode MatFactorGetSolverPackage_aij_superlu_dist(Mat A,const MatSolverPackage *type)
+static PetscErrorCode MatCholeskyFactorSymbolic_SuperLU_DIST(Mat F,Mat A,IS r,const MatFactorInfo *info)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (!A->symmetric) SETERRQ(PetscObjectComm((PetscObject)A),PETSC_ERR_ARG_WRONG,"Input matrix must be symmetric\n");
+  ierr = MatLUFactorSymbolic_SuperLU_DIST(F,A,r,r,info);CHKERRQ(ierr);
+  F->ops->choleskyfactornumeric = MatLUFactorNumeric_SuperLU_DIST;
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode MatFactorGetSolverType_aij_superlu_dist(Mat A,MatSolverType *type)
 {
   PetscFunctionBegin;
   *type = MATSOLVERSUPERLU_DIST;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatGetFactor_aij_superlu_dist"
+static PetscErrorCode MatView_Info_SuperLU_DIST(Mat A,PetscViewer viewer)
+{
+  Mat_SuperLU_DIST       *lu=(Mat_SuperLU_DIST*)A->data;
+  superlu_dist_options_t options;
+  PetscErrorCode         ierr;
+
+  PetscFunctionBegin;
+  /* check if matrix is superlu_dist type */
+  if (A->ops->solve != MatSolve_SuperLU_DIST) PetscFunctionReturn(0);
+
+  options = lu->options;
+  ierr    = PetscViewerASCIIPrintf(viewer,"SuperLU_DIST run parameters:\n");CHKERRQ(ierr);
+  ierr    = PetscViewerASCIIPrintf(viewer,"  Process grid nprow %D x npcol %D \n",lu->nprow,lu->npcol);CHKERRQ(ierr);
+  ierr    = PetscViewerASCIIPrintf(viewer,"  Equilibrate matrix %s \n",PetscBools[options.Equil != NO]);CHKERRQ(ierr);
+  ierr    = PetscViewerASCIIPrintf(viewer,"  Matrix input mode %d \n",lu->MatInputMode);CHKERRQ(ierr);
+  ierr    = PetscViewerASCIIPrintf(viewer,"  Replace tiny pivots %s \n",PetscBools[options.ReplaceTinyPivot != NO]);CHKERRQ(ierr);
+  ierr    = PetscViewerASCIIPrintf(viewer,"  Use iterative refinement %s \n",PetscBools[options.IterRefine == SLU_DOUBLE]);CHKERRQ(ierr);
+  ierr    = PetscViewerASCIIPrintf(viewer,"  Processors in row %d col partition %d \n",lu->nprow,lu->npcol);CHKERRQ(ierr);
+
+  switch (options.RowPerm) {
+  case NOROWPERM:
+    ierr = PetscViewerASCIIPrintf(viewer,"  Row permutation NOROWPERM\n");CHKERRQ(ierr);
+    break;
+  case LargeDiag_MC64:
+    ierr = PetscViewerASCIIPrintf(viewer,"  Row permutation LargeDiag_MC64\n");CHKERRQ(ierr);
+    break;
+  case LargeDiag_AWPM:
+    ierr = PetscViewerASCIIPrintf(viewer,"  Row permutation LargeDiag_AWPM\n");CHKERRQ(ierr);
+    break;
+  case MY_PERMR:
+    ierr = PetscViewerASCIIPrintf(viewer,"  Row permutation MY_PERMR\n");CHKERRQ(ierr);
+    break;
+  default:
+    SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Unknown column permutation");
+  }
+
+  switch (options.ColPerm) {
+  case NATURAL:
+    ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation NATURAL\n");CHKERRQ(ierr);
+    break;
+  case MMD_AT_PLUS_A:
+    ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation MMD_AT_PLUS_A\n");CHKERRQ(ierr);
+    break;
+  case MMD_ATA:
+    ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation MMD_ATA\n");CHKERRQ(ierr);
+    break;
+  case METIS_AT_PLUS_A:
+    ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation METIS_AT_PLUS_A\n");CHKERRQ(ierr);
+    break;
+  case PARMETIS:
+    ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation PARMETIS\n");CHKERRQ(ierr);
+    break;
+  default:
+    SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Unknown column permutation");
+  }
+
+  ierr = PetscViewerASCIIPrintf(viewer,"  Parallel symbolic factorization %s \n",PetscBools[options.ParSymbFact != NO]);CHKERRQ(ierr);
+
+  if (lu->FactPattern == SamePattern) {
+    ierr = PetscViewerASCIIPrintf(viewer,"  Repeated factorization SamePattern\n");CHKERRQ(ierr);
+  } else if (lu->FactPattern == SamePattern_SameRowPerm) {
+    ierr = PetscViewerASCIIPrintf(viewer,"  Repeated factorization SamePattern_SameRowPerm\n");CHKERRQ(ierr);
+  } else if (lu->FactPattern == DOFACT) {
+    ierr = PetscViewerASCIIPrintf(viewer,"  Repeated factorization DOFACT\n");CHKERRQ(ierr);
+  } else {
+    SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Unknown factorization pattern");
+  }
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode MatView_SuperLU_DIST(Mat A,PetscViewer viewer)
+{
+  PetscErrorCode    ierr;
+  PetscBool         iascii;
+  PetscViewerFormat format;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
+  if (iascii) {
+    ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
+    if (format == PETSC_VIEWER_ASCII_INFO) {
+      ierr = MatView_Info_SuperLU_DIST(A,viewer);CHKERRQ(ierr);
+    }
+  }
+  PetscFunctionReturn(0);
+}
+
 static PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Mat *F)
 {
   Mat                    B;
@@ -507,40 +642,41 @@ static PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Ma
   superlu_dist_options_t options;
   PetscBool              flg;
   const char             *colperm[]     = {"NATURAL","MMD_AT_PLUS_A","MMD_ATA","METIS_AT_PLUS_A","PARMETIS"};
-  const char             *rowperm[]     = {"LargeDiag","NATURAL"};
-  const char             *factPattern[] = {"SamePattern","SamePattern_SameRowPerm"};
+  const char             *rowperm[]     = {"NOROWPERM","LargeDiag_MC64","LargeDiag_AWPM","MY_PERMR"};
+  const char             *factPattern[] = {"SamePattern","SamePattern_SameRowPerm","DOFACT"};
   PetscBool              set;
 
   PetscFunctionBegin;
   /* Create the factorization matrix */
   ierr = MatCreate(PetscObjectComm((PetscObject)A),&B);CHKERRQ(ierr);
   ierr = MatSetSizes(B,A->rmap->n,A->cmap->n,M,N);CHKERRQ(ierr);
-  ierr = MatSetType(B,((PetscObject)A)->type_name);CHKERRQ(ierr);
-  ierr = MatSeqAIJSetPreallocation(B,0,NULL);CHKERRQ(ierr);
-  ierr = MatMPIAIJSetPreallocation(B,0,NULL,0,NULL);CHKERRQ(ierr);
+  ierr = PetscStrallocpy("superlu_dist",&((PetscObject)B)->type_name);CHKERRQ(ierr);
+  ierr = MatSetUp(B);CHKERRQ(ierr);
+  B->ops->getinfo = MatGetInfo_External;
+  B->ops->view    = MatView_SuperLU_DIST;
+  B->ops->destroy = MatDestroy_SuperLU_DIST;
 
-  B->ops->lufactorsymbolic = MatLUFactorSymbolic_SuperLU_DIST;
-  B->ops->view             = MatView_SuperLU_DIST;
-  B->ops->destroy          = MatDestroy_SuperLU_DIST;
-  B->ops->getdiagonal      = MatGetDiagonal_SuperLU_DIST;
-
-  ierr = PetscObjectComposeFunction((PetscObject)B,"MatFactorGetSolverPackage_C",MatFactorGetSolverPackage_aij_superlu_dist);CHKERRQ(ierr);
-
-  B->factortype = MAT_FACTOR_LU;
+  if (ftype == MAT_FACTOR_LU) {
+    B->factortype = MAT_FACTOR_LU;
+    B->ops->lufactorsymbolic       = MatLUFactorSymbolic_SuperLU_DIST;
+  } else {
+    B->factortype = MAT_FACTOR_CHOLESKY;
+    B->ops->choleskyfactorsymbolic = MatCholeskyFactorSymbolic_SuperLU_DIST;
+  }
 
   /* set solvertype */
   ierr = PetscFree(B->solvertype);CHKERRQ(ierr);
   ierr = PetscStrallocpy(MATSOLVERSUPERLU_DIST,&B->solvertype);CHKERRQ(ierr);
 
-  ierr     = PetscNewLog(B,&lu);CHKERRQ(ierr);
-  B->spptr = lu;
+  ierr    = PetscNewLog(B,&lu);CHKERRQ(ierr);
+  B->data = lu;
 
   /* Set the default input options:
      options.Fact              = DOFACT;
      options.Equil             = YES;
      options.ParSymbFact       = NO;
      options.ColPerm           = METIS_AT_PLUS_A;
-     options.RowPerm           = LargeDiag;
+     options.RowPerm           = LargeDiag_MC64;
      options.ReplaceTinyPivot  = YES;
      options.IterRefine        = DOUBLE;
      options.Trans             = NOTRANS;
@@ -553,12 +689,12 @@ static PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Ma
   ierr = MPI_Comm_dup(PetscObjectComm((PetscObject)A),&(lu->comm_superlu));CHKERRQ(ierr);
   ierr = MPI_Comm_size(PetscObjectComm((PetscObject)A),&size);CHKERRQ(ierr);
   /* Default num of process columns and rows */
-  lu->npcol = (int_t) (0.5 + PetscSqrtReal((PetscReal)size));
-  if (!lu->npcol) lu->npcol = 1;
-  while (lu->npcol > 0) {
-    lu->nprow = (int_t) (size/lu->npcol);
+  lu->nprow = (int_t) (0.5 + PetscSqrtReal((PetscReal)size));
+  if (!lu->nprow) lu->nprow = 1;
+  while (lu->nprow > 0) {
+    lu->npcol = (int_t) (size/lu->nprow);
     if (size == lu->nprow * lu->npcol) break;
-    lu->npcol--;
+    lu->nprow--;
   }
 
   ierr = PetscOptionsBegin(PetscObjectComm((PetscObject)A),((PetscObject)A)->prefix,"SuperLU_Dist Options","Mat");CHKERRQ(ierr);
@@ -574,15 +710,23 @@ static PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Ma
   ierr = PetscOptionsBool("-mat_superlu_dist_equil","Equilibrate matrix","None",options.Equil ? PETSC_TRUE : PETSC_FALSE,&flg,&set);CHKERRQ(ierr);
   if (set && !flg) options.Equil = NO;
 
-  ierr = PetscOptionsEList("-mat_superlu_dist_rowperm","Row permutation","None",rowperm,2,rowperm[0],&indx,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsEList("-mat_superlu_dist_rowperm","Row permutation","None",rowperm,4,rowperm[1],&indx,&flg);CHKERRQ(ierr);
   if (flg) {
     switch (indx) {
     case 0:
-      options.RowPerm = LargeDiag;
-      break;
-    case 1:
       options.RowPerm = NOROWPERM;
       break;
+    case 1:
+      options.RowPerm = LargeDiag_MC64;
+      break;
+    case 2:
+      options.RowPerm = LargeDiag_AWPM;
+      break;
+    case 3:
+      options.RowPerm = MY_PERMR;
+      break;
+    default:
+      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Unknown row permutation");
     }
   }
 
@@ -630,8 +774,8 @@ static PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Ma
     }
   }
 
-  lu->FactPattern = SamePattern_SameRowPerm;
-  ierr = PetscOptionsEList("-mat_superlu_dist_fact","Sparsity pattern for repeated matrix factorization","None",factPattern,2,factPattern[0],&indx,&flg);CHKERRQ(ierr);
+  lu->FactPattern = SamePattern;
+  ierr = PetscOptionsEList("-mat_superlu_dist_fact","Sparsity pattern for repeated matrix factorization","None",factPattern,3,factPattern[0],&indx,&flg);CHKERRQ(ierr);
   if (flg) {
     switch (indx) {
     case 0:
@@ -640,6 +784,9 @@ static PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Ma
     case 1:
       lu->FactPattern = SamePattern_SameRowPerm;
       break;
+    case 2:
+      lu->FactPattern = DOFACT;
+      break;
     }
   }
 
@@ -653,106 +800,37 @@ static PetscErrorCode MatGetFactor_aij_superlu_dist(Mat A,MatFactorType ftype,Ma
   if (PetscLogPrintInfo) options.PrintStat = YES;
   else options.PrintStat = NO;
   ierr = PetscOptionsBool("-mat_superlu_dist_statprint","Print factorization information","None",(PetscBool)options.PrintStat,(PetscBool*)&options.PrintStat,NULL);CHKERRQ(ierr);
-  PetscOptionsEnd();
+  ierr = PetscOptionsEnd();CHKERRQ(ierr);
 
   lu->options              = options;
   lu->options.Fact         = DOFACT;
   lu->matsolve_iscalled    = PETSC_FALSE;
   lu->matmatsolve_iscalled = PETSC_FALSE;
 
+  ierr = PetscObjectComposeFunction((PetscObject)B,"MatFactorGetSolverType_C",MatFactorGetSolverType_aij_superlu_dist);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)B,"MatSuperluDistGetDiagU_C",MatSuperluDistGetDiagU_SuperLU_DIST);CHKERRQ(ierr);
+
   *F = B;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolverPackageRegister_SuperLU_DIST"
-PETSC_EXTERN PetscErrorCode MatSolverPackageRegister_SuperLU_DIST(void)
+PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_SuperLU_DIST(void)
 {
   PetscErrorCode ierr;
   PetscFunctionBegin;
-  ierr = MatSolverPackageRegister(MATSOLVERSUPERLU_DIST,MATMPIAIJ,  MAT_FACTOR_LU,MatGetFactor_aij_superlu_dist);CHKERRQ(ierr);
-  ierr = MatSolverPackageRegister(MATSOLVERSUPERLU_DIST,MATSEQAIJ,  MAT_FACTOR_LU,MatGetFactor_aij_superlu_dist);CHKERRQ(ierr);
+  ierr = MatSolverTypeRegister(MATSOLVERSUPERLU_DIST,MATMPIAIJ,  MAT_FACTOR_LU,MatGetFactor_aij_superlu_dist);CHKERRQ(ierr);
+  ierr = MatSolverTypeRegister(MATSOLVERSUPERLU_DIST,MATSEQAIJ,  MAT_FACTOR_LU,MatGetFactor_aij_superlu_dist);CHKERRQ(ierr);
+  ierr = MatSolverTypeRegister(MATSOLVERSUPERLU_DIST,MATMPIAIJ,  MAT_FACTOR_CHOLESKY,MatGetFactor_aij_superlu_dist);CHKERRQ(ierr);
+  ierr = MatSolverTypeRegister(MATSOLVERSUPERLU_DIST,MATSEQAIJ,  MAT_FACTOR_CHOLESKY,MatGetFactor_aij_superlu_dist);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFactorInfo_SuperLU_DIST"
-PetscErrorCode MatFactorInfo_SuperLU_DIST(Mat A,PetscViewer viewer)
-{
-  Mat_SuperLU_DIST       *lu=(Mat_SuperLU_DIST*)A->spptr;
-  superlu_dist_options_t options;
-  PetscErrorCode         ierr;
-
-  PetscFunctionBegin;
-  /* check if matrix is superlu_dist type */
-  if (A->ops->solve != MatSolve_SuperLU_DIST) PetscFunctionReturn(0);
-
-  options = lu->options;
-  ierr    = PetscViewerASCIIPrintf(viewer,"SuperLU_DIST run parameters:\n");CHKERRQ(ierr);
-  ierr    = PetscViewerASCIIPrintf(viewer,"  Process grid nprow %D x npcol %D \n",lu->nprow,lu->npcol);CHKERRQ(ierr);
-  ierr    = PetscViewerASCIIPrintf(viewer,"  Equilibrate matrix %s \n",PetscBools[options.Equil != NO]);CHKERRQ(ierr);
-  ierr    = PetscViewerASCIIPrintf(viewer,"  Matrix input mode %d \n",lu->MatInputMode);CHKERRQ(ierr);
-  ierr    = PetscViewerASCIIPrintf(viewer,"  Replace tiny pivots %s \n",PetscBools[options.ReplaceTinyPivot != NO]);CHKERRQ(ierr);
-  ierr    = PetscViewerASCIIPrintf(viewer,"  Use iterative refinement %s \n",PetscBools[options.IterRefine == SLU_DOUBLE]);CHKERRQ(ierr);
-  ierr    = PetscViewerASCIIPrintf(viewer,"  Processors in row %d col partition %d \n",lu->nprow,lu->npcol);CHKERRQ(ierr);
-  ierr    = PetscViewerASCIIPrintf(viewer,"  Row permutation %s \n",(options.RowPerm == NOROWPERM) ? "NATURAL" : "LargeDiag");CHKERRQ(ierr);
-
-  switch (options.ColPerm) {
-  case NATURAL:
-    ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation NATURAL\n");CHKERRQ(ierr);
-    break;
-  case MMD_AT_PLUS_A:
-    ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation MMD_AT_PLUS_A\n");CHKERRQ(ierr);
-    break;
-  case MMD_ATA:
-    ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation MMD_ATA\n");CHKERRQ(ierr);
-    break;
-  case METIS_AT_PLUS_A:
-    ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation METIS_AT_PLUS_A\n");CHKERRQ(ierr);
-    break;
-  case PARMETIS:
-    ierr = PetscViewerASCIIPrintf(viewer,"  Column permutation PARMETIS\n");CHKERRQ(ierr);
-    break;
-  default:
-    SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Unknown column permutation");
-  }
-
-  ierr = PetscViewerASCIIPrintf(viewer,"  Parallel symbolic factorization %s \n",PetscBools[options.ParSymbFact != NO]);CHKERRQ(ierr);
-
-  if (lu->FactPattern == SamePattern) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  Repeated factorization SamePattern\n");CHKERRQ(ierr);
-  } else {
-    ierr = PetscViewerASCIIPrintf(viewer,"  Repeated factorization SamePattern_SameRowPerm\n");CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "MatView_SuperLU_DIST"
-PetscErrorCode MatView_SuperLU_DIST(Mat A,PetscViewer viewer)
-{
-  PetscErrorCode    ierr;
-  PetscBool         iascii;
-  PetscViewerFormat format;
-
-  PetscFunctionBegin;
-  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
-  if (iascii) {
-    ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
-    if (format == PETSC_VIEWER_ASCII_INFO) {
-      ierr = MatFactorInfo_SuperLU_DIST(A,viewer);CHKERRQ(ierr);
-    }
-  }
-  PetscFunctionReturn(0);
-}
-
-
 /*MC
   MATSOLVERSUPERLU_DIST - Parallel direct solver package for LU factorization
 
   Use ./configure --download-superlu_dist --download-parmetis --download-metis --download-ptscotch  to have PETSc installed with SuperLU_DIST
 
-  Use -pc_type lu -pc_factor_mat_solver_package superlu_dist to us this direct solver
+  Use -pc_type lu -pc_factor_mat_solver_type superlu_dist to use this direct solver
 
    Works with AIJ matrices
 
@@ -761,10 +839,10 @@ PetscErrorCode MatView_SuperLU_DIST(Mat A,PetscViewer viewer)
 . -mat_superlu_dist_c <n> - number of columns in processor partition
 . -mat_superlu_dist_matinput <0,1> - matrix input mode; 0=global, 1=distributed
 . -mat_superlu_dist_equil - equilibrate the matrix
-. -mat_superlu_dist_rowperm <LargeDiag,NATURAL> - row permutation
+. -mat_superlu_dist_rowperm <NOROWPERM,LargeDiag_MC64,LargeDiag_AWPM,MY_PERMR> - row permutation
 . -mat_superlu_dist_colperm <MMD_AT_PLUS_A,MMD_ATA,NATURAL> - column permutation
 . -mat_superlu_dist_replacetinypivot - replace tiny pivots
-. -mat_superlu_dist_fact <SamePattern> - (choose one of) SamePattern SamePattern_SameRowPerm
+. -mat_superlu_dist_fact <SamePattern> - (choose one of) SamePattern SamePattern_SameRowPerm DOFACT
 . -mat_superlu_dist_iterrefine - use iterative refinement
 - -mat_superlu_dist_statprint - print factorization information
 
@@ -772,7 +850,6 @@ PetscErrorCode MatView_SuperLU_DIST(Mat A,PetscViewer viewer)
 
 .seealso: PCLU
 
-.seealso: PCFactorSetMatSolverPackage(), MatSolverPackage
+.seealso: PCFactorSetMatSolverType(), MatSolverType
 
 M*/
-
