@@ -16,6 +16,7 @@
   PetscInt  maxnz;                    /* allocated nonzeros */ \
   PetscInt  *imax;                    /* maximum space allocated for each row */ \
   PetscInt  *ilen;                    /* actual length of each row */ \
+  PetscInt  *ipre;                    /* space preallocated for each row by user */ \
   PetscBool free_imax_ilen;  \
   PetscInt  reallocs;                 /* number of mallocs done during MatSetValues() \
                                         as more values are set than were prealloced */\
@@ -35,8 +36,9 @@
   PetscScalar       *solve_work;      /* work space used in MatSolve */                    \
   IS                row, col, icol;   /* index sets, used for reorderings */ \
   PetscBool         pivotinblocks;    /* pivot inside factorization of each diagonal block */ \
-  Mat               parent             /* set if this matrix was formed with MatDuplicate(...,MAT_SHARE_NONZERO_PATTERN,....);
-                                         means that this shares some data structures with the parent including diag, ilen, imax, i, j */
+  Mat               parent;           /* set if this matrix was formed with MatDuplicate(...,MAT_SHARE_NONZERO_PATTERN,....); \
+                                         means that this shares some data structures with the parent including diag, ilen, imax, i, j */\
+  Mat_SubSppt       *submatis1         /* used by MatCreateSubMatrices_MPIXAIJ_Local */
 
 typedef struct {
   MatTransposeColoring matcoloring;
@@ -46,7 +48,8 @@ typedef struct {
   PetscErrorCode (*destroy)(Mat);
 } Mat_MatMatTransMult;
 
-typedef struct { /* for MatTransposeMatMult_SeqAIJ_SeqDense() */
+typedef struct { /* used by MatTransposeMatMult() */
+  Mat          At;           /* transpose of the first matrix */
   Mat          mA;           /* maij matrix of A */
   Vec          bt,ct;        /* vectors to hold locally transposed arrays of B and C */
   PetscErrorCode (*destroy)(Mat);
@@ -115,20 +118,19 @@ typedef struct {
   PetscBool   ibdiagvalid;                    /* inverses of block diagonals are valid. */
   PetscScalar fshift,omega;                   /* last used omega and fshift */
 
-  ISColoring coloring;                        /* set with MatADSetColoring() used by MatADSetValues() */
+  ISColoring  coloring;                       /* set with MatADSetColoring() used by MatADSetValues() */
 
-  PetscScalar       *matmult_abdense;    /* used by MatMatMult() */
-  Mat_PtAP          *ptap;               /* used by MatPtAP() */
-  Mat_MatMatMatMult *matmatmatmult;      /* used by MatMatMatMult() */
-  Mat_RARt          *rart;               /* used by MatRARt() */
-  Mat_MatMatTransMult *abt;              /* used by MatMatTransposeMult() */
+  PetscScalar         *matmult_abdense;    /* used by MatMatMult() */
+  Mat_PtAP            *ptap;               /* used by MatPtAP() */
+  Mat_MatMatMatMult   *matmatmatmult;      /* used by MatMatMatMult() */
+  Mat_RARt            *rart;               /* used by MatRARt() */
+  Mat_MatMatTransMult *abt;                /* used by MatMatTransposeMult() */
+  Mat_MatTransMatMult *atb;                /* used by MatTransposeMatMult() */
 } Mat_SeqAIJ;
 
 /*
   Frees the a, i, and j arrays from the XAIJ (AIJ, BAIJ, and SBAIJ) matrix types
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatSeqXAIJFreeAIJ"
 PETSC_STATIC_INLINE PetscErrorCode MatSeqXAIJFreeAIJ(Mat AA,MatScalar **a,PetscInt **j,PetscInt **i)
 {
   PetscErrorCode ierr;
@@ -179,6 +181,36 @@ PETSC_STATIC_INLINE PetscErrorCode MatSeqXAIJFreeAIJ(Mat AA,MatScalar **a,PetscI
     Ain->reallocs++; \
   } \
 
+#define MatSeqXAIJReallocateAIJ_structure_only(Amat,AM,BS2,NROW,ROW,COL,RMAX,AI,AJ,RP,AIMAX,NONEW,datatype) \
+  if (NROW >= RMAX) { \
+    Mat_SeqAIJ *Ain = (Mat_SeqAIJ*)Amat->data; \
+    /* there is no extra room in row, therefore enlarge */ \
+    PetscInt CHUNKSIZE = 15,new_nz = AI[AM] + CHUNKSIZE,len,*new_i=0,*new_j=0; \
+ \
+    if (NONEW == -2) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"New nonzero at (%D,%D) caused a malloc\nUse MatSetOption(A, MAT_NEW_NONZERO_ALLOCATION_ERR, PETSC_FALSE) to turn off this check",ROW,COL); \
+    /* malloc new storage space */ \
+    ierr = PetscMalloc1(new_nz,&new_j);CHKERRQ(ierr); \
+    ierr = PetscMalloc1(AM+1,&new_i);CHKERRQ(ierr);\
+ \
+    /* copy over old data into new slots */ \
+    for (ii=0; ii<ROW+1; ii++) {new_i[ii] = AI[ii];} \
+    for (ii=ROW+1; ii<AM+1; ii++) {new_i[ii] = AI[ii]+CHUNKSIZE;} \
+    ierr = PetscMemcpy(new_j,AJ,(AI[ROW]+NROW)*sizeof(PetscInt));CHKERRQ(ierr); \
+    len  = (new_nz - CHUNKSIZE - AI[ROW] - NROW); \
+    ierr = PetscMemcpy(new_j+AI[ROW]+NROW+CHUNKSIZE,AJ+AI[ROW]+NROW,len*sizeof(PetscInt));CHKERRQ(ierr); \
+ \
+    /* free up old matrix storage */ \
+    ierr              = MatSeqXAIJFreeAIJ(A,&Ain->a,&Ain->j,&Ain->i);CHKERRQ(ierr); \
+    Ain->a            = NULL;                   \
+    AI                = Ain->i = new_i; AJ = Ain->j = new_j;  \
+    Ain->singlemalloc = PETSC_FALSE; \
+    Ain->free_a       = PETSC_FALSE;             \
+ \
+    RP          = AJ + AI[ROW];    \
+    RMAX        = AIMAX[ROW] = AIMAX[ROW] + CHUNKSIZE; \
+    Ain->maxnz += BS2*CHUNKSIZE; \
+    Ain->reallocs++; \
+  } \
 
 PETSC_INTERN PetscErrorCode MatSeqAIJSetPreallocation_SeqAIJ(Mat,PetscInt,const PetscInt*);
 PETSC_INTERN PetscErrorCode MatILUFactorSymbolic_SeqAIJ_inplace(Mat,Mat,IS,IS,const MatFactorInfo*);
@@ -204,11 +236,10 @@ PETSC_INTERN PetscErrorCode MatMultTransposeAdd_SeqAIJ(Mat A,Vec,Vec,Vec);
 PETSC_INTERN PetscErrorCode MatSOR_SeqAIJ(Mat,Vec,PetscReal,MatSORType,PetscReal,PetscInt,PetscInt,Vec);
 
 PETSC_INTERN PetscErrorCode MatSetOption_SeqAIJ(Mat,MatOption,PetscBool);
-PETSC_INTERN PetscErrorCode MatSetColoring_SeqAIJ(Mat,ISColoring);
-PETSC_INTERN PetscErrorCode MatSetValuesAdifor_SeqAIJ(Mat,PetscInt,void*);
 
 PETSC_INTERN PetscErrorCode MatGetSymbolicTranspose_SeqAIJ(Mat,PetscInt *[],PetscInt *[]);
 PETSC_INTERN PetscErrorCode MatGetSymbolicTransposeReduced_SeqAIJ(Mat,PetscInt,PetscInt,PetscInt *[],PetscInt *[]);
+PETSC_INTERN PetscErrorCode MatTranspose_SeqAIJ_FAST(Mat,MatReuse,Mat *);
 PETSC_INTERN PetscErrorCode MatRestoreSymbolicTranspose_SeqAIJ(Mat,PetscInt *[],PetscInt *[]);
 PETSC_INTERN PetscErrorCode MatTransposeSymbolic_SeqAIJ(Mat,Mat*);
 PETSC_INTERN PetscErrorCode MatTranspose_SeqAIJ(Mat,MatReuse,Mat*);
@@ -238,21 +269,22 @@ PETSC_INTERN PetscErrorCode MatEqual_SeqAIJ(Mat,Mat,PetscBool*);
 PETSC_INTERN PetscErrorCode MatFDColoringCreate_SeqXAIJ(Mat,ISColoring,MatFDColoring);
 PETSC_INTERN PetscErrorCode MatFDColoringSetUp_SeqXAIJ(Mat,ISColoring,MatFDColoring);
 PETSC_INTERN PetscErrorCode MatFDColoringSetUpBlocked_AIJ_Private(Mat,MatFDColoring,PetscInt);
-PETSC_INTERN PetscErrorCode MatFDColoringApply_AIJ(Mat,MatFDColoring,Vec,void*);
 PETSC_INTERN PetscErrorCode MatLoad_SeqAIJ(Mat,PetscViewer);
 PETSC_INTERN PetscErrorCode RegisterApplyPtAPRoutines_Private(Mat);
 
 PETSC_INTERN PetscErrorCode MatMatMult_SeqAIJ_SeqAIJ(Mat,Mat,MatReuse,PetscReal,Mat*);
 PETSC_INTERN PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ(Mat,Mat,PetscReal,Mat*);
+PETSC_INTERN PetscErrorCode MatMatMultSymbolic_SeqDense_SeqAIJ(Mat,Mat,PetscReal,Mat*);
 PETSC_INTERN PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_Scalable(Mat,Mat,PetscReal,Mat*);
 PETSC_INTERN PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_Scalable_fast(Mat,Mat,PetscReal,Mat*);
 PETSC_INTERN PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_Heap(Mat,Mat,PetscReal,Mat*);
 PETSC_INTERN PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_BTHeap(Mat,Mat,PetscReal,Mat*);
+PETSC_INTERN PetscErrorCode MatMatMult_SeqAIJ_SeqAIJ_Combined(Mat,Mat,PetscReal,Mat*);
 PETSC_INTERN PetscErrorCode MatMatMultNumeric_SeqAIJ_SeqAIJ(Mat,Mat,Mat);
+PETSC_INTERN PetscErrorCode MatMatMultNumeric_SeqDense_SeqAIJ(Mat,Mat,Mat);
 PETSC_INTERN PetscErrorCode MatMatMultNumeric_SeqAIJ_SeqAIJ_Scalable(Mat,Mat,Mat);
 
 PETSC_INTERN PetscErrorCode MatPtAP_SeqAIJ_SeqAIJ(Mat,Mat,MatReuse,PetscReal,Mat*);
-PETSC_INTERN PetscErrorCode MatPtAPSymbolic_SeqAIJ_SeqAIJ_DenseAxpy(Mat,Mat,PetscReal,Mat*);
 PETSC_INTERN PetscErrorCode MatPtAPSymbolic_SeqAIJ_SeqAIJ_SparseAxpy(Mat,Mat,PetscReal,Mat*);
 PETSC_INTERN PetscErrorCode MatPtAPNumeric_SeqAIJ_SeqAIJ(Mat,Mat,Mat);
 PETSC_INTERN PetscErrorCode MatPtAPNumeric_SeqAIJ_SeqAIJ_SparseAxpy(Mat,Mat,Mat);
@@ -267,6 +299,7 @@ PETSC_INTERN PetscErrorCode MatRARtNumeric_SeqAIJ_SeqAIJ_colorrart(Mat,Mat,Mat);
 PETSC_INTERN PetscErrorCode MatTransposeMatMult_SeqAIJ_SeqAIJ(Mat,Mat,MatReuse,PetscReal,Mat*);
 PETSC_INTERN PetscErrorCode MatTransposeMatMultSymbolic_SeqAIJ_SeqAIJ(Mat,Mat,PetscReal,Mat*);
 PETSC_INTERN PetscErrorCode MatTransposeMatMultNumeric_SeqAIJ_SeqAIJ(Mat,Mat,Mat);
+PETSC_INTERN PetscErrorCode MatDestroy_SeqAIJ_MatTransMatMult(Mat);
 
 PETSC_INTERN PetscErrorCode MatTransposeMatMult_SeqAIJ_SeqDense(Mat,Mat,MatReuse,PetscReal,Mat*);
 PETSC_INTERN PetscErrorCode MatTransposeMatMultSymbolic_SeqAIJ_SeqDense(Mat,Mat,PetscReal,Mat*);
@@ -286,6 +319,9 @@ PETSC_INTERN PetscErrorCode MatMatMatMultNumeric_SeqAIJ_SeqAIJ_SeqAIJ(Mat,Mat,Ma
 PETSC_INTERN PetscErrorCode MatSetValues_SeqAIJ(Mat,PetscInt,const PetscInt[],PetscInt,const PetscInt[],const PetscScalar[],InsertMode);
 PETSC_INTERN PetscErrorCode MatGetRow_SeqAIJ(Mat,PetscInt,PetscInt*,PetscInt**,PetscScalar**);
 PETSC_INTERN PetscErrorCode MatRestoreRow_SeqAIJ(Mat,PetscInt,PetscInt*,PetscInt**,PetscScalar**);
+PETSC_INTERN PetscErrorCode MatScale_SeqAIJ(Mat,PetscScalar);
+PETSC_INTERN PetscErrorCode MatDiagonalScale_SeqAIJ(Mat,Vec,Vec);
+PETSC_INTERN PetscErrorCode MatDiagonalSet_SeqAIJ(Mat,Vec,InsertMode);
 PETSC_INTERN PetscErrorCode MatAXPY_SeqAIJ(Mat,PetscScalar,Mat,MatStructure);
 PETSC_INTERN PetscErrorCode MatGetRowIJ_SeqAIJ(Mat,PetscInt,PetscBool,PetscBool,PetscInt*,const PetscInt *[],const PetscInt *[],PetscBool*);
 PETSC_INTERN PetscErrorCode MatRestoreRowIJ_SeqAIJ(Mat,PetscInt,PetscBool,PetscBool,PetscInt*,const PetscInt *[],const PetscInt *[],PetscBool*);
@@ -304,21 +340,34 @@ PETSC_INTERN PetscErrorCode MatSeqAIJCheckInode_FactorLU(Mat);
 
 PETSC_INTERN PetscErrorCode MatAXPYGetPreallocation_SeqAIJ(Mat,Mat,PetscInt*);
 
+PETSC_INTERN PetscErrorCode MatMatMatMult_Transpose_AIJ_AIJ(Mat,Mat,Mat,MatReuse,PetscReal,Mat*);
+PETSC_EXTERN PetscErrorCode MatlabEnginePut_SeqAIJ(PetscObject,void*);
+PETSC_EXTERN PetscErrorCode MatlabEngineGet_SeqAIJ(PetscObject,void*);
 PETSC_INTERN PetscErrorCode MatConvert_SeqAIJ_SeqSBAIJ(Mat,MatType,MatReuse,Mat*);
 PETSC_INTERN PetscErrorCode MatConvert_SeqAIJ_SeqBAIJ(Mat,MatType,MatReuse,Mat*);
+PETSC_INTERN PetscErrorCode MatConvert_SeqAIJ_SeqDense(Mat,MatType,MatReuse,Mat*);
+PETSC_INTERN PetscErrorCode MatConvert_SeqAIJ_SeqAIJCRL(Mat,MatType,MatReuse,Mat*);
+PETSC_INTERN PetscErrorCode MatConvert_SeqAIJ_Elemental(Mat,MatType,MatReuse,Mat*);
+PETSC_INTERN PetscErrorCode MatConvert_AIJ_HYPRE(Mat,MatType,MatReuse,Mat*);
 PETSC_INTERN PetscErrorCode MatConvert_SeqAIJ_SeqAIJPERM(Mat,MatType,MatReuse,Mat*);
+PETSC_INTERN PetscErrorCode MatConvert_SeqAIJ_SeqAIJSELL(Mat,MatType,MatReuse,Mat*);
+PETSC_INTERN PetscErrorCode MatConvert_SeqAIJ_SeqAIJMKL(Mat,MatType,MatReuse,Mat*);
+PETSC_INTERN PetscErrorCode MatConvert_SeqAIJ_SeqAIJViennaCL(Mat,MatType,MatReuse,Mat*);
 PETSC_INTERN PetscErrorCode MatReorderForNonzeroDiagonal_SeqAIJ(Mat,PetscReal,IS,IS);
 PETSC_INTERN PetscErrorCode MatMatMult_SeqDense_SeqAIJ(Mat,Mat,MatReuse,PetscReal,Mat*);
 PETSC_INTERN PetscErrorCode MatRARt_SeqAIJ_SeqAIJ(Mat,Mat,MatReuse,PetscReal,Mat*);
 PETSC_EXTERN PetscErrorCode MatCreate_SeqAIJ(Mat);
 PETSC_INTERN PetscErrorCode MatAssemblyEnd_SeqAIJ(Mat,MatAssemblyType);
-PETSC_INTERN PetscErrorCode MatDestroy_SeqAIJ(Mat);
 
 PETSC_INTERN PetscErrorCode MatAXPYGetPreallocation_SeqX_private(PetscInt,const PetscInt*,const PetscInt*,const PetscInt*,const PetscInt*,PetscInt*);
 PETSC_INTERN PetscErrorCode MatCreateMPIMatConcatenateSeqMat_SeqAIJ(MPI_Comm,Mat,PetscInt,MatReuse,Mat*);
 PETSC_INTERN PetscErrorCode MatCreateMPIMatConcatenateSeqMat_MPIAIJ(MPI_Comm,Mat,PetscInt,MatReuse,Mat*);
 
 PETSC_INTERN PetscErrorCode MatSetSeqMat_SeqAIJ(Mat,IS,IS,MatStructure,Mat);
+PETSC_INTERN PetscErrorCode MatDestroySubMatrix_Private(Mat_SubSppt*);
+PETSC_INTERN PetscErrorCode MatDestroySubMatrix_SeqAIJ(Mat);
+PETSC_INTERN PetscErrorCode MatDestroySubMatrix_Dummy(Mat);
+PETSC_INTERN PetscErrorCode MatCreateSubMatrix_SeqAIJ(Mat,IS,IS,PetscInt,MatReuse,Mat*);
 
 /*
     PetscSparseDenseMinusDot - The inner kernel of triangular solves and Gauss-Siedel smoothing. \sum_i xv[i] * r[xi[i]] for CSR storage
@@ -333,8 +382,11 @@ PETSC_INTERN PetscErrorCode MatSetSeqMat_SeqAIJ(Mat,IS,IS,MatStructure,Mat);
 .  sum - negative the sum of results
 
   PETSc compile flags:
-+   PETSC_KERNEL_USE_UNROLL_4 -   don't use this; it changes nnz and hence is WRONG
--   PETSC_KERNEL_USE_UNROLL_2 -
++   PETSC_KERNEL_USE_UNROLL_4
+-   PETSC_KERNEL_USE_UNROLL_2
+
+  Developer Notes:
+    The macro changes sum but not other parameters
 
 .seealso: PetscSparseDensePlusDot()
 
@@ -342,15 +394,20 @@ PETSC_INTERN PetscErrorCode MatSetSeqMat_SeqAIJ(Mat,IS,IS,MatStructure,Mat);
 #if defined(PETSC_KERNEL_USE_UNROLL_4)
 #define PetscSparseDenseMinusDot(sum,r,xv,xi,nnz) { \
     if (nnz > 0) { \
-      switch (nnz & 0x3) { \
+      PetscInt nnz2=nnz,rem=nnz&0x3; \
+      switch (rem) { \
       case 3: sum -= *xv++ *r[*xi++]; \
       case 2: sum -= *xv++ *r[*xi++]; \
       case 1: sum -= *xv++ *r[*xi++]; \
-        nnz       -= 4;} \
-      while (nnz > 0) { \
-        sum -=  xv[0] * r[xi[0]] - xv[1] * r[xi[1]] - \
-               xv[2] * r[xi[2]] - xv[3] * r[xi[3]]; \
-        xv += 4; xi += 4; nnz -= 4; }}}
+        nnz2      -= rem;} \
+      while (nnz2 > 0) { \
+        sum -=  xv[0] * r[xi[0]] + xv[1] * r[xi[1]] + \
+                xv[2] * r[xi[2]] + xv[3] * r[xi[3]]; \
+        xv += 4; xi += 4; nnz2 -= 4; \
+      } \
+      xv -= nnz; xi -= nnz; \
+    } \
+  }
 
 #elif defined(PETSC_KERNEL_USE_UNROLL_2)
 #define PetscSparseDenseMinusDot(sum,r,xv,xi,nnz) { \
@@ -380,8 +437,11 @@ PETSC_INTERN PetscErrorCode MatSetSeqMat_SeqAIJ(Mat,IS,IS,MatStructure,Mat);
 .  sum - the sum of results
 
   PETSc compile flags:
-+   PETSC_KERNEL_USE_UNROLL_4 -  don't use this; it changes nnz and hence is WRONG
--   PETSC_KERNEL_USE_UNROLL_2 -
++   PETSC_KERNEL_USE_UNROLL_4
+-   PETSC_KERNEL_USE_UNROLL_2
+
+  Developer Notes:
+    The macro changes sum but not other parameters
 
 .seealso: PetscSparseDenseMinusDot()
 
@@ -389,15 +449,20 @@ PETSC_INTERN PetscErrorCode MatSetSeqMat_SeqAIJ(Mat,IS,IS,MatStructure,Mat);
 #if defined(PETSC_KERNEL_USE_UNROLL_4)
 #define PetscSparseDensePlusDot(sum,r,xv,xi,nnz) { \
     if (nnz > 0) { \
-      switch (nnz & 0x3) { \
+      PetscInt nnz2=nnz,rem=nnz&0x3; \
+      switch (rem) { \
       case 3: sum += *xv++ *r[*xi++]; \
       case 2: sum += *xv++ *r[*xi++]; \
       case 1: sum += *xv++ *r[*xi++]; \
-        nnz       -= 4;} \
-      while (nnz > 0) { \
+        nnz2      -= rem;} \
+      while (nnz2 > 0) { \
         sum +=  xv[0] * r[xi[0]] + xv[1] * r[xi[1]] + \
-               xv[2] * r[xi[2]] + xv[3] * r[xi[3]]; \
-        xv += 4; xi += 4; nnz -= 4; }}}
+                xv[2] * r[xi[2]] + xv[3] * r[xi[3]]; \
+        xv += 4; xi += 4; nnz2 -= 4; \
+      } \
+      xv -= nnz; xi -= nnz; \
+    } \
+  }
 
 #elif defined(PETSC_KERNEL_USE_UNROLL_2)
 #define PetscSparseDensePlusDot(sum,r,xv,xi,nnz) { \
@@ -406,12 +471,55 @@ PETSC_INTERN PetscErrorCode MatSetSeqMat_SeqAIJ(Mat,IS,IS,MatStructure,Mat);
                                     sum += (xv[__i]*r[__i1] + xv[__i+1]*r[__i2]);} \
     if (nnz & 0x1) sum += xv[__i] * r[xi[__i]];}
 
+#elif defined(PETSC_USE_AVX512_KERNELS) && defined(PETSC_HAVE_IMMINTRIN_H) && defined(__AVX512F__) && defined(PETSC_USE_REAL_DOUBLE) && !defined(PETSC_USE_COMPLEX) && !defined(PETSC_USE_64BIT_INDICES)
+#define PetscSparseDensePlusDot(sum,r,xv,xi,nnz) PetscSparseDensePlusDot_AVX512_Private(&(sum),(r),(xv),(xi),(nnz))
+
 #else
 #define PetscSparseDensePlusDot(sum,r,xv,xi,nnz) { \
     PetscInt __i; \
     for (__i=0; __i<nnz; __i++) sum += xv[__i] * r[xi[__i]];}
 #endif
 
+#if defined(PETSC_USE_AVX512_KERNELS) && defined(PETSC_HAVE_IMMINTRIN_H) && defined(__AVX512F__) && defined(PETSC_USE_REAL_DOUBLE) && !defined(PETSC_USE_COMPLEX) && !defined(PETSC_USE_64BIT_INDICES)
+  #include <immintrin.h>
+  #if !defined(_MM_SCALE_8)
+  #define _MM_SCALE_8    8
+  #endif
+
+PETSC_STATIC_INLINE void PetscSparseDensePlusDot_AVX512_Private(PetscScalar *sum,const PetscScalar *x,const MatScalar *aa,const PetscInt *aj,PetscInt n)
+{
+  __m512d  vec_x,vec_y,vec_vals;
+  __m256i  vec_idx;
+  __mmask8 mask;
+  PetscInt j;
+
+  vec_y = _mm512_setzero_pd();
+  for (j=0; j<(n>>3); j++) {
+    vec_idx  = _mm256_loadu_si256((__m256i const*)aj);
+    vec_vals = _mm512_loadu_pd(aa);
+    vec_x    = _mm512_i32gather_pd(vec_idx,x,_MM_SCALE_8);
+    vec_y    = _mm512_fmadd_pd(vec_x,vec_vals,vec_y);
+    aj += 8; aa += 8;
+  }
+  /* masked load does not work on KNL, it requires avx512vl */
+  if ((n&0x07)>2) {
+    mask     = (__mmask8)(0xff >> (8-(n&0x07)));
+    vec_idx  = _mm256_loadu_si256((__m256i const*)aj);
+    vec_vals = _mm512_loadu_pd(aa);
+    vec_x    = _mm512_mask_i32gather_pd(vec_x,mask,vec_idx,x,_MM_SCALE_8);
+    vec_y    = _mm512_mask3_fmadd_pd(vec_x,vec_vals,vec_y,mask);
+  } else if ((n&0x07)==2) {
+    *sum += aa[0]*x[aj[0]];
+    *sum += aa[1]*x[aj[1]];
+  } else if ((n&0x07)==1) {
+    *sum += aa[0]*x[aj[0]];
+  }
+  if (n>2) *sum += _mm512_reduce_add_pd(vec_y);
+/*
+  for(j=0;j<(n&0x07);j++) *sum += aa[j]*x[aj[j]];
+*/
+}
+#endif
 
 /*
     PetscSparseDenseMaxDot - The inner kernel of a modified matrix-vector product \max_i xv[i] * r[xi[i]] for CSR storage
@@ -437,13 +545,15 @@ PETSC_INTERN PetscErrorCode MatSetSeqMat_SeqAIJ(Mat,IS,IS,MatStructure,Mat);
  */
 #define MatRowMergeMax_SeqAIJ(mat,nrows,ta) {       \
     PetscInt _j,_row,_nz,*_col;                     \
-    for (_row=0; _row<nrows; _row++) {\
-      _nz = mat->i[_row+1] - mat->i[_row]; \
-      for (_j=0; _j<_nz; _j++) {                \
-        _col = _j + mat->j + mat->i[_row];       \
-        PetscTableAdd(ta,*_col+1,1,INSERT_VALUES); \
+    if (mat) { \
+      for (_row=0; _row<nrows; _row++) {   \
+        _nz = mat->i[_row+1] - mat->i[_row];    \
+        for (_j=0; _j<_nz; _j++) {               \
+          _col = _j + mat->j + mat->i[_row];       \
+          PetscTableAdd(ta,*_col+1,1,INSERT_VALUES);                    \
+        }                                                               \
       }                                                                 \
-    }                                                                   \
+    }    \
 }
 
 /*
