@@ -7,14 +7,19 @@
   Works with MATAIJ  matrices
 
   Options Database Keys:
-+ -pc_factor_levels <l>
-- -pc_factor_drop_tolerance
++ -pc_factor_levels <l> - number of levels of fill
+- -pc_factor_drop_tolerance - is not currently hooked up to do anything
 
-  Level: beginner
+  Level: intermediate
 
    Contributed by: Bas van 't Hof
 
-.seealso: PCFactorSetMatSolverPackage(), MatSolverPackage, PCFactorSetLevels(), PCFactorSetDropTolerance()
+   Notes:
+    Since this currently hooked up to use drop tolerance it should produce the same factors and hence convergence as the PETSc ICC, for higher 
+     levels of fill it does not. This needs to be investigated. Unless you are interested in drop tolerance ICC and willing to work through the code
+     we recommend not using this functionality.
+
+.seealso: PCFactorSetMatSolverType(), MatSolverType, PCFactorSetLevels(), PCFactorSetDropTolerance()
 
 M*/
 
@@ -22,15 +27,13 @@ M*/
   spbas_memory_requirement:
     Calculate the number of bytes needed to store tha matrix
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_memory_requirement"
-long int spbas_memory_requirement(spbas_matrix matrix)
+size_t spbas_memory_requirement(spbas_matrix matrix)
 {
-  long int memreq = 6 * sizeof(PetscInt)  + /* nrows, ncols, nnz, n_alloc_icol, n_alloc_val, col_idx_type */
+  size_t memreq = 6 * sizeof(PetscInt)  + /* nrows, ncols, nnz, n_alloc_icol, n_alloc_val, col_idx_type */
                     sizeof(PetscBool)               + /* block_data */
                     sizeof(PetscScalar**)           + /* values */
                     sizeof(PetscScalar*)            + /* alloc_val */
-                    2 * sizeof(int**)               + /* icols, icols0 */
+                    2 * sizeof(PetscInt**)          + /* icols, icols0 */
                     2 * sizeof(PetscInt*)           + /* row_nnz, alloc_icol */
                     matrix.nrows * sizeof(PetscInt) + /* row_nnz[*] */
                     matrix.nrows * sizeof(PetscInt*); /* icols[*] */
@@ -55,8 +58,6 @@ long int spbas_memory_requirement(spbas_matrix matrix)
   spbas_allocate_pattern:
     allocate the pattern arrays row_nnz, icols and optionally values
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_allocate_pattern"
 PetscErrorCode spbas_allocate_pattern(spbas_matrix * result, PetscBool do_values)
 {
   PetscErrorCode ierr;
@@ -92,8 +93,6 @@ spbas_allocate_data:
    in case of !block_data:
        Allocate the arrays icols[i] and optionally values[i]
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_allocate_data"
 PetscErrorCode spbas_allocate_data(spbas_matrix * result)
 {
   PetscInt       i;
@@ -101,7 +100,7 @@ PetscErrorCode spbas_allocate_data(spbas_matrix * result)
   PetscInt       nrows = result->nrows;
   PetscInt       r_nnz;
   PetscErrorCode ierr;
-  PetscBool      do_values  = (result->values != NULL) ? PETSC_TRUE : PETSC_FALSE;
+  PetscBool      do_values  = (result->values) ? PETSC_TRUE : PETSC_FALSE;
   PetscBool      block_data = result->block_data;
 
   PetscFunctionBegin;
@@ -149,8 +148,6 @@ PetscErrorCode spbas_allocate_data(spbas_matrix * result)
        + after (return 1)
        + is identical (return 0).
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_row_order_icol"
 int spbas_row_order_icol(PetscInt i1, PetscInt i2, PetscInt *irow_in, PetscInt *icol_in,PetscInt col_idx_type)
 {
   PetscInt j;
@@ -186,8 +183,6 @@ int spbas_row_order_icol(PetscInt i1, PetscInt i2, PetscInt *irow_in, PetscInt *
     return a sorting of the rows in which identical sparseness patterns are
     next to each other
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_mergesort_icols"
 PetscErrorCode spbas_mergesort_icols(PetscInt nrows, PetscInt * irow_in, PetscInt * icol_in,PetscInt col_idx_type, PetscInt *isort)
 {
   PetscErrorCode ierr;
@@ -256,13 +251,11 @@ PetscErrorCode spbas_mergesort_icols(PetscInt nrows, PetscInt * irow_in, PetscIn
      given in compressed row storage. The compressed sparseness pattern may
      require (much) less memory.
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_compress_pattern"
 PetscErrorCode spbas_compress_pattern(PetscInt *irow_in, PetscInt *icol_in, PetscInt nrows, PetscInt ncols, PetscInt col_idx_type, spbas_matrix *B,PetscReal *mem_reduction)
 {
   PetscInt        nnz      = irow_in[nrows];
-  long int        mem_orig = (nrows + nnz) * sizeof(PetscInt);
-  long int        mem_compressed;
+  size_t          mem_orig = (nrows + nnz) * sizeof(PetscInt);
+  size_t          mem_compressed;
   PetscErrorCode  ierr;
   PetscInt        *isort;
   PetscInt        *icols;
@@ -371,8 +364,6 @@ PetscErrorCode spbas_compress_pattern(PetscInt *irow_in, PetscInt *icol_in, Pets
 /*
   spbas_delete : de-allocate the arrays owned by this matrix
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_delete"
 PetscErrorCode spbas_delete(spbas_matrix matrix)
 {
   PetscInt       i;
@@ -401,8 +392,6 @@ PetscErrorCode spbas_delete(spbas_matrix matrix)
 spbas_matrix_to_crs:
    Convert an spbas_matrix to compessed row storage
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_matrix_to_crs"
 PetscErrorCode spbas_matrix_to_crs(spbas_matrix matrix_A,MatScalar **val_out, PetscInt **irow_out, PetscInt **icol_out)
 {
   PetscInt       nrows = matrix_A.nrows;
@@ -459,8 +448,6 @@ PetscErrorCode spbas_matrix_to_crs(spbas_matrix matrix_A,MatScalar **val_out, Pe
     spbas_transpose
        return the transpose of a matrix
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_transpose"
 PetscErrorCode spbas_transpose(spbas_matrix in_matrix, spbas_matrix * result)
 {
   PetscInt       col_idx_type = in_matrix.col_idx_type;
@@ -546,7 +533,7 @@ PetscErrorCode spbas_transpose(spbas_matrix in_matrix, spbas_matrix * result)
 /*
    spbas_mergesort
 
-      mergesort for an array of intergers and an array of associated
+      mergesort for an array of integers and an array of associated
       reals
 
       on output, icol[0..nnz-1] is increasing;
@@ -555,8 +542,6 @@ PetscErrorCode spbas_transpose(spbas_matrix in_matrix, spbas_matrix * result)
       NB: val may be NULL: in that case, only the integers are sorted
 
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_mergesort"
 PetscErrorCode spbas_mergesort(PetscInt nnz, PetscInt *icol, PetscScalar *val)
 {
   PetscInt       istep;       /* Chunk-sizes of already sorted parts of arrays */
@@ -654,8 +639,6 @@ PetscErrorCode spbas_mergesort(PetscInt nnz, PetscInt *icol, PetscScalar *val)
   spbas_apply_reordering_rows:
     apply the given reordering to the rows:  matrix_A = matrix_A(perm,:);
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_apply_reordering_rows"
 PetscErrorCode spbas_apply_reordering_rows(spbas_matrix *matrix_A, const PetscInt *permutation)
 {
   PetscInt       i,j,ip;
@@ -698,8 +681,6 @@ PetscErrorCode spbas_apply_reordering_rows(spbas_matrix *matrix_A, const PetscIn
   spbas_apply_reordering_cols:
     apply the given reordering to the columns:  matrix_A(:,perm) = matrix_A;
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_apply_reordering_cols"
 PetscErrorCode spbas_apply_reordering_cols(spbas_matrix *matrix_A,const PetscInt *permutation)
 {
   PetscInt       i,j;
@@ -730,8 +711,6 @@ PetscErrorCode spbas_apply_reordering_cols(spbas_matrix *matrix_A,const PetscInt
   spbas_apply_reordering:
     apply the given reordering:  matrix_A(perm,perm) = matrix_A;
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_apply_reordering"
 PetscErrorCode spbas_apply_reordering(spbas_matrix *matrix_A, const PetscInt *permutation, const PetscInt * inv_perm)
 {
   PetscErrorCode ierr;
@@ -742,8 +721,6 @@ PetscErrorCode spbas_apply_reordering(spbas_matrix *matrix_A, const PetscInt *pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "spbas_pattern_only"
 PetscErrorCode spbas_pattern_only(PetscInt nrows, PetscInt ncols, PetscInt *ai, PetscInt *aj, spbas_matrix * result)
 {
   spbas_matrix   retval;
@@ -782,8 +759,6 @@ PetscErrorCode spbas_pattern_only(PetscInt nrows, PetscInt ncols, PetscInt *ai,
       Mark the columns in row 'row' which are nonzero in
           matrix^2log(marker).
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_mark_row_power"
 PetscErrorCode spbas_mark_row_power(PetscInt *iwork,             /* marker-vector */
                                     PetscInt row,                /* row for which the columns are marked */
                                     spbas_matrix * in_matrix,    /* matrix for which the power is being  calculated */
@@ -824,8 +799,6 @@ PetscErrorCode spbas_mark_row_power(PetscInt *iwork,             /* marker-vecto
       are inside the band width are found and stored in the output sparseness
       pattern.
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_power"
 PetscErrorCode spbas_power(spbas_matrix in_matrix,PetscInt power, spbas_matrix * result)
 {
   spbas_matrix   retval;
@@ -901,8 +874,6 @@ PetscErrorCode spbas_power(spbas_matrix in_matrix,PetscInt power, spbas_matrix *
    spbas_keep_upper:
       remove the lower part of the matrix: keep the upper part
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_keep_upper"
 PetscErrorCode spbas_keep_upper(spbas_matrix * inout_matrix)
 {
   PetscInt i, j;
