@@ -5,8 +5,6 @@
       Return PETSC_ERR_MEM if there is insufficient space to store the
       row, so garbage collection and/or re-allocation may be done.
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_cholesky_row_alloc"
 PetscErrorCode spbas_cholesky_row_alloc(spbas_matrix retval, PetscInt k, PetscInt r_nnz,PetscInt * n_alloc_used)
 {
   PetscFunctionBegin;
@@ -30,8 +28,6 @@ PetscErrorCode spbas_cholesky_row_alloc(spbas_matrix retval, PetscInt k, PetscIn
          PetscRealloc does not seem to exist.
 
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_cholesky_garbage_collect"
 PetscErrorCode spbas_cholesky_garbage_collect(spbas_matrix *result,         /* I/O: the Cholesky factor matrix being constructed.
                                                                                     Only the storage, not the contents of this matrix is changed in this function */
                                               PetscInt     i_row,           /* I  : Number of rows for which the final contents are known */
@@ -157,7 +153,7 @@ PetscErrorCode spbas_cholesky_garbage_collect(spbas_matrix *result,         /* I
     i_last = i_here + result->row_nnz[i];
     if (result->row_nnz[i]>0) {
       if (*n_alloc_used > i_here || i_last > n_alloc) {
-        ierr = PetscMemcpy((void*) &icol_rescue[n_rescue], (void*) result->icols[i], result->row_nnz[i] * sizeof(int));CHKERRQ(ierr);
+        ierr = PetscMemcpy((void*) &icol_rescue[n_rescue], (void*) result->icols[i], result->row_nnz[i] * sizeof(PetscInt));CHKERRQ(ierr);
         ierr = PetscMemcpy((void*) &val_rescue[n_rescue], (void*) result->values[i], result->row_nnz[i] * sizeof(PetscScalar));CHKERRQ(ierr);
 
         n_rescue += result->row_nnz[i];
@@ -179,7 +175,7 @@ PetscErrorCode spbas_cholesky_garbage_collect(spbas_matrix *result,         /* I
 
         Allocate new icol-data, copy old contents */
     ierr = PetscMalloc1(n_alloc, &result->alloc_icol);CHKERRQ(ierr);
-    ierr = PetscMemcpy(result->alloc_icol, alloc_icol_old, n_copy*sizeof(int));CHKERRQ(ierr);
+    ierr = PetscMemcpy(result->alloc_icol, alloc_icol_old, n_copy*sizeof(PetscInt));CHKERRQ(ierr);
 
     /* Update administration, Reset pointers to new arrays  */
     result->n_alloc_icol = n_alloc;
@@ -225,7 +221,7 @@ PetscErrorCode spbas_cholesky_garbage_collect(spbas_matrix *result,         /* I
 
     if (result->row_nnz[i]>0) {
       if (*n_alloc_used > i_here || i_last > n_alloc) {
-        ierr = PetscMemcpy((void*) result->icols[i],  (void*) &icol_rescue[n_rescue], result->row_nnz[i] * sizeof(int));CHKERRQ(ierr);
+        ierr = PetscMemcpy((void*) result->icols[i],  (void*) &icol_rescue[n_rescue], result->row_nnz[i] * sizeof(PetscInt));CHKERRQ(ierr);
         ierr = PetscMemcpy((void*) result->values[i], (void*) &val_rescue[n_rescue],result->row_nnz[i] * sizeof(PetscScalar));CHKERRQ(ierr);
 
         n_rescue += result->row_nnz[i];
@@ -259,8 +255,6 @@ PetscErrorCode spbas_cholesky_garbage_collect(spbas_matrix *result,         /* I
      with a larger epsdiag_in, a less sparse pattern, and/or a smaller
      droptol
 */
-#undef __FUNCT__
-#define __FUNCT__ "spbas_incomplete_cholesky"
 PetscErrorCode spbas_incomplete_cholesky(Mat A, const PetscInt *rip, const PetscInt *riip, spbas_matrix pattern, PetscReal droptol, PetscReal epsdiag_in, spbas_matrix * matrix_L)
 {
   PetscInt        jL;
@@ -310,7 +304,7 @@ PetscErrorCode spbas_incomplete_cholesky(Mat A, const PetscInt *rip, const Petsc
   retval.block_data   = PETSC_TRUE;
 
   ierr       = spbas_allocate_pattern(&retval, do_values);CHKERRQ(ierr);
-  ierr       = PetscMemzero((void*) retval.row_nnz, nrows*sizeof(int));CHKERRQ(ierr);
+  ierr       = PetscMemzero((void*) retval.row_nnz, nrows*sizeof(PetscInt));CHKERRQ(ierr);
   ierr       = spbas_allocate_data(&retval);CHKERRQ(ierr);
   retval.nnz = 0;
 
@@ -318,11 +312,10 @@ PetscErrorCode spbas_incomplete_cholesky(Mat A, const PetscInt *rip, const Petsc
   ierr = PetscMalloc1(nrows, &val);CHKERRQ(ierr);
   ierr = PetscMalloc1(nrows, &lvec);CHKERRQ(ierr);
   ierr = PetscMalloc1(nrows, &max_row_nnz);CHKERRQ(ierr);
-  if (!(diag && val && lvec && max_row_nnz)) SETERRQ(PETSC_COMM_SELF, PETSC_ERR_MEM, "Allocation error in spbas_incomplete_cholesky\n");
 
   ierr = PetscMemzero((void*) val, nrows*sizeof(PetscScalar));CHKERRQ(ierr);
   ierr = PetscMemzero((void*) lvec, nrows*sizeof(PetscScalar));CHKERRQ(ierr);
-  ierr = PetscMemzero((void*) max_row_nnz, nrows*sizeof(int));CHKERRQ(ierr);
+  ierr = PetscMemzero((void*) max_row_nnz, nrows*sizeof(PetscInt));CHKERRQ(ierr);
 
   /* Check correct format of sparseness pattern */
   if (pattern.col_idx_type != SPBAS_DIAGONAL_OFFSETS) SETERRQ(PETSC_COMM_SELF, PETSC_ERR_SUP_SYS, "Error in spbas_incomplete_cholesky: must have diagonal offsets in pattern\n");
