@@ -177,8 +177,6 @@ typedef struct  {
 #define Factorization_Pivot_Tolerance pow(2.2204460492503131E-16, 2.0 / 3.0)
 #define Factorization_Small_Tolerance 1e-15 /* pow(DBL_EPSILON, 0.8) */
 
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_LUSOL"
 PetscErrorCode MatDestroy_LUSOL(Mat A)
 {
   PetscErrorCode ierr;
@@ -205,8 +203,6 @@ PetscErrorCode MatDestroy_LUSOL(Mat A)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__  "MatSolve_LUSOL"
 PetscErrorCode MatSolve_LUSOL(Mat A,Vec b,Vec x)
 {
   Mat_LUSOL      *lusol=(Mat_LUSOL*)A->spptr;
@@ -237,8 +233,6 @@ PetscErrorCode MatSolve_LUSOL(Mat A,Vec b,Vec x)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatLUFactorNumeric_LUSOL"
 PetscErrorCode MatLUFactorNumeric_LUSOL(Mat F,Mat A,const MatFactorInfo *info)
 {
   Mat_SeqAIJ     *a;
@@ -336,8 +330,6 @@ PetscErrorCode MatLUFactorNumeric_LUSOL(Mat F,Mat A,const MatFactorInfo *info)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatLUFactorSymbolic_LUSOL"
 PetscErrorCode MatLUFactorSymbolic_LUSOL(Mat F,Mat A, IS r, IS c,const MatFactorInfo *info)
 {
   /************************************************************************/
@@ -422,17 +414,13 @@ PetscErrorCode MatLUFactorSymbolic_LUSOL(Mat F,Mat A, IS r, IS c,const MatFactor
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFactorGetSolverPackage_seqaij_lusol"
-PetscErrorCode MatFactorGetSolverPackage_seqaij_lusol(Mat A,const MatSolverPackage *type)
+PetscErrorCode MatFactorGetSolverType_seqaij_lusol(Mat A,MatSolverType *type)
 {
   PetscFunctionBegin;
   *type = MATSOLVERLUSOL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatGetFactor_seqaij_lusol"
 PETSC_EXTERN PetscErrorCode MatGetFactor_seqaij_lusol(Mat A,MatFactorType ftype,Mat *F)
 {
   Mat            B;
@@ -453,20 +441,21 @@ PETSC_EXTERN PetscErrorCode MatGetFactor_seqaij_lusol(Mat A,MatFactorType ftype,
   B->ops->lufactorsymbolic = MatLUFactorSymbolic_LUSOL;
   B->ops->destroy          = MatDestroy_LUSOL;
 
-  ierr = PetscObjectComposeFunction((PetscObject)B,"MatFactorGetSolverPackage_C",MatFactorGetSolverPackage_seqaij_lusol);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)B,"MatFactorGetSolverType_C",MatFactorGetSolverType_seqaij_lusol);CHKERRQ(ierr);
 
   B->factortype = MAT_FACTOR_LU;
+  ierr = PetscFree(B->solvertype);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(MATSOLVERLUSOL,&B->solvertype);CHKERRQ(ierr);
+
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolverPackageRegister_Lusol"
-PETSC_EXTERN PetscErrorCode MatSolverPackageRegister_Lusol(void)
+PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_Lusol(void)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = MatSolverPackageRegister(MATSOLVERLUSOL,MATSEQAIJ,        MAT_FACTOR_LU,MatGetFactor_seqaij_lusol);CHKERRQ(ierr);
+  ierr = MatSolverTypeRegister(MATSOLVERLUSOL,MATSEQAIJ,        MAT_FACTOR_LU,MatGetFactor_seqaij_lusol);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -481,6 +470,6 @@ PETSC_EXTERN PetscErrorCode MatSolverPackageRegister_Lusol(void)
 
    Level: beginner
 
-.seealso: PCLU, PCFactorSetMatSolverPackage(), MatSolverPackage
+.seealso: PCLU, PCFactorSetMatSolverType(), MatSolverType
 
 M*/
