@@ -6,25 +6,34 @@
 
 #include <../src/mat/impls/aij/seq/aij.h> /*I "petscmat.h" I*/
 #include <../src/mat/utils/freespace.h>
-#include <../src/mat/utils/petscheap.h>
 #include <petscbt.h>
 #include <petsc/private/isimpl.h>
 #include <../src/mat/impls/dense/seq/dense.h>
 
 static PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_LLCondensed(Mat,Mat,PetscReal,Mat*);
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatMult_SeqAIJ_SeqAIJ"
+#if defined(PETSC_HAVE_HYPRE)
+PETSC_INTERN PetscErrorCode MatMatMultSymbolic_AIJ_AIJ_wHYPRE(Mat,Mat,PetscReal,Mat*);
+#endif
+
 PETSC_INTERN PetscErrorCode MatMatMult_SeqAIJ_SeqAIJ(Mat A,Mat B,MatReuse scall,PetscReal fill,Mat *C)
 {
   PetscErrorCode ierr;
-  const char     *algTypes[6] = {"sorted","scalable","scalable_fast","heap","btheap","llcondensed"};
-  PetscInt       alg=0; /* set default algorithm */
+#if !defined(PETSC_HAVE_HYPRE)
+  const char     *algTypes[7] = {"sorted","scalable","scalable_fast","heap","btheap","llcondensed","combined"};
+  PetscInt       nalg = 7;
+#else
+  const char     *algTypes[8] = {"sorted","scalable","scalable_fast","heap","btheap","llcondensed","combined","hypre"};
+  PetscInt       nalg = 8;
+#endif
+  PetscInt       alg = 0; /* set default algorithm */
+  PetscBool      combined = PETSC_FALSE;  /* Indicates whether the symbolic stage already computed the numerical values. */
 
   PetscFunctionBegin;
   if (scall == MAT_INITIAL_MATRIX) {
     ierr = PetscObjectOptionsBegin((PetscObject)A);CHKERRQ(ierr);
-    ierr = PetscOptionsEList("-matmatmult_via","Algorithmic approach","MatMatMult",algTypes,6,algTypes[0],&alg,NULL);CHKERRQ(ierr);
+    PetscOptionsObject->alreadyprinted = PETSC_FALSE; /* a hack to ensure the option shows in '-help' */
+    ierr = PetscOptionsEList("-matmatmult_via","Algorithmic approach","MatMatMult",algTypes,nalg,algTypes[0],&alg,NULL);CHKERRQ(ierr);
     ierr = PetscOptionsEnd();CHKERRQ(ierr);
     ierr = PetscLogEventBegin(MAT_MatMultSymbolic,A,B,0,0);CHKERRQ(ierr);
     switch (alg) {
@@ -43,6 +52,15 @@ PETSC_INTERN PetscErrorCode MatMatMult_SeqAIJ_SeqAIJ(Mat A,Mat B,MatReuse scall,
     case 5:
       ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ_LLCondensed(A,B,fill,C);CHKERRQ(ierr);
       break;
+    case 6:
+      ierr = MatMatMult_SeqAIJ_SeqAIJ_Combined(A,B,fill,C);CHKERRQ(ierr);
+      combined = PETSC_TRUE;
+      break;
+#if defined(PETSC_HAVE_HYPRE)
+    case 7:
+      ierr = MatMatMultSymbolic_AIJ_AIJ_wHYPRE(A,B,fill,C);CHKERRQ(ierr);
+      break;
+#endif
     default:
       ierr = MatMatMultSymbolic_SeqAIJ_SeqAIJ(A,B,fill,C);CHKERRQ(ierr);
      break;
@@ -51,13 +69,13 @@ PETSC_INTERN PetscErrorCode MatMatMult_SeqAIJ_SeqAIJ(Mat A,Mat B,MatReuse scall,
   }
 
   ierr = PetscLogEventBegin(MAT_MatMultNumeric,A,B,0,0);CHKERRQ(ierr);
-  ierr = (*(*C)->ops->matmultnumeric)(A,B,*C);CHKERRQ(ierr);
+  if (!combined) {
+    ierr = (*(*C)->ops->matmultnumeric)(A,B,*C);CHKERRQ(ierr);
+  }
   ierr = PetscLogEventEnd(MAT_MatMultNumeric,A,B,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatMultSymbolic_SeqAIJ_SeqAIJ_LLCondensed"
 static PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_LLCondensed(Mat A,Mat B,PetscReal fill,Mat *C)
 {
   PetscErrorCode     ierr;
@@ -160,8 +178,6 @@ static PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_LLCondensed(Mat A,Mat B,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatMultNumeric_SeqAIJ_SeqAIJ"
 PetscErrorCode MatMatMultNumeric_SeqAIJ_SeqAIJ(Mat A,Mat B,Mat C)
 {
   PetscErrorCode ierr;
@@ -225,8 +241,6 @@ PetscErrorCode MatMatMultNumeric_SeqAIJ_SeqAIJ(Mat A,Mat B,Mat C)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatMultNumeric_SeqAIJ_SeqAIJ_Scalable"
 PetscErrorCode MatMatMultNumeric_SeqAIJ_SeqAIJ_Scalable(Mat A,Mat B,Mat C)
 {
   PetscErrorCode ierr;
@@ -241,6 +255,12 @@ PetscErrorCode MatMatMultNumeric_SeqAIJ_SeqAIJ_Scalable(Mat A,Mat B,Mat C)
   PetscInt       nextb;
 
   PetscFunctionBegin;
+  if (!ca) { /* first call of MatMatMultNumeric_SeqAIJ_SeqAIJ, allocate ca and matmult_abdense */
+    ierr      = PetscMalloc1(ci[cm]+1,&ca);CHKERRQ(ierr);
+    c->a      = ca;
+    c->free_a = PETSC_TRUE;
+  } 
+
   /* clean old values in C */
   ierr = PetscMemzero(ca,ci[cm]*sizeof(MatScalar));CHKERRQ(ierr);
   /* Traverse A row-wise. */
@@ -274,8 +294,6 @@ PetscErrorCode MatMatMultNumeric_SeqAIJ_SeqAIJ_Scalable(Mat A,Mat B,Mat C)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatMultSymbolic_SeqAIJ_SeqAIJ_Scalable_fast"
 PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_Scalable_fast(Mat A,Mat B,PetscReal fill,Mat *C)
 {
   PetscErrorCode     ierr;
@@ -382,8 +400,6 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_Scalable_fast(Mat A,Mat B,PetscR
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatMultSymbolic_SeqAIJ_SeqAIJ_Scalable"
 PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_Scalable(Mat A,Mat B,PetscReal fill,Mat *C)
 {
   PetscErrorCode     ierr;
@@ -489,8 +505,6 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_Scalable(Mat A,Mat B,PetscReal f
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatMultSymbolic_SeqAIJ_SeqAIJ_Heap"
 PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_Heap(Mat A,Mat B,PetscReal fill,Mat *C)
 {
   PetscErrorCode     ierr;
@@ -597,8 +611,6 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_Heap(Mat A,Mat B,PetscReal fill,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatMultSymbolic_SeqAIJ_SeqAIJ_BTHeap"
 PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_BTHeap(Mat A,Mat B,PetscReal fill,Mat *C)
 {
   PetscErrorCode     ierr;
@@ -719,8 +731,6 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ_BTHeap(Mat A,Mat B,PetscReal fil
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatMultSymbolic_SeqAIJ_SeqAIJ"
 /* concatenate unique entries and then sort */
 PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ(Mat A,Mat B,PetscReal fill,Mat *C)
 {
@@ -810,8 +820,6 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqAIJ(Mat A,Mat B,PetscReal fill,Mat *
 }
 
 /* This routine is not used. Should be removed! */
-#undef __FUNCT__
-#define __FUNCT__ "MatMatTransposeMult_SeqAIJ_SeqAIJ"
 PetscErrorCode MatMatTransposeMult_SeqAIJ_SeqAIJ(Mat A,Mat B,MatReuse scall,PetscReal fill,Mat *C)
 {
   PetscErrorCode ierr;
@@ -828,8 +836,6 @@ PetscErrorCode MatMatTransposeMult_SeqAIJ_SeqAIJ(Mat A,Mat B,MatReuse scall,Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_SeqAIJ_MatMatMultTrans"
 PetscErrorCode MatDestroy_SeqAIJ_MatMatMultTrans(Mat A)
 {
   PetscErrorCode      ierr;
@@ -845,8 +851,6 @@ PetscErrorCode MatDestroy_SeqAIJ_MatMatMultTrans(Mat A)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatTransposeMultSymbolic_SeqAIJ_SeqAIJ"
 PetscErrorCode MatMatTransposeMultSymbolic_SeqAIJ_SeqAIJ(Mat A,Mat B,PetscReal fill,Mat *C)
 {
   PetscErrorCode      ierr;
@@ -926,8 +930,6 @@ PetscErrorCode MatMatTransposeMultSymbolic_SeqAIJ_SeqAIJ(Mat A,Mat B,PetscReal f
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatTransposeMultNumeric_SeqAIJ_SeqAIJ"
 PetscErrorCode MatMatTransposeMultNumeric_SeqAIJ_SeqAIJ(Mat A,Mat B,Mat C)
 {
   PetscErrorCode      ierr;
@@ -999,26 +1001,65 @@ PetscErrorCode MatMatTransposeMultNumeric_SeqAIJ_SeqAIJ(Mat A,Mat B,Mat C)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatTransposeMatMult_SeqAIJ_SeqAIJ"
-PetscErrorCode MatTransposeMatMult_SeqAIJ_SeqAIJ(Mat A,Mat B,MatReuse scall,PetscReal fill,Mat *C)
+PetscErrorCode MatDestroy_SeqAIJ_MatTransMatMult(Mat A)
 {
-  PetscErrorCode ierr;
+  PetscErrorCode      ierr;
+  Mat_SeqAIJ          *a = (Mat_SeqAIJ*)A->data;
+  Mat_MatTransMatMult *atb = a->atb;
 
   PetscFunctionBegin;
+  ierr = MatDestroy(&atb->At);CHKERRQ(ierr);
+  ierr = (atb->destroy)(A);CHKERRQ(ierr);
+  ierr = PetscFree(atb);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode MatTransposeMatMult_SeqAIJ_SeqAIJ(Mat A,Mat B,MatReuse scall,PetscReal fill,Mat *C)
+{
+  PetscErrorCode      ierr;
+  const char          *algTypes[2] = {"matmatmult","outerproduct"};
+  PetscInt            alg=0; /* set default algorithm */
+  Mat                 At;
+  Mat_MatTransMatMult *atb;
+  Mat_SeqAIJ          *c;
+
+  PetscFunctionBegin;
   if (scall == MAT_INITIAL_MATRIX) {
-    ierr = PetscLogEventBegin(MAT_TransposeMatMultSymbolic,A,B,0,0);CHKERRQ(ierr);
-    ierr = MatTransposeMatMultSymbolic_SeqAIJ_SeqAIJ(A,B,fill,C);CHKERRQ(ierr);
-    ierr = PetscLogEventEnd(MAT_TransposeMatMultSymbolic,A,B,0,0);CHKERRQ(ierr);
+    ierr = PetscObjectOptionsBegin((PetscObject)A);CHKERRQ(ierr);
+    PetscOptionsObject->alreadyprinted = PETSC_FALSE; /* a hack to ensure the option shows in '-help' */
+    ierr = PetscOptionsEList("-mattransposematmult_via","Algorithmic approach","MatTransposeMatMult",algTypes,2,algTypes[0],&alg,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsEnd();CHKERRQ(ierr);
+
+    switch (alg) {
+    case 1:
+      ierr = MatTransposeMatMultSymbolic_SeqAIJ_SeqAIJ(A,B,fill,C);CHKERRQ(ierr);
+      break;
+    default:
+      ierr = PetscNew(&atb);CHKERRQ(ierr);
+      ierr = MatTranspose_SeqAIJ(A,MAT_INITIAL_MATRIX,&At);CHKERRQ(ierr);
+      ierr = MatMatMult_SeqAIJ_SeqAIJ(At,B,MAT_INITIAL_MATRIX,fill,C);CHKERRQ(ierr);
+
+      c                  = (Mat_SeqAIJ*)(*C)->data;
+      c->atb             = atb;
+      atb->At            = At;
+      atb->destroy       = (*C)->ops->destroy;
+      (*C)->ops->destroy = MatDestroy_SeqAIJ_MatTransMatMult;
+
+      break;
+    }
   }
-  ierr = PetscLogEventBegin(MAT_TransposeMatMultNumeric,A,B,0,0);CHKERRQ(ierr);
-  ierr = MatTransposeMatMultNumeric_SeqAIJ_SeqAIJ(A,B,*C);CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(MAT_TransposeMatMultNumeric,A,B,0,0);CHKERRQ(ierr);
+  if (alg) {
+    ierr = (*(*C)->ops->mattransposemultnumeric)(A,B,*C);CHKERRQ(ierr);
+  } else if (!alg && scall == MAT_REUSE_MATRIX) {
+    c   = (Mat_SeqAIJ*)(*C)->data;
+    atb = c->atb;
+    At  = atb->At;
+    ierr = MatTranspose_SeqAIJ(A,MAT_REUSE_MATRIX,&At);CHKERRQ(ierr);
+    ierr = MatMatMult_SeqAIJ_SeqAIJ(At,B,MAT_REUSE_MATRIX,fill,C);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatTransposeMatMultSymbolic_SeqAIJ_SeqAIJ"
 PetscErrorCode MatTransposeMatMultSymbolic_SeqAIJ_SeqAIJ(Mat A,Mat B,PetscReal fill,Mat *C)
 {
   PetscErrorCode ierr;
@@ -1037,11 +1078,11 @@ PetscErrorCode MatTransposeMatMultSymbolic_SeqAIJ_SeqAIJ(Mat A,Mat B,PetscReal f
   /* clean up */
   ierr = MatDestroy(&At);CHKERRQ(ierr);
   ierr = MatRestoreSymbolicTranspose_SeqAIJ(A,&ati,&atj);CHKERRQ(ierr);
+
+  (*C)->ops->mattransposemultnumeric = MatTransposeMatMultNumeric_SeqAIJ_SeqAIJ;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatTransposeMatMultNumeric_SeqAIJ_SeqAIJ"
 PetscErrorCode MatTransposeMatMultNumeric_SeqAIJ_SeqAIJ(Mat A,Mat B,Mat C)
 {
   PetscErrorCode ierr;
@@ -1093,8 +1134,6 @@ PetscErrorCode MatTransposeMatMultNumeric_SeqAIJ_SeqAIJ(Mat A,Mat B,Mat C)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatMult_SeqAIJ_SeqDense"
 PETSC_INTERN PetscErrorCode MatMatMult_SeqAIJ_SeqDense(Mat A,Mat B,MatReuse scall,PetscReal fill,Mat *C)
 {
   PetscErrorCode ierr;
@@ -1111,8 +1150,6 @@ PETSC_INTERN PetscErrorCode MatMatMult_SeqAIJ_SeqDense(Mat A,Mat B,MatReuse scal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatMultSymbolic_SeqAIJ_SeqDense"
 PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqDense(Mat A,Mat B,PetscReal fill,Mat *C)
 {
   PetscErrorCode ierr;
@@ -1124,8 +1161,6 @@ PetscErrorCode MatMatMultSymbolic_SeqAIJ_SeqDense(Mat A,Mat B,PetscReal fill,Mat
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatMultNumeric_SeqAIJ_SeqDense"
 PetscErrorCode MatMatMultNumeric_SeqAIJ_SeqDense(Mat A,Mat B,Mat C)
 {
   Mat_SeqAIJ        *a=(Mat_SeqAIJ*)A->data;
@@ -1191,8 +1226,6 @@ PetscErrorCode MatMatMultNumeric_SeqAIJ_SeqDense(Mat A,Mat B,Mat C)
 /*
    Note very similar to MatMult_SeqAIJ(), should generate both codes from same base
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatMatMultNumericAdd_SeqAIJ_SeqDense"
 PetscErrorCode MatMatMultNumericAdd_SeqAIJ_SeqDense(Mat A,Mat B,Mat C)
 {
   Mat_SeqAIJ     *a=(Mat_SeqAIJ*)A->data;
@@ -1299,8 +1332,6 @@ PetscErrorCode MatMatMultNumericAdd_SeqAIJ_SeqDense(Mat A,Mat B,Mat C)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatTransColoringApplySpToDen_SeqAIJ"
 PetscErrorCode  MatTransColoringApplySpToDen_SeqAIJ(MatTransposeColoring coloring,Mat B,Mat Btdense)
 {
   PetscErrorCode ierr;
@@ -1331,8 +1362,6 @@ PetscErrorCode  MatTransColoringApplySpToDen_SeqAIJ(MatTransposeColoring colorin
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatTransColoringApplyDenToSp_SeqAIJ"
 PetscErrorCode MatTransColoringApplyDenToSp_SeqAIJ(MatTransposeColoring matcoloring,Mat Cden,Mat Csp)
 {
   PetscErrorCode ierr;
@@ -1396,8 +1425,6 @@ PetscErrorCode MatTransColoringApplyDenToSp_SeqAIJ(MatTransposeColoring matcolor
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatTransposeColoringCreate_SeqAIJ"
 PetscErrorCode MatTransposeColoringCreate_SeqAIJ(Mat mat,ISColoring iscoloring,MatTransposeColoring c)
 {
   PetscErrorCode ierr;
@@ -1507,3 +1534,120 @@ PetscErrorCode MatTransposeColoringCreate_SeqAIJ(Mat mat,ISColoring iscoloring,M
   ierr = PetscFree(idxhit);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
+/* Needed for MatMatMult_SeqAIJ_SeqAIJ_Combined() */
+/* Append value to an array if the value is not present yet. A bitarray */
+/* was used to determine if there is already an entry at this position. */
+void appendToArray(PetscInt val, PetscInt *array, PetscInt *cnzi)
+{
+  array[(*cnzi)++] = val;
+}
+
+/* This algorithm combines the symbolic and numeric phase of matrix-matrix multiplication. */
+PetscErrorCode MatMatMult_SeqAIJ_SeqAIJ_Combined(Mat A,Mat B,PetscReal fill,Mat *C)
+{
+  PetscErrorCode     ierr;
+  PetscLogDouble     flops=0.0;
+  Mat_SeqAIJ         *a  = (Mat_SeqAIJ*)A->data, *b = (Mat_SeqAIJ*)B->data, *c;
+  const PetscInt     *ai = a->i,*bi = b->i, *aj = a->j;
+  PetscInt           *ci,*cj,*cj_i;
+  PetscScalar        *ca, *ca_i;
+  PetscInt           c_maxmem = 0, a_maxrownnz = 0, a_rownnz, a_col;
+  PetscInt           am = A->rmap->N, bn = B->cmap->N, bm = B->rmap->N;
+  PetscInt           i, k, ndouble = 0;
+  PetscReal          afill;
+  PetscScalar        *c_row_val_dense;
+  PetscBool          *c_row_idx_flags;
+  PetscInt           *aj_i = a->j;
+  PetscScalar        *aa_i = a->a;
+
+  PetscFunctionBegin;
+  /* Step 1: Determine upper bounds on memory for C */
+  for (i=0; i<am; i++) { /* iterate over all rows of A */
+    const PetscInt anzi  = ai[i+1] - ai[i]; /* number of nonzeros in this row of A, this is the number of rows of B that we merge */
+    const PetscInt *acol = aj + ai[i]; /* column indices of nonzero entries in this row */
+    a_rownnz = 0;
+    for (k=0;k<anzi;++k) a_rownnz += bi[acol[k]+1] - bi[acol[k]];
+    a_maxrownnz = PetscMax(a_maxrownnz, a_rownnz);
+    c_maxmem += a_rownnz;
+  }
+  ierr = PetscMalloc1(am+1, &ci);               CHKERRQ(ierr);
+  ierr = PetscMalloc1(bn, &c_row_val_dense);    CHKERRQ(ierr);
+  ierr = PetscMalloc1(bn, &c_row_idx_flags);    CHKERRQ(ierr);
+  ierr = PetscMalloc1(c_maxmem,&cj);            CHKERRQ(ierr);
+  ierr = PetscMalloc1(c_maxmem,&ca);            CHKERRQ(ierr);
+  ca_i = ca;
+  cj_i = cj;
+  ci[0] = 0;
+  ierr = PetscMemzero(c_row_val_dense, bn * sizeof(PetscScalar));CHKERRQ(ierr);
+  ierr = PetscMemzero(c_row_idx_flags, bn * sizeof(PetscBool));CHKERRQ(ierr);
+  for (i=0; i<am; i++) {
+    /* Step 2: Initialize the dense row vector for C  */
+    const PetscInt anzi     = ai[i+1] - ai[i]; /* number of nonzeros in this row of A, this is the number of rows of B that we merge */
+    PetscInt cnzi           = 0;
+    PetscInt *bj_i;
+    PetscScalar *ba_i;
+
+    /* Step 3: Do the numerical calculations */
+    for (a_col=0; a_col<anzi; a_col++) {          /* iterate over all non zero values in a row of A */
+      PetscInt a_col_index = aj_i[a_col];
+      const PetscInt bnzi = bi[a_col_index+1] - bi[a_col_index];
+      flops += 2*bnzi;
+      bj_i = b->j + bi[a_col_index];   /* points to the current row in bj */
+      ba_i = b->a + bi[a_col_index];   /* points to the current row in ba */
+      for (k=0; k<bnzi; ++k) { /* iterate over all non zeros of this row in B */
+        if (c_row_idx_flags[ bj_i[k] ] == PETSC_FALSE) {
+          appendToArray(bj_i[k], cj_i, &cnzi);
+          c_row_idx_flags[ bj_i[k] ] = PETSC_TRUE;
+        }
+        c_row_val_dense[ bj_i[k] ] += aa_i[a_col] * ba_i[k];
+      }
+    }
+
+    /* Sort array */
+    ierr = PetscSortInt(cnzi, cj_i);CHKERRQ(ierr);
+    /* Step 4 */
+    for (k=0; k < cnzi; k++) {
+      ca_i[k] = c_row_val_dense[cj_i[k]];
+      c_row_val_dense[cj_i[k]] = 0.;
+      c_row_idx_flags[cj_i[k]] = PETSC_FALSE;
+    }
+    /* terminate current row */
+    aa_i += anzi;
+    aj_i += anzi;
+    ca_i += cnzi;
+    cj_i += cnzi;
+    ci[i+1] = ci[i] + cnzi;
+    flops += cnzi;
+  }
+
+  /* Step 5 */
+  /* Create the new matrix */
+  ierr = MatCreateSeqAIJWithArrays(PetscObjectComm((PetscObject)A),am,bn,ci,cj,NULL,C);CHKERRQ(ierr);
+  ierr = MatSetBlockSizesFromMats(*C,A,B);CHKERRQ(ierr);
+
+  /* MatCreateSeqAIJWithArrays flags matrix so PETSc doesn't free the user's arrays. */
+  /* These are PETSc arrays, so change flags so arrays can be deleted by PETSc */
+  c          = (Mat_SeqAIJ*)((*C)->data);
+  c->a       = ca;
+  c->free_a  = PETSC_TRUE;
+  c->free_ij = PETSC_TRUE;
+  c->nonew   = 0;
+
+  /* set MatInfo */
+  afill = (PetscReal)ci[am]/(ai[am]+bi[bm]) + 1.e-5;
+  if (afill < 1.0) afill = 1.0;
+  c->maxnz                     = ci[am];
+  c->nz                        = ci[am];
+  (*C)->info.mallocs           = ndouble;
+  (*C)->info.fill_ratio_given  = fill;
+  (*C)->info.fill_ratio_needed = afill;
+
+  ierr = PetscFree(c_row_val_dense);CHKERRQ(ierr);
+  ierr = PetscFree(c_row_idx_flags);CHKERRQ(ierr);
+
+  ierr = MatAssemblyBegin(*C,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+  ierr = MatAssemblyEnd(*C,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+  ierr = PetscLogFlops(flops);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
