@@ -31,7 +31,7 @@ EXTERN_C_BEGIN
 EXTERN_C_END
 
 /*
-     This is the data we are "ADDING" to the SeqAIJ matrix type to get the SuperLU matrix type
+     This is the data that defines the SuperLU factored matrix type
 */
 typedef struct {
   SuperMatrix       A,L,U,B,X;
@@ -49,37 +49,23 @@ typedef struct {
   SuperLUStat_t     stat;
   Mat               A_dup;
   PetscScalar       *rhs_dup;
+  GlobalLU_t        Glu;
+  PetscBool         needconversion;
 
   /* Flag to clean up (non-global) SuperLU objects during Destroy */
   PetscBool CleanUpSuperLU;
 } Mat_SuperLU;
 
-extern PetscErrorCode MatFactorInfo_SuperLU(Mat,PetscViewer);
-extern PetscErrorCode MatLUFactorNumeric_SuperLU(Mat,Mat,const MatFactorInfo*);
-extern PetscErrorCode MatDestroy_SuperLU(Mat);
-extern PetscErrorCode MatView_SuperLU(Mat,PetscViewer);
-extern PetscErrorCode MatAssemblyEnd_SuperLU(Mat,MatAssemblyType);
-extern PetscErrorCode MatSolve_SuperLU(Mat,Vec,Vec);
-extern PetscErrorCode MatMatSolve_SuperLU(Mat,Mat,Mat);
-extern PetscErrorCode MatSolveTranspose_SuperLU(Mat,Vec,Vec);
-extern PetscErrorCode MatLUFactorSymbolic_SuperLU(Mat,Mat,IS,IS,const MatFactorInfo*);
-extern PetscErrorCode MatDuplicate_SuperLU(Mat, MatDuplicateOption, Mat*);
-
 /*
     Utility function
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatFactorInfo_SuperLU"
-PetscErrorCode MatFactorInfo_SuperLU(Mat A,PetscViewer viewer)
+static PetscErrorCode MatView_Info_SuperLU(Mat A,PetscViewer viewer)
 {
-  Mat_SuperLU       *lu= (Mat_SuperLU*)A->spptr;
+  Mat_SuperLU       *lu= (Mat_SuperLU*)A->data;
   PetscErrorCode    ierr;
   superlu_options_t options;
 
   PetscFunctionBegin;
-  /* check if matrix is superlu_dist type */
-  if (A->ops->solve != MatSolve_SuperLU) PetscFunctionReturn(0);
-
   options = lu->options;
 
   ierr = PetscViewerASCIIPrintf(viewer,"SuperLU run parameters:\n");CHKERRQ(ierr);
@@ -105,212 +91,11 @@ PetscErrorCode MatFactorInfo_SuperLU(Mat A,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-/*
-    These are the methods provided to REPLACE the corresponding methods of the
-   SeqAIJ matrix class. Other methods of SeqAIJ are not replaced
-*/
-#undef __FUNCT__
-#define __FUNCT__ "MatLUFactorNumeric_SuperLU"
-PetscErrorCode MatLUFactorNumeric_SuperLU(Mat F,Mat A,const MatFactorInfo *info)
-{
-  Mat_SuperLU    *lu = (Mat_SuperLU*)F->spptr;
-  Mat_SeqAIJ     *aa;
-  PetscErrorCode ierr;
-  PetscInt       sinfo;
-  PetscReal      ferr, berr;
-  NCformat       *Ustore;
-  SCformat       *Lstore;
-
-  PetscFunctionBegin;
-  if (lu->flg == SAME_NONZERO_PATTERN) { /* successing numerical factorization */
-    lu->options.Fact = SamePattern;
-    /* Ref: ~SuperLU_3.0/EXAMPLE/dlinsolx2.c */
-    Destroy_SuperMatrix_Store(&lu->A);
-    if (lu->options.Equil) {
-      ierr = MatCopy_SeqAIJ(A,lu->A_dup,SAME_NONZERO_PATTERN);CHKERRQ(ierr);
-    }
-    if (lu->lwork >= 0) {
-      PetscStackCall("SuperLU:Destroy_SuperNode_Matrix",Destroy_SuperNode_Matrix(&lu->L));
-      PetscStackCall("SuperLU:Destroy_CompCol_Matrix",Destroy_CompCol_Matrix(&lu->U));
-      lu->options.Fact = SamePattern;
-    }
-  }
-
-  /* Create the SuperMatrix for lu->A=A^T:
-       Since SuperLU likes column-oriented matrices,we pass it the transpose,
-       and then solve A^T X = B in MatSolve(). */
-  if (lu->options.Equil) {
-    aa = (Mat_SeqAIJ*)(lu->A_dup)->data;
-  } else {
-    aa = (Mat_SeqAIJ*)(A)->data;
-  }
-#if defined(PETSC_USE_COMPLEX)
-#if defined(PETSC_USE_REAL_SINGLE)
-  PetscStackCall("SuperLU:cCreate_CompCol_Matrix",cCreate_CompCol_Matrix(&lu->A,A->cmap->n,A->rmap->n,aa->nz,(singlecomplex*)aa->a,aa->j,aa->i,SLU_NC,SLU_C,SLU_GE));
-#else
-  PetscStackCall("SuperLU:zCreate_CompCol_Matrix",zCreate_CompCol_Matrix(&lu->A,A->cmap->n,A->rmap->n,aa->nz,(doublecomplex*)aa->a,aa->j,aa->i,SLU_NC,SLU_Z,SLU_GE));
-#endif
-#else
-#if defined(PETSC_USE_REAL_SINGLE)
-  PetscStackCall("SuperLU:sCreate_CompCol_Matrix",sCreate_CompCol_Matrix(&lu->A,A->cmap->n,A->rmap->n,aa->nz,aa->a,aa->j,aa->i,SLU_NC,SLU_S,SLU_GE));
-#else
-  PetscStackCall("SuperLU:dCreate_CompCol_Matrix",dCreate_CompCol_Matrix(&lu->A,A->cmap->n,A->rmap->n,aa->nz,aa->a,aa->j,aa->i,SLU_NC,SLU_D,SLU_GE));
-#endif
-#endif
-
-  /* Numerical factorization */
-  lu->B.ncol = 0;  /* Indicate not to solve the system */
-  if (F->factortype == MAT_FACTOR_LU) {
-#if defined(PETSC_USE_COMPLEX)
-#if defined(PETSC_USE_REAL_SINGLE)
-    PetscStackCall("SuperLU:cgssvx",cgssvx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
-                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond, &ferr, &berr,
-                                     &lu->mem_usage, &lu->stat, &sinfo));
-#else
-    PetscStackCall("SuperLU:zgssvx",zgssvx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
-                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond, &ferr, &berr,
-                                     &lu->mem_usage, &lu->stat, &sinfo));
-#endif
-#else
-#if defined(PETSC_USE_REAL_SINGLE)
-    PetscStackCall("SuperLU:sgssvx",sgssvx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
-                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond, &ferr, &berr,
-                                     &lu->mem_usage, &lu->stat, &sinfo));
-#else
-    PetscStackCall("SuperLU:dgssvx",dgssvx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
-                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond, &ferr, &berr,
-                                     &lu->mem_usage, &lu->stat, &sinfo));
-#endif
-#endif
-  } else if (F->factortype == MAT_FACTOR_ILU) {
-    /* Compute the incomplete factorization, condition number and pivot growth */
-#if defined(PETSC_USE_COMPLEX)
-#if defined(PETSC_USE_REAL_SINGLE)
-    PetscStackCall("SuperLU:cgsisx",cgsisx(&lu->options, &lu->A, lu->perm_c, lu->perm_r,lu->etree, lu->equed, lu->R, lu->C,
-                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond,
-                                     &lu->mem_usage, &lu->stat, &sinfo));
-#else
-    PetscStackCall("SuperLU:zgsisx",zgsisx(&lu->options, &lu->A, lu->perm_c, lu->perm_r,lu->etree, lu->equed, lu->R, lu->C,
-                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond,
-                                     &lu->mem_usage, &lu->stat, &sinfo));
-#endif
-#else
-#if defined(PETSC_USE_REAL_SINGLE)
-    PetscStackCall("SuperLU:sgsisx",sgsisx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
-                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond,
-                                     &lu->mem_usage, &lu->stat, &sinfo));
-#else
-    PetscStackCall("SuperLU:dgsisx",dgsisx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
-                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond,
-                                     &lu->mem_usage, &lu->stat, &sinfo));
-#endif
-#endif
-  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Factor type not supported");
-  if (!sinfo || sinfo == lu->A.ncol+1) {
-    if (lu->options.PivotGrowth) {
-      ierr = PetscPrintf(PETSC_COMM_SELF,"  Recip. pivot growth = %e\n", lu->rpg);
-    }
-    if (lu->options.ConditionNumber) {
-      ierr = PetscPrintf(PETSC_COMM_SELF,"  Recip. condition number = %e\n", lu->rcond);
-    }
-  } else if (sinfo > 0) {
-    if (lu->lwork == -1) {
-      ierr = PetscPrintf(PETSC_COMM_SELF,"  ** Estimated memory: %D bytes\n", sinfo - lu->A.ncol);
-    } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_MAT_LU_ZRPVT,"Zero pivot in row %D",sinfo);
-  } else SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_LIB, "info = %D, the %D-th argument in gssvx() had an illegal value", sinfo,-sinfo);
-
-  if (lu->options.PrintStat) {
-    ierr = PetscPrintf(PETSC_COMM_SELF,"MatLUFactorNumeric_SuperLU():\n");
-    PetscStackCall("SuperLU:StatPrint",StatPrint(&lu->stat));
-    Lstore = (SCformat*) lu->L.Store;
-    Ustore = (NCformat*) lu->U.Store;
-    ierr   = PetscPrintf(PETSC_COMM_SELF,"  No of nonzeros in factor L = %D\n", Lstore->nnz);
-    ierr   = PetscPrintf(PETSC_COMM_SELF,"  No of nonzeros in factor U = %D\n", Ustore->nnz);
-    ierr   = PetscPrintf(PETSC_COMM_SELF,"  No of nonzeros in L+U = %D\n", Lstore->nnz + Ustore->nnz - lu->A.ncol);
-    ierr   = PetscPrintf(PETSC_COMM_SELF,"  L\\U MB %.3f\ttotal MB needed %.3f\n",
-                         lu->mem_usage.for_lu/1e6, lu->mem_usage.total_needed/1e6);
-  }
-
-  lu->flg                = SAME_NONZERO_PATTERN;
-  F->ops->solve          = MatSolve_SuperLU;
-  F->ops->solvetranspose = MatSolveTranspose_SuperLU;
-  F->ops->matsolve       = NULL;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "MatGetDiagonal_SuperLU"
-PetscErrorCode MatGetDiagonal_SuperLU(Mat A,Vec v)
-{
-  PetscFunctionBegin;
-  SETERRQ(PetscObjectComm((PetscObject)A),PETSC_ERR_SUP,"Mat type: SuperLU factor");
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_SuperLU"
-PetscErrorCode MatDestroy_SuperLU(Mat A)
-{
-  PetscErrorCode ierr;
-  Mat_SuperLU    *lu=(Mat_SuperLU*)A->spptr;
-
-  PetscFunctionBegin;
-  if (lu && lu->CleanUpSuperLU) { /* Free the SuperLU datastructures */
-    PetscStackCall("SuperLU:Destroy_SuperMatrix_Store",Destroy_SuperMatrix_Store(&lu->A));
-    PetscStackCall("SuperLU:Destroy_SuperMatrix_Store",Destroy_SuperMatrix_Store(&lu->B));
-    PetscStackCall("SuperLU:Destroy_SuperMatrix_Store",Destroy_SuperMatrix_Store(&lu->X));
-    PetscStackCall("SuperLU:StatFree",StatFree(&lu->stat));
-    if (lu->lwork >= 0) {
-      PetscStackCall("SuperLU:Destroy_SuperNode_Matrix",Destroy_SuperNode_Matrix(&lu->L));
-      PetscStackCall("SuperLU:Destroy_CompCol_Matrix",Destroy_CompCol_Matrix(&lu->U));
-    }
-  }
-  if (lu) {
-    ierr = PetscFree(lu->etree);CHKERRQ(ierr);
-    ierr = PetscFree(lu->perm_r);CHKERRQ(ierr);
-    ierr = PetscFree(lu->perm_c);CHKERRQ(ierr);
-    ierr = PetscFree(lu->R);CHKERRQ(ierr);
-    ierr = PetscFree(lu->C);CHKERRQ(ierr);
-    ierr = PetscFree(lu->rhs_dup);CHKERRQ(ierr);
-    ierr = MatDestroy(&lu->A_dup);CHKERRQ(ierr);
-  }
-  ierr = PetscFree(A->spptr);CHKERRQ(ierr);
-
-  /* clear composed functions */
-  ierr = PetscObjectComposeFunction((PetscObject)A,"MatFactorGetSolverPackage_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)A,"MatSuperluSetILUDropTol_C",NULL);CHKERRQ(ierr);
-
-  ierr = MatDestroy_SeqAIJ(A);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "MatView_SuperLU"
-PetscErrorCode MatView_SuperLU(Mat A,PetscViewer viewer)
-{
-  PetscErrorCode    ierr;
-  PetscBool         iascii;
-  PetscViewerFormat format;
-
-  PetscFunctionBegin;
-  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
-  if (iascii) {
-    ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
-    if (format == PETSC_VIEWER_ASCII_INFO) {
-      ierr = MatFactorInfo_SuperLU(A,viewer);CHKERRQ(ierr);
-    }
-  }
-  PetscFunctionReturn(0);
-}
-
-
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SuperLU_Private"
 PetscErrorCode MatSolve_SuperLU_Private(Mat A,Vec b,Vec x)
 {
-  Mat_SuperLU       *lu = (Mat_SuperLU*)A->spptr;
+  Mat_SuperLU       *lu = (Mat_SuperLU*)A->data;
   const PetscScalar *barray;
-  PetscScalar       *xarray;  
+  PetscScalar       *xarray;
   PetscErrorCode    ierr;
   PetscInt          info,i,n;
   PetscReal         ferr,berr;
@@ -356,21 +141,21 @@ PetscErrorCode MatSolve_SuperLU_Private(Mat A,Vec b,Vec x)
 #if defined(PETSC_USE_REAL_SINGLE)
     PetscStackCall("SuperLU:cgssvx",cgssvx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
                                      &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond, &ferr, &berr,
-                                     &lu->mem_usage, &lu->stat, &info));
+                                     &lu->Glu, &lu->mem_usage, &lu->stat, &info));
 #else
     PetscStackCall("SuperLU:zgssvx",zgssvx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
                                      &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond, &ferr, &berr,
-                                     &lu->mem_usage, &lu->stat, &info));
+                                     &lu->Glu, &lu->mem_usage, &lu->stat, &info));
 #endif
 #else
 #if defined(PETSC_USE_REAL_SINGLE)
     PetscStackCall("SuperLU:sgssvx",sgssvx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
                                      &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond, &ferr, &berr,
-                                     &lu->mem_usage, &lu->stat, &info));
+                                     &lu->Glu, &lu->mem_usage, &lu->stat, &info));
 #else
     PetscStackCall("SuperLU:dgssvx",dgssvx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
                                      &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond, &ferr, &berr,
-                                     &lu->mem_usage, &lu->stat, &info));
+                                     &lu->Glu,&lu->mem_usage, &lu->stat, &info));
 #endif
 #endif
   } else if (A->factortype == MAT_FACTOR_ILU) {
@@ -378,21 +163,21 @@ PetscErrorCode MatSolve_SuperLU_Private(Mat A,Vec b,Vec x)
 #if defined(PETSC_USE_REAL_SINGLE)
     PetscStackCall("SuperLU:cgsisx",cgsisx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
                                      &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond,
-                                     &lu->mem_usage, &lu->stat, &info));
+                                     &lu->Glu, &lu->mem_usage, &lu->stat, &info));
 #else
     PetscStackCall("SuperLU:zgsisx",zgsisx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
                                      &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond,
-                                     &lu->mem_usage, &lu->stat, &info));
+                                     &lu->Glu, &lu->mem_usage, &lu->stat, &info));
 #endif
 #else
 #if defined(PETSC_USE_REAL_SINGLE)
     PetscStackCall("SuperLU:sgsisx",sgsisx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
                                      &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond,
-                                     &lu->mem_usage, &lu->stat, &info));
+                                     &lu->Glu, &lu->mem_usage, &lu->stat, &info));
 #else
     PetscStackCall("SuperLU:dgsisx",dgsisx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
                                      &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond,
-                                     &lu->mem_usage, &lu->stat, &info));
+                                     &lu->Glu, &lu->mem_usage, &lu->stat, &info));
 #endif
 #endif
   } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Factor type not supported");
@@ -424,39 +209,239 @@ PetscErrorCode MatSolve_SuperLU_Private(Mat A,Vec b,Vec x)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SuperLU"
 PetscErrorCode MatSolve_SuperLU(Mat A,Vec b,Vec x)
 {
-  Mat_SuperLU    *lu = (Mat_SuperLU*)A->spptr;
+  Mat_SuperLU    *lu = (Mat_SuperLU*)A->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  lu->options.Trans = TRANS;
+  if (A->factorerrortype) {
+    ierr = PetscInfo(A,"MatSolve is called with singular matrix factor, skip\n");CHKERRQ(ierr);
+    ierr = VecSetInf(x);CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  }
 
+  lu->options.Trans = TRANS;
   ierr = MatSolve_SuperLU_Private(A,b,x);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolveTranspose_SuperLU"
 PetscErrorCode MatSolveTranspose_SuperLU(Mat A,Vec b,Vec x)
 {
-  Mat_SuperLU    *lu = (Mat_SuperLU*)A->spptr;
+  Mat_SuperLU    *lu = (Mat_SuperLU*)A->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  lu->options.Trans = NOTRANS;
+  if (A->factorerrortype) {
+    ierr = PetscInfo(A,"MatSolve is called with singular matrix factor, skip\n");CHKERRQ(ierr);
+    ierr = VecSetInf(x);CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  }
 
+  lu->options.Trans = NOTRANS;
   ierr = MatSolve_SuperLU_Private(A,b,x);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMatSolve_SuperLU"
+static PetscErrorCode MatLUFactorNumeric_SuperLU(Mat F,Mat A,const MatFactorInfo *info)
+{
+  Mat_SuperLU    *lu = (Mat_SuperLU*)F->data;
+  Mat_SeqAIJ     *aa;
+  PetscErrorCode ierr;
+  PetscInt       sinfo;
+  PetscReal      ferr, berr;
+  NCformat       *Ustore;
+  SCformat       *Lstore;
+
+  PetscFunctionBegin;
+  if (lu->flg == SAME_NONZERO_PATTERN) { /* successing numerical factorization */
+    lu->options.Fact = SamePattern;
+    /* Ref: ~SuperLU_3.0/EXAMPLE/dlinsolx2.c */
+    Destroy_SuperMatrix_Store(&lu->A);
+    if (lu->A_dup) {
+      ierr = MatCopy_SeqAIJ(A,lu->A_dup,SAME_NONZERO_PATTERN);CHKERRQ(ierr);
+    }
+
+    if (lu->lwork >= 0) {
+      PetscStackCall("SuperLU:Destroy_SuperNode_Matrix",Destroy_SuperNode_Matrix(&lu->L));
+      PetscStackCall("SuperLU:Destroy_CompCol_Matrix",Destroy_CompCol_Matrix(&lu->U));
+      lu->options.Fact = SamePattern;
+    }
+  }
+
+  /* Create the SuperMatrix for lu->A=A^T:
+       Since SuperLU likes column-oriented matrices,we pass it the transpose,
+       and then solve A^T X = B in MatSolve(). */
+  if (lu->A_dup) {
+    aa = (Mat_SeqAIJ*)(lu->A_dup)->data;
+  } else {
+    aa = (Mat_SeqAIJ*)(A)->data;
+  }
+#if defined(PETSC_USE_COMPLEX)
+#if defined(PETSC_USE_REAL_SINGLE)
+  PetscStackCall("SuperLU:cCreate_CompCol_Matrix",cCreate_CompCol_Matrix(&lu->A,A->cmap->n,A->rmap->n,aa->nz,(singlecomplex*)aa->a,aa->j,aa->i,SLU_NC,SLU_C,SLU_GE));
+#else
+  PetscStackCall("SuperLU:zCreate_CompCol_Matrix",zCreate_CompCol_Matrix(&lu->A,A->cmap->n,A->rmap->n,aa->nz,(doublecomplex*)aa->a,aa->j,aa->i,SLU_NC,SLU_Z,SLU_GE));
+#endif
+#else
+#if defined(PETSC_USE_REAL_SINGLE)
+  PetscStackCall("SuperLU:sCreate_CompCol_Matrix",sCreate_CompCol_Matrix(&lu->A,A->cmap->n,A->rmap->n,aa->nz,aa->a,aa->j,aa->i,SLU_NC,SLU_S,SLU_GE));
+#else
+  PetscStackCall("SuperLU:dCreate_CompCol_Matrix",dCreate_CompCol_Matrix(&lu->A,A->cmap->n,A->rmap->n,aa->nz,aa->a,aa->j,aa->i,SLU_NC,SLU_D,SLU_GE));
+#endif
+#endif
+
+  /* Numerical factorization */
+  lu->B.ncol = 0;  /* Indicate not to solve the system */
+  if (F->factortype == MAT_FACTOR_LU) {
+#if defined(PETSC_USE_COMPLEX)
+#if defined(PETSC_USE_REAL_SINGLE)
+    PetscStackCall("SuperLU:cgssvx",cgssvx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
+                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond, &ferr, &berr,
+                                     &lu->Glu, &lu->mem_usage, &lu->stat, &sinfo));
+#else
+    PetscStackCall("SuperLU:zgssvx",zgssvx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
+                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond, &ferr, &berr,
+                                     &lu->Glu, &lu->mem_usage, &lu->stat, &sinfo));
+#endif
+#else
+#if defined(PETSC_USE_REAL_SINGLE)
+    PetscStackCall("SuperLU:sgssvx",sgssvx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
+                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond, &ferr, &berr,
+                                     &lu->Glu, &lu->mem_usage, &lu->stat, &sinfo));
+#else
+    PetscStackCall("SuperLU:dgssvx",dgssvx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
+                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond, &ferr, &berr,
+                                     &lu->Glu,&lu->mem_usage, &lu->stat, &sinfo));
+#endif
+#endif
+  } else if (F->factortype == MAT_FACTOR_ILU) {
+    /* Compute the incomplete factorization, condition number and pivot growth */
+#if defined(PETSC_USE_COMPLEX)
+#if defined(PETSC_USE_REAL_SINGLE)
+    PetscStackCall("SuperLU:cgsisx",cgsisx(&lu->options, &lu->A, lu->perm_c, lu->perm_r,lu->etree, lu->equed, lu->R, lu->C,
+                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond,
+                                     &lu->Glu, &lu->mem_usage, &lu->stat, &sinfo));
+#else
+    PetscStackCall("SuperLU:zgsisx",zgsisx(&lu->options, &lu->A, lu->perm_c, lu->perm_r,lu->etree, lu->equed, lu->R, lu->C,
+                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond,
+                                     &lu->Glu, &lu->mem_usage, &lu->stat, &sinfo));
+#endif
+#else
+#if defined(PETSC_USE_REAL_SINGLE)
+    PetscStackCall("SuperLU:sgsisx",sgsisx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
+                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond,
+                                     &lu->Glu, &lu->mem_usage, &lu->stat, &sinfo));
+#else
+    PetscStackCall("SuperLU:dgsisx",dgsisx(&lu->options, &lu->A, lu->perm_c, lu->perm_r, lu->etree, lu->equed, lu->R, lu->C,
+                                     &lu->L, &lu->U, lu->work, lu->lwork, &lu->B, &lu->X, &lu->rpg, &lu->rcond,
+                                     &lu->Glu, &lu->mem_usage, &lu->stat, &sinfo));
+#endif
+#endif
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Factor type not supported");
+  if (!sinfo || sinfo == lu->A.ncol+1) {
+    if (lu->options.PivotGrowth) {
+      ierr = PetscPrintf(PETSC_COMM_SELF,"  Recip. pivot growth = %e\n", lu->rpg);CHKERRQ(ierr);
+    }
+    if (lu->options.ConditionNumber) {
+      ierr = PetscPrintf(PETSC_COMM_SELF,"  Recip. condition number = %e\n", lu->rcond);CHKERRQ(ierr);
+    }
+  } else if (sinfo > 0) {
+    if (A->erroriffailure) {
+      SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_MAT_LU_ZRPVT,"Zero pivot in row %D",sinfo);
+    } else {
+      if (sinfo <= lu->A.ncol) {
+        if (lu->options.ILU_FillTol == 0.0) {
+          F->factorerrortype = MAT_FACTOR_NUMERIC_ZEROPIVOT;
+        }
+        ierr = PetscInfo2(F,"Number of zero pivots %D, ILU_FillTol %g\n",sinfo,lu->options.ILU_FillTol);CHKERRQ(ierr);
+      } else if (sinfo == lu->A.ncol + 1) {
+        /*
+         U is nonsingular, but RCOND is less than machine
+ 		      precision, meaning that the matrix is singular to
+ 		      working precision. Nevertheless, the solution and
+ 		      error bounds are computed because there are a number
+ 		      of situations where the computed solution can be more
+ 		      accurate than the value of RCOND would suggest.
+         */
+        ierr = PetscInfo1(F,"Matrix factor U is nonsingular, but is singular to working precision. The solution is computed. info %D",sinfo);CHKERRQ(ierr);
+      } else { /* sinfo > lu->A.ncol + 1 */
+        F->factorerrortype = MAT_FACTOR_OUTMEMORY;
+        ierr = PetscInfo1(F,"Number of bytes allocated when memory allocation fails %D\n",sinfo);CHKERRQ(ierr);
+      }
+    }
+  } else SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_LIB, "info = %D, the %D-th argument in gssvx() had an illegal value", sinfo,-sinfo);
+
+  if (lu->options.PrintStat) {
+    ierr = PetscPrintf(PETSC_COMM_SELF,"MatLUFactorNumeric_SuperLU():\n");CHKERRQ(ierr);
+    PetscStackCall("SuperLU:StatPrint",StatPrint(&lu->stat));
+    Lstore = (SCformat*) lu->L.Store;
+    Ustore = (NCformat*) lu->U.Store;
+    ierr   = PetscPrintf(PETSC_COMM_SELF,"  No of nonzeros in factor L = %D\n", Lstore->nnz);
+    ierr   = PetscPrintf(PETSC_COMM_SELF,"  No of nonzeros in factor U = %D\n", Ustore->nnz);
+    ierr   = PetscPrintf(PETSC_COMM_SELF,"  No of nonzeros in L+U = %D\n", Lstore->nnz + Ustore->nnz - lu->A.ncol);
+    ierr   = PetscPrintf(PETSC_COMM_SELF,"  L\\U MB %.3f\ttotal MB needed %.3f\n",
+                         lu->mem_usage.for_lu/1e6, lu->mem_usage.total_needed/1e6);
+  }
+
+  lu->flg                = SAME_NONZERO_PATTERN;
+  F->ops->solve          = MatSolve_SuperLU;
+  F->ops->solvetranspose = MatSolveTranspose_SuperLU;
+  F->ops->matsolve       = NULL;
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode MatDestroy_SuperLU(Mat A)
+{
+  PetscErrorCode ierr;
+  Mat_SuperLU    *lu=(Mat_SuperLU*)A->data;
+
+  PetscFunctionBegin;
+  if (lu->CleanUpSuperLU) { /* Free the SuperLU datastructures */
+    PetscStackCall("SuperLU:Destroy_SuperMatrix_Store",Destroy_SuperMatrix_Store(&lu->A));
+    PetscStackCall("SuperLU:Destroy_SuperMatrix_Store",Destroy_SuperMatrix_Store(&lu->B));
+    PetscStackCall("SuperLU:Destroy_SuperMatrix_Store",Destroy_SuperMatrix_Store(&lu->X));
+    PetscStackCall("SuperLU:StatFree",StatFree(&lu->stat));
+    if (lu->lwork >= 0) {
+      PetscStackCall("SuperLU:Destroy_SuperNode_Matrix",Destroy_SuperNode_Matrix(&lu->L));
+      PetscStackCall("SuperLU:Destroy_CompCol_Matrix",Destroy_CompCol_Matrix(&lu->U));
+    }
+  }
+  ierr = PetscFree(lu->etree);CHKERRQ(ierr);
+  ierr = PetscFree(lu->perm_r);CHKERRQ(ierr);
+  ierr = PetscFree(lu->perm_c);CHKERRQ(ierr);
+  ierr = PetscFree(lu->R);CHKERRQ(ierr);
+  ierr = PetscFree(lu->C);CHKERRQ(ierr);
+  ierr = PetscFree(lu->rhs_dup);CHKERRQ(ierr);
+  ierr = MatDestroy(&lu->A_dup);CHKERRQ(ierr);
+  ierr = PetscFree(A->data);CHKERRQ(ierr);
+
+  /* clear composed functions */
+  ierr = PetscObjectComposeFunction((PetscObject)A,"MatFactorGetSolverType_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)A,"MatSuperluSetILUDropTol_C",NULL);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode MatView_SuperLU(Mat A,PetscViewer viewer)
+{
+  PetscErrorCode    ierr;
+  PetscBool         iascii;
+  PetscViewerFormat format;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
+  if (iascii) {
+    ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
+    if (format == PETSC_VIEWER_ASCII_INFO) {
+      ierr = MatView_Info_SuperLU(A,viewer);CHKERRQ(ierr);
+    }
+  }
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode MatMatSolve_SuperLU(Mat A,Mat B,Mat X)
 {
-  Mat_SuperLU    *lu = (Mat_SuperLU*)A->spptr;
+  Mat_SuperLU    *lu = (Mat_SuperLU*)A->data;
   PetscBool      flg;
   PetscErrorCode ierr;
 
@@ -473,32 +458,36 @@ PetscErrorCode MatMatSolve_SuperLU(Mat A,Mat B,Mat X)
 /*
    Note the r permutation is ignored
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatLUFactorSymbolic_SuperLU"
-PetscErrorCode MatLUFactorSymbolic_SuperLU(Mat F,Mat A,IS r,IS c,const MatFactorInfo *info)
+static PetscErrorCode MatLUFactorSymbolic_SuperLU(Mat F,Mat A,IS r,IS c,const MatFactorInfo *info)
 {
-  Mat_SuperLU *lu = (Mat_SuperLU*)(F->spptr);
+  Mat_SuperLU    *lu = (Mat_SuperLU*)(F->data);
+  PetscErrorCode ierr;
 
   PetscFunctionBegin;
   lu->flg                 = DIFFERENT_NONZERO_PATTERN;
   lu->CleanUpSuperLU      = PETSC_TRUE;
   F->ops->lufactornumeric = MatLUFactorNumeric_SuperLU;
+
+  /* if we are here, the nonzero pattern has changed unless the user explicitly called MatLUFactorSymbolic */
+  ierr = MatDestroy(&lu->A_dup);CHKERRQ(ierr);
+  if (lu->needconversion) {
+    ierr = MatConvert(A,MATSEQAIJ,MAT_INITIAL_MATRIX,&lu->A_dup);CHKERRQ(ierr);
+  }
+  if (lu->options.Equil == YES && !lu->A_dup) { /* superlu overwrites input matrix and rhs when Equil is used, thus create A_dup to keep user's A unchanged */
+    ierr = MatDuplicate_SeqAIJ(A,MAT_COPY_VALUES,&lu->A_dup);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSuperluSetILUDropTol_SuperLU"
 static PetscErrorCode MatSuperluSetILUDropTol_SuperLU(Mat F,PetscReal dtol)
 {
-  Mat_SuperLU *lu= (Mat_SuperLU*)F->spptr;
+  Mat_SuperLU *lu= (Mat_SuperLU*)F->data;
 
   PetscFunctionBegin;
   lu->options.ILU_DropTol = dtol;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSuperluSetILUDropTol"
 /*@
   MatSuperluSetILUDropTol - Set SuperLU ILU drop tolerance
    Logically Collective on Mat
@@ -512,7 +501,8 @@ static PetscErrorCode MatSuperluSetILUDropTol_SuperLU(Mat F,PetscReal dtol)
 
    Level: beginner
 
-   References: SuperLU Users' Guide
+   References:
+.      SuperLU Users' Guide
 
 .seealso: MatGetFactor()
 @*/
@@ -527,23 +517,20 @@ PetscErrorCode MatSuperluSetILUDropTol(Mat F,PetscReal dtol)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFactorGetSolverPackage_seqaij_superlu"
-PetscErrorCode MatFactorGetSolverPackage_seqaij_superlu(Mat A,const MatSolverPackage *type)
+PetscErrorCode MatFactorGetSolverType_seqaij_superlu(Mat A,MatSolverType *type)
 {
   PetscFunctionBegin;
   *type = MATSOLVERSUPERLU;
   PetscFunctionReturn(0);
 }
 
-
 /*MC
   MATSOLVERSUPERLU = "superlu" - A solver package providing solvers LU and ILU for sequential matrices
   via the external package SuperLU.
 
   Use ./configure --download-superlu to have PETSc installed with SuperLU
 
-  Use -pc_type lu -pc_factor_mat_solver_package superlu to us this direct solver
+  Use -pc_type lu -pc_factor_mat_solver_type superlu to use this direct solver
 
   Options Database Keys:
 + -mat_superlu_equil <FALSE>            - Equil (None)
@@ -564,16 +551,15 @@ PetscErrorCode MatFactorGetSolverPackage_seqaij_superlu(Mat A,const MatSolverPac
 . -mat_superlu_ilu_norm <0>             - ILU_Norm (None)
 - -mat_superlu_ilu_milu <0>             - ILU_MILU (None)
 
-   Notes: Do not confuse this with MATSOLVERSUPERLU_DIST which is for parallel sparse solves
+   Notes:
+    Do not confuse this with MATSOLVERSUPERLU_DIST which is for parallel sparse solves
 
    Level: beginner
 
-.seealso: PCLU, PCILU, MATSOLVERSUPERLU_DIST, MATSOLVERMUMPS, PCFactorSetMatSolverPackage(), MatSolverPackage
+.seealso: PCLU, PCILU, MATSOLVERSUPERLU_DIST, MATSOLVERMUMPS, PCFactorSetMatSolverType(), MatSolverType
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "MatGetFactor_seqaij_superlu"
-PETSC_EXTERN PetscErrorCode MatGetFactor_seqaij_superlu(Mat A,MatFactorType ftype,Mat *F)
+static PetscErrorCode MatGetFactor_seqaij_superlu(Mat A,MatFactorType ftype,Mat *F)
 {
   Mat            B;
   Mat_SuperLU    *lu;
@@ -588,17 +574,19 @@ PETSC_EXTERN PetscErrorCode MatGetFactor_seqaij_superlu(Mat A,MatFactorType ftyp
   PetscFunctionBegin;
   ierr = MatCreate(PetscObjectComm((PetscObject)A),&B);CHKERRQ(ierr);
   ierr = MatSetSizes(B,A->rmap->n,A->cmap->n,PETSC_DETERMINE,PETSC_DETERMINE);CHKERRQ(ierr);
-  ierr = MatSetType(B,((PetscObject)A)->type_name);CHKERRQ(ierr);
-  ierr = MatSeqAIJSetPreallocation(B,0,NULL);CHKERRQ(ierr);
-
+  ierr = PetscStrallocpy("superlu",&((PetscObject)B)->type_name);CHKERRQ(ierr);
+  ierr = MatSetUp(B);CHKERRQ(ierr);
   if (ftype == MAT_FACTOR_LU || ftype == MAT_FACTOR_ILU) {
     B->ops->lufactorsymbolic  = MatLUFactorSymbolic_SuperLU;
     B->ops->ilufactorsymbolic = MatLUFactorSymbolic_SuperLU;
   } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Factor type not supported");
 
+  ierr = PetscFree(B->solvertype);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(MATSOLVERSUPERLU,&B->solvertype);CHKERRQ(ierr);
+
+  B->ops->getinfo     = MatGetInfo_External;
   B->ops->destroy     = MatDestroy_SuperLU;
   B->ops->view        = MatView_SuperLU;
-  B->ops->getdiagonal = MatGetDiagonal_SuperLU;
   B->factortype       = ftype;
   B->assembled        = PETSC_TRUE;           /* required by -ksp_view */
   B->preallocated     = PETSC_TRUE;
@@ -665,11 +653,7 @@ PETSC_EXTERN PetscErrorCode MatGetFactor_seqaij_superlu(Mat A,MatFactorType ftyp
   if (flg) lu->options.ILU_Norm = (norm_t)indx;
   ierr = PetscOptionsInt("-mat_superlu_ilu_milu","ILU_MILU","None",lu->options.ILU_MILU,&indx,&flg);CHKERRQ(ierr);
   if (flg) lu->options.ILU_MILU = (milu_t)indx;
-  PetscOptionsEnd();
-  if (lu->options.Equil == YES) {
-    /* superlu overwrites input matrix and rhs when Equil is used, thus create A_dup to keep user's A unchanged */
-    ierr = MatDuplicate_SeqAIJ(A,MAT_COPY_VALUES,&lu->A_dup);CHKERRQ(ierr);
-  }
+  ierr = PetscOptionsEnd();CHKERRQ(ierr);
 
   /* Allocate spaces (notice sizes are for the transpose) */
   ierr = PetscMalloc1(m,&lu->etree);CHKERRQ(ierr);
@@ -697,22 +681,34 @@ PETSC_EXTERN PetscErrorCode MatGetFactor_seqaij_superlu(Mat A,MatFactorType ftyp
 #endif
 #endif
 
-  ierr     = PetscObjectComposeFunction((PetscObject)B,"MatFactorGetSolverPackage_C",MatFactorGetSolverPackage_seqaij_superlu);CHKERRQ(ierr);
+  ierr     = PetscObjectComposeFunction((PetscObject)B,"MatFactorGetSolverType_C",MatFactorGetSolverType_seqaij_superlu);CHKERRQ(ierr);
   ierr     = PetscObjectComposeFunction((PetscObject)B,"MatSuperluSetILUDropTol_C",MatSuperluSetILUDropTol_SuperLU);CHKERRQ(ierr);
-  B->spptr = lu;
+  B->data  = lu;
 
   *F       = B;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolverPackageRegister_SuperLU"
-PETSC_EXTERN PetscErrorCode MatSolverPackageRegister_SuperLU(void)
+static PetscErrorCode MatGetFactor_seqsell_superlu(Mat A,MatFactorType ftype,Mat *F)
+{
+  Mat_SuperLU    *lu;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = MatGetFactor_seqaij_superlu(A,ftype,F);CHKERRQ(ierr);
+  lu   = (Mat_SuperLU*)((*F)->data);
+  lu->needconversion = PETSC_TRUE;
+  PetscFunctionReturn(0);
+}
+
+PETSC_EXTERN PetscErrorCode MatSolverTypeRegister_SuperLU(void)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = MatSolverPackageRegister(MATSOLVERSUPERLU,MATSEQAIJ,       MAT_FACTOR_LU,MatGetFactor_seqaij_superlu);CHKERRQ(ierr);
-  ierr = MatSolverPackageRegister(MATSOLVERSUPERLU,MATSEQAIJ,       MAT_FACTOR_ILU,MatGetFactor_seqaij_superlu);CHKERRQ(ierr);
+  ierr = MatSolverTypeRegister(MATSOLVERSUPERLU,MATSEQAIJ,MAT_FACTOR_LU,MatGetFactor_seqaij_superlu);CHKERRQ(ierr);
+  ierr = MatSolverTypeRegister(MATSOLVERSUPERLU,MATSEQAIJ,MAT_FACTOR_ILU,MatGetFactor_seqaij_superlu);CHKERRQ(ierr);
+  ierr = MatSolverTypeRegister(MATSOLVERSUPERLU,MATSEQSELL,MAT_FACTOR_LU,MatGetFactor_seqsell_superlu);CHKERRQ(ierr);
+  ierr = MatSolverTypeRegister(MATSOLVERSUPERLU,MATSEQSELL,MAT_FACTOR_ILU,MatGetFactor_seqsell_superlu);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
