@@ -6,17 +6,15 @@
   (i,j) info for the transpose.  Someday, this info could be
   maintained so successive calls to Get will not recompute the info.
 
-  Also defined is a "faster" implementation of MatTranspose for SeqAIJ
-  matrices which avoids calls to MatSetValues.  This routine has not
-  been adopted as the standard yet as it is somewhat untested.
+  Also defined is a faster implementation of MatTranspose for SeqAIJ
+  matrices which avoids calls to MatSetValues. This routine is the new
+  standard since it is much faster than MatTranspose_AIJ.
 
 */
 
 #include <../src/mat/impls/aij/seq/aij.h>
 
 
-#undef __FUNCT__
-#define __FUNCT__ "MatGetSymbolicTranspose_SeqAIJ"
 PetscErrorCode MatGetSymbolicTranspose_SeqAIJ(Mat A,PetscInt *Ati[],PetscInt *Atj[])
 {
   PetscErrorCode ierr;
@@ -70,8 +68,6 @@ PetscErrorCode MatGetSymbolicTranspose_SeqAIJ(Mat A,PetscInt *Ati[],PetscInt *At
   MatGetSymbolicTransposeReduced_SeqAIJ() - Get symbolic matrix structure of submatrix A[rstart:rend,:],
      modified from MatGetSymbolicTranspose_SeqAIJ()
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatGetSymbolicTransposeReduced_SeqAIJ"
 PetscErrorCode MatGetSymbolicTransposeReduced_SeqAIJ(Mat A,PetscInt rstart,PetscInt rend,PetscInt *Ati[],PetscInt *Atj[])
 {
   PetscErrorCode ierr;
@@ -122,8 +118,6 @@ PetscErrorCode MatGetSymbolicTransposeReduced_SeqAIJ(Mat A,PetscInt rstart,Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatTranspose_SeqAIJ_FAST"
 PetscErrorCode MatTranspose_SeqAIJ_FAST(Mat A,MatReuse reuse,Mat *B)
 {
   PetscErrorCode ierr;
@@ -137,7 +131,7 @@ PetscErrorCode MatTranspose_SeqAIJ_FAST(Mat A,MatReuse reuse,Mat *B)
   PetscFunctionBegin;
   ierr = PetscLogEventBegin(MAT_Transpose_SeqAIJ,A,0,0,0);CHKERRQ(ierr);
 
-  if (reuse == MAT_INITIAL_MATRIX || *B == A) {
+  if (reuse == MAT_INITIAL_MATRIX || reuse == MAT_INPLACE_MATRIX) {
     /* Allocate space for symbolic transpose info and work array */
     ierr = PetscCalloc1(an+1,&ati);CHKERRQ(ierr);
     ierr = PetscMalloc1(ai[am],&atj);CHKERRQ(ierr);
@@ -175,26 +169,27 @@ PetscErrorCode MatTranspose_SeqAIJ_FAST(Mat A,MatReuse reuse,Mat *B)
 
   /* Clean up temporary space and complete requests. */
   ierr = PetscFree(atfill);CHKERRQ(ierr);
-  if (reuse == MAT_INITIAL_MATRIX) {
+  if (reuse == MAT_INITIAL_MATRIX || reuse == MAT_INPLACE_MATRIX) {
     ierr = MatCreateSeqAIJWithArrays(PetscObjectComm((PetscObject)A),an,am,ati,atj,ata,&At);CHKERRQ(ierr);
 
     at          = (Mat_SeqAIJ*)(At->data);
     at->free_a  = PETSC_TRUE;
     at->free_ij = PETSC_TRUE;
     at->nonew   = 0;
+    at->maxnz   = ati[an];
+
+    ierr = MatSetType(At,((PetscObject)A)->type_name);CHKERRQ(ierr);
   }
 
-  if (reuse == MAT_INITIAL_MATRIX || *B != A) {
+  if (reuse == MAT_INITIAL_MATRIX || reuse == MAT_REUSE_MATRIX) {
     *B = At;
   } else {
-    ierr = MatHeaderMerge(A,At);CHKERRQ(ierr);
+    ierr = MatHeaderMerge(A,&At);CHKERRQ(ierr);
   }
   ierr = PetscLogEventEnd(MAT_Transpose_SeqAIJ,A,0,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatRestoreSymbolicTranspose_SeqAIJ"
 PetscErrorCode MatRestoreSymbolicTranspose_SeqAIJ(Mat A,PetscInt *ati[],PetscInt *atj[])
 {
   PetscErrorCode ierr;
@@ -205,4 +200,3 @@ PetscErrorCode MatRestoreSymbolicTranspose_SeqAIJ(Mat A,PetscInt *ati[],PetscInt
   ierr = PetscFree(*atj);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-
