@@ -1,7 +1,5 @@
 #include <../src/mat/impls/baij/mpi/mpibaij.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "MatGetMultiProcBlock_MPIBAIJ"
 PetscErrorCode  MatGetMultiProcBlock_MPIBAIJ(Mat mat, MPI_Comm subComm, MatReuse scall,Mat *subMat)
 {
   PetscErrorCode ierr;
