@@ -1,11 +1,11 @@
 
 #include <../src/mat/impls/baij/seq/baij.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "MatConvert_SeqBAIJ_SeqAIJ"
-PETSC_EXTERN PetscErrorCode MatConvert_SeqBAIJ_SeqAIJ(Mat A, MatType newtype,MatReuse reuse,Mat *newmat)
+PETSC_INTERN PetscErrorCode MatConvert_SeqBAIJ_SeqAIJ(Mat A, MatType newtype,MatReuse reuse,Mat *newmat)
 {
   Mat            B;
+  Mat_SeqAIJ     *b;
+  PetscBool      roworiented;
   Mat_SeqBAIJ    *a = (Mat_SeqBAIJ*)A->data;
   PetscErrorCode ierr;
   PetscInt       bs = A->rmap->bs,*ai = a->i,*aj = a->j,n = A->rmap->N/bs,i,j,k;
@@ -13,20 +13,30 @@ PETSC_EXTERN PetscErrorCode MatConvert_SeqBAIJ_SeqAIJ(Mat A, MatType newtype,Mat
   MatScalar      *aa = a->a;
 
   PetscFunctionBegin;
-  ierr = PetscMalloc1(n*bs,&rowlengths);CHKERRQ(ierr);
-  for (i=0; i<n; i++) {
-    maxlen = PetscMax(maxlen,(ai[i+1] - ai[i]));
-    for (j=0; j<bs; j++) {
-      rowlengths[i*bs+j] = bs*(ai[i+1] - ai[i]);
+  if (reuse == MAT_REUSE_MATRIX) {
+    B = *newmat;
+    for (i=0; i<n; i++) {
+      maxlen = PetscMax(maxlen,(ai[i+1] - ai[i]));
     }
+  } else {
+    ierr = PetscMalloc1(n*bs,&rowlengths);CHKERRQ(ierr);
+    for (i=0; i<n; i++) {
+      maxlen = PetscMax(maxlen,(ai[i+1] - ai[i]));
+      for (j=0; j<bs; j++) {
+        rowlengths[i*bs+j] = bs*(ai[i+1] - ai[i]);
+      }
+    }
+    ierr = MatCreate(PetscObjectComm((PetscObject)A),&B);CHKERRQ(ierr);
+    ierr = MatSetType(B,MATSEQAIJ);CHKERRQ(ierr);
+    ierr = MatSetSizes(B,A->rmap->n,A->cmap->n,A->rmap->N,A->cmap->N);CHKERRQ(ierr);
+    ierr = MatSetBlockSizes(B,A->rmap->bs,A->cmap->bs);CHKERRQ(ierr);
+    ierr = MatSeqAIJSetPreallocation(B,0,rowlengths);CHKERRQ(ierr);
+    ierr = PetscFree(rowlengths);CHKERRQ(ierr);
   }
-  ierr = MatCreate(PetscObjectComm((PetscObject)A),&B);CHKERRQ(ierr);
-  ierr = MatSetSizes(B,A->rmap->n,A->cmap->n,A->rmap->N,A->cmap->N);CHKERRQ(ierr);
-  ierr = MatSetType(B,MATSEQAIJ);CHKERRQ(ierr);
-  ierr = MatSeqAIJSetPreallocation(B,0,rowlengths);CHKERRQ(ierr);
-  ierr = MatSetOption(B,MAT_ROW_ORIENTED,PETSC_FALSE);CHKERRQ(ierr);
-  ierr = PetscFree(rowlengths);CHKERRQ(ierr);
+  b = (Mat_SeqAIJ*)B->data;
+  roworiented = b->roworiented;
 
+  ierr = MatSetOption(B,MAT_ROW_ORIENTED,PETSC_FALSE);CHKERRQ(ierr);
   ierr = PetscMalloc1(bs,&rows);CHKERRQ(ierr);
   ierr = PetscMalloc1(bs*maxlen,&cols);CHKERRQ(ierr);
   for (i=0; i<n; i++) {
@@ -47,11 +57,10 @@ PETSC_EXTERN PetscErrorCode MatConvert_SeqBAIJ_SeqAIJ(Mat A, MatType newtype,Mat
   ierr = PetscFree(rows);CHKERRQ(ierr);
   ierr = MatAssemblyBegin(B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
+  ierr = MatSetOption(B,MAT_ROW_ORIENTED,roworiented);CHKERRQ(ierr);
 
-  B->rmap->bs = A->rmap->bs;
-
-  if (reuse == MAT_REUSE_MATRIX) {
-    ierr = MatHeaderReplace(A,B);CHKERRQ(ierr);
+  if (reuse == MAT_INPLACE_MATRIX) {
+    ierr = MatHeaderReplace(A,&B);CHKERRQ(ierr);
   } else {
     *newmat = B;
   }
@@ -60,9 +69,7 @@ PETSC_EXTERN PetscErrorCode MatConvert_SeqBAIJ_SeqAIJ(Mat A, MatType newtype,Mat
 
 #include <../src/mat/impls/aij/seq/aij.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "MatConvert_SeqAIJ_SeqBAIJ"
-PETSC_EXTERN PetscErrorCode MatConvert_SeqAIJ_SeqBAIJ(Mat A,MatType newtype,MatReuse reuse,Mat *newmat)
+PETSC_INTERN PetscErrorCode MatConvert_SeqAIJ_SeqBAIJ(Mat A,MatType newtype,MatReuse reuse,Mat *newmat)
 {
   Mat            B;
   Mat_SeqAIJ     *a = (Mat_SeqAIJ*)A->data;
@@ -71,21 +78,21 @@ PETSC_EXTERN PetscErrorCode MatConvert_SeqAIJ_SeqBAIJ(Mat A,MatType newtype,MatR
   PetscInt       *ai=a->i,m=A->rmap->N,n=A->cmap->N,i,*rowlengths;
 
   PetscFunctionBegin;
-  if (n != m) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Matrix must be square");
   if (A->rmap->bs > 1) {
     ierr = MatConvert_Basic(A,newtype,reuse,newmat);CHKERRQ(ierr);
     PetscFunctionReturn(0);
   }
-
-  ierr = PetscMalloc1(m,&rowlengths);CHKERRQ(ierr);
-  for (i=0; i<m; i++) {
-    rowlengths[i] = ai[i+1] - ai[i];
-  }
-  ierr = MatCreate(PetscObjectComm((PetscObject)A),&B);CHKERRQ(ierr);
-  ierr = MatSetSizes(B,m,n,m,n);CHKERRQ(ierr);
-  ierr = MatSetType(B,MATSEQBAIJ);CHKERRQ(ierr);
-  ierr = MatSeqBAIJSetPreallocation_SeqBAIJ(B,1,0,rowlengths);CHKERRQ(ierr);
-  ierr = PetscFree(rowlengths);CHKERRQ(ierr);
+  if (reuse != MAT_REUSE_MATRIX) {
+    ierr = PetscMalloc1(m,&rowlengths);CHKERRQ(ierr);
+    for (i=0; i<m; i++) {
+      rowlengths[i] = ai[i+1] - ai[i];
+    }
+    ierr = MatCreate(PetscObjectComm((PetscObject)A),&B);CHKERRQ(ierr);
+    ierr = MatSetSizes(B,m,n,m,n);CHKERRQ(ierr);
+    ierr = MatSetType(B,MATSEQBAIJ);CHKERRQ(ierr);
+    ierr = MatSeqBAIJSetPreallocation(B,1,0,rowlengths);CHKERRQ(ierr);
+    ierr = PetscFree(rowlengths);CHKERRQ(ierr);
+  } else B = *newmat;
 
   ierr = MatSetOption(B,MAT_ROW_ORIENTED,PETSC_TRUE);CHKERRQ(ierr);
 
@@ -99,8 +106,8 @@ PETSC_EXTERN PetscErrorCode MatConvert_SeqAIJ_SeqBAIJ(Mat A,MatType newtype,MatR
   ierr = MatAssemblyBegin(B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
 
-  if (reuse == MAT_REUSE_MATRIX) {
-    ierr = MatHeaderReplace(A,B);CHKERRQ(ierr);
+  if (reuse == MAT_INPLACE_MATRIX) {
+    ierr = MatHeaderReplace(A,&B);CHKERRQ(ierr);
   } else {
     *newmat = B;
   }
