@@ -1,10 +1,8 @@
 #include <../src/mat/impls/baij/seq/baij.h>
-#include <petsc-private/kernels/blockinvert.h>
+#include <petsc/private/kernels/blockinvert.h>
 
 /* bs = 15 for PFLOTRAN. Block operations are done by accessing all the columns   of the block at once */
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_15_NaturalOrdering_ver2"
 PetscErrorCode MatSolve_SeqBAIJ_15_NaturalOrdering_ver2(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a=(Mat_SeqBAIJ*)A->data;
@@ -127,8 +125,6 @@ PetscErrorCode MatSolve_SeqBAIJ_15_NaturalOrdering_ver2(Mat A,Vec bb,Vec xx)
 /* bs = 15 for PFLOTRAN. Block operations are done by accessing one column at at time */
 /* Default MatSolve for block size 15 */
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_15_NaturalOrdering_ver1"
 PetscErrorCode MatSolve_SeqBAIJ_15_NaturalOrdering_ver1(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a=(Mat_SeqBAIJ*)A->data;
@@ -234,8 +230,408 @@ PetscErrorCode MatSolve_SeqBAIJ_15_NaturalOrdering_ver1(Mat A,Vec bb,Vec xx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_7_NaturalOrdering_inplace"
+/* Block operations are done by accessing one column at at time */
+/* Default MatSolve for block size 14 */
+
+PetscErrorCode MatSolve_SeqBAIJ_14_NaturalOrdering(Mat A,Vec bb,Vec xx)
+{
+  Mat_SeqBAIJ       *a=(Mat_SeqBAIJ*)A->data;
+  PetscErrorCode    ierr;
+  const PetscInt    n=a->mbs,*ai=a->i,*aj=a->j,*adiag=a->diag,*vi,bs=A->rmap->bs,bs2=a->bs2;
+  PetscInt          i,k,nz,idx,idt,m;
+  const MatScalar   *aa=a->a,*v;
+  PetscScalar       s[14];
+  PetscScalar       *x,xv;
+  const PetscScalar *b;
+
+  PetscFunctionBegin;
+  ierr = VecGetArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecGetArray(xx,&x);CHKERRQ(ierr);
+
+  /* forward solve the lower triangular */
+  for (i=0; i<n; i++) {
+    v         = aa + bs2*ai[i];
+    vi        = aj + ai[i];
+    nz        = ai[i+1] - ai[i];
+    idt       = bs*i;
+    x[idt]    = b[idt];    x[1+idt]  = b[1+idt];  x[2+idt]  = b[2+idt];  x[3+idt]  = b[3+idt];  x[4+idt]  = b[4+idt];
+    x[5+idt]  = b[5+idt];  x[6+idt]  = b[6+idt];  x[7+idt]  = b[7+idt];  x[8+idt]  = b[8+idt];  x[9+idt] = b[9+idt];
+    x[10+idt] = b[10+idt]; x[11+idt] = b[11+idt]; x[12+idt] = b[12+idt]; x[13+idt] = b[13+idt];
+    for (m=0; m<nz; m++) {
+      idx = bs*vi[m];
+      for (k=0; k<bs; k++) {
+        xv         = x[k + idx];
+        x[idt]    -= v[0]*xv;
+        x[1+idt]  -= v[1]*xv;
+        x[2+idt]  -= v[2]*xv;
+        x[3+idt]  -= v[3]*xv;
+        x[4+idt]  -= v[4]*xv;
+        x[5+idt]  -= v[5]*xv;
+        x[6+idt]  -= v[6]*xv;
+        x[7+idt]  -= v[7]*xv;
+        x[8+idt]  -= v[8]*xv;
+        x[9+idt]  -= v[9]*xv;
+        x[10+idt] -= v[10]*xv;
+        x[11+idt] -= v[11]*xv;
+        x[12+idt] -= v[12]*xv;
+        x[13+idt] -= v[13]*xv;
+        v         += 14;
+      }
+    }
+  }
+  /* backward solve the upper triangular */
+  for (i=n-1; i>=0; i--) {
+    v     = aa + bs2*(adiag[i+1]+1);
+    vi    = aj + adiag[i+1]+1;
+    nz    = adiag[i] - adiag[i+1] - 1;
+    idt   = bs*i;
+    s[0]  = x[idt];    s[1]  = x[1+idt];  s[2]  = x[2+idt];  s[3]  = x[3+idt];  s[4]  = x[4+idt];
+    s[5]  = x[5+idt];  s[6]  = x[6+idt];  s[7]  = x[7+idt];  s[8]  = x[8+idt];  s[9]  = x[9+idt];
+    s[10] = x[10+idt]; s[11] = x[11+idt]; s[12] = x[12+idt]; s[13] = x[13+idt];
+
+    for (m=0; m<nz; m++) {
+      idx = bs*vi[m];
+      for (k=0; k<bs; k++) {
+        xv     = x[k + idx];
+        s[0]  -= v[0]*xv;
+        s[1]  -= v[1]*xv;
+        s[2]  -= v[2]*xv;
+        s[3]  -= v[3]*xv;
+        s[4]  -= v[4]*xv;
+        s[5]  -= v[5]*xv;
+        s[6]  -= v[6]*xv;
+        s[7]  -= v[7]*xv;
+        s[8]  -= v[8]*xv;
+        s[9]  -= v[9]*xv;
+        s[10] -= v[10]*xv;
+        s[11] -= v[11]*xv;
+        s[12] -= v[12]*xv;
+        s[13] -= v[13]*xv;
+        v     += 14;
+      }
+    }
+    ierr = PetscMemzero(x+idt,bs*sizeof(MatScalar));CHKERRQ(ierr);
+    for (k=0; k<bs; k++) {
+      x[idt]    += v[0]*s[k];
+      x[1+idt]  += v[1]*s[k];
+      x[2+idt]  += v[2]*s[k];
+      x[3+idt]  += v[3]*s[k];
+      x[4+idt]  += v[4]*s[k];
+      x[5+idt]  += v[5]*s[k];
+      x[6+idt]  += v[6]*s[k];
+      x[7+idt]  += v[7]*s[k];
+      x[8+idt]  += v[8]*s[k];
+      x[9+idt]  += v[9]*s[k];
+      x[10+idt] += v[10]*s[k];
+      x[11+idt] += v[11]*s[k];
+      x[12+idt] += v[12]*s[k];
+      x[13+idt] += v[13]*s[k];
+      v         += 14;
+    }
+  }
+  ierr = VecRestoreArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecRestoreArray(xx,&x);CHKERRQ(ierr);
+  ierr = PetscLogFlops(2.0*bs2*(a->nz) - bs*A->cmap->n);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/* Block operations are done by accessing one column at at time */
+/* Default MatSolve for block size 13 */
+
+PetscErrorCode MatSolve_SeqBAIJ_13_NaturalOrdering(Mat A,Vec bb,Vec xx)
+{
+  Mat_SeqBAIJ       *a=(Mat_SeqBAIJ*)A->data;
+  PetscErrorCode    ierr;
+  const PetscInt    n=a->mbs,*ai=a->i,*aj=a->j,*adiag=a->diag,*vi,bs=A->rmap->bs,bs2=a->bs2;
+  PetscInt          i,k,nz,idx,idt,m;
+  const MatScalar   *aa=a->a,*v;
+  PetscScalar       s[13];
+  PetscScalar       *x,xv;
+  const PetscScalar *b;
+
+  PetscFunctionBegin;
+  ierr = VecGetArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecGetArray(xx,&x);CHKERRQ(ierr);
+
+  /* forward solve the lower triangular */
+  for (i=0; i<n; i++) {
+    v         = aa + bs2*ai[i];
+    vi        = aj + ai[i];
+    nz        = ai[i+1] - ai[i];
+    idt       = bs*i;
+    x[idt]    = b[idt];    x[1+idt]  = b[1+idt];  x[2+idt]  = b[2+idt];  x[3+idt]  = b[3+idt];  x[4+idt]  = b[4+idt];
+    x[5+idt]  = b[5+idt];  x[6+idt]  = b[6+idt];  x[7+idt]  = b[7+idt];  x[8+idt]  = b[8+idt];  x[9+idt] = b[9+idt];
+    x[10+idt] = b[10+idt]; x[11+idt] = b[11+idt]; x[12+idt] = b[12+idt];
+    for (m=0; m<nz; m++) {
+      idx = bs*vi[m];
+      for (k=0; k<bs; k++) {
+        xv         = x[k + idx];
+        x[idt]    -= v[0]*xv;
+        x[1+idt]  -= v[1]*xv;
+        x[2+idt]  -= v[2]*xv;
+        x[3+idt]  -= v[3]*xv;
+        x[4+idt]  -= v[4]*xv;
+        x[5+idt]  -= v[5]*xv;
+        x[6+idt]  -= v[6]*xv;
+        x[7+idt]  -= v[7]*xv;
+        x[8+idt]  -= v[8]*xv;
+        x[9+idt]  -= v[9]*xv;
+        x[10+idt] -= v[10]*xv;
+        x[11+idt] -= v[11]*xv;
+        x[12+idt] -= v[12]*xv;
+        v         += 13;
+      }
+    }
+  }
+  /* backward solve the upper triangular */
+  for (i=n-1; i>=0; i--) {
+    v     = aa + bs2*(adiag[i+1]+1);
+    vi    = aj + adiag[i+1]+1;
+    nz    = adiag[i] - adiag[i+1] - 1;
+    idt   = bs*i;
+    s[0]  = x[idt];    s[1]  = x[1+idt];  s[2]  = x[2+idt];  s[3]  = x[3+idt];  s[4]  = x[4+idt];
+    s[5]  = x[5+idt];  s[6]  = x[6+idt];  s[7]  = x[7+idt];  s[8]  = x[8+idt];  s[9]  = x[9+idt];
+    s[10] = x[10+idt]; s[11] = x[11+idt]; s[12] = x[12+idt];
+
+    for (m=0; m<nz; m++) {
+      idx = bs*vi[m];
+      for (k=0; k<bs; k++) {
+        xv     = x[k + idx];
+        s[0]  -= v[0]*xv;
+        s[1]  -= v[1]*xv;
+        s[2]  -= v[2]*xv;
+        s[3]  -= v[3]*xv;
+        s[4]  -= v[4]*xv;
+        s[5]  -= v[5]*xv;
+        s[6]  -= v[6]*xv;
+        s[7]  -= v[7]*xv;
+        s[8]  -= v[8]*xv;
+        s[9]  -= v[9]*xv;
+        s[10] -= v[10]*xv;
+        s[11] -= v[11]*xv;
+        s[12] -= v[12]*xv;
+        v     += 13;
+      }
+    }
+    ierr = PetscMemzero(x+idt,bs*sizeof(MatScalar));CHKERRQ(ierr);
+    for (k=0; k<bs; k++) {
+      x[idt]    += v[0]*s[k];
+      x[1+idt]  += v[1]*s[k];
+      x[2+idt]  += v[2]*s[k];
+      x[3+idt]  += v[3]*s[k];
+      x[4+idt]  += v[4]*s[k];
+      x[5+idt]  += v[5]*s[k];
+      x[6+idt]  += v[6]*s[k];
+      x[7+idt]  += v[7]*s[k];
+      x[8+idt]  += v[8]*s[k];
+      x[9+idt]  += v[9]*s[k];
+      x[10+idt] += v[10]*s[k];
+      x[11+idt] += v[11]*s[k];
+      x[12+idt] += v[12]*s[k];
+      v         += 13;
+    }
+  }
+  ierr = VecRestoreArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecRestoreArray(xx,&x);CHKERRQ(ierr);
+  ierr = PetscLogFlops(2.0*bs2*(a->nz) - bs*A->cmap->n);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/* Block operations are done by accessing one column at at time */
+/* Default MatSolve for block size 12 */
+
+PetscErrorCode MatSolve_SeqBAIJ_12_NaturalOrdering(Mat A,Vec bb,Vec xx)
+{
+  Mat_SeqBAIJ       *a=(Mat_SeqBAIJ*)A->data;
+  PetscErrorCode    ierr;
+  const PetscInt    n=a->mbs,*ai=a->i,*aj=a->j,*adiag=a->diag,*vi,bs=A->rmap->bs,bs2=a->bs2;
+  PetscInt          i,k,nz,idx,idt,m;
+  const MatScalar   *aa=a->a,*v;
+  PetscScalar       s[12];
+  PetscScalar       *x,xv;
+  const PetscScalar *b;
+
+  PetscFunctionBegin;
+  ierr = VecGetArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecGetArray(xx,&x);CHKERRQ(ierr);
+
+  /* forward solve the lower triangular */
+  for (i=0; i<n; i++) {
+    v         = aa + bs2*ai[i];
+    vi        = aj + ai[i];
+    nz        = ai[i+1] - ai[i];
+    idt       = bs*i;
+    x[idt]    = b[idt];    x[1+idt]  = b[1+idt];  x[2+idt]  = b[2+idt];  x[3+idt]  = b[3+idt];  x[4+idt]  = b[4+idt];
+    x[5+idt]  = b[5+idt];  x[6+idt]  = b[6+idt];  x[7+idt]  = b[7+idt];  x[8+idt]  = b[8+idt];  x[9+idt] = b[9+idt];
+    x[10+idt] = b[10+idt]; x[11+idt] = b[11+idt];
+    for (m=0; m<nz; m++) {
+      idx = bs*vi[m];
+      for (k=0; k<bs; k++) {
+        xv         = x[k + idx];
+        x[idt]    -= v[0]*xv;
+        x[1+idt]  -= v[1]*xv;
+        x[2+idt]  -= v[2]*xv;
+        x[3+idt]  -= v[3]*xv;
+        x[4+idt]  -= v[4]*xv;
+        x[5+idt]  -= v[5]*xv;
+        x[6+idt]  -= v[6]*xv;
+        x[7+idt]  -= v[7]*xv;
+        x[8+idt]  -= v[8]*xv;
+        x[9+idt]  -= v[9]*xv;
+        x[10+idt] -= v[10]*xv;
+        x[11+idt] -= v[11]*xv;
+        v         += 12;
+      }
+    }
+  }
+  /* backward solve the upper triangular */
+  for (i=n-1; i>=0; i--) {
+    v     = aa + bs2*(adiag[i+1]+1);
+    vi    = aj + adiag[i+1]+1;
+    nz    = adiag[i] - adiag[i+1] - 1;
+    idt   = bs*i;
+    s[0]  = x[idt];    s[1]  = x[1+idt];  s[2]  = x[2+idt];  s[3]  = x[3+idt];  s[4]  = x[4+idt];
+    s[5]  = x[5+idt];  s[6]  = x[6+idt];  s[7]  = x[7+idt];  s[8]  = x[8+idt];  s[9]  = x[9+idt];
+    s[10] = x[10+idt]; s[11] = x[11+idt];
+
+    for (m=0; m<nz; m++) {
+      idx = bs*vi[m];
+      for (k=0; k<bs; k++) {
+        xv     = x[k + idx];
+        s[0]  -= v[0]*xv;
+        s[1]  -= v[1]*xv;
+        s[2]  -= v[2]*xv;
+        s[3]  -= v[3]*xv;
+        s[4]  -= v[4]*xv;
+        s[5]  -= v[5]*xv;
+        s[6]  -= v[6]*xv;
+        s[7]  -= v[7]*xv;
+        s[8]  -= v[8]*xv;
+        s[9]  -= v[9]*xv;
+        s[10] -= v[10]*xv;
+        s[11] -= v[11]*xv;
+        v     += 12;
+      }
+    }
+    ierr = PetscMemzero(x+idt,bs*sizeof(MatScalar));CHKERRQ(ierr);
+    for (k=0; k<bs; k++) {
+      x[idt]    += v[0]*s[k];
+      x[1+idt]  += v[1]*s[k];
+      x[2+idt]  += v[2]*s[k];
+      x[3+idt]  += v[3]*s[k];
+      x[4+idt]  += v[4]*s[k];
+      x[5+idt]  += v[5]*s[k];
+      x[6+idt]  += v[6]*s[k];
+      x[7+idt]  += v[7]*s[k];
+      x[8+idt]  += v[8]*s[k];
+      x[9+idt]  += v[9]*s[k];
+      x[10+idt] += v[10]*s[k];
+      x[11+idt] += v[11]*s[k];
+      v         += 12;
+    }
+  }
+  ierr = VecRestoreArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecRestoreArray(xx,&x);CHKERRQ(ierr);
+  ierr = PetscLogFlops(2.0*bs2*(a->nz) - bs*A->cmap->n);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/* Block operations are done by accessing one column at at time */
+/* Default MatSolve for block size 11 */
+
+PetscErrorCode MatSolve_SeqBAIJ_11_NaturalOrdering(Mat A,Vec bb,Vec xx)
+{
+  Mat_SeqBAIJ       *a=(Mat_SeqBAIJ*)A->data;
+  PetscErrorCode    ierr;
+  const PetscInt    n=a->mbs,*ai=a->i,*aj=a->j,*adiag=a->diag,*vi,bs=A->rmap->bs,bs2=a->bs2;
+  PetscInt          i,k,nz,idx,idt,m;
+  const MatScalar   *aa=a->a,*v;
+  PetscScalar       s[11];
+  PetscScalar       *x,xv;
+  const PetscScalar *b;
+
+  PetscFunctionBegin;
+  ierr = VecGetArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecGetArray(xx,&x);CHKERRQ(ierr);
+
+  /* forward solve the lower triangular */
+  for (i=0; i<n; i++) {
+    v         = aa + bs2*ai[i];
+    vi        = aj + ai[i];
+    nz        = ai[i+1] - ai[i];
+    idt       = bs*i;
+    x[idt]    = b[idt];    x[1+idt]  = b[1+idt];  x[2+idt]  = b[2+idt];  x[3+idt]  = b[3+idt];  x[4+idt]  = b[4+idt];
+    x[5+idt]  = b[5+idt];  x[6+idt]  = b[6+idt];  x[7+idt]  = b[7+idt];  x[8+idt]  = b[8+idt];  x[9+idt] = b[9+idt];
+    x[10+idt] = b[10+idt];
+    for (m=0; m<nz; m++) {
+      idx = bs*vi[m];
+      for (k=0; k<11; k++) {
+        xv         = x[k + idx];
+        x[idt]    -= v[0]*xv;
+        x[1+idt]  -= v[1]*xv;
+        x[2+idt]  -= v[2]*xv;
+        x[3+idt]  -= v[3]*xv;
+        x[4+idt]  -= v[4]*xv;
+        x[5+idt]  -= v[5]*xv;
+        x[6+idt]  -= v[6]*xv;
+        x[7+idt]  -= v[7]*xv;
+        x[8+idt]  -= v[8]*xv;
+        x[9+idt]  -= v[9]*xv;
+        x[10+idt] -= v[10]*xv;
+        v         += 11;
+      }
+    }
+  }
+  /* backward solve the upper triangular */
+  for (i=n-1; i>=0; i--) {
+    v     = aa + bs2*(adiag[i+1]+1);
+    vi    = aj + adiag[i+1]+1;
+    nz    = adiag[i] - adiag[i+1] - 1;
+    idt   = bs*i;
+    s[0]  = x[idt];    s[1]  = x[1+idt];  s[2]  = x[2+idt];  s[3]  = x[3+idt];  s[4]  = x[4+idt];
+    s[5]  = x[5+idt];  s[6]  = x[6+idt];  s[7]  = x[7+idt];  s[8]  = x[8+idt];  s[9]  = x[9+idt];
+    s[10] = x[10+idt];
+
+    for (m=0; m<nz; m++) {
+      idx = bs*vi[m];
+      for (k=0; k<11; k++) {
+        xv     = x[k + idx];
+        s[0]  -= v[0]*xv;
+        s[1]  -= v[1]*xv;
+        s[2]  -= v[2]*xv;
+        s[3]  -= v[3]*xv;
+        s[4]  -= v[4]*xv;
+        s[5]  -= v[5]*xv;
+        s[6]  -= v[6]*xv;
+        s[7]  -= v[7]*xv;
+        s[8]  -= v[8]*xv;
+        s[9]  -= v[9]*xv;
+        s[10] -= v[10]*xv;
+        v     += 11;
+      }
+    }
+    ierr = PetscMemzero(x+idt,bs*sizeof(MatScalar));CHKERRQ(ierr);
+    for (k=0; k<11; k++) {
+      x[idt]    += v[0]*s[k];
+      x[1+idt]  += v[1]*s[k];
+      x[2+idt]  += v[2]*s[k];
+      x[3+idt]  += v[3]*s[k];
+      x[4+idt]  += v[4]*s[k];
+      x[5+idt]  += v[5]*s[k];
+      x[6+idt]  += v[6]*s[k];
+      x[7+idt]  += v[7]*s[k];
+      x[8+idt]  += v[8]*s[k];
+      x[9+idt]  += v[9]*s[k];
+      x[10+idt] += v[10]*s[k];
+      v         += 11;
+    }
+  }
+  ierr = VecRestoreArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecRestoreArray(xx,&x);CHKERRQ(ierr);
+  ierr = PetscLogFlops(2.0*bs2*(a->nz) - bs*A->cmap->n);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode MatSolve_SeqBAIJ_7_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a   = (Mat_SeqBAIJ*)A->data;
@@ -331,8 +727,6 @@ PetscErrorCode MatSolve_SeqBAIJ_7_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_7_NaturalOrdering"
 PetscErrorCode MatSolve_SeqBAIJ_7_NaturalOrdering(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
@@ -419,8 +813,6 @@ PetscErrorCode MatSolve_SeqBAIJ_7_NaturalOrdering(Mat A,Vec bb,Vec xx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_6_NaturalOrdering_inplace"
 PetscErrorCode MatSolve_SeqBAIJ_6_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
@@ -500,8 +892,6 @@ PetscErrorCode MatSolve_SeqBAIJ_6_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_6_NaturalOrdering"
 PetscErrorCode MatSolve_SeqBAIJ_6_NaturalOrdering(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
@@ -584,8 +974,6 @@ PetscErrorCode MatSolve_SeqBAIJ_6_NaturalOrdering(Mat A,Vec bb,Vec xx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_5_NaturalOrdering_inplace"
 PetscErrorCode MatSolve_SeqBAIJ_5_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a   = (Mat_SeqBAIJ*)A->data;
@@ -656,8 +1044,6 @@ PetscErrorCode MatSolve_SeqBAIJ_5_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_5_NaturalOrdering"
 PetscErrorCode MatSolve_SeqBAIJ_5_NaturalOrdering(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
@@ -733,8 +1119,6 @@ PetscErrorCode MatSolve_SeqBAIJ_5_NaturalOrdering(Mat A,Vec bb,Vec xx)
       Special case where the matrix was ILU(0) factored in the natural
    ordering. This eliminates the need for the column and row permutation.
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_4_NaturalOrdering_inplace"
 PetscErrorCode MatSolve_SeqBAIJ_4_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
@@ -750,12 +1134,7 @@ PetscErrorCode MatSolve_SeqBAIJ_4_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
   ierr = VecGetArrayRead(bb,&b);CHKERRQ(ierr);
   ierr = VecGetArray(xx,&x);CHKERRQ(ierr);
 
-#if defined(PETSC_USE_FORTRAN_KERNEL_SOLVEBAIJBLAS)
-  {
-    static PetscScalar w[2000]; /* very BAD need to fix */
-    fortransolvebaij4blas_(&n,x,ai,aj,diag,aa,b,w);
-  }
-#elif defined(PETSC_USE_FORTRAN_KERNEL_SOLVEBAIJ)
+#if defined(PETSC_USE_FORTRAN_KERNEL_SOLVEBAIJ)
   {
     static PetscScalar w[2000]; /* very BAD need to fix */
     fortransolvebaij4_(&n,x,ai,aj,diag,aa,b,w);
@@ -826,8 +1205,6 @@ PetscErrorCode MatSolve_SeqBAIJ_4_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_4_NaturalOrdering"
 PetscErrorCode MatSolve_SeqBAIJ_4_NaturalOrdering(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
@@ -901,8 +1278,6 @@ PetscErrorCode MatSolve_SeqBAIJ_4_NaturalOrdering(Mat A,Vec bb,Vec xx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_4_NaturalOrdering_Demotion"
 PetscErrorCode MatSolve_SeqBAIJ_4_NaturalOrdering_Demotion(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
@@ -996,8 +1371,6 @@ PetscErrorCode MatSolve_SeqBAIJ_4_NaturalOrdering_Demotion(Mat A,Vec bb,Vec xx)
 #if defined(PETSC_HAVE_SSE)
 
 #include PETSC_HAVE_SSE
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_4_NaturalOrdering_SSE_Demotion_usj"
 PetscErrorCode MatSolve_SeqBAIJ_4_NaturalOrdering_SSE_Demotion_usj(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ    *a = (Mat_SeqBAIJ*)A->data;
@@ -1193,8 +1566,6 @@ PetscErrorCode MatSolve_SeqBAIJ_4_NaturalOrdering_SSE_Demotion_usj(Mat A,Vec bb,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_4_NaturalOrdering_SSE_Demotion"
 PetscErrorCode MatSolve_SeqBAIJ_4_NaturalOrdering_SSE_Demotion(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ    *a = (Mat_SeqBAIJ*)A->data;
@@ -1398,8 +1769,6 @@ PetscErrorCode MatSolve_SeqBAIJ_4_NaturalOrdering_SSE_Demotion(Mat A,Vec bb,Vec
       Special case where the matrix was ILU(0) factored in the natural
    ordering. This eliminates the need for the column and row permutation.
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_3_NaturalOrdering_inplace"
 PetscErrorCode MatSolve_SeqBAIJ_3_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
@@ -1464,8 +1833,6 @@ PetscErrorCode MatSolve_SeqBAIJ_3_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_3_NaturalOrdering"
 PetscErrorCode MatSolve_SeqBAIJ_3_NaturalOrdering(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
@@ -1535,12 +1902,102 @@ PetscErrorCode MatSolve_SeqBAIJ_3_NaturalOrdering(Mat A,Vec bb,Vec xx)
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode MatForwardSolve_SeqBAIJ_3_NaturalOrdering(Mat A,Vec bb,Vec xx)
+{
+  Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
+  const PetscInt    n  =a->mbs,*vi,*ai=a->i,*aj=a->j;
+  PetscErrorCode    ierr;
+  PetscInt          i,k,nz,idx,jdx;
+  const PetscInt    bs = A->rmap->bs,bs2 = a->bs2;
+  const MatScalar   *aa=a->a,*v;
+  PetscScalar       *x;
+  const PetscScalar *b;
+  PetscScalar       s1,s2,s3,x1,x2,x3;
+
+  PetscFunctionBegin;
+  ierr = VecGetArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecGetArray(xx,&x);CHKERRQ(ierr);
+  /* forward solve the lower triangular */
+  idx  = 0;
+  x[0] = b[idx]; x[1] = b[1+idx];x[2] = b[2+idx];
+  for (i=1; i<n; i++) {
+    v   = aa + bs2*ai[i];
+    vi  = aj + ai[i];
+    nz  = ai[i+1] - ai[i];
+    idx = bs*i;
+    s1  = b[idx];s2 = b[1+idx];s3 = b[2+idx];
+    for (k=0; k<nz; k++) {
+      jdx = bs*vi[k];
+      x1  = x[jdx];x2 = x[1+jdx]; x3 =x[2+jdx];
+      s1 -= v[0]*x1 + v[3]*x2 + v[6]*x3;
+      s2 -= v[1]*x1 + v[4]*x2 + v[7]*x3;
+      s3 -= v[2]*x1 + v[5]*x2 + v[8]*x3;
+
+      v +=  bs2;
+    }
+
+    x[idx]   = s1;
+    x[1+idx] = s2;
+    x[2+idx] = s3;
+  }
+
+  ierr = VecRestoreArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecRestoreArray(xx,&x);CHKERRQ(ierr);
+  ierr = PetscLogFlops(1.0*bs2*(a->nz) - bs*A->cmap->n);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+
+PetscErrorCode MatBackwardSolve_SeqBAIJ_3_NaturalOrdering(Mat A,Vec bb,Vec xx)
+{
+  Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
+  const PetscInt    n  =a->mbs,*vi,*aj=a->j,*adiag=a->diag;
+  PetscErrorCode    ierr;
+  PetscInt          i,k,nz,idx,idt;
+  const PetscInt    bs = A->rmap->bs,bs2 = a->bs2;
+  const MatScalar   *aa=a->a,*v;
+  PetscScalar       *x;
+  const PetscScalar *b;
+  PetscScalar       s1,s2,s3,x1,x2,x3;
+
+  PetscFunctionBegin;
+  ierr = VecGetArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecGetArray(xx,&x);CHKERRQ(ierr);
+
+  /* backward solve the upper triangular */
+  for (i=n-1; i>=0; i--) {
+    v   = aa + bs2*(adiag[i+1]+1);
+    vi  = aj + adiag[i+1]+1;
+    nz  = adiag[i] - adiag[i+1]-1;
+    idt = bs*i;
+    s1  = b[idt];  s2 = b[1+idt];s3 = b[2+idt];
+
+    for (k=0; k<nz; k++) {
+      idx = bs*vi[k];
+      x1  = x[idx];   x2 = x[1+idx]; x3 = x[2+idx];
+      s1 -= v[0]*x1 + v[3]*x2 + v[6]*x3;
+      s2 -= v[1]*x1 + v[4]*x2 + v[7]*x3;
+      s3 -= v[2]*x1 + v[5]*x2 + v[8]*x3;
+
+      v +=  bs2;
+    }
+    /* x = inv_diagonal*x */
+    x[idt]   = v[0]*s1 + v[3]*s2 + v[6]*s3;
+    x[1+idt] = v[1]*s1 + v[4]*s2 + v[7]*s3;
+    x[2+idt] = v[2]*s1 + v[5]*s2 + v[8]*s3;
+
+  }
+
+  ierr = VecRestoreArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecRestoreArray(xx,&x);CHKERRQ(ierr);
+  ierr = PetscLogFlops(2.0*bs2*(a->nz) - bs*A->cmap->n);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*
       Special case where the matrix was ILU(0) factored in the natural
    ordering. This eliminates the need for the column and row permutation.
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_2_NaturalOrdering_inplace"
 PetscErrorCode MatSolve_SeqBAIJ_2_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
@@ -1599,8 +2056,6 @@ PetscErrorCode MatSolve_SeqBAIJ_2_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_2_NaturalOrdering"
 PetscErrorCode MatSolve_SeqBAIJ_2_NaturalOrdering(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
@@ -1663,12 +2118,93 @@ PetscErrorCode MatSolve_SeqBAIJ_2_NaturalOrdering(Mat A,Vec bb,Vec xx)
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode MatForwardSolve_SeqBAIJ_2_NaturalOrdering(Mat A,Vec bb,Vec xx)
+{
+  Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
+  const PetscInt    n  = a->mbs,*vi,*ai=a->i,*aj=a->j;
+  PetscInt          i,k,nz,idx,jdx;
+  PetscErrorCode    ierr;
+  const MatScalar   *aa=a->a,*v;
+  PetscScalar       *x,s1,s2,x1,x2;
+  const PetscScalar *b;
+
+  PetscFunctionBegin;
+  ierr = VecGetArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecGetArray(xx,&x);CHKERRQ(ierr);
+  /* forward solve the lower triangular */
+  idx  = 0;
+  x[0] = b[idx]; x[1] = b[1+idx];
+  for (i=1; i<n; i++) {
+    v   = aa + 4*ai[i];
+    vi  = aj + ai[i];
+    nz  = ai[i+1] - ai[i];
+    idx = 2*i;
+    s1  = b[idx];s2 = b[1+idx];
+    PetscPrefetchBlock(vi+nz,nz,0,PETSC_PREFETCH_HINT_NTA);
+    PetscPrefetchBlock(v+4*nz,4*nz,0,PETSC_PREFETCH_HINT_NTA);
+    for (k=0; k<nz; k++) {
+      jdx = 2*vi[k];
+      x1  = x[jdx];x2 = x[1+jdx];
+      s1 -= v[0]*x1 + v[2]*x2;
+      s2 -= v[1]*x1 + v[3]*x2;
+      v  +=  4;
+    }
+    x[idx]   = s1;
+    x[1+idx] = s2;
+  }
+
+
+  ierr = VecRestoreArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecRestoreArray(xx,&x);CHKERRQ(ierr);
+  ierr = PetscLogFlops(4.0*(a->nz) - A->cmap->n);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode MatBackwardSolve_SeqBAIJ_2_NaturalOrdering(Mat A,Vec bb,Vec xx)
+{
+  Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
+  const PetscInt    n  = a->mbs,*vi,*aj=a->j,*adiag=a->diag;
+  PetscInt          i,k,nz,idx,idt;
+  PetscErrorCode    ierr;
+  const MatScalar   *aa=a->a,*v;
+  PetscScalar       *x,s1,s2,x1,x2;
+  const PetscScalar *b;
+
+  PetscFunctionBegin;
+  ierr = VecGetArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecGetArray(xx,&x);CHKERRQ(ierr);
+
+  /* backward solve the upper triangular */
+  for (i=n-1; i>=0; i--) {
+    v   = aa + 4*(adiag[i+1]+1);
+    vi  = aj + adiag[i+1]+1;
+    nz  = adiag[i] - adiag[i+1]-1;
+    idt = 2*i;
+    s1  = b[idt];  s2 = b[1+idt];
+    PetscPrefetchBlock(vi+nz,nz,0,PETSC_PREFETCH_HINT_NTA);
+    PetscPrefetchBlock(v+4*nz,4*nz,0,PETSC_PREFETCH_HINT_NTA);
+    for (k=0; k<nz; k++) {
+      idx = 2*vi[k];
+      x1  = x[idx];   x2 = x[1+idx];
+      s1 -= v[0]*x1 + v[2]*x2;
+      s2 -= v[1]*x1 + v[3]*x2;
+      v  += 4;
+    }
+    /* x = inv_diagonal*x */
+    x[idt]   = v[0]*s1 + v[2]*s2;
+    x[1+idt] = v[1]*s1 + v[3]*s2;
+  }
+
+  ierr = VecRestoreArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecRestoreArray(xx,&x);CHKERRQ(ierr);
+  ierr = PetscLogFlops(4.0*a->nz - A->cmap->n);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*
       Special case where the matrix was ILU(0) factored in the natural
    ordering. This eliminates the need for the column and row permutation.
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_1_NaturalOrdering_inplace"
 PetscErrorCode MatSolve_SeqBAIJ_1_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
@@ -1724,8 +2260,72 @@ PetscErrorCode MatSolve_SeqBAIJ_1_NaturalOrdering_inplace(Mat A,Vec bb,Vec xx)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_SeqBAIJ_1_NaturalOrdering"
+PetscErrorCode MatForwardSolve_SeqBAIJ_1_NaturalOrdering(Mat A,Vec bb,Vec xx)
+{
+  Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
+  PetscErrorCode    ierr;
+  const PetscInt    n = a->mbs,*ai = a->i,*aj = a->j,*vi;
+  PetscScalar       *x,sum;
+  const PetscScalar *b;
+  const MatScalar   *aa = a->a,*v;
+  PetscInt          i,nz;
+
+  PetscFunctionBegin;
+  if (!n) PetscFunctionReturn(0);
+
+  ierr = VecGetArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecGetArray(xx,&x);CHKERRQ(ierr);
+
+  /* forward solve the lower triangular */
+  x[0] = b[0];
+  v    = aa;
+  vi   = aj;
+  for (i=1; i<n; i++) {
+    nz  = ai[i+1] - ai[i];
+    sum = b[i];
+    PetscSparseDenseMinusDot(sum,x,v,vi,nz);
+    v   += nz;
+    vi  += nz;
+    x[i] = sum;
+  }
+  ierr = PetscLogFlops(a->nz - A->cmap->n);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecRestoreArray(xx,&x);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode MatBackwardSolve_SeqBAIJ_1_NaturalOrdering(Mat A,Vec bb,Vec xx)
+{
+  Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
+  PetscErrorCode    ierr;
+  const PetscInt    n = a->mbs,*aj = a->j,*adiag = a->diag,*vi;
+  PetscScalar       *x,sum;
+  const PetscScalar *b;
+  const MatScalar   *aa = a->a,*v;
+  PetscInt          i,nz;
+
+  PetscFunctionBegin;
+  if (!n) PetscFunctionReturn(0);
+
+  ierr = VecGetArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecGetArray(xx,&x);CHKERRQ(ierr);
+
+  /* backward solve the upper triangular */
+  for (i=n-1; i>=0; i--) {
+    v   = aa + adiag[i+1] + 1;
+    vi  = aj + adiag[i+1] + 1;
+    nz  = adiag[i] - adiag[i+1]-1;
+    sum = b[i];
+    PetscSparseDenseMinusDot(sum,x,v,vi,nz);
+    x[i] = sum*v[nz]; /* x[i]=aa[adiag[i]]*sum; v++; */
+  }
+
+  ierr = PetscLogFlops(2.0*a->nz - A->cmap->n);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(bb,&b);CHKERRQ(ierr);
+  ierr = VecRestoreArray(xx,&x);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode MatSolve_SeqBAIJ_1_NaturalOrdering(Mat A,Vec bb,Vec xx)
 {
   Mat_SeqBAIJ       *a = (Mat_SeqBAIJ*)A->data;
