@@ -17,8 +17,6 @@ typedef struct {
   Vec               leftwork,rightwork;
 } Mat_Composite;
 
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_Composite"
 PetscErrorCode MatDestroy_Composite(Mat mat)
 {
   PetscErrorCode    ierr;
@@ -44,8 +42,6 @@ PetscErrorCode MatDestroy_Composite(Mat mat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_Composite_Multiplicative"
 PetscErrorCode MatMult_Composite_Multiplicative(Mat A,Vec x,Vec y)
 {
   Mat_Composite     *shell = (Mat_Composite*)A->data;
@@ -80,8 +76,6 @@ PetscErrorCode MatMult_Composite_Multiplicative(Mat A,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMultTranspose_Composite_Multiplicative"
 PetscErrorCode MatMultTranspose_Composite_Multiplicative(Mat A,Vec x,Vec y)
 {
   Mat_Composite     *shell = (Mat_Composite*)A->data;
@@ -116,8 +110,6 @@ PetscErrorCode MatMultTranspose_Composite_Multiplicative(Mat A,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_Composite"
 PetscErrorCode MatMult_Composite(Mat A,Vec x,Vec y)
 {
   Mat_Composite     *shell = (Mat_Composite*)A->data;
@@ -146,8 +138,6 @@ PetscErrorCode MatMult_Composite(Mat A,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMultTranspose_Composite"
 PetscErrorCode MatMultTranspose_Composite(Mat A,Vec x,Vec y)
 {
   Mat_Composite     *shell = (Mat_Composite*)A->data;
@@ -176,8 +166,6 @@ PetscErrorCode MatMultTranspose_Composite(Mat A,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatGetDiagonal_Composite"
 PetscErrorCode MatGetDiagonal_Composite(Mat A,Vec v)
 {
   Mat_Composite     *shell = (Mat_Composite*)A->data;
@@ -200,23 +188,19 @@ PetscErrorCode MatGetDiagonal_Composite(Mat A,Vec v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatAssemblyEnd_Composite"
 PetscErrorCode MatAssemblyEnd_Composite(Mat Y,MatAssemblyType t)
 {
   PetscErrorCode ierr;
   PetscBool      flg = PETSC_FALSE;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsGetBool(((PetscObject)Y)->prefix,"-mat_composite_merge",&flg,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(((PetscObject)Y)->options,((PetscObject)Y)->prefix,"-mat_composite_merge",&flg,NULL);CHKERRQ(ierr);
   if (flg) {
     ierr = MatCompositeMerge(Y);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatScale_Composite"
 PetscErrorCode MatScale_Composite(Mat inA,PetscScalar alpha)
 {
   Mat_Composite *a = (Mat_Composite*)inA->data;
@@ -226,8 +210,6 @@ PetscErrorCode MatScale_Composite(Mat inA,PetscScalar alpha)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatDiagonalScale_Composite"
 PetscErrorCode MatDiagonalScale_Composite(Mat inA,Vec left,Vec right)
 {
   Mat_Composite  *a = (Mat_Composite*)inA->data;
@@ -400,15 +382,14 @@ static struct _MatOps MatOps_Values = {0,
 /*MC
    MATCOMPOSITE - A matrix defined by the sum (or product) of one or more matrices (all matrices are of same size and parallel layout).
 
-   Notes: to use the product of the matrices call MatCompositeSetType(mat,MAT_COMPOSITE_MULTIPLICATIVE);
+   Notes:
+    to use the product of the matrices call MatCompositeSetType(mat,MAT_COMPOSITE_MULTIPLICATIVE);
 
   Level: advanced
 
 .seealso: MatCreateComposite(), MatCompositeAddMat(), MatSetType(), MatCompositeMerge(), MatCompositeSetType(), MatCompositeType
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCreate_Composite"
 PETSC_EXTERN PetscErrorCode MatCreate_Composite(Mat A)
 {
   Mat_Composite  *b;
@@ -430,9 +411,7 @@ PETSC_EXTERN PetscErrorCode MatCreate_Composite(Mat A)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCreateComposite"
-/*@C
+/*@
    MatCreateComposite - Creates a matrix as the sum of zero or more matrices
 
   Collective on MPI_Comm
@@ -443,7 +422,7 @@ PETSC_EXTERN PetscErrorCode MatCreate_Composite(Mat A)
 -  mats - the matrices
 
    Output Parameter:
-.  A - the matrix
+.  mat - the matrix
 
    Level: advanced
 
@@ -485,8 +464,6 @@ PetscErrorCode  MatCreateComposite(MPI_Comm comm,PetscInt nmat,const Mat *mats,M
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCompositeAddMat"
 /*@
     MatCompositeAddMat - add another matrix to a composite matrix
 
@@ -528,9 +505,7 @@ PetscErrorCode  MatCompositeAddMat(Mat mat,Mat smat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCompositeSetType"
-/*@C
+/*@
    MatCompositeSetType - Indicates if the matrix is defined as the sum of a set of matrices or the product
 
   Collective on MPI_Comm
@@ -572,9 +547,7 @@ PetscErrorCode  MatCompositeSetType(Mat mat,MatCompositeType type)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCompositeMerge"
-/*@C
+/*@
    MatCompositeMerge - Given a composite matrix, replaces it with a "regular" matrix
      by summing all the matrices inside the composite matrix.
 
@@ -627,7 +600,7 @@ PetscErrorCode  MatCompositeMerge(Mat mat)
   if ((left = shell->left)) {ierr = PetscObjectReference((PetscObject)left);CHKERRQ(ierr);}
   if ((right = shell->right)) {ierr = PetscObjectReference((PetscObject)right);CHKERRQ(ierr);}
 
-  ierr = MatHeaderReplace(mat,tmat);CHKERRQ(ierr);
+  ierr = MatHeaderReplace(mat,&tmat);CHKERRQ(ierr);
 
   ierr = MatDiagonalScale(mat,left,right);CHKERRQ(ierr);
   ierr = MatScale(mat,scale);CHKERRQ(ierr);
