@@ -38,8 +38,6 @@ extern PetscErrorCode VecDestroy_MPIFFTW(Vec);
    Output parameter:
      y - vector that stores result of FDFT
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_SeqFFTW"
 PetscErrorCode MatMult_SeqFFTW(Mat A,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -120,7 +118,11 @@ PetscErrorCode MatMult_SeqFFTW(Mat A,Vec x,Vec y)
     fftw_execute(fftw->p_forward);
   } else { /* use existing plan */
     if (fftw->finarray != x_array || fftw->foutarray != y_array) { /* use existing plan on new arrays */
+#if defined(PETSC_USE_COMPLEX)
       fftw_execute_dft(fftw->p_forward,(fftw_complex*)x_array,(fftw_complex*)y_array);
+#else
+      fftw_execute_dft_r2c(fftw->p_forward,(double*)x_array,(fftw_complex*)y_array);
+#endif
     } else {
       fftw_execute(fftw->p_forward);
     }
@@ -139,8 +141,6 @@ PetscErrorCode MatMult_SeqFFTW(Mat A,Vec x,Vec y)
      y - vector that stores result of BDFT
 */
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMultTranspose_SeqFFTW"
 PetscErrorCode MatMultTranspose_SeqFFTW(Mat A,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -200,7 +200,11 @@ PetscErrorCode MatMultTranspose_SeqFFTW(Mat A,Vec x,Vec y)
     fftw_execute(fftw->p_backward);CHKERRQ(ierr);
   } else { /* use existing plan */
     if (fftw->binarray != x_array || fftw->boutarray != y_array) { /* use existing plan on new arrays */
+#if defined(PETSC_USE_COMPLEX)
       fftw_execute_dft(fftw->p_backward,(fftw_complex*)x_array,(fftw_complex*)y_array);
+#else
+      fftw_execute_dft_c2r(fftw->p_backward,(fftw_complex*)x_array,(double*)y_array);
+#endif
     } else {
       fftw_execute(fftw->p_backward);CHKERRQ(ierr);
     }
@@ -218,8 +222,6 @@ PetscErrorCode MatMultTranspose_SeqFFTW(Mat A,Vec x,Vec y)
    Output parameter:
    y   - vector that stores result of FDFT
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_MPIFFTW"
 PetscErrorCode MatMult_MPIFFTW(Mat A,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -290,8 +292,6 @@ PetscErrorCode MatMult_MPIFFTW(Mat A,Vec x,Vec y)
    Output parameter:
      y - vector that stores result of BDFT
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatMultTranspose_MPIFFTW"
 PetscErrorCode MatMultTranspose_MPIFFTW(Mat A,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -352,8 +352,6 @@ PetscErrorCode MatMultTranspose_MPIFFTW(Mat A,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_FFTW"
 PetscErrorCode MatDestroy_FFTW(Mat A)
 {
   Mat_FFT        *fft  = (Mat_FFT*)A->data;
@@ -373,8 +371,6 @@ PetscErrorCode MatDestroy_FFTW(Mat A)
 }
 
 #include <../src/vec/vec/impls/mpi/pvecimpl.h>   /*I  "petscvec.h"   I*/
-#undef __FUNCT__
-#define __FUNCT__ "VecDestroy_MPIFFTW"
 PetscErrorCode VecDestroy_MPIFFTW(Vec v)
 {
   PetscErrorCode ierr;
@@ -388,8 +384,6 @@ PetscErrorCode VecDestroy_MPIFFTW(Vec v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCreateVecsFFTW"
 /*@
    MatCreateVecsFFTW - Get vector(s) compatible with the matrix, i.e. with the
      parallel layout determined by FFTW
@@ -426,12 +420,10 @@ PetscErrorCode MatCreateVecsFFTW(Mat A,Vec *x,Vec *y,Vec *z)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscTryMethod(A,"MatCreateVecsFFTW_C",(Mat,Vec*,Vec*,Vec*),(A,x,y,z));CHKERRQ(ierr);
+  ierr = PetscUseMethod(A,"MatCreateVecsFFTW_C",(Mat,Vec*,Vec*,Vec*),(A,x,y,z));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCreateVecsFFTW_FFTW"
 PetscErrorCode  MatCreateVecsFFTW_FFTW(Mat A,Vec *fin,Vec *fout,Vec *bout)
 {
   PetscErrorCode ierr;
@@ -646,8 +638,6 @@ PetscErrorCode  MatCreateVecsFFTW_FFTW(Mat A,Vec *fin,Vec *fout,Vec *bout)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecScatterPetscToFFTW"
 /*@
    VecScatterPetscToFFTW - Copies the PETSc vector to the vector that goes into FFTW block.
 
@@ -676,12 +666,10 @@ PetscErrorCode VecScatterPetscToFFTW(Mat A,Vec x,Vec y)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscTryMethod(A,"VecScatterPetscToFFTW_C",(Mat,Vec,Vec),(A,x,y));CHKERRQ(ierr);
+  ierr = PetscUseMethod(A,"VecScatterPetscToFFTW_C",(Mat,Vec,Vec),(A,x,y));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecScatterPetscToFFTW_FFTW"
 PetscErrorCode VecScatterPetscToFFTW_FFTW(Mat A,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -887,8 +875,6 @@ PetscErrorCode VecScatterPetscToFFTW_FFTW(Mat A,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecScatterFFTWToPetsc"
 /*@
    VecScatterFFTWToPetsc - Converts FFTW output to the PETSc vector.
 
@@ -913,12 +899,10 @@ PetscErrorCode VecScatterFFTWToPetsc(Mat A,Vec x,Vec y)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscTryMethod(A,"VecScatterFFTWToPetsc_C",(Mat,Vec,Vec),(A,x,y));CHKERRQ(ierr);
+  ierr = PetscUseMethod(A,"VecScatterFFTWToPetsc_C",(Mat,Vec,Vec),(A,x,y));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecScatterFFTWToPetsc_FFTW"
 PetscErrorCode VecScatterFFTWToPetsc_FFTW(Mat A,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -1116,8 +1100,6 @@ PetscErrorCode VecScatterFFTWToPetsc_FFTW(Mat A,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCreate_FFTW"
 /*
     MatCreate_FFTW - Creates a matrix object that provides FFT via the external package FFTW
 
@@ -1243,6 +1225,7 @@ PETSC_EXTERN PetscErrorCode MatCreate_FFTW(Mat A)
       break;
     }
   }
+  free(pdim);
   ierr      = PetscObjectChangeTypeName((PetscObject)A,MATFFTW);CHKERRQ(ierr);
   ierr      = PetscNewLog(A,&fftw);CHKERRQ(ierr);
   fft->data = (void*)fftw;
