@@ -1,5 +1,5 @@
 
-#include <petsc-private/matimpl.h>
+#include <petsc/private/matimpl.h>
 #include <../src/mat/impls/mffd/mffdimpl.h>   /*I  "petscmat.h"   I*/
 
 PetscFunctionList MatMFFDList              = 0;
@@ -9,8 +9,6 @@ PetscClassId  MATMFFD_CLASSID;
 PetscLogEvent MATMFFD_Mult;
 
 static PetscBool MatMFFDPackageInitialized = PETSC_FALSE;
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDFinalizePackage"
 /*@C
   MatMFFDFinalizePackage - This function destroys everything in the MatMFFD package. It is
   called from PetscFinalize().
@@ -18,7 +16,7 @@ static PetscBool MatMFFDPackageInitialized = PETSC_FALSE;
   Level: developer
 
 .keywords: Petsc, destroy, package
-.seealso: PetscFinalize()
+.seealso: PetscFinalize(), MatCreateMFFD(), MatCreateSNESMF()
 @*/
 PetscErrorCode  MatMFFDFinalizePackage(void)
 {
@@ -31,8 +29,6 @@ PetscErrorCode  MatMFFDFinalizePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDInitializePackage"
 /*@C
   MatMFFDInitializePackage - This function initializes everything in the MatMFFD package. It is called
   from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to MatCreate_MFFD()
@@ -46,8 +42,7 @@ PetscErrorCode  MatMFFDFinalizePackage(void)
 PetscErrorCode  MatMFFDInitializePackage(void)
 {
   char           logList[256];
-  char           *className;
-  PetscBool      opt;
+  PetscBool      opt,pkg;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -58,30 +53,24 @@ PetscErrorCode  MatMFFDInitializePackage(void)
   /* Register Constructors */
   ierr = MatMFFDRegisterAll();CHKERRQ(ierr);
   /* Register Events */
-  ierr = PetscLogEventRegister("MatMult MF",          MATMFFD_CLASSID,&MATMFFD_Mult);CHKERRQ(ierr);
-
+  ierr = PetscLogEventRegister("MatMult MF",MATMFFD_CLASSID,&MATMFFD_Mult);CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL, "-info_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "matmffd", &className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscInfoDeactivateClass(MATMFFD_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("matmffd",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) {ierr = PetscInfoDeactivateClass(MATMFFD_CLASSID);CHKERRQ(ierr);}
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(NULL, "-log_summary_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "matmffd", &className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscLogEventDeactivateClass(MATMFFD_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("matmffd",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) {ierr = PetscLogEventExcludeClass(MATMFFD_CLASSID);CHKERRQ(ierr);}
   }
+  /* Register package finalizer */
   ierr = PetscRegisterFinalize(MatMFFDFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetType"
 /*@C
     MatMFFDSetType - Sets the method that is used to compute the
     differencing parameter for finite differene matrix-free formulations.
@@ -101,7 +90,7 @@ PetscErrorCode  MatMFFDInitializePackage(void)
           F'(u)a  ~=  ----------------
                               h
 
-.seealso: MatCreateSNESMF(), MatMFFDRegister(), MatMFFDSetFunction()
+.seealso: MatCreateSNESMF(), MatMFFDRegister(), MatMFFDSetFunction(), MatCreateMFFD()
 @*/
 PetscErrorCode  MatMFFDSetType(Mat mat,MatMFFDType ftype)
 {
@@ -132,33 +121,39 @@ PetscErrorCode  MatMFFDSetType(Mat mat,MatMFFDType ftype)
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode MatGetDiagonal_MFFD(Mat,Vec);
+
 typedef PetscErrorCode (*FCN1)(void*,Vec); /* force argument to next function to not be extern C*/
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetFunctioniBase_MFFD"
-PetscErrorCode  MatMFFDSetFunctioniBase_MFFD(Mat mat,FCN1 func)
+static PetscErrorCode  MatMFFDSetFunctioniBase_MFFD(Mat mat,FCN1 func)
 {
   MatMFFD ctx = (MatMFFD)mat->data;
 
   PetscFunctionBegin;
   ctx->funcisetbase = func;
+  /* allow users to compose their own getdiagonal and allow MatHasOperation
+     to return false if the two functions pointers are not set */
+  if (!mat->ops->getdiagonal && func) {
+    mat->ops->getdiagonal = MatGetDiagonal_MFFD;
+  }
   PetscFunctionReturn(0);
 }
 
 typedef PetscErrorCode (*FCN2)(void*,PetscInt,Vec,PetscScalar*); /* force argument to next function to not be extern C*/
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetFunctioni_MFFD"
-PetscErrorCode  MatMFFDSetFunctioni_MFFD(Mat mat,FCN2 funci)
+static PetscErrorCode  MatMFFDSetFunctioni_MFFD(Mat mat,FCN2 funci)
 {
   MatMFFD ctx = (MatMFFD)mat->data;
 
   PetscFunctionBegin;
   ctx->funci = funci;
+  /* allow users to compose their own getdiagonal and allow MatHasOperation
+     to return false if the two functions pointers are not set */
+  if (!mat->ops->getdiagonal && funci) {
+    mat->ops->getdiagonal = MatGetDiagonal_MFFD;
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDResetHHistory_MFFD"
-PetscErrorCode  MatMFFDResetHHistory_MFFD(Mat J)
+static PetscErrorCode  MatMFFDResetHHistory_MFFD(Mat J)
 {
   MatMFFD ctx = (MatMFFD)J->data;
 
@@ -167,8 +162,6 @@ PetscErrorCode  MatMFFDResetHHistory_MFFD(Mat J)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDRegister"
 /*@C
    MatMFFDRegister - Adds a method to the MatMFFD registry.
 
@@ -191,7 +184,7 @@ PetscErrorCode  MatMFFDResetHHistory_MFFD(Mat J)
    Then, your solver can be chosen with the procedural interface via
 $     MatMFFDSetType(mfctx,"my_h")
    or at runtime via the option
-$     -snes_mf_type my_h
+$     -mat_mffd_type my_h
 
 .keywords: MatMFFD, register
 
@@ -202,28 +195,13 @@ PetscErrorCode  MatMFFDRegister(const char sname[],PetscErrorCode (*function)(Ma
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = MatInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&MatMFFDList,sname,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDAddNullSpace_MFFD"
-PetscErrorCode  MatMFFDAddNullSpace_MFFD(Mat J,MatNullSpace nullsp)
-{
-  PetscErrorCode ierr;
-  MatMFFD        ctx = (MatMFFD)J->data;
-
-  PetscFunctionBegin;
-  ierr = PetscObjectReference((PetscObject)nullsp);CHKERRQ(ierr);
-  if (ctx->sp) { ierr = MatNullSpaceDestroy(&ctx->sp);CHKERRQ(ierr); }
-  ctx->sp = nullsp;
-  PetscFunctionReturn(0);
-}
-
 /* ----------------------------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_MFFD"
-PetscErrorCode MatDestroy_MFFD(Mat mat)
+static PetscErrorCode MatDestroy_MFFD(Mat mat)
 {
   PetscErrorCode ierr;
   MatMFFD        ctx = (MatMFFD)mat->data;
@@ -233,11 +211,12 @@ PetscErrorCode MatDestroy_MFFD(Mat mat)
   ierr = VecDestroy(&ctx->drscale);CHKERRQ(ierr);
   ierr = VecDestroy(&ctx->dlscale);CHKERRQ(ierr);
   ierr = VecDestroy(&ctx->dshift);CHKERRQ(ierr);
+  ierr = VecDestroy(&ctx->dshiftw);CHKERRQ(ierr);
+  ierr = VecDestroy(&ctx->current_u);CHKERRQ(ierr);
   if (ctx->current_f_allocated) {
     ierr = VecDestroy(&ctx->current_f);CHKERRQ(ierr);
   }
   if (ctx->ops->destroy) {ierr = (*ctx->ops->destroy)(ctx);CHKERRQ(ierr);}
-  ierr      = MatNullSpaceDestroy(&ctx->sp);CHKERRQ(ierr);
   ierr      = PetscHeaderDestroy(&ctx);CHKERRQ(ierr);
   mat->data = 0;
 
@@ -249,17 +228,14 @@ PetscErrorCode MatDestroy_MFFD(Mat mat)
   ierr = PetscObjectComposeFunction((PetscObject)mat,"MatMFFDSetCheckh_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)mat,"MatMFFDSetPeriod_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)mat,"MatMFFDResetHHistory_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)mat,"MatMFFDAddNullSpace_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatView_MFFD"
 /*
    MatMFFDView_MFFD - Views matrix-free parameters.
 
 */
-PetscErrorCode MatView_MFFD(Mat J,PetscViewer viewer)
+static PetscErrorCode MatView_MFFD(Mat J,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   MatMFFD        ctx = (MatMFFD)J->data;
@@ -271,7 +247,7 @@ PetscErrorCode MatView_MFFD(Mat J,PetscViewer viewer)
   if (iascii) {
     ierr = PetscViewerASCIIPrintf(viewer,"Matrix-free approximation:\n");CHKERRQ(ierr);
     ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"err=%G (relative error in function evaluation)\n",ctx->error_rel);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"err=%g (relative error in function evaluation)\n",(double)ctx->error_rel);CHKERRQ(ierr);
     if (!((PetscObject)ctx)->type_name) {
       ierr = PetscViewerASCIIPrintf(viewer,"The compute h routine has not yet been set\n");CHKERRQ(ierr);
     } else {
@@ -282,12 +258,12 @@ PetscErrorCode MatView_MFFD(Mat J,PetscViewer viewer)
     }
     ierr = PetscObjectGetOptionsPrefix((PetscObject)J, &prefix);CHKERRQ(ierr);
 
-    ierr = PetscOptionsHasName(prefix, "-mat_mffd_view_base", &viewbase);CHKERRQ(ierr);
+    ierr = PetscOptionsHasName(((PetscObject)J)->options,prefix, "-mat_mffd_view_base", &viewbase);CHKERRQ(ierr);
     if (viewbase) {
       ierr = PetscViewerASCIIPrintf(viewer, "Base:\n");CHKERRQ(ierr);
       ierr = VecView(ctx->current_u, viewer);CHKERRQ(ierr);
     }
-    ierr = PetscOptionsHasName(prefix, "-mat_mffd_view_function", &viewfunction);CHKERRQ(ierr);
+    ierr = PetscOptionsHasName(((PetscObject)J)->options,prefix, "-mat_mffd_view_function", &viewfunction);CHKERRQ(ierr);
     if (viewfunction) {
       ierr = PetscViewerASCIIPrintf(viewer, "Function:\n");CHKERRQ(ierr);
       ierr = VecView(ctx->current_f, viewer);CHKERRQ(ierr);
@@ -297,8 +273,6 @@ PetscErrorCode MatView_MFFD(Mat J,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatAssemblyEnd_MFFD"
 /*
    MatAssemblyEnd_MFFD - Resets the ctx->ncurrenth to zero. This
    allows the user to indicate the beginning of a new linear solve by calling
@@ -306,7 +280,8 @@ PetscErrorCode MatView_MFFD(Mat J,PetscViewer viewer)
    MatCreateMFFD_WP() to properly compute ||U|| only the first time
    in the linear solver rather than every time.
 
-   This function is referenced directly from MatAssemblyEnd_SNESMF(), which may be in a different shared library.
+   This function is referenced directly from MatAssemblyEnd_SNESMF(), which may be in a different shared library hence
+   it must be labeled as PETSC_EXTERN
 */
 PETSC_EXTERN PetscErrorCode MatAssemblyEnd_MFFD(Mat J,MatAssemblyType mt)
 {
@@ -320,8 +295,6 @@ PETSC_EXTERN PetscErrorCode MatAssemblyEnd_MFFD(Mat J,MatAssemblyType mt)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_MFFD"
 /*
   MatMult_MFFD - Default matrix-free form for Jacobian-vector product, y = F'(u)*a:
 
@@ -330,7 +303,7 @@ PETSC_EXTERN PetscErrorCode MatAssemblyEnd_MFFD(Mat J,MatAssemblyType mt)
         u = current iterate
         h = difference interval
 */
-PetscErrorCode MatMult_MFFD(Mat mat,Vec a,Vec y)
+static PetscErrorCode MatMult_MFFD(Mat mat,Vec a,Vec y)
 {
   MatMFFD        ctx = (MatMFFD)mat->data;
   PetscScalar    h;
@@ -352,7 +325,7 @@ PetscErrorCode MatMult_MFFD(Mat mat,Vec a,Vec y)
   /*
       Compute differencing parameter
   */
-  if (!ctx->ops->compute) {
+  if (!((PetscObject)ctx)->type_name) {
     ierr = MatMFFDSetType(mat,MATMFFD_WP);CHKERRQ(ierr);
     ierr = MatSetFromOptions(mat);CHKERRQ(ierr);
   }
@@ -362,7 +335,7 @@ PetscErrorCode MatMult_MFFD(Mat mat,Vec a,Vec y)
     PetscFunctionReturn(0);
   }
 
-  if (PetscIsInfOrNanScalar(h)) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Computed Nan differencing parameter h");
+  if (mat->erroriffailure && PetscIsInfOrNanScalar(h)) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Computed Nan differencing parameter h");
   if (ctx->checkh) {
     ierr = (*ctx->checkh)(ctx->checkhctx,U,a,&h);CHKERRQ(ierr);
   }
@@ -370,7 +343,7 @@ PetscErrorCode MatMult_MFFD(Mat mat,Vec a,Vec y)
   /* keep a record of the current differencing parameter h */
   ctx->currenth = h;
 #if defined(PETSC_USE_COMPLEX)
-  ierr = PetscInfo2(mat,"Current differencing parameter: %G + %G i\n",PetscRealPart(h),PetscImaginaryPart(h));CHKERRQ(ierr);
+  ierr = PetscInfo2(mat,"Current differencing parameter: %g + %g i\n",(double)PetscRealPart(h),(double)PetscImaginaryPart(h));CHKERRQ(ierr);
 #else
   ierr = PetscInfo1(mat,"Current differencing parameter: %15.12e\n",h);CHKERRQ(ierr);
 #endif
@@ -403,18 +376,19 @@ PetscErrorCode MatMult_MFFD(Mat mat,Vec a,Vec y)
     ierr = VecPointwiseMult(y,ctx->dlscale,y);CHKERRQ(ierr);
   }
   if (ctx->dshift) {
-    ierr = VecPointwiseMult(ctx->dshift,a,U);CHKERRQ(ierr);
-    ierr = VecAXPY(y,1.0,U);CHKERRQ(ierr);
+    if (!ctx->dshiftw) {
+      ierr = VecDuplicate(y,&ctx->dshiftw);CHKERRQ(ierr);
+    }
+    ierr = VecPointwiseMult(ctx->dshift,a,ctx->dshiftw);CHKERRQ(ierr);
+    ierr = VecAXPY(y,1.0,ctx->dshiftw);CHKERRQ(ierr);
   }
 
-  if (ctx->sp) {ierr = MatNullSpaceRemove(ctx->sp,y,NULL);CHKERRQ(ierr);}
+  if (mat->nullsp) {ierr = MatNullSpaceRemove(mat->nullsp,y);CHKERRQ(ierr);}
 
   ierr = PetscLogEventEnd(MATMFFD_Mult,a,y,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatGetDiagonal_MFFD"
 /*
   MatGetDiagonal_MFFD - Gets the diagonal for a matrix free matrix
 
@@ -433,8 +407,9 @@ PetscErrorCode MatGetDiagonal_MFFD(Mat mat,Vec a)
   PetscInt       i,rstart,rend;
 
   PetscFunctionBegin;
-  if (!ctx->funci) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Requires calling MatMFFDSetFunctioni() first");
-
+  if (!ctx->func) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Requires calling MatMFFDSetFunction() first");
+  if (!ctx->funci) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Requires calling MatMFFDSetFunctioni() first");
+  if (!ctx->funcisetbase) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Requires calling MatMFFDSetFunctioniBase() first");
   w    = ctx->w;
   U    = ctx->current_u;
   ierr = (*ctx->func)(ctx->funcctx,U,a);CHKERRQ(ierr);
@@ -469,9 +444,7 @@ PetscErrorCode MatGetDiagonal_MFFD(Mat mat,Vec a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatDiagonalScale_MFFD"
-PetscErrorCode MatDiagonalScale_MFFD(Mat mat,Vec ll,Vec rr)
+static PetscErrorCode MatDiagonalScale_MFFD(Mat mat,Vec ll,Vec rr)
 {
   MatMFFD        aij = (MatMFFD)mat->data;
   PetscErrorCode ierr;
@@ -492,9 +465,7 @@ PetscErrorCode MatDiagonalScale_MFFD(Mat mat,Vec ll,Vec rr)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatDiagonalSet_MFFD"
-PetscErrorCode MatDiagonalSet_MFFD(Mat mat,Vec ll,InsertMode mode)
+static PetscErrorCode MatDiagonalSet_MFFD(Mat mat,Vec ll,InsertMode mode)
 {
   MatMFFD        aij = (MatMFFD)mat->data;
   PetscErrorCode ierr;
@@ -508,9 +479,7 @@ PetscErrorCode MatDiagonalSet_MFFD(Mat mat,Vec ll,InsertMode mode)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatShift_MFFD"
-PetscErrorCode MatShift_MFFD(Mat Y,PetscScalar a)
+static PetscErrorCode MatShift_MFFD(Mat Y,PetscScalar a)
 {
   MatMFFD shell = (MatMFFD)Y->data;
 
@@ -519,9 +488,7 @@ PetscErrorCode MatShift_MFFD(Mat Y,PetscScalar a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatScale_MFFD"
-PetscErrorCode MatScale_MFFD(Mat Y,PetscScalar a)
+static PetscErrorCode MatScale_MFFD(Mat Y,PetscScalar a)
 {
   MatMFFD shell = (MatMFFD)Y->data;
 
@@ -530,8 +497,6 @@ PetscErrorCode MatScale_MFFD(Mat Y,PetscScalar a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetBase_MFFD"
 PETSC_EXTERN PetscErrorCode MatMFFDSetBase_MFFD(Mat J,Vec U,Vec F)
 {
   PetscErrorCode ierr;
@@ -539,19 +504,24 @@ PETSC_EXTERN PetscErrorCode MatMFFDSetBase_MFFD(Mat J,Vec U,Vec F)
 
   PetscFunctionBegin;
   ierr = MatMFFDResetHHistory(J);CHKERRQ(ierr);
-
-  ctx->current_u = U;
+  if (!ctx->current_u) {
+    ierr = VecDuplicate(U,&ctx->current_u);CHKERRQ(ierr);
+    ierr = VecLockPush(ctx->current_u);CHKERRQ(ierr);
+  }
+  ierr = VecLockPop(ctx->current_u);CHKERRQ(ierr);
+  ierr = VecCopy(U,ctx->current_u);CHKERRQ(ierr);
+  ierr = VecLockPush(ctx->current_u);CHKERRQ(ierr);
   if (F) {
     if (ctx->current_f_allocated) {ierr = VecDestroy(&ctx->current_f);CHKERRQ(ierr);}
     ctx->current_f           = F;
     ctx->current_f_allocated = PETSC_FALSE;
   } else if (!ctx->current_f_allocated) {
-    ierr = VecDuplicate(ctx->current_u, &ctx->current_f);CHKERRQ(ierr);
+    ierr = MatCreateVecs(J,NULL,&ctx->current_f);CHKERRQ(ierr);
 
     ctx->current_f_allocated = PETSC_TRUE;
   }
   if (!ctx->w) {
-    ierr = VecDuplicate(ctx->current_u, &ctx->w);CHKERRQ(ierr);
+    ierr = VecDuplicate(ctx->current_u,&ctx->w);CHKERRQ(ierr);
   }
   J->assembled = PETSC_TRUE;
   PetscFunctionReturn(0);
@@ -559,9 +529,7 @@ PETSC_EXTERN PetscErrorCode MatMFFDSetBase_MFFD(Mat J,Vec U,Vec F)
 
 typedef PetscErrorCode (*FCN3)(void*,Vec,Vec,PetscScalar*); /* force argument to next function to not be extern C*/
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetCheckh_MFFD"
-PetscErrorCode  MatMFFDSetCheckh_MFFD(Mat J,FCN3 fun,void *ectx)
+static PetscErrorCode  MatMFFDSetCheckh_MFFD(Mat J,FCN3 fun,void *ectx)
 {
   MatMFFD ctx = (MatMFFD)J->data;
 
@@ -571,8 +539,6 @@ PetscErrorCode  MatMFFDSetCheckh_MFFD(Mat J,FCN3 fun,void *ectx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetOptionsPrefix"
 /*@C
    MatMFFDSetOptionsPrefix - Sets the prefix used for searching for all
    MatMFFD options in the database.
@@ -591,7 +557,7 @@ PetscErrorCode  MatMFFDSetCheckh_MFFD(Mat J,FCN3 fun,void *ectx)
 
 .keywords: SNES, matrix-free, parameters
 
-.seealso: MatSetFromOptions(), MatCreateSNESMF()
+.seealso: MatSetFromOptions(), MatCreateSNESMF(), MatCreateMFFD()
 @*/
 PetscErrorCode  MatMFFDSetOptionsPrefix(Mat mat,const char prefix[])
 
@@ -606,9 +572,7 @@ PetscErrorCode  MatMFFDSetOptionsPrefix(Mat mat,const char prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSetFromOptions_MFFD"
-PetscErrorCode  MatSetFromOptions_MFFD(Mat mat)
+static PetscErrorCode  MatSetFromOptions_MFFD(PetscOptionItems *PetscOptionsObject,Mat mat)
 {
   MatMFFD        mfctx = (MatMFFD)mat->data;
   PetscErrorCode ierr;
@@ -619,7 +583,7 @@ PetscErrorCode  MatSetFromOptions_MFFD(Mat mat)
   PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
   PetscValidHeaderSpecific(mfctx,MATMFFD_CLASSID,1);
   ierr = PetscObjectOptionsBegin((PetscObject)mfctx);CHKERRQ(ierr);
-  ierr = PetscOptionsList("-mat_mffd_type","Matrix free type","MatMFFDSetType",MatMFFDList,((PetscObject)mfctx)->type_name,ftype,256,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsFList("-mat_mffd_type","Matrix free type","MatMFFDSetType",MatMFFDList,((PetscObject)mfctx)->type_name,ftype,256,&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = MatMFFDSetType(mat,ftype);CHKERRQ(ierr);
   }
@@ -633,27 +597,22 @@ PetscErrorCode  MatSetFromOptions_MFFD(Mat mat)
     ierr = MatMFFDSetCheckh(mat,MatMFFDCheckPositivity,0);CHKERRQ(ierr);
   }
   if (mfctx->ops->setfromoptions) {
-    ierr = (*mfctx->ops->setfromoptions)(mfctx);CHKERRQ(ierr);
+    ierr = (*mfctx->ops->setfromoptions)(PetscOptionsObject,mfctx);CHKERRQ(ierr);
   }
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetPeriod_MFFD"
-PetscErrorCode  MatMFFDSetPeriod_MFFD(Mat mat,PetscInt period)
+static PetscErrorCode  MatMFFDSetPeriod_MFFD(Mat mat,PetscInt period)
 {
   MatMFFD ctx = (MatMFFD)mat->data;
 
   PetscFunctionBegin;
-  PetscValidLogicalCollectiveInt(mat,period,2);
   ctx->recomputeperiod = period;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetFunction_MFFD"
-PetscErrorCode  MatMFFDSetFunction_MFFD(Mat mat,PetscErrorCode (*func)(void*,Vec,Vec),void *funcctx)
+static PetscErrorCode  MatMFFDSetFunction_MFFD(Mat mat,PetscErrorCode (*func)(void*,Vec,Vec),void *funcctx)
 {
   MatMFFD ctx = (MatMFFD)mat->data;
 
@@ -663,40 +622,42 @@ PetscErrorCode  MatMFFDSetFunction_MFFD(Mat mat,PetscErrorCode (*func)(void*,Vec
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetFunctionError_MFFD"
-PetscErrorCode  MatMFFDSetFunctionError_MFFD(Mat mat,PetscReal error)
+static PetscErrorCode  MatMFFDSetFunctionError_MFFD(Mat mat,PetscReal error)
 {
   MatMFFD ctx = (MatMFFD)mat->data;
 
   PetscFunctionBegin;
-  PetscValidLogicalCollectiveReal(mat,error,2);
   if (error != PETSC_DEFAULT) ctx->error_rel = error;
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode MatMissingDiagonal_MFFD(Mat A,PetscBool  *missing,PetscInt *d)
+{
+  PetscFunctionBegin;
+  *missing = PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
+
 /*MC
   MATMFFD - MATMFFD = "mffd" - A matrix free matrix type.
 
   Level: advanced
 
-.seealso: MatCreateMFFD(), MatCreateSNESMF(), MatMFFDSetFunction()
+.seealso: MatCreateMFFD(), MatCreateSNESMF(), MatMFFDSetFunction(), MatMFFDSetType(),  
+          MatMFFDSetFunctionError(), MatMFFDDSSetUmin(), MatMFFDSetFunction()
+          MatMFFDSetHHistory(), MatMFFDResetHHistory(), MatCreateSNESMF(),
+          MatMFFDGetH(),
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "MatCreate_MFFD"
 PETSC_EXTERN PetscErrorCode MatCreate_MFFD(Mat A)
 {
   MatMFFD        mfctx;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-#if !defined(PETSC_USE_DYNAMIC_LIBRARIES)
   ierr = MatMFFDInitializePackage();CHKERRQ(ierr);
-#endif
 
-  ierr = PetscHeaderCreate(mfctx,_p_MatMFFD,struct _MFOps,MATMFFD_CLASSID,"MatMFFD","Matrix-free Finite Differencing","Mat",PetscObjectComm((PetscObject)A),MatDestroy_MFFD,MatView_MFFD);CHKERRQ(ierr);
+  ierr = PetscHeaderCreate(mfctx,MATMFFD_CLASSID,"MatMFFD","Matrix-free Finite Differencing","Mat",PetscObjectComm((PetscObject)A),MatDestroy_MFFD,MatView_MFFD);CHKERRQ(ierr);
 
-  mfctx->sp                       = 0;
   mfctx->error_rel                = PETSC_SQRT_MACHINE_EPSILON;
   mfctx->recomputeperiod          = 1;
   mfctx->count                    = 0;
@@ -727,20 +688,17 @@ PETSC_EXTERN PetscErrorCode MatCreate_MFFD(Mat A)
 
   A->data = mfctx;
 
-  A->ops->mult           = MatMult_MFFD;
-  A->ops->destroy        = MatDestroy_MFFD;
-  A->ops->view           = MatView_MFFD;
-  A->ops->assemblyend    = MatAssemblyEnd_MFFD;
-  A->ops->getdiagonal    = MatGetDiagonal_MFFD;
-  A->ops->scale          = MatScale_MFFD;
-  A->ops->shift          = MatShift_MFFD;
-  A->ops->diagonalscale  = MatDiagonalScale_MFFD;
-  A->ops->diagonalset    = MatDiagonalSet_MFFD;
-  A->ops->setfromoptions = MatSetFromOptions_MFFD;
-  A->assembled           = PETSC_TRUE;
-
-  ierr = PetscLayoutSetUp(A->rmap);CHKERRQ(ierr);
-  ierr = PetscLayoutSetUp(A->cmap);CHKERRQ(ierr);
+  A->ops->mult            = MatMult_MFFD;
+  A->ops->destroy         = MatDestroy_MFFD;
+  A->ops->view            = MatView_MFFD;
+  A->ops->assemblyend     = MatAssemblyEnd_MFFD;
+  A->ops->scale           = MatScale_MFFD;
+  A->ops->shift           = MatShift_MFFD;
+  A->ops->diagonalscale   = MatDiagonalScale_MFFD;
+  A->ops->diagonalset     = MatDiagonalSet_MFFD;
+  A->ops->setfromoptions  = MatSetFromOptions_MFFD;
+  A->ops->missingdiagonal = MatMissingDiagonal_MFFD;
+  A->assembled            = PETSC_TRUE;
 
   ierr = PetscObjectComposeFunction((PetscObject)A,"MatMFFDSetBase_C",MatMFFDSetBase_MFFD);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)A,"MatMFFDSetFunctioniBase_C",MatMFFDSetFunctioniBase_MFFD);CHKERRQ(ierr);
@@ -750,7 +708,6 @@ PETSC_EXTERN PetscErrorCode MatCreate_MFFD(Mat A)
   ierr = PetscObjectComposeFunction((PetscObject)A,"MatMFFDSetPeriod_C",MatMFFDSetPeriod_MFFD);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)A,"MatMFFDSetFunctionError_C",MatMFFDSetFunctionError_MFFD);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)A,"MatMFFDResetHHistory_C",MatMFFDResetHHistory_MFFD);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)A,"MatMFFDAddNullSpace_C",MatMFFDAddNullSpace_MFFD);CHKERRQ(ierr);
 
   mfctx->mat = A;
 
@@ -758,8 +715,6 @@ PETSC_EXTERN PetscErrorCode MatCreate_MFFD(Mat A)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCreateMFFD"
 /*@
    MatCreateMFFD - Creates a matrix-free matrix. See also MatCreateSNESMF()
 
@@ -804,8 +759,11 @@ PETSC_EXTERN PetscErrorCode MatCreate_MFFD(Mat A)
      umin = minimum iterate parameter
 .ve
 
+   You can call SNESSetJacobian() with MatMFFDComputeJacobian() if you are using matrix and not a different
+   preconditioner matrix
+
    The user can set the error_rel via MatMFFDSetFunctionError() and
-   umin via MatMFFDDSSetUmin(); see the <A href="../../docs/manual.pdf#nameddest=ch_snes">SNES chapter of the users manual</A> for details.
+   umin via MatMFFDDSSetUmin(); see Users-Manual: ch_snes for details.
 
    The user should call MatDestroy() when finished with the matrix-free
    matrix context.
@@ -834,9 +792,6 @@ PetscErrorCode  MatCreateMFFD(MPI_Comm comm,PetscInt m,PetscInt n,PetscInt M,Pet
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDGetH"
 /*@
    MatMFFDGetH - Gets the last value that was used as the differencing
    parameter.
@@ -862,6 +817,8 @@ PetscErrorCode  MatMFFDGetH(Mat mat,PetscScalar *h)
   PetscBool      match;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
+  PetscValidPointer(h,2);
   ierr = PetscObjectTypeCompare((PetscObject)mat,MATMFFD,&match);CHKERRQ(ierr);
   if (!match) SETERRQ(PetscObjectComm((PetscObject)mat),PETSC_ERR_ARG_WRONG,"Not a MFFD matrix");
 
@@ -869,18 +826,23 @@ PetscErrorCode  MatMFFDGetH(Mat mat,PetscScalar *h)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetFunction"
 /*@C
    MatMFFDSetFunction - Sets the function used in applying the matrix free.
 
    Logically Collective on Mat
 
    Input Parameters:
-+  mat - the matrix free matrix created via MatCreateSNESMF()
++  mat - the matrix free matrix created via MatCreateSNESMF() or MatCreateMFFD()
 .  func - the function to use
 -  funcctx - optional function context passed to function
 
+   Calling Sequence of func:
+$     func (void *funcctx, Vec x, Vec f)
+
++  funcctx - user provided context
+.  x - input vector
+-  f - computed output function
+
    Level: advanced
 
    Notes:
@@ -899,12 +861,11 @@ PetscErrorCode  MatMFFDSetFunction(Mat mat,PetscErrorCode (*func)(void*,Vec,Vec)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
   ierr = PetscTryMethod(mat,"MatMFFDSetFunction_C",(Mat,PetscErrorCode (*)(void*,Vec,Vec),void*),(mat,func,funcctx));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetFunctioni"
 /*@C
    MatMFFDSetFunctioni - Sets the function for a single component
 
@@ -918,12 +879,13 @@ PetscErrorCode  MatMFFDSetFunction(Mat mat,PetscErrorCode (*func)(void*,Vec,Vec)
 
    Notes:
     If you use this you MUST call MatAssemblyBegin()/MatAssemblyEnd() on the matrix free
-    matrix inside your compute Jacobian routine
-
+    matrix inside your compute Jacobian routine.
+    This function is necessary to compute the diagonal of the matrix.
+    funci must not contain any MPI call as it is called inside a loop on the local portion of the vector.
 
 .keywords: SNES, matrix-free, function
 
-.seealso: MatCreateSNESMF(),MatMFFDGetH(), MatMFFDSetHHistory(), MatMFFDResetHHistory(), SNESetFunction()
+.seealso: MatCreateSNESMF(),MatMFFDGetH(), MatMFFDSetHHistory(), MatMFFDResetHHistory(), SNESetFunction(), MatGetDiagonal()
 
 @*/
 PetscErrorCode  MatMFFDSetFunctioni(Mat mat,PetscErrorCode (*funci)(void*,PetscInt,Vec,PetscScalar*))
@@ -936,9 +898,6 @@ PetscErrorCode  MatMFFDSetFunctioni(Mat mat,PetscErrorCode (*funci)(void*,PetscI
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetFunctioniBase"
 /*@C
    MatMFFDSetFunctioniBase - Sets the base vector for a single component function evaluation
 
@@ -952,13 +911,14 @@ PetscErrorCode  MatMFFDSetFunctioni(Mat mat,PetscErrorCode (*funci)(void*,PetscI
 
    Notes:
     If you use this you MUST call MatAssemblyBegin()/MatAssemblyEnd() on the matrix free
-    matrix inside your compute Jacobian routine
+    matrix inside your compute Jacobian routine.
+    This function is necessary to compute the diagonal of the matrix.
 
 
 .keywords: SNES, matrix-free, function
 
 .seealso: MatCreateSNESMF(),MatMFFDGetH(), MatCreateMFFD(), MATMFFD
-          MatMFFDSetHHistory(), MatMFFDResetHHistory(), SNESetFunction()
+          MatMFFDSetHHistory(), MatMFFDResetHHistory(), SNESetFunction(), MatGetDiagonal()
 @*/
 PetscErrorCode  MatMFFDSetFunctioniBase(Mat mat,PetscErrorCode (*func)(void*,Vec))
 {
@@ -970,8 +930,6 @@ PetscErrorCode  MatMFFDSetFunctioniBase(Mat mat,PetscErrorCode (*func)(void*,Vec
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetPeriod"
 /*@
    MatMFFDSetPeriod - Sets how often h is recomputed, by default it is everytime
 
@@ -997,12 +955,12 @@ PetscErrorCode  MatMFFDSetPeriod(Mat mat,PetscInt period)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
+  PetscValidLogicalCollectiveInt(mat,period,2);
   ierr = PetscTryMethod(mat,"MatMFFDSetPeriod_C",(Mat,PetscInt),(mat,period));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetFunctionError"
 /*@
    MatMFFDSetFunctionError - Sets the error_rel for the approximation of
    matrix-vector products using finite differences.
@@ -1037,42 +995,12 @@ PetscErrorCode  MatMFFDSetFunctionError(Mat mat,PetscReal error)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
+  PetscValidLogicalCollectiveReal(mat,error,2);
   ierr = PetscTryMethod(mat,"MatMFFDSetFunctionError_C",(Mat,PetscReal),(mat,error));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDAddNullSpace"
-/*@
-   MatMFFDAddNullSpace - Provides a null space that an operator is
-   supposed to have.  Since roundoff will create a small component in
-   the null space, if you know the null space you may have it
-   automatically removed.
-
-   Logically Collective on Mat
-
-   Input Parameters:
-+  J - the matrix-free matrix context
--  nullsp - object created with MatNullSpaceCreate()
-
-   Level: advanced
-
-.keywords: SNES, matrix-free, null space
-
-.seealso: MatNullSpaceCreate(), MatMFFDGetH(), MatCreateSNESMF(), MatCreateMFFD(), MATMFFD
-          MatMFFDSetHHistory(), MatMFFDResetHHistory()
-@*/
-PetscErrorCode  MatMFFDAddNullSpace(Mat J,MatNullSpace nullsp)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  ierr = PetscTryMethod(J,"MatMFFDAddNullSpace_C",(Mat,MatNullSpace),(J,nullsp));CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetHHistory"
 /*@
    MatMFFDSetHHistory - Sets an array to collect a history of the
    differencing values (h) computed for the matrix-free product.
@@ -1104,6 +1032,9 @@ PetscErrorCode  MatMFFDSetHHistory(Mat J,PetscScalar history[],PetscInt nhistory
   PetscBool      match;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(J,MAT_CLASSID,1);
+  if (history) PetscValidPointer(history,2);
+  PetscValidLogicalCollectiveInt(J,nhistory,3);
   ierr = PetscObjectTypeCompare((PetscObject)J,MATMFFD,&match);CHKERRQ(ierr);
   if (!match) SETERRQ(PetscObjectComm((PetscObject)J),PETSC_ERR_ARG_WRONG,"Not a MFFD matrix");
   ctx->historyh    = history;
@@ -1112,9 +1043,6 @@ PetscErrorCode  MatMFFDSetHHistory(Mat J,PetscScalar history[],PetscInt nhistory
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDResetHHistory"
 /*@
    MatMFFDResetHHistory - Resets the counter to zero to begin
    collecting a new set of differencing histories.
@@ -1140,13 +1068,11 @@ PetscErrorCode  MatMFFDResetHHistory(Mat J)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(J,MAT_CLASSID,1);
   ierr = PetscTryMethod(J,"MatMFFDResetHHistory_C",(Mat),(J));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetBase"
 /*@
     MatMFFDSetBase - Sets the vector U at which matrix vector products of the
         Jacobian are computed
@@ -1158,7 +1084,8 @@ PetscErrorCode  MatMFFDResetHHistory(Mat J)
 .   U - the vector
 -   F - (optional) vector that contains F(u) if it has been already computed
 
-    Notes: This is rarely used directly
+    Notes:
+    This is rarely used directly
 
     If F is provided then it is not recomputed. Otherwise the function is evaluated at the base
     point during the first MatMult() after each call to MatMFFDSetBase().
@@ -1178,8 +1105,6 @@ PetscErrorCode  MatMFFDSetBase(Mat J,Vec U,Vec F)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetCheckh"
 /*@C
     MatMFFDSetCheckh - Sets a function that checks the computed h and adjusts
         it to satisfy some criteria
@@ -1196,10 +1121,14 @@ PetscErrorCode  MatMFFDSetBase(Mat J,Vec U,Vec F)
 
     Level: advanced
 
-    Notes: For example, MatMFFDSetCheckPositivity() insures that all entries
+    Notes:
+    For example, MatMFFDCheckPositivity() insures that all entries
        of U + h*a are non-negative
 
-.seealso:  MatMFFDSetCheckPositivity()
+     The function you provide is called after the default h has been computed and allows you to
+     modify it.
+
+.seealso:  MatMFFDCheckPositivity()
 @*/
 PetscErrorCode  MatMFFDSetCheckh(Mat J,PetscErrorCode (*fun)(void*,Vec,Vec,PetscScalar*),void *ctx)
 {
@@ -1211,8 +1140,6 @@ PetscErrorCode  MatMFFDSetCheckh(Mat J,PetscErrorCode (*fun)(void*,Vec,Vec,Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMFFDSetCheckPositivity"
 /*@
     MatMFFDCheckPositivity - Checks that all entries in U + h*a are positive or
         zero, decreases h until this is satisfied.
@@ -1230,7 +1157,8 @@ PetscErrorCode  MatMFFDSetCheckh(Mat J,PetscErrorCode (*fun)(void*,Vec,Vec,Petsc
 
     Level: advanced
 
-    Notes: This is rarely used directly, rather it is passed as an argument to
+    Notes:
+    This is rarely used directly, rather it is passed as an argument to
            MatMFFDSetCheckh()
 
 .seealso:  MatMFFDSetCheckh()
@@ -1244,11 +1172,14 @@ PetscErrorCode  MatMFFDCheckPositivity(void *dummy,Vec U,Vec a,PetscScalar *h)
   MPI_Comm       comm;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(U,VEC_CLASSID,2);
+  PetscValidHeaderSpecific(a,VEC_CLASSID,3);
+  PetscValidPointer(h,4);
   ierr   = PetscObjectGetComm((PetscObject)U,&comm);CHKERRQ(ierr);
   ierr   = VecGetArray(U,&u_vec);CHKERRQ(ierr);
   ierr   = VecGetArray(a,&a_vec);CHKERRQ(ierr);
   ierr   = VecGetLocalSize(U,&n);CHKERRQ(ierr);
-  minval = PetscAbsScalar(*h*1.01);
+  minval = PetscAbsScalar(*h)*PetscRealConstant(1.01);
   for (i=0; i<n; i++) {
     if (PetscRealPart(u_vec[i] + *h*a_vec[i]) <= 0.0) {
       val = PetscAbsScalar(u_vec[i]/a_vec[i]);
@@ -1257,18 +1188,11 @@ PetscErrorCode  MatMFFDCheckPositivity(void *dummy,Vec U,Vec a,PetscScalar *h)
   }
   ierr = VecRestoreArray(U,&u_vec);CHKERRQ(ierr);
   ierr = VecRestoreArray(a,&a_vec);CHKERRQ(ierr);
-  ierr = MPI_Allreduce(&minval,&val,1,MPIU_REAL,MPIU_MIN,comm);CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(&minval,&val,1,MPIU_REAL,MPIU_MIN,comm);CHKERRQ(ierr);
   if (val <= PetscAbsScalar(*h)) {
-    ierr = PetscInfo2(U,"Scaling back h from %G to %G\n",PetscRealPart(*h),.99*val);CHKERRQ(ierr);
+    ierr = PetscInfo2(U,"Scaling back h from %g to %g\n",(double)PetscRealPart(*h),(double)(.99*val));CHKERRQ(ierr);
     if (PetscRealPart(*h) > 0.0) *h =  0.99*val;
     else                         *h = -0.99*val;
   }
   PetscFunctionReturn(0);
 }
-
-
-
-
-
-
-
