@@ -4,11 +4,6 @@
 */
 #include <../src/mat/impls/sbaij/mpi/mpisbaij.h>
 
-extern PetscErrorCode MatSetValues_SeqSBAIJ(Mat,PetscInt,const PetscInt [],PetscInt,const PetscInt [],const PetscScalar [],InsertMode);
-
-
-#undef __FUNCT__
-#define __FUNCT__ "MatSetUpMultiply_MPISBAIJ"
 PetscErrorCode MatSetUpMultiply_MPISBAIJ(Mat mat)
 {
   Mat_MPISBAIJ   *sbaij = (Mat_MPISBAIJ*)mat->data;
@@ -147,8 +142,6 @@ PetscErrorCode MatSetUpMultiply_MPISBAIJ(Mat mat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSetUpMultiply_MPISBAIJ_2comm"
 PetscErrorCode MatSetUpMultiply_MPISBAIJ_2comm(Mat mat)
 {
   Mat_MPISBAIJ   *baij = (Mat_MPISBAIJ*)mat->data;
@@ -281,8 +274,6 @@ PetscErrorCode MatSetUpMultiply_MPISBAIJ_2comm(Mat mat)
    Kind of slow! But that's what application programmers get when
    they are sloppy.
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatDisAssemble_MPISBAIJ"
 PetscErrorCode MatDisAssemble_MPISBAIJ(Mat A)
 {
   Mat_MPISBAIJ   *baij  = (Mat_MPISBAIJ*)A->data;
@@ -336,7 +327,10 @@ PetscErrorCode MatDisAssemble_MPISBAIJ(Mat A)
   ierr = MatSeqBAIJSetPreallocation(Bnew,B->rmap->bs,0,nz);CHKERRQ(ierr);
   ierr = PetscFree(nz);CHKERRQ(ierr);
 
-  ((Mat_SeqSBAIJ*)Bnew->data)->nonew = Bbaij->nonew; /* Inherit insertion error options. */
+  if (Bbaij->nonew >= 0) { /* Inherit insertion error options (if positive). */
+    ((Mat_SeqSBAIJ*)Bnew->data)->nonew = Bbaij->nonew;
+  }
+
   /*
    Ensure that B's nonzerostate is monotonically increasing.
    Or should this follow the MatSetValues() loop to preserve B's nonzerstate across a MatDisAssemble() call?
