@@ -18,8 +18,6 @@
 */
 static Mat static_F;
 
-#undef __FUNCT__
-#define __FUNCT__ "CholmodErrorHandler"
 static void CholmodErrorHandler(int status,const char *file,int line,const char *message)
 {
   PetscErrorCode ierr;
@@ -35,12 +33,10 @@ static void CholmodErrorHandler(int status,const char *file,int line,const char
   PetscFunctionReturnVoid();
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CholmodStart"
 PetscErrorCode  CholmodStart(Mat F)
 {
   PetscErrorCode ierr;
-  Mat_CHOLMOD    *chol=(Mat_CHOLMOD*)F->spptr;
+  Mat_CHOLMOD    *chol=(Mat_CHOLMOD*)F->data;
   cholmod_common *c;
   PetscBool      flg;
 
@@ -127,8 +123,6 @@ PetscErrorCode  CholmodStart(Mat F)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatWrapCholmod_seqsbaij"
 static PetscErrorCode MatWrapCholmod_seqsbaij(Mat A,PetscBool values,cholmod_sparse *C,PetscBool  *aijalloc)
 {
   Mat_SeqSBAIJ   *sbaij = (Mat_SeqSBAIJ*)A->data;
@@ -153,8 +147,6 @@ static PetscErrorCode MatWrapCholmod_seqsbaij(Mat A,PetscBool values,cholmod_spa
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecWrapCholmodRead"
 static PetscErrorCode VecWrapCholmodRead(Vec X,cholmod_dense *Y)
 {
   PetscErrorCode    ierr;
@@ -177,33 +169,26 @@ static PetscErrorCode VecWrapCholmodRead(Vec X,cholmod_dense *Y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecUnWrapCholmodRead"
 static PetscErrorCode VecUnWrapCholmodRead(Vec X,cholmod_dense *Y)
 {
   PetscErrorCode    ierr;
 
   PetscFunctionBegin;
-  ierr = VecRestoreArrayRead(X,PETSC_NULL);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(X,NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatDestroy_CHOLMOD"
-PetscErrorCode  MatDestroy_CHOLMOD(Mat F)
+PETSC_INTERN PetscErrorCode  MatDestroy_CHOLMOD(Mat F)
 {
   PetscErrorCode ierr;
-  Mat_CHOLMOD    *chol=(Mat_CHOLMOD*)F->spptr;
+  Mat_CHOLMOD    *chol=(Mat_CHOLMOD*)F->data;
 
   PetscFunctionBegin;
-  if (chol) {
-    ierr = !cholmod_X_free_factor(&chol->factor,chol->common);CHKERRQ(ierr);
-    ierr = !cholmod_X_finish(chol->common);CHKERRQ(ierr);
-    ierr = PetscFree(chol->common);CHKERRQ(ierr);
-    ierr = PetscFree(chol->matrix);CHKERRQ(ierr);
-    ierr = (*chol->Destroy)(F);CHKERRQ(ierr);
-  }
-  ierr = PetscFree(F->spptr);CHKERRQ(ierr);
+  ierr = !cholmod_X_free_factor(&chol->factor,chol->common);CHKERRQ(ierr);
+  ierr = !cholmod_X_finish(chol->common);CHKERRQ(ierr);
+  ierr = PetscFree(chol->common);CHKERRQ(ierr);
+  ierr = PetscFree(chol->matrix);CHKERRQ(ierr);
+  ierr = PetscFree(F->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -211,11 +196,9 @@ static PetscErrorCode MatSolve_CHOLMOD(Mat,Vec,Vec);
 
 /*static const char *const CholmodOrderingMethods[] = {"User","AMD","METIS","NESDIS(default)","Natural","NESDIS(small=20000)","NESDIS(small=4,no constrained)","NESDIS()"};*/
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFactorInfo_CHOLMOD"
-static PetscErrorCode MatFactorInfo_CHOLMOD(Mat F,PetscViewer viewer)
+static PetscErrorCode MatView_Info_CHOLMOD(Mat F,PetscViewer viewer)
 {
-  Mat_CHOLMOD          *chol = (Mat_CHOLMOD*)F->spptr;
+  Mat_CHOLMOD          *chol = (Mat_CHOLMOD*)F->data;
   const cholmod_common *c    = chol->common;
   PetscErrorCode       ierr;
   PetscInt             i;
@@ -269,31 +252,26 @@ static PetscErrorCode MatFactorInfo_CHOLMOD(Mat F,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatView_CHOLMOD"
-PetscErrorCode  MatView_CHOLMOD(Mat F,PetscViewer viewer)
+PETSC_INTERN PetscErrorCode  MatView_CHOLMOD(Mat F,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
   PetscBool         iascii;
   PetscViewerFormat format;
 
   PetscFunctionBegin;
-  ierr = MatView_SeqSBAIJ(F,viewer);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     if (format == PETSC_VIEWER_ASCII_INFO) {
-      ierr = MatFactorInfo_CHOLMOD(F,viewer);CHKERRQ(ierr);
+      ierr = MatView_Info_CHOLMOD(F,viewer);CHKERRQ(ierr);
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSolve_CHOLMOD"
 static PetscErrorCode MatSolve_CHOLMOD(Mat F,Vec B,Vec X)
 {
-  Mat_CHOLMOD    *chol = (Mat_CHOLMOD*)F->spptr;
+  Mat_CHOLMOD    *chol = (Mat_CHOLMOD*)F->data;
   cholmod_dense  cholB,*cholX;
   PetscScalar    *x;
   PetscErrorCode ierr;
@@ -311,11 +289,9 @@ static PetscErrorCode MatSolve_CHOLMOD(Mat F,Vec B,Vec X)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCholeskyFactorNumeric_CHOLMOD"
 static PetscErrorCode MatCholeskyFactorNumeric_CHOLMOD(Mat F,Mat A,const MatFactorInfo *info)
 {
-  Mat_CHOLMOD    *chol = (Mat_CHOLMOD*)F->spptr;
+  Mat_CHOLMOD    *chol = (Mat_CHOLMOD*)F->data;
   cholmod_sparse cholA;
   PetscBool      aijalloc;
   PetscErrorCode ierr;
@@ -334,11 +310,9 @@ static PetscErrorCode MatCholeskyFactorNumeric_CHOLMOD(Mat F,Mat A,const MatFact
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCholeskyFactorSymbolic_CHOLMOD"
-PetscErrorCode  MatCholeskyFactorSymbolic_CHOLMOD(Mat F,Mat A,IS perm,const MatFactorInfo *info)
+PETSC_INTERN PetscErrorCode  MatCholeskyFactorSymbolic_CHOLMOD(Mat F,Mat A,IS perm,const MatFactorInfo *info)
 {
-  Mat_CHOLMOD    *chol = (Mat_CHOLMOD*)F->spptr;
+  Mat_CHOLMOD    *chol = (Mat_CHOLMOD*)F->data;
   PetscErrorCode ierr;
   cholmod_sparse cholA;
   PetscBool      aijalloc;
@@ -368,9 +342,7 @@ PetscErrorCode  MatCholeskyFactorSymbolic_CHOLMOD(Mat F,Mat A,IS perm,const MatF
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFactorGetSolverPackage_seqsbaij_cholmod"
-PetscErrorCode MatFactorGetSolverPackage_seqsbaij_cholmod(Mat A,const MatSolverPackage *type)
+static PetscErrorCode MatFactorGetSolverType_seqsbaij_cholmod(Mat A,MatSolverType *type)
 {
   PetscFunctionBegin;
   *type = MATSOLVERCHOLMOD;
@@ -383,7 +355,7 @@ PetscErrorCode MatFactorGetSolverPackage_seqsbaij_cholmod(Mat A,const MatSolverP
 
   Use ./configure --download-suitesparse to install PETSc to use CHOLMOD
 
-  Use -pc_type lu -pc_factor_mat_solver_package cholmod to use this direct solver
+  Use -pc_type cholesky -pc_factor_mat_solver_type cholmod to use this direct solver
 
   Consult CHOLMOD documentation for more information about the Common parameters
   which correspond to the options database keys below.
@@ -407,12 +379,10 @@ PetscErrorCode MatFactorGetSolverPackage_seqsbaij_cholmod(Mat A,const MatSolverP
 
    Note: CHOLMOD is part of SuiteSparse http://faculty.cse.tamu.edu/davis/suitesparse.html
 
-.seealso: PCCHOLESKY, PCFactorSetMatSolverPackage(), MatSolverPackage
+.seealso: PCCHOLESKY, PCFactorSetMatSolverType(), MatSolverType
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "MatGetFactor_seqsbaij_cholmod"
-PETSC_EXTERN PetscErrorCode MatGetFactor_seqsbaij_cholmod(Mat A,MatFactorType ftype,Mat *F)
+PETSC_INTERN PetscErrorCode MatGetFactor_seqsbaij_cholmod(Mat A,MatFactorType ftype,Mat *F)
 {
   Mat            B;
   Mat_CHOLMOD    *chol;
@@ -427,23 +397,27 @@ PETSC_EXTERN PetscErrorCode MatGetFactor_seqsbaij_cholmod(Mat A,MatFactorType ft
   /* Create the factorization matrix F */
   ierr = MatCreate(PetscObjectComm((PetscObject)A),&B);CHKERRQ(ierr);
   ierr = MatSetSizes(B,PETSC_DECIDE,PETSC_DECIDE,m,n);CHKERRQ(ierr);
-  ierr = MatSetType(B,((PetscObject)A)->type_name);CHKERRQ(ierr);
-  ierr = MatSeqSBAIJSetPreallocation(B,1,0,NULL);CHKERRQ(ierr);
+  ierr = PetscStrallocpy("cholmod",&((PetscObject)B)->type_name);CHKERRQ(ierr);
+  ierr = MatSetUp(B);CHKERRQ(ierr);
   ierr = PetscNewLog(B,&chol);CHKERRQ(ierr);
 
   chol->Wrap    = MatWrapCholmod_seqsbaij;
-  chol->Destroy = MatDestroy_SeqSBAIJ;
-  B->spptr      = chol;
+  B->data       = chol;
 
+  B->ops->getinfo                = MatGetInfo_External;
   B->ops->view                   = MatView_CHOLMOD;
   B->ops->choleskyfactorsymbolic = MatCholeskyFactorSymbolic_CHOLMOD;
   B->ops->destroy                = MatDestroy_CHOLMOD;
-  ierr                           = PetscObjectComposeFunction((PetscObject)B,"MatFactorGetSolverPackage_C",MatFactorGetSolverPackage_seqsbaij_cholmod);CHKERRQ(ierr);
+  ierr                           = PetscObjectComposeFunction((PetscObject)B,"MatFactorGetSolverType_C",MatFactorGetSolverType_seqsbaij_cholmod);CHKERRQ(ierr);
   B->factortype                  = MAT_FACTOR_CHOLESKY;
   B->assembled                   = PETSC_TRUE; /* required by -ksp_view */
   B->preallocated                = PETSC_TRUE;
 
   ierr = CholmodStart(B);CHKERRQ(ierr);
+
+  ierr = PetscFree(B->solvertype);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(MATSOLVERCHOLMOD,&B->solvertype);CHKERRQ(ierr);
+
   *F   = B;
   PetscFunctionReturn(0);
 }
