@@ -2,12 +2,45 @@
 /*
     This is included by sbaij.c to generate unsigned short and regular versions of these two functions
 */
-#undef __FUNCT__
+
+/* We cut-and-past below from aij.h to make a "no_function" version of PetscSparseDensePlusDot().
+ * This is necessary because the USESHORT case cannot use the inlined functions that may be employed. */
+
+#if defined(PETSC_KERNEL_USE_UNROLL_4)
+#define PetscSparseDensePlusDot_no_function(sum,r,xv,xi,nnz) { \
+    if (nnz > 0) { \
+      PetscInt nnz2=nnz,rem=nnz&0x3; \
+      switch (rem) { \
+      case 3: sum += *xv++ *r[*xi++]; \
+      case 2: sum += *xv++ *r[*xi++]; \
+      case 1: sum += *xv++ *r[*xi++]; \
+        nnz2      -= rem;} \
+      while (nnz2 > 0) { \
+        sum +=  xv[0] * r[xi[0]] + xv[1] * r[xi[1]] + \
+                xv[2] * r[xi[2]] + xv[3] * r[xi[3]]; \
+        xv += 4; xi += 4; nnz2 -= 4; \
+      } \
+      xv -= nnz; xi -= nnz; \
+    } \
+  }
+
+#elif defined(PETSC_KERNEL_USE_UNROLL_2)
+#define PetscSparseDensePlusDot_no_function(sum,r,xv,xi,nnz) { \
+    PetscInt __i,__i1,__i2; \
+    for (__i=0; __i<nnz-1; __i+=2) {__i1 = xi[__i]; __i2=xi[__i+1]; \
+                                    sum += (xv[__i]*r[__i1] + xv[__i+1]*r[__i2]);} \
+    if (nnz & 0x1) sum += xv[__i] * r[xi[__i]];}
+
+#else
+#define PetscSparseDensePlusDot_no_function(sum,r,xv,xi,nnz) { \
+    PetscInt __i; \
+    for (__i=0; __i<nnz; __i++) sum += xv[__i] * r[xi[__i]];}
+#endif
+
+
 #if defined(USESHORT)
-#define __FUNCT__ "MatMult_SeqSBAIJ_1_Hermitian_ushort"
 PetscErrorCode MatMult_SeqSBAIJ_1_Hermitian_ushort(Mat A,Vec xx,Vec zz)
 #else
-#define __FUNCT__ "MatMult_SeqSBAIJ_1_Hermitian"
 PetscErrorCode MatMult_SeqSBAIJ_1_Hermitian(Mat A,Vec xx,Vec zz)
 #endif
 {
@@ -62,12 +95,9 @@ PetscErrorCode MatMult_SeqSBAIJ_1_Hermitian(Mat A,Vec xx,Vec zz)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
 #if defined(USESHORT)
-#define __FUNCT__ "MatMult_SeqSBAIJ_1_ushort"
 PetscErrorCode MatMult_SeqSBAIJ_1_ushort(Mat A,Vec xx,Vec zz)
 #else
-#define __FUNCT__ "MatMult_SeqSBAIJ_1"
 PetscErrorCode MatMult_SeqSBAIJ_1(Mat A,Vec xx,Vec zz)
 #endif
 {
@@ -124,12 +154,9 @@ PetscErrorCode MatMult_SeqSBAIJ_1(Mat A,Vec xx,Vec zz)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
 #if defined(USESHORT)
-#define __FUNCT__ "MatSOR_SeqSBAIJ_ushort"
 PetscErrorCode MatSOR_SeqSBAIJ_ushort(Mat A,Vec bb,PetscReal omega,MatSORType flag,PetscReal fshift,PetscInt its,PetscInt lits,Vec xx)
 #else
-#define __FUNCT__ "MatSOR_SeqSBAIJ"
 PetscErrorCode MatSOR_SeqSBAIJ(Mat A,Vec bb,PetscReal omega,MatSORType flag,PetscReal fshift,PetscInt its,PetscInt lits,Vec xx)
 #endif
 {
@@ -184,7 +211,11 @@ PetscErrorCode MatSOR_SeqSBAIJ(Mat A,Vec bb,PetscReal omega,MatSORType flag,Pets
       vj  = aj + ai[i] + 1;
       v   = aa + ai[i] + 1;
       sum = b[i]*d/omega;
+#ifdef USESHORT
+      PetscSparseDensePlusDot_no_function(sum,b,v,vj,nz);
+#else
       PetscSparseDensePlusDot(sum,b,v,vj,nz);
+#endif
       x[i] = sum;
     }
     ierr = PetscLogFlops(a->nz);CHKERRQ(ierr);
