@@ -7,8 +7,6 @@
 
 PetscClassId MAT_NULLSPACE_CLASSID;
 
-#undef __FUNCT__
-#define __FUNCT__ "MatNullSpaceSetFunction"
 /*@C
    MatNullSpaceSetFunction - set a function that removes a null space from a vector
    out of null spaces.
@@ -35,8 +33,6 @@ PetscErrorCode  MatNullSpaceSetFunction(MatNullSpace sp, PetscErrorCode (*rem)(M
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatNullSpaceGetVecs"
 /*@C
    MatNullSpaceGetVecs - get vectors defining the null space
 
@@ -68,8 +64,6 @@ PetscErrorCode MatNullSpaceGetVecs(MatNullSpace sp,PetscBool *has_const,PetscInt
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatNullSpaceCreateRigidBody"
 /*@
    MatNullSpaceCreateRigidBody - create rigid body modes from coordinates
 
@@ -83,7 +77,15 @@ PetscErrorCode MatNullSpaceGetVecs(MatNullSpace sp,PetscBool *has_const,PetscInt
 
    Level: advanced
 
-.seealso: MatNullSpaceCreate()
+   Notes:
+    If you are solving an elasticity problems you should likely use this, in conjunction with ee MatSetNearNullspace(), to provide information that 
+           the PCGAMG preconditioner can use to construct a much more efficient preconditioner.
+
+           If you are solving an elasticity problem with pure Neumann boundary conditions you can use this in conjunction with MatSetNullspace() to
+           provide this information to the linear solver so it can handle the null space appropriately in the linear solution.
+
+
+.seealso: MatNullSpaceCreate(), MatSetNearNullspace(), MatSetNullspace()
 @*/
 PetscErrorCode MatNullSpaceCreateRigidBody(Vec coords,MatNullSpace *sp)
 {
@@ -161,8 +163,6 @@ PetscErrorCode MatNullSpaceCreateRigidBody(Vec coords,MatNullSpace *sp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatNullSpaceView"
 /*@C
    MatNullSpaceView - Visualizes a null space object.
 
@@ -186,7 +186,9 @@ PetscErrorCode MatNullSpaceView(MatNullSpace sp,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(sp,MAT_NULLSPACE_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)sp));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)sp),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(sp,1,viewer,2);
 
@@ -209,9 +211,7 @@ PetscErrorCode MatNullSpaceView(MatNullSpace sp,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatNullSpaceCreate"
-/*@
+/*@C
    MatNullSpaceCreate - Creates a data structure used to project vectors
    out of null spaces.
 
@@ -231,7 +231,8 @@ PetscErrorCode MatNullSpaceView(MatNullSpace sp,PetscViewer viewer)
 
    Level: advanced
 
-   Notes: See MatNullSpaceSetFunction() as an alternative way of providing the null space information instead of setting vecs.
+   Notes:
+    See MatNullSpaceSetFunction() as an alternative way of providing the null space information instead of setting vecs.
 
       If has_cnst is PETSC_TRUE you do not need to pass a constant vector in as a fourth argument to this routine, nor do you
        need to pass in a function that eliminates the constant function into MatNullSpaceSetFunction().
@@ -254,6 +255,38 @@ PetscErrorCode  MatNullSpaceCreate(MPI_Comm comm,PetscBool has_cnst,PetscInt n,c
   if (n) PetscValidPointer(vecs,4);
   for (i=0; i<n; i++) PetscValidHeaderSpecific(vecs[i],VEC_CLASSID,4);
   PetscValidPointer(SP,5);
+  if (n) {
+    for (i=0; i<n; i++) {
+      /* prevent the user from changes values in the vector */
+      ierr = VecLockPush(vecs[i]);CHKERRQ(ierr);
+    }
+  }
+#if defined(PETSC_USE_DEBUG)
+  if (n) {
+    PetscScalar *dots;
+    for (i=0; i<n; i++) {
+      PetscReal norm;
+      ierr = VecNorm(vecs[i],NORM_2,&norm);CHKERRQ(ierr);
+      if (PetscAbsReal(norm - 1) > PETSC_SQRT_MACHINE_EPSILON) SETERRQ2(PetscObjectComm((PetscObject)vecs[i]),PETSC_ERR_ARG_WRONG,"Vector %D must have 2-norm of 1.0, it is %g",i,(double)norm);
+    }
+    if (has_cnst) {
+      for (i=0; i<n; i++) {
+        PetscScalar sum;
+        ierr = VecSum(vecs[i],&sum);CHKERRQ(ierr);
+        if (PetscAbsScalar(sum) > PETSC_SQRT_MACHINE_EPSILON) SETERRQ2(PetscObjectComm((PetscObject)vecs[i]),PETSC_ERR_ARG_WRONG,"Vector %D must be orthogonal to constant vector, inner product is %g",i,(double)PetscAbsScalar(sum));
+      }
+    }
+    ierr = PetscMalloc1(n-1,&dots);CHKERRQ(ierr);
+    for (i=0; i<n-1; i++) {
+      PetscInt j;
+      ierr = VecMDot(vecs[i],n-i-1,vecs+i+1,dots);CHKERRQ(ierr);
+      for (j=0;j<n-i-1;j++) {
+        if (PetscAbsScalar(dots[j]) > PETSC_SQRT_MACHINE_EPSILON) SETERRQ3(PetscObjectComm((PetscObject)vecs[i]),PETSC_ERR_ARG_WRONG,"Vector %D must be orthogonal to vector %D, inner product is %g",i,i+j+1,(double)PetscAbsScalar(dots[j]));
+      }
+    }
+    PetscFree(dots);CHKERRQ(ierr);
+  }
+#endif
 
   *SP = NULL;
   ierr = MatInitializePackage();CHKERRQ(ierr);
@@ -281,8 +314,6 @@ PetscErrorCode  MatNullSpaceCreate(MPI_Comm comm,PetscBool has_cnst,PetscInt n,c
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatNullSpaceDestroy"
 /*@
    MatNullSpaceDestroy - Destroys a data structure used to project vectors
    out of null spaces.
@@ -301,27 +332,30 @@ PetscErrorCode  MatNullSpaceCreate(MPI_Comm comm,PetscBool has_cnst,PetscInt n,c
 PetscErrorCode  MatNullSpaceDestroy(MatNullSpace *sp)
 {
   PetscErrorCode ierr;
+  PetscInt       i;
 
   PetscFunctionBegin;
   if (!*sp) PetscFunctionReturn(0);
   PetscValidHeaderSpecific((*sp),MAT_NULLSPACE_CLASSID,1);
   if (--((PetscObject)(*sp))->refct > 0) {*sp = 0; PetscFunctionReturn(0);}
 
+  for (i=0; i < (*sp)->n; i++) {
+    ierr = VecLockPop((*sp)->vecs[i]);CHKERRQ(ierr);
+  }
+
   ierr = VecDestroyVecs((*sp)->n,&(*sp)->vecs);CHKERRQ(ierr);
   ierr = PetscFree((*sp)->alpha);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(sp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatNullSpaceRemove"
 /*@C
    MatNullSpaceRemove - Removes all the components of a null space from a vector.
 
    Collective on MatNullSpace
 
    Input Parameters:
-+  sp - the null space context
++  sp - the null space context (if this is NULL then no null space is removed)
 -  vec - the vector from which the null space is to be removed
 
    Level: advanced
@@ -337,6 +371,7 @@ PetscErrorCode  MatNullSpaceRemove(MatNullSpace sp,Vec vec)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  if (!sp) PetscFunctionReturn(0);
   PetscValidHeaderSpecific(sp,MAT_NULLSPACE_CLASSID,1);
   PetscValidHeaderSpecific(vec,VEC_CLASSID,2);
 
@@ -361,8 +396,6 @@ PetscErrorCode  MatNullSpaceRemove(MatNullSpace sp,Vec vec)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatNullSpaceTest"
 /*@
    MatNullSpaceTest  - Tests if the claimed null space is really a
      null space of a matrix
@@ -385,7 +418,7 @@ PetscErrorCode  MatNullSpaceRemove(MatNullSpace sp,Vec vec)
 PetscErrorCode  MatNullSpaceTest(MatNullSpace sp,Mat mat,PetscBool  *isNull)
 {
   PetscScalar    sum;
-  PetscReal      nrm;
+  PetscReal      nrm,tol = 10. * PETSC_SQRT_MACHINE_EPSILON;
   PetscInt       j,n,N;
   PetscErrorCode ierr;
   Vec            l,r;
@@ -396,8 +429,8 @@ PetscErrorCode  MatNullSpaceTest(MatNullSpace sp,Mat mat,PetscBool  *isNull)
   PetscValidHeaderSpecific(sp,MAT_NULLSPACE_CLASSID,1);
   PetscValidHeaderSpecific(mat,MAT_CLASSID,2);
   n    = sp->n;
-  ierr = PetscOptionsGetBool(NULL,"-mat_null_space_test_view",&flg1,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetBool(NULL,"-mat_null_space_test_view_draw",&flg2,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(((PetscObject)sp)->options,((PetscObject)mat)->prefix,"-mat_null_space_test_view",&flg1,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(((PetscObject)sp)->options,((PetscObject)mat)->prefix,"-mat_null_space_test_view_draw",&flg2,NULL);CHKERRQ(ierr);
 
   if (n) {
     ierr = VecDuplicate(sp->vecs[0],&l);CHKERRQ(ierr);
@@ -413,7 +446,7 @@ PetscErrorCode  MatNullSpaceTest(MatNullSpace sp,Mat mat,PetscBool  *isNull)
     ierr = VecSet(l,sum);CHKERRQ(ierr);
     ierr = MatMult(mat,l,r);CHKERRQ(ierr);
     ierr = VecNorm(r,NORM_2,&nrm);CHKERRQ(ierr);
-    if (nrm >= 1.e-7) consistent = PETSC_FALSE;
+    if (nrm >= tol) consistent = PETSC_FALSE;
     if (flg1) {
       if (consistent) {
         ierr = PetscPrintf(PetscObjectComm((PetscObject)sp),"Constants are likely null vector");CHKERRQ(ierr);
@@ -430,7 +463,7 @@ PetscErrorCode  MatNullSpaceTest(MatNullSpace sp,Mat mat,PetscBool  *isNull)
   for (j=0; j<n; j++) {
     ierr = (*mat->ops->mult)(mat,sp->vecs[j],l);CHKERRQ(ierr);
     ierr = VecNorm(l,NORM_2,&nrm);CHKERRQ(ierr);
-    if (nrm >= 1.e-7) consistent = PETSC_FALSE;
+    if (nrm >= tol) consistent = PETSC_FALSE;
     if (flg1) {
       if (consistent) {
         ierr = PetscPrintf(PetscObjectComm((PetscObject)sp),"Null vector %D is likely null vector",j);CHKERRQ(ierr);
