@@ -14,12 +14,6 @@ PETSC_EXTERN PetscErrorCode MatCreate_MPIBAIJ(Mat);
 PETSC_EXTERN PetscErrorCode MatCreate_SeqSBAIJ(Mat);
 PETSC_EXTERN PetscErrorCode MatCreate_MPISBAIJ(Mat);
 
-PETSC_EXTERN PetscErrorCode MatCreate_SeqBSTRM(Mat);
-PETSC_EXTERN PetscErrorCode MatCreate_MPIBSTRM(Mat);
-
-PETSC_EXTERN PetscErrorCode MatCreate_SeqSBSTRM(Mat);
-PETSC_EXTERN PetscErrorCode MatCreate_MPISBSTRM(Mat);
-
 PETSC_EXTERN PetscErrorCode MatCreate_SeqDense(Mat);
 PETSC_EXTERN PetscErrorCode MatCreate_MPIDense(Mat);
 
@@ -30,6 +24,17 @@ PETSC_EXTERN PetscErrorCode MatCreate_Composite(Mat);
 PETSC_EXTERN PetscErrorCode MatCreate_SeqAIJPERM(Mat);
 PETSC_EXTERN PetscErrorCode MatCreate_MPIAIJPERM(Mat);
 
+PETSC_EXTERN PetscErrorCode MatCreate_SeqAIJSELL(Mat);
+PETSC_EXTERN PetscErrorCode MatCreate_MPIAIJSELL(Mat);
+
+#if defined PETSC_HAVE_MKL_SPARSE
+PETSC_EXTERN PetscErrorCode MatCreate_SeqAIJMKL(Mat);
+PETSC_EXTERN PetscErrorCode MatCreate_MPIAIJMKL(Mat);
+
+PETSC_EXTERN PetscErrorCode MatCreate_SeqBAIJMKL(Mat);
+PETSC_EXTERN PetscErrorCode MatCreate_MPIBAIJMKL(Mat);
+#endif
+
 PETSC_EXTERN PetscErrorCode MatCreate_SeqAIJCRL(Mat);
 PETSC_EXTERN PetscErrorCode MatCreate_MPIAIJCRL(Mat);
 
@@ -37,12 +42,10 @@ PETSC_EXTERN PetscErrorCode MatCreate_Scatter(Mat);
 PETSC_EXTERN PetscErrorCode MatCreate_BlockMat(Mat);
 PETSC_EXTERN PetscErrorCode MatCreate_Nest(Mat);
 
-#if defined PETSC_HAVE_CUSP
-PETSC_EXTERN PetscErrorCode MatCreate_SeqAIJCUSP(Mat);
-PETSC_EXTERN PetscErrorCode MatCreate_MPIAIJCUSP(Mat);
-#endif
+PETSC_EXTERN PetscErrorCode MatCreate_SeqSELL(Mat);
+PETSC_EXTERN PetscErrorCode MatCreate_MPISELL(Mat);
 
-#if defined PETSC_HAVE_CUSP
+#if defined PETSC_HAVE_VECCUDA
 PETSC_EXTERN PetscErrorCode MatCreate_SeqAIJCUSPARSE(Mat);
 PETSC_EXTERN PetscErrorCode MatCreate_MPIAIJCUSPARSE(Mat);
 #endif
@@ -57,15 +60,13 @@ PETSC_EXTERN PetscErrorCode MatCreate_FFTW(Mat);
 #endif
 PETSC_EXTERN PetscErrorCode MatCreate_Elemental(Mat);
 
-/*
-    This is used by MatSetType() to make sure that at least one
-    MatRegisterAll() is called. In general, if there is more than one
-    DLL, then MatRegisterAll() may be called several times.
-*/
-extern PetscBool MatRegisterAllCalled;
+PETSC_EXTERN PetscErrorCode MatCreate_Preallocator(Mat);
+PETSC_EXTERN PetscErrorCode MatCreate_Dummy(Mat);
+
+#if defined PETSC_HAVE_HYPRE
+PETSC_EXTERN PetscErrorCode MatCreate_HYPRE(Mat);
+#endif
 
-#undef __FUNCT__
-#define __FUNCT__ "MatRegisterAll"
 /*@C
   MatRegisterAll - Registers all of the matrix types in PETSc
 
@@ -75,7 +76,7 @@ extern PetscBool MatRegisterAllCalled;
 
 .keywords: KSP, register, all
 
-.seealso:  MatRegisterDestroy()
+.seealso:  MatRegister()
 @*/
 PetscErrorCode  MatRegisterAll(void)
 {
@@ -103,6 +104,20 @@ PetscErrorCode  MatRegisterAll(void)
   ierr = MatRegister(MATMPIAIJPERM,     MatCreate_MPIAIJPERM);CHKERRQ(ierr);
   ierr = MatRegister(MATSEQAIJPERM,     MatCreate_SeqAIJPERM);CHKERRQ(ierr);
 
+  ierr = MatRegisterBaseName(MATAIJSELL,MATSEQAIJSELL,MATMPIAIJSELL);CHKERRQ(ierr);
+  ierr = MatRegister(MATMPIAIJSELL,     MatCreate_MPIAIJSELL);CHKERRQ(ierr);
+  ierr = MatRegister(MATSEQAIJSELL,     MatCreate_SeqAIJSELL);CHKERRQ(ierr);
+
+#if defined PETSC_HAVE_MKL_SPARSE
+  ierr = MatRegisterBaseName(MATAIJMKL, MATSEQAIJMKL,MATMPIAIJMKL);CHKERRQ(ierr);
+  ierr = MatRegister(MATMPIAIJMKL,      MatCreate_MPIAIJMKL);CHKERRQ(ierr);
+  ierr = MatRegister(MATSEQAIJMKL,      MatCreate_SeqAIJMKL);CHKERRQ(ierr);
+
+  ierr = MatRegisterBaseName(MATBAIJMKL,MATSEQBAIJMKL,MATMPIBAIJMKL);CHKERRQ(ierr);
+  ierr = MatRegister(MATMPIBAIJMKL,      MatCreate_MPIBAIJMKL);CHKERRQ(ierr);
+  ierr = MatRegister(MATSEQBAIJMKL,      MatCreate_SeqBAIJMKL);CHKERRQ(ierr);
+#endif
+
   ierr = MatRegisterBaseName(MATAIJCRL,MATSEQAIJCRL,MATMPIAIJCRL);CHKERRQ(ierr);
   ierr = MatRegister(MATSEQAIJCRL,      MatCreate_SeqAIJCRL);CHKERRQ(ierr);
   ierr = MatRegister(MATMPIAIJCRL,      MatCreate_MPIAIJCRL);CHKERRQ(ierr);
@@ -115,14 +130,6 @@ PetscErrorCode  MatRegisterAll(void)
   ierr = MatRegister(MATMPISBAIJ,       MatCreate_MPISBAIJ);CHKERRQ(ierr);
   ierr = MatRegister(MATSEQSBAIJ,       MatCreate_SeqSBAIJ);CHKERRQ(ierr);
 
-  ierr = MatRegisterBaseName(MATBSTRM,MATSEQBSTRM,MATMPIBSTRM);CHKERRQ(ierr);
-  ierr = MatRegister(MATMPIBSTRM,       MatCreate_MPIBSTRM);CHKERRQ(ierr);
-  ierr = MatRegister(MATSEQBSTRM,       MatCreate_SeqBSTRM);CHKERRQ(ierr);
-
-  ierr = MatRegisterBaseName(MATSBSTRM,MATSEQSBSTRM,MATMPISBSTRM);CHKERRQ(ierr);
-  ierr = MatRegister(MATMPISBSTRM,      MatCreate_MPISBSTRM);CHKERRQ(ierr);
-  ierr = MatRegister(MATSEQSBSTRM,      MatCreate_SeqSBSTRM);CHKERRQ(ierr);
-
   ierr = MatRegisterBaseName(MATDENSE,MATSEQDENSE,MATMPIDENSE);CHKERRQ(ierr);
   ierr = MatRegister(MATMPIDENSE,       MatCreate_MPIDense);CHKERRQ(ierr);
   ierr = MatRegister(MATSEQDENSE,       MatCreate_SeqDense);CHKERRQ(ierr);
@@ -132,14 +139,11 @@ PetscErrorCode  MatRegisterAll(void)
   ierr = MatRegister(MATBLOCKMAT,       MatCreate_BlockMat);CHKERRQ(ierr);
   ierr = MatRegister(MATNEST,           MatCreate_Nest);CHKERRQ(ierr);
 
+  ierr = MatRegisterBaseName(MATSELL,MATSEQSELL,MATMPISELL);CHKERRQ(ierr);
+  ierr = MatRegister(MATMPISELL,         MatCreate_MPISELL);CHKERRQ(ierr);
+  ierr = MatRegister(MATSEQSELL,         MatCreate_SeqSELL);CHKERRQ(ierr);
 
-#if defined PETSC_HAVE_CUSP
-  ierr = MatRegisterBaseName(MATAIJCUSP,MATSEQAIJCUSP,MATMPIAIJCUSP);CHKERRQ(ierr);
-  ierr = MatRegister(MATSEQAIJCUSP,     MatCreate_SeqAIJCUSP);CHKERRQ(ierr);
-  ierr = MatRegister(MATMPIAIJCUSP,     MatCreate_MPIAIJCUSP);CHKERRQ(ierr);
-#endif
-
-#if defined PETSC_HAVE_CUSP
+#if defined PETSC_HAVE_VECCUDA
   ierr = MatRegisterBaseName(MATAIJCUSPARSE,MATSEQAIJCUSPARSE,MATMPIAIJCUSPARSE);CHKERRQ(ierr);
   ierr = MatRegister(MATSEQAIJCUSPARSE, MatCreate_SeqAIJCUSPARSE);CHKERRQ(ierr);
   ierr = MatRegister(MATMPIAIJCUSPARSE, MatCreate_MPIAIJCUSPARSE);CHKERRQ(ierr);
@@ -151,14 +155,18 @@ PetscErrorCode  MatRegisterAll(void)
   ierr = MatRegister(MATMPIAIJVIENNACL, MatCreate_MPIAIJViennaCL);CHKERRQ(ierr);
 #endif
 
-
 #if defined PETSC_HAVE_FFTW
   ierr = MatRegister(MATFFTW,           MatCreate_FFTW);CHKERRQ(ierr);
 #endif
 #if defined PETSC_HAVE_ELEMENTAL
   ierr = MatRegister(MATELEMENTAL,      MatCreate_Elemental);CHKERRQ(ierr);
 #endif
+
+  ierr = MatRegister(MATPREALLOCATOR,   MatCreate_Preallocator);CHKERRQ(ierr);
+  ierr = MatRegister(MATDUMMY,          MatCreate_Dummy);CHKERRQ(ierr);
+
+#if defined PETSC_HAVE_HYPRE
+  ierr = MatRegister(MATHYPRE,          MatCreate_HYPRE);CHKERRQ(ierr);
+#endif
   PetscFunctionReturn(0);
 }
-
-
