@@ -7,8 +7,6 @@
 #include <petsc/private/matimpl.h>        /*I "petscmat.h" I*/
 #include <petsc/private/isimpl.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringSetF"
 PetscErrorCode  MatFDColoringSetF(MatFDColoring fd,Vec F)
 {
   PetscErrorCode ierr;
@@ -24,8 +22,6 @@ PetscErrorCode  MatFDColoringSetF(MatFDColoring fd,Vec F)
 }
 
 #include <petscdraw.h>
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringView_Draw_Zoom"
 static PetscErrorCode MatFDColoringView_Draw_Zoom(PetscDraw draw,void *Aa)
 {
   MatFDColoring  fd = (MatFDColoring)Aa;
@@ -48,8 +44,6 @@ static PetscErrorCode MatFDColoringView_Draw_Zoom(PetscDraw draw,void *Aa)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringView_Draw"
 static PetscErrorCode MatFDColoringView_Draw(MatFDColoring fd,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -59,20 +53,19 @@ static PetscErrorCode MatFDColoringView_Draw(MatFDColoring fd,PetscViewer viewer
 
   PetscFunctionBegin;
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
-  ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr); if (isnull) PetscFunctionReturn(0);
-
-  ierr = PetscObjectCompose((PetscObject)fd,"Zoomviewer",(PetscObject)viewer);CHKERRQ(ierr);
+  ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr);
+  if (isnull) PetscFunctionReturn(0);
 
   xr   = fd->N; yr  = fd->M; h = yr/10.0; w = xr/10.0;
   xr  += w;     yr += h;    xl = -w;     yl = -h;
   ierr = PetscDrawSetCoordinates(draw,xl,yl,xr,yr);CHKERRQ(ierr);
+  ierr = PetscObjectCompose((PetscObject)fd,"Zoomviewer",(PetscObject)viewer);CHKERRQ(ierr);
   ierr = PetscDrawZoom(draw,MatFDColoringView_Draw_Zoom,fd);CHKERRQ(ierr);
   ierr = PetscObjectCompose((PetscObject)fd,"Zoomviewer",NULL);CHKERRQ(ierr);
+  ierr = PetscDrawSave(draw);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringView"
 /*@C
    MatFDColoringView - Views a finite difference coloring context.
 
@@ -138,10 +131,12 @@ PetscErrorCode  MatFDColoringView(MatFDColoring c,PetscViewer viewer)
           ierr = PetscViewerASCIIPrintf(viewer,"      %D\n",c->columns[i][j]);CHKERRQ(ierr);
         }
         ierr = PetscViewerASCIIPrintf(viewer,"    Number of rows %D\n",c->nrows[i]);CHKERRQ(ierr);
-        for (j=0; j<c->nrows[i]; j++) {
-          row  = c->matentry[nz].row;
-          col  = c->matentry[nz++].col;
-          ierr = PetscViewerASCIIPrintf(viewer,"      %D %D \n",row,col);CHKERRQ(ierr);
+        if (c->matentry) {
+          for (j=0; j<c->nrows[i]; j++) {
+            row  = c->matentry[nz].row;
+            col  = c->matentry[nz++].col;
+            ierr = PetscViewerASCIIPrintf(viewer,"      %D %D \n",row,col);CHKERRQ(ierr);
+          }
         }
       }
     }
@@ -150,8 +145,6 @@ PetscErrorCode  MatFDColoringView(MatFDColoring c,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringSetParameters"
 /*@
    MatFDColoringSetParameters - Sets the parameters for the sparse approximation of
    a Jacobian matrix using finite differences.
@@ -161,9 +154,13 @@ PetscErrorCode  MatFDColoringView(MatFDColoring c,PetscViewer viewer)
    The Jacobian is estimated with the differencing approximation
 .vb
        F'(u)_{:,i} = [F(u+h*dx_{i}) - F(u)]/h where
-       h = error_rel*u[i]                 if  abs(u[i]) > umin
-         = +/- error_rel*umin             otherwise, with +/- determined by the sign of u[i]
-       dx_{i} = (0, ... 1, .... 0)
+       htype = 'ds':
+         h = error_rel*u[i]                 if  abs(u[i]) > umin
+           = +/- error_rel*umin             otherwise, with +/- determined by the sign of u[i]
+         dx_{i} = (0, ... 1, .... 0)
+
+       htype = 'wp':
+         h = error_rel * sqrt(1 + ||u||)
 .ve
 
    Input Parameters:
@@ -189,8 +186,6 @@ PetscErrorCode MatFDColoringSetParameters(MatFDColoring matfd,PetscReal error,Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringSetBlockSize"
 /*@
    MatFDColoringSetBlockSize - Sets block size for efficient inserting entries of Jacobian matrix.
 
@@ -219,8 +214,6 @@ PetscErrorCode MatFDColoringSetBlockSize(MatFDColoring matfd,PetscInt brows,Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringSetUp"
 /*@
    MatFDColoringSetUp - Sets up the internal data structures of matrix coloring context for the later use.
 
@@ -256,8 +249,6 @@ PetscErrorCode MatFDColoringSetUp(Mat mat,ISColoring iscoloring,MatFDColoring co
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringGetFunction"
 /*@C
    MatFDColoringGetFunction - Gets the function to use for computing the Jacobian.
 
@@ -286,8 +277,6 @@ PetscErrorCode  MatFDColoringGetFunction(MatFDColoring matfd,PetscErrorCode (**f
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringSetFunction"
 /*@C
    MatFDColoringSetFunction - Sets the function to use for computing the Jacobian.
 
@@ -304,7 +293,8 @@ PetscErrorCode  MatFDColoringGetFunction(MatFDColoring matfd,PetscErrorCode (**f
 
    Level: advanced
 
-   Notes: This function is usually used automatically by SNES (when one uses SNESSetJacobian() with the argument
+   Notes:
+    This function is usually used automatically by SNES (when one uses SNESSetJacobian() with the argument
      SNESComputeJacobianDefaultColor()) and only needs to be used by someone computing a matrix via coloring directly by
      calling MatFDColoringApply()
 
@@ -326,8 +316,6 @@ PetscErrorCode  MatFDColoringSetFunction(MatFDColoring matfd,PetscErrorCode (*f)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringSetFromOptions"
 /*@
    MatFDColoringSetFromOptions - Sets coloring finite difference parameters from
    the options database.
@@ -386,13 +374,48 @@ PetscErrorCode  MatFDColoringSetFromOptions(MatFDColoring matfd)
   }
 
   /* process any options handlers added with PetscObjectAddOptionsHandler() */
-  ierr = PetscObjectProcessOptionsHandlers((PetscObject)matfd);CHKERRQ(ierr);
+  ierr = PetscObjectProcessOptionsHandlers(PetscOptionsObject,(PetscObject)matfd);CHKERRQ(ierr);
   PetscOptionsEnd();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringViewFromOptions"
+/*@C
+   MatFDColoringSetType - Sets the approach for computing the finite difference parameter
+
+   Collective on MatFDColoring
+
+   Input Parameters:
++  coloring - the coloring context
+-  type - either MATMFFD_WP or MATMFFD_DS
+
+   Options Database Keys:
+.  -mat_fd_type - "wp" or "ds"
+
+   Note: It is goofy that the argument type is MatMFFDType since the MatFDColoring actually computes the matrix entries
+         but the process of computing the entries is the same as as with the MatMFFD operation so we should reuse the names instead of
+         introducing another one.
+
+   Level: intermediate
+
+.keywords: Mat, finite differences, parameters
+
+.seealso: MatFDColoringCreate(), MatFDColoringView(), MatFDColoringSetParameters()
+
+@*/
+PetscErrorCode  MatFDColoringSetType(MatFDColoring matfd,MatMFFDType type)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(matfd,MAT_FDCOLORING_CLASSID,1);
+  /*
+     It is goofy to handle the strings this way but currently there is no code to free a dynamically created matfd->htype
+     and this function is being provided as patch for a release so it shouldn't change the implementaton
+  */
+  if (type[0] == 'w' && type[1] == 'p') matfd->htype = "wp";
+  else if (type[0] == 'd' && type[1] == 's') matfd->htype = "ds";
+  else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Unknown finite differencing type %s",type);
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode MatFDColoringViewFromOptions(MatFDColoring fd,const char prefix[],const char optionname[])
 {
   PetscErrorCode    ierr;
@@ -415,8 +438,6 @@ PetscErrorCode MatFDColoringViewFromOptions(MatFDColoring fd,const char prefix[]
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringCreate"
 /*@
    MatFDColoringCreate - Creates a matrix coloring context for finite difference
    computation of Jacobians.
@@ -476,8 +497,6 @@ PetscErrorCode  MatFDColoringCreate(Mat mat,ISColoring iscoloring,MatFDColoring
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringDestroy"
 /*@
     MatFDColoringDestroy - Destroys a matrix coloring context that was created
     via MatFDColoringCreate().
@@ -521,8 +540,6 @@ PetscErrorCode  MatFDColoringDestroy(MatFDColoring *c)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringGetPerturbedColumns"
 /*@C
     MatFDColoringGetPerturbedColumns - Returns the indices of the columns that
       that are currently being perturbed.
@@ -536,13 +553,24 @@ PetscErrorCode  MatFDColoringDestroy(MatFDColoring *c)
 +   n - the number of local columns being perturbed
 -   cols - the column indices, in global numbering
 
-   Level: intermediate
+   Level: advanced
+
+   Fortran Note:
+   This routine has a different interface for Fortran
+$     #include <petsc/finclude/petscmat.h>
+$          use petscmat
+$          PetscInt, pointer :: array(:)
+$          PetscErrorCode  ierr
+$          MatFDColoring   i
+$          call MatFDColoringGetPerturbedColumnsF90(i,array,ierr)
+$      use the entries of array ...
+$          call MatFDColoringRestorePerturbedColumnsF90(i,array,ierr)
 
 .seealso: MatFDColoringCreate(), MatFDColoringDestroy(), MatFDColoringView(), MatFDColoringApply()
 
 .keywords: coloring, Jacobian, finite differences
 @*/
-PetscErrorCode  MatFDColoringGetPerturbedColumns(MatFDColoring coloring,PetscInt *n,PetscInt *cols[])
+PetscErrorCode  MatFDColoringGetPerturbedColumns(MatFDColoring coloring,PetscInt *n,const PetscInt *cols[])
 {
   PetscFunctionBegin;
   if (coloring->currentcolor >= 0) {
@@ -554,8 +582,6 @@ PetscErrorCode  MatFDColoringGetPerturbedColumns(MatFDColoring coloring,PetscInt
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatFDColoringApply"
 /*@
     MatFDColoringApply - Given a matrix for which a MatFDColoring context
     has been created, computes the Jacobian for a function via finite differences.
@@ -594,7 +620,7 @@ PetscErrorCode  MatFDColoringApply(Mat J,MatFDColoring coloring,Vec x1,void *sct
   if (!coloring->setupcalled) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Must call MatFDColoringSetUp()");
 
   ierr = MatSetUnfactored(J);CHKERRQ(ierr);
-  ierr = PetscOptionsGetBool(NULL,"-mat_fd_coloring_dont_rezero",&flg,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(((PetscObject)coloring)->options,NULL,"-mat_fd_coloring_dont_rezero",&flg,NULL);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscInfo(coloring,"Not calling MatZeroEntries()\n");CHKERRQ(ierr);
   } else {
@@ -608,5 +634,9 @@ PetscErrorCode  MatFDColoringApply(Mat J,MatFDColoring coloring,Vec x1,void *sct
   ierr = PetscLogEventBegin(MAT_FDColoringApply,coloring,J,x1,0);CHKERRQ(ierr);
   ierr = (*J->ops->fdcoloringapply)(J,coloring,x1,sctx);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(MAT_FDColoringApply,coloring,J,x1,0);CHKERRQ(ierr);
+  if (!coloring->viewed) {
+    ierr = MatFDColoringViewFromOptions(coloring,NULL,"-mat_fd_coloring_view");CHKERRQ(ierr);
+    coloring->viewed = PETSC_TRUE;
+  }
   PetscFunctionReturn(0);
 }
