@@ -30,8 +30,6 @@ PETSC_EXTERN PetscInt mc64ad_dist(const PetscInt*, PetscInt*, PetscInt*, const P
   MatGetOrdering_WBM - Find the nonsymmetric reordering of the graph which maximizes the product of diagonal entries,
     using weighted bipartite graph matching. This is MC64 in the Harwell-Boeing library.
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatGetOrdering_WBM"
 PETSC_INTERN PetscErrorCode MatGetOrdering_WBM(Mat mat, MatOrderingType type, IS *row, IS *col)
 {
   PetscScalar    *a, *dw;
