@@ -28,11 +28,11 @@ PETSC_EXTERN void PETSC_STDCALL matpartitioningview_(MatPartitioning *part,Petsc
   *ierr = MatPartitioningView(*part,v);
 }
 
-PETSC_EXTERN void PETSC_STDCALL matpartitioningsettype_(MatPartitioning *part,CHAR type PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len))
+PETSC_EXTERN void PETSC_STDCALL matpartitioningsettype_(MatPartitioning *part,char* type PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   char *t;
   FIXCHAR(type,len,t);
-  *ierr = MatPartitioningSetType(*part,t);
+  *ierr = MatPartitioningSetType(*part,t);if (*ierr) return;
   FREECHAR(type,t);
 }
 
