@@ -24,8 +24,6 @@ typedef struct {
 
 #define SIZE_LOG 10000          /* size of buffer for mesg_log */
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningApply_Party"
 static PetscErrorCode MatPartitioningApply_Party(MatPartitioning part,IS *partitioning)
 {
   PetscErrorCode        ierr;
@@ -48,16 +46,19 @@ static PetscErrorCode MatPartitioningApply_Party(MatPartitioning part,IS *partit
   ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)mat),&rank);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)mat,MATMPIADJ,&flg);CHKERRQ(ierr);
   if (size>1) {
-    if (flg) SETERRQ(PetscObjectComm((PetscObject)mat),PETSC_ERR_SUP,"Distributed matrix format MPIAdj is not supported for sequential partitioners");
-    ierr   = PetscInfo(part,"Converting distributed matrix to sequential: this could be a performance loss\n");CHKERRQ(ierr);
-    ierr   = MatGetSize(mat,&M,&N);CHKERRQ(ierr);
-    ierr   = ISCreateStride(PETSC_COMM_SELF,M,0,1,&isrow);CHKERRQ(ierr);
-    ierr   = ISCreateStride(PETSC_COMM_SELF,N,0,1,&iscol);CHKERRQ(ierr);
-    ierr   = MatGetSubMatrices(mat,1,&isrow,&iscol,MAT_INITIAL_MATRIX,&A);CHKERRQ(ierr);
-    ierr   = ISDestroy(&isrow);CHKERRQ(ierr);
-    ierr   = ISDestroy(&iscol);CHKERRQ(ierr);
-    matSeq = *A;
-    ierr   = PetscFree(A);CHKERRQ(ierr);
+    if (flg) {
+      ierr = MatMPIAdjToSeq(mat,&matSeq);CHKERRQ(ierr);
+     } else {
+      ierr   = PetscInfo(part,"Converting distributed matrix to sequential: this could be a performance loss\n");CHKERRQ(ierr);
+      ierr   = MatGetSize(mat,&M,&N);CHKERRQ(ierr);
+      ierr   = ISCreateStride(PETSC_COMM_SELF,M,0,1,&isrow);CHKERRQ(ierr);
+      ierr   = ISCreateStride(PETSC_COMM_SELF,N,0,1,&iscol);CHKERRQ(ierr);
+      ierr   = MatCreateSubMatrices(mat,1,&isrow,&iscol,MAT_INITIAL_MATRIX,&A);CHKERRQ(ierr);
+      ierr   = ISDestroy(&isrow);CHKERRQ(ierr);
+      ierr   = ISDestroy(&iscol);CHKERRQ(ierr);
+      matSeq = *A;
+      ierr   = PetscFree(A);CHKERRQ(ierr);
+     }
   } else {
     ierr   = PetscObjectReference((PetscObject)mat);CHKERRQ(ierr);
     matSeq = mat;
@@ -123,12 +124,8 @@ static PetscErrorCode MatPartitioningApply_Party(MatPartitioning part,IS *partit
   for (i=0; i<mat->rmap->N; i++) parttab[i] = part_party[i];
 
   /* creation of the index set */
-  nb_locals = mat->rmap->N / size;
-  locals    = parttab + rank*nb_locals;
-  if (rank < mat->rmap->N % size) {
-    nb_locals++;
-    locals += rank;
-  } else locals += mat->rmap->N % size;
+  nb_locals = mat->rmap->n;
+  locals    = parttab + mat->rmap->rstart;
 
   ierr = ISCreateGeneral(PetscObjectComm((PetscObject)part),nb_locals,locals,PETSC_COPY_VALUES,partitioning);CHKERRQ(ierr);
 
@@ -140,8 +137,6 @@ static PetscErrorCode MatPartitioningApply_Party(MatPartitioning part,IS *partit
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningView_Party"
 PetscErrorCode MatPartitioningView_Party(MatPartitioning part,PetscViewer viewer)
 {
   MatPartitioning_Party *party = (MatPartitioning_Party*)part->data;
@@ -167,8 +162,6 @@ PetscErrorCode MatPartitioningView_Party(MatPartitioning part,PetscViewer viewer
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningPartySetGlobal"
 /*@C
    MatPartitioningPartySetGlobal - Set global method for Party partitioner.
 
@@ -201,8 +194,6 @@ PetscErrorCode MatPartitioningPartySetGlobal(MatPartitioning part,const char *gl
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningPartySetGlobal_Party"
 PetscErrorCode MatPartitioningPartySetGlobal_Party(MatPartitioning part,const char *global)
 {
   MatPartitioning_Party *party = (MatPartitioning_Party*)part->data;
@@ -213,8 +204,6 @@ PetscErrorCode MatPartitioningPartySetGlobal_Party(MatPartitioning part,const ch
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningPartySetLocal"
 /*@C
    MatPartitioningPartySetLocal - Set local method for Party partitioner.
 
@@ -245,8 +234,6 @@ PetscErrorCode MatPartitioningPartySetLocal(MatPartitioning part,const char *loc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningPartySetLocal_Party"
 PetscErrorCode MatPartitioningPartySetLocal_Party(MatPartitioning part,const char *local)
 
 {
@@ -258,8 +245,6 @@ PetscErrorCode MatPartitioningPartySetLocal_Party(MatPartitioning part,const cha
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningPartySetCoarseLevel"
 /*@
    MatPartitioningPartySetCoarseLevel - Set the coarse level parameter for the
    Party partitioner.
@@ -286,8 +271,6 @@ PetscErrorCode MatPartitioningPartySetCoarseLevel(MatPartitioning part,PetscReal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningPartySetCoarseLevel_Party"
 PetscErrorCode MatPartitioningPartySetCoarseLevel_Party(MatPartitioning part,PetscReal level)
 {
   MatPartitioning_Party *party = (MatPartitioning_Party*)part->data;
@@ -299,8 +282,6 @@ PetscErrorCode MatPartitioningPartySetCoarseLevel_Party(MatPartitioning part,Pet
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningPartySetMatchOptimization"
 /*@
    MatPartitioningPartySetMatchOptimization - Activate matching optimization for
    graph reduction.
@@ -327,8 +308,6 @@ PetscErrorCode MatPartitioningPartySetMatchOptimization(MatPartitioning part,Pet
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningPartySetMatchOptimization_Party"
 PetscErrorCode MatPartitioningPartySetMatchOptimization_Party(MatPartitioning part,PetscBool opt)
 {
   MatPartitioning_Party *party = (MatPartitioning_Party*)part->data;
@@ -338,8 +317,6 @@ PetscErrorCode MatPartitioningPartySetMatchOptimization_Party(MatPartitioning pa
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningPartySetBipart"
 /*@
    MatPartitioningPartySetBipart - Activate or deactivate recursive bisection.
 
@@ -365,8 +342,6 @@ PetscErrorCode MatPartitioningPartySetBipart(MatPartitioning part,PetscBool bp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningPartySetBipart_Party"
 PetscErrorCode MatPartitioningPartySetBipart_Party(MatPartitioning part,PetscBool bp)
 {
   MatPartitioning_Party *party = (MatPartitioning_Party*)part->data;
@@ -376,9 +351,7 @@ PetscErrorCode MatPartitioningPartySetBipart_Party(MatPartitioning part,PetscBoo
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningSetFromOptions_Party"
-PetscErrorCode MatPartitioningSetFromOptions_Party(PetscOptions *PetscOptionsObject,MatPartitioning part)
+PetscErrorCode MatPartitioningSetFromOptions_Party(PetscOptionItems *PetscOptionsObject,MatPartitioning part)
 {
   PetscErrorCode        ierr;
   PetscBool             flag;
@@ -401,8 +374,6 @@ PetscErrorCode MatPartitioningSetFromOptions_Party(PetscOptions *PetscOptionsObj
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningDestroy_Party"
 PetscErrorCode MatPartitioningDestroy_Party(MatPartitioning part)
 {
   MatPartitioning_Party *party = (MatPartitioning_Party*)part->data;
@@ -424,7 +395,8 @@ PetscErrorCode MatPartitioningDestroy_Party(MatPartitioning part)
 
    Level: beginner
 
-   Notes: See http://wwwcs.upb.de/fachbereich/AG/monien/RESEARCH/PART/party.html
+   Notes:
+    See http://wwwcs.upb.de/fachbereich/AG/monien/RESEARCH/PART/party.html
 
 .keywords: Partitioning, create, context
 
@@ -432,8 +404,6 @@ PetscErrorCode MatPartitioningDestroy_Party(MatPartitioning part)
 
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningCreate_Party"
 PETSC_EXTERN PetscErrorCode MatPartitioningCreate_Party(MatPartitioning part)
 {
   PetscErrorCode        ierr;
