@@ -7,8 +7,6 @@ PetscClassId MAT_PARTITIONING_CLASSID;
 /*
    Simplest partitioning, keeps the current partitioning.
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningApply_Current"
 static PetscErrorCode MatPartitioningApply_Current(MatPartitioning part,IS *partitioning)
 {
   PetscErrorCode ierr;
@@ -29,8 +27,37 @@ static PetscErrorCode MatPartitioningApply_Current(MatPartitioning part,IS *part
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningApply_Square"
+/*
+   partition an index to rebalance the computation
+*/
+static PetscErrorCode MatPartitioningApply_Average(MatPartitioning part,IS *partitioning)
+{
+  PetscErrorCode ierr;
+  PetscInt       m,M,nparts,*indices,r,d,*parts,i,start,end,loc;
+
+  PetscFunctionBegin;
+  ierr = MatGetSize(part->adj,&M,NULL);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(part->adj,&m,NULL);CHKERRQ(ierr);
+  nparts = part->n;
+  ierr = PetscCalloc1(nparts,&parts);CHKERRQ(ierr);
+  d = M/nparts;
+  for (i=0; i<nparts; i++) parts[i] = d;
+  r = M%nparts;
+  for (i=0; i<r; i++) parts[i] += 1;
+  for (i=1; i<nparts; i++) parts[i] += parts[i-1];
+  ierr = PetscCalloc1(m,&indices);CHKERRQ(ierr);
+  ierr = MatGetOwnershipRange(part->adj,&start,&end);CHKERRQ(ierr);
+  for (i=start; i<end; i++) {
+    ierr = PetscFindInt(i,nparts,parts,&loc);CHKERRQ(ierr);
+    if (loc<0) loc = -(loc+1);
+    else loc = loc+1;
+    indices[i-start] = loc;
+  }
+  ierr = PetscFree(parts);CHKERRQ(ierr);
+  ierr = ISCreateGeneral(PetscObjectComm((PetscObject)part),m,indices,PETSC_OWN_POINTER,partitioning);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 static PetscErrorCode MatPartitioningApply_Square(MatPartitioning part,IS *partitioning)
 {
   PetscErrorCode ierr;
@@ -57,8 +84,6 @@ static PetscErrorCode MatPartitioningApply_Square(MatPartitioning part,IS *parti
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningCreate_Current"
 PETSC_EXTERN PetscErrorCode MatPartitioningCreate_Current(MatPartitioning part)
 {
   PetscFunctionBegin;
@@ -68,8 +93,15 @@ PETSC_EXTERN PetscErrorCode MatPartitioningCreate_Current(MatPartitioning part)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningCreate_Square"
+PETSC_EXTERN PetscErrorCode MatPartitioningCreate_Average(MatPartitioning part)
+{
+  PetscFunctionBegin;
+  part->ops->apply   = MatPartitioningApply_Average;
+  part->ops->view    = 0;
+  part->ops->destroy = 0;
+  PetscFunctionReturn(0);
+}
+
 PETSC_EXTERN PetscErrorCode MatPartitioningCreate_Square(MatPartitioning part)
 {
   PetscFunctionBegin;
@@ -80,14 +112,62 @@ PETSC_EXTERN PetscErrorCode MatPartitioningCreate_Square(MatPartitioning part)
 }
 
 
+/* gets as input the "sizes" array computed by ParMetis_*_NodeND and returns
+       seps[  0 :         2*p) : the start and end node of each subdomain
+       seps[2*p : 2*p+2*(p-1)) : the start and end node of each separator
+     levels[  0 :         p-1) : level in the tree for each separator (-1 root, -2 and -3 first level and so on)
+   The arrays must be large enough
+*/
+PETSC_INTERN PetscErrorCode MatPartitioningSizesToSep_Private(PetscInt p, PetscInt sizes[], PetscInt seps[], PetscInt level[])
+{
+  PetscInt       l2p,i,pTree,pStartTree;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  l2p = PetscLog2Real(p);
+  if (l2p - (PetscInt)PetscLog2Real(p)) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"%D is not a power of 2",p);
+  if (!p) PetscFunctionReturn(0);
+  ierr = PetscMemzero(seps,(2*p-2)*sizeof(PetscInt));CHKERRQ(ierr);
+  ierr = PetscMemzero(level,(p-1)*sizeof(PetscInt));CHKERRQ(ierr);
+  seps[2*p-2] = sizes[2*p-2];
+  pTree = p;
+  pStartTree = 0;
+  while (pTree != 1) {
+    for (i = pStartTree; i < pStartTree + pTree; i++) {
+      seps[i] += sizes[i];
+      seps[pStartTree + pTree + (i-pStartTree)/2] += seps[i];
+    }
+    pStartTree += pTree;
+    pTree = pTree/2;
+  }
+  seps[2*p-2] -= sizes[2*p-2];
+
+  pStartTree = 2*p-2;
+  pTree      = 1;
+  while (pStartTree > 0) {
+    for (i = pStartTree; i < pStartTree + pTree; i++) {
+      PetscInt k = 2*i - (pStartTree +2*pTree);
+      PetscInt n = seps[k+1];
+
+      seps[k+1]  = seps[i]   - sizes[k+1];
+      seps[k]    = seps[k+1] + sizes[k+1] - n - sizes[k];
+      level[i-p] = -pTree - i + pStartTree;
+    }
+    pTree *= 2;
+    pStartTree -= pTree;
+  }
+  /* I know there should be a formula */
+  ierr = PetscSortIntWithArrayPair(p-1,seps+p,sizes+p,level);CHKERRQ(ierr);
+  for (i=2*p-2;i>=0;i--) { seps[2*i] = seps[i]; seps[2*i+1] = seps[i] + sizes[i] - 1; }
+  PetscFunctionReturn(0);
+}
+
 /* ===========================================================================================*/
 
 PetscFunctionList MatPartitioningList              = 0;
 PetscBool         MatPartitioningRegisterAllCalled = PETSC_FALSE;
 
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningRegister"
 /*@C
    MatPartitioningRegister - Adds a new sparse matrix partitioning to the  matrix package.
 
@@ -118,12 +198,11 @@ PetscErrorCode  MatPartitioningRegister(const char sname[],PetscErrorCode (*func
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = MatInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&MatPartitioningList,sname,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningGetType"
 /*@C
    MatPartitioningGetType - Gets the Partitioning method type and name (as a string)
         from the partitioning context.
@@ -151,8 +230,6 @@ PetscErrorCode  MatPartitioningGetType(MatPartitioning partitioning,MatPartition
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningSetNParts"
 /*@C
    MatPartitioningSetNParts - Set how many partitions need to be created;
         by default this is one per processor. Certain partitioning schemes may
@@ -179,8 +256,47 @@ PetscErrorCode  MatPartitioningSetNParts(MatPartitioning part,PetscInt n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningApply"
+/*@
+   MatPartitioningApplyND - Gets a nested dissection partitioning for a matrix.
+
+   Collective on Mat
+
+   Input Parameters:
+.  matp - the matrix partitioning object
+
+   Output Parameters:
+.   partitioning - the partitioning. For each local node, a positive value indicates the processor
+                   number the node has been assigned to. Negative x values indicate the separator level -(x+1).
+
+   Level: beginner
+
+   The user can define additional partitionings; see MatPartitioningRegister().
+
+.keywords: matrix, get, partitioning
+
+.seealso:  MatPartitioningRegister(), MatPartitioningCreate(),
+           MatPartitioningDestroy(), MatPartitioningSetAdjacency(), ISPartitioningToNumbering(),
+           ISPartitioningCount()
+@*/
+PetscErrorCode  MatPartitioningApplyND(MatPartitioning matp,IS *partitioning)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(matp,MAT_PARTITIONING_CLASSID,1);
+  PetscValidPointer(partitioning,2);
+  if (!matp->adj->assembled) SETERRQ(PetscObjectComm((PetscObject)matp),PETSC_ERR_ARG_WRONGSTATE,"Not for unassembled matrix");
+  if (matp->adj->factortype) SETERRQ(PetscObjectComm((PetscObject)matp),PETSC_ERR_ARG_WRONGSTATE,"Not for factored matrix");
+  if (!matp->ops->applynd) SETERRQ1(PetscObjectComm((PetscObject)matp),PETSC_ERR_SUP,"Nested dissection not provided by MatPartitioningType %s",((PetscObject)matp)->type_name);
+  ierr = PetscLogEventBegin(MAT_PartitioningND,matp,0,0,0);CHKERRQ(ierr);
+  ierr = (*matp->ops->applynd)(matp,partitioning);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(MAT_PartitioningND,matp,0,0,0);CHKERRQ(ierr);
+
+  ierr = MatPartitioningViewFromOptions(matp,NULL,"-mat_partitioning_view");CHKERRQ(ierr);
+  ierr = ISViewFromOptions(*partitioning,NULL,"-mat_partitioning_view");CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    MatPartitioningApply - Gets a partitioning for a matrix.
 
@@ -213,7 +329,6 @@ $    -mat_partitioning_view
 PetscErrorCode  MatPartitioningApply(MatPartitioning matp,IS *partitioning)
 {
   PetscErrorCode ierr;
-  PetscBool      flag = PETSC_FALSE;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(matp,MAT_PARTITIONING_CLASSID,1);
@@ -225,18 +340,11 @@ PetscErrorCode  MatPartitioningApply(MatPartitioning matp,IS *partitioning)
   ierr = (*matp->ops->apply)(matp,partitioning);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(MAT_Partitioning,matp,0,0,0);CHKERRQ(ierr);
 
-  ierr = PetscOptionsGetBool(NULL,"-mat_partitioning_view",&flag,NULL);CHKERRQ(ierr);
-  if (flag) {
-    PetscViewer viewer;
-    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)matp),&viewer);CHKERRQ(ierr);
-    ierr = MatPartitioningView(matp,viewer);CHKERRQ(ierr);
-    ierr = ISView(*partitioning,viewer);CHKERRQ(ierr);
-  }
+  ierr = MatPartitioningViewFromOptions(matp,NULL,"-mat_partitioning_view");CHKERRQ(ierr);
+  ierr = ISViewFromOptions(*partitioning,NULL,"-mat_partitioning_view");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningSetAdjacency"
 /*@
    MatPartitioningSetAdjacency - Sets the adjacency graph (matrix) of the thing to be
       partitioned.
@@ -262,8 +370,6 @@ PetscErrorCode  MatPartitioningSetAdjacency(MatPartitioning part,Mat adj)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningDestroy"
 /*@
    MatPartitioningDestroy - Destroys the partitioning context.
 
@@ -296,8 +402,6 @@ PetscErrorCode  MatPartitioningDestroy(MatPartitioning *part)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningSetVertexWeights"
 /*@C
    MatPartitioningSetVertexWeights - Sets the weights for vertices for a partitioning.
 
@@ -330,8 +434,6 @@ PetscErrorCode  MatPartitioningSetVertexWeights(MatPartitioning part,const Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningSetPartitionWeights"
 /*@C
    MatPartitioningSetPartitionWeights - Sets the weights for each partition.
 
@@ -369,8 +471,6 @@ PetscErrorCode  MatPartitioningSetPartitionWeights(MatPartitioning part,const Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningCreate"
 /*@
    MatPartitioningCreate - Creates a partitioning context.
 
@@ -411,8 +511,6 @@ PetscErrorCode  MatPartitioningCreate(MPI_Comm comm,MatPartitioning *newp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningView"
 /*@C
    MatPartitioningView - Prints the partitioning data structure.
 
@@ -467,8 +565,6 @@ PetscErrorCode  MatPartitioningView(MatPartitioning part,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningSetType"
 /*@C
    MatPartitioningSetType - Sets the type of partitioner to use
 
@@ -502,7 +598,7 @@ PetscErrorCode  MatPartitioningSetType(MatPartitioning part,MatPartitioningType
   ierr = PetscObjectTypeCompare((PetscObject)part,type,&match);CHKERRQ(ierr);
   if (match) PetscFunctionReturn(0);
 
-  if (part->setupcalled) {
+  if (part->ops->destroy) {
     ierr =  (*part->ops->destroy)(part);CHKERRQ(ierr);
 
     part->ops->destroy = NULL;
@@ -523,8 +619,6 @@ PetscErrorCode  MatPartitioningSetType(MatPartitioning part,MatPartitioningType
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatPartitioningSetFromOptions"
 /*@
    MatPartitioningSetFromOptions - Sets various partitioning options from the
         options database.
@@ -539,6 +633,11 @@ $  -mat_partitioning_type  <type>
 $      Use -help for a list of available methods
 $      (for instance, parmetis)
 
+
+   Notes:
+    If the partitioner has not been set by the user it uses one of the installed partitioner such as ParMetis. If there are
+   no installed partitioners it uses current which means no repartioning.
+
    Level: beginner
 
 .keywords: partitioning, set, method, type
@@ -555,6 +654,12 @@ PetscErrorCode  MatPartitioningSetFromOptions(MatPartitioning part)
   if (!((PetscObject)part)->type_name) {
 #if defined(PETSC_HAVE_PARMETIS)
     def = MATPARTITIONINGPARMETIS;
+#elif defined(PETSC_HAVE_CHACO)
+    def = MATPARTITIONINGCHACO;
+#elif defined(PETSC_HAVE_PARTY)
+    def = MATPARTITIONINGPARTY;
+#elif defined(PETSC_HAVE_PTSCOTCH)
+    def = MATPARTITIONINGPTSCOTCH;
 #else
     def = MATPARTITIONINGCURRENT;
 #endif
