@@ -1,9 +1,16 @@
 
 #include <petsc/private/matimpl.h>       /*I "petscmat.h"  I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "MatShift_Basic"
-PetscErrorCode MatShift_Basic(Mat Y,PetscScalar a)
+PETSC_INTERN PetscErrorCode MatSetBlockSizes_Default(Mat mat,PetscInt rbs, PetscInt cbs)
+{
+  PetscFunctionBegin;
+  if (!mat->preallocated) PetscFunctionReturn(0);
+  if (mat->rmap->bs > 0 && mat->rmap->bs != rbs) SETERRQ2(PetscObjectComm((PetscObject)mat),PETSC_ERR_SUP,"Cannot change row block size %D to %D\n",mat->rmap->bs,rbs);
+  if (mat->cmap->bs > 0 && mat->cmap->bs != cbs) SETERRQ2(PetscObjectComm((PetscObject)mat),PETSC_ERR_SUP,"Cannot change column block size %D to %D\n",mat->cmap->bs,cbs);
+  PetscFunctionReturn(0);
+}
+
+PETSC_INTERN PetscErrorCode MatShift_Basic(Mat Y,PetscScalar a)
 {
   PetscErrorCode ierr;
   PetscInt       i,start,end;
@@ -23,8 +30,6 @@ PetscErrorCode MatShift_Basic(Mat Y,PetscScalar a)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatCreate"
 /*@
    MatCreate - Creates a matrix where the type is determined
    from either a call to MatSetType() or from the options database
@@ -84,16 +89,16 @@ PetscErrorCode  MatCreate(MPI_Comm comm,Mat *A)
   ierr = PetscHeaderCreate(B,MAT_CLASSID,"Mat","Matrix","Mat",comm,MatDestroy,MatView);CHKERRQ(ierr);
   ierr = PetscLayoutCreate(comm,&B->rmap);CHKERRQ(ierr);
   ierr = PetscLayoutCreate(comm,&B->cmap);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(VECSTANDARD,&B->defaultvectype);CHKERRQ(ierr);
 
-  B->preallocated = PETSC_FALSE;
-  *A              = B;
+  B->congruentlayouts = PETSC_DECIDE;
+  B->preallocated     = PETSC_FALSE;
+  *A                  = B;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSetErrorIfFPE"
 /*@
-   MatSetErrorIfFPE - Causes Mat to generate an error if a FPE, for example a zero pivot, is detected.
+   MatSetErrorIfFailure - Causes Mat to generate an error, for example a zero pivot, is detected.
 
    Logically Collective on Mat
 
@@ -107,17 +112,15 @@ PetscErrorCode  MatCreate(MPI_Comm comm,Mat *A)
 
 .seealso: PCSetErrorIfFailure()
 @*/
-PetscErrorCode  MatSetErrorIfFPE(Mat mat,PetscBool flg)
+PetscErrorCode  MatSetErrorIfFailure(Mat mat,PetscBool flg)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(mat,MAT_CLASSID,1);
   PetscValidLogicalCollectiveBool(mat,flg,2);
-  mat->erroriffpe = flg;
+  mat->erroriffailure = flg;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSetSizes"
 /*@
   MatSetSizes - Sets the local and global sizes, and checks to determine compatibility
 
@@ -158,17 +161,15 @@ PetscErrorCode  MatSetSizes(Mat A, PetscInt m, PetscInt n, PetscInt M, PetscInt
   if (N > 0) PetscValidLogicalCollectiveInt(A,N,5);
   if (M > 0 && m > M) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Local column size %D cannot be larger than global column size %D",m,M);
   if (N > 0 && n > N) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Local row size %D cannot be larger than global row size %D",n,N);
-  if ((A->rmap->n >= 0 && A->rmap->N >= 0) && (A->rmap->n != m || A->rmap->N != M)) SETERRQ4(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot change/reset row sizes to %D local %D global after previously setting them to %D local %D global",m,M,A->rmap->n,A->rmap->N);
-  if ((A->cmap->n >= 0 && A->cmap->N >= 0) && (A->cmap->n != n || A->cmap->N != N)) SETERRQ4(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot change/reset column sizes to %D local %D global after previously setting them to %D local %D global",n,N,A->cmap->n,A->cmap->N);
+  if ((A->rmap->n >= 0 && A->rmap->N >= 0) && (A->rmap->n != m || (M > 0 && A->rmap->N != M))) SETERRQ4(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot change/reset row sizes to %D local %D global after previously setting them to %D local %D global",m,M,A->rmap->n,A->rmap->N);
+  if ((A->cmap->n >= 0 && A->cmap->N >= 0) && (A->cmap->n != n || (N > 0 && A->cmap->N != N))) SETERRQ4(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot change/reset column sizes to %D local %D global after previously setting them to %D local %D global",n,N,A->cmap->n,A->cmap->N);
   A->rmap->n = m;
   A->cmap->n = n;
-  A->rmap->N = M;
-  A->cmap->N = N;
+  A->rmap->N = M > -1 ? M : A->rmap->N;
+  A->cmap->N = N > -1 ? N : A->cmap->N;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatSetFromOptions"
 /*@
    MatSetFromOptions - Creates a matrix where the type is determined
    from the options database. Generates a parallel MPI matrix if the
@@ -234,6 +235,7 @@ PetscErrorCode  MatSetFromOptions(Mat B)
   ierr = PetscOptionsName("-mat_is_symmetric","Checks if mat is symmetric on MatAssemblyEnd()","MatIsSymmetric",&B->checksymmetryonassembly);CHKERRQ(ierr);
   ierr = PetscOptionsReal("-mat_is_symmetric","Checks if mat is symmetric on MatAssemblyEnd()","MatIsSymmetric",B->checksymmetrytol,&B->checksymmetrytol,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsBool("-mat_null_space_test","Checks if provided null space is correct in MatAssemblyEnd()","MatSetNullSpaceTest",B->checknullspaceonassembly,&B->checknullspaceonassembly,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-mat_error_if_failure","Generate an error if an error occurs when factoring the matrix","MatSetErrorIfFailure",B->erroriffailure,&B->erroriffailure,NULL);CHKERRQ(ierr);
 
   if (B->ops->setfromoptions) {
     ierr = (*B->ops->setfromoptions)(PetscOptionsObject,B);CHKERRQ(ierr);
@@ -247,15 +249,13 @@ PetscErrorCode  MatSetFromOptions(Mat B)
   if (set) {ierr = MatSetOption(B,MAT_NEW_NONZERO_ALLOCATION_ERR,flg);CHKERRQ(ierr);}
 
   /* process any options handlers added with PetscObjectAddOptionsHandler() */
-  ierr = PetscObjectProcessOptionsHandlers((PetscObject)B);CHKERRQ(ierr);
+  ierr = PetscObjectProcessOptionsHandlers(PetscOptionsObject,(PetscObject)B);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatXAIJSetPreallocation"
-/*@
-   MatXAIJSetPreallocation - set preallocation for serial and parallel AIJ, BAIJ, and SBAIJ matrices
+/*@C
+   MatXAIJSetPreallocation - set preallocation for serial and parallel AIJ, BAIJ, and SBAIJ matrices and their unassembled versions.
 
    Collective on Mat
 
@@ -276,6 +276,7 @@ PetscErrorCode MatXAIJSetPreallocation(Mat A,PetscInt bs,const PetscInt dnnz[],c
 {
   PetscErrorCode ierr;
   void           (*aij)(void);
+  void           (*is)(void);
 
   PetscFunctionBegin;
   ierr = MatSetBlockSize(A,bs);CHKERRQ(ierr);
@@ -287,17 +288,19 @@ PetscErrorCode MatXAIJSetPreallocation(Mat A,PetscInt bs,const PetscInt dnnz[],c
   ierr = MatSeqSBAIJSetPreallocation(A,bs,0,dnnzu);CHKERRQ(ierr);
   ierr = MatMPISBAIJSetPreallocation(A,bs,0,dnnzu,0,onnzu);CHKERRQ(ierr);
   /*
-    In general, we have to do extra work to preallocate for scalar (AIJ) matrices so we check whether it will do any
+    In general, we have to do extra work to preallocate for scalar (AIJ) or unassembled (IS) matrices so we check whether it will do any
     good before going on with it.
   */
   ierr = PetscObjectQueryFunction((PetscObject)A,"MatMPIAIJSetPreallocation_C",&aij);CHKERRQ(ierr);
-  if (!aij) {
+  ierr = PetscObjectQueryFunction((PetscObject)A,"MatISSetPreallocation_C",&is);CHKERRQ(ierr);
+  if (!aij && !is) {
     ierr = PetscObjectQueryFunction((PetscObject)A,"MatSeqAIJSetPreallocation_C",&aij);CHKERRQ(ierr);
   }
-  if (aij) {
+  if (aij || is) {
     if (bs == 1) {
       ierr = MatSeqAIJSetPreallocation(A,0,dnnz);CHKERRQ(ierr);
       ierr = MatMPIAIJSetPreallocation(A,0,dnnz,0,onnz);CHKERRQ(ierr);
+      ierr = MatISSetPreallocation(A,0,dnnz,0,onnz);CHKERRQ(ierr);
     } else {                    /* Convert block-row precallocation to scalar-row */
       PetscInt i,m,*sdnnz,*sonnz;
       ierr = MatGetLocalSize(A,&m,NULL);CHKERRQ(ierr);
@@ -308,6 +311,7 @@ PetscErrorCode MatXAIJSetPreallocation(Mat A,PetscInt bs,const PetscInt dnnz[],c
       }
       ierr = MatSeqAIJSetPreallocation(A,0,dnnz ? sdnnz : NULL);CHKERRQ(ierr);
       ierr = MatMPIAIJSetPreallocation(A,0,dnnz ? sdnnz : NULL,0,onnz ? sonnz : NULL);CHKERRQ(ierr);
+      ierr = MatISSetPreallocation(A,0,dnnz ? sdnnz : NULL,0,onnz ? sonnz : NULL);CHKERRQ(ierr);
       ierr = PetscFree2(sdnnz,sonnz);CHKERRQ(ierr);
     }
   }
@@ -319,16 +323,13 @@ PetscErrorCode MatXAIJSetPreallocation(Mat A,PetscInt bs,const PetscInt dnnz[],c
 
         This is somewhat different from MatHeaderReplace() it would be nice to merge the code
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatHeaderMerge"
-PetscErrorCode MatHeaderMerge(Mat A,Mat C)
+PetscErrorCode MatHeaderMerge(Mat A,Mat *C)
 {
   PetscErrorCode ierr;
   PetscInt       refct;
   PetscOps       Abops;
   struct _MatOps Aops;
   char           *mtype,*mname;
-  void           *spptr;
 
   PetscFunctionBegin;
   /* save the parts of A we need */
@@ -337,7 +338,6 @@ PetscErrorCode MatHeaderMerge(Mat A,Mat C)
   refct = ((PetscObject)A)->refct;
   mtype = ((PetscObject)A)->type_name;
   mname = ((PetscObject)A)->name;
-  spptr = A->spptr;
 
   /* zero these so the destroy below does not free them */
   ((PetscObject)A)->type_name = 0;
@@ -346,15 +346,14 @@ PetscErrorCode MatHeaderMerge(Mat A,Mat C)
   /* free all the interior data structures from mat */
   ierr = (*A->ops->destroy)(A);CHKERRQ(ierr);
 
-  ierr = PetscFree(C->spptr);CHKERRQ(ierr);
-
+  ierr = PetscFree(A->defaultvectype);CHKERRQ(ierr);
   ierr = PetscLayoutDestroy(&A->rmap);CHKERRQ(ierr);
   ierr = PetscLayoutDestroy(&A->cmap);CHKERRQ(ierr);
   ierr = PetscFunctionListDestroy(&((PetscObject)A)->qlist);CHKERRQ(ierr);
   ierr = PetscObjectListDestroy(&((PetscObject)A)->olist);CHKERRQ(ierr);
 
   /* copy C over to A */
-  ierr = PetscMemcpy(A,C,sizeof(struct _p_Mat));CHKERRQ(ierr);
+  ierr = PetscMemcpy(A,*C,sizeof(struct _p_Mat));CHKERRQ(ierr);
 
   /* return the parts of A we saved */
   ((PetscObject)A)->bops[0]   = Abops;
@@ -362,13 +361,12 @@ PetscErrorCode MatHeaderMerge(Mat A,Mat C)
   ((PetscObject)A)->refct     = refct;
   ((PetscObject)A)->type_name = mtype;
   ((PetscObject)A)->name      = mname;
-  A->spptr                    = spptr;
 
   /* since these two are copied into A we do not want them destroyed in C */
-  ((PetscObject)C)->qlist = 0;
-  ((PetscObject)C)->olist = 0;
+  ((PetscObject)*C)->qlist = 0;
+  ((PetscObject)*C)->olist = 0;
 
-  ierr = PetscHeaderDestroy(&C);CHKERRQ(ierr);
+  ierr = PetscHeaderDestroy(C);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 /*
@@ -380,36 +378,30 @@ PetscErrorCode MatHeaderMerge(Mat A,Mat C)
 
         Used in DM hence is declared PETSC_EXTERN
 */
-#undef __FUNCT__
-#define __FUNCT__ "MatHeaderReplace"
-PETSC_EXTERN PetscErrorCode MatHeaderReplace(Mat A,Mat C)
+PETSC_EXTERN PetscErrorCode MatHeaderReplace(Mat A,Mat *C)
 {
   PetscErrorCode   ierr;
   PetscInt         refct;
   PetscObjectState state;
+  struct _p_Mat    buffer;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(A,MAT_CLASSID,1);
-  PetscValidHeaderSpecific(C,MAT_CLASSID,2);
-  if (A == C) PetscFunctionReturn(0);
-  PetscCheckSameComm(A,1,C,2);
-  if (((PetscObject)C)->refct != 1) SETERRQ1(PetscObjectComm((PetscObject)C),PETSC_ERR_ARG_WRONGSTATE,"Object C has refct %D > 1, would leave hanging reference",((PetscObject)C)->refct);
+  PetscValidHeaderSpecific(*C,MAT_CLASSID,2);
+  if (A == *C) PetscFunctionReturn(0);
+  PetscCheckSameComm(A,1,*C,2);
+  if (((PetscObject)*C)->refct != 1) SETERRQ1(PetscObjectComm((PetscObject)C),PETSC_ERR_ARG_WRONGSTATE,"Object C has refct %D > 1, would leave hanging reference",((PetscObject)*C)->refct);
 
-  /* free all the interior data structures from mat */
-  ierr = (*A->ops->destroy)(A);CHKERRQ(ierr);
-  ierr = PetscHeaderDestroy_Private((PetscObject)A);CHKERRQ(ierr);
-  ierr = PetscLayoutDestroy(&A->rmap);CHKERRQ(ierr);
-  ierr = PetscLayoutDestroy(&A->cmap);CHKERRQ(ierr);
-  ierr = PetscFree(A->spptr);CHKERRQ(ierr);
-
-  /* copy C over to A */
+  /* swap C and A */
   refct = ((PetscObject)A)->refct;
   state = ((PetscObject)A)->state;
-  ierr  = PetscMemcpy(A,C,sizeof(struct _p_Mat));CHKERRQ(ierr);
-
+  ierr  = PetscMemcpy(&buffer,A,sizeof(struct _p_Mat));CHKERRQ(ierr);
+  ierr  = PetscMemcpy(A,*C,sizeof(struct _p_Mat));CHKERRQ(ierr);
+  ierr  = PetscMemcpy(*C,&buffer,sizeof(struct _p_Mat));CHKERRQ(ierr);
   ((PetscObject)A)->refct = refct;
   ((PetscObject)A)->state = state + 1;
 
-  ierr = PetscFree(C);CHKERRQ(ierr);
+  ((PetscObject)*C)->refct = 1;
+  ierr = MatDestroy(C);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
