@@ -34,10 +34,8 @@ extern PetscErrorCode FormInitialGuess(AppCtx*,DM,Vec);
 extern PetscErrorCode MatMult_MyShell(Mat,Vec,Vec);
 extern PetscErrorCode MatAssemblyEnd_MyShell(Mat,MatAssemblyType);
 extern PetscErrorCode PCApply_MyShell(PC,Vec,Vec);
-PETSC_EXTERN PetscErrorCode SNESComputeJacobian_MyShell(SNES,Vec,Mat,Mat,void*);
+extern PetscErrorCode SNESComputeJacobian_MyShell(SNES,Vec,Mat,Mat,void*);
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   AppCtx         user;                /* user-defined work context */
@@ -53,9 +51,7 @@ int main(int argc,char **argv)
   KSP            ksp;
   PetscBool      errorinmatmult = PETSC_FALSE,errorinpcapply = PETSC_FALSE,errorinpcsetup = PETSC_FALSE;
 
-  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return(1);
-
-  PetscFunctionBeginUser;
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   ierr = PetscOptionsGetBool(NULL,NULL,"-error_in_matmult",&errorinmatmult,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsGetBool(NULL,NULL,"-error_in_pcapply",&errorinpcapply,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsGetBool(NULL,NULL,"-error_in_pcsetup",&errorinpcsetup,NULL);CHKERRQ(ierr);
@@ -71,7 +67,9 @@ int main(int argc,char **argv)
       Create distributed array object to manage parallel grid and vectors
       for principal unknowns (x) and governing residuals (f)
   */
-  ierr = DMDACreate2d(PETSC_COMM_WORLD,DM_BOUNDARY_NONE,DM_BOUNDARY_NONE,DMDA_STENCIL_STAR,-4,-4,PETSC_DECIDE,PETSC_DECIDE,4,1,0,0,&da);CHKERRQ(ierr);
+  ierr = DMDACreate2d(PETSC_COMM_WORLD,DM_BOUNDARY_NONE,DM_BOUNDARY_NONE,DMDA_STENCIL_STAR,4,4,PETSC_DECIDE,PETSC_DECIDE,4,1,0,0,&da);CHKERRQ(ierr);
+  ierr = DMSetFromOptions(da);CHKERRQ(ierr);
+  ierr = DMSetUp(da);CHKERRQ(ierr);
   ierr = SNESSetDM(user.snes,da);CHKERRQ(ierr);
 
   ierr = DMDAGetInfo(da,0,&mx,&my,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,
@@ -149,14 +147,12 @@ int main(int argc,char **argv)
   ierr = DMDestroy(&da);CHKERRQ(ierr);
   ierr = SNESDestroy(&user.snes);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
 /* ------------------------------------------------------------------- */
 
 
-#undef __FUNCT__
-#define __FUNCT__ "FormInitialGuess"
 /*
    FormInitialGuess - Forms initial approximation.
 
@@ -216,8 +212,6 @@ PetscErrorCode FormInitialGuess(AppCtx *user,DM da,Vec X)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FormFunctionLocal"
 PetscErrorCode FormFunctionLocal(DMDALocalInfo *info,Field **x,Field **f,void *ptr)
 {
   AppCtx          *user = (AppCtx*)ptr;
@@ -354,8 +348,6 @@ PetscErrorCode FormFunctionLocal(DMDALocalInfo *info,Field **x,Field **f,void *p
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMult_MyShell"
 PetscErrorCode MatMult_MyShell(Mat A,Vec x,Vec y)
 {
   PetscErrorCode  ierr;
@@ -373,8 +365,6 @@ PetscErrorCode MatMult_MyShell(Mat A,Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "MatAssemblyEnd_MyShell"
 PetscErrorCode MatAssemblyEnd_MyShell(Mat A,MatAssemblyType tp)
 {
   PetscErrorCode ierr;
@@ -386,8 +376,6 @@ PetscErrorCode MatAssemblyEnd_MyShell(Mat A,MatAssemblyType tp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PCApply_MyShell"
 PetscErrorCode PCApply_MyShell(PC pc,Vec x,Vec y)
 {
   PetscErrorCode ierr;
@@ -405,8 +393,6 @@ PetscErrorCode PCApply_MyShell(PC pc,Vec x,Vec y)
 
 PETSC_EXTERN PetscErrorCode SNESComputeJacobian_DMDA(SNES,Vec,Mat,Mat,void*);
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESComputeJacobian_MyShell"
 PetscErrorCode SNESComputeJacobian_MyShell(SNES snes,Vec X,Mat A,Mat B,void *ctx)
 {
   static PetscInt fail = 0;
@@ -419,3 +405,45 @@ PetscErrorCode SNESComputeJacobian_MyShell(SNES snes,Vec X,Mat A,Mat B,void *ctx
   }
   PetscFunctionReturn(0);
 }
+
+
+/*TEST
+
+   test:
+      args: -snes_converged_reason -ksp_converged_reason
+
+   test:
+      suffix: 2
+      args: -snes_converged_reason -ksp_converged_reason -error_in_matmult
+
+   test:
+      suffix: 3
+      args: -snes_converged_reason -ksp_converged_reason -error_in_pcapply
+
+   test:
+      suffix: 4
+      args: -snes_converged_reason -ksp_converged_reason -error_in_pcsetup
+
+   test:
+      suffix: 5
+      args: -snes_converged_reason -ksp_converged_reason -error_in_pcsetup -pc_type bjacobi
+
+   test:
+      suffix: 5_fieldsplit
+      args: -snes_converged_reason -ksp_converged_reason -error_in_pcsetup -pc_type fieldsplit
+      output_file: output/ex69_5.out
+
+   test:
+      suffix: 6
+      args: -snes_converged_reason -ksp_converged_reason -error_in_domainmf -snes_mf
+
+   test:
+      suffix: 7
+      args: -snes_converged_reason -ksp_converged_reason -error_in_domain
+
+   test:
+      suffix: 8
+      args: -snes_converged_reason -ksp_converged_reason -error_in_domain -snes_mf
+      TODO: Need to determine if deprecated
+
+TEST*/
