@@ -19,6 +19,8 @@ The flow can be driven with the lid or with bouyancy or both:\n\
    Concepts: multicomponent
    Processors: n
 T*/
+
+
 /*F-----------------------------------------------------------------------
 
     We thank David E. Keyes for contributing the driven cavity discretization within this example code.
@@ -84,15 +86,13 @@ typedef struct {
 PetscErrorCode FormFunctionLocal(DMDALocalInfo*,Field**,Field**,void*);
 
 typedef struct {
-  PassiveReal lidvelocity,prandtl,grashof;  /* physical parameters */
+  PetscReal   lidvelocity,prandtl,grashof;  /* physical parameters */
   PetscBool   draw_contours;                /* flag - 1 indicates drawing contours */
 } AppCtx;
 
 extern PetscErrorCode FormInitialGuess(AppCtx*,DM,Vec);
 extern PetscErrorCode NonlinearGS(SNES,Vec,Vec,void*);
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   AppCtx         user;                /* user-defined work context */
@@ -113,12 +113,13 @@ int main(int argc,char **argv)
       Create distributed array object to manage parallel grid and vectors
       for principal unknowns (x) and governing residuals (f)
   */
-  ierr = DMDACreate2d(PETSC_COMM_WORLD,DM_BOUNDARY_NONE,DM_BOUNDARY_NONE,DMDA_STENCIL_STAR,-4,-4,PETSC_DECIDE,PETSC_DECIDE,4,1,0,0,&da);CHKERRQ(ierr);
+  ierr = DMDACreate2d(PETSC_COMM_WORLD,DM_BOUNDARY_NONE,DM_BOUNDARY_NONE,DMDA_STENCIL_STAR,4,4,PETSC_DECIDE,PETSC_DECIDE,4,1,0,0,&da);CHKERRQ(ierr);
+  ierr = DMSetFromOptions(da);CHKERRQ(ierr);
+  ierr = DMSetUp(da);CHKERRQ(ierr);
   ierr = SNESSetDM(snes,(DM)da);CHKERRQ(ierr);
   ierr = SNESSetNGS(snes, NonlinearGS, (void*)&user);CHKERRQ(ierr);
 
-  ierr = DMDAGetInfo(da,0,&mx,&my,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,
-                     PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE);CHKERRQ(ierr);
+  ierr = DMDAGetInfo(da,0,&mx,&my,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE);CHKERRQ(ierr);
   /*
      Problem parameters (velocity of lid, prandtl, and grashof numbers)
   */
@@ -126,10 +127,10 @@ int main(int argc,char **argv)
   user.prandtl     = 1.0;
   user.grashof     = 1.0;
 
-  ierr = PetscOptionsGetReal(NULL,"-lidvelocity",&user.lidvelocity,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetReal(NULL,"-prandtl",&user.prandtl,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetReal(NULL,"-grashof",&user.grashof,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsHasName(NULL,"-contours",&user.draw_contours);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-lidvelocity",&user.lidvelocity,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-prandtl",&user.prandtl,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-grashof",&user.grashof,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-contours",&user.draw_contours);CHKERRQ(ierr);
 
   ierr = DMDASetFieldName(da,0,"x_velocity");CHKERRQ(ierr);
   ierr = DMDASetFieldName(da,1,"y_velocity");CHKERRQ(ierr);
@@ -177,14 +178,11 @@ int main(int argc,char **argv)
   ierr = DMDestroy(&da);CHKERRQ(ierr);
   ierr = SNESDestroy(&snes);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
 /* ------------------------------------------------------------------- */
 
-
-#undef __FUNCT__
-#define __FUNCT__ "FormInitialGuess"
 /*
    FormInitialGuess - Forms initial approximation.
 
@@ -244,8 +242,6 @@ PetscErrorCode FormInitialGuess(AppCtx *user,DM da,Vec X)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FormFunctionLocal"
 PetscErrorCode FormFunctionLocal(DMDALocalInfo *info,Field **x,Field **f,void *ptr)
 {
   AppCtx         *user = (AppCtx*)ptr;
@@ -374,9 +370,6 @@ PetscErrorCode FormFunctionLocal(DMDALocalInfo *info,Field **x,Field **f,void *p
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "NonlinearGS"
 PetscErrorCode NonlinearGS(SNES snes, Vec X, Vec B, void *ctx)
 {
   DMDALocalInfo  info;
@@ -440,7 +433,6 @@ PetscErrorCode NonlinearGS(SNES snes, Vec X, Vec B, void *ctx)
   /* Test whether we are on the bottom edge of the global array */
   if (yints == 0) {
     j     = 0;
-    yints = yints + 1;
     /* bottom edge */
     for (i=info.xs; i<info.xs+info.xm; i++) {
 
@@ -459,7 +451,6 @@ PetscErrorCode NonlinearGS(SNES snes, Vec X, Vec B, void *ctx)
   /* Test whether we are on the top edge of the global array */
   if (yinte == info.my) {
     j     = info.my - 1;
-    yinte = yinte - 1;
     /* top edge */
     for (i=info.xs; i<info.xs+info.xm; i++) {
       if (B) {
@@ -477,7 +468,6 @@ PetscErrorCode NonlinearGS(SNES snes, Vec X, Vec B, void *ctx)
   /* Test whether we are on the left edge of the global array */
   if (xints == 0) {
     i     = 0;
-    xints = xints + 1;
     /* left edge */
     for (j=info.ys; j<info.ys+info.ym; j++) {
       if (B) {
@@ -495,7 +485,6 @@ PetscErrorCode NonlinearGS(SNES snes, Vec X, Vec B, void *ctx)
   /* Test whether we are on the right edge of the global array */
   if (xinte == info.mx) {
     i     = info.mx - 1;
-    xinte = xinte - 1;
     /* right edge */
     for (j=info.ys; j<info.ys+info.ym; j++) {
       if (B) {
@@ -515,7 +504,6 @@ PetscErrorCode NonlinearGS(SNES snes, Vec X, Vec B, void *ctx)
       for (i=info.xs; i<info.xs + info.xm; i++) {
         ptconverged = PETSC_FALSE;
         pfnorm0     = 0.0;
-        pfnorm      = 0.0;
         fu          = 0.0;
         fv          = 0.0;
         fomega      = 0.0;
@@ -572,8 +560,7 @@ PetscErrorCode NonlinearGS(SNES snes, Vec X, Vec B, void *ctx)
             u     = x[j][i].temp;
             uxx   = (2.0*u - x[j][i-1].temp - x[j][i+1].temp)*hydhx;
             uyy   = (2.0*u - x[j-1][i].temp - x[j+1][i].temp)*hxdhy;
-            ftemp =  uxx + uyy  + prandtl*((vxp*(u - x[j][i-1].temp) + vxm*(x[j][i+1].temp - u))*hy +
-                                           (vyp*(u - x[j-1][i].temp) + vym*(x[j+1][i].temp - u))*hx) - bjitemp;
+            ftemp =  uxx + uyy  + prandtl*((vxp*(u - x[j][i-1].temp) + vxm*(x[j][i+1].temp - u))*hy + (vyp*(u - x[j-1][i].temp) + vym*(x[j+1][i].temp - u))*hx) - bjitemp;
             dftdt = 2.0*(hydhx + hxdhy) + prandtl*((vxp - vxm)*hy + (vyp - vym)*hx);
             if (PetscRealPart(vx) > 0.0) dftdu = prandtl*(u - x[j][i-1].temp)*hy;
             else dftdu = prandtl*(x[j][i+1].temp - u)*hy;
@@ -590,8 +577,6 @@ PetscErrorCode NonlinearGS(SNES snes, Vec X, Vec B, void *ctx)
            */
             yu = fu / dfudu;
             yv = fv / dfvdv;
-            yo = fomega / dfodo;
-            yt = ftemp / dftdt;
             yo = (fomega - (dfodu*yu + dfodv*yv)) / dfodo;
             yt = (ftemp - (dftdu*yu + dftdv*yv)) / dftdt;
 
@@ -636,11 +621,11 @@ PetscErrorCode NonlinearGS(SNES snes, Vec X, Vec B, void *ctx)
           pfnorm = PetscRealPart(fu*fu + fv*fv + fomega*fomega + ftemp*ftemp);
           pfnorm = PetscSqrtReal(pfnorm);
           pynorm = PetscRealPart(yu*yu + yv*yv + yo*yo + yt*yt);
-          pfnorm = PetscSqrtReal(pynorm);
+          pynorm = PetscSqrtReal(pynorm);
           pxnorm = PetscRealPart(x[j][i].u*x[j][i].u + x[j][i].v*x[j][i].v + x[j][i].omega*x[j][i].omega + x[j][i].temp*x[j][i].temp);
           pxnorm = PetscSqrtReal(pxnorm);
           if (l == 0) pfnorm0 = pfnorm;
-          if (rtol*pfnorm0 > pfnorm ||
+          if (rtol*pfnorm0 >pfnorm ||
               atol > pfnorm ||
               pxnorm*stol > pynorm) ptconverged = PETSC_TRUE;
         }
@@ -660,3 +645,447 @@ PetscErrorCode NonlinearGS(SNES snes, Vec X, Vec B, void *ctx)
   }
   PetscFunctionReturn(0);
 }
+
+
+/*TEST
+
+   test:
+      nsize: 2
+      args: -da_refine 3 -snes_monitor_short -pc_type mg -ksp_type fgmres -pc_mg_type full
+      requires: !single
+
+   test:
+      suffix: 10
+      nsize: 3
+      args: -snes_monitor_short -ksp_monitor_short -pc_type fieldsplit -pc_fieldsplit_type symmetric_multiplicative -snes_view -da_refine 1 -ksp_type fgmres
+      requires: !single
+
+   test:
+      suffix: 11
+      nsize: 4
+      requires: pastix
+      args: -snes_monitor_short -pc_type redundant -dm_mat_type mpiaij -redundant_pc_factor_mat_solver_type pastix -pc_redundant_number 2 -da_refine 4 -ksp_type fgmres
+
+   test:
+      suffix: 12
+      nsize: 12
+      requires: pastix
+      args: -snes_monitor_short -pc_type redundant -dm_mat_type mpiaij -redundant_pc_factor_mat_solver_type pastix -pc_redundant_number 5 -da_refine 4 -ksp_type fgmres
+
+   test:
+      suffix: 13
+      nsize: 3
+      args: -snes_monitor_short -ksp_monitor_short -pc_type fieldsplit -pc_fieldsplit_type multiplicative -snes_view -da_refine 1 -ksp_type fgmres -snes_mf_operator
+      requires: !single
+
+   test:
+      suffix: 14
+      nsize: 4
+      args: -snes_monitor_short -pc_type mg -dm_mat_type baij -mg_coarse_pc_type bjacobi -da_refine 3 -ksp_type fgmres
+      requires: !single
+
+   test:
+      suffix: 14_ds
+      nsize: 4
+      args: -snes_converged_reason -pc_type mg -dm_mat_type baij -mg_coarse_pc_type bjacobi -da_refine 3 -ksp_type fgmres -mat_fd_type ds
+      output_file: output/ex19_2.out
+      requires: !single
+
+   test:
+      suffix: 17
+      args: -snes_monitor_short -ksp_pc_side right
+      requires: !single
+
+   test:
+      suffix: 18
+      args: -ksp_monitor_snes_lg -ksp_pc_side right
+      requires: x !single
+
+   test:
+      suffix: 2
+      nsize: 4
+      args: -da_refine 3 -snes_converged_reason -pc_type mg -mat_fd_type ds
+      requires: !single
+
+   test:
+      suffix: 2_bcols1
+      nsize: 4
+      args: -da_refine 3 -snes_converged_reason -pc_type mg -mat_fd_type ds -mat_fd_coloring_bcols 1> ex19_1.tmp 2>&1
+      output_file: output/ex19_2.out
+      requires: !single
+
+   test:
+      suffix: 3
+      nsize: 4
+      requires: mumps
+      args: -da_refine 3 -snes_monitor_short -pc_type redundant -dm_mat_type mpiaij -redundant_ksp_type preonly -redundant_pc_factor_mat_solver_type mumps -pc_redundant_number 2
+
+   test:
+      suffix: 4
+      nsize: 12
+      requires: mumps
+      args: -da_refine 3 -snes_monitor_short -pc_type redundant -dm_mat_type mpiaij -redundant_ksp_type preonly -redundant_pc_factor_mat_solver_type mumps -pc_redundant_number 5
+      output_file: output/ex19_3.out
+
+   test:
+      suffix: 6
+      args: -snes_monitor_short -ksp_monitor_short -pc_type fieldsplit -snes_view -ksp_type fgmres -da_refine 1
+      requires: !single
+
+   test:
+      suffix: 7
+      nsize: 3
+      args: -snes_monitor_short -ksp_monitor_short -pc_type fieldsplit -snes_view -da_refine 1 -ksp_type fgmres
+
+      requires: !single
+   test:
+      suffix: 8
+      args: -snes_monitor_short -ksp_monitor_short -pc_type fieldsplit -pc_fieldsplit_block_size 2 -pc_fieldsplit_0_fields 0,1 -pc_fieldsplit_1_fields 0,1 -pc_fieldsplit_type multiplicative -snes_view -fieldsplit_pc_type lu -da_refine 1 -ksp_type fgmres
+      requires: !single
+
+   test:
+      suffix: 9
+      nsize: 3
+      args: -snes_monitor_short -ksp_monitor_short -pc_type fieldsplit -pc_fieldsplit_type multiplicative -snes_view -da_refine 1 -ksp_type fgmres
+      requires: !single
+
+   test:
+      suffix: aspin
+      nsize: 4
+      args: -da_refine 3 -da_overlap 2 -snes_monitor_short -snes_type aspin -grashof 4e4 -lidvelocity 100 -ksp_monitor_short
+      requires: !single
+
+   test:
+      suffix: bcgsl
+      nsize: 2
+      args: -ksp_type bcgsl -ksp_monitor_short -da_refine 2 -ksp_bcgsl_ell 3 -snes_view
+      requires: !single
+
+   test:
+      suffix: bcols1
+      nsize: 2
+      args: -da_refine 3 -snes_monitor_short -pc_type mg -ksp_type fgmres -pc_mg_type full -mat_fd_coloring_bcols 1
+      output_file: output/ex19_1.out
+      requires: !single
+
+   test:
+      suffix: bjacobi
+      nsize: 4
+      args: -da_refine 4 -ksp_type fgmres -pc_type bjacobi -pc_bjacobi_blocks 2 -sub_ksp_type gmres -sub_ksp_max_it 2 -sub_pc_type bjacobi -sub_sub_ksp_type preonly -sub_sub_pc_type ilu -snes_monitor_short
+      requires: !single
+
+   test:
+      suffix: cgne
+      args: -da_refine 2 -pc_type lu -ksp_type cgne -ksp_monitor_short -ksp_converged_reason -ksp_view -ksp_norm_type unpreconditioned
+      filter: grep -v HERMITIAN
+      requires: !single
+
+   test:
+      suffix: cgs
+      args: -da_refine 1 -ksp_monitor_short -ksp_type cgs
+      requires: !single
+
+   test:
+      suffix: composite_fieldsplit
+      args: -ksp_type fgmres -pc_type composite -pc_composite_type MULTIPLICATIVE -pc_composite_pcs fieldsplit,none -sub_0_pc_fieldsplit_block_size 4 -sub_0_pc_fieldsplit_type additive -sub_0_pc_fieldsplit_0_fields 0,1,2 -sub_0_pc_fieldsplit_1_fields 3 -snes_monitor_short -ksp_monitor_short
+      requires: !single
+
+   test:
+      suffix: composite_fieldsplit_bjacobi
+      args: -ksp_type fgmres -pc_type composite -pc_composite_type MULTIPLICATIVE -pc_composite_pcs fieldsplit,bjacobi -sub_0_pc_fieldsplit_block_size 4 -sub_0_pc_fieldsplit_type additive -sub_0_pc_fieldsplit_0_fields 0,1,2 -sub_0_pc_fieldsplit_1_fields 3 -sub_1_pc_bjacobi_blocks 16 -sub_1_sub_pc_type lu -snes_monitor_short -ksp_monitor_short
+      requires: !single
+
+   test:
+      suffix: composite_fieldsplit_bjacobi_2
+      nsize: 4
+      args: -ksp_type fgmres -pc_type composite -pc_composite_type MULTIPLICATIVE -pc_composite_pcs fieldsplit,bjacobi -sub_0_pc_fieldsplit_block_size 4 -sub_0_pc_fieldsplit_type additive -sub_0_pc_fieldsplit_0_fields 0,1,2 -sub_0_pc_fieldsplit_1_fields 3 -sub_1_pc_bjacobi_blocks 16 -sub_1_sub_pc_type lu -snes_monitor_short -ksp_monitor_short
+      requires: !single
+
+   test:
+      suffix: composite_gs_newton
+      nsize: 2
+      args: -da_refine 3 -grashof 4e4 -lidvelocity 100 -snes_monitor_short -snes_type composite -snes_composite_type additiveoptimal -snes_composite_sneses ngs,newtonls -sub_0_snes_max_it 20 -sub_1_pc_type mg
+      requires: !single
+
+   test:
+      suffix: cuda
+      requires: cuda !single
+      args: -dm_vec_type cuda -dm_mat_type aijcusparse -pc_type none -ksp_type fgmres -snes_monitor_short -snes_rtol 1.e-5
+
+   test:
+      suffix: draw
+      args: -pc_type fieldsplit -snes_view draw -fieldsplit_x_velocity_pc_type mg -fieldsplit_x_velocity_pc_mg_galerkin pmat -fieldsplit_x_velocity_pc_mg_levels 2 -da_refine 1 -fieldsplit_x_velocity_mg_coarse_pc_type svd
+      requires: x !single
+
+   test:
+      suffix: drawports
+      args: -snes_monitor_solution draw::draw_ports -da_refine 1
+      output_file: output/ex19_draw.out
+      requires: x !single
+
+   test:
+      suffix: fas
+      args: -da_refine 4 -snes_monitor_short -snes_type fas -fas_levels_snes_type ngs -fas_levels_snes_ngs_sweeps 3 -fas_levels_snes_ngs_atol 0.0 -fas_levels_snes_ngs_stol 0.0 -grashof 4e4 -snes_fas_smoothup 6 -snes_fas_smoothdown 6 -lidvelocity 100
+      requires: !single
+
+   test:
+      suffix: fas_full
+      args: -da_refine 4 -snes_monitor_short -snes_type fas -snes_fas_type full -snes_fas_full_downsweep -fas_levels_snes_type ngs -fas_levels_snes_ngs_sweeps 3 -fas_levels_snes_ngs_atol 0.0 -fas_levels_snes_ngs_stol 0.0 -grashof 4e4 -snes_fas_smoothup 6 -snes_fas_smoothdown 6 -lidvelocity 100
+      requires: !single
+
+   test:
+      suffix: fdcoloring_ds
+      args: -da_refine 3 -snes_converged_reason -pc_type mg -mat_fd_type ds
+      output_file: output/ex19_2.out
+      requires: !single
+
+   test:
+      suffix: fdcoloring_ds_baij
+      args: -da_refine 3 -snes_converged_reason -pc_type mg -mat_fd_type ds -dm_mat_type baij
+      output_file: output/ex19_2.out
+      requires: !single
+
+   test:
+      suffix: fdcoloring_ds_bcols1
+      args: -da_refine 3 -snes_converged_reason -pc_type mg -mat_fd_type ds -mat_fd_coloring_bcols 1
+      output_file: output/ex19_2.out
+      requires: !single
+
+   test:
+      suffix: fdcoloring_wp
+      args: -da_refine 3 -snes_monitor_short -pc_type mg
+      requires: !single
+
+   test:
+      suffix: fdcoloring_wp_baij
+      args: -da_refine 3 -snes_monitor_short -pc_type mg -dm_mat_type baij
+      output_file: output/ex19_fdcoloring_wp.out
+      requires: !single
+
+   test:
+      suffix: fdcoloring_wp_bcols1
+      args: -da_refine 3 -snes_monitor_short -pc_type mg -mat_fd_coloring_bcols 1
+      output_file: output/ex19_fdcoloring_wp.out
+      requires: !single
+
+   test:
+      suffix: fieldsplit_2
+      args: -ksp_type fgmres -pc_type fieldsplit -pc_fieldsplit_block_size 4 -pc_fieldsplit_type additive -pc_fieldsplit_0_fields 0,1,2 -pc_fieldsplit_1_fields 3 -snes_monitor_short -ksp_monitor_short
+      requires: !single
+
+   test:
+      suffix: fieldsplit_3
+      args: -ksp_type fgmres -pc_type fieldsplit -pc_fieldsplit_block_size 4 -pc_fieldsplit_type additive -pc_fieldsplit_0_fields 0,1,2 -pc_fieldsplit_1_fields 3 -fieldsplit_0_pc_type lu -fieldsplit_1_pc_type lu -snes_monitor_short -ksp_monitor_short
+      requires: !single
+
+   test:
+      suffix: fieldsplit_4
+      args: -ksp_type fgmres -pc_type fieldsplit -pc_fieldsplit_block_size 4 -pc_fieldsplit_type SCHUR -pc_fieldsplit_0_fields 0,1,2 -pc_fieldsplit_1_fields 3 -fieldsplit_0_pc_type lu -fieldsplit_1_pc_type lu -snes_monitor_short -ksp_monitor_short
+      requires: !single
+
+   test:
+      suffix: fieldsplit_hypre
+      nsize: 2
+      requires: hypre mumps
+      args: -pc_type fieldsplit -pc_fieldsplit_block_size 4 -pc_fieldsplit_type SCHUR -pc_fieldsplit_0_fields 0,1,2 -pc_fieldsplit_1_fields 3 -fieldsplit_0_pc_type lu -fieldsplit_0_pc_factor_mat_solver_type mumps -fieldsplit_1_pc_type hypre -fieldsplit_1_pc_hypre_type boomeramg -snes_monitor_short -ksp_monitor_short
+
+   test:
+      suffix: fieldsplit_mumps
+      nsize: 2
+      requires: mumps
+      args: -pc_type fieldsplit -pc_fieldsplit_block_size 4 -pc_fieldsplit_type SCHUR -pc_fieldsplit_0_fields 0,1,2 -pc_fieldsplit_1_fields 3 -fieldsplit_0_pc_type lu -fieldsplit_1_pc_type lu -snes_monitor_short -ksp_monitor_short -fieldsplit_0_pc_factor_mat_solver_type mumps -fieldsplit_1_pc_factor_mat_solver_type mumps
+      output_file: output/ex19_fieldsplit_5.out
+
+   test:
+      suffix: greedy_coloring
+      nsize: 2
+      args: -da_refine 3 -snes_monitor_short -snes_fd_color -snes_fd_color_use_mat -mat_coloring_type greedy -mat_coloring_weight_type lf -mat_coloring_view> ex19_greedy_coloring.tmp 2>&1
+      requires: !single
+
+   test:
+      suffix: hypre
+      nsize: 2
+      requires: hypre
+      args: -da_refine 3 -snes_monitor_short -pc_type hypre
+
+   test:
+      suffix: ibcgs
+      nsize: 2
+      args: -ksp_type ibcgs -ksp_monitor_short -da_refine 2 -snes_view
+      requires: !complex !single
+
+   test:
+      suffix: kaczmarz
+      nsize: 2
+      args: -pc_type kaczmarz -ksp_monitor_short -snes_monitor_short -snes_view
+      requires: !single
+
+   test:
+      suffix: klu
+      requires: suitesparse
+      args: -da_grid_x 20 -da_grid_y 20 -pc_type lu -pc_factor_mat_solver_type klu
+      output_file: output/ex19_superlu.out
+
+   test:
+      suffix: klu_2
+      requires: suitesparse
+      args: -da_grid_x 20 -da_grid_y 20 -pc_type lu -pc_factor_mat_solver_type klu -mat_klu_ordering PETSC
+      output_file: output/ex19_superlu.out
+
+   test:
+      suffix: klu_3
+      requires: suitesparse
+      args: -da_grid_x 20 -da_grid_y 20 -pc_type lu -pc_factor_mat_solver_type klu -mat_klu_use_btf 0
+      output_file: output/ex19_superlu.out
+
+   test:
+      suffix: ml
+      nsize: 2
+      requires: ml
+      args: -da_refine 3 -snes_monitor_short -pc_type ml
+
+   test:
+      suffix: ngmres_fas
+      args: -da_refine 4 -snes_monitor_short -snes_type ngmres -npc_fas_levels_snes_type ngs -npc_fas_levels_snes_ngs_sweeps 3 -npc_fas_levels_snes_ngs_atol 0.0 -npc_fas_levels_snes_ngs_stol 0.0 -npc_snes_type fas -npc_fas_levels_snes_type ngs -npc_snes_max_it 1 -npc_snes_fas_smoothup 6 -npc_snes_fas_smoothdown 6 -lidvelocity 100 -grashof 4e4
+      requires: !single
+
+   test:
+      suffix: ngmres_fas_gssecant
+      args: -da_refine 3 -snes_monitor_short -snes_type ngmres -npc_snes_type fas -npc_fas_levels_snes_type ngs -npc_fas_levels_snes_max_it 6 -npc_fas_levels_snes_ngs_secant -npc_fas_levels_snes_ngs_max_it 1 -npc_fas_coarse_snes_max_it 1 -lidvelocity 100 -grashof 4e4
+      requires: !single
+
+   test:
+      suffix: ngmres_fas_ms
+      nsize: 2
+      args: -snes_grid_sequence 2 -lidvelocity 200 -grashof 1e4 -snes_monitor_short -snes_view -snes_converged_reason -snes_type ngmres -npc_snes_type fas -npc_fas_coarse_snes_type newtonls -npc_fas_coarse_ksp_type preonly -npc_snes_max_it 1
+      requires: !single
+
+   test:
+      suffix: ngmres_nasm
+      nsize: 4
+      args: -da_refine 4 -da_overlap 2 -snes_monitor_short -snes_type ngmres -snes_max_it 10 -npc_snes_type nasm -npc_snes_nasm_type basic -grashof 4e4 -lidvelocity 100
+      requires: !single
+
+   test:
+      suffix: ngs
+      args: -ksp_monitor_short -snes_type ngs -snes_view -snes_monitor -snes_rtol 1e-4
+      requires: !single
+
+   test:
+      suffix: ngs_fd
+      args: -ksp_monitor_short -snes_type ngs -snes_ngs_secant -snes_view -snes_monitor -snes_rtol 1e-4
+      requires: !single
+
+   test:
+      suffix: parms
+      nsize: 2
+      requires: parms
+      args: -pc_type parms -ksp_monitor_short -snes_view
+
+   test:
+      suffix: superlu
+      requires: superlu
+      args: -da_grid_x 20 -da_grid_y 20 -pc_type lu -pc_factor_mat_solver_type superlu
+
+   test:
+      suffix: superlu_sell
+      requires: superlu
+      args: -da_grid_x 20 -da_grid_y 20 -pc_type lu -pc_factor_mat_solver_type superlu -dm_mat_type sell -pc_factor_mat_ordering_type natural
+      output_file: output/ex19_superlu.out
+
+   test:
+      suffix: superlu_dist
+      requires: superlu_dist
+      args: -da_grid_x 20 -da_grid_y 20 -pc_type lu -pc_factor_mat_solver_type superlu_dist
+      output_file: output/ex19_superlu.out
+
+   test:
+      suffix: superlu_dist_2
+      nsize: 2
+      requires: superlu_dist
+      args: -da_grid_x 20 -da_grid_y 20 -pc_type lu -pc_factor_mat_solver_type superlu_dist
+      output_file: output/ex19_superlu.out
+
+   test:
+      suffix: superlu_equil
+      requires: superlu
+      args: -da_grid_x 20 -da_grid_y 20 -{snes,ksp}_monitor_short -pc_type lu -pc_factor_mat_solver_type superlu -mat_superlu_equil
+
+   test:
+      suffix: superlu_equil_sell
+      requires: superlu
+      args: -da_grid_x 20 -da_grid_y 20 -{snes,ksp}_monitor_short -pc_type lu -pc_factor_mat_solver_type superlu -mat_superlu_equil -dm_mat_type sell -pc_factor_mat_ordering_type natural
+      output_file: output/ex19_superlu_equil.out
+
+   test:
+      suffix: tcqmr
+      args: -da_refine 1 -ksp_monitor_short -ksp_type tcqmr
+      requires: !single
+
+   test:
+      suffix: tfqmr
+      args: -da_refine 1 -ksp_monitor_short -ksp_type tfqmr
+      requires: !single
+
+   test:
+      suffix: umfpack
+      requires: suitesparse
+      args: -da_refine 2 -pc_type lu -pc_factor_mat_solver_type umfpack -snes_view -snes_monitor_short -ksp_monitor_short
+
+   test:
+      suffix: tut_1
+      nsize: 4
+      requires: !single
+      args: -da_refine 5 -snes_monitor -ksp_monitor -snes_view
+
+   test:
+      suffix: tut_2
+      nsize: 4
+      requires: !single
+      args: -da_refine 5 -snes_monitor -ksp_monitor -snes_view -pc_type mg
+
+   test:
+      suffix: tut_3
+      nsize: 4
+      requires: hypre !single
+      args: -da_refine 5 -snes_monitor -ksp_monitor -snes_view -pc_type hypre
+
+   test:
+      suffix: tut_8
+      nsize: 4
+      requires: ml !single
+      args: -da_refine 5 -snes_monitor -ksp_monitor -snes_view -pc_type ml
+
+   test:
+      suffix: tut_4
+      nsize: 1
+      requires: !single
+      args: -da_refine 5 -log_view
+      filter: head -n 2
+      filter_output: head -n 2
+
+   test:
+      suffix: tut_5
+      nsize: 1
+      requires: !single
+      args: -da_refine 5 -log_view -pc_type mg
+      filter: head -n 2
+      filter_output: head -n 2
+
+   test:
+      suffix: tut_6
+      nsize: 4
+      requires: !single
+      args: -da_refine 5 -log_view
+      filter: head -n 2
+      filter_output: head -n 2
+
+   test:
+      suffix: tut_7
+      nsize: 4
+      requires: !single
+      args: -da_refine 5 -log_view -pc_type mg
+      filter: head -n 2
+      filter_output: head -n 2
+
+TEST*/
