@@ -23,6 +23,10 @@ static const char help[] = "1D multiphysics prototype with analytic Jacobians to
  * without copying values to extract submatrices.
  */
 
+/*T
+   TODO: Need to determine if deprecated
+T*/
+
 #include <petscsnes.h>
 #include <petscdm.h>
 #include <petscdmda.h>
@@ -35,8 +39,6 @@ struct _UserCtx {
   Vec      Uloc,Kloc;
 };
 
-#undef __FUNCT__
-#define __FUNCT__ "FormFunctionLocal_U"
 static PetscErrorCode FormFunctionLocal_U(User user,DMDALocalInfo *info,const PetscScalar u[],const PetscScalar k[],PetscScalar f[])
 {
   PetscReal hx = 1./info->mx;
@@ -51,8 +53,6 @@ static PetscErrorCode FormFunctionLocal_U(User user,DMDALocalInfo *info,const Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FormFunctionLocal_K"
 static PetscErrorCode FormFunctionLocal_K(User user,DMDALocalInfo *info,const PetscScalar u[],const PetscScalar k[],PetscScalar f[])
 {
   PetscReal hx = 1./info->mx;
@@ -70,8 +70,6 @@ static PetscErrorCode FormFunctionLocal_K(User user,DMDALocalInfo *info,const Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FormFunction_All"
 static PetscErrorCode FormFunction_All(SNES snes,Vec X,Vec F,void *ctx)
 {
   User           user = (User)ctx;
@@ -130,8 +128,6 @@ static PetscErrorCode FormFunction_All(SNES snes,Vec X,Vec F,void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FormJacobianLocal_U"
 static PetscErrorCode FormJacobianLocal_U(User user,DMDALocalInfo *info,const PetscScalar u[],const PetscScalar k[],Mat Buu)
 {
   PetscReal      hx = 1./info->mx;
@@ -154,8 +150,6 @@ static PetscErrorCode FormJacobianLocal_U(User user,DMDALocalInfo *info,const Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FormJacobianLocal_K"
 static PetscErrorCode FormJacobianLocal_K(User user,DMDALocalInfo *info,const PetscScalar u[],const PetscScalar k[],Mat Bkk)
 {
   PetscReal      hx = 1./info->mx;
@@ -171,8 +165,6 @@ static PetscErrorCode FormJacobianLocal_K(User user,DMDALocalInfo *info,const Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FormJacobianLocal_UK"
 static PetscErrorCode FormJacobianLocal_UK(User user,DMDALocalInfo *info,DMDALocalInfo *infok,const PetscScalar u[],const PetscScalar k[],Mat Buk)
 {
   PetscReal      hx = 1./info->mx;
@@ -193,8 +185,6 @@ static PetscErrorCode FormJacobianLocal_UK(User user,DMDALocalInfo *info,DMDALoc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FormJacobianLocal_KU"
 static PetscErrorCode FormJacobianLocal_KU(User user,DMDALocalInfo *info,DMDALocalInfo *infok,const PetscScalar u[],const PetscScalar k[],Mat Bku)
 {
   PetscErrorCode ierr;
@@ -228,8 +218,6 @@ static PetscErrorCode FormJacobianLocal_KU(User user,DMDALocalInfo *info,DMDALoc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FormJacobian_All"
 static PetscErrorCode FormJacobian_All(SNES snes,Vec X,Mat J,Mat B,void *ctx)
 {
   User           user = (User)ctx;
@@ -264,8 +252,9 @@ static PetscErrorCode FormJacobian_All(SNES snes,Vec X,Mat J,Mat B,void *ctx)
     ierr = DMDAVecRestoreArray(dak,Kloc,&k);CHKERRQ(ierr);
     break;
   case 2: {
-    Mat Buu,Buk,Bku,Bkk;
-    IS  *is;
+    Mat       Buu,Buk,Bku,Bkk;
+    PetscBool nest;
+    IS        *is;
     ierr = DMCompositeScatter(user->pack,X,Uloc,Kloc);CHKERRQ(ierr);
     ierr = DMDAVecGetArray(dau,Uloc,&u);CHKERRQ(ierr);
     ierr = DMDAVecGetArray(dak,Kloc,&k);CHKERRQ(ierr);
@@ -275,8 +264,17 @@ static PetscErrorCode FormJacobian_All(SNES snes,Vec X,Mat J,Mat B,void *ctx)
     ierr = MatGetLocalSubMatrix(B,is[1],is[0],&Bku);CHKERRQ(ierr);
     ierr = MatGetLocalSubMatrix(B,is[1],is[1],&Bkk);CHKERRQ(ierr);
     ierr = FormJacobianLocal_U(user,&infou,u,k,Buu);CHKERRQ(ierr);
-    ierr = FormJacobianLocal_UK(user,&infou,&infok,u,k,Buk);CHKERRQ(ierr);
-    ierr = FormJacobianLocal_KU(user,&infou,&infok,u,k,Bku);CHKERRQ(ierr);
+    ierr = PetscObjectTypeCompare((PetscObject)B,MATNEST,&nest);CHKERRQ(ierr);
+    if (!nest) {
+      /*
+         DMCreateMatrix_Composite()  for a nested matrix does not generate off-block matrices that one can call MatSetValuesLocal() on, it just creates dummy
+         matrices with no entries; there cannot insert values into them. Commit b6480e041dd2293a65f96222772d68cdb4ed6306
+         changed Mat_Nest() from returning NULL pointers for these submatrices to dummy matrices because PCFIELDSPLIT could not
+         handle the returned null matrices.
+      */
+      ierr = FormJacobianLocal_UK(user,&infou,&infok,u,k,Buk);CHKERRQ(ierr);
+      ierr = FormJacobianLocal_KU(user,&infou,&infok,u,k,Bku);CHKERRQ(ierr);
+    }
     ierr = FormJacobianLocal_K(user,&infok,u,k,Bkk);CHKERRQ(ierr);
     ierr = MatRestoreLocalSubMatrix(B,is[0],is[0],&Buu);CHKERRQ(ierr);
     ierr = MatRestoreLocalSubMatrix(B,is[0],is[1],&Buk);CHKERRQ(ierr);
@@ -300,8 +298,6 @@ static PetscErrorCode FormJacobian_All(SNES snes,Vec X,Mat J,Mat B,void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FormInitial_Coupled"
 static PetscErrorCode FormInitial_Coupled(User user,Vec X)
 {
   PetscErrorCode ierr;
@@ -328,8 +324,6 @@ static PetscErrorCode FormInitial_Coupled(User user,Vec X)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc, char *argv[])
 {
   PetscErrorCode ierr;
@@ -341,12 +335,13 @@ int main(int argc, char *argv[])
   Vec            X,F,Xu,Xk,Fu,Fk;
   Mat            B;
   IS             *isg;
-  PetscBool      view_draw,pass_dm;
+  PetscBool      pass_dm;
 
-  PetscInitialize(&argc,&argv,0,help);
-  ierr = DMDACreate1d(PETSC_COMM_WORLD,DM_BOUNDARY_NONE,-10,1,1,NULL,&dau);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&argv,0,help);if (ierr) return ierr;
+  ierr = DMDACreate1d(PETSC_COMM_WORLD,DM_BOUNDARY_NONE,10,1,1,NULL,&dau);CHKERRQ(ierr);
   ierr = DMSetOptionsPrefix(dau,"u_");CHKERRQ(ierr);
   ierr = DMSetFromOptions(dau);CHKERRQ(ierr);
+  ierr = DMSetUp(dau);CHKERRQ(ierr);
   ierr = DMDAGetOwnershipRanges(dau,&lxu,0,0);CHKERRQ(ierr);
   ierr = DMDAGetInfo(dau,0, &m,0,0, &sizes,0,0, 0,0,0,0,0,0);CHKERRQ(ierr);
   ierr = PetscMalloc1(sizes,&lxk);CHKERRQ(ierr);
@@ -355,6 +350,7 @@ int main(int argc, char *argv[])
   ierr = DMDACreate1d(PETSC_COMM_WORLD,DM_BOUNDARY_NONE,m-1,1,1,lxk,&dak);CHKERRQ(ierr);
   ierr = DMSetOptionsPrefix(dak,"k_");CHKERRQ(ierr);
   ierr = DMSetFromOptions(dak);CHKERRQ(ierr);
+  ierr = DMSetUp(dak);CHKERRQ(ierr);
   ierr = PetscFree(lxk);CHKERRQ(ierr);
 
   ierr = DMCompositeCreate(PETSC_COMM_WORLD,&pack);CHKERRQ(ierr);
@@ -378,10 +374,9 @@ int main(int argc, char *argv[])
 
   ierr = PetscOptionsBegin(PETSC_COMM_WORLD,NULL,"Coupled problem options","SNES");CHKERRQ(ierr);
   {
-    user->ptype = 0; view_draw = PETSC_FALSE; pass_dm = PETSC_TRUE;
+    user->ptype = 0; pass_dm = PETSC_TRUE;
 
     ierr = PetscOptionsInt("-problem_type","0: solve for u only, 1: solve for k only, 2: solve for both",0,user->ptype,&user->ptype,NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsBool("-view_draw","Draw the final coupled solution regardless of whether only one physics was solved",0,view_draw,&view_draw,NULL);CHKERRQ(ierr);
     ierr = PetscOptionsBool("-pass_dm","Pass the packed DM to SNES to use when determining splits and forward into splits",0,pass_dm,&pass_dm,NULL);CHKERRQ(ierr);
   }
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
@@ -437,16 +432,17 @@ int main(int argc, char *argv[])
     ierr = SNESSolve(snes,NULL,X);CHKERRQ(ierr);
     break;
   }
-  if (view_draw) {ierr = VecView(X,PETSC_VIEWER_DRAW_WORLD);CHKERRQ(ierr);}
+  ierr = VecViewFromOptions(X,NULL,"-view_sol");CHKERRQ(ierr);
+
   if (0) {
     PetscInt  col      = 0;
     PetscBool mult_dup = PETSC_FALSE,view_dup = PETSC_FALSE;
     Mat       D;
     Vec       Y;
 
-    ierr = PetscOptionsGetInt(0,"-col",&col,0);CHKERRQ(ierr);
-    ierr = PetscOptionsGetBool(0,"-mult_dup",&mult_dup,0);CHKERRQ(ierr);
-    ierr = PetscOptionsGetBool(0,"-view_dup",&view_dup,0);CHKERRQ(ierr);
+    ierr = PetscOptionsGetInt(NULL,0,"-col",&col,0);CHKERRQ(ierr);
+    ierr = PetscOptionsGetBool(NULL,0,"-mult_dup",&mult_dup,0);CHKERRQ(ierr);
+    ierr = PetscOptionsGetBool(NULL,0,"-view_dup",&view_dup,0);CHKERRQ(ierr);
 
     ierr = VecDuplicate(X,&Y);CHKERRQ(ierr);
     /* ierr = MatAssemblyBegin(B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr); */
@@ -477,6 +473,43 @@ int main(int argc, char *argv[])
   ierr = DMDestroy(&dak);CHKERRQ(ierr);
   ierr = DMDestroy(&pack);CHKERRQ(ierr);
   ierr = SNESDestroy(&snes);CHKERRQ(ierr);
-  PetscFinalize();
-  return 0;
+  ierr = PetscFinalize();
+  return ierr;
 }
+
+/*TEST
+
+   build:
+      requires: c99
+
+   test:
+      suffix: 0
+      nsize: 3
+      args: -u_da_grid_x 20 -snes_converged_reason -snes_monitor_short -problem_type 0
+
+   test:
+      suffix: 1
+      nsize: 3
+      args: -u_da_grid_x 20 -snes_converged_reason -snes_monitor_short -problem_type 1
+
+   test:
+      suffix: 2
+      nsize: 3
+      args: -u_da_grid_x 20 -snes_converged_reason -snes_monitor_short -problem_type 2
+
+   test:
+      suffix: 3
+      nsize: 3
+      args: -u_da_grid_x 20 -snes_converged_reason -snes_monitor_short -ksp_monitor_short -problem_type 2 -snes_mf_operator -pack_dm_mat_type {{aij nest}} -pc_type fieldsplit -pc_fieldsplit_dm_splits -pc_fieldsplit_type additive -fieldsplit_u_ksp_type gmres -fieldsplit_k_pc_type jacobi
+
+   test:
+      suffix: 4
+      nsize: 6
+      args: -u_da_grid_x 257 -snes_converged_reason -snes_monitor_short -ksp_monitor_short -problem_type 2 -snes_mf_operator -pack_dm_mat_type aij -pc_type fieldsplit -pc_fieldsplit_type multiplicative -fieldsplit_u_ksp_type gmres -fieldsplit_u_ksp_pc_side right -fieldsplit_u_pc_type mg -fieldsplit_u_pc_mg_levels 4 -fieldsplit_u_mg_levels_ksp_type richardson -fieldsplit_u_mg_levels_ksp_max_it 1 -fieldsplit_u_mg_levels_pc_type sor -fieldsplit_u_pc_mg_galerkin pmat -fieldsplit_u_ksp_converged_reason -fieldsplit_k_pc_type jacobi
+      requires: !single
+
+   test:
+      suffix: glvis_composite_da_1d
+      args: -u_da_grid_x 20 -problem_type 0 -snes_monitor_solution glvis:
+
+TEST*/
