@@ -1,29 +1,33 @@
-
 static char help[] = "Newton methods to solve u'' + u^{2} = f in parallel.\n\
 This example employs a user-defined monitoring routine and optionally a user-defined\n\
-routine to check candidate iterates produced by line search routines.  This code also\n\
-demonstrates use of the macro __FUNCT__ to define routine names for use in error handling.\n\
+routine to check candidate iterates produced by line search routines.\n\
 The command line options include:\n\
-  -check_iterates : activate checking of iterates\n\
-  -check_tol <tol>: set tolerance for iterate checking\n\n";
+  -pre_check_iterates : activate checking of iterates\n\
+  -post_check_iterates : activate checking of iterates\n\
+  -check_tol <tol>: set tolerance for iterate checking\n\
+  -user_precond : activate a (trivial) user-defined preconditioner\n\n";
 
 /*T
    Concepts: SNES^basic parallel example
    Concepts: SNES^setting a user-defined monitoring routine
-   Concepts: error handling^using the macro __FUNCT__ to define routine names;
    Processors: n
 T*/
 
+
+
 /*
-   Include "petscdraw.h" so that we can use distributed arrays (DMDAs).
-   Include "petscdraw.h" so that we can use PETSc drawing routines.
+   Include "petscdm.h" so that we can use data management objects (DMs)
+   Include "petscdmda.h" so that we can use distributed arrays (DMDAs).
    Include "petscsnes.h" so that we can use SNES solvers.  Note that this
    file automatically includes:
-     petscsys.h       - base PETSc routines   petscvec.h - vectors
-     petscmat.h - matrices
-     petscis.h     - index sets            petscksp.h - Krylov subspace methods
-     petscviewer.h - viewers               petscpc.h  - preconditioners
-     petscksp.h   - linear solvers
+     petscsys.h    - base PETSc routines
+     petscvec.h    - vectors
+     petscmat.h    - matrices
+     petscis.h     - index sets
+     petscksp.h    - Krylov subspace methods
+     petscviewer.h - viewers
+     petscpc.h     - preconditioners
+     petscksp.h    - linear solvers
 */
 
 #include <petscdm.h>
@@ -31,14 +35,7 @@ T*/
 #include <petscsnes.h>
 
 /*
-   User-defined routines.  Note that immediately before each routine below,
-   we define the macro __FUNCT__ to be a string containing the routine name.
-   If defined, this macro is used in the PETSc error handlers to provide a
-   complete traceback of routine names.  All PETSc library routines use this
-   macro, and users can optionally employ it as well in their application
-   codes.  Note that users can get a traceback of PETSc errors regardless of
-   whether they define __FUNCT__ in application codes; this macro merely
-   provides the added traceback detail of the application routine names.
+   User-defined routines.
 */
 PetscErrorCode FormJacobian(SNES,Vec,Mat,Mat,void*);
 PetscErrorCode FormFunction(SNES,Vec,Vec,void*);
@@ -81,12 +78,10 @@ typedef struct {
   PetscInt its0; /* num of prevous outer KSP iterations */
 } SetSubKSPCtx;
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   SNES           snes;                 /* SNES context */
-  SNESLineSearch linesearch;          /* SNESLineSearch context */
+  SNESLineSearch linesearch;           /* SNESLineSearch context */
   Mat            J;                    /* Jacobian matrix */
   ApplicationCtx ctx;                  /* user-defined context */
   Vec            x,r,U,F;              /* vectors */
@@ -100,11 +95,10 @@ int main(int argc,char **argv)
   PetscReal      abstol,rtol,stol,norm;
   PetscBool      flg;
 
-
-  PetscInitialize(&argc,&argv,(char*)0,help);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   ierr  = MPI_Comm_rank(PETSC_COMM_WORLD,&ctx.rank);CHKERRQ(ierr);
   ierr  = MPI_Comm_size(PETSC_COMM_WORLD,&ctx.size);CHKERRQ(ierr);
-  ierr  = PetscOptionsGetInt(NULL,"-n",&N,NULL);CHKERRQ(ierr);
+  ierr  = PetscOptionsGetInt(NULL,NULL,"-n",&N,NULL);CHKERRQ(ierr);
   ctx.h = 1.0/(N-1);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -121,6 +115,8 @@ int main(int argc,char **argv)
      Create distributed array (DMDA) to manage parallel grid and vectors
   */
   ierr = DMDACreate1d(PETSC_COMM_WORLD,DM_BOUNDARY_NONE,N,1,1,NULL,&ctx.da);CHKERRQ(ierr);
+  ierr = DMSetFromOptions(ctx.da);CHKERRQ(ierr);
+  ierr = DMSetUp(ctx.da);CHKERRQ(ierr);
 
   /*
      Extract global and local vectors from DMDA; then duplicate for remaining
@@ -162,9 +158,9 @@ int main(int argc,char **argv)
   ierr = SNESSetJacobian(snes,J,J,FormJacobian,&ctx);CHKERRQ(ierr);
 
   /*
-     Optional allow user provided preconditioner
+     Optionally allow user-provided preconditioner
    */
-  ierr = PetscOptionsHasName(NULL,"-user_precond",&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-user_precond",&flg);CHKERRQ(ierr);
   if (flg) {
     KSP ksp;
     PC  pc;
@@ -201,7 +197,7 @@ int main(int argc,char **argv)
      iterates that are determined by line search methods
   */
   ierr = SNESGetLineSearch(snes, &linesearch);CHKERRQ(ierr);
-  ierr = PetscOptionsHasName(NULL,"-post_check_iterates",&post_check);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-post_check_iterates",&post_check);CHKERRQ(ierr);
 
   if (post_check) {
     ierr = PetscPrintf(PETSC_COMM_WORLD,"Activating post step checking routine\n");CHKERRQ(ierr);
@@ -211,16 +207,16 @@ int main(int argc,char **argv)
     checkP.tolerance = 1.0;
     checkP.user      = &ctx;
 
-    ierr = PetscOptionsGetReal(NULL,"-check_tol",&checkP.tolerance,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsGetReal(NULL,NULL,"-check_tol",&checkP.tolerance,NULL);CHKERRQ(ierr);
   }
 
-  ierr = PetscOptionsHasName(NULL,"-post_setsubksp",&post_setsubksp);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-post_setsubksp",&post_setsubksp);CHKERRQ(ierr);
   if (post_setsubksp) {
     ierr = PetscPrintf(PETSC_COMM_WORLD,"Activating post setsubksp\n");CHKERRQ(ierr);
     ierr = SNESLineSearchSetPostCheck(linesearch,PostSetSubKSP,&checkP1);CHKERRQ(ierr);
   }
 
-  ierr = PetscOptionsHasName(NULL,"-pre_check_iterates",&pre_check);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-pre_check_iterates",&pre_check);CHKERRQ(ierr);
   if (pre_check) {
     ierr = PetscPrintf(PETSC_COMM_WORLD,"Activating pre step checking routine\n");CHKERRQ(ierr);
     ierr = SNESLineSearchSetPreCheck(linesearch,PreCheck,&checkP);CHKERRQ(ierr);
@@ -307,11 +303,10 @@ int main(int argc,char **argv)
   ierr = SNESDestroy(&snes);CHKERRQ(ierr);
   ierr = DMDestroy(&ctx.da);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  PetscFunctionReturn(0);
+  return ierr;
 }
+
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "FormInitialGuess"
 /*
    FormInitialGuess - Computes initial guess.
 
@@ -327,9 +322,8 @@ PetscErrorCode FormInitialGuess(Vec x)
   ierr = VecSet(x,pfive);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "FormFunction"
 /*
    FormFunction - Evaluates nonlinear function, F(x).
 
@@ -411,9 +405,8 @@ PetscErrorCode FormFunction(SNES snes,Vec x,Vec f,void *ctx)
   ierr = DMRestoreLocalVector(da,&xlocal);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "FormJacobian"
 /*
    FormJacobian - Evaluates Jacobian matrix.
 
@@ -492,9 +485,8 @@ PetscErrorCode FormJacobian(SNES snes,Vec x,Mat jac,Mat B,void *ctx)
 
   PetscFunctionReturn(0);
 }
+
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "Monitor"
 /*
    Monitor - Optional user-defined monitoring routine that views the
    current iterate with an x-window plot. Set by SNESMonitorSet().
@@ -524,8 +516,6 @@ PetscErrorCode Monitor(SNES snes,PetscInt its,PetscReal fnorm,void *ctx)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "PreCheck"
 /*
    PreCheck - Optional user-defined routine that checks the validity of
    candidate steps of a line search method.  Set by SNESLineSearchSetPreCheck().
@@ -547,8 +537,6 @@ PetscErrorCode PreCheck(SNESLineSearch linesearch,Vec xcurrent,Vec y, PetscBool
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "PostCheck"
 /*
    PostCheck - Optional user-defined routine that checks the validity of
    candidate steps of a line search method.  Set by SNESLineSearchSetPostCheck().
@@ -585,7 +573,6 @@ PetscErrorCode PostCheck(SNESLineSearch linesearch,Vec xcurrent,Vec y,Vec x,Pets
   check = (StepCheckCtx*)ctx;
   user  = check->user;
   ierr  = SNESGetIterationNumber(snes,&iter);CHKERRQ(ierr);
-  ierr  = SNESLineSearchGetPreCheck(linesearch, NULL, (void**)&check);CHKERRQ(ierr);
 
   /* iteration 1 indicates we are working on the second iteration */
   if (iter > 0) {
@@ -610,8 +597,7 @@ PetscErrorCode PostCheck(SNESLineSearch linesearch,Vec xcurrent,Vec y,Vec x,Pets
         tmp        = xa[i];
         xa[i]      = .5*(xa[i] + xa_last[i]);
         *changed_x = PETSC_TRUE;
-        ierr       = PetscPrintf(PETSC_COMM_WORLD,"  Altering entry %D: x=%g, x_last=%g, diff=%g, x_new=%g\n",
-                                 i,(double)PetscAbsScalar(tmp),(double)PetscAbsScalar(xa_last[i]),(double)rdiff,(double)PetscAbsScalar(xa[i]));CHKERRQ(ierr);
+        ierr       = PetscPrintf(PETSC_COMM_WORLD,"  Altering entry %D: x=%g, x_last=%g, diff=%g, x_new=%g\n",i,(double)PetscAbsScalar(tmp),(double)PetscAbsScalar(xa_last[i]),(double)rdiff,(double)PetscAbsScalar(xa[i]));CHKERRQ(ierr);
       }
     }
     ierr = DMDAVecRestoreArray(da,check->last_step,&xa_last);CHKERRQ(ierr);
@@ -621,10 +607,7 @@ PetscErrorCode PostCheck(SNESLineSearch linesearch,Vec xcurrent,Vec y,Vec x,Pets
   PetscFunctionReturn(0);
 }
 
-
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "PostSetSubKSP"
 /*
    PostSetSubKSP - Optional user-defined routine that reset SubKSP options when hierarchical bjacobi PC is used
    e.g,
@@ -696,8 +679,29 @@ PetscErrorCode MatrixFreePreconditioner(PC pc,Vec x,Vec y)
 }
 
 
+/*TEST
 
+   test:
+      args: -nox -snes_monitor_cancel -snes_monitor_short -ksp_gmres_cgs_refinement_type refine_always
 
+   test:
+      suffix: 2
+      nsize: 3
+      args: -nox -pc_type asm -mat_type mpiaij -snes_monitor_cancel -snes_monitor_short -ksp_gmres_cgs_refinement_type refine_always
 
+   test:
+      suffix: 3
+      nsize: 2
+      args: -nox -snes_monitor_cancel -snes_monitor_short -ksp_gmres_cgs_refinement_type refine_always
 
+   test:
+      suffix: 4
+      args: -nox -pre_check_iterates -post_check_iterates
 
+   test:
+      suffix: 5
+      requires: double !complex !single
+      nsize: 2
+      args: -nox -snes_test_jacobian  -snes_test_jacobian_view
+
+TEST*/
