@@ -120,11 +120,8 @@ extern PetscErrorCode ViscosityField(DM,Vec,Vec);
 extern PetscErrorCode StressField(DM);
 extern PetscErrorCode SNESConverged_Interactive(SNES, PetscInt, PetscReal, PetscReal, PetscReal, SNESConvergedReason*, void*);
 extern PetscErrorCode InteractiveHandler(int, void*);
-extern PetscBool  OptionsHasName(const char pre[],const char name[]);
 
 /*-----------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 /*-----------------------------------------------------------------------*/
 {
@@ -137,13 +134,13 @@ int main(int argc,char **argv)
   MPI_Comm       comm;
   DM             da;
 
-  PetscInitialize(&argc,&argv,(char*)0,help);
-  PetscOptionsSetValue("-file","ex30_output");
-  PetscOptionsSetValue("-snes_monitor_short",NULL);
-  PetscOptionsSetValue("-snes_max_it","20");
-  PetscOptionsSetValue("-ksp_max_it","1500");
-  PetscOptionsSetValue("-ksp_gmres_restart","300");
-  PetscOptionsInsert(&argc,&argv,NULL);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
+  PetscOptionsSetValue(NULL,"-file","ex30_output");
+  PetscOptionsSetValue(NULL,"-snes_monitor_short",NULL);
+  PetscOptionsSetValue(NULL,"-snes_max_it","20");
+  PetscOptionsSetValue(NULL,"-ksp_max_it","1500");
+  PetscOptionsSetValue(NULL,"-ksp_gmres_restart","300");
+  PetscOptionsInsert(NULL,&argc,&argv,NULL);
 
   comm = PETSC_COMM_WORLD;
 
@@ -157,6 +154,8 @@ int main(int argc,char **argv)
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   ierr = SNESCreate(comm,&snes);CHKERRQ(ierr);
   ierr = DMDACreate2d(comm,grid.bx,grid.by,grid.stencil,grid.ni,grid.nj,PETSC_DECIDE,PETSC_DECIDE,grid.dof,grid.stencil_width,0,0,&da);CHKERRQ(ierr);
+  ierr = DMSetFromOptions(da);CHKERRQ(ierr);
+  ierr = DMSetUp(da);CHKERRQ(ierr);
   ierr = SNESSetDM(snes,da);CHKERRQ(ierr);
   ierr = DMDASetFieldName(da,0,"x-velocity");CHKERRQ(ierr);
   ierr = DMDASetFieldName(da,1,"y-velocity");CHKERRQ(ierr);
@@ -205,7 +204,7 @@ int main(int argc,char **argv)
   ierr = DMDestroy(&da);CHKERRQ(ierr);
   ierr = PetscPopSignalHandler();CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
 /*=====================================================================
@@ -213,14 +212,12 @@ int main(int argc,char **argv)
   =====================================================================*/
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "UpdateSolution"
 /*  manages solve: adaptive continuation method  */
 PetscErrorCode UpdateSolution(SNES snes, AppCtx *user, PetscInt *nits)
 {
   KSP                 ksp;
   PC                  pc;
-  SNESConvergedReason reason;
+  SNESConvergedReason reason = SNES_CONVERGED_ITERATING;
   Parameter           *param   = user->param;
   PetscReal           cont_incr=0.3;
   PetscInt            its;
@@ -268,8 +265,8 @@ PetscErrorCode UpdateSolution(SNES snes, AppCtx *user, PetscInt *nits)
 
       if (reason<0) {
         /* NOT converged */
-        cont_incr = -fabs(cont_incr)/2.0;
-        if (fabs(cont_incr)<0.01) goto done;
+        cont_incr = -PetscAbsReal(cont_incr)/2.0;
+        if (PetscAbsReal(cont_incr)<0.01) goto done;
 
       } else {
         /* converged */
@@ -284,8 +281,8 @@ PetscErrorCode UpdateSolution(SNES snes, AppCtx *user, PetscInt *nits)
     }
   }
 done:
-  if (param->stop_solve && !q) PetscPrintf(PETSC_COMM_WORLD,"USER SIGNAL: stopping solve.\n");
-  if (reason<0 && !q) PetscPrintf(PETSC_COMM_WORLD,"FAILED TO CONVERGE: stopping solve.\n");
+  if (param->stop_solve && !q) {ierr = PetscPrintf(PETSC_COMM_WORLD,"USER SIGNAL: stopping solve.\n");CHKERRQ(ierr);}
+  if (reason<0 && !q) {ierr = PetscPrintf(PETSC_COMM_WORLD,"FAILED TO CONVERGE: stopping solve.\n");CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
 
@@ -296,8 +293,6 @@ done:
 
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "UInterp"
 PETSC_STATIC_INLINE PetscScalar UInterp(Field **x, PetscInt i, PetscInt j)
 /*---------------------------------------------------------------------*/
 {
@@ -305,8 +300,6 @@ PETSC_STATIC_INLINE PetscScalar UInterp(Field **x, PetscInt i, PetscInt j)
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "WInterp"
 PETSC_STATIC_INLINE PetscScalar WInterp(Field **x, PetscInt i, PetscInt j)
 /*---------------------------------------------------------------------*/
 {
@@ -314,8 +307,6 @@ PETSC_STATIC_INLINE PetscScalar WInterp(Field **x, PetscInt i, PetscInt j)
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "PInterp"
 PETSC_STATIC_INLINE PetscScalar PInterp(Field **x, PetscInt i, PetscInt j)
 /*---------------------------------------------------------------------*/
 {
@@ -323,8 +314,6 @@ PETSC_STATIC_INLINE PetscScalar PInterp(Field **x, PetscInt i, PetscInt j)
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TInterp"
 PETSC_STATIC_INLINE PetscScalar TInterp(Field **x, PetscInt i, PetscInt j)
 /*---------------------------------------------------------------------*/
 {
@@ -332,10 +321,8 @@ PETSC_STATIC_INLINE PetscScalar TInterp(Field **x, PetscInt i, PetscInt j)
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "HorizVelocity"
 /*  isoviscous analytic solution for IC */
-PETSC_STATIC_INLINE PassiveScalar HorizVelocity(PetscInt i, PetscInt j, AppCtx *user)
+PETSC_STATIC_INLINE PetscScalar HorizVelocity(PetscInt i, PetscInt j, AppCtx *user)
 /*---------------------------------------------------------------------*/
 {
   Parameter   *param = user->param;
@@ -347,13 +334,11 @@ PETSC_STATIC_INLINE PassiveScalar HorizVelocity(PetscInt i, PetscInt j, AppCtx *
   r  = PetscSqrtReal(x*x+z*z);
   st = z/r;
   ct = x/r;
-  th = atan(z/x);
+  th = PetscAtanReal(z/x);
   return ct*(c*th*st+d*(st+th*ct)) + st*(c*(st-th*ct)+d*th*st);
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "VertVelocity"
 /*  isoviscous analytic solution for IC */
 PETSC_STATIC_INLINE PetscScalar VertVelocity(PetscInt i, PetscInt j, AppCtx *user)
 /*---------------------------------------------------------------------*/
@@ -364,13 +349,11 @@ PETSC_STATIC_INLINE PetscScalar VertVelocity(PetscInt i, PetscInt j, AppCtx *use
   PetscReal   x, z, r;
 
   x = (i - grid->jlid - 0.5)*grid->dx;  z = (j - grid->jlid)*grid->dz;
-  r = PetscSqrtReal(x*x+z*z); st = z/r;  ct = x/r;  th = atan(z/x);
+  r = PetscSqrtReal(x*x+z*z); st = z/r;  ct = x/r;  th = PetscAtanReal(z/x);
   return st*(c*th*st+d*(st+th*ct)) - ct*(c*(st-th*ct)+d*th*st);
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "Pressure"
 /*  isoviscous analytic solution for IC */
 PETSC_STATIC_INLINE PetscScalar Pressure(PetscInt i, PetscInt j, AppCtx *user)
 /*---------------------------------------------------------------------*/
@@ -384,8 +367,6 @@ PETSC_STATIC_INLINE PetscScalar Pressure(PetscInt i, PetscInt j, AppCtx *user)
   return (-2.0*(c*ct-d*st)/r);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CalcSecInv"
 /*  computes the second invariant of the strain rate tensor */
 PETSC_STATIC_INLINE PetscScalar CalcSecInv(Field **x, PetscInt i, PetscInt j, PetscInt ipos, AppCtx *user)
 /*---------------------------------------------------------------------*/
@@ -445,10 +426,8 @@ PETSC_STATIC_INLINE PetscScalar CalcSecInv(Field **x, PetscInt i, PetscInt j, Pe
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "Viscosity"
 /*  computes the shear viscosity */
-PETSC_STATIC_INLINE PetscScalar Viscosity(PetscScalar T, PetscScalar eps, PassiveScalar z, Parameter *param)
+PETSC_STATIC_INLINE PetscScalar Viscosity(PetscScalar T, PetscScalar eps, PetscScalar z, Parameter *param)
 /*---------------------------------------------------------------------*/
 {
   PetscReal   result   =0.0;
@@ -491,8 +470,6 @@ PETSC_STATIC_INLINE PetscScalar Viscosity(PetscScalar T, PetscScalar eps, Passiv
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "XMomentumResidual"
 /*  computes the residual of the x-component of eqn (1) above */
 PETSC_STATIC_INLINE PetscScalar XMomentumResidual(Field **x, PetscInt i, PetscInt j, AppCtx *user)
 /*---------------------------------------------------------------------*/
@@ -547,8 +524,6 @@ PETSC_STATIC_INLINE PetscScalar XMomentumResidual(Field **x, PetscInt i, PetscIn
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "ZMomentumResidual"
 /*  computes the residual of the z-component of eqn (1) above */
 PETSC_STATIC_INLINE PetscScalar ZMomentumResidual(Field **x, PetscInt i, PetscInt j, AppCtx *user)
 /*---------------------------------------------------------------------*/
@@ -606,8 +581,6 @@ PETSC_STATIC_INLINE PetscScalar ZMomentumResidual(Field **x, PetscInt i, PetscIn
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "ContinuityResidual"
 /*  computes the residual of eqn (2) above */
 PETSC_STATIC_INLINE PetscScalar ContinuityResidual(Field **x, PetscInt i, PetscInt j, AppCtx *user)
 /*---------------------------------------------------------------------*/
@@ -622,8 +595,6 @@ PETSC_STATIC_INLINE PetscScalar ContinuityResidual(Field **x, PetscInt i, PetscI
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "EnergyResidual"
 /*  computes the residual of eqn (3) above */
 PETSC_STATIC_INLINE PetscScalar EnergyResidual(Field **x, PetscInt i, PetscInt j, AppCtx *user)
 /*---------------------------------------------------------------------*/
@@ -684,8 +655,6 @@ PETSC_STATIC_INLINE PetscScalar EnergyResidual(Field **x, PetscInt i, PetscInt j
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "ShearStress"
 /*  computes the shear stress---used on the boundaries */
 PETSC_STATIC_INLINE PetscScalar ShearStress(Field **x, PetscInt i, PetscInt j, PetscInt ipos, AppCtx *user)
 /*---------------------------------------------------------------------*/
@@ -721,8 +690,6 @@ PETSC_STATIC_INLINE PetscScalar ShearStress(Field **x, PetscInt i, PetscInt j, P
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "XNormalStress"
 /*  computes the normal stress---used on the boundaries */
 PETSC_STATIC_INLINE PetscScalar XNormalStress(Field **x, PetscInt i, PetscInt j, PetscInt ipos, AppCtx *user)
 /*---------------------------------------------------------------------*/
@@ -761,8 +728,6 @@ PETSC_STATIC_INLINE PetscScalar XNormalStress(Field **x, PetscInt i, PetscInt j,
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "ZNormalStress"
 /*  computes the normal stress---used on the boundaries */
 PETSC_STATIC_INLINE PetscScalar ZNormalStress(Field **x, PetscInt i, PetscInt j, PetscInt ipos, AppCtx *user)
 /*---------------------------------------------------------------------*/
@@ -805,8 +770,6 @@ PETSC_STATIC_INLINE PetscScalar ZNormalStress(Field **x, PetscInt i, PetscInt j,
   =====================================================================*/
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "SetParams"
 /* initializes the problem parameters and checks for
    command line changes */
 PetscErrorCode SetParams(Parameter *param, GridInfo *grid)
@@ -823,25 +786,25 @@ PetscErrorCode SetParams(Parameter *param, GridInfo *grid)
   param->lid_depth   = 35.0;                                               /* km */
   param->fault_depth = 35.0;                                               /* km */
 
-  ierr = PetscOptionsGetReal(NULL,"-slab_dip",&(param->slab_dip),NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetReal(NULL,"-width",&(param->width),NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetReal(NULL,"-depth",&(param->depth),NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetReal(NULL,"-lid_depth",&(param->lid_depth),NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetReal(NULL,"-fault_depth",&(param->fault_depth),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-slab_dip",&(param->slab_dip),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-width",&(param->width),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-depth",&(param->depth),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-lid_depth",&(param->lid_depth),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-fault_depth",&(param->fault_depth),NULL);CHKERRQ(ierr);
 
   param->slab_dip = param->slab_dip*PETSC_PI/180.0;                    /* radians */
 
   /* grid information */
-  ierr     = PetscOptionsGetInt(NULL, "-jfault",&(grid->jfault),NULL);CHKERRQ(ierr);
+  ierr     = PetscOptionsGetInt(NULL,NULL, "-jfault",&(grid->jfault),NULL);CHKERRQ(ierr);
   grid->ni = 82;
-  ierr     = PetscOptionsGetInt(NULL, "-ni",&(grid->ni),NULL);CHKERRQ(ierr);
+  ierr     = PetscOptionsGetInt(NULL,NULL, "-ni",&(grid->ni),NULL);CHKERRQ(ierr);
 
   grid->dx            = param->width/((PetscReal)(grid->ni-2));               /* km */
-  grid->dz            = grid->dx*tan(param->slab_dip);                     /* km */
+  grid->dz            = grid->dx*PetscTanReal(param->slab_dip);               /* km */
   grid->nj            = (PetscInt)(param->depth/grid->dz + 3.0);         /* gridpoints*/
   param->depth        = grid->dz*(grid->nj-2);                             /* km */
   grid->inose         = 0;                                          /* gridpoints*/
-  ierr                = PetscOptionsGetInt(NULL,"-inose",&(grid->inose),NULL);CHKERRQ(ierr);
+  ierr                = PetscOptionsGetInt(NULL,NULL,"-inose",&(grid->inose),NULL);CHKERRQ(ierr);
   grid->bx            = DM_BOUNDARY_NONE;
   grid->by            = DM_BOUNDARY_NONE;
   grid->stencil       = DMDA_STENCIL_BOX;
@@ -852,7 +815,7 @@ PetscErrorCode SetParams(Parameter *param, GridInfo *grid)
   /* boundary conditions */
   param->pv_analytic = PETSC_FALSE;
   param->ibound      = BC_NOSTRESS;
-  ierr               = PetscOptionsGetInt(NULL,"-ibound",&(param->ibound),NULL);CHKERRQ(ierr);
+  ierr               = PetscOptionsGetInt(NULL,NULL,"-ibound",&(param->ibound),NULL);CHKERRQ(ierr);
 
   /* physical constants */
   param->slab_velocity = 5.0;               /* cm/yr */
@@ -861,11 +824,11 @@ PetscErrorCode SetParams(Parameter *param, GridInfo *grid)
   param->kappa         = 0.7272e-6;         /* m^2/sec */
   param->potentialT    = 1300.0;            /* degrees C */
 
-  ierr = PetscOptionsGetReal(NULL,"-slab_velocity",&(param->slab_velocity),NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetReal(NULL,"-slab_age",&(param->slab_age),NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetReal(NULL,"-lid_age",&(param->lid_age),NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetReal(NULL,"-kappa",&(param->kappa),NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetReal(NULL,"-potentialT",&(param->potentialT),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-slab_velocity",&(param->slab_velocity),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-slab_age",&(param->slab_age),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-lid_age",&(param->lid_age),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-kappa",&(param->kappa),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-potentialT",&(param->potentialT),NULL);CHKERRQ(ierr);
 
   /* viscosity */
   param->ivisc        = 3;                  /* 0=isovisc, 1=difn creep, 2=disl creep, 3=full */
@@ -885,26 +848,26 @@ PetscErrorCode SetParams(Parameter *param, GridInfo *grid)
   param->dislocation.Estar = 530e3;           /* J/mol */
   param->dislocation.Vstar = 14e-6;           /* m^3/mol */
 
-  ierr = PetscOptionsGetInt(NULL, "-ivisc",&(param->ivisc),NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetReal(NULL,"-visc_cutoff",&(param->visc_cutoff),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL, "-ivisc",&(param->ivisc),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-visc_cutoff",&(param->visc_cutoff),NULL);CHKERRQ(ierr);
 
   param->output_ivisc = param->ivisc;
 
-  ierr = PetscOptionsGetInt(NULL,"-output_ivisc",&(param->output_ivisc),NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetReal(NULL,"-vstar",&(param->dislocation.Vstar),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-output_ivisc",&(param->output_ivisc),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-vstar",&(param->dislocation.Vstar),NULL);CHKERRQ(ierr);
 
   /* output options */
   param->quiet      = PETSC_FALSE;
   param->param_test = PETSC_FALSE;
 
-  ierr = PetscOptionsHasName(NULL,"-quiet",&(param->quiet));CHKERRQ(ierr);
-  ierr = PetscOptionsHasName(NULL,"-test",&(param->param_test));CHKERRQ(ierr);
-  ierr = PetscOptionsGetString(NULL,"-file",param->filename,PETSC_MAX_PATH_LEN,&(param->output_to_file));
+  ierr = PetscOptionsHasName(NULL,NULL,"-quiet",&(param->quiet));CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(NULL,NULL,"-test",&(param->param_test));CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-file",param->filename,PETSC_MAX_PATH_LEN,&(param->output_to_file));CHKERRQ(ierr);
 
   /* advection */
   param->adv_scheme = ADVECT_FROMM;       /* advection scheme: 0=finite vol, 1=Fromm */
 
-  ierr = PetscOptionsGetInt(NULL,"-adv_scheme",&(param->adv_scheme),NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-adv_scheme",&(param->adv_scheme),NULL);CHKERRQ(ierr);
 
   /* misc. flags */
   param->stop_solve    = PETSC_FALSE;
@@ -939,14 +902,12 @@ PetscErrorCode SetParams(Parameter *param, GridInfo *grid)
                         / param->kappa;                         /* m^2/sec */
   param->z_scale = param->L * alpha_g_on_cp_units_inverse_km;
   param->skt     = PetscSqrtReal(param->kappa*param->slab_age*SEC_PER_YR);
-  ierr           = PetscOptionsGetReal(NULL,"-peclet",&(param->peclet),NULL);CHKERRQ(ierr);
+  ierr           = PetscOptionsGetReal(NULL,NULL,"-peclet",&(param->peclet),NULL);CHKERRQ(ierr);
 
   return ierr_out;
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "ReportParams"
 /*  prints a report of the problem parameters to stdout */
 PetscErrorCode ReportParams(Parameter *param, GridInfo *grid)
 /*---------------------------------------------------------------------*/
@@ -957,20 +918,20 @@ PetscErrorCode ReportParams(Parameter *param, GridInfo *grid)
   ierr = PetscGetDate(date,30);CHKERRQ(ierr);
 
   if (!(param->quiet)) {
-    PetscPrintf(PETSC_COMM_WORLD,"---------------------BEGIN ex30 PARAM REPORT-------------------\n");
+    ierr = PetscPrintf(PETSC_COMM_WORLD,"---------------------BEGIN ex30 PARAM REPORT-------------------\n");CHKERRQ(ierr);
     /* PetscPrintf(PETSC_COMM_WORLD,"                   %s\n",&(date[0]));*/
 
-    PetscPrintf(PETSC_COMM_WORLD,"Domain: \n");
+    ierr = PetscPrintf(PETSC_COMM_WORLD,"Domain: \n");CHKERRQ(ierr);
     ierr = PetscPrintf(PETSC_COMM_WORLD,"  Width = %g km,         Depth = %g km\n",(double)param->width,(double)param->depth);CHKERRQ(ierr);
     ierr = PetscPrintf(PETSC_COMM_WORLD,"  Slab dip = %g degrees,  Slab velocity = %g cm/yr\n",(double)(param->slab_dip*180.0/PETSC_PI),(double)param->slab_velocity);CHKERRQ(ierr);
-    ierr = PetscPrintf(PETSC_COMM_WORLD,"  Lid depth = %5.2f km,   Fault depth = %5.2f km\n",param->lid_depth*param->L,param->fault_depth*param->L);CHKERRQ(ierr);
+    ierr = PetscPrintf(PETSC_COMM_WORLD,"  Lid depth = %5.2f km,   Fault depth = %5.2f km\n",(double)(param->lid_depth*param->L),(double)(param->fault_depth*param->L));CHKERRQ(ierr);
 
-    PetscPrintf(PETSC_COMM_WORLD,"\nGrid: \n");
-    ierr = PetscPrintf(PETSC_COMM_WORLD,"  [ni,nj] = %D, %D       [dx,dz] = %g, %g km\n",grid->ni,grid->nj,(double)grid->dx*param->L,(double)(grid->dz*param->L));CHKERRQ(ierr);
+    ierr = PetscPrintf(PETSC_COMM_WORLD,"\nGrid: \n");CHKERRQ(ierr);
+    ierr = PetscPrintf(PETSC_COMM_WORLD,"  [ni,nj] = %D, %D       [dx,dz] = %g, %g km\n",grid->ni,grid->nj,(double)(grid->dx*param->L),(double)(grid->dz*param->L));CHKERRQ(ierr);
     ierr = PetscPrintf(PETSC_COMM_WORLD,"  jlid = %3D              jfault = %3D \n",grid->jlid,grid->jfault);CHKERRQ(ierr);
     ierr = PetscPrintf(PETSC_COMM_WORLD,"  Pe = %g\n",(double)param->peclet);CHKERRQ(ierr);
 
-    PetscPrintf(PETSC_COMM_WORLD,"\nRheology:");
+    ierr = PetscPrintf(PETSC_COMM_WORLD,"\nRheology:");CHKERRQ(ierr);
     if (param->ivisc==VISC_CONST) {
       ierr = PetscPrintf(PETSC_COMM_WORLD,"                 Isoviscous \n");CHKERRQ(ierr);
       if (param->pv_analytic) {
@@ -990,7 +951,7 @@ PetscErrorCode ReportParams(Parameter *param, GridInfo *grid)
       ierr_out = 1;
     }
 
-    PetscPrintf(PETSC_COMM_WORLD,"Boundary condition:");
+    ierr = PetscPrintf(PETSC_COMM_WORLD,"Boundary condition:");CHKERRQ(ierr);
     if (param->ibound==BC_ANALYTIC) {
       ierr = PetscPrintf(PETSC_COMM_WORLD,"       Isoviscous Analytic Dirichlet \n");CHKERRQ(ierr);
     } else if (param->ibound==BC_NOSTRESS) {
@@ -1004,12 +965,12 @@ PetscErrorCode ReportParams(Parameter *param, GridInfo *grid)
 
     if (param->output_to_file)
 #if defined(PETSC_HAVE_MATLAB_ENGINE)
-      PetscPrintf(PETSC_COMM_WORLD,"Output Destination:       Mat file \"%s\"\n",param->filename);
+      ierr = PetscPrintf(PETSC_COMM_WORLD,"Output Destination:       Mat file \"%s\"\n",param->filename);CHKERRQ(ierr);
 #else
-      PetscPrintf(PETSC_COMM_WORLD,"Output Destination:       PETSc binary file \"%s\"\n",param->filename);
+      ierr = PetscPrintf(PETSC_COMM_WORLD,"Output Destination:       PETSc binary file \"%s\"\n",param->filename);CHKERRQ(ierr);
 #endif
     if (param->output_ivisc != param->ivisc) {
-      PetscPrintf(PETSC_COMM_WORLD,"                          Output viscosity: -ivisc %D\n",param->output_ivisc);
+      ierr = PetscPrintf(PETSC_COMM_WORLD,"                          Output viscosity: -ivisc %D\n",param->output_ivisc);CHKERRQ(ierr);
     }
 
     ierr = PetscPrintf(PETSC_COMM_WORLD,"---------------------END ex30 PARAM REPORT---------------------\n");CHKERRQ(ierr);
@@ -1019,8 +980,6 @@ PetscErrorCode ReportParams(Parameter *param, GridInfo *grid)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "Initialize"
 /*  generates an inital guess using the analytic solution for isoviscous
     corner flow */
 PetscErrorCode Initialize(DM da)
@@ -1067,8 +1026,6 @@ PetscErrorCode Initialize(DM da)
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "DoOutput"
 /*  controls output to a file */
 PetscErrorCode DoOutput(SNES snes, PetscInt its)
 /*---------------------------------------------------------------------*/
@@ -1105,23 +1062,23 @@ PetscErrorCode DoOutput(SNES snes, PetscInt its)
     if (!rank) { /* on processor 0 */
       ierr = VecSetSizes(pars, 20, PETSC_DETERMINE);CHKERRQ(ierr);
       ierr = VecSetFromOptions(pars);CHKERRQ(ierr);
-      ierr = VecSetValue(pars,0, (PetscScalar)(grid->ni),INSERT_VALUES);
-      ierr = VecSetValue(pars,1, (PetscScalar)(grid->nj),INSERT_VALUES);
-      ierr = VecSetValue(pars,2, (PetscScalar)(grid->dx),INSERT_VALUES);
-      ierr = VecSetValue(pars,3, (PetscScalar)(grid->dz),INSERT_VALUES);
-      ierr = VecSetValue(pars,4, (PetscScalar)(param->L),INSERT_VALUES);
-      ierr = VecSetValue(pars,5, (PetscScalar)(param->V),INSERT_VALUES);
+      ierr = VecSetValue(pars,0, (PetscScalar)(grid->ni),INSERT_VALUES);CHKERRQ(ierr);
+      ierr = VecSetValue(pars,1, (PetscScalar)(grid->nj),INSERT_VALUES);CHKERRQ(ierr);
+      ierr = VecSetValue(pars,2, (PetscScalar)(grid->dx),INSERT_VALUES);CHKERRQ(ierr);
+      ierr = VecSetValue(pars,3, (PetscScalar)(grid->dz),INSERT_VALUES);CHKERRQ(ierr);
+      ierr = VecSetValue(pars,4, (PetscScalar)(param->L),INSERT_VALUES);CHKERRQ(ierr);
+      ierr = VecSetValue(pars,5, (PetscScalar)(param->V),INSERT_VALUES);CHKERRQ(ierr);
       /* skipped 6 intentionally */
-      ierr = VecSetValue(pars,7, (PetscScalar)(param->slab_dip),INSERT_VALUES);
-      ierr = VecSetValue(pars,8, (PetscScalar)(grid->jlid),INSERT_VALUES);
-      ierr = VecSetValue(pars,9, (PetscScalar)(param->lid_depth),INSERT_VALUES);
-      ierr = VecSetValue(pars,10,(PetscScalar)(grid->jfault),INSERT_VALUES);
-      ierr = VecSetValue(pars,11,(PetscScalar)(param->fault_depth),INSERT_VALUES);
-      ierr = VecSetValue(pars,12,(PetscScalar)(param->potentialT),INSERT_VALUES);
-      ierr = VecSetValue(pars,13,(PetscScalar)(param->ivisc),INSERT_VALUES);
-      ierr = VecSetValue(pars,14,(PetscScalar)(param->visc_cutoff),INSERT_VALUES);
-      ierr = VecSetValue(pars,15,(PetscScalar)(param->ibound),INSERT_VALUES);
-      ierr = VecSetValue(pars,16,(PetscScalar)(its),INSERT_VALUES);
+      ierr = VecSetValue(pars,7, (PetscScalar)(param->slab_dip),INSERT_VALUES);CHKERRQ(ierr);
+      ierr = VecSetValue(pars,8, (PetscScalar)(grid->jlid),INSERT_VALUES);CHKERRQ(ierr);
+      ierr = VecSetValue(pars,9, (PetscScalar)(param->lid_depth),INSERT_VALUES);CHKERRQ(ierr);
+      ierr = VecSetValue(pars,10,(PetscScalar)(grid->jfault),INSERT_VALUES);CHKERRQ(ierr);
+      ierr = VecSetValue(pars,11,(PetscScalar)(param->fault_depth),INSERT_VALUES);CHKERRQ(ierr);
+      ierr = VecSetValue(pars,12,(PetscScalar)(param->potentialT),INSERT_VALUES);CHKERRQ(ierr);
+      ierr = VecSetValue(pars,13,(PetscScalar)(param->ivisc),INSERT_VALUES);CHKERRQ(ierr);
+      ierr = VecSetValue(pars,14,(PetscScalar)(param->visc_cutoff),INSERT_VALUES);CHKERRQ(ierr);
+      ierr = VecSetValue(pars,15,(PetscScalar)(param->ibound),INSERT_VALUES);CHKERRQ(ierr);
+      ierr = VecSetValue(pars,16,(PetscScalar)(its),INSERT_VALUES);CHKERRQ(ierr);
     } else { /* on some other processor */
       ierr = VecSetSizes(pars, 0, PETSC_DETERMINE);CHKERRQ(ierr);
       ierr = VecSetFromOptions(pars);CHKERRQ(ierr);
@@ -1136,16 +1093,16 @@ PetscErrorCode DoOutput(SNES snes, PetscInt its)
 #endif
 
     /* send vectors to viewer */
-    ierr = PetscObjectSetName((PetscObject)res,"res");
+    ierr = PetscObjectSetName((PetscObject)res,"res");CHKERRQ(ierr);
     ierr = VecView(res,viewer);CHKERRQ(ierr);
-    ierr = PetscObjectSetName((PetscObject)user->x,"out");
+    ierr = PetscObjectSetName((PetscObject)user->x,"out");CHKERRQ(ierr);
     ierr = VecView(user->x, viewer);CHKERRQ(ierr);
-    ierr = PetscObjectSetName((PetscObject)(user->Xguess),"aux");
+    ierr = PetscObjectSetName((PetscObject)(user->Xguess),"aux");CHKERRQ(ierr);
     ierr = VecView(user->Xguess, viewer);CHKERRQ(ierr);
     ierr = StressField(da);CHKERRQ(ierr); /* compute stress fields */
-    ierr = PetscObjectSetName((PetscObject)(user->Xguess),"str");
+    ierr = PetscObjectSetName((PetscObject)(user->Xguess),"str");CHKERRQ(ierr);
     ierr = VecView(user->Xguess, viewer);CHKERRQ(ierr);
-    ierr = PetscObjectSetName((PetscObject)pars,"par");
+    ierr = PetscObjectSetName((PetscObject)pars,"par");CHKERRQ(ierr);
     ierr = VecView(pars, viewer);CHKERRQ(ierr);
 
     /* destroy viewer and vector */
@@ -1158,8 +1115,6 @@ PetscErrorCode DoOutput(SNES snes, PetscInt its)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "ViscosityField"
 /* Compute both the second invariant of the strain rate tensor and the viscosity, at both cell centers and cell corners */
 PetscErrorCode ViscosityField(DM da, Vec X, Vec V)
 /* ------------------------------------------------------------------- */
@@ -1169,7 +1124,7 @@ PetscErrorCode ViscosityField(DM da, Vec X, Vec V)
   GridInfo       *grid;
   Vec            localX;
   Field          **v, **x;
-  PassiveReal    eps, /* dx,*/ dz, T, epsC, TC;
+  PetscReal      eps, /* dx,*/ dz, T, epsC, TC;
   PetscInt       i,j,is,js,im,jm,ilim,jlim,ivt;
   PetscErrorCode ierr;
 
@@ -1219,8 +1174,6 @@ PetscErrorCode ViscosityField(DM da, Vec X, Vec V)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "StressField"
 /* post-processing: compute stress everywhere */
 PetscErrorCode StressField(DM da)
 /* ------------------------------------------------------------------- */
@@ -1264,11 +1217,9 @@ PetscErrorCode StressField(DM da)
   =====================================================================*/
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "SlabVel"
 /* returns the velocity of the subducting slab and handles fault nodes
    for BC */
-PETSC_STATIC_INLINE PassiveScalar SlabVel(char c, PetscInt i, PetscInt j, AppCtx *user)
+PETSC_STATIC_INLINE PetscScalar SlabVel(char c, PetscInt i, PetscInt j, AppCtx *user)
 /*---------------------------------------------------------------------*/
 {
   Parameter *param = user->param;
@@ -1287,44 +1238,27 @@ PETSC_STATIC_INLINE PassiveScalar SlabVel(char c, PetscInt i, PetscInt j, AppCtx
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "PlateModel"
 /*  solution to diffusive half-space cooling model for BC */
-PETSC_STATIC_INLINE PassiveScalar PlateModel(PetscInt j, PetscInt plate, AppCtx *user)
+PETSC_STATIC_INLINE PetscScalar PlateModel(PetscInt j, PetscInt plate, AppCtx *user)
 /*---------------------------------------------------------------------*/
 {
   Parameter     *param = user->param;
-  PassiveScalar z;
+  PetscScalar   z;
   if (plate==PLATE_LID) z = (j-0.5)*user->grid->dz;
   else z = (j-0.5)*user->grid->dz*param->cb;  /* PLATE_SLAB */
 #if defined(PETSC_HAVE_ERF)
-  return (erf(PetscRealPart(z*param->L/2.0/param->skt)));
+  return (PetscReal)(erf((double)PetscRealPart(z*param->L/2.0/param->skt)));
 #else
-  SETERRQ(PETSC_COMM_SELF,1,"erf() not available on this machine");
+  (*PetscErrorPrintf)("erf() not available on this machine\n");
+  MPI_Abort(PETSC_COMM_SELF,1);
 #endif
 }
 
-
-/* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "OptionsHasName"
-/*  utility function */
-PetscBool  OptionsHasName(const char pre[],const char name[])
-/* ------------------------------------------------------------------- */
-{
-  PetscBool      retval;
-  PetscErrorCode ierr;
-  ierr = PetscOptionsHasName(pre,name,&retval);CHKERRABORT(PETSC_COMM_WORLD,ierr);
-  return retval;
-}
-
 /*=====================================================================
   INTERACTIVE SIGNAL HANDLING
   =====================================================================*/
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "SNESConverged_Interactive"
 PetscErrorCode SNESConverged_Interactive(SNES snes, PetscInt it,PetscReal xnorm, PetscReal snorm, PetscReal fnorm, SNESConvergedReason *reason, void *ctx)
 /* ------------------------------------------------------------------- */
 {
@@ -1336,7 +1270,7 @@ PetscErrorCode SNESConverged_Interactive(SNES snes, PetscInt it,PetscReal xnorm,
   PetscFunctionBeginUser;
   if (param->interrupted) {
     param->interrupted = PETSC_FALSE;
-    PetscPrintf(PETSC_COMM_WORLD,"USER SIGNAL: exiting SNES solve. \n");
+    ierr = PetscPrintf(PETSC_COMM_WORLD,"USER SIGNAL: exiting SNES solve. \n");CHKERRQ(ierr);
     *reason = SNES_CONVERGED_FNORM_ABS;
     PetscFunctionReturn(0);
   } else if (param->toggle_kspmon) {
@@ -1348,12 +1282,14 @@ PetscErrorCode SNESConverged_Interactive(SNES snes, PetscInt it,PetscReal xnorm,
       ierr = KSPMonitorCancel(ksp);CHKERRQ(ierr);
 
       param->kspmon = PETSC_FALSE;
-      PetscPrintf(PETSC_COMM_WORLD,"USER SIGNAL: deactivating ksp singular value monitor. \n");
+      ierr = PetscPrintf(PETSC_COMM_WORLD,"USER SIGNAL: deactivating ksp singular value monitor. \n");CHKERRQ(ierr);
     } else {
-      ierr = KSPMonitorSet(ksp,KSPMonitorSingularValue,NULL,NULL);CHKERRQ(ierr);
+      PetscViewerAndFormat *vf;
+      ierr = PetscViewerAndFormatCreate(PETSC_VIEWER_STDOUT_WORLD,PETSC_VIEWER_DEFAULT,&vf);CHKERRQ(ierr);
+      ierr = KSPMonitorSet(ksp,(PetscErrorCode (*)(KSP,PetscInt,PetscReal,void*))KSPMonitorSingularValue,vf,(PetscErrorCode (*)(void**))PetscViewerAndFormatDestroy);CHKERRQ(ierr);
 
       param->kspmon = PETSC_TRUE;
-      PetscPrintf(PETSC_COMM_WORLD,"USER SIGNAL: activating ksp singular value monitor. \n");
+      ierr = PetscPrintf(PETSC_COMM_WORLD,"USER SIGNAL: activating ksp singular value monitor. \n");CHKERRQ(ierr);
     }
   }
   PetscFunctionReturn(SNESConvergedDefault(snes,it,xnorm,snorm,fnorm,reason,ctx));
@@ -1361,8 +1297,6 @@ PetscErrorCode SNESConverged_Interactive(SNES snes, PetscInt it,PetscReal xnorm,
 
 /* ------------------------------------------------------------------- */
 #include <signal.h>
-#undef __FUNCT__
-#define __FUNCT__ "InteractiveHandler"
 PetscErrorCode InteractiveHandler(int signum, void *ctx)
 /* ------------------------------------------------------------------- */
 {
@@ -1384,8 +1318,6 @@ PetscErrorCode InteractiveHandler(int signum, void *ctx)
 }
 
 /*---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "FormFunctionLocal"
 /*  main call-back function that computes the processor-local piece
     of the residual */
 PetscErrorCode FormFunctionLocal(DMDALocalInfo *info,Field **x,Field **f,void *ptr)
@@ -1511,3 +1443,16 @@ PetscErrorCode FormFunctionLocal(DMDALocalInfo *info,Field **x,Field **f,void *p
   }
   PetscFunctionReturn(0);
 }
+
+
+/*TEST
+
+   build:
+      requires: !complex erf
+
+   test:
+      args: -ni 18
+      filter: grep -v Destination
+      requires: !single
+
+TEST*/
