@@ -1,12 +1,8 @@
 #include <../src/snes/impls/fas/fasimpls.h> /*I  "petscsnes.h"  I*/
 
 
-extern PetscErrorCode SNESFASCycleCreateSmoother_Private(SNES, SNES*);
-
 /* -------------- functions called on the fine level -------------- */
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASSetType"
 /*@
     SNESFASSetType - Sets the update and correction type used for FAS.
 
@@ -36,8 +32,6 @@ PetscErrorCode  SNESFASSetType(SNES snes,SNESFASType fastype)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASGetType"
 /*@
 SNESFASGetType - Sets the update and correction type used for FAS.
 
@@ -64,8 +58,6 @@ PetscErrorCode  SNESFASGetType(SNES snes,SNESFASType *fastype)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASSetLevels"
 /*@C
    SNESFASSetLevels - Sets the number of levels to use with FAS.
    Must be called before any other FAS routine.
@@ -74,8 +66,7 @@ PetscErrorCode  SNESFASGetType(SNES snes,SNESFASType *fastype)
 +  snes   - the snes context
 .  levels - the number of levels
 -  comms  - optional communicators for each level; this is to allow solving the coarser
-            problems on smaller sets of processors. Use NULL_OBJECT for default in
-            Fortran.
+            problems on smaller sets of processors. 
 
    Level: intermediate
 
@@ -111,6 +102,7 @@ PetscErrorCode SNESFASSetLevels(SNES snes, PetscInt levels, MPI_Comm * comms)
   prevsnes      = snes;
   /* setup the finest level */
   ierr = SNESGetOptionsPrefix(snes, &optionsprefix);CHKERRQ(ierr);
+  ierr = PetscObjectComposedDataSetInt((PetscObject) snes, PetscMGLevelId, levels-1);CHKERRQ(ierr);
   for (i = levels - 1; i >= 0; i--) {
     if (comms) comm = comms[i];
     fas->level  = i;
@@ -126,6 +118,7 @@ PetscErrorCode SNESFASSetLevels(SNES snes, PetscInt levels, MPI_Comm * comms)
       ierr = SNESSetType(fas->next, SNESFAS);CHKERRQ(ierr);
       ierr = SNESSetTolerances(fas->next, fas->next->abstol, fas->next->rtol, fas->next->stol, fas->n_cycles, fas->next->max_funcs);CHKERRQ(ierr);
       ierr = PetscObjectIncrementTabLevel((PetscObject)fas->next, (PetscObject)snes, levels - i);CHKERRQ(ierr);
+      ierr = PetscObjectComposedDataSetInt((PetscObject) fas->next, PetscMGLevelId, i-1);CHKERRQ(ierr);
 
       ((SNES_FAS*)fas->next->data)->previous = prevsnes;
 
@@ -137,8 +130,6 @@ PetscErrorCode SNESFASSetLevels(SNES snes, PetscInt levels, MPI_Comm * comms)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASGetLevels"
 /*@
    SNESFASGetLevels - Gets the number of levels in a FAS, including fine and coarse grids
 
@@ -164,8 +155,6 @@ PetscErrorCode SNESFASGetLevels(SNES snes, PetscInt *levels)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASGetCycleSNES"
 /*@
    SNESFASGetCycleSNES - Gets the SNES corresponding to a particular
    level of the FAS hierarchy.
@@ -199,8 +188,6 @@ PetscErrorCode SNESFASGetCycleSNES(SNES snes,PetscInt level,SNES *lsnes)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASSetNumberSmoothUp"
 /*@
    SNESFASSetNumberSmoothUp - Sets the number of post-smoothing steps to
    use on all levels.
@@ -239,8 +226,6 @@ PetscErrorCode SNESFASSetNumberSmoothUp(SNES snes, PetscInt n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASSetNumberSmoothDown"
 /*@
    SNESFASSetNumberSmoothDown - Sets the number of pre-smoothing steps to
    use on all levels.
@@ -279,8 +264,6 @@ PetscErrorCode SNESFASSetNumberSmoothDown(SNES snes, PetscInt n)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASSetContinuation"
 /*@
    SNESFASSetContinuation - Sets the FAS cycle to default to exact Newton solves on the upsweep
 
@@ -295,7 +278,8 @@ PetscErrorCode SNESFASSetNumberSmoothDown(SNES snes, PetscInt n)
 
    Level: advanced
 
-   Notes: This sets the prefix on the upsweep smoothers to -fas_continuation
+   Notes:
+    This sets the prefix on the upsweep smoothers to -fas_continuation
 
 .keywords: FAS, MG, smoother, continuation
 
@@ -326,8 +310,6 @@ PetscErrorCode SNESFASSetContinuation(SNES snes,PetscBool continuation)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASSetCycles"
 /*@
    SNESFASSetCycles - Sets the number of FAS multigrid cycles to use each time a grid is visited.  Use SNESFASSetCyclesOnLevel() for more
    complicated cycling.
@@ -367,8 +349,6 @@ PetscErrorCode SNESFASSetCycles(SNES snes, PetscInt cycles)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASSetMonitor"
 /*@
    SNESFASSetMonitor - Sets the method-specific cycle monitoring
 
@@ -376,6 +356,7 @@ PetscErrorCode SNESFASSetCycles(SNES snes, PetscInt cycles)
 
    Input Parameters:
 +  snes   - the FAS context
+.  vf     - viewer and format structure (may be NULL if flg is FALSE)
 -  flg    - monitor or not
 
    Level: advanced
@@ -384,7 +365,7 @@ PetscErrorCode SNESFASSetCycles(SNES snes, PetscInt cycles)
 
 .seealso: SNESFASSetCyclesOnLevel()
 @*/
-PetscErrorCode SNESFASSetMonitor(SNES snes, PetscBool flg)
+PetscErrorCode SNESFASSetMonitor(SNES snes, PetscViewerAndFormat *vf, PetscBool flg)
 {
   SNES_FAS       *fas = (SNES_FAS*)snes->data;
   PetscErrorCode ierr;
@@ -399,10 +380,10 @@ PetscErrorCode SNESFASSetMonitor(SNES snes, PetscBool flg)
       ierr = SNESFASGetCycleSNES(snes, i, &levelsnes);CHKERRQ(ierr);
       fas  = (SNES_FAS*)levelsnes->data;
       if (flg) {
-        fas->monitor = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)levelsnes));CHKERRQ(ierr);
         /* set the monitors for the upsmoother and downsmoother */
         ierr = SNESMonitorCancel(levelsnes);CHKERRQ(ierr);
-        ierr = SNESMonitorSet(levelsnes,SNESMonitorDefault,NULL,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
+        /* Only register destroy on finest level */
+        ierr = SNESMonitorSet(levelsnes,(PetscErrorCode (*)(SNES,PetscInt,PetscReal,void*))SNESMonitorDefault,vf,(!i ? (PetscErrorCode (*)(void**))PetscViewerAndFormatDestroy : NULL));CHKERRQ(ierr);
       } else if (i != fas->levels - 1) {
         /* unset the monitors on the coarse levels */
         ierr = SNESMonitorCancel(levelsnes);CHKERRQ(ierr);
@@ -412,8 +393,6 @@ PetscErrorCode SNESFASSetMonitor(SNES snes, PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASSetLog"
 /*@
    SNESFASSetLog - Sets or unsets time logging for various FAS stages on all levels
 
@@ -445,16 +424,14 @@ PetscErrorCode SNESFASSetLog(SNES snes, PetscBool flg)
       ierr = SNESFASGetCycleSNES(snes, i, &levelsnes);CHKERRQ(ierr);
       fas  = (SNES_FAS*)levelsnes->data;
       if (flg) {
-        sprintf(eventname,"FASSetup %d",(int)i);
+        sprintf(eventname,"FASSetup  %d",(int)i);
         ierr = PetscLogEventRegister(eventname,((PetscObject)snes)->classid,&fas->eventsmoothsetup);CHKERRQ(ierr);
         sprintf(eventname,"FASSmooth %d",(int)i);
         ierr = PetscLogEventRegister(eventname,((PetscObject)snes)->classid,&fas->eventsmoothsolve);CHKERRQ(ierr);
-        sprintf(eventname,"FASResid %d",(int)i);
+        sprintf(eventname,"FASResid  %d",(int)i);
         ierr = PetscLogEventRegister(eventname,((PetscObject)snes)->classid,&fas->eventresidual);CHKERRQ(ierr);
-        if (i) {
-          sprintf(eventname,"FASInterp %d",(int)i);
-          ierr = PetscLogEventRegister(eventname,((PetscObject)snes)->classid,&fas->eventinterprestrict);CHKERRQ(ierr);
-        }
+        sprintf(eventname,"FASInterp %d",(int)i);
+        ierr = PetscLogEventRegister(eventname,((PetscObject)snes)->classid,&fas->eventinterprestrict);CHKERRQ(ierr);
       } else {
         fas->eventsmoothsetup    = 0;
         fas->eventsmoothsolve    = 0;
@@ -466,8 +443,6 @@ PetscErrorCode SNESFASSetLog(SNES snes, PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASCycleCreateSmoother_Private"
 /*
 Creates the default smoother type.
 
@@ -504,14 +479,13 @@ PetscErrorCode SNESFASCycleCreateSmoother_Private(SNES snes, SNES *smooth)
   ierr    = PetscObjectIncrementTabLevel((PetscObject)nsmooth, (PetscObject)snes, 1);CHKERRQ(ierr);
   ierr    = PetscLogObjectParent((PetscObject)snes,(PetscObject)nsmooth);CHKERRQ(ierr);
   ierr    = PetscObjectCopyFortranFunctionPointers((PetscObject)snes, (PetscObject)nsmooth);CHKERRQ(ierr);
+  ierr    = PetscObjectComposedDataSetInt((PetscObject) nsmooth, PetscMGLevelId, fas->level);CHKERRQ(ierr);
   *smooth = nsmooth;
   PetscFunctionReturn(0);
 }
 
 /* ------------- Functions called on a particular level ----------------- */
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASCycleSetCycles"
 /*@
    SNESFASCycleSetCycles - Sets the number of cycles on a particular level.
 
@@ -540,8 +514,6 @@ PetscErrorCode SNESFASCycleSetCycles(SNES snes, PetscInt cycles)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASCycleGetSmoother"
 /*@
    SNESFASCycleGetSmoother - Gets the smoother on a particular cycle level.
 
@@ -569,8 +541,6 @@ PetscErrorCode SNESFASCycleGetSmoother(SNES snes, SNES *smooth)
   *smooth = fas->smoothd;
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASCycleGetSmootherUp"
 /*@
    SNESFASCycleGetSmootherUp - Gets the up smoother on a particular cycle level.
 
@@ -604,8 +574,6 @@ PetscErrorCode SNESFASCycleGetSmootherUp(SNES snes, SNES *smoothu)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASCycleGetSmootherDown"
 /*@
    SNESFASCycleGetSmootherDown - Gets the down smoother on a particular cycle level.
 
@@ -635,8 +603,6 @@ PetscErrorCode SNESFASCycleGetSmootherDown(SNES snes, SNES *smoothd)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASCycleGetCorrection"
 /*@
    SNESFASCycleGetCorrection - Gets the coarse correction FAS context for this level
 
@@ -668,8 +634,6 @@ PetscErrorCode SNESFASCycleGetCorrection(SNES snes, SNES *correction)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASCycleGetInterpolation"
 /*@
    SNESFASCycleGetInterpolation - Gets the interpolation on this level
 
@@ -699,8 +663,6 @@ PetscErrorCode SNESFASCycleGetInterpolation(SNES snes, Mat *mat)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASCycleGetRestriction"
 /*@
    SNESFASCycleGetRestriction - Gets the restriction on this level
 
@@ -730,8 +692,6 @@ PetscErrorCode SNESFASCycleGetRestriction(SNES snes, Mat *mat)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASCycleGetInjection"
 /*@
    SNESFASCycleGetInjection - Gets the injection on this level
 
@@ -760,8 +720,6 @@ PetscErrorCode SNESFASCycleGetInjection(SNES snes, Mat *mat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASCycleGetRScale"
 /*@
    SNESFASCycleGetRScale - Gets the injection on this level
 
@@ -790,8 +748,6 @@ PetscErrorCode SNESFASCycleGetRScale(SNES snes, Vec *vec)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASCycleIsFine"
 /*@
    SNESFASCycleIsFine - Determines if a given cycle is the fine level.
 
@@ -823,8 +779,6 @@ PetscErrorCode SNESFASCycleIsFine(SNES snes, PetscBool *flg)
 
 /* ---------- functions called on the finest level that return level-specific information ---------- */
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASSetInterpolation"
 /*@
    SNESFASSetInterpolation - Sets the function to be used to calculate the
    interpolation from l-1 to the lth level
@@ -863,8 +817,6 @@ PetscErrorCode SNESFASSetInterpolation(SNES snes, PetscInt level, Mat mat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASGetInterpolation"
 /*@
    SNESFASGetInterpolation - Gets the matrix used to calculate the
    interpolation from l-1 to the lth level
@@ -895,8 +847,6 @@ PetscErrorCode SNESFASGetInterpolation(SNES snes, PetscInt level, Mat *mat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASSetRestriction"
 /*@
    SNESFASSetRestriction - Sets the function to be used to restrict the defect
    from level l to l-1.
@@ -938,8 +888,6 @@ PetscErrorCode SNESFASSetRestriction(SNES snes, PetscInt level, Mat mat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASGetRestriction"
 /*@
    SNESFASGetRestriction - Gets the matrix used to calculate the
    restriction from l to the l-1th level
@@ -971,8 +919,6 @@ PetscErrorCode SNESFASGetRestriction(SNES snes, PetscInt level, Mat *mat)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASSetInjection"
 /*@
    SNESFASSetInjection - Sets the function to be used to inject the solution
    from level l to l-1.
@@ -1009,8 +955,6 @@ PetscErrorCode SNESFASSetInjection(SNES snes, PetscInt level, Mat mat)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASGetInjection"
 /*@
    SNESFASGetInjection - Gets the matrix used to calculate the
    injection from l-1 to the lth level
@@ -1041,8 +985,6 @@ PetscErrorCode SNESFASGetInjection(SNES snes, PetscInt level, Mat *mat)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASSetRScale"
 /*@
    SNESFASSetRScale - Sets the scaling factor of the restriction
    operator from level l to l-1.
@@ -1077,8 +1019,6 @@ PetscErrorCode SNESFASSetRScale(SNES snes, PetscInt level, Vec rscale)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASGetSmoother"
 /*@
    SNESFASGetSmoother - Gets the default smoother on a level.
 
@@ -1111,8 +1051,6 @@ PetscErrorCode SNESFASGetSmoother(SNES snes, PetscInt level, SNES *smooth)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASGetSmootherDown"
 /*@
    SNESFASGetSmootherDown - Gets the downsmoother on a level.
 
@@ -1149,8 +1087,6 @@ PetscErrorCode SNESFASGetSmootherDown(SNES snes, PetscInt level, SNES *smooth)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASGetSmootherUp"
 /*@
    SNESFASGetSmootherUp - Gets the upsmoother on a level.
 
@@ -1187,24 +1123,21 @@ PetscErrorCode SNESFASGetSmootherUp(SNES snes, PetscInt level, SNES *smooth)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASGetCoarseSolve"
 /*@
-   SNESFASGetCoarseSolve - Gets the coarsest solver.
+  SNESFASGetCoarseSolve - Gets the coarsest solver.
 
-   Input Parameters:
-+  snes   - the multigrid context
+  Input Parameters:
+. snes - the multigrid context
 
-   Output Parameters:
-   solve  - the coarse-level solver
+  Output Parameters:
+. coarse - the coarse-level solver
 
-   Level: advanced
+  Level: advanced
 
 .keywords: FAS, MG, get, multigrid, solver, coarse
-
 .seealso: SNESFASSetInjection(), SNESFASSetRestriction()
 @*/
-PetscErrorCode SNESFASGetCoarseSolve(SNES snes, SNES *smooth)
+PetscErrorCode SNESFASGetCoarseSolve(SNES snes, SNES *coarse)
 {
   SNES_FAS       *fas;
   PetscErrorCode ierr;
@@ -1217,12 +1150,10 @@ PetscErrorCode SNESFASGetCoarseSolve(SNES snes, SNES *smooth)
   if (!fas->smoothd) {
     ierr = SNESFASCycleCreateSmoother_Private(levelsnes, &fas->smoothd);CHKERRQ(ierr);
   }
-  *smooth = fas->smoothd;
+  *coarse = fas->smoothd;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASFullSetDownSweep"
 /*@
    SNESFASFullSetDownSweep - Smooth during the initial downsweep for SNESFAS
 
