@@ -1,7 +1,5 @@
 #include <../src/snes/impls/fas/fasimpls.h> /*I  "petscsnes.h"  I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASGetGalerkin"
 /*@
    SNESFASGetGalerkin - Gets if the coarse problems are formed by projection to the fine problem
 
@@ -26,8 +24,6 @@ PetscErrorCode SNESFASGetGalerkin(SNES snes, PetscBool *flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASSetGalerkin"
 /*@
    SNESFASSetGalerkin - Sets coarse problems as formed by projection to the fine problem
 
@@ -52,17 +48,29 @@ PetscErrorCode SNESFASSetGalerkin(SNES snes, PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFASGalerkinDefaultFunction"
-/*
-SNESFASGalerkinDefaultFunction
+/*@C
+   SNESFASGalerkinFunctionDefault - Computes the Galerkin FAS function
 
- */
-PetscErrorCode SNESFASGalerkinDefaultFunction(SNES snes, Vec X, Vec F, void * ctx)
+   Input Parameters:
+.  snes - the nonlinear solver context
+.  X - input vector
+.  ctx - the FAS context
+
+   Output Parameter:
+.  F - output vector
+
+   Notes:
+   The Galerkin FAS function evalutation is defined as
+$  F^l(x^l) = I^l_0 F^0(P^0_l x^l)
+
+   Level: developer
+
+.keywords: FAS, galerkin
+
+.seealso: SNESFASGetGalerkin(), SNESFASSetGalerkin()
+@*/
+PetscErrorCode SNESFASGalerkinFunctionDefault(SNES snes, Vec X, Vec F, void *ctx)
 {
-  /* the Galerkin FAS function evalutation is defined as
-   F^l(x^l) = I^l_0F^0(P^0_lx^l)
-   */
   SNES           fassnes;
   SNES_FAS       *fas;
   SNES_FAS       *prevfas;
