@@ -7,8 +7,6 @@
     0 = (J^T F)^T W = F^T J W iff W not in the null space of J. Thanks for Jorge More
     for this trick. One assumes that the probability that W is in the null space of J is very, very small.
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESNEWTONLSCheckLocalMin_Private"
 static PetscErrorCode SNESNEWTONLSCheckLocalMin_Private(SNES snes,Mat A,Vec F,PetscReal fnorm,PetscBool  *ismin)
 {
   PetscReal      a1;
@@ -48,8 +46,6 @@ static PetscErrorCode SNESNEWTONLSCheckLocalMin_Private(SNES snes,Mat A,Vec F,Pe
 /*
      Checks if J^T(F - J*X) = 0
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESNEWTONLSCheckResidual_Private"
 static PetscErrorCode SNESNEWTONLSCheckResidual_Private(SNES snes,Mat A,Vec F,Vec X)
 {
   PetscReal      a1,a2;
@@ -135,8 +131,6 @@ static PetscErrorCode SNESNEWTONLSCheckResidual_Private(SNES snes,Mat A,Vec F,Ve
    Unconstrained Optimization and Nonlinear Equations" by Dennis
    and Schnabel.
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESSolve_NEWTONLS"
 PetscErrorCode SNESSolve_NEWTONLS(SNES snes)
 {
   PetscErrorCode       ierr;
@@ -166,9 +160,9 @@ PetscErrorCode SNESSolve_NEWTONLS(SNES snes)
   ierr       = SNESGetLineSearch(snes, &linesearch);CHKERRQ(ierr);
 
   /* compute the preconditioned function first in the case of left preconditioning with preconditioned function */
-  if (snes->pc && snes->pcside == PC_LEFT && snes->functype == SNES_FUNCTION_PRECONDITIONED) {
+  if (snes->npc && snes->npcside== PC_LEFT && snes->functype == SNES_FUNCTION_PRECONDITIONED) {
     ierr = SNESApplyNPC(snes,X,NULL,F);CHKERRQ(ierr);
-    ierr = SNESGetConvergedReason(snes->pc,&reason);CHKERRQ(ierr);
+    ierr = SNESGetConvergedReason(snes->npc,&reason);CHKERRQ(ierr);
     if (reason < 0  && reason != SNES_DIVERGED_MAX_IT) {
       snes->reason = SNES_DIVERGED_INNER;
       PetscFunctionReturn(0);
@@ -202,21 +196,21 @@ PetscErrorCode SNESSolve_NEWTONLS(SNES snes)
     }
 
     /* apply the nonlinear preconditioner */
-    if (snes->pc) {
-      if (snes->pcside == PC_RIGHT) {
-        ierr = SNESSetInitialFunction(snes->pc, F);CHKERRQ(ierr);
-        ierr = PetscLogEventBegin(SNES_NPCSolve,snes->pc,X,snes->vec_rhs,0);CHKERRQ(ierr);
-        ierr = SNESSolve(snes->pc, snes->vec_rhs, X);CHKERRQ(ierr);
-        ierr = PetscLogEventEnd(SNES_NPCSolve,snes->pc,X,snes->vec_rhs,0);CHKERRQ(ierr);
-        ierr = SNESGetConvergedReason(snes->pc,&reason);CHKERRQ(ierr);
+    if (snes->npc) {
+      if (snes->npcside== PC_RIGHT) {
+        ierr = SNESSetInitialFunction(snes->npc, F);CHKERRQ(ierr);
+        ierr = PetscLogEventBegin(SNES_NPCSolve,snes->npc,X,snes->vec_rhs,0);CHKERRQ(ierr);
+        ierr = SNESSolve(snes->npc, snes->vec_rhs, X);CHKERRQ(ierr);
+        ierr = PetscLogEventEnd(SNES_NPCSolve,snes->npc,X,snes->vec_rhs,0);CHKERRQ(ierr);
+        ierr = SNESGetConvergedReason(snes->npc,&reason);CHKERRQ(ierr);
         if (reason < 0  && reason != SNES_DIVERGED_MAX_IT) {
           snes->reason = SNES_DIVERGED_INNER;
           PetscFunctionReturn(0);
         }
         ierr = SNESGetNPCFunction(snes,F,&fnorm);CHKERRQ(ierr);
-      } else if (snes->pcside == PC_LEFT && snes->functype == SNES_FUNCTION_UNPRECONDITIONED) {
+      } else if (snes->npcside== PC_LEFT && snes->functype == SNES_FUNCTION_UNPRECONDITIONED) {
         ierr = SNESApplyNPC(snes,X,F,F);CHKERRQ(ierr);
-        ierr = SNESGetConvergedReason(snes->pc,&reason);CHKERRQ(ierr);
+        ierr = SNESGetConvergedReason(snes->npc,&reason);CHKERRQ(ierr);
         if (reason < 0  && reason != SNES_DIVERGED_MAX_IT) {
           snes->reason = SNES_DIVERGED_INNER;
           PetscFunctionReturn(0);
@@ -229,9 +223,8 @@ PetscErrorCode SNESSolve_NEWTONLS(SNES snes)
     ierr = KSPSetOperators(snes->ksp,snes->jacobian,snes->jacobian_pre);CHKERRQ(ierr);
     ierr = KSPSolve(snes->ksp,F,Y);CHKERRQ(ierr);
     SNESCheckKSPSolve(snes);
-    ierr              = KSPGetIterationNumber(snes->ksp,&lits);CHKERRQ(ierr);
-    snes->linear_its += lits;
-    ierr              = PetscInfo2(snes,"iter=%D, linear solve iterations=%D\n",snes->iter,lits);CHKERRQ(ierr);
+    ierr = KSPGetIterationNumber(snes->ksp,&lits);CHKERRQ(ierr);
+    ierr = PetscInfo2(snes,"iter=%D, linear solve iterations=%D\n",snes->iter,lits);CHKERRQ(ierr);
 
     if (PetscLogPrintInfo) {
       ierr = SNESNEWTONLSCheckResidual_Private(snes,snes->jacobian,F,Y);CHKERRQ(ierr);
@@ -294,21 +287,17 @@ PetscErrorCode SNESSolve_NEWTONLS(SNES snes)
    SNESSetUp(), since these actions will automatically occur during
    the call to SNESSolve().
  */
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetUp_NEWTONLS"
 PetscErrorCode SNESSetUp_NEWTONLS(SNES snes)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   ierr = SNESSetUpMatrices(snes);CHKERRQ(ierr);
-  if (snes->pcside == PC_LEFT && snes->functype == SNES_FUNCTION_DEFAULT) snes->functype = SNES_FUNCTION_PRECONDITIONED;
+  if (snes->npcside== PC_LEFT && snes->functype == SNES_FUNCTION_DEFAULT) snes->functype = SNES_FUNCTION_PRECONDITIONED;
   PetscFunctionReturn(0);
 }
 /* -------------------------------------------------------------------------- */
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESReset_NEWTONLS"
 PetscErrorCode SNESReset_NEWTONLS(SNES snes)
 {
   PetscFunctionBegin;
@@ -324,8 +313,6 @@ PetscErrorCode SNESReset_NEWTONLS(SNES snes)
 
    Application Interface Routine: SNESDestroy()
  */
-#undef __FUNCT__
-#define __FUNCT__ "SNESDestroy_NEWTONLS"
 PetscErrorCode SNESDestroy_NEWTONLS(SNES snes)
 {
   PetscErrorCode ierr;
@@ -346,8 +333,6 @@ PetscErrorCode SNESDestroy_NEWTONLS(SNES snes)
 
    Application Interface Routine: SNESView()
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESView_NEWTONLS"
 static PetscErrorCode SNESView_NEWTONLS(SNES snes,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -369,8 +354,6 @@ static PetscErrorCode SNESView_NEWTONLS(SNES snes,PetscViewer viewer)
 
    Application Interface Routine: SNESSetFromOptions()
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetFromOptions_NEWTONLS"
 static PetscErrorCode SNESSetFromOptions_NEWTONLS(PetscOptionItems *PetscOptionsObject,SNES snes)
 {
   PetscErrorCode ierr;
@@ -391,14 +374,15 @@ static PetscErrorCode SNESSetFromOptions_NEWTONLS(PetscOptionItems *PetscOptions
    Options Database:
 +   -snes_linesearch_type <bt> - bt,basic.  Select line search type
 .   -snes_linesearch_order <3> - 2, 3. Selects the order of the line search for bt
-.   -snes_linesearch_norms <true> - Turns on/off computation of the norms for basic linesearch
+.   -snes_linesearch_norms <true> - Turns on/off computation of the norms for basic linesearch (SNESLineSearchSetComputeNorms())
 .   -snes_linesearch_alpha <alpha> - Sets alpha used in determining if reduction in function norm is sufficient
 .   -snes_linesearch_maxstep <maxstep> - Sets the maximum stepsize the line search will use (if the 2-norm(y) > maxstep then scale y to be y = (maxstep/2-norm(y)) *y)
 .   -snes_linesearch_minlambda <minlambda>  - Sets the minimum lambda the line search will tolerate
 .   -snes_linesearch_monitor - print information about progress of line searches
 -   -snes_linesearch_damping - damping factor used for basic line search
 
-    Notes: This is the default nonlinear solver in SNES
+    Notes:
+    This is the default nonlinear solver in SNES
 
    Level: beginner
 
@@ -406,8 +390,6 @@ static PetscErrorCode SNESSetFromOptions_NEWTONLS(PetscOptionItems *PetscOptions
            SNESLineSearchSetPostCheck(), SNESLineSearchSetPreCheck() SNESLineSearchSetComputeNorms()
 
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "SNESCreate_NEWTONLS"
 PETSC_EXTERN PetscErrorCode SNESCreate_NEWTONLS(SNES snes)
 {
   PetscErrorCode ierr;
@@ -421,9 +403,12 @@ PETSC_EXTERN PetscErrorCode SNESCreate_NEWTONLS(SNES snes)
   snes->ops->view           = SNESView_NEWTONLS;
   snes->ops->reset          = SNESReset_NEWTONLS;
 
-  snes->pcside  = PC_RIGHT;
+  snes->npcside = PC_RIGHT;
   snes->usesksp = PETSC_TRUE;
-  snes->usespc  = PETSC_TRUE;
+  snes->usesnpc = PETSC_TRUE;
+
+  snes->alwayscomputesfinalresidual = PETSC_TRUE;
+
   ierr          = PetscNewLog(snes,&neP);CHKERRQ(ierr);
   snes->data    = (void*)neP;
   PetscFunctionReturn(0);
