@@ -28,8 +28,6 @@ typedef struct {
   PetscBool     norms;          /* Compute norms, usually only for monitoring purposes */
 } SNES_MS;
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMSRegisterAll"
 /*@C
   SNESMSRegisterAll - Registers all of the multi-stage methods in SNESMS
 
@@ -105,8 +103,6 @@ PetscErrorCode SNESMSRegisterAll(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMSRegisterDestroy"
 /*@C
    SNESMSRegisterDestroy - Frees the list of schemes that were registered by TSRosWRegister().
 
@@ -135,8 +131,6 @@ PetscErrorCode SNESMSRegisterDestroy(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMSInitializePackage"
 /*@C
   SNESMSInitializePackage - This function initializes everything in the SNESMS package. It is called
   from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to SNESCreate_MS()
@@ -160,8 +154,6 @@ PetscErrorCode SNESMSInitializePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMSFinalizePackage"
 /*@C
   SNESMSFinalizePackage - This function destroys everything in the SNESMS package. It is
   called from PetscFinalize().
@@ -182,8 +174,6 @@ PetscErrorCode SNESMSFinalizePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMSRegister"
 /*@C
    SNESMSRegister - register a multistage scheme
 
@@ -220,6 +210,7 @@ PetscErrorCode SNESMSRegister(SNESMSType name,PetscInt nstages,PetscInt nregiste
   PetscValidPointer(delta,5);
   PetscValidPointer(betasub,6);
 
+  ierr          = SNESMSInitializePackage();CHKERRQ(ierr);
   ierr          = PetscNew(&link);CHKERRQ(ierr);
   t             = &link->tab;
   ierr          = PetscStrallocpy(name,&t->name);CHKERRQ(ierr);
@@ -237,8 +228,6 @@ PetscErrorCode SNESMSRegister(SNESMSType name,PetscInt nstages,PetscInt nregiste
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMSStep_3Sstar"
 /*
   X - initial state, updated in-place.
   F - residual, computed at the initial X on input
@@ -281,8 +270,6 @@ static PetscErrorCode SNESMSStep_3Sstar(SNES snes,Vec X,Vec F)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESSolve_MS"
 static PetscErrorCode SNESSolve_MS(SNES snes)
 {
   SNES_MS        *ms = (SNES_MS*)snes->data;
@@ -292,10 +279,7 @@ static PetscErrorCode SNESSolve_MS(SNES snes)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-
-  if (snes->xl || snes->xu || snes->ops->computevariablebounds) {
-    SETERRQ1(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_WRONGSTATE, "SNES solver %s does not support bounds", ((PetscObject)snes)->type_name);
-  }
+  if (snes->xl || snes->xu || snes->ops->computevariablebounds) SETERRQ1(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_WRONGSTATE, "SNES solver %s does not support bounds", ((PetscObject)snes)->type_name);
 
   ierr = PetscCitationsRegister(SNESCitation,&SNEScite);CHKERRQ(ierr);
   snes->reason = SNES_CONVERGED_ITERATING;
@@ -363,8 +347,6 @@ static PetscErrorCode SNESSolve_MS(SNES snes)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetUp_MS"
 static PetscErrorCode SNESSetUp_MS(SNES snes)
 {
   SNES_MS        *ms = (SNES_MS*)snes->data;
@@ -377,8 +359,6 @@ static PetscErrorCode SNESSetUp_MS(SNES snes)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESReset_MS"
 static PetscErrorCode SNESReset_MS(SNES snes)
 {
 
@@ -386,8 +366,6 @@ static PetscErrorCode SNESReset_MS(SNES snes)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESDestroy_MS"
 static PetscErrorCode SNESDestroy_MS(SNES snes)
 {
   PetscErrorCode ierr;
@@ -398,8 +376,6 @@ static PetscErrorCode SNESDestroy_MS(SNES snes)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESView_MS"
 static PetscErrorCode SNESView_MS(SNES snes,PetscViewer viewer)
 {
   PetscBool      iascii;
@@ -415,9 +391,7 @@ static PetscErrorCode SNESView_MS(SNES snes,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetFromOptions_MS"
-static PetscErrorCode SNESSetFromOptions_MS(PetscOptions *PetscOptionsObject,SNES snes)
+static PetscErrorCode SNESSetFromOptions_MS(PetscOptionItems *PetscOptionsObject,SNES snes)
 {
   SNES_MS        *ms = (SNES_MS*)snes->data;
   PetscErrorCode ierr;
@@ -433,7 +407,7 @@ static PetscErrorCode SNESSetFromOptions_MS(PetscOptions *PetscOptionsObject,SNE
 
     ierr = PetscStrncpy(mstype,SNESMSDefault,sizeof(mstype));CHKERRQ(ierr);
     for (link=SNESMSTableauList,count=0; link; link=link->next,count++) ;
-    ierr = PetscMalloc1(count,&namelist);CHKERRQ(ierr);
+    ierr = PetscMalloc1(count,(char***)&namelist);CHKERRQ(ierr);
     for (link=SNESMSTableauList,count=0; link; link=link->next,count++) namelist[count] = link->tab.name;
     ierr = PetscOptionsEList("-snes_ms_type","Multistage smoother type","SNESMSSetType",(const char*const*)namelist,count,mstype,&choice,&flg);CHKERRQ(ierr);
     ierr = SNESMSSetType(snes,flg ? namelist[choice] : mstype);CHKERRQ(ierr);
@@ -445,8 +419,6 @@ static PetscErrorCode SNESSetFromOptions_MS(PetscOptions *PetscOptionsObject,SNE
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMSSetType_MS"
 PetscErrorCode  SNESMSSetType_MS(SNES snes,SNESMSType mstype)
 {
   PetscErrorCode    ierr;
@@ -471,8 +443,6 @@ PetscErrorCode  SNESMSSetType_MS(SNES snes,SNESMSType mstype)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMSSetType"
 /*@C
   SNESMSSetType - Set the type of multistage smoother
 
@@ -514,20 +484,15 @@ PetscErrorCode SNESMSSetType(SNES snes,SNESMSType rostype)
       The methods are specified in low storage form (Ketcheson 2010). New methods can be registered with SNESMSRegister().
 
       References:
-
-      Ketcheson (2010) Runge-Kutta methods with minimum storage implementations.
-
-      Jameson (1983) Solution of the Euler equations for two dimensional transonic flow by a multigrid method.
-
-      Pierce and Giles (1997) Preconditioned multigrid methods for compressible flow calculations on stretched meshes.
++   1. -   Ketcheson (2010) Runge Kutta methods with minimum storage implementations.
+.   2. -   Jameson (1983) Solution of the Euler equations for two dimensional transonic flow by a multigrid method.
+-   3. -   Pierce and Giles (1997) Preconditioned multigrid methods for compressible flow calculations on stretched meshes.
 
       Level: beginner
 
 .seealso:  SNESCreate(), SNES, SNESSetType(), SNESMS, SNESFAS, KSPCHEBYSHEV
 
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "SNESCreate_MS"
 PETSC_EXTERN PetscErrorCode SNESCreate_MS(SNES snes)
 {
   PetscErrorCode ierr;
@@ -543,9 +508,11 @@ PETSC_EXTERN PetscErrorCode SNESCreate_MS(SNES snes)
   snes->ops->view           = SNESView_MS;
   snes->ops->reset          = SNESReset_MS;
 
-  snes->usespc  = PETSC_FALSE;
+  snes->usesnpc = PETSC_FALSE;
   snes->usesksp = PETSC_TRUE;
 
+  snes->alwayscomputesfinalresidual = PETSC_FALSE;
+
   ierr        = PetscNewLog(snes,&ms);CHKERRQ(ierr);
   snes->data  = (void*)ms;
   ms->damping = 0.9;
