@@ -1,8 +1,6 @@
 #include <petsc/private/snesimpl.h>             /*I   "petscsnes.h"   I*/
 #include <petscdm.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "MatMultASPIN"
 PetscErrorCode MatMultASPIN(Mat m,Vec X,Vec Y)
 {
   PetscErrorCode ierr;
@@ -53,8 +51,18 @@ PetscErrorCode MatMultASPIN(Mat m,Vec X,Vec Y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESCreate_ASPIN"
+static PetscErrorCode SNESDestroy_ASPIN(SNES snes)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = SNESDestroy(&snes->npc);CHKERRQ(ierr);
+  /* reset NEWTONLS and free the data */
+  ierr = SNESReset(snes);CHKERRQ(ierr);
+  ierr = PetscFree(snes->data);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /* -------------------------------------------------------------------------- */
 /*MC
       SNESASPIN - Helper SNES type for Additive-Schwarz Preconditioned Inexact Newton
@@ -65,8 +73,9 @@ PetscErrorCode MatMultASPIN(Mat m,Vec X,Vec Y)
 .  -npc_sub_ksp_ - options prefix of the subdomain Krylov solver
 -  -npc_sub_pc_ - options prefix of the subdomain preconditioner
 
-    Notes: This routine sets up an instance of NETWONLS with nonlinear left preconditioning.  It differs from other
-    similar functionality in SNES as it creates a linear shell matrix that corresponds to the product:
+    Notes:
+    This routine sets up an instance of NETWONLS with nonlinear left preconditioning.  It differs from other
+    similar functionality in SNES as it creates a linear shell matrix that corresponds to the product
 
     \sum_{i=0}^{N_b}J_b({X^b_{converged}})^{-1}J(X + \sum_{i=0}^{N_b}(X^b_{converged} - X^b))
 
@@ -76,6 +85,11 @@ PetscErrorCode MatMultASPIN(Mat m,Vec X,Vec Y)
 
    Level: intermediate
 
+   References:
++  1. - X. C. Cai and D. E. Keyes, "Nonlinearly preconditioned inexact Newton algorithms",  SIAM J. Sci. Comput., 24, 2002.
+-  2. - Peter R. Brune, Matthew G. Knepley, Barry F. Smith, and Xuemin Tu, "Composing Scalable Nonlinear Algebraic Solvers",
+   SIAM Review, 57(4), 2015
+
 .seealso:  SNESCreate(), SNES, SNESSetType(), SNESNEWTONLS, SNESNASM, SNESGetNPC(), SNESGetNPCSide()
 
 M*/
@@ -86,7 +100,6 @@ PETSC_EXTERN PetscErrorCode SNESCreate_ASPIN(SNES snes)
   KSP            ksp;
   PC             pc;
   Mat            aspinmat;
-  MPI_Comm       comm;
   Vec            F;
   PetscInt       n;
   SNESLineSearch linesearch;
@@ -103,19 +116,19 @@ PETSC_EXTERN PetscErrorCode SNESCreate_ASPIN(SNES snes)
   ierr = SNESGetKSP(snes,&ksp);CHKERRQ(ierr);
   ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
   ierr = PCSetType(pc,PCNONE);CHKERRQ(ierr);
-  ierr = PetscObjectGetComm((PetscObject)snes,&comm);CHKERRQ(ierr);
   ierr = SNESGetLineSearch(snes,&linesearch);CHKERRQ(ierr);
   ierr = SNESLineSearchSetType(linesearch,SNESLINESEARCHBT);CHKERRQ(ierr);
 
   /* set up the shell matrix */
   ierr = SNESGetFunction(snes,&F,NULL,NULL);CHKERRQ(ierr);
   ierr = VecGetLocalSize(F,&n);CHKERRQ(ierr);
-  ierr = MatCreateShell(comm,n,n,PETSC_DECIDE,PETSC_DECIDE,snes,&aspinmat);CHKERRQ(ierr);
+  ierr = MatCreateShell(PetscObjectComm((PetscObject)snes),n,n,PETSC_DECIDE,PETSC_DECIDE,snes,&aspinmat);CHKERRQ(ierr);
   ierr = MatSetType(aspinmat,MATSHELL);CHKERRQ(ierr);
   ierr = MatShellSetOperation(aspinmat,MATOP_MULT,(void(*)(void))MatMultASPIN);CHKERRQ(ierr);
-
   ierr = SNESSetJacobian(snes,aspinmat,NULL,NULL,NULL);CHKERRQ(ierr);
   ierr = MatDestroy(&aspinmat);CHKERRQ(ierr);
 
+  snes->ops->destroy = SNESDestroy_ASPIN;
+
   PetscFunctionReturn(0);
 }
