@@ -8,7 +8,9 @@ typedef struct {
   Vec        *xl;                 /* solution local vectors */
   Vec        *y;                  /* step vectors */
   Vec        *b;                  /* rhs vectors */
+  Vec        weight;              /* weighting for adding updates on overlaps, in global space */
   VecScatter *oscatter;           /* scatter from global space to the subdomain global space */
+  VecScatter *oscatter_copy;      /* copy of the above */
   VecScatter *iscatter;           /* scatter from global space to the nonoverlapping subdomain space */
   VecScatter *gscatter;           /* scatter from global space to the subdomain local space */
   PCASMType  type;                /* ASM type */
@@ -16,6 +18,7 @@ typedef struct {
   PetscBool  finaljacobian;       /* compute the jacobian of the converged solution */
   PetscReal  damping;             /* damping parameter for updates from the blocks */
   PetscBool  same_local_solves;   /* flag to determine if the solvers have been individually modified */
+  PetscBool  weight_set;          /* use a weight in the overlap updates */
 
   /* logging events */
   PetscLogEvent eventrestrictinterp;
@@ -28,9 +31,7 @@ typedef struct {
 const char *const SNESNASMTypes[] = {"NONE","RESTRICT","INTERPOLATE","BASIC","PCASMType","PC_ASM_",0};
 const char *const SNESNASMFJTypes[] = {"FINALOUTER","FINALINNER","INITIAL"};
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESReset_NASM"
-PetscErrorCode SNESReset_NASM(SNES snes)
+static PetscErrorCode SNESReset_NASM(SNES snes)
 {
   SNES_NASM      *nasm = (SNES_NASM*)snes->data;
   PetscErrorCode ierr;
@@ -45,30 +46,34 @@ PetscErrorCode SNESReset_NASM(SNES snes)
 
     if (nasm->subsnes) { ierr = SNESDestroy(&nasm->subsnes[i]);CHKERRQ(ierr); }
     if (nasm->oscatter) { ierr = VecScatterDestroy(&nasm->oscatter[i]);CHKERRQ(ierr); }
+    if (nasm->oscatter_copy) { ierr = VecScatterDestroy(&nasm->oscatter_copy[i]);CHKERRQ(ierr); }
     if (nasm->iscatter) { ierr = VecScatterDestroy(&nasm->iscatter[i]);CHKERRQ(ierr); }
     if (nasm->gscatter) { ierr = VecScatterDestroy(&nasm->gscatter[i]);CHKERRQ(ierr); }
   }
 
-  if (nasm->x) {ierr = PetscFree(nasm->x);CHKERRQ(ierr);}
-  if (nasm->xl) {ierr = PetscFree(nasm->xl);CHKERRQ(ierr);}
-  if (nasm->y) {ierr = PetscFree(nasm->y);CHKERRQ(ierr);}
-  if (nasm->b) {ierr = PetscFree(nasm->b);CHKERRQ(ierr);}
+  ierr = PetscFree(nasm->x);CHKERRQ(ierr);
+  ierr = PetscFree(nasm->xl);CHKERRQ(ierr);
+  ierr = PetscFree(nasm->y);CHKERRQ(ierr);
+  ierr = PetscFree(nasm->b);CHKERRQ(ierr);
 
   if (nasm->xinit) {ierr = VecDestroy(&nasm->xinit);CHKERRQ(ierr);}
 
-  if (nasm->subsnes) {ierr = PetscFree(nasm->subsnes);CHKERRQ(ierr);}
-  if (nasm->oscatter) {ierr = PetscFree(nasm->oscatter);CHKERRQ(ierr);}
-  if (nasm->iscatter) {ierr = PetscFree(nasm->iscatter);CHKERRQ(ierr);}
-  if (nasm->gscatter) {ierr = PetscFree(nasm->gscatter);CHKERRQ(ierr);}
+  ierr = PetscFree(nasm->subsnes);CHKERRQ(ierr);
+  ierr = PetscFree(nasm->oscatter);CHKERRQ(ierr);
+  ierr = PetscFree(nasm->oscatter_copy);CHKERRQ(ierr);
+  ierr = PetscFree(nasm->iscatter);CHKERRQ(ierr);
+  ierr = PetscFree(nasm->gscatter);CHKERRQ(ierr);
+
+  if (nasm->weight_set) {
+    ierr = VecDestroy(&nasm->weight);CHKERRQ(ierr);
+  }
 
   nasm->eventrestrictinterp = 0;
   nasm->eventsubsolve = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESDestroy_NASM"
-PetscErrorCode SNESDestroy_NASM(SNES snes)
+static PetscErrorCode SNESDestroy_NASM(SNES snes)
 {
   PetscErrorCode ierr;
 
@@ -78,9 +83,7 @@ PetscErrorCode SNESDestroy_NASM(SNES snes)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMGlobalToLocalSubDomainDirichletHook_Private"
-PetscErrorCode DMGlobalToLocalSubDomainDirichletHook_Private(DM dm,Vec g,InsertMode mode,Vec l,void *ctx)
+static PetscErrorCode DMGlobalToLocalSubDomainDirichletHook_Private(DM dm,Vec g,InsertMode mode,Vec l,void *ctx)
 {
   PetscErrorCode ierr;
   Vec            bcs = (Vec)ctx;
@@ -90,9 +93,7 @@ PetscErrorCode DMGlobalToLocalSubDomainDirichletHook_Private(DM dm,Vec g,InsertM
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetUp_NASM"
-PetscErrorCode SNESSetUp_NASM(SNES snes)
+static PetscErrorCode SNESSetUp_NASM(SNES snes)
 {
   SNES_NASM      *nasm = (SNES_NASM*)snes->data;
   PetscErrorCode ierr;
@@ -113,11 +114,16 @@ PetscErrorCode SNESSetUp_NASM(SNES snes)
       ierr         = DMCreateDomainDecomposition(dm,&nasm->n,NULL,NULL,NULL,&subdms);CHKERRQ(ierr);
       if (!subdms) SETERRQ(PetscObjectComm((PetscObject)dm),PETSC_ERR_ARG_WRONGSTATE,"DM has no default decomposition defined.  Set subsolves manually with SNESNASMSetSubdomains().");
       ierr = DMCreateDomainDecompositionScatters(dm,nasm->n,subdms,&nasm->iscatter,&nasm->oscatter,&nasm->gscatter);CHKERRQ(ierr);
+      ierr = PetscMalloc1(nasm->n, &nasm->oscatter_copy);CHKERRQ(ierr);
+      for (i=0; i<nasm->n; i++) {
+        ierr = VecScatterCopy(nasm->oscatter[i], &nasm->oscatter_copy[i]);CHKERRQ(ierr);
+      }
 
       ierr = SNESGetOptionsPrefix(snes, &optionsprefix);CHKERRQ(ierr);
       ierr = PetscMalloc1(nasm->n,&nasm->subsnes);CHKERRQ(ierr);
       for (i=0; i<nasm->n; i++) {
         ierr = SNESCreate(PETSC_COMM_SELF,&nasm->subsnes[i]);CHKERRQ(ierr);
+        ierr = PetscObjectIncrementTabLevel((PetscObject)nasm->subsnes[i], (PetscObject)snes, 1);CHKERRQ(ierr);
         ierr = SNESAppendOptionsPrefix(nasm->subsnes[i],optionsprefix);CHKERRQ(ierr);
         ierr = SNESAppendOptionsPrefix(nasm->subsnes[i],"sub_");CHKERRQ(ierr);
         ierr = SNESSetDM(nasm->subsnes[i],subdms[i]);CHKERRQ(ierr);
@@ -156,8 +162,8 @@ PetscErrorCode SNESSetUp_NASM(SNES snes)
     if (!nasm->xl[i]) {
       ierr = SNESGetDM(nasm->subsnes[i],&subdm);CHKERRQ(ierr);
       ierr = DMCreateLocalVector(subdm,&nasm->xl[i]);CHKERRQ(ierr);
+      ierr = DMGlobalToLocalHookAdd(subdm,DMGlobalToLocalSubDomainDirichletHook_Private,NULL,nasm->xl[i]);CHKERRQ(ierr);
     }
-    ierr = DMGlobalToLocalHookAdd(subdm,DMGlobalToLocalSubDomainDirichletHook_Private,NULL,nasm->xl[i]);CHKERRQ(ierr);
   }
   if (nasm->finaljacobian) {
     ierr = SNESSetUpMatrices(snes);CHKERRQ(ierr);
@@ -171,9 +177,7 @@ PetscErrorCode SNESSetUp_NASM(SNES snes)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetFromOptions_NASM"
-PetscErrorCode SNESSetFromOptions_NASM(PetscOptions *PetscOptionsObject,SNES snes)
+static PetscErrorCode SNESSetFromOptions_NASM(PetscOptionItems *PetscOptionsObject,SNES snes)
 {
   PetscErrorCode    ierr;
   PCASMType         asmtype;
@@ -186,7 +190,7 @@ PetscErrorCode SNESSetFromOptions_NASM(PetscOptions *PetscOptionsObject,SNES sne
   if (flg) {ierr = SNESNASMSetType(snes,asmtype);CHKERRQ(ierr);}
   flg    = PETSC_FALSE;
   monflg = PETSC_TRUE;
-  ierr   = PetscOptionsReal("-snes_nasm_damping","Log times for subSNES solves and restriction","SNESNASMSetDamping",nasm->damping,&nasm->damping,&flg);CHKERRQ(ierr);
+  ierr   = PetscOptionsReal("-snes_nasm_damping","The new solution is obtained as old solution plus dmp times (sum of the solutions on the subdomains)","SNESNASMSetDamping",nasm->damping,&nasm->damping,&flg);CHKERRQ(ierr);
   if (flg) {ierr = SNESNASMSetDamping(snes,nasm->damping);CHKERRQ(ierr);}
   subviewflg = PETSC_FALSE;
   ierr   = PetscOptionsBool("-snes_nasm_sub_view","Print detailed information for every processor when using -snes_view","",subviewflg,&subviewflg,&flg);CHKERRQ(ierr);
@@ -207,9 +211,7 @@ PetscErrorCode SNESSetFromOptions_NASM(PetscOptions *PetscOptionsObject,SNES sne
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESView_NASM"
-PetscErrorCode SNESView_NASM(SNES snes, PetscViewer viewer)
+static PetscErrorCode SNESView_NASM(SNES snes, PetscViewer viewer)
 {
   SNES_NASM      *nasm = (SNES_NASM*)snes->data;
   PetscErrorCode ierr;
@@ -225,53 +227,51 @@ PetscErrorCode SNESView_NASM(SNES snes, PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERSTRING,&isstring);CHKERRQ(ierr);
   ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
   ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
-  ierr = MPI_Allreduce(&nasm->n,&N,1,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(&nasm->n,&N,1,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
   if (iascii) {
-    ierr = PetscViewerASCIIPrintf(viewer, "  Nonlinear Additive Schwarz: total subdomain blocks = %D\n",N);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer, "  total subdomain blocks = %D\n",N);CHKERRQ(ierr);
     if (nasm->same_local_solves) {
       if (nasm->subsnes) {
         ierr = PetscViewerASCIIPrintf(viewer,"  Local solve is the same for all blocks:\n");CHKERRQ(ierr);
         ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
-        ierr = PetscViewerGetSingleton(viewer,&sviewer);CHKERRQ(ierr);
+        ierr = PetscViewerGetSubViewer(viewer,PETSC_COMM_SELF,&sviewer);CHKERRQ(ierr);
         if (!rank) {
           ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
           ierr = SNESView(nasm->subsnes[0],sviewer);CHKERRQ(ierr);
           ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
         }
-        ierr = PetscViewerRestoreSingleton(viewer,&sviewer);CHKERRQ(ierr);
+        ierr = PetscViewerRestoreSubViewer(viewer,PETSC_COMM_SELF,&sviewer);CHKERRQ(ierr);
         ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
       }
     } else {
       /* print the solver on each block */
-      ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_TRUE);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
       ierr = PetscViewerASCIISynchronizedPrintf(viewer,"  [%d] number of local blocks = %D\n",(int)rank,nasm->n);CHKERRQ(ierr);
       ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
-      ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_FALSE);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPopSynchronized(viewer);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPrintf(viewer,"  Local solve info for each block is in the following SNES objects:\n");CHKERRQ(ierr);
       ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPrintf(viewer,"- - - - - - - - - - - - - - - - - -\n");CHKERRQ(ierr);
-      ierr = PetscViewerGetSingleton(viewer,&sviewer);CHKERRQ(ierr);
+      ierr = PetscViewerGetSubViewer(viewer,PETSC_COMM_SELF,&sviewer);CHKERRQ(ierr);
       for (i=0; i<nasm->n; i++) {
         ierr = VecGetLocalSize(nasm->x[i],&bsz);CHKERRQ(ierr);
         ierr = PetscViewerASCIIPrintf(sviewer,"[%d] local block number %D, size = %D\n",(int)rank,i,bsz);CHKERRQ(ierr);
         ierr = SNESView(nasm->subsnes[i],sviewer);CHKERRQ(ierr);
         ierr = PetscViewerASCIIPrintf(sviewer,"- - - - - - - - - - - - - - - - - -\n");CHKERRQ(ierr);
       }
-      ierr = PetscViewerRestoreSingleton(viewer,&sviewer);CHKERRQ(ierr);
+      ierr = PetscViewerRestoreSubViewer(viewer,PETSC_COMM_SELF,&sviewer);CHKERRQ(ierr);
       ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     }
   } else if (isstring) {
     ierr = PetscViewerStringSPrintf(viewer," blocks=%D,type=%s",N,SNESNASMTypes[nasm->type]);CHKERRQ(ierr);
-    ierr = PetscViewerGetSingleton(viewer,&sviewer);CHKERRQ(ierr);
+    ierr = PetscViewerGetSubViewer(viewer,PETSC_COMM_SELF,&sviewer);CHKERRQ(ierr);
     if (nasm->subsnes && !rank) {ierr = SNESView(nasm->subsnes[0],sviewer);CHKERRQ(ierr);}
-    ierr = PetscViewerRestoreSingleton(viewer,&sviewer);CHKERRQ(ierr);
+    ierr = PetscViewerRestoreSubViewer(viewer,PETSC_COMM_SELF,&sviewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMSetType"
 /*@
    SNESNASMSetType - Set the type of subdomain update used
 
@@ -298,9 +298,7 @@ PetscErrorCode SNESNASMSetType(SNES snes,PCASMType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMSetType_NASM"
-PetscErrorCode SNESNASMSetType_NASM(SNES snes,PCASMType type)
+static PetscErrorCode SNESNASMSetType_NASM(SNES snes,PCASMType type)
 {
   SNES_NASM      *nasm = (SNES_NASM*)snes->data;
 
@@ -310,8 +308,6 @@ PetscErrorCode SNESNASMSetType_NASM(SNES snes,PCASMType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMGetType"
 /*@
    SNESNASMGetType - Get the type of subdomain update used
 
@@ -332,17 +328,13 @@ PetscErrorCode SNESNASMSetType_NASM(SNES snes,PCASMType type)
 PetscErrorCode SNESNASMGetType(SNES snes,PCASMType *type)
 {
   PetscErrorCode ierr;
-  PetscErrorCode (*f)(SNES,PCASMType*);
 
   PetscFunctionBegin;
-  ierr = PetscObjectQueryFunction((PetscObject)snes,"SNESNASMGetType_C",&f);CHKERRQ(ierr);
-  if (f) {ierr = (f)(snes,type);CHKERRQ(ierr);}
+  ierr = PetscUseMethod(snes,"SNESNASMGetType_C",(SNES,PCASMType*),(snes,type));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMGetType_NASM"
-PetscErrorCode SNESNASMGetType_NASM(SNES snes,PCASMType *type)
+static PetscErrorCode SNESNASMGetType_NASM(SNES snes,PCASMType *type)
 {
   SNES_NASM      *nasm = (SNES_NASM*)snes->data;
 
@@ -351,8 +343,6 @@ PetscErrorCode SNESNASMGetType_NASM(SNES snes,PCASMType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMSetSubdomains"
 /*@
    SNESNASMSetSubdomains - Manually Set the context required to restrict and solve subdomain problems.
 
@@ -383,9 +373,7 @@ PetscErrorCode SNESNASMSetSubdomains(SNES snes,PetscInt n,SNES subsnes[],VecScat
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMSetSubdomains_NASM"
-PetscErrorCode SNESNASMSetSubdomains_NASM(SNES snes,PetscInt n,SNES subsnes[],VecScatter iscatter[],VecScatter oscatter[],VecScatter gscatter[])
+static PetscErrorCode SNESNASMSetSubdomains_NASM(SNES snes,PetscInt n,SNES subsnes[],VecScatter iscatter[],VecScatter oscatter[],VecScatter gscatter[])
 {
   PetscInt       i;
   PetscErrorCode ierr;
@@ -409,8 +397,10 @@ PetscErrorCode SNESNASMSetSubdomains_NASM(SNES snes,PetscInt n,SNES subsnes[],Ve
   }
   if (oscatter) {
     ierr = PetscMalloc1(n,&nasm->oscatter);CHKERRQ(ierr);
+    ierr = PetscMalloc1(n,&nasm->oscatter_copy);CHKERRQ(ierr);
     for (i=0; i<n; i++) {
       nasm->oscatter[i] = oscatter[i];
+      ierr = VecScatterCopy(oscatter[i], &nasm->oscatter_copy[i]);CHKERRQ(ierr);
     }
   }
   if (iscatter) {
@@ -436,8 +426,6 @@ PetscErrorCode SNESNASMSetSubdomains_NASM(SNES snes,PetscInt n,SNES subsnes[],Ve
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMGetSubdomains"
 /*@
    SNESNASMGetSubdomains - Get the local subdomain context.
 
@@ -470,9 +458,7 @@ PetscErrorCode SNESNASMGetSubdomains(SNES snes,PetscInt *n,SNES *subsnes[],VecSc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMGetSubdomains_NASM"
-PetscErrorCode SNESNASMGetSubdomains_NASM(SNES snes,PetscInt *n,SNES *subsnes[],VecScatter *iscatter[],VecScatter *oscatter[],VecScatter *gscatter[])
+static PetscErrorCode SNESNASMGetSubdomains_NASM(SNES snes,PetscInt *n,SNES *subsnes[],VecScatter *iscatter[],VecScatter *oscatter[],VecScatter *gscatter[])
 {
   SNES_NASM      *nasm = (SNES_NASM*)snes->data;
 
@@ -488,8 +474,6 @@ PetscErrorCode SNESNASMGetSubdomains_NASM(SNES snes,PetscInt *n,SNES *subsnes[],
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMGetSubdomainVecs"
 /*@
    SNESNASMGetSubdomainVecs - Get the processor-local subdomain vectors
 
@@ -522,9 +506,7 @@ PetscErrorCode SNESNASMGetSubdomainVecs(SNES snes,PetscInt *n,Vec **x,Vec **y,Ve
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMGetSubdomainVecs_NASM"
-PetscErrorCode SNESNASMGetSubdomainVecs_NASM(SNES snes,PetscInt *n,Vec **x,Vec **y,Vec **b,Vec **xl)
+static PetscErrorCode SNESNASMGetSubdomainVecs_NASM(SNES snes,PetscInt *n,Vec **x,Vec **y,Vec **b,Vec **xl)
 {
   SNES_NASM      *nasm = (SNES_NASM*)snes->data;
 
@@ -537,8 +519,6 @@ PetscErrorCode SNESNASMGetSubdomainVecs_NASM(SNES snes,PetscInt *n,Vec **x,Vec *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMSetComputeFinalJacobian"
 /*@
    SNESNASMSetComputeFinalJacobian - Schedules the computation of the global and subdomain jacobians upon convergence
 
@@ -550,7 +530,8 @@ PetscErrorCode SNESNASMGetSubdomainVecs_NASM(SNES snes,PetscInt *n,Vec **x,Vec *
 
    Level: developer
 
-   Notes: This is used almost exclusively in the implementation of ASPIN, where the converged subdomain and global jacobian
+   Notes:
+    This is used almost exclusively in the implementation of ASPIN, where the converged subdomain and global jacobian
    is needed at each linear iteration.
 
 .keywords: SNES, NASM, ASPIN
@@ -568,9 +549,7 @@ PetscErrorCode SNESNASMSetComputeFinalJacobian(SNES snes,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMSetComputeFinalJacobian_NASM"
-PetscErrorCode SNESNASMSetComputeFinalJacobian_NASM(SNES snes,PetscBool flg)
+static PetscErrorCode SNESNASMSetComputeFinalJacobian_NASM(SNES snes,PetscBool flg)
 {
   SNES_NASM      *nasm = (SNES_NASM*)snes->data;
 
@@ -580,8 +559,6 @@ PetscErrorCode SNESNASMSetComputeFinalJacobian_NASM(SNES snes,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMSetDamping"
 /*@
    SNESNASMSetDamping - Sets the update damping for NASM
 
@@ -593,6 +570,9 @@ PetscErrorCode SNESNASMSetComputeFinalJacobian_NASM(SNES snes,PetscBool flg)
 
    Level: intermediate
 
+   Notes:
+    The new solution is obtained as old solution plus dmp times (sum of the solutions on the subdomains)
+
 .keywords: SNES, NASM, damping
 
 .seealso: SNESNASM, SNESNASMGetDamping()
@@ -608,9 +588,7 @@ PetscErrorCode SNESNASMSetDamping(SNES snes,PetscReal dmp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMSetDamping_NASM"
-PetscErrorCode SNESNASMSetDamping_NASM(SNES snes,PetscReal dmp)
+static PetscErrorCode SNESNASMSetDamping_NASM(SNES snes,PetscReal dmp)
 {
   SNES_NASM      *nasm = (SNES_NASM*)snes->data;
 
@@ -619,8 +597,6 @@ PetscErrorCode SNESNASMSetDamping_NASM(SNES snes,PetscReal dmp)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMGetDamping"
 /*@
    SNESNASMGetDamping - Gets the update damping for NASM
 
@@ -638,18 +614,14 @@ PetscErrorCode SNESNASMSetDamping_NASM(SNES snes,PetscReal dmp)
 @*/
 PetscErrorCode SNESNASMGetDamping(SNES snes,PetscReal *dmp)
 {
-  PetscErrorCode (*f)(SNES,PetscReal*);
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscObjectQueryFunction((PetscObject)snes,"SNESNASMGetDamping_C",(void (**)(void))&f);CHKERRQ(ierr);
-  if (f) {ierr = (f)(snes,dmp);CHKERRQ(ierr);}
+  ierr = PetscUseMethod(snes,"SNESNASMGetDamping_C",(SNES,PetscReal*),(snes,dmp));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMGetDamping_NASM"
-PetscErrorCode SNESNASMGetDamping_NASM(SNES snes,PetscReal *dmp)
+static PetscErrorCode SNESNASMGetDamping_NASM(SNES snes,PetscReal *dmp)
 {
   SNES_NASM      *nasm = (SNES_NASM*)snes->data;
 
@@ -659,8 +631,6 @@ PetscErrorCode SNESNASMGetDamping_NASM(SNES snes,PetscReal *dmp)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMSolveLocal_Private"
 /*
   Input Parameters:
 + snes - The solver
@@ -679,27 +649,29 @@ PetscErrorCode SNESNASMSolveLocal_Private(SNES snes,Vec B,Vec Y,Vec X)
   PetscInt       i;
   PetscReal      dmp;
   PetscErrorCode ierr;
-  Vec            Xlloc,Xl,Bl,Yl;
-  VecScatter     iscat,oscat,gscat;
+  Vec            Xl,Bl,Yl,Xlloc;
+  VecScatter     iscat,oscat,gscat,oscat_copy;
   DM             dm,subdm;
   PCASMType      type;
 
   PetscFunctionBegin;
   ierr = SNESNASMGetType(snes,&type);CHKERRQ(ierr);
   ierr = SNESGetDM(snes,&dm);CHKERRQ(ierr);
-  ierr = SNESNASMGetDamping(snes,&dmp);CHKERRQ(ierr);
   ierr = VecSet(Y,0);CHKERRQ(ierr);
   if (nasm->eventrestrictinterp) {ierr = PetscLogEventBegin(nasm->eventrestrictinterp,snes,0,0,0);CHKERRQ(ierr);}
   for (i=0; i<nasm->n; i++) {
-    /* scatter the solution to the local solution */
-    Xlloc = nasm->xl[i];
-    gscat   = nasm->gscatter[i];
+    /* scatter the solution to the global solution and the local solution */
+    Xl      = nasm->x[i];
+    Xlloc   = nasm->xl[i];
     oscat   = nasm->oscatter[i];
+    oscat_copy = nasm->oscatter_copy[i];
+    gscat   = nasm->gscatter[i];
+    ierr = VecScatterBegin(oscat,X,Xl,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
     ierr = VecScatterBegin(gscat,X,Xlloc,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
     if (B) {
       /* scatter the RHS to the local RHS */
       Bl   = nasm->b[i];
-      ierr = VecScatterBegin(oscat,B,Bl,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
+      ierr = VecScatterBegin(oscat_copy,B,Bl,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
     }
   }
   if (nasm->eventrestrictinterp) {ierr = PetscLogEventEnd(nasm->eventrestrictinterp,snes,0,0,0);CHKERRQ(ierr);}
@@ -714,19 +686,20 @@ PetscErrorCode SNESNASMSolveLocal_Private(SNES snes,Vec B,Vec Y,Vec X)
     ierr    = SNESGetDM(subsnes,&subdm);CHKERRQ(ierr);
     iscat   = nasm->iscatter[i];
     oscat   = nasm->oscatter[i];
+    oscat_copy = nasm->oscatter_copy[i];
     gscat   = nasm->gscatter[i];
+    ierr = VecScatterEnd(oscat,X,Xl,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
     ierr = VecScatterEnd(gscat,X,Xlloc,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
     if (B) {
       Bl   = nasm->b[i];
-      ierr = VecScatterEnd(oscat,B,Bl,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
+      ierr = VecScatterEnd(oscat_copy,B,Bl,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
     } else Bl = NULL;
+
     ierr = DMSubDomainRestrict(dm,oscat,gscat,subdm);CHKERRQ(ierr);
-    /* Could scatter directly from X */
-    ierr = DMLocalToGlobalBegin(subdm,Xlloc,INSERT_VALUES,Xl);CHKERRQ(ierr);
-    ierr = DMLocalToGlobalEnd(subdm,Xlloc,INSERT_VALUES,Xl);CHKERRQ(ierr);
     ierr = VecCopy(Xl,Yl);CHKERRQ(ierr);
     ierr = SNESSolve(subsnes,Bl,Xl);CHKERRQ(ierr);
     ierr = VecAYPX(Yl,-1.0,Xl);CHKERRQ(ierr);
+    ierr = VecScale(Yl, nasm->damping);CHKERRQ(ierr);
     if (type == PC_ASM_BASIC) {
       ierr = VecScatterBegin(oscat,Yl,Y,ADD_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
     } else if (type == PC_ASM_RESTRICT) {
@@ -745,14 +718,16 @@ PetscErrorCode SNESNASMSolveLocal_Private(SNES snes,Vec B,Vec Y,Vec X)
       ierr = VecScatterEnd(iscat,Yl,Y,ADD_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
     } else SETERRQ(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_WRONGSTATE,"Only basic and restrict types are supported for SNESNASM");
   }
+  if (nasm->weight_set) {
+    ierr = VecPointwiseMult(Y,Y,nasm->weight);CHKERRQ(ierr);
+  }
   if (nasm->eventrestrictinterp) {ierr = PetscLogEventEnd(nasm->eventrestrictinterp,snes,0,0,0);CHKERRQ(ierr);}
+  ierr = SNESNASMGetDamping(snes,&dmp);CHKERRQ(ierr);
   ierr = VecAXPY(X,dmp,Y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNASMComputeFinalJacobian_Private"
-PetscErrorCode SNESNASMComputeFinalJacobian_Private(SNES snes, Vec Xfinal)
+static PetscErrorCode SNESNASMComputeFinalJacobian_Private(SNES snes, Vec Xfinal)
 {
   Vec            X = Xfinal;
   SNES_NASM      *nasm = (SNES_NASM*)snes->data;
@@ -774,7 +749,6 @@ PetscErrorCode SNESNASMComputeFinalJacobian_Private(SNES snes, Vec Xfinal)
     for (i=0; i<nasm->n; i++) {
       Xlloc = nasm->xl[i];
       gscat = nasm->gscatter[i];
-      oscat = nasm->oscatter[i];
       ierr = VecScatterBegin(gscat,X,Xlloc,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
     }
   }
@@ -802,9 +776,7 @@ PetscErrorCode SNESNASMComputeFinalJacobian_Private(SNES snes, Vec Xfinal)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESSolve_NASM"
-PetscErrorCode SNESSolve_NASM(SNES snes)
+static PetscErrorCode SNESSolve_NASM(SNES snes)
 {
   Vec              F;
   Vec              X;
@@ -818,9 +790,7 @@ PetscErrorCode SNESSolve_NASM(SNES snes)
 
   PetscFunctionBegin;
 
-  if (snes->xl || snes->xu || snes->ops->computevariablebounds) {
-    SETERRQ1(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_WRONGSTATE, "SNES solver %s does not support bounds", ((PetscObject)snes)->type_name);
-  }
+  if (snes->xl || snes->xu || snes->ops->computevariablebounds) SETERRQ1(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_WRONGSTATE, "SNES solver %s does not support bounds", ((PetscObject)snes)->type_name);
 
   ierr = PetscCitationsRegister(SNESCitation,&SNEScite);CHKERRQ(ierr);
   X = snes->vec_sol;
@@ -865,7 +835,7 @@ PetscErrorCode SNESSolve_NASM(SNES snes)
   /* copy the initial solution over for later */
   if (nasm->fjtype == 2) {ierr = VecCopy(X,nasm->xinit);CHKERRQ(ierr);}
 
-  for (i = 0; i < snes->max_its; i++) {
+  for (i=0; i < snes->max_its; i++) {
     ierr = SNESNASMSolveLocal_Private(snes,B,Y,X);CHKERRQ(ierr);
     if (normschedule == SNES_NORM_ALWAYS || ((i == snes->max_its - 1) && (normschedule == SNES_NORM_INITIAL_FINAL_ONLY || normschedule == SNES_NORM_FINAL_ONLY))) {
       ierr = SNESComputeFunction(snes,X,F);CHKERRQ(ierr);
@@ -901,19 +871,22 @@ PetscErrorCode SNESSolve_NASM(SNES snes)
    Options Database:
 +  -snes_nasm_log - enable logging events for the communication and solve stages
 .  -snes_nasm_type <basic,restrict> - type of subdomain update used
+.  -snes_asm_damping <dmp> - the new solution is obtained as old solution plus dmp times (sum of the solutions on the subdomains)
 .  -snes_nasm_finaljacobian - compute the local and global jacobians of the final iterate
-.  -snes_nasm_finaljacobian_type <finalinner,finalouter,initial> pick state the jacobian is calculated at
+.  -snes_nasm_finaljacobian_type <finalinner,finalouter,initial> - pick state the jacobian is calculated at
 .  -sub_snes_ - options prefix of the subdomain nonlinear solves
 .  -sub_ksp_ - options prefix of the subdomain Krylov solver
 -  -sub_pc_ - options prefix of the subdomain preconditioner
 
    Level: advanced
 
-.seealso: SNESCreate(), SNES, SNESSetType(), SNESType (for list of available types)
+   References:
+.  1. - Peter R. Brune, Matthew G. Knepley, Barry F. Smith, and Xuemin Tu, "Composing Scalable Nonlinear Algebraic Solvers",
+   SIAM Review, 57(4), 2015
+
+.seealso: SNESCreate(), SNES, SNESSetType(), SNESType (for list of available types), SNESNASMSetType(), SNESNASMGetType(), SNESNASMSetSubdomains(), SNESNASMGetSubdomains(), SNESNASMGetSubdomainVecs(), SNESNASMSetComputeFinalJacobian(), SNESNASMSetDamping(), SNESNASMGetDamping()
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESCreate_NASM"
 PETSC_EXTERN PetscErrorCode SNESCreate_NASM(SNES snes)
 {
   SNES_NASM      *nasm;
@@ -930,13 +903,15 @@ PETSC_EXTERN PetscErrorCode SNESCreate_NASM(SNES snes)
   nasm->y        = 0;
   nasm->b        = 0;
   nasm->oscatter = 0;
+  nasm->oscatter_copy = 0;
   nasm->iscatter = 0;
   nasm->gscatter = 0;
   nasm->damping  = 1.;
 
-  nasm->type = PC_ASM_BASIC;
-  nasm->finaljacobian = PETSC_FALSE;
+  nasm->type              = PC_ASM_BASIC;
+  nasm->finaljacobian     = PETSC_FALSE;
   nasm->same_local_solves = PETSC_TRUE;
+  nasm->weight_set        = PETSC_FALSE;
 
   snes->ops->destroy        = SNESDestroy_NASM;
   snes->ops->setup          = SNESSetUp_NASM;
@@ -946,7 +921,9 @@ PETSC_EXTERN PetscErrorCode SNESCreate_NASM(SNES snes)
   snes->ops->reset          = SNESReset_NASM;
 
   snes->usesksp = PETSC_FALSE;
-  snes->usespc  = PETSC_FALSE;
+  snes->usesnpc = PETSC_FALSE;
+
+  snes->alwayscomputesfinalresidual = PETSC_FALSE;
 
   nasm->fjtype              = 0;
   nasm->xinit               = NULL;
@@ -969,3 +946,86 @@ PETSC_EXTERN PetscErrorCode SNESCreate_NASM(SNES snes)
   PetscFunctionReturn(0);
 }
 
+/*@
+   SNESNASMGetSNES - Gets a subsolver
+
+   Not collective
+
+   Input Parameters:
++  snes - the SNES context
+-  i - the number of the subsnes to get
+
+   Output Parameters:
+.  subsnes - the subsolver context
+
+   Level: intermediate
+
+.keywords: SNES, NASM
+
+.seealso: SNESNASM, SNESNASMGetNumber()
+@*/
+PetscErrorCode SNESNASMGetSNES(SNES snes,PetscInt i,SNES *subsnes)
+{
+  SNES_NASM      *nasm = (SNES_NASM*)snes->data;
+
+  PetscFunctionBegin;
+  if (i < 0 || i >= nasm->n) SETERRQ(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_OUTOFRANGE,"No such subsolver");
+  *subsnes = nasm->subsnes[i];
+  PetscFunctionReturn(0);
+}
+
+/*@
+   SNESNASMGetNumber - Gets number of subsolvers
+
+   Not collective
+
+   Input Parameters:
+.  snes - the SNES context
+
+   Output Parameters:
+.  n - the number of subsolvers
+
+   Level: intermediate
+
+.keywords: SNES, NASM
+
+.seealso: SNESNASM, SNESNASMGetSNES()
+@*/
+PetscErrorCode SNESNASMGetNumber(SNES snes,PetscInt *n)
+{
+  SNES_NASM      *nasm = (SNES_NASM*)snes->data;
+
+  PetscFunctionBegin;
+  *n = nasm->n;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   SNESNASMSetWeight - Sets weight to use when adding overlapping updates
+
+   Collective
+
+   Input Parameters:
++  snes - the SNES context
+-  weight - the weights to use (typically 1/N for each dof, where N is the number of patches it appears in)
+
+   Level: intermediate
+
+.keywords: SNES, NASM
+
+.seealso: SNESNASM
+@*/
+PetscErrorCode SNESNASMSetWeight(SNES snes,Vec weight)
+{
+  SNES_NASM      *nasm = (SNES_NASM*)snes->data;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+
+  ierr = VecDestroy(&nasm->weight);CHKERRQ(ierr);
+  nasm->weight_set = PETSC_TRUE;
+  nasm->weight     = weight;
+  ierr = PetscObjectReference((PetscObject)nasm->weight);CHKERRQ(ierr);
+
+  PetscFunctionReturn(0);
+}
