@@ -1,8 +1,6 @@
 #include <../src/snes/impls/ncg/snesncgimpl.h> /*I "petscsnes.h" I*/
 const char *const SNESNCGTypes[] = {"FR","PRP","HS","DY","CD","SNESNCGType","SNES_NCG_",0};
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESReset_NCG"
 PetscErrorCode SNESReset_NCG(SNES snes)
 {
   PetscFunctionBegin;
@@ -19,8 +17,6 @@ PetscErrorCode SNESReset_NCG(SNES snes)
 
   Application Interface Routine: SNESDestroy()
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESDestroy_NCG"
 PetscErrorCode SNESDestroy_NCG(SNES snes)
 {
   PetscErrorCode ierr;
@@ -43,15 +39,13 @@ PetscErrorCode SNESDestroy_NCG(SNES snes)
 
 PETSC_EXTERN PetscErrorCode SNESLineSearchCreate_NCGLinear(SNESLineSearch);
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetUp_NCG"
 PetscErrorCode SNESSetUp_NCG(SNES snes)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   ierr = SNESSetWorkVecs(snes,2);CHKERRQ(ierr);
-  if (snes->pcside == PC_RIGHT) SETERRQ(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_WRONGSTATE, "SNESNCG only supports left preconditioning");
+  if (snes->npcside== PC_RIGHT) SETERRQ(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_WRONGSTATE, "SNESNCG only supports left preconditioning");
   if (snes->functype == SNES_FUNCTION_DEFAULT) snes->functype = SNES_FUNCTION_UNPRECONDITIONED;
   PetscFunctionReturn(0);
 }
@@ -63,8 +57,6 @@ PetscErrorCode SNESSetUp_NCG(SNES snes)
 
   Application Interface Routine: SNESSetFromOptions()
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetFromOptions_NCG"
 static PetscErrorCode SNESSetFromOptions_NCG(PetscOptionItems *PetscOptionsObject,SNES snes)
 {
   SNES_NCG       *ncg = (SNES_NCG*)snes->data;
@@ -84,7 +76,7 @@ static PetscErrorCode SNESSetFromOptions_NCG(PetscOptionItems *PetscOptionsObjec
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   if (!snes->linesearch) {
     ierr = SNESGetLineSearch(snes, &linesearch);CHKERRQ(ierr);
-    if (!snes->pc) {
+    if (!snes->npc) {
       ierr = SNESLineSearchSetType(linesearch, SNESLINESEARCHCP);CHKERRQ(ierr);
     } else {
       ierr = SNESLineSearchSetType(linesearch, SNESLINESEARCHL2);CHKERRQ(ierr);
@@ -103,8 +95,6 @@ static PetscErrorCode SNESSetFromOptions_NCG(PetscOptionItems *PetscOptionsObjec
 
   Application Interface Routine: SNESView()
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESView_NCG"
 static PetscErrorCode SNESView_NCG(SNES snes, PetscViewer viewer)
 {
   SNES_NCG      *ncg = (SNES_NCG *) snes->data;
@@ -114,13 +104,11 @@ static PetscErrorCode SNESView_NCG(SNES snes, PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject) viewer, PETSCVIEWERASCII, &iascii);CHKERRQ(ierr);
   if (iascii) {
-    ierr = PetscViewerASCIIPrintf(viewer, "ncg type: %s\n", SNESNCGTypes[ncg->type]);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer, "  type: %s\n", SNESNCGTypes[ncg->type]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESLineSearchApply_NCGLinear"
 PetscErrorCode SNESLineSearchApply_NCGLinear(SNESLineSearch linesearch)
 {
   PetscScalar    alpha, ptAp;
@@ -162,8 +150,6 @@ PetscErrorCode SNESLineSearchApply_NCGLinear(SNESLineSearch linesearch)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESLineSearchCreate_NCGLinear"
 PETSC_EXTERN PetscErrorCode SNESLineSearchCreate_NCGLinear(SNESLineSearch linesearch)
 {
   PetscFunctionBegin;
@@ -176,8 +162,6 @@ PETSC_EXTERN PetscErrorCode SNESLineSearchCreate_NCGLinear(SNESLineSearch linese
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNCGComputeYtJtF_Private"
 /*
 
  Assuming F = SNESComputeFunction(X) compute Y^tJ^tF using a simple secant approximation of the jacobian.
@@ -200,8 +184,6 @@ PetscErrorCode SNESNCGComputeYtJtF_Private(SNES snes, Vec X, Vec F, Vec Y, Vec W
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNCGSetType"
 /*@
     SNESNCGSetType - Sets the conjugate update type for SNESNCG.
 
@@ -238,8 +220,6 @@ PetscErrorCode SNESNCGSetType(SNES snes, SNESNCGType btype)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESNCGSetType_NCG"
 PetscErrorCode SNESNCGSetType_NCG(SNES snes, SNESNCGType btype)
 {
   SNES_NCG *ncg = (SNES_NCG*)snes->data;
@@ -260,8 +240,6 @@ PetscErrorCode SNESNCGSetType_NCG(SNES snes, SNESNCGType btype)
 
   Application Interface Routine: SNESSolve()
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESSolve_NCG"
 PetscErrorCode SNESSolve_NCG(SNES snes)
 {
   SNES_NCG             *ncg = (SNES_NCG*)snes->data;
@@ -296,9 +274,9 @@ PetscErrorCode SNESSolve_NCG(SNES snes)
 
   /* compute the initial function and preconditioned update dX */
 
-  if (snes->pc && snes->functype == SNES_FUNCTION_PRECONDITIONED) {
+  if (snes->npc && snes->functype == SNES_FUNCTION_PRECONDITIONED) {
     ierr = SNESApplyNPC(snes,X,NULL,dX);CHKERRQ(ierr);
-    ierr = SNESGetConvergedReason(snes->pc,&reason);CHKERRQ(ierr);
+    ierr = SNESGetConvergedReason(snes->npc,&reason);CHKERRQ(ierr);
     if (reason < 0  && reason != SNES_DIVERGED_MAX_IT) {
       snes->reason = SNES_DIVERGED_INNER;
       PetscFunctionReturn(0);
@@ -315,10 +293,10 @@ PetscErrorCode SNESSolve_NCG(SNES snes)
     SNESCheckFunctionNorm(snes,fnorm);
     ierr = VecCopy(F,dX);CHKERRQ(ierr);
   }
-  if (snes->pc) {
+  if (snes->npc) {
     if (snes->functype == SNES_FUNCTION_UNPRECONDITIONED) {
       ierr = SNESApplyNPC(snes,X,F,dX);CHKERRQ(ierr);
-      ierr = SNESGetConvergedReason(snes->pc,&reason);CHKERRQ(ierr);
+      ierr = SNESGetConvergedReason(snes->npc,&reason);CHKERRQ(ierr);
       if (reason < 0  && reason != SNES_DIVERGED_MAX_IT) {
         snes->reason = SNES_DIVERGED_INNER;
         PetscFunctionReturn(0);
@@ -360,7 +338,7 @@ PetscErrorCode SNESSolve_NCG(SNES snes)
         PetscFunctionReturn(0);
       }
     }
-    if (snes->nfuncs >= snes->max_funcs) {
+    if (snes->nfuncs >= snes->max_funcs && snes->max_funcs >= 0) {
       snes->reason = SNES_DIVERGED_FUNCTION_COUNT;
       PetscFunctionReturn(0);
     }
@@ -380,10 +358,10 @@ PetscErrorCode SNESSolve_NCG(SNES snes)
     if (snes->ops->update) {
       ierr = (*snes->ops->update)(snes, snes->iter);CHKERRQ(ierr);
     }
-    if (snes->pc) {
+    if (snes->npc) {
       if (snes->functype == SNES_FUNCTION_PRECONDITIONED) {
         ierr = SNESApplyNPC(snes,X,NULL,dX);CHKERRQ(ierr);
-        ierr = SNESGetConvergedReason(snes->pc,&reason);CHKERRQ(ierr);
+        ierr = SNESGetConvergedReason(snes->npc,&reason);CHKERRQ(ierr);
         if (reason < 0  && reason != SNES_DIVERGED_MAX_IT) {
           snes->reason = SNES_DIVERGED_INNER;
           PetscFunctionReturn(0);
@@ -391,7 +369,7 @@ PetscErrorCode SNESSolve_NCG(SNES snes)
         ierr = VecCopy(dX,F);CHKERRQ(ierr);
       } else {
         ierr = SNESApplyNPC(snes,X,F,dX);CHKERRQ(ierr);
-        ierr = SNESGetConvergedReason(snes->pc,&reason);CHKERRQ(ierr);
+        ierr = SNESGetConvergedReason(snes->npc,&reason);CHKERRQ(ierr);
         if (reason < 0  && reason != SNES_DIVERGED_MAX_IT) {
           snes->reason = SNES_DIVERGED_INNER;
           PetscFunctionReturn(0);
@@ -467,7 +445,8 @@ PetscErrorCode SNESSolve_NCG(SNES snes)
 .   -snes_linesearch_type <cp,l2,basic> - Line search type.
 -   -snes_ncg_monitor - Print relevant information about the ncg iteration.
 
-   Notes: This solves the nonlinear system of equations F(x) = 0 using the nonlinear generalization of the conjugate
+   Notes:
+    This solves the nonlinear system of equations F(x) = 0 using the nonlinear generalization of the conjugate
           gradient method.  This may be used with a nonlinear preconditioner used to pick the new search directions, but otherwise
           chooses the initial search direction as F(x) for the initial guess x.
 
@@ -478,10 +457,8 @@ PetscErrorCode SNESSolve_NCG(SNES snes)
    SIAM Review, 57(4), 2015
 
 
-.seealso:  SNESCreate(), SNES, SNESSetType(), SNESNEWTONLS, SNESNEWTONTR, SNESNGMRES, SNESNQN
+.seealso:  SNESCreate(), SNES, SNESSetType(), SNESNEWTONLS, SNESNEWTONTR, SNESNGMRES, SNESQN
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "SNESCreate_NCG"
 PETSC_EXTERN PetscErrorCode SNESCreate_NCG(SNES snes)
 {
   PetscErrorCode ierr;
@@ -496,8 +473,10 @@ PETSC_EXTERN PetscErrorCode SNESCreate_NCG(SNES snes)
   snes->ops->reset          = SNESReset_NCG;
 
   snes->usesksp = PETSC_FALSE;
-  snes->usespc  = PETSC_TRUE;
-  snes->pcside  = PC_LEFT;
+  snes->usesnpc = PETSC_TRUE;
+  snes->npcside = PC_LEFT;
+
+  snes->alwayscomputesfinalresidual = PETSC_TRUE;
 
   if (!snes->tolerancesset) {
     snes->max_funcs = 30000;
