@@ -1,8 +1,6 @@
 #include <../src/snes/impls/richardson/snesrichardsonimpl.h>
 
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESReset_NRichardson"
 PetscErrorCode SNESReset_NRichardson(SNES snes)
 {
   PetscFunctionBegin;
@@ -17,8 +15,6 @@ PetscErrorCode SNESReset_NRichardson(SNES snes)
 
   Application Interface Routine: SNESDestroy()
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESDestroy_NRichardson"
 PetscErrorCode SNESDestroy_NRichardson(SNES snes)
 {
   PetscErrorCode ierr;
@@ -39,12 +35,10 @@ PetscErrorCode SNESDestroy_NRichardson(SNES snes)
 
    Application Interface Routine: SNESSetUp()
  */
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetUp_NRichardson"
 PetscErrorCode SNESSetUp_NRichardson(SNES snes)
 {
   PetscFunctionBegin;
-  if (snes->pcside == PC_RIGHT) {SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"NRichardson only supports left preconditioning");}
+  if (snes->npcside== PC_RIGHT) {SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"NRichardson only supports left preconditioning");}
   if (snes->functype == SNES_FUNCTION_DEFAULT) snes->functype = SNES_FUNCTION_UNPRECONDITIONED;
   PetscFunctionReturn(0);
 }
@@ -57,9 +51,7 @@ PetscErrorCode SNESSetUp_NRichardson(SNES snes)
 
   Application Interface Routine: SNESSetFromOptions()
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetFromOptions_NRichardson"
-static PetscErrorCode SNESSetFromOptions_NRichardson(PetscOptions *PetscOptionsObject,SNES snes)
+static PetscErrorCode SNESSetFromOptions_NRichardson(PetscOptionItems *PetscOptionsObject,SNES snes)
 {
   PetscErrorCode ierr;
   SNESLineSearch linesearch;
@@ -83,8 +75,6 @@ static PetscErrorCode SNESSetFromOptions_NRichardson(PetscOptions *PetscOptionsO
 
   Application Interface Routine: SNESView()
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESView_NRichardson"
 static PetscErrorCode SNESView_NRichardson(SNES snes, PetscViewer viewer)
 {
   PetscBool      iascii;
@@ -108,8 +98,6 @@ static PetscErrorCode SNESView_NRichardson(SNES snes, PetscViewer viewer)
 
   Application Interface Routine: SNESSolve()
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESSolve_NRichardson"
 PetscErrorCode SNESSolve_NRichardson(SNES snes)
 {
   Vec                  X, Y, F;
@@ -120,10 +108,7 @@ PetscErrorCode SNESSolve_NRichardson(SNES snes)
   SNESConvergedReason  reason;
 
   PetscFunctionBegin;
-
-  if (snes->xl || snes->xu || snes->ops->computevariablebounds) {
-    SETERRQ1(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_WRONGSTATE, "SNES solver %s does not support bounds", ((PetscObject)snes)->type_name);
-  }
+  if (snes->xl || snes->xu || snes->ops->computevariablebounds) SETERRQ1(PetscObjectComm((PetscObject)snes),PETSC_ERR_ARG_WRONGSTATE, "SNES solver %s does not support bounds", ((PetscObject)snes)->type_name);
 
   snes->reason = SNES_CONVERGED_ITERATING;
 
@@ -137,9 +122,9 @@ PetscErrorCode SNESSolve_NRichardson(SNES snes)
   snes->norm = 0.;
   ierr       = PetscObjectSAWsGrantAccess((PetscObject)snes);CHKERRQ(ierr);
 
-  if (snes->pc && snes->functype == SNES_FUNCTION_PRECONDITIONED) {
+  if (snes->npc && snes->functype == SNES_FUNCTION_PRECONDITIONED) {
     ierr = SNESApplyNPC(snes,X,NULL,F);CHKERRQ(ierr);
-    ierr = SNESGetConvergedReason(snes->pc,&reason);CHKERRQ(ierr);
+    ierr = SNESGetConvergedReason(snes->npc,&reason);CHKERRQ(ierr);
     if (reason < 0  && reason != SNES_DIVERGED_MAX_IT) {
       snes->reason = SNES_DIVERGED_INNER;
       PetscFunctionReturn(0);
@@ -153,9 +138,9 @@ PetscErrorCode SNESSolve_NRichardson(SNES snes)
     ierr = VecNorm(F,NORM_2,&fnorm);CHKERRQ(ierr);
     SNESCheckFunctionNorm(snes,fnorm);
   }
-  if (snes->pc && snes->functype == SNES_FUNCTION_UNPRECONDITIONED) {
+  if (snes->npc && snes->functype == SNES_FUNCTION_UNPRECONDITIONED) {
       ierr = SNESApplyNPC(snes,X,F,Y);CHKERRQ(ierr);
-      ierr = SNESGetConvergedReason(snes->pc,&reason);CHKERRQ(ierr);
+      ierr = SNESGetConvergedReason(snes->npc,&reason);CHKERRQ(ierr);
       if (reason < 0  && reason != SNES_DIVERGED_MAX_IT) {
         snes->reason = SNES_DIVERGED_INNER;
         PetscFunctionReturn(0);
@@ -195,7 +180,7 @@ PetscErrorCode SNESSolve_NRichardson(SNES snes)
         break;
       }
     }
-    if (snes->nfuncs >= snes->max_funcs) {
+    if (snes->nfuncs >= snes->max_funcs && snes->max_funcs >= 0) {
       snes->reason = SNES_DIVERGED_FUNCTION_COUNT;
       break;
     }
@@ -216,7 +201,7 @@ PetscErrorCode SNESSolve_NRichardson(SNES snes)
       ierr = (*snes->ops->update)(snes, snes->iter);CHKERRQ(ierr);
     }
 
-    if (snes->pc) {
+    if (snes->npc) {
       if (snes->functype == SNES_FUNCTION_PRECONDITIONED) {
         ierr = SNESApplyNPC(snes,X,NULL,Y);CHKERRQ(ierr);
         ierr = VecNorm(F,NORM_2,&fnorm);CHKERRQ(ierr);
@@ -224,7 +209,7 @@ PetscErrorCode SNESSolve_NRichardson(SNES snes)
       } else {
         ierr = SNESApplyNPC(snes,X,F,Y);CHKERRQ(ierr);
       }
-      ierr = SNESGetConvergedReason(snes->pc,&reason);CHKERRQ(ierr);
+      ierr = SNESGetConvergedReason(snes->npc,&reason);CHKERRQ(ierr);
       if (reason < 0  && reason != SNES_DIVERGED_MAX_IT) {
         snes->reason = SNES_DIVERGED_INNER;
         PetscFunctionReturn(0);
@@ -249,7 +234,8 @@ PetscErrorCode SNESSolve_NRichardson(SNES snes)
 +   -snes_linesearch_type <l2,cp,basic> Line search type.
 -   -snes_linesearch_damping<1.0> Damping for the line search.
 
-  Notes: If no inner nonlinear preconditioner is provided then solves F(x) - b = 0 using x^{n+1} = x^{n} - lambda
+  Notes:
+    If no inner nonlinear preconditioner is provided then solves F(x) - b = 0 using x^{n+1} = x^{n} - lambda
             (F(x^n) - b) where lambda is obtained either SNESLineSearchSetDamping(), -snes_damping or a line search.  If
             an inner nonlinear preconditioner is provided (either with -npc_snes_type or SNESSetNPC()) then the inner
             solver is called an initial solution x^n and the nonlinear Richardson uses x^{n+1} = x^{n} + lambda d^{n}
@@ -260,10 +246,10 @@ PetscErrorCode SNESSolve_NRichardson(SNES snes)
 
      This uses no derivative information thus will be much slower then Newton's method obtained with -snes_type ls
 
+     Only supports left non-linear preconditioning.
+
 .seealso:  SNESCreate(), SNES, SNESSetType(), SNESNEWTONLS, SNESNEWTONTR, SNESNGMRES, SNESQN, SNESNCG
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "SNESCreate_NRichardson"
 PETSC_EXTERN PetscErrorCode SNESCreate_NRichardson(SNES snes)
 {
   PetscErrorCode   ierr;
@@ -278,9 +264,11 @@ PETSC_EXTERN PetscErrorCode SNESCreate_NRichardson(SNES snes)
   snes->ops->reset          = SNESReset_NRichardson;
 
   snes->usesksp = PETSC_FALSE;
-  snes->usespc  = PETSC_TRUE;
+  snes->usesnpc = PETSC_TRUE;
 
-  snes->pcside = PC_LEFT;
+  snes->npcside= PC_LEFT;
+
+  snes->alwayscomputesfinalresidual = PETSC_TRUE;
 
   ierr       = PetscNewLog(snes,&neP);CHKERRQ(ierr);
   snes->data = (void*) neP;
