@@ -1,12 +1,8 @@
 
 #include <../src/snes/impls/vi/rs/virsimpl.h> /*I "petscsnes.h" I*/
-#include <petsc/private/kspimpl.h>
-#include <petsc/private/matimpl.h>
 #include <petsc/private/dmimpl.h>
 #include <petsc/private/vecimpl.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESVIGetInactiveSet"
 /*
    SNESVIGetInactiveSet - Gets the global indices for the inactive set variables (these correspond to the degrees of freedom the linear
      system is solved on)
@@ -41,12 +37,12 @@ typedef struct {
   PetscErrorCode (*createinterpolation)(DM,DM,Mat*,Vec*);  /* DM's original routines */
   PetscErrorCode (*coarsen)(DM, MPI_Comm, DM*);
   PetscErrorCode (*createglobalvector)(DM,Vec*);
+  PetscErrorCode (*getinjection)(DM,DM,Mat*);
+  PetscErrorCode (*hascreateinjection)(DM,PetscBool*);
 
   DM dm;                                                  /* when destroying this object we need to reset the above function into the base DM */
 } DM_SNESVI;
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCreateGlobalVector_SNESVI"
 /*
      DMCreateGlobalVector_SNESVI - Creates global vector of the size of the reduced space
 
@@ -65,8 +61,15 @@ PetscErrorCode  DMCreateGlobalVector_SNESVI(DM dm,Vec *vec)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCreateInterpolation_SNESVI"
+static PetscErrorCode DMHasCreateInjection_SNESVI(DM dm, PetscBool *flg)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  PetscValidPointer(flg,2);
+  *flg = PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
+
 /*
      DMCreateInterpolation_SNESVI - Modifieds the interpolation obtained from the DM by removing all rows and columns associated with active constraints.
 
@@ -87,16 +90,15 @@ PetscErrorCode  DMCreateInterpolation_SNESVI(DM dm1,DM dm2,Mat *mat,Vec *vec)
   ierr = PetscContainerGetPointer(isnes,(void**)&dmsnesvi2);CHKERRQ(ierr);
 
   ierr = (*dmsnesvi1->createinterpolation)(dm1,dm2,&interp,NULL);CHKERRQ(ierr);
-  ierr = MatGetSubMatrix(interp,dmsnesvi2->inactive,dmsnesvi1->inactive,MAT_INITIAL_MATRIX,mat);CHKERRQ(ierr);
+  ierr = MatCreateSubMatrix(interp,dmsnesvi2->inactive,dmsnesvi1->inactive,MAT_INITIAL_MATRIX,mat);CHKERRQ(ierr);
   ierr = MatDestroy(&interp);CHKERRQ(ierr);
   *vec = 0;
   PetscFunctionReturn(0);
 }
 
-extern PetscErrorCode  DMSetVI(DM,IS);
+static PetscErrorCode DMSetVI(DM,IS);
+static PetscErrorCode DMDestroyVI(DM);
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCoarsen_SNESVI"
 /*
      DMCoarsen_SNESVI - Computes the regular coarsened DM then computes additional information about its inactive set
 
@@ -178,8 +180,6 @@ PetscErrorCode  DMCoarsen_SNESVI(DM dm1,MPI_Comm comm,DM *dm2)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDestroy_SNESVI"
 PetscErrorCode DMDestroy_SNESVI(DM_SNESVI *dmsnesvi)
 {
   PetscErrorCode ierr;
@@ -189,6 +189,8 @@ PetscErrorCode DMDestroy_SNESVI(DM_SNESVI *dmsnesvi)
   dmsnesvi->dm->ops->createinterpolation = dmsnesvi->createinterpolation;
   dmsnesvi->dm->ops->coarsen             = dmsnesvi->coarsen;
   dmsnesvi->dm->ops->createglobalvector  = dmsnesvi->createglobalvector;
+  dmsnesvi->dm->ops->getinjection        = dmsnesvi->getinjection;
+  dmsnesvi->dm->ops->hascreateinjection  = dmsnesvi->hascreateinjection;
   /* need to clear out this vectors because some of them may not have a reference to the DM
     but they are counted as having references to the DM in DMDestroy() */
   ierr = DMClearGlobalVectors(dmsnesvi->dm);CHKERRQ(ierr);
@@ -198,14 +200,12 @@ PetscErrorCode DMDestroy_SNESVI(DM_SNESVI *dmsnesvi)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSetVI"
 /*
      DMSetVI - Marks a DM as associated with a VI problem. This causes the interpolation/restriction operators to
                be restricted to only those variables NOT associated with active constraints.
 
 */
-PetscErrorCode  DMSetVI(DM dm,IS inactive)
+static PetscErrorCode DMSetVI(DM dm,IS inactive)
 {
   PetscErrorCode ierr;
   PetscContainer isnes;
@@ -231,6 +231,10 @@ PetscErrorCode  DMSetVI(DM dm,IS inactive)
     dm->ops->coarsen              = DMCoarsen_SNESVI;
     dmsnesvi->createglobalvector  = dm->ops->createglobalvector;
     dm->ops->createglobalvector   = DMCreateGlobalVector_SNESVI;
+    dmsnesvi->getinjection        = dm->ops->getinjection;
+    dm->ops->getinjection         = NULL;
+    dmsnesvi->hascreateinjection  = dm->ops->hascreateinjection;
+    dm->ops->hascreateinjection   = DMHasCreateInjection_SNESVI;
   } else {
     ierr = PetscContainerGetPointer(isnes,(void**)&dmsnesvi);CHKERRQ(ierr);
     ierr = ISDestroy(&dmsnesvi->inactive);CHKERRQ(ierr);
@@ -243,13 +247,11 @@ PetscErrorCode  DMSetVI(DM dm,IS inactive)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDestroyVI"
 /*
      DMDestroyVI - Frees the DM_SNESVI object contained in the DM
          - also resets the function pointers in the DM for createinterpolation() etc to use the original DM
 */
-PetscErrorCode  DMDestroyVI(DM dm)
+static PetscErrorCode DMDestroyVI(DM dm)
 {
   PetscErrorCode ierr;
 
@@ -262,10 +264,6 @@ PetscErrorCode  DMDestroyVI(DM dm)
 /* --------------------------------------------------------------------------------------------------------*/
 
 
-
-
-#undef __FUNCT__
-#define __FUNCT__ "SNESCreateIndexSets_VINEWTONRSLS"
 PetscErrorCode SNESCreateIndexSets_VINEWTONRSLS(SNES snes,Vec X,Vec F,IS *ISact,IS *ISinact)
 {
   PetscErrorCode ierr;
@@ -277,8 +275,6 @@ PetscErrorCode SNESCreateIndexSets_VINEWTONRSLS(SNES snes,Vec X,Vec F,IS *ISact,
 }
 
 /* Create active and inactive set vectors. The local size of this vector is set and petsc computes the global size */
-#undef __FUNCT__
-#define __FUNCT__ "SNESCreateSubVectors_VINEWTONRSLS"
 PetscErrorCode SNESCreateSubVectors_VINEWTONRSLS(SNES snes,PetscInt n,Vec *newv)
 {
   PetscErrorCode ierr;
@@ -293,8 +289,6 @@ PetscErrorCode SNESCreateSubVectors_VINEWTONRSLS(SNES snes,PetscInt n,Vec *newv)
 }
 
 /* Resets the snes PC and KSP when the active set sizes change */
-#undef __FUNCT__
-#define __FUNCT__ "SNESVIResetPCandKSP"
 PetscErrorCode SNESVIResetPCandKSP(SNES snes,Mat Amat,Mat Pmat)
 {
   PetscErrorCode ierr;
@@ -303,11 +297,12 @@ PetscErrorCode SNESVIResetPCandKSP(SNES snes,Mat Amat,Mat Pmat)
   PetscFunctionBegin;
   ierr = SNESGetKSP(snes,&snesksp);CHKERRQ(ierr);
   ierr = KSPReset(snesksp);CHKERRQ(ierr);
+  ierr = KSPResetFromOptions(snesksp);CHKERRQ(ierr);
 
   /*
   KSP                    kspnew;
   PC                     pcnew;
-  const MatSolverPackage stype;
+  MatSolverType          stype;
 
 
   ierr = KSPCreate(PetscObjectComm((PetscObject)snes),&kspnew);CHKERRQ(ierr);
@@ -319,8 +314,8 @@ PetscErrorCode SNESVIResetPCandKSP(SNES snes,Mat Amat,Mat Pmat)
   ierr = KSPGetPC(kspnew,&pcnew);CHKERRQ(ierr);
   ierr = PCSetType(kspnew->pc,((PetscObject)snesksp->pc)->type_name);CHKERRQ(ierr);
   ierr = PCSetOperators(kspnew->pc,Amat,Pmat);CHKERRQ(ierr);
-  ierr = PCFactorGetMatSolverPackage(snesksp->pc,&stype);CHKERRQ(ierr);
-  ierr = PCFactorSetMatSolverPackage(kspnew->pc,stype);CHKERRQ(ierr);
+  ierr = PCFactorGetMatSolverType(snesksp->pc,&stype);CHKERRQ(ierr);
+  ierr = PCFactorSetMatSolverType(kspnew->pc,stype);CHKERRQ(ierr);
   ierr = KSPDestroy(&snesksp);CHKERRQ(ierr);
   snes->ksp = kspnew;
   ierr = PetscLogObjectParent((PetscObject)snes,(PetscObject)kspnew);CHKERRQ(ierr);
@@ -331,8 +326,6 @@ PetscErrorCode SNESVIResetPCandKSP(SNES snes,Mat Amat,Mat Pmat)
 /* Variational Inequality solver using reduce space method. No semismooth algorithm is
    implemented in this algorithm. It basically identifies the active constraints and does
    a linear solve on the other variables (those not associated with the active constraints). */
-#undef __FUNCT__
-#define __FUNCT__ "SNESSolve_VINEWTONRSLS"
 PetscErrorCode SNESSolve_VINEWTONRSLS(SNES snes)
 {
   SNES_VINEWTONRSLS    *vi = (SNES_VINEWTONRSLS*)snes->data;
@@ -349,7 +342,7 @@ PetscErrorCode SNESSolve_VINEWTONRSLS(SNES snes)
   /* Multigrid must use Galerkin for coarse grids with active set/reduced space methods; cannot rediscretize on coarser grids*/
   ierr = SNESGetKSP(snes,&ksp);CHKERRQ(ierr);
   ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
-  ierr = PCMGSetGalerkin(pc,PETSC_TRUE);CHKERRQ(ierr);
+  ierr = PCMGSetGalerkin(pc,PC_MG_GALERKIN_BOTH);CHKERRQ(ierr);
 
   snes->numFailures            = 0;
   snes->numLinearSolveFailures = 0;
@@ -372,8 +365,7 @@ PetscErrorCode SNESSolve_VINEWTONRSLS(SNES snes)
   ierr = SNESVIProjectOntoBounds(snes,X);CHKERRQ(ierr);
   ierr = SNESComputeFunction(snes,X,F);CHKERRQ(ierr);
   ierr = SNESVIComputeInactiveSetFnorm(snes,F,X,&fnorm);CHKERRQ(ierr);
-  ierr = VecNormBegin(X,NORM_2,&xnorm);CHKERRQ(ierr);        /* xnorm <- ||x||  */
-  ierr = VecNormEnd(X,NORM_2,&xnorm);CHKERRQ(ierr);
+  ierr = VecNorm(X,NORM_2,&xnorm);CHKERRQ(ierr);        /* xnorm <- ||x||  */
   SNESCheckFunctionNorm(snes,fnorm);
   ierr       = PetscObjectSAWsTakeAccess((PetscObject)snes);CHKERRQ(ierr);
   snes->norm = fnorm;
@@ -425,7 +417,7 @@ PetscErrorCode SNESSolve_VINEWTONRSLS(SNES snes)
 
 
     /* Create inactive set submatrix */
-    ierr = MatGetSubMatrix(snes->jacobian,vi->IS_inact,vi->IS_inact,MAT_INITIAL_MATRIX,&jac_inact_inact);CHKERRQ(ierr);
+    ierr = MatCreateSubMatrix(snes->jacobian,vi->IS_inact,vi->IS_inact,MAT_INITIAL_MATRIX,&jac_inact_inact);CHKERRQ(ierr);
 
     if (0) {                    /* Dead code (temporary developer hack) */
       IS keptrows;
@@ -433,8 +425,9 @@ PetscErrorCode SNESSolve_VINEWTONRSLS(SNES snes)
       if (keptrows) {
         PetscInt       cnt,*nrows,k;
         const PetscInt *krows,*inact;
-        PetscInt       rstart=jac_inact_inact->rmap->rstart;
+        PetscInt       rstart;
 
+        ierr = MatGetOwnershipRange(jac_inact_inact,&rstart,NULL);CHKERRQ(ierr);
         ierr = MatDestroy(&jac_inact_inact);CHKERRQ(ierr);
         ierr = ISDestroy(&IS_act);CHKERRQ(ierr);
 
@@ -450,7 +443,7 @@ PetscErrorCode SNESSolve_VINEWTONRSLS(SNES snes)
 
         ierr = ISCreateGeneral(PetscObjectComm((PetscObject)snes),cnt,nrows,PETSC_OWN_POINTER,&vi->IS_inact);CHKERRQ(ierr);
         ierr = ISComplement(vi->IS_inact,F->map->rstart,F->map->rend,&IS_act);CHKERRQ(ierr);
-        ierr = MatGetSubMatrix(snes->jacobian,vi->IS_inact,vi->IS_inact,MAT_INITIAL_MATRIX,&jac_inact_inact);CHKERRQ(ierr);
+        ierr = MatCreateSubMatrix(snes->jacobian,vi->IS_inact,vi->IS_inact,MAT_INITIAL_MATRIX,&jac_inact_inact);CHKERRQ(ierr);
       }
     }
     ierr = DMSetVI(snes->dm,vi->IS_inact);CHKERRQ(ierr);
@@ -490,12 +483,13 @@ PetscErrorCode SNESSolve_VINEWTONRSLS(SNES snes)
     /* Active set direction = 0 */
     ierr = VecSet(Y_act,0);CHKERRQ(ierr);
     if (snes->jacobian != snes->jacobian_pre) {
-      ierr = MatGetSubMatrix(snes->jacobian_pre,vi->IS_inact,vi->IS_inact,MAT_INITIAL_MATRIX,&prejac_inact_inact);CHKERRQ(ierr);
+      ierr = MatCreateSubMatrix(snes->jacobian_pre,vi->IS_inact,vi->IS_inact,MAT_INITIAL_MATRIX,&prejac_inact_inact);CHKERRQ(ierr);
     } else prejac_inact_inact = jac_inact_inact;
 
     ierr = ISEqual(vi->IS_inact_prev,vi->IS_inact,&isequal);CHKERRQ(ierr);
     if (!isequal) {
       ierr = SNESVIResetPCandKSP(snes,jac_inact_inact,prejac_inact_inact);CHKERRQ(ierr);
+      ierr = PCFieldSplitRestrictIS(pc,vi->IS_inact);CHKERRQ(ierr);
     }
 
     /*      ierr = ISView(vi->IS_inact,0);CHKERRQ(ierr); */
@@ -536,15 +530,6 @@ PetscErrorCode SNESSolve_VINEWTONRSLS(SNES snes)
     }
 
     ierr = KSPSolve(snes->ksp,F_inact,Y_inact);CHKERRQ(ierr);
-    ierr = KSPGetConvergedReason(snes->ksp,&kspreason);CHKERRQ(ierr);
-    if (kspreason < 0) {
-      if (++snes->numLinearSolveFailures >= snes->maxLinearSolveFailures) {
-        ierr         = PetscInfo2(snes,"iter=%D, number linear solve failures %D greater than current SNES allowed, stopping solve\n",snes->iter,snes->numLinearSolveFailures);CHKERRQ(ierr);
-        snes->reason = SNES_DIVERGED_LINEAR_SOLVE;
-        break;
-      }
-     }
-
     ierr = VecScatterBegin(scat_act,Y_act,Y,INSERT_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
     ierr = VecScatterEnd(scat_act,Y_act,Y,INSERT_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
     ierr = VecScatterBegin(scat_inact,Y_inact,Y,INSERT_VALUES,SCATTER_REVERSE);CHKERRQ(ierr);
@@ -565,6 +550,16 @@ PetscErrorCode SNESSolve_VINEWTONRSLS(SNES snes)
     if (snes->jacobian != snes->jacobian_pre) {
       ierr = MatDestroy(&prejac_inact_inact);CHKERRQ(ierr);
     }
+
+    ierr = KSPGetConvergedReason(snes->ksp,&kspreason);CHKERRQ(ierr);
+    if (kspreason < 0) {
+      if (++snes->numLinearSolveFailures >= snes->maxLinearSolveFailures) {
+        ierr         = PetscInfo2(snes,"iter=%D, number linear solve failures %D greater than current SNES allowed, stopping solve\n",snes->iter,snes->numLinearSolveFailures);CHKERRQ(ierr);
+        snes->reason = SNES_DIVERGED_LINEAR_SOLVE;
+        break;
+      }
+    }
+
     ierr              = KSPGetIterationNumber(snes->ksp,&lits);CHKERRQ(ierr);
     snes->linear_its += lits;
     ierr              = PetscInfo2(snes,"iter=%D, linear solve iterations=%D\n",snes->iter,lits);CHKERRQ(ierr);
@@ -627,8 +622,6 @@ PetscErrorCode SNESSolve_VINEWTONRSLS(SNES snes)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESVISetRedundancyCheck"
 PetscErrorCode SNESVISetRedundancyCheck(SNES snes,PetscErrorCode (*func)(SNES,IS,IS*,void*),void *ctx)
 {
   SNES_VINEWTONRSLS *vi = (SNES_VINEWTONRSLS*)snes->data;
@@ -645,8 +638,6 @@ PetscErrorCode SNESVISetRedundancyCheck(SNES snes,PetscErrorCode (*func)(SNES,IS
 #include <mex.h>
 typedef struct {char *funcname; mxArray *ctx;} SNESMatlabContext;
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESVIRedundancyCheck_Matlab"
 PetscErrorCode SNESVIRedundancyCheck_Matlab(SNES snes,IS is_act,IS *is_redact,void *ctx)
 {
   PetscErrorCode    ierr;
@@ -684,8 +675,6 @@ PetscErrorCode SNESVIRedundancyCheck_Matlab(SNES snes,IS is_act,IS *is_redact,vo
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESVISetRedundancyCheckMatlab"
 PetscErrorCode SNESVISetRedundancyCheckMatlab(SNES snes,const char *func,mxArray *ctx)
 {
   PetscErrorCode    ierr;
@@ -717,8 +706,6 @@ PetscErrorCode SNESVISetRedundancyCheckMatlab(SNES snes,const char *func,mxArray
    SNESSetUp(), since these actions will automatically occur during
    the call to SNESSolve().
  */
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetUp_VINEWTONRSLS"
 PetscErrorCode SNESSetUp_VINEWTONRSLS(SNES snes)
 {
   PetscErrorCode    ierr;
@@ -747,8 +734,6 @@ PetscErrorCode SNESSetUp_VINEWTONRSLS(SNES snes)
   PetscFunctionReturn(0);
 }
 /* -------------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "SNESReset_VINEWTONRSLS"
 PetscErrorCode SNESReset_VINEWTONRSLS(SNES snes)
 {
   SNES_VINEWTONRSLS *vi = (SNES_VINEWTONRSLS*) snes->data;
@@ -765,20 +750,18 @@ PetscErrorCode SNESReset_VINEWTONRSLS(SNES snes)
       SNESVINEWTONRSLS - Reduced space active set solvers for variational inequalities based on Newton's method
 
    Options Database:
-+   -snes_type <vinewtonssls,vinewtonrsls> a semi-smooth solver, a reduced space active set method
++   -snes_type <vinewtonssls,vinewtonrsls> - a semi-smooth solver, a reduced space active set method
 -   -snes_vi_monitor - prints the number of active constraints at each iteration.
 
    Level: beginner
 
    References:
-  - T. S. Munson, and S. Benson. Flexible Complementarity Solvers for Large-Scale
+.  1. - T. S. Munson, and S. Benson. Flexible Complementarity Solvers for Large Scale
      Applications, Optimization Methods and Software, 21 (2006).
 
-.seealso:  SNESVISetVariableBounds(), SNESVISetComputeVariableBounds(), SNESCreate(), SNES, SNESSetType(), SNESVINEWTONSSLS, SNESNEWTONTR, SNESLineSearchSet(),SNESLineSearchSetPostCheck(), SNESLineSearchSetPreCheck()
+.seealso:  SNESVISetVariableBounds(), SNESVISetComputeVariableBounds(), SNESCreate(), SNES, SNESSetType(), SNESVINEWTONSSLS, SNESNEWTONTR, SNESLineSearchSetType(),SNESLineSearchSetPostCheck(), SNESLineSearchSetPreCheck()
 
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "SNESCreate_VINEWTONRSLS"
 PETSC_EXTERN PetscErrorCode SNESCreate_VINEWTONRSLS(SNES snes)
 {
   PetscErrorCode    ierr;
@@ -794,7 +777,9 @@ PETSC_EXTERN PetscErrorCode SNESCreate_VINEWTONRSLS(SNES snes)
   snes->ops->converged      = SNESConvergedDefault_VI;
 
   snes->usesksp = PETSC_TRUE;
-  snes->usespc  = PETSC_FALSE;
+  snes->usesnpc = PETSC_FALSE;
+
+  snes->alwayscomputesfinalresidual = PETSC_TRUE;
 
   ierr                = PetscNewLog(snes,&vi);CHKERRQ(ierr);
   snes->data          = (void*)vi;
