@@ -4,8 +4,6 @@
 
 static PetscBool SNESPackageInitialized = PETSC_FALSE;
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESFinalizePackage"
 /*@C
   SNESFinalizePackage - This function destroys everything in the Petsc interface to the SNES package. It is
   called from PetscFinalize().
@@ -28,8 +26,6 @@ PetscErrorCode  SNESFinalizePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESInitializePackage"
 /*@C
   SNESInitializePackage - This function initializes everything in the SNES package. It is called
   from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to SNESCreate()
@@ -43,8 +39,7 @@ PetscErrorCode  SNESFinalizePackage(void)
 PetscErrorCode  SNESInitializePackage(void)
 {
   char           logList[256];
-  char           *className;
-  PetscBool      opt;
+  PetscBool      opt,pkg,cls;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -54,42 +49,46 @@ PetscErrorCode  SNESInitializePackage(void)
   ierr = SNESMSInitializePackage();CHKERRQ(ierr);
   /* Register Classes */
   ierr = PetscClassIdRegister("SNES",&SNES_CLASSID);CHKERRQ(ierr);
-  ierr = PetscClassIdRegister("SNESLineSearch",&SNESLINESEARCH_CLASSID);CHKERRQ(ierr);
   ierr = PetscClassIdRegister("DMSNES",&DMSNES_CLASSID);CHKERRQ(ierr);
+  ierr = PetscClassIdRegister("SNESLineSearch",&SNESLINESEARCH_CLASSID);CHKERRQ(ierr);
   /* Register Constructors */
   ierr = SNESRegisterAll();CHKERRQ(ierr);
   ierr = SNESLineSearchRegisterAll();CHKERRQ(ierr);
   /* Register Events */
   ierr = PetscLogEventRegister("SNESSolve",            SNES_CLASSID,&SNES_Solve);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("SNESFunctionEval",     SNES_CLASSID,&SNES_FunctionEval);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("SNESObjectiveEval",    SNES_CLASSID,&SNES_ObjectiveEval);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("SNESNGSEval",          SNES_CLASSID,&SNES_NGSEval);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("SNESNGSFuncEval",      SNES_CLASSID,&SNES_NGSFuncEval);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("SNESJacobianEval",     SNES_CLASSID,&SNES_JacobianEval);CHKERRQ(ierr);
-  ierr = PetscLogEventRegister("SNESLineSearch",       SNESLINESEARCH_CLASSID,&SNESLineSearch_Apply);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("SNESNPCSolve",         SNES_CLASSID,&SNES_NPCSolve);CHKERRQ(ierr);
+  ierr = PetscLogEventRegister("SNESLineSearch",       SNESLINESEARCH_CLASSID,&SNESLINESEARCH_Apply);CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL, "-info_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "snes", &className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscInfoDeactivateClass(SNES_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("snes",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) {ierr = PetscInfoDeactivateClass(SNES_CLASSID);CHKERRQ(ierr);}
+    ierr = PetscStrInList("dm",logList,',',&cls);CHKERRQ(ierr);
+    if (pkg || cls) {ierr = PetscInfoDeactivateClass(DMSNES_CLASSID);CHKERRQ(ierr);}
+    ierr = PetscStrInList("sneslinesearch",logList,',',&cls);CHKERRQ(ierr);
+    if (pkg || cls) {ierr = PetscInfoDeactivateClass(SNESLINESEARCH_CLASSID);CHKERRQ(ierr);}
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(NULL, "-log_summary_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "snes", &className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscLogEventDeactivateClass(SNES_CLASSID);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("snes",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) {ierr = PetscLogEventExcludeClass(SNES_CLASSID);CHKERRQ(ierr);}
+    ierr = PetscStrInList("dm",logList,',',&cls);CHKERRQ(ierr);
+    if (pkg || cls) {ierr = PetscLogEventExcludeClass(DMSNES_CLASSID);CHKERRQ(ierr);}
+    ierr = PetscStrInList("sneslinesearch",logList,',',&cls);CHKERRQ(ierr);
+    if (pkg || cls) {ierr = PetscLogEventExcludeClass(SNESLINESEARCH_CLASSID);CHKERRQ(ierr);}
   }
+  /* Register package finalizer */
   ierr = PetscRegisterFinalize(SNESFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #if defined(PETSC_HAVE_DYNAMIC_LIBRARIES)
-#undef __FUNCT__
-#define __FUNCT__ "PetscDLLibraryRegister_petscsnes"
 /*
   PetscDLLibraryRegister - This function is called when the dynamic library it is in is opened.
 
