@@ -1,6 +1,10 @@
 
 #include <petsc/private/snesimpl.h>
 
+PETSC_INTERN PetscErrorCode SNESDiffParameterCreate_More(SNES,Vec,void**);
+PETSC_INTERN PetscErrorCode SNESDiffParameterCompute_More(SNES,void*,Vec,Vec,PetscReal*,PetscReal*);
+PETSC_INTERN PetscErrorCode SNESDiffParameterDestroy_More(void*);
+
 /* Data used by Jorge's diff parameter computation method */
 typedef struct {
   Vec      *workv;           /* work vectors */
@@ -14,13 +18,12 @@ typedef struct {
 } DIFFPAR_MORE;
 
 
-extern PetscErrorCode SNESNoise_dnest_(PetscInt*,PetscScalar*,PetscScalar*,PetscScalar*,PetscScalar*,PetscScalar*,PetscInt*,PetscScalar*);
+PETSC_INTERN PetscErrorCode SNESDefaultMatrixFreeSetParameters2(Mat,double,double,double);
+PETSC_INTERN PetscErrorCode SNESUnSetMatrixFreeParameter(SNES snes);
+PETSC_INTERN PetscErrorCode SNESNoise_dnest_(PetscInt*,PetscScalar*,PetscScalar*,PetscScalar*,PetscScalar*,PetscScalar*,PetscInt*,PetscScalar*);
+
 static PetscErrorCode JacMatMultCompare(SNES,Vec,Vec,double);
-extern PetscErrorCode SNESDefaultMatrixFreeSetParameters2(Mat,double,double,double);
-extern PetscErrorCode SNESUnSetMatrixFreeParameter(SNES snes);
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESDiffParameterCreate_More"
 PetscErrorCode SNESDiffParameterCreate_More(SNES snes,Vec x,void **outneP)
 {
   DIFFPAR_MORE   *neP;
@@ -51,7 +54,7 @@ PetscErrorCode SNESDiffParameterCreate_More(SNES snes,Vec x,void **outneP)
   ierr = PetscRandomDestroy(&rctx);CHKERRQ(ierr);
 
   /* Open output file */
-  ierr = PetscOptionsGetString(((PetscObject)snes)->prefix,"-snes_mf_noise_file",noise_file,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(((PetscObject)snes)->options,((PetscObject)snes)->prefix,"-snes_mf_noise_file",noise_file,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
   if (flg) neP->fp = fopen(noise_file,"w");
   else     neP->fp = fopen("noise.out","w");
   if (!neP->fp) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Cannot open file");
@@ -61,8 +64,6 @@ PetscErrorCode SNESDiffParameterCreate_More(SNES snes,Vec x,void **outneP)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESDiffParameterDestroy_More"
 PetscErrorCode SNESDiffParameterDestroy_More(void *nePv)
 {
   DIFFPAR_MORE   *neP = (DIFFPAR_MORE*)nePv;
@@ -78,8 +79,6 @@ PetscErrorCode SNESDiffParameterDestroy_More(void *nePv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESDiffParameterCompute_More"
 PetscErrorCode SNESDiffParameterCompute_More(SNES snes,void *nePv,Vec x,Vec p,double *fnoise,double *hopt)
 {
   DIFFPAR_MORE   *neP = (DIFFPAR_MORE*)nePv;
@@ -134,8 +133,8 @@ PetscErrorCode SNESDiffParameterCompute_More(SNES snes,void *nePv,Vec x,Vec p,do
     /* Construct the difference table */
     for (i=0; i<nf; i++) tab[i][0] = fval[i];
 
-    for (j=0; j<6; j++) {
-      for (i=0; i<nf-j; i++) {
+    for (j=0; j<nf-1; j++) {
+      for (i=0; i<nf-j-1; i++) {
         tab[i][j+1] = tab[i+1][j] - tab[i][j];
       }
     }
@@ -212,15 +211,13 @@ theend:
   fcount = neP->function_count - fcount;
   ierr   = PetscInfo5(snes,"fct_now = %D, fct_cum = %D, rerrf=%g, sqrt(noise)=%g, h_more=%g\n",fcount,neP->function_count,(double)rerrf,(double)PetscSqrtReal(*fnoise),(double)*hopt);CHKERRQ(ierr);
 
-  ierr = PetscOptionsGetBool(NULL,"-noise_test",&noise_test,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-noise_test",&noise_test,NULL);CHKERRQ(ierr);
   if (noise_test) {
     ierr = JacMatMultCompare(snes,x,p,*hopt);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "JacMatMultCompare"
 PetscErrorCode JacMatMultCompare(SNES snes,Vec x,Vec p,double hopt)
 {
   Vec            yy1, yy2; /* work vectors */
@@ -254,11 +251,12 @@ PetscErrorCode JacMatMultCompare(SNES snes,Vec x,Vec p,double hopt)
   ierr = VecNorm(yy1,NORM_2,&yy1n);CHKERRQ(ierr);
 
   /* View product vector if desired */
-  ierr = PetscOptionsGetBool(NULL,"-print_vecs",&printv,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-print_vecs",&printv,NULL);CHKERRQ(ierr);
   if (printv) {
     ierr = PetscViewerASCIIOpen(comm,"y1.out",&view2);CHKERRQ(ierr);
-    ierr = PetscViewerSetFormat(view2,PETSC_VIEWER_ASCII_COMMON);CHKERRQ(ierr);
+    ierr = PetscViewerPushFormat(view2,PETSC_VIEWER_ASCII_COMMON);CHKERRQ(ierr);
     ierr = VecView(yy1,view2);CHKERRQ(ierr);
+    ierr = PetscViewerPopFormat(view2);CHKERRQ(ierr);
     ierr = PetscViewerDestroy(&view2);CHKERRQ(ierr);
   }
 
@@ -277,8 +275,9 @@ PetscErrorCode JacMatMultCompare(SNES snes,Vec x,Vec p,double hopt)
     if (printv) {
       sprintf(filename,"y2.%d.out",(int)i);
       ierr = PetscViewerASCIIOpen(comm,filename,&view2);CHKERRQ(ierr);
-      ierr = PetscViewerSetFormat(view2,PETSC_VIEWER_ASCII_COMMON);CHKERRQ(ierr);
+      ierr = PetscViewerPushFormat(view2,PETSC_VIEWER_ASCII_COMMON);CHKERRQ(ierr);
       ierr = VecView(yy2,view2);CHKERRQ(ierr);
+      ierr = PetscViewerPopFormat(view2);CHKERRQ(ierr);
       ierr = PetscViewerDestroy(&view2);CHKERRQ(ierr);
     }
 
