@@ -1,8 +1,7 @@
 
 #include <petsc/private/snesimpl.h>    /*I  "petscsnes.h"  I*/
+#include <petscdm.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESComputeJacobianDefault"
 /*@C
    SNESComputeJacobianDefault - Computes the Jacobian using finite differences.
 
@@ -54,7 +53,9 @@ PetscErrorCode  SNESComputeJacobianDefault(SNES snes,Vec x1,Mat J,Mat B,void *ct
   const PetscInt    *ranges;
 
   PetscFunctionBegin;
-  ierr     = PetscOptionsGetReal(((PetscObject)snes)->prefix,"-snes_test_err",&epsilon,0);CHKERRQ(ierr);
+  /* Since this Jacobian will possibly have "extra" nonzero locations just turn off errors for these locations */
+  ierr = MatSetOption(B,MAT_NEW_NONZERO_ALLOCATION_ERR, PETSC_FALSE);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(((PetscObject)snes)->options,((PetscObject)snes)->prefix,"-snes_test_err",&epsilon,0);CHKERRQ(ierr);
 
   ierr = PetscObjectGetComm((PetscObject)x1,&comm);CHKERRQ(ierr);
   ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
@@ -63,12 +64,17 @@ PetscErrorCode  SNESComputeJacobianDefault(SNES snes,Vec x1,Mat J,Mat B,void *ct
     ierr = MatZeroEntries(B);CHKERRQ(ierr);
   }
   if (!snes->nvwork) {
-    snes->nvwork = 3;
-
-    ierr = VecDuplicateVecs(x1,snes->nvwork,&snes->vwork);CHKERRQ(ierr);
-    ierr = PetscLogObjectParents(snes,snes->nvwork,snes->vwork);CHKERRQ(ierr);
+    if (snes->dm) {
+      ierr = DMGetGlobalVector(snes->dm,&j1a);CHKERRQ(ierr);
+      ierr = DMGetGlobalVector(snes->dm,&j2a);CHKERRQ(ierr);
+      ierr = DMGetGlobalVector(snes->dm,&x2);CHKERRQ(ierr);
+    } else {
+      snes->nvwork = 3;
+      ierr = VecDuplicateVecs(x1,snes->nvwork,&snes->vwork);CHKERRQ(ierr);
+      ierr = PetscLogObjectParents(snes,snes->nvwork,snes->vwork);CHKERRQ(ierr);
+      j1a = snes->vwork[0]; j2a = snes->vwork[1]; x2 = snes->vwork[2];
+    }
   }
-  j1a = snes->vwork[0]; j2a = snes->vwork[1]; x2 = snes->vwork[2];
 
   ierr = VecGetSize(x1,&N);CHKERRQ(ierr);
   ierr = VecGetOwnershipRange(x1,&start,&end);CHKERRQ(ierr);
@@ -123,6 +129,11 @@ PetscErrorCode  SNESComputeJacobianDefault(SNES snes,Vec x1,Mat J,Mat B,void *ct
     }
     ierr = VecRestoreArray(j2a,&y);CHKERRQ(ierr);
   }
+  if (snes->dm) {
+    ierr = DMRestoreGlobalVector(snes->dm,&j1a);CHKERRQ(ierr);
+    ierr = DMRestoreGlobalVector(snes->dm,&j2a);CHKERRQ(ierr);
+    ierr = DMRestoreGlobalVector(snes->dm,&x2);CHKERRQ(ierr);
+  }
   ierr = MatAssemblyBegin(B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   ierr = MatAssemblyEnd(B,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
   if (B != J) {
