@@ -3,7 +3,6 @@
 
 PETSC_EXTERN PetscErrorCode SNESCreate_NEWTONLS(SNES);
 PETSC_EXTERN PetscErrorCode SNESCreate_NEWTONTR(SNES);
-PETSC_EXTERN PetscErrorCode SNESCreate_Test(SNES);
 PETSC_EXTERN PetscErrorCode SNESCreate_NRichardson(SNES);
 PETSC_EXTERN PetscErrorCode SNESCreate_KSPONLY(SNES);
 PETSC_EXTERN PetscErrorCode SNESCreate_VINEWTONRSLS(SNES);
@@ -20,7 +19,7 @@ PETSC_EXTERN PetscErrorCode SNESCreate_Anderson(SNES);
 PETSC_EXTERN PetscErrorCode SNESCreate_ASPIN(SNES);
 PETSC_EXTERN PetscErrorCode SNESCreate_Composite(SNES);
 
-const char *SNESConvergedReasons_Shifted[] = {" "," ","DIVERGED_LOCAL_MIN","DIVERGED_INNER","DIVERGED_LINE_SEARCH","DIVERGED_MAX_IT",
+const char *SNESConvergedReasons_Shifted[] = {" ","DIVERGED_DTOL","DIVERGED_LOCAL_MIN","DIVERGED_INNER","DIVERGED_LINE_SEARCH","DIVERGED_MAX_IT",
                                               "DIVERGED_FNORM_NAN","DIVERGED_LINEAR_SOLVE","DIVERGED_FUNCTION_COUNT","DIVERGED_FUNCTION_DOMAIN",
                                               "CONVERGED_ITERATING"," ","CONVERGED_FNORM_ABS","CONVERGED_FNORM_RELATIVE",
                                               "CONVERGED_SNORM_RELATIVE","CONVERGED_ITS"," ","CONVERGED_TR_DELTA","SNESConvergedReason","",0};
@@ -32,15 +31,6 @@ const char *const *const SNESNormSchedules = SNESNormSchedules_Shifted + 1;
 const char *SNESFunctionTypes_Shifted[]    = {"DEFAULT","UNPRECONDITIONED","PRECONDITIONED","SNESFunctionType","SNES_FUNCTION_",0};
 const char *const *const SNESFunctionTypes = SNESFunctionTypes_Shifted + 1;
 
-/*
-      This is used by SNESSetType() to make sure that at least one
-    SNESRegisterAll() is called. In general, if there is more than one
-    DLL then SNESRegisterAll() may be called several times.
-*/
-extern PetscBool SNESRegisterAllCalled;
-
-#undef __FUNCT__
-#define __FUNCT__ "SNESRegisterAll"
 /*@C
    SNESRegisterAll - Registers all of the nonlinear solver methods in the SNES package.
 
@@ -62,7 +52,6 @@ PetscErrorCode  SNESRegisterAll(void)
 
   ierr = SNESRegister(SNESNEWTONLS,     SNESCreate_NEWTONLS);CHKERRQ(ierr);
   ierr = SNESRegister(SNESNEWTONTR,     SNESCreate_NEWTONTR);CHKERRQ(ierr);
-  ierr = SNESRegister(SNESTEST,         SNESCreate_Test);CHKERRQ(ierr);
   ierr = SNESRegister(SNESNRICHARDSON,  SNESCreate_NRichardson);CHKERRQ(ierr);
   ierr = SNESRegister(SNESKSPONLY,      SNESCreate_KSPONLY);CHKERRQ(ierr);
   ierr = SNESRegister(SNESVINEWTONRSLS, SNESCreate_VINEWTONRSLS);CHKERRQ(ierr);
