@@ -3,8 +3,6 @@
 #include <petscdm.h>
 #include <petscblaslapack.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMonitorSolution"
 /*@C
    SNESMonitorSolution - Monitors progress of the SNES solvers by calling
    VecView() for the approximate solution at each iteration.
@@ -17,6 +15,9 @@
 .  fgnorm - 2-norm of residual
 -  dummy -  a viewer
 
+   Options Database Keys:
+.  -snes_monitor_solution [ascii binary draw][:filename][:viewer format] - plots solution at each iteration
+
    Level: intermediate
 
 .keywords: SNES, nonlinear, vector, monitor, view
@@ -38,8 +39,6 @@ PetscErrorCode  SNESMonitorSolution(SNES snes,PetscInt its,PetscReal fgnorm,Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMonitorResidual"
 /*@C
    SNESMonitorResidual - Monitors progress of the SNES solvers by calling
    VecView() for the residual at each iteration.
@@ -52,6 +51,9 @@ PetscErrorCode  SNESMonitorSolution(SNES snes,PetscInt its,PetscReal fgnorm,Pets
 .  fgnorm - 2-norm of residual
 -  dummy -  a viewer
 
+   Options Database Keys:
+.  -snes_monitor_residual [ascii binary draw][:filename][:viewer format] - plots residual (not its norm) at each iteration
+
    Level: intermediate
 
 .keywords: SNES, nonlinear, vector, monitor, view
@@ -73,8 +75,6 @@ PetscErrorCode  SNESMonitorResidual(SNES snes,PetscInt its,PetscReal fgnorm,Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMonitorSolutionUpdate"
 /*@C
    SNESMonitorSolutionUpdate - Monitors progress of the SNES solvers by calling
    VecView() for the UPDATE to the solution at each iteration.
@@ -87,6 +87,9 @@ PetscErrorCode  SNESMonitorResidual(SNES snes,PetscInt its,PetscReal fgnorm,Pets
 .  fgnorm - 2-norm of residual
 -  dummy - a viewer
 
+   Options Database Keys:
+.  -snes_monitor_solution_update [ascii binary draw][:filename][:viewer format] - plots update to solution at each iteration
+
    Level: intermediate
 
 .keywords: SNES, nonlinear, vector, monitor, view
@@ -108,8 +111,6 @@ PetscErrorCode  SNESMonitorSolutionUpdate(SNES snes,PetscInt its,PetscReal fgnor
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorSNES"
 /*@C
    KSPMonitorSNES - Print the residual norm of the nonlinear function at each iteration of the linear iterative solver.
 
@@ -158,8 +159,6 @@ PetscErrorCode  KSPMonitorSNES(KSP ksp,PetscInt n,PetscReal rnorm,void *dummy)
 
 #include <petscdraw.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorSNESLGResidualNormCreate"
 /*@C
    KSPMonitorSNESLGResidualNormCreate - Creates a line graph context for use with
    KSP to monitor convergence of preconditioned residual norms.
@@ -212,8 +211,6 @@ PetscErrorCode  KSPMonitorSNESLGResidualNormCreate(MPI_Comm comm,const char host
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorSNESLGResidualNorm"
 PetscErrorCode  KSPMonitorSNESLGResidualNorm(KSP ksp,PetscInt n,PetscReal rnorm,PetscObject *objs)
 {
   SNES           snes = (SNES) objs[0];
@@ -246,8 +243,6 @@ PetscErrorCode  KSPMonitorSNESLGResidualNorm(KSP ksp,PetscInt n,PetscReal rnorm,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "KSPMonitorSNESLGResidualNormDestroy"
 /*@
    KSPMonitorSNESLGResidualNormDestroy - Destroys a line graph context that was created
    with KSPMonitorSNESLGResidualNormCreate().
@@ -274,8 +269,6 @@ PetscErrorCode  KSPMonitorSNESLGResidualNormDestroy(PetscObject **objs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMonitorDefault"
 /*@C
    SNESMonitorDefault - Monitors progress of the SNES solvers (default).
 
@@ -311,8 +304,50 @@ PetscErrorCode  SNESMonitorDefault(SNES snes,PetscInt its,PetscReal fgnorm,Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMonitorJacUpdateSpectrum"
+/*@C
+   SNESMonitorScaling - Monitors the largest value in each row of the Jacobian.
+
+   Collective on SNES
+
+   Input Parameters:
++  snes - the SNES context
+.  its - iteration number
+.  fgnorm - 2-norm of residual
+-  vf - viewer and format structure
+
+   Notes:
+   This routine prints the largest value in each row of the Jacobian 
+
+   Level: intermediate
+
+.keywords: SNES, nonlinear, default, monitor, norm
+
+.seealso: SNESMonitorSet(), SNESMonitorSolution()
+@*/
+PetscErrorCode  SNESMonitorScaling(SNES snes,PetscInt its,PetscReal fgnorm,PetscViewerAndFormat *vf)
+{
+  PetscErrorCode ierr;
+  PetscViewer    viewer = vf->viewer;
+  KSP            ksp;
+  Mat            J;
+  Vec            v;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,4);
+  ierr = SNESGetKSP(snes,&ksp);CHKERRQ(ierr);
+  ierr = KSPGetOperators(ksp,&J,NULL);CHKERRQ(ierr);
+  ierr = MatCreateVecs(J,&v,NULL);CHKERRQ(ierr);
+  ierr = MatGetRowMaxAbs(J,v,NULL);CHKERRQ(ierr);
+  ierr = PetscViewerPushFormat(viewer,vf->format);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIAddTab(viewer,((PetscObject)snes)->tablevel);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"%3D SNES Jacobian maximum row entries \n");CHKERRQ(ierr);
+  ierr = VecView(v,viewer);CHKERRQ(ierr);
+  ierr = PetscViewerASCIISubtractTab(viewer,((PetscObject)snes)->tablevel);CHKERRQ(ierr);
+  ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr) ;
+  ierr = VecDestroy(&v);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode SNESMonitorJacUpdateSpectrum(SNES snes,PetscInt it,PetscReal fnorm,PetscViewerAndFormat *vf)
 {
 #if defined(PETSC_MISSING_LAPACK_GEEV)
@@ -375,8 +410,6 @@ PetscErrorCode SNESMonitorJacUpdateSpectrum(SNES snes,PetscInt it,PetscReal fnor
 
 PETSC_INTERN PetscErrorCode  SNESMonitorRange_Private(SNES,PetscInt,PetscReal*);
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMonitorRange_Private"
 PetscErrorCode  SNESMonitorRange_Private(SNES snes,PetscInt it,PetscReal *per)
 {
   PetscErrorCode ierr;
@@ -401,8 +434,6 @@ PetscErrorCode  SNESMonitorRange_Private(SNES snes,PetscInt it,PetscReal *per)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMonitorRange"
 /*@C
    SNESMonitorRange - Prints the percentage of residual elements that are more then 10 percent of the maximum value.
 
@@ -446,8 +477,6 @@ PetscErrorCode  SNESMonitorRange(SNES snes,PetscInt it,PetscReal rnorm,PetscView
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMonitorRatio"
 /*@C
    SNESMonitorRatio - Monitors progress of the SNES solvers by printing the ratio
    of residual norm at each iteration to the previous.
@@ -462,7 +491,8 @@ PetscErrorCode  SNESMonitorRange(SNES snes,PetscInt it,PetscReal rnorm,PetscView
 
    Level: intermediate
 
-   Notes: Insure that SNESMonitorRatio() is called when you set this monitor
+   Notes:
+    Insure that SNESMonitorRatio() is called when you set this monitor
 .keywords: SNES, nonlinear, monitor, norm
 
 .seealso: SNESMonitorSet(), SNESMonitorSolution(), SNESMonitorRatio()
@@ -489,8 +519,6 @@ PetscErrorCode  SNESMonitorRatio(SNES snes,PetscInt its,PetscReal fgnorm,PetscVi
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMonitorRatioSetUp"
 /*@C
    SNESMonitorRatioSetUp - Insures the SNES object is saving its history since this monitor needs access to it
 
@@ -520,14 +548,14 @@ PetscErrorCode  SNESMonitorRatioSetUp(SNES snes,PetscViewerAndFormat *vf)
 }
 
 /* ---------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "SNESMonitorDefaultShort"
 /*
      Default (short) SNES Monitor, same as SNESMonitorDefault() except
   it prints fewer digits of the residual as the residual gets smaller.
   This is because the later digits are meaningless and are often
   different on different machines; by using this routine different
   machines will usually generate the same output.
+
+  Deprecated: Intentionally has no manual page
 */
 PetscErrorCode  SNESMonitorDefaultShort(SNES snes,PetscInt its,PetscReal fgnorm,PetscViewerAndFormat *vf)
 {
@@ -550,8 +578,6 @@ PetscErrorCode  SNESMonitorDefaultShort(SNES snes,PetscInt its,PetscReal fgnorm,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESMonitorDefaultField"
 /*@C
   SNESMonitorDefaultField - Monitors progress of the SNES solvers, separated into fields.
 
@@ -569,7 +595,7 @@ PetscErrorCode  SNESMonitorDefaultShort(SNES snes,PetscInt its,PetscReal fgnorm,
   Level: intermediate
 
 .keywords: SNES, nonlinear, field, monitor, norm
-.seealso: SNESMonitorSet(), SNESMonitorSolution(), SNESMonitorDefault(), SNESMonitorDefaultShort()
+.seealso: SNESMonitorSet(), SNESMonitorSolution(), SNESMonitorDefault()
 @*/
 PetscErrorCode SNESMonitorDefaultField(SNES snes, PetscInt its, PetscReal fgnorm, PetscViewerAndFormat *vf)
 {
@@ -589,8 +615,8 @@ PetscErrorCode SNESMonitorDefaultField(SNES snes, PetscInt its, PetscReal fgnorm
     PetscSection s, gs;
     PetscInt     Nf, f;
 
-    ierr = DMGetDefaultSection(dm, &s);CHKERRQ(ierr);
-    ierr = DMGetDefaultGlobalSection(dm, &gs);CHKERRQ(ierr);
+    ierr = DMGetSection(dm, &s);CHKERRQ(ierr);
+    ierr = DMGetGlobalSection(dm, &gs);CHKERRQ(ierr);
     if (!s || !gs) {ierr = SNESMonitorDefault(snes, its, fgnorm, vf);CHKERRQ(ierr);}
     ierr = PetscSectionGetNumFields(s, &Nf);CHKERRQ(ierr);
     if (Nf > 256) SETERRQ1(PetscObjectComm((PetscObject) snes), PETSC_ERR_SUP, "Do not support %d fields > 256", Nf);
@@ -610,8 +636,6 @@ PetscErrorCode SNESMonitorDefaultField(SNES snes, PetscInt its, PetscReal fgnorm
   PetscFunctionReturn(0);
 }
 /* ---------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "SNESConvergedDefault"
 /*@C
    SNESConvergedDefault - Convergence test of the solvers for
    systems of nonlinear equations (default).
@@ -661,15 +685,16 @@ PetscErrorCode  SNESConvergedDefault(SNES snes,PetscInt it,PetscReal xnorm,Petsc
 
   if (!it) {
     /* set parameter for default relative tolerance convergence test */
-    snes->ttol = fnorm*snes->rtol;
+    snes->ttol   = fnorm*snes->rtol;
+    snes->rnorm0 = fnorm;
   }
   if (PetscIsInfOrNanReal(fnorm)) {
     ierr    = PetscInfo(snes,"Failed to converged, function norm is NaN\n");CHKERRQ(ierr);
     *reason = SNES_DIVERGED_FNORM_NAN;
-  } else if (fnorm < snes->abstol) {
+  } else if (fnorm < snes->abstol && (it || !snes->forceiteration)) {
     ierr    = PetscInfo2(snes,"Converged due to function norm %14.12e < %14.12e\n",(double)fnorm,(double)snes->abstol);CHKERRQ(ierr);
     *reason = SNES_CONVERGED_FNORM_ABS;
-  } else if (snes->nfuncs >= snes->max_funcs) {
+  } else if (snes->nfuncs >= snes->max_funcs && snes->max_funcs >= 0) {
     ierr    = PetscInfo2(snes,"Exceeded maximum number of function evaluations: %D > %D\n",snes->nfuncs,snes->max_funcs);CHKERRQ(ierr);
     *reason = SNES_DIVERGED_FUNCTION_COUNT;
   }
@@ -681,13 +706,15 @@ PetscErrorCode  SNESConvergedDefault(SNES snes,PetscInt it,PetscReal xnorm,Petsc
     } else if (snorm < snes->stol*xnorm) {
       ierr    = PetscInfo3(snes,"Converged due to small update length: %14.12e < %14.12e * %14.12e\n",(double)snorm,(double)snes->stol,(double)xnorm);CHKERRQ(ierr);
       *reason = SNES_CONVERGED_SNORM_RELATIVE;
+    } else if (snes->divtol > 0 && (fnorm > snes->divtol*snes->rnorm0)) {
+      ierr    = PetscInfo3(snes,"Diverged due to increase in function norm: %14.12e > %14.12e * %14.12e\n",(double)fnorm,(double)snes->divtol,(double)snes->rnorm0);CHKERRQ(ierr);
+      *reason = SNES_DIVERGED_DTOL;
     }
+    
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESConvergedSkip"
 /*@C
    SNESConvergedSkip - Convergence test for SNES that NEVER returns as
    converged, UNLESS the maximum number of iteration have been reached.
@@ -733,8 +760,6 @@ PetscErrorCode  SNESConvergedSkip(SNES snes,PetscInt it,PetscReal xnorm,PetscRea
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESSetWorkVecs"
 /*@C
   SNESSetWorkVecs - Gets a number of work vectors.
 
@@ -742,9 +767,7 @@ PetscErrorCode  SNESConvergedSkip(SNES snes,PetscInt it,PetscReal xnorm,PetscRea
 . snes  - the SNES context
 . nw - number of work vectors to allocate
 
-   Level: developer
-
-   Developers Note: This is PETSC_EXTERN because it may be used by user written plugin SNES implementations
+  Level: developer
 
 @*/
 PetscErrorCode SNESSetWorkVecs(SNES snes,PetscInt nw)
