@@ -1,12 +1,10 @@
-#include <petsc-private/linesearchimpl.h> /*I  "petscsnes.h"  I*/
-#include <petsc-private/snesimpl.h>
+#include <petsc/private/linesearchimpl.h> /*I  "petscsnes.h"  I*/
+#include <petsc/private/snesimpl.h>
 
 typedef struct {
   PetscReal alpha;        /* sufficient decrease parameter */
 } SNESLineSearch_BT;
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESLineSearchBTSetAlpha"
 /*@
    SNESLineSearchBTSetAlpha - Sets the descent parameter, alpha, in the BT linesearch variant.
 
@@ -20,18 +18,14 @@ typedef struct {
 @*/
 PetscErrorCode SNESLineSearchBTSetAlpha(SNESLineSearch linesearch, PetscReal alpha)
 {
-  SNESLineSearch_BT *bt;
+  SNESLineSearch_BT *bt = (SNESLineSearch_BT*)linesearch->data;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(linesearch,SNESLINESEARCH_CLASSID,1);
-  bt        = (SNESLineSearch_BT*)linesearch->data;
   bt->alpha = alpha;
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "SNESLineSearchBTGetAlpha"
 /*@
    SNESLineSearchBTGetAlpha - Gets the descent parameter, alpha, in the BT linesearch variant.
 
@@ -47,17 +41,14 @@ PetscErrorCode SNESLineSearchBTSetAlpha(SNESLineSearch linesearch, PetscReal alp
 @*/
 PetscErrorCode SNESLineSearchBTGetAlpha(SNESLineSearch linesearch, PetscReal *alpha)
 {
-  SNESLineSearch_BT *bt;
+  SNESLineSearch_BT *bt = (SNESLineSearch_BT*)linesearch->data;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(linesearch,SNESLINESEARCH_CLASSID,1);
-  bt     = (SNESLineSearch_BT*)linesearch->data;
   *alpha = bt->alpha;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESLineSearchApply_BT"
 static PetscErrorCode  SNESLineSearchApply_BT(SNESLineSearch linesearch)
 {
   PetscBool         changed_y,changed_w;
@@ -69,10 +60,9 @@ static PetscErrorCode  SNESLineSearchApply_BT(SNESLineSearch linesearch)
   PetscReal         t1,t2,a,b,d;
   PetscReal         f;
   PetscReal         g,gprev;
-  PetscBool         domainerror;
   PetscViewer       monitor;
   PetscInt          max_its,count;
-  SNESLineSearch_BT *bt;
+  SNESLineSearch_BT *bt = (SNESLineSearch_BT*)linesearch->data;
   Mat               jac;
   PetscErrorCode    (*objective)(SNES,Vec,PetscReal*,void*);
 
@@ -81,21 +71,17 @@ static PetscErrorCode  SNESLineSearchApply_BT(SNESLineSearch linesearch)
   ierr = SNESLineSearchGetNorms(linesearch, &xnorm, &fnorm, &ynorm);CHKERRQ(ierr);
   ierr = SNESLineSearchGetLambda(linesearch, &lambda);CHKERRQ(ierr);
   ierr = SNESLineSearchGetSNES(linesearch, &snes);CHKERRQ(ierr);
-  ierr = SNESLineSearchGetMonitor(linesearch, &monitor);CHKERRQ(ierr);
+  ierr = SNESLineSearchGetDefaultMonitor(linesearch, &monitor);CHKERRQ(ierr);
   ierr = SNESLineSearchGetTolerances(linesearch,&minlambda,&maxstep,NULL,NULL,NULL,&max_its);CHKERRQ(ierr);
   ierr = SNESGetTolerances(snes,NULL,NULL,&stol,NULL,NULL);CHKERRQ(ierr);
   ierr = SNESGetObjective(snes,&objective,NULL);CHKERRQ(ierr);
-  bt   = (SNESLineSearch_BT*)linesearch->data;
-
   alpha = bt->alpha;
 
   ierr = SNESGetJacobian(snes, &jac, NULL, NULL, NULL);CHKERRQ(ierr);
-
   if (!jac && !objective) SETERRQ(PetscObjectComm((PetscObject)linesearch), PETSC_ERR_USER, "SNESLineSearchBT requires a Jacobian matrix");
 
-  /* precheck */
   ierr = SNESLineSearchPreCheck(linesearch,X,Y,&changed_y);CHKERRQ(ierr);
-  ierr = SNESLineSearchSetSuccess(linesearch, PETSC_TRUE);CHKERRQ(ierr);
+  ierr = SNESLineSearchSetReason(linesearch, SNES_LINESEARCH_SUCCEEDED);CHKERRQ(ierr);
 
   ierr = VecNormBegin(Y, NORM_2, &ynorm);CHKERRQ(ierr);
   ierr = VecNormBegin(X, NORM_2, &xnorm);CHKERRQ(ierr);
@@ -110,7 +96,8 @@ static PetscErrorCode  SNESLineSearchApply_BT(SNESLineSearch linesearch)
     }
     ierr = VecCopy(X,W);CHKERRQ(ierr);
     ierr = VecCopy(F,G);CHKERRQ(ierr);
-    ierr = SNESLineSearchSetSuccess(linesearch, PETSC_FALSE);CHKERRQ(ierr);
+    ierr = SNESLineSearchSetNorms(linesearch,xnorm,fnorm,ynorm);CHKERRQ(ierr);
+    ierr = SNESLineSearchSetReason(linesearch, SNES_LINESEARCH_FAILED_REDUCT);CHKERRQ(ierr);
     PetscFunctionReturn(0);
   }
   if (ynorm > maxstep) {        /* Step too big, so scale back */
@@ -142,40 +129,45 @@ static PetscErrorCode  SNESLineSearchApply_BT(SNESLineSearch linesearch)
     if (initslope == 0.0) initslope = -1.0;
   }
 
-  ierr = VecWAXPY(W,-lambda,Y,X);CHKERRQ(ierr);
-  if (linesearch->ops->viproject) {
-    ierr = (*linesearch->ops->viproject)(snes, W);CHKERRQ(ierr);
-  }
-  if (snes->nfuncs >= snes->max_funcs) {
-    ierr         = PetscInfo(snes,"Exceeded maximum function evaluations, while checking full step length!\n");CHKERRQ(ierr);
-    snes->reason = SNES_DIVERGED_FUNCTION_COUNT;
-    ierr         = SNESLineSearchSetSuccess(linesearch, PETSC_FALSE);CHKERRQ(ierr);
-    PetscFunctionReturn(0);
-  }
-
-  if (objective) {
-    ierr = SNESComputeObjective(snes,W,&g);CHKERRQ(ierr);
-  } else {
-    ierr = SNESComputeFunction(snes,W,G);CHKERRQ(ierr);
-    ierr = SNESGetFunctionDomainError(snes, &domainerror);CHKERRQ(ierr);
-    if (domainerror) {
-      ierr = SNESLineSearchSetSuccess(linesearch, PETSC_FALSE);CHKERRQ(ierr);
+  while (PETSC_TRUE) {
+    ierr = VecWAXPY(W,-lambda,Y,X);CHKERRQ(ierr);
+    if (linesearch->ops->viproject) {
+      ierr = (*linesearch->ops->viproject)(snes, W);CHKERRQ(ierr);
+    }
+    if (snes->nfuncs >= snes->max_funcs && snes->max_funcs >= 0) {
+      ierr         = PetscInfo(snes,"Exceeded maximum function evaluations, while checking full step length!\n");CHKERRQ(ierr);
+      snes->reason = SNES_DIVERGED_FUNCTION_COUNT;
+      ierr         = SNESLineSearchSetReason(linesearch, SNES_LINESEARCH_FAILED_FUNCTION);CHKERRQ(ierr);
       PetscFunctionReturn(0);
     }
-    if (linesearch->ops->vinorm) {
-      gnorm = fnorm;
-      ierr  = (*linesearch->ops->vinorm)(snes, G, W, &gnorm);CHKERRQ(ierr);
+
+    if (objective) {
+      ierr = SNESComputeObjective(snes,W,&g);CHKERRQ(ierr);
     } else {
-      ierr = VecNorm(G,NORM_2,&gnorm);CHKERRQ(ierr);
+      ierr = (*linesearch->ops->snesfunc)(snes,W,G);CHKERRQ(ierr);
+      if (linesearch->ops->vinorm) {
+        gnorm = fnorm;
+        ierr  = (*linesearch->ops->vinorm)(snes, G, W, &gnorm);CHKERRQ(ierr);
+      } else {
+        ierr = VecNorm(G,NORM_2,&gnorm);CHKERRQ(ierr);
+      }
+      g = PetscSqr(gnorm);
     }
-    g = PetscSqr(gnorm);
+    ierr = SNESLineSearchMonitor(linesearch);CHKERRQ(ierr);
+
+    if (!PetscIsInfOrNanReal(g)) break;
+    if (monitor) {
+      ierr = PetscViewerASCIIAddTab(monitor,((PetscObject)linesearch)->tablevel);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(monitor,"    Line search: objective function at lambdas = %g is Inf or Nan, cutting lambda\n",(double)lambda);CHKERRQ(ierr);
+      ierr = PetscViewerASCIISubtractTab(monitor,((PetscObject)linesearch)->tablevel);CHKERRQ(ierr);
+    }
+    if (lambda <= minlambda) {
+      ierr = SNESLineSearchSetReason(linesearch, SNES_LINESEARCH_FAILED_REDUCT);CHKERRQ(ierr);
+      PetscFunctionReturn(0);
+    }
+    lambda = .5*lambda;
   }
 
-  if (PetscIsInfOrNanReal(g)) {
-    ierr = SNESLineSearchSetSuccess(linesearch, PETSC_FALSE);CHKERRQ(ierr);
-    ierr = PetscInfo(monitor,"Aborted due to Nan or Inf in function evaluation\n");CHKERRQ(ierr);
-    PetscFunctionReturn(0);
-  }
   if (!objective) {
     ierr = PetscInfo2(snes,"Initial fnorm %14.12e gnorm %14.12e\n", (double)fnorm, (double)gnorm);CHKERRQ(ierr);
   }
@@ -185,15 +177,20 @@ static PetscErrorCode  SNESLineSearchApply_BT(SNESLineSearch linesearch)
       if (!objective) {
         ierr = PetscViewerASCIIPrintf(monitor,"    Line search: Using full step: fnorm %14.12e gnorm %14.12e\n", (double)fnorm, (double)gnorm);CHKERRQ(ierr);
       } else {
-        ierr = PetscViewerASCIIPrintf(monitor,"    Line search: Using full step: obj0 %14.12e obj %14.12e\n", (double)f, (double)g);CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(monitor,"    Line search: Using full step: obj %14.12e obj %14.12e\n", (double)f, (double)g);CHKERRQ(ierr);
       }
       ierr = PetscViewerASCIISubtractTab(monitor,((PetscObject)linesearch)->tablevel);CHKERRQ(ierr);
     }
   } else {
     /* Since the full step didn't work and the step is tiny, quit */
     if (stol*xnorm > ynorm) {
-      ierr = SNESLineSearchSetSuccess(linesearch, PETSC_FALSE);CHKERRQ(ierr);
-      ierr = PetscInfo2(monitor,"Aborted due to ynorm < stol*xnorm (%14.12e < %14.12e) and inadequate full step.\n",ynorm,stol*xnorm);CHKERRQ(ierr);
+      ierr = SNESLineSearchSetNorms(linesearch,xnorm,fnorm,ynorm);CHKERRQ(ierr);
+      ierr = SNESLineSearchSetReason(linesearch, SNES_LINESEARCH_SUCCEEDED);CHKERRQ(ierr);
+      if (monitor) {
+        ierr = PetscViewerASCIIAddTab(monitor,((PetscObject)linesearch)->tablevel);CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(monitor,"    Line search: Ended due to ynorm < stol*xnorm (%14.12e < %14.12e).\n",(double)ynorm,(double)stol*xnorm);CHKERRQ(ierr);
+        ierr = PetscViewerASCIISubtractTab(monitor,((PetscObject)linesearch)->tablevel);CHKERRQ(ierr);
+      }
       PetscFunctionReturn(0);
     }
     /* Fit points with quadratic */
@@ -208,30 +205,27 @@ static PetscErrorCode  SNESLineSearchApply_BT(SNESLineSearch linesearch)
     if (linesearch->ops->viproject) {
       ierr = (*linesearch->ops->viproject)(snes, W);CHKERRQ(ierr);
     }
-    if (snes->nfuncs >= snes->max_funcs) {
+    if (snes->nfuncs >= snes->max_funcs && snes->max_funcs >= 0) {
       ierr         = PetscInfo1(snes,"Exceeded maximum function evaluations, while attempting quadratic backtracking! %D \n",snes->nfuncs);CHKERRQ(ierr);
       snes->reason = SNES_DIVERGED_FUNCTION_COUNT;
-      ierr         = SNESLineSearchSetSuccess(linesearch, PETSC_FALSE);CHKERRQ(ierr);
+      ierr         = SNESLineSearchSetReason(linesearch, SNES_LINESEARCH_FAILED_FUNCTION);CHKERRQ(ierr);
       PetscFunctionReturn(0);
     }
     if (objective) {
       ierr = SNESComputeObjective(snes,W,&g);CHKERRQ(ierr);
     } else {
-      ierr = SNESComputeFunction(snes,W,G);CHKERRQ(ierr);
-      ierr = SNESGetFunctionDomainError(snes, &domainerror);CHKERRQ(ierr);
-      if (domainerror) PetscFunctionReturn(0);
-
+      ierr = (*linesearch->ops->snesfunc)(snes,W,G);CHKERRQ(ierr);
       if (linesearch->ops->vinorm) {
         gnorm = fnorm;
         ierr = (*linesearch->ops->vinorm)(snes, G, W, &gnorm);CHKERRQ(ierr);
       } else {
         ierr = VecNorm(G,NORM_2,&gnorm);CHKERRQ(ierr);
-        g    = gnorm*gnorm;
       }
+      g = PetscSqr(gnorm);
     }
     if (PetscIsInfOrNanReal(g)) {
-      ierr = SNESLineSearchSetSuccess(linesearch, PETSC_FALSE);CHKERRQ(ierr);
-      ierr = PetscInfo(monitor,"Aborted due to Nan or Inf in function evaluation\n");CHKERRQ(ierr);
+      ierr = SNESLineSearchSetReason(linesearch, SNES_LINESEARCH_FAILED_NANORINF);CHKERRQ(ierr);
+      ierr = PetscInfo(snes,"Aborted due to Nan or Inf in function evaluation\n");CHKERRQ(ierr);
       PetscFunctionReturn(0);
     }
     if (monitor) {
@@ -257,17 +251,15 @@ static PetscErrorCode  SNESLineSearchApply_BT(SNESLineSearch linesearch)
             ierr = PetscViewerASCIIAddTab(monitor,((PetscObject)linesearch)->tablevel);CHKERRQ(ierr);
             ierr = PetscViewerASCIIPrintf(monitor,"    Line search: unable to find good step length! After %D tries \n",count);CHKERRQ(ierr);
             if (!objective) {
-              ierr = PetscViewerASCIIPrintf(monitor,
-                                            "    Line search: fnorm=%18.16e, gnorm=%18.16e, ynorm=%18.16e, minlambda=%18.16e, lambda=%18.16e, initial slope=%18.16e\n",
-                                            (double)fnorm, (double)gnorm, (double)ynorm, (double)minlambda, (double)lambda, (double)initslope);CHKERRQ(ierr);
+              ierr = PetscViewerASCIIPrintf(monitor,"    Line search: fnorm=%18.16e, gnorm=%18.16e, ynorm=%18.16e, minlambda=%18.16e, lambda=%18.16e, initial slope=%18.16e\n",
+                                                         (double)fnorm, (double)gnorm, (double)ynorm, (double)minlambda, (double)lambda, (double)initslope);CHKERRQ(ierr);
             } else {
-              ierr = PetscViewerASCIIPrintf(monitor,
-                                            "    Line search: obj(0)=%18.16e, obj=%18.16e, ynorm=%18.16e, minlambda=%18.16e, lambda=%18.16e, initial slope=%18.16e\n",
-                                            (double)f, (double)g, (double)ynorm, (double)minlambda, (double)lambda, (double)initslope);CHKERRQ(ierr);
+              ierr = PetscViewerASCIIPrintf(monitor,"    Line search: obj(0)=%18.16e, obj=%18.16e, ynorm=%18.16e, minlambda=%18.16e, lambda=%18.16e, initial slope=%18.16e\n",
+                                                         (double)f, (double)g, (double)ynorm, (double)minlambda, (double)lambda, (double)initslope);CHKERRQ(ierr);
             }
             ierr = PetscViewerASCIISubtractTab(monitor,((PetscObject)linesearch)->tablevel);CHKERRQ(ierr);
           }
-          ierr = SNESLineSearchSetSuccess(linesearch, PETSC_FALSE);CHKERRQ(ierr);
+          ierr = SNESLineSearchSetReason(linesearch, SNES_LINESEARCH_FAILED_REDUCT);CHKERRQ(ierr);
           PetscFunctionReturn(0);
         }
         if (linesearch->order == SNES_LINESEARCH_ORDER_CUBIC) {
@@ -292,36 +284,31 @@ static PetscErrorCode  SNESLineSearchApply_BT(SNESLineSearch linesearch)
         if (linesearch->ops->viproject) {
           ierr = (*linesearch->ops->viproject)(snes,W);CHKERRQ(ierr);
         }
-        if (snes->nfuncs >= snes->max_funcs) {
+        if (snes->nfuncs >= snes->max_funcs && snes->max_funcs >= 0) {
           ierr = PetscInfo1(snes,"Exceeded maximum function evaluations, while looking for good step length! %D \n",count);CHKERRQ(ierr);
           if (!objective) {
             ierr = PetscInfo5(snes,"fnorm=%18.16e, gnorm=%18.16e, ynorm=%18.16e, lambda=%18.16e, initial slope=%18.16e\n",
                               (double)fnorm,(double)gnorm,(double)ynorm,(double)lambda,(double)initslope);CHKERRQ(ierr);
           }
-          ierr         = SNESLineSearchSetSuccess(linesearch, PETSC_FALSE);CHKERRQ(ierr);
+          ierr         = SNESLineSearchSetReason(linesearch, SNES_LINESEARCH_FAILED_FUNCTION);CHKERRQ(ierr);
           snes->reason = SNES_DIVERGED_FUNCTION_COUNT;
           PetscFunctionReturn(0);
         }
         if (objective) {
           ierr = SNESComputeObjective(snes,W,&g);CHKERRQ(ierr);
         } else {
-          ierr = SNESComputeFunction(snes,W,G);CHKERRQ(ierr);
-          ierr = SNESGetFunctionDomainError(snes, &domainerror);CHKERRQ(ierr);
-          if (domainerror) {
-            ierr = SNESLineSearchSetSuccess(linesearch, PETSC_FALSE);CHKERRQ(ierr);
-            PetscFunctionReturn(0);
-          }
+          ierr = (*linesearch->ops->snesfunc)(snes,W,G);CHKERRQ(ierr);
           if (linesearch->ops->vinorm) {
             gnorm = fnorm;
             ierr  = (*linesearch->ops->vinorm)(snes, G, W, &gnorm);CHKERRQ(ierr);
           } else {
             ierr = VecNorm(G,NORM_2,&gnorm);CHKERRQ(ierr);
-            g    = gnorm*gnorm;
           }
+          g = PetscSqr(gnorm);
         }
-        if (PetscIsInfOrNanReal(gnorm)) {
-          ierr = SNESLineSearchSetSuccess(linesearch, PETSC_FALSE);CHKERRQ(ierr);
-          ierr = PetscInfo(monitor,"Aborted due to Nan or Inf in function evaluation\n");CHKERRQ(ierr);
+        if (PetscIsInfOrNanReal(g)) {
+          ierr = SNESLineSearchSetReason(linesearch, SNES_LINESEARCH_FAILED_NANORINF);CHKERRQ(ierr);
+          ierr = PetscInfo(snes,"Aborted due to Nan or Inf in function evaluation\n");CHKERRQ(ierr);
           PetscFunctionReturn(0);
         }
         if (.5*g < .5*f + lambda*alpha*initslope) { /* is reduction enough? */
@@ -375,12 +362,7 @@ static PetscErrorCode  SNESLineSearchApply_BT(SNESLineSearch linesearch)
     }
   }
   if (changed_y || changed_w || objective) { /* recompute the function norm if the step has changed or the objective isn't the norm */
-    ierr = SNESComputeFunction(snes,W,G);CHKERRQ(ierr);
-    ierr = SNESGetFunctionDomainError(snes, &domainerror);CHKERRQ(ierr);
-    if (domainerror) {
-      ierr = SNESLineSearchSetSuccess(linesearch, PETSC_FALSE);CHKERRQ(ierr);
-      PetscFunctionReturn(0);
-    }
+    ierr = (*linesearch->ops->snesfunc)(snes,W,G);CHKERRQ(ierr);
     if (linesearch->ops->vinorm) {
       gnorm = fnorm;
       ierr  = (*linesearch->ops->vinorm)(snes, G, W, &gnorm);CHKERRQ(ierr);
@@ -389,8 +371,8 @@ static PetscErrorCode  SNESLineSearchApply_BT(SNESLineSearch linesearch)
     }
     ierr = VecNorm(Y,NORM_2,&ynorm);CHKERRQ(ierr);
     if (PetscIsInfOrNanReal(gnorm)) {
-      ierr = SNESLineSearchSetSuccess(linesearch, PETSC_FALSE);CHKERRQ(ierr);
-      ierr = PetscInfo(monitor,"Aborted due to Nan or Inf in function evaluation\n");CHKERRQ(ierr);
+      ierr = SNESLineSearchSetReason(linesearch,SNES_LINESEARCH_FAILED_NANORINF);CHKERRQ(ierr);
+      ierr = PetscInfo(snes,"Aborted due to Nan or Inf in function evaluation\n");CHKERRQ(ierr);
       PetscFunctionReturn(0);
     }
   }
@@ -404,31 +386,25 @@ static PetscErrorCode  SNESLineSearchApply_BT(SNESLineSearch linesearch)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESLineSearchView_BT"
 PetscErrorCode SNESLineSearchView_BT(SNESLineSearch linesearch, PetscViewer viewer)
 {
   PetscErrorCode    ierr;
   PetscBool         iascii;
-  SNESLineSearch_BT *bt;
+  SNESLineSearch_BT *bt = (SNESLineSearch_BT*)linesearch->data;
 
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
-  bt   = (SNESLineSearch_BT*)linesearch->data;
   if (iascii) {
     if (linesearch->order == SNES_LINESEARCH_ORDER_CUBIC) {
       ierr = PetscViewerASCIIPrintf(viewer, "  interpolation: cubic\n");CHKERRQ(ierr);
     } else if (linesearch->order == SNES_LINESEARCH_ORDER_QUADRATIC) {
       ierr = PetscViewerASCIIPrintf(viewer, "  interpolation: quadratic\n");CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIIPrintf(viewer, "  alpha=%e\n", bt->alpha);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer, "  alpha=%e\n", (double)bt->alpha);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "SNESLineSearchDestroy_BT"
 static PetscErrorCode SNESLineSearchDestroy_BT(SNESLineSearch linesearch)
 {
   PetscErrorCode ierr;
@@ -438,38 +414,30 @@ static PetscErrorCode SNESLineSearchDestroy_BT(SNESLineSearch linesearch)
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "SNESLineSearchSetFromOptions_BT"
-static PetscErrorCode SNESLineSearchSetFromOptions_BT(SNESLineSearch linesearch)
+static PetscErrorCode SNESLineSearchSetFromOptions_BT(PetscOptionItems *PetscOptionsObject,SNESLineSearch linesearch)
 {
-
   PetscErrorCode    ierr;
-  SNESLineSearch_BT *bt;
+  SNESLineSearch_BT *bt = (SNESLineSearch_BT*)linesearch->data;
 
   PetscFunctionBegin;
-  bt = (SNESLineSearch_BT*)linesearch->data;
-
-  ierr = PetscOptionsHead("SNESLineSearch BT options");CHKERRQ(ierr);
+  ierr = PetscOptionsHead(PetscOptionsObject,"SNESLineSearch BT options");CHKERRQ(ierr);
   ierr = PetscOptionsReal("-snes_linesearch_alpha",   "Descent tolerance",        "SNESLineSearchBT", bt->alpha, &bt->alpha, NULL);CHKERRQ(ierr);
-
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "SNESLineSearchCreate_BT"
 /*MC
    SNESLINESEARCHBT - Backtracking line search.
 
    This line search finds the minimum of a polynomial fitting of the L2 norm of the
-   function. If this fit does not satisfy the conditions for progress, the interval shrinks
+   function or the objective function if it is provided with SNESSetObjective(). If this fit does not satisfy the conditions for progress, the interval shrinks
    and the fit is reattempted at most max_it times or until lambda is below minlambda.
 
    Options Database Keys:
 +  -snes_linesearch_alpha<1e-4> - slope descent parameter
 .  -snes_linesearch_damping<1.0> - initial step length
+.  -snes_linesearch_maxstep <length> - if the length the initial step is larger than this then the
+                                       step is scaled back to be of this length at the beginning of the line search
 .  -snes_linesearch_max_it<40> - maximum number of shrinking step
 .  -snes_linesearch_minlambda<1e-12> - minimum step length allowed
 -  -snes_linesearch_order<cubic,quadratic> - order of the approximation
@@ -498,7 +466,7 @@ PETSC_EXTERN PetscErrorCode SNESLineSearchCreate_BT(SNESLineSearch linesearch)
   linesearch->ops->view           = SNESLineSearchView_BT;
   linesearch->ops->setup          = NULL;
 
-  ierr = PetscNewLog(linesearch, SNESLineSearch_BT, &bt);CHKERRQ(ierr);
+  ierr = PetscNewLog(linesearch,&bt);CHKERRQ(ierr);
 
   linesearch->data    = (void*)bt;
   linesearch->max_its = 40;
