@@ -1,8 +1,6 @@
 #include <petsc/private/linesearchimpl.h>
 #include <petscsnes.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESLineSearchApply_CP"
 static PetscErrorCode SNESLineSearchApply_CP(SNESLineSearch linesearch)
 {
   PetscBool      changed_y, changed_w;
@@ -10,12 +8,10 @@ static PetscErrorCode SNESLineSearchApply_CP(SNESLineSearch linesearch)
   Vec            X, Y, F, W;
   SNES           snes;
   PetscReal      xnorm, ynorm, gnorm, steptol, atol, rtol, ltol, maxstep;
-
-  PetscReal   lambda, lambda_old, lambda_update, delLambda;
-  PetscScalar fty, fty_init, fty_old, fty_mid1, fty_mid2, s;
-  PetscInt    i, max_its;
-
-  PetscViewer monitor;
+  PetscReal      lambda, lambda_old, lambda_update, delLambda;
+  PetscScalar    fty, fty_init, fty_old, fty_mid1, fty_mid2, s;
+  PetscInt       i, max_its;
+  PetscViewer    monitor;
 
   PetscFunctionBegin;
   ierr = SNESLineSearchGetVecs(linesearch, &X, &F, &Y, &W, NULL);CHKERRQ(ierr);
@@ -31,6 +27,15 @@ static PetscErrorCode SNESLineSearchApply_CP(SNESLineSearch linesearch)
   lambda_old = 0.0;
 
   ierr = VecDot(F,Y,&fty_old);CHKERRQ(ierr);
+  if (PetscAbsScalar(fty_old) < atol) {
+    if (monitor) {
+      ierr = PetscViewerASCIIAddTab(monitor,((PetscObject)linesearch)->tablevel);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(monitor,"    Line search terminated ended at initial point because dot(F,Y) = %g < atol = %g\n",(double)PetscAbsScalar(fty_old), (double)atol);CHKERRQ(ierr);
+      ierr = PetscViewerASCIISubtractTab(monitor,((PetscObject)linesearch)->tablevel);CHKERRQ(ierr);
+    }
+    PetscFunctionReturn(0);
+  }
+
   fty_init = fty_old;
 
   for (i = 0; i < max_its; i++) {
@@ -86,6 +91,7 @@ static PetscErrorCode SNESLineSearchApply_CP(SNESLineSearch linesearch)
     }
     /* if the solve is going in the wrong direction, fix it */
     if (PetscRealPart(s) > 0.) s = -s;
+    if (s == 0.0) break;
     lambda_update =  lambda - PetscRealPart(fty / s);
 
     /* switch directions if we stepped out of bounds */
@@ -133,8 +139,6 @@ static PetscErrorCode SNESLineSearchApply_CP(SNESLineSearch linesearch)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESLineSearchCreate_CP"
 /*MC
    SNESLINESEARCHCP - Critical point line search. This line search assumes that there exists some
    artificial G(x) for which the SNESFunction F(x) = grad G(x).  Therefore, this line search seeks
