@@ -4,12 +4,12 @@
 typedef struct {
   PetscErrorCode (*residuallocal)(DM,Vec,Vec,void*);
   PetscErrorCode (*jacobianlocal)(DM,Vec,Mat,Mat,void*);
+  PetscErrorCode (*boundarylocal)(DM,Vec,void*);
   void *residuallocalctx;
   void *jacobianlocalctx;
+  void *boundarylocalctx;
 } DMSNES_Local;
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESDestroy_DMLocal"
 static PetscErrorCode DMSNESDestroy_DMLocal(DMSNES sdm)
 {
   PetscErrorCode ierr;
@@ -19,8 +19,6 @@ static PetscErrorCode DMSNESDestroy_DMLocal(DMSNES sdm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESDuplicate_DMLocal"
 static PetscErrorCode DMSNESDuplicate_DMLocal(DMSNES oldsdm,DMSNES sdm)
 {
   PetscErrorCode ierr;
@@ -33,8 +31,6 @@ static PetscErrorCode DMSNESDuplicate_DMLocal(DMSNES oldsdm,DMSNES sdm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMLocalSNESGetContext"
 static PetscErrorCode DMLocalSNESGetContext(DM dm,DMSNES sdm,DMSNES_Local **dmlocalsnes)
 {
   PetscErrorCode ierr;
@@ -51,8 +47,6 @@ static PetscErrorCode DMLocalSNESGetContext(DM dm,DMSNES sdm,DMSNES_Local **dmlo
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESComputeFunction_DMLocal"
 static PetscErrorCode SNESComputeFunction_DMLocal(SNES snes,Vec X,Vec F,void *ctx)
 {
   PetscErrorCode ierr;
@@ -68,6 +62,7 @@ static PetscErrorCode SNESComputeFunction_DMLocal(SNES snes,Vec X,Vec F,void *ct
   ierr = DMGetLocalVector(dm,&Xloc);CHKERRQ(ierr);
   ierr = DMGetLocalVector(dm,&Floc);CHKERRQ(ierr);
   ierr = VecZeroEntries(Xloc);CHKERRQ(ierr);
+  if (dmlocalsnes->boundarylocal) {ierr = (*dmlocalsnes->boundarylocal)(dm,Xloc,dmlocalsnes->boundarylocalctx);CHKERRQ(ierr);}
   ierr = VecZeroEntries(Floc);CHKERRQ(ierr);
   ierr = DMGlobalToLocalBegin(dm,X,INSERT_VALUES,Xloc);CHKERRQ(ierr);
   ierr = DMGlobalToLocalEnd(dm,X,INSERT_VALUES,Xloc);CHKERRQ(ierr);
@@ -79,11 +74,26 @@ static PetscErrorCode SNESComputeFunction_DMLocal(SNES snes,Vec X,Vec F,void *ct
   ierr = DMLocalToGlobalEnd(dm,Floc,ADD_VALUES,F);CHKERRQ(ierr);
   ierr = DMRestoreLocalVector(dm,&Floc);CHKERRQ(ierr);
   ierr = DMRestoreLocalVector(dm,&Xloc);CHKERRQ(ierr);
+  {
+    char        name[PETSC_MAX_PATH_LEN];
+    char        oldname[PETSC_MAX_PATH_LEN];
+    const char *tmp;
+    PetscInt    it;
+
+    ierr = SNESGetIterationNumber(snes, &it);CHKERRQ(ierr);
+    ierr = PetscSNPrintf(name, PETSC_MAX_PATH_LEN, "Solution, Iterate %d", (int) it);CHKERRQ(ierr);
+    ierr = PetscObjectGetName((PetscObject) X, &tmp);CHKERRQ(ierr);
+    ierr = PetscStrncpy(oldname, tmp, PETSC_MAX_PATH_LEN-1);CHKERRQ(ierr);
+    ierr = PetscObjectSetName((PetscObject) X, name);CHKERRQ(ierr);
+    ierr = VecViewFromOptions(X, (PetscObject) snes, "-dmsnes_solution_vec_view");CHKERRQ(ierr);
+    ierr = PetscObjectSetName((PetscObject) X, oldname);CHKERRQ(ierr);
+    ierr = PetscSNPrintf(name, PETSC_MAX_PATH_LEN, "Residual, Iterate %d", (int) it);CHKERRQ(ierr);
+    ierr = PetscObjectSetName((PetscObject) F, name);CHKERRQ(ierr);
+    ierr = VecViewFromOptions(F, (PetscObject) snes, "-dmsnes_residual_vec_view");CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESComputeJacobian_DMLocal"
 static PetscErrorCode SNESComputeJacobian_DMLocal(SNES snes,Vec X,Mat A,Mat B,void *ctx)
 {
   PetscErrorCode ierr;
@@ -96,6 +106,7 @@ static PetscErrorCode SNESComputeJacobian_DMLocal(SNES snes,Vec X,Mat A,Mat B,vo
   if (dmlocalsnes->jacobianlocal) {
     ierr = DMGetLocalVector(dm,&Xloc);CHKERRQ(ierr);
     ierr = VecZeroEntries(Xloc);CHKERRQ(ierr);
+    if (dmlocalsnes->boundarylocal) {ierr = (*dmlocalsnes->boundarylocal)(dm,Xloc,dmlocalsnes->boundarylocalctx);CHKERRQ(ierr);}
     ierr = DMGlobalToLocalBegin(dm,X,INSERT_VALUES,Xloc);CHKERRQ(ierr);
     ierr = DMGlobalToLocalEnd(dm,X,INSERT_VALUES,Xloc);CHKERRQ(ierr);
     CHKMEMQ;
@@ -141,8 +152,6 @@ static PetscErrorCode SNESComputeJacobian_DMLocal(SNES snes,Vec X,Mat A,Mat B,vo
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESSetFunctionLocal"
 /*@C
    DMSNESSetFunctionLocal - set a local residual evaluation function. This function is called with local vector
       containing the local vector information PLUS ghost point information. It should compute a result for all local
@@ -180,8 +189,39 @@ PetscErrorCode DMSNESSetFunctionLocal(DM dm,PetscErrorCode (*func)(DM,Vec,Vec,vo
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESSetJacobianLocal"
+/*@C
+   DMSNESSetBoundaryLocal - set a local boundary value function. This function is called with local vector
+      containing the local vector information PLUS ghost point information. It should insert values into the local
+      vector that do not come from the global vector, such as essential boundary condition data.
+
+   Logically Collective
+
+   Input Arguments:
++  dm - DM to associate callback with
+.  func - local boundary value evaluation
+-  ctx - optional context for local boundary value evaluation
+
+   Level: intermediate
+
+.seealso: DMSNESSetFunctionLocal(), DMDASNESSetJacobianLocal()
+@*/
+PetscErrorCode DMSNESSetBoundaryLocal(DM dm,PetscErrorCode (*func)(DM,Vec,void*),void *ctx)
+{
+  PetscErrorCode ierr;
+  DMSNES         sdm;
+  DMSNES_Local   *dmlocalsnes;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(dm,DM_CLASSID,1);
+  ierr = DMGetDMSNESWrite(dm,&sdm);CHKERRQ(ierr);
+  ierr = DMLocalSNESGetContext(dm,sdm,&dmlocalsnes);CHKERRQ(ierr);
+
+  dmlocalsnes->boundarylocal    = func;
+  dmlocalsnes->boundarylocalctx = ctx;
+
+  PetscFunctionReturn(0);
+}
+
 /*@C
    DMSNESSetJacobianLocal - set a local Jacobian evaluation function
 
