@@ -1,8 +1,6 @@
-#include <petsc-private/snesimpl.h>   /*I "petscsnes.h" I*/
-#include <petsc-private/dmimpl.h>     /*I "petscdm.h" I*/
+#include <petsc/private/snesimpl.h>   /*I "petscsnes.h" I*/
+#include <petsc/private/dmimpl.h>     /*I "petscdm.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESDestroy"
 static PetscErrorCode DMSNESDestroy(DMSNES *kdm)
 {
   PetscErrorCode ierr;
@@ -16,20 +14,16 @@ static PetscErrorCode DMSNESDestroy(DMSNES *kdm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESLoad"
 PetscErrorCode DMSNESLoad(DMSNES kdm,PetscViewer viewer)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscViewerBinaryRead(viewer,&kdm->ops->computefunction,1,PETSC_FUNCTION);CHKERRQ(ierr);
-  ierr = PetscViewerBinaryRead(viewer,&kdm->ops->computejacobian,1,PETSC_FUNCTION);CHKERRQ(ierr);
+  ierr = PetscViewerBinaryRead(viewer,&kdm->ops->computefunction,1,NULL,PETSC_FUNCTION);CHKERRQ(ierr);
+  ierr = PetscViewerBinaryRead(viewer,&kdm->ops->computejacobian,1,NULL,PETSC_FUNCTION);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESView"
 PetscErrorCode DMSNESView(DMSNES kdm,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -39,7 +33,7 @@ PetscErrorCode DMSNESView(DMSNES kdm,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERBINARY,&isbinary);CHKERRQ(ierr);
   if (isascii) {
-#if defined(PETSC_SERIALIZE_FUNCTIONS)
+#if defined(PETSC_SERIALIZE_FUNCTIONS) && defined(PETSC_SERIALIZE_FUNCTIONS_VIEW)
     const char *fname;
 
     ierr = PetscFPTFind(kdm->ops->computefunction,&fname);CHKERRQ(ierr);
@@ -54,30 +48,28 @@ PetscErrorCode DMSNESView(DMSNES kdm,PetscViewer viewer)
   } else if (isbinary) {
     struct {
       PetscErrorCode (*func)(SNES,Vec,Vec,void*);
-      PetscErrorCode (*jac)(SNES,Vec,Mat*,Mat*,MatStructure*,void*);
-    } funcstruct = {kdm->ops->computefunction,kdm->ops->computejacobian};
-    ierr = PetscViewerBinaryWrite(viewer,&funcstruct,2,PETSC_FUNCTION,PETSC_FALSE);CHKERRQ(ierr);
+    } funcstruct;
+    struct {
+      PetscErrorCode (*jac)(SNES,Vec,Mat,Mat,void*);
+    } jacstruct;
+    funcstruct.func = kdm->ops->computefunction;
+    jacstruct.jac   = kdm->ops->computejacobian;
+    ierr = PetscViewerBinaryWrite(viewer,&funcstruct,1,PETSC_FUNCTION,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = PetscViewerBinaryWrite(viewer,&jacstruct,1,PETSC_FUNCTION,PETSC_FALSE);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESCreate"
 static PetscErrorCode DMSNESCreate(MPI_Comm comm,DMSNES *kdm)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-#if !defined(PETSC_USE_DYNAMIC_LIBRARIES)
   ierr = SNESInitializePackage();CHKERRQ(ierr);
-#endif
-  ierr = PetscHeaderCreate(*kdm, _p_DMSNES, struct _DMSNESOps, DMSNES_CLASSID,  "DMSNES", "DMSNES", "DMSNES", comm, DMSNESDestroy, DMSNESView);CHKERRQ(ierr);
-  ierr = PetscMemzero((*kdm)->ops, sizeof(struct _DMSNESOps));CHKERRQ(ierr);
+  ierr = PetscHeaderCreate(*kdm, DMSNES_CLASSID,  "DMSNES", "DMSNES", "DMSNES", comm, DMSNESDestroy, DMSNESView);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCoarsenHook_DMSNES"
 /* Attaches the DMSNES to the coarse level.
  * Under what conditions should we copy versus duplicate?
  */
@@ -90,8 +82,6 @@ static PetscErrorCode DMCoarsenHook_DMSNES(DM dm,DM dmc,void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMRestrictHook_DMSNES"
 /* This could restrict auxiliary information to the coarse level.
  */
 static PetscErrorCode DMRestrictHook_DMSNES(DM dm,Mat Restrict,Vec rscale,Mat Inject,DM dmc,void *ctx)
@@ -101,8 +91,6 @@ static PetscErrorCode DMRestrictHook_DMSNES(DM dm,Mat Restrict,Vec rscale,Mat In
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSubDomainHook_DMSNES"
 /* Attaches the DMSNES to the subdomain. */
 static PetscErrorCode DMSubDomainHook_DMSNES(DM dm,DM subdm,void *ctx)
 {
@@ -113,8 +101,6 @@ static PetscErrorCode DMSubDomainHook_DMSNES(DM dm,DM subdm,void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSubDomainRestrictHook_DMSNES"
 /* This could restrict auxiliary information to the coarse level.
  */
 static PetscErrorCode DMSubDomainRestrictHook_DMSNES(DM dm,VecScatter gscat,VecScatter lscat,DM subdm,void *ctx)
@@ -124,8 +110,6 @@ static PetscErrorCode DMSubDomainRestrictHook_DMSNES(DM dm,VecScatter gscat,VecS
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMRefineHook_DMSNES"
 static PetscErrorCode DMRefineHook_DMSNES(DM dm,DM dmf,void *ctx)
 {
   PetscErrorCode ierr;
@@ -135,8 +119,6 @@ static PetscErrorCode DMRefineHook_DMSNES(DM dm,DM dmf,void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMInterpolateHook_DMSNES"
 /* This could restrict auxiliary information to the coarse level.
  */
 static PetscErrorCode DMInterpolateHook_DMSNES(DM dm,Mat Interp,DM dmf,void *ctx)
@@ -146,8 +128,6 @@ static PetscErrorCode DMInterpolateHook_DMSNES(DM dm,Mat Interp,DM dmf,void *ctx
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESCopy"
 /*@C
    DMSNESCopy - copies the information in a DMSNES to another DMSNES
 
@@ -195,8 +175,6 @@ PetscErrorCode DMSNESCopy(DMSNES kdm,DMSNES nkdm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMGetDMSNES"
 /*@C
    DMGetDMSNES - get read-only private DMSNES context from a DM
 
@@ -235,8 +213,6 @@ PetscErrorCode DMGetDMSNES(DM dm,DMSNES *snesdm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMGetDMSNESWrite"
 /*@C
    DMGetDMSNESWrite - get write access to private DMSNES context from a DM
 
@@ -273,8 +249,6 @@ PetscErrorCode DMGetDMSNESWrite(DM dm,DMSNES *snesdm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCopyDMSNES"
 /*@C
    DMCopyDMSNES - copies a DM context to a new DM
 
@@ -298,19 +272,14 @@ PetscErrorCode DMCopyDMSNES(DM dmsrc,DM dmdest)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dmsrc,DM_CLASSID,1);
   PetscValidHeaderSpecific(dmdest,DM_CLASSID,2);
-  ierr = DMSNESDestroy((DMSNES*)&dmdest->dmsnes);CHKERRQ(ierr);
-
-  dmdest->dmsnes = dmsrc->dmsnes;
-
-  ierr = PetscObjectReference(dmdest->dmsnes);CHKERRQ(ierr);
+  if (!dmdest->dmsnes) {ierr = DMSNESCreate(PetscObjectComm((PetscObject) dmdest), (DMSNES *) &dmdest->dmsnes);CHKERRQ(ierr);}
+  ierr = DMSNESCopy((DMSNES) dmsrc->dmsnes, (DMSNES) dmdest->dmsnes);CHKERRQ(ierr);
   ierr = DMCoarsenHookAdd(dmdest,DMCoarsenHook_DMSNES,NULL,NULL);CHKERRQ(ierr);
   ierr = DMRefineHookAdd(dmdest,DMRefineHook_DMSNES,NULL,NULL);CHKERRQ(ierr);
   ierr = DMSubDomainHookAdd(dmdest,DMSubDomainHook_DMSNES,DMSubDomainRestrictHook_DMSNES,NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESSetFunction"
 /*@C
    DMSNESSetFunction - set SNES residual evaluation function
 
@@ -318,7 +287,7 @@ PetscErrorCode DMCopyDMSNES(DM dmsrc,DM dmdest)
 
    Input Arguments:
 +  dm - DM to be used with SNES
-.  SNESFunction - residual evaluation function
+.  f - residual evaluation function; see SNESFunction for details
 -  ctx - context for residual evaluation
 
    Level: advanced
@@ -330,23 +299,21 @@ PetscErrorCode DMCopyDMSNES(DM dmsrc,DM dmdest)
 
 .seealso: DMSNESSetContext(), SNESSetFunction(), DMSNESSetJacobian(), SNESFunction
 @*/
-PetscErrorCode DMSNESSetFunction(DM dm,PetscErrorCode (*SNESFunction)(SNES,Vec,Vec,void*),void *ctx)
+PetscErrorCode DMSNESSetFunction(DM dm,PetscErrorCode (*f)(SNES,Vec,Vec,void*),void *ctx)
 {
   PetscErrorCode ierr;
   DMSNES         sdm;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
-  if (SNESFunction || ctx) {
+  if (f || ctx) {
     ierr = DMGetDMSNESWrite(dm,&sdm);CHKERRQ(ierr);
   }
-  if (SNESFunction) sdm->ops->computefunction = SNESFunction;
+  if (f) sdm->ops->computefunction = f;
   if (ctx) sdm->functionctx = ctx;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESGetFunction"
 /*@C
    DMSNESGetFunction - get SNES residual evaluation function
 
@@ -356,7 +323,7 @@ PetscErrorCode DMSNESSetFunction(DM dm,PetscErrorCode (*SNESFunction)(SNES,Vec,V
 .  dm - DM to be used with SNES
 
    Output Arguments:
-+  SNESFunction - residual evaluation function
++  f - residual evaluation function; see SNESFunction for details
 -  ctx - context for residual evaluation
 
    Level: advanced
@@ -367,7 +334,7 @@ PetscErrorCode DMSNESSetFunction(DM dm,PetscErrorCode (*SNESFunction)(SNES,Vec,V
 
 .seealso: DMSNESSetContext(), DMSNESSetFunction(), SNESSetFunction(), SNESFunction
 @*/
-PetscErrorCode DMSNESGetFunction(DM dm,PetscErrorCode (**SNESFunction)(SNES,Vec,Vec,void*),void **ctx)
+PetscErrorCode DMSNESGetFunction(DM dm,PetscErrorCode (**f)(SNES,Vec,Vec,void*),void **ctx)
 {
   PetscErrorCode ierr;
   DMSNES         sdm;
@@ -375,13 +342,11 @@ PetscErrorCode DMSNESGetFunction(DM dm,PetscErrorCode (**SNESFunction)(SNES,Vec,
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
   ierr = DMGetDMSNES(dm,&sdm);CHKERRQ(ierr);
-  if (SNESFunction) *SNESFunction = sdm->ops->computefunction;
+  if (f) *f = sdm->ops->computefunction;
   if (ctx) *ctx = sdm->functionctx;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESSetObjective"
 /*@C
    DMSNESSetObjective - set SNES objective evaluation function
 
@@ -389,30 +354,28 @@ PetscErrorCode DMSNESGetFunction(DM dm,PetscErrorCode (**SNESFunction)(SNES,Vec,
 
    Input Arguments:
 +  dm - DM to be used with SNES
-.  SNESObjectiveFunction - residual evaluation function
+.  obj - objective evaluation function; see SNESObjectiveFunction for details
 -  ctx - context for residual evaluation
 
    Level: advanced
 
 .seealso: DMSNESSetContext(), SNESGetObjective(), DMSNESSetFunction()
 @*/
-PetscErrorCode DMSNESSetObjective(DM dm,PetscErrorCode (*SNESObjectiveFunction)(SNES,Vec,PetscReal*,void*),void *ctx)
+PetscErrorCode DMSNESSetObjective(DM dm,PetscErrorCode (*obj)(SNES,Vec,PetscReal*,void*),void *ctx)
 {
   PetscErrorCode ierr;
   DMSNES         sdm;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
-  if (SNESObjectiveFunction || ctx) {
+  if (obj || ctx) {
     ierr = DMGetDMSNESWrite(dm,&sdm);CHKERRQ(ierr);
   }
-  if (SNESObjectiveFunction) sdm->ops->computeobjective = SNESObjectiveFunction;
+  if (obj) sdm->ops->computeobjective = obj;
   if (ctx) sdm->objectivectx = ctx;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESGetObjective"
 /*@C
    DMSNESGetObjective - get SNES objective evaluation function
 
@@ -422,7 +385,7 @@ PetscErrorCode DMSNESSetObjective(DM dm,PetscErrorCode (*SNESObjectiveFunction)(
 .  dm - DM to be used with SNES
 
    Output Arguments:
-+  SNESObjectiveFunction- residual evaluation function
++  obj- residual evaluation function; see SNESObjectiveFunction for details
 -  ctx - context for residual evaluation
 
    Level: advanced
@@ -433,7 +396,7 @@ PetscErrorCode DMSNESSetObjective(DM dm,PetscErrorCode (*SNESObjectiveFunction)(
 
 .seealso: DMSNESSetContext(), DMSNESSetObjective(), SNESSetFunction()
 @*/
-PetscErrorCode DMSNESGetObjective(DM dm,PetscErrorCode (**SNESObjectiveFunction)(SNES,Vec,PetscReal*,void*),void **ctx)
+PetscErrorCode DMSNESGetObjective(DM dm,PetscErrorCode (**obj)(SNES,Vec,PetscReal*,void*),void **ctx)
 {
   PetscErrorCode ierr;
   DMSNES         sdm;
@@ -441,51 +404,47 @@ PetscErrorCode DMSNESGetObjective(DM dm,PetscErrorCode (**SNESObjectiveFunction)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
   ierr = DMGetDMSNES(dm,&sdm);CHKERRQ(ierr);
-  if (SNESObjectiveFunction) *SNESObjectiveFunction = sdm->ops->computeobjective;
+  if (obj) *obj = sdm->ops->computeobjective;
   if (ctx) *ctx = sdm->objectivectx;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESSetGS"
 /*@C
-   DMSNESSetGS - set SNES Gauss-Seidel relaxation function
+   DMSNESSetNGS - set SNES Gauss-Seidel relaxation function
 
    Not Collective
 
    Input Argument:
 +  dm - DM to be used with SNES
-.  SNESGSFunction - relaxation function
+.  f  - relaxation function, see SNESGSFunction
 -  ctx - context for residual evaluation
 
    Level: advanced
 
    Note:
-   SNESSetGS() is normally used, but it calls this function internally because the user context is actually
+   SNESSetNGS() is normally used, but it calls this function internally because the user context is actually
    associated with the DM.  This makes the interface consistent regardless of whether the user interacts with a DM or
    not. If DM took a more central role at some later date, this could become the primary method of setting the residual.
 
 .seealso: DMSNESSetContext(), SNESSetFunction(), DMSNESSetJacobian(), DMSNESSetFunction(), SNESGSFunction
 @*/
-PetscErrorCode DMSNESSetGS(DM dm,PetscErrorCode (*SNESGSFunction)(SNES,Vec,Vec,void*),void *ctx)
+PetscErrorCode DMSNESSetNGS(DM dm,PetscErrorCode (*f)(SNES,Vec,Vec,void*),void *ctx)
 {
   PetscErrorCode ierr;
   DMSNES         sdm;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
-  if (SNESGSFunction || ctx) {
+  if (f || ctx) {
     ierr = DMGetDMSNESWrite(dm,&sdm);CHKERRQ(ierr);
   }
-  if (SNESGSFunction) sdm->ops->computegs = SNESGSFunction;
+  if (f) sdm->ops->computegs = f;
   if (ctx) sdm->gsctx = ctx;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESGetGS"
 /*@C
-   DMSNESGetGS - get SNES Gauss-Seidel relaxation function
+   DMSNESGetNGS - get SNES Gauss-Seidel relaxation function
 
    Not Collective
 
@@ -493,19 +452,19 @@ PetscErrorCode DMSNESSetGS(DM dm,PetscErrorCode (*SNESGSFunction)(SNES,Vec,Vec,v
 .  dm - DM to be used with SNES
 
    Output Arguments:
-+  SNESGSFunction - relaxation function which performs Gauss-Seidel sweeps
++  f - relaxation function which performs Gauss-Seidel sweeps, see SNESGSFunction
 -  ctx - context for residual evaluation
 
    Level: advanced
 
    Note:
-   SNESGetGS() is normally used, but it calls this function internally because the user context is actually
+   SNESGetNGS() is normally used, but it calls this function internally because the user context is actually
    associated with the DM.  This makes the interface consistent regardless of whether the user interacts with a DM or
    not. If DM took a more central role at some later date, this could become the primary method of setting the residual.
 
-.seealso: DMSNESSetContext(), SNESGetGS(), DMSNESGetJacobian(), DMSNESGetFunction(), SNESGSFunction
+.seealso: DMSNESSetContext(), SNESGetNGS(), DMSNESGetJacobian(), DMSNESGetFunction(), SNESNGSFunction
 @*/
-PetscErrorCode DMSNESGetGS(DM dm,PetscErrorCode (**SNESGSFunction)(SNES,Vec,Vec,void*),void **ctx)
+PetscErrorCode DMSNESGetNGS(DM dm,PetscErrorCode (**f)(SNES,Vec,Vec,void*),void **ctx)
 {
   PetscErrorCode ierr;
   DMSNES         sdm;
@@ -513,13 +472,11 @@ PetscErrorCode DMSNESGetGS(DM dm,PetscErrorCode (**SNESGSFunction)(SNES,Vec,Vec,
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
   ierr = DMGetDMSNES(dm,&sdm);CHKERRQ(ierr);
-  if (SNESGSFunction) *SNESGSFunction = sdm->ops->computegs;
+  if (f) *f = sdm->ops->computegs;
   if (ctx) *ctx = sdm->gsctx;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESSetJacobian"
 /*@C
    DMSNESSetJacobian - set SNES Jacobian evaluation function
 
@@ -527,7 +484,7 @@ PetscErrorCode DMSNESGetGS(DM dm,PetscErrorCode (**SNESGSFunction)(SNES,Vec,Vec,
 
    Input Argument:
 +  dm - DM to be used with SNES
-.  SNESJacobianFunction - Jacobian evaluation function
+.  J - Jacobian evaluation function
 -  ctx - context for residual evaluation
 
    Level: advanced
@@ -539,23 +496,21 @@ PetscErrorCode DMSNESGetGS(DM dm,PetscErrorCode (**SNESGSFunction)(SNES,Vec,Vec,
 
 .seealso: DMSNESSetContext(), SNESSetFunction(), DMSNESGetJacobian(), SNESSetJacobian(), SNESJacobianFunction
 @*/
-PetscErrorCode DMSNESSetJacobian(DM dm,PetscErrorCode (*SNESJacobianFunction)(SNES,Vec,Mat*,Mat*,MatStructure*,void*),void *ctx)
+PetscErrorCode DMSNESSetJacobian(DM dm,PetscErrorCode (*J)(SNES,Vec,Mat,Mat,void*),void *ctx)
 {
   PetscErrorCode ierr;
   DMSNES         sdm;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
-  if (SNESJacobianFunction || ctx) {
+  if (J || ctx) {
     ierr = DMGetDMSNESWrite(dm,&sdm);CHKERRQ(ierr);
   }
-  if (SNESJacobianFunction) sdm->ops->computejacobian = SNESJacobianFunction;
+  if (J) sdm->ops->computejacobian = J;
   if (ctx) sdm->jacobianctx = ctx;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESGetJacobian"
 /*@C
    DMSNESGetJacobian - get SNES Jacobian evaluation function
 
@@ -565,7 +520,7 @@ PetscErrorCode DMSNESSetJacobian(DM dm,PetscErrorCode (*SNESJacobianFunction)(SN
 .  dm - DM to be used with SNES
 
    Output Arguments:
-+  SNESJacobianFunction - Jacobian evaluation function
++  J - Jacobian evaluation function; see SNESJacobianFunction for all calling sequence
 -  ctx - context for residual evaluation
 
    Level: advanced
@@ -577,7 +532,7 @@ PetscErrorCode DMSNESSetJacobian(DM dm,PetscErrorCode (*SNESJacobianFunction)(SN
 
 .seealso: DMSNESSetContext(), SNESSetFunction(), DMSNESSetJacobian(), SNESJacobianFunction
 @*/
-PetscErrorCode DMSNESGetJacobian(DM dm,PetscErrorCode (**SNESJacobianFunction)(SNES,Vec,Mat*,Mat*,MatStructure*,void*),void **ctx)
+PetscErrorCode DMSNESGetJacobian(DM dm,PetscErrorCode (**J)(SNES,Vec,Mat,Mat,void*),void **ctx)
 {
   PetscErrorCode ierr;
   DMSNES         sdm;
@@ -585,13 +540,11 @@ PetscErrorCode DMSNESGetJacobian(DM dm,PetscErrorCode (**SNESJacobianFunction)(S
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
   ierr = DMGetDMSNES(dm,&sdm);CHKERRQ(ierr);
-  if (SNESJacobianFunction) *SNESJacobianFunction = sdm->ops->computejacobian;
+  if (J) *J = sdm->ops->computejacobian;
   if (ctx) *ctx = sdm->jacobianctx;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESSetPicard"
 /*@C
    DMSNESSetPicard - set SNES Picard iteration matrix and RHS evaluation functions.
 
@@ -599,15 +552,15 @@ PetscErrorCode DMSNESGetJacobian(DM dm,PetscErrorCode (**SNESJacobianFunction)(S
 
    Input Argument:
 +  dm - DM to be used with SNES
-.  SNESFunction - RHS evaluation function
-.  SNESJacobianFunction - Picard matrix evaluation function
+.  b - RHS evaluation function
+.  J - Picard matrix evaluation function
 -  ctx - context for residual evaluation
 
    Level: advanced
 
 .seealso: SNESSetPicard(), DMSNESSetFunction(), DMSNESSetJacobian()
 @*/
-PetscErrorCode DMSNESSetPicard(DM dm,PetscErrorCode (*SNESFunction)(SNES,Vec,Vec,void*),PetscErrorCode (*SNESJacobianFunction)(SNES,Vec,Mat*,Mat*,MatStructure*,void*),void *ctx)
+PetscErrorCode DMSNESSetPicard(DM dm,PetscErrorCode (*b)(SNES,Vec,Vec,void*),PetscErrorCode (*J)(SNES,Vec,Mat,Mat,void*),void *ctx)
 {
   PetscErrorCode ierr;
   DMSNES         sdm;
@@ -615,14 +568,12 @@ PetscErrorCode DMSNESSetPicard(DM dm,PetscErrorCode (*SNESFunction)(SNES,Vec,Vec
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
   ierr = DMGetDMSNES(dm,&sdm);CHKERRQ(ierr);
-  if (SNESFunction) sdm->ops->computepfunction = SNESFunction;
-  if (SNESJacobianFunction) sdm->ops->computepjacobian = SNESJacobianFunction;
+  if (b) sdm->ops->computepfunction = b;
+  if (J) sdm->ops->computepjacobian = J;
   if (ctx) sdm->pctx = ctx;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSNESGetPicard"
 /*@C
    DMSNESGetPicard - get SNES Picard iteration evaluation functions
 
@@ -632,15 +583,15 @@ PetscErrorCode DMSNESSetPicard(DM dm,PetscErrorCode (*SNESFunction)(SNES,Vec,Vec
 .  dm - DM to be used with SNES
 
    Output Arguments:
-+  SNESFunction - Jacobian evaluation function;
-.  SNESJacobianFunction  - RHS evaluation function;
++  b - RHS evaluation function; see SNESFunction for details
+.  J  - RHS evaluation function; see SNESJacobianFunction for detailsa
 -  ctx - context for residual evaluation
 
    Level: advanced
 
 .seealso: DMSNESSetContext(), SNESSetFunction(), DMSNESSetJacobian()
 @*/
-PetscErrorCode DMSNESGetPicard(DM dm,PetscErrorCode (**SNESFunction)(SNES,Vec,Vec,void*),PetscErrorCode (**SNESJacobianFunction)(SNES,Vec,Mat*,Mat*,MatStructure*,void*),void **ctx)
+PetscErrorCode DMSNESGetPicard(DM dm,PetscErrorCode (**b)(SNES,Vec,Vec,void*),PetscErrorCode (**J)(SNES,Vec,Mat,Mat,void*),void **ctx)
 {
   PetscErrorCode ierr;
   DMSNES         sdm;
@@ -648,8 +599,8 @@ PetscErrorCode DMSNESGetPicard(DM dm,PetscErrorCode (**SNESFunction)(SNES,Vec,Ve
   PetscFunctionBegin;
   PetscValidHeaderSpecific(dm,DM_CLASSID,1);
   ierr = DMGetDMSNES(dm,&sdm);CHKERRQ(ierr);
-  if (SNESFunction) *SNESFunction = sdm->ops->computepfunction;
-  if (SNESJacobianFunction) *SNESJacobianFunction = sdm->ops->computepjacobian;
+  if (b) *b = sdm->ops->computepfunction;
+  if (J) *J = sdm->ops->computepjacobian;
   if (ctx) *ctx = sdm->pctx;
   PetscFunctionReturn(0);
 }
