@@ -1,11 +1,10 @@
 
 static char help[] = "Demonstrates use of color map\n";
 
+
 #include <petscsys.h>
 #include <petscdraw.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   PetscDraw      draw;
@@ -27,6 +26,17 @@ int main(int argc,char **argv)
   ierr = PetscDrawSave(draw);CHKERRQ(ierr);
   ierr = PetscDrawDestroy(&draw);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
+
+
+/*TEST
+
+   build:
+     requires: x
+
+   test:
+     output_file: output/ex1_1.out
+
+TEST*/
