@@ -8,8 +8,6 @@
 PetscClassId PETSC_DRAW_CLASSID;
 
 static PetscBool PetscDrawPackageInitialized = PETSC_FALSE;
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawFinalizePackage"
 /*@C
   PetscDrawFinalizePackage - This function destroys everything in the Petsc interface to the Draw package. It is
   called from PetscFinalize().
@@ -30,8 +28,6 @@ PetscErrorCode  PetscDrawFinalizePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawInitializePackage"
 /*@C
   PetscInitializeDrawPackage - This function initializes everything in the PetscDraw package. It is called
   from PetscDLLibraryRegister() when using dynamic libraries, and on the call to PetscInitialize()
@@ -45,8 +41,7 @@ PetscErrorCode  PetscDrawFinalizePackage(void)
 PetscErrorCode  PetscDrawInitializePackage(void)
 {
   char           logList[256];
-  char           *className;
-  PetscBool      opt;
+  PetscBool      opt,pkg;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -54,35 +49,44 @@ PetscErrorCode  PetscDrawInitializePackage(void)
   PetscDrawPackageInitialized = PETSC_TRUE;
   /* Register Classes */
   ierr = PetscClassIdRegister("Draw",&PETSC_DRAW_CLASSID);CHKERRQ(ierr);
-  ierr = PetscClassIdRegister("Axis",&PETSC_DRAWAXIS_CLASSID);CHKERRQ(ierr);
+  ierr = PetscClassIdRegister("Draw Axis",&PETSC_DRAWAXIS_CLASSID);CHKERRQ(ierr);
   ierr = PetscClassIdRegister("Line Graph",&PETSC_DRAWLG_CLASSID);CHKERRQ(ierr);
   ierr = PetscClassIdRegister("Histogram",&PETSC_DRAWHG_CLASSID);CHKERRQ(ierr);
-  ierr = PetscClassIdRegister("Bar Grap",&PETSC_DRAWBAR_CLASSID);CHKERRQ(ierr);
+  ierr = PetscClassIdRegister("Bar Graph",&PETSC_DRAWBAR_CLASSID);CHKERRQ(ierr);
   ierr = PetscClassIdRegister("Scatter Plot",&PETSC_DRAWSP_CLASSID);CHKERRQ(ierr);
   /* Register Constructors */
   ierr = PetscDrawRegisterAll();CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL, "-info_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "draw", &className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscInfoDeactivateClass(0);CHKERRQ(ierr);
+    ierr = PetscStrInList("draw",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) {
+      ierr = PetscInfoDeactivateClass(PETSC_DRAW_CLASSID);CHKERRQ(ierr);
+      ierr = PetscInfoDeactivateClass(PETSC_DRAWAXIS_CLASSID);CHKERRQ(ierr);
+      ierr = PetscInfoDeactivateClass(PETSC_DRAWLG_CLASSID);CHKERRQ(ierr);
+      ierr = PetscInfoDeactivateClass(PETSC_DRAWHG_CLASSID);CHKERRQ(ierr);
+      ierr = PetscInfoDeactivateClass(PETSC_DRAWBAR_CLASSID);CHKERRQ(ierr);
+      ierr = PetscInfoDeactivateClass(PETSC_DRAWSP_CLASSID);CHKERRQ(ierr);
     }
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(NULL, "-log_summary_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "draw", &className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscLogEventDeactivateClass(0);CHKERRQ(ierr);
+    ierr = PetscStrInList("draw",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) {
+      ierr = PetscLogEventExcludeClass(PETSC_DRAW_CLASSID);CHKERRQ(ierr);
+      ierr = PetscLogEventExcludeClass(PETSC_DRAWAXIS_CLASSID);CHKERRQ(ierr);
+      ierr = PetscLogEventExcludeClass(PETSC_DRAWLG_CLASSID);CHKERRQ(ierr);
+      ierr = PetscLogEventExcludeClass(PETSC_DRAWHG_CLASSID);CHKERRQ(ierr);
+      ierr = PetscLogEventExcludeClass(PETSC_DRAWBAR_CLASSID);CHKERRQ(ierr);
+      ierr = PetscLogEventExcludeClass(PETSC_DRAWSP_CLASSID);CHKERRQ(ierr);
     }
   }
+  /* Register package finalizer */
   ierr = PetscRegisterFinalize(PetscDrawFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawResizeWindow"
 /*@
    PetscDrawResizeWindow - Allows one to resize a window from a program.
 
@@ -101,14 +105,41 @@ PetscErrorCode  PetscDrawResizeWindow(PetscDraw draw,int w,int h)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
+  PetscValidLogicalCollectiveInt(draw,w,2);
+  PetscValidLogicalCollectiveInt(draw,h,3);
   if (draw->ops->resizewindow) {
     ierr = (*draw->ops->resizewindow)(draw,w,h);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawCheckResizedWindow"
+/*@
+   PetscDrawGetWindowSize - Gets the size of the window.
+
+   Not collective
+
+   Input Parameter:
+.  draw - the window
+
+   Output Parameters:
+.  w,h - the window width and height
+
+   Level: intermediate
+
+.seealso: PetscDrawResizeWindow(), PetscDrawCheckResizedWindow()
+@*/
+PetscErrorCode  PetscDrawGetWindowSize(PetscDraw draw,int *w,int *h)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
+  if (w) PetscValidPointer(w,2);
+  if (h) PetscValidPointer(h,3);
+  if (w) *w = draw->w;
+  if (h) *h = draw->h;
+  PetscFunctionReturn(0);
+}
+
 /*@
    PetscDrawCheckResizedWindow - Checks if the user has resized the window.
 
@@ -127,14 +158,13 @@ PetscErrorCode  PetscDrawCheckResizedWindow(PetscDraw draw)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
   if (draw->ops->checkresizedwindow) {
     ierr = (*draw->ops->checkresizedwindow)(draw);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawGetTitle"
 /*@C
    PetscDrawGetTitle - Gets pointer to title of a PetscDraw context.
 
@@ -150,7 +180,7 @@ PetscErrorCode  PetscDrawCheckResizedWindow(PetscDraw draw)
 
 .seealso: PetscDrawSetTitle()
 @*/
-PetscErrorCode  PetscDrawGetTitle(PetscDraw draw,char **title)
+PetscErrorCode  PetscDrawGetTitle(PetscDraw draw,const char *title[])
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
@@ -159,12 +189,10 @@ PetscErrorCode  PetscDrawGetTitle(PetscDraw draw,char **title)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawSetTitle"
 /*@C
    PetscDrawSetTitle - Sets the title of a PetscDraw context.
 
-   Not collective (any processor or all may call this)
+   Collective on PetscDraw
 
    Input Parameters:
 +  draw - the graphics context
@@ -172,13 +200,13 @@ PetscErrorCode  PetscDrawGetTitle(PetscDraw draw,char **title)
 
    Level: intermediate
 
-   Note: The title is positioned in the windowing system title bar for the window. Hence it will not be saved with -draw_save 
+   Note: The title is positioned in the windowing system title bar for the window. Hence it will not be saved with -draw_save
    in the image.
 
    A copy of the string is made, so you may destroy the
    title string after calling this routine.
 
-   You can use PetscDrawAxisSetLabels() to indicate a title within the window 
+   You can use PetscDrawAxisSetLabels() to indicate a title within the window
 
 .seealso: PetscDrawGetTitle(), PetscDrawAppendTitle()
 @*/
@@ -192,17 +220,15 @@ PetscErrorCode  PetscDrawSetTitle(PetscDraw draw,const char title[])
   ierr = PetscFree(draw->title);CHKERRQ(ierr);
   ierr = PetscStrallocpy(title,&draw->title);CHKERRQ(ierr);
   if (draw->ops->settitle) {
-    ierr = (*draw->ops->settitle)(draw,title);CHKERRQ(ierr);
+    ierr = (*draw->ops->settitle)(draw,draw->title);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawAppendTitle"
 /*@C
    PetscDrawAppendTitle - Appends to the title of a PetscDraw context.
 
-   Not collective (any processor or all can call this)
+   Collective on PetscDraw
 
    Input Parameters:
 +  draw - the graphics context
@@ -219,22 +245,21 @@ PetscErrorCode  PetscDrawSetTitle(PetscDraw draw,const char title[])
 PetscErrorCode  PetscDrawAppendTitle(PetscDraw draw,const char title[])
 {
   PetscErrorCode ierr;
-  size_t         len1,len2,len;
-  char           *newtitle;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
-  if (!title) PetscFunctionReturn(0);
+  if (title) PetscValidCharPointer(title,2);
+  if (!title || !title[0]) PetscFunctionReturn(0);
 
   if (draw->title) {
+    size_t len1,len2;
+    char   *newtitle;
     ierr = PetscStrlen(title,&len1);CHKERRQ(ierr);
     ierr = PetscStrlen(draw->title,&len2);CHKERRQ(ierr);
-    len  = len1 + len2;
-    ierr = PetscMalloc1(len + 1,&newtitle);CHKERRQ(ierr);
+    ierr = PetscMalloc1(len1 + len2 + 1,&newtitle);CHKERRQ(ierr);
     ierr = PetscStrcpy(newtitle,draw->title);CHKERRQ(ierr);
     ierr = PetscStrcat(newtitle,title);CHKERRQ(ierr);
     ierr = PetscFree(draw->title);CHKERRQ(ierr);
-
     draw->title = newtitle;
   } else {
     ierr = PetscStrallocpy(title,&draw->title);CHKERRQ(ierr);
@@ -245,8 +270,22 @@ PetscErrorCode  PetscDrawAppendTitle(PetscDraw draw,const char title[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawDestroy"
+static PetscErrorCode PetscDrawDestroy_Private(PetscDraw draw)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (!draw->ops->save && !draw->ops->getimage) PetscFunctionReturn(0);
+  ierr = PetscDrawSaveMovie(draw);CHKERRQ(ierr);
+  if (draw->savefinalfilename) {
+    draw->savesinglefile = PETSC_TRUE;
+    ierr = PetscDrawSetSave(draw,draw->savefinalfilename);CHKERRQ(ierr);
+    ierr = PetscDrawSave(draw);CHKERRQ(ierr);
+  }
+  ierr = PetscBarrier((PetscObject)draw);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    PetscDrawDestroy - Deletes a draw context.
 
@@ -271,27 +310,28 @@ PetscErrorCode  PetscDrawDestroy(PetscDraw *draw)
 
   if ((*draw)->pause == -2) {
     (*draw)->pause = -1;
-
     ierr = PetscDrawPause(*draw);CHKERRQ(ierr);
   }
 
   /* if memory was published then destroy it */
   ierr = PetscObjectSAWsViewOff((PetscObject)*draw);CHKERRQ(ierr);
 
+  ierr = PetscDrawDestroy_Private(*draw);CHKERRQ(ierr);
+
   if ((*draw)->ops->destroy) {
     ierr = (*(*draw)->ops->destroy)(*draw);CHKERRQ(ierr);
   }
+  ierr = PetscDrawDestroy(&(*draw)->popup);CHKERRQ(ierr);
   ierr = PetscFree((*draw)->title);CHKERRQ(ierr);
   ierr = PetscFree((*draw)->display);CHKERRQ(ierr);
   ierr = PetscFree((*draw)->savefilename);CHKERRQ(ierr);
-  ierr = PetscFree((*draw)->savefilenameext);CHKERRQ(ierr);
+  ierr = PetscFree((*draw)->saveimageext);CHKERRQ(ierr);
+  ierr = PetscFree((*draw)->savemovieext);CHKERRQ(ierr);
   ierr = PetscFree((*draw)->savefinalfilename);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(draw);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawGetPopup"
 /*@
    PetscDrawGetPopup - Creates a popup window associated with a PetscDraw window.
 
@@ -305,6 +345,8 @@ PetscErrorCode  PetscDrawDestroy(PetscDraw *draw)
 
    Level: advanced
 
+.seealso: PetscDrawScalePopup(), PetscDrawCreate()
+
 @*/
 PetscErrorCode  PetscDrawGetPopup(PetscDraw draw,PetscDraw *popup)
 {
@@ -319,45 +361,14 @@ PetscErrorCode  PetscDrawGetPopup(PetscDraw draw,PetscDraw *popup)
     ierr = (*draw->ops->getpopup)(draw,popup);CHKERRQ(ierr);
     if (*popup) {
       ierr = PetscObjectSetOptionsPrefix((PetscObject)*popup,"popup_");CHKERRQ(ierr);
+      (*popup)->pause = 0.0;
       ierr = PetscDrawSetFromOptions(*popup);CHKERRQ(ierr);
     }
   } else *popup = NULL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawDestroy_Null"
-PetscErrorCode PetscDrawDestroy_Null(PetscDraw draw)
-{
-  PetscFunctionBegin;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawOpenNull"
-/*
-  PetscDrawOpenNull - Opens a null drawing context. All draw commands to
-  it are ignored.
-
-  Output Parameter:
-. win - the drawing context
-
-   Level: advanced
-
-*/
-PetscErrorCode  PetscDrawOpenNull(MPI_Comm comm,PetscDraw *win)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  ierr = PetscDrawCreate(comm,NULL,NULL,0,0,1,1,win);CHKERRQ(ierr);
-  ierr = PetscDrawSetType(*win,PETSC_DRAW_NULL);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawSetDisplay"
-/*@
+/*@C
   PetscDrawSetDisplay - Sets the display where a PetscDraw object will be displayed
 
   Input Parameter:
@@ -366,6 +377,8 @@ PetscErrorCode  PetscDrawOpenNull(MPI_Comm comm,PetscDraw *win)
 
   Level: advanced
 
+.seealso: PetscDrawCreate()
+
 @*/
 PetscErrorCode  PetscDrawSetDisplay(PetscDraw draw,const char display[])
 {
@@ -377,35 +390,34 @@ PetscErrorCode  PetscDrawSetDisplay(PetscDraw draw,const char display[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawCreate_Null"
-/*
-  PetscDrawCreate_Null - Opens a null drawing context. All draw commands to
-  it are ignored.
 
-  Input Parameter:
-. win - the drawing context
-*/
-PETSC_EXTERN PetscErrorCode PetscDrawCreate_Null(PetscDraw draw)
+/*@
+   PetscDrawSetDoubleBuffer - Sets a window to be double buffered.
+
+   Logically Collective on PetscDraw
+
+   Input Parameter:
+.  draw - the drawing context
+
+   Level: intermediate
+
+   Concepts: drawing^double buffer
+   Concepts: graphics^double buffer
+   Concepts: double buffer
+
+@*/
+PetscErrorCode  PetscDrawSetDoubleBuffer(PetscDraw draw)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscMemzero(draw->ops,sizeof(struct _PetscDrawOps));CHKERRQ(ierr);
-
-  draw->ops->destroy = PetscDrawDestroy_Null;
-  draw->ops->view    = 0;
-  draw->pause        = 0.0;
-  draw->coor_xl      = 0.0;  draw->coor_xr = 1.0;
-  draw->coor_yl      = 0.0;  draw->coor_yr = 1.0;
-  draw->port_xl      = 0.0;  draw->port_xr = 1.0;
-  draw->port_yl      = 0.0;  draw->port_yr = 1.0;
-  draw->popup        = 0;
+  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
+  if (draw->ops->setdoublebuffer) {
+    ierr = (*draw->ops->setdoublebuffer)(draw);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawGetSingleton"
 /*@C
    PetscDrawGetSingleton - Gain access to a PetscDraw object as if it were owned
         by the one process.
@@ -433,8 +445,10 @@ PetscErrorCode  PetscDrawGetSingleton(PetscDraw draw,PetscDraw *sdraw)
   PetscValidPointer(sdraw,2);
 
   ierr = MPI_Comm_size(PetscObjectComm((PetscObject)draw),&size);CHKERRQ(ierr);
-  if (size == 1) *sdraw = draw;
-  else {
+  if (size == 1) {
+    ierr = PetscObjectReference((PetscObject)draw);CHKERRQ(ierr);
+    *sdraw = draw;
+  } else {
     if (draw->ops->getsingleton) {
       ierr = (*draw->ops->getsingleton)(draw,sdraw);CHKERRQ(ierr);
     } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot get singleton for this type %s of draw object",((PetscObject)draw)->type_name);
@@ -442,8 +456,6 @@ PetscErrorCode  PetscDrawGetSingleton(PetscDraw draw,PetscDraw *sdraw)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawRestoreSingleton"
 /*@C
    PetscDrawRestoreSingleton - Remove access to a PetscDraw object as if it were owned
         by the one process.
@@ -470,17 +482,15 @@ PetscErrorCode  PetscDrawRestoreSingleton(PetscDraw draw,PetscDraw *sdraw)
   PetscValidHeaderSpecific(*sdraw,PETSC_DRAW_CLASSID,2);
 
   ierr = MPI_Comm_size(PetscObjectComm((PetscObject)draw),&size);CHKERRQ(ierr);
-  if (size != 1) {
+  if (size == 1) {
+    if (draw == *sdraw) {
+      ierr = PetscObjectDereference((PetscObject)draw);CHKERRQ(ierr);
+      *sdraw = NULL;
+    } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Cannot restore singleton, it is not the parent draw");
+  } else {
     if (draw->ops->restoresingleton) {
       ierr = (*draw->ops->restoresingleton)(draw,sdraw);CHKERRQ(ierr);
     } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Cannot restore singleton for this type %s of draw object",((PetscObject)draw)->type_name);
   }
   PetscFunctionReturn(0);
 }
-
-
-
-
-
-
-
