@@ -13,8 +13,6 @@
 */
 PetscFunctionList PetscDrawList = 0;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawView"
 /*@C
    PetscDrawView - Prints the PetscDraw data structure.
 
@@ -53,7 +51,9 @@ PetscErrorCode  PetscDrawView(PetscDraw indraw,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(indraw,PETSC_DRAW_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)indraw));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)indraw),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   PetscCheckSameComm(indraw,1,viewer,2);
 
@@ -69,8 +69,8 @@ PetscErrorCode  PetscDrawView(PetscDraw indraw,PetscViewer viewer)
 
     ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
     ierr = PetscDrawGetCurrentPoint(draw,&x,&y);CHKERRQ(ierr);
-    ierr   = PetscStrcpy(str,"PetscDraw: ");CHKERRQ(ierr);
-    ierr   = PetscStrcat(str,((PetscObject)indraw)->type_name);CHKERRQ(ierr);
+    ierr   = PetscStrncpy(str,"PetscDraw: ",sizeof(str));CHKERRQ(ierr);
+    ierr   = PetscStrlcat(str,((PetscObject)indraw)->type_name,sizeof(str));CHKERRQ(ierr);
     ierr   = PetscDrawStringBoxed(draw,x,y,PETSC_DRAW_RED,PETSC_DRAW_BLACK,str,NULL,&h);CHKERRQ(ierr);
     bottom = y - h;
     ierr = PetscDrawPushCurrentPoint(draw,x,bottom);CHKERRQ(ierr);
@@ -90,8 +90,6 @@ PetscErrorCode  PetscDrawView(PetscDraw indraw,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawCreate"
 /*@C
    PetscDrawCreate - Creates a graphics context.
 
@@ -113,13 +111,23 @@ PetscErrorCode  PetscDrawView(PetscDraw indraw,PetscViewer viewer)
    Concepts: graphics^creating context
    Concepts: drawing^creating context
 
-.seealso: PetscDrawSetFromOptions(), PetscDrawDestroy(), PetscDrawSetType()
+.seealso: PetscDrawSetType(), PetscDrawSetFromOptions(), PetscDrawDestroy(), PetscDrawSetType(), PetscDrawLGCreate(), PetscDrawSPCreate(),
+          PetscDrawViewPortsCreate(), PetscDrawViewPortsSet(), PetscDrawAxisCreate(), PetscDrawHGCreate(), PetscDrawBarCreate(),
+          PetscViewerDrawGetDraw(), PetscDrawSetFromOptions(), PetscDrawSetSave(), PetscDrawSetSaveMovie(), PetscDrawSetSaveFinalImage(),
+          PetscDrawOpenX(), PetscDrawOpenImage(), PetscDrawIsNull(), PetscDrawGetPopup(), PetscDrawCheckResizedWindow(), PetscDrawResizeWindow(),
+          PetscDrawGetWindowSize(), PetscDrawLine(), PetscDrawArrow(), PetscDrawLineSetWidth(), PetscDrawLineGetWidth(), PetscDrawMarker(),
+          PetscDrawPoint(), PetscDrawRectangle(), PetscDrawTriangle(), PetscDrawEllipse(), PetscDrawString(), PetscDrawStringCentered(),
+          PetscDrawStringBoxed(), PetscDrawStringBoxed(), PetscDrawStringVertical(), PetscDrawSetViewPort(), PetscDrawGetViewPort(),
+          PetscDrawSplitViewPort(), PetscDrawSetTitle(), PetscDrawAppendTitle(), PetscDrawGetTitle(), PetscDrawSetPause(), PetscDrawGetPause(),
+          PetscDrawPause(), PetscDrawSetDoubleBuffer(), PetscDrawClear(), PetscDrawFlush(), PetscDrawGetSingleton(), PetscDrawGetMouseButton(),
+          PetscDrawZoom(), PetscDrawGetBoundingBox()
+
 @*/
 PetscErrorCode  PetscDrawCreate(MPI_Comm comm,const char display[],const char title[],int x,int y,int w,int h,PetscDraw *indraw)
 {
   PetscDraw      draw;
   PetscErrorCode ierr;
-  PetscReal      dpause;
+  PetscReal      dpause = 0.0;
   PetscBool      flag;
 
   PetscFunctionBegin;
@@ -127,9 +135,9 @@ PetscErrorCode  PetscDrawCreate(MPI_Comm comm,const char display[],const char ti
   *indraw = 0;
   ierr = PetscHeaderCreate(draw,PETSC_DRAW_CLASSID,"Draw","Graphics","Draw",comm,PetscDrawDestroy,PetscDrawView);CHKERRQ(ierr);
 
-  draw->data    = 0;
-  ierr          = PetscStrallocpy(title,&draw->title);CHKERRQ(ierr);
+  draw->data    = NULL;
   ierr          = PetscStrallocpy(display,&draw->display);CHKERRQ(ierr);
+  ierr          = PetscStrallocpy(title,&draw->title);CHKERRQ(ierr);
   draw->x       = x;
   draw->y       = y;
   draw->w       = w;
@@ -143,13 +151,17 @@ PetscErrorCode  PetscDrawCreate(MPI_Comm comm,const char display[],const char ti
   draw->port_xr = 1.0;
   draw->port_yl = 0.0;
   draw->port_yr = 1.0;
-  draw->popup   = 0;
+  draw->popup   = NULL;
 
-  ierr = PetscOptionsGetReal(NULL,"-draw_pause",&dpause,&flag);CHKERRQ(ierr);
+  ierr = PetscOptionsGetReal(NULL,NULL,"-draw_pause",&dpause,&flag);CHKERRQ(ierr);
   if (flag) draw->pause = dpause;
-  draw->savefilename  = NULL;
-  draw->savefilemovie = PETSC_FALSE;
-  draw->savefilecount = -1;
+
+  draw->savefilename   = NULL;
+  draw->saveimageext   = NULL;
+  draw->savemovieext   = NULL;
+  draw->savefilecount  = 0;
+  draw->savesinglefile = PETSC_FALSE;
+  draw->savemoviefps   = PETSC_DECIDE;
 
   ierr = PetscDrawSetCurrentPoint(draw,.5,.9);CHKERRQ(ierr);
 
@@ -162,8 +174,6 @@ PetscErrorCode  PetscDrawCreate(MPI_Comm comm,const char display[],const char ti
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawSetType"
 /*@C
    PetscDrawSetType - Builds graphics object for a particular implementation
 
@@ -176,19 +186,19 @@ PetscErrorCode  PetscDrawCreate(MPI_Comm comm,const char display[],const char ti
    Options Database Command:
 .  -draw_type  <type> - Sets the type; use -help for a list of available methods (for instance, x)
 
-   See PetscDrawSetFromOptions for additional options database keys
+   See PetscDrawSetFromOptions() for additional options database keys
 
    Level: intermediate
 
    Notes:
    See "petsc/include/petscdraw.h" for available methods (for instance,
-   PETSC_DRAW_X)
+   PETSC_DRAW_X, PETSC_DRAW_TIKZ or PETSC_DRAW_IMAGE)
 
    Concepts: drawing^X windows
    Concepts: X windows^graphics
    Concepts: drawing^Microsoft Windows
 
-.seealso: PetscDrawSetFromOptions(), PetscDrawCreate(), PetscDrawDestroy()
+.seealso: PetscDrawSetFromOptions(), PetscDrawCreate(), PetscDrawDestroy(), PetscDrawType
 @*/
 PetscErrorCode  PetscDrawSetType(PetscDraw draw,PetscDrawType type)
 {
@@ -204,7 +214,7 @@ PetscErrorCode  PetscDrawSetType(PetscDraw draw,PetscDrawType type)
   if (match) PetscFunctionReturn(0);
 
   /*  User requests no graphics */
-  ierr = PetscOptionsHasName(NULL,"-nox",&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsHasName(((PetscObject)draw)->options,NULL,"-nox",&flg);CHKERRQ(ierr);
 
   /*
      This is not ideal, but it allows codes to continue to run if X graphics
@@ -217,24 +227,30 @@ PetscErrorCode  PetscDrawSetType(PetscDraw draw,PetscDrawType type)
     if (match) {
       PetscBool dontwarn = PETSC_TRUE;
       flg  = PETSC_TRUE;
-      ierr = PetscOptionsHasName(NULL,"-nox_warning",&dontwarn);CHKERRQ(ierr);
+      ierr = PetscOptionsHasName(NULL,NULL,"-nox_warning",&dontwarn);CHKERRQ(ierr);
       if (!dontwarn) (*PetscErrorPrintf)("PETSc installed without X windows on this machine\nproceeding without graphics\n");
     }
   }
 #endif
-  if (flg) type = PETSC_DRAW_NULL;
+  if (flg) {
+    ierr = PetscOptionsHasName(NULL,NULL,"-draw_double_buffer",NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsHasName(NULL,NULL,"-draw_virtual",NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsHasName(NULL,NULL,"-draw_fast",NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsHasName(NULL,NULL,"-draw_ports",NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsHasName(NULL,NULL,"-draw_coordinates",NULL);CHKERRQ(ierr);
+    ierr = PetscStrcmp(type,"tikz",&flg);CHKERRQ(ierr);
+    if (!flg) type = PETSC_DRAW_NULL;
+  }
 
   ierr =  PetscFunctionListFind(PetscDrawList,type,&r);CHKERRQ(ierr);
   if (!r) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_UNKNOWN_TYPE,"Unknown PetscDraw type given: %s",type);
   if (draw->ops->destroy) {ierr = (*draw->ops->destroy)(draw);CHKERRQ(ierr);}
   ierr = PetscMemzero(draw->ops,sizeof(struct _PetscDrawOps));CHKERRQ(ierr);
-  ierr       = PetscObjectChangeTypeName((PetscObject)draw,type);CHKERRQ(ierr);
-  ierr       = (*r)(draw);CHKERRQ(ierr);
+  ierr = PetscObjectChangeTypeName((PetscObject)draw,type);CHKERRQ(ierr);
+  ierr = (*r)(draw);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawGetType"
 /*@C
    PetscDrawGetType - Gets the PetscDraw type as a string from the PetscDraw object.
 
@@ -248,6 +264,8 @@ PetscErrorCode  PetscDrawSetType(PetscDraw draw,PetscDrawType type)
 
    Level: advanced
 
+.seealso: PetscDrawSetType(), PetscDrawType
+
 @*/
 PetscErrorCode  PetscDrawGetType(PetscDraw draw,PetscDrawType *type)
 {
@@ -258,8 +276,6 @@ PetscErrorCode  PetscDrawGetType(PetscDraw draw,PetscDrawType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawRegister"
 /*@C
    PetscDrawRegister - Adds a method to the graphics package.
 
@@ -287,19 +303,44 @@ $     -draw_type my_draw_type
    Concepts: graphics^registering new draw classes
    Concepts: PetscDraw^registering new draw classes
 
-.seealso: PetscDrawRegisterAll(), PetscDrawRegisterDestroy()
+.seealso: PetscDrawRegisterAll(), PetscDrawRegisterDestroy(), PetscDrawType, PetscDrawSetType()
 @*/
 PetscErrorCode  PetscDrawRegister(const char *sname,PetscErrorCode (*function)(PetscDraw))
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = PetscDrawInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&PetscDrawList,sname,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawSetFromOptions"
+/*@C
+   PetscDrawSetOptionsPrefix - Sets the prefix used for searching for all
+   PetscDraw options in the database.
+
+   Logically Collective on PetscDraw
+
+   Input Parameter:
++  draw - the draw context
+-  prefix - the prefix to prepend to all option names
+
+   Level: advanced
+
+.keywords: PetscDraw, set, options, prefix, database
+
+.seealso: PetscDrawSetFromOptions(), PetscDrawCreate()
+@*/
+PetscErrorCode  PetscDrawSetOptionsPrefix(PetscDraw draw,const char prefix[])
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
+  ierr = PetscObjectSetOptionsPrefix((PetscObject)draw,prefix);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    PetscDrawSetFromOptions - Sets the graphics type from the options database.
       Defaults to a PETSc X windows graphics.
@@ -317,18 +358,19 @@ PetscErrorCode  PetscDrawRegister(const char *sname,PetscErrorCode (*function)(P
 .   -draw_save [optional filename] - (X windows only) saves each image before it is cleared to a file
 .   -draw_save_final_image [optional filename] - (X windows only) saves the final image displayed in a window
 .   -draw_save_movie - converts image files to a movie  at the end of the run. See PetscDrawSetSave()
-.   -draw_save_on_flush - saves an image on each flush in addition to each clear
--   -draw_save_single_file - saves each new image in the same file, normally each new image is saved in a new file with filename_%d
+.   -draw_save_single_file - saves each new image in the same file, normally each new image is saved in a new file with 'filename/filename_%d.ext'
+.   -draw_save_on_clear - saves an image on each clear, mainly for debugging
+-   -draw_save_on_flush - saves an image on each flush, mainly for debugging
 
    Level: intermediate
 
    Notes:
     Must be called after PetscDrawCreate() before the PetscDraw is used.
 
-    Concepts: drawing^setting options
-    Concepts: graphics^setting options
+   Concepts: drawing^setting options
+   Concepts: graphics^setting options
 
-.seealso: PetscDrawCreate(), PetscDrawSetType(), PetscDrawSetSave(), PetscDrawSetSaveFinalImage()
+.seealso: PetscDrawCreate(), PetscDrawSetType(), PetscDrawSetSave(), PetscDrawSetSaveFinalImage(), PetscDrawPause(), PetscDrawSetPause()
 
 @*/
 PetscErrorCode  PetscDrawSetFromOptions(PetscDraw draw)
@@ -337,7 +379,6 @@ PetscErrorCode  PetscDrawSetFromOptions(PetscDraw draw)
   PetscBool         flg,nox;
   char              vtype[256];
   const char        *def;
-  PetscReal         dpause;
 #if !defined(PETSC_USE_WINDOWS_GRAPHICS) && !defined(PETSC_HAVE_X)
   PetscBool         warn;
 #endif
@@ -345,13 +386,11 @@ PetscErrorCode  PetscDrawSetFromOptions(PetscDraw draw)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
 
-  if (!PetscDrawList) {
-    ierr = PetscDrawRegisterAll();CHKERRQ(ierr);
-  }
+  ierr = PetscDrawRegisterAll();CHKERRQ(ierr);
 
   if (((PetscObject)draw)->type_name) def = ((PetscObject)draw)->type_name;
   else {
-    ierr = PetscOptionsHasName(NULL,"-nox",&nox);CHKERRQ(ierr);
+    ierr = PetscOptionsHasName(((PetscObject)draw)->options,NULL,"-nox",&nox);CHKERRQ(ierr);
     def  = PETSC_DRAW_NULL;
 #if defined(PETSC_USE_WINDOWS_GRAPHICS)
     if (!nox) def = PETSC_DRAW_WIN32;
@@ -362,7 +401,7 @@ PetscErrorCode  PetscDrawSetFromOptions(PetscDraw draw)
 #elif defined(PETSC_HAVE_OPENGLES)
     if (!nox) def = PETSC_DRAW_OPENGLES;
 #else
-    ierr = PetscOptionsHasName(NULL,"-nox_warning",&warn);CHKERRQ(ierr);
+    ierr = PetscOptionsHasName(NULL,NULL,"-nox_warning",&warn);CHKERRQ(ierr);
     if (!nox && !warn) (*PetscErrorPrintf)("PETSc installed without X windows, Microsoft Graphics, OpenGL ES, or GLUT/OpenGL on this machine\nproceeding without graphics\n");
 #endif
   }
@@ -374,152 +413,30 @@ PetscErrorCode  PetscDrawSetFromOptions(PetscDraw draw)
     ierr = PetscDrawSetType(draw,def);CHKERRQ(ierr);
   }
   ierr = PetscOptionsName("-nox","Run without graphics","None",&nox);CHKERRQ(ierr);
-#if defined(PETSC_HAVE_X)
   {
     char      filename[PETSC_MAX_PATH_LEN];
-    PetscBool save,movie = PETSC_FALSE;
-    ierr = PetscOptionsBool("-draw_save_movie","Make a movie from the images saved (X Windows only)","PetscDrawSetSave",movie,&movie,NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsBool("-draw_save_single_file","Each new image replaces previous image in file","PetscDrawSetSave",draw->savesinglefile,&draw->savesinglefile,NULL);CHKERRQ(ierr);
-    ierr = PetscOptionsString("-draw_save","Save graphics to file (X Windows only)","PetscDrawSetSave",filename,filename,PETSC_MAX_PATH_LEN,&save);CHKERRQ(ierr);
-    if (save) {
-      ierr = PetscDrawSetSave(draw,filename,movie);CHKERRQ(ierr);
-    }
-    ierr = PetscOptionsString("-draw_save_final_image","Save graphics to file (X Windows only)","PetscDrawSetSaveFinalImage",filename,filename,PETSC_MAX_PATH_LEN,&save);CHKERRQ(ierr);
-    if (save) {
-      ierr = PetscDrawSetSaveFinalImage(draw,filename);CHKERRQ(ierr);
-    }
-    ierr = PetscOptionsBool("-draw_save_on_flush","Save graphics to file (X Windows only) on each flush","PetscDrawSetSave",draw->saveonflush,&draw->saveonflush,NULL);CHKERRQ(ierr);
+    char      movieext[32];
+    PetscBool image,movie;
+    ierr = PetscSNPrintf(filename,sizeof(filename),"%s%s",draw->savefilename?draw->savefilename:"",draw->saveimageext?draw->saveimageext:"");CHKERRQ(ierr);
+    ierr = PetscSNPrintf(movieext,sizeof(movieext),"%s",draw->savemovieext?draw->savemovieext:"");CHKERRQ(ierr);
+    ierr = PetscOptionsString("-draw_save","Save graphics to image file","PetscDrawSetSave",filename,filename,sizeof(filename),&image);CHKERRQ(ierr);
+    ierr = PetscOptionsString("-draw_save_movie","Make a movie from saved images","PetscDrawSetSaveMovie",movieext,movieext,sizeof(movieext),&movie);CHKERRQ(ierr);
+    ierr = PetscOptionsInt("-draw_save_movie_fps","Set frames per second in saved movie",PETSC_FUNCTION_NAME,draw->savemoviefps,&draw->savemoviefps,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsBool("-draw_save_single_file","Each new image replaces previous image in file",PETSC_FUNCTION_NAME,draw->savesinglefile,&draw->savesinglefile,NULL);CHKERRQ(ierr);
+    if (image) {ierr = PetscDrawSetSave(draw,filename);CHKERRQ(ierr);}
+    if (movie) {ierr = PetscDrawSetSaveMovie(draw,movieext);CHKERRQ(ierr);}
+    ierr = PetscOptionsString("-draw_save_final_image","Save final graphics to image file","PetscDrawSetSaveFinalImage",filename,filename,sizeof(filename),&image);CHKERRQ(ierr);
+    if (image) {ierr = PetscDrawSetSaveFinalImage(draw,filename);CHKERRQ(ierr);}
+    ierr = PetscOptionsBool("-draw_save_on_clear","Save graphics to file on each clear",PETSC_FUNCTION_NAME,draw->saveonclear,&draw->saveonclear,NULL);CHKERRQ(ierr);
+    ierr = PetscOptionsBool("-draw_save_on_flush","Save graphics to file on each flush",PETSC_FUNCTION_NAME,draw->saveonflush,&draw->saveonflush,NULL);CHKERRQ(ierr);
   }
-#endif
-  ierr = PetscOptionsGetReal(NULL,"-draw_pause",&dpause,&flg);CHKERRQ(ierr);
-  if (flg) draw->pause = dpause;
-
+  ierr = PetscOptionsReal("-draw_pause","Amount of time that program pauses after plots","PetscDrawSetPause",draw->pause,&draw->pause,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsEnum("-draw_marker_type","Type of marker to use on plots","PetscDrawSetMarkerType",PetscDrawMarkerTypes,(PetscEnum)draw->markertype,(PetscEnum *)&draw->markertype,NULL);CHKERRQ(ierr);
 
   /* process any options handlers added with PetscObjectAddOptionsHandler() */
-  ierr = PetscObjectProcessOptionsHandlers((PetscObject)draw);CHKERRQ(ierr);
+  ierr = PetscObjectProcessOptionsHandlers(PetscOptionsObject,(PetscObject)draw);CHKERRQ(ierr);
 
   ierr = PetscDrawViewFromOptions(draw,NULL,"-draw_view");CHKERRQ(ierr);
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawSetSave"
-/*@C
-   PetscDrawSetSave - Saves images produced in a PetscDraw into a file as a Gif file using AfterImage
-
-   Collective on PetscDraw
-
-   Input Parameter:
-+  draw      - the graphics context
-.  filename  - name of the file, if .ext then uses name of draw object plus .ext using .ext to determine the image type, if NULL use .Gif image type
--  movie - produce a movie of all the images
-
-   Options Database Command:
-+  -draw_save  <filename> - filename could be name.ext or .ext (where .ext determines the type of graphics file to save, for example .Gif)
-.  -draw_save_movie
-.  -draw_save_final_image [optional filename] - (X windows only) saves the final image displayed in a window
-.  -draw_save_on_flush - saves an image on each flush in addition to each clear
--  -draw_save_single_file - saves each new image in the same file, normally each new image is saved in a new file with filename_%d
-
-   Level: intermediate
-
-   Concepts: X windows^graphics
-
-   Notes: You should call this BEFORE calling PetscDrawClear() and creating your image.
-
-   Requires that PETSc be configured with the option --with-afterimage to save the images and ffmpeg must be in your path to make the movie
-
-   The .ext formats that are supported depend on what formats AfterImage was configured with; on the Apple Mac both .Gif and .Jpeg are supported.
-
-   If X windows generates an error message about X_CreateWindow() failing then Afterimage was installed without X windows. Reinstall Afterimage using the
-   ./configure flags --x-includes=/pathtoXincludes --x-libraries=/pathtoXlibraries   For example under Mac OS X Mountain Lion --x-includes=/opt/X11/include -x-libraries=/opt/X11/lib
-
-
-.seealso: PetscDrawSetFromOptions(), PetscDrawCreate(), PetscDrawDestroy(), PetscDrawSetSaveFinalImage()
-@*/
-PetscErrorCode  PetscDrawSetSave(PetscDraw draw,const char *filename,PetscBool movie)
-{
-  PetscErrorCode ierr;
-  char           *ext;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
-  ierr = PetscFree(draw->savefilename);CHKERRQ(ierr);
-
-  /* determine extension of filename */
-  if (filename && filename[0]) {
-    ierr = PetscStrchr(filename,'.',&ext);CHKERRQ(ierr);
-    if (!ext) SETERRQ1(PetscObjectComm((PetscObject)draw),PETSC_ERR_ARG_INCOMP,"Filename %s should end with graphics extension (for example .Gif)",filename);
-  } else {
-    ext = (char *)".Gif";
-  }
-  if (ext == filename) filename = NULL;
-  ierr = PetscStrallocpy(ext,&draw->savefilenameext);CHKERRQ(ierr);
-
-  draw->savefilemovie = movie;
-  if (filename && filename[0]) {
-    size_t  l1,l2;
-    ierr = PetscStrlen(filename,&l1);CHKERRQ(ierr);
-    ierr = PetscStrlen(ext,&l2);CHKERRQ(ierr);
-    ierr = PetscMalloc1(l1-l2+1,&draw->savefilename);CHKERRQ(ierr);
-    ierr = PetscStrncpy(draw->savefilename,filename,l1-l2+1);CHKERRQ(ierr);
-  } else {
-    const char *name;
-
-    ierr = PetscObjectGetName((PetscObject)draw,&name);CHKERRQ(ierr);
-    ierr = PetscStrallocpy(name,&draw->savefilename);CHKERRQ(ierr);
-  }
-  ierr = PetscInfo2(NULL,"Will save images to file %s%s\n",draw->savefilename,draw->savefilenameext);CHKERRQ(ierr);
-  if (draw->ops->setsave) {
-    ierr = (*draw->ops->setsave)(draw,draw->savefilename);CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawSetSaveFinalImage"
-/*@C
-   PetscDrawSetSaveFinalImage - Saves the finale image produced in a PetscDraw into a file as a Gif file using AfterImage
-
-   Collective on PetscDraw
-
-   Input Parameter:
-+  draw      - the graphics context
--  filename  - name of the file, if NULL uses name of draw object
-
-   Options Database Command:
-.  -draw_save_final_image  <filename>
-
-   Level: intermediate
-
-   Concepts: X windows^graphics
-
-   Notes: You should call this BEFORE calling PetscDrawClear() and creating your image.
-
-   Requires that PETSc be configured with the option --with-afterimage to save the images and ffmpeg must be in your path to make the movie
-
-   If X windows generates an error message about X_CreateWindow() failing then Afterimage was installed without X windows. Reinstall Afterimage using the
-   ./configure flags --x-includes=/pathtoXincludes --x-libraries=/pathtoXlibraries   For example under Mac OS X Mountain Lion --x-includes=/opt/X11/include -x-libraries=/opt/X11/lib
-
-
-.seealso: PetscDrawSetFromOptions(), PetscDrawCreate(), PetscDrawDestroy(), PetscDrawSetSave()
-@*/
-PetscErrorCode  PetscDrawSetSaveFinalImage(PetscDraw draw,const char *filename)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
-  ierr = PetscFree(draw->savefinalfilename);CHKERRQ(ierr);
-
-  if (filename && filename[0]) {
-    ierr = PetscStrallocpy(filename,&draw->savefinalfilename);CHKERRQ(ierr);
-  } else {
-    const char *name;
-    ierr = PetscObjectGetName((PetscObject)draw,&name);CHKERRQ(ierr);
-    ierr = PetscStrallocpy(name,&draw->savefinalfilename);CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
-}
