@@ -4,6 +4,7 @@
 */
 #include <petsc/private/drawimpl.h>  /*I "petscdraw.h" I*/
 
+PETSC_EXTERN PetscErrorCode PetscDrawCreate_Image(PetscDraw);
 PETSC_EXTERN PetscErrorCode PetscDrawCreate_TikZ(PetscDraw);
 #if defined(PETSC_HAVE_X)
 PETSC_EXTERN PetscErrorCode PetscDrawCreate_X(PetscDraw);
@@ -19,10 +20,8 @@ PETSC_EXTERN PetscErrorCode PetscDrawCreate_Null(PetscDraw);
 PETSC_EXTERN PetscErrorCode PetscDrawCreate_Win32(PetscDraw);
 #endif
 
-PetscBool PetscDrawRegisterAllCalled;
+PetscBool PetscDrawRegisterAllCalled = PETSC_FALSE;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawRegisterAll"
 /*@C
   PetscDrawRegisterAll - Registers all of the graphics methods in the PetscDraw package.
 
@@ -40,6 +39,7 @@ PetscErrorCode  PetscDrawRegisterAll(void)
   if (PetscDrawRegisterAllCalled) PetscFunctionReturn(0);
   PetscDrawRegisterAllCalled = PETSC_TRUE;
 
+  ierr = PetscDrawRegister(PETSC_DRAW_IMAGE,    PetscDrawCreate_Image);CHKERRQ(ierr);
   ierr = PetscDrawRegister(PETSC_DRAW_TIKZ,     PetscDrawCreate_TikZ);CHKERRQ(ierr);
 #if defined(PETSC_HAVE_OPENGLES)
   ierr = PetscDrawRegister(PETSC_DRAW_OPENGLES, PetscDrawCreate_OpenGLES);CHKERRQ(ierr);
