@@ -1,11 +1,82 @@
-
 /*
        Provides the calling sequences for all the basic PetscDraw routines.
 */
 #include <petsc/private/drawimpl.h>  /*I "petscdraw.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawStringCentered"
+/*@C
+   PetscDrawString - PetscDraws text onto a drawable.
+
+   Not Collective
+
+   Input Parameters:
++  draw - the drawing context
+.  xl,yl - the coordinates of lower left corner of text
+.  cl - the color of the text
+-  text - the text to draw
+
+   Level: beginner
+
+   Concepts: drawing^string
+   Concepts: string^drawing
+
+.seealso: PetscDrawStringVertical(), PetscDrawStringCentered(), PetscDrawStringBoxed(), PetscDrawStringBoxed(), PetscDrawStringSetSize(),
+          PetscDrawStringGetSize(), PetscDrawLine(), PetscDrawRectangle(), PetscDrawTriangle(), PetscDrawEllipse(),
+          PetscDrawMarker(), PetscDrawPoint()
+
+@*/
+PetscErrorCode  PetscDrawString(PetscDraw draw,PetscReal xl,PetscReal yl,int cl,const char text[])
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
+  PetscValidCharPointer(text,5);
+  if (!draw->ops->string) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"This draw type %s does not support drawing strings",((PetscObject)draw)->type_name);
+  ierr = (*draw->ops->string)(draw,xl,yl,cl,text);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   PetscDrawStringVertical - PetscDraws text onto a drawable.
+
+   Not Collective
+
+   Input Parameters:
++  draw - the drawing context
+.  xl,yl - the coordinates of upper left corner of text
+.  cl - the color of the text
+-  text - the text to draw
+
+   Level: beginner
+
+   Concepts: string^drawing vertical
+
+.seealso: PetscDrawString(), PetscDrawStringCentered(), PetscDrawStringBoxed(), PetscDrawStringBoxed(), PetscDrawStringSetSize(),
+          PetscDrawStringGetSize()
+
+@*/
+PetscErrorCode  PetscDrawStringVertical(PetscDraw draw,PetscReal xl,PetscReal yl,int cl,const char text[])
+{
+  int            i;
+  char           chr[2] = {0, 0};
+  PetscReal      tw,th;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
+  PetscValidCharPointer(text,5);
+
+  if (draw->ops->stringvertical) {
+    ierr = (*draw->ops->stringvertical)(draw,xl,yl,cl,text);CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  }
+  ierr = PetscDrawStringGetSize(draw,&tw,&th);CHKERRQ(ierr);
+  for (i = 0; (chr[0] = text[i]); i++) {
+    ierr = PetscDrawString(draw,xl,yl-th*(i+1),cl,chr);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 /*@C
    PetscDrawStringCentered - PetscDraws text onto a drawable centered at a point
 
@@ -23,68 +94,27 @@
    Concepts: drawing^string
    Concepts: string^drawing
 
-.seealso: PetscDrawStringVertical(), PetscDrawString(), PetscDrawStringBoxed()
+.seealso: PetscDrawStringVertical(), PetscDrawString(), PetscDrawStringBoxed(), PetscDrawStringBoxed(), PetscDrawStringSetSize(),
+          PetscDrawStringGetSize()
 
 @*/
 PetscErrorCode  PetscDrawStringCentered(PetscDraw draw,PetscReal xc,PetscReal yl,int cl,const char text[])
 {
   PetscErrorCode ierr;
-  PetscBool      isnull;
   size_t         len;
   PetscReal      tw,th;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
   PetscValidCharPointer(text,5);
-  ierr = PetscObjectTypeCompare((PetscObject)draw,PETSC_DRAW_NULL,&isnull);CHKERRQ(ierr);
-  if (isnull) PetscFunctionReturn(0);
 
   ierr = PetscDrawStringGetSize(draw,&tw,&th);CHKERRQ(ierr);
-  ierr =  PetscStrlen(text,&len);CHKERRQ(ierr);
-  xc   = xc - .5*len*tw;
+  ierr = PetscStrlen(text,&len);CHKERRQ(ierr);
+  xc   = xc - len*tw/2;
   ierr = PetscDrawString(draw,xc,yl,cl,text);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawString"
-/*@C
-   PetscDrawString - PetscDraws text onto a drawable.
-
-   Not Collective
-
-   Input Parameters:
-+  draw - the drawing context
-.  xl - the coordinates of lower left corner of text
-.  yl - the coordinates of lower left corner of text
-.  cl - the color of the text
--  text - the text to draw
-
-   Level: beginner
-
-   Concepts: drawing^string
-   Concepts: string^drawing
-
-.seealso: PetscDrawStringVertical(), PetscDrawStringCentered(), PetscDrawStringBoxed()
-
-@*/
-PetscErrorCode  PetscDrawString(PetscDraw draw,PetscReal xl,PetscReal yl,int cl,const char text[])
-{
-  PetscErrorCode ierr;
-  PetscBool      isnull;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
-  PetscValidCharPointer(text,5);
-  ierr = PetscObjectTypeCompare((PetscObject)draw,PETSC_DRAW_NULL,&isnull);CHKERRQ(ierr);
-  if (isnull) PetscFunctionReturn(0);
-  ierr = (*draw->ops->string)(draw,xl,yl,cl,text);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawStringBoxed"
 /*@C
    PetscDrawStringBoxed - Draws a string with a box around it
 
@@ -106,13 +136,13 @@ PetscErrorCode  PetscDrawString(PetscDraw draw,PetscReal xl,PetscReal yl,int cl,
    Concepts: drawing^string
    Concepts: string^drawing
 
-.seealso: PetscDrawStringVertical(), PetscDrawStringBoxedSize(), PetscDrawString(), PetscDrawStringCentered()
+.seealso: PetscDrawStringVertical(), PetscDrawString(), PetscDrawStringCentered(), PetscDrawStringBoxed(), PetscDrawStringSetSize(),
+          PetscDrawStringGetSize()
 
 @*/
 PetscErrorCode  PetscDrawStringBoxed(PetscDraw draw,PetscReal sxl,PetscReal syl,int sc,int bc,const char text[],PetscReal *w,PetscReal *h)
 {
   PetscErrorCode ierr;
-  PetscBool      isnull;
   PetscReal      top,left,right,bottom,tw,th;
   size_t         len,mlen = 0;
   char           **array;
@@ -121,8 +151,6 @@ PetscErrorCode  PetscDrawStringBoxed(PetscDraw draw,PetscReal sxl,PetscReal syl,
   PetscFunctionBegin;
   PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
   PetscValidCharPointer(text,5);
-  ierr = PetscObjectTypeCompare((PetscObject)draw,PETSC_DRAW_NULL,&isnull);CHKERRQ(ierr);
-  if (isnull) PetscFunctionReturn(0);
 
   if (draw->ops->boxedstring) {
     ierr = (*draw->ops->boxedstring)(draw,sxl,syl,sc,bc,text,w,h);CHKERRQ(ierr);
@@ -162,3 +190,67 @@ PetscErrorCode  PetscDrawStringBoxed(PetscDraw draw,PetscReal sxl,PetscReal syl,
   ierr = PetscStrToArrayDestroy(cnt,array);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
+/*@
+   PetscDrawStringSetSize - Sets the size for character text.
+
+   Not Collective
+
+   Input Parameters:
++  draw - the drawing context
+.  width - the width in user coordinates
+-  height - the character height in user coordinates
+
+   Level: advanced
+
+   Note:
+   Only a limited range of sizes are available.
+
+   Concepts: string^drawing size
+
+.seealso: PetscDrawStringVertical(), PetscDrawString(), PetscDrawStringCentered(), PetscDrawStringBoxed(), PetscDrawStringBoxed(),
+          PetscDrawStringGetSize()
+
+@*/
+PetscErrorCode  PetscDrawStringSetSize(PetscDraw draw,PetscReal width,PetscReal height)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
+  if (draw->ops->stringsetsize) {
+    ierr = (*draw->ops->stringsetsize)(draw,width,height);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+/*@
+   PetscDrawStringGetSize - Gets the size for character text.  The width is
+   relative to the user coordinates of the window.
+
+   Not Collective
+
+   Input Parameters:
++  draw - the drawing context
+.  width - the width in user coordinates
+-  height - the character height
+
+   Level: advanced
+
+   Concepts: string^drawing size
+
+.seealso: PetscDrawStringVertical(), PetscDrawString(), PetscDrawStringCentered(), PetscDrawStringBoxed(), PetscDrawStringBoxed(),
+          PetscDrawStringSetSize()
+
+@*/
+PetscErrorCode  PetscDrawStringGetSize(PetscDraw draw,PetscReal *width,PetscReal *height)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
+  if (!draw->ops->stringgetsize) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"This draw type %s does not support getting string size",((PetscObject)draw)->type_name);
+  ierr = (*draw->ops->stringgetsize)(draw,width,height);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
