@@ -4,8 +4,6 @@
 */
 #include <petsc/private/drawimpl.h>  /*I "petscdraw.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawTriangle"
 /*@
    PetscDrawTriangle - PetscDraws a triangle  onto a drawable.
 
@@ -22,70 +20,75 @@
    Concepts: graphics^triangle
    Concepts: triangle
 
+.seealso: PetscDrawLine(), PetscDrawRectangle(), PetscDrawEllipse(), PetscDrawMarker(), PetscDrawPoint(), PetscDrawArrow()
 @*/
 PetscErrorCode  PetscDrawTriangle(PetscDraw draw,PetscReal x1,PetscReal y_1,PetscReal x2,PetscReal y2,PetscReal x3,PetscReal y3,int c1,int c2,int c3)
 {
   PetscErrorCode ierr;
-  PetscBool      isnull;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
-  ierr = PetscObjectTypeCompare((PetscObject)draw,PETSC_DRAW_NULL,&isnull);CHKERRQ(ierr);
-  if (isnull) PetscFunctionReturn(0);
-  if (!draw->ops->triangle) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"No triangles with this PetscDrawType");
+  if (!draw->ops->triangle) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"This draw type %s does not support drawing triangles",((PetscObject)draw)->type_name);
   ierr = (*draw->ops->triangle)(draw,x1,y_1,x2,y2,x3,y3,c1,c2,c3);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawScalePopup"
 /*@
-       PetscDrawScalePopup - PetscDraws a contour scale window.
+   PetscDrawScalePopup - draws a contour scale window.
 
-     Collective on PetscDraw
+   Collective on PetscDraw
 
-  Input Parameters:
-+    popup - the window (often a window obtained via PetscDrawGetPopup()
-.    min - minimum value being plotted
--    max - maximum value being plotted
+   Input Parameters:
++  popup - the window (often a window obtained via PetscDrawGetPopup()
+.  min - minimum value being plotted
+-  max - maximum value being plotted
 
-  Level: intermediate
+   Level: intermediate
 
-  Notes:
-     All processors that share the draw MUST call this routine
+   Notes:
+    All processors that share the draw MUST call this routine
+
+.seealso: PetscDrawGetPopup(), PetscDrawTensorContour()
 
 @*/
 PetscErrorCode  PetscDrawScalePopup(PetscDraw popup,PetscReal min,PetscReal max)
 {
-  PetscReal      xl = 0.0,yl = 0.0,xr = 2.0,yr = 1.0,value;
+  PetscBool      isnull;
+  PetscReal      xl = 0.0,yl = 0.0,xr = 1.0,yr = 1.0;
+  PetscMPIInt    rank;
   PetscErrorCode ierr;
-  int            i,c = PETSC_DRAW_BASIC_COLORS,rank;
+  int            i;
   char           string[32];
-  MPI_Comm       comm;
 
   PetscFunctionBegin;
+  if (!popup) PetscFunctionReturn(0);
+  PetscValidHeaderSpecific(popup,PETSC_DRAW_CLASSID,1);
+  ierr = PetscDrawIsNull(popup,&isnull);CHKERRQ(ierr);
+  if (isnull) PetscFunctionReturn(0);
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)popup),&rank);CHKERRQ(ierr);
+
   ierr = PetscDrawCheckResizedWindow(popup);CHKERRQ(ierr);
-  ierr = PetscDrawSynchronizedClear(popup);CHKERRQ(ierr);
-  ierr = PetscObjectGetComm((PetscObject)popup,&comm);CHKERRQ(ierr);
-  ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
-  if (rank) PetscFunctionReturn(0);
-
-  for (i=0; i<10; i++) {
-    ierr = PetscDrawRectangle(popup,xl,yl,xr,yr,c,c,c,c);CHKERRQ(ierr);
-
-    yl += .1; yr += .1;
-    c = (int)((double)c + (245. - PETSC_DRAW_BASIC_COLORS)/9.);
-  }
-  for (i=0; i<10; i++) {
-    value = min + i*(max-min)/9.0;
-    /* look for a value that should be zero, but is not due to round-off */
-    if (PetscAbsReal(value) < 1.e-10 && max-min > 1.e-6) value = 0.0;
-    sprintf(string,"%18.16e",(double)value);
-    ierr = PetscDrawString(popup,.2,.02 + i/10.0,PETSC_DRAW_BLACK,string);CHKERRQ(ierr);
-  }
+  ierr = PetscDrawClear(popup);CHKERRQ(ierr);
   ierr = PetscDrawSetTitle(popup,"Contour Scale");CHKERRQ(ierr);
+  ierr = PetscDrawSetCoordinates(popup,xl,yl,xr,yr);CHKERRQ(ierr);
+  ierr = PetscDrawCollectiveBegin(popup);CHKERRQ(ierr);
+  if (!rank) {
+    for (i=0; i<10; i++) {
+      int c = PetscDrawRealToColor((PetscReal)i/9,0,1);
+      ierr = PetscDrawRectangle(popup,xl,yl,xr,yr,c,c,c,c);CHKERRQ(ierr);
+      yl += 0.1;
+    }
+    for (i=0; i<10; i++) {
+      PetscReal value = min + i*(max-min)/9;
+      /* look for a value that should be zero, but is not due to round-off */
+      if (PetscAbsReal(value) < 1.e-10 && max-min > 1.e-6) value = 0.0;
+      ierr = PetscSNPrintf(string,sizeof(string),"%18.16e",(double)value);CHKERRQ(ierr);
+      ierr = PetscDrawString(popup,0.2,0.02+i/10.0,PETSC_DRAW_BLACK,string);CHKERRQ(ierr);
+    }
+  }
+  ierr = PetscDrawCollectiveEnd(popup);CHKERRQ(ierr);
   ierr = PetscDrawFlush(popup);CHKERRQ(ierr);
+  ierr = PetscDrawSave(popup);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -95,8 +98,6 @@ typedef struct {
   PetscBool showgrid;
 } ZoomCtx;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawTensorContour_Zoom"
 static PetscErrorCode PetscDrawTensorContour_Zoom(PetscDraw win,void *dctx)
 {
   PetscErrorCode ierr;
@@ -104,7 +105,7 @@ static PetscErrorCode PetscDrawTensorContour_Zoom(PetscDraw win,void *dctx)
   ZoomCtx        *ctx = (ZoomCtx*)dctx;
 
   PetscFunctionBegin;
-  ierr = PetscDrawTensorContourPatch(win,ctx->m,ctx->n,ctx->x,ctx->y,ctx->max,ctx->min,ctx->v);CHKERRQ(ierr);
+  ierr = PetscDrawTensorContourPatch(win,ctx->m,ctx->n,ctx->x,ctx->y,ctx->min,ctx->max,ctx->v);CHKERRQ(ierr);
   if (ctx->showgrid) {
     for (i=0; i<ctx->m; i++) {
       ierr = PetscDrawLine(win,ctx->x[i],ctx->y[0],ctx->x[i],ctx->y[ctx->n-1],PETSC_DRAW_BLACK);CHKERRQ(ierr);
@@ -116,8 +117,6 @@ static PetscErrorCode PetscDrawTensorContour_Zoom(PetscDraw win,void *dctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawTensorContour"
 /*@C
    PetscDrawTensorContour - PetscDraws a contour plot for a two-dimensional array
    that is stored as a PETSc vector.
@@ -125,7 +124,7 @@ static PetscErrorCode PetscDrawTensorContour_Zoom(PetscDraw win,void *dctx)
    Collective on PetscDraw, but PetscDraw must be sequential
 
    Input Parameters:
-+   win   - the window to draw in
++   draw  - the draw context
 .   m,n   - the global number of mesh points in the x and y directions
 .   xi,yi - the locations of the global mesh points (optional, use NULL
             to indicate uniform spacing on [0,1])
@@ -140,32 +139,28 @@ static PetscErrorCode PetscDrawTensorContour_Zoom(PetscDraw win,void *dctx)
    Concepts: contour plot
    Concepts: drawing^contour plot
 
-.seealso: PetscDrawTensorContourPatch()
+.seealso: PetscDrawTensorContourPatch(), PetscDrawScalePopup()
 
 @*/
-PetscErrorCode  PetscDrawTensorContour(PetscDraw win,int m,int n,const PetscReal xi[],const PetscReal yi[],PetscReal *v)
+PetscErrorCode  PetscDrawTensorContour(PetscDraw draw,int m,int n,const PetscReal xi[],const PetscReal yi[],PetscReal *v)
 {
   PetscErrorCode ierr;
   int            N = m*n;
   PetscBool      isnull;
   PetscDraw      popup;
-  MPI_Comm       comm;
   int            xin=1,yin=1,i;
   PetscMPIInt    size;
   PetscReal      h;
   ZoomCtx        ctx;
 
   PetscFunctionBegin;
-  ierr = PetscDrawIsNull(win,&isnull);CHKERRQ(ierr); if (isnull) PetscFunctionReturn(0);
-  ierr = PetscObjectGetComm((PetscObject)win,&comm);CHKERRQ(ierr);
-  ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
+  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
+  ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr);
+  if (isnull) PetscFunctionReturn(0);
+  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)draw),&size);CHKERRQ(ierr);
   if (size > 1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"May only be used with single processor PetscDraw");
   if (N <= 0) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"n %d and m %d must be positive",m,n);
 
-  /* create scale window */
-  ierr = PetscDrawGetPopup(win,&popup);CHKERRQ(ierr);
-  ierr = PetscDrawCheckResizedWindow(win);CHKERRQ(ierr);
-
   ctx.v   = v;
   ctx.m   = m;
   ctx.n   = n;
@@ -177,10 +172,11 @@ PetscErrorCode  PetscDrawTensorContour(PetscDraw win,int m,int n,const PetscReal
   if (ctx.max - ctx.min < 1.e-7) {ctx.min -= 5.e-8; ctx.max += 5.e-8;}
 
   /* PetscDraw the scale window */
-  if (popup) {ierr = PetscDrawScalePopup(popup,ctx.min,ctx.max);CHKERRQ(ierr);}
+  ierr = PetscDrawGetPopup(draw,&popup);CHKERRQ(ierr);
+  ierr = PetscDrawScalePopup(popup,ctx.min,ctx.max);CHKERRQ(ierr);
 
   ctx.showgrid = PETSC_FALSE;
-  ierr         = PetscOptionsGetBool(NULL,"-draw_contour_grid",&ctx.showgrid,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(((PetscObject)draw)->options,NULL,"-draw_contour_grid",&ctx.showgrid,NULL);CHKERRQ(ierr);
 
   /* fill up x and y coordinates */
   if (!xi) {
@@ -199,15 +195,13 @@ PetscErrorCode  PetscDrawTensorContour(PetscDraw win,int m,int n,const PetscReal
     for (i=1; i<ctx.n; i++) ctx.y[i] = ctx.y[i-1] + h;
   } else ctx.y = (PetscReal*)yi;
 
-  ierr = PetscDrawZoom(win,(PetscErrorCode (*)(PetscDraw,void*))PetscDrawTensorContour_Zoom,&ctx);CHKERRQ(ierr);
+  ierr = PetscDrawZoom(draw,PetscDrawTensorContour_Zoom,&ctx);CHKERRQ(ierr);
 
   if (!xin) {ierr = PetscFree(ctx.x);CHKERRQ(ierr);}
   if (!yin) {ierr = PetscFree(ctx.y);CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawTensorContourPatch"
 /*@
    PetscDrawTensorContourPatch - PetscDraws a rectangular patch of a contour plot
    for a two-dimensional array.
@@ -215,10 +209,10 @@ PetscErrorCode  PetscDrawTensorContour(PetscDraw win,int m,int n,const PetscReal
    Not Collective
 
    Input Parameters:
-+  win - the window to draw in
++  draw - the draw context
 .  m,n - the number of local mesh points in the x and y direction
 .  x,y - the locations of the local mesh points
-.  max,min - the maximum and minimum value in the entire contour
+.  min,max - the minimum and maximum value in the entire contour
 -  v - the data
 
    Options Database Keys:
@@ -235,22 +229,21 @@ PetscErrorCode  PetscDrawTensorContour(PetscDraw win,int m,int n,const PetscReal
 .seealso: PetscDrawTensorContour()
 
 @*/
-PetscErrorCode  PetscDrawTensorContourPatch(PetscDraw draw,int m,int n,PetscReal *x,PetscReal *y,PetscReal max,PetscReal min,PetscReal *v)
+PetscErrorCode  PetscDrawTensorContourPatch(PetscDraw draw,int m,int n,PetscReal *x,PetscReal *y,PetscReal min,PetscReal max,PetscReal *v)
 {
   PetscErrorCode ierr;
   int            c1,c2,c3,c4,i,j;
-  PetscReal      x1,x2,x3,x4,y_1,y2,y3,y4,scale;
+  PetscReal      x1,x2,x3,x4,y_1,y2,y3,y4;
 
   PetscFunctionBegin;
-  scale = (245.0 - PETSC_DRAW_BASIC_COLORS)/(max - min);
-
+  PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
   /* PetscDraw the contour plot patch */
   for (j=0; j<n-1; j++) {
     for (i=0; i<m-1; i++) {
-      x1 = x[i];  y_1 = y[j]; c1 = (int)(PETSC_DRAW_BASIC_COLORS + scale*(v[i+j*m] - min));
-      x2 = x[i+1];y2 = y_1;   c2 = (int)(PETSC_DRAW_BASIC_COLORS + scale*(v[i+j*m+1]-min));
-      x3 = x2;    y3 = y[j+1];c3 = (int)(PETSC_DRAW_BASIC_COLORS + scale*(v[i+j*m+1+m]-min));
-      x4 = x1;    y4 = y3;    c4 = (int)(PETSC_DRAW_BASIC_COLORS + scale*(v[i+j*m+m]-min));
+      x1 = x[i];   y_1 = y[j];  c1 = PetscDrawRealToColor(v[i+j*m],min,max);
+      x2 = x[i+1]; y2 = y_1;    c2 = PetscDrawRealToColor(v[i+j*m+1],min,max);
+      x3 = x2;     y3 = y[j+1]; c3 = PetscDrawRealToColor(v[i+j*m+1+m],min,max);
+      x4 = x1;     y4 = y3;     c4 = PetscDrawRealToColor(v[i+j*m+m],min,max);
 
       ierr = PetscDrawTriangle(draw,x1,y_1,x2,y2,x3,y3,c1,c2,c3);CHKERRQ(ierr);
       ierr = PetscDrawTriangle(draw,x1,y_1,x3,y3,x4,y4,c1,c3,c4);CHKERRQ(ierr);
