@@ -4,8 +4,6 @@
 */
 #include <petsc/private/drawimpl.h>  /*I "petscdraw.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawSetViewPort"
 /*@
    PetscDrawSetViewPort - Sets the portion of the window (page) to which draw
    routines will write.
@@ -39,8 +37,6 @@ PetscErrorCode  PetscDrawSetViewPort(PetscDraw draw,PetscReal xl,PetscReal yl,Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawGetViewPort"
 /*@
    PetscDrawGetViewPort - Gets the portion of the window (page) to which draw
    routines will write.
@@ -76,8 +72,6 @@ PetscErrorCode  PetscDrawGetViewPort(PetscDraw draw,PetscReal *xl,PetscReal *yl,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawSplitViewPort"
 /*@
    PetscDrawSplitViewPort - Splits a window shared by several processes into smaller
    view ports. One for each process.
@@ -137,11 +131,8 @@ PetscErrorCode  PetscDrawSplitViewPort(PetscDraw draw)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawViewPortsCreate"
 /*@C
-   PetscDrawViewPortsCreate - Splits a window into smaller
-       view ports. Each processor shares all the viewports.
+   PetscDrawViewPortsCreate - Splits a window into smaller view ports. Each processor shares all the viewports.
 
    Collective on PetscDraw
 
@@ -152,6 +143,9 @@ PetscErrorCode  PetscDrawSplitViewPort(PetscDraw draw)
    Output Parameter:
 .  ports - a PetscDrawViewPorts context (C structure)
 
+   Options Database:
+.  -draw_ports - display multiple fields in the same window with PetscDrawPorts instead of in seperate windows
+
    Level: advanced
 
    Concepts: drawing^in subset of window
@@ -187,10 +181,11 @@ PetscErrorCode  PetscDrawViewPortsCreate(PetscDraw draw,PetscInt nports,PetscDra
   while (n*n < nports) n++;
   h = 1.0/n;
 
-  ierr = PetscMalloc1(n*n,&xl);CHKERRQ(ierr); ports->xl = xl;
-  ierr = PetscMalloc1(n*n,&xr);CHKERRQ(ierr); ports->xr = xr;
-  ierr = PetscMalloc1(n*n,&yl);CHKERRQ(ierr); ports->yl = yl;
-  ierr = PetscMalloc1(n*n,&yr);CHKERRQ(ierr); ports->yr = yr;
+  ierr = PetscMalloc4(n*n,&xl,n*n,&xr,n*n,&yl,n*n,&yr);CHKERRQ(ierr);
+  ports->xl = xl;
+  ports->xr = xr;
+  ports->yl = yl;
+  ports->yr = yr;
 
   ierr = PetscDrawSetCoordinates(draw,0.0,0.0,1.0,1.0);CHKERRQ(ierr);
   ierr = PetscDrawCollectiveBegin(draw);CHKERRQ(ierr);
@@ -217,8 +212,6 @@ PetscErrorCode  PetscDrawViewPortsCreate(PetscDraw draw,PetscInt nports,PetscDra
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawViewPortsCreateRect"
 /*@C
    PetscDrawViewPortsCreateRect - Splits a window into smaller
        view ports. Each processor shares all the viewports. The number
@@ -268,10 +261,11 @@ PetscErrorCode  PetscDrawViewPortsCreateRect(PetscDraw draw,PetscInt nx,PetscInt
   /* save previous drawport of window */
   ierr = PetscDrawGetViewPort(draw,&ports->port_xl,&ports->port_yl,&ports->port_xr,&ports->port_yr);CHKERRQ(ierr);
 
-  ierr = PetscMalloc1(n,&xl);CHKERRQ(ierr); ports->xl = xl;
-  ierr = PetscMalloc1(n,&xr);CHKERRQ(ierr); ports->xr = xr;
-  ierr = PetscMalloc1(n,&yl);CHKERRQ(ierr); ports->yl = yl;
-  ierr = PetscMalloc1(n,&yr);CHKERRQ(ierr); ports->yr = yr;
+  ierr = PetscMalloc4(n,&xl,n,&xr,n,&yl,n,&yr);CHKERRQ(ierr);
+  ports->xr = xr;
+  ports->xl = xl;
+  ports->yl = yl;
+  ports->yr = yr;
 
   ierr = PetscDrawSetCoordinates(draw,0.0,0.0,1.0,1.0);CHKERRQ(ierr);
   ierr = PetscDrawCollectiveBegin(draw);CHKERRQ(ierr);
@@ -302,8 +296,6 @@ PetscErrorCode  PetscDrawViewPortsCreateRect(PetscDraw draw,PetscInt nx,PetscInt
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawViewPortsDestroy"
 /*@C
    PetscDrawViewPortsDestroy - frees a PetscDrawViewPorts object
 
@@ -327,16 +319,11 @@ PetscErrorCode  PetscDrawViewPortsDestroy(PetscDrawViewPorts *ports)
   /* reset Drawport of Window back to previous value */
   ierr = PetscDrawSetViewPort(ports->draw,ports->port_xl,ports->port_yl,ports->port_xr,ports->port_yr);CHKERRQ(ierr);
   ierr = PetscDrawDestroy(&ports->draw);CHKERRQ(ierr);
-  ierr = PetscFree(ports->xl);CHKERRQ(ierr);
-  ierr = PetscFree(ports->xr);CHKERRQ(ierr);
-  ierr = PetscFree(ports->yl);CHKERRQ(ierr);
-  ierr = PetscFree(ports->yr);CHKERRQ(ierr);
+  ierr = PetscFree4(ports->xl,ports->xr,ports->yl,ports->yr);CHKERRQ(ierr);
   ierr = PetscFree(ports);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawViewPortsSet"
 /*@C
    PetscDrawViewPortsSet - sets a draw object to use a particular subport
 
