@@ -2,12 +2,10 @@
 
 PetscClassId PETSC_DRAWAXIS_CLASSID = 0;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawAxisCreate"
 /*@
    PetscDrawAxisCreate - Generate the axis data structure.
 
-   Collective over PetscDraw
+   Collective on PetscDraw
 
    Input Parameters:
 .  win - PetscDraw object where axis to to be made
@@ -15,57 +13,57 @@ PetscClassId PETSC_DRAWAXIS_CLASSID = 0;
    Ouput Parameters:
 .  axis - the axis datastructure
 
+   Notes:
+    the MPI communicator that owns the underlying draw object owns the PetscDrawAxis object, but calls to set PetscDrawAxis options are ignored by all processes
+          except the first MPI process in the communicator
+
    Level: advanced
 
+.seealso: PetscDrawLGCreate(), PetscDrawLG, PetscDrawSPCreate(), PetscDrawSP, PetscDrawHGCreate(), PetscDrawHG, PetscDrawBarCreate(), PetscDrawBar, PetscDrawLGGetAxis(), PetscDrawSPGetAxis(),
+          PetscDrawHGGetAxis(), PetscDrawBarGetAxis(), PetscDrawAxis, PetscDrawAxisDestroy(), PetscDrawAxisSetColors(), PetscDrawAxisSetLabels(), PetscDrawAxisSetLimits(), PetscDrawAxisGetLimits(), PetscDrawAxisSetHoldLimits(),
+          PetscDrawAxisDraw()
 @*/
 PetscErrorCode  PetscDrawAxisCreate(PetscDraw draw,PetscDrawAxis *axis)
 {
   PetscDrawAxis  ad;
-  PetscObject    obj = (PetscObject)draw;
   PetscErrorCode ierr;
-  PetscBool      isnull;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(draw,PETSC_DRAW_CLASSID,1);
   PetscValidPointer(axis,2);
-  ierr = PetscObjectTypeCompare(obj,PETSC_DRAW_NULL,&isnull);CHKERRQ(ierr);
-  if (isnull) {
-    ierr = PetscDrawOpenNull(PetscObjectComm((PetscObject)obj),(PetscDraw*)axis);CHKERRQ(ierr);
 
-    (*axis)->win = draw;
-    PetscFunctionReturn(0);
-  }
-  ierr = PetscHeaderCreate(ad,PETSC_DRAWAXIS_CLASSID,"PetscDrawAxis","Draw Axis","Draw",PetscObjectComm((PetscObject)draw),PetscDrawAxisDestroy,NULL);CHKERRQ(ierr);
+  ierr = PetscHeaderCreate(ad,PETSC_DRAWAXIS_CLASSID,"DrawAxis","Draw Axis","Draw",PetscObjectComm((PetscObject)draw),PetscDrawAxisDestroy,NULL);CHKERRQ(ierr);
   ierr = PetscLogObjectParent((PetscObject)draw,(PetscObject)ad);CHKERRQ(ierr);
 
+  ierr = PetscObjectReference((PetscObject)draw);CHKERRQ(ierr);
+  ad->win = draw;
+
   ad->xticks    = PetscADefTicks;
   ad->yticks    = PetscADefTicks;
   ad->xlabelstr = PetscADefLabel;
   ad->ylabelstr = PetscADefLabel;
-  ad->win       = draw;
   ad->ac        = PETSC_DRAW_BLACK;
   ad->tc        = PETSC_DRAW_BLACK;
   ad->cc        = PETSC_DRAW_BLACK;
-  ad->xlabel    = 0;
-  ad->ylabel    = 0;
-  ad->toplabel  = 0;
+  ad->xlabel    = NULL;
+  ad->ylabel    = NULL;
+  ad->toplabel  = NULL;
 
   *axis = ad;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawAxisDestroy"
 /*@
     PetscDrawAxisDestroy - Frees the space used by an axis structure.
 
-    Collective over PetscDrawAxis
+    Collective on PetscDrawAxis
 
     Input Parameters:
 .   axis - the axis context
 
     Level: advanced
 
+.seealso: PetscDrawAxisCreate(), PetscDrawAxis
 @*/
 PetscErrorCode  PetscDrawAxisDestroy(PetscDrawAxis *axis)
 {
@@ -73,22 +71,22 @@ PetscErrorCode  PetscDrawAxisDestroy(PetscDrawAxis *axis)
 
   PetscFunctionBegin;
   if (!*axis) PetscFunctionReturn(0);
-  if (--((PetscObject)(*axis))->refct > 0) PetscFunctionReturn(0);
+  PetscValidHeaderSpecific(*axis,PETSC_DRAWAXIS_CLASSID,1);
+  if (--((PetscObject)(*axis))->refct > 0) {*axis = NULL; PetscFunctionReturn(0);}
 
   ierr = PetscFree((*axis)->toplabel);CHKERRQ(ierr);
   ierr = PetscFree((*axis)->xlabel);CHKERRQ(ierr);
   ierr = PetscFree((*axis)->ylabel);CHKERRQ(ierr);
+  ierr = PetscDrawDestroy(&(*axis)->win);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(axis);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawAxisSetColors"
 /*@
     PetscDrawAxisSetColors -  Sets the colors to be used for the axis,
                          tickmarks, and text.
 
-    Not Collective (ignored on all processors except processor 0 of PetscDrawAxis)
+    Logically Collective on PetscDrawAxis
 
     Input Parameters:
 +   axis - the axis
@@ -98,39 +96,43 @@ PetscErrorCode  PetscDrawAxisDestroy(PetscDrawAxis *axis)
 
     Level: advanced
 
+.seealso: PetscDrawAxisCreate(), PetscDrawAxis, PetscDrawAxisSetLabels(), PetscDrawAxisDraw(), PetscDrawAxisSetLimits()
 @*/
 PetscErrorCode  PetscDrawAxisSetColors(PetscDrawAxis axis,int ac,int tc,int cc)
 {
   PetscFunctionBegin;
-  if (!axis) PetscFunctionReturn(0);
+  PetscValidHeaderSpecific(axis,PETSC_DRAWAXIS_CLASSID,1);
+  PetscValidLogicalCollectiveInt(axis,ac,2);
+  PetscValidLogicalCollectiveInt(axis,tc,3);
+  PetscValidLogicalCollectiveInt(axis,cc,4);
   axis->ac = ac; axis->tc = tc; axis->cc = cc;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawAxisSetLabels"
 /*@C
     PetscDrawAxisSetLabels -  Sets the x and y axis labels.
 
-    Not Collective (ignored on all processors except processor 0 of PetscDrawAxis)
+    Logically Collective on PetscDrawAxis
 
     Input Parameters:
 +   axis - the axis
 .   top - the label at the top of the image
 -   xlabel,ylabel - the labes for the x and y axis
 
-    Notes: Must be called before PetscDrawAxisDraw() or PetscDrawLGDraw()
+    Notes:
+    Must be called before PetscDrawAxisDraw() or PetscDrawLGDraw()
            There should be no newlines in the arguments
 
     Level: advanced
 
+.seealso: PetscDrawAxisCreate(), PetscDrawAxis, PetscDrawAxisSetColors(), PetscDrawAxisDraw(), PetscDrawAxisSetLimits()
 @*/
 PetscErrorCode  PetscDrawAxisSetLabels(PetscDrawAxis axis,const char top[],const char xlabel[],const char ylabel[])
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (!axis) PetscFunctionReturn(0);
+  PetscValidHeaderSpecific(axis,PETSC_DRAWAXIS_CLASSID,1);
   ierr = PetscFree(axis->xlabel);CHKERRQ(ierr);
   ierr = PetscFree(axis->ylabel);CHKERRQ(ierr);
   ierr = PetscFree(axis->toplabel);CHKERRQ(ierr);
@@ -140,13 +142,70 @@ PetscErrorCode  PetscDrawAxisSetLabels(PetscDrawAxis axis,const char top[],const
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawAxisSetHoldLimits"
+/*@
+    PetscDrawAxisSetLimits -  Sets the limits (in user coords) of the axis
+
+    Logically Collective on PetscDrawAxis
+
+    Input Parameters:
++   axis - the axis
+.   xmin,xmax - limits in x
+-   ymin,ymax - limits in y
+
+    Options Database:
+.   -drawaxis_hold - hold the initial set of axis limits for future plotting
+
+    Level: advanced
+
+.seealso:  PetscDrawAxisSetHoldLimits(), PetscDrawAxisGetLimits(), PetscDrawAxisSetLabels(), PetscDrawAxisSetColors()
+
+@*/
+PetscErrorCode  PetscDrawAxisSetLimits(PetscDrawAxis axis,PetscReal xmin,PetscReal xmax,PetscReal ymin,PetscReal ymax)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(axis,PETSC_DRAWAXIS_CLASSID,1);
+  if (axis->hold) PetscFunctionReturn(0);
+  axis->xlow = xmin;
+  axis->xhigh= xmax;
+  axis->ylow = ymin;
+  axis->yhigh= ymax;
+  ierr = PetscOptionsHasName(((PetscObject)axis)->options,((PetscObject)axis)->prefix,"-drawaxis_hold",&axis->hold);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+    PetscDrawAxisGetLimits -  Gets the limits (in user coords) of the axis
+
+    Not Collective
+
+    Input Parameters:
++   axis - the axis
+.   xmin,xmax - limits in x
+-   ymin,ymax - limits in y
+
+    Level: advanced
+
+.seealso:  PetscDrawAxisCreate(), PetscDrawAxis, PetscDrawAxisSetHoldLimits(), PetscDrawAxisSetLimits(), PetscDrawAxisSetLabels(), PetscDrawAxisSetColors()
+
+@*/
+PetscErrorCode  PetscDrawAxisGetLimits(PetscDrawAxis axis,PetscReal *xmin,PetscReal *xmax,PetscReal *ymin,PetscReal *ymax)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(axis,PETSC_DRAWAXIS_CLASSID,1);
+  *xmin = axis->xlow;
+  *xmax = axis->xhigh;
+  *ymin = axis->ylow;
+  *ymax = axis->yhigh;
+  PetscFunctionReturn(0);
+}
+
 /*@
     PetscDrawAxisSetHoldLimits -  Causes an axis to keep the same limits until this is called
         again
 
-    Not Collective (ignored on all processors except processor 0 of PetscDrawAxis)
+    Logically Collective on PetscDrawAxis
 
     Input Parameters:
 +   axis - the axis
@@ -158,23 +217,22 @@ PetscErrorCode  PetscDrawAxisSetLabels(PetscDrawAxis axis,const char top[],const
         Once this has been called with PETSC_TRUE the limits will not change if you call
      PetscDrawAxisSetLimits() until you call this with PETSC_FALSE
 
-.seealso:  PetscDrawAxisSetLimits()
+.seealso:  PetscDrawAxisCreate(), PetscDrawAxis, PetscDrawAxisGetLimits(), PetscDrawAxisSetLimits(), PetscDrawAxisSetLabels(), PetscDrawAxisSetColors()
 
 @*/
 PetscErrorCode  PetscDrawAxisSetHoldLimits(PetscDrawAxis axis,PetscBool hold)
 {
   PetscFunctionBegin;
-  if (!axis) PetscFunctionReturn(0);
+  PetscValidHeaderSpecific(axis,PETSC_DRAWAXIS_CLASSID,1);
+  PetscValidLogicalCollectiveBool(axis,hold,2);
   axis->hold = hold;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawAxisDraw"
 /*@
     PetscDrawAxisDraw - PetscDraws an axis.
 
-    Not Collective (ignored on all processors except processor 0 of PetscDrawAxis)
+    Collective on PetscDrawAxis
 
     Input Parameter:
 .   axis - Axis structure
@@ -186,99 +244,135 @@ PetscErrorCode  PetscDrawAxisSetHoldLimits(PetscDrawAxis axis,PetscBool hold)
     By picking special routines for the ticks and labels, special
     effects may be generated.  These routines are part of the Axis
     structure (axis).
+
+.seealso:  PetscDrawAxisCreate(), PetscDrawAxis, PetscDrawAxisGetLimits(), PetscDrawAxisSetLimits(), PetscDrawAxisSetLabels(), PetscDrawAxisSetColors()
+
 @*/
 PetscErrorCode  PetscDrawAxisDraw(PetscDrawAxis axis)
 {
-  int            i,ntick,numx,numy,ac = axis->ac,tc = axis->tc,cc = axis->cc,rank;
-  size_t         len;
-  PetscReal      tickloc[MAXSEGS],sep,h,w,tw,th,xl,xr,yl,yr;
+  int            i,ntick,numx,numy,ac,tc,cc;
+  PetscMPIInt    rank;
+  size_t         len,ytlen=0;
+  PetscReal      coors[4],tickloc[MAXSEGS],sep,tw,th;
+  PetscReal      xl,xr,yl,yr,dxl=0,dyl=0,dxr=0,dyr=0;
   char           *p;
-  PetscDraw      draw = axis->win;
+  PetscDraw      draw;
+  PetscBool      isnull;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (!axis) PetscFunctionReturn(0);
+  PetscValidHeaderSpecific(axis,PETSC_DRAWAXIS_CLASSID,1);
+  ierr = PetscDrawIsNull(axis->win,&isnull);CHKERRQ(ierr);
+  if (isnull) PetscFunctionReturn(0);
   ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)axis),&rank);CHKERRQ(ierr);
-  if (rank) PetscFunctionReturn(0);
 
+  draw = axis->win;
+
+  ac = axis->ac; tc = axis->tc; cc = axis->cc;
   if (axis->xlow == axis->xhigh) {axis->xlow -= .5; axis->xhigh += .5;}
-  /*  if ((axis->yhigh - axis->ylow) <= 1.e-5*PetscMax(PetscAbsReal(axis->yhigh),PetscAbsReal(axis->ylow))) {
-    axis->ylow  -= 1.e-5*PetscMax(PetscAbsReal(axis->yhigh),PetscAbsReal(axis->ylow));
-    axis->yhigh += 1.e-5*PetscMax(PetscAbsReal(axis->yhigh),PetscAbsReal(axis->ylow));
-   } */
   if (axis->ylow == axis->yhigh) {axis->ylow -= .5; axis->yhigh += .5;}
 
-  xl   = axis->xlow; xr = axis->xhigh; yl = axis->ylow; yr = axis->yhigh;
-  ierr = PetscDrawSetCoordinates(draw,xl,yl,xr,yr);CHKERRQ(ierr);
+  ierr = PetscDrawCollectiveBegin(draw);CHKERRQ(ierr);
+  if (rank) goto finally;
+
+  /* get cannonical string size */
+  ierr = PetscDrawSetCoordinates(draw,0,0,1,1);CHKERRQ(ierr);
   ierr = PetscDrawStringGetSize(draw,&tw,&th);CHKERRQ(ierr);
-  numx = (int)(.15*(xr-xl)/tw); if (numx > 6) numx = 6;if (numx< 2) numx = 2;
-  numy = (int)(.50*(yr-yl)/th); if (numy > 6) numy = 6;if (numy< 2) numy = 2;
-  xl  -= 11*tw; xr += 2*tw; yl -= 2.5*th; yr += 2*th;
-  if (axis->xlabel) yl -= 2*th;
-  if (axis->ylabel) xl -= 2*tw;
+  /* lower spacing */
+  if (axis->xlabelstr) dyl += 1.5*th;
+  if (axis->xlabel)    dyl += 1.5*th;
+  /* left spacing */
+  if (axis->ylabelstr) dxl += 7.5*tw;
+  if (axis->ylabel)    dxl += 2.0*tw;
+  /* right and top spacing */
+  if (axis->xlabelstr) dxr = 2.5*tw;
+  if (axis->ylabelstr) dyr = 0.5*th;
+  if (axis->toplabel)  dyr = 1.5*th;
+  /* extra spacing */
+  dxl += 0.7*tw; dxr += 0.5*tw;
+  dyl += 0.2*th; dyr += 0.2*th;
+  /* determine coordinates */
+  xl = (dxl*axis->xhigh + dxr*axis->xlow - axis->xlow)  / (dxl + dxr - 1);
+  xr = (dxl*axis->xhigh + dxr*axis->xlow - axis->xhigh) / (dxl + dxr - 1);
+  yl = (dyl*axis->yhigh + dyr*axis->ylow - axis->ylow)  / (dyl + dyr - 1);
+  yr = (dyl*axis->yhigh + dyr*axis->ylow - axis->yhigh) / (dyl + dyr - 1);
   ierr = PetscDrawSetCoordinates(draw,xl,yl,xr,yr);CHKERRQ(ierr);
   ierr = PetscDrawStringGetSize(draw,&tw,&th);CHKERRQ(ierr);
 
+  /* PetscDraw the axis lines */
   ierr = PetscDrawLine(draw,axis->xlow,axis->ylow,axis->xhigh,axis->ylow,ac);CHKERRQ(ierr);
   ierr = PetscDrawLine(draw,axis->xlow,axis->ylow,axis->xlow,axis->yhigh,ac);CHKERRQ(ierr);
+  ierr = PetscDrawLine(draw,axis->xlow,axis->yhigh,axis->xhigh,axis->yhigh,ac);CHKERRQ(ierr);
+  ierr = PetscDrawLine(draw,axis->xhigh,axis->ylow,axis->xhigh,axis->yhigh,ac);CHKERRQ(ierr);
 
+  /* PetscDraw the top label */
   if (axis->toplabel) {
-    h    = axis->yhigh;
-    ierr = PetscDrawStringCentered(draw,.5*(xl+xr),axis->yhigh,cc,axis->toplabel);CHKERRQ(ierr);
+    PetscReal x = (axis->xlow + axis->xhigh)/2, y = axis->yhigh + 0.5*th;
+    ierr = PetscDrawStringCentered(draw,x,y,cc,axis->toplabel);CHKERRQ(ierr);
   }
 
-  /* PetscDraw the ticks and labels */
+  /* PetscDraw the X ticks and labels */
   if (axis->xticks) {
+    numx = (int)(.15*(axis->xhigh-axis->xlow)/tw); numx = PetscClipInterval(numx,2,6);
     ierr = (*axis->xticks)(axis->xlow,axis->xhigh,numx,&ntick,tickloc,MAXSEGS);CHKERRQ(ierr);
     /* PetscDraw in tick marks */
     for (i=0; i<ntick; i++) {
-      ierr = PetscDrawLine(draw,tickloc[i],axis->ylow-.5*th,tickloc[i],axis->ylow+.5*th,tc);CHKERRQ(ierr);
+      ierr = PetscDrawLine(draw,tickloc[i],axis->ylow,tickloc[i],axis->ylow+.5*th,tc);CHKERRQ(ierr);
+      ierr = PetscDrawLine(draw,tickloc[i],axis->yhigh,tickloc[i],axis->yhigh-.5*th,tc);CHKERRQ(ierr);
     }
     /* label ticks */
-    for (i=0; i<ntick; i++) {
-      if (axis->xlabelstr) {
+    if (axis->xlabelstr) {
+      for (i=0; i<ntick; i++) {
         if (i < ntick - 1) sep = tickloc[i+1] - tickloc[i];
         else if (i > 0)    sep = tickloc[i]   - tickloc[i-1];
         else               sep = 0.0;
         ierr = (*axis->xlabelstr)(tickloc[i],sep,&p);CHKERRQ(ierr);
-        ierr = PetscDrawStringCentered(draw,tickloc[i],axis->ylow-1.2*th,cc,p);CHKERRQ(ierr);
+        ierr = PetscDrawStringCentered(draw,tickloc[i],axis->ylow-1.5*th,cc,p);CHKERRQ(ierr);
       }
     }
   }
   if (axis->xlabel) {
-    h    = axis->ylow - 2.5*th;
-    ierr = PetscDrawStringCentered(draw,.5*(xl + xr),h,cc,axis->xlabel);CHKERRQ(ierr);
+    PetscReal x = (axis->xlow + axis->xhigh)/2, y = axis->ylow - 1.5*th;
+    if (axis->xlabelstr) y -= 1.5*th;
+    ierr = PetscDrawStringCentered(draw,x,y,cc,axis->xlabel);CHKERRQ(ierr);
   }
+
+  /* PetscDraw the Y ticks and labels */
   if (axis->yticks) {
+    numy = (int)(.50*(axis->yhigh-axis->ylow)/th); numy = PetscClipInterval(numy,2,6);
     ierr = (*axis->yticks)(axis->ylow,axis->yhigh,numy,&ntick,tickloc,MAXSEGS);CHKERRQ(ierr);
     /* PetscDraw in tick marks */
     for (i=0; i<ntick; i++) {
-      ierr = PetscDrawLine(draw,axis->xlow -.5*tw,tickloc[i],axis->xlow+.5*tw,tickloc[i],tc);CHKERRQ(ierr);
+      ierr = PetscDrawLine(draw,axis->xlow,tickloc[i],axis->xlow+.5*tw,tickloc[i],tc);CHKERRQ(ierr);
+      ierr = PetscDrawLine(draw,axis->xhigh,tickloc[i],axis->xhigh-.5*tw,tickloc[i],tc);CHKERRQ(ierr);
     }
     /* label ticks */
-    for (i=0; i<ntick; i++) {
-      if (axis->ylabelstr) {
+    if (axis->ylabelstr) {
+      for (i=0; i<ntick; i++) {
         if (i < ntick - 1) sep = tickloc[i+1] - tickloc[i];
         else if (i > 0)    sep = tickloc[i]   - tickloc[i-1];
         else               sep = 0.0;
-        ierr = (*axis->xlabelstr)(tickloc[i],sep,&p);CHKERRQ(ierr);
-        ierr = PetscStrlen(p,&len);CHKERRQ(ierr);
-        w    = axis->xlow - len * tw - 1.2*tw;
-        ierr = PetscDrawString(draw,w,tickloc[i]-.5*th,cc,p);CHKERRQ(ierr);
+        ierr = (*axis->ylabelstr)(tickloc[i],sep,&p);CHKERRQ(ierr);
+        ierr = PetscStrlen(p,&len);CHKERRQ(ierr); ytlen = PetscMax(ytlen,len);
+        ierr = PetscDrawString(draw,axis->xlow-(len+.5)*tw,tickloc[i]-.5*th,cc,p);CHKERRQ(ierr);
       }
     }
   }
   if (axis->ylabel) {
+    PetscReal x = axis->xlow - 2.0*tw, y = (axis->ylow + axis->yhigh)/2;
+    if (axis->ylabelstr) x -= (ytlen+.5)*tw;
     ierr = PetscStrlen(axis->ylabel,&len);CHKERRQ(ierr);
-    h    = yl + .5*(yr - yl) + .5*len*th;
-    w    = xl + 1.5*tw;
-    ierr = PetscDrawStringVertical(draw,w,h,cc,axis->ylabel);CHKERRQ(ierr);
+    ierr = PetscDrawStringVertical(draw,x,y+len*th/2,cc,axis->ylabel);CHKERRQ(ierr);
   }
+
+  ierr = PetscDrawGetCoordinates(draw,&coors[0],&coors[1],&coors[2],&coors[3]);CHKERRQ(ierr);
+finally:
+  ierr = PetscDrawCollectiveEnd(draw);CHKERRQ(ierr);
+  ierr = MPI_Bcast(coors,4,MPIU_REAL,0,PetscObjectComm((PetscObject)draw));CHKERRQ(ierr);
+  ierr = PetscDrawSetCoordinates(draw,coors[0],coors[1],coors[2],coors[3]);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStripe0"
 /*
     Removes all zeros but one from .0000
 */
@@ -306,8 +400,6 @@ PetscErrorCode PetscStripe0(char *buf)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStripAllZeros"
 /*
     Removes all zeros but one from .0000
 */
@@ -327,8 +419,6 @@ PetscErrorCode PetscStripAllZeros(char *buf)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStripTrailingZeros"
 /*
     Removes trailing zeros
 */
@@ -358,8 +448,6 @@ PetscErrorCode PetscStripTrailingZeros(char *buf)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStripInitialZero"
 /*
     Removes leading 0 from 0.22 or -0.22
 */
@@ -378,8 +466,6 @@ PetscErrorCode PetscStripInitialZero(char *buf)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStripZeros"
 /*
      Removes the extraneous zeros in numbers like 1.10000e6
 */
@@ -401,8 +487,6 @@ PetscErrorCode PetscStripZeros(char *buf)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStripZerosPlus"
 /*
       Removes the plus in something like 1.1e+2 or 1.1e+02
 */
@@ -432,10 +516,3 @@ PetscErrorCode PetscStripZerosPlus(char *buf)
   }
   PetscFunctionReturn(0);
 }
-
-
-
-
-
-
-
