@@ -29,12 +29,10 @@ struct _p_PetscDrawHG {
 
 #define CHUNKSIZE 100
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawHGCreate"
 /*@C
    PetscDrawHGCreate - Creates a histogram data structure.
 
-   Collective over PetscDraw
+   Collective on PetscDraw
 
    Input Parameters:
 +  draw  - The window where the graph will be made
@@ -43,30 +41,41 @@ struct _p_PetscDrawHG {
    Output Parameters:
 .  hist - The histogram context
 
+   Notes:
+    The difference between a bar chart, PetscDrawBar, and a histogram, PetscDrawHG, is explained here http://stattrek.com/statistics/charts/histogram.aspx?Tutorial=AP
+
+   The histogram is only displayed when PetscDrawHGDraw() is called.
+
+   The MPI communicator that owns the PetscDraw owns this PetscDrawHG, but the calls to set options and add data are ignored on all processes except the
+   zeroth MPI process in the communicator. All MPI processes in the communicator must call PetscDrawHGDraw() to display the updated graph.
+
    Level: intermediate
 
    Concepts: histogram^creating
 
-.seealso: PetscDrawHGDestroy()
+.seealso: PetscDrawHGDestroy(), PetscDrawHG, PetscDrawBarCreate(), PetscDrawBar, PetscDrawLGCreate(), PetscDrawLG, PetscDrawSPCreate(), PetscDrawSP,
+          PetscDrawHGSetNumberBins(), PetscDrawHGReset(), PetscDrawHGAddValue(), PetscDrawHGDraw(), PetscDrawHGSave(), PetscDrawHGView(), PetscDrawHGSetColor(),
+          PetscDrawHGSetLimits(), PetscDrawHGCalcStats(), PetscDrawHGIntegerBins(), PetscDrawHGGetAxis(), PetscDrawAxis, PetscDrawHGGetDraw() 
 
 @*/
-PetscErrorCode  PetscDrawHGCreate(PetscDraw draw, int bins, PetscDrawHG *hist)
+PetscErrorCode  PetscDrawHGCreate(PetscDraw draw,int bins,PetscDrawHG *hist)
 {
   PetscDrawHG    h;
-  PetscBool      isnull;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(draw, PETSC_DRAW_CLASSID,1);
+  PetscValidLogicalCollectiveInt(draw,bins,2);
   PetscValidPointer(hist,3);
-  ierr = PetscHeaderCreate(h, PETSC_DRAWHG_CLASSID,  "PetscDrawHG", "Histogram", "Draw", PetscObjectComm((PetscObject)draw), PetscDrawHGDestroy, NULL);CHKERRQ(ierr);
+
+  ierr = PetscHeaderCreate(h,PETSC_DRAWHG_CLASSID,"DrawHG","Histogram","Draw",PetscObjectComm((PetscObject)draw),PetscDrawHGDestroy,NULL);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent((PetscObject)draw,(PetscObject)h);CHKERRQ(ierr);
+
+  ierr = PetscObjectReference((PetscObject)draw);CHKERRQ(ierr);
+  h->win = draw;
 
   h->view        = NULL;
   h->destroy     = NULL;
-  h->win         = draw;
-
-  ierr = PetscObjectReference((PetscObject) draw);CHKERRQ(ierr);
-
   h->color       = PETSC_DRAW_GREEN;
   h->xmin        = PETSC_MAX_REAL;
   h->xmax        = PETSC_MIN_REAL;
@@ -75,64 +84,61 @@ PetscErrorCode  PetscDrawHGCreate(PetscDraw draw, int bins, PetscDrawHG *hist)
   h->numBins     = bins;
   h->maxBins     = bins;
 
-  ierr = PetscMalloc1(h->maxBins, &h->bins);CHKERRQ(ierr);
+  ierr = PetscMalloc1(h->maxBins,&h->bins);CHKERRQ(ierr);
 
   h->numValues   = 0;
   h->maxValues   = CHUNKSIZE;
   h->calcStats   = PETSC_FALSE;
   h->integerBins = PETSC_FALSE;
 
-  ierr = PetscMalloc1(h->maxValues, &h->values);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory((PetscObject)h, (h->maxBins + h->maxValues)*sizeof(PetscReal));CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject) draw, PETSC_DRAW_NULL, &isnull);CHKERRQ(ierr);
+  ierr = PetscMalloc1(h->maxValues,&h->values);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)h,(h->maxBins + h->maxValues)*sizeof(PetscReal));CHKERRQ(ierr);
+
+  ierr = PetscDrawAxisCreate(draw,&h->axis);CHKERRQ(ierr);
+  ierr = PetscLogObjectParent((PetscObject)h,(PetscObject)h->axis);CHKERRQ(ierr);
 
-  if (!isnull) {
-    ierr = PetscDrawAxisCreate(draw, &h->axis);CHKERRQ(ierr);
-    ierr = PetscLogObjectParent((PetscObject)h, (PetscObject)h->axis);CHKERRQ(ierr);
-  } else h->axis = NULL;
   *hist = h;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawHGSetNumberBins"
 /*@
    PetscDrawHGSetNumberBins - Change the number of bins that are to be drawn.
 
-   Not Collective (ignored except on processor 0 of PetscDrawHG)
+   Logically Collective on PetscDrawHG
 
    Input Parameter:
 +  hist - The histogram context.
--  dim  - The number of curves.
+-  bins  - The number of bins.
 
    Level: intermediate
 
    Concepts: histogram^setting number of bins
 
+.seealso: PetscDrawHGCreate(), PetscDrawHG, PetscDrawHGDraw(), PetscDrawHGIntegerBins()
+
 @*/
 PetscErrorCode  PetscDrawHGSetNumberBins(PetscDrawHG hist, int bins)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(hist, PETSC_DRAWHG_CLASSID,1);
+  PetscValidHeaderSpecific(hist,PETSC_DRAWHG_CLASSID,1);
+  PetscValidLogicalCollectiveInt(hist,bins,2);
+
   if (hist->maxBins < bins) {
     ierr = PetscFree(hist->bins);CHKERRQ(ierr);
     ierr = PetscMalloc1(bins, &hist->bins);CHKERRQ(ierr);
     ierr = PetscLogObjectMemory((PetscObject)hist, (bins - hist->maxBins) * sizeof(PetscReal));CHKERRQ(ierr);
-
     hist->maxBins = bins;
   }
   hist->numBins = bins;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawHGReset"
 /*@
   PetscDrawHGReset - Clears histogram to allow for reuse with new data.
 
-  Not Collective (ignored except on processor 0 of PetscDrawHG)
+  Logically Collective on PetscDrawHG
 
   Input Parameter:
 . hist - The histogram context.
@@ -140,11 +146,15 @@ PetscErrorCode  PetscDrawHGSetNumberBins(PetscDrawHG hist, int bins)
   Level: intermediate
 
   Concepts: histogram^resetting
+
+.seealso: PetscDrawHGCreate(), PetscDrawHG, PetscDrawHGDraw(), PetscDrawHGAddValue()
+
 @*/
 PetscErrorCode  PetscDrawHGReset(PetscDrawHG hist)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(hist, PETSC_DRAWHG_CLASSID,1);
+  PetscValidHeaderSpecific(hist,PETSC_DRAWHG_CLASSID,1);
+
   hist->xmin      = PETSC_MAX_REAL;
   hist->xmax      = PETSC_MIN_REAL;
   hist->ymin      = 0.0;
@@ -153,19 +163,17 @@ PetscErrorCode  PetscDrawHGReset(PetscDrawHG hist)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawHGDestroy"
 /*@C
   PetscDrawHGDestroy - Frees all space taken up by histogram data structure.
 
-  Collective over PetscDrawHG
+  Collective on PetscDrawHG
 
   Input Parameter:
 . hist - The histogram context
 
   Level: intermediate
 
-.seealso:  PetscDrawHGCreate()
+.seealso:  PetscDrawHGCreate(), PetscDrawHG
 @*/
 PetscErrorCode  PetscDrawHGDestroy(PetscDrawHG *hist)
 {
@@ -173,23 +181,21 @@ PetscErrorCode  PetscDrawHGDestroy(PetscDrawHG *hist)
 
   PetscFunctionBegin;
   if (!*hist) PetscFunctionReturn(0);
-  PetscValidHeader(*hist,1);
+  PetscValidHeaderSpecific(*hist,PETSC_DRAWHG_CLASSID,1);
+  if (--((PetscObject)(*hist))->refct > 0) {*hist = NULL; PetscFunctionReturn(0);}
 
-  if (--((PetscObject)(*hist))->refct > 0) PetscFunctionReturn(0);
-  ierr = PetscDrawAxisDestroy(&(*hist)->axis);CHKERRQ(ierr);
-  ierr = PetscDrawDestroy(&(*hist)->win);CHKERRQ(ierr);
   ierr = PetscFree((*hist)->bins);CHKERRQ(ierr);
   ierr = PetscFree((*hist)->values);CHKERRQ(ierr);
+  ierr = PetscDrawAxisDestroy(&(*hist)->axis);CHKERRQ(ierr);
+  ierr = PetscDrawDestroy(&(*hist)->win);CHKERRQ(ierr);
   ierr = PetscHeaderDestroy(hist);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawHGAddValue"
 /*@
   PetscDrawHGAddValue - Adds another value to the histogram.
 
-  Not Collective (ignored except on processor 0 of PetscDrawHG)
+  Logically Collective on PetscDrawHG
 
   Input Parameters:
 + hist  - The histogram
@@ -199,12 +205,13 @@ PetscErrorCode  PetscDrawHGDestroy(PetscDrawHG *hist)
 
   Concepts: histogram^adding values
 
-.seealso: PetscDrawHGAddValues()
+.seealso: PetscDrawHGCreate(), PetscDrawHG, PetscDrawHGDraw(), PetscDrawHGAddValue(), PetscDrawHGReset()
 @*/
 PetscErrorCode  PetscDrawHGAddValue(PetscDrawHG hist, PetscReal value)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(hist, PETSC_DRAWHG_CLASSID,1);
+  PetscValidHeaderSpecific(hist,PETSC_DRAWHG_CLASSID,1);
+
   /* Allocate more memory if necessary */
   if (hist->numValues >= hist->maxValues) {
     PetscReal      *tmp;
@@ -247,43 +254,41 @@ PetscErrorCode  PetscDrawHGAddValue(PetscDrawHG hist, PetscReal value)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawHGDraw"
 /*@
   PetscDrawHGDraw - Redraws a histogram.
 
-  Not Collective (ignored except on processor 0 of PetscDrawHG)
+  Collective on PetscDrawHG
 
   Input Parameter:
 . hist - The histogram context
 
   Level: intermediate
 
+.seealso: PetscDrawHGCreate(), PetscDrawHG, PetscDrawHGDraw(), PetscDrawHGAddValue(), PetscDrawHGReset()
+
 @*/
 PetscErrorCode  PetscDrawHGDraw(PetscDrawHG hist)
 {
-  PetscDraw      draw = hist->win;
+  PetscDraw      draw;
   PetscBool      isnull;
   PetscReal      xmin,xmax,ymin,ymax,*bins,*values,binSize,binLeft,binRight,maxHeight,mean,var;
   char           title[256];
   char           xlabel[256];
   PetscInt       numBins,numBinsOld,numValues,initSize,i,p,bcolor,color;
+  PetscMPIInt    rank;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(hist, PETSC_DRAWHG_CLASSID,1);
-  ierr = PetscObjectTypeCompare((PetscObject) draw, PETSC_DRAW_NULL, &isnull);CHKERRQ(ierr);
+  PetscValidHeaderSpecific(hist,PETSC_DRAWHG_CLASSID,1);
+  ierr = PetscDrawIsNull(hist->win,&isnull);CHKERRQ(ierr);
   if (isnull) PetscFunctionReturn(0);
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)hist),&rank);CHKERRQ(ierr);
+
   if ((hist->xmin >= hist->xmax) || (hist->ymin >= hist->ymax)) PetscFunctionReturn(0);
   if (hist->numValues < 1) PetscFunctionReturn(0);
 
-#if 0
-  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)hist),&rank);CHKERRQ(ierr);
-  if (rank) PetscFunctionReturn(0);
-#endif
-
   color = hist->color;
-  if (color == PETSC_DRAW_ROTATE) bcolor = 2;
+  if (color == PETSC_DRAW_ROTATE) bcolor = PETSC_DRAW_BLACK+1;
   else bcolor = color;
 
   xmin      = hist->xmin;
@@ -295,7 +300,10 @@ PetscErrorCode  PetscDrawHGDraw(PetscDrawHG hist)
   mean      = 0.0;
   var       = 0.0;
 
+  draw = hist->win;
+  ierr = PetscDrawCheckResizedWindow(draw);CHKERRQ(ierr);
   ierr = PetscDrawClear(draw);CHKERRQ(ierr);
+
   if (xmin == xmax) {
     /* Calculate number of points in each bin */
     bins    = hist->bins;
@@ -318,17 +326,16 @@ PetscErrorCode  PetscDrawHGDraw(PetscDrawHG hist)
       ierr = PetscDrawAxisSetLabels(hist->axis, title, xlabel, NULL);CHKERRQ(ierr);
     }
     ierr = PetscDrawAxisDraw(hist->axis);CHKERRQ(ierr);
-    /* Draw bins */
-    binLeft  = xmin;
-    binRight = xmax;
-    ierr = PetscDrawRectangle(draw,binLeft,ymin,binRight,bins[0],bcolor,bcolor,bcolor,bcolor);CHKERRQ(ierr);
-
-    if (color == PETSC_DRAW_ROTATE && bins[0] != 0.0) bcolor++;
-    if (bcolor > 31) bcolor = 2;
-
-    ierr = PetscDrawLine(draw,binLeft,ymin,binLeft,bins[0],PETSC_DRAW_BLACK);CHKERRQ(ierr);
-    ierr = PetscDrawLine(draw,binRight,ymin,binRight,bins[0],PETSC_DRAW_BLACK);CHKERRQ(ierr);
-    ierr = PetscDrawLine(draw,binLeft,bins[0],binRight,bins[0],PETSC_DRAW_BLACK);CHKERRQ(ierr);
+    ierr = PetscDrawCollectiveBegin(draw);CHKERRQ(ierr);
+    if (!rank) { /* Draw bins */
+      binLeft  = xmin;
+      binRight = xmax;
+      ierr = PetscDrawRectangle(draw,binLeft,ymin,binRight,bins[0],bcolor,bcolor,bcolor,bcolor);CHKERRQ(ierr);
+      ierr = PetscDrawLine(draw,binLeft,ymin,binLeft,bins[0],PETSC_DRAW_BLACK);CHKERRQ(ierr);
+      ierr = PetscDrawLine(draw,binRight,ymin,binRight,bins[0],PETSC_DRAW_BLACK);CHKERRQ(ierr);
+      ierr = PetscDrawLine(draw,binLeft,bins[0],binRight,bins[0],PETSC_DRAW_BLACK);CHKERRQ(ierr);
+    }
+    ierr = PetscDrawCollectiveEnd(draw);CHKERRQ(ierr);
   } else {
     numBins    = hist->numBins;
     numBinsOld = hist->numBins;
@@ -367,32 +374,57 @@ PetscErrorCode  PetscDrawHGDraw(PetscDrawHG hist)
       mean /= numValues;
       if (numValues > 1) var = (var - numValues*mean*mean) / (numValues-1);
       else var = 0.0;
-
       ierr = PetscSNPrintf(title, 256,"Mean: %g  Var: %g", (double)mean, (double)var);CHKERRQ(ierr);
       ierr = PetscSNPrintf(xlabel,256, "Total: %D", numValues);CHKERRQ(ierr);
       ierr = PetscDrawAxisSetLabels(hist->axis, title, xlabel, NULL);CHKERRQ(ierr);
     }
     ierr = PetscDrawAxisDraw(hist->axis);CHKERRQ(ierr);
-    /* Draw bins */
-    for (i = 0; i < numBins; i++) {
-      binLeft  = xmin + binSize*i;
-      binRight = xmin + binSize*(i+1);
-      ierr = PetscDrawRectangle(draw,binLeft,ymin,binRight,bins[i],bcolor,bcolor,bcolor,bcolor);CHKERRQ(ierr);
-      if (color == PETSC_DRAW_ROTATE && bins[i]) bcolor++;
-      if (bcolor > 31) bcolor = 2;
-      ierr = PetscDrawLine(draw,binLeft,ymin,binLeft,bins[i],PETSC_DRAW_BLACK);CHKERRQ(ierr);
-      ierr = PetscDrawLine(draw,binRight,ymin,binRight,bins[i],PETSC_DRAW_BLACK);CHKERRQ(ierr);
-      ierr = PetscDrawLine(draw,binLeft,bins[i],binRight,bins[i],PETSC_DRAW_BLACK);CHKERRQ(ierr);
+    ierr = PetscDrawCollectiveBegin(draw);CHKERRQ(ierr);
+    if (!rank) { /* Draw bins */
+      for (i = 0; i < numBins; i++) {
+        binLeft  = xmin + binSize*i;
+        binRight = xmin + binSize*(i+1);
+        ierr = PetscDrawRectangle(draw,binLeft,ymin,binRight,bins[i],bcolor,bcolor,bcolor,bcolor);CHKERRQ(ierr);
+        ierr = PetscDrawLine(draw,binLeft,ymin,binLeft,bins[i],PETSC_DRAW_BLACK);CHKERRQ(ierr);
+        ierr = PetscDrawLine(draw,binRight,ymin,binRight,bins[i],PETSC_DRAW_BLACK);CHKERRQ(ierr);
+        ierr = PetscDrawLine(draw,binLeft,bins[i],binRight,bins[i],PETSC_DRAW_BLACK);CHKERRQ(ierr);
+        if (color == PETSC_DRAW_ROTATE && bins[i]) bcolor++;
+        if (bcolor > PETSC_DRAW_BASIC_COLORS-1) bcolor = PETSC_DRAW_BLACK+1;
+      }
     }
-    ierr = PetscDrawHGSetNumberBins(hist, numBinsOld);CHKERRQ(ierr);
+    ierr = PetscDrawCollectiveEnd(draw);CHKERRQ(ierr);
+    ierr = PetscDrawHGSetNumberBins(hist,numBinsOld);CHKERRQ(ierr);
   }
-  ierr = PetscDrawSynchronizedFlush(draw);CHKERRQ(ierr);
+
+  ierr = PetscDrawFlush(draw);CHKERRQ(ierr);
   ierr = PetscDrawPause(draw);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawHGView"
+/*@
+  PetscDrawHGSave - Saves a drawn image
+
+  Collective on PetscDrawHG
+
+  Input Parameter:
+. hist - The histogram context
+
+  Level: intermediate
+
+  Concepts: histogram^saving
+
+.seealso:  PetscDrawHGCreate(), PetscDrawHGGetDraw(), PetscDrawSetSave(), PetscDrawSave(), PetscDrawHGDraw()
+@*/
+PetscErrorCode  PetscDrawHGSave(PetscDrawHG hg)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(hg,PETSC_DRAWHG_CLASSID,1);
+  ierr = PetscDrawSave(hg->win);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
   PetscDrawHGView - Prints the histogram information.
 
@@ -403,6 +435,8 @@ PetscErrorCode  PetscDrawHGDraw(PetscDrawHG hist)
 
   Level: beginner
 
+.seealso:  PetscDrawHGCreate(), PetscDrawHGGetDraw(), PetscDrawSetSave(), PetscDrawSave(), PetscDrawHGDraw()
+
 .keywords:  draw, histogram
 @*/
 PetscErrorCode  PetscDrawHGView(PetscDrawHG hist,PetscViewer viewer)
@@ -412,7 +446,8 @@ PetscErrorCode  PetscDrawHGView(PetscDrawHG hist,PetscViewer viewer)
   PetscInt       numBins,numBinsOld,numValues,initSize,i,p;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(hist, PETSC_DRAWHG_CLASSID,1);
+  PetscValidHeaderSpecific(hist,PETSC_DRAWHG_CLASSID,1);
+
   if ((hist->xmin > hist->xmax) || (hist->ymin >= hist->ymax)) PetscFunctionReturn(0);
   if (hist->numValues < 1) PetscFunctionReturn(0);
 
@@ -485,12 +520,10 @@ PetscErrorCode  PetscDrawHGView(PetscDrawHG hist,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawHGSetColor"
 /*@
   PetscDrawHGSetColor - Sets the color the bars will be drawn with.
 
-  Not Collective (ignored except on processor 0 of PetscDrawHG)
+  Logically Collective on PetscDrawHG
 
   Input Parameters:
 + hist - The histogram context
@@ -499,23 +532,24 @@ PetscErrorCode  PetscDrawHGView(PetscDrawHG hist,PetscViewer viewer)
 
   Level: intermediate
 
+.seealso:  PetscDrawHGCreate(), PetscDrawHGGetDraw(), PetscDrawSetSave(), PetscDrawSave(), PetscDrawHGDraw(), PetscDrawHGGetAxis()
+
 @*/
-PetscErrorCode  PetscDrawHGSetColor(PetscDrawHG hist, int color)
+PetscErrorCode  PetscDrawHGSetColor(PetscDrawHG hist,int color)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(hist, PETSC_DRAWHG_CLASSID,1);
+  PetscValidHeaderSpecific(hist,PETSC_DRAWHG_CLASSID,1);
+
   hist->color = color;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawHGSetLimits"
 /*@
   PetscDrawHGSetLimits - Sets the axis limits for a histogram. If more
   points are added after this call, the limits will be adjusted to
   include those additional points.
 
-  Not Collective (ignored except on processor 0 of PetscDrawHG)
+  Logically Collective on PetscDrawHG
 
   Input Parameters:
 + hist - The histogram context
@@ -524,11 +558,15 @@ PetscErrorCode  PetscDrawHGSetColor(PetscDrawHG hist, int color)
   Level: intermediate
 
   Concepts: histogram^setting axis
+
+.seealso:  PetscDrawHGCreate(), PetscDrawHGGetDraw(), PetscDrawSetSave(), PetscDrawSave(), PetscDrawHGDraw(), PetscDrawHGGetAxis()
+
 @*/
 PetscErrorCode  PetscDrawHGSetLimits(PetscDrawHG hist, PetscReal x_min, PetscReal x_max, int y_min, int y_max)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(hist, PETSC_DRAWHG_CLASSID,1);
+  PetscValidHeaderSpecific(hist,PETSC_DRAWHG_CLASSID,1);
+
   hist->xmin = x_min;
   hist->xmax = x_max;
   hist->ymin = y_min;
@@ -536,8 +574,6 @@ PetscErrorCode  PetscDrawHGSetLimits(PetscDrawHG hist, PetscReal x_min, PetscRea
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawHGCalcStats"
 /*@
   PetscDrawHGCalcStats - Turns on calculation of descriptive statistics
 
@@ -551,17 +587,18 @@ PetscErrorCode  PetscDrawHGSetLimits(PetscDrawHG hist, PetscReal x_min, PetscRea
 
 .keywords:  draw, histogram, statistics
 
+.seealso:  PetscDrawHGCreate(), PetscDrawHGAddValue(), PetscDrawHGView(), PetscDrawHGDraw()
+
 @*/
 PetscErrorCode  PetscDrawHGCalcStats(PetscDrawHG hist, PetscBool calc)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(hist, PETSC_DRAWHG_CLASSID,1);
+  PetscValidHeaderSpecific(hist,PETSC_DRAWHG_CLASSID,1);
+
   hist->calcStats = calc;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawHGIntegerBins"
 /*@
   PetscDrawHGIntegerBins - Turns on integer width bins
 
@@ -574,24 +611,26 @@ PetscErrorCode  PetscDrawHGCalcStats(PetscDrawHG hist, PetscBool calc)
   Level: intermediate
 
 .keywords:  draw, histogram, statistics
+
+.seealso:  PetscDrawHGCreate(), PetscDrawHGAddValue(), PetscDrawHGView(), PetscDrawHGDraw(), PetscDrawHGSetColor()
+
 @*/
 PetscErrorCode  PetscDrawHGIntegerBins(PetscDrawHG hist, PetscBool ints)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(hist, PETSC_DRAWHG_CLASSID,1);
+  PetscValidHeaderSpecific(hist,PETSC_DRAWHG_CLASSID,1);
+
   hist->integerBins = ints;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawHGGetAxis"
 /*@C
   PetscDrawHGGetAxis - Gets the axis context associated with a histogram.
   This is useful if one wants to change some axis property, such as
   labels, color, etc. The axis context should not be destroyed by the
   application code.
 
-  Not Collective (ignored except on processor 0 of PetscDrawHG)
+  Not Collective, PetscDrawAxis is parallel if PetscDrawHG is parallel
 
   Input Parameter:
 . hist - The histogram context
@@ -601,18 +640,18 @@ PetscErrorCode  PetscDrawHGIntegerBins(PetscDrawHG hist, PetscBool ints)
 
   Level: intermediate
 
+.seealso:  PetscDrawHGCreate(), PetscDrawHGAddValue(), PetscDrawHGView(), PetscDrawHGDraw(), PetscDrawHGSetColor(), PetscDrawAxis, PetscDrawHGSetLimits()
+
 @*/
-PetscErrorCode  PetscDrawHGGetAxis(PetscDrawHG hist, PetscDrawAxis *axis)
+PetscErrorCode  PetscDrawHGGetAxis(PetscDrawHG hist,PetscDrawAxis *axis)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(hist, PETSC_DRAWHG_CLASSID,1);
+  PetscValidHeaderSpecific(hist,PETSC_DRAWHG_CLASSID,1);
   PetscValidPointer(axis,2);
   *axis = hist->axis;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawHGGetDraw"
 /*@C
   PetscDrawHGGetDraw - Gets the draw context associated with a histogram.
 
@@ -622,17 +661,19 @@ PetscErrorCode  PetscDrawHGGetAxis(PetscDrawHG hist, PetscDrawAxis *axis)
 . hist - The histogram context
 
   Output Parameter:
-. win  - The draw context
+. draw  - The draw context
 
   Level: intermediate
 
+.seealso:  PetscDrawHGCreate(), PetscDrawHGAddValue(), PetscDrawHGView(), PetscDrawHGDraw(), PetscDrawHGSetColor(), PetscDrawAxis, PetscDrawHGSetLimits()
+
 @*/
-PetscErrorCode  PetscDrawHGGetDraw(PetscDrawHG hist, PetscDraw *win)
+PetscErrorCode  PetscDrawHGGetDraw(PetscDrawHG hist,PetscDraw *draw)
 {
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(hist, PETSC_DRAWHG_CLASSID,1);
-  PetscValidPointer(win,2);
-  *win = hist->win;
+  PetscValidHeaderSpecific(hist,PETSC_DRAWHG_CLASSID,1);
+  PetscValidPointer(draw,2);
+  *draw = hist->win;
   PetscFunctionReturn(0);
 }
 
