@@ -1,8 +1,6 @@
 
 #include <petscdraw.h>     /*I "petscdraw.h"  I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscDrawZoom"
 /*@C
     PetscDrawZoom - Allows one to create a graphic that users may zoom into.
 
