@@ -18,8 +18,6 @@
 /* Logging support */
 PetscClassId PETSC_RANDOM_CLASSID;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscRandomDestroy"
 /*@
    PetscRandomDestroy - Destroys a context that has been formed by
    PetscRandomCreate().
@@ -49,9 +47,7 @@ PetscErrorCode  PetscRandomDestroy(PetscRandom *r)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscRandomGetSeed"
-/*@
+/*@C
    PetscRandomGetSeed - Gets the random seed.
 
    Not collective
@@ -79,9 +75,7 @@ PetscErrorCode  PetscRandomGetSeed(PetscRandom r,unsigned long *seed)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscRandomSetSeed"
-/*@
+/*@C
    PetscRandomSetSeed - Sets the random seed. You MUST call PetscRandomSeed() after this call to have the new seed used.
 
    Not collective
@@ -115,8 +109,6 @@ PetscErrorCode  PetscRandomSetSeed(PetscRandom r,unsigned long seed)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "PetscRandomSetTypeFromOptions_Private"
 /*
   PetscRandomSetTypeFromOptions_Private - Sets the type of random generator from user options. Defaults to type PETSCRAND48 or PETSCRAND.
 
@@ -154,8 +146,6 @@ static PetscErrorCode PetscRandomSetTypeFromOptions_Private(PetscOptionItems *Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscRandomSetFromOptions"
 /*@
   PetscRandomSetFromOptions - Configures the random number generator from the options database.
 
@@ -169,7 +159,8 @@ static PetscErrorCode PetscRandomSetTypeFromOptions_Private(PetscOptionItems *Pe
 - -random_no_imaginary_part - makes the imaginary part of the random number zero, this is useful when you want the
                               same code to produce the same result when run with real numbers or complex numbers for regression testing purposes
 
-  Notes:  To see all options, run your program with the -help option.
+  Notes:
+    To see all options, run your program with the -help option.
           Must be called after PetscRandomCreate() but before the rnd is used.
 
   Level: beginner
@@ -220,8 +211,6 @@ PetscErrorCode  PetscRandomSetFromOptions(PetscRandom rnd)
 #if defined(PETSC_HAVE_SAWS)
 #include <petscviewersaws.h>
 #endif
-#undef __FUNCT__
-#define __FUNCT__ "PetscRandomView"
 /*@C
    PetscRandomView - Views a random number generator object.
 
@@ -271,7 +260,7 @@ PetscErrorCode  PetscRandomView(PetscRandom rnd,PetscViewer viewer)
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)rnd,viewer);CHKERRQ(ierr);
     ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)rnd),&rank);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] Random type %s, seed %D\n",rank,((PetscObject)rnd)->type_name,rnd->seed);CHKERRQ(ierr);
+    ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] Random type %s, seed %lu\n",rank,((PetscObject)rnd)->type_name,rnd->seed);CHKERRQ(ierr);
     ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPopSynchronized(viewer);CHKERRQ(ierr);
 #if defined(PETSC_HAVE_SAWS)
@@ -293,8 +282,6 @@ PetscErrorCode  PetscRandomView(PetscRandom rnd,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscRandomCreate"
 /*@
    PetscRandomCreate - Creates a context for generating random numbers,
    and initializes the random-number generator.
@@ -302,7 +289,7 @@ PetscErrorCode  PetscRandomView(PetscRandom rnd,PetscViewer viewer)
    Collective on MPI_Comm
 
    Input Parameters:
-+  comm - MPI communicator
+.  comm - MPI communicator
 
    Output Parameter:
 .  r  - the random number generator context
@@ -363,8 +350,6 @@ PetscErrorCode  PetscRandomCreate(MPI_Comm comm,PetscRandom *r)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscRandomSeed"
 /*@
    PetscRandomSeed - Seed the generator.
 
