@@ -4,8 +4,6 @@
 PetscFunctionList PetscRandomList              = NULL;
 PetscBool         PetscRandomRegisterAllCalled = PETSC_FALSE;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscRandomSetType"
 /*@C
   PetscRandomSetType - Builds a context for generating particular type of random numbers.
 
@@ -54,8 +52,6 @@ PetscErrorCode  PetscRandomSetType(PetscRandom rnd, PetscRandomType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscRandomGetType"
 /*@C
   PetscRandomGetType - Gets the type name (as a string) from the PetscRandom.
 
@@ -81,8 +77,6 @@ PetscErrorCode  PetscRandomGetType(PetscRandom rnd, PetscRandomType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscRandomRegister"
 /*@C
   PetscRandomRegister -  Adds a new PetscRandom component implementation
 
@@ -110,7 +104,8 @@ PetscErrorCode  PetscRandomGetType(PetscRandom rnd, PetscRandomType *type)
     -random_type my_random_name
 .ve
 
-  Notes: For an example of the code needed to interface your own random number generator see
+  Notes:
+    For an example of the code needed to interface your own random number generator see
          src/sys/random/impls/rand/rand.c
 
   Level: advanced
@@ -124,6 +119,7 @@ PetscErrorCode  PetscRandomRegister(const char sname[], PetscErrorCode (*functio
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = PetscRandomInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&PetscRandomList,sname,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -137,9 +133,11 @@ PETSC_EXTERN PetscErrorCode PetscRandomCreate_Rand48(PetscRandom);
 #if defined(PETSC_HAVE_SPRNG)
 PETSC_EXTERN PetscErrorCode PetscRandomCreate_Sprng(PetscRandom);
 #endif
+PETSC_EXTERN PetscErrorCode PetscRandomCreate_Rander48(PetscRandom);
+#if defined(PETSC_HAVE_RANDOM123)
+PETSC_EXTERN PetscErrorCode PetscRandomCreate_Random123(PetscRandom);
+#endif
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscRandomRegisterAll"
 /*@C
   PetscRandomRegisterAll - Registers all of the components in the PetscRandom package.
 
@@ -165,6 +163,10 @@ PetscErrorCode  PetscRandomRegisterAll(void)
 #endif
 #if defined(PETSC_HAVE_SPRNG)
   ierr = PetscRandomRegister(PETSCSPRNG, PetscRandomCreate_Sprng);CHKERRQ(ierr);
+#endif
+  ierr = PetscRandomRegister(PETSCRANDER48,PetscRandomCreate_Rander48);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_RANDOM123)
+  ierr = PetscRandomRegister(PETSCRANDOM123, PetscRandomCreate_Random123);CHKERRQ(ierr);
 #endif
   PetscFunctionReturn(0);
 }
