@@ -4,8 +4,6 @@ static char help[] = "Tests dynamic loading of viewer.\n\n";
 #include <petscsys.h>
 #include <petscviewer.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **args)
 {
   PetscErrorCode ierr;
@@ -19,8 +17,15 @@ int main(int argc,char **args)
   ierr = PetscViewerASCIIOpen(PETSC_COMM_WORLD,"stdout",&viewer);CHKERRQ(ierr);
   ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
 
 
+
+
+/*TEST
+
+   test:
+
+TEST*/
