@@ -3,8 +3,6 @@
 
 #define QUEUESTRINGSIZE 8192
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerFileClose_ASCII"
 static PetscErrorCode PetscViewerFileClose_ASCII(PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -22,12 +20,12 @@ static PetscErrorCode PetscViewerFileClose_ASCII(PetscViewer viewer)
     if (vascii->storecompressed) {
       char par[PETSC_MAX_PATH_LEN],buf[PETSC_MAX_PATH_LEN];
       FILE *fp;
-      ierr = PetscStrcpy(par,"gzip ");CHKERRQ(ierr);
-      ierr = PetscStrcat(par,vascii->filename);CHKERRQ(ierr);
+      ierr = PetscStrncpy(par,"gzip ",sizeof(par));CHKERRQ(ierr);
+      ierr = PetscStrlcat(par,vascii->filename,sizeof(par));CHKERRQ(ierr);
 #if defined(PETSC_HAVE_POPEN)
       ierr = PetscPOpen(PETSC_COMM_SELF,NULL,par,"r",&fp);CHKERRQ(ierr);
       if (fgets(buf,1024,fp)) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_LIB,"Error from compression command %s\n%s",par,buf);
-      ierr = PetscPClose(PETSC_COMM_SELF,fp,NULL);CHKERRQ(ierr);
+      ierr = PetscPClose(PETSC_COMM_SELF,fp);CHKERRQ(ierr);
 #else
       SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP_SYS,"Cannot run external programs on this machine");
 #endif
@@ -38,8 +36,6 @@ static PetscErrorCode PetscViewerFileClose_ASCII(PetscViewer viewer)
 }
 
 /* ----------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDestroy_ASCII"
 PetscErrorCode PetscViewerDestroy_ASCII(PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -54,16 +50,16 @@ PetscErrorCode PetscViewerDestroy_ASCII(PetscViewer viewer)
 
   /* remove the viewer from the list in the MPI Communicator */
   if (Petsc_Viewer_keyval == MPI_KEYVAL_INVALID) {
-    ierr = MPI_Keyval_create(MPI_NULL_COPY_FN,Petsc_DelViewer,&Petsc_Viewer_keyval,(void*)0);CHKERRQ(ierr);
+    ierr = MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN,Petsc_DelViewer,&Petsc_Viewer_keyval,(void*)0);CHKERRQ(ierr);
   }
 
-  ierr = MPI_Attr_get(PetscObjectComm((PetscObject)viewer),Petsc_Viewer_keyval,(void**)&vlink,(PetscMPIInt*)&flg);CHKERRQ(ierr);
+  ierr = MPI_Comm_get_attr(PetscObjectComm((PetscObject)viewer),Petsc_Viewer_keyval,(void**)&vlink,(PetscMPIInt*)&flg);CHKERRQ(ierr);
   if (flg) {
     if (vlink && vlink->viewer == viewer) {
       if (vlink->next) {
-        ierr = MPI_Attr_put(PetscObjectComm((PetscObject)viewer),Petsc_Viewer_keyval,vlink->next);CHKERRQ(ierr);
+        ierr = MPI_Comm_set_attr(PetscObjectComm((PetscObject)viewer),Petsc_Viewer_keyval,vlink->next);CHKERRQ(ierr);
       } else {
-        ierr = MPI_Attr_delete(PetscObjectComm((PetscObject)viewer),Petsc_Viewer_keyval);CHKERRQ(ierr);
+        ierr = MPI_Comm_delete_attr(PetscObjectComm((PetscObject)viewer),Petsc_Viewer_keyval);CHKERRQ(ierr);
       }
       ierr = PetscFree(vlink);CHKERRQ(ierr);
     } else {
@@ -77,74 +73,116 @@ PetscErrorCode PetscViewerDestroy_ASCII(PetscViewer viewer)
       }
     }
   }
+
+  if (Petsc_Viewer_Stdout_keyval != MPI_KEYVAL_INVALID) {
+    PetscViewer aviewer;
+    ierr = MPI_Comm_get_attr(PetscObjectComm((PetscObject)viewer),Petsc_Viewer_Stdout_keyval,(void**)&aviewer,(PetscMPIInt*)&flg);CHKERRQ(ierr);
+    if (flg && aviewer == viewer) {
+      ierr = MPI_Comm_delete_attr(PetscObjectComm((PetscObject)viewer),Petsc_Viewer_Stdout_keyval);CHKERRQ(ierr);
+    }
+  }
+  if (Petsc_Viewer_Stderr_keyval != MPI_KEYVAL_INVALID) {
+    PetscViewer aviewer;
+    ierr = MPI_Comm_get_attr(PetscObjectComm((PetscObject)viewer),Petsc_Viewer_Stderr_keyval,(void**)&aviewer,(PetscMPIInt*)&flg);CHKERRQ(ierr);
+    if (flg && aviewer == viewer) {
+      ierr = MPI_Comm_delete_attr(PetscObjectComm((PetscObject)viewer),Petsc_Viewer_Stderr_keyval);CHKERRQ(ierr);
+    }
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDestroy_ASCII_Singleton"
-PetscErrorCode PetscViewerDestroy_ASCII_Singleton(PetscViewer viewer)
+PetscErrorCode PetscViewerDestroy_ASCII_SubViewer(PetscViewer viewer)
 {
   PetscViewer_ASCII *vascii = (PetscViewer_ASCII*)viewer->data;
   PetscErrorCode    ierr;
 
   PetscFunctionBegin;
-  ierr = PetscViewerRestoreSingleton(vascii->bviewer,&viewer);CHKERRQ(ierr);
-  vascii->bviewer = NULL;
+  ierr = PetscViewerRestoreSubViewer(vascii->bviewer,0,&viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDestroy_ASCII_Subcomm"
-PetscErrorCode PetscViewerDestroy_ASCII_Subcomm(PetscViewer viewer)
-{
-  PetscViewer_ASCII *vascii = (PetscViewer_ASCII*)viewer->data;
-  PetscErrorCode    ierr;
-
-  PetscFunctionBegin;
-  ierr = PetscViewerRestoreSubcomm(vascii->subviewer,PetscObjectComm((PetscObject)viewer),&viewer);CHKERRQ(ierr);
-  vascii->subviewer = NULL;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerFlush_ASCII_Singleton_0"
-PetscErrorCode PetscViewerFlush_ASCII_Singleton_0(PetscViewer viewer)
-{
-  PetscViewer_ASCII *vascii = (PetscViewer_ASCII*)viewer->data;
-  int               err;
-
-  PetscFunctionBegin;
-  err = fflush(vascii->fd);
-  if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerFlush_ASCII"
 PetscErrorCode PetscViewerFlush_ASCII(PetscViewer viewer)
 {
-  PetscMPIInt       rank;
   PetscErrorCode    ierr;
   PetscViewer_ASCII *vascii = (PetscViewer_ASCII*)viewer->data;
   int               err;
+  MPI_Comm          comm;
+  PetscMPIInt       rank,size;
+  FILE              *fd = vascii->fd;
 
   PetscFunctionBegin;
-  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)viewer),&rank);CHKERRQ(ierr);
-  /* fflush() fails on OSX for read-only descriptors */
-  if (!rank && (vascii->mode != FILE_MODE_READ)) {
+  ierr = PetscObjectGetComm((PetscObject)viewer,&comm);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
+
+  if (!vascii->bviewer && !rank && (vascii->mode != FILE_MODE_READ)) {
     err = fflush(vascii->fd);
     if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() call failed");
   }
 
   if (vascii->allowsynchronized) {
-    /* Also flush anything printed with PetscViewerASCIISynchronizedPrintf()  */
-    ierr = PetscSynchronizedFlush(PetscObjectComm((PetscObject)viewer),vascii->fd);CHKERRQ(ierr);
+    PetscMPIInt   tag,i,j,n = 0,dummy = 0;
+    char          *message;
+    MPI_Status    status;
+
+    ierr = PetscCommDuplicate(comm,&comm,&tag);CHKERRQ(ierr);
+
+    /* First processor waits for messages from all other processors */
+    if (!rank) {
+      /* flush my own messages that I may have queued up */
+      PrintfQueue next = vascii->petsc_printfqueuebase,previous;
+      for (i=0; i<vascii->petsc_printfqueuelength; i++) {
+        if (!vascii->bviewer) {
+          ierr = PetscFPrintf(comm,fd,"%s",next->string);CHKERRQ(ierr);
+        } else {
+          ierr = PetscViewerASCIISynchronizedPrintf(vascii->bviewer,"%s",next->string);CHKERRQ(ierr);
+        }
+        previous = next;
+        next     = next->next;
+        ierr     = PetscFree(previous->string);CHKERRQ(ierr);
+        ierr     = PetscFree(previous);CHKERRQ(ierr);
+      }
+      vascii->petsc_printfqueue       = 0;
+      vascii->petsc_printfqueuelength = 0;
+      for (i=1; i<size; i++) {
+        /* to prevent a flood of messages to process zero, request each message separately */
+        ierr = MPI_Send(&dummy,1,MPI_INT,i,tag,comm);CHKERRQ(ierr);
+        ierr = MPI_Recv(&n,1,MPI_INT,i,tag,comm,&status);CHKERRQ(ierr);
+        for (j=0; j<n; j++) {
+          PetscMPIInt size = 0;
+
+          ierr = MPI_Recv(&size,1,MPI_INT,i,tag,comm,&status);CHKERRQ(ierr);
+          ierr = PetscMalloc1(size, &message);CHKERRQ(ierr);
+          ierr = MPI_Recv(message,size,MPI_CHAR,i,tag,comm,&status);CHKERRQ(ierr);
+          if (!vascii->bviewer) {
+            ierr = PetscFPrintf(comm,fd,"%s",message);CHKERRQ(ierr);
+          } else {
+            ierr = PetscViewerASCIISynchronizedPrintf(vascii->bviewer,"%s",message);CHKERRQ(ierr);
+          }
+          ierr = PetscFree(message);CHKERRQ(ierr);
+        }
+      }
+    } else { /* other processors send queue to processor 0 */
+      PrintfQueue next = vascii->petsc_printfqueuebase,previous;
+
+      ierr = MPI_Recv(&dummy,1,MPI_INT,0,tag,comm,&status);CHKERRQ(ierr);
+      ierr = MPI_Send(&vascii->petsc_printfqueuelength,1,MPI_INT,0,tag,comm);CHKERRQ(ierr);
+      for (i=0; i<vascii->petsc_printfqueuelength; i++) {
+        ierr     = MPI_Send(&next->size,1,MPI_INT,0,tag,comm);CHKERRQ(ierr);
+        ierr     = MPI_Send(next->string,next->size,MPI_CHAR,0,tag,comm);CHKERRQ(ierr);
+        previous = next;
+        next     = next->next;
+        ierr     = PetscFree(previous->string);CHKERRQ(ierr);
+        ierr     = PetscFree(previous);CHKERRQ(ierr);
+      }
+      vascii->petsc_printfqueue       = 0;
+      vascii->petsc_printfqueuelength = 0;
+    }
+    ierr = PetscCommDestroy(&comm);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerASCIIGetPointer"
 /*@C
     PetscViewerASCIIGetPointer - Extracts the file pointer from an ASCII PetscViewer.
 
@@ -173,8 +211,6 @@ PetscErrorCode  PetscViewerASCIIGetPointer(PetscViewer viewer,FILE **fd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerFileGetMode_ASCII"
 PetscErrorCode  PetscViewerFileGetMode_ASCII(PetscViewer viewer, PetscFileMode *mode)
 {
   PetscViewer_ASCII *vascii = (PetscViewer_ASCII*)viewer->data;
@@ -184,8 +220,6 @@ PetscErrorCode  PetscViewerFileGetMode_ASCII(PetscViewer viewer, PetscFileMode *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerFileSetMode_ASCII"
 PetscErrorCode  PetscViewerFileSetMode_ASCII(PetscViewer viewer, PetscFileMode mode)
 {
   PetscViewer_ASCII *vascii = (PetscViewer_ASCII*)viewer->data;
@@ -199,17 +233,15 @@ PetscErrorCode  PetscViewerFileSetMode_ASCII(PetscViewer viewer, PetscFileMode m
    If petsc_history is on, then all Petsc*Printf() results are saved
    if the appropriate (usually .petschistory) file.
 */
-extern FILE *petsc_history;
+PETSC_INTERN FILE *petsc_history;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerASCIISetTab"
 /*@
     PetscViewerASCIISetTab - Causes PetscViewer to tab in a number of times
 
     Not Collective, but only first processor in set has any effect
 
     Input Parameters:
-+    viewer - optained with PetscViewerASCIIOpen()
++    viewer - obtained with PetscViewerASCIIOpen()
 -    tabs - number of tabs
 
     Level: developer
@@ -237,15 +269,13 @@ PetscErrorCode  PetscViewerASCIISetTab(PetscViewer viewer,PetscInt tabs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerASCIIGetTab"
 /*@
     PetscViewerASCIIGetTab - Return the number of tabs used by PetscViewer.
 
     Not Collective, meaningful on first processor only.
 
     Input Parameters:
-.    viewer - optained with PetscViewerASCIIOpen()
+.    viewer - obtained with PetscViewerASCIIOpen()
     Output Parameters:
 .    tabs - number of tabs
 
@@ -274,15 +304,13 @@ PetscErrorCode  PetscViewerASCIIGetTab(PetscViewer viewer,PetscInt *tabs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerASCIIAddTab"
 /*@
     PetscViewerASCIIAddTab - Add to the number of times an ASCII viewer tabs before printing
 
     Not Collective, but only first processor in set has any effect
 
     Input Parameters:
-+    viewer - optained with PetscViewerASCIIOpen()
++    viewer - obtained with PetscViewerASCIIOpen()
 -    tabs - number of tabs
 
     Level: developer
@@ -310,15 +338,13 @@ PetscErrorCode  PetscViewerASCIIAddTab(PetscViewer viewer,PetscInt tabs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerASCIISubtractTab"
 /*@
     PetscViewerASCIISubtractTab - Subtracts from the number of times an ASCII viewer tabs before printing
 
     Not Collective, but only first processor in set has any effect
 
     Input Parameters:
-+    viewer - optained with PetscViewerASCIIOpen()
++    viewer - obtained with PetscViewerASCIIOpen()
 -    tabs - number of tabs
 
     Level: developer
@@ -346,27 +372,24 @@ PetscErrorCode  PetscViewerASCIISubtractTab(PetscViewer viewer,PetscInt tabs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerASCIISynchronizedAllow"
 /*@C
-    PetscViewerASCIISynchronizedAllow - Allows calls to PetscViewerASCIISynchronizedPrintf() for this viewer
+    PetscViewerASCIIPushSynchronized - Allows calls to PetscViewerASCIISynchronizedPrintf() for this viewer
 
     Collective on PetscViewer
 
     Input Parameters:
-+    viewer - optained with PetscViewerASCIIOpen()
--    allow - PETSC_TRUE to allow the synchronized printing
+.    viewer - obtained with PetscViewerASCIIOpen()
 
     Level: intermediate
 
   Concepts: PetscViewerASCII^formating
   Concepts: tab^setting
 
-.seealso: PetscPrintf(), PetscSynchronizedPrintf(), PetscViewerASCIIPrintf(),
+.seealso: PetscViewerASCIIPopSynchronized(), PetscPrintf(), PetscSynchronizedPrintf(), PetscViewerASCIIPrintf(),
           PetscViewerASCIIPopTab(), PetscViewerASCIISynchronizedPrintf(), PetscViewerASCIIOpen(),
           PetscViewerCreate(), PetscViewerDestroy(), PetscViewerSetType(), PetscViewerASCIIGetPointer()
 @*/
-PetscErrorCode  PetscViewerASCIISynchronizedAllow(PetscViewer viewer,PetscBool allow)
+PetscErrorCode  PetscViewerASCIIPushSynchronized(PetscViewer viewer)
 {
   PetscViewer_ASCII *ascii = (PetscViewer_ASCII*)viewer->data;
   PetscBool         iascii;
@@ -375,20 +398,51 @@ PetscErrorCode  PetscViewerASCIISynchronizedAllow(PetscViewer viewer,PetscBool a
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
-  if (iascii) ascii->allowsynchronized = allow;
+  if (iascii) ascii->allowsynchronized++;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerASCIIPushTab"
-/*@
+/*@C
+    PetscViewerASCIIPopSynchronized - Undoes most recent PetscViewerASCIIPushSynchronized() for this viewer
+
+    Collective on PetscViewer
+
+    Input Parameters:
+.    viewer - obtained with PetscViewerASCIIOpen()
+
+    Level: intermediate
+
+  Concepts: PetscViewerASCII^formating
+  Concepts: tab^setting
+
+.seealso: PetscViewerASCIIPushSynchronized(), PetscPrintf(), PetscSynchronizedPrintf(), PetscViewerASCIIPrintf(),
+          PetscViewerASCIIPopTab(), PetscViewerASCIISynchronizedPrintf(), PetscViewerASCIIOpen(),
+          PetscViewerCreate(), PetscViewerDestroy(), PetscViewerSetType(), PetscViewerASCIIGetPointer()
+@*/
+PetscErrorCode  PetscViewerASCIIPopSynchronized(PetscViewer viewer)
+{
+  PetscViewer_ASCII *ascii = (PetscViewer_ASCII*)viewer->data;
+  PetscBool         iascii;
+  PetscErrorCode    ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
+  if (iascii) {
+    ascii->allowsynchronized--;
+    if (ascii->allowsynchronized < 0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Called more times than PetscViewerASCIIPushSynchronized()");
+  }
+  PetscFunctionReturn(0);
+}
+
+/*@C
     PetscViewerASCIIPushTab - Adds one more tab to the amount that PetscViewerASCIIPrintf()
      lines are tabbed.
 
     Not Collective, but only first processor in set has any effect
 
     Input Parameters:
-.    viewer - optained with PetscViewerASCIIOpen()
+.    viewer - obtained with PetscViewerASCIIOpen()
 
     Level: developer
 
@@ -415,16 +469,14 @@ PetscErrorCode  PetscViewerASCIIPushTab(PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerASCIIPopTab"
-/*@
+/*@C
     PetscViewerASCIIPopTab - Removes one tab from the amount that PetscViewerASCIIPrintf()
      lines are tabbed.
 
     Not Collective, but only first processor in set has any effect
 
     Input Parameters:
-.    viewer - optained with PetscViewerASCIIOpen()
+.    viewer - obtained with PetscViewerASCIIOpen()
 
     Level: developer
 
@@ -454,15 +506,13 @@ PetscErrorCode  PetscViewerASCIIPopTab(PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerASCIIUseTabs"
 /*@
     PetscViewerASCIIUseTabs - Turns on or off the use of tabs with the ASCII PetscViewer
 
     Not Collective, but only first processor in set has any effect
 
     Input Parameters:
-+    viewer - optained with PetscViewerASCIIOpen()
++    viewer - obtained with PetscViewerASCIIOpen()
 -    flg - PETSC_TRUE or PETSC_FALSE
 
     Level: developer
@@ -498,10 +548,7 @@ PetscErrorCode  PetscViewerASCIIUseTabs(PetscViewer viewer,PetscBool flg)
 
 /* ----------------------------------------------------------------------- */
 
-#include <../src/sys/fileio/mprint.h> /* defines the queue datastructures and variables */
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerASCIIPrintf"
 /*@C
     PetscViewerASCIIPrintf - Prints to a file, only from the first
     processor in the PetscViewer
@@ -509,7 +556,7 @@ PetscErrorCode  PetscViewerASCIIUseTabs(PetscViewer viewer,PetscBool flg)
     Not Collective, but only first processor in set has any effect
 
     Input Parameters:
-+    viewer - optained with PetscViewerASCIIOpen()
++    viewer - obtained with PetscViewerASCIIOpen()
 -    format - the usual printf() format string
 
     Level: developer
@@ -524,7 +571,7 @@ PetscErrorCode  PetscViewerASCIIUseTabs(PetscViewer viewer,PetscBool flg)
 
 .seealso: PetscPrintf(), PetscSynchronizedPrintf(), PetscViewerASCIIOpen(),
           PetscViewerASCIIPushTab(), PetscViewerASCIIPopTab(), PetscViewerASCIISynchronizedPrintf(),
-          PetscViewerCreate(), PetscViewerDestroy(), PetscViewerSetType(), PetscViewerASCIIGetPointer(), PetscViewerASCIISynchronizedAllow()
+          PetscViewerCreate(), PetscViewerDestroy(), PetscViewerSetType(), PetscViewerASCIIGetPointer(), PetscViewerASCIIPushSynchronized()
 @*/
 PetscErrorCode  PetscViewerASCIIPrintf(PetscViewer viewer,const char format[],...)
 {
@@ -533,7 +580,7 @@ PetscErrorCode  PetscViewerASCIIPrintf(PetscViewer viewer,const char format[],..
   PetscInt          tab,intab = ascii->tab;
   PetscErrorCode    ierr;
   FILE              *fd = ascii->fd;
-  PetscBool         iascii,issingleton = PETSC_FALSE;
+  PetscBool         iascii;
   int               err;
 
   PetscFunctionBegin;
@@ -541,16 +588,34 @@ PetscErrorCode  PetscViewerASCIIPrintf(PetscViewer viewer,const char format[],..
   PetscValidCharPointer(format,2);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (!iascii) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Not ASCII PetscViewer");
-  if (ascii->bviewer) {
-    viewer      = ascii->bviewer;
-    ascii       = (PetscViewer_ASCII*)viewer->data;
-    fd          = ascii->fd;
-    issingleton = PETSC_TRUE;
-  }
-
   ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)viewer),&rank);CHKERRQ(ierr);
-  if (!rank) {
+  if (rank) PetscFunctionReturn(0);
+
+  if (ascii->bviewer) { /* pass string up to parent viewer */
+    char        *string;
+    va_list     Argp;
+    size_t      fullLength;
+
+    ierr = PetscCalloc1(QUEUESTRINGSIZE, &string);CHKERRQ(ierr);
+    va_start(Argp,format);
+    ierr = PetscVSNPrintf(string,QUEUESTRINGSIZE,format,&fullLength,Argp);CHKERRQ(ierr);
+    va_end(Argp);
+    ierr = PetscViewerASCIISynchronizedPrintf(viewer,"%s",string);CHKERRQ(ierr);
+    ierr = PetscFree(string);CHKERRQ(ierr);
+  } else { /* write directly to file */
     va_list Argp;
+    /* flush my own messages that I may have queued up */
+    PrintfQueue next = ascii->petsc_printfqueuebase,previous;
+    PetscInt    i;
+    for (i=0; i<ascii->petsc_printfqueuelength; i++) {
+      ierr = PetscFPrintf(PETSC_COMM_SELF,fd,"%s",next->string);CHKERRQ(ierr);
+      previous = next;
+      next     = next->next;
+      ierr     = PetscFree(previous->string);CHKERRQ(ierr);
+      ierr     = PetscFree(previous);CHKERRQ(ierr);
+    }
+    ascii->petsc_printfqueue       = 0;
+    ascii->petsc_printfqueuelength = 0;
     tab = intab;
     while (tab--) {
       ierr = PetscFPrintf(PETSC_COMM_SELF,fd,"  ");CHKERRQ(ierr);
@@ -571,37 +636,10 @@ PetscErrorCode  PetscViewerASCIIPrintf(PetscViewer viewer,const char format[],..
       if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");
     }
     va_end(Argp);
-  } else if (issingleton) {
-    char        *string;
-    va_list     Argp;
-    size_t      fullLength;
-    PrintfQueue next;
-
-    ierr = PetscNew(&next);CHKERRQ(ierr);
-    if (petsc_printfqueue) {
-      petsc_printfqueue->next = next;
-      petsc_printfqueue       = next;
-    } else {
-      petsc_printfqueuebase = petsc_printfqueue = next;
-    }
-    petsc_printfqueuelength++;
-    next->size = QUEUESTRINGSIZE;
-    ierr       = PetscCalloc1(next->size, &next->string);CHKERRQ(ierr);
-    string     = next->string;
-    tab        = intab;
-    tab       *= 2;
-    while (tab--) {
-      *string++ = ' ';
-    }
-    va_start(Argp,format);
-    ierr = PetscVSNPrintf(string,next->size-2*ascii->tab,format,&fullLength,Argp);CHKERRQ(ierr);
-    va_end(Argp);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerFileSetName"
 /*@C
      PetscViewerFileSetName - Sets the name of the file the PetscViewer uses.
 
@@ -620,16 +658,16 @@ PetscErrorCode  PetscViewerASCIIPrintf(PetscViewer viewer,const char format[],..
 PetscErrorCode  PetscViewerFileSetName(PetscViewer viewer,const char name[])
 {
   PetscErrorCode ierr;
+  char           b[PETSC_MAX_PATH_LEN];
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
   PetscValidCharPointer(name,2);
-  ierr = PetscTryMethod(viewer,"PetscViewerFileSetName_C",(PetscViewer,const char[]),(viewer,name));CHKERRQ(ierr);
+  ierr = PetscStrreplace(PetscObjectComm((PetscObject)viewer),name,b,sizeof(b));CHKERRQ(ierr);
+  ierr = PetscTryMethod(viewer,"PetscViewerFileSetName_C",(PetscViewer,const char[]),(viewer,b));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerFileGetName"
 /*@C
      PetscViewerFileGetName - Gets the name of the file the PetscViewer uses.
 
@@ -652,12 +690,11 @@ PetscErrorCode  PetscViewerFileGetName(PetscViewer viewer,const char **name)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
-  ierr = PetscTryMethod(viewer,"PetscViewerFileGetName_C",(PetscViewer,const char**),(viewer,name));CHKERRQ(ierr);
+  PetscValidPointer(name,2);
+  ierr = PetscUseMethod(viewer,"PetscViewerFileGetName_C",(PetscViewer,const char**),(viewer,name));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerFileGetName_ASCII"
 PetscErrorCode  PetscViewerFileGetName_ASCII(PetscViewer viewer,const char **name)
 {
   PetscViewer_ASCII *vascii = (PetscViewer_ASCII*)viewer->data;
@@ -667,8 +704,6 @@ PetscErrorCode  PetscViewerFileGetName_ASCII(PetscViewer viewer,const char **nam
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerFileSetName_ASCII"
 PetscErrorCode  PetscViewerFileSetName_ASCII(PetscViewer viewer,const char name[])
 {
   PetscErrorCode    ierr;
@@ -690,6 +725,7 @@ PetscErrorCode  PetscViewerFileSetName_ASCII(PetscViewer viewer,const char name[
   if (gz) {
     ierr = PetscStrlen(gz,&len);CHKERRQ(ierr);
     if (len == 3) {
+      if (vascii->mode != FILE_MODE_WRITE) SETERRQ(PetscObjectComm((PetscObject)viewer),PETSC_ERR_SUP,"Cannot open ASCII PetscViewer file that is compressed; uncompress it manually first");
       *gz = 0;
       vascii->storecompressed = PETSC_TRUE;
     }
@@ -742,114 +778,48 @@ PetscErrorCode  PetscViewerFileSetName_ASCII(PetscViewer viewer,const char name[
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerGetSingleton_ASCII"
-PetscErrorCode PetscViewerGetSingleton_ASCII(PetscViewer viewer,PetscViewer *outviewer)
+PetscErrorCode PetscViewerGetSubViewer_ASCII(PetscViewer viewer,MPI_Comm subcomm,PetscViewer *outviewer)
 {
   PetscMPIInt       rank;
   PetscErrorCode    ierr;
   PetscViewer_ASCII *vascii = (PetscViewer_ASCII*)viewer->data,*ovascii;
-  const char        *name;
 
   PetscFunctionBegin;
-  if (vascii->sviewer) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Singleton already obtained from PetscViewer and not restored");
-  ierr         = PetscViewerCreate(PETSC_COMM_SELF,outviewer);CHKERRQ(ierr);
+  if (vascii->sviewer) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"SubViewer already obtained from PetscViewer and not restored");
+  ierr         = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
+  ierr         = PetscViewerCreate(subcomm,outviewer);CHKERRQ(ierr);
   ierr         = PetscViewerSetType(*outviewer,PETSCVIEWERASCII);CHKERRQ(ierr);
+  ierr         = PetscViewerASCIIPushSynchronized(*outviewer);CHKERRQ(ierr);
   ovascii      = (PetscViewer_ASCII*)(*outviewer)->data;
   ovascii->fd  = vascii->fd;
   ovascii->tab = vascii->tab;
+  ovascii->closefile = PETSC_FALSE;
 
   vascii->sviewer = *outviewer;
 
   (*outviewer)->format  = viewer->format;
 
-  ierr = PetscObjectGetName((PetscObject)viewer,&name);CHKERRQ(ierr);
-  ierr = PetscObjectSetName((PetscObject)(*outviewer),name);CHKERRQ(ierr);
-
   ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)viewer),&rank);CHKERRQ(ierr);
   ((PetscViewer_ASCII*)((*outviewer)->data))->bviewer = viewer;
-  (*outviewer)->ops->destroy = PetscViewerDestroy_ASCII_Singleton;
-  if (rank) (*outviewer)->ops->flush = 0;
-  else      (*outviewer)->ops->flush = PetscViewerFlush_ASCII_Singleton_0;
+  (*outviewer)->ops->destroy = PetscViewerDestroy_ASCII_SubViewer;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerRestoreSingleton_ASCII"
-PetscErrorCode PetscViewerRestoreSingleton_ASCII(PetscViewer viewer,PetscViewer *outviewer)
+PetscErrorCode PetscViewerRestoreSubViewer_ASCII(PetscViewer viewer,MPI_Comm comm,PetscViewer *outviewer)
 {
   PetscErrorCode    ierr;
-  PetscViewer_ASCII *vascii = (PetscViewer_ASCII*)(*outviewer)->data;
   PetscViewer_ASCII *ascii  = (PetscViewer_ASCII*)viewer->data;
 
   PetscFunctionBegin;
-  if (!ascii->sviewer) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Singleton never obtained from PetscViewer");
-  if (ascii->sviewer != *outviewer) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"This PetscViewer did not generate singleton");
+  if (!ascii->sviewer) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"SubViewer never obtained from PetscViewer");
+  if (ascii->sviewer != *outviewer) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"This PetscViewer did not generate this SubViewer");
 
   ascii->sviewer             = 0;
-  vascii->fd                 = PETSC_STDOUT;
   (*outviewer)->ops->destroy = PetscViewerDestroy_ASCII;
   ierr                       = PetscViewerDestroy(outviewer);CHKERRQ(ierr);
-  ierr                       = PetscViewerFlush(viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerGetSubcomm_ASCII"
-PetscErrorCode PetscViewerGetSubcomm_ASCII(PetscViewer viewer,MPI_Comm subcomm,PetscViewer *outviewer)
-{
-  PetscErrorCode    ierr;
-  PetscViewer_ASCII *vascii = (PetscViewer_ASCII*)viewer->data,*ovascii;
-  const char        *name;
-
-  PetscFunctionBegin;
-  if (vascii->sviewer) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Subcomm viewer already obtained from PetscViewer and not restored");
-  /* Note that we need to open vascii->filename for the subcomm:
-     we can't count on reusing viewer's fd since the root in comm and subcomm may differ.
-     Further, if the subcomm happens to be the same as comm, PetscViewerASCIIOpen() will
-     will return the current viewer, having increfed it.
-   */
-  ierr = PetscViewerASCIIOpen(subcomm,vascii->filename, outviewer);CHKERRQ(ierr);
-  if (*outviewer == viewer) PetscFunctionReturn(0);
-  ovascii = (PetscViewer_ASCII*)(*outviewer)->data;
-
-  ovascii->tab    = vascii->tab;
-  vascii->sviewer = *outviewer;
-
-  (*outviewer)->format  = viewer->format;
-
-  ierr = PetscObjectGetName((PetscObject)viewer,&name);CHKERRQ(ierr);
-  ierr = PetscObjectSetName((PetscObject)(*outviewer),name);CHKERRQ(ierr);
-
-  ((PetscViewer_ASCII*)((*outviewer)->data))->subviewer = viewer;
-
-  (*outviewer)->ops->destroy = PetscViewerDestroy_ASCII_Subcomm;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerRestoreSubcomm_ASCII"
-PetscErrorCode PetscViewerRestoreSubcomm_ASCII(PetscViewer viewer,MPI_Comm subcomm,PetscViewer *outviewer)
-{
-  PetscErrorCode    ierr;
-  PetscViewer_ASCII *oascii = (PetscViewer_ASCII*)(*outviewer)->data;
-  PetscViewer_ASCII *ascii  = (PetscViewer_ASCII*)viewer->data;
-
-  PetscFunctionBegin;
-  if (!ascii->sviewer) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Subcomm never obtained from PetscViewer");
-  if (ascii->sviewer != *outviewer) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"The given PetscViewer did not generate this subcomm viewer");
-
-  ascii->sviewer             = 0;
-  oascii->fd                 = PETSC_STDOUT;
-  (*outviewer)->ops->destroy = PetscViewerDestroy_ASCII;
-
-  ierr = PetscViewerDestroy(outviewer);CHKERRQ(ierr);
-  ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerView_ASCII"
 PetscErrorCode  PetscViewerView_ASCII(PetscViewer v,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -862,8 +832,17 @@ PetscErrorCode  PetscViewerView_ASCII(PetscViewer v,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerCreate_ASCII"
+/*MC
+   PETSCVIEWERASCII - A viewer that prints to stdout or an ASCII file
+
+
+.seealso:  PETSC_VIEWER_STDOUT_(),PETSC_VIEWER_STDOUT_SELF, PETSC_VIEWER_STDOUT_WORLD,PetscViewerCreate(), PetscViewerASCIIOpen(),
+           PetscViewerMatlabOpen(), VecView(), DMView(), PetscViewerMatlabPutArray(), PETSCVIEWERBINARY, PETSCVIEWERMATLAB,
+           PetscViewerFileSetName(), PetscViewerFileSetMode(), PetscViewerFormat, PetscViewerType, PetscViewerSetType()
+
+  Level: beginner
+
+M*/
 PETSC_EXTERN PetscErrorCode PetscViewerCreate_ASCII(PetscViewer viewer)
 {
   PetscViewer_ASCII *vascii;
@@ -875,10 +854,8 @@ PETSC_EXTERN PetscErrorCode PetscViewerCreate_ASCII(PetscViewer viewer)
 
   viewer->ops->destroy          = PetscViewerDestroy_ASCII;
   viewer->ops->flush            = PetscViewerFlush_ASCII;
-  viewer->ops->getsingleton     = PetscViewerGetSingleton_ASCII;
-  viewer->ops->restoresingleton = PetscViewerRestoreSingleton_ASCII;
-  viewer->ops->getsubcomm       = PetscViewerGetSubcomm_ASCII;
-  viewer->ops->restoresubcomm   = PetscViewerRestoreSubcomm_ASCII;
+  viewer->ops->getsubviewer     = PetscViewerGetSubViewer_ASCII;
+  viewer->ops->restoresubviewer = PetscViewerRestoreSubViewer_ASCII;
   viewer->ops->view             = PetscViewerView_ASCII;
   viewer->ops->read             = PetscViewerASCIIRead;
 
@@ -900,9 +877,6 @@ PETSC_EXTERN PetscErrorCode PetscViewerCreate_ASCII(PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerASCIISynchronizedPrintf"
 /*@C
     PetscViewerASCIISynchronizedPrintf - Prints synchronized output to the specified file from
     several processors.  Output of the first processor is followed by that of the
@@ -916,25 +890,26 @@ PETSC_EXTERN PetscErrorCode PetscViewerCreate_ASCII(PetscViewer viewer)
 
     Level: intermediate
 
-    Notes: You must have previously called PetscViewerASCIISynchronizeAllow() to allow this routine to be called.
+    Notes:
+    You must have previously called PetscViewerASCIISynchronizeAllow() to allow this routine to be called.
 
     Fortran Note:
       Can only print a single character* string
 
 .seealso: PetscSynchronizedPrintf(), PetscSynchronizedFlush(), PetscFPrintf(),
           PetscFOpen(), PetscViewerFlush(), PetscViewerASCIIGetPointer(), PetscViewerDestroy(), PetscViewerASCIIOpen(),
-          PetscViewerASCIIPrintf(), PetscViewerASCIISynchronizedAllow()
+          PetscViewerASCIIPrintf(), PetscViewerASCIIPushSynchronized()
 
 @*/
 PetscErrorCode  PetscViewerASCIISynchronizedPrintf(PetscViewer viewer,const char format[],...)
 {
   PetscViewer_ASCII *vascii = (PetscViewer_ASCII*)viewer->data;
   PetscErrorCode    ierr;
-  PetscMPIInt       rank,size;
+  PetscMPIInt       rank;
   PetscInt          tab = vascii->tab;
   MPI_Comm          comm;
   FILE              *fp;
-  PetscBool         iascii;
+  PetscBool         iascii,hasbviewer = PETSC_FALSE;
   int               err;
 
   PetscFunctionBegin;
@@ -942,17 +917,36 @@ PetscErrorCode  PetscViewerASCIISynchronizedPrintf(PetscViewer viewer,const char
   PetscValidCharPointer(format,2);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (!iascii) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Not ASCII PetscViewer");
-  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)viewer),&size);CHKERRQ(ierr);
-  if (size > 1 && !vascii->allowsynchronized) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"First call PetscViewerASCIISynchronizedAllow() to allow this call");
-  if (!viewer->ops->flush) PetscFunctionReturn(0); /* This viewer obtained via PetscViewerGetSubcomm_ASCII(), should not participate. */
+  if (!vascii->allowsynchronized) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"First call PetscViewerASCIIPushSynchronized() to allow this call");
 
   ierr = PetscObjectGetComm((PetscObject)viewer,&comm);CHKERRQ(ierr);
-  fp   = vascii->fd;
   ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
 
-  /* First processor prints immediately to fp */
-  if (!rank) {
+  if (vascii->bviewer) {
+    hasbviewer = PETSC_TRUE;
+    if (!rank) {
+      vascii = (PetscViewer_ASCII*)vascii->bviewer->data;
+      ierr = PetscObjectGetComm((PetscObject)viewer,&comm);CHKERRQ(ierr);
+      ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
+    }
+  }
+
+  fp   = vascii->fd;
+
+  if (!rank && !hasbviewer) {   /* First processor prints immediately to fp */
     va_list Argp;
+    /* flush my own messages that I may have queued up */
+    PrintfQueue next = vascii->petsc_printfqueuebase,previous;
+    PetscInt    i;
+    for (i=0; i<vascii->petsc_printfqueuelength; i++) {
+      ierr = PetscFPrintf(comm,fp,"%s",next->string);CHKERRQ(ierr);
+      previous = next;
+      next     = next->next;
+      ierr     = PetscFree(previous->string);CHKERRQ(ierr);
+      ierr     = PetscFree(previous);CHKERRQ(ierr);
+    }
+    vascii->petsc_printfqueue       = 0;
+    vascii->petsc_printfqueuelength = 0;
 
     while (tab--) {
       ierr = PetscFPrintf(PETSC_COMM_SELF,fp,"  ");CHKERRQ(ierr);
@@ -969,20 +963,20 @@ PetscErrorCode  PetscViewerASCIISynchronizedPrintf(PetscViewer viewer,const char
       if (err) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SYS,"fflush() failed on file");
     }
     va_end(Argp);
-  } else { /* other processors add to local queue */
+  } else { /* other processors add to queue */
     char        *string;
     va_list     Argp;
     size_t      fullLength;
     PrintfQueue next;
 
     ierr = PetscNew(&next);CHKERRQ(ierr);
-    if (petsc_printfqueue) {
-      petsc_printfqueue->next = next;
-      petsc_printfqueue       = next;
+    if (vascii->petsc_printfqueue) {
+      vascii->petsc_printfqueue->next = next;
+      vascii->petsc_printfqueue       = next;
     } else {
-      petsc_printfqueuebase = petsc_printfqueue = next;
+      vascii->petsc_printfqueuebase = vascii->petsc_printfqueue = next;
     }
-    petsc_printfqueuelength++;
+    vascii->petsc_printfqueuelength++;
     next->size = QUEUESTRINGSIZE;
     ierr       = PetscMalloc1(next->size, &next->string);CHKERRQ(ierr);
     ierr       = PetscMemzero(next->string,next->size);CHKERRQ(ierr);
@@ -994,12 +988,24 @@ PetscErrorCode  PetscViewerASCIISynchronizedPrintf(PetscViewer viewer,const char
     va_start(Argp,format);
     ierr = PetscVSNPrintf(string,next->size-2*vascii->tab,format,&fullLength,Argp);CHKERRQ(ierr);
     va_end(Argp);
+    if (fullLength > (size_t) (next->size-2*vascii->tab)) {
+      ierr       = PetscFree(next->string);CHKERRQ(ierr);
+      next->size = fullLength + 2*vascii->tab;
+      ierr       = PetscMalloc1(next->size, &next->string);CHKERRQ(ierr);
+      ierr       = PetscMemzero(next->string,next->size);CHKERRQ(ierr);
+      string     = next->string;
+      tab        = 2*vascii->tab;
+      while (tab--) {
+        *string++ = ' ';
+      }
+      va_start(Argp,format);
+      ierr = PetscVSNPrintf(string,next->size-2*vascii->tab,format,NULL,Argp);CHKERRQ(ierr);
+      va_end(Argp);
+    }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerASCIIRead"
 /*@C
    PetscViewerASCIIRead - Reads from am ASCII file
 
@@ -1018,9 +1024,9 @@ PetscErrorCode  PetscViewerASCIISynchronizedPrintf(PetscViewer viewer,const char
 
    Concepts: ascii files
 
-.seealso: PetscViewerASCIIOpen(), PetscViewerSetFormat(), PetscViewerDestroy(),
+.seealso: PetscViewerASCIIOpen(), PetscViewerPushFormat(), PetscViewerDestroy(),
           VecView(), MatView(), VecLoad(), MatLoad(), PetscViewerBinaryGetDescriptor(),
-          PetscViewerBinaryGetInfoPointer(), PetscFileMode, PetscViewer, PetscBinaryViewerRead()
+          PetscViewerBinaryGetInfoPointer(), PetscFileMode, PetscViewer, PetscViewerBinaryRead()
 @*/
 PetscErrorCode PetscViewerASCIIRead(PetscViewer viewer,void *data,PetscInt num,PetscInt *count,PetscDataType dtype)
 {
@@ -1034,18 +1040,17 @@ PetscErrorCode PetscViewerASCIIRead(PetscViewer viewer,void *data,PetscInt num,P
   for (i=0; i<num; i++) {
     if (dtype == PETSC_CHAR)         ret = fscanf(fd, "%c",  &(((char*)data)[i]));
     else if (dtype == PETSC_STRING)  ret = fscanf(fd, "%s",  &(((char*)data)[i]));
-#if PETSC_USE_64BIT_INDICES
-#if (PETSC_SIZEOF_LONG_LONG == 8)
-    else if (dtype == PETSC_INT)     ret = fscanf(fd, "%ld",  &(((PetscInt*)data)[i]));
-#else
-    else if (dtype == PETSC_INT)     ret = fscanf(fd, "%lld",  &(((PetscInt*)data)[i]));
-#endif
-#else
-    else if (dtype == PETSC_INT)     ret = fscanf(fd, "%d",  &(((PetscInt*)data)[i]));
-#endif
+    else if (dtype == PETSC_INT)     ret = fscanf(fd, "%" PetscInt_FMT,  &(((PetscInt*)data)[i]));
     else if (dtype == PETSC_ENUM)    ret = fscanf(fd, "%d",  &(((int*)data)[i]));
     else if (dtype == PETSC_FLOAT)   ret = fscanf(fd, "%f",  &(((float*)data)[i]));
     else if (dtype == PETSC_DOUBLE)  ret = fscanf(fd, "%lg", &(((double*)data)[i]));
+#if defined(PETSC_USE_REAL___FLOAT128)
+    else if (dtype == PETSC___FLOAT128) {
+      double tmp;
+      ret = fscanf(fd, "%lg", &tmp);
+      ((__float128*)data)[i] = tmp;
+    }
+#endif
     else {SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Data type %d not supported", (int) dtype);}
     if (!ret) SETERRQ1(PETSC_COMM_SELF, PETSC_ERR_ARG_WRONG, "Conversion error for data type %d", (int) dtype);
     else if (ret < 0) break; /* Proxy for EOF, need to check for it in configure */
