@@ -13,12 +13,12 @@
 #define petsc_viewer_stdout__      petsc_viewer_stdout___
 #endif
 
-PETSC_EXTERN void PETSC_STDCALL petscviewerasciiopen_(MPI_Comm *comm,CHAR name PETSC_MIXED_LEN(len),PetscViewer *lab,
+PETSC_EXTERN void PETSC_STDCALL petscviewerasciiopen_(MPI_Comm *comm,char* name PETSC_MIXED_LEN(len),PetscViewer *lab,
                                     PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   char *c1;
   FIXCHAR(name,len,c1);
-  *ierr = PetscViewerASCIIOpen(MPI_Comm_f2c(*(MPI_Fint*)&*comm),c1,lab);
+  *ierr = PetscViewerASCIIOpen(MPI_Comm_f2c(*(MPI_Fint*)&*comm),c1,lab);if (*ierr) return;
   FREECHAR(name,c1);
 }
 
