@@ -18,12 +18,12 @@ PETSC_EXTERN void PETSC_STDCALL petscviewerfilesetmode_(PetscViewer *viewer,Pets
   *ierr = PetscViewerFileSetMode(v,*type);
 }
 
-PETSC_EXTERN void PETSC_STDCALL petscviewerbinaryopen_(MPI_Comm *comm,CHAR name PETSC_MIXED_LEN(len),PetscFileMode *type,
+PETSC_EXTERN void PETSC_STDCALL petscviewerbinaryopen_(MPI_Comm *comm,char* name PETSC_MIXED_LEN(len),PetscFileMode *type,
                                           PetscViewer *binv,PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   char *c1;
   FIXCHAR(name,len,c1);
-  *ierr = PetscViewerBinaryOpen(MPI_Comm_f2c(*(MPI_Fint*)&*comm),c1,*type,binv);
+  *ierr = PetscViewerBinaryOpen(MPI_Comm_f2c(*(MPI_Fint*)&*comm),c1,*type,binv);if (*ierr) return;
   FREECHAR(name,c1);
 }
 
