@@ -2,9 +2,7 @@
 #include <../src/sys/classes/viewer/impls/draw/vdraw.h> /*I "petscdraw.h" I*/
 #include <petscviewer.h>                                /*I "petscviewer.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDestroy_Draw"
-PetscErrorCode PetscViewerDestroy_Draw(PetscViewer v)
+static PetscErrorCode PetscViewerDestroy_Draw(PetscViewer v)
 {
   PetscErrorCode   ierr;
   PetscInt         i;
@@ -17,19 +15,16 @@ PetscErrorCode PetscViewerDestroy_Draw(PetscViewer v)
     ierr = PetscDrawLGDestroy(&vdraw->drawlg[i]);CHKERRQ(ierr);
     ierr = PetscDrawDestroy(&vdraw->draw[i]);CHKERRQ(ierr);
   }
-
   ierr = PetscFree(vdraw->display);CHKERRQ(ierr);
   ierr = PetscFree(vdraw->title);CHKERRQ(ierr);
   ierr = PetscFree3(vdraw->draw,vdraw->drawlg,vdraw->drawaxis);CHKERRQ(ierr);
   ierr = PetscFree(vdraw->bounds);CHKERRQ(ierr);
   ierr = PetscFree(vdraw->drawtype);CHKERRQ(ierr);
-  ierr = PetscFree(vdraw);CHKERRQ(ierr);
+  ierr = PetscFree(v->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerFlush_Draw"
-PetscErrorCode PetscViewerFlush_Draw(PetscViewer v)
+static PetscErrorCode PetscViewerFlush_Draw(PetscViewer v)
 {
   PetscErrorCode   ierr;
   PetscInt         i;
@@ -37,22 +32,20 @@ PetscErrorCode PetscViewerFlush_Draw(PetscViewer v)
 
   PetscFunctionBegin;
   for (i=0; i<vdraw->draw_max; i++) {
-    if (vdraw->draw[i]) {ierr = PetscDrawSynchronizedFlush(vdraw->draw[i]);CHKERRQ(ierr);}
+    if (vdraw->draw[i]) {ierr = PetscDrawFlush(vdraw->draw[i]);CHKERRQ(ierr);}
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawGetDraw"
 /*@C
     PetscViewerDrawGetDraw - Returns PetscDraw object from PetscViewer object.
     This PetscDraw object may then be used to perform graphics using
     PetscDrawXXX() commands.
 
-    Not collective (but PetscDraw returned will be parallel object if PetscViewer is)
+    Collective on PetscViewer
 
     Input Parameters:
-+  viewer - the PetscViewer (created with PetscViewerDrawOpen())
++   viewer - the PetscViewer (created with PetscViewerDrawOpen())
 -   windownumber - indicates which subwindow (usually 0)
 
     Ouput Parameter:
@@ -67,17 +60,19 @@ PetscErrorCode PetscViewerFlush_Draw(PetscViewer v)
 @*/
 PetscErrorCode  PetscViewerDrawGetDraw(PetscViewer viewer,PetscInt windownumber,PetscDraw *draw)
 {
-  PetscViewer_Draw *vdraw = (PetscViewer_Draw*)viewer->data;
+  PetscViewer_Draw *vdraw;
   PetscErrorCode   ierr;
   PetscBool        isdraw;
-  char             *title;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
+  PetscValidLogicalCollectiveInt(viewer,windownumber,2);
   if (draw) PetscValidPointer(draw,3);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
   if (!isdraw) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Must be draw type PetscViewer");
   if (windownumber < 0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Window number cannot be negative");
+  vdraw = (PetscViewer_Draw*)viewer->data;
+
   windownumber += vdraw->draw_base;
   if (windownumber >= vdraw->draw_max) {
     /* allocate twice as many slots as needed */
@@ -98,17 +93,18 @@ PetscErrorCode  PetscViewerDrawGetDraw(PetscViewer viewer,PetscInt windownumber,
   }
 
   if (!vdraw->draw[windownumber]) {
-    if (!windownumber) title = vdraw->title;
-    else {
-      char tmp_str[128];
-      ierr  = PetscSNPrintf(tmp_str, 128, "%s:%d", vdraw->title,windownumber);CHKERRQ(ierr);
+    char *title = vdraw->title, tmp_str[128];
+    if (windownumber) {
+      ierr = PetscSNPrintf(tmp_str,sizeof(tmp_str),"%s:%d",vdraw->title?vdraw->title:"",windownumber);CHKERRQ(ierr);
       title = tmp_str;
     }
     ierr = PetscDrawCreate(PetscObjectComm((PetscObject)viewer),vdraw->display,title,PETSC_DECIDE,PETSC_DECIDE,vdraw->w,vdraw->h,&vdraw->draw[windownumber]);CHKERRQ(ierr);
+    ierr = PetscLogObjectParent((PetscObject)viewer,(PetscObject)vdraw->draw[windownumber]);CHKERRQ(ierr);
     if (vdraw->drawtype) {
       ierr = PetscDrawSetType(vdraw->draw[windownumber],vdraw->drawtype);CHKERRQ(ierr);
     }
     ierr = PetscDrawSetPause(vdraw->draw[windownumber],vdraw->pause);CHKERRQ(ierr);
+    ierr = PetscDrawSetOptionsPrefix(vdraw->draw[windownumber],((PetscObject)viewer)->prefix);CHKERRQ(ierr);
     ierr = PetscDrawSetFromOptions(vdraw->draw[windownumber]);CHKERRQ(ierr);
   }
   if (draw) *draw = vdraw->draw[windownumber];
@@ -116,12 +112,10 @@ PetscErrorCode  PetscViewerDrawGetDraw(PetscViewer viewer,PetscInt windownumber,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawBaseAdd"
 /*@C
     PetscViewerDrawBaseAdd - add to the base integer that is added to the windownumber passed to PetscViewerDrawGetDraw()
 
-    Not collective (but PetscDraw returned will be parallel object if PetscViewer is)
+    Logically Collective on PetscViewer
 
     Input Parameters:
 +  viewer - the PetscViewer (created with PetscViewerDrawOpen())
@@ -136,28 +130,29 @@ PetscErrorCode  PetscViewerDrawGetDraw(PetscViewer viewer,PetscInt windownumber,
 @*/
 PetscErrorCode  PetscViewerDrawBaseAdd(PetscViewer viewer,PetscInt windownumber)
 {
-  PetscViewer_Draw *vdraw = (PetscViewer_Draw*)viewer->data;
+  PetscViewer_Draw *vdraw;
   PetscErrorCode   ierr;
   PetscBool        isdraw;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
+  PetscValidLogicalCollectiveInt(viewer,windownumber,2);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
   if (!isdraw) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Must be draw type PetscViewer");
+  vdraw = (PetscViewer_Draw*)viewer->data;
+
   if (windownumber + vdraw->draw_base < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Resulting base %D cannot be negative",windownumber+vdraw->draw_base);
   vdraw->draw_base += windownumber;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawBaseSet"
 /*@C
     PetscViewerDrawBaseSet - sets the base integer that is added to the windownumber passed to PetscViewerDrawGetDraw()
 
-    Not collective (but PetscDraw returned will be parallel object if PetscViewer is)
+    Logically Collective on PetscViewer
 
     Input Parameters:
-+  viewer - the PetscViewer (created with PetscViewerDrawOpen())
++   viewer - the PetscViewer (created with PetscViewerDrawOpen())
 -   windownumber - value to set the base
 
     Level: developer
@@ -169,27 +164,28 @@ PetscErrorCode  PetscViewerDrawBaseAdd(PetscViewer viewer,PetscInt windownumber)
 @*/
 PetscErrorCode  PetscViewerDrawBaseSet(PetscViewer viewer,PetscInt windownumber)
 {
-  PetscViewer_Draw *vdraw = (PetscViewer_Draw*)viewer->data;
+  PetscViewer_Draw *vdraw;
   PetscErrorCode   ierr;
   PetscBool        isdraw;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
+  PetscValidLogicalCollectiveInt(viewer,windownumber,2);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
   if (!isdraw) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Must be draw type PetscViewer");
+  vdraw = (PetscViewer_Draw*)viewer->data;
+
   if (windownumber < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Resulting base %D cannot be negative",windownumber);
   vdraw->draw_base = windownumber;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawGetDrawLG"
 /*@C
     PetscViewerDrawGetDrawLG - Returns PetscDrawLG object from PetscViewer object.
     This PetscDrawLG object may then be used to perform graphics using
     PetscDrawLGXXX() commands.
 
-    Not Collective (but PetscDrawLG object will be parallel if PetscViewer is)
+    Collective on PetscViewer
 
     Input Parameter:
 +   PetscViewer - the PetscViewer (created with PetscViewerDrawOpen())
@@ -208,14 +204,16 @@ PetscErrorCode  PetscViewerDrawGetDrawLG(PetscViewer viewer,PetscInt windownumbe
 {
   PetscErrorCode   ierr;
   PetscBool        isdraw;
-  PetscViewer_Draw *vdraw = (PetscViewer_Draw*)viewer->data;
+  PetscViewer_Draw *vdraw;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
+  PetscValidLogicalCollectiveInt(viewer,windownumber,2);
   PetscValidPointer(drawlg,3);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
   if (!isdraw) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Must be draw type PetscViewer");
   if (windownumber < 0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Window number cannot be negative");
+  vdraw = (PetscViewer_Draw*)viewer->data;
 
   if (windownumber+vdraw->draw_base >= vdraw->draw_max || !vdraw->draw[windownumber+vdraw->draw_base]) {
     ierr = PetscViewerDrawGetDraw(viewer,windownumber,NULL);CHKERRQ(ierr);
@@ -223,19 +221,18 @@ PetscErrorCode  PetscViewerDrawGetDrawLG(PetscViewer viewer,PetscInt windownumbe
   if (!vdraw->drawlg[windownumber+vdraw->draw_base]) {
     ierr = PetscDrawLGCreate(vdraw->draw[windownumber+vdraw->draw_base],1,&vdraw->drawlg[windownumber+vdraw->draw_base]);CHKERRQ(ierr);
     ierr = PetscLogObjectParent((PetscObject)viewer,(PetscObject)vdraw->drawlg[windownumber+vdraw->draw_base]);CHKERRQ(ierr);
+    ierr = PetscDrawLGSetFromOptions(vdraw->drawlg[windownumber+vdraw->draw_base]);CHKERRQ(ierr);
   }
   *drawlg = vdraw->drawlg[windownumber+vdraw->draw_base];
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawGetDrawAxis"
 /*@C
     PetscViewerDrawGetDrawAxis - Returns PetscDrawAxis object from PetscViewer object.
     This PetscDrawAxis object may then be used to perform graphics using
     PetscDrawAxisXXX() commands.
 
-    Not Collective (but PetscDrawAxis object will be parallel if PetscViewer is)
+    Collective on PetscViewer
 
     Input Parameter:
 +   viewer - the PetscViewer (created with PetscViewerDrawOpen()
@@ -254,14 +251,16 @@ PetscErrorCode  PetscViewerDrawGetDrawAxis(PetscViewer viewer,PetscInt windownum
 {
   PetscErrorCode   ierr;
   PetscBool        isdraw;
-  PetscViewer_Draw *vdraw = (PetscViewer_Draw*)viewer->data;;
+  PetscViewer_Draw *vdraw;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
+  PetscValidLogicalCollectiveInt(viewer,windownumber,2);
   PetscValidPointer(drawaxis,3);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
   if (!isdraw) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Must be draw type PetscViewer");
   if (windownumber < 0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Window number cannot be negative");
+  vdraw = (PetscViewer_Draw*)viewer->data;
 
   if (windownumber+vdraw->draw_base >= vdraw->draw_max || !vdraw->draw[windownumber+vdraw->draw_base]) {
     ierr = PetscViewerDrawGetDraw(viewer,windownumber,NULL);CHKERRQ(ierr);
@@ -274,52 +273,108 @@ PetscErrorCode  PetscViewerDrawGetDrawAxis(PetscViewer viewer,PetscInt windownum
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawResize"
 PetscErrorCode  PetscViewerDrawResize(PetscViewer v,int w,int h)
 {
-  PetscViewer_Draw *vdraw = (PetscViewer_Draw*)v->data;
+  PetscErrorCode   ierr;
+  PetscViewer_Draw *vdraw;
+  PetscBool        isdraw;
 
   PetscFunctionBegin;
-  vdraw->h = h;
-  vdraw->w = w;
+  PetscValidHeaderSpecific(v,PETSC_VIEWER_CLASSID,1);
+  ierr = PetscObjectTypeCompare((PetscObject)v,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
+  if (!isdraw) PetscFunctionReturn(0);
+  vdraw = (PetscViewer_Draw*)v->data;
+
+  if (w >= 1) vdraw->w = w;
+  if (h >= 1) vdraw->h = h;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawSetInfo"
 PetscErrorCode  PetscViewerDrawSetInfo(PetscViewer v,const char display[],const char title[],int x,int y,int w,int h)
 {
   PetscErrorCode   ierr;
-  PetscViewer_Draw *vdraw = (PetscViewer_Draw*)v->data;
+  PetscViewer_Draw *vdraw;
+  PetscBool        isdraw;
 
   PetscFunctionBegin;
-  vdraw->h = h;
-  vdraw->w = w;
-  ierr     = PetscStrallocpy(display,&vdraw->display);CHKERRQ(ierr);
-  ierr     = PetscStrallocpy(title,&vdraw->title);CHKERRQ(ierr);
+  PetscValidHeaderSpecific(v,PETSC_VIEWER_CLASSID,1);
+  ierr = PetscObjectTypeCompare((PetscObject)v,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
+  if (!isdraw) PetscFunctionReturn(0);
+  vdraw = (PetscViewer_Draw*)v->data;
+
+  ierr = PetscStrallocpy(display,&vdraw->display);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(title,&vdraw->title);CHKERRQ(ierr);
+  if (w >= 1) vdraw->w = w;
+  if (h >= 1) vdraw->h = h;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawSetDrawType"
 PetscErrorCode  PetscViewerDrawSetDrawType(PetscViewer v,PetscDrawType drawtype)
 {
   PetscErrorCode   ierr;
-  PetscViewer_Draw *vdraw = (PetscViewer_Draw*)v->data;
-  PetscBool        flg;
+  PetscViewer_Draw *vdraw;
+  PetscBool        isdraw;
 
   PetscFunctionBegin;
-  ierr = PetscObjectTypeCompare((PetscObject)v,PETSCVIEWERDRAW,&flg);CHKERRQ(ierr);
-  if (flg) {
-    ierr = PetscFree(vdraw->drawtype);CHKERRQ(ierr);
-    ierr = PetscStrallocpy(drawtype,(char**)&vdraw->drawtype);CHKERRQ(ierr);
-  }
+  PetscValidHeaderSpecific(v,PETSC_VIEWER_CLASSID,1);
+  ierr = PetscObjectTypeCompare((PetscObject)v,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
+  if (!isdraw) PetscFunctionReturn(0);
+  vdraw = (PetscViewer_Draw*)v->data;
+
+  ierr = PetscFree(vdraw->drawtype);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(drawtype,(char**)&vdraw->drawtype);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode PetscViewerDrawGetDrawType(PetscViewer v,PetscDrawType *drawtype)
+{
+  PetscErrorCode   ierr;
+  PetscViewer_Draw *vdraw;
+  PetscBool        isdraw;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(v,PETSC_VIEWER_CLASSID,1);
+  ierr = PetscObjectTypeCompare((PetscObject)v,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
+  if (!isdraw) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Must be draw type PetscViewer");
+  vdraw = (PetscViewer_Draw*)v->data;
+
+  *drawtype = vdraw->drawtype;
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode PetscViewerDrawSetTitle(PetscViewer v,const char title[])
+{
+  PetscErrorCode   ierr;
+  PetscViewer_Draw *vdraw;
+  PetscBool        isdraw;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(v,PETSC_VIEWER_CLASSID,1);
+  ierr = PetscObjectTypeCompare((PetscObject)v,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
+  if (!isdraw) PetscFunctionReturn(0);
+  vdraw = (PetscViewer_Draw*)v->data;
+
+  ierr = PetscFree(vdraw->title);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(title,&vdraw->title);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode PetscViewerDrawGetTitle(PetscViewer v,const char *title[])
+{
+  PetscErrorCode   ierr;
+  PetscViewer_Draw *vdraw;
+  PetscBool        isdraw;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(v,PETSC_VIEWER_CLASSID,1);
+  ierr = PetscObjectTypeCompare((PetscObject)v,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
+  if (!isdraw) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Must be draw type PetscViewer");
+  vdraw = (PetscViewer_Draw*)v->data;
+
+  *title = vdraw->title;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawOpen"
 /*@C
    PetscViewerDrawOpen - Opens a window for use as a PetscViewer. If you want to
    do graphics in this window, you must call PetscViewerDrawGetDraw() and
@@ -379,54 +434,49 @@ PetscErrorCode  PetscViewerDrawOpen(MPI_Comm comm,const char display[],const cha
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerGetSingleton_Draw"
-PetscErrorCode PetscViewerGetSingleton_Draw(PetscViewer viewer,PetscViewer *sviewer)
+PetscErrorCode PetscViewerGetSubViewer_Draw(PetscViewer viewer,MPI_Comm comm,PetscViewer *sviewer)
 {
   PetscErrorCode   ierr;
   PetscMPIInt      rank;
   PetscInt         i;
-  PetscViewer_Draw *vdraw = (PetscViewer_Draw*)viewer->data,*vsdraw;
+  PetscViewer_Draw *vdraw = (PetscViewer_Draw*)viewer->data,*svdraw;
 
   PetscFunctionBegin;
-  if (vdraw->singleton_made) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Trying to get singleton without first restoring previous");
-
+  if (vdraw->singleton_made) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Trying to get SubViewer without first restoring previous");
   /* only processor zero can use the PetscViewer draw singleton */
+  if (sviewer) *sviewer = NULL;
   ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)viewer),&rank);CHKERRQ(ierr);
   if (!rank) {
-    ierr   = PetscViewerCreate(PETSC_COMM_SELF,sviewer);CHKERRQ(ierr);
-    ierr   = PetscViewerSetType(*sviewer,PETSCVIEWERDRAW);CHKERRQ(ierr);
-    vsdraw = (PetscViewer_Draw*)(*sviewer)->data;
-    for (i=0; i<vdraw->draw_max; i++) {
-      if (vdraw->draw[i]) {
-        ierr = PetscDrawGetSingleton(vdraw->draw[i],&vsdraw->draw[i]);CHKERRQ(ierr);
-      }
+    ierr = PetscViewerCreate(PETSC_COMM_SELF,sviewer);CHKERRQ(ierr);
+    ierr = PetscViewerSetType(*sviewer,PETSCVIEWERDRAW);CHKERRQ(ierr);
+    svdraw = (PetscViewer_Draw*)(*sviewer)->data;
+    (*sviewer)->format = viewer->format;
+    for (i=0; i<vdraw->draw_max; i++) { /* XXX this is wrong if svdraw->draw_max (initially 5) < vdraw->draw_max */
+      if (vdraw->draw[i]) {ierr = PetscDrawGetSingleton(vdraw->draw[i],&svdraw->draw[i]);CHKERRQ(ierr);}
     }
   }
   vdraw->singleton_made = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerRestoreSingleton_Draw"
-PetscErrorCode PetscViewerRestoreSingleton_Draw(PetscViewer viewer,PetscViewer *sviewer)
+PetscErrorCode PetscViewerRestoreSubViewer_Draw(PetscViewer viewer,MPI_Comm comm,PetscViewer *sviewer)
 {
   PetscErrorCode   ierr;
   PetscMPIInt      rank;
   PetscInt         i;
-  PetscViewer_Draw *vdraw = (PetscViewer_Draw*)viewer->data,*vsdraw;
+  PetscViewer_Draw *vdraw = (PetscViewer_Draw*)viewer->data,*svdraw;
 
   PetscFunctionBegin;
   if (!vdraw->singleton_made) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Trying to restore a singleton that was not gotten");
   ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)viewer),&rank);CHKERRQ(ierr);
   if (!rank) {
-    vsdraw = (PetscViewer_Draw*)(*sviewer)->data;
+    svdraw = (PetscViewer_Draw*)(*sviewer)->data;
     for (i=0; i<vdraw->draw_max; i++) {
-      if (vdraw->draw[i] && vsdraw->draw[i]) {
-        ierr = PetscDrawRestoreSingleton(vdraw->draw[i],&vsdraw->draw[i]);CHKERRQ(ierr);
+      if (vdraw->draw[i] && svdraw->draw[i]) {
+        ierr = PetscDrawRestoreSingleton(vdraw->draw[i],&svdraw->draw[i]);CHKERRQ(ierr);
       }
     }
-    ierr = PetscFree3(vsdraw->draw,vsdraw->drawlg,vsdraw->drawaxis);CHKERRQ(ierr);
+    ierr = PetscFree3(svdraw->draw,svdraw->drawlg,svdraw->drawaxis);CHKERRQ(ierr);
     ierr = PetscFree((*sviewer)->data);CHKERRQ(ierr);
     ierr = PetscHeaderDestroy(sviewer);CHKERRQ(ierr);
   }
@@ -434,9 +484,7 @@ PetscErrorCode PetscViewerRestoreSingleton_Draw(PetscViewer viewer,PetscViewer *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerSetFromOptions_Draw"
-PetscErrorCode PetscViewerSetFromOptions_Draw(PetscOptions *PetscOptionsObject,PetscViewer v)
+PetscErrorCode PetscViewerSetFromOptions_Draw(PetscOptionItems *PetscOptionsObject,PetscViewer v)
 {
   PetscErrorCode ierr;
   PetscReal      bounds[16];
@@ -449,13 +497,10 @@ PetscErrorCode PetscViewerSetFromOptions_Draw(PetscOptions *PetscOptionsObject,P
   if (flg) {
     ierr = PetscViewerDrawSetBounds(v,nbounds/2,bounds);CHKERRQ(ierr);
   }
-
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerView_Draw"
 PetscErrorCode PetscViewerView_Draw(PetscViewer viewer,PetscViewer v)
 {
   PetscErrorCode   ierr;
@@ -475,23 +520,33 @@ PetscErrorCode PetscViewerView_Draw(PetscViewer viewer,PetscViewer v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerCreate_Draw"
+/*MC
+   PETSCVIEWERDRAW - A viewer that generates graphics, either to the screen or a file
+
+
+.seealso:  PetscViewerDrawOpen(), PetscViewerDrawGetDraw(), PETSC_VIEWER_DRAW_(),PETSC_VIEWER_DRAW_SELF, PETSC_VIEWER_DRAW_WORLD,
+           PetscViewerCreate(), PetscViewerASCIIOpen(), PetscViewerBinaryOpen(), PETSCVIEWERBINARY, 
+           PetscViewerMatlabOpen(), VecView(), DMView(), PetscViewerMatlabPutArray(), PETSCVIEWERASCII, PETSCVIEWERMATLAB,
+           PetscViewerFileSetName(), PetscViewerFileSetMode(), PetscViewerFormat, PetscViewerType, PetscViewerSetType()
+
+  Level: beginner
+
+M*/
 PETSC_EXTERN PetscErrorCode PetscViewerCreate_Draw(PetscViewer viewer)
 {
   PetscErrorCode   ierr;
   PetscViewer_Draw *vdraw;
 
   PetscFunctionBegin;
-  ierr         = PetscNewLog(viewer,&vdraw);CHKERRQ(ierr);
+  ierr = PetscNewLog(viewer,&vdraw);CHKERRQ(ierr);
   viewer->data = (void*)vdraw;
 
   viewer->ops->flush            = PetscViewerFlush_Draw;
   viewer->ops->view             = PetscViewerView_Draw;
   viewer->ops->destroy          = PetscViewerDestroy_Draw;
   viewer->ops->setfromoptions   = PetscViewerSetFromOptions_Draw;
-  viewer->ops->getsingleton     = PetscViewerGetSingleton_Draw;
-  viewer->ops->restoresingleton = PetscViewerRestoreSingleton_Draw;
+  viewer->ops->getsubviewer     = PetscViewerGetSubViewer_Draw;
+  viewer->ops->restoresubviewer = PetscViewerRestoreSubViewer_Draw;
 
   /* these are created on the fly if requested */
   vdraw->draw_max  = 5;
@@ -504,8 +559,6 @@ PETSC_EXTERN PetscErrorCode PetscViewerCreate_Draw(PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawClear"
 /*@
     PetscViewerDrawClear - Clears a PetscDraw graphic associated with a PetscViewer.
 
@@ -522,23 +575,22 @@ PETSC_EXTERN PetscErrorCode PetscViewerCreate_Draw(PetscViewer viewer)
 PetscErrorCode  PetscViewerDrawClear(PetscViewer viewer)
 {
   PetscErrorCode   ierr;
-  PetscInt         i;
-  PetscBool        isdraw;
   PetscViewer_Draw *vdraw;
+  PetscBool        isdraw;
+  PetscInt         i;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
-  if (isdraw) {
-    vdraw = (PetscViewer_Draw*)viewer->data;
-    for (i=0; i<vdraw->draw_max; i++) {
-      if (vdraw->draw[i]) {ierr = PetscDrawClear(vdraw->draw[i]);CHKERRQ(ierr);}
-    }
+  if (!isdraw) PetscFunctionReturn(0);
+  vdraw = (PetscViewer_Draw*)viewer->data;
+
+  for (i=0; i<vdraw->draw_max; i++) {
+    if (vdraw->draw[i]) {ierr = PetscDrawClear(vdraw->draw[i]);CHKERRQ(ierr);}
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawGetPause"
 /*@
     PetscViewerDrawGetPause - Gets a pause for the first present draw
 
@@ -558,31 +610,29 @@ PetscErrorCode  PetscViewerDrawClear(PetscViewer viewer)
 PetscErrorCode  PetscViewerDrawGetPause(PetscViewer viewer,PetscReal *pause)
 {
   PetscErrorCode   ierr;
-  PetscInt         i;
-  PetscBool        isdraw;
   PetscViewer_Draw *vdraw;
+  PetscBool        isdraw;
+  PetscInt         i;
   PetscDraw        draw;
 
   PetscFunctionBegin;
-  ierr   = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
-  *pause = 0.0;
-  if (isdraw) {
-    vdraw = (PetscViewer_Draw*)viewer->data;
-    for (i=0; i<vdraw->draw_max; i++) {
-      if (vdraw->draw[i]) {
-        ierr = PetscDrawGetPause(vdraw->draw[i],pause);CHKERRQ(ierr);
-        PetscFunctionReturn(0);
-      }
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
+  if (!isdraw) {*pause = 0.0; PetscFunctionReturn(0);}
+  vdraw = (PetscViewer_Draw*)viewer->data;
+
+  for (i=0; i<vdraw->draw_max; i++) {
+    if (vdraw->draw[i]) {
+      ierr = PetscDrawGetPause(vdraw->draw[i],pause);CHKERRQ(ierr);
+      PetscFunctionReturn(0);
     }
-    /* none exist yet so create one and get its pause */
-    ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
-    ierr = PetscDrawGetPause(vdraw->draw[0],pause);CHKERRQ(ierr);
   }
+  /* none exist yet so create one and get its pause */
+  ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
+  ierr = PetscDrawGetPause(draw,pause);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawSetPause"
 /*@
     PetscViewerDrawSetPause - Sets a pause for each PetscDraw in the viewer
 
@@ -600,25 +650,24 @@ PetscErrorCode  PetscViewerDrawGetPause(PetscViewer viewer,PetscReal *pause)
 PetscErrorCode  PetscViewerDrawSetPause(PetscViewer viewer,PetscReal pause)
 {
   PetscErrorCode   ierr;
-  PetscInt         i;
+  PetscViewer_Draw *vdraw;
   PetscBool        isdraw;
+  PetscInt         i;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
-  if (isdraw) {
-    PetscViewer_Draw *vdraw = (PetscViewer_Draw*)viewer->data;
+  if (!isdraw) PetscFunctionReturn(0);
+  vdraw = (PetscViewer_Draw*)viewer->data;
 
-    vdraw->pause = pause;
-    for (i=0; i<vdraw->draw_max; i++) {
-      if (vdraw->draw[i]) {ierr = PetscDrawSetPause(vdraw->draw[i],pause);CHKERRQ(ierr);}
-    }
+  vdraw->pause = pause;
+  for (i=0; i<vdraw->draw_max; i++) {
+    if (vdraw->draw[i]) {ierr = PetscDrawSetPause(vdraw->draw[i],pause);CHKERRQ(ierr);}
   }
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawSetHold"
 /*@
     PetscViewerDrawSetHold - Holds previous image when drawing new image
 
@@ -640,16 +689,15 @@ PetscErrorCode  PetscViewerDrawSetHold(PetscViewer viewer,PetscBool hold)
   PetscBool        isdraw;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
-  if (isdraw) {
-    vdraw       = (PetscViewer_Draw*)viewer->data;
-    vdraw->hold = hold;
-  }
+  if (!isdraw) PetscFunctionReturn(0);
+  vdraw = (PetscViewer_Draw*)viewer->data;
+
+  vdraw->hold = hold;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawGetHold"
 /*@
     PetscViewerDrawGetHold - Checks if holds previous image when drawing new image
 
@@ -673,11 +721,12 @@ PetscErrorCode  PetscViewerDrawGetHold(PetscViewer viewer,PetscBool *hold)
   PetscBool        isdraw;
 
   PetscFunctionBegin;
+  PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
-  if (isdraw) {
-    vdraw = (PetscViewer_Draw*)viewer->data;
-    *hold = vdraw->hold;
-  }
+  if (!isdraw) {*hold = PETSC_FALSE; PetscFunctionReturn(0);}
+  vdraw = (PetscViewer_Draw*)viewer->data;
+
+  *hold = vdraw->hold;
   PetscFunctionReturn(0);
 }
 
@@ -686,10 +735,8 @@ PetscErrorCode  PetscViewerDrawGetHold(PetscViewer viewer,PetscBool *hold)
     The variable Petsc_Viewer_Draw_keyval is used to indicate an MPI attribute that
   is attached to a communicator, in this case the attribute is a PetscViewer.
 */
-static PetscMPIInt Petsc_Viewer_Draw_keyval = MPI_KEYVAL_INVALID;
+PetscMPIInt Petsc_Viewer_Draw_keyval = MPI_KEYVAL_INVALID;
 
-#undef __FUNCT__
-#define __FUNCT__ "PETSC_VIEWER_DRAW_"
 /*@C
     PETSC_VIEWER_DRAW_ - Creates a window PetscViewer shared by all processors
                      in a communicator.
@@ -718,17 +765,17 @@ PetscViewer  PETSC_VIEWER_DRAW_(MPI_Comm comm)
   PetscFunctionBegin;
   ierr = PetscCommDuplicate(comm,&ncomm,NULL);if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_DRAW_",__FILE__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
   if (Petsc_Viewer_Draw_keyval == MPI_KEYVAL_INVALID) {
-    ierr = MPI_Keyval_create(MPI_NULL_COPY_FN,MPI_NULL_DELETE_FN,&Petsc_Viewer_Draw_keyval,0);
+    ierr = MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN,MPI_COMM_NULL_DELETE_FN,&Petsc_Viewer_Draw_keyval,0);
     if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_DRAW_",__FILE__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
   }
-  ierr = MPI_Attr_get(ncomm,Petsc_Viewer_Draw_keyval,(void**)&viewer,&flag);
+  ierr = MPI_Comm_get_attr(ncomm,Petsc_Viewer_Draw_keyval,(void**)&viewer,&flag);
   if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_DRAW_",__FILE__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
   if (!flag) { /* PetscViewer not yet created */
     ierr = PetscViewerDrawOpen(ncomm,0,0,PETSC_DECIDE,PETSC_DECIDE,300,300,&viewer);
     if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_DRAW_",__FILE__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
     ierr = PetscObjectRegisterDestroy((PetscObject)viewer);
     if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_DRAW_",__FILE__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
-    ierr = MPI_Attr_put(ncomm,Petsc_Viewer_Draw_keyval,(void*)viewer);
+    ierr = MPI_Comm_set_attr(ncomm,Petsc_Viewer_Draw_keyval,(void*)viewer);
     if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_DRAW_",__FILE__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
   }
   ierr = PetscCommDestroy(&ncomm);
@@ -736,8 +783,6 @@ PetscViewer  PETSC_VIEWER_DRAW_(MPI_Comm comm)
   PetscFunctionReturn(viewer);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawSetBounds"
 /*@
     PetscViewerDrawSetBounds - sets the upper and lower bounds to be used in plotting
 
@@ -754,7 +799,8 @@ PetscViewer  PETSC_VIEWER_DRAW_(MPI_Comm comm)
 
     Level: intermediate
 
-    Notes: this determines the colors used in 2d contour plots generated with VecView() for DMDA in 2d. Any values in the vector below or above the
+    Notes:
+    this determines the colors used in 2d contour plots generated with VecView() for DMDA in 2d. Any values in the vector below or above the
       bounds are moved to the bound value before plotting. In this way the color index from color to physical value remains the same for all plots generated with
       this viewer. Otherwise the color to physical value meaning changes with each new image if this is not set.
 
@@ -765,21 +811,24 @@ PetscViewer  PETSC_VIEWER_DRAW_(MPI_Comm comm)
 @*/
 PetscErrorCode  PetscViewerDrawSetBounds(PetscViewer viewer,PetscInt nbounds,const PetscReal *bounds)
 {
-  PetscViewer_Draw *vdraw = (PetscViewer_Draw*)viewer->data;
+  PetscViewer_Draw *vdraw;
+  PetscBool        isdraw;
   PetscErrorCode   ierr;
 
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
-  vdraw->nbounds = nbounds;
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
+  if (!isdraw) PetscFunctionReturn(0);
+  vdraw = (PetscViewer_Draw*)viewer->data;
 
+  vdraw->nbounds = nbounds;
+  ierr = PetscFree(vdraw->bounds);CHKERRQ(ierr);
   ierr = PetscMalloc1(2*nbounds,&vdraw->bounds);CHKERRQ(ierr);
   ierr = PetscMemcpy(vdraw->bounds,bounds,2*nbounds*sizeof(PetscReal));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDrawGetBounds"
 /*@C
     PetscViewerDrawGetBounds - gets the upper and lower bounds to be used in plotting set with PetscViewerDrawSetBounds()
 
@@ -801,11 +850,17 @@ PetscErrorCode  PetscViewerDrawSetBounds(PetscViewer viewer,PetscInt nbounds,con
 @*/
 PetscErrorCode  PetscViewerDrawGetBounds(PetscViewer viewer,PetscInt *nbounds,const PetscReal **bounds)
 {
-  PetscViewer_Draw *vdraw = (PetscViewer_Draw*)viewer->data;
+  PetscViewer_Draw *vdraw;
+  PetscBool        isdraw;
+  PetscErrorCode   ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,1);
-  *nbounds = vdraw->nbounds;
-  *bounds  = vdraw->bounds;
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
+  if (!isdraw) {if (nbounds) *nbounds = 0; if (bounds) *bounds = NULL; PetscFunctionReturn(0);}
+  vdraw = (PetscViewer_Draw*)viewer->data;
+
+  if (nbounds) *nbounds = vdraw->nbounds;
+  if (bounds)  *bounds  = vdraw->bounds;
   PetscFunctionReturn(0);
 }
