@@ -1,6 +1,6 @@
 
-#include <petsc-private/viewerimpl.h>   /* "petscsys.h" */
-#include <petsc-private/pcimpl.h>
+#include <petsc/private/viewerimpl.h>   /* "petscsys.h" */
+#include <petsc/private/pcimpl.h>
 #include <../src/mat/impls/aij/seq/aij.h>
 #include <mathematica.h>
 
@@ -12,8 +12,6 @@ PetscViewer PETSC_VIEWER_MATHEMATICA_WORLD_PRIVATE = NULL;
 static void *mathematicaEnv                        = NULL;
 
 static PetscBool PetscViewerMathematicaPackageInitialized = PETSC_FALSE;
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaFinalizePackage"
 /*@C
   PetscViewerMathematicaFinalizePackage - This function destroys everything in the Petsc interface to Mathematica. It is
   called from PetscFinalize().
@@ -31,8 +29,6 @@ PetscErrorCode  PetscViewerMathematicaFinalizePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaInitializePackage"
 /*@C
   PetscViewerMathematicaInitializePackage - This function initializes everything in the Petsc interface to Mathematica. It is
   called from PetscDLLibraryRegister() when using dynamic libraries, and on the call to PetscInitialize()
@@ -58,8 +54,6 @@ PetscErrorCode  PetscViewerMathematicaInitializePackage(void)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerInitializeMathematicaWorld_Private"
 PetscErrorCode PetscViewerInitializeMathematicaWorld_Private()
 {
   PetscErrorCode ierr;
@@ -70,8 +64,6 @@ PetscErrorCode PetscViewerInitializeMathematicaWorld_Private()
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDestroy_Mathematica"
 static PetscErrorCode PetscViewerDestroy_Mathematica(PetscViewer viewer)
 {
   PetscViewer_Mathematica *vmath = (PetscViewer_Mathematica*) viewer->data;
@@ -85,8 +77,6 @@ static PetscErrorCode PetscViewerDestroy_Mathematica(PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDestroyMathematica_Private"
 PetscErrorCode PetscViewerDestroyMathematica_Private(void)
 {
   PetscErrorCode ierr;
@@ -98,8 +88,6 @@ PetscErrorCode PetscViewerDestroyMathematica_Private(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaSetupConnection_Private"
 PetscErrorCode PetscViewerMathematicaSetupConnection_Private(PetscViewer v)
 {
   PetscViewer_Mathematica *vmath = (PetscViewer_Mathematica*) v->data;
@@ -159,19 +147,15 @@ PetscErrorCode PetscViewerMathematicaSetupConnection_Private(PetscViewer v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerCreate_Mathematica"
 PETSC_EXTERN PetscErrorCode PetscViewerCreate_Mathematica(PetscViewer v)
 {
   PetscViewer_Mathematica *vmath;
   PetscErrorCode          ierr;
 
   PetscFunctionBegin;
-#if !defined(PETSC_USE_DYNAMIC_LIBRARIES)
   ierr = PetscViewerMathematicaInitializePackage();CHKERRQ(ierr);
-#endif
 
-  ierr            = PetscNewLog(v,PetscViewer_Mathematica, &vmath);CHKERRQ(ierr);
+  ierr            = PetscNewLog(v,&vmath);CHKERRQ(ierr);
   v->data         = (void*) vmath;
   v->ops->destroy = PetscViewerDestroy_Mathematica;
   v->ops->flush   = 0;
@@ -189,9 +173,7 @@ PETSC_EXTERN PetscErrorCode PetscViewerCreate_Mathematica(PetscViewer v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaParseLinkMode_Private"
-PetscErrorCode PetscViewerMathematicaParseLinkMode_Private(char *modename, LinkMode *mode)
+static PetscErrorCode PetscViewerMathematicaParseLinkMode(char *modename, LinkMode *mode)
 {
   PetscBool      isCreate, isConnect, isLaunch;
   PetscErrorCode ierr;
@@ -207,8 +189,6 @@ PetscErrorCode PetscViewerMathematicaParseLinkMode_Private(char *modename, LinkM
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaSetFromOptions"
 PetscErrorCode  PetscViewerMathematicaSetFromOptions(PetscViewer v)
 {
   PetscViewer_Mathematica *vmath = (PetscViewer_Mathematica*) v->data;
@@ -236,7 +216,7 @@ PetscErrorCode  PetscViewerMathematicaSetFromOptions(PetscViewer v)
   }
   /* Get link port */
   numPorts = size;
-  ierr     = PetscMalloc(size*sizeof(int), &ports);CHKERRQ(ierr);
+  ierr     = PetscMalloc1(size, &ports);CHKERRQ(ierr);
   ierr     = PetscOptionsGetIntArray("viewer_", "-math_linkport", ports, &numPorts, &opt);CHKERRQ(ierr);
   if (opt) {
     if (numPorts > rank) snprintf(linkname, 255, "%6d", ports[rank]);
@@ -246,7 +226,7 @@ PetscErrorCode  PetscViewerMathematicaSetFromOptions(PetscViewer v)
   ierr = PetscFree(ports);CHKERRQ(ierr);
   /* Get link host */
   numHosts = size;
-  ierr     = PetscMalloc(size*sizeof(char*), &hosts);CHKERRQ(ierr);
+  ierr     = PetscMalloc1(size, &hosts);CHKERRQ(ierr);
   ierr     = PetscOptionsGetStringArray("viewer_", "-math_linkhost", hosts, &numHosts, &opt);CHKERRQ(ierr);
   if (opt) {
     if (numHosts > rank) {
@@ -265,7 +245,7 @@ PetscErrorCode  PetscViewerMathematicaSetFromOptions(PetscViewer v)
   if (opt) {
     LinkMode mode;
 
-    ierr = PetscViewerMathematicaParseLinkMode_Private(modename, &mode);CHKERRQ(ierr);
+    ierr = PetscViewerMathematicaParseLinkMode(modename, &mode);CHKERRQ(ierr);
     ierr = PetscViewerMathematicaSetLinkMode(v, mode);CHKERRQ(ierr);
   }
   /* Get graphics type */
@@ -297,8 +277,6 @@ PetscErrorCode  PetscViewerMathematicaSetFromOptions(PetscViewer v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaSetLinkName"
 PetscErrorCode  PetscViewerMathematicaSetLinkName(PetscViewer v, const char *name)
 {
   PetscViewer_Mathematica *vmath = (PetscViewer_Mathematica*) v->data;
@@ -311,8 +289,6 @@ PetscErrorCode  PetscViewerMathematicaSetLinkName(PetscViewer v, const char *nam
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaLinkPort"
 PetscErrorCode  PetscViewerMathematicaSetLinkPort(PetscViewer v, int port)
 {
   char           name[16];
@@ -324,8 +300,6 @@ PetscErrorCode  PetscViewerMathematicaSetLinkPort(PetscViewer v, int port)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaSetLinkHost"
 PetscErrorCode  PetscViewerMathematicaSetLinkHost(PetscViewer v, const char *host)
 {
   PetscViewer_Mathematica *vmath = (PetscViewer_Mathematica*) v->data;
@@ -338,8 +312,6 @@ PetscErrorCode  PetscViewerMathematicaSetLinkHost(PetscViewer v, const char *hos
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaSetLinkHost"
 PetscErrorCode  PetscViewerMathematicaSetLinkMode(PetscViewer v, LinkMode mode)
 {
   PetscViewer_Mathematica *vmath = (PetscViewer_Mathematica*) v->data;
@@ -350,8 +322,6 @@ PetscErrorCode  PetscViewerMathematicaSetLinkMode(PetscViewer v, LinkMode mode)
 }
 
 /*----------------------------------------- Public Functions --------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaOpen"
 /*@C
   PetscViewerMathematicaOpen - Communicates with Mathemtica using MathLink.
 
@@ -381,12 +351,12 @@ $    PetscViewerMathematicaOpen(MPI_Comm comm, int port, char *machine, char *mo
 $    VecView(Vec vector, PetscViewer viewer)
 
    Options Database Keys:
-$    -viewer_math_linkhost <machine> - The host machine for the kernel
-$    -viewer_math_linkname <name>    - The full link name for the connection
-$    -viewer_math_linkport <port>    - The port for the connection
-$    -viewer_math_mode <mode>        - The mode, e.g. Launch, Connect
-$    -viewer_math_type <type>        - The plot type, e.g. Triangulation, Vector
-$    -viewer_math_graphics <output>  - The output type, e.g. Motif, PS, PSFile
++    -viewer_math_linkhost <machine> - The host machine for the kernel
+.    -viewer_math_linkname <name>    - The full link name for the connection
+.    -viewer_math_linkport <port>    - The port for the connection
+.    -viewer_math_mode <mode>        - The mode, e.g. Launch, Connect
+.    -viewer_math_type <type>        - The plot type, e.g. Triangulation, Vector
+-    -viewer_math_graphics <output>  - The output type, e.g. Motif, PS, PSFile
 
 .keywords: PetscViewer, Mathematica, open
 
@@ -402,15 +372,13 @@ PetscErrorCode  PetscViewerMathematicaOpen(MPI_Comm comm, int port, const char m
   LinkMode linkmode;
   ierr = PetscViewerMathematicaSetLinkPort(*v, port);CHKERRQ(ierr);
   ierr = PetscViewerMathematicaSetLinkHost(*v, machine);CHKERRQ(ierr);
-  ierr = PetscViewerMathematicaParseLinkMode_Private(mode, &linkmode);CHKERRQ(ierr);
+  ierr = PetscViewerMathematicaParseLinkMode(mode, &linkmode);CHKERRQ(ierr);
   ierr = PetscViewerMathematicaSetLinkMode(*v, linkmode);CHKERRQ(ierr);
 #endif
   ierr = PetscViewerSetType(*v, PETSC_VIEWER_MATHEMATICA);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaGetLink"
 /*@C
   PetscViewerMathematicaGetLink - Returns the link to Mathematica
 
@@ -433,8 +401,6 @@ PetscErrorCode  PetscViewerMathematicaGetLink(PetscViewer viewer, MLINK *link)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaSkipPackets"
 /*@C
   PetscViewerMathematicaSkipPackets - Discard packets sent by Mathematica until a certain packet type is received
 
@@ -462,8 +428,6 @@ PetscErrorCode  PetscViewerMathematicaSkipPackets(PetscViewer viewer, int type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaGetName"
 /*@C
   PetscViewerMathematicaGetName - Retrieve the default name for objects communicated to Mathematica
 
@@ -489,8 +453,6 @@ PetscErrorCode  PetscViewerMathematicaGetName(PetscViewer viewer, const char **n
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaSetName"
 /*@C
   PetscViewerMathematicaSetName - Override the default name for objects communicated to Mathematica
 
@@ -514,8 +476,6 @@ PetscErrorCode  PetscViewerMathematicaSetName(PetscViewer viewer, const char nam
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaClearName"
 /*@C
   PetscViewerMathematicaClearName - Use the default name for objects communicated to Mathematica
 
@@ -537,8 +497,6 @@ PetscErrorCode  PetscViewerMathematicaClearName(PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaGetVector"
 /*@C
   PetscViewerMathematicaGetVector - Retrieve a vector from Mathematica
 
@@ -586,8 +544,6 @@ PetscErrorCode  PetscViewerMathematicaGetVector(PetscViewer viewer, Vec v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMathematicaPutVector"
 /*@C
   PetscViewerMathematicaPutVector - Send a vector to Mathematica
 
