@@ -8,12 +8,12 @@
 #endif
 
 #if defined(PETSC_HAVE_MATLAB_ENGINE)
-PETSC_EXTERN void PETSC_STDCALL petscviewermatlabopen_(MPI_Comm *comm,CHAR name PETSC_MIXED_LEN(len),PetscFileMode *type,
+PETSC_EXTERN void PETSC_STDCALL petscviewermatlabopen_(MPI_Comm *comm,char* name PETSC_MIXED_LEN(len),PetscFileMode *type,
                            PetscViewer *binv,PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   char *c1;
   FIXCHAR(name,len,c1);
-  *ierr = PetscViewerMatlabOpen(MPI_Comm_f2c(*(MPI_Fint*)&*comm),c1,*type,binv);
+  *ierr = PetscViewerMatlabOpen(MPI_Comm_f2c(*(MPI_Fint*)&*comm),c1,*type,binv);if (*ierr) return;
   FREECHAR(name,c1);
 }
 #endif
