@@ -2,6 +2,136 @@
 #include <petsc/private/viewerimpl.h>
 #include <mat.h>
 
+
+typedef struct {
+  MATFile       *ep;
+  PetscMPIInt   rank;
+  PetscFileMode btype;
+} PetscViewer_Matlab;
+
+/*@C
+    PetscViewerMatlabPutArray - Puts an array into the MATLAB viewer.
+
+      Not collective: only processor zero saves the array
+
+    Input Parameters:
++    mfile - the viewer
+.    m,n - the dimensions of the array
+.    array - the array (represented in one dimension)
+-    name - the name of the array
+
+   Level: advanced
+
+     Notes:
+    Only writes array values on processor 0.
+
+@*/
+PetscErrorCode  PetscViewerMatlabPutArray(PetscViewer mfile,int m,int n,const PetscScalar *array,const char *name)
+{
+  PetscErrorCode     ierr;
+  PetscViewer_Matlab *ml = (PetscViewer_Matlab*)mfile->data;
+  mxArray            *mat;
+
+  PetscFunctionBegin;
+  if (!ml->rank) {
+    ierr = PetscInfo1(mfile,"Putting MATLAB array %s\n",name);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)
+    mat  = mxCreateDoubleMatrix(m,n,mxREAL);
+#else
+    mat  = mxCreateDoubleMatrix(m,n,mxCOMPLEX);
+#endif
+    ierr = PetscMemcpy(mxGetPr(mat),array,m*n*sizeof(PetscScalar));CHKERRQ(ierr);
+    matPutVariable(ml->ep,name,mat);
+
+    ierr = PetscInfo1(mfile,"Put MATLAB array %s\n",name);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode  PetscViewerMatlabPutVariable(PetscViewer viewer,const char *name,void *mat)
+{
+  PetscViewer_Matlab *ml = (PetscViewer_Matlab*)viewer->data;
+
+  PetscFunctionBegin;
+  matPutVariable(ml->ep,name,(mxArray*)mat);
+  PetscFunctionReturn(0);
+}
+
+/*@C
+    PetscViewerMatlabGetArray - Gets a variable from a MATLAB viewer into an array
+
+    Not Collective; only processor zero reads in the array
+
+    Input Parameters:
++    mfile - the MATLAB file viewer
+.    m,n - the dimensions of the array
+.    array - the array (represented in one dimension)
+-    name - the name of the array
+
+   Level: advanced
+
+     Notes:
+    Only reads in array values on processor 0.
+
+@*/
+PetscErrorCode  PetscViewerMatlabGetArray(PetscViewer mfile,int m,int n,PetscScalar *array,const char *name)
+{
+  PetscErrorCode     ierr;
+  PetscViewer_Matlab *ml = (PetscViewer_Matlab*)mfile->data;
+  mxArray            *mat;
+
+  PetscFunctionBegin;
+  if (!ml->rank) {
+    ierr = PetscInfo1(mfile,"Getting MATLAB array %s\n",name);CHKERRQ(ierr);
+    mat  = matGetVariable(ml->ep,name);
+    if (!mat) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Unable to get array %s from matlab",name);
+    ierr = PetscMemcpy(array,mxGetPr(mat),m*n*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscInfo1(mfile,"Got MATLAB array %s\n",name);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode  PetscViewerFileSetMode_Matlab(PetscViewer viewer,PetscFileMode type)
+{
+  PetscViewer_Matlab *vmatlab = (PetscViewer_Matlab*)viewer->data;
+
+  PetscFunctionBegin;
+  vmatlab->btype = type;
+  PetscFunctionReturn(0);
+}
+
+/*
+        Actually opens the file
+*/
+PetscErrorCode  PetscViewerFileSetName_Matlab(PetscViewer viewer,const char name[])
+{
+  PetscViewer_Matlab *vmatlab = (PetscViewer_Matlab*)viewer->data;
+  PetscFileMode      type     = vmatlab->btype;
+
+  PetscFunctionBegin;
+  if (type == (PetscFileMode) -1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Must call PetscViewerFileSetMode() before PetscViewerFileSetName()");
+  if (vmatlab->ep) matClose(vmatlab->ep);
+
+  /* only first processor opens file */
+  if (!vmatlab->rank) {
+    if (type == FILE_MODE_READ) vmatlab->ep = matOpen(name,"r");
+    else if (type == FILE_MODE_WRITE || type == FILE_MODE_WRITE) vmatlab->ep = matOpen(name,"w");
+    else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Unknown file type");
+  }
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode PetscViewerDestroy_Matlab(PetscViewer v)
+{
+  PetscErrorCode     ierr;
+  PetscViewer_Matlab *vf = (PetscViewer_Matlab*)v->data;
+
+  PetscFunctionBegin;
+  if (vf->ep) matClose(vf->ep);
+  ierr = PetscFree(vf);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*MC
    PETSCVIEWERMATLAB - A viewer that saves the variables into a MATLAB .mat file that may be read into MATLAB
        with load('filename').
@@ -31,153 +161,11 @@ $                xnew(:) = x;    % reshape one dimensional vector back to two di
 
               Use PetscViewerMatlabPutArray() to just put an array of doubles into the .mat file
 
-.seealso:  PETSC_VIEWER_MATLAB_(),PETSC_VIEWER_MATLAB_SELF(), PETSC_VIEWER_MATLAB_WORLD(),PetscViewerCreate(),
-           PetscViewerMatlabOpen(), VecView(), DMView(), PetscViewerMatlabPutArray(), PETSCVIEWERBINARY,
-           PETSC_ASCII_VIEWER, PetscViewerFileSetName(), PetscViewerFileSetMode()
+.seealso:  PETSC_VIEWER_MATLAB_(),PETSC_VIEWER_MATLAB_SELF, PETSC_VIEWER_MATLAB_WORLD,PetscViewerCreate(),
+           PetscViewerMatlabOpen(), VecView(), DMView(), PetscViewerMatlabPutArray(), PETSCVIEWERBINARY, PETSCVIEWERASCII, PETSCVIEWERDRAW,
+           PETSC_VIEWER_STDOUT_(), PetscViewerFileSetName(), PetscViewerFileSetMode(), PetscViewerFormat
 
 M*/
-
-typedef struct {
-  MATFile       *ep;
-  PetscMPIInt   rank;
-  PetscFileMode btype;
-} PetscViewer_Matlab;
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMatlabPutArray"
-/*@C
-    PetscViewerMatlabPutArray - Puts an array into the MATLAB viewer.
-
-      Not collective: only processor zero saves the array
-
-    Input Parameters:
-+    mfile - the viewer
-.    m,n - the dimensions of the array
-.    array - the array (represented in one dimension)
--    name - the name of the array
-
-   Level: advanced
-
-     Notes: Only writes array values on processor 0.
-
-@*/
-PetscErrorCode  PetscViewerMatlabPutArray(PetscViewer mfile,int m,int n,const PetscScalar *array,const char *name)
-{
-  PetscErrorCode     ierr;
-  PetscViewer_Matlab *ml = (PetscViewer_Matlab*)mfile->data;
-  mxArray            *mat;
-
-  PetscFunctionBegin;
-  if (!ml->rank) {
-    ierr = PetscInfo1(mfile,"Putting MATLAB array %s\n",name);CHKERRQ(ierr);
-#if !defined(PETSC_USE_COMPLEX)
-    mat  = mxCreateDoubleMatrix(m,n,mxREAL);
-#else
-    mat  = mxCreateDoubleMatrix(m,n,mxCOMPLEX);
-#endif
-    ierr = PetscMemcpy(mxGetPr(mat),array,m*n*sizeof(PetscScalar));CHKERRQ(ierr);
-    matPutVariable(ml->ep,name,mat);
-
-    ierr = PetscInfo1(mfile,"Put MATLAB array %s\n",name);CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMatlabPutVariable"
-PetscErrorCode  PetscViewerMatlabPutVariable(PetscViewer viewer,const char *name,void *mat)
-{
-  PetscViewer_Matlab *ml = (PetscViewer_Matlab*)viewer->data;
-
-  PetscFunctionBegin;
-  matPutVariable(ml->ep,name,(mxArray*)mat);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMatlabGetArray"
-/*@C
-    PetscViewerMatlabGetArray - Gets a variable from a MATLAB viewer into an array
-
-    Not Collective; only processor zero reads in the array
-
-    Input Parameters:
-+    mfile - the MATLAB file viewer
-.    m,n - the dimensions of the array
-.    array - the array (represented in one dimension)
--    name - the name of the array
-
-   Level: advanced
-
-     Notes: Only reads in array values on processor 0.
-
-@*/
-PetscErrorCode  PetscViewerMatlabGetArray(PetscViewer mfile,int m,int n,PetscScalar *array,const char *name)
-{
-  PetscErrorCode     ierr;
-  PetscViewer_Matlab *ml = (PetscViewer_Matlab*)mfile->data;
-  mxArray            *mat;
-
-  PetscFunctionBegin;
-  if (!ml->rank) {
-    ierr = PetscInfo1(mfile,"Getting MATLAB array %s\n",name);CHKERRQ(ierr);
-    mat  = matGetVariable(ml->ep,name);
-    if (!mat) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"Unable to get array %s from matlab",name);
-    ierr = PetscMemcpy(array,mxGetPr(mat),m*n*sizeof(PetscScalar));CHKERRQ(ierr);
-    ierr = PetscInfo1(mfile,"Got MATLAB array %s\n",name);CHKERRQ(ierr);
-  }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerFileSetMode_Matlab"
-PetscErrorCode  PetscViewerFileSetMode_Matlab(PetscViewer viewer,PetscFileMode type)
-{
-  PetscViewer_Matlab *vmatlab = (PetscViewer_Matlab*)viewer->data;
-
-  PetscFunctionBegin;
-  vmatlab->btype = type;
-  PetscFunctionReturn(0);
-}
-
-/*
-        Actually opens the file
-*/
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerFileSetName_Matlab"
-PetscErrorCode  PetscViewerFileSetName_Matlab(PetscViewer viewer,const char name[])
-{
-  PetscViewer_Matlab *vmatlab = (PetscViewer_Matlab*)viewer->data;
-  PetscFileMode      type     = vmatlab->btype;
-
-  PetscFunctionBegin;
-  if (type == (PetscFileMode) -1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ORDER,"Must call PetscViewerFileSetMode() before PetscViewerFileSetName()");
-  if (vmatlab->ep) matClose(vmatlab->ep);
-
-  /* only first processor opens file */
-  if (!vmatlab->rank) {
-    if (type == FILE_MODE_READ) vmatlab->ep = matOpen(name,"r");
-    else if (type == FILE_MODE_WRITE || type == FILE_MODE_WRITE) vmatlab->ep = matOpen(name,"w");
-    else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Unknown file type");
-  }
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDestroy_Matlab"
-PetscErrorCode PetscViewerDestroy_Matlab(PetscViewer v)
-{
-  PetscErrorCode     ierr;
-  PetscViewer_Matlab *vf = (PetscViewer_Matlab*)v->data;
-
-  PetscFunctionBegin;
-  if (vf->ep) matClose(vf->ep);
-  ierr = PetscFree(vf);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerCreate_Matlab"
 PETSC_EXTERN PetscErrorCode PetscViewerCreate_Matlab(PetscViewer viewer)
 {
   PetscErrorCode     ierr;
@@ -196,8 +184,6 @@ PETSC_EXTERN PetscErrorCode PetscViewerCreate_Matlab(PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerMatlabOpen"
 /*@C
    PetscViewerMatlabOpen - Opens a Matlab .mat file for output
 
@@ -226,7 +212,7 @@ $    FILE_MODE_WRITE - open existing file for MATLAB output
    Concepts: MATLAB .mat files
    Concepts: PetscViewerMatlab^creating
 
-.seealso: PetscViewerASCIIOpen(), PetscViewerSetFormat(), PetscViewerDestroy(), PETSCVIEWERBINARY, PetscViewerBinaryOpen()
+.seealso: PetscViewerASCIIOpen(), PetscViewerPushFormat(), PetscViewerDestroy(), PETSCVIEWERBINARY, PetscViewerBinaryOpen()
           VecView(), MatView(), VecLoad(), MatLoad()
 @*/
 PetscErrorCode  PetscViewerMatlabOpen(MPI_Comm comm,const char name[],PetscFileMode type,PetscViewer *binv)
@@ -243,8 +229,6 @@ PetscErrorCode  PetscViewerMatlabOpen(MPI_Comm comm,const char name[],PetscFileM
 
 static PetscMPIInt Petsc_Viewer_Matlab_keyval = MPI_KEYVAL_INVALID;
 
-#undef __FUNCT__
-#define __FUNCT__ "PETSC_VIEWER_MATLAB_"
 /*@C
      PETSC_VIEWER_MATLAB_ - Creates a Matlab PetscViewer shared by all processors
                      in a communicator.
@@ -283,10 +267,10 @@ PetscViewer  PETSC_VIEWER_MATLAB_(MPI_Comm comm)
   PetscFunctionBegin;
   ierr = PetscCommDuplicate(comm,&ncomm,NULL);if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
   if (Petsc_Viewer_Matlab_keyval == MPI_KEYVAL_INVALID) {
-    ierr = MPI_Keyval_create(MPI_NULL_COPY_FN,MPI_NULL_DELETE_FN,&Petsc_Viewer_Matlab_keyval,0);
+    ierr = MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN,MPI_COMM_NULL_DELETE_FN,&Petsc_Viewer_Matlab_keyval,0);
     if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
   }
-  ierr = MPI_Attr_get(ncomm,Petsc_Viewer_Matlab_keyval,(void**)&viewer,(int*)&flg);
+  ierr = MPI_Comm_get_attr(ncomm,Petsc_Viewer_Matlab_keyval,(void**)&viewer,(int*)&flg);
   if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
   if (!flg) { /* PetscViewer not yet created */
     ierr = PetscOptionsGetenv(ncomm,"PETSC_VIEWER_MATLAB_FILENAME",fname,PETSC_MAX_PATH_LEN,&flg);
@@ -299,7 +283,7 @@ PetscViewer  PETSC_VIEWER_MATLAB_(MPI_Comm comm)
     if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
     ierr = PetscObjectRegisterDestroy((PetscObject)viewer);
     if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
-    ierr = MPI_Attr_put(ncomm,Petsc_Viewer_Matlab_keyval,(void*)viewer);
+    ierr = MPI_Comm_set_attr(ncomm,Petsc_Viewer_Matlab_keyval,(void*)viewer);
     if (ierr) {PetscError(PETSC_COMM_SELF,__LINE__,"PETSC_VIEWER_MATLAB_",__FILE__,PETSC_ERR_PLIB,PETSC_ERROR_INITIAL," ");PetscFunctionReturn(0);}
   }
   ierr = PetscCommDestroy(&ncomm);
