@@ -1,30 +1,31 @@
 
 #include <petscdraw.h>
 #include <petscviewer.h>
-
-extern PetscLogEvent PETSC_Barrier,PETSC_BuildTwoSided,PETSC_BuildTwoSidedF;
+#include <petsc/private/viewerimpl.h>
 
 static PetscBool PetscSysPackageInitialized = PETSC_FALSE;
-#undef __FUNCT__
-#define __FUNCT__ "PetscSysFinalizePackage"
+
 /*@C
-  PetscSysFinalizePackage - This function destroys everything in the Petsc interface to Mathematica. It is
-  called from PetscFinalize().
+  PetscSysFinalizePackage - This function destroys everything in the PETSc created internally in the system library portion of PETSc.
+  It is called from PetscFinalize().
 
   Level: developer
 
-.keywords: Petsc, destroy, package, mathematica
+.keywords: Petsc, destroy, package
 .seealso: PetscFinalize()
 @*/
 PetscErrorCode  PetscSysFinalizePackage(void)
 {
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
+  if (Petsc_Seq_keyval != MPI_KEYVAL_INVALID) {
+    ierr = MPI_Comm_free_keyval(&Petsc_Seq_keyval);CHKERRQ(ierr);
+  }
   PetscSysPackageInitialized = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSysInitializePackage"
 /*@C
   PetscSysInitializePackage - This function initializes everything in the main Petsc package. It is called
   from PetscDLLibraryRegister() when using dynamic libraries, and on the call to PetscInitialize()
@@ -38,8 +39,7 @@ PetscErrorCode  PetscSysFinalizePackage(void)
 PetscErrorCode  PetscSysInitializePackage(void)
 {
   char           logList[256];
-  char           *className;
-  PetscBool      opt;
+  PetscBool      opt,pkg;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -54,20 +54,16 @@ PetscErrorCode  PetscSysInitializePackage(void)
   ierr = PetscLogEventRegister("BuildTwoSided",PETSC_SMALLEST_CLASSID,&PETSC_BuildTwoSided);CHKERRQ(ierr);
   ierr = PetscLogEventRegister("BuildTwoSidedF",PETSC_SMALLEST_CLASSID,&PETSC_BuildTwoSidedF);CHKERRQ(ierr);
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL, "-info_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "null", &className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscInfoDeactivateClass(0);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("null",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) {ierr = PetscInfoDeactivateClass(0);CHKERRQ(ierr);}
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(NULL,NULL, "-log_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "null", &className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscLogEventDeactivateClass(0);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("null",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) {ierr = PetscLogEventExcludeClass(PETSC_SMALLEST_CLASSID);CHKERRQ(ierr);}
   }
   ierr = PetscRegisterFinalize(PetscSysFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -84,11 +80,8 @@ PETSC_EXTERN PetscErrorCode PetscDLLibraryRegister_petscsnes(void);
 PETSC_EXTERN PetscErrorCode PetscDLLibraryRegister_petscts(void);
 #endif
 
-#undef __FUNCT__
 #if defined(PETSC_USE_SINGLE_LIBRARY)
-#define __FUNCT__ "PetscDLLibraryRegister_petsc"
 #else
-#define __FUNCT__ "PetscDLLibraryRegister_petscsys"
 #endif
 /*
   PetscDLLibraryRegister - This function is called when the dynamic library it is in is opened.
