@@ -4,10 +4,8 @@
 PetscClassId PETSC_VIEWER_CLASSID;
 
 static PetscBool PetscViewerPackageInitialized = PETSC_FALSE;
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerFinalizePackage"
 /*@C
-  PetscViewerFinalizePackage - This function destroys everything in the Petsc interface to Mathematica. It is
+  PetscViewerFinalizePackage - This function destroys any global objects created in the Petsc viewers. It is
   called from PetscFinalize().
 
   Level: developer
@@ -20,14 +18,37 @@ PetscErrorCode  PetscViewerFinalizePackage(void)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  if (Petsc_Viewer_keyval != MPI_KEYVAL_INVALID) {
+    ierr = MPI_Comm_free_keyval(&Petsc_Viewer_keyval);CHKERRQ(ierr);
+  }
+  if (Petsc_Viewer_Stdout_keyval != MPI_KEYVAL_INVALID) {
+    ierr = MPI_Comm_free_keyval(&Petsc_Viewer_Stdout_keyval);CHKERRQ(ierr);
+  }
+  if (Petsc_Viewer_Stderr_keyval != MPI_KEYVAL_INVALID) {
+    ierr = MPI_Comm_free_keyval(&Petsc_Viewer_Stderr_keyval);CHKERRQ(ierr);
+  }
+  if (Petsc_Viewer_Binary_keyval != MPI_KEYVAL_INVALID) {
+    ierr = MPI_Comm_free_keyval(&Petsc_Viewer_Binary_keyval);CHKERRQ(ierr);
+  }
+  if (Petsc_Viewer_Draw_keyval != MPI_KEYVAL_INVALID) {
+    ierr = MPI_Comm_free_keyval(&Petsc_Viewer_Draw_keyval);CHKERRQ(ierr);
+  }
+#if defined(PETSC_HAVE_HDF5)
+  if (Petsc_Viewer_HDF5_keyval != MPI_KEYVAL_INVALID) {
+    ierr = MPI_Comm_free_keyval(&Petsc_Viewer_HDF5_keyval);CHKERRQ(ierr);
+  }
+#endif
+#if defined(PETSC_USE_SOCKETVIEWER)
+  if (Petsc_Viewer_Socket_keyval != MPI_KEYVAL_INVALID) {
+    ierr = MPI_Comm_free_keyval(&Petsc_Viewer_Socket_keyval);CHKERRQ(ierr);
+  }
+#endif
   ierr = PetscFunctionListDestroy(&PetscViewerList);CHKERRQ(ierr);
   PetscViewerPackageInitialized = PETSC_FALSE;
   PetscViewerRegisterAllCalled  = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerInitializePackage"
 /*@C
   PetscViewerInitializePackage - This function initializes everything in the main PetscViewer package.
 
@@ -39,8 +60,7 @@ PetscErrorCode  PetscViewerFinalizePackage(void)
 PetscErrorCode  PetscViewerInitializePackage(void)
 {
   char           logList[256];
-  char           *className;
-  PetscBool      opt;
+  PetscBool      opt,pkg;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -48,35 +68,28 @@ PetscErrorCode  PetscViewerInitializePackage(void)
   PetscViewerPackageInitialized = PETSC_TRUE;
   /* Register Classes */
   ierr = PetscClassIdRegister("Viewer",&PETSC_VIEWER_CLASSID);CHKERRQ(ierr);
-
   /* Register Constructors */
   ierr = PetscViewerRegisterAll();CHKERRQ(ierr);
-
   /* Process info exclusions */
-  ierr = PetscOptionsGetString(NULL, "-info_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-info_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "viewer", &className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscInfoDeactivateClass(0);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("viewer",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) {ierr = PetscInfoDeactivateClass(PETSC_VIEWER_CLASSID);CHKERRQ(ierr);}
   }
   /* Process summary exclusions */
-  ierr = PetscOptionsGetString(NULL, "-log_summary_exclude", logList, 256, &opt);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-log_exclude",logList,sizeof(logList),&opt);CHKERRQ(ierr);
   if (opt) {
-    ierr = PetscStrstr(logList, "viewer", &className);CHKERRQ(ierr);
-    if (className) {
-      ierr = PetscLogEventDeactivateClass(0);CHKERRQ(ierr);
-    }
+    ierr = PetscStrInList("viewer",logList,',',&pkg);CHKERRQ(ierr);
+    if (pkg) {ierr = PetscLogEventExcludeClass(PETSC_VIEWER_CLASSID);CHKERRQ(ierr);}
   }
 #if defined(PETSC_HAVE_MATHEMATICA)
   ierr = PetscViewerMathematicaInitializePackage();CHKERRQ(ierr);
 #endif
+  /* Register package finalizer */
   ierr = PetscRegisterFinalize(PetscViewerFinalizePackage);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerDestroy"
 /*@
    PetscViewerDestroy - Destroys a PetscViewer.
 
@@ -109,8 +122,63 @@ PetscErrorCode  PetscViewerDestroy(PetscViewer *viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerGetType"
+/*@C
+   PetscViewerAndFormatCreate - Creates a PetscViewerAndFormat struct.
+
+   Collective on PetscViewer
+
+   Input Parameters:
++  viewer - the viewer
+-  format - the format 
+
+   Output Parameter:
+.   vf - viewer and format object
+
+   Notes:
+    This increases the reference count of the viewer so you can destroy the viewer object after this call
+   Level: developer
+
+   This is used as the context variable for many of the TS, SNES, and KSP monitor functions
+
+.seealso: PetscViewerSocketOpen(), PetscViewerASCIIOpen(), PetscViewerCreate(), PetscViewerDrawOpen(), PetscViewerAndFormatDestroy()
+
+@*/
+PetscErrorCode  PetscViewerAndFormatCreate(PetscViewer viewer, PetscViewerFormat format,PetscViewerAndFormat **vf)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectReference((PetscObject)viewer);CHKERRQ(ierr);
+  ierr = PetscNew(vf);CHKERRQ(ierr);
+  (*vf)->viewer = viewer;
+  (*vf)->format = format;
+  PetscFunctionReturn(0);
+}
+
+
+/*@C
+   PetscViewerAndFormatDestroy - Destroys a PetscViewerAndFormat struct.
+
+   Collective on PetscViewer
+
+   Input Parameters:
+.  viewer - the PetscViewerAndFormat to be destroyed.
+
+   Level: developer
+
+.seealso: PetscViewerSocketOpen(), PetscViewerASCIIOpen(), PetscViewerCreate(), PetscViewerDrawOpen(), PetscViewerAndFormatCreate()
+
+@*/
+PetscErrorCode  PetscViewerAndFormatDestroy(PetscViewerAndFormat **vf)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscViewerDestroy(&(*vf)->viewer);CHKERRQ(ierr);
+  ierr = PetscFree(*vf);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@C
    PetscViewerGetType - Returns the type of a PetscViewer.
 
@@ -148,8 +216,6 @@ PetscErrorCode  PetscViewerGetType(PetscViewer viewer,PetscViewerType *type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerSetOptionsPrefix"
 /*@C
    PetscViewerSetOptionsPrefix - Sets the prefix used for searching for all
    PetscViewer options in the database.
@@ -180,8 +246,6 @@ PetscErrorCode  PetscViewerSetOptionsPrefix(PetscViewer viewer,const char prefix
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerAppendOptionsPrefix"
 /*@C
    PetscViewerAppendOptionsPrefix - Appends to the prefix used for searching for all
    PetscViewer options in the database.
@@ -212,8 +276,6 @@ PetscErrorCode  PetscViewerAppendOptionsPrefix(PetscViewer viewer,const char pre
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerGetOptionsPrefix"
 /*@C
    PetscViewerGetOptionsPrefix - Sets the prefix used for searching for all
    PetscViewer options in the database.
@@ -226,7 +288,8 @@ PetscErrorCode  PetscViewerAppendOptionsPrefix(PetscViewer viewer,const char pre
    Output Parameter:
 .  prefix - pointer to the prefix string used
 
-   Notes: On the fortran side, the user should pass in a string 'prefix' of
+   Notes:
+    On the fortran side, the user should pass in a string 'prefix' of
    sufficient length to hold the prefix.
 
    Level: advanced
@@ -245,8 +308,6 @@ PetscErrorCode  PetscViewerGetOptionsPrefix(PetscViewer viewer,const char *prefi
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerSetUp"
 /*@
    PetscViewerSetUp - Sets up the internal viewer data structures for the later use.
 
@@ -279,8 +340,6 @@ PetscErrorCode  PetscViewerSetUp(PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerView"
 /*@C
    PetscViewerView - Visualizes a viewer object.
 
@@ -301,7 +360,7 @@ PetscErrorCode  PetscViewerSetUp(PetscViewer viewer)
 
    Level: beginner
 
-.seealso: PetscViewerSetFormat(), PetscViewerASCIIOpen(), PetscViewerDrawOpen(),
+.seealso: PetscViewerPushFormat(), PetscViewerASCIIOpen(), PetscViewerDrawOpen(),
           PetscViewerSocketOpen(), PetscViewerBinaryOpen(), PetscViewerLoad()
 @*/
 PetscErrorCode  PetscViewerView(PetscViewer v,PetscViewer viewer)
@@ -352,8 +411,6 @@ PetscErrorCode  PetscViewerView(PetscViewer v,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerRead"
 /*@C
    PetscViewerRead - Reads data from a PetscViewer
 
@@ -368,11 +425,15 @@ PetscErrorCode  PetscViewerView(PetscViewer v,PetscViewer viewer)
    Output Parameters:
 .  count - number of items of data actually read, or NULL
 
+   Notes:
+   If datatype is PETSC_STRING and num is negative, reads until a newline character is found,
+   until a maximum of (-num - 1) chars.
+
    Level: beginner
 
    Concepts: binary files, ascii files
 
-.seealso: PetscViewerASCIIOpen(), PetscViewerSetFormat(), PetscViewerDestroy(),
+.seealso: PetscViewerASCIIOpen(), PetscViewerPushFormat(), PetscViewerDestroy(),
           VecView(), MatView(), VecLoad(), MatLoad(), PetscViewerBinaryGetDescriptor(),
           PetscViewerBinaryGetInfoPointer(), PetscFileMode, PetscViewer
 @*/
@@ -385,16 +446,25 @@ PetscErrorCode  PetscViewerRead(PetscViewer viewer, void *data, PetscInt num, Pe
   if (dtype == PETSC_STRING) {
     PetscInt c, i = 0, cnt;
     char *s = (char *)data;
-    for (c = 0; c < num; c++) {
-      /* Skip leading whitespaces */
-      do {ierr = (*viewer->ops->read)(viewer, &(s[i]), 1, &cnt, PETSC_CHAR);CHKERRQ(ierr); if (count && !cnt) break;}
-      while (s[i]=='\n' || s[i]=='\t' || s[i]==' ' || s[i]=='\0' || s[i]=='\v' || s[i]=='\f' || s[i]=='\r');
-      i++;
-      /* Read strings one char at a time */
-      do {ierr = (*viewer->ops->read)(viewer, &(s[i++]), 1, &cnt, PETSC_CHAR);CHKERRQ(ierr); if (count && !cnt) break;}
-      while (s[i-1]!='\n' && s[i-1]!='\t' && s[i-1]!=' ' && s[i-1]!='\0' && s[i-1]!='\v' && s[i-1]!='\f' && s[i-1]!='\r');
+    if (num >= 0) {
+      for (c = 0; c < num; c++) {
+        /* Skip leading whitespaces */
+        do {ierr = (*viewer->ops->read)(viewer, &(s[i]), 1, &cnt, PETSC_CHAR);CHKERRQ(ierr); if (count && !cnt) break;}
+        while (s[i]=='\n' || s[i]=='\t' || s[i]==' ' || s[i]=='\0' || s[i]=='\v' || s[i]=='\f' || s[i]=='\r');
+        i++;
+        /* Read strings one char at a time */
+        do {ierr = (*viewer->ops->read)(viewer, &(s[i++]), 1, &cnt, PETSC_CHAR);CHKERRQ(ierr); if (count && !cnt) break;}
+        while (s[i-1]!='\n' && s[i-1]!='\t' && s[i-1]!=' ' && s[i-1]!='\0' && s[i-1]!='\v' && s[i-1]!='\f' && s[i-1]!='\r');
+        /* Terminate final string */
+        if (c == num-1) s[i-1] = '\0';
+      }
+    } else {
+      /* Read until a \n is encountered (-num is the max size allowed) */
+      do {ierr = (*viewer->ops->read)(viewer, &(s[i++]), 1, &cnt, PETSC_CHAR);CHKERRQ(ierr); if (i == -num && !cnt) break;}
+      while (s[i-1]!='\n');
       /* Terminate final string */
-      if (c == num-1) s[i-1] = '\0';
+      s[i-1] = '\0';
+      c      = i;
     }
     if (count) *count = c;
     else if (c < num) SETERRQ2(PetscObjectComm((PetscObject) viewer), PETSC_ERR_FILE_READ, "Insufficient data, only read %D < %D strings", c, num);
