@@ -21,6 +21,7 @@ const char *const PetscViewerFormats[] = {
   "ASCII_FACTOR_INFO",
   "ASCII_LATEX",
   "ASCII_XML",
+  "ASCII_GLVIS",
   "DRAW_BASIC",
   "DRAW_LG",
   "DRAW_CONTOUR",
@@ -30,15 +31,16 @@ const char *const PetscViewerFormats[] = {
   "VTK_VTU",
   "BINARY_MATLAB",
   "NATIVE",
+  "HDF5_PETSC",
   "HDF5_VIZ",
+  "HDF5_XDMF",
   "NOFORMAT",
+  "LOAD_BALANCE",
   "PetscViewerFormat",
   "PETSC_VIEWER_",
   0
 };
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerSetFormat"
 /*@C
    PetscViewerSetFormat - Sets the format for PetscViewers.
 
@@ -100,8 +102,6 @@ PetscErrorCode  PetscViewerSetFormat(PetscViewer viewer,PetscViewerFormat format
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerPushFormat"
 /*@C
    PetscViewerPushFormat - Sets the format for file PetscViewers.
 
@@ -131,7 +131,8 @@ PetscErrorCode  PetscViewerSetFormat(PetscViewer viewer,PetscViewerFormat format
        matrices are stored as dense), for DMDA vectors displays vectors in DMDA ordering, not natural
 .    PETSC_VIEWER_DRAW_BASIC - views the vector with a simple 1d plot
 .    PETSC_VIEWER_DRAW_LG - views the vector with a line graph
--    PETSC_VIEWER_DRAW_CONTOUR - views the vector with a contour plot
+.    PETSC_VIEWER_DRAW_CONTOUR - views the vector with a contour plot
+-    PETSC_VIEWER_ASCII_XML - saves the data in XML format, needed for PetscLogView() when viewing with PetscLogNestedBegin()
 
    These formats are most often used for viewing matrices and vectors.
    Currently, the object name is used only in the MATLAB format.
@@ -153,8 +154,6 @@ PetscErrorCode  PetscViewerPushFormat(PetscViewer viewer,PetscViewerFormat forma
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerPopFormat"
 /*@C
    PetscViewerPopFormat - Resets the format for file PetscViewers.
 
@@ -180,8 +179,6 @@ PetscErrorCode  PetscViewerPopFormat(PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscViewerGetFormat"
 PetscErrorCode  PetscViewerGetFormat(PetscViewer viewer,PetscViewerFormat *format)
 {
   PetscFunctionBegin;
