@@ -11,24 +11,22 @@
 */
 PetscDLLibrary PetscDLLibrariesLoaded = 0;
 
-#if defined(PETSC_HAVE_DYNAMIC_LIBRARIES)
+#if defined(PETSC_HAVE_DYNAMIC_LIBRARIES) && defined(PETSC_USE_SHARED_LIBRARIES)
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscLoadDynamicLibrary"
-static PetscErrorCode  PetscLoadDynamicLibrary(const char *name,PetscBool  *found)
+PetscErrorCode  PetscLoadDynamicLibrary(const char *name,PetscBool  *found)
 {
   char           libs[PETSC_MAX_PATH_LEN],dlib[PETSC_MAX_PATH_LEN];
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscStrcpy(libs,"${PETSC_LIB_DIR}/libpetsc");CHKERRQ(ierr);
-  ierr = PetscStrcat(libs,name);CHKERRQ(ierr);
+  ierr = PetscStrncpy(libs,"${PETSC_LIB_DIR}/libpetsc",sizeof(libs));CHKERRQ(ierr);
+  ierr = PetscStrlcat(libs,name,sizeof(libs));CHKERRQ(ierr);
   ierr = PetscDLLibraryRetrieve(PETSC_COMM_WORLD,libs,dlib,1024,found);CHKERRQ(ierr);
   if (*found) {
     ierr = PetscDLLibraryAppend(PETSC_COMM_WORLD,&PetscDLLibrariesLoaded,dlib);CHKERRQ(ierr);
   } else {
-    ierr = PetscStrcpy(libs,"${PETSC_DIR}/${PETSC_ARCH}/lib/libpetsc");CHKERRQ(ierr);
-    ierr = PetscStrcat(libs,name);CHKERRQ(ierr);
+    ierr = PetscStrncpy(libs,"${PETSC_DIR}/${PETSC_ARCH}/lib/libpetsc",sizeof(libs));CHKERRQ(ierr);
+    ierr = PetscStrlcat(libs,name,sizeof(libs));CHKERRQ(ierr);
     ierr = PetscDLLibraryRetrieve(PETSC_COMM_WORLD,libs,dlib,1024,found);CHKERRQ(ierr);
     if (*found) {
       ierr = PetscDLLibraryAppend(PETSC_COMM_WORLD,&PetscDLLibrariesLoaded,dlib);CHKERRQ(ierr);
@@ -40,44 +38,44 @@ static PetscErrorCode  PetscLoadDynamicLibrary(const char *name,PetscBool  *foun
 #endif
 
 #if defined(PETSC_HAVE_THREADSAFETY)
-extern PetscErrorCode AOInitializePackage(void);
-extern PetscErrorCode PetscSFInitializePackage(void);
-extern PetscErrorCode CharacteristicInitializePackage(void);
-extern PetscErrorCode ISInitializePackage(void);
-extern PetscErrorCode VecInitializePackage(void);
-extern PetscErrorCode MatInitializePackage(void);
-extern PetscErrorCode DMInitializePackage(void);
-extern PetscErrorCode PCInitializePackage(void);
-extern PetscErrorCode KSPInitializePackage(void);
-extern PetscErrorCode SNESInitializePackage(void);
-extern PetscErrorCode TSInitializePackage(void);
+PETSC_EXTERN PetscErrorCode AOInitializePackage(void);
+PETSC_EXTERN PetscErrorCode PetscSFInitializePackage(void);
+#if !defined(PETSC_USE_COMPLEX)
+PETSC_EXTERN PetscErrorCode CharacteristicInitializePackage(void);
+#endif
+PETSC_EXTERN PetscErrorCode ISInitializePackage(void);
+PETSC_EXTERN PetscErrorCode VecInitializePackage(void);
+PETSC_EXTERN PetscErrorCode MatInitializePackage(void);
+PETSC_EXTERN PetscErrorCode DMInitializePackage(void);
+PETSC_EXTERN PetscErrorCode PCInitializePackage(void);
+PETSC_EXTERN PetscErrorCode KSPInitializePackage(void);
+PETSC_EXTERN PetscErrorCode SNESInitializePackage(void);
+PETSC_EXTERN PetscErrorCode TSInitializePackage(void);
 static MPI_Comm PETSC_COMM_WORLD_INNER = 0,PETSC_COMM_SELF_INNER = 0;
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscInitialize_DynamicLibraries"
 /*
     PetscInitialize_DynamicLibraries - Adds the default dynamic link libraries to the
     search path.
 */
-PetscErrorCode  PetscInitialize_DynamicLibraries(void)
+PETSC_INTERN PetscErrorCode PetscInitialize_DynamicLibraries(void)
 {
   char           *libname[32];
   PetscErrorCode ierr;
   PetscInt       nmax,i;
-#if defined(PETSC_HAVE_DYNAMIC_LIBRARIES)
+#if defined(PETSC_USE_DYNAMIC_LIBRARIES) && defined(PETSC_USE_SHARED_LIBRARIES)
   PetscBool      preload;
 #endif
 
   PetscFunctionBegin;
   nmax = 32;
-  ierr = PetscOptionsGetStringArray(NULL,"-dll_prepend",libname,&nmax,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetStringArray(NULL,NULL,"-dll_prepend",libname,&nmax,NULL);CHKERRQ(ierr);
   for (i=0; i<nmax; i++) {
     ierr = PetscDLLibraryPrepend(PETSC_COMM_WORLD,&PetscDLLibrariesLoaded,libname[i]);CHKERRQ(ierr);
     ierr = PetscFree(libname[i]);CHKERRQ(ierr);
   }
 
-#if !defined(PETSC_HAVE_DYNAMIC_LIBRARIES)
+#if !defined(PETSC_USE_DYNAMIC_LIBRARIES) || !defined(PETSC_USE_SHARED_LIBRARIES)
   /*
       This just initializes the most basic PETSc stuff.
 
@@ -87,7 +85,7 @@ PetscErrorCode  PetscInitialize_DynamicLibraries(void)
   ierr = PetscSysInitializePackage();CHKERRQ(ierr);
 #else
   preload = PETSC_FALSE;
-  ierr = PetscOptionsGetBool(NULL,"-dynamic_library_preload",&preload,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-dynamic_library_preload",&preload,NULL);CHKERRQ(ierr);
   if (preload) {
     PetscBool found;
 #if defined(PETSC_USE_SINGLE_LIBRARY)
@@ -113,7 +111,7 @@ PetscErrorCode  PetscInitialize_DynamicLibraries(void)
 #endif
 
   nmax = 32;
-  ierr = PetscOptionsGetStringArray(NULL,"-dll_append",libname,&nmax,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetStringArray(NULL,NULL,"-dll_append",libname,&nmax,NULL);CHKERRQ(ierr);
   for (i=0; i<nmax; i++) {
     ierr = PetscDLLibraryAppend(PETSC_COMM_WORLD,&PetscDLLibrariesLoaded,libname[i]);CHKERRQ(ierr);
     ierr = PetscFree(libname[i]);CHKERRQ(ierr);
@@ -123,7 +121,9 @@ PetscErrorCode  PetscInitialize_DynamicLibraries(void)
   /* These must be done here because it is not safe for individual threads to call these initialize routines */
   ierr = AOInitializePackage();CHKERRQ(ierr);
   ierr = PetscSFInitializePackage();CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)
   ierr = CharacteristicInitializePackage();CHKERRQ(ierr);
+#endif
   ierr = ISInitializePackage();CHKERRQ(ierr);
   ierr = VecInitializePackage();CHKERRQ(ierr);
   ierr = MatInitializePackage();CHKERRQ(ierr);
@@ -138,18 +138,16 @@ PetscErrorCode  PetscInitialize_DynamicLibraries(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscFinalize_DynamicLibraries"
 /*
      PetscFinalize_DynamicLibraries - Closes the opened dynamic libraries.
 */
-PetscErrorCode PetscFinalize_DynamicLibraries(void)
+PETSC_INTERN PetscErrorCode PetscFinalize_DynamicLibraries(void)
 {
   PetscErrorCode ierr;
   PetscBool      flg = PETSC_FALSE;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsGetBool(NULL,"-dll_view",&flg,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-dll_view",&flg,NULL);CHKERRQ(ierr);
   if (flg) { ierr = PetscDLLibraryPrintPath(PetscDLLibrariesLoaded);CHKERRQ(ierr); }
   ierr = PetscDLLibraryClose(PetscDLLibrariesLoaded);CHKERRQ(ierr);
 
@@ -205,8 +203,6 @@ static PetscFunctionList dlallhead = 0;
 .seealso: PetscFunctionListDestroy(), SNESRegister(), KSPRegister(),
           PCRegister(), TSRegister(), PetscFunctionList, PetscObjectComposeFunction()
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "PetscFunctionListAdd_Private"
 PETSC_EXTERN PetscErrorCode PetscFunctionListAdd_Private(PetscFunctionList *fl,const char name[],void (*fnc)(void))
 {
   PetscFunctionList entry,ne;
@@ -220,7 +216,7 @@ PETSC_EXTERN PetscErrorCode PetscFunctionListAdd_Private(PetscFunctionList *fl,c
     entry->next    = 0;
     *fl            = entry;
 
-#if defined(PETSC_USE_LOG)
+#if defined(PETSC_USE_DEBUG)
     /* add this new list to list of all lists */
     if (!dlallhead) {
       dlallhead        = *fl;
@@ -256,8 +252,6 @@ PETSC_EXTERN PetscErrorCode PetscFunctionListAdd_Private(PetscFunctionList *fl,c
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscFunctionListDestroy"
 /*@
     PetscFunctionListDestroy - Destroys a list of registered routines.
 
@@ -305,8 +299,6 @@ PetscErrorCode  PetscFunctionListDestroy(PetscFunctionList *fl)
 /*
    Print any PetscFunctionLists that have not be destroyed
 */
-#undef __FUNCT__
-#define __FUNCT__ "PetscFunctionListPrintAll"
 PetscErrorCode  PetscFunctionListPrintAll(void)
 {
   PetscFunctionList tmp = dlallhead;
@@ -341,8 +333,6 @@ PetscErrorCode  PetscFunctionListPrintAll(void)
 
 .seealso: PetscFunctionListAdd(), PetscFunctionList, PetscObjectQueryFunction()
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "PetscFunctionListFind_Private"
 PETSC_EXTERN PetscErrorCode PetscFunctionListFind_Private(PetscFunctionList fl,const char name[],void (**r)(void))
 {
   PetscFunctionList entry = fl;
@@ -364,8 +354,6 @@ PETSC_EXTERN PetscErrorCode PetscFunctionListFind_Private(PetscFunctionList fl,c
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscFunctionListView"
 /*@
    PetscFunctionListView - prints out contents of an PetscFunctionList
 
@@ -400,8 +388,6 @@ PetscErrorCode  PetscFunctionListView(PetscFunctionList list,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscFunctionListGet"
 /*@C
    PetscFunctionListGet - Gets an array the contains the entries in PetscFunctionList, this is used
          by help etc.
@@ -434,7 +420,7 @@ PetscErrorCode  PetscFunctionListGet(PetscFunctionList list,const char ***array,
     list = list->next;
     count++;
   }
-  ierr  = PetscMalloc1(count+1,array);CHKERRQ(ierr);
+  ierr  = PetscMalloc1(count+1,(char***)array);CHKERRQ(ierr);
   count = 0;
   while (klist) {
     (*array)[count] = klist->name;
@@ -447,8 +433,6 @@ PetscErrorCode  PetscFunctionListGet(PetscFunctionList list,const char ***array,
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscFunctionListPrintTypes"
 /*@C
    PetscFunctionListPrintTypes - Prints the methods available.
 
@@ -476,8 +460,8 @@ PetscErrorCode  PetscFunctionListGet(PetscFunctionList list,const char ***array,
   PetscFunctionBegin;
   if (!fd) fd = PETSC_STDOUT;
 
-  ierr = PetscStrcpy(p,"-");CHKERRQ(ierr);
-  if (prefix) {ierr = PetscStrcat(p,prefix);CHKERRQ(ierr);}
+  ierr = PetscStrncpy(p,"-",sizeof(p));CHKERRQ(ierr);
+  if (prefix) {ierr = PetscStrlcat(p,prefix,sizeof(p));CHKERRQ(ierr);}
   ierr = PetscFPrintf(comm,fd,"  %s%s <%s>: %s (one of)",p,name+1,def,text);CHKERRQ(ierr);
 
   while (list) {
@@ -488,8 +472,6 @@ PetscErrorCode  PetscFunctionListGet(PetscFunctionList list,const char ***array,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscFunctionListDuplicate"
 /*@
     PetscFunctionListDuplicate - Creates a new list from a given object list.
 
