@@ -14,8 +14,6 @@ struct _EH {
 
 static EH eh = 0;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscEmacsClientErrorHandler"
 /*@C
    PetscEmacsClientErrorHandler - Error handler that uses the emacsclient program to
     load the file where the error occured. Then calls the "previous" error handler.
@@ -25,7 +23,6 @@ static EH eh = 0;
    Input Parameters:
 +  comm - communicator over which error occured
 .  line - the line number of the error (indicated by __LINE__)
-.  func - the function where error is detected (indicated by __FUNCT__)
 .  file - the file in which the error was detected (indicated by __FILE__)
 .  mess - an error text string, usually just printed to the screen
 .  n - the generic error number
@@ -62,14 +59,13 @@ PetscErrorCode  PetscEmacsClientErrorHandler(MPI_Comm comm,int line,const char *
   char           command[PETSC_MAX_PATH_LEN];
   const char     *pdir;
   FILE           *fp;
-  int            rval;
 
   PetscFunctionBegin;
   ierr = PetscGetPetscDir(&pdir);if (ierr) PetscFunctionReturn(ierr);
   sprintf(command,"cd %s; emacsclient --no-wait +%d %s\n",pdir,line,file);
 #if defined(PETSC_HAVE_POPEN)
   ierr = PetscPOpen(MPI_COMM_WORLD,(char*)ctx,command,"r",&fp);if (ierr) PetscFunctionReturn(ierr);
-  ierr = PetscPClose(MPI_COMM_WORLD,fp,&rval);if (ierr) PetscFunctionReturn(ierr);
+  ierr = PetscPClose(MPI_COMM_WORLD,fp);if (ierr) PetscFunctionReturn(ierr);
 #else
   SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP_SYS,"Cannot run external programs on this machine");
 #endif
@@ -82,8 +78,6 @@ PetscErrorCode  PetscEmacsClientErrorHandler(MPI_Comm comm,int line,const char *
   PetscFunctionReturn(ierr);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscPushErrorHandler"
 /*@C
    PetscPushErrorHandler - Sets a routine to be called on detection of errors.
 
@@ -99,7 +93,6 @@ $    int handler(MPI_Comm comm,int line,char *func,char *file,PetscErrorCode n,i
 
 +  comm - communicator over which error occured
 .  line - the line number of the error (indicated by __LINE__)
-.  func - the function where the error occured (indicated by __FUNCT__)
 .  file - the file in which the error was detected (indicated by __FILE__)
 .  n - the generic error number (see list defined in include/petscerror.h)
 .  p - PETSC_ERROR_INITIAL if error just detected, otherwise PETSC_ERROR_REPEAT
@@ -116,7 +109,8 @@ $    int handler(MPI_Comm comm,int line,char *func,char *file,PetscErrorCode n,i
    The currently available PETSc error handlers include PetscTraceBackErrorHandler(),
    PetscAttachDebuggerErrorHandler(), PetscAbortErrorHandler(), and PetscMPIAbortErrorHandler(), PetscReturnErrorHandler().
 
-   Fortran Notes: You can only push one error handler from Fortran before poping it.
+   Fortran Notes:
+    You can only push one error handler from Fortran before poping it.
 
 .seealso: PetscPopErrorHandler(), PetscAttachDebuggerErrorHandler(), PetscAbortErrorHandler(), PetscTraceBackErrorHandler(), PetscPushSignalHandler()
 
@@ -136,8 +130,6 @@ PetscErrorCode  PetscPushErrorHandler(PetscErrorCode (*handler)(MPI_Comm comm,in
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscPopErrorHandler"
 /*@
    PetscPopErrorHandler - Removes the latest error handler that was
    pushed with PetscPushErrorHandler().
@@ -163,8 +155,6 @@ PetscErrorCode  PetscPopErrorHandler(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscReturnErrorHandler"
 /*@C
   PetscReturnErrorHandler - Error handler that causes a return to the current
   level.
@@ -174,7 +164,6 @@ PetscErrorCode  PetscPopErrorHandler(void)
    Input Parameters:
 +  comm - communicator over which error occurred
 .  line - the line number of the error (indicated by __LINE__)
-.  func - the function where error is detected (indicated by __FUNCT__)
 .  file - the file in which the error was detected (indicated by __FILE__)
 .  mess - an error text string, usually just printed to the screen
 .  n - the generic error number
@@ -247,7 +236,7 @@ static const char *PetscErrorStrings[] = {
           "Overflow in integer operation: http://www.mcs.anl.gov/petsc/documentation/faq.html#64-bit-indices",
   /*85 */ "Null argument, when expecting valid pointer",
   /*86 */ "Unknown type. Check for miss-spelling or missing package: http://www.mcs.anl.gov/petsc/documentation/installation.html#external",
-  /*87 */ "Not used",
+  /*87 */ "MPI library at runtime is not compatible with MPI used at compile time",
   /*88 */ "Error in system call",
   /*89 */ "Object Type not set: http://www.mcs.anl.gov/petsc/documentation/faq.html#objecttypenotset"
   /*90 */ "  ",
@@ -259,8 +248,6 @@ static const char *PetscErrorStrings[] = {
   /*96 */ "  ",
 };
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscErrorMessage"
 /*@C
    PetscErrorMessage - returns the text string associated with a PETSc error code.
 
@@ -277,7 +264,7 @@ static const char *PetscErrorStrings[] = {
 
    Concepts: error handler^messages
 
-.seealso:  PetscPushErrorHandler(), PetscAttachDebuggerErrorHandler(),
+.seealso:  PetscPushErrorHandler(), PetscAttachDebuggerErrorHandler(), PetscError(), SETERRQ(), CHKERRQ() 
           PetscAbortErrorHandler(), PetscTraceBackErrorHandler()
  @*/
 PetscErrorCode  PetscErrorMessage(int errnum,const char *text[],char **specific)
@@ -317,8 +304,6 @@ static void PetscCxxErrorThrow() {
 }
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscError"
 /*@C
    PetscError - Routine that is called when an error has been detected,
    usually called through the macro SETERRQ(PETSC_COMM_SELF,).
@@ -328,9 +313,8 @@ static void PetscCxxErrorThrow() {
    Input Parameters:
 +  comm - communicator over which error occurred.  ALL ranks of this communicator MUST call this routine
 .  line - the line number of the error (indicated by __LINE__)
-.  func - the function where the error occured (indicated by __FUNCT__)
+.  func - the function name in which the error was detected
 .  file - the file in which the error was detected (indicated by __FILE__)
-.  mess - an error text string, usually just printed to the screen
 .  n - the generic error number
 .  p - PETSC_ERROR_INITIAL indicates the error was initially detected, PETSC_ERROR_REPEAT indicates this is a traceback from a previously detected error
 -  mess - formatted message string - aka printf
@@ -343,6 +327,10 @@ static void PetscCxxErrorThrow() {
    sequence
 $     SETERRQ(comm,n,mess)
 
+   Fortran Note:
+   This routine is used differently from Fortran
+$    PetscError(MPI_Comm comm,PetscErrorCode n,PetscErrorType p,char *message)
+
    Experienced users can set the error handler with PetscPushErrorHandler().
 
    Developer Note: Since this is called after an error condition it should not be calling any error handlers (currently it ignores any error codes)
@@ -351,14 +339,14 @@ $     SETERRQ(comm,n,mess)
 
    Concepts: error^setting condition
 
-.seealso: PetscTraceBackErrorHandler(), PetscPushErrorHandler(), SETERRQ(), CHKERRQ(), CHKMEMQ, SETERRQ1(), SETERRQ2()
+.seealso: PetscTraceBackErrorHandler(), PetscPushErrorHandler(), SETERRQ(), CHKERRQ(), CHKMEMQ, SETERRQ1(), SETERRQ2(), PetscErrorMessage()
 @*/
-PetscErrorCode  PetscError(MPI_Comm comm,int line,const char *func,const char *file,PetscErrorCode n,PetscErrorType p,const char *mess,...)
+PetscErrorCode PetscError(MPI_Comm comm,int line,const char *func,const char *file,PetscErrorCode n,PetscErrorType p,const char *mess,...)
 {
   va_list        Argp;
   size_t         fullLength;
   char           buf[2048],*lbuf = 0;
-  PetscBool      ismain,isunknown;
+  PetscBool      ismain;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -372,7 +360,7 @@ PetscErrorCode  PetscError(MPI_Comm comm,int line,const char *func,const char *f
     PetscVSNPrintf(buf,2048,mess,&fullLength,Argp);
     va_end(Argp);
     lbuf = buf;
-    if (p == 1) PetscStrncpy(PetscErrorBaseMessage,lbuf,1023);
+    if (p == PETSC_ERROR_INITIAL) PetscStrncpy(PetscErrorBaseMessage,lbuf,1023);
   }
 
   if (!eh) ierr = PetscTraceBackErrorHandler(comm,line,func,file,n,p,lbuf,0);
@@ -386,8 +374,7 @@ PetscErrorCode  PetscError(MPI_Comm comm,int line,const char *func,const char *f
     PetscStrncmp() does its own error checking which is problamatic
   */
   PetscStrncmp(func,"main",4,&ismain);
-  PetscStrncmp(func,"unknown",7,&isunknown);
-  if (ismain || isunknown) MPI_Abort(PETSC_COMM_WORLD,(int)ierr);
+  if (ismain) MPI_Abort(PETSC_COMM_WORLD,(int)ierr);
 
 #if defined(PETSC_CLANGUAGE_CXX)
   if (p == PETSC_ERROR_IN_CXX) {
@@ -399,8 +386,6 @@ PetscErrorCode  PetscError(MPI_Comm comm,int line,const char *func,const char *f
 
 /* -------------------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscIntView"
 /*@C
     PetscIntView - Prints an array of integers; useful for debugging.
 
@@ -413,7 +398,8 @@ PetscErrorCode  PetscError(MPI_Comm comm,int line,const char *func,const char *f
 
   Level: intermediate
 
-    Developer Notes: idx cannot be const because may be passed to binary viewer where byte swappping is done
+    Developer Notes:
+    idx cannot be const because may be passed to binary viewer where byte swapping is done
 
 .seealso: PetscRealView()
 @*/
@@ -488,8 +474,6 @@ PetscErrorCode  PetscIntView(PetscInt N,const PetscInt idx[],PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscRealView"
 /*@C
     PetscRealView - Prints an array of doubles; useful for debugging.
 
@@ -502,7 +486,8 @@ PetscErrorCode  PetscIntView(PetscInt N,const PetscInt idx[],PetscViewer viewer)
 
   Level: intermediate
 
-    Developer Notes: idx cannot be const because may be passed to binary viewer where byte swappping is done
+    Developer Notes:
+    idx cannot be const because may be passed to binary viewer where byte swapping is done
 
 .seealso: PetscIntView()
 @*/
@@ -577,8 +562,6 @@ PetscErrorCode  PetscRealView(PetscInt N,const PetscReal idx[],PetscViewer viewe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscScalarView"
 /*@C
     PetscScalarView - Prints an array of scalars; useful for debugging.
 
@@ -591,7 +574,8 @@ PetscErrorCode  PetscRealView(PetscInt N,const PetscReal idx[],PetscViewer viewe
 
   Level: intermediate
 
-    Developer Notes: idx cannot be const because may be passed to binary viewer where byte swappping is done
+    Developer Notes:
+    idx cannot be const because may be passed to binary viewer where byte swapping is done
 
 .seealso: PetscIntView(), PetscRealView()
 @*/
