@@ -3,8 +3,6 @@ static char help[] = "Demonstrates PETSc error handlers.\n";
 
 #include <petscsys.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "CreateError"
 int CreateError(int n)
 {
   PetscErrorCode ierr;
@@ -13,17 +11,25 @@ int CreateError(int n)
   return 0;
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   PetscErrorCode ierr;
-  PetscInitialize(&argc,&argv,(char*)0,help);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   ierr = PetscFPrintf(PETSC_COMM_WORLD,stdout,"Demonstrates PETSc Error Handlers\n");CHKERRQ(ierr);
   ierr = PetscFPrintf(PETSC_COMM_WORLD,stdout,"The error is a contrived error to test error handling\n");CHKERRQ(ierr);
   ierr = PetscSynchronizedFlush(PETSC_COMM_WORLD,PETSC_STDOUT);CHKERRQ(ierr);
   ierr = CreateError(5);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
+
+
+/*TEST
+
+ # Testing errors so only look for errors
+   test:
+     filter: Error: egrep "(PETSC ERROR)" | egrep "(Error Created|CreateError\(\)|main\(\))" | cut -f1,2,3,4,5,6 -d" "
+
+
+TEST*/
