@@ -5,6 +5,8 @@ static char help[] = "Demonstrates using the PetscBag Object\n\n";
    Concepts: bags;
    Processors: n
 T*/
+
+
 #include <petscsys.h>
 #include <petscbag.h>
 #include <petscviewer.h>
@@ -46,8 +48,6 @@ typedef struct {
 } Parameter;
 
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   PetscErrorCode ierr;
@@ -66,7 +66,7 @@ int main(int argc,char **argv)
                  runtime.  The user can use the "help" variable place
                  additional help messages in this printout.
   */
-  ierr = PetscInitialize(&argc,&argv,(char*)0,help);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
 
   /* Create an empty bag */
   ierr = PetscBagCreate(PETSC_COMM_WORLD,sizeof(Parameter),&bag);CHKERRQ(ierr);
@@ -99,7 +99,7 @@ int main(int argc,char **argv)
 
 
   /* This option allows loading user-provided PetscBag */
-  ierr = PetscOptionsGetString(NULL,"-f",filename,sizeof(filename),&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-f",filename,sizeof(filename),&flg);CHKERRQ(ierr);
   if (!flg) {
 
     /* write bag to stdio & binary file */
@@ -125,5 +125,21 @@ int main(int argc,char **argv)
   /* clean up and exit */
   ierr = PetscBagDestroy(&bag);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
+
+
+/*TEST
+
+   test:
+      args: -pbag_rho 44 -pbag_do_output true
+      requires: yaml !complex
+
+   test:
+      suffix: yaml
+      requires: yaml !complex
+      args: -options_file_yaml bag.yml -options_view
+      filter:  grep -v saws_port_auto_select |grep -v malloc_dump | grep -v display
+      localrunfiles: bag.yml
+
+TEST*/
