@@ -1,4 +1,41 @@
-#include <../src/sys/f90-src/f90impl.h>
+#include <petsc/private/f90impl.h>
+
+/*@C
+
+   Converts a MPI_Fint that contains a Fortran MPI_Datatype to its C MPI_Datatype equivalent
+
+   Not Collective
+
+   Input Parameter:
+.  unit - The Fortran MPI_Datatype
+
+   Output Parameter:
+.  dtype - the corresponding C MPI_Datatype
+
+   Level: developer
+
+   Developer Notes:
+    The MPI documentation in multiple places says that one can never us
+   Fortran MPI_Datatypes in C (or vis-versa) but this is problematic since users could never
+   call C routines from Fortran that have MPI_Datatype arguments. Jed states that the Fortran
+   MPI_Datatypes will always be available in C if the MPI was built to support Fortran. This function
+   relys on this.
+@*/
+PetscErrorCode PetscMPIFortranDatatypeToC(MPI_Fint unit,MPI_Datatype *dtype)
+{
+  MPI_Datatype ftype;
+
+  PetscFunctionBegin;
+  ftype = MPI_Type_f2c(unit);
+  if (ftype == MPI_INTEGER) *dtype = MPI_INT;
+  else if (ftype == MPI_INTEGER8) *dtype = MPIU_INT64;
+  else if (ftype == MPI_DOUBLE_PRECISION) *dtype = MPI_DOUBLE;
+#if defined(PETSC_HAVE_COMPLEX)
+  else if (ftype == MPI_COMPLEX16) *dtype = MPIU_C_DOUBLE_COMPLEX;
+#endif
+  else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Unknown Fortran MPI_Datatype");
+  PetscFunctionReturn(0);
+}
 
 /*************************************************************************/
 
@@ -43,60 +80,54 @@ PETSC_EXTERN void PETSC_STDCALL f90array1dcreatefortranaddr_(void *,PetscInt *,P
 PETSC_EXTERN void PETSC_STDCALL f90array1daccessfortranaddr_(F90Array1d*,void** PETSC_F90_2PTR_PROTO_NOVAR);
 PETSC_EXTERN void PETSC_STDCALL f90array1ddestroyfortranaddr_(F90Array1d *ptr PETSC_F90_2PTR_PROTO_NOVAR);
 
-#undef __FUNCT__
-#define __FUNCT__ "F90Array1dCreate"
-PetscErrorCode F90Array1dCreate(void *array,PetscDataType type,PetscInt start,PetscInt len,F90Array1d *ptr PETSC_F90_2PTR_PROTO(ptrd))
+PetscErrorCode F90Array1dCreate(void *array,MPI_Datatype type,PetscInt start,PetscInt len,F90Array1d *ptr PETSC_F90_2PTR_PROTO(ptrd))
 {
   PetscFunctionBegin;
-  if (type == PETSC_SCALAR) {
+  if (type == MPIU_SCALAR) {
     if (!len) array = PETSC_NULL_SCALAR_Fortran;
     f90array1dcreatescalar_(array,&start,&len,ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_REAL) {
+  } else if (type == MPIU_REAL) {
     if (!len) array = PETSC_NULL_REAL_Fortran;
     f90array1dcreatereal_(array,&start,&len,ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_INT) {
+  } else if (type == MPIU_INT) {
     if (!len) array = PETSC_NULL_INTEGER_Fortran;
     f90array1dcreateint_(array,&start,&len,ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_FORTRANADDR) {
+  } else if (type == MPIU_FORTRANADDR) {
     f90array1dcreatefortranaddr_(array,&start,&len,ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"unsupported PetscDataType: %d",(PetscInt)type);
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Unsupported MPI_Datatype");
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "F90Array1dAccess"
-PetscErrorCode  F90Array1dAccess(F90Array1d *ptr,PetscDataType type,void **array PETSC_F90_2PTR_PROTO(ptrd))
+PetscErrorCode  F90Array1dAccess(F90Array1d *ptr,MPI_Datatype type,void **array PETSC_F90_2PTR_PROTO(ptrd))
 {
   PetscFunctionBegin;
-  if (type == PETSC_SCALAR) {
+  if (type == MPIU_SCALAR) {
     f90array1daccessscalar_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
     if (*array == PETSC_NULL_SCALAR_Fortran) *array = 0;
-  } else if (type == PETSC_REAL) {
+  } else if (type == MPIU_REAL) {
     f90array1daccessreal_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
     if (*array == PETSC_NULL_REAL_Fortran) *array = 0;
-  } else if (type == PETSC_INT) {
+  } else if (type == MPIU_INT) {
     f90array1daccessint_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
     if (*array == PETSC_NULL_INTEGER_Fortran) *array = 0;
-  } else if (type == PETSC_FORTRANADDR) {
+  } else if (type == MPIU_FORTRANADDR) {
     f90array1daccessfortranaddr_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
-  } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"unsupported PetscDataType: %d",(PetscInt)type);
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Unsupported MPI_Datatype");
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "F90Array1dDestroy"
-PetscErrorCode  F90Array1dDestroy(F90Array1d *ptr,PetscDataType type PETSC_F90_2PTR_PROTO(ptrd))
+PetscErrorCode  F90Array1dDestroy(F90Array1d *ptr,MPI_Datatype type PETSC_F90_2PTR_PROTO(ptrd))
 {
   PetscFunctionBegin;
-  if (type == PETSC_SCALAR) {
+  if (type == MPIU_SCALAR) {
     f90array1ddestroyscalar_(ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_REAL) {
+  } else if (type == MPIU_REAL) {
     f90array1ddestroyreal_(ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_INT) {
+  } else if (type == MPIU_INT) {
     f90array1ddestroyint_(ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_FORTRANADDR) {
+  } else if (type == MPIU_FORTRANADDR) {
     f90array1ddestroyfortranaddr_(ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"unsupported PetscDataType: %d",(PetscInt)type);
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Unsupported MPI_Datatype");
   PetscFunctionReturn(0);
 }
 
@@ -143,54 +174,48 @@ PETSC_EXTERN void PETSC_STDCALL f90array2dcreatefortranaddr_(void *,PetscInt *,P
 PETSC_EXTERN void PETSC_STDCALL f90array2daccessfortranaddr_(F90Array2d*,void** PETSC_F90_2PTR_PROTO_NOVAR);
 PETSC_EXTERN void PETSC_STDCALL f90array2ddestroyfortranaddr_(F90Array2d *ptr PETSC_F90_2PTR_PROTO_NOVAR);
 
-#undef __FUNCT__
-#define __FUNCT__ "F90Array2dCreate"
-PetscErrorCode F90Array2dCreate(void *array,PetscDataType type,PetscInt start1,PetscInt len1,PetscInt start2,PetscInt len2,F90Array2d *ptr PETSC_F90_2PTR_PROTO(ptrd))
+PetscErrorCode F90Array2dCreate(void *array,MPI_Datatype type,PetscInt start1,PetscInt len1,PetscInt start2,PetscInt len2,F90Array2d *ptr PETSC_F90_2PTR_PROTO(ptrd))
 {
   PetscFunctionBegin;
-  if (type == PETSC_SCALAR) {
+  if (type == MPIU_SCALAR) {
     f90array2dcreatescalar_(array,&start1,&len1,&start2,&len2,ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_REAL) {
+  } else if (type == MPIU_REAL) {
     f90array2dcreatereal_(array,&start1,&len1,&start2,&len2,ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_INT) {
+  } else if (type == MPIU_INT) {
     f90array2dcreateint_(array,&start1,&len1,&start2,&len2,ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_FORTRANADDR) {
+  } else if (type == MPIU_FORTRANADDR) {
     f90array2dcreatefortranaddr_(array,&start1,&len1,&start2,&len2,ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"unsupported PetscDataType: %d",(PetscInt)type);
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Unsupported MPI_Datatype");
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "F90Array2dAccess"
-PetscErrorCode  F90Array2dAccess(F90Array2d *ptr,PetscDataType type,void **array PETSC_F90_2PTR_PROTO(ptrd))
+PetscErrorCode  F90Array2dAccess(F90Array2d *ptr,MPI_Datatype type,void **array PETSC_F90_2PTR_PROTO(ptrd))
 {
   PetscFunctionBegin;
-  if (type == PETSC_SCALAR) {
+  if (type == MPIU_SCALAR) {
     f90array2daccessscalar_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_REAL) {
+  } else if (type == MPIU_REAL) {
     f90array2daccessreal_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_INT) {
+  } else if (type == MPIU_INT) {
     f90array2daccessint_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_FORTRANADDR) {
+  } else if (type == MPIU_FORTRANADDR) {
     f90array2daccessfortranaddr_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
-  } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"unsupported PetscDataType: %d",(PetscInt)type);
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Unsupported MPI_Datatype");
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "F90Array2dDestroy"
-PetscErrorCode  F90Array2dDestroy(F90Array2d *ptr,PetscDataType type PETSC_F90_2PTR_PROTO(ptrd))
+PetscErrorCode  F90Array2dDestroy(F90Array2d *ptr,MPI_Datatype type PETSC_F90_2PTR_PROTO(ptrd))
 {
   PetscFunctionBegin;
-  if (type == PETSC_SCALAR) {
+  if (type == MPIU_SCALAR) {
     f90array2ddestroyscalar_(ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_REAL) {
+  } else if (type == MPIU_REAL) {
     f90array2ddestroyreal_(ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_INT) {
+  } else if (type == MPIU_INT) {
     f90array2ddestroyint_(ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_FORTRANADDR) {
+  } else if (type == MPIU_FORTRANADDR) {
     f90array2ddestroyfortranaddr_(ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"unsupported PetscDataType: %d",(PetscInt)type);
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Unsupported MPI_Datatype");
   PetscFunctionReturn(0);
 }
 
@@ -237,54 +262,48 @@ PETSC_EXTERN void PETSC_STDCALL f90array3dcreatefortranaddr_(void *,PetscInt *,P
 PETSC_EXTERN void PETSC_STDCALL f90array3daccessfortranaddr_(F90Array3d*,void** PETSC_F90_2PTR_PROTO_NOVAR);
 PETSC_EXTERN void PETSC_STDCALL f90array3ddestroyfortranaddr_(F90Array3d *ptr PETSC_F90_2PTR_PROTO_NOVAR);
 
-#undef __FUNCT__
-#define __FUNCT__ "F90Array3dCreate"
-PetscErrorCode F90Array3dCreate(void *array,PetscDataType type,PetscInt start1,PetscInt len1,PetscInt start2,PetscInt len2,PetscInt start3,PetscInt len3,F90Array3d *ptr PETSC_F90_2PTR_PROTO(ptrd))
+PetscErrorCode F90Array3dCreate(void *array,MPI_Datatype type,PetscInt start1,PetscInt len1,PetscInt start2,PetscInt len2,PetscInt start3,PetscInt len3,F90Array3d *ptr PETSC_F90_2PTR_PROTO(ptrd))
 {
   PetscFunctionBegin;
-  if (type == PETSC_SCALAR) {
+  if (type == MPIU_SCALAR) {
     f90array3dcreatescalar_(array,&start1,&len1,&start2,&len2,&start3,&len3,ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_REAL) {
+  } else if (type == MPIU_REAL) {
     f90array3dcreatereal_(array,&start1,&len1,&start2,&len2,&start3,&len3,ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_INT) {
+  } else if (type == MPIU_INT) {
     f90array3dcreateint_(array,&start1,&len1,&start2,&len2,&start3,&len3,ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_FORTRANADDR) {
+  } else if (type == MPIU_FORTRANADDR) {
     f90array3dcreatefortranaddr_(array,&start1,&len1,&start2,&len2,&start3,&len3,ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"unsupported PetscDataType: %d",(PetscInt)type);
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Unsupported MPI_Datatype");
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "F90Array3dAccess"
-PetscErrorCode  F90Array3dAccess(F90Array3d *ptr,PetscDataType type,void **array PETSC_F90_2PTR_PROTO(ptrd))
+PetscErrorCode  F90Array3dAccess(F90Array3d *ptr,MPI_Datatype type,void **array PETSC_F90_2PTR_PROTO(ptrd))
 {
   PetscFunctionBegin;
-  if (type == PETSC_SCALAR) {
+  if (type == MPIU_SCALAR) {
     f90array3daccessscalar_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_REAL) {
+  } else if (type == MPIU_REAL) {
     f90array3daccessreal_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_INT) {
+  } else if (type == MPIU_INT) {
     f90array3daccessint_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_FORTRANADDR) {
+  } else if (type == MPIU_FORTRANADDR) {
     f90array3daccessfortranaddr_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
-  } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"unsupported PetscDataType: %d",(PetscInt)type);
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Unsupported MPI_Datatype");
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "F90Array3dDestroy"
-PetscErrorCode  F90Array3dDestroy(F90Array3d *ptr,PetscDataType type PETSC_F90_2PTR_PROTO(ptrd))
+PetscErrorCode  F90Array3dDestroy(F90Array3d *ptr,MPI_Datatype type PETSC_F90_2PTR_PROTO(ptrd))
 {
   PetscFunctionBegin;
-  if (type == PETSC_SCALAR) {
+  if (type == MPIU_SCALAR) {
     f90array3ddestroyscalar_(ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_REAL) {
+  } else if (type == MPIU_REAL) {
     f90array3ddestroyreal_(ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_INT) {
+  } else if (type == MPIU_INT) {
     f90array3ddestroyint_(ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_FORTRANADDR) {
+  } else if (type == MPIU_FORTRANADDR) {
     f90array3ddestroyfortranaddr_(ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"unsupported PetscDataType: %d",(PetscInt)type);
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Unsupported MPI_Datatype");
   PetscFunctionReturn(0);
 }
 
@@ -330,42 +349,36 @@ PETSC_EXTERN void PETSC_STDCALL f90array4dcreatefortranaddr_(void *,PetscInt *,P
 PETSC_EXTERN void PETSC_STDCALL f90array4daccessfortranaddr_(F90Array4d*,void** PETSC_F90_2PTR_PROTO_NOVAR);
 PETSC_EXTERN void PETSC_STDCALL f90array4ddestroyfortranaddr_(F90Array4d *ptr PETSC_F90_2PTR_PROTO_NOVAR);
 
-#undef __FUNCT__
-#define __FUNCT__ "F90Array4dCreate"
-PetscErrorCode F90Array4dCreate(void *array,PetscDataType type,PetscInt start1,PetscInt len1,PetscInt start2,PetscInt len2,PetscInt start3,PetscInt len3,PetscInt start4,PetscInt len4,F90Array4d *ptr PETSC_F90_2PTR_PROTO(ptrd))
+PetscErrorCode F90Array4dCreate(void *array,MPI_Datatype type,PetscInt start1,PetscInt len1,PetscInt start2,PetscInt len2,PetscInt start3,PetscInt len3,PetscInt start4,PetscInt len4,F90Array4d *ptr PETSC_F90_2PTR_PROTO(ptrd))
 {
   PetscFunctionBegin;
-  if (type == PETSC_SCALAR) {
+  if (type == MPIU_SCALAR) {
     f90array4dcreatescalar_(array,&start1,&len1,&start2,&len2,&start3,&len3,&start4,&len4,ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"unsupported PetscDataType: %d",(PetscInt)type);
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Unsupported MPI_Datatype");
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "F90Array4dAccess"
-PetscErrorCode  F90Array4dAccess(F90Array4d *ptr,PetscDataType type,void **array PETSC_F90_2PTR_PROTO(ptrd))
+PetscErrorCode  F90Array4dAccess(F90Array4d *ptr,MPI_Datatype type,void **array PETSC_F90_2PTR_PROTO(ptrd))
 {
   PetscFunctionBegin;
-  if (type == PETSC_SCALAR) {
+  if (type == MPIU_SCALAR) {
     f90array4daccessscalar_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_REAL) {
+  } else if (type == MPIU_REAL) {
     f90array4daccessreal_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_INT) {
+  } else if (type == MPIU_INT) {
     f90array4daccessint_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
-  } else if (type == PETSC_FORTRANADDR) {
+  } else if (type == MPIU_FORTRANADDR) {
     f90array4daccessfortranaddr_(ptr,array PETSC_F90_2PTR_PARAM(ptrd));
-  } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"unsupported PetscDataType: %d",(PetscInt)type);
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Unsupported MPI_Datatype");
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "F90Array4dDestroy"
-PetscErrorCode  F90Array4dDestroy(F90Array4d *ptr,PetscDataType type PETSC_F90_2PTR_PROTO(ptrd))
+PetscErrorCode  F90Array4dDestroy(F90Array4d *ptr,MPI_Datatype type PETSC_F90_2PTR_PROTO(ptrd))
 {
   PetscFunctionBegin;
-  if (type == PETSC_SCALAR) {
+  if (type == MPIU_SCALAR) {
     f90array4ddestroyscalar_(ptr PETSC_F90_2PTR_PARAM(ptrd));
-  } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"unsupported PetscDataType: %d",(PetscInt)type);
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Unsupported MPI_Datatype");
   PetscFunctionReturn(0);
 }
 
