@@ -8,22 +8,22 @@
 #define petsctestfile_             petsctestfile
 #endif
 
-PETSC_EXTERN void PETSC_STDCALL petscbinaryopen_(CHAR name PETSC_MIXED_LEN(len),PetscFileMode *type,int *fd,
+PETSC_EXTERN void PETSC_STDCALL petscbinaryopen_(char* name PETSC_MIXED_LEN(len),PetscFileMode *type,int *fd,
                                     PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   char *c1;
 
   FIXCHAR(name,len,c1);
-  *ierr = PetscBinaryOpen(c1,*type,fd);
+  *ierr = PetscBinaryOpen(c1,*type,fd);if (*ierr) return;
   FREECHAR(name,c1);
 }
 
-PETSC_EXTERN void PETSC_STDCALL petsctestfile_(CHAR name PETSC_MIXED_LEN(len),CHAR mode PETSC_MIXED_LEN(len1),PetscBool *flg,PetscErrorCode *ierr PETSC_END_LEN(len) PETSC_END_LEN(len1))
+PETSC_EXTERN void PETSC_STDCALL petsctestfile_(char* name PETSC_MIXED_LEN(len),char* mode PETSC_MIXED_LEN(len1),PetscBool *flg,PetscErrorCode *ierr PETSC_END_LEN(len) PETSC_END_LEN(len1))
 {
   char *c1;
 
   FIXCHAR(name,len,c1);
-  *ierr = PetscTestFile(c1,*mode,flg);
+  *ierr = PetscTestFile(c1,*mode,flg);if (*ierr) return;
   FREECHAR(name,c1);
 }
 
