@@ -1,12 +1,11 @@
 #include <petsc/private/fortranimpl.h>
 
-void *PETSCNULLPOINTERADDRESS = NULL;
-
 /*MC
    PetscFortranAddr - a variable type in Fortran that can hold a
      regular C pointer.
 
-   Notes: Used, for example, as the file argument in PetscFOpen()
+   Notes:
+    Used, for example, as the file argument in PetscFOpen()
 
    Level: beginner
 
@@ -27,7 +26,6 @@ M*/
 */
 char *PETSC_NULL_CHARACTER_Fortran = 0;
 void *PETSC_NULL_INTEGER_Fortran   = 0;
-void *PETSC_NULL_OBJECT_Fortran    = 0;
 void *PETSC_NULL_SCALAR_Fortran    = 0;
 void *PETSC_NULL_DOUBLE_Fortran    = 0;
 void *PETSC_NULL_REAL_Fortran      = 0;
