@@ -9,8 +9,6 @@
 
 PetscStageLog petsc_stageLog = 0;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscLogGetStageLog"
 /*@C
   PetscLogGetStageLog - This function returns the default stage logging object.
 
@@ -21,7 +19,8 @@ PetscStageLog petsc_stageLog = 0;
 
   Level: developer
 
-  Developer Notes: Inline since called for EACH PetscEventLogBeginDefault() and PetscEventLogEndDefault()
+  Developer Notes:
+    Inline since called for EACH PetscEventLogBeginDefault() and PetscEventLogEndDefault()
 
 .keywords: log, stage
 .seealso: PetscStageLogCreate()
@@ -38,8 +37,6 @@ PetscErrorCode PetscLogGetStageLog(PetscStageLog *stageLog)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogGetCurrent"
 /*@C
   PetscStageLogGetCurrent - This function returns the stage from the top of the stack.
 
@@ -56,7 +53,8 @@ PetscErrorCode PetscLogGetStageLog(PetscStageLog *stageLog)
 
   Level: developer
 
-  Developer Notes: Inline since called for EACH PetscEventLogBeginDefault() and PetscEventLogEndDefault()
+  Developer Notes:
+    Inline since called for EACH PetscEventLogBeginDefault() and PetscEventLogEndDefault()
 
 .keywords: log, stage
 .seealso: PetscStageLogPush(), PetscStageLogPop(), PetscLogGetStageLog()
@@ -79,8 +77,6 @@ PetscErrorCode  PetscStageLogGetCurrent(PetscStageLog stageLog, int *stage)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogGetEventPerfLog"
 /*@C
   PetscStageLogGetEventPerfLog - This function returns the PetscEventPerfLog for the given stage.
 
@@ -95,7 +91,8 @@ PetscErrorCode  PetscStageLogGetCurrent(PetscStageLog stageLog, int *stage)
 
   Level: developer
 
-  Developer Notes: Inline since called for EACH PetscEventLogBeginDefault() and PetscEventLogEndDefault()
+  Developer Notes:
+    Inline since called for EACH PetscEventLogBeginDefault() and PetscEventLogEndDefault()
 
 .keywords: log, stage
 .seealso: PetscStageLogPush(), PetscStageLogPop(), PetscLogGetStageLog()
@@ -109,8 +106,6 @@ PetscErrorCode  PetscStageLogGetEventPerfLog(PetscStageLog stageLog, int stage,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageInfoDestroy"
 /*@C
   PetscStageInfoDestroy - This destroys a PetscStageInfo object.
 
@@ -130,13 +125,11 @@ PetscErrorCode  PetscStageInfoDestroy(PetscStageInfo *stageInfo)
 
   PetscFunctionBegin;
   ierr = PetscFree(stageInfo->name);CHKERRQ(ierr);
-  ierr = EventPerfLogDestroy(stageInfo->eventLog);CHKERRQ(ierr);
-  ierr = ClassPerfLogDestroy(stageInfo->classLog);CHKERRQ(ierr);
+  ierr = PetscEventPerfLogDestroy(stageInfo->eventLog);CHKERRQ(ierr);
+  ierr = PetscClassPerfLogDestroy(stageInfo->classLog);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogDestroy"
 /*@C
   PetscStageLogDestroy - This destroys a PetscStageLog object.
 
@@ -158,7 +151,7 @@ PetscErrorCode  PetscStageLogDestroy(PetscStageLog stageLog)
   PetscFunctionBegin;
   if (!stageLog) PetscFunctionReturn(0);
   ierr = PetscIntStackDestroy(stageLog->stack);CHKERRQ(ierr);
-  ierr = EventRegLogDestroy(stageLog->eventLog);CHKERRQ(ierr);
+  ierr = PetscEventRegLogDestroy(stageLog->eventLog);CHKERRQ(ierr);
   ierr = PetscClassRegLogDestroy(stageLog->classLog);CHKERRQ(ierr);
   for (stage = 0; stage < stageLog->numStages; stage++) {
     ierr = PetscStageInfoDestroy(&stageLog->stageInfo[stage]);CHKERRQ(ierr);
@@ -168,8 +161,6 @@ PetscErrorCode  PetscStageLogDestroy(PetscStageLog stageLog)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogRegister"
 /*@C
   PetscStageLogRegister - Registers a stage name for logging operations in an application code.
 
@@ -190,50 +181,41 @@ PetscErrorCode  PetscStageLogDestroy(PetscStageLog stageLog)
 PetscErrorCode  PetscStageLogRegister(PetscStageLog stageLog, const char sname[], int *stage)
 {
   PetscStageInfo *stageInfo;
-  char           *str;
-  int            s, st;
+  int            s;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidCharPointer(sname,2);
   PetscValidIntPointer(stage,3);
-  for (st = 0; st < stageLog->numStages; ++st) {
+  /* Check stage already registered */
+  for (s = 0; s < stageLog->numStages; ++s) {
     PetscBool same;
 
-    ierr = PetscStrcmp(stageLog->stageInfo[st].name, sname, &same);CHKERRQ(ierr);
+    ierr = PetscStrcmp(stageLog->stageInfo[s].name, sname, &same);CHKERRQ(ierr);
     if (same) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG, "Duplicate stage name given: %s", sname);
   }
+  /* Create new stage */
   s = stageLog->numStages++;
   if (stageLog->numStages > stageLog->maxStages) {
     ierr = PetscMalloc1(stageLog->maxStages*2, &stageInfo);CHKERRQ(ierr);
     ierr = PetscMemcpy(stageInfo, stageLog->stageInfo, stageLog->maxStages * sizeof(PetscStageInfo));CHKERRQ(ierr);
     ierr = PetscFree(stageLog->stageInfo);CHKERRQ(ierr);
-
     stageLog->stageInfo  = stageInfo;
     stageLog->maxStages *= 2;
   }
-  /* Setup stage */
-  ierr = PetscStrallocpy(sname, &str);CHKERRQ(ierr);
-
-  stageLog->stageInfo[s].name                   = str;
-  stageLog->stageInfo[s].used                   = PETSC_FALSE;
-  stageLog->stageInfo[s].perfInfo.active        = PETSC_TRUE;
-  stageLog->stageInfo[s].perfInfo.visible       = PETSC_TRUE;
-  stageLog->stageInfo[s].perfInfo.count         = 0;
-  stageLog->stageInfo[s].perfInfo.flops         = 0.0;
-  stageLog->stageInfo[s].perfInfo.time          = 0.0;
-  stageLog->stageInfo[s].perfInfo.numMessages   = 0.0;
-  stageLog->stageInfo[s].perfInfo.messageLength = 0.0;
-  stageLog->stageInfo[s].perfInfo.numReductions = 0.0;
-
-  ierr = EventPerfLogCreate(&stageLog->stageInfo[s].eventLog);CHKERRQ(ierr);
-  ierr = ClassPerfLogCreate(&stageLog->stageInfo[s].classLog);CHKERRQ(ierr);
+  /* Setup new stage info */
+  stageInfo = &stageLog->stageInfo[s];
+  ierr = PetscMemzero(stageInfo,sizeof(PetscStageInfo));CHKERRQ(ierr);
+  ierr = PetscStrallocpy(sname,&stageInfo->name);CHKERRQ(ierr);
+  stageInfo->used             = PETSC_FALSE;
+  stageInfo->perfInfo.active  = PETSC_TRUE;
+  stageInfo->perfInfo.visible = PETSC_TRUE;
+  ierr = PetscEventPerfLogCreate(&stageInfo->eventLog);CHKERRQ(ierr);
+  ierr = PetscClassPerfLogCreate(&stageInfo->classLog);CHKERRQ(ierr);
   *stage = s;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogPush"
 /*@C
   PetscStageLogPush - This function pushes a stage on the stack.
 
@@ -244,7 +226,7 @@ PetscErrorCode  PetscStageLogRegister(PetscStageLog stageLog, const char sname[]
 - stage - The stage to log
 
   Database Options:
-. -log_summary - Activates logging
+. -log_view - Activates logging
 
   Usage:
   If the option -log_sumary is used to run the program containing the
@@ -308,8 +290,6 @@ PetscErrorCode  PetscStageLogPush(PetscStageLog stageLog, int stage)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogPop"
 /*@C
   PetscStageLogPop - This function pops a stage from the stack.
 
@@ -374,8 +354,6 @@ PetscErrorCode  PetscStageLogPop(PetscStageLog stageLog)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogGetClassRegLog"
 /*@C
   PetscStageLogGetClassRegLog - This function returns the PetscClassRegLog for the given stage.
 
@@ -400,8 +378,6 @@ PetscErrorCode  PetscStageLogGetClassRegLog(PetscStageLog stageLog, PetscClassRe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogGetEventRegLog"
 /*@C
   PetscStageLogGetEventRegLog - This function returns the PetscEventRegLog.
 
@@ -426,10 +402,8 @@ PetscErrorCode  PetscStageLogGetEventRegLog(PetscStageLog stageLog, PetscEventRe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogGetClassPerfLog"
 /*@C
-  PetscStageLogGetClassPerfLog - This function returns the ClassPerfLog for the given stage.
+  PetscStageLogGetClassPerfLog - This function returns the PetscClassPerfLog for the given stage.
 
   Not Collective
 
@@ -438,7 +412,7 @@ PetscErrorCode  PetscStageLogGetEventRegLog(PetscStageLog stageLog, PetscEventRe
 - stage    - The stage
 
   Output Parameter:
-. classLog - The ClassPerfLog
+. classLog - The PetscClassPerfLog
 
   Level: developer
 
@@ -455,8 +429,6 @@ PetscErrorCode  PetscStageLogGetClassPerfLog(PetscStageLog stageLog, int stage,
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogSetActive"
 /*@C
   PetscStageLogSetActive - This function determines whether events will be logged during this state.
 
@@ -480,8 +452,6 @@ PetscErrorCode  PetscStageLogSetActive(PetscStageLog stageLog, int stage, PetscB
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogGetActive"
 /*@C
   PetscStageLogGetActive - This function returns whether events will be logged suring this stage.
 
@@ -508,8 +478,6 @@ PetscErrorCode  PetscStageLogGetActive(PetscStageLog stageLog, int stage, PetscB
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogSetVisible"
 /*@C
   PetscStageLogSetVisible - This function determines whether a stage is printed during PetscLogView()
 
@@ -521,7 +489,7 @@ PetscErrorCode  PetscStageLogGetActive(PetscStageLog stageLog, int stage, PetscB
 - isVisible - The visibility flag, PETSC_TRUE for printing, otherwise PETSC_FALSE (default is PETSC_TRUE)
 
   Database Options:
-. -log_summary - Activates log summary
+. -log_view - Activates log summary
 
   Level: developer
 
@@ -536,8 +504,6 @@ PetscErrorCode  PetscStageLogSetVisible(PetscStageLog stageLog, int stage, Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogGetVisible"
 /*@C
   PetscStageLogGetVisible - This function returns whether a stage is printed during PetscLogView()
 
@@ -551,7 +517,7 @@ PetscErrorCode  PetscStageLogSetVisible(PetscStageLog stageLog, int stage, Petsc
 . isVisible - The visibility flag, PETSC_TRUE for printing, otherwise PETSC_FALSE (default is PETSC_TRUE)
 
   Database Options:
-. -log_summary - Activates log summary
+. -log_view - Activates log summary
 
   Level: developer
 
@@ -567,8 +533,6 @@ PetscErrorCode  PetscStageLogGetVisible(PetscStageLog stageLog, int stage, Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogGetStage"
 /*@C
   PetscStageLogGetStage - This function returns the stage id given the stage name.
 
@@ -598,14 +562,14 @@ PetscErrorCode  PetscStageLogGetStage(PetscStageLog stageLog, const char name[],
   *stage = -1;
   for (s = 0; s < stageLog->numStages; s++) {
     ierr = PetscStrcasecmp(stageLog->stageInfo[s].name, name, &match);CHKERRQ(ierr);
-    *stage = s;
-    if (match) break;
+    if (match) {
+      *stage = s;
+      break;
+    }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStageLogCreate"
 /*@C
   PetscStageLogCreate - This creates a PetscStageLog object.
 
@@ -633,7 +597,7 @@ PetscErrorCode  PetscStageLogCreate(PetscStageLog *stageLog)
 
   ierr = PetscIntStackCreate(&l->stack);CHKERRQ(ierr);
   ierr = PetscMalloc1(l->maxStages, &l->stageInfo);CHKERRQ(ierr);
-  ierr = EventRegLogCreate(&l->eventLog);CHKERRQ(ierr);
+  ierr = PetscEventRegLogCreate(&l->eventLog);CHKERRQ(ierr);
   ierr = PetscClassRegLogCreate(&l->classLog);CHKERRQ(ierr);
 
   *stageLog = l;
