@@ -3,21 +3,17 @@
 #include <petscviewer.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
-#define petscmallocdump_           PETSCMALLOCDUMP
-#define petscmallocdumplog_        PETSCMALLOCDUMPLOG
-#define petscmallocvalidate_       PETSCMALLOCVALIDATE
-#define petscmemoryshowusage_      PETSCMEMORYSHOWUSAGE
-#define petscmemorysetgetmaximumusage_ PETSCMEMORYSETGETMAXIMUMUSAGE
+#define petscmallocdump_               PETSCMALLOCDUMP
+#define petscmallocdumplog_            PETSCMALLOCDUMPLOG
+#define petscmallocvalidate_           PETSCMALLOCVALIDATE
+#define petscmemoryview_               PETSCMEMORYVIEW
 #elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
-#define petscmallocdump_           petscmallocdump
-#define petscmallocdumplog_        petscmallocdumplog
-#define petscmallocvalidate_       petscmallocvalidate
-#define petscmemoryshowusage_      petscmemoryshowusage
-#define petscmemorysetgetmaximumusage_ petscmemorysetgetmaximumusage
+#define petscmallocdump_               petscmallocdump
+#define petscmallocdumplog_            petscmallocdumplog
+#define petscmallocvalidate_           petscmallocvalidate
+#define petscmemoryview_               petscmemoryview
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscFixSlashN"
 static PetscErrorCode PetscFixSlashN(const char *in, char **out)
 {
   PetscErrorCode ierr;
@@ -47,12 +43,7 @@ PETSC_EXTERN void PETSC_STDCALL petscmallocvalidate_(PetscErrorCode *ierr)
   *ierr = PetscMallocValidate(0,"Unknown Fortran",0);
 }
 
-PETSC_EXTERN void PETSC_STDCALL petscmemorysetgetmaximumusage_(PetscErrorCode *ierr)
-{
-  *ierr = PetscMemorySetGetMaximumUsage();
-}
-
-PETSC_EXTERN void PETSC_STDCALL petscmemoryshowusage_(PetscViewer *vin, CHAR message PETSC_MIXED_LEN(len), PetscErrorCode *ierr PETSC_END_LEN(len))
+PETSC_EXTERN void PETSC_STDCALL petscmemoryview_(PetscViewer *vin, char* message PETSC_MIXED_LEN(len), PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   PetscViewer v;
   char        *msg, *tmp;
@@ -61,7 +52,7 @@ PETSC_EXTERN void PETSC_STDCALL petscmemoryshowusage_(PetscViewer *vin, CHAR mes
   *ierr = PetscFixSlashN(msg,&tmp);if (*ierr) return;
   FREECHAR(message,msg);
   PetscPatchDefaultViewers_Fortran(vin,v);
-  *ierr = PetscMemoryShowUsage(v,tmp);if (*ierr) return;
+  *ierr = PetscMemoryView(v,tmp);if (*ierr) return;
   *ierr = PetscFree(tmp);
 }
 
