@@ -13,10 +13,12 @@
 /*
      These are defined in mal.c and ensure that malloced space is PetscScalar aligned
 */
-extern PetscErrorCode  PetscMallocAlign(size_t,int,const char[],const char[],void**);
-extern PetscErrorCode  PetscFreeAlign(void*,int,const char[],const char[]);
-extern PetscErrorCode  PetscTrMallocDefault(size_t,int,const char[],const char[],void**);
-extern PetscErrorCode  PetscTrFreeDefault(void*,int,const char[],const char[]);
+PETSC_EXTERN PetscErrorCode PetscMallocAlign(size_t,int,const char[],const char[],void**);
+PETSC_EXTERN PetscErrorCode PetscFreeAlign(void*,int,const char[],const char[]);
+PETSC_EXTERN PetscErrorCode PetscReallocAlign(size_t,int,const char[],const char[],void**);
+PETSC_EXTERN PetscErrorCode PetscTrMallocDefault(size_t,int,const char[],const char[],void**);
+PETSC_EXTERN PetscErrorCode PetscTrFreeDefault(void*,int,const char[],const char[]);
+PETSC_EXTERN PetscErrorCode PetscTrReallocDefault(size_t,int,const char[],const char[],void**);
 
 
 #define CLASSID_VALUE  ((PetscClassId) 0xf0e0d0c9)
@@ -66,14 +68,13 @@ static size_t     PetscLogMallocThreshold = 0;
 static size_t     *PetscLogMallocLength;
 static const char **PetscLogMallocFile,**PetscLogMallocFunction;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSetUseTrMalloc_Private"
-PetscErrorCode PetscSetUseTrMalloc_Private(void)
+PETSC_INTERN PetscErrorCode PetscSetUseTrMalloc_Private(void)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   ierr = PetscMallocSet(PetscTrMallocDefault,PetscTrFreeDefault);CHKERRQ(ierr);
+  PetscTrRealloc = PetscTrReallocDefault;
 
   TRallocated       = 0;
   TRfrags           = 0;
@@ -86,8 +87,6 @@ PetscErrorCode PetscSetUseTrMalloc_Private(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMallocValidate"
 /*@C
    PetscMallocValidate - Test the memory for corruption.  This can be used to
    check for memory overwrites.
@@ -110,7 +109,6 @@ PetscErrorCode PetscSetUseTrMalloc_Private(void)
     routine.
 
     The line, function, file are given by the C preprocessor as
-    __LINE__, __FUNCT__, __FILE__
 
     The Fortran calling sequence is simply PetscMallocValidate(ierr)
 
@@ -154,15 +152,12 @@ PetscErrorCode  PetscMallocValidate(int line,const char function[],const char fi
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscTrMallocDefault"
 /*
     PetscTrMallocDefault - Malloc with tracing.
 
     Input Parameters:
 +   a   - number of bytes to allocate
 .   lineno - line number where used.  Use __LINE__ for this
-.   function - function calling routine. Use __FUNCT__ for this
 -   filename  - file name where used.  Use __FILE__ for this
 
     Returns:
@@ -240,15 +235,12 @@ PetscErrorCode  PetscTrMallocDefault(size_t a,int lineno,const char function[],c
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscTrFreeDefault"
 /*
    PetscTrFreeDefault - Free with tracing.
 
    Input Parameters:
 .   a    - pointer to a block allocated with PetscTrMalloc
 .   lineno - line number where used.  Use __LINE__ for this
-.   function - function calling routine. Use __FUNCT__ for this
 .   file  - file name where used.  Use __FILE__ for this
  */
 PetscErrorCode  PetscTrFreeDefault(void *aa,int line,const char function[],const char file[])
@@ -319,8 +311,134 @@ PetscErrorCode  PetscTrFreeDefault(void *aa,int line,const char function[],const
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMemoryView"
+
+/*
+  PetscTrReallocDefault - Realloc with tracing.
+
+  Input Parameters:
++ len      - number of bytes to allocate
+. lineno   - line number where used.  Use __LINE__ for this
+. filename - file name where used.  Use __FILE__ for this
+- result   - double aligned pointer to initial storage.
+
+  Output Parameter:
+. result - double aligned pointer to requested storage, or null if not available.
+
+  Level: developer
+
+.seealso: PetscTrMallocDefault(), PetscTrFreeDefault()
+*/
+PetscErrorCode PetscTrReallocDefault(size_t len, int lineno, const char function[], const char filename[], void **result)
+{
+  char           *a = (char *) *result;
+  TRSPACE        *head;
+  char           *ahead, *inew;
+  PetscClassId   *nend;
+  size_t         nsize;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  /* Realloc to zero = free */
+  if (!len) {
+    ierr = PetscTrFreeDefault(*result,lineno,function,filename);CHKERRQ(ierr);
+    *result = NULL;
+    PetscFunctionReturn(0);
+  }
+
+  if (TRdebugLevel) {ierr = PetscMallocValidate(lineno,function,filename); if (ierr) PetscFunctionReturn(ierr);}
+
+  ahead = a;
+  a     = a - sizeof(TrSPACE);
+  head  = (TRSPACE *) a;
+  inew  = a;
+
+  if (head->classid != CLASSID_VALUE) {
+    (*PetscErrorPrintf)("PetscTrReallocDefault() called from %s() line %d in %s\n",function,lineno,filename);
+    (*PetscErrorPrintf)("Block at address %p is corrupted; cannot free;\nmay be block not allocated with PetscMalloc()\n",a);
+    SETERRQ(PETSC_COMM_SELF,PETSC_ERR_MEMC,"Bad location or corrupted memory");
+  }
+  nend = (PetscClassId *)(ahead + head->size);
+  if (*nend != CLASSID_VALUE) {
+    if (*nend == ALREADY_FREED) {
+      (*PetscErrorPrintf)("PetscTrReallocDefault() called from %s() line %d in %s\n",function,lineno,filename);
+      (*PetscErrorPrintf)("Block [id=%d(%.0f)] at address %p was already freed\n",head->id,(PetscLogDouble)head->size,a + sizeof(TrSPACE));
+      if (head->lineno > 0 && head->lineno < 50000 /* sanity check */) {
+        (*PetscErrorPrintf)("Block freed in %s() line %d in %s\n",head->functionname,head->lineno,head->filename);
+      } else {
+        (*PetscErrorPrintf)("Block allocated in %s() line %d in %s\n",head->functionname,-head->lineno,head->filename);
+      }
+      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Memory already freed");
+    } else {
+      /* Damaged tail */
+      (*PetscErrorPrintf)("PetscTrReallocDefault() called from %s() line %d in %s\n",function,lineno,filename);
+      (*PetscErrorPrintf)("Block [id=%d(%.0f)] at address %p is corrupted (probably write past end of array)\n",head->id,(PetscLogDouble)head->size,a);
+      (*PetscErrorPrintf)("Block allocated in %s() line %d in %s\n",head->functionname,head->lineno,head->filename);
+      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_MEMC,"Corrupted memory");
+    }
+  }
+
+  TRallocated -= head->size;
+  TRfrags--;
+  if (head->prev) head->prev->next = head->next;
+  else TRhead = head->next;
+  if (head->next) head->next->prev = head->prev;
+
+  nsize = (len + (PETSC_MEMALIGN-1)) & ~(PETSC_MEMALIGN-1);
+  ierr  = PetscReallocAlign(nsize+sizeof(TrSPACE)+sizeof(PetscClassId),lineno,function,filename,(void**)&inew);CHKERRQ(ierr);
+
+  head  = (TRSPACE*)inew;
+  inew += sizeof(TrSPACE);
+
+  if (TRhead) TRhead->prev = head;
+  head->next   = TRhead;
+  TRhead       = head;
+  head->prev   = NULL;
+  head->size   = nsize;
+  head->id     = TRid;
+  head->lineno = lineno;
+
+  head->filename                 = filename;
+  head->functionname             = function;
+  head->classid                  = CLASSID_VALUE;
+  *(PetscClassId*)(inew + nsize) = CLASSID_VALUE;
+
+  TRallocated += nsize;
+  if (TRallocated > TRMaxMem) TRMaxMem = TRallocated;
+  TRfrags++;
+
+#if defined(PETSC_USE_DEBUG)
+  if (PetscStackActive()) {
+    ierr = PetscStackCopy(petscstack,&head->stack);CHKERRQ(ierr);
+    /* fix the line number to where the malloc() was called, not the PetscFunctionBegin; */
+    head->stack.line[head->stack.currentsize-2] = lineno;
+  } else {
+    head->stack.currentsize = 0;
+  }
+#endif
+
+  /*
+         Allow logging of all mallocs made
+  */
+  if (PetscLogMalloc > -1 && PetscLogMalloc < PetscLogMallocMax && len >= PetscLogMallocThreshold) {
+    if (!PetscLogMalloc) {
+      PetscLogMallocLength = (size_t*)malloc(PetscLogMallocMax*sizeof(size_t));
+      if (!PetscLogMallocLength) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_MEM," ");
+
+      PetscLogMallocFile = (const char**)malloc(PetscLogMallocMax*sizeof(char*));
+      if (!PetscLogMallocFile) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_MEM," ");
+
+      PetscLogMallocFunction = (const char**)malloc(PetscLogMallocMax*sizeof(char*));
+      if (!PetscLogMallocFunction) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_MEM," ");
+    }
+    PetscLogMallocLength[PetscLogMalloc]     = nsize;
+    PetscLogMallocFile[PetscLogMalloc]       = filename;
+    PetscLogMallocFunction[PetscLogMalloc++] = function;
+  }
+  *result = (void*)inew;
+  PetscFunctionReturn(0);
+}
+
+
 /*@C
     PetscMemoryView - Shows the amount of memory currently being used
         in a communicator.
@@ -407,9 +525,7 @@ PetscErrorCode  PetscMemoryView(PetscViewer viewer,const char message[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMallocGetCurrentUsage"
-/*@C
+/*@
     PetscMallocGetCurrentUsage - gets the current amount of memory used that was PetscMalloc()ed
 
     Not Collective
@@ -431,9 +547,7 @@ PetscErrorCode  PetscMallocGetCurrentUsage(PetscLogDouble *space)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMallocGetMaximumUsage"
-/*@C
+/*@
     PetscMallocGetMaximumUsage - gets the maximum amount of memory used that was PetscMalloc()ed at any time
         during this run.
 
@@ -457,8 +571,6 @@ PetscErrorCode  PetscMallocGetMaximumUsage(PetscLogDouble *space)
 }
 
 #if defined(PETSC_USE_DEBUG)
-#undef __FUNCT__
-#define __FUNCT__ "PetscMallocGetStack"
 /*@C
    PetscMallocGetStack - returns a pointer to the stack for the location in the program a call to PetscMalloc() was used to obtain that memory
 
@@ -484,8 +596,6 @@ PetscErrorCode  PetscMallocGetStack(void *ptr,PetscStack **stack)
   PetscFunctionReturn(0);
 }
 #else
-#undef __FUNCT__
-#define __FUNCT__ "PetscMallocGetStack"
 PetscErrorCode  PetscMallocGetStack(void *ptr,void **stack)
 {
   PetscFunctionBegin;
@@ -494,8 +604,6 @@ PetscErrorCode  PetscMallocGetStack(void *ptr,void **stack)
 }
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMallocDump"
 /*@C
    PetscMallocDump - Dumps the allocated memory blocks to a file. The information
    printed is: size of space (in bytes), address of space, id of space,
@@ -516,7 +624,8 @@ PetscErrorCode  PetscMallocGetStack(void *ptr,void **stack)
    The calling sequence in Fortran is PetscMallocDump(integer ierr)
    The fp defaults to stdout.
 
-   Notes: uses MPI_COMM_WORLD, because this may be called in PetscFinalize() after PETSC_COMM_WORLD
+   Notes:
+    uses MPI_COMM_WORLD, because this may be called in PetscFinalize() after PETSC_COMM_WORLD
           has been freed.
 
    Concepts: memory usage
@@ -562,9 +671,7 @@ PetscErrorCode  PetscMallocDump(FILE *fp)
 
 /* ---------------------------------------------------------------------------- */
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMallocSetDumpLog"
-/*@C
+/*@
     PetscMallocSetDumpLog - Activates logging of all calls to PetscMalloc().
 
     Not Collective
@@ -588,9 +695,7 @@ PetscErrorCode PetscMallocSetDumpLog(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMallocSetDumpLogThreshold"
-/*@C
+/*@
     PetscMallocSetDumpLogThreshold - Activates logging of all calls to PetscMalloc().
 
     Not Collective
@@ -617,9 +722,7 @@ PetscErrorCode PetscMallocSetDumpLogThreshold(PetscLogDouble logmin)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMallocGetDumpLog"
-/*@C
+/*@
     PetscMallocGetDumpLog - Determine whether all calls to PetscMalloc() are being logged
 
     Not Collective
@@ -642,8 +745,6 @@ PetscErrorCode PetscMallocGetDumpLog(PetscBool *logging)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMallocDumpLog"
 /*@C
     PetscMallocDumpLog - Dumps the log of all calls to PetscMalloc(); also calls
        PetscMemoryGetMaximumUsage()
@@ -740,9 +841,7 @@ foundit:;
 
 /* ---------------------------------------------------------------------------- */
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMallocDebug"
-/*@C
+/*@
     PetscMallocDebug - Turns on/off debugging for the memory management routines.
 
     Not Collective
@@ -761,9 +860,7 @@ PetscErrorCode  PetscMallocDebug(PetscBool level)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMallocGetDebug"
-/*@C
+/*@
     PetscMallocGetDebug - Indicates if any PETSc is doing ANY memory debugging.
 
     Not Collective
