@@ -5,8 +5,6 @@
 #include <petsc/private/petscimpl.h>  /*I   "petscsys.h"    I*/
 #include <petscviewer.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscComposedQuantitiesDestroy"
 PetscErrorCode PetscComposedQuantitiesDestroy(PetscObject obj)
 {
   PetscErrorCode ierr;
@@ -43,8 +41,6 @@ PetscErrorCode PetscComposedQuantitiesDestroy(PetscObject obj)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectDestroy"
 /*@
    PetscObjectDestroy - Destroys any PetscObject, regardless of the type.
 
@@ -75,8 +71,6 @@ PetscErrorCode  PetscObjectDestroy(PetscObject *obj)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectView"
 /*@C
    PetscObjectView - Views any PetscObject, regardless of the type.
 
@@ -108,8 +102,45 @@ PetscErrorCode  PetscObjectView(PetscObject obj,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectTypeCompare"
+#define CHKERRQI(incall,ierr) if (ierr) {incall = PETSC_FALSE; CHKERRQ(ierr);}
+
+/*@C
+  PetscObjectViewFromOptions - Processes command line options to determine if/how a PetscObject is to be viewed.
+
+  Collective on PetscObject
+
+  Input Parameters:
++ obj   - the object
+. bobj  - optional other object that provides prefix (if NULL then the prefix in obj is used)
+- optionname - option to activate viewing
+
+  Level: intermediate
+
+@*/
+PetscErrorCode PetscObjectViewFromOptions(PetscObject obj,PetscObject bobj,const char optionname[])
+{
+  PetscErrorCode    ierr;
+  PetscViewer       viewer;
+  PetscBool         flg;
+  static PetscBool  incall = PETSC_FALSE;
+  PetscViewerFormat format;
+  const char        *prefix;
+
+  PetscFunctionBegin;
+  if (incall) PetscFunctionReturn(0);
+  incall = PETSC_TRUE;
+  prefix = bobj ? bobj->prefix : obj->prefix;
+  ierr   = PetscOptionsGetViewer(PetscObjectComm((PetscObject)obj),prefix,optionname,&viewer,&format,&flg);CHKERRQI(incall,ierr);
+  if (flg) {
+    ierr = PetscViewerPushFormat(viewer,format);CHKERRQI(incall,ierr);
+    ierr = PetscObjectView(obj,viewer);CHKERRQI(incall,ierr);
+    ierr = PetscViewerPopFormat(viewer);CHKERRQI(incall,ierr);
+    ierr = PetscViewerDestroy(&viewer);CHKERRQI(incall,ierr);
+  }
+  incall = PETSC_FALSE;
+  PetscFunctionReturn(0);
+}
+
 /*@C
    PetscObjectTypeCompare - Determines whether a PETSc object is of a particular type.
 
@@ -126,7 +157,7 @@ PetscErrorCode  PetscObjectView(PetscObject obj,PetscViewer viewer)
 
    Level: intermediate
 
-.seealso: VecGetType(), KSPGetType(), PCGetType(), SNESGetType()
+.seealso: VecGetType(), KSPGetType(), PCGetType(), SNESGetType(), PetscObjectBaseTypeCompare()
 
    Concepts: comparing^object types
    Concepts: types^comparing
@@ -150,8 +181,41 @@ PetscErrorCode  PetscObjectTypeCompare(PetscObject obj,const char type_name[],Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectTypeCompareAny"
+/*@C
+   PetscObjectBaseTypeCompare - Determines whether a PetscObject is of a given base type. For example the base type of MATSEQAIJPERM is MATSEQAIJ
+
+   Not Collective
+
+   Input Parameters:
++  mat - the matrix
+-  type_name - string containing a type name
+
+   Output Parameter:
+.  same - PETSC_TRUE if it is of the same base type
+
+   Level: intermediate
+
+.seealso: PetscObjectTypeCompare()
+
+
+@*/
+PetscErrorCode  PetscObjectBaseTypeCompare(PetscObject obj,const char type_name[],PetscBool  *same)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (!obj) *same = PETSC_FALSE;
+  else if (!type_name && !obj->type_name) *same = PETSC_TRUE;
+  else if (!type_name || !obj->type_name) *same = PETSC_FALSE;
+  else {
+    PetscValidHeader(obj,1);
+    PetscValidCharPointer(type_name,2);
+    PetscValidPointer(same,3);
+    ierr = PetscStrbeginswith((char*)(obj->type_name),type_name,same);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 /*@C
    PetscObjectTypeCompareAny - Determines whether a PETSc object is of any of a list of types.
 
@@ -199,8 +263,6 @@ PetscErrorCode PetscObjectTypeCompareAny(PetscObject obj,PetscBool *match,const
 static int         PetscObjectRegisterDestroy_Count = 0;
 static PetscObject PetscObjectRegisterDestroy_Objects[MAXREGDESOBJS];
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectRegisterDestroy"
 /*@C
    PetscObjectRegisterDestroy - Registers a PETSc object to be destroyed when
      PetscFinalize() is called.
@@ -229,8 +291,6 @@ PetscErrorCode  PetscObjectRegisterDestroy(PetscObject obj)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectRegisterDestroyAll"
 /*@C
    PetscObjectRegisterDestroyAll - Frees all the PETSc objects that have been registered
      with PetscObjectRegisterDestroy(). Called by PetscFinalize()
@@ -257,10 +317,8 @@ PetscErrorCode  PetscObjectRegisterDestroyAll(void)
 
 #define MAXREGFIN 256
 static int PetscRegisterFinalize_Count = 0;
-static PetscErrorCode ((*PetscRegisterFinalize_Functions[MAXREGFIN])(void));
+static PetscErrorCode (*PetscRegisterFinalize_Functions[MAXREGFIN])(void);
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscRegisterFinalize"
 /*@C
    PetscRegisterFinalize - Registers a function that is to be called in PetscFinalize()
 
@@ -289,8 +347,6 @@ PetscErrorCode  PetscRegisterFinalize(PetscErrorCode (*f)(void))
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscRegisterFinalizeAll"
 /*@C
    PetscRegisterFinalizeAll - Runs all the finalize functions set with PetscRegisterFinalize()
 
