@@ -16,8 +16,6 @@ typedef struct {
 static FortranCallbackBase *_classbase;
 static PetscClassId        _maxclassid = PETSC_SMALLEST_CLASSID;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscFortranCallbackFinalize"
 static PetscErrorCode PetscFortranCallbackFinalize(void)
 {
   PetscErrorCode ierr;
@@ -39,8 +37,6 @@ static PetscErrorCode PetscFortranCallbackFinalize(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscFortranCallbackRegister"
 /*@C
    PetscFortranCallbackRegister - register a type+subtype callback
 
@@ -65,7 +61,7 @@ PetscErrorCode PetscFortranCallbackRegister(PetscClassId classid,const char *sub
 
   PetscFunctionBegin;
   *id = 0;
-  if (classid < PETSC_SMALLEST_CLASSID || PETSC_LARGEST_CLASSID <= classid) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_CORRUPT,"ClassId %D corrupt",classid);
+  if (classid < PETSC_SMALLEST_CLASSID || PETSC_LARGEST_CLASSID < classid) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_CORRUPT,"ClassId %D corrupt",classid);
   if (classid >= _maxclassid) {
     PetscClassId        newmax = PETSC_SMALLEST_CLASSID + 2*(PETSC_LARGEST_CLASSID-PETSC_SMALLEST_CLASSID);
     FortranCallbackBase *newbase;
@@ -105,8 +101,6 @@ found:
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscFortranCallbackGetSizes"
 /*@C
    PetscFortranCallbackGetSizes - get sizes of class and subtype pointer arrays
 
