@@ -6,12 +6,12 @@
 #define petscobjecttypecompare_          petscobjecttypecompare
 #endif
 
-PETSC_EXTERN void PETSC_STDCALL petscobjecttypecompare_(PetscObject *obj,CHAR type_name PETSC_MIXED_LEN(len),
+PETSC_EXTERN void PETSC_STDCALL petscobjecttypecompare_(PetscObject *obj,char* type_name PETSC_MIXED_LEN(len),
                                      PetscBool  *same,PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   char *c1;
   FIXCHAR(type_name,len,c1);
-  *ierr = PetscObjectTypeCompare(*obj,c1,same);
+  *ierr = PetscObjectTypeCompare(*obj,c1,same);if (*ierr) return;
   FREECHAR(type_name,c1);
 }
 
