@@ -13,26 +13,28 @@
 
 #include <petsc/private/fortranimpl.h>
 
-#if defined(PETSC_HAVE_CUDA)
-#include <cublas.h>
-#endif
-
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define petscinitialize_              PETSCINITIALIZE
+#define petscinitializenoarguments_   PETSCINITIALIZENOARGUMENTS
 #define petscfinalize_                PETSCFINALIZE
 #define petscend_                     PETSCEND
 #define iargc_                        IARGC
 #define getarg_                       GETARG
 #define mpi_init_                     MPI_INIT
-#define petscgetcommoncomm_           PETSCGETCOMMONCOMM
+#define petscgetcomm_                 PETSCGETCOMM
+#define petsccommandargumentcount_    PETSCCOMMANDARGUMENTCOUNT
+#define petscgetcommandargument_      PETSCGETCOMMANDARGUMENT
 #elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
 #define petscinitialize_              petscinitialize
+#define petscinitializenoarguments_   petscinitializenoarguments
 #define petscfinalize_                petscfinalize
 #define petscend_                     petscend
 #define mpi_init_                     mpi_init
 #define iargc_                        iargc
 #define getarg_                       getarg
-#define petscgetcommoncomm_           petscgetcommoncomm
+#define petscgetcomm_                 petscgetcomm
+#define petsccommandargumentcount_    petsccommandargumentcount
+#define petscgetcommandargument_      petscgetcommandargument
 #endif
 
 #if defined(PETSC_HAVE_NAGF90)
@@ -58,7 +60,13 @@
 #define iargc_   iargc_
 #define getarg_  getarg_
 #endif
-#if defined(PETSC_HAVE_GFORTRAN_IARGC) /* gfortran from gcc4 */
+
+#if defined(PETSC_HAVE_FORTRAN_GET_COMMAND_ARGUMENT) /* Fortran 2003 */
+#undef iargc_
+#undef getarg_
+#define iargc_ petsccommandargumentcount_
+#define getarg_ petscgetcommandargument_
+#elif defined(PETSC_HAVE_GFORTRAN_IARGC) /* gfortran from gcc4 */
 #undef iargc_
 #undef getarg_
 #define iargc_  _gfortran_iargc
@@ -76,17 +84,34 @@
   contained any _.
 */
 #if defined(PETSC_HAVE_FORTRAN_UNDERSCORE_UNDERSCORE)
-#undef mpi_init_
 #define mpi_init_             mpi_init__
 #endif
 
+#if defined(PETSC_HAVE_MPIUNI)
+#if defined(mpi_init_)
+#undef mpi_init_
+#if defined(PETSC_HAVE_FORTRAN_CAPS)
+#define mpi_init_             PETSC_MPI_INIT
+#elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
+#define mpi_init_             petsc_mpi_init
+#elif defined(PETSC_HAVE_FORTRAN_UNDERSCORE_UNDERSCORE)
+#define mpi_init_             petsc_mpi_init__
+#endif
+#else    /* mpi_init_ */
+#define mpi_init_             petsc_mpi_init_
+#endif   /* mpi_init_ */
+#endif   /* PETSC_HAVE_MPIUNI */
+
 PETSC_EXTERN void PETSC_STDCALL mpi_init_(int*);
-PETSC_EXTERN void PETSC_STDCALL petscgetcommoncomm_(PetscMPIInt*);
+PETSC_EXTERN void PETSC_STDCALL petscgetcomm_(PetscMPIInt*);
 
 /*
      Different Fortran compilers handle command lines in different ways
 */
-#if defined(PETSC_USE_NARGS)
+#if defined(PETSC_HAVE_FORTRAN_GET_COMMAND_ARGUMENT) /* Fortran 2003  - same as 'else' case */
+PETSC_EXTERN int iargc_(void);
+PETSC_EXTERN void getarg_(int*,char*,int);
+#elif defined(PETSC_USE_NARGS)
 PETSC_EXTERN short __stdcall NARGS();
 PETSC_EXTERN void __stdcall GETARG(short*,char*,int,short *);
 
@@ -109,40 +134,38 @@ PETSC_EXTERN void PXFGETARG(int*,_fcd,int*,int*);
 #endif
 #endif
 
-#if (defined(PETSC_HAVE_COMPLEX) && !defined(PETSC_HAVE_MPI_C_DOUBLE_COMPLEX)) || defined(PETSC_USE_REAL___FLOAT128)
-extern MPI_Op MPIU_SUM;
+#if (defined(PETSC_HAVE_COMPLEX) && !defined(PETSC_HAVE_MPI_C_DOUBLE_COMPLEX)) || defined(PETSC_USE_REAL___FLOAT128) || defined(PETSC_USE_REAL___FP16)
+PETSC_EXTERN MPI_Op MPIU_SUM;
 
 PETSC_EXTERN void MPIAPI PetscSum_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
 
 #endif
-#if defined(PETSC_USE_REAL___FLOAT128)
+#if defined(PETSC_USE_REAL___FLOAT128) || defined(PETSC_USE_REAL___FP16)
 
 PETSC_EXTERN void MPIAPI PetscSum_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
 PETSC_EXTERN void MPIAPI PetscMax_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
 PETSC_EXTERN void MPIAPI PetscMin_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
 #endif
 
-extern MPI_Op PetscMaxSum_Op;
-
-PETSC_EXTERN void MPIAPI PetscMaxSum_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
+PETSC_INTERN void MPIAPI MPIU_MaxSum_Local(void*,void*,PetscMPIInt*,MPI_Datatype*);
 PETSC_EXTERN PetscMPIInt MPIAPI Petsc_DelCounter(MPI_Comm,PetscMPIInt,void*,void*);
 PETSC_EXTERN PetscMPIInt MPIAPI Petsc_DelComm_Inner(MPI_Comm,PetscMPIInt,void*,void*);
 PETSC_EXTERN PetscMPIInt MPIAPI Petsc_DelComm_Outer(MPI_Comm,PetscMPIInt,void*,void*);
 
-extern PetscErrorCode  PetscOptionsCheckInitial_Private(void);
-extern PetscErrorCode  PetscOptionsCheckInitial_Components(void);
-extern PetscErrorCode  PetscInitialize_DynamicLibraries(void);
+PETSC_INTERN PetscErrorCode PetscOptionsCheckInitial_Private(void);
+PETSC_INTERN PetscErrorCode PetscOptionsCheckInitial_Components(void);
+PETSC_INTERN PetscErrorCode PetscInitialize_DynamicLibraries(void);
 #if defined(PETSC_USE_LOG)
-extern PetscErrorCode  PetscLogInitialize(void);
+PETSC_INTERN PetscErrorCode PetscLogInitialize(void);
 #endif
-extern PetscErrorCode  PetscMallocAlign(size_t,int,const char[],const char[],void**);
-extern PetscErrorCode  PetscFreeAlign(void*,int,const char[],const char[]);
-extern int  PetscGlobalArgc;
-extern char **PetscGlobalArgs;
+PETSC_EXTERN PetscErrorCode PetscMallocAlign(size_t,int,const char[],const char[],void**);
+PETSC_EXTERN PetscErrorCode PetscFreeAlign(void*,int,const char[],const char[]);
+PETSC_INTERN int  PetscGlobalArgc;
+PETSC_INTERN char **PetscGlobalArgs;
 
 /*
     Reads in Fortran command line argments and sends them to
-  all processors and adds them to Options database.
+  all processors.
 */
 
 PetscErrorCode PETScParseFortranArgs_Private(int *argc,char ***argv)
@@ -176,7 +199,9 @@ PetscErrorCode PETScParseFortranArgs_Private(int *argc,char ***argv)
     ierr = PetscMemzero((*argv)[0],(*argc)*warg*sizeof(char));CHKERRQ(ierr);
     for (i=0; i<*argc; i++) {
       (*argv)[i+1] = (*argv)[i] + warg;
-#if defined(PETSC_HAVE_PXFGETARG_NEW)
+#if defined (PETSC_HAVE_FORTRAN_GET_COMMAND_ARGUMENT) /* same as 'else' case */
+      getarg_(&i,(*argv)[i],warg);
+#elif defined(PETSC_HAVE_PXFGETARG_NEW)
       {char *tmp = (*argv)[i];
       int ilen;
       getarg_(&i,tmp,&ilen,&ierr,warg);CHKERRQ(ierr);
@@ -209,16 +234,22 @@ PetscErrorCode PETScParseFortranArgs_Private(int *argc,char ***argv)
 }
 
 #if defined(PETSC_SERIALIZE_FUNCTIONS)
-extern PetscFPT PetscFPTData;
+PETSC_INTERN PetscFPT PetscFPTData;
 #endif
 
+#if defined(PETSC_HAVE_ADIOS)
+#include <adios.h>
+#include <adios_read.h>
+#endif
 /* -----------------------------------------------------------------------------------------------*/
 
 #if defined(PETSC_HAVE_SAWS)
 #include <petscviewersaws.h>
-extern PetscErrorCode  PetscInitializeSAWs(const char[]);
+PETSC_INTERN PetscErrorCode  PetscInitializeSAWs(const char[]);
 #endif
 
+PETSC_EXTERN PetscMPIInt MPIAPI Petsc_DelComm_Shm(MPI_Comm,PetscMPIInt,void *,void *);
+
 /*
     petscinitialize - Version called from Fortran.
 
@@ -226,25 +257,18 @@ extern PetscErrorCode  PetscInitializeSAWs(const char[]);
       Since this is called from Fortran it does not return error codes
 
 */
-PETSC_EXTERN void PETSC_STDCALL petscinitialize_(CHAR filename PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len))
+static void petscinitialize_internal(char* filename, PetscInt len, PetscBool readarguments, PetscErrorCode *ierr)
 {
+  int            j,i;
 #if defined (PETSC_USE_NARGS)
-  short       flg,i;
-#else
-  int         i;
-#if !defined(PETSC_HAVE_PXFGETARG_NEW) && !defined (PETSC_HAVE_PXFGETARG_NEW)
-  int         j;
+  short          flg;
 #endif
-#endif
-#if defined(PETSC_HAVE_CUDA)
-  PetscBool   flg2;
-#endif
-  int         flag;
-  PetscMPIInt size;
-  char        *t1,name[256],hostname[64];
-  PetscMPIInt f_petsc_comm_world;
+  int            flag;
+  PetscMPIInt    size;
+  char           *t1,name[256],hostname[64];
+  PetscMPIInt    f_petsc_comm_world;
 
-  *ierr = PetscMemzero(name,256); if (*ierr) return;
+  *ierr = PetscMemzero(name,sizeof(name)); if (*ierr) return;
   if (PetscInitializeCalled) {*ierr = 0; return;}
 
   /* this must be initialized in a routine, not as a constant declaration*/
@@ -256,10 +280,12 @@ PETSC_EXTERN void PETSC_STDCALL petscinitialize_(CHAR filename PETSC_MIXED_LEN(l
   _set_output_format(_TWO_DIGIT_EXPONENT);
 #endif
 
-  *ierr = PetscOptionsCreate();
+  *ierr = PetscOptionsCreateDefault();
   if (*ierr) return;
   i = 0;
-#if defined (PETSC_HAVE_PXFGETARG_NEW)
+#if defined (PETSC_HAVE_FORTRAN_GET_COMMAND_ARGUMENT) /* same as 'else' case */
+  getarg_(&i,name,sizeof(name));
+#elif defined (PETSC_HAVE_PXFGETARG_NEW)
   { int ilen,sierr;
     getarg_(&i,name,&ilen,&sierr,256);
     if (sierr) PetscStrncpy(name,"Unknown Name",256);
@@ -269,31 +295,26 @@ PETSC_EXTERN void PETSC_STDCALL petscinitialize_(CHAR filename PETSC_MIXED_LEN(l
   GETARG(&i,name,256,&flg);
 #else
   getarg_(&i,name,256);
+#endif
   /* Eliminate spaces at the end of the string */
-  for (j=254; j>=0; j--) {
+  for (j=sizeof(name)-2; j>=0; j--) {
     if (name[j] != ' ') {
       name[j+1] = 0;
       break;
     }
   }
   if (j<0) PetscStrncpy(name,"Unknown Name",256);
-#endif
   *ierr = PetscSetProgramName(name);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize: Calling PetscSetProgramName()\n");return;}
 
   /* check if PETSC_COMM_WORLD is initialized by the user in fortran */
-  petscgetcommoncomm_(&f_petsc_comm_world);
+  petscgetcomm_(&f_petsc_comm_world);
   MPI_Initialized(&flag);
   if (!flag) {
     PetscMPIInt mierr;
 
     if (f_petsc_comm_world) {(*PetscErrorPrintf)("You cannot set PETSC_COMM_WORLD if you have not initialized MPI first\n");return;}
-    /* MPI requires calling Fortran mpi_init() if main program is Fortran */
-#if defined(PETSC_HAVE_MPIUNI) && defined(MPIUNI_AVOID_MPI_NAMESPACE)
-    mierr = MPI_Init((int*)0, (char***)0);
-#else
     mpi_init_(&mierr);
-#endif
     if (mierr) {
       *ierr = mierr;
       (*PetscErrorPrintf)("PetscInitialize: Calling Fortran MPI_Init()\n");
@@ -305,6 +326,15 @@ PETSC_EXTERN void PETSC_STDCALL petscinitialize_(CHAR filename PETSC_MIXED_LEN(l
   else PETSC_COMM_WORLD = MPI_COMM_WORLD;
   PetscInitializeCalled = PETSC_TRUE;
 
+  *ierr = PetscSpinlockCreate(&PetscViewerASCIISpinLockOpen);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize: Creating global spin lock\n");return;}
+  *ierr = PetscSpinlockCreate(&PetscViewerASCIISpinLockStdout);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize: Creating global spin lock\n");return;}
+  *ierr = PetscSpinlockCreate(&PetscViewerASCIISpinLockStderr);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize: Creating global spin lock\n");return;}
+  *ierr = PetscSpinlockCreate(&PetscCommSpinLock);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize: Creating global spin lock\n");return;}
+
   *ierr = PetscErrorPrintfInitialize();
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize: Calling PetscErrorPrintfInitialize()\n");return;}
   *ierr = MPI_Comm_rank(MPI_COMM_WORLD,&PetscGlobalRank);
@@ -322,7 +352,7 @@ PETSC_EXTERN void PETSC_STDCALL petscinitialize_(CHAR filename PETSC_MIXED_LEN(l
      are not called; at least on IRIX.
   */
   {
-#if defined(PETSC_CLANGUAGE_CXX)
+#if defined(PETSC_CLANGUAGE_CXX) && !defined(PETSC_USE_REAL___FLOAT128)
     PetscComplex ic(0.0,1.0);
     PETSC_i = ic;
 #else
@@ -362,13 +392,24 @@ PETSC_EXTERN void PETSC_STDCALL petscinitialize_(CHAR filename PETSC_MIXED_LEN(l
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI ops\n");return;}
   *ierr = MPI_Op_create(PetscMin_Local,1,&MPIU_MIN);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI ops\n");return;}
+#elif defined(PETSC_USE_REAL___FP16)
+  *ierr = MPI_Type_contiguous(2,MPI_CHAR,&MPIU___FP16);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI types\n");return;}
+  *ierr = MPI_Type_commit(&MPIU___FP16);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI types\n");return;}
+  *ierr = MPI_Op_create(PetscSum_Local,1,&MPIU_SUM);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI ops\n");return;}
+  *ierr = MPI_Op_create(PetscMax_Local,1,&MPIU_MAX);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI ops\n");return;}
+  *ierr = MPI_Op_create(PetscMin_Local,1,&MPIU_MIN);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI ops\n");return;}
 #endif
 
   /*
        Create the PETSc MPI reduction operator that sums of the first
      half of the entries and maxes the second half.
   */
-  *ierr = MPI_Op_create(PetscMaxSum_Local,1,&PetscMaxSum_Op);
+  *ierr = MPI_Op_create(MPIU_MaxSum_Local,1,&MPIU_MAXSUM_OP);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI ops\n");return;}
 
   *ierr = MPI_Type_contiguous(2,MPIU_SCALAR,&MPIU_2SCALAR);
@@ -381,11 +422,13 @@ PETSC_EXTERN void PETSC_STDCALL petscinitialize_(CHAR filename PETSC_MIXED_LEN(l
   *ierr = MPI_Type_commit(&MPIU_2INT);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI types\n");return;}
 #endif
-  *ierr = MPI_Keyval_create(MPI_NULL_COPY_FN,Petsc_DelCounter,&Petsc_Counter_keyval,(void*)0);
+  *ierr = MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN,Petsc_DelCounter,&Petsc_Counter_keyval,(void*)0);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI keyvals\n");return;}
-  *ierr = MPI_Keyval_create(MPI_NULL_COPY_FN,Petsc_DelComm_Outer,&Petsc_InnerComm_keyval,(void*)0);
+  *ierr = MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN,Petsc_DelComm_Outer,&Petsc_InnerComm_keyval,(void*)0);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI keyvals\n");return;}
-  *ierr = MPI_Keyval_create(MPI_NULL_COPY_FN,Petsc_DelComm_Inner,&Petsc_OuterComm_keyval,(void*)0);
+  *ierr = MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN,Petsc_DelComm_Inner,&Petsc_OuterComm_keyval,(void*)0);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI keyvals\n");return;}
+  *ierr = MPI_Comm_create_keyval(MPI_COMM_NULL_COPY_FN,Petsc_DelComm_Shm,&Petsc_ShmComm_keyval,(void*)0);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating MPI keyvals\n");return;}
 
   /*
@@ -395,13 +438,19 @@ PETSC_EXTERN void PETSC_STDCALL petscinitialize_(CHAR filename PETSC_MIXED_LEN(l
      below.
   */
   PetscInitializeFortran();
-  PETScParseFortranArgs_Private(&PetscGlobalArgc,&PetscGlobalArgs);
-  FIXCHAR(filename,len,t1);
-  *ierr = PetscOptionsInsert(&PetscGlobalArgc,&PetscGlobalArgs,t1);
-  FREECHAR(filename,t1);
-  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating options database\n");return;}
+  if(readarguments == PETSC_TRUE) {
+    PETScParseFortranArgs_Private(&PetscGlobalArgc,&PetscGlobalArgs);
+    FIXCHAR(filename,len,t1);
+    *ierr = PetscOptionsInsert(NULL,&PetscGlobalArgc,&PetscGlobalArgs,t1);
+    if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Creating options database\n");return;}
+    FREECHAR(filename,t1);
+    if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Freeing string in creating options database\n");return;}
+  }
   *ierr = PetscOptionsCheckInitial_Private();
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Checking initial options\n");return;}
+  /* call a second time to check options database */
+  *ierr = PetscErrorPrintfInitialize();
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize: Calling PetscErrorPrintfInitialize()\n");return;}
   *ierr = PetscCitationsInitialize();
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:PetscCitationsInitialize()\n");return;}
 #if defined(PETSC_HAVE_SAWS)
@@ -429,7 +478,7 @@ PETSC_EXTERN void PETSC_STDCALL petscinitialize_(CHAR filename PETSC_MIXED_LEN(l
   *ierr = PetscOptionsCheckInitial_Components();
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:Checking initial options\n");return;}
 
-#if defined(PETSC_USE_DEBUG)
+#if defined(PETSC_USE_DEBUG) && !defined(PETSC_HAVE_THREADSAFETY)
   *ierr = PetscStackCreate();
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:PetscStackCreate()\n");return;}
 #endif
@@ -438,14 +487,29 @@ PETSC_EXTERN void PETSC_STDCALL petscinitialize_(CHAR filename PETSC_MIXED_LEN(l
   *ierr = PetscFPTCreate(10000);
   if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:PetscFPTCreate()\n");return;}
 #endif
-
-#if defined(PETSC_HAVE_CUDA)
-  flg2  = PETSC_TRUE;
-  *ierr = PetscOptionsGetBool(NULL,"-cublas",&flg2,NULL);
-  if (flg2) cublasInit();
+#if defined(PETSC_HAVE_ADIOS)
+  *ierr = adios_init_noxml(PETSC_COMM_WORLD);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:adios_init_noxml()\n");return;}
+  *ierr = adios_declare_group(&Petsc_adios_group,"PETSc","",adios_stat_default);
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:adios_declare_group()\n");return;}
+  *ierr = adios_select_method(Petsc_adios_group,"MPI","","");
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:adios_select_method()\n");return;}
+  *ierr = adios_read_init_method(ADIOS_READ_METHOD_BP,PETSC_COMM_WORLD,"");
+  if (*ierr) {(*PetscErrorPrintf)("PetscInitialize:adios_read_init_method()\n");return;}
 #endif
 }
 
+PETSC_EXTERN void PETSC_STDCALL petscinitialize_(char* filename PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len))
+{
+  petscinitialize_internal(filename, len, PETSC_TRUE, ierr);
+}
+
+PETSC_EXTERN void PETSC_STDCALL petscinitializenoarguments_(PetscErrorCode *ierr)
+{
+  petscinitialize_internal(NULL, (PetscInt) 0, PETSC_FALSE, ierr);
+}
+
+
 PETSC_EXTERN void PETSC_STDCALL petscfinalize_(PetscErrorCode *ierr)
 {
 #if defined(PETSC_HAVE_SUNMATHPRO)
@@ -458,7 +522,7 @@ PETSC_EXTERN void PETSC_STDCALL petscfinalize_(PetscErrorCode *ierr)
   *ierr = PetscFinalize();
 }
 
-void PETSC_STDCALL petscend_(PetscErrorCode *ierr)
+PETSC_EXTERN void PETSC_STDCALL petscend_(PetscErrorCode *ierr)
 {
 #if defined(PETSC_HAVE_SUNMATHPRO)
   extern void standard_arithmetic();
