@@ -1,23 +1,30 @@
 
-#include <petsc-private/fortranimpl.h>
+#include <petsc/private/fortranimpl.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define petscinitializefortran_       PETSCINITIALIZEFORTRAN
-#define petscsetcommonblock_          PETSCSETCOMMONBLOCK
+#define petscsetmoduleblock_          PETSCSETMODULEBLOCK
 #define petscsetfortranbasepointers_  PETSCSETFORTRANBASEPOINTERS
 #define petsc_null_function_          PETSC_NULL_FUNCTION
+#define petscsetmoduleblocknumeric_   PETSCSETMODULEBLOCKNUMERIC
+#define petscsetcomm_                 PETSCSETCOMM
 #elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
 #define petscinitializefortran_       petscinitializefortran
-#define petscsetcommonblock_          petscsetcommonblock
+#define petscsetmoduleblock_          petscsetmoduleblock
 #define petscsetfortranbasepointers_  petscsetfortranbasepointers
 #define petsc_null_function_          petsc_null_function
+#define petscsetmoduleblocknumeric_   petscsetmoduleblocknumeric
+#define petscsetcomm_                 petscsetcomm
 #endif
 
 #if defined(PETSC_HAVE_FORTRAN_UNDERSCORE_UNDERSCORE)
 #define petsc_null_function_  petsc_null_function__
 #endif
 
-PETSC_EXTERN void PETSC_STDCALL petscsetcommonblock_(int*,int*);
+PETSC_EXTERN void PETSC_STDCALL petscsetmoduleblock_();
+PETSC_EXTERN void PETSC_STDCALL petscsetmoduleblockmpi_(MPI_Fint*,MPI_Fint*,MPI_Fint*);
+PETSC_EXTERN void PETSC_STDCALL petscsetmoduleblocknumeric_(PetscReal*,PetscReal*,PetscReal*,PetscReal*,PetscReal*,PetscReal*,PetscReal*,PetscReal*);
+PETSC_EXTERN void PETSC_STDCALL petscsetcomm_(MPI_Fint*,MPI_Fint*);
 
 /*@C
    PetscInitializeFortran - Routine that should be called soon AFTER
@@ -41,12 +48,34 @@ PETSC_EXTERN void PETSC_STDCALL petscsetcommonblock_(int*,int*);
 @*/
 PetscErrorCode PetscInitializeFortran(void)
 {
-  PetscMPIInt c1=0,c2=0;
+  MPI_Fint c1=0,c2=0;
 
   if (PETSC_COMM_WORLD) c1 =  MPI_Comm_c2f(PETSC_COMM_WORLD);
-  /* PETSC_COMM_SELF is defined as MPI_COMM_SELF */
   c2 =  MPI_Comm_c2f(PETSC_COMM_SELF);
-  petscsetcommonblock_(&c1,&c2);
+  petscsetmoduleblock_();
+  petscsetcomm_(&c1,&c2);
+
+#if defined(PETSC_USE_REAL___FLOAT128)
+  {
+    MPI_Fint freal,fscalar,fsum;
+    freal   = MPI_Type_c2f(MPIU_REAL);
+    fscalar = MPI_Type_c2f(MPIU_SCALAR);
+    fsum    = MPI_Op_c2f(MPIU_SUM);
+    petscsetmoduleblockmpi_(&freal,&fscalar,&fsum);
+  }
+#endif
+
+  {
+    PetscReal pi = PETSC_PI;
+    PetscReal maxreal = PETSC_MAX_REAL;
+    PetscReal minreal = PETSC_MIN_REAL;
+    PetscReal eps = PETSC_MACHINE_EPSILON;
+    PetscReal seps = PETSC_SQRT_MACHINE_EPSILON;
+    PetscReal small = PETSC_SMALL;
+    PetscReal pinf = PETSC_INFINITY;
+    PetscReal pninf = PETSC_NINFINITY;
+    petscsetmoduleblocknumeric_(&pi,&maxreal,&minreal,&eps,&seps,&small,&pinf,&pninf);
+  }
   return 0;
 }
 
@@ -57,7 +86,7 @@ PETSC_EXTERN void PETSC_STDCALL petscinitializefortran_(int *ierr)
 
 PETSC_EXTERN void PETSC_STDCALL petscsetfortranbasepointers_(char *fnull_character PETSC_MIXED_LEN(len),
                                   void *fnull_integer,void *fnull_scalar,void * fnull_double,
-                                  void *fnull_real,void *fnull_object,
+                                  void *fnull_real,
                                   void* fnull_truth,void (*fnull_function)(void) PETSC_END_LEN(len))
 {
   PETSC_NULL_CHARACTER_Fortran = fnull_character;
@@ -65,7 +94,6 @@ PETSC_EXTERN void PETSC_STDCALL petscsetfortranbasepointers_(char *fnull_charact
   PETSC_NULL_SCALAR_Fortran    = fnull_scalar;
   PETSC_NULL_DOUBLE_Fortran    = fnull_double;
   PETSC_NULL_REAL_Fortran      = fnull_real;
-  PETSC_NULL_OBJECT_Fortran    = fnull_object;
   PETSC_NULL_BOOL_Fortran      = fnull_truth;
   PETSC_NULL_FUNCTION_Fortran  = fnull_function;
 }
