@@ -7,6 +7,7 @@
 */
 
 #include <petscsys.h>        /*I  "petscsys.h"   I*/
+#include <petsc/private/petscimpl.h>
 #include <petscvalgrind.h>
 #include <petscviewer.h>
 
@@ -20,6 +21,10 @@
 #include <cuda_runtime.h>
 #endif
 
+#if defined(PETSC_HAVE_VIENNACL)
+PETSC_EXTERN PetscErrorCode PetscViennaCLInit();
+#endif
+
 /* ------------------------Nasty global variables -------------------------------*/
 /*
      Indicates if PETSc started up MPI, or it was
@@ -63,6 +68,8 @@ MPI_Datatype MPIU___FLOAT128 = 0;
 #if defined(PETSC_HAVE_COMPLEX)
 MPI_Datatype MPIU___COMPLEX128 = 0;
 #endif
+#elif defined(PETSC_USE_REAL___FP16)
+MPI_Datatype MPIU___FP16 = 0;
 #endif
 MPI_Datatype MPIU_2SCALAR = 0;
 #if defined(PETSC_USE_64BIT_INDICES) || !defined(MPI_2INT)
@@ -76,15 +83,10 @@ MPI_Datatype MPIU_ENUM;
 */
 PetscErrorCode (*PetscErrorPrintf)(const char [],...)          = PetscErrorPrintfDefault;
 PetscErrorCode (*PetscHelpPrintf)(MPI_Comm,const char [],...)  = PetscHelpPrintfDefault;
-#if defined(PETSC_HAVE_MATLAB_ENGINE)
-PetscErrorCode (*PetscVFPrintf)(FILE*,const char[],va_list)    = PetscVFPrintf_Matlab;
-#else
 PetscErrorCode (*PetscVFPrintf)(FILE*,const char[],va_list)    = PetscVFPrintfDefault;
-#endif
 /*
   This is needed to turn on/off GPU synchronization
 */
-PetscBool PetscCUSPSynchronize = PETSC_FALSE;
 PetscBool PetscViennaCLSynchronize = PETSC_FALSE;
 PetscBool PetscCUDASynchronize = PETSC_FALSE;
 
@@ -92,10 +94,9 @@ PetscBool PetscCUDASynchronize = PETSC_FALSE;
 /*
    Optional file where all PETSc output from various prints is saved
 */
+PETSC_INTERN FILE *petsc_history;
 FILE *petsc_history = NULL;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscOpenHistoryFile"
 PetscErrorCode  PetscOpenHistoryFile(const char filename[],FILE **fd)
 {
   PetscErrorCode ierr;
@@ -136,9 +137,7 @@ PetscErrorCode  PetscOpenHistoryFile(const char filename[],FILE **fd)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscCloseHistoryFile"
-PetscErrorCode  PetscCloseHistoryFile(FILE **fd)
+PETSC_INTERN PetscErrorCode PetscCloseHistoryFile(FILE **fd)
 {
   PetscErrorCode ierr;
   PetscMPIInt    rank;
@@ -171,18 +170,14 @@ PetscErrorCode  PetscCloseHistoryFile(FILE **fd)
   in the debugger hence we call abort() instead of MPI_Abort().
 */
 
-#undef __FUNCT__
-#define __FUNCT__ "Petsc_MPI_AbortOnError"
-void Petsc_MPI_AbortOnError(MPI_Comm *comm,PetscMPIInt *flag)
+void Petsc_MPI_AbortOnError(MPI_Comm *comm,PetscMPIInt *flag,...)
 {
   PetscFunctionBegin;
   (*PetscErrorPrintf)("MPI error %d\n",*flag);
   abort();
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "Petsc_MPI_DebuggerOnError"
-void Petsc_MPI_DebuggerOnError(MPI_Comm *comm,PetscMPIInt *flag)
+void Petsc_MPI_DebuggerOnError(MPI_Comm *comm,PetscMPIInt *flag,...)
 {
   PetscErrorCode ierr;
 
@@ -192,8 +187,6 @@ void Petsc_MPI_DebuggerOnError(MPI_Comm *comm,PetscMPIInt *flag)
   if (ierr) MPI_Abort(*comm,*flag); /* hopeless so get out */
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscEnd"
 /*@C
    PetscEnd - Calls PetscFinalize() and then ends the program. This is useful if one
      wishes a clean exit somewhere deep in the program.
@@ -218,15 +211,14 @@ PetscErrorCode  PetscEnd(void)
 }
 
 PetscBool PetscOptionsPublish = PETSC_FALSE;
-extern PetscErrorCode PetscSetUseTrMalloc_Private(void);
-extern PetscBool      petscsetmallocvisited;
-static char           emacsmachinename[256];
+PETSC_INTERN PetscErrorCode PetscSetUseTrMalloc_Private(void);
+PETSC_INTERN PetscErrorCode PetscSetUseHBWMalloc_Private(void);
+PETSC_INTERN PetscBool      petscsetmallocvisited;
+static       char           emacsmachinename[256];
 
 PetscErrorCode (*PetscExternalVersionFunction)(MPI_Comm) = 0;
 PetscErrorCode (*PetscExternalHelpFunction)(MPI_Comm)    = 0;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSetHelpVersionFunctions"
 /*@C
    PetscSetHelpVersionFunctions - Sets functions that print help and version information
    before the PETSc help and version information is printed. Must call BEFORE PetscInitialize().
@@ -250,12 +242,10 @@ PetscErrorCode  PetscSetHelpVersionFunctions(PetscErrorCode (*help)(MPI_Comm),Pe
 }
 
 #if defined(PETSC_USE_LOG)
-extern PetscBool   PetscObjectsLog;
+PETSC_INTERN PetscBool   PetscObjectsLog;
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscOptionsCheckInitial_Private"
-PetscErrorCode  PetscOptionsCheckInitial_Private(void)
+PETSC_INTERN PetscErrorCode  PetscOptionsCheckInitial_Private(void)
 {
   char              string[64],mname[PETSC_MAX_PATH_LEN],*f;
   MPI_Comm          comm = PETSC_COMM_WORLD;
@@ -265,7 +255,7 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
   PetscInt          intensity;
   int               i;
   PetscMPIInt       rank;
-  char              version[256];
+  char              version[256],helpoptions[256];
 #if !defined(PETSC_HAVE_THREADSAFETY)
   PetscReal         logthreshold;
 #endif
@@ -273,9 +263,9 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
   PetscViewerFormat format;
   PetscBool         flg4 = PETSC_FALSE;
 #endif
-  
+
   PetscFunctionBegin;
-  ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
 
 #if !defined(PETSC_HAVE_THREADSAFETY)
   /*
@@ -301,6 +291,8 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
   if (flg3) {
     ierr = PetscMallocSetDumpLogThreshold((PetscLogDouble)logthreshold);CHKERRQ(ierr);
   }
+  ierr = PetscOptionsGetBool(NULL,NULL,"-malloc_coalesce",&flg1,&flg2);CHKERRQ(ierr);
+  if (flg2) {ierr = PetscMallocSetCoalesce(flg1);CHKERRQ(ierr);}
   flg1 = PETSC_FALSE;
   ierr = PetscOptionsGetBool(NULL,NULL,"-malloc_debug",&flg1,NULL);CHKERRQ(ierr);
   if (flg1) {
@@ -316,6 +308,10 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
     ierr = PetscMallocDebug(PETSC_TRUE);CHKERRQ(ierr);
   }
 #endif
+  flg1 = PETSC_FALSE;
+  ierr = PetscOptionsGetBool(NULL,NULL,"-malloc_hbw",&flg1,NULL);CHKERRQ(ierr);
+  /* ignore this option if malloc is already set */
+  if (flg1 && !petscsetmallocvisited) {ierr = PetscSetUseHBWMalloc_Private();CHKERRQ(ierr);}
 
   flg1 = PETSC_FALSE;
   ierr = PetscOptionsGetBool(NULL,NULL,"-malloc_info",&flg1,NULL);CHKERRQ(ierr);
@@ -342,7 +338,7 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
   */
   ierr = PetscOptionsHasName(NULL,NULL,"-v",&flg1);CHKERRQ(ierr);
   ierr = PetscOptionsHasName(NULL,NULL,"-version",&flg2);CHKERRQ(ierr);
-  ierr = PetscOptionsHasName(NULL,NULL,"-help",&flg3);CHKERRQ(ierr);
+  ierr = PetscOptionsHasHelp(NULL,&flg3);CHKERRQ(ierr);
   if (flg1 || flg2 || flg3) {
 
     /*
@@ -353,16 +349,14 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
     }
 
     ierr = PetscGetVersion(version,256);CHKERRQ(ierr);
-    ierr = (*PetscHelpPrintf)(comm,"--------------------------------------------\
-------------------------------\n");CHKERRQ(ierr);
+    ierr = (*PetscHelpPrintf)(comm,"--------------------------------------------------------------------------\n");CHKERRQ(ierr);
     ierr = (*PetscHelpPrintf)(comm,"%s\n",version);CHKERRQ(ierr);
     ierr = (*PetscHelpPrintf)(comm,"%s",PETSC_AUTHOR_INFO);CHKERRQ(ierr);
     ierr = (*PetscHelpPrintf)(comm,"See docs/changes/index.html for recent updates.\n");CHKERRQ(ierr);
     ierr = (*PetscHelpPrintf)(comm,"See docs/faq.html for problems.\n");CHKERRQ(ierr);
     ierr = (*PetscHelpPrintf)(comm,"See docs/manualpages/index.html for help. \n");CHKERRQ(ierr);
     ierr = (*PetscHelpPrintf)(comm,"Libraries linked from %s\n",PETSC_LIB_DIR);CHKERRQ(ierr);
-    ierr = (*PetscHelpPrintf)(comm,"--------------------------------------------\
-------------------------------\n");CHKERRQ(ierr);
+    ierr = (*PetscHelpPrintf)(comm,"--------------------------------------------------------------------------\n");CHKERRQ(ierr);
   }
 
   /*
@@ -374,13 +368,24 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
     }
   }
 
+  ierr = PetscOptionsGetString(NULL,NULL,"-help",helpoptions,sizeof(helpoptions),&flg1);CHKERRQ(ierr);
+  if (flg1) {
+    ierr = PetscStrcmp(helpoptions,"intro",&flg2);CHKERRQ(ierr);
+    if (flg2) {
+      ierr = PetscOptionsDestroyDefault();CHKERRQ(ierr);
+      ierr = PetscFreeMPIResources();CHKERRQ(ierr);
+      ierr = MPI_Finalize();CHKERRQ(ierr);
+      exit(0);
+    }
+  }
+
   /*
       Setup the error handling
   */
   flg1 = PETSC_FALSE;
   ierr = PetscOptionsGetBool(NULL,NULL,"-on_error_abort",&flg1,NULL);CHKERRQ(ierr);
   if (flg1) {
-    ierr = MPI_Comm_set_errhandler(PETSC_COMM_WORLD,MPI_ERRORS_ARE_FATAL);CHKERRQ(ierr);
+    ierr = MPI_Comm_set_errhandler(comm,MPI_ERRORS_ARE_FATAL);CHKERRQ(ierr);
     ierr = PetscPushErrorHandler(PetscAbortErrorHandler,0);CHKERRQ(ierr);
   }
   flg1 = PETSC_FALSE;
@@ -409,7 +414,7 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
     MPI_Errhandler err_handler;
 
     ierr = PetscSetDebuggerFromString(string);CHKERRQ(ierr);
-    ierr = MPI_Comm_create_errhandler((MPI_Handler_function*)Petsc_MPI_DebuggerOnError,&err_handler);CHKERRQ(ierr);
+    ierr = MPI_Comm_create_errhandler(Petsc_MPI_DebuggerOnError,&err_handler);CHKERRQ(ierr);
     ierr = MPI_Comm_set_errhandler(comm,err_handler);CHKERRQ(ierr);
     ierr = PetscPushErrorHandler(PetscAttachDebuggerErrorHandler,0);CHKERRQ(ierr);
   }
@@ -427,18 +432,18 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
        debugger has stated it is likely to receive a SIGUSR1
        and kill the program.
     */
-    ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
+    ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
     if (size > 2) {
       PetscMPIInt dummy = 0;
       MPI_Status  status;
       for (i=0; i<size; i++) {
         if (rank != i) {
-          ierr = MPI_Send(&dummy,1,MPI_INT,i,109,PETSC_COMM_WORLD);CHKERRQ(ierr);
+          ierr = MPI_Send(&dummy,1,MPI_INT,i,109,comm);CHKERRQ(ierr);
         }
       }
       for (i=0; i<size; i++) {
         if (rank != i) {
-          ierr = MPI_Recv(&dummy,1,MPI_INT,i,109,PETSC_COMM_WORLD,&status);CHKERRQ(ierr);
+          ierr = MPI_Recv(&dummy,1,MPI_INT,i,109,comm,&status);CHKERRQ(ierr);
         }
       }
     }
@@ -459,7 +464,7 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
       } else {
         ierr = PetscStopForDebugger();CHKERRQ(ierr);
       }
-      ierr = MPI_Comm_create_errhandler((MPI_Handler_function*)Petsc_MPI_AbortOnError,&err_handler);CHKERRQ(ierr);
+      ierr = MPI_Comm_create_errhandler(Petsc_MPI_AbortOnError,&err_handler);CHKERRQ(ierr);
       ierr = MPI_Comm_set_errhandler(comm,err_handler);CHKERRQ(ierr);
     }
     ierr = PetscFree(nodes);CHKERRQ(ierr);
@@ -492,6 +497,9 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
       ierr = PetscOpenHistoryFile(NULL,&petsc_history);CHKERRQ(ierr);
     }
   }
+
+  ierr = PetscOptionsGetBool(NULL,NULL,"-log_sync",&PetscLogSyncOn,NULL);CHKERRQ(ierr);
+
 #if defined(PETSC_HAVE_MPE)
   flg1 = PETSC_FALSE;
   ierr = PetscOptionsHasName(NULL,NULL,"-log_mpe",&flg1);CHKERRQ(ierr);
@@ -509,7 +517,7 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
     char name[PETSC_MAX_PATH_LEN],fname[PETSC_MAX_PATH_LEN];
     FILE *file;
     if (mname[0]) {
-      sprintf(name,"%s.%d",mname,rank);
+      PetscSNPrintf(name,PETSC_MAX_PATH_LEN,"%s.%d",mname,rank);
       ierr = PetscFixFilename(name,fname);CHKERRQ(ierr);
       file = fopen(fname,"w");
       if (!file) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Unable to open trace file: %s",fname);
@@ -517,14 +525,19 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
     ierr = PetscLogTraceBegin(file);CHKERRQ(ierr);
   }
 
-  ierr   = PetscOptionsGetViewer(PETSC_COMM_WORLD,NULL,"-log_view",NULL,&format,&flg4);CHKERRQ(ierr);
+  ierr = PetscOptionsGetViewer(comm,NULL,"-log_view",NULL,&format,&flg4);CHKERRQ(ierr);
   if (flg4) {
-    if (format == PETSC_VIEWER_ASCII_XML){
+    if (format == PETSC_VIEWER_ASCII_XML) {
       ierr = PetscLogNestedBegin();CHKERRQ(ierr);
     } else {
       ierr = PetscLogDefaultBegin();CHKERRQ(ierr);
     }
   }
+  if (flg4 && format == PETSC_VIEWER_ASCII_XML) {
+    PetscReal threshold = PetscRealConstant(0.01);
+    ierr = PetscOptionsGetReal(NULL,NULL,"-log_threshold",&threshold,&flg1);CHKERRQ(ierr);
+    if (flg1) {ierr = PetscLogSetThreshold((PetscLogDouble)threshold,NULL);CHKERRQ(ierr);}
+  }
 #endif
 
   ierr = PetscOptionsGetBool(NULL,NULL,"-saws_options",&PetscOptionsPublish,NULL);CHKERRQ(ierr);
@@ -542,12 +555,12 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
     for (device = 0; device < devCount; ++device) {
       err = cudaGetDeviceProperties(&prop, (int)device);
       if (err != cudaSuccess) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SYS,"error in cudaGetDeviceProperties %s",cudaGetErrorString(err));
-      ierr = PetscPrintf(PETSC_COMM_WORLD, "CUDA device %D: %s\n", device, prop.name);CHKERRQ(ierr);
+      ierr = PetscPrintf(comm, "CUDA device %D: %s\n", device, prop.name);CHKERRQ(ierr);
     }
   }
   if (!PetscCUDAInitialized) {
     PetscMPIInt size;
-    ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
+    ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
     if (size>1) {
       int         devCount;
       PetscInt    device;
@@ -567,7 +580,7 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
         if (err != cudaSuccess) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SYS,"error in cudaGetDeviceCount %s",cudaGetErrorString(err));
 
         /* next determine the rank and then set the device via a mod */
-        ierr   = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
+        ierr   = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
         device = rank % devCount;
         err    = cudaSetDevice((int)device);
         if (err != cudaSuccess) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SYS,"error in cudaSetDevice %s",cudaGetErrorString(err));
@@ -602,7 +615,7 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
   /*
        Print basic help message
   */
-  ierr = PetscOptionsHasName(NULL,NULL,"-help",&flg1);CHKERRQ(ierr);
+  ierr = PetscOptionsHasHelp(NULL,&flg1);CHKERRQ(ierr);
   if (flg1) {
     ierr = (*PetscHelpPrintf)(comm,"Options for all PETSc programs:\n");CHKERRQ(ierr);
     ierr = (*PetscHelpPrintf)(comm," -help: prints help method for each option\n");CHKERRQ(ierr);
@@ -630,7 +643,7 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
     ierr = (*PetscHelpPrintf)(comm," -malloc_info: prints total memory usage\n");CHKERRQ(ierr);
     ierr = (*PetscHelpPrintf)(comm," -malloc_log: keeps log of all memory allocations\n");CHKERRQ(ierr);
     ierr = (*PetscHelpPrintf)(comm," -malloc_debug: enables extended checking for memory corruption\n");CHKERRQ(ierr);
-    ierr = (*PetscHelpPrintf)(comm," -options_table: dump list of options inputted\n");CHKERRQ(ierr);
+    ierr = (*PetscHelpPrintf)(comm," -options_view: dump list of options inputted\n");CHKERRQ(ierr);
     ierr = (*PetscHelpPrintf)(comm," -options_left: dump list of unused options\n");CHKERRQ(ierr);
     ierr = (*PetscHelpPrintf)(comm," -options_left no: don't dump list of unused options\n");CHKERRQ(ierr);
     ierr = (*PetscHelpPrintf)(comm," -tmp tmpdir: alternative /tmp directory\n");CHKERRQ(ierr);
@@ -639,7 +652,7 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
     ierr = (*PetscHelpPrintf)(comm," -memory_view: print memory usage at end of run\n");CHKERRQ(ierr);
 #if defined(PETSC_USE_LOG)
     ierr = (*PetscHelpPrintf)(comm," -get_total_flops: total flops over all processors\n");CHKERRQ(ierr);
-    ierr = (*PetscHelpPrintf)(comm," -log[_summary _summary_python]: logging objects and events\n");CHKERRQ(ierr);
+    ierr = (*PetscHelpPrintf)(comm," -log_view [:filename:[format]]: logging objects and events\n");CHKERRQ(ierr);
     ierr = (*PetscHelpPrintf)(comm," -log_trace [filename]: prints trace of all PETSc calls\n");CHKERRQ(ierr);
 #if defined(PETSC_HAVE_MPE)
     ierr = (*PetscHelpPrintf)(comm," -log_mpe: Also create logfile viewable through Jumpshot\n");CHKERRQ(ierr);
@@ -675,23 +688,24 @@ PetscErrorCode  PetscOptionsCheckInitial_Private(void)
     }
   }
 
-#if defined(PETSC_HAVE_CUSP) || defined(PETSC_HAVE_VIENNACL) || defined(PETSC_HAVE_VECCUDA)
+#if defined(PETSC_HAVE_VIENNACL) || defined(PETSC_HAVE_VECCUDA)
   ierr = PetscOptionsHasName(NULL,NULL,"-log_summary",&flg3);CHKERRQ(ierr);
   if (!flg3) {
-  ierr = PetscOptionsHasName(NULL,NULL,"-log_view",&flg3);CHKERRQ(ierr);
+    ierr = PetscOptionsHasName(NULL,NULL,"-log_view",&flg3);CHKERRQ(ierr);
   }
 #endif
-#if defined(PETSC_HAVE_CUSP)
-  ierr = PetscOptionsGetBool(NULL,NULL,"-cusp_synchronize",&flg3,NULL);CHKERRQ(ierr);
-  PetscCUSPSynchronize = flg3;
-#elif defined(PETSC_HAVE_VIENNACL)
+#if defined(PETSC_HAVE_VIENNACL)
   ierr = PetscOptionsGetBool(NULL,NULL,"-viennacl_synchronize",&flg3,NULL);CHKERRQ(ierr);
   PetscViennaCLSynchronize = flg3;
-#elif defined(PETSC_HAVE_VECCUDA)
+#endif
+#if defined(PETSC_HAVE_VECCUDA)
   ierr = PetscOptionsGetBool(NULL,NULL,"-cuda_synchronize",&flg3,NULL);CHKERRQ(ierr);
   PetscCUDASynchronize = flg3;
 #endif
 
+#if defined(PETSC_HAVE_VIENNACL)
+  ierr = PetscViennaCLInit();CHKERRQ(ierr);
+#endif
+
   PetscFunctionReturn(0);
 }
-
