@@ -1,8 +1,6 @@
 
 #include <petsc/private/petscimpl.h>        /*I    "petscsys.h"   I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectGetName"
 /*@C
    PetscObjectGetName - Gets a string name associated with a PETSc object.
 
@@ -14,11 +12,14 @@
          PetscObjectGetName((PetscObject)mat,&name);
 -  name - the name associated with obj
 
+   Notes:
+    Calls PetscObjectName() if a name has not yet been provided to the object.
+
    Level: intermediate
 
    Concepts: object name
 
-.seealso: PetscObjectSetName()
+.seealso: PetscObjectSetName(), PetscObjectName()
 @*/
 PetscErrorCode  PetscObjectGetName(PetscObject obj,const char *name[])
 {
