@@ -2,8 +2,6 @@
 #include <petsc/private/petscimpl.h>        /*I    "petscsys.h"   I*/
 #include <petscviewer.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectSetName"
 /*@C
    PetscObjectSetName - Sets a string name associated with a PETSc object.
 
@@ -15,11 +13,13 @@
          PetscObjectSetName((PetscObject)mat,name);
 -  name - the name to give obj
 
+   Notes:
+    If this routine is not called then the object may end up being name by PetscObjectName().
    Level: advanced
 
    Concepts: object name^setting
 
-.seealso: PetscObjectGetName()
+.seealso: PetscObjectGetName(), PetscObjectName()
 @*/
 PetscErrorCode  PetscObjectSetName(PetscObject obj,const char name[])
 {
@@ -32,10 +32,8 @@ PetscErrorCode  PetscObjectSetName(PetscObject obj,const char name[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectPrintClassNamePrefixType"
 /*@C
-      PetscObjectPrintTypeNamePrefix - used in the XXXView() methods to display information about the class, name, prefix and type of an object
+      PetscObjectPrintClassNamePrefixType - used in the XXXView() methods to display information about the class, name, prefix and type of an object
 
    Input Parameters:
 +     obj - the PETSc object
@@ -43,13 +41,17 @@ PetscErrorCode  PetscObjectSetName(PetscObject obj,const char name[])
 
    Level: developer
 
-   Notes: If the viewer format is PETSC_VIEWER_ASCII_MATLAB then the information is printed after a % symbol
+   Notes:
+    If the viewer format is PETSC_VIEWER_ASCII_MATLAB then the information is printed after a % symbol
           so that MATLAB will treat it as a comment.
 
           If the viewer format is PETSC_VIEWER_ASCII_VTK*, PETSC_VIEWER_ASCII_LATEX, or
           PETSC_VIEWER_ASCII_MATRIXMARKET then don't print header information
           as these formats can't process it.
 
+   Developer Note: The flag donotPetscObjectPrintClassNamePrefixType is useful to prevent double printing of the information when recursion is used
+                   to actually print the object.
+
 .seealso: PetscObjectSetName(), PetscObjectName()
 
 @*/
@@ -63,22 +65,25 @@ PetscErrorCode PetscObjectPrintClassNamePrefixType(PetscObject obj,PetscViewer v
 
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&flg);CHKERRQ(ierr);
+  if (obj->donotPetscObjectPrintClassNamePrefixType) PetscFunctionReturn(0);
   if (!flg) PetscFunctionReturn(0);
 
   ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
-  if (format == PETSC_VIEWER_ASCII_VTK || format == PETSC_VIEWER_ASCII_VTK_CELL || format == PETSC_VIEWER_ASCII_VTK_COORDS || format == PETSC_VIEWER_ASCII_MATRIXMARKET || format == PETSC_VIEWER_ASCII_LATEX) PetscFunctionReturn(0);
+  if (format == PETSC_VIEWER_ASCII_VTK || format == PETSC_VIEWER_ASCII_VTK_CELL || format == PETSC_VIEWER_ASCII_VTK_COORDS || format == PETSC_VIEWER_ASCII_MATRIXMARKET || format == PETSC_VIEWER_ASCII_LATEX || format == PETSC_VIEWER_ASCII_GLVIS) PetscFunctionReturn(0);
 
   if (format == PETSC_VIEWER_ASCII_MATLAB) {ierr = PetscViewerASCIIPrintf(viewer,"%%");CHKERRQ(ierr);}
   ierr = PetscViewerASCIIPrintf(viewer,"%s Object:",obj->class_name);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
   if (obj->name) {
-    ierr = PetscViewerASCIIPrintf(viewer,"%s",obj->name);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer," %s",obj->name);CHKERRQ(ierr);
   }
   if (obj->prefix) {
-    ierr = PetscViewerASCIIPrintf(viewer,"(%s)",obj->prefix);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer," (%s)",obj->prefix);CHKERRQ(ierr);
   }
   ierr = PetscObjectGetComm(obj,&comm);CHKERRQ(ierr);
   ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
   ierr = PetscViewerASCIIPrintf(viewer," %d MPI processes\n",size);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
   if (format == PETSC_VIEWER_ASCII_MATLAB) {ierr = PetscViewerASCIIPrintf(viewer,"%%");CHKERRQ(ierr);}
   if (obj->type_name) {
     ierr = PetscViewerASCIIPrintf(viewer,"  type: %s\n",obj->type_name);CHKERRQ(ierr);
@@ -88,8 +93,6 @@ PetscErrorCode PetscObjectPrintClassNamePrefixType(PetscObject obj,PetscViewer v
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectName"
 /*@C
    PetscObjectName - Gives an object a name if it does not have one
 
@@ -104,7 +107,8 @@ PetscErrorCode PetscObjectPrintClassNamePrefixType(PetscObject obj,PetscViewer v
 
    Concepts: object name^setting default
 
-   Notes: This is used in a small number of places when an object NEEDS a name, for example when it is saved to MATLAB with that variable name.
+   Notes:
+    This is used in a small number of places when an object NEEDS a name, for example when it is saved to MATLAB with that variable name.
           Use PetscObjectSetName() to set the name of an object to what you want. The SAWs viewer requires that no two published objects
           share the same name.
 
@@ -125,7 +129,7 @@ PetscErrorCode  PetscObjectName(PetscObject obj)
   PetscValidHeader(obj,1);
   if (!obj->name) {
     union {MPI_Comm comm; void *ptr; char raw[sizeof(MPI_Comm)]; } ucomm;
-    ierr = MPI_Attr_get(obj->comm,Petsc_Counter_keyval,(void*)&counter,&flg);CHKERRQ(ierr);
+    ierr = MPI_Comm_get_attr(obj->comm,Petsc_Counter_keyval,(void*)&counter,&flg);CHKERRQ(ierr);
     if (!flg) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_CORRUPT,"Bad MPI communicator supplied; must be a PETSc communicator");
     ucomm.ptr = NULL;
     ucomm.comm = obj->comm;
@@ -141,8 +145,6 @@ PetscErrorCode  PetscObjectName(PetscObject obj)
 
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscObjectChangeTypeName"
 PetscErrorCode  PetscObjectChangeTypeName(PetscObject obj,const char type_name[])
 {
   PetscErrorCode ierr;
