@@ -7,20 +7,31 @@
 
 const char *const PetscSubcommTypes[] = {"GENERAL","CONTIGUOUS","INTERLACED","PetscSubcommType","PETSC_SUBCOMM_",0};
 
-extern PetscErrorCode PetscSubcommCreate_contiguous(PetscSubcomm);
-extern PetscErrorCode PetscSubcommCreate_interlaced(PetscSubcomm);
-#undef __FUNCT__
-#define __FUNCT__ "PetscSubcommSetFromOptions"
+static PetscErrorCode PetscSubcommCreate_contiguous(PetscSubcomm);
+static PetscErrorCode PetscSubcommCreate_interlaced(PetscSubcomm);
+
+/*@C
+   PetscSubcommSetFromOptions - Allows setting options from a PetscSubcomm
+
+   Collective on PetscSubcomm
+
+   Input Parameter:
+.  psubcomm - PetscSubcomm context
+
+   Level: beginner
+
+@*/
 PetscErrorCode PetscSubcommSetFromOptions(PetscSubcomm psubcomm)
 {
-  PetscErrorCode ierr;
+  PetscErrorCode   ierr;
   PetscSubcommType type;
-  PetscBool      flg;
+  PetscBool        flg;
 
   PetscFunctionBegin;
   if (!psubcomm) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_NULL,"Must call PetscSubcommCreate firt");
-  type = psubcomm->type;
-  ierr = PetscOptionsGetEnum(NULL,"-psubcomm_type",PetscSubcommTypes,(PetscEnum*)&type,&flg);CHKERRQ(ierr);
+
+  ierr = PetscOptionsBegin(psubcomm->parent,psubcomm->subcommprefix,"Options for PetscSubcomm",NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsEnum("-psubcomm_type",NULL,NULL,PetscSubcommTypes,(PetscEnum)psubcomm->type,(PetscEnum*)&type,&flg);CHKERRQ(ierr);
   if (flg && psubcomm->type != type) {
     /* free old structures */
     ierr = PetscCommDestroy(&(psubcomm)->dupparent);CHKERRQ(ierr);
@@ -40,15 +51,53 @@ PetscErrorCode PetscSubcommSetFromOptions(PetscSubcomm psubcomm)
     }
   }
 
-  ierr = PetscOptionsHasName(NULL, "-psubcomm_view", &flg);CHKERRQ(ierr);
+  ierr = PetscOptionsName("-psubcomm_view","Triggers display of PetscSubcomm context","PetscSubcommView",&flg);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscSubcommView(psubcomm,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);
   }
+  ierr = PetscOptionsEnd();CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@C
+  PetscSubcommSetOptionsPrefix - Sets the prefix used for searching for all
+  PetscSubcomm items in the options database.
+
+  Logically collective on PetscSubcomm.
+
+  Level: Intermediate
+
+  Input Parameters:
++   psubcomm - PetscSubcomm context
+-   prefix - the prefix to prepend all PetscSubcomm item names with.
+
+@*/
+PetscErrorCode PetscSubcommSetOptionsPrefix(PetscSubcomm psubcomm,const char pre[])
+{
+  PetscErrorCode   ierr;
+
+  PetscFunctionBegin;
+   if (!pre) {
+    ierr = PetscFree(psubcomm->subcommprefix);CHKERRQ(ierr);
+  } else {
+    if (pre[0] == '-') SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Options prefix should not begin with a hypen");
+    ierr = PetscFree(psubcomm->subcommprefix);CHKERRQ(ierr);
+    ierr = PetscStrallocpy(pre,&(psubcomm->subcommprefix));CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSubcommView"
+/*@C
+   PetscSubcommView - Views a PetscSubcomm of values as either ASCII text or a binary file
+
+   Collective on PetscSubcomm
+
+   Input Parameter:
++  psubcomm - PetscSubcomm context
+-  viewer - location to view the values
+
+   Level: beginner
+@*/
 PetscErrorCode PetscSubcommView(PetscSubcomm psubcomm,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -76,8 +125,6 @@ PetscErrorCode PetscSubcommView(PetscSubcomm psubcomm,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSubcommSetNumber"
 /*@C
   PetscSubcommSetNumber - Set total number of subcommunicators.
 
@@ -109,8 +156,6 @@ PetscErrorCode  PetscSubcommSetNumber(PetscSubcomm psubcomm,PetscInt nsubcomm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSubcommSetType"
 /*@C
   PetscSubcommSetType - Set type of subcommunicators.
 
@@ -142,8 +187,6 @@ PetscErrorCode  PetscSubcommSetType(PetscSubcomm psubcomm,PetscSubcommType subco
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSubcommSetTypeGeneral"
 /*@C
   PetscSubcommSetTypeGeneral - Set a PetscSubcomm from user's specifications
 
@@ -213,8 +256,18 @@ PetscErrorCode PetscSubcommSetTypeGeneral(PetscSubcomm psubcomm,PetscMPIInt colo
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSubcommDestroy"
+/*@C
+  PetscSubcommDestroy - Destroys a PetscSubcomm object
+
+   Collective on PetscSubcomm
+
+   Input Parameter:
+   .  psubcomm - the PetscSubcomm context
+
+   Level: advanced
+
+.seealso: PetscSubcommCreate(),PetscSubcommSetType()
+@*/
 PetscErrorCode  PetscSubcommDestroy(PetscSubcomm *psubcomm)
 {
   PetscErrorCode ierr;
@@ -224,12 +277,11 @@ PetscErrorCode  PetscSubcommDestroy(PetscSubcomm *psubcomm)
   ierr = PetscCommDestroy(&(*psubcomm)->dupparent);CHKERRQ(ierr);
   ierr = PetscCommDestroy(&(*psubcomm)->child);CHKERRQ(ierr);
   ierr = PetscFree((*psubcomm)->subsize);CHKERRQ(ierr);
+  if ((*psubcomm)->subcommprefix) { ierr = PetscFree((*psubcomm)->subcommprefix);CHKERRQ(ierr); }
   ierr = PetscFree((*psubcomm));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSubcommCreate"
 /*@C
   PetscSubcommCreate - Create a PetscSubcomm context.
 
@@ -269,9 +321,7 @@ PetscErrorCode  PetscSubcommCreate(MPI_Comm comm,PetscSubcomm *psubcomm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSubcommCreate_contiguous"
-PetscErrorCode PetscSubcommCreate_contiguous(PetscSubcomm psubcomm)
+static PetscErrorCode PetscSubcommCreate_contiguous(PetscSubcomm psubcomm)
 {
   PetscErrorCode ierr;
   PetscMPIInt    rank,size,*subsize,duprank=-1,subrank=-1;
@@ -318,8 +368,6 @@ PetscErrorCode PetscSubcommCreate_contiguous(PetscSubcomm psubcomm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSubcommCreate_interlaced"
 /*
    Note:
    In PCREDUNDANT, to avoid data scattering from subcomm back to original comm, we create subcommunicators
@@ -345,7 +393,7 @@ PetscErrorCode PetscSubcommCreate_contiguous(PetscSubcomm psubcomm)
                     subcomm[0] subcomm[1]  subcomm[2]
 */
 
-PetscErrorCode PetscSubcommCreate_interlaced(PetscSubcomm psubcomm)
+static PetscErrorCode PetscSubcommCreate_interlaced(PetscSubcomm psubcomm)
 {
   PetscErrorCode ierr;
   PetscMPIInt    rank,size,*subsize,duprank,subrank;
