@@ -1,4 +1,4 @@
-#include <petsc-private/fortranimpl.h>
+#include <petsc/private/fortranimpl.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define petscpythoninitialize_ PETSCPYTHONINITIALIZE
@@ -8,7 +8,7 @@
 #define petscpythonfinalize_   petscpythonfinalize
 #endif
 
-PETSC_EXTERN void PETSC_STDCALL petscpythoninitialize_(CHAR n1 PETSC_MIXED_LEN(l1),CHAR n2 PETSC_MIXED_LEN(l2), PetscErrorCode *ierr PETSC_END_LEN(l1) PETSC_END_LEN(l2))
+PETSC_EXTERN void PETSC_STDCALL petscpythoninitialize_(char* n1 PETSC_MIXED_LEN(l1),char* n2 PETSC_MIXED_LEN(l2), PetscErrorCode *ierr PETSC_END_LEN(l1) PETSC_END_LEN(l2))
 {
   char *t1,*t2;
   FIXCHAR(n1,l1,t1);
