@@ -6,8 +6,6 @@
 #define PETSC_PYTHON_EXE "python"
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscPythonFindExecutable"
 static PetscErrorCode PetscPythonFindExecutable(char pythonexe[PETSC_MAX_PATH_LEN])
 {
   PetscBool      flag;
@@ -16,22 +14,50 @@ static PetscErrorCode PetscPythonFindExecutable(char pythonexe[PETSC_MAX_PATH_LE
   PetscFunctionBegin;
   /* get the path for the Python interpreter executable */
   ierr = PetscStrncpy(pythonexe,PETSC_PYTHON_EXE,PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
-  ierr = PetscOptionsGetString(NULL,"-python",pythonexe,PETSC_MAX_PATH_LEN,&flag);CHKERRQ(ierr);
+  ierr = PetscOptionsGetString(NULL,NULL,"-python",pythonexe,PETSC_MAX_PATH_LEN,&flag);CHKERRQ(ierr);
   if (!flag || pythonexe[0]==0) {
     ierr = PetscStrncpy(pythonexe,PETSC_PYTHON_EXE,PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscPythonFindLibrary"
-static PetscErrorCode PetscPythonFindLibrary(char pythonexe[PETSC_MAX_PATH_LEN],char pythonlib[PETSC_MAX_PATH_LEN])
+/*
+    Python does not appear to have a universal way to indicate the location of Python dynamic library so try several possibilities
+*/
+static PetscErrorCode PetscPythonFindLibraryName(const char pythonexe[PETSC_MAX_PATH_LEN],const char attempt[PETSC_MAX_PATH_LEN],char pythonlib[PETSC_MAX_PATH_LEN],PetscBool *found)
 {
-  const char     cmdline[] = "-c 'import sys; print(sys.exec_prefix); print(sys.version[:3])'";
-  char           command[PETSC_MAX_PATH_LEN+1+sizeof(cmdline)+1];
-  char           prefix[PETSC_MAX_PATH_LEN],version[8],sep[2]={PETSC_DIR_SEPARATOR, 0},*eol;
+  char           command[2*PETSC_MAX_PATH_LEN];
   FILE           *fp = NULL;
-  char           path[PETSC_MAX_PATH_LEN+1];
+  char           *eol;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  /* call Python to find out the name of the Python dynamic library */
+  ierr = PetscStrncpy(command,pythonexe,PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
+  ierr = PetscStrcat(command," ");CHKERRQ(ierr);
+  ierr = PetscStrcat(command,attempt);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_POPEN)
+  ierr = PetscPOpen(PETSC_COMM_SELF,NULL,command,"r",&fp);CHKERRQ(ierr);
+  if (!fgets(pythonlib,PETSC_MAX_PATH_LEN,fp)) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Python: bad output from executable: %s",pythonexe);
+  ierr = PetscPClose(PETSC_COMM_SELF,fp);CHKERRQ(ierr);
+#else
+  SETERRQ(PETSC_COMM_SELF,1,"Python: Aborted due to missing popen()");
+#endif
+  /* remove newlines */
+  ierr = PetscStrchr(pythonlib,'\n',&eol);CHKERRQ(ierr);
+  if (eol) eol[0] = 0;
+  ierr = PetscTestFile(pythonlib,'r',found);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+
+static PetscErrorCode PetscPythonFindLibrary(const char pythonexe[PETSC_MAX_PATH_LEN],char pythonlib[PETSC_MAX_PATH_LEN])
+{
+  const char     cmdline1[] = "-c 'import os;from distutils import sysconfig; print(os.path.join(sysconfig.get_config_var(\"LIBDIR\"),sysconfig.get_config_var(\"LDLIBRARY\")))'";
+  const char     cmdline2[] = "-c 'import os;from distutils import sysconfig; import sys;print(os.path.join(sysconfig.get_config_var(\"LIBDIR\"),\"libpython\"+sys.version[:3]+\".dylib\"))'";
+  const char     cmdline3[] = "-c 'import os;from distutils import sysconfig; print(os.path.join(sysconfig.get_config_var(\"LIBPL\"),sysconfig.get_config_var(\"LDLIBRARY\")))'";
+  const char     cmdline4[] = "-c 'from distutils import sysconfig; print(sysconfig.get_config_var(\"LIBPYTHON\"))'";
+
   PetscBool      found = PETSC_FALSE;
   PetscErrorCode ierr;
 
@@ -41,53 +67,17 @@ static PetscErrorCode PetscPythonFindLibrary(char pythonexe[PETSC_MAX_PATH_LEN],
   PetscFunctionReturn(0);
 #endif
 
-  /* call Python to find out the name of the Python dynamic library */
-  ierr = PetscStrncpy(command,pythonexe,PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
-  ierr = PetscStrcat(command," ");CHKERRQ(ierr);
-  ierr = PetscStrcat(command,cmdline);CHKERRQ(ierr);
-#if defined(PETSC_HAVE_POPEN)
-  ierr = PetscPOpen(PETSC_COMM_SELF,NULL,command,"r",&fp);CHKERRQ(ierr);
-  if (!fgets(prefix,sizeof(prefix),fp)) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Python: bad output from executable: %s",pythonexe);
-  if (!fgets(version,sizeof(version),fp)) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Python: bad output from executable: %s",pythonexe);
-  ierr = PetscPClose(PETSC_COMM_SELF,fp,NULL);CHKERRQ(ierr);
-#else
-  SETERRQ(PETSC_COMM_SELF,1,"Python: Aborted due to missing popen()");
-#endif
-  /* remove newlines */
-  ierr = PetscStrchr(prefix,'\n',&eol);CHKERRQ(ierr);
-  if (eol) eol[0] = 0;
-  ierr = PetscStrchr(version,'\n',&eol);CHKERRQ(ierr);
-  if (eol) eol[0] = 0;
-
-  /* test for $prefix/lib64/libpythonX.X[.so]*/
-  ierr = PetscStrcpy(pythonlib,prefix);CHKERRQ(ierr);
-  ierr = PetscStrcat(pythonlib,sep);CHKERRQ(ierr);
-  ierr = PetscStrcat(pythonlib,"lib64");CHKERRQ(ierr);
-  ierr = PetscTestDirectory(pythonlib,'r',&found);CHKERRQ(ierr);
-  if (found) {
-    ierr = PetscStrcat(pythonlib,sep);CHKERRQ(ierr);
-    ierr = PetscStrcat(pythonlib,"libpython");CHKERRQ(ierr);
-    ierr = PetscStrcat(pythonlib,version);CHKERRQ(ierr);
-    ierr = PetscDLLibraryRetrieve(PETSC_COMM_SELF,pythonlib,path,PETSC_MAX_PATH_LEN,&found);CHKERRQ(ierr);
-    if (found) PetscFunctionReturn(0);
+  ierr = PetscPythonFindLibraryName(pythonexe,cmdline1,pythonlib,&found);CHKERRQ(ierr);
+  if (!found) {
+    ierr = PetscPythonFindLibraryName(pythonexe,cmdline2,pythonlib,&found);CHKERRQ(ierr);
   }
-
-  /* test for $prefix/lib/libpythonX.X[.so]*/
-  ierr = PetscStrcpy(pythonlib,prefix);CHKERRQ(ierr);
-  ierr = PetscStrcat(pythonlib,sep);CHKERRQ(ierr);
-  ierr = PetscStrcat(pythonlib,"lib");CHKERRQ(ierr);
-  ierr = PetscTestDirectory(pythonlib,'r',&found);CHKERRQ(ierr);
-  if (found) {
-    ierr = PetscStrcat(pythonlib,sep);CHKERRQ(ierr);
-    ierr = PetscStrcat(pythonlib,"libpython");CHKERRQ(ierr);
-    ierr = PetscStrcat(pythonlib,version);CHKERRQ(ierr);
-    ierr = PetscDLLibraryRetrieve(PETSC_COMM_SELF,pythonlib,path,PETSC_MAX_PATH_LEN,&found);CHKERRQ(ierr);
-    if (found) PetscFunctionReturn(0);
+  if (!found) {
+    ierr = PetscPythonFindLibraryName(pythonexe,cmdline3,pythonlib,&found);CHKERRQ(ierr);
   }
-
-  /* nothing good found */
-  ierr = PetscMemzero(pythonlib,PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
-  ierr = PetscInfo(0,"Python dynamic library not found\n");CHKERRQ(ierr);
+  if (!found) {
+    ierr = PetscPythonFindLibraryName(pythonexe,cmdline4,pythonlib,&found);CHKERRQ(ierr);
+  }
+  ierr = PetscInfo2(0,"Python library  %s found %d\n",pythonlib,found);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -103,7 +93,7 @@ static int       (*Py_IsInitialized)(void);
 static void      (*Py_InitializeEx)(int);
 static void      (*Py_Finalize)(void);
 
-static void      (*PySys_SetArgv)(int,char**);
+static void      (*PySys_SetArgv)(int,void*);
 static PyObject* (*PySys_GetObject)(const char*);
 static PyObject* (*PyObject_CallMethod)(PyObject*,const char*, const char*, ...);
 static PyObject* (*PyImport_ImportModule)(const char*);
@@ -126,8 +116,6 @@ static void      (*PyErr_Restore)(PyObject*,PyObject*,PyObject*);
 #define PetscDLPyLibClose(comm) \
   do { } while (0)
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscPythonLoadLibrary"
 static PetscErrorCode PetscPythonLoadLibrary(const char pythonlib[])
 {
   PetscErrorCode ierr;
@@ -155,12 +143,12 @@ static PetscErrorCode PetscPythonLoadLibrary(const char pythonlib[])
   ierr = PetscDLPyLibSym("PyErr_Display",            &PyErr_Display           );CHKERRQ(ierr);
   ierr = PetscDLPyLibSym("PyErr_Restore",            &PyErr_Restore           );CHKERRQ(ierr);
   /* XXX TODO: check that ALL symbols were there !!! */
-  if (!Py_None)          SETERRQ(PETSC_COMM_SELF,1,"Python: failed to load symbols from dynamic library");
-  if (!Py_GetVersion)    SETERRQ(PETSC_COMM_SELF,1,"Python: failed to load symbols from dynamic library");
-  if (!Py_IsInitialized) SETERRQ(PETSC_COMM_SELF,1,"Python: failed to load symbols from dynamic library");
-  if (!Py_InitializeEx)  SETERRQ(PETSC_COMM_SELF,1,"Python: failed to load symbols from dynamic library");
-  if (!Py_Finalize)      SETERRQ(PETSC_COMM_SELF,1,"Python: failed to load symbols from dynamic library");
-  ierr = PetscInfo(0,"Python: all required symbols loaded from Python dynamic library\n");CHKERRQ(ierr);
+  if (!Py_None)          SETERRQ1(PETSC_COMM_SELF,1,"Python: failed to load symbols from Python dynamic library %s",pythonlib);
+  if (!Py_GetVersion)    SETERRQ1(PETSC_COMM_SELF,1,"Python: failed to load symbols from Python dynamic library %s",pythonlib);
+  if (!Py_IsInitialized) SETERRQ1(PETSC_COMM_SELF,1,"Python: failed to load symbols from Python dynamic library %s",pythonlib);
+  if (!Py_InitializeEx)  SETERRQ1(PETSC_COMM_SELF,1,"Python: failed to load symbols from Python dynamic library %s",pythonlib);
+  if (!Py_Finalize)      SETERRQ1(PETSC_COMM_SELF,1,"Python: failed to load symbols from Python dynamic library %s",pythonlib);
+  ierr = PetscInfo1(0,"Python: all required symbols loaded from Python dynamic library %s\n",pythonlib);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -170,8 +158,6 @@ static char      PetscPythonExe[PETSC_MAX_PATH_LEN] = { 0 };
 static char      PetscPythonLib[PETSC_MAX_PATH_LEN] = { 0 };
 static PetscBool PetscBeganPython = PETSC_FALSE;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscPythonFinalize"
 /*@C
   PetscPythonFinalize - Finalize Python.
 
@@ -187,8 +173,6 @@ PetscErrorCode  PetscPythonFinalize(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscPythonInitialize"
 /*@C
   PetscPythonInitialize - Initialize Python and import petsc4py.
 
@@ -235,13 +219,12 @@ PetscErrorCode  PetscPythonInitialize(const char pyexe[],const char pylib[])
     /*  build 'sys.argv' list */
     py_version = Py_GetVersion();
     if (py_version[0] == '2') {
-      int argc = 0; char **argv = 0;
-      ierr = PetscGetArgs(&argc,&argv);CHKERRQ(ierr);
+      int argc = 0; char *argv[1] = {NULL};
       PySys_SetArgv(argc,argv);
     }
     if (py_version[0] == '3') {
-      /* XXX 'argv' is type 'wchar_t**' */
-      PySys_SetArgv(0,NULL);
+      int argc = 0; wchar_t *argv[1] = {NULL};
+      PySys_SetArgv(argc,argv);
     }
     /* add PETSC_LIB_DIR in front of 'sys.path' */
     sys_path = PySys_GetObject("path");
@@ -252,7 +235,6 @@ PetscErrorCode  PetscPythonInitialize(const char pyexe[],const char pylib[])
     /* register finalizer */
     if (!registered) {
       ierr = PetscRegisterFinalize(PetscPythonFinalize);CHKERRQ(ierr);
-
       registered = PETSC_TRUE;
     }
     PetscBeganPython = PETSC_TRUE;
@@ -270,8 +252,6 @@ PetscErrorCode  PetscPythonInitialize(const char pyexe[],const char pylib[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscPythonPrintError"
 /*@C
   PetscPythonPrintError - Print Python errors.
 
@@ -301,8 +281,6 @@ PetscErrorCode  PetscPythonPrintError(void)
 PETSC_EXTERN PetscErrorCode (*PetscPythonMonitorSet_C)(PetscObject,const char[]);
 PetscErrorCode (*PetscPythonMonitorSet_C)(PetscObject,const char[]) = NULL;
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscPythonMonitorSet"
 /*@C
   PetscPythonMonitorSet - Set Python monitor
 
@@ -318,9 +296,9 @@ PetscErrorCode PetscPythonMonitorSet(PetscObject obj, const char url[])
   PetscFunctionBegin;
   PetscValidHeader(obj,1);
   PetscValidCharPointer(url,2);
-  if (PetscPythonMonitorSet_C == NULL) {
+  if (!PetscPythonMonitorSet_C) {
     ierr = PetscPythonInitialize(NULL,NULL);CHKERRQ(ierr);
-    if (PetscPythonMonitorSet_C == NULL) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_LIB,"Couldn't initialize Python support for monitors");
+    if (!PetscPythonMonitorSet_C) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_LIB,"Couldn't initialize Python support for monitors");
   }
   ierr = PetscPythonMonitorSet_C(obj,url);CHKERRQ(ierr);
   PetscFunctionReturn(0);
