@@ -1,4 +1,4 @@
-#define PETSC_DESIRE_FEATURE_TEST_MACROS /* for getdomainname */
+#define PETSC_DESIRE_FEATURE_TEST_MACROS /* for getdomainname() */
 /*
       Code for manipulating files.
 */
@@ -19,8 +19,6 @@
 #include <netdb.h>
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscGetHostName"
 /*@C
     PetscGetHostName - Returns the name of the host. This attempts to
     return the entire Internet name. It may not return the same name
@@ -80,6 +78,7 @@ PetscErrorCode  PetscGetHostName(char name[],size_t nlen)
     ierr = PetscStrlen(name,&l);CHKERRQ(ierr);
     if (l == nlen-1) PetscFunctionReturn(0);
     name[l++] = '.';
+    name[l]   = 0;
 #if defined(PETSC_HAVE_SYSINFO_3ARG)
     sysinfo(SI_SRPC_DOMAIN,name+l,nlen-l);
 #elif defined(PETSC_HAVE_GETDOMAINNAME)
