@@ -2,7 +2,7 @@
 /*
    This file contains routines for sorting integers. Values are sorted in place.
  */
-#include <petsc-private/petscimpl.h>                /*I  "petscsys.h"  I*/
+#include <petsc/private/petscimpl.h>                /*I  "petscsys.h"  I*/
 
 #define SWAP(a,b,t) {t=a;a=b;b=t;}
 
@@ -19,8 +19,6 @@
 
 /* -----------------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSortInt_Private"
 /*
    A simple version of quicksort; taken from Kernighan and Ritchie, page 87.
    Assumes 0 origin for v, number of elements = right+1 (right is index of
@@ -50,8 +48,6 @@ static void PetscSortInt_Private(PetscInt *v,PetscInt right)
   PetscSortInt_Private(v+j+1,right-(j+1));
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSortInt"
 /*@
    PetscSortInt - Sorts an array of integers in place in increasing order.
 
@@ -86,8 +82,39 @@ PetscErrorCode  PetscSortInt(PetscInt n,PetscInt i[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSortRemoveDupsInt"
+/*@
+   PetscSortedRemoveDupsInt - Removes all duplicate entries of a sorted input array
+
+   Not Collective
+
+   Input Parameters:
++  n  - number of values
+-  ii  - sorted array of integers
+
+   Output Parameter:
+.  n - number of non-redundant values
+
+   Level: intermediate
+
+   Concepts: sorting^ints
+
+.seealso: PetscSortInt()
+@*/
+PetscErrorCode  PetscSortedRemoveDupsInt(PetscInt *n,PetscInt ii[])
+{
+  PetscInt i,s = 0,N = *n, b = 0;
+
+  PetscFunctionBegin;
+  for (i=0; i<N-1; i++) {
+    if (PetscUnlikely(ii[b+s+1] < ii[b])) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Input array is not sorted");
+    if (ii[b+s+1] != ii[b]) {
+      ii[b+1] = ii[b+s+1]; b++;
+    } else s++;
+  }
+  *n = N - s;
+  PetscFunctionReturn(0);
+}
+
 /*@
    PetscSortRemoveDupsInt - Sorts an array of integers in place in increasing order removes all duplicate entries
 
@@ -104,26 +131,18 @@ PetscErrorCode  PetscSortInt(PetscInt n,PetscInt i[])
 
    Concepts: sorting^ints
 
-.seealso: PetscSortReal(), PetscSortIntWithPermutation(), PetscSortInt()
+.seealso: PetscSortReal(), PetscSortIntWithPermutation(), PetscSortInt(), PetscSortedRemoveDupsInt()
 @*/
 PetscErrorCode  PetscSortRemoveDupsInt(PetscInt *n,PetscInt ii[])
 {
   PetscErrorCode ierr;
-  PetscInt       i,s = 0,N = *n, b = 0;
 
   PetscFunctionBegin;
-  ierr = PetscSortInt(N,ii);CHKERRQ(ierr);
-  for (i=0; i<N-1; i++) {
-    if (ii[b+s+1] != ii[b]) {
-      ii[b+1] = ii[b+s+1]; b++;
-    } else s++;
-  }
-  *n = N - s;
+  ierr = PetscSortInt(*n,ii);CHKERRQ(ierr);
+  ierr = PetscSortedRemoveDupsInt(n,ii);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscFindInt"
 /*@
   PetscFindInt - Finds integer in a sorted array of integers
 
@@ -160,12 +179,45 @@ PetscErrorCode PetscFindInt(PetscInt key, PetscInt n, const PetscInt ii[], Petsc
   PetscFunctionReturn(0);
 }
 
+/*@
+  PetscFindMPIInt - Finds MPI integer in a sorted array of integers
+
+   Not Collective
+
+   Input Parameters:
++  key - the integer to locate
+.  n   - number of values in the array
+-  ii  - array of integers
+
+   Output Parameter:
+.  loc - the location if found, otherwise -(slot+1) where slot is the place the value would go
+
+   Level: intermediate
+
+   Concepts: sorting^ints
+
+.seealso: PetscSortInt(), PetscSortIntWithArray(), PetscSortRemoveDupsInt()
+@*/
+PetscErrorCode PetscFindMPIInt(PetscMPIInt key, PetscInt n, const PetscMPIInt ii[], PetscInt *loc)
+{
+  PetscInt lo = 0,hi = n;
+
+  PetscFunctionBegin;
+  PetscValidPointer(loc,4);
+  if (!n) {*loc = -1; PetscFunctionReturn(0);}
+  PetscValidPointer(ii,3);
+  while (hi - lo > 1) {
+    PetscInt mid = lo + (hi - lo)/2;
+    if (key < ii[mid]) hi = mid;
+    else               lo = mid;
+  }
+  *loc = key == ii[lo] ? lo : -(lo + (key > ii[lo]) + 1);
+  PetscFunctionReturn(0);
+}
 
 /* -----------------------------------------------------------------------*/
 #define SWAP2(a,b,c,d,t) {t=a;a=b;b=t;t=c;c=d;d=t;}
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSortIntWithArray_Private"
 /*
    A simple version of quicksort; taken from Kernighan and Ritchie, page 87.
    Assumes 0 origin for v, number of elements = right+1 (right is index of
@@ -195,8 +247,6 @@ static PetscErrorCode PetscSortIntWithArray_Private(PetscInt *v,PetscInt *V,Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSortIntWithArray"
 /*@
    PetscSortIntWithArray - Sorts an array of integers in place in increasing order;
        changes a second array to match the sorted first array.
@@ -239,8 +289,6 @@ PetscErrorCode  PetscSortIntWithArray(PetscInt n,PetscInt i[],PetscInt Ii[])
 
 #define SWAP3(a,b,c,d,e,f,t) {t=a;a=b;b=t;t=c;c=d;d=t;t=e;e=f;f=t;}
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSortIntWithArrayPair_Private"
 /*
    A simple version of quicksort; taken from Kernighan and Ritchie, page 87.
    Assumes 0 origin for v, number of elements = right+1 (right is index of
@@ -270,8 +318,6 @@ static PetscErrorCode PetscSortIntWithArrayPair_Private(PetscInt *L,PetscInt *J,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSortIntWithArrayPair"
 /*@
    PetscSortIntWithArrayPair - Sorts an array of integers in place in increasing order;
        changes a pair of integer arrays to match the sorted first array.
@@ -290,7 +336,7 @@ static PetscErrorCode PetscSortIntWithArrayPair_Private(PetscInt *L,PetscInt *J,
 
 .seealso: PetscSortReal(), PetscSortIntPermutation(), PetscSortIntWithArray()
 @*/
-PetscErrorCode  PetscSortIntWithArrayPair(PetscInt n,PetscInt *L,PetscInt *J, PetscInt *K)
+PetscErrorCode  PetscSortIntWithArrayPair(PetscInt n,PetscInt L[],PetscInt J[], PetscInt K[])
 {
   PetscErrorCode ierr;
   PetscInt       j,k,tmp,ik;
@@ -312,8 +358,6 @@ PetscErrorCode  PetscSortIntWithArrayPair(PetscInt n,PetscInt *L,PetscInt *J, Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSortMPIInt_Private"
 /*
    A simple version of quicksort; taken from Kernighan and Ritchie, page 87.
    Assumes 0 origin for v, number of elements = right+1 (right is index of
@@ -344,8 +388,6 @@ static void PetscSortMPIInt_Private(PetscMPIInt *v,PetscInt right)
   PetscSortMPIInt_Private(v+j+1,right-(j+1));
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSortMPIInt"
 /*@
    PetscSortMPIInt - Sorts an array of MPI integers in place in increasing order.
 
@@ -381,8 +423,6 @@ PetscErrorCode  PetscSortMPIInt(PetscInt n,PetscMPIInt i[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSortRemoveDupsMPIInt"
 /*@
    PetscSortRemoveDupsMPIInt - Sorts an array of MPI integers in place in increasing order removes all duplicate entries
 
@@ -417,8 +457,6 @@ PetscErrorCode  PetscSortRemoveDupsMPIInt(PetscInt *n,PetscMPIInt ii[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSortMPIIntWithArray_Private"
 /*
    A simple version of quicksort; taken from Kernighan and Ritchie, page 87.
    Assumes 0 origin for v, number of elements = right+1 (right is index of
@@ -448,8 +486,6 @@ static PetscErrorCode PetscSortMPIIntWithArray_Private(PetscMPIInt *v,PetscMPIIn
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSortMPIIntWithArray"
 /*@
    PetscSortMPIIntWithArray - Sorts an array of integers in place in increasing order;
        changes a second array to match the sorted first array.
@@ -492,8 +528,6 @@ PetscErrorCode  PetscSortMPIIntWithArray(PetscMPIInt n,PetscMPIInt i[],PetscMPII
 /* -----------------------------------------------------------------------*/
 #define SWAP2IntScalar(a,b,c,d,t,ts) {t=a;a=b;b=t;ts=c;c=d;d=ts;}
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSortIntWithScalarArray_Private"
 /*
    Modified from PetscSortIntWithArray_Private().
 */
@@ -522,8 +556,6 @@ static PetscErrorCode PetscSortIntWithScalarArray_Private(PetscInt *v,PetscScala
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSortIntWithScalarArray"
 /*@
    PetscSortIntWithScalarArray - Sorts an array of integers in place in increasing order;
        changes a second SCALAR array to match the sorted first INTEGER array.
@@ -564,12 +596,147 @@ PetscErrorCode  PetscSortIntWithScalarArray(PetscInt n,PetscInt i[],PetscScalar
   PetscFunctionReturn(0);
 }
 
+#define SWAP2IntData(a,b,c,d,t,td,siz)          \
+  do {                                          \
+  PetscErrorCode _ierr;                         \
+  t=a;a=b;b=t;                                  \
+  _ierr = PetscMemcpy(td,c,siz);CHKERRQ(_ierr); \
+  _ierr = PetscMemcpy(c,d,siz);CHKERRQ(_ierr);  \
+  _ierr = PetscMemcpy(d,td,siz);CHKERRQ(_ierr); \
+} while(0)
+
+/*
+   Modified from PetscSortIntWithArray_Private().
+*/
+static PetscErrorCode PetscSortIntWithDataArray_Private(PetscInt *v,char *V,PetscInt right,size_t size,void *work)
+{
+  PetscErrorCode ierr;
+  PetscInt       i,vl,last,tmp;
+
+  PetscFunctionBegin;
+  if (right <= 1) {
+    if (right == 1) {
+      if (v[0] > v[1]) SWAP2IntData(v[0],v[1],V,V+size,tmp,work,size);
+    }
+    PetscFunctionReturn(0);
+  }
+  SWAP2IntData(v[0],v[right/2],V,V+size*(right/2),tmp,work,size);
+  vl   = v[0];
+  last = 0;
+  for (i=1; i<=right; i++) {
+    if (v[i] < vl) {last++; SWAP2IntData(v[last],v[i],V+size*last,V+size*i,tmp,work,size);}
+  }
+  SWAP2IntData(v[0],v[last],V,V + size*last,tmp,work,size);
+  ierr = PetscSortIntWithDataArray_Private(v,V,last-1,size,work);CHKERRQ(ierr);
+  ierr = PetscSortIntWithDataArray_Private(v+last+1,V+size*(last+1),right-(last+1),size,work);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@C
+   PetscSortIntWithDataArray - Sorts an array of integers in place in increasing order;
+       changes a second array to match the sorted first INTEGER array.  Unlike other sort routines, the user must
+       provide workspace (the size of an element in the data array) to use when sorting.
+
+   Not Collective
+
+   Input Parameters:
++  n  - number of values
+.  i  - array of integers
+.  Ii - second array of data
+.  size - sizeof elements in the data array in bytes
+-  work - workspace of "size" bytes used when sorting
+
+   Level: intermediate
+
+   Concepts: sorting^ints with array
+
+.seealso: PetscSortReal(), PetscSortIntPermutation(), PetscSortInt(), PetscSortIntWithArray()
+@*/
+PetscErrorCode  PetscSortIntWithDataArray(PetscInt n,PetscInt i[],void *Ii,size_t size,void *work)
+{
+  char           *V = (char *) Ii;
+  PetscErrorCode ierr;
+  PetscInt       j,k,tmp,ik;
+
+  PetscFunctionBegin;
+  if (n<8) {
+    for (k=0; k<n; k++) {
+      ik = i[k];
+      for (j=k+1; j<n; j++) {
+        if (ik > i[j]) {
+          SWAP2IntData(i[k],i[j],V+size*k,V+size*j,tmp,work,size);
+          ik = i[k];
+        }
+      }
+    }
+  } else {
+    ierr = PetscSortIntWithDataArray_Private(i,V,n-1,size,work);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscMergeIntArrayPair"
 /*@
-   PetscMergeIntArrayPair -     Merges two SORTED integer arrays along with an additional array of integers.
+   PetscMergeIntArray -     Merges two SORTED integer arrays, removes duplicate elements.
+
+   Not Collective
+
+   Input Parameters:
++  an  - number of values in the first array
+.  aI  - first sorted array of integers
+.  bn  - number of values in the second array
+-  bI  - second array of integers
+
+   Output Parameters:
++  n   - number of values in the merged array
+-  L   - merged sorted array, this is allocated if an array is not provided
+
+   Level: intermediate
+
+   Concepts: merging^arrays
+
+.seealso: PetscSortReal(), PetscSortIntPermutation(), PetscSortInt(), PetscSortIntWithArray()
+@*/
+PetscErrorCode  PetscMergeIntArray(PetscInt an,const PetscInt aI[], PetscInt bn, const PetscInt bI[],  PetscInt *n, PetscInt **L)
+{
+  PetscErrorCode ierr;
+  PetscInt       *L_ = *L, ak, bk, k;
+
+  if (!L_) {
+    ierr = PetscMalloc1(an+bn, L);CHKERRQ(ierr);
+    L_   = *L;
+  }
+  k = ak = bk = 0;
+  while (ak < an && bk < bn) {
+    if (aI[ak] == bI[bk]) {
+      L_[k] = aI[ak];
+      ++ak;
+      ++bk;
+      ++k;
+    } else if (aI[ak] < bI[bk]) {
+      L_[k] = aI[ak];
+      ++ak;
+      ++k;
+    } else {
+      L_[k] = bI[bk];
+      ++bk;
+      ++k;
+    }
+  }
+  if (ak < an) {
+    ierr = PetscMemcpy(L_+k,aI+ak,(an-ak)*sizeof(PetscInt));CHKERRQ(ierr);
+    k   += (an-ak);
+  }
+  if (bk < bn) {
+    ierr = PetscMemcpy(L_+k,bI+bk,(bn-bk)*sizeof(PetscInt));CHKERRQ(ierr);
+    k   += (bn-bk);
+  }
+  *n = k;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   PetscMergeIntArrayPair -     Merges two SORTED integer arrays that share NO common values along with an additional array of integers.
                                 The additional arrays are the same length as sorted arrays and are merged
                                 in the order determined by the merging of the sorted pair.
 
@@ -585,30 +752,35 @@ PetscErrorCode  PetscSortIntWithScalarArray(PetscInt n,PetscInt i[],PetscScalar
 
    Output Parameters:
 +  n   - number of values in the merged array (== an + bn)
-.  I   - merged sorted array
+.  L   - merged sorted array 
 -  J   - merged additional array
 
+   Notes:
+    if L or J point to non-null arrays then this routine will assume they are of the approproate size and use them, otherwise this routine will allocate space for them 
    Level: intermediate
 
    Concepts: merging^arrays
 
 .seealso: PetscSortReal(), PetscSortIntPermutation(), PetscSortInt(), PetscSortIntWithArray()
 @*/
-PetscErrorCode  PetscMergeIntArrayPair(PetscInt an,const PetscInt *aI, const PetscInt *aJ, PetscInt bn, const PetscInt *bI, const PetscInt *bJ, PetscInt *n, PetscInt **L, PetscInt **J)
+PetscErrorCode  PetscMergeIntArrayPair(PetscInt an,const PetscInt aI[], const PetscInt aJ[], PetscInt bn, const PetscInt bI[], const PetscInt bJ[], PetscInt *n, PetscInt **L, PetscInt **J)
 {
   PetscErrorCode ierr;
-  PetscInt       n_, *L_ = *L, *J_= *J, ak, bk, k;
+  PetscInt       n_, *L_, *J_, ak, bk, k;
 
+  PetscFunctionBegin;
+  PetscValidIntPointer(L,8);
+  PetscValidIntPointer(J,9);
   n_ = an + bn;
   *n = n_;
-  if (!L_) {
-    ierr = PetscMalloc(n_*sizeof(PetscInt), L);CHKERRQ(ierr);
-    L_   = *L;
+  if (!*L) {
+    ierr = PetscMalloc1(n_, L);CHKERRQ(ierr);
   }
-  if (!J_) {
-    ierr = PetscMalloc(n_*sizeof(PetscInt), &J_);CHKERRQ(ierr);
-    J_   = *J;
+  L_ = *L;
+  if (!*J) {
+    ierr = PetscMalloc1(n_, J);CHKERRQ(ierr);
   }
+  J_   = *J;
   k = ak = bk = 0;
   while (ak < an && bk < bn) {
     if (aI[ak] <= bI[bk]) {
@@ -635,10 +807,46 @@ PetscErrorCode  PetscMergeIntArrayPair(PetscInt an,const PetscInt *aI, const Pet
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscProcessTree"
 /*@
+   PetscMergeMPIIntArray -     Merges two SORTED integer arrays.
+
+   Not Collective
+
+   Input Parameters:
++  an  - number of values in the first array
+.  aI  - first sorted array of integers
+.  bn  - number of values in the second array
+-  bI  - second array of integers
+
+   Output Parameters:
++  n   - number of values in the merged array (<= an + bn)
+-  L   - merged sorted array, allocated if address of NULL pointer is passed
+
+   Level: intermediate
+
+   Concepts: merging^arrays
+
+.seealso: PetscSortReal(), PetscSortIntPermutation(), PetscSortInt(), PetscSortIntWithArray()
+@*/
+PetscErrorCode PetscMergeMPIIntArray(PetscInt an,const PetscMPIInt aI[],PetscInt bn,const PetscMPIInt bI[],PetscInt *n,PetscMPIInt **L)
+{
+  PetscErrorCode ierr;
+  PetscInt       ai,bi,k;
+
+  PetscFunctionBegin;
+  if (!*L) {ierr = PetscMalloc1((an+bn),L);CHKERRQ(ierr);}
+  for (ai=0,bi=0,k=0; ai<an || bi<bn; ) {
+    PetscInt t = -1;
+    for ( ; ai<an && (!bn || aI[ai] <= bI[bi]); ai++) (*L)[k++] = t = aI[ai];
+    for ( ; bi<bn && bI[bi] == t; bi++);
+    for ( ; bi<bn && (!an || bI[bi] <= aI[ai]); bi++) (*L)[k++] = t = bI[bi];
+    for ( ; ai<an && aI[ai] == t; ai++);
+  }
+  *n = k;
+  PetscFunctionReturn(0);
+}
+
+/*@C
    PetscProcessTree - Prepares tree data to be displayed graphically
 
    Not Collective
@@ -655,8 +863,10 @@ PetscErrorCode  PetscMergeIntArrayPair(PetscInt an,const PetscInt *aI, const Pet
 .  Idbylevel - a list of ids on each of the levels, first level followed by second etc
 -  Column - for each id tells its column index
 
-   Level: intermediate
+   Level: developer
 
+   Notes:
+    This code is not currently used
 
 .seealso: PetscSortReal(), PetscSortIntWithPermutation()
 @*/
@@ -679,8 +889,7 @@ PetscErrorCode  PetscProcessTree(PetscInt n,const PetscBool mask[],const PetscIn
   }
 
   /* determine the level in the tree of each node */
-  ierr = PetscMalloc(n*sizeof(PetscInt),&level);CHKERRQ(ierr);
-  ierr = PetscMemzero(level,n*sizeof(PetscInt));CHKERRQ(ierr);
+  ierr = PetscCalloc1(n,&level);CHKERRQ(ierr);
 
   level[0] = 1;
   while (!done) {
@@ -697,8 +906,7 @@ PetscErrorCode  PetscProcessTree(PetscInt n,const PetscBool mask[],const PetscIn
   }
 
   /* count the number of nodes on each level and its max */
-  ierr = PetscMalloc(nlevels*sizeof(PetscInt),&levelcnt);CHKERRQ(ierr);
-  ierr = PetscMemzero(levelcnt,nlevels*sizeof(PetscInt));CHKERRQ(ierr);
+  ierr = PetscCalloc1(nlevels,&levelcnt);CHKERRQ(ierr);
   for (i=0; i<n; i++) {
     if (mask[i]) continue;
     levelcnt[level[i]-1]++;
@@ -706,8 +914,8 @@ PetscErrorCode  PetscProcessTree(PetscInt n,const PetscBool mask[],const PetscIn
   for (i=0; i<nlevels;i++) levelmax = PetscMax(levelmax,levelcnt[i]);
 
   /* for each level sort the ids by the parent id */
-  ierr = PetscMalloc2(levelmax,PetscInt,&workid,levelmax,PetscInt,&workparentid);CHKERRQ(ierr);
-  ierr = PetscMalloc(nmask*sizeof(PetscInt),&idbylevel);CHKERRQ(ierr);
+  ierr = PetscMalloc2(levelmax,&workid,levelmax,&workparentid);CHKERRQ(ierr);
+  ierr = PetscMalloc1(nmask,&idbylevel);CHKERRQ(ierr);
   for (j=1; j<=nlevels;j++) {
     cnt = 0;
     for (i=0; i<n; i++) {
@@ -728,7 +936,7 @@ PetscErrorCode  PetscProcessTree(PetscInt n,const PetscBool mask[],const PetscIn
   ierr = PetscFree2(workid,workparentid);CHKERRQ(ierr);
 
   /* for each node list its column */
-  ierr = PetscMalloc(n*sizeof(PetscInt),&column);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n,&column);CHKERRQ(ierr);
   cnt = 0;
   for (j=0; j<nlevels; j++) {
     for (i=0; i<levelcnt[j]; i++) {
