@@ -13,8 +13,6 @@
 #  include <strings.h>          /* strcasecmp */
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrToArray"
 /*@C
    PetscStrToArray - Separates a string by a charactor (for example ' ' or '\n') and creates an array of strings
 
@@ -30,11 +28,13 @@
 
    Level: intermediate
 
-   Notes: this may be called before PetscInitialize() or after PetscFinalize()
+   Notes:
+    this may be called before PetscInitialize() or after PetscFinalize()
 
    Not for use in Fortran
 
-   Developer Notes: Using raw malloc() and does not call error handlers since this may be used before PETSc is initialized. Used
+   Developer Notes:
+    Using raw malloc() and does not call error handlers since this may be used before PETSc is initialized. Used
      to generate argc, args arguments passed to MPI_Init()
 
 .seealso: PetscStrToArrayDestroy(), PetscToken, PetscTokenCreate()
@@ -49,6 +49,9 @@ PetscErrorCode  PetscStrToArray(const char s[],char sp,int *argc,char ***args)
   else    n = strlen(s);
   *argc = 0;
   *args = NULL;
+  for (; n>0; n--) {   /* remove separator chars at the end - and will empty the string if all chars are separator chars */
+    if (s[n-1] != sp) break;
+  }
   if (!n) {
     return(0);
   }
@@ -59,9 +62,6 @@ PetscErrorCode  PetscStrToArray(const char s[],char sp,int *argc,char ***args)
     if ((s[i] == sp || s[i] == 0) && !flg) {flg = PETSC_TRUE; (*argc)++;}
     else if (s[i] != sp) {flg = PETSC_FALSE;}
   }
-  if (!*argc) { /* string only has separator characters */
-    return(0);
-  }
   (*args) = (char**) malloc(((*argc)+1)*sizeof(char*)); if (!*args) return PETSC_ERR_MEM;
   lens    = (int*) malloc((*argc)*sizeof(int)); if (!lens) return PETSC_ERR_MEM;
   for (i=0; i<*argc; i++) lens[i] = 0;
@@ -98,8 +98,6 @@ PetscErrorCode  PetscStrToArray(const char s[],char sp,int *argc,char ***args)
   return 0;
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrToArrayDestroy"
 /*@C
    PetscStrToArrayDestroy - Frees array created with PetscStrToArray().
 
@@ -113,7 +111,8 @@ PetscErrorCode  PetscStrToArray(const char s[],char sp,int *argc,char ***args)
 
    Concepts: command line arguments
 
-   Notes: This may be called before PetscInitialize() or after PetscFinalize()
+   Notes:
+    This may be called before PetscInitialize() or after PetscFinalize()
 
    Not for use in Fortran
 
@@ -129,8 +128,6 @@ PetscErrorCode  PetscStrToArrayDestroy(int argc,char **args)
   return 0;
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrlen"
 /*@C
    PetscStrlen - Gets length of a string
 
@@ -162,8 +159,6 @@ PetscErrorCode  PetscStrlen(const char s[],size_t *len)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrallocpy"
 /*@C
    PetscStrallocpy - Allocates space to hold a copy of a string then copies the string
 
@@ -201,8 +196,6 @@ PetscErrorCode  PetscStrallocpy(const char s[],char *t[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrArrayallocpy"
 /*@C
    PetscStrArrayallocpy - Allocates space to hold a copy of an array of strings then copies the strings
 
@@ -239,8 +232,6 @@ PetscErrorCode  PetscStrArrayallocpy(const char *const *list,char ***t)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrArrayDestroy"
 /*@C
    PetscStrArrayDestroy - Frees array of strings created with PetscStrArrayallocpy().
 
@@ -253,7 +244,8 @@ PetscErrorCode  PetscStrArrayallocpy(const char *const *list,char ***t)
 
    Concepts: command line arguments
 
-   Notes: Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
 .seealso: PetscStrArrayallocpy()
 
@@ -273,8 +265,6 @@ PetscErrorCode PetscStrArrayDestroy(char ***list)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrNArrayallocpy"
 /*@C
    PetscStrNArrayallocpy - Allocates space to hold a copy of an array of strings then copies the strings
 
@@ -310,8 +300,6 @@ PetscErrorCode  PetscStrNArrayallocpy(PetscInt n,const char *const *list,char **
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrNArrayDestroy"
 /*@C
    PetscStrNArrayDestroy - Frees array of strings created with PetscStrArrayallocpy().
 
@@ -323,7 +311,8 @@ PetscErrorCode  PetscStrNArrayallocpy(PetscInt n,const char *const *list,char **
 
    Level: intermediate
 
-   Notes: Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
 .seealso: PetscStrArrayallocpy()
 
@@ -342,8 +331,6 @@ PetscErrorCode PetscStrNArrayDestroy(PetscInt n,char ***list)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrcpy"
 /*@C
    PetscStrcpy - Copies a string
 
@@ -364,7 +351,7 @@ PetscErrorCode PetscStrNArrayDestroy(PetscInt n,char ***list)
 
   Concepts: string copy
 
-.seealso: PetscStrncpy(), PetscStrcat(), PetscStrncat()
+.seealso: PetscStrncpy(), PetscStrcat(), PetscStrlcat()
 
 @*/
 
@@ -377,8 +364,6 @@ PetscErrorCode  PetscStrcpy(char s[],const char t[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrncpy"
 /*@C
    PetscStrncpy - Copies a string up to a certain length
 
@@ -397,18 +382,21 @@ PetscErrorCode  PetscStrcpy(char s[],const char t[])
      Null string returns a string starting with zero
 
      If the string that is being copied is of length n or larger then the entire string is not
-     copied and the file location of s is set to NULL. This is different then the behavior of 
-     strncpy() which leaves s non-terminated.
+     copied and the final location of s is set to NULL. This is different then the behavior of 
+     strncpy() which leaves s non-terminated if there is not room for the entire string.
 
   Concepts: string copy
 
-.seealso: PetscStrcpy(), PetscStrcat(), PetscStrncat()
+  Developers Note: Should this be PetscStrlcpy() to reflect its behavior which is like strlcpy() not strncpy()
+
+.seealso: PetscStrcpy(), PetscStrcat(), PetscStrlcat()
 
 @*/
 PetscErrorCode  PetscStrncpy(char s[],const char t[],size_t n)
 {
   PetscFunctionBegin;
   if (t && !s) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_NULL,"Trying to copy string into null pointer");
+  if (s && !n) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_NULL,"Requires an output string of length at least 1 to hold the termination character");
   if (t) {
     if (n > 1) {
       strncpy(s,t,n-1);
@@ -420,8 +408,6 @@ PetscErrorCode  PetscStrncpy(char s[],const char t[],size_t n)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrcat"
 /*@C
    PetscStrcat - Concatenates a string onto a given string
 
@@ -433,11 +419,12 @@ PetscErrorCode  PetscStrncpy(char s[],const char t[],size_t n)
 
    Level: intermediate
 
-   Notes: Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
   Concepts: string copy
 
-.seealso: PetscStrcpy(), PetscStrncpy(), PetscStrncat()
+.seealso: PetscStrcpy(), PetscStrncpy(), PetscStrlcat()
 
 @*/
 PetscErrorCode  PetscStrcat(char s[],const char t[])
@@ -448,40 +435,47 @@ PetscErrorCode  PetscStrcat(char s[],const char t[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrncat"
 /*@C
-   PetscStrncat - Concatenates a string onto a given string, up to a given length
+   PetscStrlcat - Concatenates a string onto a given string, up to a given length
 
    Not Collective
 
    Input Parameters:
-+  s - pointer to string to be added to end
++  s - pointer to string to be added to at end
 .  t - string to be added to
-.  n - maximum length to copy
+-  n - length of the original allocated string
 
    Level: intermediate
 
-  Notes:    Not for use in Fortran
+  Notes:
+  Not for use in Fortran
+
+  Unlike the system call strncat(), the length passed in is the length of the
+  original allocated space, not the length of the left-over space. This is
+  similar to the BSD system call strlcat().
 
   Concepts: string copy
 
 .seealso: PetscStrcpy(), PetscStrncpy(), PetscStrcat()
 
 @*/
-PetscErrorCode  PetscStrncat(char s[],const char t[],size_t n)
+PetscErrorCode  PetscStrlcat(char s[],const char t[],size_t n)
 {
+  size_t         len;
+  PetscErrorCode ierr;
+
   PetscFunctionBegin;
-  strncat(s,t,n);
+  if (t && !n) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"String buffer length must be positive");
+  if (!t) PetscFunctionReturn(0);
+  ierr = PetscStrlen(t,&len);CHKERRQ(ierr);
+  strncat(s,t,n - len);
+  s[n-1] = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrcmpNoError"
 /*
    Only to be used with PetscCheck__FUNCT__()!
 
-   Will be removed once we eliminate the __FUNCT__ paradigm
 */
 void  PetscStrcmpNoError(const char a[],const char b[],PetscBool  *flg)
 {
@@ -496,8 +490,6 @@ void  PetscStrcmpNoError(const char a[],const char b[],PetscBool  *flg)
   }
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrcmp"
 /*@C
    PetscStrcmp - Compares two strings,
 
@@ -512,7 +504,8 @@ void  PetscStrcmpNoError(const char a[],const char b[],PetscBool  *flg)
 
    Level: intermediate
 
-   Notes:    Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
 .seealso: PetscStrgrt(), PetscStrncmp(), PetscStrcasecmp()
 
@@ -532,8 +525,6 @@ PetscErrorCode  PetscStrcmp(const char a[],const char b[],PetscBool  *flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrgrt"
 /*@C
    PetscStrgrt - If first string is greater than the second
 
@@ -573,8 +564,6 @@ PetscErrorCode  PetscStrgrt(const char a[],const char b[],PetscBool  *t)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrcasecmp"
 /*@C
    PetscStrcasecmp - Returns true if the two strings are the same
      except possibly for case.
@@ -630,8 +619,6 @@ PetscErrorCode  PetscStrcasecmp(const char a[],const char b[],PetscBool  *t)
 
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrncmp"
 /*@C
    PetscStrncmp - Compares two strings, up to a certain length
 
@@ -647,7 +634,8 @@ PetscErrorCode  PetscStrcasecmp(const char a[],const char b[],PetscBool  *t)
 
    Level: intermediate
 
-   Notes:    Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
 .seealso: PetscStrgrt(), PetscStrcmp(), PetscStrcasecmp()
 
@@ -663,8 +651,6 @@ PetscErrorCode  PetscStrncmp(const char a[],const char b[],size_t n,PetscBool  *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrchr"
 /*@C
    PetscStrchr - Locates first occurance of a character in a string
 
@@ -679,7 +665,8 @@ PetscErrorCode  PetscStrncmp(const char a[],const char b[],size_t n,PetscBool  *
 
    Level: intermediate
 
-   Notes:    Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
 @*/
 PetscErrorCode  PetscStrchr(const char a[],char b,char *c[])
@@ -689,8 +676,6 @@ PetscErrorCode  PetscStrchr(const char a[],char b,char *c[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrrchr"
 /*@C
    PetscStrrchr - Locates one location past the last occurance of a character in a string,
       if the character is not found then returns entire string
@@ -706,7 +691,8 @@ PetscErrorCode  PetscStrchr(const char a[],char b,char *c[])
 
    Level: intermediate
 
-   Notes:    Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
 @*/
 PetscErrorCode  PetscStrrchr(const char a[],char b,char *tmp[])
@@ -718,8 +704,6 @@ PetscErrorCode  PetscStrrchr(const char a[],char b,char *tmp[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrtolower"
 /*@C
    PetscStrtolower - Converts string to lower case
 
@@ -730,7 +714,8 @@ PetscErrorCode  PetscStrrchr(const char a[],char b,char *tmp[])
 
    Level: intermediate
 
-   Notes:    Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
 @*/
 PetscErrorCode  PetscStrtolower(char a[])
@@ -743,10 +728,8 @@ PetscErrorCode  PetscStrtolower(char a[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrtoupper"
 /*@C
-   PetscStrtolower - Converts string to upper case
+   PetscStrtoupper - Converts string to upper case
 
    Not Collective
 
@@ -755,7 +738,8 @@ PetscErrorCode  PetscStrtolower(char a[])
 
    Level: intermediate
 
-   Notes:    Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
 @*/
 PetscErrorCode  PetscStrtoupper(char a[])
@@ -768,8 +752,6 @@ PetscErrorCode  PetscStrtoupper(char a[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrendswith"
 /*@C
    PetscStrendswith - Determines if a string ends with a certain string
 
@@ -782,7 +764,8 @@ PetscErrorCode  PetscStrtoupper(char a[])
    Output Parameter:
 .  flg - PETSC_TRUE or PETSC_FALSE
 
-   Notes:     Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
    Level: intermediate
 
@@ -804,8 +787,6 @@ PetscErrorCode  PetscStrendswith(const char a[],const char b[],PetscBool *flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrbeginswith"
 /*@C
    PetscStrbeginswith - Determines if a string begins with a certain string
 
@@ -813,15 +794,19 @@ PetscErrorCode  PetscStrendswith(const char a[],const char b[],PetscBool *flg)
 
    Input Parameters:
 +  a - pointer to string
--  b - string to beginwith
+-  b - string to begin with
 
    Output Parameter:
 .  flg - PETSC_TRUE or PETSC_FALSE
 
-   Notes:     Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
    Level: intermediate
 
+.seealso: PetscStrendswithwhich(), PetscStrendswith(), PetscStrtoupper, PetscStrtolower(), PetscStrrchr(), PetscStrchr(),
+          PetscStrncmp(), PetscStrlen(), PetscStrncmp(), PetscStrcmp()
+
 @*/
 PetscErrorCode  PetscStrbeginswith(const char a[],const char b[],PetscBool *flg)
 {
@@ -836,8 +821,6 @@ PetscErrorCode  PetscStrbeginswith(const char a[],const char b[],PetscBool *flg)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrendswithwhich"
 /*@C
    PetscStrendswithwhich - Determines if a string ends with one of several possible strings
 
@@ -850,7 +833,8 @@ PetscErrorCode  PetscStrbeginswith(const char a[],const char b[],PetscBool *flg)
    Output Parameter:
 .  cnt - the index of the string it ends with or 1+the last possible index
 
-   Notes:     Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
    Level: intermediate
 
@@ -870,8 +854,6 @@ PetscErrorCode  PetscStrendswithwhich(const char a[],const char *const *bs,Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrrstr"
 /*@C
    PetscStrrstr - Locates last occurance of string in another string
 
@@ -884,7 +866,8 @@ PetscErrorCode  PetscStrendswithwhich(const char a[],const char *const *bs,Petsc
    Output Parameter:
 .  tmp - location of occurance
 
-   Notes:     Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
    Level: intermediate
 
@@ -902,8 +885,6 @@ PetscErrorCode  PetscStrrstr(const char a[],const char b[],char *tmp[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrstr"
 /*@C
    PetscStrstr - Locates first occurance of string in another string
 
@@ -916,7 +897,8 @@ PetscErrorCode  PetscStrrstr(const char a[],const char b[],char *tmp[])
    Output Parameter:
 .  tmp - location of occurance, is a NULL if the string is not found
 
-   Notes: Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
    Level: intermediate
 
@@ -930,8 +912,6 @@ PetscErrorCode  PetscStrstr(const char haystack[],const char needle[],char *tmp[
 
 struct _p_PetscToken {char token;char *array;char *current;};
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscTokenFind"
 /*@C
    PetscTokenFind - Locates next "token" in a string
 
@@ -988,8 +968,6 @@ PetscErrorCode  PetscTokenFind(PetscToken a,char *result[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscTokenCreate"
 /*@C
    PetscTokenCreate - Creates a PetscToken used to find tokens in a string
 
@@ -1026,8 +1004,6 @@ PetscErrorCode  PetscTokenCreate(const char a[],const char b,PetscToken *t)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscTokenDestroy"
 /*@C
    PetscTokenDestroy - Destroys a PetscToken
 
@@ -1038,7 +1014,8 @@ PetscErrorCode  PetscTokenCreate(const char a[],const char b,PetscToken *t)
 
    Level: intermediate
 
-   Notes:     Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
 .seealso: PetscTokenCreate(), PetscTokenFind()
 @*/
@@ -1053,9 +1030,45 @@ PetscErrorCode  PetscTokenDestroy(PetscToken *a)
   PetscFunctionReturn(0);
 }
 
+/*@C
+   PetscStrInList - search string in character-delimited list
+
+   Not Collective
+
+   Input Parameters:
++  str - the string to look for
+.  list - the list to search in
+-  sep - the separator character
+
+   Output Parameter:
+.  found - whether str is in list
+
+   Level: intermediate
+
+   Notes:
+    Not for use in Fortran
+
+.seealso: PetscTokenCreate(), PetscTokenFind(), PetscStrcmp()
+@*/
+PetscErrorCode PetscStrInList(const char str[],const char list[],char sep,PetscBool *found)
+{
+  PetscToken     token;
+  char           *item;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  *found = PETSC_FALSE;
+  ierr = PetscTokenCreate(list,sep,&token);CHKERRQ(ierr);
+  ierr = PetscTokenFind(token,&item);CHKERRQ(ierr);
+  while (item) {
+    ierr = PetscStrcmp(str,item,found);CHKERRQ(ierr);
+    if (*found) break;
+    ierr = PetscTokenFind(token,&item);CHKERRQ(ierr);
+  }
+  ierr = PetscTokenDestroy(&token);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscGetPetscDir"
 /*@C
    PetscGetPetscDir - Gets the directory PETSc is installed in
 
@@ -1066,7 +1079,8 @@ PetscErrorCode  PetscTokenDestroy(PetscToken *a)
 
    Level: developer
 
-   Notes: Not for use in Fortran
+   Notes:
+    Not for use in Fortran
 
 @*/
 PetscErrorCode  PetscGetPetscDir(const char *dir[])
@@ -1076,8 +1090,6 @@ PetscErrorCode  PetscGetPetscDir(const char *dir[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscStrreplace"
 /*@C
    PetscStrreplace - Replaces substrings in string with other substrings
 
@@ -1109,7 +1121,7 @@ PetscErrorCode  PetscStrreplace(MPI_Comm comm,const char aa[],char b[],size_t le
   size_t         l,l1,l2,l3;
   char           *work,*par,*epar,env[1024],*tfree,*a = (char*)aa;
   const char     *s[] = {"${PETSC_ARCH}","${PETSC_DIR}","${PETSC_LIB_DIR}","${DISPLAY}","${HOMEDIRECTORY}","${WORKINGDIRECTORY}","${USERNAME}","${HOSTNAME}",0};
-  const char     *r[] = {0,0,0,0,0,0,0,0,0};
+  char           *r[] = {0,0,0,0,0,0,0,0,0};
   PetscBool      flag;
 
   PetscFunctionBegin;
@@ -1120,25 +1132,25 @@ PetscErrorCode  PetscStrreplace(MPI_Comm comm,const char aa[],char b[],size_t le
   ierr = PetscMalloc1(len,&work);CHKERRQ(ierr);
 
   /* get values for replaced variables */
-  ierr = PetscStrallocpy(PETSC_ARCH,(char**)&r[0]);CHKERRQ(ierr);
-  ierr = PetscStrallocpy(PETSC_DIR,(char**)&r[1]);CHKERRQ(ierr);
-  ierr = PetscStrallocpy(PETSC_LIB_DIR,(char**)&r[2]);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(PETSC_ARCH,&r[0]);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(PETSC_DIR,&r[1]);CHKERRQ(ierr);
+  ierr = PetscStrallocpy(PETSC_LIB_DIR,&r[2]);CHKERRQ(ierr);
   ierr = PetscMalloc1(256,&r[3]);CHKERRQ(ierr);
   ierr = PetscMalloc1(PETSC_MAX_PATH_LEN,&r[4]);CHKERRQ(ierr);
   ierr = PetscMalloc1(PETSC_MAX_PATH_LEN,&r[5]);CHKERRQ(ierr);
   ierr = PetscMalloc1(256,&r[6]);CHKERRQ(ierr);
   ierr = PetscMalloc1(256,&r[7]);CHKERRQ(ierr);
-  ierr = PetscGetDisplay((char*)r[3],256);CHKERRQ(ierr);
-  ierr = PetscGetHomeDirectory((char*)r[4],PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
-  ierr = PetscGetWorkingDirectory((char*)r[5],PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
-  ierr = PetscGetUserName((char*)r[6],256);CHKERRQ(ierr);
-  ierr = PetscGetHostName((char*)r[7],256);CHKERRQ(ierr);
+  ierr = PetscGetDisplay(r[3],256);CHKERRQ(ierr);
+  ierr = PetscGetHomeDirectory(r[4],PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
+  ierr = PetscGetWorkingDirectory(r[5],PETSC_MAX_PATH_LEN);CHKERRQ(ierr);
+  ierr = PetscGetUserName(r[6],256);CHKERRQ(ierr);
+  ierr = PetscGetHostName(r[7],256);CHKERRQ(ierr);
 
   /* replace that are in environment */
   ierr = PetscOptionsGetenv(comm,"PETSC_LIB_DIR",env,1024,&flag);CHKERRQ(ierr);
   if (flag) {
     ierr = PetscFree(r[2]);CHKERRQ(ierr);
-    ierr = PetscStrallocpy(env,(char**)&r[2]);CHKERRQ(ierr);
+    ierr = PetscStrallocpy(env,&r[2]);CHKERRQ(ierr);
   }
 
   /* replace the requested strings */
@@ -1192,8 +1204,6 @@ PetscErrorCode  PetscStrreplace(MPI_Comm comm,const char aa[],char b[],size_t le
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscEListFind"
 /*@C
    PetscEListFind - searches list of strings for given string, using case insensitive matching
 
@@ -1232,10 +1242,8 @@ PetscErrorCode PetscEListFind(PetscInt n,const char *const *list,const char *str
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscEnumFind"
 /*@C
-   PetscEListFind - searches enum list of strings for given string, using case insensitive matching
+   PetscEnumFind - searches enum list of strings for given string, using case insensitive matching
 
    Not Collective
 
