@@ -1,17 +1,14 @@
 #include <petsctaolinesearch.h>
-#include <../src/tao/matrix/lmvmmat.h>
+#include <../src/tao/unconstrained/impls/lmvm/lmvm.h>
 #include <../src/tao/bound/impls/blmvm/blmvm.h>
 
 /*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TaoSolve_BLMVM"
 static PetscErrorCode TaoSolve_BLMVM(Tao tao)
 {
   PetscErrorCode               ierr;
   TAO_BLMVM                    *blmP = (TAO_BLMVM *)tao->data;
-  TaoConvergedReason           reason = TAO_CONTINUE_ITERATING;
   TaoLineSearchConvergedReason ls_status = TAOLINESEARCH_CONTINUE_ITERATING;
-  PetscReal                    f, fold, gdx, gnorm;
+  PetscReal                    f, fold, gdx, gnorm, gnorm2;
   PetscReal                    stepsize = 1.0,delta;
 
   PetscFunctionBegin;
@@ -20,33 +17,40 @@ static PetscErrorCode TaoSolve_BLMVM(Tao tao)
   ierr = VecMedian(tao->XL,tao->solution,tao->XU,tao->solution);CHKERRQ(ierr);
   ierr = TaoLineSearchSetVariableBounds(tao->linesearch,tao->XL,tao->XU);CHKERRQ(ierr);
 
+
   /* Check convergence criteria */
   ierr = TaoComputeObjectiveAndGradient(tao, tao->solution,&f,blmP->unprojected_gradient);CHKERRQ(ierr);
   ierr = VecBoundGradientProjection(blmP->unprojected_gradient,tao->solution, tao->XL,tao->XU,tao->gradient);CHKERRQ(ierr);
 
-  ierr = VecNorm(tao->gradient,NORM_2,&gnorm);CHKERRQ(ierr);
-  if (PetscIsInfOrNanReal(f) || PetscIsInfOrNanReal(gnorm)) SETERRQ(PETSC_COMM_SELF,1, "User provided compute function generated Inf pr NaN");
+  ierr = TaoGradientNorm(tao, tao->gradient,NORM_2,&gnorm);CHKERRQ(ierr);
+  if (PetscIsInfOrNanReal(f) || PetscIsInfOrNanReal(gnorm)) SETERRQ(PETSC_COMM_SELF,1, "User provided compute function generated Inf or NaN");
 
-  ierr = TaoMonitor(tao, tao->niter, f, gnorm, 0.0, stepsize, &reason);CHKERRQ(ierr);
-  if (reason != TAO_CONTINUE_ITERATING) PetscFunctionReturn(0);
-
-  /* Set initial scaling for the function */
-  if (f != 0.0) {
-    delta = 2.0*PetscAbsScalar(f) / (gnorm*gnorm);
-  } else {
-    delta = 2.0 / (gnorm*gnorm);
-  }
-  ierr = MatLMVMSetDelta(blmP->M,delta);CHKERRQ(ierr);
+  tao->reason = TAO_CONTINUE_ITERATING;
+  ierr = TaoLogConvergenceHistory(tao,f,gnorm,0.0,tao->ksp_its);CHKERRQ(ierr);
+  ierr = TaoMonitor(tao,tao->niter,f,gnorm,0.0,stepsize);CHKERRQ(ierr);
+  ierr = (*tao->ops->convergencetest)(tao,tao->cnvP);CHKERRQ(ierr);
+  if (tao->reason != TAO_CONTINUE_ITERATING) PetscFunctionReturn(0);
 
   /* Set counter for gradient/reset steps */
-  blmP->grad = 0;
-  blmP->reset = 0;
+  if (!blmP->recycle) {
+    blmP->grad = 0;
+    blmP->reset = 0;
+    ierr = MatLMVMReset(blmP->M, PETSC_FALSE);CHKERRQ(ierr);
+  }
 
   /* Have not converged; continue with Newton method */
-  while (reason == TAO_CONTINUE_ITERATING) {
+  while (tao->reason == TAO_CONTINUE_ITERATING) {
     /* Compute direction */
+    gnorm2 = gnorm*gnorm;
+    if (gnorm2 == 0.0) gnorm2 = PETSC_MACHINE_EPSILON;
+    if (f == 0.0) {
+      delta = 2.0 / gnorm2;
+    } else {
+      delta = 2.0 * PetscAbsScalar(f) / gnorm2;
+    }
+    ierr = MatSymBrdnSetDelta(blmP->M, delta);CHKERRQ(ierr);
     ierr = MatLMVMUpdate(blmP->M, tao->solution, tao->gradient);CHKERRQ(ierr);
-    ierr = MatLMVMSolve(blmP->M, blmP->unprojected_gradient, tao->stepdirection);CHKERRQ(ierr);
+    ierr = MatSolve(blmP->M, blmP->unprojected_gradient, tao->stepdirection);CHKERRQ(ierr);
     ierr = VecBoundGradientProjection(tao->stepdirection,tao->solution,tao->XL,tao->XU,tao->gradient);CHKERRQ(ierr);
 
     /* Check for success (descent direction) */
@@ -56,15 +60,9 @@ static PetscErrorCode TaoSolve_BLMVM(Tao tao)
          Use steepest descent direction (scaled) */
       ++blmP->grad;
 
-      if (f != 0.0) {
-        delta = 2.0*PetscAbsScalar(f) / (gnorm*gnorm);
-      } else {
-        delta = 2.0 / (gnorm*gnorm);
-      }
-      ierr = MatLMVMSetDelta(blmP->M,delta);CHKERRQ(ierr);
-      ierr = MatLMVMReset(blmP->M);CHKERRQ(ierr);
+      ierr = MatLMVMReset(blmP->M, PETSC_FALSE);CHKERRQ(ierr);
       ierr = MatLMVMUpdate(blmP->M, tao->solution, blmP->unprojected_gradient);CHKERRQ(ierr);
-      ierr = MatLMVMSolve(blmP->M,blmP->unprojected_gradient, tao->stepdirection);CHKERRQ(ierr);
+      ierr = MatSolve(blmP->M,blmP->unprojected_gradient, tao->stepdirection);CHKERRQ(ierr);
     }
     ierr = VecScale(tao->stepdirection,-1.0);CHKERRQ(ierr);
 
@@ -85,18 +83,12 @@ static PetscErrorCode TaoSolve_BLMVM(Tao tao)
       ierr = VecCopy(blmP->Xold, tao->solution);CHKERRQ(ierr);
       ierr = VecCopy(blmP->Gold, blmP->unprojected_gradient);CHKERRQ(ierr);
 
-      if (f != 0.0) {
-        delta = 2.0* PetscAbsScalar(f) / (gnorm*gnorm);
-      } else {
-        delta = 2.0/ (gnorm*gnorm);
-      }
-      ierr = MatLMVMSetDelta(blmP->M,delta);CHKERRQ(ierr);
-      ierr = MatLMVMReset(blmP->M);CHKERRQ(ierr);
+      ierr = MatLMVMReset(blmP->M, PETSC_FALSE);CHKERRQ(ierr);
       ierr = MatLMVMUpdate(blmP->M, tao->solution, blmP->unprojected_gradient);CHKERRQ(ierr);
-      ierr = MatLMVMSolve(blmP->M, blmP->unprojected_gradient, tao->stepdirection);CHKERRQ(ierr);
+      ierr = MatSolve(blmP->M, blmP->unprojected_gradient, tao->stepdirection);CHKERRQ(ierr);
       ierr = VecScale(tao->stepdirection, -1.0);CHKERRQ(ierr);
 
-      /* This may be incorrect; linesearch has values fo stepmax and stepmin
+      /* This may be incorrect; linesearch has values for stepmax and stepmin
          that should be reset. */
       ierr = TaoLineSearchSetInitialStepLength(tao->linesearch,1.0);CHKERRQ(ierr);
       ierr = TaoLineSearchApply(tao->linesearch,tao->solution,&f, blmP->unprojected_gradient, tao->stepdirection,  &stepsize, &ls_status);CHKERRQ(ierr);
@@ -110,22 +102,21 @@ static PetscErrorCode TaoSolve_BLMVM(Tao tao)
 
     /* Check for converged */
     ierr = VecBoundGradientProjection(blmP->unprojected_gradient, tao->solution, tao->XL, tao->XU, tao->gradient);CHKERRQ(ierr);
-    ierr = VecNorm(tao->gradient, NORM_2, &gnorm);CHKERRQ(ierr);
+    ierr = TaoGradientNorm(tao, tao->gradient, NORM_2, &gnorm);CHKERRQ(ierr);
 
 
     if (PetscIsInfOrNanReal(f) || PetscIsInfOrNanReal(gnorm)) SETERRQ(PETSC_COMM_SELF,1, "User provided compute function generated Not-a-Number");
     tao->niter++;
-    ierr = TaoMonitor(tao, tao->niter, f, gnorm, 0.0, stepsize, &reason);CHKERRQ(ierr);
+    ierr = TaoLogConvergenceHistory(tao,f,gnorm,0.0,tao->ksp_its);CHKERRQ(ierr);
+    ierr = TaoMonitor(tao,tao->niter,f,gnorm,0.0,stepsize);CHKERRQ(ierr);
+    ierr = (*tao->ops->convergencetest)(tao,tao->cnvP);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetup_BLMVM"
 static PetscErrorCode TaoSetup_BLMVM(Tao tao)
 {
   TAO_BLMVM      *blmP = (TAO_BLMVM *)tao->data;
-  PetscInt       n,N;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
@@ -148,17 +139,17 @@ static PetscErrorCode TaoSetup_BLMVM(Tao tao)
     ierr = VecDuplicate(tao->solution,&tao->XU);CHKERRQ(ierr);
     ierr = VecSet(tao->XU,PETSC_INFINITY);CHKERRQ(ierr);
   }
-  /* Create matrix for the limited memory approximation */
-  ierr = VecGetLocalSize(tao->solution,&n);CHKERRQ(ierr);
-  ierr = VecGetSize(tao->solution,&N);CHKERRQ(ierr);
-  ierr = MatCreateLMVM(((PetscObject)tao)->comm,n,N,&blmP->M);CHKERRQ(ierr);
-  ierr = MatLMVMAllocateVectors(blmP->M,tao->solution);CHKERRQ(ierr);
+  /* Allocate matrix for the limited memory approximation */
+  ierr = MatLMVMAllocate(blmP->M,tao->solution,blmP->unprojected_gradient);CHKERRQ(ierr);
+
+  /* If the user has set a matrix to solve as the initial H0, set the options prefix here, and set up the KSP */
+  if (blmP->H0) {
+    ierr = MatLMVMSetJ0(blmP->M, blmP->H0);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
 /* ---------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "TaoDestroy_BLMVM"
 static PetscErrorCode TaoDestroy_BLMVM(Tao tao)
 {
   TAO_BLMVM      *blmP = (TAO_BLMVM *)tao->data;
@@ -166,33 +157,38 @@ static PetscErrorCode TaoDestroy_BLMVM(Tao tao)
 
   PetscFunctionBegin;
   if (tao->setupcalled) {
-    ierr = MatDestroy(&blmP->M);CHKERRQ(ierr);
     ierr = VecDestroy(&blmP->unprojected_gradient);CHKERRQ(ierr);
     ierr = VecDestroy(&blmP->Xold);CHKERRQ(ierr);
     ierr = VecDestroy(&blmP->Gold);CHKERRQ(ierr);
   }
+  ierr = MatDestroy(&blmP->M);CHKERRQ(ierr);
+  if (blmP->H0) {
+    PetscObjectDereference((PetscObject)blmP->H0);
+  }
   ierr = PetscFree(tao->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 /*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetFromOptions_BLMVM"
-static PetscErrorCode TaoSetFromOptions_BLMVM(PetscOptions* PetscOptionsObject,Tao tao)
+static PetscErrorCode TaoSetFromOptions_BLMVM(PetscOptionItems* PetscOptionsObject,Tao tao)
 {
+  TAO_BLMVM      *blmP = (TAO_BLMVM *)tao->data;
   PetscErrorCode ierr;
+  PetscBool      is_spd;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"Limited-memory variable-metric method for bound constrained optimization");CHKERRQ(ierr);
-  ierr = TaoLineSearchSetFromOptions(tao->linesearch);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-tao_blmvm_recycle","enable recycling of the BFGS matrix between subsequent TaoSolve() calls","",blmP->recycle,&blmP->recycle,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsTail();CHKERRQ(ierr);
+  ierr = TaoLineSearchSetFromOptions(tao->linesearch);CHKERRQ(ierr);
+  ierr = MatSetFromOptions(blmP->M);CHKERRQ(ierr);
+  ierr = MatGetOption(blmP->M, MAT_SPD, &is_spd);CHKERRQ(ierr);
+  if (!is_spd) SETERRQ(PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_INCOMP, "LMVM matrix must be symmetric positive-definite");
   PetscFunctionReturn(0);
 }
 
 
 /*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TaoView_BLMVM"
 static int TaoView_BLMVM(Tao tao, PetscViewer viewer)
 {
   TAO_BLMVM      *lmP = (TAO_BLMVM *)tao->data;
@@ -202,15 +198,14 @@ static int TaoView_BLMVM(Tao tao, PetscViewer viewer)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer, PETSCVIEWERASCII, &isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer, "Gradient steps: %D\n", lmP->grad);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
+    ierr = PetscViewerPushFormat(viewer, PETSC_VIEWER_ASCII_INFO);CHKERRQ(ierr);
+    ierr = MatView(lmP->M, viewer);CHKERRQ(ierr);
+    ierr = PetscViewerPopFormat(viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoComputeDual_BLMVM"
 static PetscErrorCode TaoComputeDual_BLMVM(Tao tao, Vec DXL, Vec DXU)
 {
   TAO_BLMVM      *blm = (TAO_BLMVM *) tao->data;
@@ -240,26 +235,10 @@ static PetscErrorCode TaoComputeDual_BLMVM(Tao tao, Vec DXL, Vec DXU)
          of TAOLMVM
 
   Options Database Keys:
-+     -tao_lmm_vectors - number of vectors to use for approximation
-.     -tao_lmm_scale_type - "none","scalar","broyden"
-.     -tao_lmm_limit_type - "none","average","relative","absolute"
-.     -tao_lmm_rescale_type - "none","scalar","gl"
-.     -tao_lmm_limit_mu - mu limiting factor
-.     -tao_lmm_limit_nu - nu limiting factor
-.     -tao_lmm_delta_min - minimum delta value
-.     -tao_lmm_delta_max - maximum delta value
-.     -tao_lmm_broyden_phi - phi factor for Broyden scaling
-.     -tao_lmm_scalar_alpha - alpha factor for scalar scaling
-.     -tao_lmm_rescale_alpha - alpha factor for rescaling diagonal
-.     -tao_lmm_rescale_beta - beta factor for rescaling diagonal
-.     -tao_lmm_scalar_history - amount of history for scalar scaling
-.     -tao_lmm_rescale_history - amount of history for rescaling diagonal
--     -tao_lmm_eps - rejection tolerance
+.     -tao_lmm_recycle - enable recycling of LMVM information between subsequent TaoSolve calls
 
   Level: beginner
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "TaoCreate_BLMVM"
 PETSC_EXTERN PetscErrorCode TaoCreate_BLMVM(Tao tao)
 {
   TAO_BLMVM      *blmP;
@@ -275,18 +254,122 @@ PETSC_EXTERN PetscErrorCode TaoCreate_BLMVM(Tao tao)
   tao->ops->computedual = TaoComputeDual_BLMVM;
 
   ierr = PetscNewLog(tao,&blmP);CHKERRQ(ierr);
+  blmP->H0 = NULL;
+  blmP->recycle = PETSC_FALSE;
   tao->data = (void*)blmP;
 
   /* Override default settings (unless already changed) */
   if (!tao->max_it_changed) tao->max_it = 2000;
   if (!tao->max_funcs_changed) tao->max_funcs = 4000;
-  if (!tao->fatol_changed) tao->fatol = 1.0e-4;
-  if (!tao->frtol_changed) tao->frtol = 1.0e-4;
 
   ierr = TaoLineSearchCreate(((PetscObject)tao)->comm, &tao->linesearch);CHKERRQ(ierr);
+  ierr = PetscObjectIncrementTabLevel((PetscObject)tao->linesearch, (PetscObject)tao, 1);CHKERRQ(ierr);
   ierr = TaoLineSearchSetType(tao->linesearch, morethuente_type);CHKERRQ(ierr);
   ierr = TaoLineSearchUseTaoRoutines(tao->linesearch,tao);CHKERRQ(ierr);
   ierr = TaoLineSearchSetOptionsPrefix(tao->linesearch,tao->hdr.prefix);CHKERRQ(ierr);
+  
+  ierr = KSPInitializePackage();CHKERRQ(ierr);
+  ierr = MatCreate(((PetscObject)tao)->comm, &blmP->M);CHKERRQ(ierr);
+  ierr = MatSetType(blmP->M, MATLMVMBFGS);CHKERRQ(ierr);
+  ierr = PetscObjectIncrementTabLevel((PetscObject)blmP->M, (PetscObject)tao, 1);CHKERRQ(ierr);
+  ierr = MatSetOptionsPrefix(blmP->M, "tao_blmvm_");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode TaoLMVMRecycle(Tao tao, PetscBool flg)
+{
+  TAO_LMVM       *lmP;
+  TAO_BLMVM      *blmP;
+  TaoType        type;
+  PetscBool      is_lmvm, is_blmvm;
+  PetscErrorCode ierr;
+  
+  PetscFunctionBegin;
+  ierr = TaoGetType(tao, &type);CHKERRQ(ierr);
+  ierr = PetscStrcmp(type, TAOLMVM,  &is_lmvm);CHKERRQ(ierr);
+  ierr = PetscStrcmp(type, TAOBLMVM, &is_blmvm);CHKERRQ(ierr);
+  
+  if (is_lmvm) {
+    lmP = (TAO_LMVM *)tao->data;
+    lmP->recycle = flg;
+  } else if (is_blmvm) {
+    blmP = (TAO_BLMVM *)tao->data;
+    blmP->recycle = flg;
+  } else SETERRQ(PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_WRONGSTATE, "This routine applies to TAO_LMVM and TAO_BLMVM.");
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode TaoLMVMSetH0(Tao tao, Mat H0)
+{
+  TAO_LMVM       *lmP;
+  TAO_BLMVM      *blmP;
+  TaoType        type;
+  PetscBool      is_lmvm, is_blmvm;
+  PetscErrorCode ierr;
+  
+  PetscFunctionBegin;
+  ierr = TaoGetType(tao, &type);CHKERRQ(ierr);
+  ierr = PetscStrcmp(type, TAOLMVM,  &is_lmvm);CHKERRQ(ierr);
+  ierr = PetscStrcmp(type, TAOBLMVM, &is_blmvm);CHKERRQ(ierr);
+
+  if (is_lmvm) {
+    lmP = (TAO_LMVM *)tao->data;
+    ierr = PetscObjectReference((PetscObject)H0);CHKERRQ(ierr);
+    lmP->H0 = H0;
+  } else if (is_blmvm) {
+    blmP = (TAO_BLMVM *)tao->data;
+    ierr = PetscObjectReference((PetscObject)H0);CHKERRQ(ierr);
+    blmP->H0 = H0;
+  } else SETERRQ(PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_WRONGSTATE, "This routine applies to TAO_LMVM and TAO_BLMVM.");
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode TaoLMVMGetH0(Tao tao, Mat *H0)
+{
+  TAO_LMVM       *lmP;
+  TAO_BLMVM      *blmP;
+  TaoType        type;
+  PetscBool      is_lmvm, is_blmvm;
+  Mat            M;
+
+  PetscErrorCode ierr;
+  
+  PetscFunctionBegin;
+  ierr = TaoGetType(tao, &type);CHKERRQ(ierr);
+  ierr = PetscStrcmp(type, TAOLMVM,  &is_lmvm);CHKERRQ(ierr);
+  ierr = PetscStrcmp(type, TAOBLMVM, &is_blmvm);CHKERRQ(ierr);
+
+  if (is_lmvm) {
+    lmP = (TAO_LMVM *)tao->data;
+    M = lmP->M;
+  } else if (is_blmvm) {
+    blmP = (TAO_BLMVM *)tao->data;
+    M = blmP->M;
+  } else SETERRQ(PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_WRONGSTATE, "This routine applies to TAO_LMVM and TAO_BLMVM.");
+  ierr = MatLMVMGetJ0(M, H0);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode TaoLMVMGetH0KSP(Tao tao, KSP *ksp)
+{
+  TAO_LMVM       *lmP;
+  TAO_BLMVM      *blmP;
+  TaoType        type;
+  PetscBool      is_lmvm, is_blmvm;
+  Mat            M;
+  PetscErrorCode ierr;
+
+  ierr = TaoGetType(tao, &type);CHKERRQ(ierr);
+  ierr = PetscStrcmp(type, TAOLMVM,  &is_lmvm);CHKERRQ(ierr);
+  ierr = PetscStrcmp(type, TAOBLMVM, &is_blmvm);CHKERRQ(ierr);
+
+  if (is_lmvm) {
+    lmP = (TAO_LMVM *)tao->data;
+    M = lmP->M;
+  } else if (is_blmvm) {
+    blmP = (TAO_BLMVM *)tao->data;
+    M = blmP->M;
+  } else SETERRQ(PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_WRONGSTATE, "This routine applies to TAO_LMVM and TAO_BLMVM.");
+  ierr = MatLMVMGetJ0KSP(M, ksp);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
