@@ -1,7 +1,5 @@
 #include <../src/tao/complementarity/impls/ssls/ssls.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetUp_SSFLS"
 PetscErrorCode TaoSetUp_SSFLS(Tao tao)
 {
   TAO_SSLS       *ssls = (TAO_SSLS *)tao->data;
@@ -27,14 +25,11 @@ PetscErrorCode TaoSetUp_SSFLS(Tao tao)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSolve_SSFLS"
 static PetscErrorCode TaoSolve_SSFLS(Tao tao)
 {
   TAO_SSLS                     *ssls = (TAO_SSLS *)tao->data;
   PetscReal                    psi, ndpsi, normd, innerd, t=0;
   PetscReal                    delta, rho;
-  TaoConvergedReason           reason;
   TaoLineSearchConvergedReason ls_reason;
   PetscErrorCode               ierr;
 
@@ -55,11 +50,14 @@ static PetscErrorCode TaoSolve_SSFLS(Tao tao)
   ierr = TaoLineSearchComputeObjectiveAndGradient(tao->linesearch,tao->solution,&psi,ssls->dpsi);CHKERRQ(ierr);
   ierr = VecNorm(ssls->dpsi,NORM_2,&ndpsi);CHKERRQ(ierr);
 
+  tao->reason = TAO_CONTINUE_ITERATING;
   while (PETSC_TRUE) {
     ierr=PetscInfo3(tao, "iter: %D, merit: %g, ndpsi: %g\n",tao->niter, (double)ssls->merit, (double)ndpsi);CHKERRQ(ierr);
     /* Check the termination criteria */
-    ierr = TaoMonitor(tao,tao->niter,ssls->merit,ndpsi,0.0,t,&reason);CHKERRQ(ierr);
-    if (reason!=TAO_CONTINUE_ITERATING) break;
+    ierr = TaoLogConvergenceHistory(tao,ssls->merit,ndpsi,0.0,tao->ksp_its);CHKERRQ(ierr);
+    ierr = TaoMonitor(tao,tao->niter,ssls->merit,ndpsi,0.0,t);CHKERRQ(ierr);
+    ierr = (*tao->ops->convergencetest)(tao,tao->cnvP);CHKERRQ(ierr);
+    if (tao->reason!=TAO_CONTINUE_ITERATING) break;
     tao->niter++;
 
     /* Calculate direction.  (Really negative of newton direction.  Therefore,
@@ -93,8 +91,6 @@ static PetscErrorCode TaoSolve_SSFLS(Tao tao)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoDestroy_SSFLS"
 PetscErrorCode TaoDestroy_SSFLS(Tao tao)
 {
   TAO_SSLS       *ssls = (TAO_SSLS *)tao->data;
@@ -124,8 +120,6 @@ PetscErrorCode TaoDestroy_SSFLS(Tao tao)
    Level: beginner
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoCreate_SSFLS"
 PETSC_EXTERN PetscErrorCode TaoCreate_SSFLS(Tao tao)
 {
   TAO_SSLS       *ssls;
@@ -145,18 +139,18 @@ PETSC_EXTERN PetscErrorCode TaoCreate_SSFLS(Tao tao)
   ssls->rho = 2.1;
 
   ierr = TaoLineSearchCreate(((PetscObject)tao)->comm,&tao->linesearch);CHKERRQ(ierr);
+  ierr = PetscObjectIncrementTabLevel((PetscObject)tao->linesearch, (PetscObject)tao, 1);CHKERRQ(ierr);
   ierr = TaoLineSearchSetType(tao->linesearch,armijo_type);CHKERRQ(ierr);
   ierr = TaoLineSearchSetOptionsPrefix(tao->linesearch,tao->hdr.prefix);CHKERRQ(ierr);
   ierr = TaoLineSearchSetFromOptions(tao->linesearch);CHKERRQ(ierr);
   /* Linesearch objective and objectivegradient routines are  set in solve routine */
   ierr = KSPCreate(((PetscObject)tao)->comm,&tao->ksp);CHKERRQ(ierr);
+  ierr = PetscObjectIncrementTabLevel((PetscObject)tao->ksp, (PetscObject)tao, 1);CHKERRQ(ierr);
   ierr = KSPSetOptionsPrefix(tao->ksp,tao->hdr.prefix);CHKERRQ(ierr);
 
   /* Override default settings (unless already changed) */
   if (!tao->max_it_changed) tao->max_it = 2000;
   if (!tao->max_funcs_changed) tao->max_funcs = 4000;
-  if (!tao->fatol_changed) tao->fatol = 0;
-  if (!tao->frtol_changed) tao->frtol = 0;
   if (!tao->gttol_changed) tao->gttol = 0;
   if (!tao->grtol_changed) tao->grtol = 0;
 #if defined(PETSC_USE_REAL_SINGLE)
@@ -168,5 +162,3 @@ PETSC_EXTERN PetscErrorCode TaoCreate_SSFLS(Tao tao)
 #endif
   PetscFunctionReturn(0);
 }
-
-
