@@ -1,7 +1,5 @@
 #include <../src/tao/complementarity/impls/ssls/ssls.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetUp_SSILS"
 PetscErrorCode TaoSetUp_SSILS(Tao tao)
 {
   TAO_SSLS       *ssls = (TAO_SSLS *)tao->data;
@@ -19,8 +17,6 @@ PetscErrorCode TaoSetUp_SSILS(Tao tao)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoDestroy_SSILS"
 PetscErrorCode TaoDestroy_SSILS(Tao tao)
 {
   TAO_SSLS       *ssls = (TAO_SSLS *)tao->data;
@@ -37,14 +33,11 @@ PetscErrorCode TaoDestroy_SSILS(Tao tao)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSolve_SSILS"
 static PetscErrorCode TaoSolve_SSILS(Tao tao)
 {
   TAO_SSLS                     *ssls = (TAO_SSLS *)tao->data;
   PetscReal                    psi, ndpsi, normd, innerd, t=0;
   PetscReal                    delta, rho;
-  TaoConvergedReason           reason;
   TaoLineSearchConvergedReason ls_reason;
   PetscErrorCode               ierr;
 
@@ -64,11 +57,14 @@ static PetscErrorCode TaoSolve_SSILS(Tao tao)
   ierr = TaoLineSearchComputeObjectiveAndGradient(tao->linesearch,tao->solution,&psi,ssls->dpsi);CHKERRQ(ierr);
   ierr = VecNorm(ssls->dpsi,NORM_2,&ndpsi);CHKERRQ(ierr);
 
+  tao->reason = TAO_CONTINUE_ITERATING;
   while (PETSC_TRUE) {
     ierr=PetscInfo3(tao, "iter: %D, merit: %g, ndpsi: %g\n",tao->niter, (double)ssls->merit, (double)ndpsi);CHKERRQ(ierr);
     /* Check the termination criteria */
-    ierr = TaoMonitor(tao,tao->niter,ssls->merit,ndpsi,0.0,t,&reason);CHKERRQ(ierr);
-    if (reason!=TAO_CONTINUE_ITERATING) break;
+    ierr = TaoLogConvergenceHistory(tao,ssls->merit,ndpsi,0.0,tao->ksp_its);CHKERRQ(ierr);
+    ierr = TaoMonitor(tao,tao->niter,ssls->merit,ndpsi,0.0,t);CHKERRQ(ierr);
+    ierr = (*tao->ops->convergencetest)(tao,tao->cnvP);CHKERRQ(ierr);
+    if (tao->reason!=TAO_CONTINUE_ITERATING) break;
     tao->niter++;
 
     /* Calculate direction.  (Really negative of newton direction.  Therefore,
@@ -81,7 +77,7 @@ static PetscErrorCode TaoSolve_SSILS(Tao tao)
     ierr = VecDot(tao->stepdirection,ssls->dpsi,&innerd);CHKERRQ(ierr);
 
     /* Make sure that we have a descent direction */
-    if (innerd <= delta*pow(normd, rho)) {
+    if (innerd <= delta*PetscPowReal(normd, rho)) {
       ierr = PetscInfo(tao, "newton direction not descent\n");CHKERRQ(ierr);
       ierr = VecCopy(ssls->dpsi,tao->stepdirection);CHKERRQ(ierr);
       ierr = VecDot(tao->stepdirection,ssls->dpsi,&innerd);CHKERRQ(ierr);
@@ -108,8 +104,6 @@ static PetscErrorCode TaoSolve_SSILS(Tao tao)
 
    Level: beginner
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "TaoCreate_SSILS"
 PETSC_EXTERN PetscErrorCode TaoCreate_SSILS(Tao tao)
 {
   TAO_SSLS       *ssls;
@@ -129,18 +123,18 @@ PETSC_EXTERN PetscErrorCode TaoCreate_SSILS(Tao tao)
   ssls->rho = 2.1;
 
   ierr = TaoLineSearchCreate(((PetscObject)tao)->comm,&tao->linesearch);CHKERRQ(ierr);
+  ierr = PetscObjectIncrementTabLevel((PetscObject)tao->linesearch, (PetscObject)tao, 1);CHKERRQ(ierr);
   ierr = TaoLineSearchSetType(tao->linesearch,armijo_type);CHKERRQ(ierr);
   ierr = TaoLineSearchSetOptionsPrefix(tao->linesearch,tao->hdr.prefix);CHKERRQ(ierr);
   ierr = TaoLineSearchSetFromOptions(tao->linesearch);CHKERRQ(ierr);
   /* Note: linesearch objective and objectivegradient routines are set in solve routine */
   ierr = KSPCreate(((PetscObject)tao)->comm,&tao->ksp);CHKERRQ(ierr);
+  ierr = PetscObjectIncrementTabLevel((PetscObject)tao->ksp, (PetscObject)tao, 1);CHKERRQ(ierr);
   ierr = KSPSetOptionsPrefix(tao->ksp,tao->hdr.prefix);CHKERRQ(ierr);
 
   /* Override default settings (unless already changed) */
   if (!tao->max_it_changed) tao->max_it = 2000;
   if (!tao->max_funcs_changed) tao->max_funcs = 4000;
-  if (!tao->fatol_changed) tao->fatol = 0;
-  if (!tao->frtol_changed) tao->frtol = 0;
   if (!tao->gttol_changed) tao->gttol = 0;
   if (!tao->grtol_changed) tao->grtol = 0;
 #if defined(PETSC_USE_REAL_SINGLE)
@@ -152,5 +146,3 @@ PETSC_EXTERN PetscErrorCode TaoCreate_SSILS(Tao tao)
 #endif
   PetscFunctionReturn(0);
 }
-
-
