@@ -1,7 +1,5 @@
 #include <petsc/private/taoimpl.h> /*I "petsctao.h" I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetInitialVector"
 /*@
   TaoSetInitialVector - Sets the initial guess for the solve
 
@@ -30,8 +28,81 @@ PetscErrorCode TaoSetInitialVector(Tao tao, Vec x0)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoComputeGradient"
+PetscErrorCode TaoTestGradient(Tao tao,Vec x,Vec g1)
+{
+  Vec               g2,g3;
+  PetscBool         complete_print = PETSC_FALSE,test = PETSC_FALSE;
+  PetscReal         hcnorm,fdnorm,hcmax,fdmax,diffmax,diffnorm;
+  PetscScalar       dot;
+  MPI_Comm          comm;
+  PetscViewer       viewer,mviewer;
+  PetscViewerFormat format;
+  PetscInt          tabs;
+  static PetscBool  directionsprinted = PETSC_FALSE;
+  PetscErrorCode    ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectOptionsBegin((PetscObject)tao);CHKERRQ(ierr);
+  ierr = PetscOptionsName("-tao_test_gradient","Compare hand-coded and finite difference Gradients","None",&test);CHKERRQ(ierr);
+  ierr = PetscOptionsViewer("-tao_test_gradient_view","View difference between hand-coded and finite difference Gradients element entries","None",&mviewer,&format,&complete_print);CHKERRQ(ierr);
+  ierr = PetscOptionsEnd();CHKERRQ(ierr);
+  if (!test) PetscFunctionReturn(0);
+
+  ierr = PetscObjectGetComm((PetscObject)tao,&comm);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIGetStdout(comm,&viewer);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIGetTab(viewer, &tabs);CHKERRQ(ierr);
+  ierr = PetscViewerASCIISetTab(viewer, ((PetscObject)tao)->tablevel);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  ---------- Testing Gradient -------------\n");CHKERRQ(ierr);
+  if (!complete_print && !directionsprinted) {
+    ierr = PetscViewerASCIIPrintf(viewer,"  Run with -tao_test_gradient_view and optionally -tao_test_gradient <threshold> to show difference\n");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"    of hand-coded and finite difference gradient entries greater than <threshold>.\n");CHKERRQ(ierr);
+  }
+  if (!directionsprinted) {
+    ierr = PetscViewerASCIIPrintf(viewer,"  Testing hand-coded Gradient, if (for double precision runs) ||G - Gfd||_F/||G||_F is\n");CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"    O(1.e-8), the hand-coded Hessian is probably correct.\n");CHKERRQ(ierr);
+    directionsprinted = PETSC_TRUE;
+  }
+  if (complete_print) {
+    ierr = PetscViewerPushFormat(mviewer,format);CHKERRQ(ierr);
+  }
+
+  ierr = VecDuplicate(x,&g2);CHKERRQ(ierr);
+  ierr = VecDuplicate(x,&g3);CHKERRQ(ierr);
+
+  /* Compute finite difference gradient, assume the gradient is already computed by TaoComputeGradient() and put into g1 */
+  ierr = TaoDefaultComputeGradient(tao,x,g2,NULL);CHKERRQ(ierr);
+
+  ierr = VecNorm(g2,NORM_2,&fdnorm);CHKERRQ(ierr);
+  ierr = VecNorm(g1,NORM_2,&hcnorm);CHKERRQ(ierr);
+  ierr = VecNorm(g2,NORM_INFINITY,&fdmax);CHKERRQ(ierr);
+  ierr = VecNorm(g1,NORM_INFINITY,&hcmax);CHKERRQ(ierr);
+  ierr = VecDot(g1,g2,&dot);CHKERRQ(ierr);
+  ierr = VecCopy(g1,g3);CHKERRQ(ierr);
+  ierr = VecAXPY(g3,-1.0,g2);CHKERRQ(ierr);
+  ierr = VecNorm(g3,NORM_2,&diffnorm);CHKERRQ(ierr);
+  ierr = VecNorm(g3,NORM_INFINITY,&diffmax);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  ||Gfd|| %g, ||G|| = %g, angle cosine = (Gfd'G)/||Gfd||||G|| = %g\n", (double)fdnorm, (double)hcnorm, (double)(PetscRealPart(dot)/(fdnorm*hcnorm)));CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  2-norm ||G - Gfd||/||G|| = %g, ||G - Gfd|| = %g\n",(double)(diffnorm/PetscMax(hcnorm,fdnorm)),(double)diffnorm);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPrintf(viewer,"  max-norm ||G - Gfd||/||G|| = %g, ||G - Gfd|| = %g\n",(double)(diffmax/PetscMax(hcmax,fdmax)),(double)diffmax);CHKERRQ(ierr);
+
+  if (complete_print) {
+    ierr = PetscViewerASCIIPrintf(viewer,"  Hand-coded gradient ----------\n");CHKERRQ(ierr);
+    ierr = VecView(g1,mviewer);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  Finite difference gradient ----------\n");CHKERRQ(ierr);
+    ierr = VecView(g2,mviewer);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  Hand-coded minus finite-difference gradient ----------\n");CHKERRQ(ierr);
+    ierr = VecView(g3,mviewer);CHKERRQ(ierr);
+  }
+  ierr = VecDestroy(&g2);CHKERRQ(ierr);
+  ierr = VecDestroy(&g3);CHKERRQ(ierr);
+
+  if (complete_print) {
+    ierr = PetscViewerPopFormat(mviewer);CHKERRQ(ierr);
+  }
+  ierr = PetscViewerASCIISetTab(viewer,tabs);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
   TaoComputeGradient - Computes the gradient of the objective function
 
@@ -44,7 +115,12 @@ PetscErrorCode TaoSetInitialVector(Tao tao, Vec x0)
   Output Parameter:
 . G - gradient vector
 
-  Notes: TaoComputeGradient() is typically used within minimization implementations,
+  Options Database Keys:
++    -tao_test_gradient - compare the user provided gradient with one compute via finite differences to check for errors
+-    -tao_test_gradient_view - display the user provided gradient, the finite difference gradient and the difference between them to help users detect the location of errors in the user provided gradient
+
+  Notes:
+    TaoComputeGradient() is typically used within minimization implementations,
   so most users would not generally call this routine themselves.
 
   Level: advanced
@@ -62,26 +138,28 @@ PetscErrorCode TaoComputeGradient(Tao tao, Vec X, Vec G)
   PetscValidHeaderSpecific(G,VEC_CLASSID,2);
   PetscCheckSameComm(tao,1,X,2);
   PetscCheckSameComm(tao,1,G,3);
+  ierr = VecLockPush(X);CHKERRQ(ierr);
   if (tao->ops->computegradient) {
-    ierr = PetscLogEventBegin(Tao_GradientEval,tao,X,G,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventBegin(TAO_GradientEval,tao,X,G,NULL);CHKERRQ(ierr);
     PetscStackPush("Tao user gradient evaluation routine");
     ierr = (*tao->ops->computegradient)(tao,X,G,tao->user_gradP);CHKERRQ(ierr);
     PetscStackPop;
-    ierr = PetscLogEventEnd(Tao_GradientEval,tao,X,G,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(TAO_GradientEval,tao,X,G,NULL);CHKERRQ(ierr);
     tao->ngrads++;
   } else if (tao->ops->computeobjectiveandgradient) {
-    ierr = PetscLogEventBegin(Tao_ObjGradientEval,tao,X,G,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventBegin(TAO_ObjGradEval,tao,X,G,NULL);CHKERRQ(ierr);
     PetscStackPush("Tao user objective/gradient evaluation routine");
     ierr = (*tao->ops->computeobjectiveandgradient)(tao,X,&dummy,G,tao->user_objgradP);CHKERRQ(ierr);
     PetscStackPop;
-    ierr = PetscLogEventEnd(Tao_ObjGradientEval,tao,X,G,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(TAO_ObjGradEval,tao,X,G,NULL);CHKERRQ(ierr);
     tao->nfuncgrads++;
-  }  else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"TaoSetGradientRoutine() has not been called");
+  } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"TaoSetGradientRoutine() has not been called");
+  ierr = VecLockPop(X);CHKERRQ(ierr);
+
+  ierr = TaoTestGradient(tao,X,G);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoComputeObjective"
 /*@
   TaoComputeObjective - Computes the objective function value at a given point
 
@@ -94,7 +172,8 @@ PetscErrorCode TaoComputeGradient(Tao tao, Vec X, Vec G)
   Output Parameter:
 . f - Objective value at X
 
-  Notes: TaoComputeObjective() is typically used within minimization implementations,
+  Notes:
+    TaoComputeObjective() is typically used within minimization implementations,
   so most users would not generally call this routine themselves.
 
   Level: advanced
@@ -110,30 +189,30 @@ PetscErrorCode TaoComputeObjective(Tao tao, Vec X, PetscReal *f)
   PetscValidHeaderSpecific(tao,TAO_CLASSID,1);
   PetscValidHeaderSpecific(X,VEC_CLASSID,2);
   PetscCheckSameComm(tao,1,X,2);
+  ierr = VecLockPush(X);CHKERRQ(ierr);
   if (tao->ops->computeobjective) {
-    ierr = PetscLogEventBegin(Tao_ObjectiveEval,tao,X,NULL,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventBegin(TAO_ObjectiveEval,tao,X,NULL,NULL);CHKERRQ(ierr);
     PetscStackPush("Tao user objective evaluation routine");
     ierr = (*tao->ops->computeobjective)(tao,X,f,tao->user_objP);CHKERRQ(ierr);
     PetscStackPop;
-    ierr = PetscLogEventEnd(Tao_ObjectiveEval,tao,X,NULL,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(TAO_ObjectiveEval,tao,X,NULL,NULL);CHKERRQ(ierr);
     tao->nfuncs++;
   } else if (tao->ops->computeobjectiveandgradient) {
     ierr = PetscInfo(tao,"Duplicating variable vector in order to call func/grad routine\n");CHKERRQ(ierr);
     ierr = VecDuplicate(X,&temp);CHKERRQ(ierr);
-    ierr = PetscLogEventBegin(Tao_ObjGradientEval,tao,X,NULL,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventBegin(TAO_ObjGradEval,tao,X,NULL,NULL);CHKERRQ(ierr);
     PetscStackPush("Tao user objective/gradient evaluation routine");
     ierr = (*tao->ops->computeobjectiveandgradient)(tao,X,f,temp,tao->user_objgradP);CHKERRQ(ierr);
     PetscStackPop;
-    ierr = PetscLogEventEnd(Tao_ObjGradientEval,tao,X,NULL,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(TAO_ObjGradEval,tao,X,NULL,NULL);CHKERRQ(ierr);
     ierr = VecDestroy(&temp);CHKERRQ(ierr);
     tao->nfuncgrads++;
   }  else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"TaoSetObjectiveRoutine() has not been called");
-  ierr = PetscInfo1(tao,"TAO Function evaluation: %14.12e\n",(double)(*f));CHKERRQ(ierr);
+  ierr = PetscInfo1(tao,"TAO Function evaluation: %20.19e\n",(double)(*f));CHKERRQ(ierr);
+  ierr = VecLockPop(X);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoComputeObjectiveAndGradient"
 /*@
   TaoComputeObjectiveAndGradient - Computes the objective function value at a given point
 
@@ -147,7 +226,8 @@ PetscErrorCode TaoComputeObjective(Tao tao, Vec X, PetscReal *f)
 + f - Objective value at X
 - g - Gradient vector at X
 
-  Notes: TaoComputeObjectiveAndGradient() is typically used within minimization implementations,
+  Notes:
+    TaoComputeObjectiveAndGradient() is typically used within minimization implementations,
   so most users would not generally call this routine themselves.
 
   Level: advanced
@@ -164,37 +244,40 @@ PetscErrorCode TaoComputeObjectiveAndGradient(Tao tao, Vec X, PetscReal *f, Vec
   PetscValidHeaderSpecific(G,VEC_CLASSID,4);
   PetscCheckSameComm(tao,1,X,2);
   PetscCheckSameComm(tao,1,G,4);
+  ierr = VecLockPush(X);CHKERRQ(ierr);
   if (tao->ops->computeobjectiveandgradient) {
-    ierr = PetscLogEventBegin(Tao_ObjGradientEval,tao,X,G,NULL);CHKERRQ(ierr);
-    PetscStackPush("Tao user objective/gradient evaluation routine");
-    ierr = (*tao->ops->computeobjectiveandgradient)(tao,X,f,G,tao->user_objgradP);CHKERRQ(ierr);
-    PetscStackPop;
+    ierr = PetscLogEventBegin(TAO_ObjGradEval,tao,X,G,NULL);CHKERRQ(ierr);
     if (tao->ops->computegradient == TaoDefaultComputeGradient) {
-      /* Overwrite gradient with finite difference gradient */
-      ierr = TaoDefaultComputeGradient(tao,X,G,tao->user_objgradP);CHKERRQ(ierr);
+      ierr = TaoComputeObjective(tao,X,f);CHKERRQ(ierr);
+      ierr = TaoDefaultComputeGradient(tao,X,G,NULL);CHKERRQ(ierr);
+    } else {
+      PetscStackPush("Tao user objective/gradient evaluation routine");
+      ierr = (*tao->ops->computeobjectiveandgradient)(tao,X,f,G,tao->user_objgradP);CHKERRQ(ierr);
+      PetscStackPop;
     }
-    ierr = PetscLogEventEnd(Tao_ObjGradientEval,tao,X,G,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(TAO_ObjGradEval,tao,X,G,NULL);CHKERRQ(ierr);
     tao->nfuncgrads++;
   } else if (tao->ops->computeobjective && tao->ops->computegradient) {
-    ierr = PetscLogEventBegin(Tao_ObjectiveEval,tao,X,NULL,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventBegin(TAO_ObjectiveEval,tao,X,NULL,NULL);CHKERRQ(ierr);
     PetscStackPush("Tao user objective evaluation routine");
     ierr = (*tao->ops->computeobjective)(tao,X,f,tao->user_objP);CHKERRQ(ierr);
     PetscStackPop;
-    ierr = PetscLogEventEnd(Tao_ObjectiveEval,tao,X,NULL,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(TAO_ObjectiveEval,tao,X,NULL,NULL);CHKERRQ(ierr);
     tao->nfuncs++;
-    ierr = PetscLogEventBegin(Tao_GradientEval,tao,X,G,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventBegin(TAO_GradientEval,tao,X,G,NULL);CHKERRQ(ierr);
     PetscStackPush("Tao user gradient evaluation routine");
     ierr = (*tao->ops->computegradient)(tao,X,G,tao->user_gradP);CHKERRQ(ierr);
     PetscStackPop;
-    ierr = PetscLogEventEnd(Tao_GradientEval,tao,X,G,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(TAO_GradientEval,tao,X,G,NULL);CHKERRQ(ierr);
     tao->ngrads++;
   } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"TaoSetObjectiveRoutine() or TaoSetGradientRoutine() not set");
-  ierr = PetscInfo1(tao,"TAO Function evaluation: %14.12e\n",(double)(*f));CHKERRQ(ierr);
+  ierr = PetscInfo1(tao,"TAO Function evaluation: %20.19e\n",(double)(*f));CHKERRQ(ierr);
+  ierr = VecLockPop(X);CHKERRQ(ierr);
+
+  ierr = TaoTestGradient(tao,X,G);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetObjectiveRoutine"
 /*@C
   TaoSetObjectiveRoutine - Sets the function evaluation routine for minimization
 
@@ -226,8 +309,6 @@ PetscErrorCode TaoSetObjectiveRoutine(Tao tao, PetscErrorCode (*func)(Tao, Vec,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetSeparableObjectiveRoutine"
 /*@C
   TaoSetSeparableObjectiveRoutine - Sets the function evaluation routine for least-square applications
 
@@ -261,8 +342,60 @@ PetscErrorCode TaoSetSeparableObjectiveRoutine(Tao tao, Vec sepobj, PetscErrorCo
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoComputeSeparableObjective"
+/*@
+  TaoSetSeparableObjectiveWeights - Give weights for the separable objective values. A vector can be used if only diagonal terms are used, otherwise a matrix can be give. If this function is not used, or if sigma_v and sigma_w are both NULL, then the default identity matrix will be used for weights.
+
+  Collective on Tao
+
+  Input Parameters:
++ tao - the Tao context
+. sigma_v - vector of weights (diagonal terms only)
+. n       - the number of weights (if using off-diagonal)
+. rows    - index list of rows for sigma_w
+. cols    - index list of columns for sigma_w
+- vals - array of weights
+
+
+
+  Note: Either sigma_v or sigma_w (or both) should be NULL
+
+  Level: intermediate
+
+.seealso: TaoSetSeparableObjectiveRoutine()
+@*/
+PetscErrorCode TaoSetSeparableObjectiveWeights(Tao tao, Vec sigma_v, PetscInt n, PetscInt *rows, PetscInt *cols, PetscReal *vals)
+{
+  PetscErrorCode ierr;
+  PetscInt       i;
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(tao,TAO_CLASSID,1);
+  ierr = VecDestroy(&tao->sep_weights_v);CHKERRQ(ierr);
+  tao->sep_weights_v=sigma_v;
+  if (sigma_v) {
+    ierr = PetscObjectReference((PetscObject)sigma_v);CHKERRQ(ierr);
+  }
+  if (vals) {
+    if (tao->sep_weights_n) {
+      ierr = PetscFree(tao->sep_weights_rows);CHKERRQ(ierr);
+      ierr = PetscFree(tao->sep_weights_cols);CHKERRQ(ierr);
+      ierr = PetscFree(tao->sep_weights_w);CHKERRQ(ierr);
+    }
+    ierr = PetscMalloc1(n,&tao->sep_weights_rows);CHKERRQ(ierr);
+    ierr = PetscMalloc1(n,&tao->sep_weights_cols);CHKERRQ(ierr);
+    ierr = PetscMalloc1(n,&tao->sep_weights_w);CHKERRQ(ierr);
+    tao->sep_weights_n=n;
+    for (i=0;i<n;i++) {
+      tao->sep_weights_rows[i]=rows[i];
+      tao->sep_weights_cols[i]=cols[i];
+      tao->sep_weights_w[i]=vals[i];
+    }
+  } else {
+    tao->sep_weights_n=0;
+    tao->sep_weights_rows=0;
+    tao->sep_weights_cols=0;
+  }
+  PetscFunctionReturn(0);
+}
 /*@
   TaoComputeSeparableObjective - Computes a separable objective function vector at a given point (for least-square applications)
 
@@ -275,7 +408,8 @@ PetscErrorCode TaoSetSeparableObjectiveRoutine(Tao tao, Vec sepobj, PetscErrorCo
   Output Parameter:
 . f - Objective vector at X
 
-  Notes: TaoComputeSeparableObjective() is typically used within minimization implementations,
+  Notes:
+    TaoComputeSeparableObjective() is typically used within minimization implementations,
   so most users would not generally call this routine themselves.
 
   Level: advanced
@@ -293,19 +427,17 @@ PetscErrorCode TaoComputeSeparableObjective(Tao tao, Vec X, Vec F)
   PetscCheckSameComm(tao,1,X,2);
   PetscCheckSameComm(tao,1,F,3);
   if (tao->ops->computeseparableobjective) {
-    ierr = PetscLogEventBegin(Tao_ObjectiveEval,tao,X,NULL,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventBegin(TAO_ObjectiveEval,tao,X,NULL,NULL);CHKERRQ(ierr);
     PetscStackPush("Tao user separable objective evaluation routine");
     ierr = (*tao->ops->computeseparableobjective)(tao,X,F,tao->user_sepobjP);CHKERRQ(ierr);
     PetscStackPop;
-    ierr = PetscLogEventEnd(Tao_ObjectiveEval,tao,X,NULL,NULL);CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(TAO_ObjectiveEval,tao,X,NULL,NULL);CHKERRQ(ierr);
     tao->nfuncs++;
   } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"TaoSetSeparableObjectiveRoutine() has not been called");
   ierr = PetscInfo(tao,"TAO separable function evaluation.\n");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetGradientRoutine"
 /*@C
   TaoSetGradientRoutine - Sets the gradient evaluation routine for minimization
 
@@ -337,9 +469,6 @@ PetscErrorCode TaoSetGradientRoutine(Tao tao,  PetscErrorCode (*func)(Tao, Vec,
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetObjectiveAndGradientRoutine"
 /*@C
   TaoSetObjectiveAndGradientRoutine - Sets a combined objective function and gradient evaluation routine for minimization
 
@@ -372,8 +501,6 @@ PetscErrorCode TaoSetObjectiveAndGradientRoutine(Tao tao, PetscErrorCode (*func)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoIsObjectiveDefined"
 /*@
   TaoIsObjectiveDefined -- Checks to see if the user has
   declared an objective-only routine.  Useful for determining when
@@ -399,8 +526,6 @@ PetscErrorCode TaoIsObjectiveDefined(Tao tao, PetscBool *flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoIsGradientDefined"
 /*@
   TaoIsGradientDefined -- Checks to see if the user has
   declared an objective-only routine.  Useful for determining when
@@ -425,9 +550,6 @@ PetscErrorCode TaoIsGradientDefined(Tao tao, PetscBool *flg)
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "TaoIsObjectiveAndGradientDefined"
 /*@
   TaoIsObjectiveAndGradientDefined -- Checks to see if the user has
   declared a joint objective/gradient routine.  Useful for determining when
@@ -451,6 +573,3 @@ PetscErrorCode TaoIsObjectiveAndGradientDefined(Tao tao, PetscBool *flg)
   else *flg = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
-
-
-
