@@ -1,14 +1,11 @@
 #include <../src/tao/leastsquares/impls/pounders/pounders.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "pounders_h"
 static PetscErrorCode pounders_h(Tao subtao, Vec v, Mat H, Mat Hpre, void *ctx)
 {
   PetscFunctionBegin;
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__ "pounders_fg"
+
 static PetscErrorCode  pounders_fg(Tao subtao, Vec x, PetscReal *f, Vec g, void *ctx)
 {
   TAO_POUNDERS   *mfqP = (TAO_POUNDERS*)ctx;
@@ -29,8 +26,34 @@ static PetscErrorCode  pounders_fg(Tao subtao, Vec x, PetscReal *f, Vec g, void
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "gqtwrap"
+static PetscErrorCode pounders_feval(Tao tao, Vec x, Vec F, PetscReal *fsum)
+{
+  PetscErrorCode ierr;
+  TAO_POUNDERS   *mfqP = (TAO_POUNDERS*)tao->data;
+  PetscInt i,row,col;
+  PetscReal fr,fc;
+  PetscFunctionBegin;
+  ierr = TaoComputeSeparableObjective(tao,x,F);CHKERRQ(ierr);
+  if (tao->sep_weights_v) {
+    ierr = VecPointwiseMult(mfqP->workfvec,tao->sep_weights_v,F);CHKERRQ(ierr);
+    ierr = VecDot(mfqP->workfvec,mfqP->workfvec,fsum);CHKERRQ(ierr);
+  } else if (tao->sep_weights_w) {
+    *fsum=0;
+    for (i=0;i<tao->sep_weights_n;i++) {
+      row=tao->sep_weights_rows[i];
+      col=tao->sep_weights_cols[i];
+      ierr = VecGetValues(F,1,&row,&fr);CHKERRQ(ierr);
+      ierr = VecGetValues(F,1,&col,&fc);CHKERRQ(ierr);
+      *fsum += tao->sep_weights_w[i]*fc*fr;
+    }
+  } else {
+    ierr = VecDot(F,F,fsum);CHKERRQ(ierr);
+  }
+  ierr = PetscInfo1(tao,"Separable objective norm: %20.19e\n",(double)*fsum);CHKERRQ(ierr);
+  if (PetscIsInfOrNanReal(*fsum)) SETERRQ(PETSC_COMM_SELF,1, "User provided compute function generated Inf or NaN");
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode gqtwrap(Tao tao,PetscReal *gnorm, PetscReal *qmin)
 {
   PetscErrorCode ierr;
@@ -43,7 +66,7 @@ PetscErrorCode gqtwrap(Tao tao,PetscReal *gnorm, PetscReal *qmin)
   TAO_POUNDERS   *mfqP = (TAO_POUNDERS*)tao->data;
 
   PetscFunctionBegin;
-  if (! mfqP->usegqt) {
+  if (!mfqP->usegqt) {
     PetscReal maxval;
     PetscInt  i,j;
 
@@ -53,11 +76,12 @@ PetscErrorCode gqtwrap(Tao tao,PetscReal *gnorm, PetscReal *qmin)
 
     ierr = VecSet(mfqP->subx,0.0);CHKERRQ(ierr);
 
-    ierr = VecSet(mfqP->subndel,-mfqP->delta);CHKERRQ(ierr);
-    ierr = VecSet(mfqP->subpdel,mfqP->delta);CHKERRQ(ierr);
+    ierr = VecSet(mfqP->subndel,-1.0);CHKERRQ(ierr);
+    ierr = VecSet(mfqP->subpdel,+1.0);CHKERRQ(ierr);
 
+    /* Complete the lower triangle of the Hessian matrix */
     for (i=0;i<mfqP->n;i++) {
-      for (j=i;j<mfqP->n;j++) {
+      for (j=i+1;j<mfqP->n;j++) {
         mfqP->Hres[j+mfqP->n*i] = mfqP->Hres[mfqP->n*j+i];
       }
     }
@@ -66,7 +90,7 @@ PetscErrorCode gqtwrap(Tao tao,PetscReal *gnorm, PetscReal *qmin)
     ierr = MatAssemblyEnd(mfqP->subH,MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
 
     ierr = TaoResetStatistics(mfqP->subtao);CHKERRQ(ierr);
-    ierr = TaoSetTolerances(mfqP->subtao,PETSC_DEFAULT,PETSC_DEFAULT,*gnorm,*gnorm,PETSC_DEFAULT);CHKERRQ(ierr);
+    /* ierr = TaoSetTolerances(mfqP->subtao,*gnorm,*gnorm,PETSC_DEFAULT);CHKERRQ(ierr); */
     /* enforce bound constraints -- experimental */
     if (tao->XU && tao->XL) {
       ierr = VecCopy(tao->XU,mfqP->subxu);CHKERRQ(ierr);
@@ -124,8 +148,99 @@ PetscErrorCode gqtwrap(Tao tao,PetscReal *gnorm, PetscReal *qmin)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "phi2eval"
+static PetscErrorCode pounders_update_res(Tao tao)
+{
+  TAO_POUNDERS   *mfqP = (TAO_POUNDERS*)tao->data;
+  PetscInt i,row,col;
+  PetscBLASInt blasn=mfqP->n,blasn2=blasn*blasn,blasm=mfqP->m,ione=1;
+  PetscReal zero=0.0,one=1.0,wii,factor;
+  PetscErrorCode ierr;
+  PetscFunctionBegin;
+
+  for (i=0;i<mfqP->n;i++) {
+    mfqP->Gres[i]=0;
+  }
+  for (i=0;i<mfqP->n*mfqP->n;i++) {
+    mfqP->Hres[i]=0;
+  }
+
+  /* Compute Gres= sum_ij[wij * (cjgi + cigj)] */
+  if (tao->sep_weights_v) {
+    /* Vector(diagonal) weights: gres = sum_i(wii*ci*gi) */
+    for (i=0;i<mfqP->m;i++) {
+      ierr = VecGetValues(tao->sep_weights_v,1,&i,&factor);CHKERRQ(ierr);
+      factor=factor*mfqP->C[i];
+      PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&blasn,&factor,&mfqP->Fdiff[blasn*i],&ione,mfqP->Gres,&ione));
+    }
+
+    /* compute Hres = sum_ij [wij * (*ci*Hj + cj*Hi + gi gj' + gj gi') ] */
+    /* vector(diagonal weights) Hres = sum_i(wii*(ci*Hi + gi * gi' )*/
+    for (i=0;i<mfqP->m;i++) {
+      ierr = VecGetValues(tao->sep_weights_v,1,&i,&wii);CHKERRQ(ierr);
+      if (tao->niter>1) {
+        factor=wii*mfqP->C[i];
+        /* add wii * ci * Hi */
+        PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&blasn2,&factor,&mfqP->H[i],&blasm,mfqP->Hres,&ione));
+      }
+      /* add wii * gi * gi' */
+      PetscStackCallBLAS("BLASgemm",BLASgemm_("N","T",&blasn,&blasn,&ione,&wii,&mfqP->Fdiff[blasn*i],&blasn,&mfqP->Fdiff[blasn*i],&blasn,&one,mfqP->Hres,&blasn));
+    }
+  } else if (tao->sep_weights_w) {
+    /* General case: .5 * Gres= sum_ij[wij * (cjgi + cigj)] */
+    for (i=0;i<tao->sep_weights_n;i++) {
+      row=tao->sep_weights_rows[i];
+      col=tao->sep_weights_cols[i];
+
+      factor = tao->sep_weights_w[i]*mfqP->C[col]/2.0;
+      PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&blasn,&factor,&mfqP->Fdiff[blasn*row],&ione,mfqP->Gres,&ione));
+      factor = tao->sep_weights_w[i]*mfqP->C[row]/2.0;
+      PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&blasn,&factor,&mfqP->Fdiff[blasn*col],&ione,mfqP->Gres,&ione));
+    }
+
+    /* compute Hres = sum_ij [wij * (*ci*Hj + cj*Hi + gi gj' + gj gi') ] */
+    /* .5 * sum_ij [wij * (*ci*Hj + cj*Hi + gi gj' + gj gi') ] */
+    for (i=0;i<tao->sep_weights_n;i++) {
+      row=tao->sep_weights_rows[i];
+      col=tao->sep_weights_cols[i];
+      factor=tao->sep_weights_w[i]/2.0;
+      /* add wij * gi gj' + wij * gj gi' */
+      PetscStackCallBLAS("BLASgemm",BLASgemm_("N","T",&blasn,&blasn,&ione,&factor,&mfqP->Fdiff[blasn*row],&blasn,&mfqP->Fdiff[blasn*col],&blasn,&one,mfqP->Hres,&blasn));
+      PetscStackCallBLAS("BLASgemm",BLASgemm_("N","T",&blasn,&blasn,&ione,&factor,&mfqP->Fdiff[blasn*col],&blasn,&mfqP->Fdiff[blasn*row],&blasn,&one,mfqP->Hres,&blasn));
+    }
+    if (tao->niter > 1) {
+      for (i=0;i<tao->sep_weights_n;i++) {
+        row=tao->sep_weights_rows[i];
+        col=tao->sep_weights_cols[i];
+
+        /* add  wij*cj*Hi */
+        factor = tao->sep_weights_w[i]*mfqP->C[col]/2.0;
+        PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&blasn2,&factor,&mfqP->H[row],&blasm,mfqP->Hres,&ione));
+
+        /* add wij*ci*Hj */
+        factor = tao->sep_weights_w[i]*mfqP->C[row]/2.0;
+        PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&blasn2,&factor,&mfqP->H[col],&blasm,mfqP->Hres,&ione));
+      }
+    }
+  } else {
+    /* Default: Gres= sum_i[cigi] = G*c' */
+    ierr = PetscInfo(tao,"Identity weights\n");CHKERRQ(ierr);
+    PetscStackCallBLAS("BLASgemv",BLASgemv_("N",&blasn,&blasm,&one,mfqP->Fdiff,&blasn,mfqP->C,&ione,&zero,mfqP->Gres,&ione));
+
+    /* compute Hres = sum_ij [wij * (*ci*Hj + cj*Hi + gi gj' + gj gi') ] */
+    /*  Hres = G*G' + 0.5 sum {F(xkin,i)*H(:,:,i)}  */
+    PetscStackCallBLAS("BLASgemm",BLASgemm_("N","T",&blasn,&blasn,&blasm,&one,mfqP->Fdiff, &blasn,mfqP->Fdiff, &blasn,&zero,mfqP->Hres,&blasn));
+
+    /* sum(F(xkin,i)*H(:,:,i)) */
+    if (tao->niter>1) {
+      for (i=0;i<mfqP->m;i++) {
+        factor = mfqP->C[i];
+        PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&blasn2,&factor,&mfqP->H[i],&blasm,mfqP->Hres,&ione));
+      }
+    }
+  }
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode phi2eval(PetscReal *x, PetscInt n, PetscReal *phi)
 {
 /* Phi = .5*[x(1)^2  sqrt(2)*x(1)*x(2) ... sqrt(2)*x(1)*x(n) ... x(2)^2 sqrt(2)*x(2)*x(3) .. x(n)^2] */
@@ -145,8 +260,6 @@ PetscErrorCode phi2eval(PetscReal *x, PetscInt n, PetscReal *phi)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "getquadpounders"
 PetscErrorCode getquadpounders(TAO_POUNDERS *mfqP)
 {
 /* Computes the parameters of the quadratic Q(x) = c + g'*x + 0.5*x*G*x'
@@ -230,8 +343,6 @@ PetscErrorCode getquadpounders(TAO_POUNDERS *mfqP)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "morepoints"
 PetscErrorCode morepoints(TAO_POUNDERS *mfqP)
 {
   /* Assumes mfqP->model_indices[0]  is minimum index
@@ -326,7 +437,9 @@ PetscErrorCode morepoints(TAO_POUNDERS *mfqP)
 
     /* Get svd for L_tmp(:,n+2:np+1) (L_tmp is modified in process) */
     blasint = mfqP->nmodelpoints - mfqP->n;
+    ierr = PetscFPTrapPush(PETSC_FP_TRAP_OFF);CHKERRQ(ierr);
     PetscStackCallBLAS("LAPACKgesvd",LAPACKgesvd_("N","N",&blasint2,&blasint,&mfqP->L_tmp[(mfqP->n+1)*blasint2],&blasint2,mfqP->beta,mfqP->work,&blasn,mfqP->work,&blasn,mfqP->npmaxwork,&blasnmax,&info));
+    ierr = PetscFPTrapPop();CHKERRQ(ierr);
     if (info != 0) SETERRQ1(PETSC_COMM_SELF,1,"LAPACK routine gesvd returned with value %d\n",info);
 
     if (mfqP->beta[PetscMin(blasint,blasint2)-1] > mfqP->theta2) {
@@ -382,8 +495,6 @@ PetscErrorCode morepoints(TAO_POUNDERS *mfqP)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "addpoint"
 /* Only call from modelimprove, addpoint() needs ->Q_tmp and ->work to be set */
 PetscErrorCode addpoint(Tao tao, TAO_POUNDERS *mfqP, PetscInt index)
 {
@@ -405,10 +516,7 @@ PetscErrorCode addpoint(Tao tao, TAO_POUNDERS *mfqP, PetscInt index)
   /* Compute value of new vector */
   ierr = VecDuplicate(mfqP->Fhist[0],&mfqP->Fhist[mfqP->nHist]);CHKERRQ(ierr);
   CHKMEMQ;
-  ierr = TaoComputeSeparableObjective(tao,mfqP->Xhist[mfqP->nHist],mfqP->Fhist[mfqP->nHist]);CHKERRQ(ierr);
-  ierr = VecNorm(mfqP->Fhist[mfqP->nHist],NORM_2,&mfqP->Fres[mfqP->nHist]);CHKERRQ(ierr);
-  if (PetscIsInfOrNanReal(mfqP->Fres[mfqP->nHist])) SETERRQ(PETSC_COMM_SELF,1, "User provided compute function generated Inf or NaN");
-  mfqP->Fres[mfqP->nHist]*=mfqP->Fres[mfqP->nHist];
+  ierr = pounders_feval(tao,mfqP->Xhist[mfqP->nHist],mfqP->Fhist[mfqP->nHist],&mfqP->Fres[mfqP->nHist]);CHKERRQ(ierr);
 
   /* Add new vector to model */
   mfqP->model_indices[mfqP->nmodelpoints] = mfqP->nHist;
@@ -417,8 +525,6 @@ PetscErrorCode addpoint(Tao tao, TAO_POUNDERS *mfqP, PetscInt index)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "modelimprove"
 PetscErrorCode modelimprove(Tao tao, TAO_POUNDERS *mfqP, PetscInt addallpoints)
 {
   /* modeld = Q(:,np+1:n)' */
@@ -470,8 +576,6 @@ PetscErrorCode modelimprove(Tao tao, TAO_POUNDERS *mfqP, PetscInt addallpoints)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "affpoints"
 PetscErrorCode affpoints(TAO_POUNDERS *mfqP, PetscReal *xmin,PetscReal c)
 {
   PetscInt        i,j;
@@ -490,7 +594,7 @@ PetscErrorCode affpoints(TAO_POUNDERS *mfqP, PetscReal *xmin,PetscReal c)
     ierr = VecRestoreArrayRead(mfqP->Xhist[i],&x);CHKERRQ(ierr);
     PetscStackCallBLAS("BLAScopy",BLAScopy_(&blasn,mfqP->work,&ione,mfqP->work2,&ione));
     normd = BLASnrm2_(&blasn,mfqP->work,&ione);
-    if (normd <= c*c) {
+    if (normd <= c) {
       blasj=PetscMax((mfqP->n - mfqP->nmodelpoints),0);
       if (!mfqP->q_is_I) {
         /* project D onto null */
@@ -521,14 +625,11 @@ PetscErrorCode affpoints(TAO_POUNDERS *mfqP, PetscReal *xmin,PetscReal c)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSolve_POUNDERS"
 static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
 {
   TAO_POUNDERS       *mfqP = (TAO_POUNDERS *)tao->data;
   PetscInt           i,ii,j,k,l;
   PetscReal          step=1.0;
-  TaoConvergedReason reason = TAO_CONTINUE_ITERATING;
   PetscInt           low,high;
   PetscReal          minnorm;
   PetscReal          *x,*f;
@@ -539,7 +640,7 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
   PetscBool          valid,same;
   PetscReal          mdec, rho, normxsp;
   PetscReal          one=1.0,zero=0.0,ratio;
-  PetscBLASInt       blasm,blasn,blasnpmax,blasn2;
+  PetscBLASInt       blasm,blasn,blasncopy,blasnpmax;
   PetscErrorCode     ierr;
   static PetscBool   set = PETSC_FALSE;
 
@@ -562,6 +663,7 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
   if (tao->XL && tao->XU) {
     /* Check x0 <= XU */
     PetscReal val;
+
     ierr = VecCopy(tao->solution,mfqP->Xhist[0]);CHKERRQ(ierr);
     ierr = VecAXPY(mfqP->Xhist[0],-1.0,tao->XU);CHKERRQ(ierr);
     ierr = VecMax(mfqP->Xhist[0],NULL,&val);CHKERRQ(ierr);
@@ -582,35 +684,30 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
     if (val > 1e-10) SETERRQ(PETSC_COMM_WORLD,1,"X0 + delta > upper bound");
   }
 
-  CHKMEMQ;
   blasm = mfqP->m; blasn=mfqP->n; blasnpmax = mfqP->npmax;
-  for (i=0;i<mfqP->n*mfqP->n*mfqP->m;i++) mfqP->H[i]=0;
+  for (i=0;i<mfqP->n*mfqP->n*mfqP->m;++i) mfqP->H[i]=0;
 
   ierr = VecCopy(tao->solution,mfqP->Xhist[0]);CHKERRQ(ierr);
-  CHKMEMQ;
-  ierr = TaoComputeSeparableObjective(tao,tao->solution,mfqP->Fhist[0]);CHKERRQ(ierr);
 
-  ierr = VecNorm(mfqP->Fhist[0],NORM_2,&mfqP->Fres[0]);CHKERRQ(ierr);
-  if (PetscIsInfOrNanReal(mfqP->Fres[0])) SETERRQ(PETSC_COMM_SELF,1, "User provided compute function generated Inf or NaN");
-  mfqP->Fres[0]*=mfqP->Fres[0];
+  /* This provides enough information to approximate the gradient of the objective */
+  /* using a forward difference scheme. */
+
+  ierr = PetscInfo1(tao,"Initialize simplex; delta = %10.9e\n",(double)mfqP->delta);CHKERRQ(ierr);
+  ierr = pounders_feval(tao,mfqP->Xhist[0],mfqP->Fhist[0],&mfqP->Fres[0]);CHKERRQ(ierr);
   mfqP->minindex = 0;
   minnorm = mfqP->Fres[0];
-  ierr = TaoMonitor(tao, tao->niter, minnorm, PETSC_INFINITY, 0.0, step, &reason);CHKERRQ(ierr);
-  tao->niter++;
 
   ierr = VecGetOwnershipRange(mfqP->Xhist[0],&low,&high);CHKERRQ(ierr);
-  for (i=1;i<mfqP->n+1;i++) {
-    ierr = VecCopy(tao->solution,mfqP->Xhist[i]);CHKERRQ(ierr);
+  for (i=1;i<mfqP->n+1;++i) {
+    ierr = VecCopy(mfqP->Xhist[0],mfqP->Xhist[i]);CHKERRQ(ierr);
+
     if (i-1 >= low && i-1 < high) {
       ierr = VecGetArray(mfqP->Xhist[i],&x);CHKERRQ(ierr);
       x[i-1-low] += mfqP->delta;
       ierr = VecRestoreArray(mfqP->Xhist[i],&x);CHKERRQ(ierr);
     }
     CHKMEMQ;
-    ierr = TaoComputeSeparableObjective(tao,mfqP->Xhist[i],mfqP->Fhist[i]);CHKERRQ(ierr);
-    ierr = VecNorm(mfqP->Fhist[i],NORM_2,&mfqP->Fres[i]);CHKERRQ(ierr);
-    if (PetscIsInfOrNanReal(mfqP->Fres[i])) SETERRQ(PETSC_COMM_SELF,1, "User provided compute function generated Inf or NaN");
-    mfqP->Fres[i]*=mfqP->Fres[i];
+    ierr = pounders_feval(tao,mfqP->Xhist[i],mfqP->Fhist[i],&mfqP->Fres[i]);CHKERRQ(ierr);
     if (mfqP->Fres[i] < minnorm) {
       mfqP->minindex = i;
       minnorm = mfqP->Fres[i];
@@ -618,6 +715,8 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
   }
   ierr = VecCopy(mfqP->Xhist[mfqP->minindex],tao->solution);CHKERRQ(ierr);
   ierr = VecCopy(mfqP->Fhist[mfqP->minindex],tao->sep_objective);CHKERRQ(ierr);
+  ierr = PetscInfo1(tao,"Finalize simplex; minnorm = %10.9e\n",(double)minnorm);CHKERRQ(ierr);
+
   /* Gather mpi vecs to one big local vec */
 
   /* Begin serial code */
@@ -627,7 +726,8 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
   /* (Column oriented for blas calls) */
   ii=0;
 
-  if (mfqP->size == 1) {
+  ierr = PetscInfo1(tao,"Build matrix: %D\n",(PetscInt)mfqP->size);CHKERRQ(ierr);
+  if (1 == mfqP->size) {
     ierr = VecGetArrayRead(mfqP->Xhist[mfqP->minindex],&xmint);CHKERRQ(ierr);
     for (i=0;i<mfqP->n;i++) mfqP->xmin[i] = xmint[i];
     ierr = VecRestoreArrayRead(mfqP->Xhist[mfqP->minindex],&xmint);CHKERRQ(ierr);
@@ -646,16 +746,18 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
         mfqP->Fdiff[ii+mfqP->n*j] = f[j] - fmin[j];
       }
       ierr = VecRestoreArray(mfqP->Fhist[i],&f);CHKERRQ(ierr);
-      mfqP->model_indices[ii++] = i;
 
+      mfqP->model_indices[ii++] = i;
     }
     for (j=0;j<mfqP->m;j++) {
       mfqP->C[j] = fmin[j];
     }
     ierr = VecRestoreArrayRead(mfqP->Fhist[mfqP->minindex],&fmin);CHKERRQ(ierr);
   } else {
+    ierr = VecSet(mfqP->localxmin,0);CHKERRQ(ierr);
     ierr = VecScatterBegin(mfqP->scatterx,mfqP->Xhist[mfqP->minindex],mfqP->localxmin,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
     ierr = VecScatterEnd(mfqP->scatterx,mfqP->Xhist[mfqP->minindex],mfqP->localxmin,INSERT_VALUES,SCATTER_FORWARD);CHKERRQ(ierr);
+
     ierr = VecGetArrayRead(mfqP->localxmin,&xmint);CHKERRQ(ierr);
     for (i=0;i<mfqP->n;i++) mfqP->xmin[i] = xmint[i];
     ierr = VecRestoreArrayRead(mfqP->localxmin,&xmint);CHKERRQ(ierr);
@@ -666,12 +768,11 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
     for (i=0;i<mfqP->n+1;i++) {
       if (i == mfqP->minindex) continue;
 
-      mfqP->model_indices[ii++] = i;
       ierr = VecScatterBegin(mfqP->scatterx,mfqP->Xhist[ii],mfqP->localx,INSERT_VALUES, SCATTER_FORWARD);CHKERRQ(ierr);
       ierr = VecScatterEnd(mfqP->scatterx,mfqP->Xhist[ii],mfqP->localx,INSERT_VALUES, SCATTER_FORWARD);CHKERRQ(ierr);
       ierr = VecGetArray(mfqP->localx,&x);CHKERRQ(ierr);
       for (j=0;j<mfqP->n;j++) {
-        mfqP->Disp[i+mfqP->npmax*j] = (x[j] - mfqP->xmin[j])/mfqP->delta;
+        mfqP->Disp[ii+mfqP->npmax*j] = (x[j] - mfqP->xmin[j])/mfqP->delta;
       }
       ierr = VecRestoreArray(mfqP->localx,&x);CHKERRQ(ierr);
 
@@ -679,9 +780,11 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
       ierr = VecScatterEnd(mfqP->scatterf,mfqP->Fhist[ii],mfqP->localf,INSERT_VALUES, SCATTER_FORWARD);CHKERRQ(ierr);
       ierr = VecGetArray(mfqP->localf,&f);CHKERRQ(ierr);
       for (j=0;j<mfqP->m;j++) {
-        mfqP->Fdiff[i*mfqP->n+j] = f[j] - fmin[j];
+        mfqP->Fdiff[ii+mfqP->n*j] = f[j] - fmin[j];
       }
       ierr = VecRestoreArray(mfqP->localf,&f);CHKERRQ(ierr);
+
+      mfqP->model_indices[ii++] = i;
     }
     for (j=0;j<mfqP->m;j++) {
       mfqP->C[j] = fmin[j];
@@ -692,16 +795,12 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
   /* Determine the initial quadratic models */
   /* G = D(ModelIn,:) \ (F(ModelIn,1:m)-repmat(F(xkin,1:m),n,1)); */
   /* D (nxn) Fdiff (nxm)  => G (nxm) */
-  blasn2 = blasn;
-  PetscStackCallBLAS("LAPACKgesv",LAPACKgesv_(&blasn,&blasm,mfqP->Disp,&blasnpmax,mfqP->iwork,mfqP->Fdiff,&blasn2,&info));
-  ierr = PetscInfo1(tao,"gesv returned %d\n",info);CHKERRQ(ierr);
+  blasncopy = blasn;
+  PetscStackCallBLAS("LAPACKgesv",LAPACKgesv_(&blasn,&blasm,mfqP->Disp,&blasnpmax,mfqP->iwork,mfqP->Fdiff,&blasncopy,&info));
+  ierr = PetscInfo1(tao,"Linear solve return: %D\n",(PetscInt)info);CHKERRQ(ierr);
 
   cres = minnorm;
-  /* Gres = G*F(xkin,1:m)'  G (nxm)   Fk (m)   */
-  PetscStackCallBLAS("BLASgemv",BLASgemv_("N",&blasn,&blasm,&one,mfqP->Fdiff,&blasn,mfqP->C,&ione,&zero,mfqP->Gres,&ione));
-
-  /*  Hres = G*G'  */
-  PetscStackCallBLAS("BLASgemm",BLASgemm_("N","T",&blasn,&blasn,&blasm,&one,mfqP->Fdiff, &blasn,mfqP->Fdiff,&blasn,&zero,mfqP->Hres,&blasn));
+  ierr = pounders_update_res(tao);CHKERRQ(ierr);
 
   valid = PETSC_TRUE;
 
@@ -711,14 +810,20 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
   ierr = VecNorm(tao->gradient,NORM_2,&gnorm);CHKERRQ(ierr);
   gnorm *= mfqP->delta;
   ierr = VecCopy(mfqP->Xhist[mfqP->minindex],tao->solution);CHKERRQ(ierr);
-  ierr = TaoMonitor(tao, tao->niter, minnorm, gnorm, 0.0, step, &reason);CHKERRQ(ierr);
+  
+  tao->reason = TAO_CONTINUE_ITERATING;
+  ierr = TaoLogConvergenceHistory(tao,minnorm,gnorm,0.0,tao->ksp_its);CHKERRQ(ierr);
+  ierr = TaoMonitor(tao,tao->niter,minnorm,gnorm,0.0,step);CHKERRQ(ierr);
+  ierr = (*tao->ops->convergencetest)(tao,tao->cnvP);CHKERRQ(ierr);
+  
   mfqP->nHist = mfqP->n+1;
   mfqP->nmodelpoints = mfqP->n+1;
+  ierr = PetscInfo1(tao,"Initial gradient: %20.19e\n",(double)gnorm);CHKERRQ(ierr);
 
-  while (reason == TAO_CONTINUE_ITERATING) {
+  while (tao->reason == TAO_CONTINUE_ITERATING) {
     PetscReal gnm = 1e-4;
     tao->niter++;
-    /* Solve the subproblem min{Q(s): ||s|| <= delta} */
+    /* Solve the subproblem min{Q(s): ||s|| <= 1.0} */
     ierr = gqtwrap(tao,&gnm,&mdec);CHKERRQ(ierr);
     /* Evaluate the function at the new point */
 
@@ -731,10 +836,8 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
     ierr = VecAssemblyBegin(mfqP->Xhist[mfqP->nHist]);CHKERRQ(ierr);
     ierr = VecAssemblyEnd(mfqP->Xhist[mfqP->nHist]);CHKERRQ(ierr);
 
-    ierr = TaoComputeSeparableObjective(tao,mfqP->Xhist[mfqP->nHist],mfqP->Fhist[mfqP->nHist]);CHKERRQ(ierr);
-    ierr = VecNorm(mfqP->Fhist[mfqP->nHist],NORM_2,&mfqP->Fres[mfqP->nHist]);CHKERRQ(ierr);
-    if (PetscIsInfOrNanReal(mfqP->Fres[mfqP->nHist])) SETERRQ(PETSC_COMM_SELF,1, "User provided compute function generated Inf or NaN");
-    mfqP->Fres[mfqP->nHist]*=mfqP->Fres[mfqP->nHist];
+    ierr = pounders_feval(tao,mfqP->Xhist[mfqP->nHist],mfqP->Fhist[mfqP->nHist],&mfqP->Fres[mfqP->nHist]);CHKERRQ(ierr);
+
     rho = (mfqP->Fres[mfqP->minindex] - mfqP->Fres[mfqP->nHist]) / mdec;
     mfqP->nHist++;
 
@@ -804,11 +907,13 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
     /* Compute the next interpolation set */
     mfqP->q_is_I = 1;
     mfqP->nmodelpoints=0;
+    ierr = PetscInfo2(tao,"Affine Points: xmin = %20.19e, c1 = %20.19e\n",(double)*mfqP->xmin,(double)mfqP->c1);CHKERRQ(ierr);
     ierr = affpoints(mfqP,mfqP->xmin,mfqP->c1);CHKERRQ(ierr);
     if (mfqP->nmodelpoints == mfqP->n) {
       valid = PETSC_TRUE;
     } else {
       valid = PETSC_FALSE;
+      ierr = PetscInfo2(tao,"Affine Points: xmin = %20.19e, c2 = %20.19e\n",(double)*mfqP->xmin,(double)mfqP->c2);CHKERRQ(ierr);
       ierr = affpoints(mfqP,mfqP->xmin,mfqP->c2);CHKERRQ(ierr);
       if (mfqP->n > mfqP->nmodelpoints) {
         ierr = PetscInfo(tao,"Model not valid -- adding geometry points\n");CHKERRQ(ierr);
@@ -841,6 +946,7 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
     }
 
     /* Update the quadratic model */
+    ierr = PetscInfo2(tao,"Get Quad, size: %D, points: %D\n",mfqP->n,mfqP->nmodelpoints);CHKERRQ(ierr);
     ierr = getquadpounders(mfqP);CHKERRQ(ierr);
     ierr = VecGetArrayRead(mfqP->Fhist[mfqP->minindex],&fmin);CHKERRQ(ierr);
     PetscStackCallBLAS("BLAScopy",BLAScopy_(&blasm,fmin,&ione,mfqP->C,&ione));
@@ -849,7 +955,7 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
     iblas = blasm*blasn;
     PetscStackCallBLAS("BLASscal",BLASscal_(&iblas,&ratio,mfqP->Fdiff,&ione));
     PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&iblas,&one,mfqP->Gdel,&ione,mfqP->Fdiff,&ione));
-    /* H = H*(delta/deltaold) + Hdel */
+    /* H = H*(delta/deltaold)^2 + Hdel */
     iblas = blasm*blasn*blasn;
     ratio *= ratio;
     PetscStackCallBLAS("BLASscal",BLASscal_(&iblas,&ratio,mfqP->H,&ione));
@@ -857,16 +963,7 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
 
     /* Get residuals */
     cres = mfqP->Fres[mfqP->minindex];
-    /* Gres = G*F(xkin,1:m)' */
-    PetscStackCallBLAS("BLASgemv",BLASgemv_("N",&blasn,&blasm,&one,mfqP->Fdiff,&blasn,mfqP->C,&ione,&zero,mfqP->Gres,&ione));
-    /* Hres = sum i=1..m {F(xkin,i)*H(:,:,i)}   + G*G' */
-    PetscStackCallBLAS("BLASgemm",BLASgemm_("N","T",&blasn,&blasn,&blasm,&one,mfqP->Fdiff,&blasn,mfqP->Fdiff,&blasn,&zero,mfqP->Hres,&blasn));
-
-    iblas = mfqP->n*mfqP->n;
-
-    for (j=0;j<mfqP->m;j++) {
-      PetscStackCallBLAS("BLASaxpy",BLASaxpy_(&iblas,&fmin[j],&mfqP->H[j],&blasm,mfqP->Hres,&ione));
-    }
+    ierr = pounders_update_res(tao);CHKERRQ(ierr);
 
     /* Export solution and gradient residual to TAO */
     ierr = VecCopy(mfqP->Xhist[mfqP->minindex],tao->solution);CHKERRQ(ierr);
@@ -876,7 +973,9 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
     ierr = VecNorm(tao->gradient,NORM_2,&gnorm);CHKERRQ(ierr);
     gnorm *= mfqP->delta;
     /*  final criticality test */
-    ierr = TaoMonitor(tao, tao->niter, minnorm, gnorm, 0.0, step, &reason);CHKERRQ(ierr);
+    ierr = TaoLogConvergenceHistory(tao,minnorm,gnorm,0.0,tao->ksp_its);CHKERRQ(ierr);
+    ierr = TaoMonitor(tao,tao->niter,minnorm,gnorm,0.0,step);CHKERRQ(ierr);
+    ierr = (*tao->ops->convergencetest)(tao,tao->cnvP);CHKERRQ(ierr);
     /* test for repeated model */
     if (mfqP->nmodelpoints==mfqP->last_nmodelpoints) {
       same = PETSC_TRUE;
@@ -896,19 +995,16 @@ static PetscErrorCode TaoSolve_POUNDERS(Tao tao)
     mfqP->last_nmodelpoints = mfqP->nmodelpoints;
     if (same && mfqP->delta == deltaold) {
       ierr = PetscInfo(tao,"Identical model used in successive iterations\n");CHKERRQ(ierr);
-      reason = TAO_CONVERGED_STEPTOL;
       tao->reason = TAO_CONVERGED_STEPTOL;
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetUp_POUNDERS"
 static PetscErrorCode TaoSetUp_POUNDERS(Tao tao)
 {
   TAO_POUNDERS   *mfqP = (TAO_POUNDERS*)tao->data;
-  PetscInt       i;
+  PetscInt       i,j;
   IS             isfloc,isfglob,isxloc,isxglob;
   PetscErrorCode ierr;
 
@@ -924,16 +1020,17 @@ static PetscErrorCode TaoSetUp_POUNDERS(Tao tao)
   mfqP->npmax = PetscMin((mfqP->n+1)*(mfqP->n+2)/2,mfqP->npmax);
   mfqP->npmax = PetscMax(mfqP->npmax, mfqP->n+2);
 
-  ierr = PetscMalloc1(tao->max_funcs+10,&mfqP->Xhist);CHKERRQ(ierr);
-  ierr = PetscMalloc1(tao->max_funcs+10,&mfqP->Fhist);CHKERRQ(ierr);
-  for (i=0;i<mfqP->n +1;i++) {
+  ierr = PetscMalloc1(tao->max_funcs+100,&mfqP->Xhist);CHKERRQ(ierr);
+  ierr = PetscMalloc1(tao->max_funcs+100,&mfqP->Fhist);CHKERRQ(ierr);
+  for (i=0;i<mfqP->n+1;i++) {
     ierr = VecDuplicate(tao->solution,&mfqP->Xhist[i]);CHKERRQ(ierr);
     ierr = VecDuplicate(tao->sep_objective,&mfqP->Fhist[i]);CHKERRQ(ierr);
   }
   ierr = VecDuplicate(tao->solution,&mfqP->workxvec);CHKERRQ(ierr);
+  ierr = VecDuplicate(tao->sep_objective,&mfqP->workfvec);CHKERRQ(ierr);
   mfqP->nHist = 0;
 
-  ierr = PetscMalloc1(tao->max_funcs+10,&mfqP->Fres);CHKERRQ(ierr);
+  ierr = PetscMalloc1(tao->max_funcs+100,&mfqP->Fres);CHKERRQ(ierr);
   ierr = PetscMalloc1(mfqP->npmax*mfqP->m,&mfqP->RES);CHKERRQ(ierr);
   ierr = PetscMalloc1(mfqP->n,&mfqP->work);CHKERRQ(ierr);
   ierr = PetscMalloc1(mfqP->n,&mfqP->work2);CHKERRQ(ierr);
@@ -971,6 +1068,16 @@ static PetscErrorCode TaoSetUp_POUNDERS(Tao tao)
   ierr = PetscMalloc1(mfqP->n*mfqP->n*mfqP->m, &mfqP->Hdel);CHKERRQ(ierr);
   ierr = PetscMalloc1(PetscMax(mfqP->m,mfqP->n),&mfqP->indices);CHKERRQ(ierr);
   ierr = PetscMalloc1(mfqP->n,&mfqP->iwork);CHKERRQ(ierr);
+  ierr = PetscMalloc1(mfqP->m*mfqP->m,&mfqP->w);CHKERRQ(ierr);
+  for (i=0;i<mfqP->m;i++) {
+    for (j=0;j<mfqP->m;j++) {
+      if (i==j) {
+        mfqP->w[i+mfqP->m*j]=1.0;
+      } else {
+        mfqP->w[i+mfqP->m*j]=0.0;
+      }
+    }
+  }
   for (i=0;i<PetscMax(mfqP->m,mfqP->n);i++) {
     mfqP->indices[i] = i;
   }
@@ -1005,7 +1112,8 @@ static PetscErrorCode TaoSetUp_POUNDERS(Tao tao)
     ierr = VecDuplicate(mfqP->subxl,&mfqP->subpdel);CHKERRQ(ierr);
     ierr = VecDuplicate(mfqP->subxl,&mfqP->subndel);CHKERRQ(ierr);
     ierr = TaoCreate(PETSC_COMM_SELF,&mfqP->subtao);CHKERRQ(ierr);
-    ierr = TaoSetType(mfqP->subtao,TAOTRON);CHKERRQ(ierr);
+    ierr = PetscObjectIncrementTabLevel((PetscObject)mfqP->subtao, (PetscObject)tao, 1);CHKERRQ(ierr);
+    ierr = TaoSetType(mfqP->subtao,TAOBNTR);CHKERRQ(ierr);
     ierr = TaoSetOptionsPrefix(mfqP->subtao,"pounders_subsolver_");CHKERRQ(ierr);
     ierr = TaoSetInitialVector(mfqP->subtao,mfqP->subx);CHKERRQ(ierr);
     ierr = TaoSetObjectiveAndGradientRoutine(mfqP->subtao,pounders_fg,(void*)mfqP);CHKERRQ(ierr);
@@ -1023,8 +1131,6 @@ static PetscErrorCode TaoSetUp_POUNDERS(Tao tao)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoDestroy_POUNDERS"
 static PetscErrorCode TaoDestroy_POUNDERS(Tao tao)
 {
   TAO_POUNDERS   *mfqP = (TAO_POUNDERS*)tao->data;
@@ -1078,14 +1184,13 @@ static PetscErrorCode TaoDestroy_POUNDERS(Tao tao)
   ierr = PetscFree(mfqP->Hdel);CHKERRQ(ierr);
   ierr = PetscFree(mfqP->indices);CHKERRQ(ierr);
   ierr = PetscFree(mfqP->iwork);CHKERRQ(ierr);
-
+  ierr = PetscFree(mfqP->w);CHKERRQ(ierr);
   for (i=0;i<mfqP->nHist;i++) {
     ierr = VecDestroy(&mfqP->Xhist[i]);CHKERRQ(ierr);
     ierr = VecDestroy(&mfqP->Fhist[i]);CHKERRQ(ierr);
   }
-  if (mfqP->workxvec) {
-    ierr = VecDestroy(&mfqP->workxvec);CHKERRQ(ierr);
-  }
+  ierr = VecDestroy(&mfqP->workxvec);CHKERRQ(ierr);
+  ierr = VecDestroy(&mfqP->workfvec);CHKERRQ(ierr);
   ierr = PetscFree(mfqP->Xhist);CHKERRQ(ierr);
   ierr = PetscFree(mfqP->Fhist);CHKERRQ(ierr);
 
@@ -1099,9 +1204,7 @@ static PetscErrorCode TaoDestroy_POUNDERS(Tao tao)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetFromOptions_POUNDERS"
-static PetscErrorCode TaoSetFromOptions_POUNDERS(PetscOptions *PetscOptionsObject,Tao tao)
+static PetscErrorCode TaoSetFromOptions_POUNDERS(PetscOptionItems *PetscOptionsObject,Tao tao)
 {
   TAO_POUNDERS   *mfqP = (TAO_POUNDERS*)tao->data;
   PetscErrorCode ierr;
@@ -1110,38 +1213,29 @@ static PetscErrorCode TaoSetFromOptions_POUNDERS(PetscOptions *PetscOptionsObjec
   ierr = PetscOptionsHead(PetscOptionsObject,"POUNDERS method for least-squares optimization");CHKERRQ(ierr);
   ierr = PetscOptionsReal("-tao_pounders_delta","initial delta","",mfqP->delta,&mfqP->delta0,NULL);CHKERRQ(ierr);
   mfqP->delta = mfqP->delta0;
-  mfqP->npmax = PETSC_DEFAULT;
   ierr = PetscOptionsInt("-tao_pounders_npmax","max number of points in model","",mfqP->npmax,&mfqP->npmax,NULL);CHKERRQ(ierr);
-  mfqP->usegqt = PETSC_FALSE;
   ierr = PetscOptionsBool("-tao_pounders_gqt","use gqt algorithm for subproblem","",mfqP->usegqt,&mfqP->usegqt,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoView_POUNDERS"
 static PetscErrorCode TaoView_POUNDERS(Tao tao, PetscViewer viewer)
 {
   TAO_POUNDERS   *mfqP = (TAO_POUNDERS *)tao->data;
   PetscBool      isascii;
-  PetscInt       nits;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer, "initial delta: %g\n",(double)mfqP->delta0);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer, "final delta: %g\n",(double)mfqP->delta);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer, "model points: %D\n",mfqP->nmodelpoints);CHKERRQ(ierr);
     if (mfqP->usegqt) {
       ierr = PetscViewerASCIIPrintf(viewer, "subproblem solver: gqt\n");CHKERRQ(ierr);
     } else {
-      ierr = PetscViewerASCIIPrintf(viewer, "subproblem solver: %s\n",((PetscObject)mfqP->subtao)->type_name);CHKERRQ(ierr);
-      ierr = TaoGetTotalIterationNumber(mfqP->subtao,&nits);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPrintf(viewer, "total subproblem iterations: %D\n",nits);CHKERRQ(ierr);
+      ierr = TaoView(mfqP->subtao, viewer);CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -1154,11 +1248,9 @@ static PetscErrorCode TaoView_POUNDERS(Tao tao, PetscViewer viewer)
 - -tao_pounders_gqt - use gqt algorithm for subproblem instead of TRON
 
   Level: beginner
- 
+
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoCreate_POUNDERS"
 PETSC_EXTERN PetscErrorCode TaoCreate_POUNDERS(Tao tao)
 {
   TAO_POUNDERS   *mfqP = (TAO_POUNDERS*)tao->data;
@@ -1176,29 +1268,21 @@ PETSC_EXTERN PetscErrorCode TaoCreate_POUNDERS(Tao tao)
   /* Override default settings (unless already changed) */
   if (!tao->max_it_changed) tao->max_it = 2000;
   if (!tao->max_funcs_changed) tao->max_funcs = 4000;
-#if defined(PETSC_USE_REAL_SINGLE)
-  if (!tao->fatol_changed) tao->fatol = 1.0e-4;
-  if (!tao->frtol_changed) tao->frtol = 1.0e-4;
-  mfqP->deltamin=1e-3;
-#else
-  if (!tao->fatol_changed) tao->fatol = 1.0e-8;
-  if (!tao->frtol_changed) tao->frtol = 1.0e-8;
-  mfqP->deltamin=1e-6;
-#endif
+  mfqP->npmax = PETSC_DEFAULT;
   mfqP->delta0 = 0.1;
   mfqP->delta = 0.1;
   mfqP->deltamax=1e3;
-  mfqP->c2 = 100.0;
+  mfqP->deltamin=1e-6;
+  mfqP->c2 = 10.0;
   mfqP->theta1=1e-5;
   mfqP->theta2=1e-4;
   mfqP->gamma0=0.5;
   mfqP->gamma1=2.0;
   mfqP->eta0 = 0.0;
   mfqP->eta1 = 0.1;
+  mfqP->usegqt = PETSC_FALSE;
   mfqP->gqt_rtol = 0.001;
   mfqP->gqt_maxits = 50;
   mfqP->workxvec = 0;
   PetscFunctionReturn(0);
 }
-
-
