@@ -8,8 +8,6 @@
 #define REFERENCE_AVE  2
 #define REFERENCE_MEAN 3
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchDestroy_Armijo"
 static PetscErrorCode TaoLineSearchDestroy_Armijo(TaoLineSearch ls)
 {
   TaoLineSearch_ARMIJO *armP = (TaoLineSearch_ARMIJO *)ls->data;
@@ -23,24 +21,18 @@ static PetscErrorCode TaoLineSearchDestroy_Armijo(TaoLineSearch ls)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchReset_Armijo"
 static PetscErrorCode TaoLineSearchReset_Armijo(TaoLineSearch ls)
 {
   TaoLineSearch_ARMIJO *armP = (TaoLineSearch_ARMIJO *)ls->data;
   PetscErrorCode       ierr;
 
   PetscFunctionBegin;
-  if (armP->memory != NULL) {
-    ierr = PetscFree(armP->memory);CHKERRQ(ierr);
-  }
+  ierr = PetscFree(armP->memory);CHKERRQ(ierr);
   armP->memorySetup = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchSetFromOptions_Armijo"
-static PetscErrorCode TaoLineSearchSetFromOptions_Armijo(PetscOptions *PetscOptionsObject,TaoLineSearch ls)
+static PetscErrorCode TaoLineSearchSetFromOptions_Armijo(PetscOptionItems *PetscOptionsObject,TaoLineSearch ls)
 {
   TaoLineSearch_ARMIJO *armP = (TaoLineSearch_ARMIJO *)ls->data;
   PetscErrorCode       ierr;
@@ -59,8 +51,6 @@ static PetscErrorCode TaoLineSearchSetFromOptions_Armijo(PetscOptions *PetscOpti
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchView_Armijo"
 static PetscErrorCode TaoLineSearchView_Armijo(TaoLineSearch ls, PetscViewer pv)
 {
   TaoLineSearch_ARMIJO *armP = (TaoLineSearch_ARMIJO *)ls->data;
@@ -70,7 +60,6 @@ static PetscErrorCode TaoLineSearchView_Armijo(TaoLineSearch ls, PetscViewer pv)
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)pv, PETSCVIEWERASCII, &isascii);CHKERRQ(ierr);
   if (isascii) {
-    ierr = PetscViewerASCIIPrintf(pv,"  maxf=%D, ftol=%g, gtol=%g\n",ls->max_funcs, (double)ls->rtol, (double)ls->ftol);CHKERRQ(ierr);
     ierr=PetscViewerASCIIPrintf(pv,"  Armijo linesearch",armP->alpha);CHKERRQ(ierr);
     if (armP->nondescending) {
       ierr = PetscViewerASCIIPrintf(pv, " (nondescending)");CHKERRQ(ierr);
@@ -85,8 +74,6 @@ static PetscErrorCode TaoLineSearchView_Armijo(TaoLineSearch ls, PetscViewer pv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchApply_Armijo"
 /* @ TaoApply_Armijo - This routine performs a linesearch. It
    backtracks until the (nonmonotone) Armijo conditions are satisfied.
 
@@ -110,12 +97,13 @@ static PetscErrorCode TaoLineSearchApply_Armijo(TaoLineSearch ls, Vec x, PetscRe
 {
   TaoLineSearch_ARMIJO *armP = (TaoLineSearch_ARMIJO *)ls->data;
   PetscErrorCode       ierr;
-  PetscInt             i;
+  PetscInt             i,its=0;
   PetscReal            fact, ref, gdx;
   PetscInt             idx;
   PetscBool            g_computed=PETSC_FALSE; /* to prevent extra gradient computation */
 
   PetscFunctionBegin;
+  ierr = TaoLineSearchMonitor(ls, 0, *f, 0.0);CHKERRQ(ierr);
 
   ls->reason = TAOLINESEARCH_CONTINUE_ITERATING;
   if (!armP->work) {
@@ -221,6 +209,7 @@ static PetscErrorCode TaoLineSearchApply_Armijo(TaoLineSearch ls, Vec x, PetscRe
   ls->step = ls->initstep;
   while (ls->step >= ls->stepmin && (ls->nfeval+ls->nfgeval) < ls->max_funcs) {
     /* Calculate iterate */
+    ++its;
     ierr = VecCopy(x,armP->work);CHKERRQ(ierr);
     ierr = VecAXPY(armP->work,ls->step,s);CHKERRQ(ierr);
     if (ls->bounded) {
@@ -242,6 +231,8 @@ static PetscErrorCode TaoLineSearchApply_Armijo(TaoLineSearch ls, Vec x, PetscRe
       ls->f_fullstep = *f;
     }
 
+    ierr = TaoLineSearchMonitor(ls, its, *f, ls->step);CHKERRQ(ierr);
+
     if (PetscIsInfOrNanReal(*f)) {
       ls->step *= armP->beta_inf;
     } else {
@@ -293,8 +284,6 @@ static PetscErrorCode TaoLineSearchApply_Armijo(TaoLineSearch ls, Vec x, PetscRe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchCreate_Armijo"
 PETSC_EXTERN PetscErrorCode TaoLineSearchCreate_Armijo(TaoLineSearch ls)
 {
   TaoLineSearch_ARMIJO *armP;
