@@ -1,8 +1,6 @@
 
 #include <petsc/private/taolinesearchimpl.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchDestroy_Unit"
 static PetscErrorCode TaoLineSearchDestroy_Unit(TaoLineSearch ls)
 {
   PetscErrorCode ierr;
@@ -11,9 +9,7 @@ static PetscErrorCode TaoLineSearchDestroy_Unit(TaoLineSearch ls)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchSetFromOptions_Unit"
-static PetscErrorCode TaoLineSearchSetFromOptions_Unit(PetscOptions *PetscOptionsObject,TaoLineSearch ls)
+static PetscErrorCode TaoLineSearchSetFromOptions_Unit(PetscOptionItems *PetscOptionsObject,TaoLineSearch ls)
 {
   PetscErrorCode ierr;
   PetscFunctionBegin;
@@ -22,8 +18,6 @@ static PetscErrorCode TaoLineSearchSetFromOptions_Unit(PetscOptions *PetscOption
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchView_Unit"
 static PetscErrorCode TaoLineSearchView_Unit(TaoLineSearch ls,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -37,8 +31,6 @@ static PetscErrorCode TaoLineSearchView_Unit(TaoLineSearch ls,PetscViewer viewer
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchApply_Unit"
 static PetscErrorCode TaoLineSearchApply_Unit(TaoLineSearch ls,Vec x,PetscReal *f,Vec g,Vec step_direction)
 {
   PetscErrorCode ierr;
@@ -47,8 +39,10 @@ static PetscErrorCode TaoLineSearchApply_Unit(TaoLineSearch ls,Vec x,PetscReal *
 
   PetscFunctionBegin;
   /* Take unit step (newx = startx + 1.0*step_direction) */
+  ierr = TaoLineSearchMonitor(ls, 0, *f, 0.0);CHKERRQ(ierr);
   ierr = VecAXPY(x,1.0,step_direction);CHKERRQ(ierr);
   ierr = TaoLineSearchComputeObjectiveAndGradient(ls,x,&ftry,g);CHKERRQ(ierr);
+  ierr = TaoLineSearchMonitor(ls, 1, *f, 1.0);CHKERRQ(ierr);
   ierr = PetscInfo1(ls,"Tao Apply Unit Step: %4.4e\n",1.0);CHKERRQ(ierr);
   if (startf < ftry){
     ierr = PetscInfo2(ls,"Tao Apply Unit Step, FINCREASE: F old:= %12.10e, F new: %12.10e\n",(double)startf,(double)ftry);CHKERRQ(ierr);
@@ -59,8 +53,6 @@ static PetscErrorCode TaoLineSearchApply_Unit(TaoLineSearch ls,Vec x,PetscReal *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchCreate_Unit"
 /*@C
    TaoCreateUnitLineSearch - Always use step length of 1.0
 
