@@ -1,14 +1,13 @@
 #include <petsctaolinesearch.h> /*I "petsctaolinesearch.h" I*/
 #include <petsc/private/taolinesearchimpl.h>
 
-PetscBool TaoLineSearchInitialized = PETSC_FALSE;
 PetscFunctionList TaoLineSearchList = NULL;
 
 PetscClassId TAOLINESEARCH_CLASSID=0;
-PetscLogEvent TaoLineSearch_ApplyEvent = 0, TaoLineSearch_EvalEvent=0;
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchView"
+PetscLogEvent TAOLINESEARCH_Apply;
+PetscLogEvent TAOLINESEARCH_Eval;
+
 /*@C
   TaoLineSearchView - Prints information about the TaoLineSearch
 
@@ -38,13 +37,11 @@ PetscErrorCode TaoLineSearchView(TaoLineSearch ls, PetscViewer viewer)
 {
   PetscErrorCode          ierr;
   PetscBool               isascii, isstring;
-  const TaoLineSearchType type;
-  PetscBool               destroyviewer = PETSC_FALSE;
+  TaoLineSearchType       type;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ls,TAOLINESEARCH_CLASSID,1);
   if (!viewer) {
-    destroyviewer = PETSC_TRUE;
     ierr = PetscViewerASCIIGetStdout(((PetscObject)ls)->comm, &viewer);CHKERRQ(ierr);
   }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
@@ -53,23 +50,13 @@ PetscErrorCode TaoLineSearchView(TaoLineSearch ls, PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERSTRING,&isstring);CHKERRQ(ierr);
   if (isascii) {
-    if (((PetscObject)ls)->prefix) {
-      ierr = PetscViewerASCIIPrintf(viewer,"TaoLineSearch Object:(%s)\n",((PetscObject)ls)->prefix);CHKERRQ(ierr);
-    } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"TaoLineSearch Object:\n");CHKERRQ(ierr);
-    }
-    ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
-    ierr = TaoLineSearchGetType(ls,&type);CHKERRQ(ierr);
-    if (type) {
-      ierr = PetscViewerASCIIPrintf(viewer,"type: %s\n",type);CHKERRQ(ierr);
-    } else {
-      ierr = PetscViewerASCIIPrintf(viewer,"type: not set yet\n");CHKERRQ(ierr);
-    }
+    ierr = PetscObjectPrintClassNamePrefixType((PetscObject)ls, viewer);CHKERRQ(ierr);
     if (ls->ops->view) {
       ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
       ierr = (*ls->ops->view)(ls,viewer);CHKERRQ(ierr);
       ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
     }
+    ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"maximum function evaluations=%D\n",ls->max_funcs);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"tolerances: ftol=%g, rtol=%g, gtol=%g\n",(double)ls->ftol,(double)ls->rtol,(double)ls->gtol);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"total number of function evaluations=%D\n",ls->nfeval);CHKERRQ(ierr);
@@ -79,21 +66,15 @@ PetscErrorCode TaoLineSearchView(TaoLineSearch ls, PetscViewer viewer)
     if (ls->bounded) {
       ierr = PetscViewerASCIIPrintf(viewer,"using variable bounds\n");CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIIPrintf(viewer,"Termination reason: %D\n",(int)ls->reason);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"Termination reason: %d\n",(int)ls->reason);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
-
   } else if (isstring) {
     ierr = TaoLineSearchGetType(ls,&type);CHKERRQ(ierr);
     ierr = PetscViewerStringSPrintf(viewer," %-3.3s",type);CHKERRQ(ierr);
   }
-  if (destroyviewer) {
-    ierr = PetscViewerDestroy(&viewer);CHKERRQ(ierr);
-  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchCreate"
 /*@C
   TaoLineSearchCreate - Creates a TAO Line Search object.  Algorithms in TAO that use
   line-searches will automatically create one.
@@ -129,9 +110,7 @@ PetscErrorCode TaoLineSearchCreate(MPI_Comm comm, TaoLineSearch *newls)
   PetscValidPointer(newls,2);
   *newls = NULL;
 
- #ifndef PETSC_USE_DYNAMIC_LIBRARIES
   ierr = TaoLineSearchInitializePackage();CHKERRQ(ierr);
- #endif
 
   ierr = PetscHeaderCreate(ls,TAOLINESEARCH_CLASSID,"TaoLineSearch","Linesearch","Tao",comm,TaoLineSearchDestroy,TaoLineSearchView);CHKERRQ(ierr);
   ls->bounded = 0;
@@ -160,14 +139,14 @@ PetscErrorCode TaoLineSearchCreate(MPI_Comm comm, TaoLineSearch *newls)
   ls->ops->setfromoptions=0;
   ls->ops->reset=0;
   ls->ops->destroy=0;
+  ls->ops->monitor=0;
+  ls->usemonitor=PETSC_FALSE;
   ls->setupcalled=PETSC_FALSE;
   ls->usetaoroutines=PETSC_FALSE;
   *newls = ls;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchSetUp"
 /*@
   TaoLineSearchSetUp - Sets up the internal data structures for the later use
   of a Tao solver
@@ -231,8 +210,6 @@ PetscErrorCode TaoLineSearchSetUp(TaoLineSearch ls)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchReset"
 /*@
   TaoLineSearchReset - Some line searches may carry state information
   from one TaoLineSearchApply() to the next.  This function resets this
@@ -259,8 +236,6 @@ PetscErrorCode TaoLineSearchReset(TaoLineSearch ls)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchDestroy"
 /*@
   TaoLineSearchDestroy - Destroys the TAO context that was created with
   TaoLineSearchCreate()
@@ -287,12 +262,13 @@ PetscErrorCode TaoLineSearchDestroy(TaoLineSearch *ls)
   if ((*ls)->ops->destroy) {
     ierr = (*(*ls)->ops->destroy)(*ls);CHKERRQ(ierr);
   }
+  if ((*ls)->usemonitor) {
+    ierr = PetscViewerDestroy(&(*ls)->viewer);CHKERRQ(ierr);
+  }
   ierr = PetscHeaderDestroy(ls);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchApply"
 /*@
   TaoLineSearchApply - Performs a line-search in a given step direction.  Criteria for acceptable step length depends on the line-search algorithm chosen
 
@@ -403,9 +379,9 @@ PetscErrorCode TaoLineSearchApply(TaoLineSearch ls, Vec x, PetscReal *f, Vec g,
   ierr = VecDestroy(&ls->start_x);CHKERRQ(ierr);
   ls->start_x = x;
 
-  ierr = PetscLogEventBegin(TaoLineSearch_ApplyEvent,ls,0,0,0);CHKERRQ(ierr);
+  ierr = PetscLogEventBegin(TAOLINESEARCH_Apply,ls,0,0,0);CHKERRQ(ierr);
   ierr = (*ls->ops->apply)(ls,x,f,g,s);CHKERRQ(ierr);
-  ierr = PetscLogEventEnd(TaoLineSearch_ApplyEvent, ls, 0,0,0);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(TAOLINESEARCH_Apply, ls, 0,0,0);CHKERRQ(ierr);
   *reason=ls->reason;
   ls->new_f = *f;
 
@@ -417,8 +393,6 @@ PetscErrorCode TaoLineSearchApply(TaoLineSearch ls, Vec x, PetscReal *f, Vec g,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchSetType"
 /*@C
    TaoLineSearchSetType - Sets the algorithm used in a line search
 
@@ -444,7 +418,7 @@ PetscErrorCode TaoLineSearchApply(TaoLineSearch ls, Vec x, PetscReal *f, Vec g,
 
 @*/
 
-PetscErrorCode TaoLineSearchSetType(TaoLineSearch ls, const TaoLineSearchType type)
+PetscErrorCode TaoLineSearchSetType(TaoLineSearch ls, TaoLineSearchType type)
 {
   PetscErrorCode ierr;
   PetscErrorCode (*r)(TaoLineSearch);
@@ -486,8 +460,45 @@ PetscErrorCode TaoLineSearchSetType(TaoLineSearch ls, const TaoLineSearchType ty
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchSetFromOptions"
+/*@C
+  TaoLineSearchMonitor - Monitor the line search steps. This routine will otuput the
+  iteration number, step length, and function value before calling the implementation 
+  specific monitor.
+
+   Input Parameters:
++  ls - the TaoLineSearch context
+.  its - the current iterate number (>=0)
+.  f - the current objective function value
+-  step - the step length
+
+   Options Database Key:
+.  -tao_ls_monitor - Use the default monitor, which prints statistics to standard output
+
+   Level: developer
+
+@*/
+PetscErrorCode TaoLineSearchMonitor(TaoLineSearch ls, PetscInt its, PetscReal f, PetscReal step)
+{
+  PetscErrorCode ierr;
+  PetscInt       tabs;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(ls,TAOLINESEARCH_CLASSID,1);
+  if (ls->usemonitor) {
+    ierr = PetscViewerASCIIGetTab(ls->viewer, &tabs);CHKERRQ(ierr);
+    ierr = PetscViewerASCIISetTab(ls->viewer, ((PetscObject)ls)->tablevel);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(ls->viewer, "%3D LS", its);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(ls->viewer, "  Function value: %g,", (double)f);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(ls->viewer, "  Step lenght: %g\n", (double)step);CHKERRQ(ierr);
+    if (ls->ops->monitor && its > 0) {
+      ierr = PetscViewerASCIISetTab(ls->viewer, ((PetscObject)ls)->tablevel + 3);CHKERRQ(ierr);
+      ierr = (*ls->ops->monitor)(ls);CHKERRQ(ierr);
+    }
+    ierr = PetscViewerASCIISetTab(ls->viewer, tabs);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 /*@
   TaoLineSearchSetFromOptions - Sets various TaoLineSearch parameters from user
   options.
@@ -513,15 +524,13 @@ PetscErrorCode TaoLineSearchSetFromOptions(TaoLineSearch ls)
 {
   PetscErrorCode ierr;
   const char     *default_type=TAOLINESEARCHMT;
-  char           type[256];
+  char           type[256],monfilename[PETSC_MAX_PATH_LEN];
+  PetscViewer    monviewer;
   PetscBool      flg;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ls,TAOLINESEARCH_CLASSID,1);
   ierr = PetscObjectOptionsBegin((PetscObject)ls);CHKERRQ(ierr);
-  if (!TaoLineSearchInitialized) {
-    ierr = TaoLineSearchInitializePackage();CHKERRQ(ierr);
-  }
   if (((PetscObject)ls)->type_name) {
     default_type = ((PetscObject)ls)->type_name;
   }
@@ -539,6 +548,12 @@ PetscErrorCode TaoLineSearchSetFromOptions(TaoLineSearch ls)
   ierr = PetscOptionsReal("-tao_ls_rtol","relative tol for acceptable step","",ls->rtol,&ls->rtol,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsReal("-tao_ls_stepmin","lower bound for step","",ls->stepmin,&ls->stepmin,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsReal("-tao_ls_stepmax","upper bound for step","",ls->stepmax,&ls->stepmax,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsString("-tao_ls_monitor","enable the basic monitor","TaoLineSearchSetMonitor","stdout",monfilename,PETSC_MAX_PATH_LEN,&flg);CHKERRQ(ierr);
+  if (flg) {
+    ierr = PetscViewerASCIIOpen(PetscObjectComm((PetscObject)ls),monfilename,&monviewer);CHKERRQ(ierr);
+    ls->viewer = monviewer;
+    ls->usemonitor = PETSC_TRUE;
+  }
   if (ls->ops->setfromoptions) {
     ierr = (*ls->ops->setfromoptions)(PetscOptionsObject,ls);CHKERRQ(ierr);
   }
@@ -546,8 +561,6 @@ PetscErrorCode TaoLineSearchSetFromOptions(TaoLineSearch ls)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchGetType"
 /*@C
   TaoLineSearchGetType - Gets the current line search algorithm
 
@@ -562,7 +575,7 @@ PetscErrorCode TaoLineSearchSetFromOptions(TaoLineSearch ls)
   Level: developer
 
 @*/
-PetscErrorCode TaoLineSearchGetType(TaoLineSearch ls, const TaoLineSearchType *type)
+PetscErrorCode TaoLineSearchGetType(TaoLineSearch ls, TaoLineSearchType *type)
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ls,TAOLINESEARCH_CLASSID,1);
@@ -571,8 +584,6 @@ PetscErrorCode TaoLineSearchGetType(TaoLineSearch ls, const TaoLineSearchType *t
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchGetNumberFunctionEvaluations"
 /*@
   TaoLineSearchGetNumberFunctionEvaluations - Gets the number of function and gradient evaluation
   routines used by the line search in last application (not cumulative).
@@ -605,8 +616,6 @@ PetscErrorCode TaoLineSearchGetNumberFunctionEvaluations(TaoLineSearch ls, Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchIsUsingTaoRoutines"
 /*@
   TaoLineSearchIsUsingTaoRoutines - Checks whether the line search is using
   Tao evaluation routines.
@@ -630,8 +639,6 @@ PetscErrorCode TaoLineSearchIsUsingTaoRoutines(TaoLineSearch ls, PetscBool *flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchSetObjectiveRoutine"
 /*@C
   TaoLineSearchSetObjectiveRoutine - Sets the function evaluation routine for the line search
 
@@ -675,8 +682,6 @@ PetscErrorCode TaoLineSearchSetObjectiveRoutine(TaoLineSearch ls, PetscErrorCode
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchSetGradientRoutine"
 /*@C
   TaoLineSearchSetGradientRoutine - Sets the gradient evaluation routine for the line search
 
@@ -719,8 +724,6 @@ PetscErrorCode TaoLineSearchSetGradientRoutine(TaoLineSearch ls, PetscErrorCode(
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchSetObjectiveAndGradientRoutine"
 /*@C
   TaoLineSearchSetObjectiveAndGradientRoutine - Sets the objective/gradient evaluation routine for the line search
 
@@ -763,8 +766,6 @@ PetscErrorCode TaoLineSearchSetObjectiveAndGradientRoutine(TaoLineSearch ls, Pet
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchSetObjectiveAndGTSRoutine"
 /*@C
   TaoLineSearchSetObjectiveAndGTSRoutine - Sets the objective and
   (gradient'*stepdirection) evaluation routine for the line search.
@@ -816,8 +817,6 @@ PetscErrorCode TaoLineSearchSetObjectiveAndGTSRoutine(TaoLineSearch ls, PetscErr
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchUseTaoRoutines"
 /*@C
   TaoLineSearchUseTaoRoutines - Informs the TaoLineSearch to use the
   objective and gradient evaluation routines from the given Tao object.
@@ -842,8 +841,6 @@ PetscErrorCode TaoLineSearchUseTaoRoutines(TaoLineSearch ls, Tao ts)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchComputeObjective"
 /*@
   TaoLineSearchComputeObjective - Computes the objective function value at a given point
 
@@ -856,7 +853,8 @@ PetscErrorCode TaoLineSearchUseTaoRoutines(TaoLineSearch ls, Tao ts)
   Output Parameter:
 . f - Objective value at X
 
-  Notes: TaoLineSearchComputeObjective() is typically used within line searches
+  Notes:
+    TaoLineSearchComputeObjective() is typically used within line searches
   so most users would not generally call this routine themselves.
 
   Level: developer
@@ -877,8 +875,8 @@ PetscErrorCode TaoLineSearchComputeObjective(TaoLineSearch ls, Vec x, PetscReal
   if (ls->usetaoroutines) {
     ierr = TaoComputeObjective(ls->tao,x,f);CHKERRQ(ierr);
   } else {
-    ierr = PetscLogEventBegin(TaoLineSearch_EvalEvent,ls,0,0,0);CHKERRQ(ierr);
     if (!ls->ops->computeobjective && !ls->ops->computeobjectiveandgradient && !ls->ops->computeobjectiveandgts) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Line Search does not have objective function set");
+    ierr = PetscLogEventBegin(TAOLINESEARCH_Eval,ls,0,0,0);CHKERRQ(ierr);
     PetscStackPush("TaoLineSearch user objective routine");
     if (ls->ops->computeobjective) {
       ierr = (*ls->ops->computeobjective)(ls,x,f,ls->userctx_func);CHKERRQ(ierr);
@@ -890,14 +888,12 @@ PetscErrorCode TaoLineSearchComputeObjective(TaoLineSearch ls, Vec x, PetscReal
       ierr = (*ls->ops->computeobjectiveandgts)(ls,x,ls->stepdirection,f,&gts,ls->userctx_funcgts);CHKERRQ(ierr);
     }
     PetscStackPop;
-    ierr = PetscLogEventEnd(TaoLineSearch_EvalEvent,ls,0,0,0);CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(TAOLINESEARCH_Eval,ls,0,0,0);CHKERRQ(ierr);
   }
   ls->nfeval++;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchComputeObjectiveAndGradient"
 /*@
   TaoLineSearchComputeObjectiveAndGradient - Computes the objective function value at a given point
 
@@ -911,7 +907,8 @@ PetscErrorCode TaoLineSearchComputeObjective(TaoLineSearch ls, Vec x, PetscReal
 + f - Objective value at X
 - g - Gradient vector at X
 
-  Notes: TaoLineSearchComputeObjectiveAndGradient() is typically used within line searches
+  Notes:
+    TaoLineSearchComputeObjectiveAndGradient() is typically used within line searches
   so most users would not generally call this routine themselves.
 
   Level: developer
@@ -932,10 +929,9 @@ PetscErrorCode TaoLineSearchComputeObjectiveAndGradient(TaoLineSearch ls, Vec x,
   if (ls->usetaoroutines) {
       ierr = TaoComputeObjectiveAndGradient(ls->tao,x,f,g);CHKERRQ(ierr);
   } else {
-    ierr = PetscLogEventBegin(TaoLineSearch_EvalEvent,ls,0,0,0);CHKERRQ(ierr);
     if (!ls->ops->computeobjective && !ls->ops->computeobjectiveandgradient) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Line Search does not have objective function set");
-    if (!ls->ops->computegradient && !ls->ops->computeobjectiveandgradient) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Line Search does not have gradient function set");
-
+    if (!ls->ops->computegradient  && !ls->ops->computeobjectiveandgradient) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Line Search does not have gradient function set");
+    ierr = PetscLogEventBegin(TAOLINESEARCH_Eval,ls,0,0,0);CHKERRQ(ierr);
     PetscStackPush("TaoLineSearch user objective/gradient routine");
     if (ls->ops->computeobjectiveandgradient) {
       ierr = (*ls->ops->computeobjectiveandgradient)(ls,x,f,g,ls->userctx_funcgrad);CHKERRQ(ierr);
@@ -944,15 +940,13 @@ PetscErrorCode TaoLineSearchComputeObjectiveAndGradient(TaoLineSearch ls, Vec x,
       ierr = (*ls->ops->computegradient)(ls,x,g,ls->userctx_grad);CHKERRQ(ierr);
     }
     PetscStackPop;
-    ierr = PetscLogEventEnd(TaoLineSearch_EvalEvent,ls,0,0,0);CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(TAOLINESEARCH_Eval,ls,0,0,0);CHKERRQ(ierr);
     ierr = PetscInfo1(ls,"TaoLineSearch Function evaluation: %14.12e\n",(double)(*f));CHKERRQ(ierr);
   }
   ls->nfgeval++;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchComputeGradient"
 /*@
   TaoLineSearchComputeGradient - Computes the gradient of the objective function
 
@@ -965,7 +959,8 @@ PetscErrorCode TaoLineSearchComputeObjectiveAndGradient(TaoLineSearch ls, Vec x,
   Output Parameter:
 . g - gradient vector
 
-  Notes: TaoComputeGradient() is typically used within line searches
+  Notes:
+    TaoComputeGradient() is typically used within line searches
   so most users would not generally call this routine themselves.
 
   Level: developer
@@ -986,8 +981,8 @@ PetscErrorCode TaoLineSearchComputeGradient(TaoLineSearch ls, Vec x, Vec g)
   if (ls->usetaoroutines) {
     ierr = TaoComputeGradient(ls->tao,x,g);CHKERRQ(ierr);
   } else {
-    ierr = PetscLogEventBegin(TaoLineSearch_EvalEvent,ls,0,0,0);CHKERRQ(ierr);
     if (!ls->ops->computegradient && !ls->ops->computeobjectiveandgradient) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Line Search does not have gradient functions set");
+    ierr = PetscLogEventBegin(TAOLINESEARCH_Eval,ls,0,0,0);CHKERRQ(ierr);
     PetscStackPush("TaoLineSearch user gradient routine");
     if (ls->ops->computegradient) {
       ierr = (*ls->ops->computegradient)(ls,x,g,ls->userctx_grad);CHKERRQ(ierr);
@@ -995,14 +990,12 @@ PetscErrorCode TaoLineSearchComputeGradient(TaoLineSearch ls, Vec x, Vec g)
       ierr = (*ls->ops->computeobjectiveandgradient)(ls,x,&fdummy,g,ls->userctx_funcgrad);CHKERRQ(ierr);
     }
     PetscStackPop;
-    ierr = PetscLogEventEnd(TaoLineSearch_EvalEvent,ls,0,0,0);CHKERRQ(ierr);
+    ierr = PetscLogEventEnd(TAOLINESEARCH_Eval,ls,0,0,0);CHKERRQ(ierr);
   }
   ls->ngeval++;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchComputeObjectiveAndGTS"
 /*@
   TaoLineSearchComputeObjectiveAndGTS - Computes the objective function value and inner product of gradient and step direction at a given point
 
@@ -1016,7 +1009,8 @@ PetscErrorCode TaoLineSearchComputeGradient(TaoLineSearch ls, Vec x, Vec g)
 + f - Objective value at X
 - gts - inner product of gradient and step direction at X
 
-  Notes: TaoLineSearchComputeObjectiveAndGTS() is typically used within line searches
+  Notes:
+    TaoLineSearchComputeObjectiveAndGTS() is typically used within line searches
   so most users would not generally call this routine themselves.
 
   Level: developer
@@ -1032,19 +1026,17 @@ PetscErrorCode TaoLineSearchComputeObjectiveAndGTS(TaoLineSearch ls, Vec x, Pets
   PetscValidPointer(f,3);
   PetscValidPointer(gts,4);
   PetscCheckSameComm(ls,1,x,2);
-  ierr = PetscLogEventBegin(TaoLineSearch_EvalEvent,ls,0,0,0);CHKERRQ(ierr);
   if (!ls->ops->computeobjectiveandgts) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Line Search does not have objective and gts function set");
+  ierr = PetscLogEventBegin(TAOLINESEARCH_Eval,ls,0,0,0);CHKERRQ(ierr);
   PetscStackPush("TaoLineSearch user objective/gts routine");
   ierr = (*ls->ops->computeobjectiveandgts)(ls,x,ls->stepdirection,f,gts,ls->userctx_funcgts);CHKERRQ(ierr);
   PetscStackPop;
-  ierr = PetscLogEventEnd(TaoLineSearch_EvalEvent,ls,0,0,0);CHKERRQ(ierr);
+  ierr = PetscLogEventEnd(TAOLINESEARCH_Eval,ls,0,0,0);CHKERRQ(ierr);
   ierr = PetscInfo1(ls,"TaoLineSearch Function evaluation: %14.12e\n",(double)(*f));CHKERRQ(ierr);
   ls->nfeval++;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchGetSolution"
 /*@
   TaoLineSearchGetSolution - Returns the solution to the line search
 
@@ -1102,8 +1094,6 @@ PetscErrorCode TaoLineSearchGetSolution(TaoLineSearch ls, Vec x, PetscReal *f, V
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchGetStartingVector"
 /*@
   TaoLineSearchGetStartingVector - Gets a the initial point of the line
   search.
@@ -1128,8 +1118,6 @@ PetscErrorCode TaoLineSearchGetStartingVector(TaoLineSearch ls, Vec *x)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchGetStepDirection"
 /*@
   TaoLineSearchGetStepDirection - Gets the step direction of the line
   search.
@@ -1155,8 +1143,6 @@ PetscErrorCode TaoLineSearchGetStepDirection(TaoLineSearch ls, Vec *s)
 
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchGetFullStepObjective"
 /*@
   TaoLineSearchGetFullStepObjective - Returns the objective function value at the full step.  Useful for some minimization algorithms.
 
@@ -1179,8 +1165,6 @@ PetscErrorCode TaoLineSearchGetFullStepObjective(TaoLineSearch ls, PetscReal *f_
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchSetVariableBounds"
 /*@
   TaoLineSearchSetVariableBounds - Sets the upper and lower bounds.
 
@@ -1210,8 +1194,6 @@ PetscErrorCode TaoLineSearchSetVariableBounds(TaoLineSearch ls,Vec xl, Vec xu)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchSetInitialStepLength"
 /*@
   TaoLineSearchSetInitialStepLength - Sets the initial step length of a line
   search.  If this value is not set then 1.0 is assumed.
@@ -1234,8 +1216,6 @@ PetscErrorCode TaoLineSearchSetInitialStepLength(TaoLineSearch ls,PetscReal s)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchGetStepLength"
 /*@
   TaoLineSearchGetStepLength - Get the current step length
 
@@ -1259,9 +1239,7 @@ PetscErrorCode TaoLineSearchGetStepLength(TaoLineSearch ls,PetscReal *s)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchRegister"
-/*MC
+/*@C
    TaoLineSearchRegister - Adds a line-search algorithm to the registry
 
    Not collective
@@ -1285,39 +1263,16 @@ $     -tao_ls_type my_linesearch
 
    Level: developer
 
-.seealso: TaoLineSearchRegisterDestroy()
-M*/
+@*/
 PetscErrorCode TaoLineSearchRegister(const char sname[], PetscErrorCode (*func)(TaoLineSearch))
 {
   PetscErrorCode ierr;
   PetscFunctionBegin;
+  ierr = TaoLineSearchInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&TaoLineSearchList, sname, (void (*)(void))func);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchRegisterDestroy"
-/*@C
-   TaoLineSearchRegisterDestroy - Frees the list of line-search algorithms that were
-   registered by TaoLineSearchRegister().
-
-   Not Collective
-
-   Level: developer
-
-.seealso: TaoLineSearchRegister()
-@*/
-PetscErrorCode TaoLineSearchRegisterDestroy(void)
-{
-  PetscErrorCode ierr;
-  PetscFunctionBegin;
-  ierr = PetscFunctionListDestroy(&TaoLineSearchList);CHKERRQ(ierr);
-  TaoLineSearchInitialized = PETSC_FALSE;
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchAppendOptionsPrefix"
 /*@C
    TaoLineSearchAppendOptionsPrefix - Appends to the prefix used for searching
    for all TaoLineSearch options in the database.
@@ -1343,8 +1298,6 @@ PetscErrorCode TaoLineSearchAppendOptionsPrefix(TaoLineSearch ls, const char p[]
   return PetscObjectAppendOptionsPrefix((PetscObject)ls,p);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchGetOptionsPrefix"
 /*@C
   TaoLineSearchGetOptionsPrefix - Gets the prefix used for searching for all
   TaoLineSearch options in the database
@@ -1357,7 +1310,8 @@ PetscErrorCode TaoLineSearchAppendOptionsPrefix(TaoLineSearch ls, const char p[]
   Output Parameters:
 . prefix - pointer to the prefix string used is returned
 
-  Notes: On the fortran side, the user should pass in a string 'prefix' of
+  Notes:
+    On the fortran side, the user should pass in a string 'prefix' of
   sufficient length to hold the prefix.
 
   Level: advanced
@@ -1369,8 +1323,6 @@ PetscErrorCode TaoLineSearchGetOptionsPrefix(TaoLineSearch ls, const char *p[])
   return PetscObjectGetOptionsPrefix((PetscObject)ls,p);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoLineSearchSetOptionsPrefix"
 /*@C
    TaoLineSearchSetOptionsPrefix - Sets the prefix used for searching for all
    TaoLineSearch options in the database.
