@@ -18,6 +18,8 @@
    Processors: n
 T*/
 
+
+
 typedef struct {
   PetscInt n; /* Number of total variables */
   PetscInt m; /* Number of constraints */
@@ -115,8 +117,6 @@ PetscErrorCode QMatMultTranspose(Mat,Vec,Vec);
 
 static  char help[]="";
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc, char **argv)
 {
   PetscErrorCode     ierr;
@@ -126,7 +126,7 @@ int main(int argc, char **argv)
   PetscInt           ntests = 1;
   PetscInt           i;
 
-  PetscInitialize(&argc, &argv, (char*)0,help);
+  ierr = PetscInitialize(&argc, &argv, (char*)0,help);if (ierr) return ierr;
   user.mx = 8;
   ierr = PetscOptionsBegin(PETSC_COMM_WORLD,NULL,NULL,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsInt("-mx","Number of grid points in each direction","",user.mx,&user.mx,NULL);CHKERRQ(ierr);
@@ -191,12 +191,10 @@ int main(int argc, char **argv)
   ierr = TaoDestroy(&tao);CHKERRQ(ierr);
   ierr = VecDestroy(&x0);CHKERRQ(ierr);
   ierr = EllipticDestroy(&user);CHKERRQ(ierr);
-  PetscFinalize();
-  return 0;
+  ierr = PetscFinalize();
+  return ierr;
 }
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "FormFunction"
 /*
    dwork = Qy - d
    lwork = L*(u-ur)
@@ -221,8 +219,6 @@ PetscErrorCode FormFunction(Tao tao,Vec X,PetscReal *f,void *ptr)
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "FormGradient"
 /*
     state: g_s = Q' *(Qy - d)
     design: g_d = alpha*L'*L*(u-ur)
@@ -245,8 +241,6 @@ PetscErrorCode FormGradient(Tao tao,Vec X,Vec G,void *ptr)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FormFunctionGradient"
 PetscErrorCode FormFunctionGradient(Tao tao, Vec X, PetscReal *f, Vec G, void *ptr)
 {
   PetscErrorCode ierr;
@@ -271,8 +265,6 @@ PetscErrorCode FormFunctionGradient(Tao tao, Vec X, PetscReal *f, Vec G, void *p
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "FormJacobianState"
 /* A
 MatShell object
 */
@@ -301,8 +293,6 @@ PetscErrorCode FormJacobianState(Tao tao, Vec X, Mat J, Mat JPre, Mat JInv, void
   PetscFunctionReturn(0);
 }
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "FormJacobianDesign"
 /* B */
 PetscErrorCode FormJacobianDesign(Tao tao, Vec X, Mat J, void *ptr)
 {
@@ -314,8 +304,6 @@ PetscErrorCode FormJacobianDesign(Tao tao, Vec X, Mat J, void *ptr)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "StateBlockMatMult"
 PetscErrorCode StateBlockMatMult(Mat J_shell, Vec X, Vec Y)
 {
   PetscErrorCode ierr;
@@ -331,8 +319,6 @@ PetscErrorCode StateBlockMatMult(Mat J_shell, Vec X, Vec Y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "StateMatMult"
 PetscErrorCode StateMatMult(Mat J_shell, Vec X, Vec Y)
 {
   PetscErrorCode ierr;
@@ -354,8 +340,6 @@ PetscErrorCode StateMatMult(Mat J_shell, Vec X, Vec Y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "StateInvMatMult"
 PetscErrorCode StateInvMatMult(Mat J_shell, Vec X, Vec Y)
 {
   PetscErrorCode ierr;
@@ -387,8 +371,6 @@ PetscErrorCode StateInvMatMult(Mat J_shell, Vec X, Vec Y)
   }
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__ "QMatMult"
 PetscErrorCode QMatMult(Mat J_shell, Vec X, Vec Y)
 {
   PetscErrorCode ierr;
@@ -410,8 +392,6 @@ PetscErrorCode QMatMult(Mat J_shell, Vec X, Vec Y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "QMatMultTranspose"
 PetscErrorCode QMatMultTranspose(Mat J_shell, Vec X, Vec Y)
 {
   PetscErrorCode ierr;
@@ -433,8 +413,6 @@ PetscErrorCode QMatMultTranspose(Mat J_shell, Vec X, Vec Y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DesignMatMult"
 PetscErrorCode DesignMatMult(Mat J_shell, Vec X, Vec Y)
 {
   PetscErrorCode ierr;
@@ -483,8 +461,6 @@ PetscErrorCode DesignMatMult(Mat J_shell, Vec X, Vec Y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DesignMatMultTranspose"
 PetscErrorCode DesignMatMultTranspose(Mat J_shell, Vec X, Vec Y)
 {
   PetscErrorCode ierr;
@@ -531,8 +507,6 @@ PetscErrorCode DesignMatMultTranspose(Mat J_shell, Vec X, Vec Y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FormConstraints"
 PetscErrorCode FormConstraints(Tao tao, Vec X, Vec C, void *ptr)
 {
    /* C=Ay - q      A = Div * Sigma * Grad + hx*hx*hx*ones(n,n) */
@@ -570,8 +544,6 @@ PetscErrorCode FormConstraints(Tao tao, Vec X, Vec C, void *ptr)
    PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "Scatter"
 PetscErrorCode Scatter(Vec x, Vec sub1, VecScatter scat1, Vec sub2, VecScatter scat2)
 {
   PetscErrorCode ierr;
@@ -586,8 +558,6 @@ PetscErrorCode Scatter(Vec x, Vec sub1, VecScatter scat1, Vec sub2, VecScatter s
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "Gather"
 PetscErrorCode Gather(Vec x, Vec sub1, VecScatter scat1, Vec sub2, VecScatter scat2)
 {
   PetscErrorCode ierr;
@@ -602,8 +572,6 @@ PetscErrorCode Gather(Vec x, Vec sub1, VecScatter scat1, Vec sub2, VecScatter sc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EllipticInitialize"
 PetscErrorCode EllipticInitialize(AppCtx *user)
 {
   PetscErrorCode ierr;
@@ -726,7 +694,7 @@ PetscErrorCode EllipticInitialize(AppCtx *user)
   ierr = VecGetLocalSize(user->subd,&dsubnlocal);CHKERRQ(ierr);
   ierr = VecSetSizes(user->d,dsubnlocal*user->ns,user->ndata*user->ns);CHKERRQ(ierr);
   ierr = VecSetFromOptions(user->d);CHKERRQ(ierr);
-  ierr = PetscMalloc1(user->ns,&user->di_scatter);
+  ierr = PetscMalloc1(user->ns,&user->di_scatter);CHKERRQ(ierr);
 
   ierr = VecGetOwnershipRange(user->d,&lo2,&hi2);CHKERRQ(ierr);
   istart = 0;
@@ -802,7 +770,7 @@ PetscErrorCode EllipticInitialize(AppCtx *user)
             for (kk=0;kk<nrank;kk++) istart+=user->ns*(subranges[kk+1]-subranges[kk]);
             istart += (subranges[nrank+1]-subranges[nrank])*subvec;
             l = istart+offset;
-            v = 100*PetscSinScalar(2*PETSC_PI*(vx+0.25*is))*sin(2*PETSC_PI*(vy+0.25*js))*sin(2*PETSC_PI*(vz+0.25*ks));
+            v = 100*PetscSinScalar(2*PETSC_PI*(vx+0.25*is))*PetscSinScalar(2*PETSC_PI*(vy+0.25*js))*PetscSinScalar(2*PETSC_PI*(vz+0.25*ks));
             ierr = VecSetValues(user->q,1,&l,&v,INSERT_VALUES);CHKERRQ(ierr);
           }
         }
@@ -915,7 +883,7 @@ PetscErrorCode EllipticInitialize(AppCtx *user)
   ierr = MatSetFromOptions(user->Av);CHKERRQ(ierr);
   ierr = MatMPIAIJSetPreallocation(user->Av,2,NULL,2,NULL);CHKERRQ(ierr);
   ierr = MatSeqAIJSetPreallocation(user->Av,2,NULL);CHKERRQ(ierr);
-  ierr = MatGetOwnershipRange(user->Av,&istart,&iend);
+  ierr = MatGetOwnershipRange(user->Av,&istart,&iend);CHKERRQ(ierr);
 
   for (i=istart; i<iend; i++){
     if (i<m/3){
@@ -948,7 +916,7 @@ PetscErrorCode EllipticInitialize(AppCtx *user)
   ierr = MatSetFromOptions(user->L);CHKERRQ(ierr);
   ierr = MatMPIAIJSetPreallocation(user->L,2,NULL,2,NULL);CHKERRQ(ierr);
   ierr = MatSeqAIJSetPreallocation(user->L,2,NULL);CHKERRQ(ierr);
-  ierr = MatGetOwnershipRange(user->L,&istart,&iend);
+  ierr = MatGetOwnershipRange(user->L,&istart,&iend);CHKERRQ(ierr);
 
   for (i=istart; i<iend; i++){
     if (i<m/3){
@@ -1018,7 +986,7 @@ PetscErrorCode EllipticInitialize(AppCtx *user)
     ierr = MatDuplicate(user->Div,MAT_SHARE_NONZERO_PATTERN,&user->Divwork);CHKERRQ(ierr);
   } else {
     ierr = MatCreate(PETSC_COMM_WORLD,&user->Diag);CHKERRQ(ierr);
-    ierr = MatSetSizes(user->Diag,PETSC_DECIDE,PETSC_DECIDE,m,m);
+    ierr = MatSetSizes(user->Diag,PETSC_DECIDE,PETSC_DECIDE,m,m);CHKERRQ(ierr);
     ierr = MatSetFromOptions(user->Diag);CHKERRQ(ierr);
     ierr = MatMPIAIJSetPreallocation(user->Diag,1,NULL,0,NULL);CHKERRQ(ierr);
     ierr = MatSeqAIJSetPreallocation(user->Diag,1,NULL);CHKERRQ(ierr);
@@ -1056,7 +1024,7 @@ PetscErrorCode EllipticInitialize(AppCtx *user)
 
   /* First compute Av_u = Av*exp(-u) */
   ierr = VecSet(user->uwork, 0);CHKERRQ(ierr);
-  ierr = VecAXPY(user->uwork,-1.0,user->utrue); /* Note: user->utrue */
+  ierr = VecAXPY(user->uwork,-1.0,user->utrue);CHKERRQ(ierr); /* Note: user->utrue */
   ierr = VecExp(user->uwork);CHKERRQ(ierr);
   ierr = MatMult(user->Av,user->uwork,user->Av_u);CHKERRQ(ierr);
 
@@ -1086,7 +1054,7 @@ PetscErrorCode EllipticInitialize(AppCtx *user)
     ierr = MatCreateDense(PETSC_COMM_WORLD,ysubnlocal,PETSC_DECIDE,user->ndesign,1,user->ones,&user->Ones);CHKERRQ(ierr);
     ierr = MatAssemblyBegin(user->Ones, MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
     ierr = MatAssemblyEnd(user->Ones, MAT_FINAL_ASSEMBLY);CHKERRQ(ierr);
-    ierr = MatCreateLRC(user->DSG,user->Ones,user->Ones,&user->JsBlock);CHKERRQ(ierr);
+    ierr = MatCreateLRC(user->DSG,user->Ones,NULL,user->Ones,&user->JsBlock);CHKERRQ(ierr);
     ierr = MatSetUp(user->JsBlock);CHKERRQ(ierr);
   } else {
     /* Create matrix-free shell user->Js for computing (A + h^3*e*e^T)*x */
@@ -1251,8 +1219,6 @@ PetscErrorCode EllipticInitialize(AppCtx *user)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EllipticDestroy"
 PetscErrorCode EllipticDestroy(AppCtx *user)
 {
   PetscErrorCode ierr;
@@ -1323,8 +1289,6 @@ PetscErrorCode EllipticDestroy(AppCtx *user)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "EllipticMonitor"
 PetscErrorCode EllipticMonitor(Tao tao, void *ptr)
 {
   PetscErrorCode ierr;
@@ -1344,3 +1308,20 @@ PetscErrorCode EllipticMonitor(Tao tao, void *ptr)
 }
 
 
+
+
+/*TEST
+
+   build:
+      requires: !complex
+
+   test:
+      args: -tao_cmonitor -ns 1 -tao_type lcl -tao_gatol 1.e-3 -tao_max_it 11
+      requires: !single
+
+   test:
+      suffix: 2
+      args: -tao_cmonitor -tao_type lcl -tao_max_it 11 -use_ptap -use_lrc -ns 1 -tao_gatol 1.e-3
+      requires: !single
+
+TEST*/
