@@ -17,8 +17,6 @@ static PetscErrorCode solve(TAO_DF*);
    Regularizer assumed to be L2 norm = lambda*0.5*W'W ()
 */
 
-#undef __FUNCT__
-#define __FUNCT__ "make_grad_node"
 static PetscErrorCode make_grad_node(Vec X, Vec_Chain **p)
 {
   PetscErrorCode ierr;
@@ -31,8 +29,6 @@ static PetscErrorCode make_grad_node(Vec X, Vec_Chain **p)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "destroy_grad_list"
 static PetscErrorCode destroy_grad_list(Vec_Chain *head)
 {
   PetscErrorCode ierr;
@@ -50,12 +46,9 @@ static PetscErrorCode destroy_grad_list(Vec_Chain *head)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSolve_BMRM"
 static PetscErrorCode TaoSolve_BMRM(Tao tao)
 {
   PetscErrorCode     ierr;
-  TaoConvergedReason reason;
   TAO_DF             df;
   TAO_BMRM           *bmrm = (TAO_BMRM*)tao->data;
 
@@ -71,7 +64,7 @@ static PetscErrorCode TaoSolve_BMRM(Tao tao)
 
   /* Used in converged criteria check */
   PetscReal          reg;
-  PetscReal          jtwt, max_jtwt, pre_epsilon, epsilon, jw, min_jw;
+  PetscReal          jtwt = 0.0, max_jtwt, pre_epsilon, epsilon, jw, min_jw;
   PetscReal          innerSolverTol;
   MPI_Comm           comm;
 
@@ -94,7 +87,7 @@ static PetscErrorCode TaoSolve_BMRM(Tao tao)
   }
 
   df.tol = 1e-6;
-  reason = TAO_CONTINUE_ITERATING;
+  tao->reason = TAO_CONTINUE_ITERATING;
 
   /*-----------------Algorithm Begins------------------------*/
   /* make the scatter */
@@ -104,8 +97,11 @@ static PetscErrorCode TaoSolve_BMRM(Tao tao)
 
   /* NOTE: In application pass the sub-gradient of Remp(W) */
   ierr = TaoComputeObjectiveAndGradient(tao, W, &f, G);CHKERRQ(ierr);
-  ierr = TaoMonitor(tao,tao->niter,f,1.0,0.0,tao->step,&reason);CHKERRQ(ierr);
-  while (reason == TAO_CONTINUE_ITERATING) {
+  ierr = TaoLogConvergenceHistory(tao,f,1.0,0.0,tao->ksp_its);CHKERRQ(ierr);
+  ierr = TaoMonitor(tao,tao->niter,f,1.0,0.0,tao->step);CHKERRQ(ierr);
+  ierr = (*tao->ops->convergencetest)(tao,tao->cnvP);CHKERRQ(ierr);
+  
+  while (tao->reason == TAO_CONTINUE_ITERATING) {
     /* compute bt = Remp(Wt-1) - <Wt-1, At> */
     ierr = VecDot(W, G, &bt);CHKERRQ(ierr);
     bt = f - bt;
@@ -116,7 +112,7 @@ static PetscErrorCode TaoSolve_BMRM(Tao tao)
 
     /* Bring up the inner solver */
     if (!rank) {
-      ierr = ensure_df_space(tao->niter+1, &df); CHKERRQ(ierr);
+      ierr = ensure_df_space(tao->niter+1, &df);CHKERRQ(ierr);
       ierr = make_grad_node(bmrm->local_w, &pgrad);CHKERRQ(ierr);
       tail_glist->next = pgrad;
       tail_glist = pgrad;
@@ -129,6 +125,7 @@ static PetscErrorCode TaoSolve_BMRM(Tao tao)
       /* set up the Q */
       pgrad = grad_list.next;
       for (i=0; i<=tao->niter; i++) {
+        if (!pgrad) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Assert that there are at least tao->niter+1 pgrad available");
         ierr = VecDot(pgrad->V, bmrm->local_w, &reg);CHKERRQ(ierr);
         df.Q[i][tao->niter] = df.Q[tao->niter][i] = reg / lambda;
         pgrad = pgrad->next;
@@ -136,21 +133,21 @@ static PetscErrorCode TaoSolve_BMRM(Tao tao)
 
       if (tao->niter > 0) {
         df.x[tao->niter] = 0.0;
-        ierr = solve(&df); CHKERRQ(ierr);
+        ierr = solve(&df);CHKERRQ(ierr);
       } else
         df.x[0] = 1.0;
 
       /* now computing Jt*(alpha_t) which should be = Jt(wt) to check convergence */
       jtwt = 0.0;
-      ierr = VecSet(bmrm->local_w, 0.0); CHKERRQ(ierr);
+      ierr = VecSet(bmrm->local_w, 0.0);CHKERRQ(ierr);
       pgrad = grad_list.next;
       for (i=0; i<=tao->niter; i++) {
         jtwt -= df.x[i] * df.f[i];
-        ierr = VecAXPY(bmrm->local_w, -df.x[i] / lambda, pgrad->V); CHKERRQ(ierr);
+        ierr = VecAXPY(bmrm->local_w, -df.x[i] / lambda, pgrad->V);CHKERRQ(ierr);
         pgrad = pgrad->next;
       }
 
-      ierr = VecNorm(bmrm->local_w, NORM_2, &reg); CHKERRQ(ierr);
+      ierr = VecNorm(bmrm->local_w, NORM_2, &reg);CHKERRQ(ierr);
       reg = 0.5*lambda*reg*reg;
       jtwt -= reg;
     } /* end if rank == 0 */
@@ -182,7 +179,9 @@ static PetscErrorCode TaoSolve_BMRM(Tao tao)
     }
 
     tao->niter++;
-    ierr = TaoMonitor(tao,tao->niter,min_jw,epsilon,0.0,tao->step,&reason);CHKERRQ(ierr);
+    ierr = TaoLogConvergenceHistory(tao,min_jw,epsilon,0.0,tao->ksp_its);CHKERRQ(ierr);
+    ierr = TaoMonitor(tao,tao->niter,min_jw,epsilon,0.0,tao->step);CHKERRQ(ierr);
+    ierr = (*tao->ops->convergencetest)(tao,tao->cnvP);CHKERRQ(ierr);
   }
 
   /* free all the memory */
@@ -199,9 +198,7 @@ static PetscErrorCode TaoSolve_BMRM(Tao tao)
 
 /* ---------------------------------------------------------- */
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetup_BMRM"
-static PetscErrorCode TaoSetup_BMRM(Tao tao) 
+static PetscErrorCode TaoSetup_BMRM(Tao tao)
 {
 
   PetscErrorCode ierr;
@@ -209,14 +206,12 @@ static PetscErrorCode TaoSetup_BMRM(Tao tao)
   PetscFunctionBegin;
   /* Allocate some arrays */
   if (!tao->gradient) {
-    ierr = VecDuplicate(tao->solution, &tao->gradient);   CHKERRQ(ierr);
+    ierr = VecDuplicate(tao->solution, &tao->gradient);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
 /*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TaoDestroy_BMRM"
 static PetscErrorCode TaoDestroy_BMRM(Tao tao)
 {
   PetscErrorCode ierr;
@@ -226,23 +221,19 @@ static PetscErrorCode TaoDestroy_BMRM(Tao tao)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetFromOptions_BMRM"
-static PetscErrorCode TaoSetFromOptions_BMRM(PetscOptions *PetscOptionsObject,Tao tao)
+static PetscErrorCode TaoSetFromOptions_BMRM(PetscOptionItems *PetscOptionsObject,Tao tao)
 {
   PetscErrorCode ierr;
   TAO_BMRM*      bmrm = (TAO_BMRM*)tao->data;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"BMRM for regularized risk minimization");CHKERRQ(ierr);
-  ierr = PetscOptionsReal("-tao_bmrm_lambda", "regulariser weight","", 100,&bmrm->lambda,NULL); CHKERRQ(ierr);
+  ierr = PetscOptionsReal("-tao_bmrm_lambda", "regulariser weight","", 100,&bmrm->lambda,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 /*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TaoView_BMRM"
 static PetscErrorCode TaoView_BMRM(Tao tao, PetscViewer viewer)
 {
   PetscBool      isascii;
@@ -267,8 +258,6 @@ static PetscErrorCode TaoView_BMRM(Tao tao, PetscViewer viewer)
   Level: beginner
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoCreate_BMRM"
 PETSC_EXTERN PetscErrorCode TaoCreate_BMRM(Tao tao)
 {
   TAO_BMRM       *bmrm;
@@ -288,16 +277,12 @@ PETSC_EXTERN PetscErrorCode TaoCreate_BMRM(Tao tao)
   /* Override default settings (unless already changed) */
   if (!tao->max_it_changed) tao->max_it = 2000;
   if (!tao->max_funcs_changed) tao->max_funcs = 4000;
-  if (!tao->fatol_changed) tao->fatol = 1.0e-12;
-  if (!tao->frtol_changed) tao->frtol = 1.0e-12;
   if (!tao->gatol_changed) tao->gatol = 1.0e-12;
   if (!tao->grtol_changed) tao->grtol = 1.0e-12;
 
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "init_df_solver"
 PetscErrorCode init_df_solver(TAO_DF *df)
 {
   PetscInt       i, n = INCRE_DIM;
@@ -311,36 +296,34 @@ PetscErrorCode init_df_solver(TAO_DF *df)
 
   /* memory space required by Dai-Fletcher */
   df->cur_num_cp = n;
-  ierr = PetscMalloc1(n, &df->f); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->a); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->l); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->u); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->x); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->Q); CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->f);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->a);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->l);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->u);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->x);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->Q);CHKERRQ(ierr);
 
   for (i = 0; i < n; i ++) {
-    ierr = PetscMalloc1(n, &df->Q[i]); CHKERRQ(ierr);
+    ierr = PetscMalloc1(n, &df->Q[i]);CHKERRQ(ierr);
   }
 
-  ierr = PetscMalloc1(n, &df->g); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->y); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->tempv); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->d); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->Qd); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->t); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->xplus); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->tplus); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->sk); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->yk); CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->g);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->y);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->tempv);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->d);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->Qd);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->t);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->xplus);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->tplus);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->sk);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->yk);CHKERRQ(ierr);
 
-  ierr = PetscMalloc1(n, &df->ipt); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->ipt2); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->uv); CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->ipt);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->ipt2);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->uv);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ensure_df_space"
 PetscErrorCode ensure_df_space(PetscInt dim, TAO_DF *df)
 {
   PetscErrorCode ierr;
@@ -356,121 +339,117 @@ PetscErrorCode ensure_df_space(PetscInt dim, TAO_DF *df)
   n = df->cur_num_cp;
 
   /* memory space required by dai-fletcher */
-  ierr = PetscMalloc1(n, &tmp); CHKERRQ(ierr);
-  ierr = PetscMemcpy(tmp, df->f, sizeof(PetscReal)*old_n); CHKERRQ(ierr);
-  ierr = PetscFree(df->f); CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &tmp);CHKERRQ(ierr);
+  ierr = PetscMemcpy(tmp, df->f, sizeof(PetscReal)*old_n);CHKERRQ(ierr);
+  ierr = PetscFree(df->f);CHKERRQ(ierr);
   df->f = tmp;
 
-  ierr = PetscMalloc1(n, &tmp); CHKERRQ(ierr);
-  ierr = PetscMemcpy(tmp, df->a, sizeof(PetscReal)*old_n); CHKERRQ(ierr);
-  ierr = PetscFree(df->a); CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &tmp);CHKERRQ(ierr);
+  ierr = PetscMemcpy(tmp, df->a, sizeof(PetscReal)*old_n);CHKERRQ(ierr);
+  ierr = PetscFree(df->a);CHKERRQ(ierr);
   df->a = tmp;
 
-  ierr = PetscMalloc1(n, &tmp); CHKERRQ(ierr);
-  ierr = PetscMemcpy(tmp, df->l, sizeof(PetscReal)*old_n); CHKERRQ(ierr);
-  ierr = PetscFree(df->l); CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &tmp);CHKERRQ(ierr);
+  ierr = PetscMemcpy(tmp, df->l, sizeof(PetscReal)*old_n);CHKERRQ(ierr);
+  ierr = PetscFree(df->l);CHKERRQ(ierr);
   df->l = tmp;
 
-  ierr = PetscMalloc1(n, &tmp); CHKERRQ(ierr);
-  ierr = PetscMemcpy(tmp, df->u, sizeof(PetscReal)*old_n); CHKERRQ(ierr);
-  ierr = PetscFree(df->u); CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &tmp);CHKERRQ(ierr);
+  ierr = PetscMemcpy(tmp, df->u, sizeof(PetscReal)*old_n);CHKERRQ(ierr);
+  ierr = PetscFree(df->u);CHKERRQ(ierr);
   df->u = tmp;
 
-  ierr = PetscMalloc1(n, &tmp); CHKERRQ(ierr);
-  ierr = PetscMemcpy(tmp, df->x, sizeof(PetscReal)*old_n); CHKERRQ(ierr);
-  ierr = PetscFree(df->x); CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &tmp);CHKERRQ(ierr);
+  ierr = PetscMemcpy(tmp, df->x, sizeof(PetscReal)*old_n);CHKERRQ(ierr);
+  ierr = PetscFree(df->x);CHKERRQ(ierr);
   df->x = tmp;
 
-  ierr = PetscMalloc1(n, &tmp_Q); CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &tmp_Q);CHKERRQ(ierr);
   for (i = 0; i < n; i ++) {
-    ierr = PetscMalloc1(n, &tmp_Q[i]); CHKERRQ(ierr);
+    ierr = PetscMalloc1(n, &tmp_Q[i]);CHKERRQ(ierr);
     if (i < old_n) {
-      ierr = PetscMemcpy(tmp_Q[i], df->Q[i], sizeof(PetscReal)*old_n); CHKERRQ(ierr);
-      ierr = PetscFree(df->Q[i]); CHKERRQ(ierr);
+      ierr = PetscMemcpy(tmp_Q[i], df->Q[i], sizeof(PetscReal)*old_n);CHKERRQ(ierr);
+      ierr = PetscFree(df->Q[i]);CHKERRQ(ierr);
     }
   }
 
-  ierr = PetscFree(df->Q); CHKERRQ(ierr);
+  ierr = PetscFree(df->Q);CHKERRQ(ierr);
   df->Q = tmp_Q;
 
-  ierr = PetscFree(df->g); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->g); CHKERRQ(ierr);
+  ierr = PetscFree(df->g);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->g);CHKERRQ(ierr);
 
-  ierr = PetscFree(df->y); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->y); CHKERRQ(ierr);
+  ierr = PetscFree(df->y);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->y);CHKERRQ(ierr);
 
-  ierr = PetscFree(df->tempv); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->tempv); CHKERRQ(ierr);
+  ierr = PetscFree(df->tempv);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->tempv);CHKERRQ(ierr);
 
-  ierr = PetscFree(df->d); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->d); CHKERRQ(ierr);
+  ierr = PetscFree(df->d);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->d);CHKERRQ(ierr);
 
-  ierr = PetscFree(df->Qd); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->Qd); CHKERRQ(ierr);
+  ierr = PetscFree(df->Qd);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->Qd);CHKERRQ(ierr);
 
-  ierr = PetscFree(df->t); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->t); CHKERRQ(ierr);
+  ierr = PetscFree(df->t);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->t);CHKERRQ(ierr);
 
-  ierr = PetscFree(df->xplus); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->xplus); CHKERRQ(ierr);
+  ierr = PetscFree(df->xplus);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->xplus);CHKERRQ(ierr);
 
-  ierr = PetscFree(df->tplus); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->tplus); CHKERRQ(ierr);
+  ierr = PetscFree(df->tplus);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->tplus);CHKERRQ(ierr);
 
-  ierr = PetscFree(df->sk); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->sk); CHKERRQ(ierr);
+  ierr = PetscFree(df->sk);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->sk);CHKERRQ(ierr);
 
-  ierr = PetscFree(df->yk); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->yk); CHKERRQ(ierr);
+  ierr = PetscFree(df->yk);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->yk);CHKERRQ(ierr);
 
-  ierr = PetscFree(df->ipt); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->ipt); CHKERRQ(ierr);
+  ierr = PetscFree(df->ipt);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->ipt);CHKERRQ(ierr);
 
-  ierr = PetscFree(df->ipt2); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->ipt2); CHKERRQ(ierr);
+  ierr = PetscFree(df->ipt2);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->ipt2);CHKERRQ(ierr);
 
-  ierr = PetscFree(df->uv); CHKERRQ(ierr);
-  ierr = PetscMalloc1(n, &df->uv); CHKERRQ(ierr);
+  ierr = PetscFree(df->uv);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &df->uv);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "destroy_df_solver"
 PetscErrorCode destroy_df_solver(TAO_DF *df)
 {
   PetscErrorCode ierr;
   PetscInt       i;
 
   PetscFunctionBegin;
-  ierr = PetscFree(df->f); CHKERRQ(ierr);
-  ierr = PetscFree(df->a); CHKERRQ(ierr);
-  ierr = PetscFree(df->l); CHKERRQ(ierr);
-  ierr = PetscFree(df->u); CHKERRQ(ierr);
-  ierr = PetscFree(df->x); CHKERRQ(ierr);
+  ierr = PetscFree(df->f);CHKERRQ(ierr);
+  ierr = PetscFree(df->a);CHKERRQ(ierr);
+  ierr = PetscFree(df->l);CHKERRQ(ierr);
+  ierr = PetscFree(df->u);CHKERRQ(ierr);
+  ierr = PetscFree(df->x);CHKERRQ(ierr);
 
   for (i = 0; i < df->cur_num_cp; i ++) {
-    ierr = PetscFree(df->Q[i]); CHKERRQ(ierr);
+    ierr = PetscFree(df->Q[i]);CHKERRQ(ierr);
   }
-  ierr = PetscFree(df->Q); CHKERRQ(ierr);
-  ierr = PetscFree(df->ipt); CHKERRQ(ierr);
-  ierr = PetscFree(df->ipt2); CHKERRQ(ierr);
-  ierr = PetscFree(df->uv); CHKERRQ(ierr);
-  ierr = PetscFree(df->g); CHKERRQ(ierr);
-  ierr = PetscFree(df->y); CHKERRQ(ierr);
-  ierr = PetscFree(df->tempv); CHKERRQ(ierr);
-  ierr = PetscFree(df->d); CHKERRQ(ierr);
-  ierr = PetscFree(df->Qd); CHKERRQ(ierr);
-  ierr = PetscFree(df->t); CHKERRQ(ierr);
-  ierr = PetscFree(df->xplus); CHKERRQ(ierr);
-  ierr = PetscFree(df->tplus); CHKERRQ(ierr);
-  ierr = PetscFree(df->sk); CHKERRQ(ierr);
-  ierr = PetscFree(df->yk); CHKERRQ(ierr);
+  ierr = PetscFree(df->Q);CHKERRQ(ierr);
+  ierr = PetscFree(df->ipt);CHKERRQ(ierr);
+  ierr = PetscFree(df->ipt2);CHKERRQ(ierr);
+  ierr = PetscFree(df->uv);CHKERRQ(ierr);
+  ierr = PetscFree(df->g);CHKERRQ(ierr);
+  ierr = PetscFree(df->y);CHKERRQ(ierr);
+  ierr = PetscFree(df->tempv);CHKERRQ(ierr);
+  ierr = PetscFree(df->d);CHKERRQ(ierr);
+  ierr = PetscFree(df->Qd);CHKERRQ(ierr);
+  ierr = PetscFree(df->t);CHKERRQ(ierr);
+  ierr = PetscFree(df->xplus);CHKERRQ(ierr);
+  ierr = PetscFree(df->tplus);CHKERRQ(ierr);
+  ierr = PetscFree(df->sk);CHKERRQ(ierr);
+  ierr = PetscFree(df->yk);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 /* Piecewise linear monotone target function for the Dai-Fletcher projector */
-#undef __FUNCT__
-#define __FUNCT__ "phi"
 PetscReal phi(PetscReal *x,PetscInt n,PetscReal lambda,PetscReal *a,PetscReal b,PetscReal *c,PetscReal *l,PetscReal *u)
 {
   PetscReal r = 0.0;
@@ -493,8 +472,6 @@ PetscReal phi(PetscReal *x,PetscInt n,PetscReal lambda,PetscReal *a,PetscReal b,
  *
  *  \param c The point to be projected onto feasible set
  */
-#undef __FUNCT__
-#define __FUNCT__ "project"
 PetscInt project(PetscInt n,PetscReal *a,PetscReal b,PetscReal *c,PetscReal *l,PetscReal *u,PetscReal *x,PetscReal *lam_ext,TAO_DF *df)
 {
   PetscReal      lambda, lambdal, lambdau, dlambda, lambda_new;
@@ -525,7 +502,7 @@ PetscInt project(PetscInt n,PetscReal *a,PetscReal b,PetscReal *c,PetscReal *l,P
     if (r < TOL_R) return 0;
   } else  {
     /* equality constraint ,i.e., without \xi >= 0 constraint */
-    if (fabs(r) < TOL_R) return 0;
+    if (PetscAbsReal(r) < TOL_R) return 0;
   }
 
   if (r < 0.0){
@@ -562,11 +539,9 @@ PetscInt project(PetscInt n,PetscReal *a,PetscReal b,PetscReal *c,PetscReal *l,P
     rl      = r;
   }
 
-  if(fabs(dlambda) > BMRM_INFTY) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"L2N2_DaiFletcherPGM detected Infeasible QP problem!");
+  if(PetscAbsReal(dlambda) > BMRM_INFTY) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"L2N2_DaiFletcherPGM detected Infeasible QP problem!");
 
   if(ru == 0){
-    lambda = lambdau;
-    r = phi(x, n, lambda, a, b, c, l, u);
     return innerIter;
   }
 
@@ -576,8 +551,8 @@ PetscInt project(PetscInt n,PetscReal *a,PetscReal b,PetscReal *c,PetscReal *l,P
   lambda  = lambdau - dlambda;
   r       = phi(x, n, lambda, a, b, c, l, u);
 
-  while (fabs(r) > TOL_R
-         && dlambda > TOL_LAM * (1.0 + fabs(lambda))
+  while (PetscAbsReal(r) > TOL_R
+         && dlambda > TOL_LAM * (1.0 + PetscAbsReal(lambda))
          && innerIter < df->maxProjIter){
     innerIter++;
     if (r > 0.0){
@@ -630,8 +605,6 @@ PetscInt project(PetscInt n,PetscReal *a,PetscReal b,PetscReal *c,PetscReal *l,P
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "solve"
 PetscErrorCode solve(TAO_DF *df)
 {
   PetscErrorCode ierr;
@@ -667,10 +640,10 @@ PetscErrorCode solve(TAO_DF *df)
 
   it = 0;
   for (i = 0; i < dim; i++) {
-    if (fabs(x[i]) > ProdDELTAsv) ipt[it++] = i;
+    if (PetscAbsReal(x[i]) > ProdDELTAsv) ipt[it++] = i;
   }
 
-  ierr = PetscMemzero(t, dim*sizeof(PetscReal)); CHKERRQ(ierr);
+  ierr = PetscMemzero(t, dim*sizeof(PetscReal));CHKERRQ(ierr);
   for (i = 0; i < it; i++){
     tempQ = Q[ipt[i]];
     for (j = 0; j < dim; j++) t[j] += (tempQ[j]*x[ipt[i]]);
@@ -692,12 +665,10 @@ PetscErrorCode solve(TAO_DF *df)
   max = ALPHA_MIN;
   for (i = 0; i < dim; i++){
     y[i] = tempv[i] - x[i];
-    if (fabs(y[i]) > max) max = fabs(y[i]);
+    if (PetscAbsReal(y[i]) > max) max = PetscAbsReal(y[i]);
   }
 
   if (max < tol*1e-3){
-    lscount = 0;
-    innerIter    = 0;
     return 0;
   }
 
@@ -740,13 +711,13 @@ PetscErrorCode solve(TAO_DF *df)
 
     it = it2 = 0;
     for (i = 0; i < dim; i++){
-      if (fabs(d[i]) > (ProdDELTAsv*1.0e-2)) ipt[it++]   = i;
+      if (PetscAbsReal(d[i]) > (ProdDELTAsv*1.0e-2)) ipt[it++]   = i;
     }
     for (i = 0; i < dim; i++) {
-      if (fabs(y[i]) > ProdDELTAsv) ipt2[it2++] = i;
+      if (PetscAbsReal(y[i]) > ProdDELTAsv) ipt2[it2++] = i;
     }
 
-    ierr = PetscMemzero(Qd, dim*sizeof(PetscReal)); CHKERRQ(ierr);
+    ierr = PetscMemzero(Qd, dim*sizeof(PetscReal));CHKERRQ(ierr);
     /* compute Qd = Q*d */
     if (it < it2){
       for (i = 0; i < it; i++){
@@ -857,7 +828,7 @@ PetscErrorCode solve(TAO_DF *df)
         kktlam = kktlam/it;
         info   = 1;
         for (i = 0; i < it; i++) {
-          if (fabs(a[ipt[i]] * g[ipt[i]] + kktlam) > tol) {
+          if (PetscAbsReal(a[ipt[i]] * g[ipt[i]] + kktlam) > tol) {
             info = 0;
             break;
           }
