@@ -1,11 +1,48 @@
 #include <../src/tao/unconstrained/impls/neldermead/neldermead.h>
 #include <petscvec.h>
 
-static PetscErrorCode NelderMeadSort(TAO_NelderMead *nm);
-static PetscErrorCode NelderMeadReplace(TAO_NelderMead *nm, PetscInt index, Vec Xmu, PetscReal f);
+
+/*------------------------------------------------------------*/
+static PetscErrorCode NelderMeadSort(TAO_NelderMead *nm)
+{
+  PetscReal *values = nm->f_values;
+  PetscInt  *indices = nm->indices;
+  PetscInt  dim = nm->N+1;
+  PetscInt  i,j,index;
+  PetscReal val;
+
+  PetscFunctionBegin;
+  for (i=1;i<dim;i++) {
+    index = indices[i];
+    val = values[index];
+    for (j=i-1; j>=0 && values[indices[j]] > val; j--) {
+      indices[j+1] = indices[j];
+    }
+    indices[j+1] = index;
+  }
+  PetscFunctionReturn(0);
+}
+
+
+/*------------------------------------------------------------*/
+static PetscErrorCode NelderMeadReplace(TAO_NelderMead *nm, PetscInt index, Vec Xmu, PetscReal f)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  /*  Add new vector's fraction of average */
+  ierr = VecAXPY(nm->Xbar,nm->oneOverN,Xmu);CHKERRQ(ierr);
+  ierr = VecCopy(Xmu,nm->simplex[index]);CHKERRQ(ierr);
+  nm->f_values[index] = f;
+
+  ierr = NelderMeadSort(nm);CHKERRQ(ierr);
+
+  /*  Subtract last vector from average */
+  ierr = VecAXPY(nm->Xbar,-nm->oneOverN,nm->simplex[nm->indices[nm->N]]);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /* ---------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetUp_NM"
 static PetscErrorCode TaoSetUp_NM(Tao tao)
 {
   PetscErrorCode ierr;
@@ -30,9 +67,7 @@ static PetscErrorCode TaoSetUp_NM(Tao tao)
 }
 
 /* ---------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "TaoDestroy_NM"
-PetscErrorCode TaoDestroy_NM(Tao tao)
+static PetscErrorCode TaoDestroy_NM(Tao tao)
 {
   TAO_NelderMead *nm = (TAO_NelderMead*)tao->data;
   PetscErrorCode ierr;
@@ -53,9 +88,7 @@ PetscErrorCode TaoDestroy_NM(Tao tao)
 }
 
 /*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetFromOptions_NM"
-PetscErrorCode TaoSetFromOptions_NM(PetscOptions *PetscOptionsObject,Tao tao)
+static PetscErrorCode TaoSetFromOptions_NM(PetscOptionItems *PetscOptionsObject,Tao tao)
 {
   TAO_NelderMead *nm = (TAO_NelderMead*)tao->data;
   PetscErrorCode ierr;
@@ -72,9 +105,7 @@ PetscErrorCode TaoSetFromOptions_NM(PetscOptions *PetscOptionsObject,Tao tao)
 }
 
 /*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TaoView_NM"
-PetscErrorCode TaoView_NM(Tao tao,PetscViewer viewer)
+static PetscErrorCode TaoView_NM(Tao tao,PetscViewer viewer)
 {
   TAO_NelderMead *nm = (TAO_NelderMead*)tao->data;
   PetscBool      isascii;
@@ -95,13 +126,10 @@ PetscErrorCode TaoView_NM(Tao tao,PetscViewer viewer)
 }
 
 /*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TaoSolve_NM"
-PetscErrorCode TaoSolve_NM(Tao tao)
+static PetscErrorCode TaoSolve_NM(Tao tao)
 {
   PetscErrorCode     ierr;
   TAO_NelderMead     *nm = (TAO_NelderMead*)tao->data;
-  TaoConvergedReason reason;
   PetscReal          *x;
   PetscInt           i;
   Vec                Xmur=nm->Xmur, Xmue=nm->Xmue, Xmuc=nm->Xmuc, Xbar=nm->Xbar;
@@ -143,12 +171,14 @@ PetscErrorCode TaoSolve_NM(Tao tao)
     ierr = VecAXPY(Xbar,1.0,nm->simplex[nm->indices[i]]);CHKERRQ(ierr);
   }
   ierr = VecScale(Xbar,nm->oneOverN);CHKERRQ(ierr);
-  reason = TAO_CONTINUE_ITERATING;
+  tao->reason = TAO_CONTINUE_ITERATING;
   while (1) {
     shrink = 0;
     ierr = VecCopy(nm->simplex[nm->indices[0]],tao->solution);CHKERRQ(ierr);
-    ierr = TaoMonitor(tao,tao->niter++,nm->f_values[nm->indices[0]],nm->f_values[nm->indices[nm->N]]-nm->f_values[nm->indices[0]],0.0,1.0,&reason);CHKERRQ(ierr);
-    if (reason != TAO_CONTINUE_ITERATING) break;
+    ierr = TaoLogConvergenceHistory(tao, nm->f_values[nm->indices[0]], nm->f_values[nm->indices[nm->N]]-nm->f_values[nm->indices[0]], 0.0, tao->ksp_its);CHKERRQ(ierr);
+    ierr = TaoMonitor(tao,tao->niter, nm->f_values[nm->indices[0]], nm->f_values[nm->indices[nm->N]]-nm->f_values[nm->indices[0]], 0.0, 1.0);CHKERRQ(ierr);
+    ierr = (*tao->ops->convergencetest)(tao,tao->cnvP);CHKERRQ(ierr);
+    if (tao->reason != TAO_CONTINUE_ITERATING) break;
 
     /* x(mu) = (1 + mu)Xbar - mu*X_N+1 */
     ierr = VecAXPBYPCZ(Xmur,1+nm->mu_r,-nm->mu_r,0,Xbar,nm->simplex[nm->indices[nm->N]]);CHKERRQ(ierr);
@@ -222,8 +252,6 @@ PetscErrorCode TaoSolve_NM(Tao tao)
  Level: beginner
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoCreate_NM"
 PETSC_EXTERN PetscErrorCode TaoCreate_NM(Tao tao)
 {
   TAO_NelderMead *nm;
@@ -242,13 +270,6 @@ PETSC_EXTERN PetscErrorCode TaoCreate_NM(Tao tao)
   /* Override default settings (unless already changed) */
   if (!tao->max_it_changed) tao->max_it = 2000;
   if (!tao->max_funcs_changed) tao->max_funcs = 4000;
-#if defined(PETSC_USE_REAL_SINGLE)
-  if (!tao->fatol_changed) tao->fatol = 1.0e-5;
-  if (!tao->frtol_changed) tao->frtol = 1.0e-5;
-#else
-  if (!tao->fatol_changed) tao->fatol = 1.0e-8;
-  if (!tao->frtol_changed) tao->frtol = 1.0e-8;
-#endif
 
   nm->simplex = 0;
   nm->lamda = 1;
@@ -261,47 +282,3 @@ PETSC_EXTERN PetscErrorCode TaoCreate_NM(Tao tao)
   PetscFunctionReturn(0);
 }
 
-
-/*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "NelderMeadSort"
-PetscErrorCode NelderMeadSort(TAO_NelderMead *nm)
-{
-  PetscReal *values = nm->f_values;
-  PetscInt  *indices = nm->indices;
-  PetscInt  dim = nm->N+1;
-  PetscInt  i,j,index;
-  PetscReal val;
-
-  PetscFunctionBegin;
-  for (i=1;i<dim;i++) {
-    index = indices[i];
-    val = values[index];
-    for (j=i-1; j>=0 && values[indices[j]] > val; j--) {
-      indices[j+1] = indices[j];
-    }
-    indices[j+1] = index;
-  }
-  PetscFunctionReturn(0);
-}
-
-
-/*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "NelderMeadReplace"
-PetscErrorCode NelderMeadReplace(TAO_NelderMead *nm, PetscInt index, Vec Xmu, PetscReal f)
-{
-  PetscErrorCode ierr;
-
-  PetscFunctionBegin;
-  /*  Add new vector's fraction of average */
-  ierr = VecAXPY(nm->Xbar,nm->oneOverN,Xmu);CHKERRQ(ierr);
-  ierr = VecCopy(Xmu,nm->simplex[index]);CHKERRQ(ierr);
-  nm->f_values[index] = f;
-
-  ierr = NelderMeadSort(nm);CHKERRQ(ierr);
-
-  /*  Subtract last vector from average */
-  ierr = VecAXPY(nm->Xbar,-nm->oneOverN,nm->simplex[nm->indices[nm->N]]);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
