@@ -1,29 +1,7 @@
 #include <petsctaolinesearch.h>
-#include <../src/tao/matrix/lmvmmat.h>
-#include <../src/tao/unconstrained/impls/nls/nls.h>
+#include <../src/tao/unconstrained/impls/nls/nlsimpl.h>
 
 #include <petscksp.h>
-#include <petscpc.h>
-#include <petsc/private/kspimpl.h>
-#include <petsc/private/pcimpl.h>
-
-#define NLS_KSP_CG      0
-#define NLS_KSP_NASH    1
-#define NLS_KSP_STCG    2
-#define NLS_KSP_GLTR    3
-#define NLS_KSP_PETSC   4
-#define NLS_KSP_TYPES   5
-
-#define NLS_PC_NONE     0
-#define NLS_PC_AHESS    1
-#define NLS_PC_BFGS     2
-#define NLS_PC_PETSC    3
-#define NLS_PC_TYPES    4
-
-#define BFGS_SCALE_AHESS        0
-#define BFGS_SCALE_PHESS        1
-#define BFGS_SCALE_BFGS         2
-#define BFGS_SCALE_TYPES        3
 
 #define NLS_INIT_CONSTANT         0
 #define NLS_INIT_DIRECTION        1
@@ -35,20 +13,11 @@
 #define NLS_UPDATE_INTERPOLATION  2
 #define NLS_UPDATE_TYPES          3
 
-static const char *NLS_KSP[64] = {"cg", "nash", "stcg", "gltr", "petsc"};
-
-static const char *NLS_PC[64] = {"none", "ahess", "bfgs", "petsc"};
-
-static const char *BFGS_SCALE[64] = {"ahess", "phess", "bfgs"};
-
 static const char *NLS_INIT[64] = {"constant", "direction", "interpolation"};
 
 static const char *NLS_UPDATE[64] = {"step", "reduction", "interpolation"};
 
-static PetscErrorCode MatLMVMSolveShell(PC pc, Vec b, Vec x);
-/* Routine for BFGS preconditioner
-
-
+/*
  Implements Newton's Method with a line search approach for solving
  unconstrained minimization problems.  A More'-Thuente line search
  is used to guarantee that the bfgs preconditioner remains positive
@@ -59,35 +28,33 @@ static PetscErrorCode MatLMVMSolveShell(PC pc, Vec b, Vec x);
  problems.
 
  The linear system solve should be done with a conjugate gradient
- method, although any method can be used. */
+ method, although any method can be used.
+*/
 
 #define NLS_NEWTON              0
 #define NLS_BFGS                1
-#define NLS_SCALED_GRADIENT     2
-#define NLS_GRADIENT            3
+#define NLS_GRADIENT            2
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoSolve_NLS"
 static PetscErrorCode TaoSolve_NLS(Tao tao)
 {
   PetscErrorCode               ierr;
   TAO_NLS                      *nlsP = (TAO_NLS *)tao->data;
-  PC                           pc;
+  KSPType                      ksp_type;
+  PetscBool                    is_nash,is_stcg,is_gltr,is_bfgs,is_jacobi,is_symmetric,sym_set;
   KSPConvergedReason           ksp_reason;
+  PC                           pc;
   TaoLineSearchConvergedReason ls_reason;
-  TaoConvergedReason           reason;
 
   PetscReal                    fmin, ftrial, f_full, prered, actred, kappa, sigma;
   PetscReal                    tau, tau_1, tau_2, tau_max, tau_min, max_radius;
   PetscReal                    f, fold, gdx, gnorm, pert;
   PetscReal                    step = 1.0;
-  PetscReal                    delta;
   PetscReal                    norm_d = 0.0, e_min;
 
   PetscInt                     stepType;
   PetscInt                     bfgsUpdates = 0;
   PetscInt                     n,N,kspits;
-  PetscInt                     needH;
+  PetscInt                     needH = 1;
 
   PetscInt                     i_max = 5;
   PetscInt                     j_max = 1;
@@ -110,107 +77,55 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
   nlsP->ksp_iter = 0;
   nlsP->ksp_othr = 0;
 
-  /* Modify the linear solver to a trust region method if desired */
-  switch(nlsP->ksp_type) {
-  case NLS_KSP_CG:
-    ierr = KSPSetType(tao->ksp, KSPCG);CHKERRQ(ierr);
-    ierr = KSPSetFromOptions(tao->ksp);CHKERRQ(ierr);
-    break;
-
-  case NLS_KSP_NASH:
-    ierr = KSPSetType(tao->ksp, KSPNASH);CHKERRQ(ierr);
-    ierr = KSPSetFromOptions(tao->ksp);CHKERRQ(ierr);
-    break;
-
-  case NLS_KSP_STCG:
-    ierr = KSPSetType(tao->ksp, KSPSTCG);CHKERRQ(ierr);
-    ierr = KSPSetFromOptions(tao->ksp);CHKERRQ(ierr);
-    break;
-
-  case NLS_KSP_GLTR:
-    ierr = KSPSetType(tao->ksp, KSPGLTR);CHKERRQ(ierr);
-    ierr = KSPSetFromOptions(tao->ksp);CHKERRQ(ierr);
-    break;
-
-  default:
-    /* Use the method set by the ksp_type */
-    break;
-  }
-
   /* Initialize trust-region radius when using nash, stcg, or gltr
-   Will be reset during the first iteration */
-  if (NLS_KSP_NASH == nlsP->ksp_type) {
-    ierr = KSPNASHSetRadius(tao->ksp,nlsP->max_radius);CHKERRQ(ierr);
-  } else if (NLS_KSP_STCG == nlsP->ksp_type) {
-    ierr = KSPSTCGSetRadius(tao->ksp,nlsP->max_radius);CHKERRQ(ierr);
-  } else if (NLS_KSP_GLTR == nlsP->ksp_type) {
-    ierr = KSPGLTRSetRadius(tao->ksp,nlsP->max_radius);CHKERRQ(ierr);
-  }
+     Command automatically ignored for other methods
+     Will be reset during the first iteration
+  */
+  ierr = KSPGetType(tao->ksp,&ksp_type);CHKERRQ(ierr);
+  ierr = PetscStrcmp(ksp_type,KSPCGNASH,&is_nash);CHKERRQ(ierr);
+  ierr = PetscStrcmp(ksp_type,KSPCGSTCG,&is_stcg);CHKERRQ(ierr);
+  ierr = PetscStrcmp(ksp_type,KSPCGGLTR,&is_gltr);CHKERRQ(ierr);
 
-  if (NLS_KSP_NASH == nlsP->ksp_type || NLS_KSP_STCG == nlsP->ksp_type || NLS_KSP_GLTR == nlsP->ksp_type) {
+  ierr = KSPCGSetRadius(tao->ksp,nlsP->max_radius);CHKERRQ(ierr);
+
+  if (is_nash || is_stcg || is_gltr) {
+    if (tao->trust0 < 0.0) SETERRQ(PETSC_COMM_SELF,1,"Initial radius negative");
     tao->trust = tao->trust0;
-
-    if (tao->trust < 0.0) SETERRQ(PETSC_COMM_SELF,1, "Initial radius negative");
-
-    /* Modify the radius if it is too large or small */
     tao->trust = PetscMax(tao->trust, nlsP->min_radius);
     tao->trust = PetscMin(tao->trust, nlsP->max_radius);
   }
 
-  /* Get vectors we will need */
-  if (NLS_PC_BFGS == nlsP->pc_type && !nlsP->M) {
-    ierr = VecGetLocalSize(tao->solution,&n);CHKERRQ(ierr);
-    ierr = VecGetSize(tao->solution,&N);CHKERRQ(ierr);
-    ierr = MatCreateLMVM(((PetscObject)tao)->comm,n,N,&nlsP->M);CHKERRQ(ierr);
-    ierr = MatLMVMAllocateVectors(nlsP->M,tao->solution);CHKERRQ(ierr);
-  }
-
   /* Check convergence criteria */
   ierr = TaoComputeObjectiveAndGradient(tao, tao->solution, &f, tao->gradient);CHKERRQ(ierr);
-  ierr = VecNorm(tao->gradient,NORM_2,&gnorm);CHKERRQ(ierr);
+  ierr = TaoGradientNorm(tao, tao->gradient,NORM_2,&gnorm);CHKERRQ(ierr);
   if (PetscIsInfOrNanReal(f) || PetscIsInfOrNanReal(gnorm)) SETERRQ(PETSC_COMM_SELF,1, "User provided compute function generated Inf or NaN");
-  needH = 1;
+  
+  tao->reason = TAO_CONTINUE_ITERATING;
+  ierr = TaoLogConvergenceHistory(tao,f,gnorm,0.0,tao->ksp_its);CHKERRQ(ierr);
+  ierr = TaoMonitor(tao,tao->niter,f,gnorm,0.0,step);CHKERRQ(ierr);
+  ierr = (*tao->ops->convergencetest)(tao,tao->cnvP);CHKERRQ(ierr);
+  if (tao->reason != TAO_CONTINUE_ITERATING) PetscFunctionReturn(0);
 
-  ierr = TaoMonitor(tao, tao->niter, f, gnorm, 0.0, 1.0, &reason);CHKERRQ(ierr);
-  if (reason != TAO_CONTINUE_ITERATING) PetscFunctionReturn(0);
-
-  /* create vectors for the limited memory preconditioner */
-  if ((NLS_PC_BFGS == nlsP->pc_type) && (BFGS_SCALE_BFGS != nlsP->bfgs_scale_type)) {
-    if (!nlsP->Diag) {
-      ierr = VecDuplicate(tao->solution,&nlsP->Diag);CHKERRQ(ierr);
-    }
-  }
-
-  /* Modify the preconditioner to use the bfgs approximation */
+  /* Allocate the vectors needed for the BFGS approximation */
   ierr = KSPGetPC(tao->ksp, &pc);CHKERRQ(ierr);
-  switch(nlsP->pc_type) {
-  case NLS_PC_NONE:
-    ierr = PCSetType(pc, PCNONE);CHKERRQ(ierr);
-    ierr = PCSetFromOptions(pc);CHKERRQ(ierr);
-    break;
-
-  case NLS_PC_AHESS:
-    ierr = PCSetType(pc, PCJACOBI);CHKERRQ(ierr);
-    ierr = PCSetFromOptions(pc);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)pc, PCLMVM, &is_bfgs);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)pc, PCJACOBI, &is_jacobi);CHKERRQ(ierr);
+  if (is_bfgs) {
+    nlsP->bfgs_pre = pc;
+    ierr = PCLMVMGetMatLMVM(nlsP->bfgs_pre, &nlsP->M);CHKERRQ(ierr);
+    ierr = VecGetLocalSize(tao->solution, &n);CHKERRQ(ierr);
+    ierr = VecGetSize(tao->solution, &N);CHKERRQ(ierr);
+    ierr = MatSetSizes(nlsP->M, n, n, N, N);CHKERRQ(ierr);
+    ierr = MatLMVMAllocate(nlsP->M, tao->solution, tao->gradient);CHKERRQ(ierr);
+    ierr = MatIsSymmetricKnown(nlsP->M, &sym_set, &is_symmetric);CHKERRQ(ierr);
+    if (!sym_set || !is_symmetric) SETERRQ(PetscObjectComm((PetscObject)tao), PETSC_ERR_ARG_INCOMP, "LMVM matrix in the LMVM preconditioner must be symmetric.");
+  } else if (is_jacobi) {
     ierr = PCJacobiSetUseAbs(pc,PETSC_TRUE);CHKERRQ(ierr);
-    break;
-
-  case NLS_PC_BFGS:
-    ierr = PCSetType(pc, PCSHELL);CHKERRQ(ierr);
-    ierr = PCSetFromOptions(pc);CHKERRQ(ierr);
-    ierr = PCShellSetName(pc, "bfgs");CHKERRQ(ierr);
-    ierr = PCShellSetContext(pc, nlsP->M);CHKERRQ(ierr);
-    ierr = PCShellSetApply(pc, MatLMVMSolveShell);CHKERRQ(ierr);
-    break;
-
-  default:
-    /* Use the pc method set by pc_type */
-    break;
   }
 
   /* Initialize trust-region radius.  The initialization is only performed
      when we are using Nash, Steihaug-Toint or the Generalized Lanczos method. */
-  if (NLS_KSP_NASH == nlsP->ksp_type || NLS_KSP_STCG == nlsP->ksp_type || NLS_KSP_GLTR == nlsP->ksp_type) {
+  if (is_nash || is_stcg || is_gltr) {
     switch(nlsP->init_type) {
     case NLS_INIT_CONSTANT:
       /* Use the initial radius specified */
@@ -225,7 +140,7 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
         sigma = 0.0;
 
         if (needH) {
-          ierr = TaoComputeHessian(tao, tao->solution,tao->hessian,tao->hessian_pre);CHKERRQ(ierr);
+          ierr  = TaoComputeHessian(tao, tao->solution,tao->hessian,tao->hessian_pre);CHKERRQ(ierr);
           needH = 0;
         }
 
@@ -308,12 +223,14 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
           ierr = VecAXPY(tao->solution,sigma,tao->gradient);CHKERRQ(ierr);
           ierr = TaoComputeGradient(tao,tao->solution,tao->gradient);CHKERRQ(ierr);
 
-          ierr = VecNorm(tao->gradient,NORM_2,&gnorm);CHKERRQ(ierr);
+          ierr = TaoGradientNorm(tao, tao->gradient,NORM_2,&gnorm);CHKERRQ(ierr);
           if (PetscIsInfOrNanReal(gnorm)) SETERRQ(PETSC_COMM_SELF,1, "User provided compute gradient generated Inf or NaN");
           needH = 1;
 
-          ierr = TaoMonitor(tao, tao->niter, f, gnorm, 0.0, 1.0, &reason);CHKERRQ(ierr);
-          if (reason != TAO_CONTINUE_ITERATING) PetscFunctionReturn(0);
+          ierr = TaoLogConvergenceHistory(tao,f,gnorm,0.0,tao->ksp_its);CHKERRQ(ierr);
+          ierr = TaoMonitor(tao,tao->niter,f,gnorm,0.0,step);CHKERRQ(ierr);
+          ierr = (*tao->ops->convergencetest)(tao,tao->cnvP);CHKERRQ(ierr);
+          if (tao->reason != TAO_CONTINUE_ITERATING) PetscFunctionReturn(0);
         }
       }
       tao->trust = PetscMax(tao->trust, max_radius);
@@ -330,41 +247,19 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
     }
   }
 
-  /* Set initial scaling for the BFGS preconditioner
-     This step is done after computing the initial trust-region radius
-     since the function value may have decreased */
-  if (NLS_PC_BFGS == nlsP->pc_type) {
-    if (f != 0.0) {
-      delta = 2.0 * PetscAbsScalar(f) / (gnorm*gnorm);
-    } else {
-      delta = 2.0 / (gnorm*gnorm);
-    }
-    ierr = MatLMVMSetDelta(nlsP->M,delta);CHKERRQ(ierr);
-  }
-
   /* Set counter for gradient/reset steps*/
   nlsP->newt = 0;
   nlsP->bfgs = 0;
-  nlsP->sgrad = 0;
   nlsP->grad = 0;
 
   /* Have not converged; continue with Newton method */
-  while (reason == TAO_CONTINUE_ITERATING) {
+  while (tao->reason == TAO_CONTINUE_ITERATING) {
     ++tao->niter;
     tao->ksp_its=0;
 
     /* Compute the Hessian */
     if (needH) {
       ierr = TaoComputeHessian(tao,tao->solution,tao->hessian,tao->hessian_pre);CHKERRQ(ierr);
-      needH = 0;
-    }
-
-    if ((NLS_PC_BFGS == nlsP->pc_type) && (BFGS_SCALE_AHESS == nlsP->bfgs_scale_type)) {
-      /* Obtain diagonal for the bfgs preconditioner  */
-      ierr = MatGetDiagonal(tao->hessian, nlsP->Diag);CHKERRQ(ierr);
-      ierr = VecAbs(nlsP->Diag);CHKERRQ(ierr);
-      ierr = VecReciprocal(nlsP->Diag);CHKERRQ(ierr);
-      ierr = MatLMVMSetScale(nlsP->M,nlsP->Diag);CHKERRQ(ierr);
     }
 
     /* Shift the Hessian matrix */
@@ -375,43 +270,20 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
       }
     }
 
-    if (NLS_PC_BFGS == nlsP->pc_type) {
-      if (BFGS_SCALE_PHESS == nlsP->bfgs_scale_type) {
-        /* Obtain diagonal for the bfgs preconditioner  */
-        ierr = MatGetDiagonal(tao->hessian, nlsP->Diag);CHKERRQ(ierr);
-        ierr = VecAbs(nlsP->Diag);CHKERRQ(ierr);
-        ierr = VecReciprocal(nlsP->Diag);CHKERRQ(ierr);
-        ierr = MatLMVMSetScale(nlsP->M,nlsP->Diag);CHKERRQ(ierr);
-      }
-      /* Update the limited memory preconditioner */
+    if (nlsP->bfgs_pre) {
       ierr = MatLMVMUpdate(nlsP->M, tao->solution, tao->gradient);CHKERRQ(ierr);
       ++bfgsUpdates;
     }
 
     /* Solve the Newton system of equations */
     ierr = KSPSetOperators(tao->ksp,tao->hessian,tao->hessian_pre);CHKERRQ(ierr);
-    if (NLS_KSP_NASH == nlsP->ksp_type || NLS_KSP_STCG == nlsP->ksp_type ||  NLS_KSP_GLTR == nlsP->ksp_type) {
-
-      if (NLS_KSP_NASH == nlsP->ksp_type) {
-        ierr = KSPNASHSetRadius(tao->ksp,nlsP->max_radius);CHKERRQ(ierr);
-      } else if (NLS_KSP_STCG == nlsP->ksp_type) {
-         ierr = KSPSTCGSetRadius(tao->ksp,nlsP->max_radius);CHKERRQ(ierr);
-      } else if (NLS_KSP_GLTR == nlsP->ksp_type) {
-        ierr = KSPGLTRSetRadius(tao->ksp,nlsP->max_radius);CHKERRQ(ierr);
-      }
-
+    if (is_nash || is_stcg || is_gltr) {
+      ierr = KSPCGSetRadius(tao->ksp,nlsP->max_radius);CHKERRQ(ierr);
       ierr = KSPSolve(tao->ksp, tao->gradient, nlsP->D);CHKERRQ(ierr);
       ierr = KSPGetIterationNumber(tao->ksp,&kspits);CHKERRQ(ierr);
       tao->ksp_its+=kspits;
       tao->ksp_tot_its+=kspits;
-
-      if (NLS_KSP_NASH == nlsP->ksp_type) {
-        ierr = KSPNASHGetNormD(tao->ksp,&norm_d);CHKERRQ(ierr);
-      } else if (NLS_KSP_STCG == nlsP->ksp_type) {
-         ierr = KSPSTCGGetNormD(tao->ksp,&norm_d);CHKERRQ(ierr);
-      } else if (NLS_KSP_GLTR == nlsP->ksp_type) {
-        ierr = KSPGLTRGetNormD(tao->ksp,&norm_d);CHKERRQ(ierr);
-      }
+      ierr = KSPCGGetNormD(tao->ksp,&norm_d);CHKERRQ(ierr);
 
       if (0.0 == tao->trust) {
         /* Radius was uninitialized; use the norm of the direction */
@@ -430,25 +302,12 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
           tao->trust = PetscMax(tao->trust, nlsP->min_radius);
           tao->trust = PetscMin(tao->trust, nlsP->max_radius);
 
-          if (NLS_KSP_NASH == nlsP->ksp_type) {
-            ierr = KSPNASHSetRadius(tao->ksp,nlsP->max_radius);CHKERRQ(ierr);
-          } else if (NLS_KSP_STCG == nlsP->ksp_type) {
-            ierr = KSPSTCGSetRadius(tao->ksp,nlsP->max_radius);CHKERRQ(ierr);
-          } else if (NLS_KSP_GLTR == nlsP->ksp_type) {
-            ierr = KSPGLTRSetRadius(tao->ksp,nlsP->max_radius);CHKERRQ(ierr);
-          }
-
+          ierr = KSPCGSetRadius(tao->ksp,nlsP->max_radius);CHKERRQ(ierr);
           ierr = KSPSolve(tao->ksp, tao->gradient, nlsP->D);CHKERRQ(ierr);
           ierr = KSPGetIterationNumber(tao->ksp,&kspits);CHKERRQ(ierr);
           tao->ksp_its+=kspits;
           tao->ksp_tot_its+=kspits;
-          if (NLS_KSP_NASH == nlsP->ksp_type) {
-            ierr = KSPNASHGetNormD(tao->ksp,&norm_d);CHKERRQ(ierr);
-          } else if (NLS_KSP_STCG == nlsP->ksp_type) {
-            ierr = KSPSTCGGetNormD(tao->ksp,&norm_d);CHKERRQ(ierr);
-          } else if (NLS_KSP_GLTR == nlsP->ksp_type) {
-            ierr = KSPGLTRGetNormD(tao->ksp,&norm_d);CHKERRQ(ierr);
-          }
+          ierr = KSPCGGetNormD(tao->ksp,&norm_d);CHKERRQ(ierr);
 
           if (norm_d == 0.0) SETERRQ(PETSC_COMM_SELF,1, "Initial direction zero");
         }
@@ -461,17 +320,10 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
     }
     ierr = VecScale(nlsP->D, -1.0);CHKERRQ(ierr);
     ierr = KSPGetConvergedReason(tao->ksp, &ksp_reason);CHKERRQ(ierr);
-    if ((KSP_DIVERGED_INDEFINITE_PC == ksp_reason) &&  (NLS_PC_BFGS == nlsP->pc_type) && (bfgsUpdates > 1)) {
+    if ((KSP_DIVERGED_INDEFINITE_PC == ksp_reason) &&  (nlsP->bfgs_pre)) {
       /* Preconditioner is numerically indefinite; reset the
          approximate if using BFGS preconditioning. */
-
-      if (f != 0.0) {
-        delta = 2.0 * PetscAbsScalar(f) / (gnorm*gnorm);
-      } else {
-        delta = 2.0 / (gnorm*gnorm);
-      }
-      ierr = MatLMVMSetDelta(nlsP->M,delta);CHKERRQ(ierr);
-      ierr = MatLMVMReset(nlsP->M);CHKERRQ(ierr);
+      ierr = MatLMVMReset(nlsP->M, PETSC_FALSE);CHKERRQ(ierr);
       ierr = MatLMVMUpdate(nlsP->M, tao->solution, tao->gradient);CHKERRQ(ierr);
       bfgsUpdates = 1;
     }
@@ -500,8 +352,8 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
       if (pert <= 0.0) {
         /* Initialize the perturbation */
         pert = PetscMin(nlsP->imax, PetscMax(nlsP->imin, nlsP->imfac * gnorm));
-        if (NLS_KSP_GLTR == nlsP->ksp_type) {
-          ierr = KSPGLTRGetMinEig(tao->ksp,&e_min);CHKERRQ(ierr);
+        if (is_gltr) {
+          ierr = KSPCGGLTRGetMinEig(tao->ksp,&e_min);CHKERRQ(ierr);
           pert = PetscMax(pert, -e_min);
         }
       } else {
@@ -509,7 +361,7 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
         pert = PetscMin(nlsP->pmax, PetscMax(nlsP->pgfac * pert, nlsP->pmgfac * gnorm));
       }
 
-      if (NLS_PC_BFGS != nlsP->pc_type) {
+      if (!nlsP->bfgs_pre) {
         /* We don't have the bfgs matrix around and updated
            Must use gradient direction in this case */
         ierr = VecCopy(tao->gradient, nlsP->D);CHKERRQ(ierr);
@@ -518,7 +370,7 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
         stepType = NLS_GRADIENT;
       } else {
         /* Attempt to use the BFGS direction */
-        ierr = MatLMVMSolve(nlsP->M, tao->gradient, nlsP->D);CHKERRQ(ierr);
+        ierr = MatSolve(nlsP->M, tao->gradient, nlsP->D);CHKERRQ(ierr);
         ierr = VecScale(nlsP->D, -1.0);CHKERRQ(ierr);
 
         /* Check for success (descent direction) */
@@ -530,26 +382,20 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
              which is guaranteed to be descent */
 
           /* Use steepest descent direction (scaled) */
-
-          if (f != 0.0) {
-            delta = 2.0 * PetscAbsScalar(f) / (gnorm*gnorm);
-          } else {
-            delta = 2.0 / (gnorm*gnorm);
-          }
-          ierr = MatLMVMSetDelta(nlsP->M, delta);CHKERRQ(ierr);
-          ierr = MatLMVMReset(nlsP->M);CHKERRQ(ierr);
+          ierr = MatLMVMReset(nlsP->M, PETSC_FALSE);CHKERRQ(ierr);
           ierr = MatLMVMUpdate(nlsP->M, tao->solution, tao->gradient);CHKERRQ(ierr);
-          ierr = MatLMVMSolve(nlsP->M, tao->gradient, nlsP->D);CHKERRQ(ierr);
+          ierr = MatSolve(nlsP->M, tao->gradient, nlsP->D);CHKERRQ(ierr);
           ierr = VecScale(nlsP->D, -1.0);CHKERRQ(ierr);
 
           bfgsUpdates = 1;
-          ++nlsP->sgrad;
-          stepType = NLS_SCALED_GRADIENT;
+          ++nlsP->grad;
+          stepType = NLS_GRADIENT;
         } else {
+          ierr = MatLMVMGetUpdateCount(nlsP->M, &bfgsUpdates);CHKERRQ(ierr);
           if (1 == bfgsUpdates) {
             /* The first BFGS direction is always the scaled gradient */
-            ++nlsP->sgrad;
-            stepType = NLS_SCALED_GRADIENT;
+            ++nlsP->grad;
+            stepType = NLS_GRADIENT;
           } else {
             ++nlsP->bfgs;
             stepType = NLS_BFGS;
@@ -568,8 +414,8 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
         if (pert <= 0.0) {
           /* Initialize the perturbation */
           pert = PetscMin(nlsP->imax, PetscMax(nlsP->imin, nlsP->imfac * gnorm));
-          if (NLS_KSP_GLTR == nlsP->ksp_type) {
-            ierr = KSPGLTRGetMinEig(tao->ksp, &e_min);CHKERRQ(ierr);
+          if (is_gltr) {
+            ierr = KSPCGGLTRGetMinEig(tao->ksp, &e_min);CHKERRQ(ierr);
             pert = PetscMax(pert, -e_min);
           }
         } else {
@@ -611,8 +457,8 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
         if (pert <= 0.0) {
           /* Initialize the perturbation */
           pert = PetscMin(nlsP->imax, PetscMax(nlsP->imin, nlsP->imfac * gnorm));
-          if (NLS_KSP_GLTR == nlsP->ksp_type) {
-            ierr = KSPGLTRGetMinEig(tao->ksp,&e_min);CHKERRQ(ierr);
+          if (is_gltr) {
+            ierr = KSPCGGLTRGetMinEig(tao->ksp,&e_min);CHKERRQ(ierr);
             pert = PetscMax(pert, -e_min);
           }
         } else {
@@ -620,7 +466,7 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
           pert = PetscMin(nlsP->pmax, PetscMax(nlsP->pgfac * pert, nlsP->pmgfac * gnorm));
         }
 
-        if (NLS_PC_BFGS != nlsP->pc_type) {
+        if (!nlsP->bfgs_pre) {
           /* We don't have the bfgs matrix around and being updated
              Must use gradient direction in this case */
           ierr = VecCopy(tao->gradient, nlsP->D);CHKERRQ(ierr);
@@ -628,32 +474,25 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
           stepType = NLS_GRADIENT;
         } else {
           /* Attempt to use the BFGS direction */
-          ierr = MatLMVMSolve(nlsP->M, tao->gradient, nlsP->D);CHKERRQ(ierr);
+          ierr = MatSolve(nlsP->M, tao->gradient, nlsP->D);CHKERRQ(ierr);
           /* Check for success (descent direction) */
           ierr = VecDot(tao->solution, nlsP->D, &gdx);CHKERRQ(ierr);
           if ((gdx <= 0) || PetscIsInfOrNanReal(gdx)) {
             /* BFGS direction is not descent or direction produced not a number
                We can assert bfgsUpdates > 1 in this case
                Use steepest descent direction (scaled) */
-
-            if (f != 0.0) {
-              delta = 2.0 * PetscAbsScalar(f) / (gnorm*gnorm);
-            } else {
-              delta = 2.0 / (gnorm*gnorm);
-            }
-            ierr = MatLMVMSetDelta(nlsP->M, delta);CHKERRQ(ierr);
-            ierr = MatLMVMReset(nlsP->M);CHKERRQ(ierr);
+            ierr = MatLMVMReset(nlsP->M, PETSC_FALSE);CHKERRQ(ierr);
             ierr = MatLMVMUpdate(nlsP->M, tao->solution, tao->gradient);CHKERRQ(ierr);
-            ierr = MatLMVMSolve(nlsP->M, tao->gradient, nlsP->D);CHKERRQ(ierr);
+            ierr = MatSolve(nlsP->M, tao->gradient, nlsP->D);CHKERRQ(ierr);
 
             bfgsUpdates = 1;
-            ++nlsP->sgrad;
-            stepType = NLS_SCALED_GRADIENT;
+            ++nlsP->grad;
+            stepType = NLS_GRADIENT;
           } else {
             if (1 == bfgsUpdates) {
               /* The first BFGS direction is always the scaled gradient */
-              ++nlsP->sgrad;
-              stepType = NLS_SCALED_GRADIENT;
+              ++nlsP->grad;
+              stepType = NLS_GRADIENT;
             } else {
               ++nlsP->bfgs;
               stepType = NLS_BFGS;
@@ -666,33 +505,9 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
         /* Can only enter if pc_type == NLS_PC_BFGS
            Failed to obtain acceptable iterate with BFGS step
            Attempt to use the scaled gradient direction */
-
-        if (f != 0.0) {
-          delta = 2.0 * PetscAbsScalar(f) / (gnorm*gnorm);
-        } else {
-          delta = 2.0 / (gnorm*gnorm);
-        }
-        ierr = MatLMVMSetDelta(nlsP->M, delta);CHKERRQ(ierr);
-        ierr = MatLMVMReset(nlsP->M);CHKERRQ(ierr);
+        ierr = MatLMVMReset(nlsP->M, PETSC_FALSE);CHKERRQ(ierr);
         ierr = MatLMVMUpdate(nlsP->M, tao->solution, tao->gradient);CHKERRQ(ierr);
-        ierr = MatLMVMSolve(nlsP->M, tao->gradient, nlsP->D);CHKERRQ(ierr);
-
-        bfgsUpdates = 1;
-        ++nlsP->sgrad;
-        stepType = NLS_SCALED_GRADIENT;
-        break;
-
-      case NLS_SCALED_GRADIENT:
-        /* Can only enter if pc_type == NLS_PC_BFGS
-           The scaled gradient step did not produce a new iterate;
-           attemp to use the gradient direction.
-           Need to make sure we are not using a different diagonal scaling */
-
-        ierr = MatLMVMSetScale(nlsP->M,0);CHKERRQ(ierr);
-        ierr = MatLMVMSetDelta(nlsP->M,1.0);CHKERRQ(ierr);
-        ierr = MatLMVMReset(nlsP->M);CHKERRQ(ierr);
-        ierr = MatLMVMUpdate(nlsP->M, tao->solution, tao->gradient);CHKERRQ(ierr);
-        ierr = MatLMVMSolve(nlsP->M, tao->gradient, nlsP->D);CHKERRQ(ierr);
+        ierr = MatSolve(nlsP->M, tao->gradient, nlsP->D);CHKERRQ(ierr);
 
         bfgsUpdates = 1;
         ++nlsP->grad;
@@ -712,13 +527,12 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
       ierr = VecCopy(nlsP->Xold, tao->solution);CHKERRQ(ierr);
       ierr = VecCopy(nlsP->Gold, tao->gradient);CHKERRQ(ierr);
       step = 0.0;
-      reason = TAO_DIVERGED_LS_FAILURE;
       tao->reason = TAO_DIVERGED_LS_FAILURE;
       break;
     }
 
     /* Update trust region radius */
-    if (NLS_KSP_NASH == nlsP->ksp_type || NLS_KSP_STCG == nlsP->ksp_type || NLS_KSP_GLTR == nlsP->ksp_type) {
+    if (is_nash || is_stcg || is_gltr) {
       switch(nlsP->update_type) {
       case NLS_UPDATE_STEP:
         if (stepType == NLS_NEWTON) {
@@ -752,14 +566,7 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
         if (stepType == NLS_NEWTON) {
           /*  Get predicted reduction */
 
-          if (NLS_KSP_STCG == nlsP->ksp_type) {
-            ierr = KSPSTCGGetObjFcn(tao->ksp,&prered);CHKERRQ(ierr);
-          } else if (NLS_KSP_NASH == nlsP->ksp_type)  {
-            ierr = KSPNASHGetObjFcn(tao->ksp,&prered);CHKERRQ(ierr);
-          } else {
-            ierr = KSPGLTRGetObjFcn(tao->ksp,&prered);CHKERRQ(ierr);
-          }
-
+          ierr = KSPCGGetObjFcn(tao->ksp,&prered);CHKERRQ(ierr);
           if (prered >= 0.0) {
             /*  The predicted reduction has the wrong sign.  This cannot */
             /*  happen in infinite precision arithmetic.  Step should */
@@ -810,14 +617,7 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
 
       default:
         if (stepType == NLS_NEWTON) {
-
-          if (NLS_KSP_STCG == nlsP->ksp_type) {
-              ierr = KSPSTCGGetObjFcn(tao->ksp,&prered);CHKERRQ(ierr);
-          } else if (NLS_KSP_NASH == nlsP->ksp_type)  {
-              ierr = KSPNASHGetObjFcn(tao->ksp,&prered);CHKERRQ(ierr);
-          } else {
-              ierr = KSPGLTRGetObjFcn(tao->ksp,&prered);CHKERRQ(ierr);
-          }
+          ierr = KSPCGGetObjFcn(tao->ksp,&prered);CHKERRQ(ierr);
           if (prered >= 0.0) {
             /*  The predicted reduction has the wrong sign.  This cannot */
             /*  happen in infinite precision arithmetic.  Step should */
@@ -891,17 +691,17 @@ static PetscErrorCode TaoSolve_NLS(Tao tao)
     }
 
     /*  Check for termination */
-    ierr = VecNorm(tao->gradient, NORM_2, &gnorm);CHKERRQ(ierr);
+    ierr = TaoGradientNorm(tao, tao->gradient,NORM_2,&gnorm);CHKERRQ(ierr);
     if (PetscIsInfOrNanReal(f) || PetscIsInfOrNanReal(gnorm)) SETERRQ(PETSC_COMM_SELF,1,"User provided compute function generated Not-a-Number");
     needH = 1;
-    ierr = TaoMonitor(tao, tao->niter, f, gnorm, 0.0, step, &reason);CHKERRQ(ierr);
+    ierr = TaoLogConvergenceHistory(tao,f,gnorm,0.0,tao->ksp_its);CHKERRQ(ierr);
+    ierr = TaoMonitor(tao,tao->niter,f,gnorm,0.0,step);CHKERRQ(ierr);
+    ierr = (*tao->ops->convergencetest)(tao,tao->cnvP);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
 /* ---------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetUp_NLS"
 static PetscErrorCode TaoSetUp_NLS(Tao tao)
 {
   TAO_NLS        *nlsP = (TAO_NLS *)tao->data;
@@ -914,14 +714,12 @@ static PetscErrorCode TaoSetUp_NLS(Tao tao)
   if (!nlsP->D) {ierr = VecDuplicate(tao->solution,&nlsP->D);CHKERRQ(ierr);}
   if (!nlsP->Xold) {ierr = VecDuplicate(tao->solution,&nlsP->Xold);CHKERRQ(ierr);}
   if (!nlsP->Gold) {ierr = VecDuplicate(tao->solution,&nlsP->Gold);CHKERRQ(ierr);}
-  nlsP->Diag = 0;
   nlsP->M = 0;
+  nlsP->bfgs_pre = 0;
   PetscFunctionReturn(0);
 }
 
 /*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TaoDestroy_NLS"
 static PetscErrorCode TaoDestroy_NLS(Tao tao)
 {
   TAO_NLS        *nlsP = (TAO_NLS *)tao->data;
@@ -934,28 +732,21 @@ static PetscErrorCode TaoDestroy_NLS(Tao tao)
     ierr = VecDestroy(&nlsP->Xold);CHKERRQ(ierr);
     ierr = VecDestroy(&nlsP->Gold);CHKERRQ(ierr);
   }
-  ierr = VecDestroy(&nlsP->Diag);CHKERRQ(ierr);
-  ierr = MatDestroy(&nlsP->M);CHKERRQ(ierr);
   ierr = PetscFree(tao->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 /*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TaoSetFromOptions_NLS"
-static PetscErrorCode TaoSetFromOptions_NLS(PetscOptions *PetscOptionsObject,Tao tao)
+static PetscErrorCode TaoSetFromOptions_NLS(PetscOptionItems *PetscOptionsObject,Tao tao)
 {
   TAO_NLS        *nlsP = (TAO_NLS *)tao->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"Newton line search method for unconstrained optimization");CHKERRQ(ierr);
-  ierr = PetscOptionsEList("-tao_nls_ksp_type", "ksp type", "", NLS_KSP, NLS_KSP_TYPES, NLS_KSP[nlsP->ksp_type], &nlsP->ksp_type, 0);CHKERRQ(ierr);
-  ierr = PetscOptionsEList("-tao_nls_pc_type", "pc type", "", NLS_PC, NLS_PC_TYPES, NLS_PC[nlsP->pc_type], &nlsP->pc_type, 0);CHKERRQ(ierr);
-  ierr = PetscOptionsEList("-tao_nls_bfgs_scale_type", "bfgs scale type", "", BFGS_SCALE, BFGS_SCALE_TYPES, BFGS_SCALE[nlsP->bfgs_scale_type], &nlsP->bfgs_scale_type, 0);CHKERRQ(ierr);
   ierr = PetscOptionsEList("-tao_nls_init_type", "radius initialization type", "", NLS_INIT, NLS_INIT_TYPES, NLS_INIT[nlsP->init_type], &nlsP->init_type, 0);CHKERRQ(ierr);
   ierr = PetscOptionsEList("-tao_nls_update_type", "radius update type", "", NLS_UPDATE, NLS_UPDATE_TYPES, NLS_UPDATE[nlsP->update_type], &nlsP->update_type, 0);CHKERRQ(ierr);
- ierr = PetscOptionsReal("-tao_nls_sval", "perturbation starting value", "", nlsP->sval, &nlsP->sval,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsReal("-tao_nls_sval", "perturbation starting value", "", nlsP->sval, &nlsP->sval,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsReal("-tao_nls_imin", "minimum initial perturbation", "", nlsP->imin, &nlsP->imin,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsReal("-tao_nls_imax", "maximum initial perturbation", "", nlsP->imax, &nlsP->imax,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsReal("-tao_nls_imfac", "initial merit factor", "", nlsP->imfac, &nlsP->imfac,NULL);CHKERRQ(ierr);
@@ -1008,12 +799,9 @@ static PetscErrorCode TaoSetFromOptions_NLS(PetscOptions *PetscOptionsObject,Tao
 
 
 /*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TaoView_NLS"
 static PetscErrorCode TaoView_NLS(Tao tao, PetscViewer viewer)
 {
   TAO_NLS        *nlsP = (TAO_NLS *)tao->data;
-  PetscInt       nrejects;
   PetscBool      isascii;
   PetscErrorCode ierr;
 
@@ -1021,13 +809,8 @@ static PetscErrorCode TaoView_NLS(Tao tao, PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
-    if (NLS_PC_BFGS == nlsP->pc_type && nlsP->M) {
-      ierr = MatLMVMGetRejects(nlsP->M,&nrejects);CHKERRQ(ierr);
-      ierr = PetscViewerASCIIPrintf(viewer, "Rejected matrix updates: %D\n",nrejects);CHKERRQ(ierr);
-    }
     ierr = PetscViewerASCIIPrintf(viewer, "Newton steps: %D\n", nlsP->newt);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer, "BFGS steps: %D\n", nlsP->bfgs);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer, "Scaled gradient steps: %D\n", nlsP->sgrad);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer, "Gradient steps: %D\n", nlsP->grad);CHKERRQ(ierr);
 
     ierr = PetscViewerASCIIPrintf(viewer, "nls ksp atol: %D\n", nlsP->ksp_atol);CHKERRQ(ierr);
@@ -1053,10 +836,7 @@ static PetscErrorCode TaoView_NLS(Tao tao, PetscViewer viewer)
               min_t f(xk + t d_k)
 
     Options Database Keys:
-+ -tao_nls_ksp_type - "cg","nash","stcg","gltr","petsc"
-. -tao_nls_pc_type - "none","ahess","bfgs","petsc"
-. -tao_nls_bfgs_scale_type - "ahess","phess","bfgs"
-. -tao_nls_init_type - "constant","direction","interpolation"
++ -tao_nls_init_type - "constant","direction","interpolation"
 . -tao_nls_update_type - "step","direction","interpolation"
 . -tao_nls_sval - perturbation starting value
 . -tao_nls_imin - minimum initial perturbation
@@ -1100,8 +880,6 @@ static PetscErrorCode TaoView_NLS(Tao tao, PetscViewer viewer)
   Level: beginner
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TaoCreate_NLS"
 PETSC_EXTERN PetscErrorCode TaoCreate_NLS(Tao tao)
 {
   TAO_NLS        *nlsP;
@@ -1120,13 +898,6 @@ PETSC_EXTERN PetscErrorCode TaoCreate_NLS(Tao tao)
   /* Override default settings (unless already changed) */
   if (!tao->max_it_changed) tao->max_it = 50;
   if (!tao->trust0_changed) tao->trust0 = 100.0;
-#if defined(PETSC_USE_REAL_SINGLE)
-  if (!tao->fatol_changed) tao->fatol = 1.0e-5;
-  if (!tao->frtol_changed) tao->frtol = 1.0e-5;
-#else
-  if (!tao->fatol_changed) tao->fatol = 1.0e-10;
-  if (!tao->frtol_changed) tao->frtol = 1.0e-10;
-#endif
 
   tao->data = (void*)nlsP;
 
@@ -1193,35 +964,20 @@ PETSC_EXTERN PetscErrorCode TaoCreate_NLS(Tao tao)
   nlsP->max_radius = 1.0e10;
   nlsP->epsilon = 1.0e-6;
 
-  nlsP->ksp_type        = NLS_KSP_STCG;
-  nlsP->pc_type         = NLS_PC_BFGS;
-  nlsP->bfgs_scale_type = BFGS_SCALE_PHESS;
   nlsP->init_type       = NLS_INIT_INTERPOLATION;
   nlsP->update_type     = NLS_UPDATE_STEP;
 
   ierr = TaoLineSearchCreate(((PetscObject)tao)->comm,&tao->linesearch);CHKERRQ(ierr);
+  ierr = PetscObjectIncrementTabLevel((PetscObject)tao->linesearch,(PetscObject)tao,1);CHKERRQ(ierr);
   ierr = TaoLineSearchSetType(tao->linesearch,morethuente_type);CHKERRQ(ierr);
   ierr = TaoLineSearchUseTaoRoutines(tao->linesearch,tao);CHKERRQ(ierr);
   ierr = TaoLineSearchSetOptionsPrefix(tao->linesearch,tao->hdr.prefix);CHKERRQ(ierr);
 
   /*  Set linear solver to default for symmetric matrices */
   ierr = KSPCreate(((PetscObject)tao)->comm,&tao->ksp);CHKERRQ(ierr);
-  ierr = KSPSetOptionsPrefix(tao->ksp, tao->hdr.prefix);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "MatLMVMSolveShell"
-static PetscErrorCode MatLMVMSolveShell(PC pc, Vec b, Vec x)
-{
-  PetscErrorCode ierr;
-  Mat            M;
-
-  PetscFunctionBegin;
-  PetscValidHeaderSpecific(pc,PC_CLASSID,1);
-  PetscValidHeaderSpecific(b,VEC_CLASSID,2);
-  PetscValidHeaderSpecific(x,VEC_CLASSID,3);
-  ierr = PCShellGetContext(pc,(void**)&M);CHKERRQ(ierr);
-  ierr = MatLMVMSolve(M, b, x);CHKERRQ(ierr);
+  ierr = PetscObjectIncrementTabLevel((PetscObject)tao->ksp,(PetscObject)tao,1);CHKERRQ(ierr);
+  ierr = KSPSetOptionsPrefix(tao->ksp,tao->hdr.prefix);CHKERRQ(ierr);
+  ierr = KSPAppendOptionsPrefix(tao->ksp,"tao_nls_");CHKERRQ(ierr);
+  ierr = KSPSetType(tao->ksp,KSPCGSTCG);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
