@@ -1,17 +1,18 @@
 
 #include <petsc/private/tsimpl.h> /*I  "petscts.h" I*/
 
+PetscClassId TSADAPT_CLASSID;
+
 static PetscFunctionList TSAdaptList;
 static PetscBool         TSAdaptPackageInitialized;
 static PetscBool         TSAdaptRegisterAllCalled;
-static PetscClassId      TSADAPT_CLASSID;
 
-PETSC_EXTERN PetscErrorCode TSAdaptCreate_Basic(TSAdapt);
 PETSC_EXTERN PetscErrorCode TSAdaptCreate_None(TSAdapt);
+PETSC_EXTERN PetscErrorCode TSAdaptCreate_Basic(TSAdapt);
+PETSC_EXTERN PetscErrorCode TSAdaptCreate_DSP(TSAdapt);
 PETSC_EXTERN PetscErrorCode TSAdaptCreate_CFL(TSAdapt);
+PETSC_EXTERN PetscErrorCode TSAdaptCreate_GLEE(TSAdapt);
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptRegister"
 /*@C
    TSAdaptRegister -  adds a TSAdapt implementation
 
@@ -45,12 +46,11 @@ PetscErrorCode  TSAdaptRegister(const char sname[],PetscErrorCode (*function)(TS
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  ierr = TSAdaptInitializePackage();CHKERRQ(ierr);
   ierr = PetscFunctionListAdd(&TSAdaptList,sname,function);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptRegisterAll"
 /*@C
   TSAdaptRegisterAll - Registers all of the adaptivity schemes in TSAdapt
 
@@ -69,16 +69,16 @@ PetscErrorCode  TSAdaptRegisterAll(void)
   PetscFunctionBegin;
   if (TSAdaptRegisterAllCalled) PetscFunctionReturn(0);
   TSAdaptRegisterAllCalled = PETSC_TRUE;
-  ierr = TSAdaptRegister(TSADAPTBASIC,TSAdaptCreate_Basic);CHKERRQ(ierr);
   ierr = TSAdaptRegister(TSADAPTNONE, TSAdaptCreate_None);CHKERRQ(ierr);
+  ierr = TSAdaptRegister(TSADAPTBASIC,TSAdaptCreate_Basic);CHKERRQ(ierr);
+  ierr = TSAdaptRegister(TSADAPTDSP,  TSAdaptCreate_DSP);CHKERRQ(ierr);
   ierr = TSAdaptRegister(TSADAPTCFL,  TSAdaptCreate_CFL);CHKERRQ(ierr);
+  ierr = TSAdaptRegister(TSADAPTGLEE, TSAdaptCreate_GLEE);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptFinalizePackage"
 /*@C
-  TSFinalizePackage - This function destroys everything in the TS package. It is
+  TSAdaptFinalizePackage - This function destroys everything in the TS package. It is
   called from PetscFinalize().
 
   Level: developer
@@ -97,12 +97,10 @@ PetscErrorCode  TSAdaptFinalizePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptInitializePackage"
 /*@C
   TSAdaptInitializePackage - This function initializes everything in the TSAdapt package. It is
   called from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to
-  TSCreate_GL() when using static libraries.
+  TSAdaptCreate() when using static libraries.
 
   Level: developer
 
@@ -122,8 +120,24 @@ PetscErrorCode  TSAdaptInitializePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptSetType"
+/*@C
+  TSAdaptSetType - sets the approach used for the error adapter, currently there is only TSADAPTBASIC and TSADAPTNONE
+
+  Logicially Collective on TSAdapt
+
+  Input Parameter:
++ adapt - the TS adapter, most likely obtained with TSGetAdapt()
+- type - either  TSADAPTBASIC or TSADAPTNONE
+
+  Options Database:
+. -ts_adapt_type <basic or dsp or none> - to set the adapter type
+
+  Level: intermediate
+
+.keywords: TSAdapt, create
+
+.seealso: TSGetAdapt(), TSAdaptDestroy(), TSAdaptType, TSAdaptGetType()
+@*/
 PetscErrorCode  TSAdaptSetType(TSAdapt adapt,TSAdaptType type)
 {
   PetscBool      match;
@@ -131,6 +145,7 @@ PetscErrorCode  TSAdaptSetType(TSAdapt adapt,TSAdaptType type)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(adapt,TSADAPT_CLASSID,1);
+  PetscValidCharPointer(type,2);
   ierr = PetscObjectTypeCompare((PetscObject)adapt,type,&match);CHKERRQ(ierr);
   if (match) PetscFunctionReturn(0);
   ierr = PetscFunctionListFind(TSAdaptList,type,&r);CHKERRQ(ierr);
@@ -142,8 +157,31 @@ PetscErrorCode  TSAdaptSetType(TSAdapt adapt,TSAdaptType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptSetOptionsPrefix"
+/*@C
+  TSAdaptGetType - gets the TS adapter method type (as a string).
+
+  Not Collective
+
+  Input Parameter:
+. adapt - The TS adapter, most likely obtained with TSGetAdapt()
+
+  Output Parameter:
+. type - The name of TS adapter method
+
+  Level: intermediate
+
+.keywords: TSAdapt, get, type
+.seealso TSAdaptSetType()
+@*/
+PetscErrorCode TSAdaptGetType(TSAdapt adapt,TSAdaptType *type)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(adapt,TSADAPT_CLASSID,1);
+  PetscValidPointer(type,2);
+  *type = ((PetscObject)adapt)->type_name;
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode  TSAdaptSetOptionsPrefix(TSAdapt adapt,const char prefix[])
 {
   PetscErrorCode ierr;
@@ -154,8 +192,6 @@ PetscErrorCode  TSAdaptSetOptionsPrefix(TSAdapt adapt,const char prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptLoad"
 /*@C
   TSAdaptLoad - Loads a TSAdapt that has been stored in binary  with TSAdaptView().
 
@@ -196,19 +232,17 @@ PetscErrorCode  TSAdaptLoad(TSAdapt adapt,PetscViewer viewer)
   if (!isbinary) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Invalid viewer; open viewer with PetscViewerBinaryOpen()");
 
   ierr = PetscViewerBinaryRead(viewer,type,256,NULL,PETSC_CHAR);CHKERRQ(ierr);
-  ierr = TSAdaptSetType(adapt, type);CHKERRQ(ierr);
+  ierr = TSAdaptSetType(adapt,type);CHKERRQ(ierr);
   if (adapt->ops->load) {
     ierr = (*adapt->ops->load)(adapt,viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptView"
 PetscErrorCode  TSAdaptView(TSAdapt adapt,PetscViewer viewer)
 {
   PetscErrorCode ierr;
-  PetscBool      iascii,isbinary;
+  PetscBool      iascii,isbinary,isnone;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(adapt,TSADAPT_CLASSID,1);
@@ -219,7 +253,16 @@ PetscErrorCode  TSAdaptView(TSAdapt adapt,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERBINARY,&isbinary);CHKERRQ(ierr);
   if (iascii) {
     ierr = PetscObjectPrintClassNamePrefixType((PetscObject)adapt,viewer);CHKERRQ(ierr);
-    ierr = PetscViewerASCIIPrintf(viewer,"  number of candidates %D\n",adapt->candidates.n);CHKERRQ(ierr);
+    ierr = PetscObjectTypeCompare((PetscObject)adapt,TSADAPTNONE,&isnone);CHKERRQ(ierr);
+    if (!isnone) {
+      if (adapt->always_accept) {ierr = PetscViewerASCIIPrintf(viewer,"  always accepting steps\n");CHKERRQ(ierr);}
+      ierr = PetscViewerASCIIPrintf(viewer,"  safety factor %g\n",(double)adapt->safety);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  extra safety factor after step rejection %g\n",(double)adapt->reject_safety);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  clip fastest increase %g\n",(double)adapt->clip[1]);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  clip fastest decrease %g\n",(double)adapt->clip[0]);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  maximum allowed timestep %g\n",(double)adapt->dt_max);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"  minimum allowed timestep %g\n",(double)adapt->dt_min);CHKERRQ(ierr);
+    }
     if (adapt->ops->view) {
       ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
       ierr = (*adapt->ops->view)(adapt,viewer);CHKERRQ(ierr);
@@ -237,8 +280,6 @@ PetscErrorCode  TSAdaptView(TSAdapt adapt,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptReset"
 /*@
    TSAdaptReset - Resets a TSAdapt context.
 
@@ -261,8 +302,6 @@ PetscErrorCode  TSAdaptReset(TSAdapt adapt)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptDestroy"
 PetscErrorCode  TSAdaptDestroy(TSAdapt *adapt)
 {
   PetscErrorCode ierr;
@@ -280,8 +319,6 @@ PetscErrorCode  TSAdaptDestroy(TSAdapt *adapt)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptSetMonitor"
 /*@
    TSAdaptSetMonitor - Monitor the choices made by the adaptive controller
 
@@ -291,6 +328,9 @@ PetscErrorCode  TSAdaptDestroy(TSAdapt *adapt)
 +  adapt - adaptive controller context
 -  flg - PETSC_TRUE to active a monitor, PETSC_FALSE to disable
 
+   Options Database Keys:
+.  -ts_adapt_monitor - to turn on monitoring
+
    Level: intermediate
 
 .seealso: TSAdaptChoose()
@@ -310,10 +350,8 @@ PetscErrorCode TSAdaptSetMonitor(TSAdapt adapt,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptSetCheckStage"
 /*@C
-   TSAdaptSetCheckStage - set a callback to check convergence for a stage
+   TSAdaptSetCheckStage - Set a callback to check convergence for a stage
 
    Logically collective on TSAdapt
 
@@ -332,7 +370,7 @@ $  PetscErrorCode func(TSAdapt adapt,TS ts,PetscBool *accept)
 
 .seealso: TSAdaptChoose()
 @*/
-PetscErrorCode TSAdaptSetCheckStage(TSAdapt adapt,PetscErrorCode (*func)(TSAdapt,TS,PetscBool*))
+PetscErrorCode TSAdaptSetCheckStage(TSAdapt adapt,PetscErrorCode (*func)(TSAdapt,TS,PetscReal,Vec,PetscBool*))
 {
 
   PetscFunctionBegin;
@@ -341,12 +379,154 @@ PetscErrorCode TSAdaptSetCheckStage(TSAdapt adapt,PetscErrorCode (*func)(TSAdapt
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptSetStepLimits"
 /*@
-   TSAdaptSetStepLimits - Set minimum and maximum step sizes to be considered by the controller
+   TSAdaptSetAlwaysAccept - Set whether to always accept steps regardless of
+   any error or stability condition not meeting the prescribed goal.
 
-   Logically Collective
+   Logically collective on TSAdapt
+
+   Input Arguments:
++  adapt - time step adaptivity context, usually gotten with TSGetAdapt()
+-  flag - whether to always accept steps
+
+   Options Database Keys:
+.  -ts_adapt_always_accept - to always accept steps
+
+   Level: intermediate
+
+.seealso: TSAdapt, TSAdaptChoose()
+@*/
+PetscErrorCode TSAdaptSetAlwaysAccept(TSAdapt adapt,PetscBool flag)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(adapt,TSADAPT_CLASSID,1);
+  PetscValidLogicalCollectiveBool(adapt,flag,2);
+  adapt->always_accept = flag;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   TSAdaptSetSafety - Set safety factors
+
+   Logically collective on TSAdapt
+
+   Input Arguments:
++  adapt - adaptive controller context
+.  safety - safety factor relative to target error/stability goal
+-  reject_safety - extra safety factor to apply if the last step was rejected
+
+   Options Database Keys:
++  -ts_adapt_safety <safety> - to set safety factor
+-  -ts_adapt_reject_safety <reject_safety> - to set reject safety factor
+
+   Level: intermediate
+
+.seealso: TSAdapt, TSAdaptGetSafety(), TSAdaptChoose()
+@*/
+PetscErrorCode TSAdaptSetSafety(TSAdapt adapt,PetscReal safety,PetscReal reject_safety)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(adapt,TSADAPT_CLASSID,1);
+  PetscValidLogicalCollectiveReal(adapt,safety,2);
+  PetscValidLogicalCollectiveReal(adapt,reject_safety,3);
+  if (safety != PETSC_DEFAULT && safety < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Safety factor %g must be non negative",(double)safety);
+  if (safety != PETSC_DEFAULT && safety > 1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Safety factor %g must be less than one",(double)safety);
+  if (reject_safety != PETSC_DEFAULT && reject_safety < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Reject safety factor %g must be non negative",(double)reject_safety);
+  if (reject_safety != PETSC_DEFAULT && reject_safety > 1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Reject safety factor %g must be less than one",(double)reject_safety);
+  if (safety != PETSC_DEFAULT) adapt->safety = safety;
+  if (reject_safety != PETSC_DEFAULT) adapt->reject_safety = reject_safety;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   TSAdaptGetSafety - Get safety factors
+
+   Not Collective
+
+   Input Arguments:
+.  adapt - adaptive controller context
+
+   Ouput Arguments:
+.  safety - safety factor relative to target error/stability goal
++  reject_safety - extra safety factor to apply if the last step was rejected
+
+   Level: intermediate
+
+.seealso: TSAdapt, TSAdaptSetSafety(), TSAdaptChoose()
+@*/
+PetscErrorCode TSAdaptGetSafety(TSAdapt adapt,PetscReal *safety,PetscReal *reject_safety)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(adapt,TSADAPT_CLASSID,1);
+  if (safety)        PetscValidRealPointer(safety,2);
+  if (reject_safety) PetscValidRealPointer(reject_safety,3);
+  if (safety)        *safety        = adapt->safety;
+  if (reject_safety) *reject_safety = adapt->reject_safety;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   TSAdaptSetClip - Sets the admissible decrease/increase factor in step size
+
+   Logically collective on TSAdapt
+
+   Input Arguments:
++  adapt - adaptive controller context
+.  low - admissible decrease factor
+-  high - admissible increase factor
+
+   Options Database Keys:
+.  -ts_adapt_clip <low>,<high> - to set admissible time step decrease and increase factors
+
+   Level: intermediate
+
+.seealso: TSAdaptChoose(), TSAdaptGetClip()
+@*/
+PetscErrorCode TSAdaptSetClip(TSAdapt adapt,PetscReal low,PetscReal high)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(adapt,TSADAPT_CLASSID,1);
+  PetscValidLogicalCollectiveReal(adapt,low,2);
+  PetscValidLogicalCollectiveReal(adapt,high,3);
+  if (low  != PETSC_DEFAULT && low  < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Decrease factor %g must be non negative",(double)low);
+  if (low  != PETSC_DEFAULT && low  > 1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Decrease factor %g must be less than one",(double)low);
+  if (high != PETSC_DEFAULT && high < 1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Increase factor %g must be geather than one",(double)high);
+  if (low  != PETSC_DEFAULT) adapt->clip[0] = low;
+  if (high != PETSC_DEFAULT) adapt->clip[1] = high;
+  PetscFunctionReturn(0);
+}
+
+/*@
+   TSAdaptGetClip - Gets the admissible decrease/increase factor in step size
+
+   Not Collective
+
+   Input Arguments:
+.  adapt - adaptive controller context
+
+   Ouput Arguments:
++  low - optional, admissible decrease factor
+-  high - optional, admissible increase factor
+
+   Level: intermediate
+
+.seealso: TSAdaptChoose(), TSAdaptSetClip()
+@*/
+PetscErrorCode TSAdaptGetClip(TSAdapt adapt,PetscReal *low,PetscReal *high)
+{
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(adapt,TSADAPT_CLASSID,1);
+  if (low)  PetscValidRealPointer(low,2);
+  if (high) PetscValidRealPointer(high,3);
+  if (low)  *low  = adapt->clip[0];
+  if (high) *high = adapt->clip[1];
+  PetscFunctionReturn(0);
+}
+
+/*@
+   TSAdaptSetStepLimits - Set the minimum and maximum step sizes to be considered by the controller
+
+   Logically collective on TSAdapt
 
    Input Arguments:
 +  adapt - time step adaptivity context, usually gotten with TSGetAdapt()
@@ -354,25 +534,58 @@ PetscErrorCode TSAdaptSetCheckStage(TSAdapt adapt,PetscErrorCode (*func)(TSAdapt
 -  hmax - maximum time step
 
    Options Database Keys:
-+  -ts_adapt_dt_min - minimum time step
--  -ts_adapt_dt_max - maximum time step
++  -ts_adapt_dt_min <min> - to set minimum time step
+-  -ts_adapt_dt_max <max> - to set maximum time step
 
    Level: intermediate
 
-.seealso: TSAdapt
+.seealso: TSAdapt, TSAdaptGetStepLimits(), TSAdaptChoose()
 @*/
 PetscErrorCode TSAdaptSetStepLimits(TSAdapt adapt,PetscReal hmin,PetscReal hmax)
 {
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(adapt,TSADAPT_CLASSID,1);
-  if (hmin != PETSC_DECIDE) adapt->dt_min = hmin;
-  if (hmax != PETSC_DECIDE) adapt->dt_max = hmax;
+  PetscValidLogicalCollectiveReal(adapt,hmin,2);
+  PetscValidLogicalCollectiveReal(adapt,hmax,3);
+  if (hmin != PETSC_DEFAULT && hmin < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Minimum time step %g must be non negative",(double)hmin);
+  if (hmax != PETSC_DEFAULT && hmax < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Minimum time step %g must be non negative",(double)hmax);
+  if (hmin != PETSC_DEFAULT) adapt->dt_min = hmin;
+  if (hmax != PETSC_DEFAULT) adapt->dt_max = hmax;
+  hmin = adapt->dt_min;
+  hmax = adapt->dt_max;
+  if (hmax <= hmin) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Maximum time step %g must geather than minimum time step %g",(double)hmax,(double)hmin);
+  PetscFunctionReturn(0);
+}
+
+/*@
+   TSAdaptGetStepLimits - Get the minimum and maximum step sizes to be considered by the controller
+
+   Not Collective
+
+   Input Arguments:
+.  adapt - time step adaptivity context, usually gotten with TSGetAdapt()
+
+   Output Arguments:
++  hmin - minimum time step
+-  hmax - maximum time step
+
+   Level: intermediate
+
+.seealso: TSAdapt, TSAdaptSetStepLimits(), TSAdaptChoose()
+@*/
+PetscErrorCode TSAdaptGetStepLimits(TSAdapt adapt,PetscReal *hmin,PetscReal *hmax)
+{
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(adapt,TSADAPT_CLASSID,1);
+  if (hmin) PetscValidRealPointer(hmin,2);
+  if (hmax) PetscValidRealPointer(hmax,3);
+  if (hmin) *hmin = adapt->dt_min;
+  if (hmax) *hmax = adapt->dt_max;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptSetFromOptions"
 /*
    TSAdaptSetFromOptions - Sets various TSAdapt parameters from user options.
 
@@ -382,51 +595,79 @@ PetscErrorCode TSAdaptSetStepLimits(TSAdapt adapt,PetscReal hmin,PetscReal hmax)
 .  adapt - the TSAdapt context
 
    Options Database Keys:
-.  -ts_adapt_type <type> - basic
++  -ts_adapt_type <type> - algorithm to use for adaptivity
+.  -ts_adapt_always_accept - always accept steps regardless of error/stability goals
+.  -ts_adapt_safety <safety> - safety factor relative to target error/stability goal
+.  -ts_adapt_reject_safety <safety> - extra safety factor to apply if the last step was rejected
+.  -ts_adapt_clip <low,high> - admissible time step decrease and increase factors
+.  -ts_adapt_dt_min <min> - minimum timestep to use
+.  -ts_adapt_dt_max <max> - maximum timestep to use
+.  -ts_adapt_scale_solve_failed <scale> - scale timestep by this factor if a solve fails
+-  -ts_adapt_wnormtype <2 or infinity> - type of norm for computing error estimates
 
    Level: advanced
 
    Notes:
    This function is automatically called by TSSetFromOptions()
 
-.keywords: TS, TSGetAdapt(), TSAdaptSetType()
+.keywords: TS, TSGetAdapt(), TSAdaptSetType(), TSAdaptSetStepLimits()
 
-.seealso: TSGetType()
+.seealso: TSGetAdapt(), TSAdaptSetType(), TSAdaptSetAlwaysAccept(), TSAdaptSetSafety(),
+          TSAdaptSetClip(), TSAdaptSetStepLimits(), TSAdaptSetMonitor()
 */
-PetscErrorCode  TSAdaptSetFromOptions(PetscOptions *PetscOptionsObject,TSAdapt adapt)
+PetscErrorCode  TSAdaptSetFromOptions(PetscOptionItems *PetscOptionsObject,TSAdapt adapt)
 {
   PetscErrorCode ierr;
   char           type[256] = TSADAPTBASIC;
+  PetscReal      safety,reject_safety,clip[2],hmin,hmax;
   PetscBool      set,flg;
+  PetscInt       two;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(adapt,TSADAPT_CLASSID,1);
   /* This should use PetscOptionsBegin() if/when this becomes an object used outside of TS, but currently this
-  * function can only be called from inside TSSetFromOptions_GL()  */
+   * function can only be called from inside TSSetFromOptions()  */
   ierr = PetscOptionsHead(PetscOptionsObject,"TS Adaptivity options");CHKERRQ(ierr);
-  ierr = PetscOptionsFList("-ts_adapt_type","Algorithm to use for adaptivity","TSAdaptSetType",TSAdaptList,
-                          ((PetscObject)adapt)->type_name ? ((PetscObject)adapt)->type_name : type,type,sizeof(type),&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsFList("-ts_adapt_type","Algorithm to use for adaptivity","TSAdaptSetType",TSAdaptList,((PetscObject)adapt)->type_name ? ((PetscObject)adapt)->type_name : type,type,sizeof(type),&flg);CHKERRQ(ierr);
   if (flg || !((PetscObject)adapt)->type_name) {
     ierr = TSAdaptSetType(adapt,type);CHKERRQ(ierr);
   }
-  ierr = PetscOptionsReal("-ts_adapt_dt_min","Minimum time step considered","TSAdaptSetStepLimits",adapt->dt_min,&adapt->dt_min,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsReal("-ts_adapt_dt_max","Maximum time step considered","TSAdaptSetStepLimits",adapt->dt_max,&adapt->dt_max,NULL);CHKERRQ(ierr);
+
+  ierr = PetscOptionsBool("-ts_adapt_always_accept","Always accept the step","TSAdaptSetAlwaysAccept",adapt->always_accept,&flg,&set);CHKERRQ(ierr);
+  if (set) {ierr = TSAdaptSetAlwaysAccept(adapt,flg);CHKERRQ(ierr);}
+
+  safety = adapt->safety; reject_safety = adapt->reject_safety;
+  ierr = PetscOptionsReal("-ts_adapt_safety","Safety factor relative to target error/stability goal","TSAdaptSetSafety",safety,&safety,&set);CHKERRQ(ierr);
+  ierr = PetscOptionsReal("-ts_adapt_reject_safety","Extra safety factor to apply if the last step was rejected","TSAdaptSetSafety",reject_safety,&reject_safety,&flg);CHKERRQ(ierr);
+  if (set || flg) {ierr = TSAdaptSetSafety(adapt,safety,reject_safety);CHKERRQ(ierr);}
+
+  two = 2; clip[0] = adapt->clip[0]; clip[1] = adapt->clip[1];
+  ierr = PetscOptionsRealArray("-ts_adapt_clip","Admissible decrease/increase factor in step size","TSAdaptSetClip",clip,&two,&set);CHKERRQ(ierr);
+  if (set && (two != 2)) SETERRQ(PetscObjectComm((PetscObject)adapt),PETSC_ERR_ARG_OUTOFRANGE,"Must give exactly two values to -ts_adapt_clip");
+  if (set) {ierr = TSAdaptSetClip(adapt,clip[0],clip[1]);CHKERRQ(ierr);}
+
+  hmin = adapt->dt_min; hmax = adapt->dt_max;
+  ierr = PetscOptionsReal("-ts_adapt_dt_min","Minimum time step considered","TSAdaptSetStepLimits",hmin,&hmin,&set);CHKERRQ(ierr);
+  ierr = PetscOptionsReal("-ts_adapt_dt_max","Maximum time step considered","TSAdaptSetStepLimits",hmax,&hmax,&flg);CHKERRQ(ierr);
+  if (set || flg) {ierr = TSAdaptSetStepLimits(adapt,hmin,hmax);CHKERRQ(ierr);}
+
   ierr = PetscOptionsReal("-ts_adapt_scale_solve_failed","Scale step by this factor if solve fails","",adapt->scale_solve_failed,&adapt->scale_solve_failed,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsBool("-ts_adapt_monitor","Print choices made by adaptive controller","TSAdaptSetMonitor",adapt->monitor ? PETSC_TRUE : PETSC_FALSE,&flg,&set);CHKERRQ(ierr);
+
   ierr = PetscOptionsEnum("-ts_adapt_wnormtype","Type of norm computed for error estimation","",NormTypes,(PetscEnum)adapt->wnormtype,(PetscEnum*)&adapt->wnormtype,NULL);CHKERRQ(ierr);
   if (adapt->wnormtype != NORM_2 && adapt->wnormtype != NORM_INFINITY) SETERRQ(PetscObjectComm((PetscObject)adapt),PETSC_ERR_SUP,"Only 2-norm and infinite norm supported");
+
+  ierr = PetscOptionsBool("-ts_adapt_monitor","Print choices made by adaptive controller","TSAdaptSetMonitor",adapt->monitor ? PETSC_TRUE : PETSC_FALSE,&flg,&set);CHKERRQ(ierr);
   if (set) {ierr = TSAdaptSetMonitor(adapt,flg);CHKERRQ(ierr);}
+
   if (adapt->ops->setfromoptions) {ierr = (*adapt->ops->setfromoptions)(PetscOptionsObject,adapt);CHKERRQ(ierr);}
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptCandidatesClear"
 /*@
    TSAdaptCandidatesClear - clear any previously set candidate schemes
 
-   Logically Collective
+   Logically collective on TSAdapt
 
    Input Argument:
 .  adapt - adaptive controller
@@ -445,12 +686,10 @@ PetscErrorCode TSAdaptCandidatesClear(TSAdapt adapt)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptCandidateAdd"
 /*@C
    TSAdaptCandidateAdd - add a candidate scheme for the adaptive controller to select from
 
-   Logically Collective
+   Logically collective on TSAdapt
 
    Input Arguments:
 +  adapt - time step adaptivity context, obtained with TSGetAdapt() or TSAdaptCreate()
@@ -491,8 +730,6 @@ PetscErrorCode TSAdaptCandidateAdd(TSAdapt adapt,const char name[],PetscInt orde
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptCandidatesGet"
 /*@C
    TSAdaptCandidatesGet - Get the list of candidate orders of accuracy and cost
 
@@ -527,19 +764,17 @@ PetscErrorCode TSAdaptCandidatesGet(TSAdapt adapt,PetscInt *n,const PetscInt **o
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptChoose"
 /*@C
    TSAdaptChoose - choose which method and step size to use for the next step
 
-   Logically Collective
+   Collective on TSAdapt
 
    Input Arguments:
 +  adapt - adaptive contoller
 -  h - current step size
 
    Output Arguments:
-+  next_sc - scheme to use for the next step
++  next_sc - optional, scheme to use for the next step
 .  next_h - step size to use for the next step
 -  accept - PETSC_TRUE to accept the current step, PETSC_FALSE to repeat the current step with the new step size
 
@@ -554,52 +789,65 @@ PetscErrorCode TSAdaptCandidatesGet(TSAdapt adapt,PetscInt *n,const PetscInt **o
 PetscErrorCode TSAdaptChoose(TSAdapt adapt,TS ts,PetscReal h,PetscInt *next_sc,PetscReal *next_h,PetscBool *accept)
 {
   PetscErrorCode ierr;
+  PetscInt       ncandidates = adapt->candidates.n;
+  PetscInt       scheme = 0;
   PetscReal      wlte = -1.0;
+  PetscReal      wltea = -1.0;
+  PetscReal      wlter = -1.0;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(adapt,TSADAPT_CLASSID,1);
   PetscValidHeaderSpecific(ts,TS_CLASSID,2);
-  PetscValidIntPointer(next_sc,4);
+  if (next_sc) PetscValidIntPointer(next_sc,4);
   PetscValidPointer(next_h,5);
   PetscValidIntPointer(accept,6);
-  if (adapt->candidates.n < 1) SETERRQ1(PetscObjectComm((PetscObject)adapt),PETSC_ERR_ARG_WRONGSTATE,"%D candidates have been registered",adapt->candidates.n);
-  if (!adapt->candidates.inuse_set) SETERRQ1(PetscObjectComm((PetscObject)adapt),PETSC_ERR_ARG_WRONGSTATE,"The current in-use scheme is not among the %D candidates",adapt->candidates.n);
-  ierr = (*adapt->ops->choose)(adapt,ts,h,next_sc,next_h,accept,&wlte);CHKERRQ(ierr);
+  if (next_sc) *next_sc = 0;
+
+  /* Do not mess with adaptivity while handling events*/
+  if (ts->event && ts->event->status != TSEVENT_NONE) {
+    *next_h = h;
+    *accept = PETSC_TRUE;
+    PetscFunctionReturn(0);
+  }
+
+  ierr = (*adapt->ops->choose)(adapt,ts,h,&scheme,next_h,accept,&wlte,&wltea,&wlter);CHKERRQ(ierr);
+  if (scheme < 0 || (ncandidates > 0 && scheme >= ncandidates)) SETERRQ2(PetscObjectComm((PetscObject)adapt),PETSC_ERR_ARG_OUTOFRANGE,"Chosen scheme %D not in valid range 0..%D",scheme,ncandidates-1);
+  if (*next_h < 0) SETERRQ1(PetscObjectComm((PetscObject)adapt),PETSC_ERR_ARG_OUTOFRANGE,"Computed step size %g must be positive",(double)*next_h);
+  if (next_sc) *next_sc = scheme;
+
   if (*accept && ts->exact_final_time == TS_EXACTFINALTIME_MATCHSTEP) {
-    /* Reduce time step if it overshoots max time */
-    PetscReal max_time = ts->max_time;
-    PetscReal next_dt  = 0.0;
-    if (ts->ptime + ts->time_step + *next_h >= max_time) {
-      next_dt = max_time - (ts->ptime + ts->time_step);
-      if (next_dt > PETSC_SMALL) *next_h = next_dt;
-      else ts->reason = TS_CONVERGED_TIME;
-    }
+    /* Increase/reduce step size if end time of next step is close to or overshoots max time */
+    PetscReal t = ts->ptime + ts->time_step, h = *next_h;
+    PetscReal tend = t + h, tmax = ts->max_time, hmax = tmax - t;
+    PetscReal a = (PetscReal)1.01; /* allow 1% step size increase */
+    if (t < tmax && tend > tmax) *next_h = hmax;
+    if (t < tmax && tend < tmax && h > hmax/2) *next_h = hmax/2;
+    if (t < tmax && tend < tmax && h*a > hmax) *next_h = hmax;
   }
-  if (*next_sc < 0 || adapt->candidates.n <= *next_sc) SETERRQ2(PetscObjectComm((PetscObject)adapt),PETSC_ERR_ARG_OUTOFRANGE,"Chosen scheme %D not in valid range 0..%D",*next_sc,adapt->candidates.n-1);
-  if (!(*next_h > 0.)) SETERRQ1(PetscObjectComm((PetscObject)adapt),PETSC_ERR_ARG_OUTOFRANGE,"Computed step size %g must be positive",(double)*next_h);
 
   if (adapt->monitor) {
+    const char *sc_name = (scheme < ncandidates) ? adapt->candidates.name[scheme] : "";
     ierr = PetscViewerASCIIAddTab(adapt->monitor,((PetscObject)adapt)->tablevel);CHKERRQ(ierr);
     if (wlte < 0) {
-      ierr = PetscViewerASCIIPrintf(adapt->monitor,"    TSAdapt '%s': step %3D %s t=%-11g+%10.3e family='%s' scheme=%D:'%s' dt=%-10.3e\n",((PetscObject)adapt)->type_name,ts->steps,*accept ? "accepted" : "rejected",(double)ts->ptime,(double)h,((PetscObject)ts)->type_name,*next_sc,adapt->candidates.name[*next_sc],(double)*next_h);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(adapt->monitor,"    TSAdapt %s %s %D:%s step %3D %s t=%-11g+%10.3e dt=%-10.3e\n",((PetscObject)adapt)->type_name,((PetscObject)ts)->type_name,scheme,sc_name,ts->steps,*accept ? "accepted" : "rejected",(double)ts->ptime,(double)h,(double)*next_h);CHKERRQ(ierr);
     } else {
-      ierr = PetscViewerASCIIPrintf(adapt->monitor,"    TSAdapt '%s': step %3D %s t=%-11g+%10.3e wlte=%5.3g family='%s' scheme=%D:'%s' dt=%-10.3e\n",((PetscObject)adapt)->type_name,ts->steps,*accept ? "accepted" : "rejected",(double)ts->ptime,(double)h,(double)wlte,((PetscObject)ts)->type_name,*next_sc,adapt->candidates.name[*next_sc],(double)*next_h);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(adapt->monitor,"    TSAdapt %s %s %D:%s step %3D %s t=%-11g+%10.3e dt=%-10.3e wlte=%5.3g  wltea=%5.3g wlter=%5.3g\n",((PetscObject)adapt)->type_name,((PetscObject)ts)->type_name,scheme,sc_name,ts->steps,*accept ? "accepted" : "rejected",(double)ts->ptime,(double)h,(double)*next_h,(double)wlte,(double)wltea,(double)wlter);CHKERRQ(ierr);
     }
     ierr = PetscViewerASCIISubtractTab(adapt->monitor,((PetscObject)adapt)->tablevel);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptCheckStage"
 /*@
    TSAdaptCheckStage - checks whether to accept a stage, (e.g. reject and change time step size if nonlinear solve fails)
 
-   Collective
+   Collective on TSAdapt
 
    Input Arguments:
 +  adapt - adaptive controller context
--  ts - time stepper
+.  ts - time stepper
+.  t - Current simulation time
+-  Y - Current solution vector
 
    Output Arguments:
 .  accept - PETSC_TRUE to accept the stage, PETSC_FALSE to reject
@@ -608,49 +856,51 @@ PetscErrorCode TSAdaptChoose(TSAdapt adapt,TS ts,PetscReal h,PetscInt *next_sc,P
 
 .seealso:
 @*/
-PetscErrorCode TSAdaptCheckStage(TSAdapt adapt,TS ts,PetscBool *accept)
+PetscErrorCode TSAdaptCheckStage(TSAdapt adapt,TS ts,PetscReal t,Vec Y,PetscBool *accept)
 {
   PetscErrorCode      ierr;
-  SNES                snes;
-  SNESConvergedReason snesreason;
+  SNESConvergedReason snesreason = SNES_CONVERGED_ITERATING;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(adapt,TSADAPT_CLASSID,1);
   PetscValidHeaderSpecific(ts,TS_CLASSID,2);
   PetscValidIntPointer(accept,3);
-  *accept = PETSC_TRUE;
-  ierr    = TSGetSNES(ts,&snes);CHKERRQ(ierr);
-  ierr    = SNESGetConvergedReason(snes,&snesreason);CHKERRQ(ierr);
+
+  if (ts->snes) {ierr = SNESGetConvergedReason(ts->snes,&snesreason);CHKERRQ(ierr);}
   if (snesreason < 0) {
-    PetscReal dt,new_dt;
     *accept = PETSC_FALSE;
-    ierr    = TSGetTimeStep(ts,&dt);CHKERRQ(ierr);
     if (++ts->num_snes_failures >= ts->max_snes_failures && ts->max_snes_failures > 0) {
       ts->reason = TS_DIVERGED_NONLINEAR_SOLVE;
       ierr = PetscInfo2(ts,"Step=%D, nonlinear solve failures %D greater than current TS allowed, stopping solve\n",ts->steps,ts->num_snes_failures);CHKERRQ(ierr);
       if (adapt->monitor) {
         ierr = PetscViewerASCIIAddTab(adapt->monitor,((PetscObject)adapt)->tablevel);CHKERRQ(ierr);
-        ierr = PetscViewerASCIIPrintf(adapt->monitor,"    TSAdapt '%s': step %3D stage rejected t=%-11g+%10.3e, nonlinear solve failures %D greater than current TS allowed\n",((PetscObject)adapt)->type_name,ts->steps,(double)ts->ptime,dt,ts->num_snes_failures);CHKERRQ(ierr);
-        ierr = PetscViewerASCIISubtractTab(adapt->monitor,((PetscObject)adapt)->tablevel);CHKERRQ(ierr);
-      }
-    } else {
-      new_dt = dt*adapt->scale_solve_failed;
-      ierr   = TSSetTimeStep(ts,new_dt);CHKERRQ(ierr);
-      if (adapt->monitor) {
-        ierr = PetscViewerASCIIAddTab(adapt->monitor,((PetscObject)adapt)->tablevel);CHKERRQ(ierr);
-        ierr = PetscViewerASCIIPrintf(adapt->monitor,"    TSAdapt '%s': step %3D stage rejected t=%-11g+%10.3e retrying with dt=%-10.3e\n",((PetscObject)adapt)->type_name,ts->steps,(double)ts->ptime,(double)dt,(double)new_dt);CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(adapt->monitor,"    TSAdapt %s step %3D stage rejected t=%-11g+%10.3e, nonlinear solve failures %D greater than current TS allowed\n",((PetscObject)adapt)->type_name,ts->steps,(double)ts->ptime,(double)ts->time_step,ts->num_snes_failures);CHKERRQ(ierr);
         ierr = PetscViewerASCIISubtractTab(adapt->monitor,((PetscObject)adapt)->tablevel);CHKERRQ(ierr);
       }
     }
+  } else {
+    *accept = PETSC_TRUE;
+    ierr = TSFunctionDomainError(ts,t,Y,accept);CHKERRQ(ierr);
+    if(*accept && adapt->checkstage) {
+      ierr = (*adapt->checkstage)(adapt,ts,t,Y,accept);CHKERRQ(ierr);
+    }
+  }
+
+  if(!(*accept) && !ts->reason) {
+    PetscReal dt,new_dt;
+    ierr = TSGetTimeStep(ts,&dt);CHKERRQ(ierr);
+    new_dt = dt * adapt->scale_solve_failed;
+    ierr = TSSetTimeStep(ts,new_dt);CHKERRQ(ierr);
+    adapt->timestepjustincreased += 4;
+    if (adapt->monitor) {
+      ierr = PetscViewerASCIIAddTab(adapt->monitor,((PetscObject)adapt)->tablevel);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(adapt->monitor,"    TSAdapt %s step %3D stage rejected (%s) t=%-11g+%10.3e retrying with dt=%-10.3e\n",((PetscObject)adapt)->type_name,ts->steps,SNESConvergedReasons[snesreason],(double)ts->ptime,(double)dt,(double)new_dt);CHKERRQ(ierr);
+      ierr = PetscViewerASCIISubtractTab(adapt->monitor,((PetscObject)adapt)->tablevel);CHKERRQ(ierr);
+    }
   }
-  if (adapt->checkstage) {ierr = (*adapt->checkstage)(adapt,ts,accept);CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
 
-
-
-#undef __FUNCT__
-#define __FUNCT__ "TSAdaptCreate"
 /*@
   TSAdaptCreate - create an adaptive controller context for time stepping
 
@@ -682,8 +932,13 @@ PetscErrorCode  TSAdaptCreate(MPI_Comm comm,TSAdapt *inadapt)
 
   ierr = PetscHeaderCreate(adapt,TSADAPT_CLASSID,"TSAdapt","Time stepping adaptivity","TS",comm,TSAdaptDestroy,TSAdaptView);CHKERRQ(ierr);
 
+  adapt->always_accept      = PETSC_FALSE;
+  adapt->safety             = 0.9;
+  adapt->reject_safety      = 0.5;
+  adapt->clip[0]            = 0.1;
+  adapt->clip[1]            = 10.;
   adapt->dt_min             = 1e-20;
-  adapt->dt_max             = 1e50;
+  adapt->dt_max             = 1e+20;
   adapt->scale_solve_failed = 0.25;
   adapt->wnormtype          = NORM_2;
 
