@@ -2,8 +2,6 @@
 #include <petscdmda.h>
 #include <petscviewer.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicView_DA"
 PetscErrorCode CharacteristicView_DA(Characteristic c, PetscViewer viewer)
 {
   Characteristic_DA *da = (Characteristic_DA*) c->data;
@@ -22,8 +20,6 @@ PetscErrorCode CharacteristicView_DA(Characteristic c, PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicDestroy_DA"
 PetscErrorCode CharacteristicDestroy_DA(Characteristic c)
 {
   Characteristic_DA *da = (Characteristic_DA*) c->data;
@@ -34,8 +30,6 @@ PetscErrorCode CharacteristicDestroy_DA(Characteristic c)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicSetUp_DA"
 PetscErrorCode CharacteristicSetUp_DA(Characteristic c)
 {
   PetscMPIInt    blockLen[2];
@@ -72,8 +66,6 @@ PetscErrorCode CharacteristicSetUp_DA(Characteristic c)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CharacteristicCreate_DA"
 PETSC_EXTERN PetscErrorCode CharacteristicCreate_DA(Characteristic c)
 {
   Characteristic_DA *da;
@@ -93,8 +85,6 @@ PETSC_EXTERN PetscErrorCode CharacteristicCreate_DA(Characteristic c)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMDAMapCoordsToPeriodicDomain"
 /* -----------------------------------------------------------------------------
    Checks for periodicity of a DM and Maps points outside of a domain back onto the domain
    using appropriate periodicity. At the moment assumes only a 2-D DMDA.
@@ -106,7 +96,7 @@ PetscErrorCode DMDAMapCoordsToPeriodicDomain(DM da, PetscScalar *x, PetscScalar
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = DMDAGetInfo(da, &dim, &gx, &gy, 0, 0, 0, 0, 0, 0, &bx, &by, 0, 0);
+  ierr = DMDAGetInfo(da, &dim, &gx, &gy, 0, 0, 0, 0, 0, 0, &bx, &by, 0, 0);CHKERRQ(ierr);
 
   if (bx == DM_BOUNDARY_PERIODIC) {
       while (*x >= (PetscScalar)gx) *x -= (PetscScalar)gx;
@@ -116,5 +106,5 @@ PetscErrorCode DMDAMapCoordsToPeriodicDomain(DM da, PetscScalar *x, PetscScalar
       while (*y >= (PetscScalar)gy) *y -= (PetscScalar)gy;
       while (*y < 0.0)              *y += (PetscScalar)gy;
     }
-  PetscFunctionReturn(ierr);
+  PetscFunctionReturn(0);
 }
