@@ -6,7 +6,7 @@ static char help[] = "Nonlinear, time-dependent. Developed from radiative_surfac
     mpiexec -n <np> ./ex5 [options]
     ./ex5 -help  [view petsc options]
     ./ex5 -ts_type sundials -ts_view
-    ./ex5 -da_grid_x 20 -da_grid_y 20 -log_summary
+    ./ex5 -da_grid_x 20 -da_grid_y 20 -log_view
     ./ex5 -da_grid_x 20 -da_grid_y 20 -ts_type rosw -ts_atol 1.e-6 -ts_rtol 1.e-6
     ./ex5 -drawcontours -draw_pause 0.1 -draw_fields 0,1,2,3,4
 */
@@ -144,8 +144,6 @@ extern PetscErrorCode potential_temperature(PetscScalar, PetscScalar, PetscScala
 extern PetscErrorCode latentflux(PetscScalar, PetscScalar, PetscScalar, PetscScalar, PetscScalar*);             /* calculates latent heat flux */
 extern PetscErrorCode calc_gflux(PetscScalar, PetscScalar, PetscScalar*);                                       /* calculates flux between top soil layer and underlying earth */
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   PetscErrorCode ierr;
@@ -176,7 +174,7 @@ int main(int argc,char **argv)
   MatFDColoring  matfdcoloring = 0;
   PetscBool      monitor_off = PETSC_FALSE;
 
-  PetscInitialize(&argc,&argv,(char*)0,help);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
   ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank);CHKERRQ(ierr);
 
@@ -221,8 +219,9 @@ int main(int argc,char **argv)
   /*------------------------------------------*/
 
   /* Create grid */
-  ierr = DMDACreate2d(PETSC_COMM_WORLD,DM_BOUNDARY_PERIODIC,DM_BOUNDARY_PERIODIC,DMDA_STENCIL_STAR,-20,-20,
-                      PETSC_DECIDE,PETSC_DECIDE,dof,1,NULL,NULL,&da);CHKERRQ(ierr);
+  ierr = DMDACreate2d(PETSC_COMM_WORLD,DM_BOUNDARY_PERIODIC,DM_BOUNDARY_PERIODIC,DMDA_STENCIL_STAR,20,20,PETSC_DECIDE,PETSC_DECIDE,dof,1,NULL,NULL,&da);CHKERRQ(ierr);
+  ierr = DMSetFromOptions(da);CHKERRQ(ierr);
+  ierr = DMSetUp(da);CHKERRQ(ierr);
   ierr = DMDASetUniformCoordinates(da, 0.0, 1.0, 0.0, 1.0, 0.0, 1.0);CHKERRQ(ierr);
 
   /* Define output window for each variable of interest */
@@ -297,8 +296,9 @@ int main(int argc,char **argv)
   ftime = TIMESTEP*time;
   if (!rank) printf("time %d, ftime %g hour, TIMESTEP %g\n",time,(double)(ftime/3600),(double)dt);
 
-  ierr = TSSetInitialTimeStep(ts,0.0,dt);CHKERRQ(ierr);
-  ierr = TSSetDuration(ts,time,ftime);CHKERRQ(ierr);
+  ierr = TSSetTimeStep(ts,dt);CHKERRQ(ierr);
+  ierr = TSSetMaxSteps(ts,time);CHKERRQ(ierr);
+  ierr = TSSetMaxTime(ts,ftime);CHKERRQ(ierr);
   ierr = TSSetExactFinalTime(ts,TS_EXACTFINALTIME_STEPOVER);CHKERRQ(ierr);
   ierr = TSSetSolution(ts,T);CHKERRQ(ierr);
   ierr = TSSetDM(ts,da);CHKERRQ(ierr);
@@ -313,7 +313,7 @@ int main(int argc,char **argv)
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   ierr = TSSolve(ts,T);CHKERRQ(ierr);
   ierr = TSGetSolveTime(ts,&ftime);CHKERRQ(ierr);
-  ierr = TSGetTimeStepNumber(ts,&steps);CHKERRQ(ierr);
+  ierr = TSGetStepNumber(ts,&steps);CHKERRQ(ierr);
   if (!rank) PetscPrintf(PETSC_COMM_WORLD,"Solution T after %g hours %d steps\n",(double)(ftime/3600),steps);
 
 
@@ -327,15 +327,13 @@ int main(int argc,char **argv)
   ierr = TSDestroy(&ts);CHKERRQ(ierr);
   ierr = DMDestroy(&da);CHKERRQ(ierr);
 
-  PetscFinalize();
-  return 0;
+  ierr = PetscFinalize();
+  return ierr;
 }
 /*****************************end main program********************************/
 /*****************************************************************************/
 /*****************************************************************************/
 /*****************************************************************************/
-#undef __FUNCT__
-#define __FUNCT__ "calcfluxs"
 PetscErrorCode calcfluxs(PetscScalar sfctemp, PetscScalar airtemp, PetscScalar emma, PetscScalar fract, PetscScalar cloudTemp, PetscScalar *flux)
 {
   PetscFunctionBeginUser;
@@ -343,8 +341,6 @@ PetscErrorCode calcfluxs(PetscScalar sfctemp, PetscScalar airtemp, PetscScalar e
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "calcfluxa"
 PetscErrorCode calcfluxa(PetscScalar sfctemp, PetscScalar airtemp, PetscScalar emma, PetscScalar *flux)   /* this function is not currently called upon */
 {
   PetscScalar emm = 0.001;
@@ -353,8 +349,6 @@ PetscErrorCode calcfluxa(PetscScalar sfctemp, PetscScalar airtemp, PetscScalar e
   *flux = SIG*(-emm*(PetscPowScalarInt(airtemp,4)));      /* calculates flux usinge Stefan-Boltzmann relation */
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__ "sensibleflux"
 PetscErrorCode sensibleflux(PetscScalar sfctemp, PetscScalar airtemp, PetscScalar wind, PetscScalar *sheat)
 {
   PetscScalar density = 1;    /* air density */
@@ -367,8 +361,6 @@ PetscErrorCode sensibleflux(PetscScalar sfctemp, PetscScalar airtemp, PetscScala
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "latentflux"
 PetscErrorCode latentflux(PetscScalar sfctemp, PetscScalar dewtemp, PetscScalar wind, PetscScalar pressure1, PetscScalar *latentheat)
 {
   PetscScalar density = 1;   /* density of dry air */
@@ -394,8 +386,6 @@ PetscErrorCode latentflux(PetscScalar sfctemp, PetscScalar dewtemp, PetscScalar
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "potential_temperature"
 PetscErrorCode potential_temperature(PetscScalar temp, PetscScalar pressure1, PetscScalar pressure2, PetscScalar sfctemp, PetscScalar *pottemp)
 {
   PetscScalar kdry;     /* poisson constant for dry atmosphere */
@@ -434,8 +424,6 @@ extern PetscScalar calc_q(PetscScalar rv)
   return specific_humidity;
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "calc_gflux"
 PetscErrorCode calc_gflux(PetscScalar sfctemp, PetscScalar deep_grnd_temp, PetscScalar* Gflux)
 {
   PetscScalar k;                       /* thermal conductivity parameter */
@@ -448,13 +436,11 @@ PetscErrorCode calc_gflux(PetscScalar sfctemp, PetscScalar deep_grnd_temp, Petsc
   *Gflux = (k*(deep_grnd_temp - sfctemp)/dz);   /* calculates flux from deep ground layer */
   PetscFunctionReturn(0);
 }
-/* #undef __FUNCT__ */
-/* #define __FUNCT__ "emission" */
 extern PetscScalar emission(PetscScalar pwat)
 {
   PetscScalar emma;
 
-  emma = 0.725 + 0.17*log10(pwat);
+  emma = 0.725 + 0.17*PetscLog10Real(PetscRealPart(pwat));
 
   return emma;
 }
@@ -502,51 +488,49 @@ void readinput(struct in *put)
 
   ifp = fopen("ex5_control.txt", "r");
 
-  for (i=0; i<110; i++) fscanf(ifp, "%c", &x);
-  fscanf(ifp, "%lf", &tmp);
+  for (i=0; i<110; i++) { if (fscanf(ifp, "%c", &x) != 1) abort();}
+  if (fscanf(ifp, "%lf", &tmp) != 1) abort();
   put->Ts = tmp;
 
-  for (i=0; i<43; i++) fscanf(ifp, "%c", &x);
-  fscanf(ifp, "%lf", &tmp);
+  for (i=0; i<43; i++) { if (fscanf(ifp, "%c", &x) != 1) abort();}
+  if (fscanf(ifp, "%lf", &tmp) != 1) abort();
   put->Td = tmp;
 
-  for (i=0; i<43; i++) fscanf(ifp, "%c", &x);
-  fscanf(ifp, "%lf", &tmp);
+  for (i=0; i<43; i++) { if (fscanf(ifp, "%c", &x) != 1) abort();}
+  if (fscanf(ifp, "%lf", &tmp) != 1) abort();
   put->Ta = tmp;
 
-  for (i=0; i<43; i++) fscanf(ifp, "%c", &x);
-  fscanf(ifp, "%lf", &tmp);
+  for (i=0; i<43; i++) { if (fscanf(ifp, "%c", &x) != 1) abort();}
+  if (fscanf(ifp, "%lf", &tmp)!= 1) abort();
   put->Tc = tmp;
 
-  for (i=0; i<43; i++) fscanf(ifp, "%c", &x);
-  fscanf(ifp, "%lf", &tmp);
+  for (i=0; i<43; i++) { if (fscanf(ifp, "%c", &x) != 1) abort();}
+  if (fscanf(ifp, "%lf", &tmp) != 1) abort();
   put->fr = tmp;
 
-  for (i=0; i<43; i++) fscanf(ifp, "%c", &x);
-  fscanf(ifp, "%lf", &tmp);
+  for (i=0; i<43; i++) {if (fscanf(ifp, "%c", &x) != 1) abort();}
+  if (fscanf(ifp, "%lf", &tmp) != 1) abort();
   put->wnd = tmp;
 
-  for (i=0; i<43; i++) fscanf(ifp, "%c", &x);
-  fscanf(ifp, "%lf", &tmp);
+  for (i=0; i<43; i++) {if (fscanf(ifp, "%c", &x) != 1) abort();}
+  if (fscanf(ifp, "%lf", &tmp) != 1) abort();  
   put->pwt = tmp;
 
-  for (i=0; i<43; i++) fscanf(ifp, "%c", &x);
-  fscanf(ifp, "%lf", &tmp);
+  for (i=0; i<43; i++) {if (fscanf(ifp, "%c", &x) != 1) abort();}
+  if (fscanf(ifp, "%lf", &tmp) != 1) abort();  
   put->wndDir = tmp;
 
-  for (i=0; i<43; i++) fscanf(ifp, "%c", &x);
-  fscanf(ifp, "%lf", &tmp);
+  for (i=0; i<43; i++) {if (fscanf(ifp, "%c", &x) != 1) abort();}
+  if (fscanf(ifp, "%lf", &tmp) != 1) abort();  
   put->time = tmp;
 
-  for (i=0; i<63; i++) fscanf(ifp, "%c", &x);
-  fscanf(ifp, "%lf", &tmp);
+  for (i=0; i<63; i++) {if (fscanf(ifp, "%c", &x) != 1) abort();}
+  if (fscanf(ifp, "%lf", &tmp) != 1) abort();
   put->init = tmp;
 
 }
 
 /* ------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "FormInitialSolution"
 PetscErrorCode FormInitialSolution(DM da,Vec Xglobal,void *ctx)
 {
   PetscErrorCode ierr;
@@ -595,8 +579,6 @@ PetscErrorCode FormInitialSolution(DM da,Vec Xglobal,void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RhsFunc"
 /*
    RhsFunc - Evaluates nonlinear function F(u).
 
@@ -645,8 +627,7 @@ PetscErrorCode RhsFunc(TS ts,PetscReal t,Vec Xglobal,Vec F,void *ctx)
 
   PetscFunctionBeginUser;
   ierr = DMGetLocalVector(da,&localT);CHKERRQ(ierr);
-  ierr = DMDAGetInfo(da,PETSC_IGNORE,&Mx,&My,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,
-                     PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE);
+  ierr = DMDAGetInfo(da,PETSC_IGNORE,&Mx,&My,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE,PETSC_IGNORE);CHKERRQ(ierr);
 
   dhx = (PetscReal)(Mx-1)/(5000*(Mx-1));  /* dhx = 1/dx; assume 2D space domain: [0.0, 1.e5] x [0.0, 1.e5] */
   dhy = (PetscReal)(My-1)/(5000*(Mx-1));  /* dhy = 1/dy; */
@@ -676,12 +657,11 @@ PetscErrorCode RhsFunc(TS ts,PetscReal t,Vec Xglobal,Vec F,void *ctx)
       Ts = X[j][i].Ts; u = X[j][i].u; v = X[j][i].v; p = X[j][i].p; /*P = X[j][i].P; */
 
       sfctemp1 = (double)Ts;
-      sfctemp1 = (double)X[j][i].Ts;
       ierr     = calcfluxs(sfctemp1,airtemp,emma,fract,Tc,&fsfc1);CHKERRQ(ierr);        /* calculates surface net radiative flux */
       ierr     = sensibleflux(sfctemp1,airtemp,wind,&sheat);CHKERRQ(ierr);              /* calculate sensible heat flux */
       ierr     = latentflux(sfctemp1,dewtemp,wind,pressure1,&latentheat);CHKERRQ(ierr); /* calculates latent heat flux */
       ierr     = calc_gflux(sfctemp1,deep_grnd_temp,&groundflux);CHKERRQ(ierr);         /* calculates flux from earth below surface soil layer by conduction */
-      ierr     = calcfluxa(sfctemp1,airtemp,emma,&Ra);                                  /* Calculates the change in downward radiative flux */
+      ierr     = calcfluxa(sfctemp1,airtemp,emma,&Ra);CHKERRQ(ierr);                    /* Calculates the change in downward radiative flux */
       fsfc1    = fsfc1 + latentheat + sheat + groundflux;                               /* adds radiative, sensible heat, latent heat, and ground heat flux yielding net flux */
 
       /* convective coefficients for upwinding */
@@ -732,8 +712,6 @@ PetscErrorCode RhsFunc(TS ts,PetscReal t,Vec Xglobal,Vec F,void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "Monitor"
 PetscErrorCode Monitor(TS ts,PetscInt step,PetscReal time,Vec T,void *ctx)
 {
   PetscErrorCode    ierr;
@@ -758,3 +736,26 @@ PetscErrorCode Monitor(TS ts,PetscInt step,PetscReal time,Vec T,void *ctx)
   }
   PetscFunctionReturn(0);
 }
+
+
+
+/*TEST
+
+   build:
+      requires: !complex !single
+
+   test:
+      args: -ts_max_steps 130 -monitor_interval 60
+      output_file: output/ex5.out
+      TODO: crashes in test harness, runs fine outside, try valgrind?
+      requires: !complex !single
+
+   test:
+      suffix: 2
+      nsize: 4
+      args: -ts_max_steps 130 -monitor_interval 60
+      output_file: output/ex5.out
+      TODO: crashes in test harness, runs fine outside, try valgrind?
+      requires: !complex !single
+
+TEST*/
