@@ -24,20 +24,18 @@ static PetscErrorCode FormIFunction(TS,PetscReal,Vec,Vec,Vec,void*);
 static PetscErrorCode FormIJacobian(TS,PetscReal,Vec,Vec,PetscReal,Mat,Mat,void *ctx);
 static PetscErrorCode FormInitialSolution(TS,Vec,void*);
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 
 int main(int argc, char **argv)
 {
   TS                ts;
   Vec               x; /*solution vector*/
   Mat               A; /*Jacobian*/
-  PetscInt          steps,maxsteps,mx;
+  PetscInt          steps,mx;
   PetscErrorCode    ierr;
   PetscReal         ftime;
-  AppCtx      user;       /* user-defined work context */
+  AppCtx            user;       /* user-defined work context */
 
-  PetscInitialize(&argc,&argv,NULL,help);
+  ierr = PetscInitialize(&argc,&argv,NULL,help);if (ierr) return ierr;
 
   /* Initialize user application context */
   ierr = PetscOptionsBegin(PETSC_COMM_WORLD,NULL,"Allen-Cahn equation","");
@@ -52,7 +50,7 @@ int main(int argc, char **argv)
    Set runtime options
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   /*
-   * ierr = PetscOptionsGetBool(NULL,"-monitor",&monitor,NULL);CHKERRQ(ierr);
+   * ierr = PetscOptionsGetBool(NULL,NULL,"-monitor",&monitor,NULL);CHKERRQ(ierr);
    */
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    Create necessary matrix and vectors, solve same ODE on every process
@@ -71,9 +69,9 @@ int main(int argc, char **argv)
   ierr = TSSetRHSFunction(ts,NULL,RHSFunction,&user);CHKERRQ(ierr);
   ierr = TSSetIFunction(ts,NULL,FormIFunction,&user);CHKERRQ(ierr);
   ierr = TSSetIJacobian(ts,A,A,FormIJacobian,&user);CHKERRQ(ierr);
-  ftime = 142;
-  maxsteps = 100000;
-  ierr = TSSetDuration(ts,maxsteps,ftime);CHKERRQ(ierr);
+  ftime = 22;
+  ierr = TSSetMaxTime(ts,ftime);CHKERRQ(ierr);
+  ierr = TSSetExactFinalTime(ts,TS_EXACTFINALTIME_STEPOVER);CHKERRQ(ierr);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    Set initial conditions
@@ -92,7 +90,7 @@ int main(int argc, char **argv)
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   ierr = TSSolve(ts,x);CHKERRQ(ierr);
   ierr = TSGetTime(ts,&ftime);CHKERRQ(ierr);
-  ierr = TSGetTimeStepNumber(ts,&steps);CHKERRQ(ierr);
+  ierr = TSGetStepNumber(ts,&steps);CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_WORLD,"eps %g, steps %D, ftime %g\n",(double)user.param,steps,(double)ftime);CHKERRQ(ierr);
   /*   ierr = VecView(x,PETSC_VIEWER_STDOUT_WORLD);CHKERRQ(ierr);*/
 
@@ -103,12 +101,10 @@ int main(int argc, char **argv)
   ierr = VecDestroy(&x);CHKERRQ(ierr);
   ierr = TSDestroy(&ts);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  PetscFunctionReturn(0);
+  return ierr;
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "RHSFunction"
 static PetscErrorCode RHSFunction(TS ts,PetscReal t,Vec X,Vec F,void *ptr)
 {
   PetscErrorCode    ierr;
@@ -135,8 +131,6 @@ static PetscErrorCode RHSFunction(TS ts,PetscReal t,Vec X,Vec F,void *ptr)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "FormIFunction"
 static PetscErrorCode FormIFunction(TS ts,PetscReal t,Vec X,Vec Xdot,Vec F,void *ptr)
 {
   PetscErrorCode    ierr;
@@ -162,8 +156,6 @@ static PetscErrorCode FormIFunction(TS ts,PetscReal t,Vec X,Vec Xdot,Vec F,void
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "FormIJacobian"
 static PetscErrorCode FormIJacobian(TS ts,PetscReal t,Vec U, Vec Udot, PetscReal a, Mat J,Mat Jpre,void *ctx)
 {
   PetscErrorCode    ierr;
@@ -192,8 +184,6 @@ static PetscErrorCode FormIJacobian(TS ts,PetscReal t,Vec U, Vec Udot, PetscReal
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "FormInitialSolution"
 static PetscErrorCode FormInitialSolution(TS ts,Vec U,void *ctx)
 {
   AppCtx *user = (AppCtx*)ctx;
@@ -208,17 +198,26 @@ static PetscErrorCode FormInitialSolution(TS ts,Vec U,void *ctx)
   for(i=0;i<user->mx;i++) {
     x_map = user->xleft + i*hx;
     if(x_map >= 0.7065) {
-      x[i] = tanh((x_map-0.8)/(2.*PetscSqrtReal(user->param)));
+      x[i] = PetscTanhReal((x_map-0.8)/(2.*PetscSqrtReal(user->param)));
     } else if(x_map >= 0.4865) {
-      x[i] = tanh((0.613-x_map)/(2.*PetscSqrtReal(user->param)));
+      x[i] = PetscTanhReal((0.613-x_map)/(2.*PetscSqrtReal(user->param)));
     } else if(x_map >= 0.28) {
-      x[i] = tanh((x_map-0.36)/(2.*PetscSqrtReal(user->param)));
+      x[i] = PetscTanhReal((x_map-0.36)/(2.*PetscSqrtReal(user->param)));
     } else if(x_map >= -0.7) {
-      x[i] = tanh((0.2-x_map)/(2.*PetscSqrtReal(user->param)));
+      x[i] = PetscTanhReal((0.2-x_map)/(2.*PetscSqrtReal(user->param)));
     } else if(x_map >= -1) {
-      x[i] = tanh((x_map+0.9)/(2.*PetscSqrtReal(user->param)));
+      x[i] = PetscTanhReal((x_map+0.9)/(2.*PetscSqrtReal(user->param)));
     }
   }
   ierr = VecRestoreArray(U,&x);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
+
+/*TEST
+
+     test:
+       args:  -ts_rtol 1e-04 -ts_dt 0.025 -pc_type lu -ksp_error_if_not_converged TRUE  -ts_type eimex -ts_adapt_type none -ts_eimex_order_adapt -ts_eimex_max_rows 7 -ts_monitor_draw_solution
+       requires: x
+       timeoutfactor: 3
+
+TEST*/
