@@ -2,8 +2,8 @@ static const char help[] = "An elastic wave equation driven by Dieterich-Ruina f
 /*
 This whole derivation comes from Erickson, Birnir, and Lavallee [2010]. The model comes from the continuum limit in Carlson and Langer [1989],
 
-  u_{tt}   = c^2 u_{xx} - \tilde\gamma^2 u − (\gamma^2 / \xi) (\theta + \ln(u_t + 1))
-  \theta_t = −(u_t + 1) (\theta + (1 + \epsilon) \ln(u_t +1))
+  u_{tt}   = c^2 u_{xx} - \tilde\gamma^2 u - (\gamma^2 / \xi) (\theta + \ln(u_t + 1))
+  \theta_t = -(u_t + 1) (\theta + (1 + \epsilon) \ln(u_t +1))
 
 which can be reduced to a first order system,
 
@@ -29,8 +29,6 @@ struct _User {
   PetscReal c;          /* wavespeed */
 };
 
-#undef __FUNCT__
-#define __FUNCT__ "FormRHSFunction"
 static PetscErrorCode FormRHSFunction(TS ts, PetscReal t, Vec U, Vec F, void *ctx)
 {
   User              user = (User) ctx;
@@ -48,9 +46,9 @@ static PetscErrorCode FormRHSFunction(TS ts, PetscReal t, Vec U, Vec F, void *ct
   ierr = DMGetCoordinateDM(dm, &cdm);CHKERRQ(ierr);
   ierr = DMGetCoordinatesLocal(dm, &C);CHKERRQ(ierr);
   ierr = DMDAGetLocalInfo(dm, &info);CHKERRQ(ierr);
-  ierr = DMDAVecGetArrayRead(dm,  U, &u);CHKERRQ(ierr);
+  ierr = DMDAVecGetArrayRead(dm,  U, (void*)&u);CHKERRQ(ierr);
   ierr = DMDAVecGetArray(dm,  F, &f);CHKERRQ(ierr);
-  ierr = DMDAVecGetArray(cdm, C, &x);CHKERRQ(ierr);
+  ierr = DMDAVecGetArrayRead(cdm, C, (void*)&x);CHKERRQ(ierr);
   for (i = info.xs; i < info.xs+info.xm; ++i) {
     const PetscScalar hx = i+1 == info.xs+info.xm ? x[i] - x[i-1] : x[i+1] - x[i];
 
@@ -58,14 +56,12 @@ static PetscErrorCode FormRHSFunction(TS ts, PetscReal t, Vec U, Vec F, void *ct
     f[i].v  = -hx*(PetscSqr(user->gammaTilde)*u[i].u + (PetscSqr(user->gamma) / user->xi)*(u[i].th + log(u[i].v + 1)));
     f[i].th = -hx*(u[i].v + 1)*(u[i].th + (1 + user->epsilon)*log(u[i].v + 1));
   }
-  ierr = DMDAVecRestoreArrayRead(dm,  U, &u);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArrayRead(dm,  U, (void*)&u);CHKERRQ(ierr);
   ierr = DMDAVecRestoreArray(dm,  F, &f);CHKERRQ(ierr);
-  ierr = DMDAVecRestoreArrayRead(cdm, C, &x);CHKERRQ(ierr);
+  ierr = DMDAVecRestoreArrayRead(cdm, C, (void*)&x);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FormIFunction"
 static PetscErrorCode FormIFunction(TS ts, PetscReal t, Vec U, Vec Udot, Vec F, void *ctx)
 {
   User           user = (User) ctx;
@@ -116,8 +112,6 @@ static PetscErrorCode FormIFunction(TS ts, PetscReal t, Vec U, Vec Udot, Vec F,
 }
 
 /* IJacobian - Compute IJacobian = dF/dU + a dF/dUdot */
-#undef __FUNCT__
-#define __FUNCT__ "FormIJacobian"
 PetscErrorCode FormIJacobian(TS ts, PetscReal t, Vec U, Vec Udot, PetscReal a, Mat J, Mat Jpre, void *ctx)
 {
   User           user = (User) ctx;
@@ -179,8 +173,6 @@ PetscErrorCode FormIJacobian(TS ts, PetscReal t, Vec U, Vec Udot, PetscReal a, M
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "FormInitialSolution"
 PetscErrorCode FormInitialSolution(TS ts, Vec U, void *ctx)
 {
   /* User            user = (User) ctx; */
@@ -194,7 +186,7 @@ PetscErrorCode FormInitialSolution(TS ts, Vec U, void *ctx)
   PetscErrorCode  ierr;
 
   PetscFunctionBeginUser;
-  ierr = TSGetDM(ts, &dm);
+  ierr = TSGetDM(ts, &dm);CHKERRQ(ierr);
   ierr = DMGetCoordinateDM(dm, &cdm);CHKERRQ(ierr);
   ierr = DMGetCoordinatesLocal(dm, &C);CHKERRQ(ierr);
   ierr = DMDAGetLocalInfo(dm, &info);CHKERRQ(ierr);
@@ -210,22 +202,22 @@ PetscErrorCode FormInitialSolution(TS ts, Vec U, void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc, char **argv)
 {
   DM                dm;
   TS                ts;
   Vec               X;
   Mat               J;
-  PetscInt          steps, maxsteps, mx;
+  PetscInt          steps, mx;
   PetscReal         ftime, hx, dt;
   TSConvergedReason reason;
   struct _User      user;
   PetscErrorCode    ierr;
 
-  ierr = PetscInitialize(&argc, &argv, NULL, help);CHKERRQ(ierr);
-  ierr = DMDACreate1d(PETSC_COMM_WORLD, DM_BOUNDARY_NONE, -11, 3, 1, NULL, &dm);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc, &argv, NULL,help);if (ierr) return ierr;
+  ierr = DMDACreate1d(PETSC_COMM_WORLD, DM_BOUNDARY_NONE, 11, 3, 1, NULL, &dm);CHKERRQ(ierr);
+  ierr = DMSetFromOptions(dm);CHKERRQ(ierr);
+  ierr = DMSetUp(dm);CHKERRQ(ierr);
   ierr = DMDASetUniformCoordinates(dm, 0.0, 20.0, 0.0, 0.0, 0.0, 0.0);CHKERRQ(ierr);
   ierr = DMCreateGlobalVector(dm, &X);CHKERRQ(ierr);
 
@@ -252,21 +244,20 @@ int main(int argc, char **argv)
   ierr = DMCreateMatrix(dm, &J);CHKERRQ(ierr);
   ierr = TSSetIJacobian(ts, J, J, FormIJacobian, &user);CHKERRQ(ierr);
 
-  ftime    = 800.0;
-  maxsteps = 10000;
-  ierr = TSSetDuration(ts, maxsteps, ftime);CHKERRQ(ierr);
+  ftime = 800.0;
+  ierr = TSSetMaxTime(ts,ftime);CHKERRQ(ierr);
   ierr = TSSetExactFinalTime(ts,TS_EXACTFINALTIME_STEPOVER);CHKERRQ(ierr);
   ierr = FormInitialSolution(ts, X, &user);CHKERRQ(ierr);
   ierr = TSSetSolution(ts, X);CHKERRQ(ierr);
   ierr = VecGetSize(X, &mx);CHKERRQ(ierr);
   hx   = 20.0/(PetscReal)(mx-1);
   dt   = 0.4 * PetscSqr(hx) / PetscSqr(user.c); /* Diffusive stability limit */
-  ierr = TSSetInitialTimeStep(ts, 0.0, dt);CHKERRQ(ierr);
+  ierr = TSSetTimeStep(ts,dt);CHKERRQ(ierr);
   ierr = TSSetFromOptions(ts);CHKERRQ(ierr);
 
   ierr = TSSolve(ts, X);CHKERRQ(ierr);
   ierr = TSGetSolveTime(ts, &ftime);CHKERRQ(ierr);
-  ierr = TSGetTimeStepNumber(ts, &steps);CHKERRQ(ierr);
+  ierr = TSGetStepNumber(ts, &steps);CHKERRQ(ierr);
   ierr = TSGetConvergedReason(ts, &reason);CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_WORLD, "%s at time %g after %D steps\n", TSConvergedReasons[reason], (double)ftime, steps);CHKERRQ(ierr);
 
@@ -275,5 +266,15 @@ int main(int argc, char **argv)
   ierr = TSDestroy(&ts);CHKERRQ(ierr);
   ierr = DMDestroy(&dm);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
+
+/*TEST
+
+    build:
+      requires: !single  !complex c99
+
+    test:
+      TODO: broken, was not nightly tested
+
+TEST*/
