@@ -28,8 +28,6 @@ struct _Problem {
 /*
       Stiff 3-variable system from chemical reactions, due to Robertson (1966), problem ROBER in Hairer&Wanner, ODE 2, 1996
 */
-#undef __FUNCT__
-#define __FUNCT__ "RoberFunction"
 static PetscErrorCode RoberFunction(TS ts,PetscReal t,Vec X,Vec Xdot,Vec F,void *ctx)
 {
   PetscErrorCode    ierr;
@@ -49,8 +47,6 @@ static PetscErrorCode RoberFunction(TS ts,PetscReal t,Vec X,Vec Xdot,Vec F,void
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RoberJacobian"
 static PetscErrorCode RoberJacobian(TS ts,PetscReal t,Vec X,Vec Xdot,PetscReal a,Mat A,Mat B,void *ctx)
 {
   PetscErrorCode    ierr;
@@ -77,8 +73,6 @@ static PetscErrorCode RoberJacobian(TS ts,PetscReal t,Vec X,Vec Xdot,PetscReal a
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RoberSolution"
 static PetscErrorCode RoberSolution(PetscReal t,Vec X,void *ctx)
 {
   PetscErrorCode ierr;
@@ -94,8 +88,6 @@ static PetscErrorCode RoberSolution(PetscReal t,Vec X,void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "RoberCreate"
 static PetscErrorCode RoberCreate(Problem p)
 {
 
@@ -117,8 +109,6 @@ typedef struct {
   PetscReal lambda;
 } CECtx;
 
-#undef __FUNCT__
-#define __FUNCT__ "CEDestroy"
 static PetscErrorCode CEDestroy(Problem p)
 {
   PetscErrorCode ierr;
@@ -128,8 +118,6 @@ static PetscErrorCode CEDestroy(Problem p)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CEFunction"
 static PetscErrorCode CEFunction(TS ts,PetscReal t,Vec X,Vec Xdot,Vec F,void *ctx)
 {
   PetscErrorCode    ierr;
@@ -151,8 +139,6 @@ static PetscErrorCode CEFunction(TS ts,PetscReal t,Vec X,Vec Xdot,Vec F,void *ct
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CEJacobian"
 static PetscErrorCode CEJacobian(TS ts,PetscReal t,Vec X,Vec Xdot,PetscReal a,Mat A,Mat B,void *ctx)
 {
   PetscReal         l = ((CECtx*)ctx)->lambda;
@@ -178,8 +164,6 @@ static PetscErrorCode CEJacobian(TS ts,PetscReal t,Vec X,Vec Xdot,PetscReal a,Ma
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CESolution"
 static PetscErrorCode CESolution(PetscReal t,Vec X,void *ctx)
 {
   PetscReal      l = ((CECtx*)ctx)->lambda;
@@ -193,8 +177,6 @@ static PetscErrorCode CESolution(PetscReal t,Vec X,void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "CECreate"
 static PetscErrorCode CECreate(Problem p)
 {
   PetscErrorCode ierr;
@@ -224,8 +206,6 @@ static PetscErrorCode CECreate(Problem p)
 /*
 *  Stiff 3-variable oscillatory system from chemical reactions. problem OREGO in Hairer&Wanner
 */
-#undef __FUNCT__
-#define __FUNCT__ "OregoFunction"
 static PetscErrorCode OregoFunction(TS ts,PetscReal t,Vec X,Vec Xdot,Vec F,void *ctx)
 {
   PetscErrorCode    ierr;
@@ -245,8 +225,6 @@ static PetscErrorCode OregoFunction(TS ts,PetscReal t,Vec X,Vec Xdot,Vec F,void
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "OregoJacobian"
 static PetscErrorCode OregoJacobian(TS ts,PetscReal t,Vec X,Vec Xdot,PetscReal a,Mat A,Mat B,void *ctx)
 {
   PetscErrorCode    ierr;
@@ -279,8 +257,6 @@ static PetscErrorCode OregoJacobian(TS ts,PetscReal t,Vec X,Vec Xdot,PetscReal a
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "OregoSolution"
 static PetscErrorCode OregoSolution(PetscReal t,Vec X,void *ctx)
 {
   PetscErrorCode ierr;
@@ -296,8 +272,6 @@ static PetscErrorCode OregoSolution(PetscReal t,Vec X,void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "OregoCreate"
 static PetscErrorCode OregoCreate(Problem p)
 {
 
@@ -321,8 +295,6 @@ typedef struct {
   Vec      x;
 } MonitorCtx;
 
-#undef __FUNCT__
-#define __FUNCT__ "MonitorError"
 static PetscErrorCode MonitorError(TS ts,PetscInt step,PetscReal t,Vec x,void *ctx)
 {
   PetscErrorCode ierr;
@@ -337,13 +309,14 @@ static PetscErrorCode MonitorError(TS ts,PetscInt step,PetscReal t,Vec x,void *c
   ierr = VecAYPX(mon->x,-1,x);CHKERRQ(ierr);
   ierr = VecNorm(mon->x,NORM_2,&nrm_diff);CHKERRQ(ierr);
   ierr = TSGetTimeStep(ts,&h);CHKERRQ(ierr);
+  if (step < 0) {
+    ierr = PetscPrintf(mon->comm,"Interpolated final solution ");CHKERRQ(ierr);
+  }
   ierr = PetscPrintf(mon->comm,"step %4D t=%12.8e h=% 8.2e  |x|=%9.2e  |x_e|=%9.2e  |x-x_e|=%9.2e\n",step,(double)t,(double)h,(double)nrm_x,(double)nrm_exact,(double)nrm_diff);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   PetscFunctionList plist = NULL;
@@ -353,7 +326,7 @@ int main(int argc,char **argv)
   Mat               A;             /* Jacobian matrix */
   Problem           problem;
   PetscBool         use_monitor;
-  PetscInt          steps,maxsteps = 1000,nonlinits,linits,snesfails,rejects;
+  PetscInt          steps,nonlinits,linits,snesfails,rejects;
   PetscReal         ftime;
   MonitorCtx        mon;
   PetscErrorCode    ierr;
@@ -362,7 +335,7 @@ int main(int argc,char **argv)
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Initialize program
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-  PetscInitialize(&argc,&argv,(char*)0,help);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size);CHKERRQ(ierr);
   if (size > 1) SETERRQ(PETSC_COMM_WORLD,PETSC_ERR_SUP,"Only for sequential runs");
 
@@ -417,7 +390,8 @@ int main(int argc,char **argv)
   ierr = TSSetType(ts,TSROSW);CHKERRQ(ierr); /* Rosenbrock-W */
   ierr = TSSetIFunction(ts,NULL,problem->function,problem->data);CHKERRQ(ierr);
   ierr = TSSetIJacobian(ts,A,A,problem->jacobian,problem->data);CHKERRQ(ierr);
-  ierr = TSSetDuration(ts,maxsteps,problem->final_time);CHKERRQ(ierr);
+  ierr = TSSetMaxTime(ts,problem->final_time);CHKERRQ(ierr);
+  ierr = TSSetExactFinalTime(ts,TS_EXACTFINALTIME_STEPOVER);CHKERRQ(ierr);
   ierr = TSSetMaxStepRejections(ts,10);CHKERRQ(ierr);
   ierr = TSSetMaxSNESFailures(ts,-1);CHKERRQ(ierr); /* unlimited */
   if (use_monitor) {
@@ -428,7 +402,7 @@ int main(int argc,char **argv)
      Set initial conditions
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   ierr = (*problem->solution)(0,x,problem->data);CHKERRQ(ierr);
-  ierr = TSSetInitialTimeStep(ts,0.0,.001);CHKERRQ(ierr);
+  ierr = TSSetTimeStep(ts,.001);CHKERRQ(ierr);
   ierr = TSSetSolution(ts,x);CHKERRQ(ierr);
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -441,15 +415,12 @@ int main(int argc,char **argv)
      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   ierr = TSSolve(ts,x);CHKERRQ(ierr);
   ierr = TSGetSolveTime(ts,&ftime);CHKERRQ(ierr);
-  ierr = TSGetTimeStepNumber(ts,&steps);CHKERRQ(ierr);
+  ierr = TSGetStepNumber(ts,&steps);CHKERRQ(ierr);
   ierr = TSGetSNESFailures(ts,&snesfails);CHKERRQ(ierr);
   ierr = TSGetStepRejections(ts,&rejects);CHKERRQ(ierr);
   ierr = TSGetSNESIterations(ts,&nonlinits);CHKERRQ(ierr);
   ierr = TSGetKSPIterations(ts,&linits);CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_WORLD,"steps %D (%D rejected, %D SNES fails), ftime %g, nonlinits %D, linits %D\n",steps,rejects,snesfails,(double)ftime,nonlinits,linits);CHKERRQ(ierr);
-  if (problem->hasexact) {
-    ierr = MonitorError(ts,steps,ftime,x,&mon);CHKERRQ(ierr);
-  }
 
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Free work space.  All PETSc objects should be destroyed when they
@@ -467,5 +438,23 @@ int main(int argc,char **argv)
   ierr = PetscFunctionListDestroy(&plist);CHKERRQ(ierr);
 
   ierr = PetscFinalize();
-  PetscFunctionReturn(0);
+  return ierr;
 }
+
+/*TEST
+
+    test:
+      requires: !complex
+      args: -monitor_error -ts_atol 1e-2 -ts_rtol 1e-2 -ts_exact_final_time interpolate -ts_type arkimex
+
+    test:
+      suffix: 2
+      requires: !single !complex
+      args: -ts_atol 1e-2 -ts_rtol 1e-2 -ts_final_time 15 -ts_type arkimex -ts_arkimex_type 2e -problem_type orego -ts_arkimex_initial_guess_extrapolate 0
+
+    test:
+      suffix: 3
+      requires: !single !complex
+      args: -ts_atol 1e-2 -ts_rtol 1e-2 -ts_final_time 15 -ts_type arkimex -ts_arkimex_type 2e -problem_type orego -ts_arkimex_initial_guess_extrapolate 1
+
+TEST*/
