@@ -1,39 +1,4 @@
-/*
- * eimex.c
- *
- *  Created on: Jun 21, 2012
- *      Written by Hong Zhang (zhang@vt.edu), Virginia Tech
- *                 Emil Constantinescu (emconsta@mcs.anl.gov), Argonne National Laboratory
- */
-/*MC
-      EIMEX - Time stepping with Extrapolated IMEX methods.
 
-  Notes:
-  The general system is written as
-
-  G(t,X,Xdot) = F(t,X)
-
-  where G represents the stiff part and F represents the non-stiff part. The user should provide the stiff part
-  of the equation using TSSetIFunction() and the non-stiff part with TSSetRHSFunction().
-  This method is designed to be linearly implicit on G and can use an approximate and lagged Jacobian.
-
-  Another common form for the system is
-
-  y'=f(x)+g(x)
-
-  The relationship between F,G and f,g is
-
-  G = y'-g(x), F = f(x)
-
- References
-  E. Constantinescu and A. Sandu, Extrapolated implicit-explicit time stepping, SIAM Journal on Scientific
-Computing, 31 (2010), pp. 4452-4477.
-
-      Level: beginner
-
-.seealso:  TSCreate(), TS, TSSetType(), TSEIMEXSetMaxRows(), TSEIMEXSetRowCol(), TSEIMEXSetOrdAdapt()
-
- M*/
 #include <petsc/private/tsimpl.h>                /*I   "petscts.h"   I*/
 #include <petscdm.h>
 
@@ -66,8 +31,6 @@ static PetscInt Map(PetscInt i, PetscInt j, PetscInt s)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSEvaluateStep_EIMEX"
 static PetscErrorCode TSEvaluateStep_EIMEX(TS ts,PetscInt order,Vec X,PetscBool *done)
 {
   TS_EIMEX        *ext = (TS_EIMEX*)ts->data;
@@ -79,8 +42,6 @@ static PetscErrorCode TSEvaluateStep_EIMEX(TS ts,PetscInt order,Vec X,PetscBool
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSStage_EIMEX"
 static PetscErrorCode TSStage_EIMEX(TS ts,PetscInt istage)
 {
   TS_EIMEX        *ext = (TS_EIMEX*)ts->data;
@@ -107,15 +68,13 @@ static PetscErrorCode TSStage_EIMEX(TS ts,PetscInt istage)
     ierr = SNESGetLinearSolveIterations(snes,&lits);CHKERRQ(ierr);
     ts->snes_its += its; ts->ksp_its += lits;
     ierr = TSGetAdapt(ts,&adapt);CHKERRQ(ierr);
-    ierr = TSAdaptCheckStage(adapt,ts,&accept);CHKERRQ(ierr);
+    ierr = TSAdaptCheckStage(adapt,ts,ext->ctime,Y,&accept);CHKERRQ(ierr);
   }
 
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSStep_EIMEX"
 static PetscErrorCode TSStep_EIMEX(TS ts)
 {
   TS_EIMEX        *ext = (TS_EIMEX*)ts->data;
@@ -126,7 +85,7 @@ static PetscErrorCode TSStep_EIMEX(TS ts)
   PetscInt        i,j;
   PetscBool       accept = PETSC_FALSE;
   PetscErrorCode  ierr;
-  PetscReal       alpha,local_error;
+  PetscReal       alpha,local_error,local_error_a,local_error_r;
   PetscFunctionBegin;
 
   ierr = TSGetSNES(ts,&snes);CHKERRQ(ierr);
@@ -155,7 +114,7 @@ static PetscErrorCode TSStep_EIMEX(TS ts)
   if(ext->ord_adapt && ext->nstages < ext->max_rows){
 	accept = PETSC_FALSE;
 	while(!accept && ext->nstages < ext->max_rows){
-	  ierr = TSErrorWeightedNorm(ts,ts->vec_sol,T[Map(ext->nstages-1,ext->nstages-2,ext->nstages)],ts->adapt->wnormtype,&local_error);CHKERRQ(ierr);
+	  ierr = TSErrorWeightedNorm(ts,ts->vec_sol,T[Map(ext->nstages-1,ext->nstages-2,ext->nstages)],ts->adapt->wnormtype,&local_error,&local_error_a,&local_error_r);CHKERRQ(ierr);
 	  accept = (local_error < 1.0)? PETSC_TRUE : PETSC_FALSE;
 
 	  if(!accept){/* add one more stage*/
@@ -189,7 +148,6 @@ static PetscErrorCode TSStep_EIMEX(TS ts)
 	}
   }/*end if ext->ord_adapt*/
   ts->ptime += ts->time_step;
-  ts->steps++;
   ext->status = TS_STEP_COMPLETE;
 
   if (ext->status != TS_STEP_COMPLETE && !ts->reason) ts->reason = TS_DIVERGED_STEP_REJECTED;
@@ -197,15 +155,12 @@ static PetscErrorCode TSStep_EIMEX(TS ts)
 }
 
 /* cubic Hermit spline */
-#undef __FUNCT__
-#define __FUNCT__ "TSInterpolate_EIMEX"
 static PetscErrorCode TSInterpolate_EIMEX(TS ts,PetscReal itime,Vec X)
 {
   TS_EIMEX       *ext = (TS_EIMEX*)ts->data;
   PetscReal      t,a,b;
-  Vec            Y0=ext->VecSolPrev,Y1=ext->Y,
-                         Ydot=ext->Ydot,YdotI=ext->YdotI;
-  const PetscReal h = ts->time_step_prev;
+  Vec            Y0=ext->VecSolPrev,Y1=ext->Y,Ydot=ext->Ydot,YdotI=ext->YdotI;
+  const PetscReal h = ts->ptime - ts->ptime_prev;
   PetscErrorCode ierr;
   PetscFunctionBegin;
   t = (itime -ts->ptime + h)/h;
@@ -227,8 +182,6 @@ static PetscErrorCode TSInterpolate_EIMEX(TS ts,PetscReal itime,Vec X)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSReset_EIMEX"
 static PetscErrorCode TSReset_EIMEX(TS ts)
 {
   TS_EIMEX        *ext = (TS_EIMEX*)ts->data;
@@ -248,8 +201,6 @@ static PetscErrorCode TSReset_EIMEX(TS ts)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSDestroy_EIMEX"
 static PetscErrorCode TSDestroy_EIMEX(TS ts)
 {
   PetscErrorCode  ierr;
@@ -265,8 +216,6 @@ static PetscErrorCode TSDestroy_EIMEX(TS ts)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSEIMEXGetVecs"
 static PetscErrorCode TSEIMEXGetVecs(TS ts,DM dm,Vec *Z,Vec *Ydot,Vec *YdotI, Vec *YdotRHS)
 {
   TS_EIMEX       *ext = (TS_EIMEX*)ts->data;
@@ -297,8 +246,6 @@ static PetscErrorCode TSEIMEXGetVecs(TS ts,DM dm,Vec *Z,Vec *Ydot,Vec *YdotI, Ve
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSEIMEXRestoreVecs"
 static PetscErrorCode TSEIMEXRestoreVecs(TS ts,DM dm,Vec *Z,Vec *Ydot,Vec *YdotI,Vec *YdotRHS)
 {
   PetscErrorCode ierr;
@@ -334,8 +281,6 @@ static PetscErrorCode TSEIMEXRestoreVecs(TS ts,DM dm,Vec *Z,Vec *Ydot,Vec *YdotI
   In the case of Backward Euler, Fn = (U-U0)/h-g(t1,U))
   Since FormIFunction calculates G = ydot - g(t,y), ydot will be set to (U-U0)/h
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESTSFormFunction_EIMEX"
 static PetscErrorCode SNESTSFormFunction_EIMEX(SNES snes,Vec X,Vec G,TS ts)
 {
   TS_EIMEX        *ext = (TS_EIMEX*)ts->data;
@@ -363,8 +308,6 @@ static PetscErrorCode SNESTSFormFunction_EIMEX(SNES snes,Vec X,Vec G,TS ts)
 /*
  This defined the Jacobian matrix for SNES. Jn = (I/h-g'(t,y))
  */
-#undef __FUNCT__
-#define __FUNCT__ "SNESTSFormJacobian_EIMEX"
 static PetscErrorCode SNESTSFormJacobian_EIMEX(SNES snes,Vec X,Mat A,Mat B,TS ts)
 {
   TS_EIMEX        *ext = (TS_EIMEX*)ts->data;
@@ -384,8 +327,6 @@ static PetscErrorCode SNESTSFormJacobian_EIMEX(SNES snes,Vec X,Mat A,Mat B,TS ts
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCoarsenHook_TSEIMEX"
 static PetscErrorCode DMCoarsenHook_TSEIMEX(DM fine,DM coarse,void *ctx)
 {
 
@@ -393,8 +334,6 @@ static PetscErrorCode DMCoarsenHook_TSEIMEX(DM fine,DM coarse,void *ctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMRestrictHook_TSEIMEX"
 static PetscErrorCode DMRestrictHook_TSEIMEX(DM fine,Mat restrct,Vec rscale,Mat inject,DM coarse,void *ctx)
 {
   TS ts = (TS)ctx;
@@ -412,8 +351,6 @@ static PetscErrorCode DMRestrictHook_TSEIMEX(DM fine,Mat restrct,Vec rscale,Mat
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSetUp_EIMEX"
 static PetscErrorCode TSSetUp_EIMEX(TS ts)
 {
   TS_EIMEX       *ext = (TS_EIMEX*)ts->data;
@@ -439,6 +376,8 @@ static PetscErrorCode TSSetUp_EIMEX(TS ts)
     ext->nstages = ext->max_rows; /* by default nstages is the same as max_rows, this can be changed by setting order adaptivity */
   }
 
+  ierr = TSGetAdapt(ts,&ts->adapt);CHKERRQ(ierr);
+
   ierr = VecDuplicateVecs(ts->vec_sol,(1+ext->nstages)*ext->nstages/2,&ext->T);CHKERRQ(ierr);/* full T table */
   ierr = VecDuplicate(ts->vec_sol,&ext->YdotI);CHKERRQ(ierr);
   ierr = VecDuplicate(ts->vec_sol,&ext->YdotRHS);CHKERRQ(ierr);
@@ -453,9 +392,7 @@ static PetscErrorCode TSSetUp_EIMEX(TS ts)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSetFromOptions_EIMEX"
-static PetscErrorCode TSSetFromOptions_EIMEX(PetscOptions *PetscOptionsObject,TS ts)
+static PetscErrorCode TSSetFromOptions_EIMEX(PetscOptionItems *PetscOptionsObject,TS ts)
 {
   TS_EIMEX       *ext = (TS_EIMEX*)ts->data;
   PetscErrorCode ierr;
@@ -477,32 +414,18 @@ static PetscErrorCode TSSetFromOptions_EIMEX(PetscOptions *PetscOptionsObject,TS
       ierr = TSEIMEXSetRowCol(ts,tindex[0],tindex[1]);CHKERRQ(ierr);
     }
     ierr = PetscOptionsBool("-ts_eimex_order_adapt","Solve the problem with adaptive order","TSEIMEXSetOrdAdapt",ext->ord_adapt,&ext->ord_adapt,NULL);CHKERRQ(ierr);
-    ierr = SNESSetFromOptions(ts->snes);CHKERRQ(ierr);
   }
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSView_EIMEX"
 static PetscErrorCode TSView_EIMEX(TS ts,PetscViewer viewer)
 {
-  /*  TS_EIMEX         *ext = (TS_EIMEX*)ts->data; */
-  PetscBool        iascii;
-  PetscErrorCode   ierr;
-
   PetscFunctionBegin;
-  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
-  if (iascii) {
-    ierr = PetscViewerASCIIPrintf(viewer,"  EIMEX\n");CHKERRQ(ierr);
-  }
-  ierr = SNESView(ts->snes,viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSEIMEXSetMaxRows"
 /*@C
   TSEIMEXSetMaxRows - Set the maximum number of rows for EIMEX schemes
 
@@ -526,8 +449,6 @@ PetscErrorCode TSEIMEXSetMaxRows(TS ts, PetscInt nrows)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSEIMEXSetRowCol"
 /*@C
   TSEIMEXSetRowCol - Set the type index in the T table for the return value
 
@@ -551,8 +472,6 @@ PetscErrorCode TSEIMEXSetRowCol(TS ts, PetscInt row, PetscInt col)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSEIMEXSetOrdAdapt"
 /*@C
   TSEIMEXSetOrdAdapt - Set the order adaptativity
 
@@ -576,8 +495,6 @@ PetscErrorCode TSEIMEXSetOrdAdapt(TS ts, PetscBool flg)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSEIMEXSetMaxRows_EIMEX"
 static PetscErrorCode TSEIMEXSetMaxRows_EIMEX(TS ts,PetscInt nrows)
 {
   TS_EIMEX *ext = (TS_EIMEX*)ts->data;
@@ -593,8 +510,6 @@ static PetscErrorCode TSEIMEXSetMaxRows_EIMEX(TS ts,PetscInt nrows)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSEIMEXSetRowCol_EIMEX"
 static PetscErrorCode TSEIMEXSetRowCol_EIMEX(TS ts,PetscInt row,PetscInt col)
 {
   TS_EIMEX *ext = (TS_EIMEX*)ts->data;
@@ -609,8 +524,6 @@ static PetscErrorCode TSEIMEXSetRowCol_EIMEX(TS ts,PetscInt row,PetscInt col)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSEIMEXSetOrdAdapt_EIMEX"
 static PetscErrorCode TSEIMEXSetOrdAdapt_EIMEX(TS ts,PetscBool flg)
 {
   TS_EIMEX *ext = (TS_EIMEX*)ts->data;
@@ -619,22 +532,43 @@ static PetscErrorCode TSEIMEXSetOrdAdapt_EIMEX(TS ts,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-/* ------------------------------------------------------------ */
 /*MC
-      TSEIMEX - ODE solver using extrapolated IMEX schemes
-  These methods are intended for problems with well-separated time scales, especially when a slow scale is strongly nonlinear such that it is expensive to solve with a fully implicit method. The user should provide the stiff part of the equation using TSSetIFunction() and the non-stiff part with TSSetRHSFunction().
+      TSEIMEX - Time stepping with Extrapolated IMEX methods.
+
+   These methods are intended for problems with well-separated time scales, especially when a slow scale is strongly nonlinear such that it 
+   is expensive to solve with a fully implicit method. The user should provide the stiff part of the equation using TSSetIFunction() and the 
+   non-stiff part with TSSetRHSFunction().
 
    Notes:
   The default is a 3-stage scheme, it can be changed with TSEIMEXSetMaxRows() or -ts_eimex_max_rows
 
   This method currently only works with ODE, for which the stiff part G(t,X,Xdot) has the form Xdot + Ghat(t,X).
 
-  Level: beginner
+  The general system is written as
 
-.seealso:  TSCreate(), TS
-M*/
-#undef __FUNCT__
-#define __FUNCT__ "TSCreate_EIMEX"
+  G(t,X,Xdot) = F(t,X)
+
+  where G represents the stiff part and F represents the non-stiff part. The user should provide the stiff part
+  of the equation using TSSetIFunction() and the non-stiff part with TSSetRHSFunction().
+  This method is designed to be linearly implicit on G and can use an approximate and lagged Jacobian.
+
+  Another common form for the system is
+
+  y'=f(x)+g(x)
+
+  The relationship between F,G and f,g is
+
+  G = y'-g(x), F = f(x)
+
+ References
+  E. Constantinescu and A. Sandu, Extrapolated implicit-explicit time stepping, SIAM Journal on Scientific
+Computing, 31 (2010), pp. 4452-4477.
+
+      Level: beginner
+
+.seealso:  TSCreate(), TS, TSSetType(), TSEIMEXSetMaxRows(), TSEIMEXSetRowCol(), TSEIMEXSetOrdAdapt()
+
+ M*/
 PETSC_EXTERN PetscErrorCode TSCreate_EIMEX(TS ts)
 {
   TS_EIMEX       *ext;
@@ -652,6 +586,9 @@ PETSC_EXTERN PetscErrorCode TSCreate_EIMEX(TS ts)
   ts->ops->setfromoptions = TSSetFromOptions_EIMEX;
   ts->ops->snesfunction   = SNESTSFormFunction_EIMEX;
   ts->ops->snesjacobian   = SNESTSFormJacobian_EIMEX;
+  ts->default_adapt_type  = TSADAPTNONE;
+
+  ts->usessnes = PETSC_TRUE;
 
   ierr = PetscNewLog(ts,&ext);CHKERRQ(ierr);
   ts->data = (void*)ext;
