@@ -11,8 +11,6 @@
       TSPrecond_Sundials - function that we provide to SUNDIALS to
                         evaluate the preconditioner.
 */
-#undef __FUNCT__
-#define __FUNCT__ "TSPrecond_Sundials"
 PetscErrorCode TSPrecond_Sundials(realtype tn,N_Vector y,N_Vector fy,booleantype jok,booleantype *jcurPtr,
                                   realtype _gamma,void *P_data,N_Vector vtemp1,N_Vector vtemp2,N_Vector vtemp3)
 {
@@ -43,8 +41,6 @@ PetscErrorCode TSPrecond_Sundials(realtype tn,N_Vector y,N_Vector fy,booleantype
 /*
      TSPSolve_Sundials -  routine that we provide to Sundials that applies the preconditioner.
 */
-#undef __FUNCT__
-#define __FUNCT__ "TSPSolve_Sundials"
 PetscErrorCode TSPSolve_Sundials(realtype tn,N_Vector y,N_Vector fy,N_Vector r,N_Vector z,
                                  realtype _gamma,realtype delta,int lr,void *P_data,N_Vector vtemp)
 {
@@ -73,8 +69,6 @@ PetscErrorCode TSPSolve_Sundials(realtype tn,N_Vector y,N_Vector fy,N_Vector r,N
 /*
         TSFunction_Sundials - routine that we provide to Sundials that applies the right hand side.
 */
-#undef __FUNCT__
-#define __FUNCT__ "TSFunction_Sundials"
 int TSFunction_Sundials(realtype t,N_Vector y,N_Vector ydot,void *ctx)
 {
   TS             ts = (TS) ctx;
@@ -114,14 +108,12 @@ int TSFunction_Sundials(realtype t,N_Vector y,N_Vector ydot,void *ctx)
 /*
        TSStep_Sundials - Calls Sundials to integrate the ODE.
 */
-#undef __FUNCT__
-#define __FUNCT__ "TSStep_Sundials"
 PetscErrorCode TSStep_Sundials(TS ts)
 {
   TS_Sundials    *cvode = (TS_Sundials*)ts->data;
   PetscErrorCode ierr;
   PetscInt       flag;
-  long int       its,nsteps;
+  long int       nits,lits,nsteps;
   realtype       t,tout;
   PetscScalar    *y_data;
   void           *mem;
@@ -133,12 +125,12 @@ PetscErrorCode TSStep_Sundials(TS ts)
   N_VSetArrayPointer((realtype*)y_data,cvode->y);
   ierr = VecRestoreArray(ts->vec_sol,NULL);CHKERRQ(ierr);
 
-  ierr = TSPreStep(ts);CHKERRQ(ierr);
-
-  /* We would like to call TSPreStep() when starting each step (including rejections), TSPreStage(),
-   * and TSPostStage() before each stage solve, but CVode does not appear to support this. */
-  if (cvode->monitorstep) flag = CVode(mem,tout,cvode->y,&t,CV_ONE_STEP);
-  else flag = CVode(mem,tout,cvode->y,&t,CV_NORMAL);
+  /* We would like to TSPreStage() and TSPostStage()
+   * before each stage solve but CVode does not appear to support this. */
+  if (cvode->monitorstep)
+    flag = CVode(mem,tout,cvode->y,&t,CV_ONE_STEP);
+  else
+    flag = CVode(mem,tout,cvode->y,&t,CV_NORMAL);
 
   if (flag) { /* display error message */
     switch (flag) {
@@ -149,10 +141,10 @@ PetscErrorCode TSStep_Sundials(TS ts)
         SETERRQ(PETSC_COMM_SELF,PETSC_ERR_LIB,"CVode() fails, CV_TOO_CLOSE");
         break;
       case CV_TOO_MUCH_WORK: {
-        PetscReal      tcur;
+        PetscReal tcur;
         ierr = CVodeGetNumSteps(mem,&nsteps);CHKERRQ(ierr);
         ierr = CVodeGetCurrentTime(mem,&tcur);CHKERRQ(ierr);
-        SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_LIB,"CVode() fails, CV_TOO_MUCH_WORK. At t=%g, nsteps %D exceeds mxstep %D. Increase '-ts_max_steps <>' or modify TSSetDuration()",(double)tcur,nsteps,ts->max_steps);
+        SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_LIB,"CVode() fails, CV_TOO_MUCH_WORK. At t=%g, nsteps %D exceeds maxstep %D. Increase '-ts_max_steps <>' or modify TSSetMaxSteps()",(double)tcur,nsteps,ts->max_steps);
       } break;
       case CV_TOO_MUCH_ACC:
         SETERRQ(PETSC_COMM_SELF,PETSC_ERR_LIB,"CVode() fails, CV_TOO_MUCH_ACC");
@@ -192,26 +184,25 @@ PetscErrorCode TSStep_Sundials(TS ts)
     }
   }
 
+  /* log inner nonlinear and linear iterations */
+  ierr = CVodeGetNumNonlinSolvIters(mem,&nits);CHKERRQ(ierr);
+  ierr = CVSpilsGetNumLinIters(mem,&lits);CHKERRQ(ierr);
+  ts->snes_its += nits; ts->ksp_its = lits;
+
   /* copy the solution from cvode->y to cvode->update and sol */
   ierr = VecPlaceArray(cvode->w1,y_data);CHKERRQ(ierr);
   ierr = VecCopy(cvode->w1,cvode->update);CHKERRQ(ierr);
   ierr = VecResetArray(cvode->w1);CHKERRQ(ierr);
   ierr = VecCopy(cvode->update,ts->vec_sol);CHKERRQ(ierr);
-  ierr = CVodeGetNumNonlinSolvIters(mem,&its);CHKERRQ(ierr);
-  ierr = CVSpilsGetNumLinIters(mem, &its);
-  ts->snes_its = its; ts->ksp_its = its;
 
   ts->time_step = t - ts->ptime;
-  ts->ptime     = t;
-  ts->steps++;
+  ts->ptime = t;
 
   ierr = CVodeGetNumSteps(mem,&nsteps);CHKERRQ(ierr);
-  if (!cvode->monitorstep) ts->steps = nsteps;
+  if (!cvode->monitorstep) ts->steps += nsteps - 1; /* TSStep() increments the step counter by one */
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSInterpolate_Sundials"
 static PetscErrorCode TSInterpolate_Sundials(TS ts,PetscReal t,Vec X)
 {
   TS_Sundials    *cvode = (TS_Sundials*)ts->data;
@@ -236,8 +227,6 @@ static PetscErrorCode TSInterpolate_Sundials(TS ts,PetscReal t,Vec X)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSReset_Sundials"
 PetscErrorCode TSReset_Sundials(TS ts)
 {
   TS_Sundials    *cvode = (TS_Sundials*)ts->data;
@@ -252,8 +241,6 @@ PetscErrorCode TSReset_Sundials(TS ts)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSDestroy_Sundials"
 PetscErrorCode TSDestroy_Sundials(TS ts)
 {
   TS_Sundials    *cvode = (TS_Sundials*)ts->data;
@@ -276,8 +263,6 @@ PetscErrorCode TSDestroy_Sundials(TS ts)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSetUp_Sundials"
 PetscErrorCode TSSetUp_Sundials(TS ts)
 {
   TS_Sundials    *cvode = (TS_Sundials*)ts->data;
@@ -358,6 +343,7 @@ PetscErrorCode TSSetUp_Sundials(TS ts)
 
   /* Specify max num of steps to be taken by cvode in its attempt to reach the next output time */
   flag = CVodeSetMaxNumSteps(mem,ts->max_steps);
+  if (flag) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_LIB,"CVodeSetMaxNumSteps() fails, flag %d",flag);
 
   /* call CVSpgmr to use GMRES as the linear solver.        */
   /* setup the ode integrator with the given preconditioner */
@@ -387,9 +373,7 @@ const char *const TSSundialsLmmTypes[] = {"","ADAMS","BDF","TSSundialsLmmType","
 /* type of G-S orthogonalization used by CVODE linear solver */
 const char *const TSSundialsGramSchmidtTypes[] = {"","MODIFIED","CLASSICAL","TSSundialsGramSchmidtType","SUNDIALS_",0};
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSetFromOptions_Sundials"
-PetscErrorCode TSSetFromOptions_Sundials(PetscOptions *PetscOptionsObject,TS ts)
+PetscErrorCode TSSetFromOptions_Sundials(PetscOptionItems *PetscOptionsObject,TS ts)
 {
   TS_Sundials    *cvode = (TS_Sundials*)ts->data;
   PetscErrorCode ierr;
@@ -413,15 +397,13 @@ PetscErrorCode TSSetFromOptions_Sundials(PetscOptions *PetscOptionsObject,TS ts)
   ierr = PetscOptionsReal("-ts_sundials_maxdt","Maximum step size","TSSundialsSetMaxTimeStep",cvode->maxdt,&cvode->maxdt,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsReal("-ts_sundials_linear_tolerance","Convergence tolerance for linear solve","TSSundialsSetLinearTolerance",cvode->linear_tol,&cvode->linear_tol,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsInt("-ts_sundials_maxl","Max dimension of the Krylov subspace","TSSundialsSetMaxl",cvode->maxl,&cvode->maxl,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsBool("-ts_sundials_monitor_steps","Monitor SUNDIALS internel steps","TSSundialsMonitorInternalSteps",cvode->monitorstep,&cvode->monitorstep,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-ts_sundials_monitor_steps","Monitor SUNDIALS internal steps","TSSundialsMonitorInternalSteps",cvode->monitorstep,&cvode->monitorstep,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   ierr = TSSundialsGetPC(ts,&pc);CHKERRQ(ierr);
   ierr = PCSetFromOptions(pc);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSView_Sundials"
 PetscErrorCode TSView_Sundials(TS ts,PetscViewer viewer)
 {
   TS_Sundials    *cvode = (TS_Sundials*)ts->data;
@@ -494,8 +476,6 @@ PetscErrorCode TSView_Sundials(TS ts,PetscViewer viewer)
 
 
 /* --------------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsSetType_Sundials"
 PetscErrorCode  TSSundialsSetType_Sundials(TS ts,TSSundialsLmmType type)
 {
   TS_Sundials *cvode = (TS_Sundials*)ts->data;
@@ -505,8 +485,6 @@ PetscErrorCode  TSSundialsSetType_Sundials(TS ts,TSSundialsLmmType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsSetMaxl_Sundials"
 PetscErrorCode  TSSundialsSetMaxl_Sundials(TS ts,PetscInt maxl)
 {
   TS_Sundials *cvode = (TS_Sundials*)ts->data;
@@ -516,8 +494,6 @@ PetscErrorCode  TSSundialsSetMaxl_Sundials(TS ts,PetscInt maxl)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsSetLinearTolerance_Sundials"
 PetscErrorCode  TSSundialsSetLinearTolerance_Sundials(TS ts,double tol)
 {
   TS_Sundials *cvode = (TS_Sundials*)ts->data;
@@ -527,8 +503,6 @@ PetscErrorCode  TSSundialsSetLinearTolerance_Sundials(TS ts,double tol)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsSetGramSchmidtType_Sundials"
 PetscErrorCode  TSSundialsSetGramSchmidtType_Sundials(TS ts,TSSundialsGramSchmidtType type)
 {
   TS_Sundials *cvode = (TS_Sundials*)ts->data;
@@ -538,8 +512,6 @@ PetscErrorCode  TSSundialsSetGramSchmidtType_Sundials(TS ts,TSSundialsGramSchmid
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsSetTolerance_Sundials"
 PetscErrorCode  TSSundialsSetTolerance_Sundials(TS ts,double aabs,double rel)
 {
   TS_Sundials *cvode = (TS_Sundials*)ts->data;
@@ -550,8 +522,6 @@ PetscErrorCode  TSSundialsSetTolerance_Sundials(TS ts,double aabs,double rel)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsSetMinTimeStep_Sundials"
 PetscErrorCode  TSSundialsSetMinTimeStep_Sundials(TS ts,PetscReal mindt)
 {
   TS_Sundials *cvode = (TS_Sundials*)ts->data;
@@ -561,8 +531,6 @@ PetscErrorCode  TSSundialsSetMinTimeStep_Sundials(TS ts,PetscReal mindt)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsSetMaxTimeStep_Sundials"
 PetscErrorCode  TSSundialsSetMaxTimeStep_Sundials(TS ts,PetscReal maxdt)
 {
   TS_Sundials *cvode = (TS_Sundials*)ts->data;
@@ -571,8 +539,6 @@ PetscErrorCode  TSSundialsSetMaxTimeStep_Sundials(TS ts,PetscReal maxdt)
   cvode->maxdt = maxdt;
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsGetPC_Sundials"
 PetscErrorCode  TSSundialsGetPC_Sundials(TS ts,PC *pc)
 {
   SNES           snes;
@@ -586,8 +552,6 @@ PetscErrorCode  TSSundialsGetPC_Sundials(TS ts,PC *pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsGetIterations_Sundials"
 PetscErrorCode  TSSundialsGetIterations_Sundials(TS ts,int *nonlin,int *lin)
 {
   PetscFunctionBegin;
@@ -596,8 +560,6 @@ PetscErrorCode  TSSundialsGetIterations_Sundials(TS ts,int *nonlin,int *lin)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsMonitorInternalSteps_Sundials"
 PetscErrorCode  TSSundialsMonitorInternalSteps_Sundials(TS ts,PetscBool s)
 {
   TS_Sundials *cvode = (TS_Sundials*)ts->data;
@@ -608,8 +570,6 @@ PetscErrorCode  TSSundialsMonitorInternalSteps_Sundials(TS ts,PetscBool s)
 }
 /* -------------------------------------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsGetIterations"
 /*@C
    TSSundialsGetIterations - Gets the number of nonlinear and linear iterations used so far by Sundials.
 
@@ -644,8 +604,6 @@ PetscErrorCode  TSSundialsGetIterations(TS ts,int *nonlin,int *lin)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsSetType"
 /*@
    TSSundialsSetType - Sets the method that Sundials will use for integration.
 
@@ -674,8 +632,6 @@ PetscErrorCode  TSSundialsSetType(TS ts,TSSundialsLmmType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsSetMaxl"
 /*@
    TSSundialsSetMaxl - Sets the dimension of the Krylov space used by
        GMRES in the linear solver in SUNDIALS. SUNDIALS DOES NOT use restarted GMRES so
@@ -708,8 +664,6 @@ PetscErrorCode  TSSundialsSetMaxl(TS ts,PetscInt maxl)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsSetLinearTolerance"
 /*@
    TSSundialsSetLinearTolerance - Sets the tolerance used to solve the linear
        system by SUNDIALS.
@@ -742,8 +696,6 @@ PetscErrorCode  TSSundialsSetLinearTolerance(TS ts,double tol)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsSetGramSchmidtType"
 /*@
    TSSundialsSetGramSchmidtType - Sets type of orthogonalization used
         in GMRES method by SUNDIALS linear solver.
@@ -774,8 +726,6 @@ PetscErrorCode  TSSundialsSetGramSchmidtType(TS ts,TSSundialsGramSchmidtType typ
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsSetTolerance"
 /*@
    TSSundialsSetTolerance - Sets the absolute and relative tolerance used by
                          Sundials for error control.
@@ -810,8 +760,6 @@ PetscErrorCode  TSSundialsSetTolerance(TS ts,double aabs,double rel)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsGetPC"
 /*@
    TSSundialsGetPC - Extract the PC context from a time-step context for Sundials.
 
@@ -837,8 +785,6 @@ PetscErrorCode  TSSundialsGetPC(TS ts,PC *pc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsSetMinTimeStep"
 /*@
    TSSundialsSetMinTimeStep - Smallest time step to be chosen by the adaptive controller.
 
@@ -863,8 +809,6 @@ PetscErrorCode  TSSundialsSetMinTimeStep(TS ts,PetscReal mindt)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsSetMaxTimeStep"
 /*@
    TSSundialsSetMaxTimeStep - Largest time step to be chosen by the adaptive controller.
 
@@ -885,8 +829,6 @@ PetscErrorCode  TSSundialsSetMaxTimeStep(TS ts,PetscReal maxdt)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSundialsMonitorInternalSteps"
 /*@
    TSSundialsMonitorInternalSteps - Monitor Sundials internal steps (Defaults to false).
 
@@ -914,17 +856,18 @@ PetscErrorCode  TSSundialsMonitorInternalSteps(TS ts,PetscBool ft)
       TSSUNDIALS - ODE solver using the LLNL CVODE/SUNDIALS package (now called SUNDIALS)
 
    Options Database:
-+    -ts_sundials_type <bdf,adams>
++    -ts_sundials_type <bdf,adams> -
 .    -ts_sundials_gramschmidt_type <modified, classical> - type of orthogonalization inside GMRES
 .    -ts_sundials_atol <tol> - Absolute tolerance for convergence
 .    -ts_sundials_rtol <tol> - Relative tolerance for convergence
-.    -ts_sundials_linear_tolerance <tol>
+.    -ts_sundials_linear_tolerance <tol> - 
 .    -ts_sundials_maxl <maxl> - Max dimension of the Krylov subspace
--    -ts_sundials_monitor_steps - Monitor SUNDIALS internel steps
+-    -ts_sundials_monitor_steps - Monitor SUNDIALS internal steps
 
 
-    Notes: This uses its own nonlinear solver and Krylov method so PETSc SNES and KSP options do not apply
-           only PETSc PC options
+    Notes:
+    This uses its own nonlinear solver and Krylov method so PETSc SNES and KSP options do not apply,
+           only PETSc PC options.
 
     Level: beginner
 
@@ -932,8 +875,6 @@ PetscErrorCode  TSSundialsMonitorInternalSteps(TS ts,PetscBool ft)
            TSSundialsSetGramSchmidtType(), TSSundialsSetTolerance(), TSSundialsGetPC(), TSSundialsGetIterations(), TSSetExactFinalTime()
 
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "TSCreate_Sundials"
 PETSC_EXTERN PetscErrorCode TSCreate_Sundials(TS ts)
 {
   TS_Sundials    *cvode;
@@ -948,9 +889,12 @@ PETSC_EXTERN PetscErrorCode TSCreate_Sundials(TS ts)
   ts->ops->step           = TSStep_Sundials;
   ts->ops->interpolate    = TSInterpolate_Sundials;
   ts->ops->setfromoptions = TSSetFromOptions_Sundials;
+  ts->default_adapt_type  = TSADAPTNONE;
 
   ierr = PetscNewLog(ts,&cvode);CHKERRQ(ierr);
 
+  ts->usessnes = PETSC_TRUE;
+
   ts->data           = (void*)cvode;
   cvode->cvode_type  = SUNDIALS_BDF;
   cvode->gtype       = SUNDIALS_CLASSICAL_GS;
