@@ -22,12 +22,11 @@ typedef struct {
   PetscReal dt_increment;                   /* scaling that dt is incremented each time-step */
   PetscReal dt_max;                         /* maximum time step */
   PetscBool increment_dt_from_initial_dt;
+  PetscReal fatol,frtol;
 } TS_Pseudo;
 
 /* ------------------------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoComputeTimeStep"
 /*@C
     TSPseudoComputeTimeStep - Computes the next timestep for a currently running
     pseudo-timestepping process.
@@ -64,8 +63,6 @@ PetscErrorCode  TSPseudoComputeTimeStep(TS ts,PetscReal *dt)
 
 
 /* ------------------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoVerifyTimeStepDefault"
 /*@C
    TSPseudoVerifyTimeStepDefault - Default code to verify the quality of the last timestep.
 
@@ -98,8 +95,6 @@ PetscErrorCode  TSPseudoVerifyTimeStepDefault(TS ts,Vec update,void *dtctx,Petsc
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoVerifyTimeStep"
 /*@
     TSPseudoVerifyTimeStep - Verifies whether the last timestep was acceptable.
 
@@ -123,71 +118,76 @@ PetscErrorCode  TSPseudoVerifyTimeStepDefault(TS ts,Vec update,void *dtctx,Petsc
 
 .seealso: TSPseudoSetVerifyTimeStep(), TSPseudoVerifyTimeStepDefault()
 @*/
-PetscErrorCode  TSPseudoVerifyTimeStep(TS ts,Vec update,PetscReal *dt,PetscBool  *flag)
+PetscErrorCode  TSPseudoVerifyTimeStep(TS ts,Vec update,PetscReal *dt,PetscBool *flag)
 {
   TS_Pseudo      *pseudo = (TS_Pseudo*)ts->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (!pseudo->verify) {*flag = PETSC_TRUE; PetscFunctionReturn(0);}
-
-  ierr = (*pseudo->verify)(ts,update,pseudo->verifyctx,dt,flag);CHKERRQ(ierr);
+  *flag = PETSC_TRUE;
+  if(pseudo->verify) {
+    ierr = (*pseudo->verify)(ts,update,pseudo->verifyctx,dt,flag);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
 /* --------------------------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TSStep_Pseudo"
 static PetscErrorCode TSStep_Pseudo(TS ts)
 {
   TS_Pseudo           *pseudo = (TS_Pseudo*)ts->data;
-  PetscInt            its,lits,reject;
+  PetscInt            nits,lits,reject;
   PetscBool           stepok;
-  PetscReal           next_time_step;
-  SNESConvergedReason snesreason = SNES_CONVERGED_ITERATING;
+  PetscReal           next_time_step = ts->time_step;
   PetscErrorCode      ierr;
 
   PetscFunctionBegin;
   if (ts->steps == 0) pseudo->dt_initial = ts->time_step;
   ierr = VecCopy(ts->vec_sol,pseudo->update);CHKERRQ(ierr);
-  next_time_step = ts->time_step;
   ierr = TSPseudoComputeTimeStep(ts,&next_time_step);CHKERRQ(ierr);
   for (reject=0; reject<ts->max_reject; reject++,ts->reject++) {
     ts->time_step = next_time_step;
-    ierr = TSPreStep(ts);CHKERRQ(ierr);
     ierr = TSPreStage(ts,ts->ptime+ts->time_step);CHKERRQ(ierr);
     ierr = SNESSolve(ts->snes,NULL,pseudo->update);CHKERRQ(ierr);
-    ierr = SNESGetConvergedReason(ts->snes,&snesreason);CHKERRQ(ierr);
+    ierr = SNESGetIterationNumber(ts->snes,&nits);CHKERRQ(ierr);
     ierr = SNESGetLinearSolveIterations(ts->snes,&lits);CHKERRQ(ierr);
-    ierr = SNESGetIterationNumber(ts->snes,&its);CHKERRQ(ierr);
+    ts->snes_its += nits; ts->ksp_its += lits;
     ierr = TSPostStage(ts,ts->ptime+ts->time_step,0,&(pseudo->update));CHKERRQ(ierr);
-    ts->snes_its += its; ts->ksp_its += lits;
-    ierr = PetscInfo3(ts,"step=%D, nonlinear solve iterations=%D, linear solve iterations=%D\n",ts->steps,its,lits);CHKERRQ(ierr);
-    pseudo->fnorm = -1;         /* The current norm is no longer valid, monitor must recompute it. */
+    ierr = TSAdaptCheckStage(ts->adapt,ts,ts->ptime+ts->time_step,pseudo->update,&stepok);CHKERRQ(ierr);
+    if (!stepok) {next_time_step = ts->time_step; continue;}
+    pseudo->fnorm = -1; /* The current norm is no longer valid, monitor must recompute it. */
     ierr = TSPseudoVerifyTimeStep(ts,pseudo->update,&next_time_step,&stepok);CHKERRQ(ierr);
     if (stepok) break;
   }
-  if (snesreason < 0 && ts->max_snes_failures > 0 && ++ts->num_snes_failures >= ts->max_snes_failures) {
-    ts->reason = TS_DIVERGED_NONLINEAR_SOLVE;
-    ierr = PetscInfo2(ts,"step=%D, nonlinear solve solve failures %D greater than current TS allowed, stopping solve\n",ts->steps,ts->num_snes_failures);CHKERRQ(ierr);
-    PetscFunctionReturn(0);
-  }
   if (reject >= ts->max_reject) {
     ts->reason = TS_DIVERGED_STEP_REJECTED;
-    ierr = PetscInfo2(ts,"step=%D, step rejections %D greater than current TS allowed, stopping solve\n",ts->steps,reject);CHKERRQ(ierr);
+    ierr = PetscInfo2(ts,"Step=%D, step rejections %D greater than current TS allowed, stopping solve\n",ts->steps,reject);CHKERRQ(ierr);
     PetscFunctionReturn(0);
   }
+
   ierr = VecCopy(pseudo->update,ts->vec_sol);CHKERRQ(ierr);
   ts->ptime += ts->time_step;
   ts->time_step = next_time_step;
-  ts->steps++;
+
+  if (pseudo->fnorm < 0) {
+    ierr = VecZeroEntries(pseudo->xdot);CHKERRQ(ierr);
+    ierr = TSComputeIFunction(ts,ts->ptime,ts->vec_sol,pseudo->xdot,pseudo->func,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = VecNorm(pseudo->func,NORM_2,&pseudo->fnorm);CHKERRQ(ierr);
+  }
+  if (pseudo->fnorm < pseudo->fatol) {
+    ts->reason = TS_CONVERGED_PSEUDO_FATOL;
+    ierr = PetscInfo3(ts,"Step=%D, converged since fnorm %g < fatol %g\n",ts->steps,pseudo->fnorm,pseudo->frtol);CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  }
+  if (pseudo->fnorm/pseudo->fnorm_initial < pseudo->frtol) {
+    ts->reason = TS_CONVERGED_PSEUDO_FRTOL;
+    ierr = PetscInfo4(ts,"Step=%D, converged since fnorm %g / fnorm_initial %g < frtol %g\n",ts->steps,pseudo->fnorm,pseudo->fnorm_initial,pseudo->fatol);CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  }
   PetscFunctionReturn(0);
 }
 
 /*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TSReset_Pseudo"
 static PetscErrorCode TSReset_Pseudo(TS ts)
 {
   TS_Pseudo      *pseudo = (TS_Pseudo*)ts->data;
@@ -200,8 +200,6 @@ static PetscErrorCode TSReset_Pseudo(TS ts)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSDestroy_Pseudo"
 static PetscErrorCode TSDestroy_Pseudo(TS ts)
 {
   PetscErrorCode ierr;
@@ -219,8 +217,6 @@ static PetscErrorCode TSDestroy_Pseudo(TS ts)
 
 /*------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoGetXdot"
 /*
     Compute Xdot = (X^{n+1}-X^n)/dt) = 0
 */
@@ -233,6 +229,7 @@ static PetscErrorCode TSPseudoGetXdot(TS ts,Vec X,Vec *Xdot)
   PetscInt          i,n;
 
   PetscFunctionBegin;
+  *Xdot = NULL;
   ierr = VecGetArrayRead(ts->vec_sol,&xn);CHKERRQ(ierr);
   ierr = VecGetArrayRead(X,&xnp1);CHKERRQ(ierr);
   ierr = VecGetArray(pseudo->xdot,&xdot);CHKERRQ(ierr);
@@ -245,8 +242,6 @@ static PetscErrorCode TSPseudoGetXdot(TS ts,Vec X,Vec *Xdot)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESTSFormFunction_Pseudo"
 /*
     The transient residual is
 
@@ -275,8 +270,6 @@ static PetscErrorCode SNESTSFormFunction_Pseudo(SNES snes,Vec X,Vec Y,TS ts)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESTSFormJacobian_Pseudo"
 /*
    This constructs the Jacobian needed for SNES.  For DAE, this is
 
@@ -298,8 +291,6 @@ static PetscErrorCode SNESTSFormJacobian_Pseudo(SNES snes,Vec X,Mat AA,Mat BB,TS
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSetUp_Pseudo"
 static PetscErrorCode TSSetUp_Pseudo(TS ts)
 {
   TS_Pseudo      *pseudo = (TS_Pseudo*)ts->data;
@@ -313,19 +304,14 @@ static PetscErrorCode TSSetUp_Pseudo(TS ts)
 }
 /*------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoMonitorDefault"
-PetscErrorCode TSPseudoMonitorDefault(TS ts,PetscInt step,PetscReal ptime,Vec v,void *dummy)
+static PetscErrorCode TSPseudoMonitorDefault(TS ts,PetscInt step,PetscReal ptime,Vec v,void *dummy)
 {
   TS_Pseudo      *pseudo = (TS_Pseudo*)ts->data;
   PetscErrorCode ierr;
   PetscViewer    viewer = (PetscViewer) dummy;
 
   PetscFunctionBegin;
-  if (!viewer) {
-    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)ts),&viewer);CHKERRQ(ierr);
-  }
-  if (pseudo->fnorm < 0) {      /* The last computed norm is stale, recompute */
+  if (pseudo->fnorm < 0) { /* The last computed norm is stale, recompute */
     ierr = VecZeroEntries(pseudo->xdot);CHKERRQ(ierr);
     ierr = TSComputeIFunction(ts,ts->ptime,ts->vec_sol,pseudo->xdot,pseudo->func,PETSC_FALSE);CHKERRQ(ierr);
     ierr = VecNorm(pseudo->func,NORM_2,&pseudo->fnorm);CHKERRQ(ierr);
@@ -336,9 +322,7 @@ PetscErrorCode TSPseudoMonitorDefault(TS ts,PetscInt step,PetscReal ptime,Vec v,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSetFromOptions_Pseudo"
-static PetscErrorCode TSSetFromOptions_Pseudo(PetscOptions *PetscOptionsObject,TS ts)
+static PetscErrorCode TSSetFromOptions_Pseudo(PetscOptionItems *PetscOptionsObject,TS ts)
 {
   TS_Pseudo      *pseudo = (TS_Pseudo*)ts->data;
   PetscErrorCode ierr;
@@ -347,38 +331,41 @@ static PetscErrorCode TSSetFromOptions_Pseudo(PetscOptions *PetscOptionsObject,T
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"Pseudo-timestepping options");CHKERRQ(ierr);
-  ierr = PetscOptionsBool("-ts_monitor_pseudo","Monitor convergence","TSPseudoMonitorDefault",flg,&flg,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsBool("-ts_monitor_pseudo","Monitor convergence","",flg,&flg,NULL);CHKERRQ(ierr);
   if (flg) {
     ierr = PetscViewerASCIIOpen(PetscObjectComm((PetscObject)ts),"stdout",&viewer);CHKERRQ(ierr);
     ierr = TSMonitorSet(ts,TSPseudoMonitorDefault,viewer,(PetscErrorCode (*)(void**))PetscViewerDestroy);CHKERRQ(ierr);
   }
-  flg  = PETSC_FALSE;
+  flg  = pseudo->increment_dt_from_initial_dt;
   ierr = PetscOptionsBool("-ts_pseudo_increment_dt_from_initial_dt","Increase dt as a ratio from original dt","TSPseudoIncrementDtFromInitialDt",flg,&flg,NULL);CHKERRQ(ierr);
-  if (flg) {
-    ierr = TSPseudoIncrementDtFromInitialDt(ts);CHKERRQ(ierr);
-  }
+  pseudo->increment_dt_from_initial_dt = flg;
   ierr = PetscOptionsReal("-ts_pseudo_increment","Ratio to increase dt","TSPseudoSetTimeStepIncrement",pseudo->dt_increment,&pseudo->dt_increment,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsReal("-ts_pseudo_max_dt","Maximum value for dt","TSPseudoSetMaxTimeStep",pseudo->dt_max,&pseudo->dt_max,NULL);CHKERRQ(ierr);
-
-  ierr = SNESSetFromOptions(ts->snes);CHKERRQ(ierr);
+  ierr = PetscOptionsReal("-ts_pseudo_fatol","Tolerance for norm of function","",pseudo->fatol,&pseudo->fatol,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsReal("-ts_pseudo_frtol","Relative tolerance for norm of function","",pseudo->frtol,&pseudo->frtol,NULL);CHKERRQ(ierr);
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSView_Pseudo"
 static PetscErrorCode TSView_Pseudo(TS ts,PetscViewer viewer)
 {
   PetscErrorCode ierr;
+  PetscBool      isascii;
 
   PetscFunctionBegin;
-  ierr = SNESView(ts->snes,viewer);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
+  if (isascii) {
+    TS_Pseudo *pseudo = (TS_Pseudo*) ts->data;
+    ierr = PetscViewerASCIIPrintf(viewer,"  frtol - relative tolerance in function value %g\n",(double)pseudo->frtol);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  fatol - absolute tolerance in function value %g\n",(double)pseudo->fatol);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  dt_initial - initial timestep %g\n",(double)pseudo->dt_initial);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  dt_increment - increase in timestep on successful step %g\n",(double)pseudo->dt_increment);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPrintf(viewer,"  dt_max - maximum time %g\n",(double)pseudo->dt_max);CHKERRQ(ierr);
+  }
   PetscFunctionReturn(0);
 }
 
 /* ----------------------------------------------------------------------------- */
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoSetVerifyTimeStep"
 /*@C
    TSPseudoSetVerifyTimeStep - Sets a user-defined routine to verify the quality of the
    last timestep.
@@ -419,8 +406,6 @@ PetscErrorCode  TSPseudoSetVerifyTimeStep(TS ts,PetscErrorCode (*dt)(TS,Vec,void
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoSetTimeStepIncrement"
 /*@
     TSPseudoSetTimeStepIncrement - Sets the scaling increment applied to
     dt when using the TSPseudoTimeStepDefault() routine.
@@ -451,8 +436,6 @@ PetscErrorCode  TSPseudoSetTimeStepIncrement(TS ts,PetscReal inc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoSetMaxTimeStep"
 /*@
     TSPseudoSetMaxTimeStep - Sets the maximum time step
     when using the TSPseudoTimeStepDefault() routine.
@@ -483,8 +466,6 @@ PetscErrorCode  TSPseudoSetMaxTimeStep(TS ts,PetscReal maxdt)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoIncrementDtFromInitialDt"
 /*@
     TSPseudoIncrementDtFromInitialDt - Indicates that a new timestep
     is computed via the formula
@@ -517,8 +498,6 @@ PetscErrorCode  TSPseudoIncrementDtFromInitialDt(TS ts)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoSetTimeStep"
 /*@C
    TSPseudoSetTimeStep - Sets the user-defined routine to be
    called at each pseudo-timestep to update the timestep.
@@ -561,22 +540,17 @@ PetscErrorCode  TSPseudoSetTimeStep(TS ts,PetscErrorCode (*dt)(TS,PetscReal*,voi
 /* ----------------------------------------------------------------------------- */
 
 typedef PetscErrorCode (*FCN1)(TS,Vec,void*,PetscReal*,PetscBool*);  /* force argument to next function to not be extern C*/
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoSetVerifyTimeStep_Pseudo"
-PetscErrorCode  TSPseudoSetVerifyTimeStep_Pseudo(TS ts,FCN1 dt,void *ctx)
+static PetscErrorCode  TSPseudoSetVerifyTimeStep_Pseudo(TS ts,FCN1 dt,void *ctx)
 {
-  TS_Pseudo *pseudo;
+  TS_Pseudo *pseudo = (TS_Pseudo*)ts->data;
 
   PetscFunctionBegin;
-  pseudo            = (TS_Pseudo*)ts->data;
   pseudo->verify    = dt;
   pseudo->verifyctx = ctx;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoSetTimeStepIncrement_Pseudo"
-PetscErrorCode  TSPseudoSetTimeStepIncrement_Pseudo(TS ts,PetscReal inc)
+static PetscErrorCode  TSPseudoSetTimeStepIncrement_Pseudo(TS ts,PetscReal inc)
 {
   TS_Pseudo *pseudo = (TS_Pseudo*)ts->data;
 
@@ -585,9 +559,7 @@ PetscErrorCode  TSPseudoSetTimeStepIncrement_Pseudo(TS ts,PetscReal inc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoSetMaxTimeStep_Pseudo"
-PetscErrorCode  TSPseudoSetMaxTimeStep_Pseudo(TS ts,PetscReal maxdt)
+static PetscErrorCode  TSPseudoSetMaxTimeStep_Pseudo(TS ts,PetscReal maxdt)
 {
   TS_Pseudo *pseudo = (TS_Pseudo*)ts->data;
 
@@ -596,9 +568,7 @@ PetscErrorCode  TSPseudoSetMaxTimeStep_Pseudo(TS ts,PetscReal maxdt)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoIncrementDtFromInitialDt_Pseudo"
-PetscErrorCode  TSPseudoIncrementDtFromInitialDt_Pseudo(TS ts)
+static PetscErrorCode  TSPseudoIncrementDtFromInitialDt_Pseudo(TS ts)
 {
   TS_Pseudo *pseudo = (TS_Pseudo*)ts->data;
 
@@ -608,9 +578,7 @@ PetscErrorCode  TSPseudoIncrementDtFromInitialDt_Pseudo(TS ts)
 }
 
 typedef PetscErrorCode (*FCN2)(TS,PetscReal*,void*); /* force argument to next function to not be extern C*/
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoSetTimeStep_Pseudo"
-PetscErrorCode  TSPseudoSetTimeStep_Pseudo(TS ts,FCN2 dt,void *ctx)
+static PetscErrorCode  TSPseudoSetTimeStep_Pseudo(TS ts,FCN2 dt,void *ctx)
 {
   TS_Pseudo *pseudo = (TS_Pseudo*)ts->data;
 
@@ -642,13 +610,15 @@ $    G(Y) = F(Y,(Y-X)/dt)
 
   Options database keys:
 +  -ts_pseudo_increment <real> - ratio of increase dt
--  -ts_pseudo_increment_dt_from_initial_dt <truth> - Increase dt as a ratio from original dt
+.  -ts_pseudo_increment_dt_from_initial_dt <truth> - Increase dt as a ratio from original dt
+.  -ts_pseudo_fatol <atol> - stop iterating when the function norm is less than atol
+-  -ts_pseudo_frtol <rtol> - stop iterating when the function norm divided by the initial function norm is less than rtol
 
   Level: beginner
 
   References:
-  Todd S. Coffey and C. T. Kelley and David E. Keyes, Pseudotransient Continuation and Differential-Algebraic Equations, 2003.
-  C. T. Kelley and David E. Keyes, Convergence analysis of Pseudotransient Continuation, 1998.
++  1. - Todd S. Coffey and C. T. Kelley and David E. Keyes, Pseudotransient Continuation and Differential Algebraic Equations, 2003.
+-  2. - C. T. Kelley and David E. Keyes, Convergence analysis of Pseudotransient Continuation, 1998.
 
   Notes:
   The residual computed by this method includes the transient term (Xdot is computed instead of
@@ -664,8 +634,6 @@ $  Xdot = (Xpredicted - Xold)/dt = (Xold-Xold)/dt = 0
 .seealso:  TSCreate(), TS, TSSetType()
 
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "TSCreate_Pseudo"
 PETSC_EXTERN PetscErrorCode TSCreate_Pseudo(TS ts)
 {
   TS_Pseudo      *pseudo;
@@ -674,15 +642,16 @@ PETSC_EXTERN PetscErrorCode TSCreate_Pseudo(TS ts)
   SNESType       stype;
 
   PetscFunctionBegin;
-  ts->ops->reset   = TSReset_Pseudo;
-  ts->ops->destroy = TSDestroy_Pseudo;
-  ts->ops->view    = TSView_Pseudo;
-
+  ts->ops->reset          = TSReset_Pseudo;
+  ts->ops->destroy        = TSDestroy_Pseudo;
+  ts->ops->view           = TSView_Pseudo;
   ts->ops->setup          = TSSetUp_Pseudo;
   ts->ops->step           = TSStep_Pseudo;
   ts->ops->setfromoptions = TSSetFromOptions_Pseudo;
   ts->ops->snesfunction   = SNESTSFormFunction_Pseudo;
   ts->ops->snesjacobian   = SNESTSFormJacobian_Pseudo;
+  ts->default_adapt_type  = TSADAPTNONE;
+  ts->usessnes            = PETSC_TRUE;
 
   ierr = TSGetSNES(ts,&snes);CHKERRQ(ierr);
   ierr = SNESGetType(snes,&stype);CHKERRQ(ierr);
@@ -691,11 +660,20 @@ PETSC_EXTERN PetscErrorCode TSCreate_Pseudo(TS ts)
   ierr = PetscNewLog(ts,&pseudo);CHKERRQ(ierr);
   ts->data = (void*)pseudo;
 
+  pseudo->dt                           = TSPseudoTimeStepDefault;
+  pseudo->dtctx                        = NULL;
   pseudo->dt_increment                 = 1.1;
   pseudo->increment_dt_from_initial_dt = PETSC_FALSE;
-  pseudo->dt                           = TSPseudoTimeStepDefault;
   pseudo->fnorm                        = -1;
-
+  pseudo->fnorm_initial                = -1;
+  pseudo->fnorm_previous               = -1;
+ #if defined(PETSC_USE_REAL_SINGLE)
+  pseudo->fatol                        = 1.e-25;
+  pseudo->frtol                        = 1.e-5;
+#else
+  pseudo->fatol                        = 1.e-50;
+  pseudo->frtol                        = 1.e-12;
+#endif
   ierr = PetscObjectComposeFunction((PetscObject)ts,"TSPseudoSetVerifyTimeStep_C",TSPseudoSetVerifyTimeStep_Pseudo);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)ts,"TSPseudoSetTimeStepIncrement_C",TSPseudoSetTimeStepIncrement_Pseudo);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)ts,"TSPseudoSetMaxTimeStep_C",TSPseudoSetMaxTimeStep_Pseudo);CHKERRQ(ierr);
@@ -704,8 +682,6 @@ PETSC_EXTERN PetscErrorCode TSCreate_Pseudo(TS ts)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSPseudoTimeStepDefault"
 /*@C
    TSPseudoTimeStepDefault - Default code to compute pseudo-timestepping.
    Use with TSPseudoSetTimeStep().
@@ -728,21 +704,21 @@ PETSC_EXTERN PetscErrorCode TSCreate_Pseudo(TS ts)
 PetscErrorCode  TSPseudoTimeStepDefault(TS ts,PetscReal *newdt,void *dtctx)
 {
   TS_Pseudo      *pseudo = (TS_Pseudo*)ts->data;
-  PetscReal      inc     = pseudo->dt_increment,fnorm_previous = pseudo->fnorm_previous;
+  PetscReal      inc = pseudo->dt_increment;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   ierr = VecZeroEntries(pseudo->xdot);CHKERRQ(ierr);
   ierr = TSComputeIFunction(ts,ts->ptime,ts->vec_sol,pseudo->xdot,pseudo->func,PETSC_FALSE);CHKERRQ(ierr);
   ierr = VecNorm(pseudo->func,NORM_2,&pseudo->fnorm);CHKERRQ(ierr);
-  if (pseudo->fnorm_initial == 0.0) {
+  if (pseudo->fnorm_initial < 0) {
     /* first time through so compute initial function norm */
-    pseudo->fnorm_initial = pseudo->fnorm;
-    fnorm_previous        = pseudo->fnorm;
+    pseudo->fnorm_initial  = pseudo->fnorm;
+    pseudo->fnorm_previous = pseudo->fnorm;
   }
   if (pseudo->fnorm == 0.0)                      *newdt = 1.e12*inc*ts->time_step;
   else if (pseudo->increment_dt_from_initial_dt) *newdt = inc*pseudo->dt_initial*pseudo->fnorm_initial/pseudo->fnorm;
-  else                                           *newdt = inc*ts->time_step*fnorm_previous/pseudo->fnorm;
+  else                                           *newdt = inc*ts->time_step*pseudo->fnorm_previous/pseudo->fnorm;
   if (pseudo->dt_max > 0) *newdt = PetscMin(*newdt,pseudo->dt_max);
   pseudo->fnorm_previous = pseudo->fnorm;
   PetscFunctionReturn(0);
