@@ -7,10 +7,10 @@
 #define tspythonsettype_            tspythonsettype
 #endif
 
-PETSC_EXTERN void PETSC_STDCALL tspythonsettype_(TS *ts, CHAR name PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len))
+PETSC_EXTERN void PETSC_STDCALL tspythonsettype_(TS *ts, char* name PETSC_MIXED_LEN(len),PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   char *t;
   FIXCHAR(name,len,t);
-  *ierr = TSPythonSetType(*ts,t);
+  *ierr = TSPythonSetType(*ts,t);if (*ierr) return;
   FREECHAR(name,t);
 }
