@@ -54,7 +54,7 @@ typedef struct {
   Vec          Ystage;           /* Work vector for the state value at each stage */
   Vec          Zdot;             /* Ydot = Zdot + shift*Y */
   Vec          Zstage;           /* Y = Zstage + Y */
-  Vec          VecSolPrev;       /* Work vector holding the solution from the previous step (used for interpolation)*/
+  Vec          vec_sol_prev;     /* Solution from the previous step (used for interpolation and rollback)*/
   PetscScalar  *work;            /* Scalar work space of length number of stages, used to prepare VecMAXPY() */
   PetscReal    scoeff;           /* shift = scoeff/dt */
   PetscReal    stage_time;
@@ -111,7 +111,7 @@ M*/
      This is strongly A-stable with R(infty) = 0.73. The embedded method of order 2 is strongly A-stable with R(infty) = 0.73.
 
      References:
-     Rang and Angermann, New Rosenbrock-W methods of order 3 for partial differential algebraic equations of index 1, 2005.
+.  1. -   Rang and Angermann, New Rosenbrock W methods of order 3 for partial differential algebraic equations of index 1, 2005.
 
      Level: intermediate
 
@@ -126,7 +126,7 @@ M*/
      This is strongly A-stable with R(infty) = 0. The embedded method of order 2 is strongly A-stable with R(infty) = 0.48.
 
      References:
-     Rang and Angermann, New Rosenbrock-W methods of order 3 for partial differential algebraic equations of index 1, 2005.
+.  1. -   Rang and Angermann, New Rosenbrock W methods of order 3 for partial differential algebraic equations of index 1, 2005.
 
      Level: intermediate
 
@@ -141,7 +141,7 @@ M*/
      Both the third order and embedded second order methods are stiffly accurate and L-stable.
 
      References:
-     Sandu et al, Benchmarking stiff ODE solvers for atmospheric chemistry problems II, Rosenbrock solvers, 1997.
+.  1. -   Sandu et al, Benchmarking stiff ODE solvers for atmospheric chemistry problems II, Rosenbrock solvers, 1997.
 
      Level: intermediate
 
@@ -159,7 +159,7 @@ M*/
      This method is called ROS3 in the paper.
 
      References:
-     Sandu et al, Benchmarking stiff ODE solvers for atmospheric chemistry problems II, Rosenbrock solvers, 1997.
+.  1. -   Sandu et al, Benchmarking stiff ODE solvers for atmospheric chemistry problems II, Rosenbrock solvers, 1997.
 
      Level: intermediate
 
@@ -174,7 +174,7 @@ M*/
      A-stable SPP explicit order 3, 3 stages, CFL 1 (eff = 1/3)
 
      References:
-     Emil Constantinescu
+.     Emil Constantinescu
 
      Level: intermediate
 
@@ -189,7 +189,7 @@ M*/
      L-stable (A-stable embedded) SPP explicit order 3, 4 stages, CFL 2 (eff = 1/2)
 
      References:
-     Emil Constantinescu
+.     Emil Constantinescu
 
      Level: intermediate
 
@@ -204,7 +204,7 @@ M*/
      L-stable (L-stable embedded) SPP explicit order 3, 4 stages, CFL 2 (eff = 1/2)
 
      References:
-     Emil Constantinescu
+.     Emil Constantinescu
 
      Level: intermediate
 
@@ -221,9 +221,8 @@ M*/
      This method does not provide a dense output formula.
 
      References:
-     Kaps and Rentrop, Generalized Runge-Kutta methods of order four with stepsize control for stiff ordinary differential equations, 1979.
-
-     Hairer and Wanner, Solving Ordinary Differential Equations II, Section 4 Table 7.2.
++   1. -  Kaps and Rentrop, Generalized Runge Kutta methods of order four with stepsize control for stiff ordinary differential equations, 1979.
+-   2. -  Hairer and Wanner, Solving Ordinary Differential Equations II, Section 4 Table 7.2.
 
      Hairer's code ros4.f
 
@@ -242,9 +241,8 @@ M*/
      This method does not provide a dense output formula.
 
      References:
-     Shampine, Implementation of Rosenbrock methods, 1982.
-
-     Hairer and Wanner, Solving Ordinary Differential Equations II, Section 4 Table 7.2.
++   1. -  Shampine, Implementation of Rosenbrock methods, 1982.
+-   2. -  Hairer and Wanner, Solving Ordinary Differential Equations II, Section 4 Table 7.2.
 
      Hairer's code ros4.f
 
@@ -263,9 +261,8 @@ M*/
      This method does not provide a dense output formula.
 
      References:
-     van Veldhuizen, D-stability and Kaps-Rentrop methods, 1984.
-
-     Hairer and Wanner, Solving Ordinary Differential Equations II, Section 4 Table 7.2.
++   1. -  van Veldhuizen, D stability and Kaps Rentrop methods, 1984.
+-   2. -  Hairer and Wanner, Solving Ordinary Differential Equations II, Section 4 Table 7.2.
 
      Hairer's code ros4.f
 
@@ -284,7 +281,7 @@ M*/
      This method does not provide a dense output formula.
 
      References:
-     Hairer and Wanner, Solving Ordinary Differential Equations II, Section 4 Table 7.2.
+.  1. -   Hairer and Wanner, Solving Ordinary Differential Equations II, Section 4 Table 7.2.
 
      Hairer's code ros4.f
 
@@ -293,10 +290,8 @@ M*/
 .seealso: TSROSW, TSROSWGRK4T, TSROSWSHAMP4, TSROSW4L
 M*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRosWRegisterAll"
 /*@C
-  TSRosWRegisterAll - Registers all of the additive Runge-Kutta implicit-explicit methods in TSRosW
+  TSRosWRegisterAll - Registers all of the Rosenbrock-W methods in TSRosW
 
   Not Collective, but should be called by all processes which will need the schemes to be registered
 
@@ -601,8 +596,6 @@ PetscErrorCode TSRosWRegisterAll(void)
 
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRosWRegisterDestroy"
 /*@C
    TSRosWRegisterDestroy - Frees the list of schemes that were registered by TSRosWRegister().
 
@@ -633,8 +626,6 @@ PetscErrorCode TSRosWRegisterDestroy(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRosWInitializePackage"
 /*@C
   TSRosWInitializePackage - This function initializes everything in the TSRosW package. It is called
   from PetscDLLibraryRegister() when using dynamic libraries, and on the first call to TSCreate_RosW()
@@ -657,8 +648,6 @@ PetscErrorCode TSRosWInitializePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRosWFinalizePackage"
 /*@C
   TSRosWFinalizePackage - This function destroys everything in the TSRosW package. It is
   called from PetscFinalize().
@@ -678,8 +667,6 @@ PetscErrorCode TSRosWFinalizePackage(void)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRosWRegister"
 /*@C
    TSRosWRegister - register a Rosenbrock W scheme by providing the entries in the Butcher tableau and optionally embedded approximations and interpolation
 
@@ -721,6 +708,7 @@ PetscErrorCode TSRosWRegister(TSRosWType name,PetscInt order,PetscInt s,const Pe
   PetscValidPointer(b,6);
   if (bembed) PetscValidPointer(bembed,7);
 
+  ierr     = TSRosWInitializePackage();CHKERRQ(ierr);
   ierr     = PetscCalloc1(1,&link);CHKERRQ(ierr);
   t        = &link->tab;
   ierr     = PetscStrallocpy(name,&t->name);CHKERRQ(ierr);
@@ -757,16 +745,16 @@ PetscErrorCode TSRosWRegister(TSRosWType name,PetscInt order,PetscInt s,const Pe
 
   switch (s) {
   case 1: GammaInv[0] = 1./GammaInv[0]; break;
-  case 2: ierr = PetscKernel_A_gets_inverse_A_2(GammaInv,0);CHKERRQ(ierr); break;
-  case 3: ierr = PetscKernel_A_gets_inverse_A_3(GammaInv,0);CHKERRQ(ierr); break;
-  case 4: ierr = PetscKernel_A_gets_inverse_A_4(GammaInv,0);CHKERRQ(ierr); break;
+  case 2: ierr = PetscKernel_A_gets_inverse_A_2(GammaInv,0,PETSC_FALSE,NULL);CHKERRQ(ierr); break;
+  case 3: ierr = PetscKernel_A_gets_inverse_A_3(GammaInv,0,PETSC_FALSE,NULL);CHKERRQ(ierr); break;
+  case 4: ierr = PetscKernel_A_gets_inverse_A_4(GammaInv,0,PETSC_FALSE,NULL);CHKERRQ(ierr); break;
   case 5: {
     PetscInt  ipvt5[5];
     MatScalar work5[5*5];
-    ierr = PetscKernel_A_gets_inverse_A_5(GammaInv,ipvt5,work5,0);CHKERRQ(ierr); break;
+    ierr = PetscKernel_A_gets_inverse_A_5(GammaInv,ipvt5,work5,0,PETSC_FALSE,NULL);CHKERRQ(ierr); break;
   }
-  case 6: ierr = PetscKernel_A_gets_inverse_A_6(GammaInv,0);CHKERRQ(ierr); break;
-  case 7: ierr = PetscKernel_A_gets_inverse_A_7(GammaInv,0);CHKERRQ(ierr); break;
+  case 6: ierr = PetscKernel_A_gets_inverse_A_6(GammaInv,0,PETSC_FALSE,NULL);CHKERRQ(ierr); break;
+  case 7: ierr = PetscKernel_A_gets_inverse_A_7(GammaInv,0,PETSC_FALSE,NULL);CHKERRQ(ierr); break;
   default: SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Not implemented for %D stages",s);
   }
   for (i=0; i<s*s; i++) t->GammaInv[i] = PetscRealPart(GammaInv[i]);
@@ -809,8 +797,6 @@ PetscErrorCode TSRosWRegister(TSRosWType name,PetscInt order,PetscInt s,const Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRosWRegisterRos4"
 /*@C
    TSRosWRegisterRos4 - register a fourth order Rosenbrock scheme by providing paramter choices
 
@@ -863,7 +849,7 @@ PetscErrorCode TSRosWRegisterRos4(TSRosWType name,PetscReal gamma,PetscReal a2,P
   rhs[0]  = one - b3;
   rhs[1]  = one/three - a3*a3*b3;
   rhs[2]  = one/four - a3*a3*a3*b3;
-  ierr    = PetscKernel_A_gets_inverse_A_3(&M[0][0],0);CHKERRQ(ierr);
+  ierr    = PetscKernel_A_gets_inverse_A_3(&M[0][0],0,PETSC_FALSE,NULL);CHKERRQ(ierr);
   b1      = PetscRealPart(M[0][0]*rhs[0] + M[0][1]*rhs[1] + M[0][2]*rhs[2]);
   b2      = PetscRealPart(M[1][0]*rhs[0] + M[1][1]*rhs[1] + M[1][2]*rhs[2]);
   b4      = PetscRealPart(M[2][0]*rhs[0] + M[2][1]*rhs[1] + M[2][2]*rhs[2]);
@@ -876,7 +862,7 @@ PetscErrorCode TSRosWRegisterRos4(TSRosWType name,PetscReal gamma,PetscReal a2,P
   M[1][0]      = a4*a4*beta32beta2p-a3*a3*beta4jbetajp; M[1][1] = a2*a2*beta4jbetajp; M[1][2] = -a2*a2*beta32beta2p;
   M[2][0]      = b4*beta43*a3*a3-p43;                   M[2][1] = -b4*beta43*a2*a2;   M[2][2] = 0;
   rhs[0]       = one/two - gamma; rhs[1] = 0; rhs[2] = -a2*a2*p32;
-  ierr         = PetscKernel_A_gets_inverse_A_3(&M[0][0],0);CHKERRQ(ierr);
+  ierr         = PetscKernel_A_gets_inverse_A_3(&M[0][0],0,PETSC_FALSE,NULL);CHKERRQ(ierr);
   beta2p       = PetscRealPart(M[0][0]*rhs[0] + M[0][1]*rhs[1] + M[0][2]*rhs[2]);
   beta3p       = PetscRealPart(M[1][0]*rhs[0] + M[1][1]*rhs[1] + M[1][2]*rhs[2]);
   beta4p       = PetscRealPart(M[2][0]*rhs[0] + M[2][1]*rhs[1] + M[2][2]*rhs[2]);
@@ -914,8 +900,6 @@ PetscErrorCode TSRosWRegisterRos4(TSRosWType name,PetscReal gamma,PetscReal a2,P
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSEvaluateStep_RosW"
 /*
  The step completion formula is
 
@@ -963,31 +947,20 @@ static PetscErrorCode TSEvaluateStep_RosW(TS ts,PetscInt order,Vec U,PetscBool *
   }
   unavailable:
   if (done) *done = PETSC_FALSE;
-  else SETERRQ3(PetscObjectComm((PetscObject)ts),PETSC_ERR_SUP,"Rosenbrock-W '%s' of order %D cannot evaluate step at order %D",tab->name,tab->order,order);
+  else SETERRQ3(PetscObjectComm((PetscObject)ts),PETSC_ERR_SUP,"Rosenbrock-W '%s' of order %D cannot evaluate step at order %D. Consider using -ts_adapt_type none or a different method that has an embedded estimate.",tab->name,tab->order,order);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRollBack_RosW"
-PetscErrorCode TSRollBack_RosW(TS ts)
+static PetscErrorCode TSRollBack_RosW(TS ts)
 {
   TS_RosW        *ros = (TS_RosW*)ts->data;
-  RosWTableau    tab = ros->tableau;
-  const PetscInt s    = tab->s;
-  PetscScalar    *w = ros->work;
-  PetscInt       i;
-  Vec            *Y = ros->Y;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  for (i=0; i<s; i++) w[i] = -tab->bt[i];
-  ierr = VecMAXPY(ts->vec_sol,s,w,Y);CHKERRQ(ierr);
-  ros->status   = TS_STEP_INCOMPLETE;
+  ierr = VecCopy(ros->vec_sol_prev,ts->vec_sol);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSStep_RosW"
 static PetscErrorCode TSStep_RosW(TS ts)
 {
   TS_RosW         *ros = (TS_RosW*)ts->data;
@@ -1000,21 +973,20 @@ static PetscErrorCode TSStep_RosW(TS ts)
   Vec             *Y   = ros->Y,Ydot = ros->Ydot,Zdot = ros->Zdot,Zstage = ros->Zstage;
   SNES            snes;
   TSAdapt         adapt;
-  PetscInt        i,j,its,lits,reject,next_scheme;
-  PetscBool       accept;
-  PetscReal       next_time_step;
+  PetscInt        i,j,its,lits;
+  PetscInt        rejections = 0;
+  PetscBool       stageok,accept = PETSC_TRUE;
+  PetscReal       next_time_step = ts->time_step;
   PetscErrorCode  ierr;
 
   PetscFunctionBegin;
-  ierr = TSGetSNES(ts,&snes);CHKERRQ(ierr);
-  accept         = PETSC_TRUE;
-  next_time_step = ts->time_step;
-  ros->status    = TS_STEP_INCOMPLETE;
+  if (!ts->steprollback) {
+    ierr = VecCopy(ts->vec_sol,ros->vec_sol_prev);CHKERRQ(ierr);
+  }
 
-  for (reject=0; reject<ts->max_reject && !ts->reason; reject++,ts->reject++) {
+  ros->status = TS_STEP_INCOMPLETE;
+  while (!ts->reason && ros->status != TS_STEP_COMPLETE) {
     const PetscReal h = ts->time_step;
-    ierr = TSPreStep(ts);CHKERRQ(ierr);
-    ierr = VecCopy(ts->vec_sol,ros->VecSolPrev);CHKERRQ(ierr); /*move this at the end*/
     for (i=0; i<s; i++) {
       ros->stage_time = ts->ptime + h*ASum[i];
       ierr = TSPreStage(ts,ros->stage_time);CHKERRQ(ierr);
@@ -1038,6 +1010,7 @@ static PetscErrorCode TSStep_RosW(TS ts)
       ierr = VecZeroEntries(Y[i]);CHKERRQ(ierr);
 
       if (!ros->stage_explicit) {
+        ierr = TSGetSNES(ts,&snes);CHKERRQ(ierr);
         if (!ros->recompute_jacobian && !i) {
           ierr = SNESSetLagJacobian(snes,-2);CHKERRQ(ierr); /* Recompute the Jacobian on this solve, but not again */
         }
@@ -1045,58 +1018,60 @@ static PetscErrorCode TSStep_RosW(TS ts)
         ierr = SNESGetIterationNumber(snes,&its);CHKERRQ(ierr);
         ierr = SNESGetLinearSolveIterations(snes,&lits);CHKERRQ(ierr);
         ts->snes_its += its; ts->ksp_its += lits;
-        ierr = TSGetAdapt(ts,&adapt);CHKERRQ(ierr);
-        ierr = TSAdaptCheckStage(adapt,ts,&accept);CHKERRQ(ierr);
-        if (!accept) goto reject_step;
       } else {
         Mat J,Jp;
         ierr = VecZeroEntries(Ydot);CHKERRQ(ierr); /* Evaluate Y[i]=G(t,Ydot=0,Zstage) */
         ierr = TSComputeIFunction(ts,ros->stage_time,Zstage,Ydot,Y[i],PETSC_FALSE);CHKERRQ(ierr);
         ierr = VecScale(Y[i],-1.0);CHKERRQ(ierr);
-        ierr = VecAXPY(Y[i],-1.0,Zdot);CHKERRQ(ierr); /*Y[i]=F(Zstage)-Zdot[=GammaInv*Y]*/
+        ierr = VecAXPY(Y[i],-1.0,Zdot);CHKERRQ(ierr); /*Y[i] = F(Zstage)-Zdot[=GammaInv*Y]*/
 
         ierr = VecZeroEntries(Zstage);CHKERRQ(ierr); /* Zstage = GammaExplicitCorr[i,j] * Y[j] */
         for (j=0; j<i; j++) w[j] = GammaExplicitCorr[i*s+j];
         ierr = VecMAXPY(Zstage,i,w,Y);CHKERRQ(ierr);
-        /*Y[i] += Y[i] + Jac*Zstage[=Jac*GammaExplicitCorr[i,j] * Y[j]] */
+
+        /* Y[i] = Y[i] + Jac*Zstage[=Jac*GammaExplicitCorr[i,j] * Y[j]] */
         ierr = TSGetIJacobian(ts,&J,&Jp,NULL,NULL);CHKERRQ(ierr);
         ierr = TSComputeIJacobian(ts,ros->stage_time,ts->vec_sol,Ydot,0,J,Jp,PETSC_FALSE);CHKERRQ(ierr);
         ierr = MatMult(J,Zstage,Zdot);CHKERRQ(ierr);
-
         ierr = VecAXPY(Y[i],-1.0,Zdot);CHKERRQ(ierr);
-        ierr = VecScale(Y[i],h);CHKERRQ(ierr);
         ts->ksp_its += 1;
+
+        ierr = VecScale(Y[i],h);CHKERRQ(ierr);
       }
       ierr = TSPostStage(ts,ros->stage_time,i,Y);CHKERRQ(ierr);
+      ierr = TSGetAdapt(ts,&adapt);CHKERRQ(ierr);
+      ierr = TSAdaptCheckStage(adapt,ts,ros->stage_time,Y[i],&stageok);CHKERRQ(ierr);
+      if (!stageok) goto reject_step;
     }
-    ierr = TSEvaluateStep(ts,tab->order,ts->vec_sol,NULL);CHKERRQ(ierr);
-    ros->status = TS_STEP_PENDING;
 
-    /* Register only the current method as a candidate because we're not supporting multiple candidates yet. */
+    ros->status = TS_STEP_INCOMPLETE;
+    ierr = TSEvaluateStep_RosW(ts,tab->order,ts->vec_sol,NULL);CHKERRQ(ierr);
+    ros->status = TS_STEP_PENDING;
     ierr = TSGetAdapt(ts,&adapt);CHKERRQ(ierr);
     ierr = TSAdaptCandidatesClear(adapt);CHKERRQ(ierr);
-    ierr = TSAdaptCandidateAdd(adapt,tab->name,tab->order,1,tab->ccfl,1.*tab->s,PETSC_TRUE);CHKERRQ(ierr);
-    ierr = TSAdaptChoose(adapt,ts,ts->time_step,&next_scheme,&next_time_step,&accept);CHKERRQ(ierr);
-    if (accept) {
-      /* ignore next_scheme for now */
-      ts->ptime    += ts->time_step;
+    ierr = TSAdaptCandidateAdd(adapt,tab->name,tab->order,1,tab->ccfl,(PetscReal)tab->s,PETSC_TRUE);CHKERRQ(ierr);
+    ierr = TSAdaptChoose(adapt,ts,ts->time_step,NULL,&next_time_step,&accept);CHKERRQ(ierr);
+    ros->status = accept ? TS_STEP_COMPLETE : TS_STEP_INCOMPLETE;
+    if (!accept) { /* Roll back the current step */
+      ierr = TSRollBack_RosW(ts);CHKERRQ(ierr);
       ts->time_step = next_time_step;
-      ts->steps++;
-      ros->status = TS_STEP_COMPLETE;
-      break;
-    } else {                    /* Roll back the current step */
-      ts->ptime += next_time_step; /* This will be undone in rollback */
-      ros->status = TS_STEP_INCOMPLETE;
-      ierr = TSRollBack(ts);CHKERRQ(ierr);
+      goto reject_step;
+    }
+
+    ts->ptime += ts->time_step;
+    ts->time_step = next_time_step;
+    break;
+
+  reject_step:
+    ts->reject++; accept = PETSC_FALSE;
+    if (!ts->reason && ++rejections > ts->max_reject && ts->max_reject >= 0) {
+      ts->reason = TS_DIVERGED_STEP_REJECTED;
+      ierr = PetscInfo2(ts,"Step=%D, step rejections %D greater than current TS allowed, stopping solve\n",ts->steps,rejections);CHKERRQ(ierr);
     }
-reject_step: continue;
   }
-  if (ros->status != TS_STEP_COMPLETE && !ts->reason) ts->reason = TS_DIVERGED_STEP_REJECTED;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSInterpolate_RosW"
 static PetscErrorCode TSInterpolate_RosW(TS ts,PetscReal itime,Vec U)
 {
   TS_RosW         *ros = (TS_RosW*)ts->data;
@@ -1120,7 +1095,7 @@ static PetscErrorCode TSInterpolate_RosW(TS ts,PetscReal itime,Vec U)
     t = (itime - ts->ptime)/h;
     break;
   case TS_STEP_COMPLETE:
-    h = ts->time_step_prev;
+    h = ts->ptime - ts->ptime_prev;
     t = (itime - ts->ptime)/h + 1; /* In the interval [0,1] */
     break;
   default: SETERRQ(PetscObjectComm((PetscObject)ts),PETSC_ERR_PLIB,"Invalid TSStepStatus");
@@ -1134,65 +1109,53 @@ static PetscErrorCode TSInterpolate_RosW(TS ts,PetscReal itime,Vec U)
   }
 
   /* y(t+tt*h) = y(t) + Sum bt(tt) * GammaInv * Ydot */
-  /*U<-0*/
+  /* U <- 0*/
   ierr = VecZeroEntries(U);CHKERRQ(ierr);
-
-  /*U<- Sum bt_i * GammaInv(i,1:i) * Y(1:i) */
-  for (j=0; j<s; j++) w[j]=0;
+  /* U <- Sum bt_i * GammaInv(i,1:i) * Y(1:i) */
+  for (j=0; j<s; j++) w[j] = 0;
   for (j=0; j<s; j++) {
     for (i=j; i<s; i++) {
       w[j] +=  bt[i]*GammaInv[i*s+j];
     }
   }
   ierr = VecMAXPY(U,i,w,Y);CHKERRQ(ierr);
-
-  /*X<-y(t) + X*/
-  ierr = VecAXPY(U,1.0,ros->VecSolPrev);CHKERRQ(ierr);
+  /* U <- y(t) + U */
+  ierr = VecAXPY(U,1,ros->vec_sol_prev);CHKERRQ(ierr);
 
   ierr = PetscFree(bt);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 /*------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "TSReset_RosW"
-static PetscErrorCode TSReset_RosW(TS ts)
+
+static PetscErrorCode TSRosWTableauReset(TS ts)
 {
   TS_RosW        *ros = (TS_RosW*)ts->data;
-  PetscInt       s;
+  RosWTableau    tab  = ros->tableau;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (!ros->tableau) PetscFunctionReturn(0);
-  s    = ros->tableau->s;
-  ierr = VecDestroyVecs(s,&ros->Y);CHKERRQ(ierr);
-  ierr = VecDestroy(&ros->Ydot);CHKERRQ(ierr);
-  ierr = VecDestroy(&ros->Ystage);CHKERRQ(ierr);
-  ierr = VecDestroy(&ros->Zdot);CHKERRQ(ierr);
-  ierr = VecDestroy(&ros->Zstage);CHKERRQ(ierr);
-  ierr = VecDestroy(&ros->VecSolPrev);CHKERRQ(ierr);
+  if (!tab) PetscFunctionReturn(0);
+  ierr = VecDestroyVecs(tab->s,&ros->Y);CHKERRQ(ierr);
   ierr = PetscFree(ros->work);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSDestroy_RosW"
-static PetscErrorCode TSDestroy_RosW(TS ts)
+static PetscErrorCode TSReset_RosW(TS ts)
 {
+  TS_RosW        *ros = (TS_RosW*)ts->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = TSReset_RosW(ts);CHKERRQ(ierr);
-  ierr = PetscFree(ts->data);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)ts,"TSRosWGetType_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)ts,"TSRosWSetType_C",NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)ts,"TSRosWSetRecomputeJacobian_C",NULL);CHKERRQ(ierr);
+  ierr = TSRosWTableauReset(ts);CHKERRQ(ierr);
+  ierr = VecDestroy(&ros->Ydot);CHKERRQ(ierr);
+  ierr = VecDestroy(&ros->Ystage);CHKERRQ(ierr);
+  ierr = VecDestroy(&ros->Zdot);CHKERRQ(ierr);
+  ierr = VecDestroy(&ros->Zstage);CHKERRQ(ierr);
+  ierr = VecDestroy(&ros->vec_sol_prev);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-
-#undef __FUNCT__
-#define __FUNCT__ "TSRosWGetVecs"
 static PetscErrorCode TSRosWGetVecs(TS ts,DM dm,Vec *Ydot,Vec *Zdot,Vec *Ystage,Vec *Zstage)
 {
   TS_RosW        *rw = (TS_RosW*)ts->data;
@@ -1223,8 +1186,6 @@ static PetscErrorCode TSRosWGetVecs(TS ts,DM dm,Vec *Ydot,Vec *Zdot,Vec *Ystage,
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRosWRestoreVecs"
 static PetscErrorCode TSRosWRestoreVecs(TS ts,DM dm,Vec *Ydot,Vec *Zdot, Vec *Ystage, Vec *Zstage)
 {
   PetscErrorCode ierr;
@@ -1253,16 +1214,12 @@ static PetscErrorCode TSRosWRestoreVecs(TS ts,DM dm,Vec *Ydot,Vec *Zdot, Vec *Ys
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMCoarsenHook_TSRosW"
 static PetscErrorCode DMCoarsenHook_TSRosW(DM fine,DM coarse,void *ctx)
 {
   PetscFunctionBegin;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMRestrictHook_TSRosW"
 static PetscErrorCode DMRestrictHook_TSRosW(DM fine,Mat restrct,Vec rscale,Mat inject,DM coarse,void *ctx)
 {
   TS             ts = (TS)ctx;
@@ -1287,16 +1244,12 @@ static PetscErrorCode DMRestrictHook_TSRosW(DM fine,Mat restrct,Vec rscale,Mat i
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSubDomainHook_TSRosW"
 static PetscErrorCode DMSubDomainHook_TSRosW(DM fine,DM coarse,void *ctx)
 {
   PetscFunctionBegin;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "DMSubDomainRestrictHook_TSRosW"
 static PetscErrorCode DMSubDomainRestrictHook_TSRosW(DM dm,VecScatter gscat,VecScatter lscat,DM subdm,void *ctx)
 {
   TS             ts = (TS)ctx;
@@ -1329,8 +1282,6 @@ static PetscErrorCode DMSubDomainRestrictHook_TSRosW(DM dm,VecScatter gscat,VecS
   This defines the nonlinear equation that is to be solved with SNES
   G(U) = F[t0+Theta*dt, U, (U-U0)*shift] = 0
 */
-#undef __FUNCT__
-#define __FUNCT__ "SNESTSFormFunction_RosW"
 static PetscErrorCode SNESTSFormFunction_RosW(SNES snes,Vec U,Vec F,TS ts)
 {
   TS_RosW        *ros = (TS_RosW*)ts->data;
@@ -1352,8 +1303,6 @@ static PetscErrorCode SNESTSFormFunction_RosW(SNES snes,Vec U,Vec F,TS ts)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "SNESTSFormJacobian_RosW"
 static PetscErrorCode SNESTSFormJacobian_RosW(SNES snes,Vec U,Mat A,Mat B,TS ts)
 {
   TS_RosW        *ros = (TS_RosW*)ts->data;
@@ -1374,43 +1323,49 @@ static PetscErrorCode SNESTSFormJacobian_RosW(SNES snes,Vec U,Mat A,Mat B,TS ts)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSetUp_RosW"
-static PetscErrorCode TSSetUp_RosW(TS ts)
+static PetscErrorCode TSRosWTableauSetUp(TS ts)
 {
   TS_RosW        *ros = (TS_RosW*)ts->data;
   RosWTableau    tab  = ros->tableau;
-  PetscInt       s    = tab->s;
   PetscErrorCode ierr;
-  DM             dm;
 
   PetscFunctionBegin;
-  if (!ros->tableau) {
-    ierr = TSRosWSetType(ts,TSRosWDefault);CHKERRQ(ierr);
-  }
-  ierr = VecDuplicateVecs(ts->vec_sol,s,&ros->Y);CHKERRQ(ierr);
+  ierr = VecDuplicateVecs(ts->vec_sol,tab->s,&ros->Y);CHKERRQ(ierr);
+  ierr = PetscMalloc1(tab->s,&ros->work);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode TSSetUp_RosW(TS ts)
+{
+  TS_RosW        *ros = (TS_RosW*)ts->data;
+  PetscErrorCode ierr;
+  DM             dm;
+  SNES           snes;
+
+  PetscFunctionBegin;
+  ierr = TSRosWTableauSetUp(ts);CHKERRQ(ierr);
   ierr = VecDuplicate(ts->vec_sol,&ros->Ydot);CHKERRQ(ierr);
   ierr = VecDuplicate(ts->vec_sol,&ros->Ystage);CHKERRQ(ierr);
   ierr = VecDuplicate(ts->vec_sol,&ros->Zdot);CHKERRQ(ierr);
   ierr = VecDuplicate(ts->vec_sol,&ros->Zstage);CHKERRQ(ierr);
-  ierr = VecDuplicate(ts->vec_sol,&ros->VecSolPrev);CHKERRQ(ierr);
-  ierr = PetscMalloc1(s,&ros->work);CHKERRQ(ierr);
+  ierr = VecDuplicate(ts->vec_sol,&ros->vec_sol_prev);CHKERRQ(ierr);
   ierr = TSGetDM(ts,&dm);CHKERRQ(ierr);
-  if (dm) {
-    ierr = DMCoarsenHookAdd(dm,DMCoarsenHook_TSRosW,DMRestrictHook_TSRosW,ts);CHKERRQ(ierr);
-    ierr = DMSubDomainHookAdd(dm,DMSubDomainHook_TSRosW,DMSubDomainRestrictHook_TSRosW,ts);CHKERRQ(ierr);
+  ierr = DMCoarsenHookAdd(dm,DMCoarsenHook_TSRosW,DMRestrictHook_TSRosW,ts);CHKERRQ(ierr);
+  ierr = DMSubDomainHookAdd(dm,DMSubDomainHook_TSRosW,DMSubDomainRestrictHook_TSRosW,ts);CHKERRQ(ierr);
+  /* Rosenbrock methods are linearly implicit, so set that unless the user has specifically asked for something else */
+  ierr = TSGetSNES(ts,&snes);CHKERRQ(ierr);
+  if (!((PetscObject)snes)->type_name) {
+    ierr = SNESSetType(snes,SNESKSPONLY);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 /*------------------------------------------------------------*/
 
-#undef __FUNCT__
-#define __FUNCT__ "TSSetFromOptions_RosW"
-static PetscErrorCode TSSetFromOptions_RosW(PetscOptions *PetscOptionsObject,TS ts)
+static PetscErrorCode TSSetFromOptions_RosW(PetscOptionItems *PetscOptionsObject,TS ts)
 {
   TS_RosW        *ros = (TS_RosW*)ts->data;
   PetscErrorCode ierr;
-  char           rostype[256];
+  SNES           snes;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"RosW ODE solver options");CHKERRQ(ierr);
@@ -1419,67 +1374,39 @@ static PetscErrorCode TSSetFromOptions_RosW(PetscOptions *PetscOptionsObject,TS
     PetscInt        count,choice;
     PetscBool       flg;
     const char      **namelist;
-    SNES            snes;
 
-    ierr = PetscStrncpy(rostype,TSRosWDefault,sizeof(rostype));CHKERRQ(ierr);
     for (link=RosWTableauList,count=0; link; link=link->next,count++) ;
-    ierr = PetscMalloc1(count,&namelist);CHKERRQ(ierr);
+    ierr = PetscMalloc1(count,(char***)&namelist);CHKERRQ(ierr);
     for (link=RosWTableauList,count=0; link; link=link->next,count++) namelist[count] = link->tab.name;
-    ierr = PetscOptionsEList("-ts_rosw_type","Family of Rosenbrock-W method","TSRosWSetType",(const char*const*)namelist,count,rostype,&choice,&flg);CHKERRQ(ierr);
-    ierr = TSRosWSetType(ts,flg ? namelist[choice] : rostype);CHKERRQ(ierr);
+    ierr = PetscOptionsEList("-ts_rosw_type","Family of Rosenbrock-W method","TSRosWSetType",(const char*const*)namelist,count,ros->tableau->name,&choice,&flg);CHKERRQ(ierr);
+    if (flg) {ierr = TSRosWSetType(ts,namelist[choice]);CHKERRQ(ierr);}
     ierr = PetscFree(namelist);CHKERRQ(ierr);
 
     ierr = PetscOptionsBool("-ts_rosw_recompute_jacobian","Recompute the Jacobian at each stage","TSRosWSetRecomputeJacobian",ros->recompute_jacobian,&ros->recompute_jacobian,NULL);CHKERRQ(ierr);
-
-    /* Rosenbrock methods are linearly implicit, so set that unless the user has specifically asked for something else */
-    ierr = TSGetSNES(ts,&snes);CHKERRQ(ierr);
-    if (!((PetscObject)snes)->type_name) {
-      ierr = SNESSetType(snes,SNESKSPONLY);CHKERRQ(ierr);
-    }
-    ierr = SNESSetFromOptions(snes);CHKERRQ(ierr);
   }
   ierr = PetscOptionsTail();CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-#undef __FUNCT__
-#define __FUNCT__ "PetscFormatRealArray"
-static PetscErrorCode PetscFormatRealArray(char buf[],size_t len,const char *fmt,PetscInt n,const PetscReal x[])
-{
-  PetscErrorCode ierr;
-  PetscInt       i;
-  size_t         left,count;
-  char           *p;
-
-  PetscFunctionBegin;
-  for (i=0,p=buf,left=len; i<n; i++) {
-    ierr = PetscSNPrintfCount(p,left,fmt,&count,x[i]);CHKERRQ(ierr);
-    if (count >= left) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Insufficient space in buffer");
-    left -= count;
-    p    += count;
-    *p++  = ' ';
+  /* Rosenbrock methods are linearly implicit, so set that unless the user has specifically asked for something else */
+  ierr = TSGetSNES(ts,&snes);CHKERRQ(ierr);
+  if (!((PetscObject)snes)->type_name) {
+    ierr = SNESSetType(snes,SNESKSPONLY);CHKERRQ(ierr);
   }
-  p[i ? 0 : -1] = 0;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSView_RosW"
 static PetscErrorCode TSView_RosW(TS ts,PetscViewer viewer)
 {
   TS_RosW        *ros = (TS_RosW*)ts->data;
-  RosWTableau    tab  = ros->tableau;
   PetscBool      iascii;
   PetscErrorCode ierr;
-  TSAdapt        adapt;
 
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
-    TSRosWType rostype;
-    PetscInt   i;
-    PetscReal  abscissa[512];
-    char       buf[512];
+    RosWTableau tab  = ros->tableau;
+    TSRosWType  rostype;
+    char        buf[512];
+    PetscInt    i;
+    PetscReal   abscissa[512];
     ierr = TSRosWGetType(ts,&rostype);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  Rosenbrock-W %s\n",rostype);CHKERRQ(ierr);
     ierr = PetscFormatRealArray(buf,sizeof(buf),"% 8.6f",tab->s,tab->ASum);CHKERRQ(ierr);
@@ -1488,23 +1415,18 @@ static PetscErrorCode TSView_RosW(TS ts,PetscViewer viewer)
     ierr = PetscFormatRealArray(buf,sizeof(buf),"% 8.6f",tab->s,abscissa);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"  Abscissa of A+Gamma = %s\n",buf);CHKERRQ(ierr);
   }
-  ierr = TSGetAdapt(ts,&adapt);CHKERRQ(ierr);
-  ierr = TSAdaptView(adapt,viewer);CHKERRQ(ierr);
-  ierr = SNESView(ts->snes,viewer);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSLoad_RosW"
 static PetscErrorCode TSLoad_RosW(TS ts,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   SNES           snes;
-  TSAdapt        tsadapt;
+  TSAdapt        adapt;
 
   PetscFunctionBegin;
-  ierr = TSGetAdapt(ts,&tsadapt);CHKERRQ(ierr);
-  ierr = TSAdaptLoad(tsadapt,viewer);CHKERRQ(ierr);
+  ierr = TSGetAdapt(ts,&adapt);CHKERRQ(ierr);
+  ierr = TSAdaptLoad(adapt,viewer);CHKERRQ(ierr);
   ierr = TSGetSNES(ts,&snes);CHKERRQ(ierr);
   ierr = SNESLoad(snes,viewer);CHKERRQ(ierr);
   /* function and Jacobian context for SNES when used with TS is always ts object */
@@ -1513,8 +1435,6 @@ static PetscErrorCode TSLoad_RosW(TS ts,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRosWSetType"
 /*@C
   TSRosWSetType - Set the type of Rosenbrock-W scheme
 
@@ -1522,24 +1442,23 @@ static PetscErrorCode TSLoad_RosW(TS ts,PetscViewer viewer)
 
   Input Parameter:
 +  ts - timestepping context
--  rostype - type of Rosenbrock-W scheme
+-  roswtype - type of Rosenbrock-W scheme
 
   Level: beginner
 
 .seealso: TSRosWGetType(), TSROSW, TSROSW2M, TSROSW2P, TSROSWRA3PW, TSROSWRA34PW2, TSROSWRODAS3, TSROSWSANDU3, TSROSWASSP3P3S1C, TSROSWLASSP3P4S2C, TSROSWLLSSP3P4S2C, TSROSWARK3
 @*/
-PetscErrorCode TSRosWSetType(TS ts,TSRosWType rostype)
+PetscErrorCode TSRosWSetType(TS ts,TSRosWType roswtype)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ts,TS_CLASSID,1);
-  ierr = PetscTryMethod(ts,"TSRosWSetType_C",(TS,TSRosWType),(ts,rostype));CHKERRQ(ierr);
+  PetscValidCharPointer(roswtype,2);
+  ierr = PetscTryMethod(ts,"TSRosWSetType_C",(TS,TSRosWType),(ts,roswtype));CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRosWGetType"
 /*@C
   TSRosWGetType - Get the type of Rosenbrock-W scheme
 
@@ -1565,8 +1484,6 @@ PetscErrorCode TSRosWGetType(TS ts,TSRosWType *rostype)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRosWSetRecomputeJacobian"
 /*@C
   TSRosWSetRecomputeJacobian - Set whether to recompute the Jacobian at each stage. The default is to update the Jacobian once per step.
 
@@ -1590,22 +1507,16 @@ PetscErrorCode TSRosWSetRecomputeJacobian(TS ts,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRosWGetType_RosW"
-PetscErrorCode  TSRosWGetType_RosW(TS ts,TSRosWType *rostype)
+static PetscErrorCode  TSRosWGetType_RosW(TS ts,TSRosWType *rostype)
 {
   TS_RosW        *ros = (TS_RosW*)ts->data;
-  PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (!ros->tableau) {ierr = TSRosWSetType(ts,TSRosWDefault);CHKERRQ(ierr);}
   *rostype = ros->tableau->name;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRosWSetType_RosW"
-PetscErrorCode  TSRosWSetType_RosW(TS ts,TSRosWType rostype)
+static PetscErrorCode  TSRosWSetType_RosW(TS ts,TSRosWType rostype)
 {
   TS_RosW         *ros = (TS_RosW*)ts->data;
   PetscErrorCode  ierr;
@@ -1620,8 +1531,10 @@ PetscErrorCode  TSRosWSetType_RosW(TS ts,TSRosWType rostype)
   for (link = RosWTableauList; link; link=link->next) {
     ierr = PetscStrcmp(link->tab.name,rostype,&match);CHKERRQ(ierr);
     if (match) {
-      ierr = TSReset_RosW(ts);CHKERRQ(ierr);
+      if (ts->setupcalled) {ierr = TSRosWTableauReset(ts);CHKERRQ(ierr);}
       ros->tableau = &link->tab;
+      if (ts->setupcalled) {ierr = TSRosWTableauSetUp(ts);CHKERRQ(ierr);}
+      ts->default_adapt_type = ros->tableau->bembed ? TSADAPTBASIC : TSADAPTNONE;
       PetscFunctionReturn(0);
     }
   }
@@ -1629,9 +1542,7 @@ PetscErrorCode  TSRosWSetType_RosW(TS ts,TSRosWType rostype)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSRosWSetRecomputeJacobian_RosW"
-PetscErrorCode  TSRosWSetRecomputeJacobian_RosW(TS ts,PetscBool flg)
+static PetscErrorCode  TSRosWSetRecomputeJacobian_RosW(TS ts,PetscBool flg)
 {
   TS_RosW *ros = (TS_RosW*)ts->data;
 
@@ -1640,6 +1551,22 @@ PetscErrorCode  TSRosWSetRecomputeJacobian_RosW(TS ts,PetscBool flg)
   PetscFunctionReturn(0);
 }
 
+static PetscErrorCode TSDestroy_RosW(TS ts)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = TSReset_RosW(ts);CHKERRQ(ierr);
+  if (ts->dm) {
+    ierr = DMCoarsenHookRemove(ts->dm,DMCoarsenHook_TSRosW,DMRestrictHook_TSRosW,ts);CHKERRQ(ierr);
+    ierr = DMSubDomainHookRemove(ts->dm,DMSubDomainHook_TSRosW,DMSubDomainRestrictHook_TSRosW,ts);CHKERRQ(ierr);
+  }
+  ierr = PetscFree(ts->data);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ts,"TSRosWGetType_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ts,"TSRosWSetType_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ts,"TSRosWSetRecomputeJacobian_C",NULL);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
 
 /* ------------------------------------------------------------ */
 /*MC
@@ -1654,7 +1581,7 @@ PetscErrorCode  TSRosWSetRecomputeJacobian_RosW(TS ts,PetscBool flg)
 
   Consider trying TSARKIMEX if the stiff part is strongly nonlinear.
 
-  Developer notes:
+  Developer Notes:
   Rosenbrock-W methods are typically specified for autonomous ODE
 
 $  udot = f(u)
@@ -1703,8 +1630,6 @@ $  g(u_0 + sum_j a_ij y_j + y_i, ydot_i) = 0
 .seealso:  TSCreate(), TS, TSSetType(), TSRosWSetType(), TSRosWRegister(), TSROSWTHETA1, TSROSWTHETA2, TSROSW2M, TSROSW2P, TSROSWRA3PW, TSROSWRA34PW2, TSROSWRODAS3,
            TSROSWSANDU3, TSROSWASSP3P3S1C, TSROSWLASSP3P4S2C, TSROSWLLSSP3P4S2C, TSROSWGRK4T, TSROSWSHAMP4, TSROSWVELDD4, TSROSW4L
 M*/
-#undef __FUNCT__
-#define __FUNCT__ "TSCreate_RosW"
 PETSC_EXTERN PetscErrorCode TSCreate_RosW(TS ts)
 {
   TS_RosW        *ros;
@@ -1726,11 +1651,15 @@ PETSC_EXTERN PetscErrorCode TSCreate_RosW(TS ts)
   ts->ops->snesfunction   = SNESTSFormFunction_RosW;
   ts->ops->snesjacobian   = SNESTSFormJacobian_RosW;
 
+  ts->usessnes = PETSC_TRUE;
+
   ierr = PetscNewLog(ts,&ros);CHKERRQ(ierr);
   ts->data = (void*)ros;
 
   ierr = PetscObjectComposeFunction((PetscObject)ts,"TSRosWGetType_C",TSRosWGetType_RosW);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)ts,"TSRosWSetType_C",TSRosWSetType_RosW);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)ts,"TSRosWSetRecomputeJacobian_C",TSRosWSetRecomputeJacobian_RosW);CHKERRQ(ierr);
+
+  ierr = TSRosWSetType(ts,TSRosWDefault);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
