@@ -14,8 +14,6 @@ struct _n_TSMonitorSPEigCtx {
 };
 
 
-#undef __FUNCT__
-#define __FUNCT__ "TSMonitorSPEigCtxCreate"
 /*@C
    TSMonitorSPEigCtxCreate - Creates a context for use with TS to monitor the eigenvalues of the linearized operator
 
@@ -74,7 +72,7 @@ PetscErrorCode  TSMonitorSPEigCtxCreate(MPI_Comm comm,const char host[],const ch
   (*ctx)->howoften          = howoften;
   (*ctx)->computeexplicitly = PETSC_FALSE;
 
-  ierr = PetscOptionsGetBool(NULL,"-ts_monitor_sp_eig_explicitly",&(*ctx)->computeexplicitly,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetBool(NULL,NULL,"-ts_monitor_sp_eig_explicitly",&(*ctx)->computeexplicitly,NULL);CHKERRQ(ierr);
 
   (*ctx)->comm = comm;
   (*ctx)->xmin = -2.1;
@@ -84,8 +82,6 @@ PetscErrorCode  TSMonitorSPEigCtxCreate(MPI_Comm comm,const char host[],const ch
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSLinearStabilityIndicator"
 static PetscErrorCode TSLinearStabilityIndicator(TS ts, PetscReal xr,PetscReal xi,PetscBool *flg)
 {
   PetscErrorCode ierr;
@@ -98,8 +94,6 @@ static PetscErrorCode TSLinearStabilityIndicator(TS ts, PetscReal xr,PetscReal x
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSMonitorSPEig"
 PetscErrorCode TSMonitorSPEig(TS ts,PetscInt step,PetscReal ptime,Vec v,void *monctx)
 {
   TSMonitorSPEigCtx ctx = (TSMonitorSPEigCtx) monctx;
@@ -113,6 +107,7 @@ PetscErrorCode TSMonitorSPEig(TS ts,PetscInt step,PetscReal ptime,Vec v,void *mo
   SNES              snes;
 
   PetscFunctionBegin;
+  if (step < 0) PetscFunctionReturn(0); /* -1 indicates interpolated solution */
   if (!step) PetscFunctionReturn(0);
   if (((ctx->howoften > 0) && (!(step % ctx->howoften))) || ((ctx->howoften == -1) && ts->reason)) {
     ierr = VecDuplicate(v,&xdot);CHKERRQ(ierr);
@@ -173,21 +168,19 @@ PetscErrorCode TSMonitorSPEig(TS ts,PetscInt step,PetscReal ptime,Vec v,void *mo
       ierr = PetscDrawSetPause(draw,0.0);CHKERRQ(ierr);
       ierr = PetscDrawSPDraw(drawsp,PETSC_TRUE);CHKERRQ(ierr);
       ierr = PetscDrawSetPause(draw,pause);CHKERRQ(ierr);
-
       if (ts->ops->linearstability) {
         ierr = PetscDrawSPGetAxis(drawsp,&axis);CHKERRQ(ierr);
         ierr = PetscDrawAxisGetLimits(axis,&xmin,&xmax,&ymin,&ymax);CHKERRQ(ierr);
         ierr = PetscDrawIndicatorFunction(draw,xmin,xmax,ymin,ymax,PETSC_DRAW_CYAN,(PetscErrorCode (*)(void*,PetscReal,PetscReal,PetscBool*))TSLinearStabilityIndicator,ts);CHKERRQ(ierr);
         ierr = PetscDrawSPDraw(drawsp,PETSC_FALSE);CHKERRQ(ierr);
       }
+      ierr = PetscDrawSPSave(drawsp);CHKERRQ(ierr);
     }
     ierr = MatDestroy(&B);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "TSMonitorSPEigCtxDestroy"
 /*@C
    TSMonitorSPEigCtxDestroy - Destroys a scatter plot context that was created with TSMonitorSPEigCtxCreate().
 
