@@ -14,8 +14,6 @@ typedef struct {
 /*
        All processors have the same data so processor 1 prints it
 */
-#undef __FUNCT__
-#define __FUNCT__ "AOView_Basic"
 PetscErrorCode AOView_Basic(AO ao,PetscViewer viewer)
 {
   PetscErrorCode ierr;
@@ -40,8 +38,6 @@ PetscErrorCode AOView_Basic(AO ao,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AODestroy_Basic"
 PetscErrorCode AODestroy_Basic(AO ao)
 {
   AO_Basic       *aobasic = (AO_Basic*)ao->data;
@@ -53,8 +49,6 @@ PetscErrorCode AODestroy_Basic(AO ao)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOBasicGetIndices_Private"
 PetscErrorCode AOBasicGetIndices_Private(AO ao,PetscInt **app,PetscInt **petsc)
 {
   AO_Basic *basic = (AO_Basic*)ao->data;
@@ -65,8 +59,6 @@ PetscErrorCode AOBasicGetIndices_Private(AO ao,PetscInt **app,PetscInt **petsc)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOPetscToApplication_Basic"
 PetscErrorCode AOPetscToApplication_Basic(AO ao,PetscInt n,PetscInt *ia)
 {
   PetscInt i,N=ao->N;
@@ -83,8 +75,6 @@ PetscErrorCode AOPetscToApplication_Basic(AO ao,PetscInt n,PetscInt *ia)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOApplicationToPetsc_Basic"
 PetscErrorCode AOApplicationToPetsc_Basic(AO ao,PetscInt n,PetscInt *ia)
 {
   PetscInt i,N=ao->N;
@@ -101,8 +91,6 @@ PetscErrorCode AOApplicationToPetsc_Basic(AO ao,PetscInt n,PetscInt *ia)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOPetscToApplicationPermuteInt_Basic"
 PetscErrorCode AOPetscToApplicationPermuteInt_Basic(AO ao, PetscInt block, PetscInt *array)
 {
   AO_Basic       *aobasic = (AO_Basic*) ao->data;
@@ -111,7 +99,7 @@ PetscErrorCode AOPetscToApplicationPermuteInt_Basic(AO ao, PetscInt block, Petsc
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscMalloc(ao->N*block * sizeof(PetscInt), &temp);CHKERRQ(ierr);
+  ierr = PetscMalloc1(ao->N*block, &temp);CHKERRQ(ierr);
   for (i = 0; i < ao->N; i++) {
     for (j = 0; j < block; j++) temp[i*block+j] = array[aobasic->petsc[i]*block+j];
   }
@@ -120,8 +108,6 @@ PetscErrorCode AOPetscToApplicationPermuteInt_Basic(AO ao, PetscInt block, Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOApplicationToPetscPermuteInt_Basic"
 PetscErrorCode AOApplicationToPetscPermuteInt_Basic(AO ao, PetscInt block, PetscInt *array)
 {
   AO_Basic       *aobasic = (AO_Basic*) ao->data;
@@ -130,7 +116,7 @@ PetscErrorCode AOApplicationToPetscPermuteInt_Basic(AO ao, PetscInt block, Petsc
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscMalloc(ao->N*block * sizeof(PetscInt), &temp);CHKERRQ(ierr);
+  ierr = PetscMalloc1(ao->N*block, &temp);CHKERRQ(ierr);
   for (i = 0; i < ao->N; i++) {
     for (j = 0; j < block; j++) temp[i*block+j] = array[aobasic->app[i]*block+j];
   }
@@ -139,8 +125,6 @@ PetscErrorCode AOApplicationToPetscPermuteInt_Basic(AO ao, PetscInt block, Petsc
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOPetscToApplicationPermuteReal_Basic"
 PetscErrorCode AOPetscToApplicationPermuteReal_Basic(AO ao, PetscInt block, PetscReal *array)
 {
   AO_Basic       *aobasic = (AO_Basic*) ao->data;
@@ -149,7 +133,7 @@ PetscErrorCode AOPetscToApplicationPermuteReal_Basic(AO ao, PetscInt block, Pets
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscMalloc(ao->N*block * sizeof(PetscReal), &temp);CHKERRQ(ierr);
+  ierr = PetscMalloc1(ao->N*block, &temp);CHKERRQ(ierr);
   for (i = 0; i < ao->N; i++) {
     for (j = 0; j < block; j++) temp[i*block+j] = array[aobasic->petsc[i]*block+j];
   }
@@ -158,8 +142,6 @@ PetscErrorCode AOPetscToApplicationPermuteReal_Basic(AO ao, PetscInt block, Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOApplicationToPetscPermuteReal_Basic"
 PetscErrorCode AOApplicationToPetscPermuteReal_Basic(AO ao, PetscInt block, PetscReal *array)
 {
   AO_Basic       *aobasic = (AO_Basic*) ao->data;
@@ -168,7 +150,7 @@ PetscErrorCode AOApplicationToPetscPermuteReal_Basic(AO ao, PetscInt block, Pets
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscMalloc(ao->N*block * sizeof(PetscReal), &temp);CHKERRQ(ierr);
+  ierr = PetscMalloc1(ao->N*block, &temp);CHKERRQ(ierr);
   for (i = 0; i < ao->N; i++) {
     for (j = 0; j < block; j++) temp[i*block+j] = array[aobasic->app[i]*block+j];
   }
@@ -188,8 +170,6 @@ static struct _AOOps AOOps_Basic = {
   AOApplicationToPetscPermuteReal_Basic
 };
 
-#undef __FUNCT__
-#define __FUNCT__ "AOCreate_Basic"
 PETSC_EXTERN PetscErrorCode AOCreate_Basic(AO ao)
 {
   AO_Basic       *aobasic;
@@ -202,7 +182,7 @@ PETSC_EXTERN PetscErrorCode AOCreate_Basic(AO ao)
 
   PetscFunctionBegin;
   /* create special struct aobasic */
-  ierr     = PetscNewLog(ao, AO_Basic, &aobasic);CHKERRQ(ierr);
+  ierr     = PetscNewLog(ao,&aobasic);CHKERRQ(ierr);
   ao->data = (void*) aobasic;
   ierr     = PetscMemcpy(ao->ops,&AOOps_Basic,sizeof(struct _AOOps));CHKERRQ(ierr);
   ierr     = PetscObjectChangeTypeName((PetscObject)ao,AOBASIC);CHKERRQ(ierr);
@@ -216,7 +196,7 @@ PETSC_EXTERN PetscErrorCode AOCreate_Basic(AO ao)
   ierr = PetscObjectGetComm((PetscObject)isapp,&comm);CHKERRQ(ierr);
   ierr = MPI_Comm_size(comm, &size);CHKERRQ(ierr);
   ierr = MPI_Comm_rank(comm, &rank);CHKERRQ(ierr);
-  ierr = PetscMalloc2(size,PetscMPIInt, &lens,size,PetscMPIInt,&disp);CHKERRQ(ierr);
+  ierr = PetscMalloc2(size, &lens,size,&disp);CHKERRQ(ierr);
   ierr = MPI_Allgather(&count, 1, MPI_INT, lens, 1, MPI_INT, comm);CHKERRQ(ierr);
   N    =  0;
   for (i = 0; i < size; i++) {
@@ -230,7 +210,7 @@ PETSC_EXTERN PetscErrorCode AOCreate_Basic(AO ao)
   if (napp) {
     if (!ispetsc) {
       start = disp[rank];
-      ierr  = PetscMalloc((napp+1) * sizeof(PetscInt), &petsc);CHKERRQ(ierr);
+      ierr  = PetscMalloc1(napp+1, &petsc);CHKERRQ(ierr);
       for (i=0; i<napp; i++) petsc[i] = start + i;
     } else {
       ierr  = ISGetIndices(ispetsc,&mypetsc);CHKERRQ(ierr);
@@ -239,7 +219,7 @@ PETSC_EXTERN PetscErrorCode AOCreate_Basic(AO ao)
   }
 
   /* get all indices on all processors */
-  ierr = PetscMalloc2(N,PetscInt,&allpetsc,N,PetscInt,&allapp);CHKERRQ(ierr);
+  ierr = PetscMalloc2(N,&allpetsc,N,&allapp);CHKERRQ(ierr);
   ierr = MPI_Allgatherv(petsc, count, MPIU_INT, allpetsc, lens, disp, MPIU_INT, comm);CHKERRQ(ierr);
   ierr = MPI_Allgatherv((void*)myapp, count, MPIU_INT, allapp, lens, disp, MPIU_INT, comm);CHKERRQ(ierr);
   ierr = PetscFree2(lens,disp);CHKERRQ(ierr);
@@ -247,7 +227,7 @@ PETSC_EXTERN PetscErrorCode AOCreate_Basic(AO ao)
 #if defined(PETSC_USE_DEBUG)
   {
     PetscInt *sorted;
-    ierr = PetscMalloc(N*sizeof(PetscInt),&sorted);CHKERRQ(ierr);
+    ierr = PetscMalloc1(N,&sorted);CHKERRQ(ierr);
 
     ierr = PetscMemcpy(sorted,allpetsc,N*sizeof(PetscInt));CHKERRQ(ierr);
     ierr = PetscSortInt(N,sorted);CHKERRQ(ierr);
@@ -266,8 +246,8 @@ PETSC_EXTERN PetscErrorCode AOCreate_Basic(AO ao)
 #endif
 
   /* generate a list of application and PETSc node numbers */
-  ierr = PetscMalloc2(N,PetscInt, &aobasic->app,N,PetscInt,&aobasic->petsc);CHKERRQ(ierr);
-  ierr = PetscLogObjectMemory(ao,2*N*sizeof(PetscInt));CHKERRQ(ierr);
+  ierr = PetscMalloc2(N, &aobasic->app,N,&aobasic->petsc);CHKERRQ(ierr);
+  ierr = PetscLogObjectMemory((PetscObject)ao,2*N*sizeof(PetscInt));CHKERRQ(ierr);
   ierr = PetscMemzero(aobasic->app, N*sizeof(PetscInt));CHKERRQ(ierr);
   ierr = PetscMemzero(aobasic->petsc, N*sizeof(PetscInt));CHKERRQ(ierr);
   for (i = 0; i < N; i++) {
@@ -300,8 +280,6 @@ PETSC_EXTERN PetscErrorCode AOCreate_Basic(AO ao)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOCreateBasic"
 /*@C
    AOCreateBasic - Creates a basic application ordering using two integer arrays.
 
@@ -319,7 +297,8 @@ PETSC_EXTERN PetscErrorCode AOCreate_Basic(AO ao)
 
    Level: beginner
 
-    Notes: the arrays myapp and mypetsc must contain the all the integers 0 to napp-1 with no duplicates; that is there cannot be any "holes"
+    Notes:
+    the arrays myapp and mypetsc must contain the all the integers 0 to napp-1 with no duplicates; that is there cannot be any "holes"
            in the indices. Use AOCreateMapping() or AOCreateMappingIS() if you wish to have "holes" in the indices.
 
 .keywords: AO, create
@@ -347,8 +326,6 @@ PetscErrorCode  AOCreateBasic(MPI_Comm comm,PetscInt napp,const PetscInt myapp[]
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOCreateBasicIS"
 /*@C
    AOCreateBasicIS - Creates a basic application ordering using two index sets.
 
@@ -364,7 +341,8 @@ PetscErrorCode  AOCreateBasic(MPI_Comm comm,PetscInt napp,const PetscInt myapp[]
 
    Level: beginner
 
-    Notes: the index sets isapp and ispetsc must contain the all the integers 0 to napp-1 (where napp is the length of the index sets) with no duplicates;
+    Notes:
+    the index sets isapp and ispetsc must contain the all the integers 0 to napp-1 (where napp is the length of the index sets) with no duplicates;
            that is there cannot be any "holes"
 
 .keywords: AO, create
@@ -382,6 +360,7 @@ PetscErrorCode AOCreateBasicIS(IS isapp,IS ispetsc,AO *aoout)
   ierr   = AOCreate(comm,&ao);CHKERRQ(ierr);
   ierr   = AOSetIS(ao,isapp,ispetsc);CHKERRQ(ierr);
   ierr   = AOSetType(ao,AOBASIC);CHKERRQ(ierr);
+  ierr   = AOViewFromOptions(ao,NULL,"-ao_view");CHKERRQ(ierr);
   *aoout = ao;
   PetscFunctionReturn(0);
 }
