@@ -15,8 +15,6 @@ typedef struct {
   PetscInt *petscPerm;
 } AO_Mapping;
 
-#undef __FUNCT__
-#define __FUNCT__ "AODestroy_Mapping"
 PetscErrorCode AODestroy_Mapping(AO ao)
 {
   AO_Mapping     *aomap = (AO_Mapping*) ao->data;
@@ -28,8 +26,6 @@ PetscErrorCode AODestroy_Mapping(AO ao)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOView_Mapping"
 PetscErrorCode AOView_Mapping(AO ao, PetscViewer viewer)
 {
   AO_Mapping     *aomap = (AO_Mapping*) ao->data;
@@ -41,9 +37,6 @@ PetscErrorCode AOView_Mapping(AO ao, PetscViewer viewer)
   PetscFunctionBegin;
   ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)ao), &rank);CHKERRQ(ierr);
   if (rank) PetscFunctionReturn(0);
-
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_SELF;
-
   ierr = PetscObjectTypeCompare((PetscObject) viewer, PETSCVIEWERASCII, &iascii);CHKERRQ(ierr);
   if (iascii) {
     PetscViewerASCIIPrintf(viewer, "Number of elements in ordering %D\n", aomap->N);
@@ -55,8 +48,6 @@ PetscErrorCode AOView_Mapping(AO ao, PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOPetscToApplication_Mapping"
 PetscErrorCode AOPetscToApplication_Mapping(AO ao, PetscInt n, PetscInt *ia)
 {
   AO_Mapping *aomap = (AO_Mapping*) ao->data;
@@ -92,8 +83,6 @@ PetscErrorCode AOPetscToApplication_Mapping(AO ao, PetscInt n, PetscInt *ia)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOApplicationToPetsc_Mapping"
 PetscErrorCode AOApplicationToPetsc_Mapping(AO ao, PetscInt n, PetscInt *ia)
 {
   AO_Mapping *aomap = (AO_Mapping*) ao->data;
@@ -138,8 +127,6 @@ static struct _AOOps AOps = {AOView_Mapping,
                              NULL,
                              NULL};
 
-#undef __FUNCT__
-#define __FUNCT__ "AOMappingHasApplicationIndex"
 /*@C
   AOMappingHasApplicationIndex - Searches for the supplied application index.
 
@@ -180,8 +167,6 @@ PetscErrorCode  AOMappingHasApplicationIndex(AO ao, PetscInt idex, PetscBool  *h
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOMappingHasPetscIndex"
 /*@
   AOMappingHasPetscIndex - Searches for the supplied petsc index.
 
@@ -222,8 +207,6 @@ PetscErrorCode  AOMappingHasPetscIndex(AO ao, PetscInt idex, PetscBool  *hasInde
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOCreateMapping"
 /*@C
   AOCreateMapping - Creates a basic application mapping using two integer arrays.
 
@@ -241,7 +224,8 @@ PetscErrorCode  AOMappingHasPetscIndex(AO ao, PetscInt idex, PetscBool  *hasInde
 
   Level: beginner
 
-    Notes: the arrays myapp and mypetsc need NOT contain the all the integers 0 to napp-1, that is there CAN be "holes"  in the indices.
+    Notes:
+    the arrays myapp and mypetsc need NOT contain the all the integers 0 to napp-1, that is there CAN be "holes"  in the indices.
        Use AOCreateBasic() or AOCreateBasicIS() if they do not have holes for better performance.
 
 .keywords: AO, create
@@ -343,9 +327,7 @@ PetscErrorCode  AOCreateMapping(MPI_Comm comm,PetscInt napp,const PetscInt myapp
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOCreateMappingIS"
-/*@C
+/*@
   AOCreateMappingIS - Creates a basic application ordering using two index sets.
 
   Input Parameters:
@@ -361,7 +343,8 @@ PetscErrorCode  AOCreateMapping(MPI_Comm comm,PetscInt napp,const PetscInt myapp
 
   Level: beginner
 
-    Notes: the index sets isapp and ispetsc need NOT contain the all the integers 0 to N-1, that is there CAN be "holes"  in the indices.
+    Notes:
+    the index sets isapp and ispetsc need NOT contain the all the integers 0 to N-1, that is there CAN be "holes"  in the indices.
        Use AOCreateBasic() or AOCreateBasicIS() if they do not have holes for better performance.
 
 .keywords: AO, create
