@@ -8,8 +8,6 @@
 PetscClassId  AO_CLASSID;
 PetscLogEvent AO_PetscToApplication, AO_ApplicationToPetsc;
 
-#undef __FUNCT__
-#define __FUNCT__ "AOView"
 /*@C
    AOView - Displays an application ordering.
 
@@ -45,7 +43,9 @@ PetscErrorCode  AOView(AO ao,PetscViewer viewer)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ao,AO_CLASSID,1);
-  if (!viewer) viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)ao));
+  if (!viewer) {
+    ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)ao),&viewer);CHKERRQ(ierr);
+  }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
 
   ierr = PetscObjectPrintClassNamePrefixType((PetscObject)ao,viewer);CHKERRQ(ierr);
@@ -53,8 +53,6 @@ PetscErrorCode  AOView(AO ao,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AODestroy"
 /*@
    AODestroy - Destroys an application ordering.
 
@@ -92,8 +90,9 @@ PetscErrorCode  AODestroy(AO *ao)
 
 #include <../src/vec/is/is/impls/general/general.h>
 /* ---------------------------------------------------------------------*/
-#undef __FUNCT__
-#define __FUNCT__ "AOPetscToApplicationIS"
+
+PETSC_INTERN PetscErrorCode ISSetUp_General(IS);
+
 /*@
    AOPetscToApplicationIS - Maps an index set in the PETSc ordering to
    the application-defined ordering.
@@ -137,11 +136,11 @@ PetscErrorCode  AOPetscToApplicationIS(AO ao,IS is)
   ierr = ISGetLocalSize(is,&n);CHKERRQ(ierr);
   ierr = (*ao->ops->petsctoapplication)(ao,n,ia);CHKERRQ(ierr);
   ierr = ISRestoreIndices(is,(const PetscInt**)&ia);CHKERRQ(ierr);
+  /* updated cached values (sorted, min, max, etc.)*/
+  ierr = ISSetUp_General(is);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOApplicationToPetscIS"
 /*@
    AOApplicationToPetscIS - Maps an index set in the application-defined
    ordering to the PETSc ordering.
@@ -183,11 +182,11 @@ PetscErrorCode  AOApplicationToPetscIS(AO ao,IS is)
   ierr = ISGetLocalSize(is,&n);CHKERRQ(ierr);
   ierr = (*ao->ops->applicationtopetsc)(ao,n,ia);CHKERRQ(ierr);
   ierr = ISRestoreIndices(is,(const PetscInt**)&ia);CHKERRQ(ierr);
+  /* updated cached values (sorted, min, max, etc.)*/
+  ierr = ISSetUp_General(is);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOPetscToApplication"
 /*@
    AOPetscToApplication - Maps a set of integers in the PETSc ordering to
    the application-defined ordering.
@@ -227,8 +226,6 @@ PetscErrorCode  AOPetscToApplication(AO ao,PetscInt n,PetscInt ia[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOApplicationToPetsc"
 /*@
    AOApplicationToPetsc - Maps a set of integers in the application-defined
    ordering to the PETSc ordering.
@@ -268,8 +265,6 @@ PetscErrorCode  AOApplicationToPetsc(AO ao,PetscInt n,PetscInt ia[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOPetscToApplicationPermuteInt"
 /*@
   AOPetscToApplicationPermuteInt - Permutes an array of blocks of integers
   in the PETSc ordering to the application-defined ordering.
@@ -307,8 +302,6 @@ PetscErrorCode  AOPetscToApplicationPermuteInt(AO ao, PetscInt block, PetscInt a
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOApplicationToPetscPermuteInt"
 /*@
   AOApplicationToPetscPermuteInt - Permutes an array of blocks of integers
   in the application-defined ordering to the PETSc ordering.
@@ -347,8 +340,6 @@ PetscErrorCode  AOApplicationToPetscPermuteInt(AO ao, PetscInt block, PetscInt a
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOPetscToApplicationPermuteReal"
 /*@
   AOPetscToApplicationPermuteReal - Permutes an array of blocks of reals
   in the PETSc ordering to the application-defined ordering.
@@ -387,8 +378,6 @@ PetscErrorCode  AOPetscToApplicationPermuteReal(AO ao, PetscInt block, PetscReal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOApplicationToPetscPermuteReal"
 /*@
   AOApplicationToPetscPermuteReal - Permutes an array of blocks of reals
   in the application-defined ordering to the PETSc ordering.
@@ -427,8 +416,6 @@ PetscErrorCode  AOApplicationToPetscPermuteReal(AO ao, PetscInt block, PetscReal
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOSetFromOptions"
 /*@
     AOSetFromOptions - Sets AO options from the options database.
 
@@ -464,8 +451,6 @@ PetscErrorCode AOSetFromOptions(AO ao)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOSetIS"
 /*@
    AOSetIS - Sets the IS associated with the application ordering.
 
@@ -497,19 +482,17 @@ PetscErrorCode AOSetIS(AO ao,IS isapp,IS ispetsc)
     PetscInt napp,npetsc;
     ierr = ISGetLocalSize(isapp,&napp);CHKERRQ(ierr);
     ierr = ISGetLocalSize(ispetsc,&npetsc);CHKERRQ(ierr);
-    if (napp != npetsc) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"napp %D != npetsc %d. Local IS lengths must match",napp,npetsc);
+    if (napp != npetsc) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"napp %D != npetsc %D. Local IS lengths must match",napp,npetsc);
   }
+  if (isapp) {ierr = PetscObjectReference((PetscObject)isapp);CHKERRQ(ierr);}
+  if (ispetsc) {ierr = PetscObjectReference((PetscObject)ispetsc);CHKERRQ(ierr);}
   ierr = ISDestroy(&ao->isapp);CHKERRQ(ierr);
   ierr = ISDestroy(&ao->ispetsc);CHKERRQ(ierr);
   ao->isapp   = isapp;
   ao->ispetsc = ispetsc;
-  if (ao->isapp) {ierr = PetscObjectReference((PetscObject)ao->isapp);CHKERRQ(ierr);}
-  if (ao->ispetsc) {ierr = PetscObjectReference((PetscObject)ao->ispetsc);CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "AOCreate"
 /*@
    AOCreate - Creates an application ordering.
 
