@@ -3,8 +3,6 @@
 PETSC_EXTERN PetscErrorCode AOCreate_Basic(AO ao);
 PETSC_EXTERN PetscErrorCode AOCreate_MemoryScalable(AO ao);
 
-#undef __FUNCT__
-#define __FUNCT__ "AORegisterAll"
 /*@C
   AORegisterAll - Registers all of the application ordering components in the AO package.
 
@@ -20,6 +18,7 @@ PetscErrorCode  AORegisterAll(void)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
+  if (AORegisterAllCalled) PetscFunctionReturn(0);
   AORegisterAllCalled = PETSC_TRUE;
 
   ierr = AORegister(AOBASIC,          AOCreate_Basic);CHKERRQ(ierr);
