@@ -8,34 +8,63 @@
 #include <petscviewer.h>
 
 typedef struct {
-  PetscBool sorted;             /* are the blocks sorted? */
-  PetscBool borrowed_indices;   /* do not free indices when IS is destroyed */
+  PetscBool sorted;      /* are the blocks sorted? */
+  PetscBool allocated;   /* did we allocate the index array ourselves? */
   PetscInt  *idx;
 } IS_Block;
 
-#undef __FUNCT__
-#define __FUNCT__ "ISDestroy_Block"
-PetscErrorCode ISDestroy_Block(IS is)
+static PetscErrorCode ISDestroy_Block(IS is)
 {
-  IS_Block       *is_block = (IS_Block*)is->data;
+  IS_Block       *sub = (IS_Block*)is->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (!is_block->borrowed_indices) {
-    ierr = PetscFree(is_block->idx);CHKERRQ(ierr);
-  }
-  ierr = PetscObjectComposeFunction((PetscObject)is,"ISBlockSetIndices_C",0);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)is,"ISBlockGetIndices_C",0);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)is,"ISBlockRestoreIndices_C",0);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)is,"ISBlockGetSize_C",0);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunction((PetscObject)is,"ISBlockGetLocalSize_C",0);CHKERRQ(ierr);
+  if (sub->allocated) {ierr = PetscFree(sub->idx);CHKERRQ(ierr);}
+  ierr = PetscObjectComposeFunction((PetscObject)is,"ISBlockSetIndices_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)is,"ISBlockGetIndices_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)is,"ISBlockRestoreIndices_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)is,"ISBlockGetSize_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)is,"ISBlockGetLocalSize_C",NULL);CHKERRQ(ierr);
   ierr = PetscFree(is->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetIndices_Block"
-PetscErrorCode ISGetIndices_Block(IS in,const PetscInt *idx[])
+static PetscErrorCode ISLocate_Block(IS is,PetscInt key,PetscInt *location)
+{
+  IS_Block       *sub = (IS_Block*)is->data;
+  PetscInt       numIdx, i, bs, bkey, mkey;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscLayoutGetBlockSize(is->map,&bs);CHKERRQ(ierr);
+  ierr = PetscLayoutGetSize(is->map,&numIdx);CHKERRQ(ierr);
+  numIdx /= bs;
+  bkey    = key / bs;
+  mkey    = key % bs;
+  if (mkey < 0) {
+    bkey--;
+    mkey += bs;
+  }
+  if (sub->sorted) {
+    ierr = PetscFindInt(bkey,numIdx,sub->idx,location);CHKERRQ(ierr);
+  } else {
+    const PetscInt *idx = sub->idx;
+
+    *location = -1;
+    for (i = 0; i < numIdx; i++) {
+      if (idx[i] == bkey) {
+        *location = i;
+        break;
+      }
+    }
+  }
+  if (*location >= 0) {
+    *location = *location * bs + mkey;
+  }
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode ISGetIndices_Block(IS in,const PetscInt *idx[])
 {
   IS_Block       *sub = (IS_Block*)in->data;
   PetscErrorCode ierr;
@@ -58,9 +87,7 @@ PetscErrorCode ISGetIndices_Block(IS in,const PetscInt *idx[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISRestoreIndices_Block"
-PetscErrorCode ISRestoreIndices_Block(IS is,const PetscInt *idx[])
+static PetscErrorCode ISRestoreIndices_Block(IS is,const PetscInt *idx[])
 {
   IS_Block       *sub = (IS_Block*)is->data;
   PetscInt       bs;
@@ -76,9 +103,7 @@ PetscErrorCode ISRestoreIndices_Block(IS is,const PetscInt *idx[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetSize_Block"
-PetscErrorCode ISGetSize_Block(IS is,PetscInt *size)
+static PetscErrorCode ISGetSize_Block(IS is,PetscInt *size)
 {
   PetscErrorCode ierr;
 
@@ -87,9 +112,7 @@ PetscErrorCode ISGetSize_Block(IS is,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetLocalSize_Block"
-PetscErrorCode ISGetLocalSize_Block(IS is,PetscInt *size)
+static PetscErrorCode ISGetLocalSize_Block(IS is,PetscInt *size)
 {
   PetscErrorCode ierr;
 
@@ -98,9 +121,7 @@ PetscErrorCode ISGetLocalSize_Block(IS is,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISInvertPermutation_Block"
-PetscErrorCode ISInvertPermutation_Block(IS is,PetscInt nlocal,IS *isout)
+static PetscErrorCode ISInvertPermutation_Block(IS is,PetscInt nlocal,IS *isout)
 {
   IS_Block       *sub = (IS_Block*)is->data;
   PetscInt       i,*ii,bs,n,*idx = sub->idx;
@@ -121,9 +142,7 @@ PetscErrorCode ISInvertPermutation_Block(IS is,PetscInt nlocal,IS *isout)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISView_Block"
-PetscErrorCode ISView_Block(IS is, PetscViewer viewer)
+static PetscErrorCode ISView_Block(IS is, PetscViewer viewer)
 {
   IS_Block       *sub = (IS_Block*)is->data;
   PetscErrorCode ierr;
@@ -136,25 +155,42 @@ PetscErrorCode ISView_Block(IS is, PetscViewer viewer)
   n   /= bs;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
-    ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_TRUE);CHKERRQ(ierr);
-    if (is->isperm) {
-      ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Block Index set is permutation\n");CHKERRQ(ierr);
+    PetscViewerFormat fmt;
+
+    ierr = PetscViewerGetFormat(viewer,&fmt);CHKERRQ(ierr);
+    if (fmt == PETSC_VIEWER_ASCII_MATLAB) {
+      IS             ist;
+      const char     *name;
+      const PetscInt *idx;
+      PetscInt       n;
+
+      ierr = PetscObjectGetName((PetscObject)is,&name);CHKERRQ(ierr);
+      ierr = ISGetLocalSize(is,&n);CHKERRQ(ierr);
+      ierr = ISGetIndices(is,&idx);CHKERRQ(ierr);
+      ierr = ISCreateGeneral(PetscObjectComm((PetscObject)is),n,idx,PETSC_USE_POINTER,&ist);CHKERRQ(ierr);
+      ierr = PetscObjectSetName((PetscObject)ist,name);CHKERRQ(ierr);
+      ierr = ISView(ist,viewer);CHKERRQ(ierr);
+      ierr = ISDestroy(&ist);CHKERRQ(ierr);
+      ierr = ISRestoreIndices(is,&idx);CHKERRQ(ierr);
+    } else {
+      ierr = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
+      if (is->isperm) {
+        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Block Index set is permutation\n");CHKERRQ(ierr);
+      }
+      ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Block size %D\n",bs);CHKERRQ(ierr);
+      ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Number of block indices in set %D\n",n);CHKERRQ(ierr);
+      ierr = PetscViewerASCIISynchronizedPrintf(viewer,"The first indices of each block are\n");CHKERRQ(ierr);
+      for (i=0; i<n; i++) {
+        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Block %D Index %D\n",i,idx[i]);CHKERRQ(ierr);
+      }
+      ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPopSynchronized(viewer);CHKERRQ(ierr);
     }
-    ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Block size %D\n",bs);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Number of block indices in set %D\n",n);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISynchronizedPrintf(viewer,"The first indices of each block are\n");CHKERRQ(ierr);
-    for (i=0; i<n; i++) {
-      ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Block %D Index %D\n",i,idx[i]);CHKERRQ(ierr);
-    }
-    ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_FALSE);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSort_Block"
-PetscErrorCode ISSort_Block(IS is)
+static PetscErrorCode ISSort_Block(IS is)
 {
   IS_Block       *sub = (IS_Block*)is->data;
   PetscInt       bs, n;
@@ -169,28 +205,29 @@ PetscErrorCode ISSort_Block(IS is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSortRemoveDups_Block"
-PetscErrorCode ISSortRemoveDups_Block(IS is)
+static PetscErrorCode ISSortRemoveDups_Block(IS is)
 {
   IS_Block       *sub = (IS_Block*)is->data;
   PetscInt       bs, n, nb;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (sub->sorted) PetscFunctionReturn(0);
   ierr = PetscLayoutGetBlockSize(is->map, &bs);CHKERRQ(ierr);
   ierr = PetscLayoutGetLocalSize(is->map, &n);CHKERRQ(ierr);
   nb   = n/bs;
-  ierr = PetscSortRemoveDupsInt(&nb,sub->idx);CHKERRQ(ierr);
+  if (sub->sorted) {
+    ierr = PetscSortedRemoveDupsInt(&nb,sub->idx);CHKERRQ(ierr);
+  } else {
+    ierr = PetscSortRemoveDupsInt(&nb,sub->idx);CHKERRQ(ierr);
+  }
   ierr = PetscLayoutSetLocalSize(is->map, nb*bs);CHKERRQ(ierr);
+  ierr = PetscLayoutSetSize(is->map, PETSC_DECIDE);CHKERRQ(ierr);
+  ierr = PetscLayoutSetUp(is->map);CHKERRQ(ierr);
   sub->sorted = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSorted_Block"
-PetscErrorCode ISSorted_Block(IS is,PetscBool  *flg)
+static PetscErrorCode ISSorted_Block(IS is,PetscBool  *flg)
 {
   IS_Block *sub = (IS_Block*)is->data;
 
@@ -199,9 +236,7 @@ PetscErrorCode ISSorted_Block(IS is,PetscBool  *flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISDuplicate_Block"
-PetscErrorCode ISDuplicate_Block(IS is,IS *newIS)
+static PetscErrorCode ISDuplicate_Block(IS is,IS *newIS)
 {
   PetscErrorCode ierr;
   IS_Block       *sub = (IS_Block*)is->data;
@@ -215,9 +250,7 @@ PetscErrorCode ISDuplicate_Block(IS is,IS *newIS)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISIdentity_Block"
-PetscErrorCode ISIdentity_Block(IS is,PetscBool  *ident)
+static PetscErrorCode ISIdentity_Block(IS is,PetscBool  *ident)
 {
   IS_Block      *is_block = (IS_Block*)is->data;
   PetscInt       i,bs,n,*idx = is_block->idx;
@@ -239,8 +272,6 @@ PetscErrorCode ISIdentity_Block(IS is,PetscBool  *ident)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISCopy_Block"
 static PetscErrorCode ISCopy_Block(IS is,IS isy)
 {
   IS_Block       *is_block = (IS_Block*)is->data,*isy_block = (IS_Block*)isy->data;
@@ -260,8 +291,6 @@ static PetscErrorCode ISCopy_Block(IS is,IS isy)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISOnComm_Block"
 static PetscErrorCode ISOnComm_Block(IS is,MPI_Comm comm,PetscCopyMode mode,IS *newis)
 {
   PetscErrorCode ierr;
@@ -276,8 +305,6 @@ static PetscErrorCode ISOnComm_Block(IS is,MPI_Comm comm,PetscCopyMode mode,IS *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSetBlockSize_Block"
 static PetscErrorCode ISSetBlockSize_Block(IS is,PetscInt bs)
 {
   PetscErrorCode ierr;
@@ -287,8 +314,6 @@ static PetscErrorCode ISSetBlockSize_Block(IS is,PetscInt bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISToGeneral_Block"
 static PetscErrorCode ISToGeneral_Block(IS inis)
 {
   IS_Block       *sub   = (IS_Block*)inis->data;
@@ -301,8 +326,8 @@ static PetscErrorCode ISToGeneral_Block(IS inis)
   ierr = ISGetLocalSize(inis,&n);CHKERRQ(ierr);
   ierr = ISGetIndices(inis,&idx);CHKERRQ(ierr);
   if (bs == 1) {
-    PetscCopyMode mode = sub->borrowed_indices ? PETSC_USE_POINTER : PETSC_OWN_POINTER;
-    sub->borrowed_indices = PETSC_TRUE; /* prevent deallocation when changing the subtype*/
+    PetscCopyMode mode = sub->allocated ? PETSC_OWN_POINTER : PETSC_USE_POINTER;
+    sub->allocated = PETSC_FALSE; /* prevent deallocation when changing the subtype*/
     ierr = ISSetType(inis,ISGENERAL);CHKERRQ(ierr);
     ierr = ISGeneralSetIndices(inis,n,idx,mode);CHKERRQ(ierr);
   } else {
@@ -330,10 +355,9 @@ static struct _ISOps myops = { ISGetSize_Block,
                                ISToGeneral_Block,
                                ISOnComm_Block,
                                ISSetBlockSize_Block,
-                               0};
+                               0,
+                               ISLocate_Block};
 
-#undef __FUNCT__
-#define __FUNCT__ "ISBlockSetIndices"
 /*@
    ISBlockSetIndices - The indices are relative to entries, not blocks.
 
@@ -374,56 +398,57 @@ PetscErrorCode  ISBlockSetIndices(IS is,PetscInt bs,PetscInt n,const PetscInt id
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISBlockSetIndices_Block"
-PetscErrorCode  ISBlockSetIndices_Block(IS is,PetscInt bs,PetscInt n,const PetscInt idx[],PetscCopyMode mode)
+static PetscErrorCode  ISBlockSetIndices_Block(IS is,PetscInt bs,PetscInt n,const PetscInt idx[],PetscCopyMode mode)
 {
   PetscErrorCode ierr;
   PetscInt       i,min,max;
-  IS_Block       *sub   = (IS_Block*)is->data;
-  PetscBool      sorted = PETSC_TRUE;
+  IS_Block       *sub = (IS_Block*)is->data;
 
   PetscFunctionBegin;
-  if (!sub->borrowed_indices) {
-    ierr = PetscFree(sub->idx);CHKERRQ(ierr);
-  } else {
-    sub->borrowed_indices = PETSC_FALSE;
-  }
+  if (bs < 1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"block size < 1");
+  if (n < 0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"length < 0");
+  if (n) PetscValidIntPointer(idx,3);
+
   ierr = PetscLayoutSetLocalSize(is->map, n*bs);CHKERRQ(ierr);
   ierr = PetscLayoutSetBlockSize(is->map, bs);CHKERRQ(ierr);
   ierr = PetscLayoutSetUp(is->map);CHKERRQ(ierr);
-  for (i=1; i<n; i++) {
-    if (idx[i] < idx[i-1]) {sorted = PETSC_FALSE; break;}
-  }
-  if (n) min = max = idx[0];
-  else   min = max = 0;
-  for (i=1; i<n; i++) {
-    if (idx[i] < min) min = idx[i];
-    if (idx[i] > max) max = idx[i];
-  }
+
+  if (sub->allocated) {ierr = PetscFree(sub->idx);CHKERRQ(ierr);}
   if (mode == PETSC_COPY_VALUES) {
     ierr = PetscMalloc1(n,&sub->idx);CHKERRQ(ierr);
     ierr = PetscLogObjectMemory((PetscObject)is,n*sizeof(PetscInt));CHKERRQ(ierr);
     ierr = PetscMemcpy(sub->idx,idx,n*sizeof(PetscInt));CHKERRQ(ierr);
+    sub->allocated = PETSC_TRUE;
   } else if (mode == PETSC_OWN_POINTER) {
     sub->idx = (PetscInt*) idx;
     ierr = PetscLogObjectMemory((PetscObject)is,n*sizeof(PetscInt));CHKERRQ(ierr);
+    sub->allocated = PETSC_TRUE;
   } else if (mode == PETSC_USE_POINTER) {
     sub->idx = (PetscInt*) idx;
-    sub->borrowed_indices = PETSC_TRUE;
+    sub->allocated = PETSC_FALSE;
   }
 
-  sub->sorted = sorted;
-  is->min     = bs*min;
-  is->max     = bs*max+bs-1;
-  is->data    = (void*)sub;
-  ierr = PetscMemcpy(is->ops,&myops,sizeof(myops));CHKERRQ(ierr);
-  is->isperm  = PETSC_FALSE;
+  sub->sorted = PETSC_TRUE;
+  for (i=1; i<n; i++) {
+    if (idx[i] < idx[i-1]) {sub->sorted = PETSC_FALSE; break;}
+  }
+  if (n) {
+    min = max = idx[0];
+    for (i=1; i<n; i++) {
+      if (idx[i] < min) min = idx[i];
+      if (idx[i] > max) max = idx[i];
+    }
+    is->min = bs*min;
+    is->max = bs*max+bs-1;
+  } else {
+    is->min = PETSC_MAX_INT;
+    is->max = PETSC_MIN_INT;
+  }
+  is->isperm     = PETSC_FALSE;
+  is->isidentity = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISCreateBlock"
 /*@
    ISCreateBlock - Creates a data structure for an index set containing
    a list of integers. The indices are relative to entries, not blocks.
@@ -464,6 +489,7 @@ PetscErrorCode  ISCreateBlock(MPI_Comm comm,PetscInt bs,PetscInt n,const PetscIn
 
   PetscFunctionBegin;
   PetscValidPointer(is,5);
+  if (bs < 1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"block size < 1");
   if (n < 0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"length < 0");
   if (n) PetscValidIntPointer(idx,4);
 
@@ -473,9 +499,7 @@ PetscErrorCode  ISCreateBlock(MPI_Comm comm,PetscInt bs,PetscInt n,const PetscIn
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISBlockGetIndices_Block"
-PetscErrorCode  ISBlockGetIndices_Block(IS is,const PetscInt *idx[])
+static PetscErrorCode  ISBlockGetIndices_Block(IS is,const PetscInt *idx[])
 {
   IS_Block *sub = (IS_Block*)is->data;
 
@@ -484,16 +508,12 @@ PetscErrorCode  ISBlockGetIndices_Block(IS is,const PetscInt *idx[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISBlockRestoreIndices_Block"
-PetscErrorCode  ISBlockRestoreIndices_Block(IS is,const PetscInt *idx[])
+static PetscErrorCode  ISBlockRestoreIndices_Block(IS is,const PetscInt *idx[])
 {
   PetscFunctionBegin;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISBlockGetIndices"
 /*@C
    ISBlockGetIndices - Gets the indices associated with each block.
 
@@ -522,8 +542,6 @@ PetscErrorCode  ISBlockGetIndices(IS is,const PetscInt *idx[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISBlockRestoreIndices"
 /*@C
    ISBlockRestoreIndices - Restores the indices associated with each block.
 
@@ -552,8 +570,6 @@ PetscErrorCode  ISBlockRestoreIndices(IS is,const PetscInt *idx[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISBlockGetLocalSize"
 /*@
    ISBlockGetLocalSize - Returns the local number of blocks in the index set.
 
@@ -581,9 +597,7 @@ PetscErrorCode  ISBlockGetLocalSize(IS is,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISBlockGetLocalSize_Block"
-PetscErrorCode  ISBlockGetLocalSize_Block(IS is,PetscInt *size)
+static PetscErrorCode  ISBlockGetLocalSize_Block(IS is,PetscInt *size)
 {
   PetscInt       bs, n;
   PetscErrorCode ierr;
@@ -595,8 +609,6 @@ PetscErrorCode  ISBlockGetLocalSize_Block(IS is,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISBlockGetSize"
 /*@
    ISBlockGetSize - Returns the global number of blocks in the index set.
 
@@ -624,9 +636,7 @@ PetscErrorCode  ISBlockGetSize(IS is,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISBlockGetSize_Block"
-PetscErrorCode  ISBlockGetSize_Block(IS is,PetscInt *size)
+static PetscErrorCode  ISBlockGetSize_Block(IS is,PetscInt *size)
 {
   PetscInt       bs, N;
   PetscErrorCode ierr;
@@ -638,8 +648,6 @@ PetscErrorCode  ISBlockGetSize_Block(IS is,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISCreate_Block"
 PETSC_EXTERN PetscErrorCode ISCreate_Block(IS is)
 {
   PetscErrorCode ierr;
@@ -647,7 +655,8 @@ PETSC_EXTERN PetscErrorCode ISCreate_Block(IS is)
 
   PetscFunctionBegin;
   ierr = PetscNewLog(is,&sub);CHKERRQ(ierr);
-  is->data = sub;
+  is->data = (void *) sub;
+  ierr = PetscMemcpy(is->ops,&myops,sizeof(myops));CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)is,"ISBlockSetIndices_C",ISBlockSetIndices_Block);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)is,"ISBlockGetIndices_C",ISBlockGetIndices_Block);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)is,"ISBlockRestoreIndices_C",ISBlockRestoreIndices_Block);CHKERRQ(ierr);
