@@ -7,9 +7,7 @@
 #include <petscviewer.h>
 #include <petscviewerhdf5.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "ISDuplicate_General"
-PetscErrorCode ISDuplicate_General(IS is,IS *newIS)
+static PetscErrorCode ISDuplicate_General(IS is,IS *newIS)
 {
   PetscErrorCode ierr;
   IS_General     *sub = (IS_General*)is->data;
@@ -21,23 +19,19 @@ PetscErrorCode ISDuplicate_General(IS is,IS *newIS)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISDestroy_General"
-PetscErrorCode ISDestroy_General(IS is)
+static PetscErrorCode ISDestroy_General(IS is)
 {
   IS_General     *is_general = (IS_General*)is->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   if (is_general->allocated) {ierr = PetscFree(is_general->idx);CHKERRQ(ierr);}
-  ierr = PetscObjectComposeFunction((PetscObject)is,"ISGeneralSetIndices_C",0);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)is,"ISGeneralSetIndices_C",NULL);CHKERRQ(ierr);
   ierr = PetscFree(is->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISIdentity_General"
-PetscErrorCode ISIdentity_General(IS is, PetscBool *ident)
+static PetscErrorCode ISIdentity_General(IS is, PetscBool *ident)
 {
   IS_General *is_general = (IS_General*)is->data;
   PetscInt   i,n,*idx = is_general->idx;
@@ -57,8 +51,6 @@ PetscErrorCode ISIdentity_General(IS is, PetscBool *ident)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISCopy_General"
 static PetscErrorCode ISCopy_General(IS is,IS isy)
 {
   IS_General     *is_general = (IS_General*)is->data,*isy_general = (IS_General*)isy->data;
@@ -76,9 +68,7 @@ static PetscErrorCode ISCopy_General(IS is,IS isy)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISOnComm_General"
-PetscErrorCode ISOnComm_General(IS is,MPI_Comm comm,PetscCopyMode mode,IS *newis)
+static PetscErrorCode ISOnComm_General(IS is,MPI_Comm comm,PetscCopyMode mode,IS *newis)
 {
   PetscErrorCode ierr;
   IS_General     *sub = (IS_General*)is->data;
@@ -91,34 +81,15 @@ PetscErrorCode ISOnComm_General(IS is,MPI_Comm comm,PetscCopyMode mode,IS *newis
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSetBlockSize_General"
 static PetscErrorCode ISSetBlockSize_General(IS is,PetscInt bs)
 {
-#if defined(PETSC_USE_DEBUG)
-  IS_General    *sub = (IS_General*)is->data;
-  PetscInt       n;
-#endif
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   ierr = PetscLayoutSetBlockSize(is->map, bs);CHKERRQ(ierr);
-#if defined(PETSC_USE_DEBUG)
-  ierr = PetscLayoutGetLocalSize(is->map, &n);CHKERRQ(ierr);
-  {
-    PetscInt i,j;
-    for (i=0; i<n; i+=bs) {
-      for (j=0; j<bs; j++) {
-        if (sub->idx[i+j] != sub->idx[i]+j) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"Index set does not have block structure, cannot set block size to %D",bs);
-      }
-    }
-  }
-#endif
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISContiguousLocal_General"
 static PetscErrorCode ISContiguousLocal_General(IS is,PetscInt gstart,PetscInt gend,PetscInt *start,PetscBool *contig)
 {
   IS_General *sub = (IS_General*)is->data;
@@ -144,9 +115,30 @@ nomatch:
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetIndices_General"
-PetscErrorCode ISGetIndices_General(IS in,const PetscInt *idx[])
+static PetscErrorCode ISLocate_General(IS is,PetscInt key,PetscInt *location)
+{
+  IS_General     *sub = (IS_General*)is->data;
+  PetscInt       numIdx, i;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscLayoutGetLocalSize(is->map,&numIdx);CHKERRQ(ierr);
+  if (sub->sorted) {ierr =  PetscFindInt(key,numIdx,sub->idx,location);CHKERRQ(ierr);}
+  else {
+    const PetscInt *idx = sub->idx;
+
+    *location = -1;
+    for (i = 0; i < numIdx; i++) {
+      if (idx[i] == key) {
+        *location = i;
+        PetscFunctionReturn(0);
+      }
+    }
+  }
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode ISGetIndices_General(IS in,const PetscInt *idx[])
 {
   IS_General *sub = (IS_General*)in->data;
 
@@ -155,9 +147,7 @@ PetscErrorCode ISGetIndices_General(IS in,const PetscInt *idx[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISRestoreIndices_General"
-PetscErrorCode ISRestoreIndices_General(IS in,const PetscInt *idx[])
+static PetscErrorCode ISRestoreIndices_General(IS in,const PetscInt *idx[])
 {
   IS_General *sub = (IS_General*)in->data;
 
@@ -166,9 +156,7 @@ PetscErrorCode ISRestoreIndices_General(IS in,const PetscInt *idx[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetSize_General"
-PetscErrorCode ISGetSize_General(IS is,PetscInt *size)
+static PetscErrorCode ISGetSize_General(IS is,PetscInt *size)
 {
   PetscErrorCode ierr;
 
@@ -177,9 +165,7 @@ PetscErrorCode ISGetSize_General(IS is,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetLocalSize_General"
-PetscErrorCode ISGetLocalSize_General(IS is,PetscInt *size)
+static PetscErrorCode ISGetLocalSize_General(IS is,PetscInt *size)
 {
   PetscErrorCode ierr;
 
@@ -188,9 +174,7 @@ PetscErrorCode ISGetLocalSize_General(IS is,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISInvertPermutation_General"
-PetscErrorCode ISInvertPermutation_General(IS is,PetscInt nlocal,IS *isout)
+static PetscErrorCode ISInvertPermutation_General(IS is,PetscInt nlocal,IS *isout)
 {
   IS_General     *sub = (IS_General*)is->data;
   PetscInt       i,*ii,n,nstart;
@@ -209,12 +193,12 @@ PetscErrorCode ISInvertPermutation_General(IS is,PetscInt nlocal,IS *isout)
     ierr = ISSetPermutation(*isout);CHKERRQ(ierr);
   } else {
     /* crude, nonscalable get entire IS on each processor */
-    if (nlocal == PETSC_DECIDE) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Do not yet support nlocal of PETSC_DECIDE");
     ierr = ISAllGather(is,&istmp);CHKERRQ(ierr);
     ierr = ISSetPermutation(istmp);CHKERRQ(ierr);
     ierr = ISInvertPermutation(istmp,PETSC_DECIDE,&nistmp);CHKERRQ(ierr);
     ierr = ISDestroy(&istmp);CHKERRQ(ierr);
     /* get the part we need */
+    if (nlocal == PETSC_DECIDE) nlocal = n;
     ierr = MPI_Scan(&nlocal,&nstart,1,MPIU_INT,MPI_SUM,PetscObjectComm((PetscObject)is));CHKERRQ(ierr);
 #if defined(PETSC_USE_DEBUG)
     {
@@ -237,9 +221,7 @@ PetscErrorCode ISInvertPermutation_General(IS is,PetscInt nlocal,IS *isout)
 }
 
 #if defined(PETSC_HAVE_HDF5)
-#undef __FUNCT__
-#define __FUNCT__ "ISView_General_HDF5"
-PetscErrorCode ISView_General_HDF5(IS is, PetscViewer viewer)
+static PetscErrorCode ISView_General_HDF5(IS is, PetscViewer viewer)
 {
   hid_t           filespace;  /* file dataspace identifier */
   hid_t           chunkspace; /* chunk dataset property identifier */
@@ -364,10 +346,10 @@ PetscErrorCode ISView_General_HDF5(IS is, PetscViewer viewer)
 #endif
   /* To write dataset independently use H5Pset_dxpl_mpio(plist_id, H5FD_MPIO_INDEPENDENT) */
 
-  ierr   = ISGetIndices(is, &ind);CHKERRQ(ierr);
+  ierr = ISGetIndices(is, &ind);CHKERRQ(ierr);
   PetscStackCallHDF5(H5Dwrite,(dset_id, inttype, memspace, filespace, plist_id, ind));
   PetscStackCallHDF5(H5Fflush,(file_id, H5F_SCOPE_GLOBAL));
-  ierr   = ISGetIndices(is, &ind);CHKERRQ(ierr);
+  ierr = ISRestoreIndices(is, &ind);CHKERRQ(ierr);
 
   /* Close/release resources */
   if (group != file_id) PetscStackCallHDF5(H5Gclose,(group));
@@ -380,11 +362,10 @@ PetscErrorCode ISView_General_HDF5(IS is, PetscViewer viewer)
 }
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "ISView_General_Binary"
-PetscErrorCode ISView_General_Binary(IS is,PetscViewer viewer)
+static PetscErrorCode ISView_General_Binary(IS is,PetscViewer viewer)
 {
   PetscErrorCode ierr;
+  PetscBool      skipHeader,useMPIIO;
   IS_General     *isa = (IS_General*) is->data;
   PetscMPIInt    rank,size,mesgsize,tag = ((PetscObject)viewer)->tag, mesglen;
   PetscInt       n,N,len,j,tr[2];
@@ -393,18 +374,47 @@ PetscErrorCode ISView_General_Binary(IS is,PetscViewer viewer)
   PetscInt       message_count,flowcontrolcount,*values;
 
   PetscFunctionBegin;
+  /* ierr = ISGetLayout(is,&map);CHKERRQ(ierr); */
   ierr = PetscLayoutGetLocalSize(is->map, &n);CHKERRQ(ierr);
   ierr = PetscLayoutGetSize(is->map, &N);CHKERRQ(ierr);
-  ierr = PetscViewerBinaryGetDescriptor(viewer,&fdes);CHKERRQ(ierr);
-
-  /* determine maximum message to arrive */
-  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)is),&rank);CHKERRQ(ierr);
-  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)is),&size);CHKERRQ(ierr);
 
   tr[0] = IS_FILE_CLASSID;
   tr[1] = N;
-  ierr  = PetscViewerBinaryWrite(viewer,tr,2,PETSC_INT,PETSC_FALSE);CHKERRQ(ierr);
-  ierr  = MPI_Reduce(&n,&len,1,MPIU_INT,MPI_SUM,0,PetscObjectComm((PetscObject)is));CHKERRQ(ierr);
+
+  ierr = PetscViewerBinaryGetSkipHeader(viewer,&skipHeader);CHKERRQ(ierr);
+  if (!skipHeader) {
+    ierr  = PetscViewerBinaryWrite(viewer,tr,2,PETSC_INT,PETSC_FALSE);CHKERRQ(ierr);
+  }
+
+  ierr = PetscViewerBinaryGetUseMPIIO(viewer,&useMPIIO);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_MPIIO)
+  if (useMPIIO) {
+    MPI_File       mfdes;
+    MPI_Offset     off;
+    PetscMPIInt    lsize;
+    PetscInt       rstart;
+    const PetscInt *iarray;
+
+    ierr = PetscMPIIntCast(n,&lsize);CHKERRQ(ierr);
+    ierr = PetscViewerBinaryGetMPIIODescriptor(viewer,&mfdes);CHKERRQ(ierr);
+    ierr = PetscViewerBinaryGetMPIIOOffset(viewer,&off);CHKERRQ(ierr);
+    ierr = PetscLayoutGetRange(is->map,&rstart,NULL);CHKERRQ(ierr);
+    off += rstart*(MPI_Offset)sizeof(PetscInt); /* off is MPI_Offset, not PetscMPIInt */
+    ierr = MPI_File_set_view(mfdes,off,MPIU_INT,MPIU_INT,(char*)"native",MPI_INFO_NULL);CHKERRQ(ierr);
+    ierr = ISGetIndices(is,&iarray);CHKERRQ(ierr);
+    ierr = MPIU_File_write_all(mfdes,(void*)iarray,lsize,MPIU_INT,MPI_STATUS_IGNORE);CHKERRQ(ierr);
+    ierr = ISRestoreIndices(is,&iarray);CHKERRQ(ierr);
+    ierr = PetscViewerBinaryAddMPIIOOffset(viewer,N*(MPI_Offset)sizeof(PetscInt));CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  }
+#endif
+
+  ierr = PetscViewerBinaryGetDescriptor(viewer,&fdes);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)is),&rank);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)is),&size);CHKERRQ(ierr);
+
+  /* determine maximum message to arrive */
+  ierr = MPI_Reduce(&n,&len,1,MPIU_INT,MPI_MAX,0,PetscObjectComm((PetscObject)is));CHKERRQ(ierr);
 
   ierr = PetscViewerFlowControlStart(viewer,&message_count,&flowcontrolcount);CHKERRQ(ierr);
   if (!rank) {
@@ -430,9 +440,7 @@ PetscErrorCode ISView_General_Binary(IS is,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISView_General"
-PetscErrorCode ISView_General(IS is,PetscViewer viewer)
+static PetscErrorCode ISView_General(IS is,PetscViewer viewer)
 {
   IS_General     *sub = (IS_General*)is->data;
   PetscErrorCode ierr;
@@ -445,33 +453,63 @@ PetscErrorCode ISView_General(IS is,PetscViewer viewer)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERBINARY,&isbinary);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERHDF5,&ishdf5);CHKERRQ(ierr);
   if (iascii) {
-    MPI_Comm    comm;
-    PetscMPIInt rank,size;
+    MPI_Comm          comm;
+    PetscMPIInt       rank,size;
+    PetscViewerFormat fmt;
 
     ierr = PetscObjectGetComm((PetscObject)viewer,&comm);CHKERRQ(ierr);
     ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
     ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
 
-    ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_TRUE);CHKERRQ(ierr);
+    ierr = PetscViewerGetFormat(viewer,&fmt);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
     if (size > 1) {
-      if (is->isperm) {
-        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] Index set is permutation\n",rank);CHKERRQ(ierr);
-      }
-      ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] Number of indices in set %D\n",rank,n);CHKERRQ(ierr);
-      for (i=0; i<n; i++) {
-        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] %D %D\n",rank,i,idx[i]);CHKERRQ(ierr);
+      if (fmt == PETSC_VIEWER_ASCII_MATLAB) {
+        const char* name;
+
+        ierr = PetscObjectGetName((PetscObject)is,&name);CHKERRQ(ierr);
+        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"%s_%d = [...\n",name,rank);CHKERRQ(ierr);
+        for (i=0; i<n; i++) {
+          ierr = PetscViewerASCIISynchronizedPrintf(viewer,"%D\n",idx[i]+1);CHKERRQ(ierr);
+        }
+        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"];\n");CHKERRQ(ierr);
+      } else {
+        PetscInt st = 0;
+
+        if (fmt == PETSC_VIEWER_ASCII_INDEX) st = is->map->rstart;
+        if (is->isperm) {
+          ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] Index set is permutation\n",rank);CHKERRQ(ierr);
+        }
+        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] Number of indices in set %D\n",rank,n);CHKERRQ(ierr);
+        for (i=0; i<n; i++) {
+          ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] %D %D\n",rank,i + st,idx[i]);CHKERRQ(ierr);
+        }
       }
     } else {
-      if (is->isperm) {
-        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Index set is permutation\n");CHKERRQ(ierr);
-      }
-      ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Number of indices in set %D\n",n);CHKERRQ(ierr);
-      for (i=0; i<n; i++) {
-        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"%D %D\n",i,idx[i]);CHKERRQ(ierr);
+      if (fmt == PETSC_VIEWER_ASCII_MATLAB) {
+        const char* name;
+
+        ierr = PetscObjectGetName((PetscObject)is,&name);CHKERRQ(ierr);
+        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"%s = [...\n",name);CHKERRQ(ierr);
+        for (i=0; i<n; i++) {
+          ierr = PetscViewerASCIISynchronizedPrintf(viewer,"%D\n",idx[i]+1);CHKERRQ(ierr);
+        }
+        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"];\n");CHKERRQ(ierr);
+      } else {
+        PetscInt st = 0;
+
+        if (fmt == PETSC_VIEWER_ASCII_INDEX) st = is->map->rstart;
+        if (is->isperm) {
+          ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Index set is permutation\n");CHKERRQ(ierr);
+        }
+        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"Number of indices in set %D\n",n);CHKERRQ(ierr);
+        for (i=0; i<n; i++) {
+          ierr = PetscViewerASCIISynchronizedPrintf(viewer,"%D %D\n",i + st,idx[i]);CHKERRQ(ierr);
+        }
       }
     }
     ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_FALSE);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPopSynchronized(viewer);CHKERRQ(ierr);
   } else if (isbinary) {
     ierr = ISView_General_Binary(is,viewer);CHKERRQ(ierr);
   } else if (ishdf5) {
@@ -482,9 +520,7 @@ PetscErrorCode ISView_General(IS is,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSort_General"
-PetscErrorCode ISSort_General(IS is)
+static PetscErrorCode ISSort_General(IS is)
 {
   IS_General     *sub = (IS_General*)is->data;
   PetscInt       n;
@@ -498,26 +534,27 @@ PetscErrorCode ISSort_General(IS is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSortRemoveDups_General"
-PetscErrorCode ISSortRemoveDups_General(IS is)
+static PetscErrorCode ISSortRemoveDups_General(IS is)
 {
   IS_General     *sub = (IS_General*)is->data;
   PetscInt       n;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  if (sub->sorted) PetscFunctionReturn(0);
   ierr = PetscLayoutGetLocalSize(is->map, &n);CHKERRQ(ierr);
-  ierr = PetscSortRemoveDupsInt(&n,sub->idx);CHKERRQ(ierr);
+  if (sub->sorted) {
+    ierr = PetscSortedRemoveDupsInt(&n,sub->idx);CHKERRQ(ierr);
+  } else {
+    ierr = PetscSortRemoveDupsInt(&n,sub->idx);CHKERRQ(ierr);
+  }
   ierr = PetscLayoutSetLocalSize(is->map, n);CHKERRQ(ierr);
+  ierr = PetscLayoutSetSize(is->map, PETSC_DECIDE);CHKERRQ(ierr);
+  ierr = PetscLayoutSetUp(is->map);CHKERRQ(ierr);
   sub->sorted = PETSC_TRUE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSorted_General"
-PetscErrorCode ISSorted_General(IS is,PetscBool  *flg)
+static PetscErrorCode ISSorted_General(IS is,PetscBool  *flg)
 {
   IS_General *sub = (IS_General*)is->data;
 
@@ -526,8 +563,6 @@ PetscErrorCode ISSorted_General(IS is,PetscBool  *flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISToGeneral_General"
 PetscErrorCode  ISToGeneral_General(IS is)
 {
   PetscFunctionBegin;
@@ -551,41 +586,42 @@ static struct _ISOps myops = { ISGetSize_General,
                                ISToGeneral_General,
                                ISOnComm_General,
                                ISSetBlockSize_General,
-                               ISContiguousLocal_General};
+                               ISContiguousLocal_General,
+                               ISLocate_General};
 
-#undef __FUNCT__
-#define __FUNCT__ "ISCreateGeneral_Private"
-PetscErrorCode ISCreateGeneral_Private(IS is)
+PETSC_INTERN PetscErrorCode ISSetUp_General(IS);
+
+PetscErrorCode ISSetUp_General(IS is)
 {
   PetscErrorCode ierr;
-  IS_General     *sub   = (IS_General*)is->data;
-  const PetscInt *idx   = sub->idx;
-  PetscBool      sorted = PETSC_TRUE;
+  IS_General     *sub = (IS_General*)is->data;
+  const PetscInt *idx = sub->idx;
   PetscInt       n,i,min,max;
 
   PetscFunctionBegin;
   ierr = PetscLayoutGetLocalSize(is->map, &n);CHKERRQ(ierr);
-  ierr = PetscLayoutSetUp(is->map);CHKERRQ(ierr);
+
+  sub->sorted = PETSC_TRUE;
   for (i=1; i<n; i++) {
-    if (idx[i] < idx[i-1]) {sorted = PETSC_FALSE; break;}
+    if (idx[i] < idx[i-1]) {sub->sorted = PETSC_FALSE; break;}
   }
-  if (n) min = max = idx[0];
-  else   min = max = 0;
-  for (i=1; i<n; i++) {
-    if (idx[i] < min) min = idx[i];
-    if (idx[i] > max) max = idx[i];
+  if (n) {
+    min = max = idx[0];
+    for (i=1; i<n; i++) {
+      if (idx[i] < min) min = idx[i];
+      if (idx[i] > max) max = idx[i];
+    }
+    is->min = min;
+    is->max = max;
+  } else {
+    is->min = PETSC_MAX_INT;
+    is->max = PETSC_MIN_INT;
   }
-  sub->sorted    = sorted;
-  is->min        = min;
-  is->max        = max;
   is->isperm     = PETSC_FALSE;
   is->isidentity = PETSC_FALSE;
-  ierr = ISViewFromOptions(is,NULL,"-is_view");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISCreateGeneral"
 /*@
    ISCreateGeneral - Creates a data structure for an index set
    containing a list of integers.
@@ -596,7 +632,7 @@ PetscErrorCode ISCreateGeneral_Private(IS is)
 +  comm - the MPI communicator
 .  n - the length of the index set
 .  idx - the list of integers
--  mode - see PetscCopyMode for meaning of this flag.
+-  mode - PETSC_COPY_VALUES, PETSC_OWN_POINTER, or PETSC_USE_POINTER; see PetscCopyMode for meaning of this flag.
 
    Output Parameter:
 .  is - the new index set
@@ -613,7 +649,7 @@ PetscErrorCode ISCreateGeneral_Private(IS is)
   Concepts: index sets^creating
   Concepts: IS^creating
 
-.seealso: ISCreateStride(), ISCreateBlock(), ISAllGather()
+.seealso: ISCreateStride(), ISCreateBlock(), ISAllGather(), PETSC_COPY_VALUES, PETSC_OWN_POINTER, PETSC_USE_POINTER, PetscCopyMode
 @*/
 PetscErrorCode  ISCreateGeneral(MPI_Comm comm,PetscInt n,const PetscInt idx[],PetscCopyMode mode,IS *is)
 {
@@ -626,8 +662,6 @@ PetscErrorCode  ISCreateGeneral(MPI_Comm comm,PetscInt n,const PetscInt idx[],Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGeneralSetIndices"
 /*@
    ISGeneralSetIndices - Sets the indices for an ISGENERAL index set
 
@@ -655,8 +689,6 @@ PetscErrorCode  ISGeneralSetIndices(IS is,PetscInt n,const PetscInt idx[],PetscC
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGeneralSetIndices_General"
 PetscErrorCode  ISGeneralSetIndices_General(IS is,PetscInt n,const PetscInt idx[],PetscCopyMode mode)
 {
   PetscErrorCode ierr;
@@ -666,35 +698,38 @@ PetscErrorCode  ISGeneralSetIndices_General(IS is,PetscInt n,const PetscInt idx[
   if (n < 0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"length < 0");
   if (n) PetscValidIntPointer(idx,3);
 
+  ierr = PetscLayoutSetLocalSize(is->map,n);CHKERRQ(ierr);
+  ierr = PetscLayoutSetUp(is->map);CHKERRQ(ierr);
+
   if (sub->allocated) {ierr = PetscFree(sub->idx);CHKERRQ(ierr);}
-  ierr = PetscLayoutSetLocalSize(is->map, n);CHKERRQ(ierr);
   if (mode == PETSC_COPY_VALUES) {
-    ierr           = PetscMalloc1(n,&sub->idx);CHKERRQ(ierr);
-    ierr           = PetscLogObjectMemory((PetscObject)is,n*sizeof(PetscInt));CHKERRQ(ierr);
-    ierr           = PetscMemcpy(sub->idx,idx,n*sizeof(PetscInt));CHKERRQ(ierr);
+    ierr = PetscMalloc1(n,&sub->idx);CHKERRQ(ierr);
+    ierr = PetscLogObjectMemory((PetscObject)is,n*sizeof(PetscInt));CHKERRQ(ierr);
+    ierr = PetscMemcpy(sub->idx,idx,n*sizeof(PetscInt));CHKERRQ(ierr);
     sub->allocated = PETSC_TRUE;
   } else if (mode == PETSC_OWN_POINTER) {
-    sub->idx       = (PetscInt*)idx;
+    sub->idx = (PetscInt*)idx;
+    ierr = PetscLogObjectMemory((PetscObject)is,n*sizeof(PetscInt));CHKERRQ(ierr);
     sub->allocated = PETSC_TRUE;
   } else {
-    sub->idx       = (PetscInt*)idx;
+    sub->idx = (PetscInt*)idx;
     sub->allocated = PETSC_FALSE;
   }
-  ierr = ISCreateGeneral_Private(is);CHKERRQ(ierr);
+
+  ierr = ISSetUp_General(is);CHKERRQ(ierr);
+  ierr = ISViewFromOptions(is,NULL,"-is_view");CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISCreate_General"
 PETSC_EXTERN PetscErrorCode ISCreate_General(IS is)
 {
   PetscErrorCode ierr;
   IS_General     *sub;
 
   PetscFunctionBegin;
-  ierr = PetscMemcpy(is->ops,&myops,sizeof(myops));CHKERRQ(ierr);
   ierr = PetscNewLog(is,&sub);CHKERRQ(ierr);
   is->data = (void *) sub;
+  ierr = PetscMemcpy(is->ops,&myops,sizeof(myops));CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)is,"ISGeneralSetIndices_C",ISGeneralSetIndices_General);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
