@@ -11,8 +11,6 @@ typedef struct {
   PetscInt N,n,first,step;
 } IS_Stride;
 
-#undef __FUNCT__
-#define __FUNCT__ "ISIdentity_Stride"
 PetscErrorCode ISIdentity_Stride(IS is,PetscBool  *ident)
 {
   IS_Stride *is_stride = (IS_Stride*)is->data;
@@ -27,8 +25,6 @@ PetscErrorCode ISIdentity_Stride(IS is,PetscBool  *ident)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISCopy_Stride"
 static PetscErrorCode ISCopy_Stride(IS is,IS isy)
 {
   IS_Stride      *is_stride = (IS_Stride*)is->data,*isy_stride = (IS_Stride*)isy->data;
@@ -39,8 +35,6 @@ static PetscErrorCode ISCopy_Stride(IS is,IS isy)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISDuplicate_Stride"
 PetscErrorCode ISDuplicate_Stride(IS is,IS *newIS)
 {
   PetscErrorCode ierr;
@@ -51,8 +45,6 @@ PetscErrorCode ISDuplicate_Stride(IS is,IS *newIS)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISInvertPermutation_Stride"
 PetscErrorCode ISInvertPermutation_Stride(IS is,PetscInt nlocal,IS *perm)
 {
   IS_Stride      *isstride = (IS_Stride*)is->data;
@@ -74,8 +66,6 @@ PetscErrorCode ISInvertPermutation_Stride(IS is,PetscInt nlocal,IS *perm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISStrideGetInfo"
 /*@
    ISStrideGetInfo - Returns the first index in a stride index set and
    the stride width.
@@ -119,20 +109,16 @@ PetscErrorCode  ISStrideGetInfo(IS is,PetscInt *first,PetscInt *step)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISDestroy_Stride"
 PetscErrorCode ISDestroy_Stride(IS is)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscObjectComposeFunction((PetscObject)is,"ISStrideSetStride_C",0);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)is,"ISStrideSetStride_C",NULL);CHKERRQ(ierr);
   ierr = PetscFree(is->data);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISToGeneral_Stride"
 PetscErrorCode  ISToGeneral_Stride(IS inis)
 {
   PetscErrorCode ierr;
@@ -147,13 +133,26 @@ PetscErrorCode  ISToGeneral_Stride(IS inis)
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode ISLocate_Stride(IS is,PetscInt key,PetscInt *location)
+{
+  IS_Stride      *sub = (IS_Stride*)is->data;
+  PetscInt       rem, step;
+
+  PetscFunctionBegin;
+  *location = -1;
+  step      = sub->step;
+  key      -= sub->first;
+  rem       = key / step;
+  if ((rem < sub->n) && !(key % step)) {
+    *location = rem;
+  }
+  PetscFunctionReturn(0);
+}
 
 /*
      Returns a legitimate index memory even if
    the stride index set is empty.
 */
-#undef __FUNCT__
-#define __FUNCT__ "ISGetIndices_Stride"
 PetscErrorCode ISGetIndices_Stride(IS in,const PetscInt *idx[])
 {
   IS_Stride      *sub = (IS_Stride*)in->data;
@@ -161,7 +160,7 @@ PetscErrorCode ISGetIndices_Stride(IS in,const PetscInt *idx[])
   PetscInt       i,**dx = (PetscInt**)idx;
 
   PetscFunctionBegin;
-  ierr      = PetscMalloc1(sub->n,idx);CHKERRQ(ierr);
+  ierr      = PetscMalloc1(sub->n,(PetscInt**)idx);CHKERRQ(ierr);
   if (sub->n) {
     (*dx)[0] = sub->first;
     for (i=1; i<sub->n; i++) (*dx)[i] = (*dx)[i-1] + sub->step;
@@ -169,8 +168,6 @@ PetscErrorCode ISGetIndices_Stride(IS in,const PetscInt *idx[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISRestoreIndices_Stride"
 PetscErrorCode ISRestoreIndices_Stride(IS in,const PetscInt *idx[])
 {
   PetscErrorCode ierr;
@@ -180,8 +177,6 @@ PetscErrorCode ISRestoreIndices_Stride(IS in,const PetscInt *idx[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetSize_Stride"
 PetscErrorCode ISGetSize_Stride(IS is,PetscInt *size)
 {
   IS_Stride *sub = (IS_Stride*)is->data;
@@ -191,8 +186,6 @@ PetscErrorCode ISGetSize_Stride(IS is,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetLocalSize_Stride"
 PetscErrorCode ISGetLocalSize_Stride(IS is,PetscInt *size)
 {
   IS_Stride *sub = (IS_Stride*)is->data;
@@ -202,48 +195,63 @@ PetscErrorCode ISGetLocalSize_Stride(IS is,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISView_Stride"
 PetscErrorCode ISView_Stride(IS is,PetscViewer viewer)
 {
-  IS_Stride      *sub = (IS_Stride*)is->data;
-  PetscInt       i,n = sub->n;
-  PetscMPIInt    rank,size;
-  PetscBool      iascii;
-  PetscErrorCode ierr;
+  IS_Stride         *sub = (IS_Stride*)is->data;
+  PetscInt          i,n = sub->n;
+  PetscMPIInt       rank,size;
+  PetscBool         iascii;
+  PetscViewerFormat fmt;
+  PetscErrorCode    ierr;
 
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
+    PetscBool matl;
+
     ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)is),&rank);CHKERRQ(ierr);
     ierr = MPI_Comm_size(PetscObjectComm((PetscObject)is),&size);CHKERRQ(ierr);
+    ierr = PetscViewerGetFormat(viewer,&fmt);CHKERRQ(ierr);
+    matl = (PetscBool)(fmt == PETSC_VIEWER_ASCII_MATLAB);
     if (size == 1) {
-      if (is->isperm) {
-        ierr = PetscViewerASCIIPrintf(viewer,"Index set is permutation\n");CHKERRQ(ierr);
-      }
-      ierr = PetscViewerASCIIPrintf(viewer,"Number of indices in (stride) set %D\n",n);CHKERRQ(ierr);
-      for (i=0; i<n; i++) {
-        ierr = PetscViewerASCIIPrintf(viewer,"%D %D\n",i,sub->first + i*sub->step);CHKERRQ(ierr);
+      if (matl) {
+        const char* name;
+
+        ierr = PetscObjectGetName((PetscObject)is,&name);CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPrintf(viewer,"%s = [%D : %D : %D];\n",name,sub->first+1,sub->step,sub->first + sub->step*(n-1)+1);CHKERRQ(ierr);
+      } else {
+        if (is->isperm) {
+          ierr = PetscViewerASCIIPrintf(viewer,"Index set is permutation\n");CHKERRQ(ierr);
+        }
+        ierr = PetscViewerASCIIPrintf(viewer,"Number of indices in (stride) set %D\n",n);CHKERRQ(ierr);
+        for (i=0; i<n; i++) {
+          ierr = PetscViewerASCIIPrintf(viewer,"%D %D\n",i,sub->first + i*sub->step);CHKERRQ(ierr);
+        }
       }
       ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
     } else {
-      ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_TRUE);CHKERRQ(ierr);
-      if (is->isperm) {
-        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] Index set is permutation\n",rank);CHKERRQ(ierr);
-      }
-      ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] Number of indices in (stride) set %D\n",rank,n);CHKERRQ(ierr);
-      for (i=0; i<n; i++) {
-        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] %D %D\n",rank,i,sub->first + i*sub->step);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
+      if (matl) {
+        const char* name;
+
+        ierr = PetscObjectGetName((PetscObject)is,&name);CHKERRQ(ierr);
+        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"%s_%d = [%D : %D : %D];\n",name,rank,sub->first+1,sub->step,sub->first + sub->step*(n-1)+1);CHKERRQ(ierr);
+      } else {
+        if (is->isperm) {
+          ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] Index set is permutation\n",rank);CHKERRQ(ierr);
+        }
+        ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] Number of indices in (stride) set %D\n",rank,n);CHKERRQ(ierr);
+        for (i=0; i<n; i++) {
+          ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] %D %D\n",rank,i,sub->first + i*sub->step);CHKERRQ(ierr);
+        }
       }
       ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
-      ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_FALSE);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPopSynchronized(viewer);CHKERRQ(ierr);
     }
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSort_Stride"
 PetscErrorCode ISSort_Stride(IS is)
 {
   IS_Stride *sub = (IS_Stride*)is->data;
@@ -255,8 +263,6 @@ PetscErrorCode ISSort_Stride(IS is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSorted_Stride"
 PetscErrorCode ISSorted_Stride(IS is,PetscBool * flg)
 {
   IS_Stride *sub = (IS_Stride*)is->data;
@@ -267,8 +273,6 @@ PetscErrorCode ISSorted_Stride(IS is,PetscBool * flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISOnComm_Stride"
 static PetscErrorCode ISOnComm_Stride(IS is,MPI_Comm comm,PetscCopyMode mode,IS *newis)
 {
   PetscErrorCode ierr;
@@ -279,8 +283,6 @@ static PetscErrorCode ISOnComm_Stride(IS is,MPI_Comm comm,PetscCopyMode mode,IS
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSetBlockSize_Stride"
 static PetscErrorCode ISSetBlockSize_Stride(IS is,PetscInt bs)
 {
   IS_Stride     *sub = (IS_Stride*)is->data;
@@ -292,8 +294,6 @@ static PetscErrorCode ISSetBlockSize_Stride(IS is,PetscInt bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISContiguousLocal_Stride"
 static PetscErrorCode ISContiguousLocal_Stride(IS is,PetscInt gstart,PetscInt gend,PetscInt *start,PetscBool *contig)
 {
   IS_Stride *sub = (IS_Stride*)is->data;
@@ -327,11 +327,10 @@ static struct _ISOps myops = { ISGetSize_Stride,
                                ISToGeneral_Stride,
                                ISOnComm_Stride,
                                ISSetBlockSize_Stride,
-                               ISContiguousLocal_Stride};
+                               ISContiguousLocal_Stride,
+                               ISLocate_Stride};
 
 
-#undef __FUNCT__
-#define __FUNCT__ "ISStrideSetStride"
 /*@
    ISStrideSetStride - Sets the stride information for a stride index set.
 
@@ -361,8 +360,6 @@ PetscErrorCode  ISStrideSetStride(IS is,PetscInt n,PetscInt first,PetscInt step)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISStrideSetStride_Stride"
 PetscErrorCode  ISStrideSetStride_Stride(IS is,PetscInt n,PetscInt first,PetscInt step)
 {
   PetscErrorCode ierr;
@@ -371,23 +368,22 @@ PetscErrorCode  ISStrideSetStride_Stride(IS is,PetscInt n,PetscInt first,PetscIn
 
   PetscFunctionBegin;
   sub->n     = n;
-  ierr       = MPI_Allreduce(&n,&sub->N,1,MPIU_INT,MPI_SUM,PetscObjectComm((PetscObject)is));CHKERRQ(ierr);
+  ierr       = MPIU_Allreduce(&n,&sub->N,1,MPIU_INT,MPI_SUM,PetscObjectComm((PetscObject)is));CHKERRQ(ierr);
   sub->first = first;
   sub->step  = step;
   if (step > 0) {min = first; max = first + step*(n-1);}
   else          {max = first; min = first + step*(n-1);}
 
-  is->min  = n > 0 ? min : -1;
-  is->max  = n > 0 ? max : -1;
+  is->min  = n > 0 ? min : PETSC_MAX_INT;
+  is->max  = n > 0 ? max : PETSC_MIN_INT;
   is->data = (void*)sub;
 
   if ((!first && step == 1) || (first == max && step == -1 && !min)) is->isperm = PETSC_TRUE;
   else is->isperm = PETSC_FALSE;
+  is->isidentity = PETSC_FALSE;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISCreateStride"
 /*@
    ISCreateStride - Creates a data structure for an index set
    containing a list of evenly spaced integers.
@@ -427,20 +423,15 @@ PetscErrorCode  ISCreateStride(MPI_Comm comm,PetscInt n,PetscInt first,PetscInt
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISCreate_Stride"
 PETSC_EXTERN PetscErrorCode ISCreate_Stride(IS is)
 {
   PetscErrorCode ierr;
   IS_Stride      *sub;
 
   PetscFunctionBegin;
-  ierr = PetscMemcpy(is->ops,&myops,sizeof(myops));CHKERRQ(ierr);
   ierr = PetscNewLog(is,&sub);CHKERRQ(ierr);
-  is->data = sub;
+  is->data = (void *) sub;
+  ierr = PetscMemcpy(is->ops,&myops,sizeof(myops));CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)is,"ISStrideSetStride_C",ISStrideSetStride_Stride);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-
-
-
