@@ -4,12 +4,160 @@
 */
 #include <petsc/private/isimpl.h>      /*I "petscis.h" I*/
 #include <petscviewer.h>
+#include <petscsf.h>
 
 /* Logging support */
 PetscClassId IS_CLASSID;
 
-#undef __FUNCT__
-#define __FUNCT__ "ISIdentity"
+/*@
+   ISRenumber - Renumbers an index set (with multiplicities) in a contiguous way.
+
+   Collective on IS
+
+   Input Parmeters:
+.  subset - the index set
+.  subset_mult - the multiplcity of each entry in subset (optional, can be NULL)
+
+   Output Parameters:
+.  N - the maximum entry of the new IS
+.  subset_n - the new IS
+
+   Level: intermediate
+
+.seealso:
+@*/
+PetscErrorCode ISRenumber(IS subset, IS subset_mult, PetscInt *N, IS *subset_n)
+{
+  PetscSF        sf;
+  PetscLayout    map;
+  const PetscInt *idxs;
+  PetscInt       *leaf_data,*root_data,*gidxs;
+  PetscInt       N_n,n,i,lbounds[2],gbounds[2],Nl;
+  PetscInt       n_n,nlocals,start,first_index;
+  PetscMPIInt    commsize;
+  PetscBool      first_found;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = ISGetLocalSize(subset,&n);CHKERRQ(ierr);
+  if (subset_mult) {
+    PetscCheckSameComm(subset,1,subset_mult,2);
+    ierr = ISGetLocalSize(subset,&i);CHKERRQ(ierr);
+    if (i != n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Local subset and multiplicity sizes don't match! %d != %d",n,i);
+  }
+  /* create workspace layout for computing global indices of subset */
+  ierr = ISGetIndices(subset,&idxs);CHKERRQ(ierr);
+  lbounds[0] = lbounds[1] = 0;
+  for (i=0;i<n;i++) {
+    if (idxs[i] < lbounds[0]) lbounds[0] = idxs[i];
+    else if (idxs[i] > lbounds[1]) lbounds[1] = idxs[i];
+  }
+  lbounds[0] = -lbounds[0];
+  ierr = MPIU_Allreduce(lbounds,gbounds,2,MPIU_INT,MPI_MAX,PetscObjectComm((PetscObject)subset));CHKERRQ(ierr);
+  gbounds[0] = -gbounds[0];
+  N_n= gbounds[1] - gbounds[0] + 1;
+  ierr = PetscLayoutCreate(PetscObjectComm((PetscObject)subset),&map);CHKERRQ(ierr);
+  ierr = PetscLayoutSetBlockSize(map,1);CHKERRQ(ierr);
+  ierr = PetscLayoutSetSize(map,N_n);CHKERRQ(ierr);
+  ierr = PetscLayoutSetUp(map);CHKERRQ(ierr);
+  ierr = PetscLayoutGetLocalSize(map,&Nl);CHKERRQ(ierr);
+
+  /* create sf : leaf_data == multiplicity of indexes, root data == global index in layout */
+  ierr = PetscMalloc2(n,&leaf_data,Nl,&root_data);CHKERRQ(ierr);
+  if (subset_mult) {
+    const PetscInt* idxs_mult;
+
+    ierr = ISGetIndices(subset_mult,&idxs_mult);CHKERRQ(ierr);
+    ierr = PetscMemcpy(leaf_data,idxs_mult,n*sizeof(PetscInt));CHKERRQ(ierr);
+    ierr = ISRestoreIndices(subset_mult,&idxs_mult);CHKERRQ(ierr);
+  } else {
+    for (i=0;i<n;i++) leaf_data[i] = 1;
+  }
+  /* local size of new subset */
+  n_n = 0;
+  for (i=0;i<n;i++) n_n += leaf_data[i];
+
+  /* global indexes in layout */
+  ierr = PetscMalloc1(n_n,&gidxs);CHKERRQ(ierr); /* allocating possibly extra space in gidxs which will be used later */
+  for (i=0;i<n;i++) gidxs[i] = idxs[i] - gbounds[0];
+  ierr = ISRestoreIndices(subset,&idxs);CHKERRQ(ierr);
+  ierr = PetscSFCreate(PetscObjectComm((PetscObject)subset),&sf);CHKERRQ(ierr);
+  ierr = PetscSFSetGraphLayout(sf,map,n,NULL,PETSC_COPY_VALUES,gidxs);CHKERRQ(ierr);
+  ierr = PetscLayoutDestroy(&map);CHKERRQ(ierr);
+
+  /* reduce from leaves to roots */
+  ierr = PetscMemzero(root_data,Nl*sizeof(PetscInt));CHKERRQ(ierr);
+  ierr = PetscSFReduceBegin(sf,MPIU_INT,leaf_data,root_data,MPI_MAX);CHKERRQ(ierr);
+  ierr = PetscSFReduceEnd(sf,MPIU_INT,leaf_data,root_data,MPI_MAX);CHKERRQ(ierr);
+
+  /* count indexes in local part of layout */
+  nlocals = 0;
+  first_index = -1;
+  first_found = PETSC_FALSE;
+  for (i=0;i<Nl;i++) {
+    if (!first_found && root_data[i]) {
+      first_found = PETSC_TRUE;
+      first_index = i;
+    }
+    nlocals += root_data[i];
+  }
+
+  /* cumulative of number of indexes and size of subset without holes */
+#if defined(PETSC_HAVE_MPI_EXSCAN)
+  start = 0;
+  ierr = MPI_Exscan(&nlocals,&start,1,MPIU_INT,MPI_SUM,PetscObjectComm((PetscObject)subset));CHKERRQ(ierr);
+#else
+  ierr = MPI_Scan(&nlocals,&start,1,MPIU_INT,MPI_SUM,PetscObjectComm((PetscObject)subset));CHKERRQ(ierr);
+  start = start-nlocals;
+#endif
+
+  if (N) { /* compute total size of new subset if requested */
+    *N = start + nlocals;
+    ierr = MPI_Comm_size(PetscObjectComm((PetscObject)subset),&commsize);CHKERRQ(ierr);
+    ierr = MPI_Bcast(N,1,MPIU_INT,commsize-1,PetscObjectComm((PetscObject)subset));CHKERRQ(ierr);
+  }
+
+  /* adapt root data with cumulative */
+  if (first_found) {
+    PetscInt old_index;
+
+    root_data[first_index] += start;
+    old_index = first_index;
+    for (i=first_index+1;i<Nl;i++) {
+      if (root_data[i]) {
+        root_data[i] += root_data[old_index];
+        old_index = i;
+      }
+    }
+  }
+
+  /* from roots to leaves */
+  ierr = PetscSFBcastBegin(sf,MPIU_INT,root_data,leaf_data);CHKERRQ(ierr);
+  ierr = PetscSFBcastEnd(sf,MPIU_INT,root_data,leaf_data);CHKERRQ(ierr);
+  ierr = PetscSFDestroy(&sf);CHKERRQ(ierr);
+
+  /* create new IS with global indexes without holes */
+  if (subset_mult) {
+    const PetscInt* idxs_mult;
+    PetscInt        cum;
+
+    cum = 0;
+    ierr = ISGetIndices(subset_mult,&idxs_mult);CHKERRQ(ierr);
+    for (i=0;i<n;i++) {
+      PetscInt j;
+      for (j=0;j<idxs_mult[i];j++) gidxs[cum++] = leaf_data[i] - idxs_mult[i] + j;
+    }
+    ierr = ISRestoreIndices(subset_mult,&idxs_mult);CHKERRQ(ierr);
+  } else {
+    for (i=0;i<n;i++) {
+      gidxs[i] = leaf_data[i]-1;
+    }
+  }
+  ierr = ISCreateGeneral(PetscObjectComm((PetscObject)subset),n_n,gidxs,PETSC_OWN_POINTER,subset_n);CHKERRQ(ierr);
+  ierr = PetscFree2(leaf_data,root_data);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 /*@
    ISIdentity - Determines whether index set is the identity mapping.
 
@@ -43,8 +191,6 @@ PetscErrorCode  ISIdentity(IS is,PetscBool  *ident)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSetIdentity"
 /*@
    ISSetIdentity - Informs the index set that it is an identity.
 
@@ -71,8 +217,6 @@ PetscErrorCode  ISSetIdentity(IS is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISContiguousLocal"
 /*@
    ISContiguousLocal - Locates an index set with contiguous range within a global range, if possible
 
@@ -110,8 +254,6 @@ PetscErrorCode  ISContiguousLocal(IS is,PetscInt gstart,PetscInt gend,PetscInt *
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISPermutation"
 /*@
    ISPermutation - PETSC_TRUE or PETSC_FALSE depending on whether the
    index set has been declared to be a permutation.
@@ -140,8 +282,6 @@ PetscErrorCode  ISPermutation(IS is,PetscBool  *perm)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSetPermutation"
 /*@
    ISSetPermutation - Informs the index set that it is a permutation.
 
@@ -191,8 +331,6 @@ PetscErrorCode  ISSetPermutation(IS is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISDestroy"
 /*@
    ISDestroy - Destroys an index set.
 
@@ -230,8 +368,6 @@ PetscErrorCode  ISDestroy(IS *is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISInvertPermutation"
 /*@
    ISInvertPermutation - Creates a new permutation that is the inverse of
                          a given permutation.
@@ -248,7 +384,8 @@ PetscErrorCode  ISDestroy(IS *is)
 
    Level: intermediate
 
-   Notes: For parallel index sets this does the complete parallel permutation, but the
+   Notes:
+    For parallel index sets this does the complete parallel permutation, but the
     code is not efficient for huge index sets (10,000,000 indices).
 
    Concepts: inverse permutation
@@ -272,8 +409,6 @@ PetscErrorCode  ISInvertPermutation(IS is,PetscInt nlocal,IS *isout)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetSize"
 /*@
    ISGetSize - Returns the global length of an index set.
 
@@ -302,8 +437,6 @@ PetscErrorCode  ISGetSize(IS is,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetLocalSize"
 /*@
    ISGetLocalSize - Returns the local (processor) length of an index set.
 
@@ -333,8 +466,6 @@ PetscErrorCode  ISGetLocalSize(IS is,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetIndices"
 /*@C
    ISGetIndices - Returns a pointer to the indices.  The user should call
    ISRestoreIndices() after having looked at the indices.  The user should
@@ -349,7 +480,7 @@ PetscErrorCode  ISGetLocalSize(IS is,PetscInt *size)
 .  ptr - the location to put the pointer to the indices
 
    Fortran Note:
-   This routine is used differently from Fortran
+   This routine has two different interfaces from Fortran; the first is not recommend, it does not require Fortran 90
 $    IS          is
 $    integer     is_array(1)
 $    PetscOffset i_is
@@ -361,6 +492,14 @@ $      value = is_array(i_is + 1)
 $
 $      ...... other code
 $       call ISRestoreIndices(is,is_array,i_is,ierr)
+   The second Fortran interface is recommended.
+$          use petscisdef
+$          PetscInt, pointer :: array(:)
+$          PetscErrorCode  ierr
+$          IS       i
+$          call ISGetIndicesF90(i,array,ierr)
+
+
 
    See the Fortran chapter of the users manual and
    petsc/src/is/examples/[tutorials,tests] for details.
@@ -383,8 +522,6 @@ PetscErrorCode  ISGetIndices(IS is,const PetscInt *ptr[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetMinMax"
 /*@C
    ISGetMinMax - Gets the minimum and maximum values in an IS
 
@@ -399,6 +536,10 @@ PetscErrorCode  ISGetIndices(IS is,const PetscInt *ptr[])
 
    Level: intermediate
 
+   Notes:
+    Empty index sets return min=PETSC_MAX_INT and max=PETSC_MIN_INT.
+    In parallel, it returns the min and max of the local portion of the IS
+
    Concepts: index sets^getting indices
    Concepts: indices of index set
 
@@ -413,8 +554,53 @@ PetscErrorCode  ISGetMinMax(IS is,PetscInt *min,PetscInt *max)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISRestoreIndices"
+/*@
+  ISLocate - determine the location of an index within the local component of an index set
+
+  Not Collective
+
+  Input Parameter:
++ is - the index set
+- key - the search key
+
+  Output Parameter:
+. location - if >= 0, a location within the index set that is equal to the key, otherwise the key is not in the index set
+
+  Level: intermediate
+@*/
+PetscErrorCode ISLocate(IS is, PetscInt key, PetscInt *location)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  if (is->ops->locate) {
+    ierr = (*is->ops->locate)(is,key,location);CHKERRQ(ierr);
+  } else {
+    PetscInt       numIdx;
+    PetscBool      sorted;
+    const PetscInt *idx;
+
+    ierr = ISGetLocalSize(is,&numIdx);CHKERRQ(ierr);
+    ierr = ISGetIndices(is,&idx);CHKERRQ(ierr);
+    ierr = ISSorted(is,&sorted);CHKERRQ(ierr);
+    if (sorted) {
+      ierr = PetscFindInt(key,numIdx,idx,location);CHKERRQ(ierr);
+    } else {
+      PetscInt i;
+
+      *location = -1;
+      for (i = 0; i < numIdx; i++) {
+        if (idx[i] == key) {
+          *location = i;
+          break;
+        }
+      }
+    }
+    ierr = ISRestoreIndices(is,&idx);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
 /*@C
    ISRestoreIndices - Restores an index set to a usable state after a call
                       to ISGetIndices().
@@ -462,8 +648,6 @@ PetscErrorCode  ISRestoreIndices(IS is,const PetscInt *ptr[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGatherTotal_Private"
 static PetscErrorCode ISGatherTotal_Private(IS is)
 {
   PetscErrorCode ierr;
@@ -496,8 +680,6 @@ static PetscErrorCode ISGatherTotal_Private(IS is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetTotalIndices"
 /*@C
    ISGetTotalIndices - Retrieve an array containing all indices across the communicator.
 
@@ -512,7 +694,8 @@ static PetscErrorCode ISGatherTotal_Private(IS is)
 
    Level: intermediate
 
-   Notes: this is potentially nonscalable, but depends on the size of the total index set
+   Notes:
+    this is potentially nonscalable, but depends on the size of the total index set
      and the size of the communicator. This may be feasible for index sets defined on
      subcommunicators, such that the set size does not grow with PETSC_WORLD_COMM.
      Note also that there is no way to tell where the local part of the indices starts
@@ -542,8 +725,6 @@ PetscErrorCode ISGetTotalIndices(IS is, const PetscInt *indices[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISRestoreTotalIndices"
 /*@C
    ISRestoreTotalIndices - Restore the index array obtained with ISGetTotalIndices().
 
@@ -575,8 +756,6 @@ PetscErrorCode  ISRestoreTotalIndices(IS is, const PetscInt *indices[])
   }
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__ "ISGetNonlocalIndices"
 /*@C
    ISGetNonlocalIndices - Retrieve an array of indices from remote processors
                        in this communicator.
@@ -594,7 +773,8 @@ PetscErrorCode  ISRestoreTotalIndices(IS is, const PetscInt *indices[])
 
    Level: intermediate
 
-   Notes: restore the indices using ISRestoreNonlocalIndices().
+   Notes:
+    restore the indices using ISRestoreNonlocalIndices().
           The same scalability considerations as those for ISGetTotalIndices
           apply here.
 
@@ -618,7 +798,7 @@ PetscErrorCode  ISGetNonlocalIndices(IS is, const PetscInt *indices[])
     }
     ierr = ISGetLocalSize(is,&n);CHKERRQ(ierr);
     ierr = ISGetSize(is,&N);CHKERRQ(ierr);
-    ierr = PetscMalloc(sizeof(PetscInt)*(N-n), &(is->nonlocal));CHKERRQ(ierr);
+    ierr = PetscMalloc1(N-n, &(is->nonlocal));CHKERRQ(ierr);
     ierr = PetscMemcpy(is->nonlocal, is->total, sizeof(PetscInt)*is->local_offset);CHKERRQ(ierr);
     ierr = PetscMemcpy(is->nonlocal+is->local_offset, is->total+is->local_offset+n, sizeof(PetscInt)*(N - is->local_offset - n));CHKERRQ(ierr);
     *indices = is->nonlocal;
@@ -626,8 +806,6 @@ PetscErrorCode  ISGetNonlocalIndices(IS is, const PetscInt *indices[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISRestoreNonlocalIndices"
 /*@C
    ISRestoreTotalIndices - Restore the index array obtained with ISGetNonlocalIndices().
 
@@ -652,8 +830,6 @@ PetscErrorCode  ISRestoreNonlocalIndices(IS is, const PetscInt *indices[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetNonlocalIS"
 /*@
    ISGetNonlocalIS - Gather all nonlocal indices for this IS and present
                      them as another sequential index set.
@@ -670,7 +846,8 @@ PetscErrorCode  ISRestoreNonlocalIndices(IS is, const PetscInt *indices[])
 
    Level: intermediate
 
-   Notes: complement represents the result of ISGetNonlocalIndices as an IS.
+   Notes:
+    complement represents the result of ISGetNonlocalIndices as an IS.
           Therefore scalability issues similar to ISGetNonlocalIndices apply.
           The resulting IS must be restored using ISRestoreNonlocalIS().
 
@@ -702,8 +879,6 @@ PetscErrorCode  ISGetNonlocalIS(IS is, IS *complement)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "ISRestoreNonlocalIS"
 /*@
    ISRestoreNonlocalIS - Restore the IS obtained with ISGetNonlocalIS().
 
@@ -735,8 +910,6 @@ PetscErrorCode  ISRestoreNonlocalIS(IS is, IS *complement)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISView"
 /*@C
    ISView - Displays an index set.
 
@@ -767,8 +940,6 @@ PetscErrorCode  ISView(IS is,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISLoad"
 /*@
   ISLoad - Loads a vector that has been stored in binary or HDF5 format with ISView().
 
@@ -805,8 +976,6 @@ PetscErrorCode ISLoad(IS is, PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSort"
 /*@
    ISSort - Sorts the indices of an index set.
 
@@ -832,8 +1001,6 @@ PetscErrorCode  ISSort(IS is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSortRemoveDups"
 /*@
   ISSortRemoveDups - Sorts the indices of an index set, removing duplicates.
 
@@ -859,8 +1026,6 @@ PetscErrorCode ISSortRemoveDups(IS is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISToGeneral"
 /*@
    ISToGeneral - Converts an IS object of any type to ISGENERAL type
 
@@ -888,8 +1053,6 @@ PetscErrorCode  ISToGeneral(IS is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSorted"
 /*@
    ISSorted - Checks the indices to determine whether they have been sorted.
 
@@ -902,7 +1065,8 @@ PetscErrorCode  ISToGeneral(IS is)
 .  flg - output flag, either PETSC_TRUE if the index set is sorted,
          or PETSC_FALSE otherwise.
 
-   Notes: For parallel IS objects this only indicates if the local part of the IS
+   Notes:
+    For parallel IS objects this only indicates if the local part of the IS
           is sorted. So some processors may return PETSC_TRUE while others may
           return PETSC_FALSE.
 
@@ -921,8 +1085,6 @@ PetscErrorCode  ISSorted(IS is,PetscBool  *flg)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISDuplicate"
 /*@
    ISDuplicate - Creates a duplicate copy of an index set.
 
@@ -953,8 +1115,6 @@ PetscErrorCode  ISDuplicate(IS is,IS *newIS)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISCopy"
 /*@
    ISCopy - Copies an index set.
 
@@ -989,8 +1149,6 @@ PetscErrorCode  ISCopy(IS is,IS isy)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISOnComm"
 /*@
    ISOnComm - Split a parallel IS on subcomms (usually self) or concatenate index sets on subcomms into a parallel index set
 
@@ -1034,8 +1192,6 @@ PetscErrorCode  ISOnComm(IS is,MPI_Comm comm,PetscCopyMode mode,IS *newis)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSetBlockSize"
 /*@
    ISSetBlockSize - informs an index set that it has a given block size
 
@@ -1061,8 +1217,6 @@ PetscErrorCode  ISSetBlockSize(IS is,PetscInt bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetBlockSize"
 /*@
    ISGetBlockSize - Returns the number of elements in a block.
 
@@ -1090,8 +1244,6 @@ PetscErrorCode  ISGetBlockSize(IS is,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISGetIndicesCopy"
 PetscErrorCode ISGetIndicesCopy(IS is, PetscInt idx[])
 {
   PetscErrorCode ierr;
@@ -1125,16 +1277,13 @@ PetscErrorCode ISGetIndicesCopy(IS is, PetscInt idx[])
 
     Example of Usage:
 .vb
-    PetscScalar, pointer xx_v(:)
+    PetscInt, pointer xx_v(:)
     ....
     call ISGetIndicesF90(x,xx_v,ierr)
     a = xx_v(3)
     call ISRestoreIndicesF90(x,xx_v,ierr)
 .ve
 
-    Notes:
-    Not yet supported for all F90 compilers.
-
     Level: intermediate
 
 .seealso:  ISRestoreIndicesF90(), ISGetIndices(), ISRestoreIndices()
@@ -1163,16 +1312,13 @@ M*/
 
     Example of Usage:
 .vb
-    PetscScalar, pointer xx_v(:)
+    PetscInt, pointer xx_v(:)
     ....
     call ISGetIndicesF90(x,xx_v,ierr)
     a = xx_v(3)
     call ISRestoreIndicesF90(x,xx_v,ierr)
 .ve
 
-    Notes:
-    Not yet supported for all F90 compilers.
-
     Level: intermediate
 
 .seealso:  ISGetIndicesF90(), ISGetIndices(), ISRestoreIndices()
@@ -1197,16 +1343,13 @@ M*/
 -   ierr - error code
     Example of Usage:
 .vb
-    PetscScalar, pointer xx_v(:)
+    PetscInt, pointer xx_v(:)
     ....
     call ISBlockGetIndicesF90(x,xx_v,ierr)
     a = xx_v(3)
     call ISBlockRestoreIndicesF90(x,xx_v,ierr)
 .ve
 
-    Notes:
-    Not yet supported for all F90 compilers
-
     Level: intermediate
 
 .seealso:  ISBlockRestoreIndicesF90(), ISGetIndices(), ISRestoreIndices(),
@@ -1236,7 +1379,7 @@ M*/
 
     Example of Usage:
 .vb
-    PetscScalar, pointer xx_v(:)
+    PetscInt, pointer xx_v(:)
     ....
     call ISBlockGetIndicesF90(x,xx_v,ierr)
     a = xx_v(3)
