@@ -1,6 +1,6 @@
 
 #include <petscis.h>
-#include <../src/sys/f90-src/f90impl.h>
+#include <petsc/private/f90impl.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define iscoloringgetisf90_        ISCOLORINGGETISF90
@@ -19,15 +19,15 @@ PETSC_EXTERN void PETSC_STDCALL iscoloringgetisf90_(ISColoring *iscoloring,Petsc
   *__ierr = ISColoringGetIS(*iscoloring,n,&lis); if (*__ierr) return;
   *__ierr = PetscMalloc1(*n,&newisint); if (*__ierr) return;
   for (i=0; i<*n; i++) newisint[i] = (PetscFortranAddr)lis[i];
-  *__ierr = F90Array1dCreate(newisint,PETSC_FORTRANADDR,1,*n,ptr PETSC_F90_2PTR_PARAM(ptrd));
+  *__ierr = F90Array1dCreate(newisint,MPIU_FORTRANADDR,1,*n,ptr PETSC_F90_2PTR_PARAM(ptrd));
 }
 
 PETSC_EXTERN void PETSC_STDCALL iscoloringrestoreisf90_(ISColoring *iscoloring,F90Array1d *ptr,int *__ierr PETSC_F90_2PTR_PROTO(ptrd))
 {
   PetscFortranAddr *is;
 
-  *__ierr = F90Array1dAccess(ptr,PETSC_FORTRANADDR,(void**)&is PETSC_F90_2PTR_PARAM(ptrd));if (*__ierr) return;
-  *__ierr = F90Array1dDestroy(ptr,PETSC_FORTRANADDR PETSC_F90_2PTR_PARAM(ptrd));if (*__ierr) return;
+  *__ierr = F90Array1dAccess(ptr,MPIU_FORTRANADDR,(void**)&is PETSC_F90_2PTR_PARAM(ptrd));if (*__ierr) return;
+  *__ierr = F90Array1dDestroy(ptr,MPIU_FORTRANADDR PETSC_F90_2PTR_PARAM(ptrd));if (*__ierr) return;
   *__ierr = ISColoringRestoreIS(*iscoloring,(IS**)is);if (*__ierr) return;
   *__ierr = PetscFree(is);
 }
