@@ -1,11 +1,10 @@
 
 #include <petsc/private/isimpl.h>    /*I "petscis.h"  I*/
 #include <petscviewer.h>
+#include <petscsf.h>
 
 const char *const ISColoringTypes[] = {"global","ghosted","ISColoringType","IS_COLORING_",0};
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringReference"
 PetscErrorCode ISColoringReference(ISColoring coloring)
 {
   PetscFunctionBegin;
@@ -13,8 +12,6 @@ PetscErrorCode ISColoringReference(ISColoring coloring)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringSetType"
 PetscErrorCode ISColoringSetType(ISColoring coloring,ISColoringType type)
 {
   PetscFunctionBegin;
@@ -22,8 +19,6 @@ PetscErrorCode ISColoringSetType(ISColoring coloring,ISColoringType type)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringDestroy"
 /*@
    ISColoringDestroy - Destroys a coloring context.
 
@@ -58,10 +53,8 @@ PetscErrorCode  ISColoringDestroy(ISColoring *iscoloring)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringViewFromOptions"
 /*
-  ISColoringViewFromOptions - Processes command line options to determine if/how an ISColoring object is to be viewed. 
+  ISColoringViewFromOptions - Processes command line options to determine if/how an ISColoring object is to be viewed.
 
   Collective on ISColoring
 
@@ -95,8 +88,6 @@ PetscErrorCode ISColoringViewFromOptions(ISColoring obj,PetscObject bobj,const c
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringView"
 /*@C
    ISColoringView - Views a coloring context.
 
@@ -133,11 +124,11 @@ PetscErrorCode  ISColoringView(ISColoring iscoloring,PetscViewer viewer)
     ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
     ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
     ierr = PetscViewerASCIIPrintf(viewer,"ISColoring Object: %d MPI processes\n",size);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_TRUE);CHKERRQ(ierr);
+    ierr = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
     ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] Number of colors %d\n",rank,iscoloring->n);CHKERRQ(ierr);
     ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
-    ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_FALSE);CHKERRQ(ierr);
-  } else SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Viewer type %s not supported for ISColoring",((PetscObject)viewer)->type_name);
+    ierr = PetscViewerASCIIPopSynchronized(viewer);CHKERRQ(ierr);
+  }
 
   ierr = ISColoringGetIS(iscoloring,PETSC_IGNORE,&is);CHKERRQ(ierr);
   for (i=0; i<iscoloring->n; i++) {
@@ -147,8 +138,6 @@ PetscErrorCode  ISColoringView(ISColoring iscoloring,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringGetIS"
 /*@C
    ISColoringGetIS - Extracts index sets from the coloring context
 
@@ -198,7 +187,7 @@ PetscErrorCode  ISColoringGetIS(ISColoring iscoloring,PetscInt *nn,IS *isis[])
         ierr = MPI_Scan(&iscoloring->N,&base,1,MPIU_INT,MPI_SUM,iscoloring->comm);CHKERRQ(ierr);
         base -= iscoloring->N;
         for (i=0; i<n; i++) ii[colors[i]][mcolors[colors[i]]++] = i + base; /* global idx */
-      } else if (iscoloring->ctype == IS_COLORING_GHOSTED) {
+      } else if (iscoloring->ctype == IS_COLORING_LOCAL) {
         for (i=0; i<n; i++) ii[colors[i]][mcolors[colors[i]]++] = i;   /* local idx */
       } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Not provided for this ISColoringType type");
 
@@ -217,8 +206,6 @@ PetscErrorCode  ISColoringGetIS(ISColoring iscoloring,PetscInt *nn,IS *isis[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringRestoreIS"
 /*@C
    ISColoringRestoreIS - Restores the index sets extracted from the coloring context
 
@@ -242,8 +229,6 @@ PetscErrorCode  ISColoringRestoreIS(ISColoring iscoloring,IS *is[])
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "ISColoringCreate"
 /*@
     ISColoringCreate - Generates an ISColoring context from lists (provided
     by each processor) of colors for each node.
@@ -265,7 +250,8 @@ PetscErrorCode  ISColoringRestoreIS(ISColoring iscoloring,IS *is[])
 
    Level: advanced
 
-    Notes: By default sets coloring type to  IS_COLORING_GLOBAL
+    Notes:
+    By default sets coloring type to  IS_COLORING_GLOBAL
 
 .seealso: MatColoringCreate(), ISColoringView(), ISColoringDestroy(), ISColoringSetType()
 
@@ -309,7 +295,7 @@ PetscErrorCode  ISColoringCreate(MPI_Comm comm,PetscInt ncolors,PetscInt n,const
     if (ncwork < colors[i]) ncwork = colors[i];
   }
   ncwork++;
-  ierr = MPI_Allreduce(&ncwork,&nc,1,MPIU_INT,MPI_MAX,comm);CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(&ncwork,&nc,1,MPIU_INT,MPI_MAX,comm);CHKERRQ(ierr);
   if (nc > ncolors) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Number of colors passed in %D is less then the actual number of colors in array %D",ncolors,nc);
   (*iscoloring)->n      = nc;
   (*iscoloring)->is     = 0;
@@ -333,8 +319,121 @@ PetscErrorCode  ISColoringCreate(MPI_Comm comm,PetscInt ncolors,PetscInt n,const
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISPartitioningToNumbering"
+/*@
+    ISBuildTwoSided - Takes an IS that describes where we will go. Generates an IS that contains new numbers from remote or local
+    on the IS.
+
+    Collective on IS
+
+    Input Parameters
+.   ito - an IS describes where we will go. Negative target rank will be ignored
+.   toindx - an IS describes what indices should send. NULL means sending natural numbering
+
+    Output Parameter:
+.   rows - contains new numbers from remote or local
+
+   Level: advanced
+
+.seealso: MatPartitioningCreate(), ISPartitioningToNumbering(), ISPartitioningCount()
+
+@*/
+PetscErrorCode  ISBuildTwoSided(IS ito,IS toindx, IS *rows)
+{
+   const PetscInt *ito_indices,*toindx_indices;
+   PetscInt       *send_indices,rstart,*recv_indices,nrecvs,nsends;
+   PetscInt       *tosizes,*fromsizes,i,j,*tosizes_tmp,*tooffsets_tmp,ito_ln;
+   PetscMPIInt    *toranks,*fromranks,size,target_rank,*fromperm_newtoold,nto,nfrom;
+   PetscLayout     isrmap;
+   MPI_Comm        comm;
+   PetscSF         sf;
+   PetscSFNode    *iremote;
+   PetscErrorCode  ierr;
+
+   PetscFunctionBegin;
+   ierr = PetscObjectGetComm((PetscObject)ito,&comm);CHKERRQ(ierr);
+   ierr = MPI_Comm_size(comm,&size);CHKERRQ(ierr);
+   ierr = ISGetLocalSize(ito,&ito_ln);CHKERRQ(ierr);
+   /* why we do not have ISGetLayout? */
+   isrmap = ito->map;
+   ierr = PetscLayoutGetRange(isrmap,&rstart,NULL);CHKERRQ(ierr);
+   ierr = ISGetIndices(ito,&ito_indices);CHKERRQ(ierr);
+   ierr = PetscCalloc2(size,&tosizes_tmp,size+1,&tooffsets_tmp);CHKERRQ(ierr);
+   for (i=0; i<ito_ln; i++) {
+     if (ito_indices[i]<0) continue;
+#if defined(PETSC_USE_DEBUG)
+     if (ito_indices[i]>=size) SETERRQ2(comm,PETSC_ERR_ARG_OUTOFRANGE,"target rank %d is larger than communicator size %d ",ito_indices[i],size);
+#endif
+     tosizes_tmp[ito_indices[i]]++;
+   }
+   nto = 0;
+   for (i=0; i<size; i++) {
+     tooffsets_tmp[i+1] = tooffsets_tmp[i]+tosizes_tmp[i];
+     if (tosizes_tmp[i]>0) nto++;
+   }
+   ierr = PetscCalloc2(nto,&toranks,2*nto,&tosizes);CHKERRQ(ierr);
+   nto  = 0;
+   for (i=0; i<size; i++) {
+     if (tosizes_tmp[i]>0) {
+       toranks[nto]     = i;
+       tosizes[2*nto]   = tosizes_tmp[i];/* size */
+       tosizes[2*nto+1] = tooffsets_tmp[i];/* offset */
+       nto++;
+     }
+   }
+   nsends = tooffsets_tmp[size];
+   ierr   = PetscCalloc1(nsends,&send_indices);CHKERRQ(ierr);
+   if (toindx) {
+     ierr = ISGetIndices(toindx,&toindx_indices);CHKERRQ(ierr);
+   }
+   for (i=0; i<ito_ln; i++) {
+     if (ito_indices[i]<0) continue;
+     target_rank = ito_indices[i];
+     send_indices[tooffsets_tmp[target_rank]] = toindx? toindx_indices[i]:(i+rstart);
+     tooffsets_tmp[target_rank]++;
+   }
+   if (toindx) {
+     ierr = ISRestoreIndices(toindx,&toindx_indices);CHKERRQ(ierr);
+   }
+   ierr = ISRestoreIndices(ito,&ito_indices);CHKERRQ(ierr);
+   ierr = PetscFree2(tosizes_tmp,tooffsets_tmp);CHKERRQ(ierr);
+   ierr = PetscCommBuildTwoSided(comm,2,MPIU_INT,nto,toranks,tosizes,&nfrom,&fromranks,&fromsizes);CHKERRQ(ierr);
+   ierr = PetscFree2(toranks,tosizes);CHKERRQ(ierr);
+   ierr = PetscCalloc1(nfrom,&fromperm_newtoold);CHKERRQ(ierr);
+   for (i=0; i<nfrom; i++) fromperm_newtoold[i] = i;
+   ierr   = PetscSortMPIIntWithArray(nfrom,fromranks,fromperm_newtoold);CHKERRQ(ierr);
+   nrecvs = 0;
+   for (i=0; i<nfrom; i++) nrecvs += fromsizes[i*2];
+   ierr   = PetscCalloc1(nrecvs,&recv_indices);CHKERRQ(ierr);
+   ierr   = PetscCalloc1(nrecvs,&iremote);CHKERRQ(ierr);
+   nrecvs = 0;
+   for (i=0; i<nfrom; i++) {
+     for (j=0; j<fromsizes[2*fromperm_newtoold[i]]; j++) {
+       iremote[nrecvs].rank    = fromranks[i];
+       iremote[nrecvs++].index = fromsizes[2*fromperm_newtoold[i]+1]+j;
+     }
+   }
+   ierr = PetscSFCreate(comm,&sf);CHKERRQ(ierr);
+   ierr = PetscSFSetGraph(sf,nsends,nrecvs,NULL,PETSC_OWN_POINTER,iremote,PETSC_OWN_POINTER);CHKERRQ(ierr);
+   ierr = PetscSFSetType(sf,PETSCSFBASIC);CHKERRQ(ierr);
+   /* how to put a prefix ? */
+   ierr = PetscSFSetFromOptions(sf);CHKERRQ(ierr);
+   ierr = PetscSFBcastBegin(sf,MPIU_INT,send_indices,recv_indices);CHKERRQ(ierr);
+   ierr = PetscSFBcastEnd(sf,MPIU_INT,send_indices,recv_indices);CHKERRQ(ierr);
+   ierr = PetscSFDestroy(&sf);CHKERRQ(ierr);
+   ierr = PetscFree(fromranks);CHKERRQ(ierr);
+   ierr = PetscFree(fromsizes);CHKERRQ(ierr);
+   ierr = PetscFree(fromperm_newtoold);CHKERRQ(ierr);
+   ierr = PetscFree(send_indices);CHKERRQ(ierr);
+   if (rows) {
+     ierr = PetscSortInt(nrecvs,recv_indices);CHKERRQ(ierr);
+     ierr = ISCreateGeneral(comm,nrecvs,recv_indices,PETSC_OWN_POINTER,rows);CHKERRQ(ierr);
+   } else {
+     ierr = PetscFree(recv_indices);CHKERRQ(ierr);
+   }
+   PetscFunctionReturn(0);
+}
+
+
 /*@
     ISPartitioningToNumbering - Takes an ISPartitioning and on each processor
     generates an IS that contains a new global node number for each index based
@@ -344,6 +443,7 @@ PetscErrorCode  ISColoringCreate(MPI_Comm comm,PetscInt ncolors,PetscInt n,const
 
     Input Parameters
 .   partitioning - a partitioning as generated by MatPartitioningApply()
+                   or MatPartitioningApplyND()
 
     Output Parameter:
 .   is - on each processor the index set that defines the global numbers
@@ -358,19 +458,29 @@ PetscErrorCode  ISColoringCreate(MPI_Comm comm,PetscInt ncolors,PetscInt n,const
 PetscErrorCode  ISPartitioningToNumbering(IS part,IS *is)
 {
   MPI_Comm       comm;
+  IS             ndorder;
   PetscInt       i,np,npt,n,*starts = NULL,*sums = NULL,*lsizes = NULL,*newi = NULL;
   const PetscInt *indices = NULL;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = PetscObjectGetComm((PetscObject)part,&comm);CHKERRQ(ierr);
+  PetscValidHeaderSpecific(part,IS_CLASSID,1);
+  PetscValidPointer(is,2);
+  /* see if the partitioning comes from nested dissection */
+  ierr = PetscObjectQuery((PetscObject)part,"_petsc_matpartitioning_ndorder",(PetscObject*)&ndorder);CHKERRQ(ierr);
+  if (ndorder) {
+    ierr = PetscObjectReference((PetscObject)ndorder);CHKERRQ(ierr);
+    *is  = ndorder;
+    PetscFunctionReturn(0);
+  }
 
+  ierr = PetscObjectGetComm((PetscObject)part,&comm);CHKERRQ(ierr);
   /* count the number of partitions, i.e., virtual processors */
   ierr = ISGetLocalSize(part,&n);CHKERRQ(ierr);
   ierr = ISGetIndices(part,&indices);CHKERRQ(ierr);
   np   = 0;
   for (i=0; i<n; i++) np = PetscMax(np,indices[i]);
-  ierr = MPI_Allreduce(&np,&npt,1,MPIU_INT,MPI_MAX,comm);CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(&np,&npt,1,MPIU_INT,MPI_MAX,comm);CHKERRQ(ierr);
   np   = npt+1; /* so that it looks like a MPI_Comm_size output */
 
   /*
@@ -381,7 +491,7 @@ PetscErrorCode  ISPartitioningToNumbering(IS part,IS *is)
   ierr = PetscMalloc3(np,&lsizes,np,&starts,np,&sums);CHKERRQ(ierr);
   ierr = PetscMemzero(lsizes,np*sizeof(PetscInt));CHKERRQ(ierr);
   for (i=0; i<n; i++) lsizes[indices[i]]++;
-  ierr = MPI_Allreduce(lsizes,sums,np,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(lsizes,sums,np,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
   ierr = MPI_Scan(lsizes,starts,np,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
   for (i=0; i<np; i++) starts[i] -= lsizes[i];
   for (i=1; i<np; i++) {
@@ -402,8 +512,6 @@ PetscErrorCode  ISPartitioningToNumbering(IS part,IS *is)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISPartitioningCount"
 /*@
     ISPartitioningCount - Takes a ISPartitioning and determines the number of
     resulting elements on each (partition) process
@@ -411,7 +519,8 @@ PetscErrorCode  ISPartitioningToNumbering(IS part,IS *is)
     Collective on IS
 
     Input Parameters:
-+   partitioning - a partitioning as generated by MatPartitioningApply()
++   partitioning - a partitioning as generated by MatPartitioningApply() or
+                   MatPartitioningApplyND()
 -   len - length of the array count, this is the total number of partitions
 
     Output Parameter:
@@ -426,10 +535,11 @@ PetscErrorCode  ISPartitioningToNumbering(IS part,IS *is)
         of count) is the size of the communicator associated with IS,
         but it can be set by MatPartitioningSetNParts. The resulting array
         of lengths can for instance serve as input of PCBJacobiSetTotalBlocks.
-
+        If the partitioning has been obtained by MatPartitioningApplyND(),
+        the returned count does not include the separators.
 
 .seealso: MatPartitioningCreate(), AOCreateBasic(), ISPartitioningToNumbering(),
-        MatPartitioningSetNParts(), MatPartitioningApply()
+        MatPartitioningSetNParts(), MatPartitioningApply(), MatPartitioningApplyND()
 
 @*/
 PetscErrorCode  ISPartitioningCount(IS part,PetscInt len,PetscInt count[])
@@ -455,7 +565,7 @@ PetscErrorCode  ISPartitioningCount(IS part,PetscInt len,PetscInt count[])
   {
     PetscInt np = 0,npt;
     for (i=0; i<n; i++) np = PetscMax(np,indices[i]);
-    ierr = MPI_Allreduce(&np,&npt,1,MPIU_INT,MPI_MAX,comm);CHKERRQ(ierr);
+    ierr = MPIU_Allreduce(&np,&npt,1,MPIU_INT,MPI_MAX,comm);CHKERRQ(ierr);
     np   = npt+1; /* so that it looks like a MPI_Comm_size output */
     if (np > len) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_SIZ,"Length of count array %D is less than number of partitions %D",len,np);
   }
@@ -467,16 +577,16 @@ PetscErrorCode  ISPartitioningCount(IS part,PetscInt len,PetscInt count[])
         starts - global number of first element in each partition on this processor
   */
   ierr = PetscCalloc1(len,&lsizes);CHKERRQ(ierr);
-  for (i=0; i<n; i++) lsizes[indices[i]]++;
+  for (i=0; i<n; i++) {
+    if (indices[i] > -1) lsizes[indices[i]]++;
+  }
   ierr = ISRestoreIndices(part,&indices);CHKERRQ(ierr);
   ierr = PetscMPIIntCast(len,&npp);CHKERRQ(ierr);
-  ierr = MPI_Allreduce(lsizes,count,npp,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(lsizes,count,npp,MPIU_INT,MPI_SUM,comm);CHKERRQ(ierr);
   ierr = PetscFree(lsizes);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISAllGather"
 /*@
     ISAllGather - Given an index set (IS) on each processor, generates a large
     index set (same on each processor) by concatenating together each
@@ -535,7 +645,10 @@ PetscErrorCode  ISAllGather(IS is,IS *isout)
     ierr       = PetscMPIIntCast(n,&nn);CHKERRQ(ierr);
     ierr       = MPI_Allgather(&nn,1,MPI_INT,sizes,1,MPI_INT,comm);CHKERRQ(ierr);
     offsets[0] = 0;
-    for (i=1; i<size; i++) offsets[i] = offsets[i-1] + sizes[i-1];
+    for (i=1; i<size; i++) {
+      PetscInt s = offsets[i-1] + sizes[i-1];
+      ierr = PetscMPIIntCast(s,&offsets[i]);CHKERRQ(ierr);
+    }
     N = offsets[size-1] + sizes[size-1];
 
     ierr = PetscMalloc1(N,&indices);CHKERRQ(ierr);
@@ -549,8 +662,6 @@ PetscErrorCode  ISAllGather(IS is,IS *isout)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISAllGatherColors"
 /*@C
     ISAllGatherColors - Given a a set of colors on each processor, generates a large
     set (same on each processor) by concatenating together each processors colors
@@ -603,8 +714,6 @@ PetscErrorCode  ISAllGatherColors(MPI_Comm comm,PetscInt n,ISColoringValue *lind
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISComplement"
 /*@
     ISComplement - Given an index set (IS) generates the complement index set. That is all
        all indices that are NOT in the given set.
@@ -619,7 +728,8 @@ PetscErrorCode  ISAllGatherColors(MPI_Comm comm,PetscInt n,ISColoringValue *lind
     Output Parameter:
 .   isout - the complement
 
-    Notes:  The communicator for this new IS is the same as for the input IS
+    Notes:
+    The communicator for this new IS is the same as for the input IS
 
       For a parallel IS, this will generate the local part of the complement on each process
 
