@@ -2,8 +2,6 @@
 #include <petsc/private/isimpl.h>                    /*I "petscis.h"  I*/
 #include <petscbt.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "ISDifference"
 /*@
    ISDifference - Computes the difference between two index sets.
 
@@ -91,8 +89,6 @@ PetscErrorCode  ISDifference(IS is1,IS is2,IS *isout)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISSum"
 /*@
    ISSum - Computes the sum (union) of two index sets.
 
@@ -217,8 +213,6 @@ PetscErrorCode  ISSum(IS is1,IS is2,IS *is3)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISExpand"
 /*@
    ISExpand - Computes the union of two index sets, by concatenating 2 lists and
    removing duplicates.
@@ -256,10 +250,13 @@ PetscErrorCode ISExpand(IS is1,IS is2,IS *isout)
   MPI_Comm       comm;
 
   PetscFunctionBegin;
-  PetscValidHeaderSpecific(is1,IS_CLASSID,1);
-  PetscValidHeaderSpecific(is2,IS_CLASSID,2);
+  if (is1) PetscValidHeaderSpecific(is1,IS_CLASSID,1);
+  if (is2) PetscValidHeaderSpecific(is2,IS_CLASSID,2);
   PetscValidPointer(isout,3);
 
+  if (!is1 && !is2) SETERRQ(PETSC_COMM_SELF, PETSC_ERR_ARG_WRONG, "Both arguments cannot be NULL");
+  if (!is1) {ierr = ISDuplicate(is2, isout);CHKERRQ(ierr);PetscFunctionReturn(0);}
+  if (!is2) {ierr = ISDuplicate(is1, isout);CHKERRQ(ierr);PetscFunctionReturn(0);}
   ierr = ISGetIndices(is1,&i1);CHKERRQ(ierr);
   ierr = ISGetLocalSize(is1,&n1);CHKERRQ(ierr);
   ierr = ISGetIndices(is2,&i2);CHKERRQ(ierr);
@@ -305,8 +302,128 @@ PetscErrorCode ISExpand(IS is1,IS is2,IS *isout)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "ISConcatenate"
+/*@
+   ISIntersect - Computes the union of two index sets, by concatenating 2 lists, sorting,
+   and finding duplicates.
+
+   Collective on IS
+
+   Input Parameter:
++  is1 - first index set
+-  is2 - second index set
+
+   Output Parameters:
+.  isout - the sorted intersection of is1 and is2
+
+   Notes:
+   Negative values are removed from the lists. This requires O(min(is1,is2))
+   memory and O(max(is1,is2)log(max(is1,is2))) work
+
+   The IS's do not need to be sorted.
+
+   Level: intermediate
+
+.seealso: ISDestroy(), ISView(), ISDifference(), ISSum(), ISExpand()
+
+   Concepts: index sets^intersection
+   Concepts: IS^intersection
+
+@*/
+PetscErrorCode ISIntersect(IS is1,IS is2,IS *isout)
+{
+  PetscErrorCode ierr;
+  PetscInt       i,n1,n2,nout,*iout;
+  const PetscInt *i1,*i2;
+  IS             is1sorted = NULL, is2sorted = NULL;
+  PetscBool      sorted;
+  MPI_Comm       comm;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(is1,IS_CLASSID,1);
+  PetscValidHeaderSpecific(is2,IS_CLASSID,2);
+  PetscValidPointer(isout,3);
+
+  ierr = ISGetLocalSize(is1,&n1);CHKERRQ(ierr);
+  ierr = ISGetLocalSize(is2,&n2);CHKERRQ(ierr);
+  if (n1 < n2) {
+    IS tempis = is1;
+    int ntemp = n1;
+
+    is1 = is2;
+    is2 = tempis;
+    n1  = n2;
+    n2  = ntemp;
+  }
+  ierr = ISSorted(is1,&sorted);CHKERRQ(ierr);
+  if (!sorted) {
+    ierr = ISDuplicate(is1,&is1sorted);CHKERRQ(ierr);
+    ierr = ISSort(is1sorted);CHKERRQ(ierr);
+    ierr = ISGetIndices(is1sorted,&i1);CHKERRQ(ierr);
+  } else {
+    is1sorted = is1;
+    ierr = PetscObjectReference((PetscObject)is1);CHKERRQ(ierr);
+    ierr = ISGetIndices(is1,&i1);CHKERRQ(ierr);
+  }
+  ierr = ISSorted(is2,&sorted);CHKERRQ(ierr);
+  if (!sorted) {
+    ierr = ISDuplicate(is2,&is2sorted);CHKERRQ(ierr);
+    ierr = ISSort(is2sorted);CHKERRQ(ierr);
+    ierr = ISGetIndices(is2sorted,&i2);CHKERRQ(ierr);
+  } else {
+    is2sorted = is2;
+    ierr = PetscObjectReference((PetscObject)is2);CHKERRQ(ierr);
+    ierr = ISGetIndices(is2,&i2);CHKERRQ(ierr);
+  }
+
+  ierr = PetscMalloc1(n2,&iout);CHKERRQ(ierr);
+
+  for (i = 0, nout = 0; i < n2; i++) {
+    PetscInt key = i2[i];
+    PetscInt loc;
+
+    ierr = ISLocate(is1sorted,key,&loc);CHKERRQ(ierr);
+    if (loc >= 0) {
+      if (!nout || iout[nout-1] < key) {
+        iout[nout++] = key;
+      }
+    }
+  }
+  ierr = PetscRealloc(sizeof (PetscInt) * (size_t) nout,&iout);CHKERRQ(ierr);
+
+  /* create the new IS containing the sum */
+  ierr = PetscObjectGetComm((PetscObject)is1,&comm);CHKERRQ(ierr);
+  ierr = ISCreateGeneral(comm,nout,iout,PETSC_OWN_POINTER,isout);CHKERRQ(ierr);
+
+  ierr = ISRestoreIndices(is2sorted,&i2);CHKERRQ(ierr);
+  ierr = ISDestroy(&is2sorted);CHKERRQ(ierr);
+  ierr = ISRestoreIndices(is1sorted,&i1);CHKERRQ(ierr);
+  ierr = ISDestroy(&is1sorted);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode ISIntersect_Caching_Internal(IS is1, IS is2, IS *isect)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  *isect = NULL;
+  if (is2 && is1) {
+    char           composeStr[33] = {0};
+    PetscObjectId  is2id;
+
+    ierr = PetscObjectGetId((PetscObject)is2,&is2id);CHKERRQ(ierr);
+    ierr = PetscSNPrintf(composeStr,32,"ISIntersect_Caching_%x",is2id);CHKERRQ(ierr);
+    ierr = PetscObjectQuery((PetscObject) is1, composeStr, (PetscObject *) isect);CHKERRQ(ierr);
+    if (*isect == NULL) {
+      ierr = ISIntersect(is1, is2, isect);CHKERRQ(ierr);
+      ierr = PetscObjectCompose((PetscObject) is1, composeStr, (PetscObject) *isect);CHKERRQ(ierr);
+    } else {
+      ierr = PetscObjectReference((PetscObject) *isect);CHKERRQ(ierr);
+    }
+  }
+  PetscFunctionReturn(0);
+}
+
 /*@
    ISConcatenate - Forms a new IS by locally concatenating the indices from an IS list without reordering.
 
@@ -318,12 +435,11 @@ PetscErrorCode ISExpand(IS is1,IS is2,IS *isout)
 .  len     - size of islist array (nonnegative)
 -  islist  - array of index sets
 
-
-
    Output Parameters:
 .  isout   - The concatenated index set; empty, if len == 0.
 
-   Notes: The semantics of calling this on comm imply that the comms of the members if islist also contain this rank.
+   Notes:
+    The semantics of calling this on comm imply that the comms of the members if islist also contain this rank.
 
    Level: intermediate
 
@@ -341,9 +457,9 @@ PetscErrorCode ISConcatenate(MPI_Comm comm, PetscInt len, const IS islist[], IS
   PetscInt *idx;
 
   PetscFunctionBegin;
-  PetscValidPointer(islist,2);
+  PetscValidPointer(islist,3);
 #if defined(PETSC_USE_DEBUG)
-  for (i = 0; i < len; ++i) PetscValidHeaderSpecific(islist[i], IS_CLASSID, 1);
+  for (i = 0; i < len; ++i) if (islist[i]) PetscValidHeaderSpecific(islist[i], IS_CLASSID, 3);
 #endif
   PetscValidPointer(isout, 4);
   if (!len) {
@@ -353,17 +469,21 @@ PetscErrorCode ISConcatenate(MPI_Comm comm, PetscInt len, const IS islist[], IS
   if (len < 0) SETERRQ1(PETSC_COMM_SELF, PETSC_ERR_ARG_WRONG, "Negative array length: %D", len);
   N = 0;
   for (i = 0; i < len; ++i) {
-    ierr = ISGetLocalSize(islist[i], &n);CHKERRQ(ierr);
-    N   += n;
+    if (islist[i]) {
+      ierr = ISGetLocalSize(islist[i], &n);CHKERRQ(ierr);
+      N   += n;
+    }
   }
-  ierr = PetscMalloc(sizeof(PetscInt)*N, &idx);CHKERRQ(ierr);
+  ierr = PetscMalloc1(N, &idx);CHKERRQ(ierr);
   N = 0;
   for (i = 0; i < len; ++i) {
-    ierr = ISGetLocalSize(islist[i], &n);CHKERRQ(ierr);
-    ierr = ISGetIndices(islist[i], &iidx);CHKERRQ(ierr);
-    ierr = PetscMemcpy(idx+N,iidx, sizeof(PetscInt)*n);CHKERRQ(ierr);
-    ierr = ISRestoreIndices(islist[i], &iidx);CHKERRQ(ierr);
-    N   += n;
+    if (islist[i]) {
+      ierr = ISGetLocalSize(islist[i], &n);CHKERRQ(ierr);
+      ierr = ISGetIndices(islist[i], &iidx);CHKERRQ(ierr);
+      ierr = PetscMemcpy(idx+N,iidx, sizeof(PetscInt)*n);CHKERRQ(ierr);
+      ierr = ISRestoreIndices(islist[i], &iidx);CHKERRQ(ierr);
+      N   += n;
+    }
   }
   ierr = ISCreateGeneral(comm, N, idx, PETSC_OWN_POINTER, isout);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -403,8 +523,6 @@ PetscErrorCode ISConcatenate(MPI_Comm comm, PetscInt len, const IS islist[], IS
 
 .seealso ISPairToList()
 @*/
-#undef  __FUNCT__
-#define __FUNCT__ "ISListToPair"
 PetscErrorCode ISListToPair(MPI_Comm comm, PetscInt listlen, IS islist[], IS *xis, IS *yis)
 {
   PetscErrorCode ierr;
@@ -464,8 +582,6 @@ PetscErrorCode ISListToPair(MPI_Comm comm, PetscInt listlen, IS islist[], IS *xi
 
 .seealso ISListToPair()
  @*/
-#undef  __FUNCT__
-#define __FUNCT__ "ISPairToList"
 PetscErrorCode ISPairToList(IS xis, IS yis, PetscInt *listlen, IS **islist)
 {
   PetscErrorCode ierr;
@@ -504,14 +620,14 @@ PetscErrorCode ISPairToList(IS xis, IS yis, PetscInt *listlen, IS **islist)
     lhigh = PetscMax(lhigh,colors[lstart]);
     ++lcount;
   }
-  ierr     = MPI_Allreduce(&llow,&low,1,MPI_INT,MPI_MIN,comm);CHKERRQ(ierr);
-  ierr     = MPI_Allreduce(&lhigh,&high,1,MPI_INT,MPI_MAX,comm);CHKERRQ(ierr);
+  ierr     = MPIU_Allreduce(&llow,&low,1,MPI_INT,MPI_MIN,comm);CHKERRQ(ierr);
+  ierr     = MPIU_Allreduce(&lhigh,&high,1,MPI_INT,MPI_MAX,comm);CHKERRQ(ierr);
   *listlen = 0;
   if (low <= high) {
     if (lcount > 0) {
       *listlen = lcount;
       if (!*islist) {
-        ierr = PetscMalloc(sizeof(IS)*lcount, islist);CHKERRQ(ierr);
+        ierr = PetscMalloc1(lcount, islist);CHKERRQ(ierr);
       }
     }
     /*
@@ -537,7 +653,7 @@ PetscErrorCode ISPairToList(IS xis, IS yis, PetscInt *listlen, IS **islist)
       }
       color = (PetscMPIInt)(colors[lstart] == l);
       /* Check whether a proper subcommunicator exists. */
-      ierr = MPI_Allreduce(&color,&subsize,1,MPI_INT,MPI_SUM,comm);CHKERRQ(ierr);
+      ierr = MPIU_Allreduce(&color,&subsize,1,MPI_INT,MPI_SUM,comm);CHKERRQ(ierr);
 
       if (subsize == 1) subcomm = PETSC_COMM_SELF;
       else if (subsize == size) subcomm = comm;
@@ -594,13 +710,11 @@ PetscErrorCode ISPairToList(IS xis, IS yis, PetscInt *listlen, IS **islist)
 
 .seealso ISLocalToGlobalMapping
  @*/
-#undef  __FUNCT__
-#define __FUNCT__ "ISEmbed"
 PetscErrorCode ISEmbed(IS a, IS b, PetscBool drop, IS *c)
 {
   PetscErrorCode             ierr;
   ISLocalToGlobalMapping     ltog;
-  ISGlobalToLocalMappingType gtoltype = IS_GTOLM_DROP;
+  ISGlobalToLocalMappingMode gtoltype = IS_GTOLM_DROP;
   PetscInt                   alen, clen, *cindices, *cindices2;
   const PetscInt             *aindices;
 
@@ -613,7 +727,8 @@ PetscErrorCode ISEmbed(IS a, IS b, PetscBool drop, IS *c)
   ierr = ISGetIndices(a, &aindices);CHKERRQ(ierr);
   ierr = PetscMalloc1(alen, &cindices);CHKERRQ(ierr);
   if (!drop) gtoltype = IS_GTOLM_MASK;
-  ISGlobalToLocalMappingApply(ltog,gtoltype,alen,aindices,&clen,cindices);CHKERRQ(ierr);
+  ierr = ISGlobalToLocalMappingApply(ltog,gtoltype,alen,aindices,&clen,cindices);CHKERRQ(ierr);
+  ierr = ISLocalToGlobalMappingDestroy(&ltog);CHKERRQ(ierr);
   if (clen != alen) {
     cindices2 = cindices;
     ierr      = PetscMalloc1(clen, &cindices);CHKERRQ(ierr);
@@ -647,8 +762,6 @@ PetscErrorCode ISEmbed(IS a, IS b, PetscBool drop, IS *c)
 
 .seealso ISLocalToGlobalMapping, ISSort(), PetscIntSortWithPermutation()
  @*/
-#undef  __FUNCT__
-#define __FUNCT__ "ISSortPermutation"
 PetscErrorCode ISSortPermutation(IS f,PetscBool always,IS *h)
 {
   PetscErrorCode  ierr;
