@@ -35,15 +35,16 @@ struct _n_PetscSFBasicPack {
   size_t           unitbytes;   /* Number of bytes in a unit */
   PetscInt         bs;          /* Number of basic units in a unit */
   const void       *key;        /* Array used as key for operation */
-  char             *root;       /* Packed root data, contiguous by leaf rank */
-  char             *leaf;       /* Packed leaf data, contiguous by root rank */
+  char             **root;      /* Packed root data, indexed by leaf rank */
+  char             **leaf;      /* Packed leaf data, indexed by root rank */
   MPI_Request      *requests;   /* Array of root requests followed by leaf requests */
   PetscSFBasicPack next;
 };
 
 typedef struct {
   PetscMPIInt      tag;
-  PetscInt         niranks;     /* Number of incoming ranks (ranks accessing my roots) */
+  PetscMPIInt      niranks;     /* Number of incoming ranks (ranks accessing my roots) */
+  PetscMPIInt      ndiranks;    /* Number of incoming ranks (ranks accessing my roots) in distinguished set */
   PetscMPIInt      *iranks;     /* Array of ranks that reference my roots */
   PetscInt         itotal;      /* Total number of graph edges referencing my roots */
   PetscInt         *ioffset;    /* Array of length niranks+1 holding offset in irootloc[] for each rank */
@@ -52,11 +53,13 @@ typedef struct {
   PetscSFBasicPack inuse;       /* Buffers being used for transactions that have not yet completed */
 } PetscSF_Basic;
 
-#if !defined(PETSC_HAVE_MPI_TYPE_DUP) /* Danger: type is not reference counted; subject to ABA problem */
-PETSC_STATIC_INLINE PetscErrorCode MPI_Type_dup(MPI_Datatype datatype,MPI_Datatype *newtype)
+#if !defined(PETSC_HAVE_MPI_TYPE_DUP)
+PETSC_STATIC_INLINE int MPI_Type_dup(MPI_Datatype datatype,MPI_Datatype *newtype)
 {
-  *newtype = datatype;
-  return 0;
+  int ierr;
+  ierr = MPI_Type_contiguous(1,datatype,newtype); if (ierr) return ierr;
+  ierr = MPI_Type_commit(newtype); if (ierr) return ierr;
+  return MPI_SUCCESS;
 }
 #endif
 
@@ -528,53 +531,76 @@ DEF_Block(int,6)
 DEF_Block(int,7)
 DEF_Block(int,8)
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFSetUp_Basic"
 static PetscErrorCode PetscSFSetUp_Basic(PetscSF sf)
 {
   PetscSF_Basic *bas = (PetscSF_Basic*)sf->data;
   PetscErrorCode ierr;
   PetscInt *rlengths,*ilengths,i;
+  PetscMPIInt rank,niranks,*iranks;
   MPI_Comm comm;
+  MPI_Group group;
   MPI_Request *rootreqs,*leafreqs;
 
   PetscFunctionBegin;
+  ierr = MPI_Comm_group(PETSC_COMM_SELF,&group);CHKERRQ(ierr);
+  ierr = PetscSFSetUpRanks(sf,group);CHKERRQ(ierr);
+  ierr = MPI_Group_free(&group);CHKERRQ(ierr);
   ierr = PetscObjectGetComm((PetscObject)sf,&comm);CHKERRQ(ierr);
   ierr = PetscObjectGetNewTag((PetscObject)sf,&bas->tag);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
   /*
    * Inform roots about how many leaves and from which ranks
    */
   ierr = PetscMalloc1(sf->nranks,&rlengths);CHKERRQ(ierr);
-  /* Determine number, sending ranks, and length of incoming  */
+  /* Determine number, sending ranks, and length of incoming */
   for (i=0; i<sf->nranks; i++) {
     rlengths[i] = sf->roffset[i+1] - sf->roffset[i]; /* Number of roots referenced by my leaves; for rank sf->ranks[i] */
   }
-  ierr = PetscCommBuildTwoSided(comm,1,MPIU_INT,sf->nranks,sf->ranks,rlengths,&bas->niranks,&bas->iranks,(void**)&ilengths);CHKERRQ(ierr);
+  ierr = PetscCommBuildTwoSided(comm,1,MPIU_INT,sf->nranks-sf->ndranks,sf->ranks+sf->ndranks,rlengths+sf->ndranks,&niranks,&iranks,(void**)&ilengths);CHKERRQ(ierr);
+
+  /* Partition into distinguished and non-distinguished incoming ranks */
+  bas->ndiranks = sf->ndranks;
+  bas->niranks = bas->ndiranks + niranks;
+  ierr = PetscMalloc2(bas->niranks,&bas->iranks,bas->niranks+1,&bas->ioffset);CHKERRQ(ierr);
+  bas->ioffset[0] = 0;
+  for (i=0; i<bas->ndiranks; i++) {
+    bas->iranks[i] = sf->ranks[i];
+    bas->ioffset[i+1] = bas->ioffset[i] + rlengths[i];
+  }
+  if (bas->ndiranks > 1 || (bas->ndiranks == 1 && bas->iranks[0] != rank)) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Broken setup for shared ranks");
+  for ( ; i<bas->niranks; i++) {
+    bas->iranks[i] = iranks[i-bas->ndiranks];
+    bas->ioffset[i+1] = bas->ioffset[i] + ilengths[i-bas->ndiranks];
+  }
+  bas->itotal = bas->ioffset[i];
   ierr = PetscFree(rlengths);CHKERRQ(ierr);
+  ierr = PetscFree(iranks);CHKERRQ(ierr);
+  ierr = PetscFree(ilengths);CHKERRQ(ierr);
 
   /* Send leaf identities to roots */
-  for (i=0,bas->itotal=0; i<bas->niranks; i++) bas->itotal += ilengths[i];
-  ierr = PetscMalloc2(bas->niranks+1,&bas->ioffset,bas->itotal,&bas->irootloc);CHKERRQ(ierr);
-  ierr = PetscMalloc2(bas->niranks,&rootreqs,sf->nranks,&leafreqs);CHKERRQ(ierr);
-  bas->ioffset[0] = 0;
-  for (i=0; i<bas->niranks; i++) {
-    bas->ioffset[i+1] = bas->ioffset[i] + ilengths[i];
-    ierr = MPI_Irecv(bas->irootloc+bas->ioffset[i],ilengths[i],MPIU_INT,bas->iranks[i],bas->tag,comm,&rootreqs[i]);CHKERRQ(ierr);
+  ierr = PetscMalloc1(bas->itotal,&bas->irootloc);CHKERRQ(ierr);
+  ierr = PetscMalloc2(bas->niranks-bas->ndiranks,&rootreqs,sf->nranks-sf->ndranks,&leafreqs);CHKERRQ(ierr);
+  for (i=bas->ndiranks; i<bas->niranks; i++) {
+    ierr = MPI_Irecv(bas->irootloc+bas->ioffset[i],bas->ioffset[i+1]-bas->ioffset[i],MPIU_INT,bas->iranks[i],bas->tag,comm,&rootreqs[i-bas->ndiranks]);CHKERRQ(ierr);
   }
   for (i=0; i<sf->nranks; i++) {
     PetscMPIInt npoints;
     ierr = PetscMPIIntCast(sf->roffset[i+1] - sf->roffset[i],&npoints);CHKERRQ(ierr);
-    ierr = MPI_Isend(sf->rremote+sf->roffset[i],npoints,MPIU_INT,sf->ranks[i],bas->tag,comm,&leafreqs[i]);CHKERRQ(ierr);
+    if (i < sf->ndranks) {
+      if (sf->ranks[i] != rank) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Cannot interpret distinguished leaf rank");
+      if (bas->iranks[0] != rank) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Cannot interpret distinguished root rank");
+      if (npoints != bas->ioffset[1]-bas->ioffset[0]) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Distinguished rank exchange has mismatched lengths");
+      ierr = PetscMemcpy(bas->irootloc+bas->ioffset[0],sf->rremote+sf->roffset[i],npoints*sizeof(bas->irootloc[0]));CHKERRQ(ierr);
+      continue;
+    }
+    ierr = MPI_Isend(sf->rremote+sf->roffset[i],npoints,MPIU_INT,sf->ranks[i],bas->tag,comm,&leafreqs[i-sf->ndranks]);CHKERRQ(ierr);
   }
-  ierr = MPI_Waitall(bas->niranks,rootreqs,MPI_STATUSES_IGNORE);CHKERRQ(ierr);
-  ierr = MPI_Waitall(sf->nranks,leafreqs,MPI_STATUSES_IGNORE);CHKERRQ(ierr);
-  ierr = PetscFree(ilengths);CHKERRQ(ierr);
+  ierr = MPI_Waitall(bas->niranks-bas->ndiranks,rootreqs,MPI_STATUSES_IGNORE);CHKERRQ(ierr);
+  ierr = MPI_Waitall(sf->nranks-sf->ndranks,leafreqs,MPI_STATUSES_IGNORE);CHKERRQ(ierr);
   ierr = PetscFree2(rootreqs,leafreqs);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFBasicPackTypeSetup"
 static PetscErrorCode PetscSFBasicPackTypeSetup(PetscSFBasicPack link,MPI_Datatype unit)
 {
   PetscErrorCode ierr;
@@ -636,8 +662,9 @@ static PetscErrorCode PetscSFBasicPackTypeSetup(PetscSFBasicPack link,MPI_Dataty
     link->unitbytes *= nPetscComplexContig;
 #endif
   } else {
-    PetscMPIInt bytes;
-    ierr = MPI_Type_size(unit,&bytes);CHKERRQ(ierr);
+    MPI_Aint lb,bytes;
+    ierr = MPI_Type_get_extent(unit,&lb,&bytes);CHKERRQ(ierr);
+    if (lb != 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Datatype with nonzero lower bound %ld\n",(long)lb);
     if (bytes % sizeof(int)) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"No support for type size not divisible by %D",sizeof(int));
     switch (bytes / sizeof(int)) {
     case 1: PackInit_block_int_1(link); break;
@@ -655,8 +682,6 @@ static PetscErrorCode PetscSFBasicPackTypeSetup(PetscSFBasicPack link,MPI_Dataty
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFBasicPackGetUnpackOp"
 static PetscErrorCode PetscSFBasicPackGetUnpackOp(PetscSF sf,PetscSFBasicPack link,MPI_Op op,void (**UnpackOp)(PetscInt,PetscInt,const PetscInt*,void*,const void*))
 {
   PetscFunctionBegin;
@@ -674,11 +699,9 @@ static PetscErrorCode PetscSFBasicPackGetUnpackOp(PetscSF sf,PetscSFBasicPack li
   else if (op == MPI_BXOR) *UnpackOp = link->UnpackBXOR;
   else if (op == MPI_MAXLOC) *UnpackOp = link->UnpackMaxloc;
   else if (op == MPI_MINLOC) *UnpackOp = link->UnpackMinloc;
-  else SETERRQ(PetscObjectComm((PetscObject)sf),PETSC_ERR_SUP,"No support for MPI_Op");
+  else *UnpackOp = NULL;
   PetscFunctionReturn(0);
 }
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFBasicPackGetFetchAndOp"
 static PetscErrorCode PetscSFBasicPackGetFetchAndOp(PetscSF sf,PetscSFBasicPack link,MPI_Op op,void (**FetchAndOp)(PetscInt,PetscInt,const PetscInt*,void*,void*))
 {
   PetscFunctionBegin;
@@ -700,64 +723,56 @@ static PetscErrorCode PetscSFBasicPackGetFetchAndOp(PetscSF sf,PetscSFBasicPack
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFBasicPackGetReqs"
 static PetscErrorCode PetscSFBasicPackGetReqs(PetscSF sf,PetscSFBasicPack link,MPI_Request **rootreqs,MPI_Request **leafreqs)
 {
   PetscSF_Basic *bas = (PetscSF_Basic*)sf->data;
 
   PetscFunctionBegin;
   if (rootreqs) *rootreqs = link->requests;
-  if (leafreqs) *leafreqs = link->requests + bas->niranks;
+  if (leafreqs) *leafreqs = link->requests + (bas->niranks - bas->ndiranks);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFBasicPackWaitall"
 static PetscErrorCode PetscSFBasicPackWaitall(PetscSF sf,PetscSFBasicPack link)
 {
   PetscSF_Basic  *bas = (PetscSF_Basic*)sf->data;
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
-  ierr = MPI_Waitall(bas->niranks+sf->nranks,link->requests,MPI_STATUSES_IGNORE);CHKERRQ(ierr);
+  ierr = MPI_Waitall(bas->niranks+sf->nranks-(bas->ndiranks+sf->ndranks),link->requests,MPI_STATUSES_IGNORE);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFBasicGetRootInfo"
-static PetscErrorCode PetscSFBasicGetRootInfo(PetscSF sf,PetscInt *nrootranks,const PetscMPIInt **rootranks,const PetscInt **rootoffset,const PetscInt **rootloc)
+static PetscErrorCode PetscSFBasicGetRootInfo(PetscSF sf,PetscInt *nrootranks,PetscInt *ndrootranks,const PetscMPIInt **rootranks,const PetscInt **rootoffset,const PetscInt **rootloc)
 {
   PetscSF_Basic *bas = (PetscSF_Basic*)sf->data;
 
   PetscFunctionBegin;
-  if (nrootranks) *nrootranks = bas->niranks;
-  if (rootranks)  *rootranks  = bas->iranks;
-  if (rootoffset) *rootoffset = bas->ioffset;
-  if (rootloc)    *rootloc    = bas->irootloc;
+  if (nrootranks)  *nrootranks  = bas->niranks;
+  if (ndrootranks) *ndrootranks = bas->ndiranks;
+  if (rootranks)   *rootranks   = bas->iranks;
+  if (rootoffset)  *rootoffset  = bas->ioffset;
+  if (rootloc)     *rootloc     = bas->irootloc;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFBasicGetLeafInfo"
-static PetscErrorCode PetscSFBasicGetLeafInfo(PetscSF sf,PetscInt *nleafranks,const PetscMPIInt **leafranks,const PetscInt **leafoffset,const PetscInt **leafloc)
+static PetscErrorCode PetscSFBasicGetLeafInfo(PetscSF sf,PetscInt *nleafranks,PetscInt *ndleafranks,const PetscMPIInt **leafranks,const PetscInt **leafoffset,const PetscInt **leafloc)
 {
   PetscFunctionBegin;
-  if (nleafranks) *nleafranks = sf->nranks;
-  if (leafranks)  *leafranks  = sf->ranks;
-  if (leafoffset) *leafoffset = sf->roffset;
-  if (leafloc)    *leafloc    = sf->rmine;
+  if (nleafranks)  *nleafranks  = sf->nranks;
+  if (ndleafranks) *ndleafranks = sf->ndranks;
+  if (leafranks)   *leafranks   = sf->ranks;
+  if (leafoffset)  *leafoffset  = sf->roffset;
+  if (leafloc)     *leafloc     = sf->rmine;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFBasicGetPack"
 static PetscErrorCode PetscSFBasicGetPack(PetscSF sf,MPI_Datatype unit,const void *key,PetscSFBasicPack *mylink)
 {
   PetscSF_Basic    *bas = (PetscSF_Basic*)sf->data;
   PetscErrorCode   ierr;
   PetscSFBasicPack link,*p;
-  PetscInt         nrootranks,nleafranks;
+  PetscInt         nrootranks,ndrootranks,nleafranks,ndleafranks,i;
   const PetscInt   *rootoffset,*leafoffset;
 
   PetscFunctionBegin;
@@ -772,12 +787,23 @@ static PetscErrorCode PetscSFBasicGetPack(PetscSF sf,MPI_Datatype unit,const voi
   }
 
   /* Create new composite types for each send rank */
-  ierr = PetscSFBasicGetRootInfo(sf,&nrootranks,NULL,&rootoffset,NULL);CHKERRQ(ierr);
-  ierr = PetscSFBasicGetLeafInfo(sf,&nleafranks,NULL,&leafoffset,NULL);CHKERRQ(ierr);
+  ierr = PetscSFBasicGetRootInfo(sf,&nrootranks,&ndrootranks,NULL,&rootoffset,NULL);CHKERRQ(ierr);
+  ierr = PetscSFBasicGetLeafInfo(sf,&nleafranks,&ndleafranks,NULL,&leafoffset,NULL);CHKERRQ(ierr);
   ierr = PetscNew(&link);CHKERRQ(ierr);
   ierr = PetscSFBasicPackTypeSetup(link,unit);CHKERRQ(ierr);
-  ierr = PetscMalloc2(rootoffset[nrootranks]*link->unitbytes,&link->root,leafoffset[nleafranks]*link->unitbytes,&link->leaf);CHKERRQ(ierr);
-  ierr = PetscMalloc1(nrootranks+nleafranks,&link->requests);CHKERRQ(ierr);
+  ierr = PetscMalloc2(nrootranks,&link->root,nleafranks,&link->leaf);CHKERRQ(ierr);
+  for (i=0; i<nrootranks; i++) {
+    ierr = PetscMalloc((rootoffset[i+1]-rootoffset[i])*link->unitbytes,&link->root[i]);CHKERRQ(ierr);
+  }
+  for (i=0; i<nleafranks; i++) {
+    if (i < ndleafranks) {      /* Leaf buffers for distinguished ranks are pointers directly into root buffers */
+      if (ndrootranks != 1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_PLIB,"Cannot match distinguished ranks");
+      link->leaf[i] = link->root[0];
+      continue;
+    }
+    ierr = PetscMalloc((leafoffset[i+1]-leafoffset[i])*link->unitbytes,&link->leaf[i]);CHKERRQ(ierr);
+  }
+  ierr = PetscCalloc1(nrootranks+nleafranks,&link->requests);CHKERRQ(ierr);
 
 found:
   link->key  = key;
@@ -788,8 +814,6 @@ found:
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFBasicGetPackInUse"
 static PetscErrorCode PetscSFBasicGetPackInUse(PetscSF sf,MPI_Datatype unit,const void *key,PetscCopyMode cmode,PetscSFBasicPack *mylink)
 {
   PetscSF_Basic    *bas = (PetscSF_Basic*)sf->data;
@@ -815,8 +839,6 @@ static PetscErrorCode PetscSFBasicGetPackInUse(PetscSF sf,MPI_Datatype unit,cons
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFBasicReclaimPack"
 static PetscErrorCode PetscSFBasicReclaimPack(PetscSF sf,PetscSFBasicPack *link)
 {
   PetscSF_Basic *bas = (PetscSF_Basic*)sf->data;
@@ -829,9 +851,7 @@ static PetscErrorCode PetscSFBasicReclaimPack(PetscSF sf,PetscSFBasicPack *link)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFSetFromOptions_Basic"
-static PetscErrorCode PetscSFSetFromOptions_Basic(PetscOptions *PetscOptionsObject,PetscSF sf)
+static PetscErrorCode PetscSFSetFromOptions_Basic(PetscOptionItems *PetscOptionsObject,PetscSF sf)
 {
   PetscErrorCode ierr;
 
@@ -841,8 +861,6 @@ static PetscErrorCode PetscSFSetFromOptions_Basic(PetscOptions *PetscOptionsObje
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFReset_Basic"
 static PetscErrorCode PetscSFReset_Basic(PetscSF sf)
 {
   PetscSF_Basic    *bas = (PetscSF_Basic*)sf->data;
@@ -850,14 +868,15 @@ static PetscErrorCode PetscSFReset_Basic(PetscSF sf)
   PetscSFBasicPack link,next;
 
   PetscFunctionBegin;
-  ierr = PetscFree(bas->iranks);CHKERRQ(ierr);
-  ierr = PetscFree2(bas->ioffset,bas->irootloc);CHKERRQ(ierr);
   if (bas->inuse) SETERRQ(PetscObjectComm((PetscObject)sf),PETSC_ERR_ARG_WRONGSTATE,"Outstanding operation has not been completed");
+  ierr = PetscFree2(bas->iranks,bas->ioffset);CHKERRQ(ierr);
+  ierr = PetscFree(bas->irootloc);CHKERRQ(ierr);
   for (link=bas->avail; link; link=next) {
+    PetscInt i;
     next = link->next;
-#if defined(PETSC_HAVE_MPI_TYPE_DUP)
     ierr = MPI_Type_free(&link->unit);CHKERRQ(ierr);
-#endif
+    for (i=0; i<bas->niranks; i++) {ierr = PetscFree(link->root[i]);CHKERRQ(ierr);}
+    for (i=sf->ndranks; i<sf->nranks; i++) {ierr = PetscFree(link->leaf[i]);CHKERRQ(ierr);} /* Free only non-distinguished leaf buffers */
     ierr = PetscFree2(link->root,link->leaf);CHKERRQ(ierr);
     ierr = PetscFree(link->requests);CHKERRQ(ierr);
     ierr = PetscFree(link);CHKERRQ(ierr);
@@ -866,8 +885,6 @@ static PetscErrorCode PetscSFReset_Basic(PetscSF sf)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFDestroy_Basic"
 static PetscErrorCode PetscSFDestroy_Basic(PetscSF sf)
 {
   PetscErrorCode ierr;
@@ -878,8 +895,6 @@ static PetscErrorCode PetscSFDestroy_Basic(PetscSF sf)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFView_Basic"
 static PetscErrorCode PetscSFView_Basic(PetscSF sf,PetscViewer viewer)
 {
   /* PetscSF_Basic *bas = (PetscSF_Basic*)sf->data; */
@@ -894,130 +909,138 @@ static PetscErrorCode PetscSFView_Basic(PetscSF sf,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFBcastBegin_Basic"
 /* Send from roots to leaves */
 static PetscErrorCode PetscSFBcastBegin_Basic(PetscSF sf,MPI_Datatype unit,const void *rootdata,void *leafdata)
 {
   PetscSF_Basic     *bas = (PetscSF_Basic*)sf->data;
   PetscErrorCode    ierr;
   PetscSFBasicPack  link;
-  PetscInt          i,nrootranks,nleafranks;
+  PetscInt          i,nrootranks,ndrootranks,nleafranks,ndleafranks;
   const PetscInt    *rootoffset,*leafoffset,*rootloc,*leafloc;
   const PetscMPIInt *rootranks,*leafranks;
   MPI_Request       *rootreqs,*leafreqs;
-  size_t            unitbytes;
 
   PetscFunctionBegin;
-  ierr = PetscSFBasicGetRootInfo(sf,&nrootranks,&rootranks,&rootoffset,&rootloc);CHKERRQ(ierr);
-  ierr = PetscSFBasicGetLeafInfo(sf,&nleafranks,&leafranks,&leafoffset,&leafloc);CHKERRQ(ierr);
+  ierr = PetscSFBasicGetRootInfo(sf,&nrootranks,&ndrootranks,&rootranks,&rootoffset,&rootloc);CHKERRQ(ierr);
+  ierr = PetscSFBasicGetLeafInfo(sf,&nleafranks,&ndleafranks,&leafranks,&leafoffset,&leafloc);CHKERRQ(ierr);
   ierr = PetscSFBasicGetPack(sf,unit,rootdata,&link);CHKERRQ(ierr);
 
-  unitbytes = link->unitbytes;
-
   ierr = PetscSFBasicPackGetReqs(sf,link,&rootreqs,&leafreqs);CHKERRQ(ierr);
-  /* Eagerly post leaf receives */
-  for (i=0; i<nleafranks; i++) {
+  /* Eagerly post leaf receives, but only from non-distinguished ranks -- distinguished ranks will receive via shared memory */
+  for (i=ndleafranks; i<nleafranks; i++) {
     PetscMPIInt n = leafoffset[i+1] - leafoffset[i];
-    ierr = MPI_Irecv(link->leaf+leafoffset[i]*unitbytes,n,unit,leafranks[i],bas->tag,PetscObjectComm((PetscObject)sf),&leafreqs[i]);CHKERRQ(ierr);
+    ierr = MPI_Irecv(link->leaf[i],n,unit,leafranks[i],bas->tag,PetscObjectComm((PetscObject)sf),&leafreqs[i-ndleafranks]);CHKERRQ(ierr);
   }
   /* Pack and send root data */
   for (i=0; i<nrootranks; i++) {
     PetscMPIInt n          = rootoffset[i+1] - rootoffset[i];
-    void        *packstart = link->root+rootoffset[i]*unitbytes;
+    void        *packstart = link->root[i];
     (*link->Pack)(n,link->bs,rootloc+rootoffset[i],rootdata,packstart);
-    ierr = MPI_Isend(packstart,n,unit,rootranks[i],bas->tag,PetscObjectComm((PetscObject)sf),&rootreqs[i]);CHKERRQ(ierr);
+    if (i < ndrootranks) continue; /* shared memory */
+    ierr = MPI_Isend(packstart,n,unit,rootranks[i],bas->tag,PetscObjectComm((PetscObject)sf),&rootreqs[i-ndrootranks]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFBcastEnd_Basic"
 PetscErrorCode PetscSFBcastEnd_Basic(PetscSF sf,MPI_Datatype unit,const void *rootdata,void *leafdata)
 {
   PetscErrorCode   ierr;
   PetscSFBasicPack link;
-  PetscInt         i,nleafranks;
+  PetscInt         i,nleafranks,ndleafranks;
   const PetscInt   *leafoffset,*leafloc;
 
   PetscFunctionBegin;
   ierr = PetscSFBasicGetPackInUse(sf,unit,rootdata,PETSC_OWN_POINTER,&link);CHKERRQ(ierr);
   ierr = PetscSFBasicPackWaitall(sf,link);CHKERRQ(ierr);
-  ierr = PetscSFBasicGetLeafInfo(sf,&nleafranks,NULL,&leafoffset,&leafloc);CHKERRQ(ierr);
+  ierr = PetscSFBasicGetLeafInfo(sf,&nleafranks,&ndleafranks,NULL,&leafoffset,&leafloc);CHKERRQ(ierr);
   for (i=0; i<nleafranks; i++) {
     PetscMPIInt n          = leafoffset[i+1] - leafoffset[i];
-    const void  *packstart = link->leaf+leafoffset[i]*link->unitbytes;
+    const void  *packstart = link->leaf[i];
     (*link->UnpackInsert)(n,link->bs,leafloc+leafoffset[i],leafdata,packstart);
   }
   ierr = PetscSFBasicReclaimPack(sf,&link);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFReduceBegin_Basic"
 /* leaf -> root with reduction */
 PetscErrorCode PetscSFReduceBegin_Basic(PetscSF sf,MPI_Datatype unit,const void *leafdata,void *rootdata,MPI_Op op)
 {
   PetscSF_Basic     *bas = (PetscSF_Basic*)sf->data;
   PetscSFBasicPack  link;
   PetscErrorCode    ierr;
-  PetscInt          i,nrootranks,nleafranks;
+  PetscInt          i,nrootranks,ndrootranks,nleafranks,ndleafranks;
   const PetscInt    *rootoffset,*leafoffset,*rootloc,*leafloc;
   const PetscMPIInt *rootranks,*leafranks;
   MPI_Request       *rootreqs,*leafreqs;
-  size_t            unitbytes;
 
   PetscFunctionBegin;
-  ierr = PetscSFBasicGetRootInfo(sf,&nrootranks,&rootranks,&rootoffset,&rootloc);CHKERRQ(ierr);
-  ierr = PetscSFBasicGetLeafInfo(sf,&nleafranks,&leafranks,&leafoffset,&leafloc);CHKERRQ(ierr);
+  ierr = PetscSFBasicGetRootInfo(sf,&nrootranks,&ndrootranks,&rootranks,&rootoffset,&rootloc);CHKERRQ(ierr);
+  ierr = PetscSFBasicGetLeafInfo(sf,&nleafranks,&ndleafranks,&leafranks,&leafoffset,&leafloc);CHKERRQ(ierr);
   ierr = PetscSFBasicGetPack(sf,unit,rootdata,&link);CHKERRQ(ierr);
 
-  unitbytes = link->unitbytes;
-
   ierr = PetscSFBasicPackGetReqs(sf,link,&rootreqs,&leafreqs);CHKERRQ(ierr);
-  /* Eagerly post root receives */
-  for (i=0; i<nrootranks; i++) {
+  /* Eagerly post root receives for non-distinguished ranks */
+  for (i=ndrootranks; i<nrootranks; i++) {
     PetscMPIInt n = rootoffset[i+1] - rootoffset[i];
-    ierr = MPI_Irecv(link->root+rootoffset[i]*unitbytes,n,unit,rootranks[i],bas->tag,PetscObjectComm((PetscObject)sf),&rootreqs[i]);CHKERRQ(ierr);
+    ierr = MPI_Irecv(link->root[i],n,unit,rootranks[i],bas->tag,PetscObjectComm((PetscObject)sf),&rootreqs[i-ndrootranks]);CHKERRQ(ierr);
   }
   /* Pack and send leaf data */
   for (i=0; i<nleafranks; i++) {
     PetscMPIInt n          = leafoffset[i+1] - leafoffset[i];
-    void        *packstart = link->leaf+leafoffset[i]*unitbytes;
+    void        *packstart = link->leaf[i];
     (*link->Pack)(n,link->bs,leafloc+leafoffset[i],leafdata,packstart);
-    ierr = MPI_Isend(packstart,n,unit,leafranks[i],bas->tag,PetscObjectComm((PetscObject)sf),&leafreqs[i]);CHKERRQ(ierr);
+    if (i < ndleafranks) continue; /* shared memory */
+    ierr = MPI_Isend(packstart,n,unit,leafranks[i],bas->tag,PetscObjectComm((PetscObject)sf),&leafreqs[i-ndleafranks]);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFReduceEnd_Basic"
 static PetscErrorCode PetscSFReduceEnd_Basic(PetscSF sf,MPI_Datatype unit,const void *leafdata,void *rootdata,MPI_Op op)
 {
   void             (*UnpackOp)(PetscInt,PetscInt,const PetscInt*,void*,const void*);
   PetscErrorCode   ierr;
   PetscSFBasicPack link;
   PetscInt         i,nrootranks;
+  PetscMPIInt      typesize = -1;
   const PetscInt   *rootoffset,*rootloc;
 
   PetscFunctionBegin;
   ierr = PetscSFBasicGetPackInUse(sf,unit,rootdata,PETSC_OWN_POINTER,&link);CHKERRQ(ierr);
   /* This implementation could be changed to unpack as receives arrive, at the cost of non-determinism */
   ierr = PetscSFBasicPackWaitall(sf,link);CHKERRQ(ierr);
-  ierr = PetscSFBasicGetRootInfo(sf,&nrootranks,NULL,&rootoffset,&rootloc);CHKERRQ(ierr);
+  ierr = PetscSFBasicGetRootInfo(sf,&nrootranks,NULL,NULL,&rootoffset,&rootloc);CHKERRQ(ierr);
   ierr = PetscSFBasicPackGetUnpackOp(sf,link,op,&UnpackOp);CHKERRQ(ierr);
+  if (UnpackOp) {
+    typesize = link->unitbytes;
+  }
+  else {
+    ierr = MPI_Type_size(unit,&typesize);CHKERRQ(ierr);
+  }
   for (i=0; i<nrootranks; i++) {
-    PetscMPIInt n          = rootoffset[i+1] - rootoffset[i];
-    const void  *packstart = link->root+rootoffset[i]*link->unitbytes;
+    PetscMPIInt n   = rootoffset[i+1] - rootoffset[i];
+    char *packstart = (char *) link->root[i];
 
-    (*UnpackOp)(n,link->bs,rootloc+rootoffset[i],rootdata,packstart);
+    if (UnpackOp) {
+      (*UnpackOp)(n,link->bs,rootloc+rootoffset[i],rootdata,(const void *)packstart);
+    }
+#if defined(PETSC_HAVE_MPI_REDUCE_LOCAL)
+    else if (n) { /* the op should be defined to operate on the whole datatype, so we ignore link->bs */
+      PetscInt j;
+
+      for (j = 0; j < n; j++) {
+        ierr = MPI_Reduce_local(packstart+j*typesize,((char *) rootdata)+(rootloc[rootoffset[i]+j])*typesize,1,unit,op);CHKERRQ(ierr);
+      }
+    }
+#else
+    else {
+      SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"No unpacking reduction operation for this MPI_Op");
+    }
+#endif
   }
   ierr = PetscSFBasicReclaimPack(sf,&link);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFFetchAndOpBegin_Basic"
 static PetscErrorCode PetscSFFetchAndOpBegin_Basic(PetscSF sf,MPI_Datatype unit,void *rootdata,const void *leafdata,void *leafupdate,MPI_Op op)
 {
   PetscErrorCode ierr;
@@ -1027,54 +1050,49 @@ static PetscErrorCode PetscSFFetchAndOpBegin_Basic(PetscSF sf,MPI_Datatype unit,
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFFetchAndOpEnd_Basic"
 static PetscErrorCode PetscSFFetchAndOpEnd_Basic(PetscSF sf,MPI_Datatype unit,void *rootdata,const void *leafdata,void *leafupdate,MPI_Op op)
 {
   PetscSF_Basic     *bas = (PetscSF_Basic*)sf->data;
   void              (*FetchAndOp)(PetscInt,PetscInt,const PetscInt*,void*,void*);
   PetscErrorCode    ierr;
   PetscSFBasicPack  link;
-  PetscInt          i,nrootranks,nleafranks;
+  PetscInt          i,nrootranks,ndrootranks,nleafranks,ndleafranks;
   const PetscInt    *rootoffset,*leafoffset,*rootloc,*leafloc;
   const PetscMPIInt *rootranks,*leafranks;
   MPI_Request       *rootreqs,*leafreqs;
-  size_t            unitbytes;
 
   PetscFunctionBegin;
   ierr = PetscSFBasicGetPackInUse(sf,unit,rootdata,PETSC_OWN_POINTER,&link);CHKERRQ(ierr);
   /* This implementation could be changed to unpack as receives arrive, at the cost of non-determinism */
   ierr      = PetscSFBasicPackWaitall(sf,link);CHKERRQ(ierr);
-  unitbytes = link->unitbytes;
-  ierr      = PetscSFBasicGetRootInfo(sf,&nrootranks,&rootranks,&rootoffset,&rootloc);CHKERRQ(ierr);
-  ierr      = PetscSFBasicGetLeafInfo(sf,&nleafranks,&leafranks,&leafoffset,&leafloc);CHKERRQ(ierr);
+  ierr      = PetscSFBasicGetRootInfo(sf,&nrootranks,&ndrootranks,&rootranks,&rootoffset,&rootloc);CHKERRQ(ierr);
+  ierr      = PetscSFBasicGetLeafInfo(sf,&nleafranks,&ndleafranks,&leafranks,&leafoffset,&leafloc);CHKERRQ(ierr);
   ierr      = PetscSFBasicPackGetReqs(sf,link,&rootreqs,&leafreqs);CHKERRQ(ierr);
   /* Post leaf receives */
-  for (i=0; i<nleafranks; i++) {
+  for (i=ndleafranks; i<nleafranks; i++) {
     PetscMPIInt n = leafoffset[i+1] - leafoffset[i];
-    ierr = MPI_Irecv(link->leaf+leafoffset[i]*unitbytes,n,unit,leafranks[i],bas->tag,PetscObjectComm((PetscObject)sf),&leafreqs[i]);CHKERRQ(ierr);
+    ierr = MPI_Irecv(link->leaf[i],n,unit,leafranks[i],bas->tag,PetscObjectComm((PetscObject)sf),&leafreqs[i-ndleafranks]);CHKERRQ(ierr);
   }
   /* Process local fetch-and-op, post root sends */
   ierr = PetscSFBasicPackGetFetchAndOp(sf,link,op,&FetchAndOp);CHKERRQ(ierr);
   for (i=0; i<nrootranks; i++) {
     PetscMPIInt n          = rootoffset[i+1] - rootoffset[i];
-    void        *packstart = link->root+rootoffset[i]*unitbytes;
+    void        *packstart = link->root[i];
 
     (*FetchAndOp)(n,link->bs,rootloc+rootoffset[i],rootdata,packstart);
-    ierr = MPI_Isend(packstart,n,unit,rootranks[i],bas->tag,PetscObjectComm((PetscObject)sf),&rootreqs[i]);CHKERRQ(ierr);
+    if (i < ndrootranks) continue; /* shared memory */
+    ierr = MPI_Isend(packstart,n,unit,rootranks[i],bas->tag,PetscObjectComm((PetscObject)sf),&rootreqs[i-ndrootranks]);CHKERRQ(ierr);
   }
   ierr = PetscSFBasicPackWaitall(sf,link);CHKERRQ(ierr);
   for (i=0; i<nleafranks; i++) {
     PetscMPIInt n          = leafoffset[i+1] - leafoffset[i];
-    const void  *packstart = link->leaf+leafoffset[i]*unitbytes;
+    const void  *packstart = link->leaf[i];
     (*link->UnpackInsert)(n,link->bs,leafloc+leafoffset[i],leafupdate,packstart);
   }
   ierr = PetscSFBasicReclaimPack(sf,&link);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscSFCreate_Basic"
 PETSC_EXTERN PetscErrorCode PetscSFCreate_Basic(PetscSF sf)
 {
   PetscSF_Basic  *bas = (PetscSF_Basic*)sf->data;
@@ -1093,7 +1111,7 @@ PETSC_EXTERN PetscErrorCode PetscSFCreate_Basic(PetscSF sf)
   sf->ops->FetchAndOpBegin = PetscSFFetchAndOpBegin_Basic;
   sf->ops->FetchAndOpEnd   = PetscSFFetchAndOpEnd_Basic;
 
-  ierr     = PetscNewLog(sf,&bas);CHKERRQ(ierr);
+  ierr = PetscNewLog(sf,&bas);CHKERRQ(ierr);
   sf->data = (void*)bas;
   PetscFunctionReturn(0);
 }
