@@ -1,8 +1,8 @@
 static char help[] = "Tests VecMDot(),VecDot(),VecMTDot(), and VecTDot()\n";
+
+
 #include <petscvec.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc, char **argv)
 {
   PetscErrorCode ierr;
@@ -11,9 +11,9 @@ int main(int argc, char **argv)
   PetscRandom    rctx;
   PetscScalar    *val_dot,*val_mdot,*tval_dot,*tval_mdot;
 
-  PetscInitialize(&argc,&argv,(char*)0,help);
-  ierr = PetscOptionsGetInt(NULL,"-n",&n,NULL);CHKERRQ(ierr);
-  ierr = PetscOptionsGetInt(NULL,"-k",&k,NULL);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
+  ierr = PetscOptionsGetInt(NULL,NULL,"-n",&n,NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsGetInt(NULL,NULL,"-k",&k,NULL);CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_WORLD,"Test with %D random vectors of length %D",k,n);CHKERRQ(ierr);
   ierr = PetscPrintf(PETSC_COMM_WORLD,"\n",k,n);CHKERRQ(ierr);
   ierr = PetscRandomCreate(PETSC_COMM_WORLD,&rctx);CHKERRQ(ierr);
@@ -38,12 +38,12 @@ int main(int argc, char **argv)
       }
       /* Check result */
       for (j=0;j<i;j++) {
-        if (fabs(val_mdot[j] - val_dot[j])/fabs(val_dot[j]) > 1e-5) {
-          ierr = PetscPrintf(PETSC_COMM_WORLD, "[TEST FAILED] i=%D, j=%D, val_mdot[j]=%g, val_dot[j]=%g\n", i, j, val_mdot[j], val_dot[j]);CHKERRQ(ierr);
+        if (PetscAbsScalar(val_mdot[j] - val_dot[j])/PetscAbsScalar(val_dot[j]) > 1e-5) {
+          ierr = PetscPrintf(PETSC_COMM_WORLD, "[TEST FAILED] i=%D, j=%D, val_mdot[j]=%g, val_dot[j]=%g\n",i,j,(double)PetscAbsScalar(val_mdot[j]), (double)PetscAbsScalar(val_dot[j]));CHKERRQ(ierr);
           break;
         }
-        if (fabs(tval_mdot[j] - tval_dot[j])/fabs(tval_dot[j]) > 1e-5) {
-          ierr = PetscPrintf(PETSC_COMM_WORLD, "[TEST FAILED] i=%D, j=%D, tval_mdot[j]=%g, tval_dot[j]=%g\n", i, j, tval_mdot[j], tval_dot[j]);CHKERRQ(ierr);
+        if (PetscAbsScalar(tval_mdot[j] - tval_dot[j])/PetscAbsScalar(tval_dot[j]) > 1e-5) {
+          ierr = PetscPrintf(PETSC_COMM_WORLD, "[TEST FAILED] i=%D, j=%D, tval_mdot[j]=%g, tval_dot[j]=%g\n",i,j,(double)PetscAbsScalar(tval_mdot[j]), (double)PetscAbsScalar(tval_dot[j]));CHKERRQ(ierr);
           break;
         }
       }
@@ -57,6 +57,19 @@ int main(int argc, char **argv)
   ierr = VecDestroyVecs(k,&V);CHKERRQ(ierr);
   ierr = VecDestroy(&t);CHKERRQ(ierr);
   ierr = PetscRandomDestroy(&rctx);CHKERRQ(ierr);
-  ierr = PetscFinalize();CHKERRQ(ierr);
-  return 0;
+  ierr = PetscFinalize();
+  return ierr;
 }
+
+
+/*TEST
+
+   test:
+
+   test:
+      suffix: cuda
+      args: -vec_type cuda
+      output_file: output/ex43_1.out
+      requires: veccuda
+
+TEST*/
