@@ -6,6 +6,8 @@ static char help[] = "Basic vector routines.\n\n";
    Processors: n
 T*/
 
+
+
 /*
   Include "petscvec.h" so that we can use vectors.  Note that this file
   automatically includes:
@@ -15,8 +17,6 @@ T*/
 
 #include <petscvec.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   Vec            x,y,w;               /* vectors */
@@ -26,8 +26,8 @@ int main(int argc,char **argv)
   PetscErrorCode ierr;
   PetscScalar    one = 1.0,two = 2.0,three = 3.0,dots[3],dot;
 
-  PetscInitialize(&argc,&argv,(char*)0,help);
-  ierr = PetscOptionsGetInt(NULL,"-n",&n,NULL);CHKERRQ(ierr);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
+  ierr = PetscOptionsGetInt(NULL,NULL,"-n",&n,NULL);CHKERRQ(ierr);
 
   /*
      Create a vector, specifying only its global dimension.
@@ -161,6 +161,31 @@ int main(int argc,char **argv)
   ierr = VecDestroy(&w);CHKERRQ(ierr);
   ierr = VecDestroyVecs(3,&z);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
+
+
+/*TEST
+
+   test:
+
+   test:
+      suffix: 2
+      nsize: 2
+      output_file: output/ex1_1.out
+
+   test:
+      suffix: 2_cuda
+      nsize: 2
+      args: -vec_type cuda
+      output_file: output/ex1_1.out
+      requires: veccuda
+
+   test:
+      suffix: cuda
+      args: -vec_type cuda
+      output_file: output/ex1_1.out
+      requires: veccuda
+
+TEST*/
