@@ -15,8 +15,6 @@ T*/
 
 #include <petscvec.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "main"
 int main(int argc,char **argv)
 {
   Vec            x;               /* vectors */
@@ -25,7 +23,7 @@ int main(int argc,char **argv)
   PetscErrorCode ierr;
   PetscScalar    one = 1.0;
 
-  PetscInitialize(&argc,&argv,(char*)0,help);
+  ierr = PetscInitialize(&argc,&argv,(char*)0,help);if (ierr) return ierr;
   ierr = PetscOptionsGetInt(NULL,NULL,"-n",&n,NULL);CHKERRQ(ierr);
 
   /*
@@ -79,6 +77,12 @@ int main(int argc,char **argv)
   */
   ierr = VecDestroy(&x);CHKERRQ(ierr);
   ierr = PetscFinalize();
-  return 0;
+  return ierr;
 }
 
+/*TEST
+
+     test:
+       nsize: 2
+
+TEST*/
