@@ -4,13 +4,9 @@
 */
 
 #include <petsc/private/vecimpl.h>          /*I "petscvec.h" I*/
-EXTERN_C_BEGIN
+#include <../src/vec/vec/impls/hypre/vhyp.h>
 #include <HYPRE.h>
-#include <HYPRE_parcsr_ls.h>
-EXTERN_C_END
 
-#undef __FUNCT__
-#define __FUNCT__ "VecHYPRE_IJVectorCreate"
 PETSC_EXTERN PetscErrorCode VecHYPRE_IJVectorCreate(Vec v,HYPRE_IJVector *ij)
 {
   PetscErrorCode ierr;
@@ -23,8 +19,6 @@ PETSC_EXTERN PetscErrorCode VecHYPRE_IJVectorCreate(Vec v,HYPRE_IJVector *ij)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecHYPRE_IJVectorCopy"
 PETSC_EXTERN PetscErrorCode VecHYPRE_IJVectorCopy(Vec v,HYPRE_IJVector ij)
 {
   PetscErrorCode ierr;
@@ -39,8 +33,6 @@ PETSC_EXTERN PetscErrorCode VecHYPRE_IJVectorCopy(Vec v,HYPRE_IJVector ij)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecHYPRE_IJVectorCopyFrom"
 PetscErrorCode VecHYPRE_IJVectorCopyFrom(HYPRE_IJVector ij,Vec v)
 {
   PetscErrorCode ierr;
