@@ -3,10 +3,10 @@
      Code for some of the parallel vector primatives.
 */
 #include <../src/vec/vec/impls/mpi/pvecimpl.h>   /*I  "petscvec.h"   I*/
+#include <petsc/private/glvisviewerimpl.h>
+#include <petsc/private/glvisvecimpl.h>
 #include <petscviewerhdf5.h>
 
-#undef __FUNCT__
-#define __FUNCT__ "VecDestroy_MPI"
 PetscErrorCode VecDestroy_MPI(Vec v)
 {
   Vec_MPI        *x = (Vec_MPI*)v->data;
@@ -24,6 +24,8 @@ PetscErrorCode VecDestroy_MPI(Vec v)
     ierr = VecDestroy(&x->localrep);CHKERRQ(ierr);
     ierr = VecScatterDestroy(&x->localupdate);CHKERRQ(ierr);
   }
+  ierr = VecAssemblyReset_MPI(v);CHKERRQ(ierr);
+
   /* Destroy the stashes: note the order - so that the tags are freed properly */
   ierr = VecStashDestroy_Private(&v->bstash);CHKERRQ(ierr);
   ierr = VecStashDestroy_Private(&v->stash);CHKERRQ(ierr);
@@ -31,8 +33,6 @@ PetscErrorCode VecDestroy_MPI(Vec v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecView_MPI_ASCII"
 PetscErrorCode VecView_MPI_ASCII(Vec xin,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -45,13 +45,24 @@ PetscErrorCode VecView_MPI_ASCII(Vec xin,PetscViewer viewer)
   PetscViewerFormat format;
 
   PetscFunctionBegin;
+  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)xin),&size);CHKERRQ(ierr);
+  ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
+  if (format == PETSC_VIEWER_LOAD_BALANCE) {
+    PetscInt nmax = 0,nmin = xin->map->n,navg;
+    for (i=0; i<(PetscInt)size; i++) {
+      nmax = PetscMax(nmax,xin->map->range[i+1] - xin->map->range[i]);
+      nmin = PetscMin(nmin,xin->map->range[i+1] - xin->map->range[i]);
+    }
+    navg = xin->map->N/size;
+    ierr = PetscViewerASCIIPrintf(viewer,"  Load Balance - Local vector size Min %D  avg %D  max %D\n",nmin,navg,nmax);CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  }
+
   ierr = VecGetArrayRead(xin,&xarray);CHKERRQ(ierr);
   /* determine maximum message to arrive */
   ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)xin),&rank);CHKERRQ(ierr);
   ierr = MPI_Reduce(&work,&len,1,MPIU_INT,MPI_MAX,0,PetscObjectComm((PetscObject)xin));CHKERRQ(ierr);
-  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)xin),&size);CHKERRQ(ierr);
-
-  ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
+  if (format == PETSC_VIEWER_ASCII_GLVIS) { rank = 0, len = 0; } /* no parallel distributed write support from GLVis */
   if (!rank) {
     ierr = PetscMalloc1(len,&values);CHKERRQ(ierr);
     /*
@@ -275,6 +286,38 @@ PetscErrorCode VecView_MPI_ASCII(Vec xin,PetscViewer viewer)
           ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
         }
       }
+    } else if (format == PETSC_VIEWER_ASCII_GLVIS) {
+      /* GLVis ASCII visualization/dump: this function mimicks mfem::GridFunction::Save() */
+      const PetscScalar       *array;
+      PetscInt                i,n,vdim, ordering = 1; /* mfem::FiniteElementSpace::Ordering::byVDIM */
+      PetscContainer          glvis_container;
+      PetscViewerGLVisVecInfo glvis_vec_info;
+      PetscViewerGLVisInfo    glvis_info;
+      PetscErrorCode          ierr;
+
+      /* mfem::FiniteElementSpace::Save() */
+      ierr = VecGetBlockSize(xin,&vdim);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"FiniteElementSpace\n");CHKERRQ(ierr);
+      ierr = PetscObjectQuery((PetscObject)xin,"_glvis_info_container",(PetscObject*)&glvis_container);CHKERRQ(ierr);
+      if (!glvis_container) SETERRQ(PetscObjectComm((PetscObject)xin),PETSC_ERR_PLIB,"Missing GLVis container");
+      ierr = PetscContainerGetPointer(glvis_container,(void**)&glvis_vec_info);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"%s\n",glvis_vec_info->fec_type);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"VDim: %d\n",vdim);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"Ordering: %d\n",ordering);CHKERRQ(ierr);
+      ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
+      /* mfem::Vector::Print() */
+      ierr = PetscObjectQuery((PetscObject)viewer,"_glvis_info_container",(PetscObject*)&glvis_container);CHKERRQ(ierr);
+      if (!glvis_container) SETERRQ(PetscObjectComm((PetscObject)viewer),PETSC_ERR_PLIB,"Missing GLVis container");
+      ierr = PetscContainerGetPointer(glvis_container,(void**)&glvis_info);CHKERRQ(ierr);
+      if (glvis_info->enabled) {
+        ierr = VecGetLocalSize(xin,&n);CHKERRQ(ierr);
+        ierr = VecGetArrayRead(xin,&array);CHKERRQ(ierr);
+        for (i=0;i<n;i++) {
+          ierr = PetscViewerASCIIPrintf(viewer,glvis_info->fmt,(double)PetscRealPart(array[i]));CHKERRQ(ierr);
+          ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
+        }
+        ierr = VecRestoreArrayRead(xin,&array);CHKERRQ(ierr);
+      }
     } else if (format == PETSC_VIEWER_ASCII_INFO || format == PETSC_VIEWER_ASCII_INFO_DETAIL) {
       /* No info */
     } else {
@@ -338,13 +381,11 @@ PetscErrorCode VecView_MPI_ASCII(Vec xin,PetscViewer viewer)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecView_MPI_Binary"
 PetscErrorCode VecView_MPI_Binary(Vec xin,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
   PetscMPIInt       rank,size,mesgsize,tag = ((PetscObject)viewer)->tag, mesglen;
-  PetscInt          len,n = xin->map->n,j,tr[2];
+  PetscInt          len,n,j,tr[2];
   int               fdes;
   MPI_Status        status;
   PetscScalar       *values;
@@ -419,7 +460,7 @@ PetscErrorCode VecView_MPI_Binary(Vec xin,PetscViewer viewer)
     MPI_File     mfdes;
     PetscMPIInt  lsize;
 
-    ierr = PetscMPIIntCast(n,&lsize);CHKERRQ(ierr);
+    ierr = PetscMPIIntCast(xin->map->n,&lsize);CHKERRQ(ierr);
     ierr = PetscViewerBinaryGetMPIIODescriptor(viewer,&mfdes);CHKERRQ(ierr);
     ierr = PetscViewerBinaryGetMPIIOOffset(viewer,&off);CHKERRQ(ierr);
     off += xin->map->rstart*sizeof(PetscScalar); /* off is MPI_Offset, not PetscMPIInt */
@@ -444,89 +485,67 @@ PetscErrorCode VecView_MPI_Binary(Vec xin,PetscViewer viewer)
 }
 
 #include <petscdraw.h>
-#undef __FUNCT__
-#define __FUNCT__ "VecView_MPI_Draw_LG"
 PetscErrorCode VecView_MPI_Draw_LG(Vec xin,PetscViewer viewer)
 {
-  PetscDraw      draw;
-  PetscBool      isnull;
-  PetscErrorCode ierr;
-
-#if defined(PETSC_USE_64BIT_INDICES)
-  PetscFunctionBegin;
-  ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
-  ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr);
-  if (isnull) PetscFunctionReturn(0);
-  SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Not supported with 64 bit integers");
-#else
-  PetscMPIInt       size,rank;
-  int               i,N = xin->map->N,*lens;
-  PetscReal         *xx,*yy;
+  PetscDraw         draw;
+  PetscBool         isnull;
   PetscDrawLG       lg;
+  PetscMPIInt       i,size,rank,n,N,*lens = NULL,*disp = NULL;
+  PetscReal         *values, *xx = NULL,*yy = NULL;
   const PetscScalar *xarray;
+  int               colors[] = {PETSC_DRAW_RED};
+  PetscErrorCode    ierr;
 
   PetscFunctionBegin;
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
   ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr);
   if (isnull) PetscFunctionReturn(0);
-
-  ierr = VecGetArrayRead(xin,&xarray);CHKERRQ(ierr);
-  ierr = PetscViewerDrawGetDrawLG(viewer,0,&lg);CHKERRQ(ierr);
-  ierr = PetscDrawCheckResizedWindow(draw);CHKERRQ(ierr);
   ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)xin),&rank);CHKERRQ(ierr);
   ierr = MPI_Comm_size(PetscObjectComm((PetscObject)xin),&size);CHKERRQ(ierr);
-  if (!rank) {
-    ierr = PetscDrawLGReset(lg);CHKERRQ(ierr);
-    ierr = PetscMalloc2(N,&xx,N,&yy);CHKERRQ(ierr);
-    for (i=0; i<N; i++) xx[i] = (PetscReal) i;
-    ierr = PetscMalloc1(size,&lens);CHKERRQ(ierr);
-    for (i=0; i<size; i++) lens[i] = xin->map->range[i+1] - xin->map->range[i];
+  ierr = PetscMPIIntCast(xin->map->n,&n);CHKERRQ(ierr);
+  ierr = PetscMPIIntCast(xin->map->N,&N);CHKERRQ(ierr);
 
-#if !defined(PETSC_USE_COMPLEX)
-    ierr = MPI_Gatherv((void*)xarray,xin->map->n,MPIU_REAL,yy,lens,xin->map->range,MPIU_REAL,0,PetscObjectComm((PetscObject)xin));CHKERRQ(ierr);
+  ierr = VecGetArrayRead(xin,&xarray);CHKERRQ(ierr);
+#if defined(PETSC_USE_COMPLEX)
+  ierr = PetscMalloc1(n+1,&values);CHKERRQ(ierr);
+  for (i=0; i<n; i++) values[i] = PetscRealPart(xarray[i]);
 #else
-    {
-      PetscReal *xr;
-      ierr = PetscMalloc1(xin->map->n+1,&xr);CHKERRQ(ierr);
-      for (i=0; i<xin->map->n; i++) xr[i] = PetscRealPart(xarray[i]);
-
-      ierr = MPI_Gatherv(xr,xin->map->n,MPIU_REAL,yy,lens,xin->map->range,MPIU_REAL,0,PetscObjectComm((PetscObject)xin));CHKERRQ(ierr);
-      ierr = PetscFree(xr);CHKERRQ(ierr);
-    }
+  values = (PetscReal*)xarray;
 #endif
-    ierr = PetscFree(lens);CHKERRQ(ierr);
+  if (!rank) {
+    ierr = PetscMalloc2(N,&xx,N,&yy);CHKERRQ(ierr);
+    for (i=0; i<N; i++) xx[i] = (PetscReal)i;
+    ierr = PetscMalloc2(size,&lens,size,&disp);CHKERRQ(ierr);
+    for (i=0; i<size; i++) lens[i] = (PetscMPIInt)xin->map->range[i+1] - (PetscMPIInt)xin->map->range[i];
+    for (i=0; i<size; i++) disp[i] = (PetscMPIInt)xin->map->range[i];
+  }
+  ierr = MPI_Gatherv(values,n,MPIU_REAL,yy,lens,disp,MPIU_REAL,0,PetscObjectComm((PetscObject)xin));CHKERRQ(ierr);
+  ierr = PetscFree2(lens,disp);CHKERRQ(ierr);
+#if defined(PETSC_USE_COMPLEX)
+  ierr = PetscFree(values);CHKERRQ(ierr);
+#endif
+  ierr = VecRestoreArrayRead(xin,&xarray);CHKERRQ(ierr);
+
+  ierr = PetscViewerDrawGetDrawLG(viewer,0,&lg);CHKERRQ(ierr);
+  ierr = PetscDrawLGReset(lg);CHKERRQ(ierr);
+  ierr = PetscDrawLGSetDimension(lg,1);CHKERRQ(ierr);
+  ierr = PetscDrawLGSetColors(lg,colors);CHKERRQ(ierr);
+  if (!rank) {
     ierr = PetscDrawLGAddPoints(lg,N,&xx,&yy);CHKERRQ(ierr);
     ierr = PetscFree2(xx,yy);CHKERRQ(ierr);
-  } else {
-#if !defined(PETSC_USE_COMPLEX)
-    ierr = MPI_Gatherv((void*)xarray,xin->map->n,MPIU_REAL,0,0,0,MPIU_REAL,0,PetscObjectComm((PetscObject)xin));CHKERRQ(ierr);
-#else
-    {
-      PetscReal *xr;
-      ierr = PetscMalloc1(xin->map->n+1,&xr);CHKERRQ(ierr);
-      for (i=0; i<xin->map->n; i++) xr[i] = PetscRealPart(xarray[i]);
-
-      ierr = MPI_Gatherv(xr,xin->map->n,MPIU_REAL,0,0,0,MPIU_REAL,0,PetscObjectComm((PetscObject)xin));CHKERRQ(ierr);
-      ierr = PetscFree(xr);CHKERRQ(ierr);
-    }
-#endif
   }
   ierr = PetscDrawLGDraw(lg);CHKERRQ(ierr);
-  ierr = PetscDrawSynchronizedFlush(draw);CHKERRQ(ierr);
-  ierr = VecRestoreArrayRead(xin,&xarray);CHKERRQ(ierr);
-#endif
+  ierr = PetscDrawLGSave(lg);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecView_MPI_Draw"
 PetscErrorCode  VecView_MPI_Draw(Vec xin,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
   PetscMPIInt       rank,size,tag = ((PetscObject)viewer)->tag;
   PetscInt          i,start,end;
   MPI_Status        status;
-  PetscReal         coors[4],ymin,ymax,xmin,xmax,tmp = 0.0;
+  PetscReal         min,max,tmp = 0.0;
   PetscDraw         draw;
   PetscBool         isnull;
   PetscDrawAxis     axis;
@@ -534,69 +553,52 @@ PetscErrorCode  VecView_MPI_Draw(Vec xin,PetscViewer viewer)
 
   PetscFunctionBegin;
   ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
-  ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr); if (isnull) PetscFunctionReturn(0);
-
-  ierr = VecGetArrayRead(xin,&xarray);CHKERRQ(ierr);
-  ierr = PetscDrawCheckResizedWindow(draw);CHKERRQ(ierr);
-  xmin = 1.e20; xmax = -1.e20;
-  for (i=0; i<xin->map->n; i++) {
-#if defined(PETSC_USE_COMPLEX)
-    if (PetscRealPart(xarray[i]) < xmin) xmin = PetscRealPart(xarray[i]);
-    if (PetscRealPart(xarray[i]) > xmax) xmax = PetscRealPart(xarray[i]);
-#else
-    if (xarray[i] < xmin) xmin = xarray[i];
-    if (xarray[i] > xmax) xmax = xarray[i];
-#endif
-  }
-  if (xmin + 1.e-10 > xmax) {
-    xmin -= 1.e-5;
-    xmax += 1.e-5;
-  }
-  ierr = MPI_Reduce(&xmin,&ymin,1,MPIU_REAL,MPIU_MIN,0,PetscObjectComm((PetscObject)xin));CHKERRQ(ierr);
-  ierr = MPI_Reduce(&xmax,&ymax,1,MPIU_REAL,MPIU_MAX,0,PetscObjectComm((PetscObject)xin));CHKERRQ(ierr);
+  ierr = PetscDrawIsNull(draw,&isnull);CHKERRQ(ierr);
+  if (isnull) PetscFunctionReturn(0);
   ierr = MPI_Comm_size(PetscObjectComm((PetscObject)xin),&size);CHKERRQ(ierr);
   ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)xin),&rank);CHKERRQ(ierr);
+
+  ierr = VecMin(xin,NULL,&min);CHKERRQ(ierr);
+  ierr = VecMax(xin,NULL,&max);CHKERRQ(ierr);
+  if (min == max) {
+    min -= 1.e-5;
+    max += 1.e-5;
+  }
+
+  ierr = PetscDrawCheckResizedWindow(draw);CHKERRQ(ierr);
+  ierr = PetscDrawClear(draw);CHKERRQ(ierr);
+
   ierr = PetscDrawAxisCreate(draw,&axis);CHKERRQ(ierr);
-  ierr = PetscLogObjectParent((PetscObject)draw,(PetscObject)axis);CHKERRQ(ierr);
-  if (!rank) {
-    ierr = PetscDrawClear(draw);CHKERRQ(ierr);
-    ierr = PetscDrawFlush(draw);CHKERRQ(ierr);
-    ierr = PetscDrawAxisSetLimits(axis,0.0,(double)xin->map->N,ymin,ymax);CHKERRQ(ierr);
-    ierr = PetscDrawAxisDraw(axis);CHKERRQ(ierr);
-    ierr = PetscDrawGetCoordinates(draw,coors,coors+1,coors+2,coors+3);CHKERRQ(ierr);
-  }
+  ierr = PetscDrawAxisSetLimits(axis,0.0,(PetscReal)xin->map->N,min,max);CHKERRQ(ierr);
+  ierr = PetscDrawAxisDraw(axis);CHKERRQ(ierr);
   ierr = PetscDrawAxisDestroy(&axis);CHKERRQ(ierr);
-  ierr = MPI_Bcast(coors,4,MPIU_REAL,0,PetscObjectComm((PetscObject)xin));CHKERRQ(ierr);
-  if (rank) {ierr = PetscDrawSetCoordinates(draw,coors[0],coors[1],coors[2],coors[3]);CHKERRQ(ierr);}
+
   /* draw local part of vector */
+  ierr = VecGetArrayRead(xin,&xarray);CHKERRQ(ierr);
   ierr = VecGetOwnershipRange(xin,&start,&end);CHKERRQ(ierr);
-  if (rank < size-1) { /*send value to right */
+  if (rank < size-1) { /* send value to right */
     ierr = MPI_Send((void*)&xarray[xin->map->n-1],1,MPIU_REAL,rank+1,tag,PetscObjectComm((PetscObject)xin));CHKERRQ(ierr);
   }
-  for (i=1; i<xin->map->n; i++) {
-#if !defined(PETSC_USE_COMPLEX)
-    ierr = PetscDrawLine(draw,(PetscReal)(i-1+start),xarray[i-1],(PetscReal)(i+start),xarray[i],PETSC_DRAW_RED);CHKERRQ(ierr);
-#else
-    ierr = PetscDrawLine(draw,(PetscReal)(i-1+start),PetscRealPart(xarray[i-1]),(PetscReal)(i+start),PetscRealPart(xarray[i]),PETSC_DRAW_RED);CHKERRQ(ierr);
-#endif
-  }
   if (rank) { /* receive value from right */
     ierr = MPI_Recv(&tmp,1,MPIU_REAL,rank-1,tag,PetscObjectComm((PetscObject)xin),&status);CHKERRQ(ierr);
-#if !defined(PETSC_USE_COMPLEX)
-    ierr = PetscDrawLine(draw,(PetscReal)start-1,tmp,(PetscReal)start,xarray[0],PETSC_DRAW_RED);CHKERRQ(ierr);
-#else
-    ierr = PetscDrawLine(draw,(PetscReal)start-1,tmp,(PetscReal)start,PetscRealPart(xarray[0]),PETSC_DRAW_RED);CHKERRQ(ierr);
-#endif
   }
-  ierr = PetscDrawSynchronizedFlush(draw);CHKERRQ(ierr);
-  ierr = PetscDrawPause(draw);CHKERRQ(ierr);
+  ierr = PetscDrawCollectiveBegin(draw);CHKERRQ(ierr);
+  if (rank) {
+    ierr = PetscDrawLine(draw,(PetscReal)start-1,tmp,(PetscReal)start,PetscRealPart(xarray[0]),PETSC_DRAW_RED);CHKERRQ(ierr);
+  }
+  for (i=1; i<xin->map->n; i++) {
+    ierr = PetscDrawLine(draw,(PetscReal)(i-1+start),PetscRealPart(xarray[i-1]),(PetscReal)(i+start),PetscRealPart(xarray[i]),PETSC_DRAW_RED);CHKERRQ(ierr);
+  }
+  ierr = PetscDrawCollectiveEnd(draw);CHKERRQ(ierr);
   ierr = VecRestoreArrayRead(xin,&xarray);CHKERRQ(ierr);
+
+  ierr = PetscDrawFlush(draw);CHKERRQ(ierr);
+  ierr = PetscDrawPause(draw);CHKERRQ(ierr);
+  ierr = PetscDrawSave(draw);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 #if defined(PETSC_HAVE_MATLAB_ENGINE)
-#undef __FUNCT__
-#define __FUNCT__ "VecView_MPI_Matlab"
 PetscErrorCode VecView_MPI_Matlab(Vec xin,PetscViewer viewer)
 {
   PetscErrorCode    ierr;
@@ -629,9 +631,67 @@ PetscErrorCode VecView_MPI_Matlab(Vec xin,PetscViewer viewer)
 }
 #endif
 
+#if defined(PETSC_HAVE_ADIOS)
+#include <adios.h>
+#include <adios_read.h>
+#include <petsc/private/vieweradiosimpl.h>
+#include <petsc/private/viewerimpl.h>
+
+PetscErrorCode VecView_MPI_ADIOS(Vec xin, PetscViewer viewer)
+{
+  PetscViewer_ADIOS *adios = (PetscViewer_ADIOS*)viewer->data;
+  PetscErrorCode    ierr;
+  const char        *vecname;
+  int64_t           id;
+  PetscInt          n,N,rstart;
+  const PetscScalar *array;
+  char              nglobalname[16],nlocalname[16],coffset[16];
+
+  PetscFunctionBegin;
+  ierr = PetscObjectGetName((PetscObject) xin, &vecname);CHKERRQ(ierr);
+
+  ierr = VecGetLocalSize(xin,&n);CHKERRQ(ierr);
+  ierr = VecGetSize(xin,&N);CHKERRQ(ierr);
+  ierr = VecGetOwnershipRange(xin,&rstart,NULL);CHKERRQ(ierr);
+
+  sprintf(nlocalname,"%d",(int)n);
+  sprintf(nglobalname,"%d",(int)N);
+  sprintf(coffset,"%d",(int)rstart);
+  id   = adios_define_var(Petsc_adios_group,vecname,"",adios_double,nlocalname,nglobalname,coffset);CHKERRQ(ierr);
+  ierr = VecGetArrayRead(xin,&array);CHKERRQ(ierr);
+  ierr = adios_write_byid(adios->adios_handle,id,array);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(xin,&array);CHKERRQ(ierr);
+
+  PetscFunctionReturn(0);
+}
+#endif
+
+#if defined(PETSC_HAVE_ADIOS2)
+#include <adios2_c.h>
+#include <petsc/private/vieweradios2impl.h>
+#include <petsc/private/viewerimpl.h>
+
+PetscErrorCode VecView_MPI_ADIOS2(Vec xin, PetscViewer viewer)
+{
+  PetscErrorCode     ierr;
+  PetscViewer_ADIOS2 *adios2 = (PetscViewer_ADIOS2*)viewer->data;
+  PetscInt           n,N,rstart;
+  const char         *vecname;
+  const PetscScalar  *array;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectGetName((PetscObject) xin, &vecname);CHKERRQ(ierr);
+  ierr = VecGetLocalSize(xin,&n);CHKERRQ(ierr);
+  ierr = VecGetSize(xin,&N);CHKERRQ(ierr);
+  ierr = VecGetOwnershipRange(xin,&rstart,NULL);CHKERRQ(ierr);
+
+  ierr = VecGetArrayRead(xin,&array);CHKERRQ(ierr);
+  ierr = VecRestoreArrayRead(xin,&array);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+#endif
+
 #if defined(PETSC_HAVE_HDF5)
-#undef __FUNCT__
-#define __FUNCT__ "VecView_MPI_HDF5"
 PetscErrorCode VecView_MPI_HDF5(Vec xin, PetscViewer viewer)
 {
   /* TODO: It looks like we can remove the H5Sclose(filespace) and H5Dget_space(dset_id). Why do we do this? */
@@ -649,6 +709,7 @@ PetscErrorCode VecView_MPI_HDF5(Vec xin, PetscViewer viewer)
   hsize_t           maxDims[4], dims[4], chunkDims[4], count[4],offset[4];
   PetscInt          timestep;
   PetscInt          low;
+  PetscInt          chunksize;
   const PetscScalar *x;
   const char        *vecname;
   PetscErrorCode    ierr;
@@ -673,6 +734,7 @@ PetscErrorCode VecView_MPI_HDF5(Vec xin, PetscViewer viewer)
    * permit extending dataset).
    */
   dim = 0;
+  chunksize = 1;
   if (timestep >= 0) {
     dims[dim]      = timestep+1;
     maxDims[dim]   = H5S_UNLIMITED;
@@ -683,19 +745,49 @@ PetscErrorCode VecView_MPI_HDF5(Vec xin, PetscViewer viewer)
 
   maxDims[dim]   = dims[dim];
   chunkDims[dim] = dims[dim];
+  chunksize      *= chunkDims[dim];
   ++dim;
   if (bs > 1 || dim2) {
     dims[dim]      = bs;
     maxDims[dim]   = dims[dim];
     chunkDims[dim] = dims[dim];
+    chunksize      *= chunkDims[dim];
     ++dim;
   }
 #if defined(PETSC_USE_COMPLEX)
   dims[dim]      = 2;
   maxDims[dim]   = dims[dim];
   chunkDims[dim] = dims[dim];
+  chunksize      *= chunkDims[dim];
+  /* hdf5 chunks must be less than the max of 32 bit int */
+  if (chunksize > PETSC_HDF5_INT_MAX/64 ) {
+    if (bs > 1 || dim2) {
+      if (chunkDims[dim-2] > (PetscInt)PetscSqrtReal((PetscReal)(PETSC_HDF5_INT_MAX/128))) {
+        chunkDims[dim-2] = (PetscInt)PetscSqrtReal((PetscReal)(PETSC_HDF5_INT_MAX/128));
+      } if (chunkDims[dim-1] > (PetscInt)PetscSqrtReal((PetscReal)(PETSC_HDF5_INT_MAX/128))) {
+        chunkDims[dim-1] = (PetscInt)PetscSqrtReal((PetscReal)(PETSC_HDF5_INT_MAX/128));
+      }
+    } else {
+      chunkDims[dim-1] = PETSC_HDF5_INT_MAX/128;
+    }
+  }
   ++dim;
+#else 
+  /* hdf5 chunks must be less than the max of 32 bit int */
+  if (chunksize > PETSC_HDF5_INT_MAX/64) {
+    if (bs > 1 || dim2) {
+      if (chunkDims[dim-2] > (PetscInt)PetscSqrtReal((PetscReal)(PETSC_HDF5_INT_MAX/64))) {
+        chunkDims[dim-2] = (PetscInt)PetscSqrtReal((PetscReal)(PETSC_HDF5_INT_MAX/64));
+      } if (chunkDims[dim-1] > (PetscInt)PetscSqrtReal((PetscReal)(PETSC_HDF5_INT_MAX/64))) {
+        chunkDims[dim-1] = (PetscInt)PetscSqrtReal((PetscReal)(PETSC_HDF5_INT_MAX/64));
+      }
+    } else {
+      chunkDims[dim-1] = PETSC_HDF5_INT_MAX/64;
+    }
+  }
 #endif
+
+
   PetscStackCallHDF5Return(filespace,H5Screate_simple,(dim, dims, maxDims));
 
 #if defined(PETSC_USE_REAL_SINGLE)
@@ -703,6 +795,8 @@ PetscErrorCode VecView_MPI_HDF5(Vec xin, PetscViewer viewer)
   filescalartype = H5T_NATIVE_FLOAT;
 #elif defined(PETSC_USE_REAL___FLOAT128)
 #error "HDF5 output with 128 bit floats not supported."
+#elif defined(PETSC_USE_REAL___FP16)
+#error "HDF5 output with 16 bit floats not supported."
 #else
   memscalartype = H5T_NATIVE_DOUBLE;
   if (spoutput == PETSC_TRUE) filescalartype = H5T_NATIVE_FLOAT;
@@ -788,6 +882,17 @@ PetscErrorCode VecView_MPI_HDF5(Vec xin, PetscViewer viewer)
   PetscStackCallHDF5(H5Fflush,(file_id, H5F_SCOPE_GLOBAL));
   ierr   = VecRestoreArrayRead(xin, &x);CHKERRQ(ierr);
 
+#if defined(PETSC_USE_COMPLEX)
+  {
+    PetscInt   one = 1;
+    const char *groupname;
+    char       vecgroup[PETSC_MAX_PATH_LEN];
+
+    ierr = PetscViewerHDF5GetGroup(viewer,&groupname);CHKERRQ(ierr);
+    ierr = PetscSNPrintf(vecgroup,PETSC_MAX_PATH_LEN,"%s/%s",groupname ? groupname : "",vecname);CHKERRQ(ierr);
+    ierr = PetscViewerHDF5WriteAttribute(viewer,vecgroup,"complex",PETSC_INT,&one);CHKERRQ(ierr);
+  }
+#endif
   /* Close/release resources */
   if (group != file_id) {
     PetscStackCallHDF5(H5Gclose,(group));
@@ -801,9 +906,7 @@ PetscErrorCode VecView_MPI_HDF5(Vec xin, PetscViewer viewer)
 }
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "VecView_MPI"
-PetscErrorCode VecView_MPI(Vec xin,PetscViewer viewer)
+PETSC_EXTERN PetscErrorCode VecView_MPI(Vec xin,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   PetscBool      iascii,isbinary,isdraw;
@@ -813,9 +916,16 @@ PetscErrorCode VecView_MPI(Vec xin,PetscViewer viewer)
 #if defined(PETSC_HAVE_HDF5)
   PetscBool      ishdf5;
 #endif
-#if defined(PETSC_HAVE_MATLAB_ENGINE) && !defined(PETSC_USE_COMPLEX) && !defined(PETSC_USE_REAL_SINGLE) && !defined(PETSC_USE_REAL___FLOAT128)
+#if defined(PETSC_HAVE_MATLAB_ENGINE)
   PetscBool      ismatlab;
 #endif
+#if defined(PETSC_HAVE_ADIOS)
+  PetscBool      isadios;
+#endif
+#if defined(PETSC_HAVE_ADIOS2)
+  PetscBool      isadios2;
+#endif
+  PetscBool      isglvis;
 
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
@@ -827,8 +937,15 @@ PetscErrorCode VecView_MPI(Vec xin,PetscViewer viewer)
 #if defined(PETSC_HAVE_HDF5)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERHDF5,&ishdf5);CHKERRQ(ierr);
 #endif
-#if defined(PETSC_HAVE_MATLAB_ENGINE) && !defined(PETSC_USE_COMPLEX) && !defined(PETSC_USE_REAL_SINGLE) && !defined(PETSC_USE_REAL___FLOAT128)
+#if defined(PETSC_HAVE_MATLAB_ENGINE)
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERMATLAB,&ismatlab);CHKERRQ(ierr);
+#endif
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERGLVIS,&isglvis);CHKERRQ(ierr);
+#if defined(PETSC_HAVE_ADIOS)
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERADIOS,&isadios);CHKERRQ(ierr);
+#endif
+#if defined(PETSC_HAVE_ADIOS2)
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERADIOS2,&isadios2);CHKERRQ(ierr);
 #endif
   if (iascii) {
     ierr = VecView_MPI_ASCII(xin,viewer);CHKERRQ(ierr);
@@ -836,7 +953,6 @@ PetscErrorCode VecView_MPI(Vec xin,PetscViewer viewer)
     ierr = VecView_MPI_Binary(xin,viewer);CHKERRQ(ierr);
   } else if (isdraw) {
     PetscViewerFormat format;
-
     ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
     if (format == PETSC_VIEWER_DRAW_LG) {
       ierr = VecView_MPI_Draw_LG(xin,viewer);CHKERRQ(ierr);
@@ -851,16 +967,24 @@ PetscErrorCode VecView_MPI(Vec xin,PetscViewer viewer)
   } else if (ishdf5) {
     ierr = VecView_MPI_HDF5(xin,viewer);CHKERRQ(ierr);
 #endif
-#if defined(PETSC_HAVE_MATLAB_ENGINE) && !defined(PETSC_USE_COMPLEX) && !defined(PETSC_USE_REAL_SINGLE) && !defined(PETSC_USE_REAL___FLOAT128)
+#if defined(PETSC_HAVE_ADIOS)
+  } else if (isadios) {
+    ierr = VecView_MPI_ADIOS(xin,viewer);CHKERRQ(ierr);
+#endif
+#if defined(PETSC_HAVE_ADIOS2)
+  } else if (isadios2) {
+    ierr = VecView_MPI_ADIOS2(xin,viewer);CHKERRQ(ierr);
+#endif
+#if defined(PETSC_HAVE_MATLAB_ENGINE)
   } else if (ismatlab) {
     ierr = VecView_MPI_Matlab(xin,viewer);CHKERRQ(ierr);
 #endif
+  } else if (isglvis) {
+    ierr = VecView_GLVis(xin,viewer);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecGetSize_MPI"
 PetscErrorCode VecGetSize_MPI(Vec xin,PetscInt *N)
 {
   PetscFunctionBegin;
@@ -868,8 +992,6 @@ PetscErrorCode VecGetSize_MPI(Vec xin,PetscInt *N)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecGetValues_MPI"
 PetscErrorCode VecGetValues_MPI(Vec xin,PetscInt ni,const PetscInt ix[],PetscScalar y[])
 {
   const PetscScalar *xx;
@@ -890,8 +1012,6 @@ PetscErrorCode VecGetValues_MPI(Vec xin,PetscInt ni,const PetscInt ix[],PetscSca
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetValues_MPI"
 PetscErrorCode VecSetValues_MPI(Vec xin,PetscInt ni,const PetscInt ix[],const PetscScalar y[],InsertMode addv)
 {
   PetscErrorCode ierr;
@@ -933,7 +1053,7 @@ PetscErrorCode VecSetValues_MPI(Vec xin,PetscInt ni,const PetscInt ix[],const Pe
         xx[row-start] += y[i];
       } else if (!xin->stash.donotstash) {
 #if defined(PETSC_USE_DEBUG)
-        if (ix[i] > xin->map->N) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Out of range index value %D maximum %D",ix[i],xin->map->N);
+        if (ix[i] >= xin->map->N) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Out of range index value %D maximum %D",ix[i],xin->map->N);
 #endif
         ierr = VecStashValue_Private(&xin->stash,row,y[i]);CHKERRQ(ierr);
       }
@@ -943,8 +1063,6 @@ PetscErrorCode VecSetValues_MPI(Vec xin,PetscInt ni,const PetscInt ix[],const Pe
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetValuesBlocked_MPI"
 PetscErrorCode VecSetValuesBlocked_MPI(Vec xin,PetscInt ni,const PetscInt ix[],const PetscScalar yin[],InsertMode addv)
 {
   PetscMPIInt    rank    = xin->stash.rank;
@@ -966,7 +1084,7 @@ PetscErrorCode VecSetValuesBlocked_MPI(Vec xin,PetscInt ni,const PetscInt ix[],c
       if ((row = bs*ix[i]) >= start && row < end) {
         for (j=0; j<bs; j++) xx[row-start+j] = y[j];
       } else if (!xin->stash.donotstash) {
-        if (ix[i] < 0) continue;
+        if (ix[i] < 0) { y += bs; continue; }
 #if defined(PETSC_USE_DEBUG)
         if (ix[i] >= xin->map->N) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Out of range index value %D max %D",ix[i],xin->map->N);
 #endif
@@ -979,7 +1097,7 @@ PetscErrorCode VecSetValuesBlocked_MPI(Vec xin,PetscInt ni,const PetscInt ix[],c
       if ((row = bs*ix[i]) >= start && row < end) {
         for (j=0; j<bs; j++) xx[row-start+j] += y[j];
       } else if (!xin->stash.donotstash) {
-        if (ix[i] < 0) continue;
+        if (ix[i] < 0) { y += bs; continue; }
 #if defined(PETSC_USE_DEBUG)
         if (ix[i] > xin->map->N) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Out of range index value %D max %D",ix[i],xin->map->N);
 #endif
@@ -996,8 +1114,6 @@ PetscErrorCode VecSetValuesBlocked_MPI(Vec xin,PetscInt ni,const PetscInt ix[],c
    Since nsends or nreceives may be zero we add 1 in certain mallocs
 to make sure we never malloc an empty one.
 */
-#undef __FUNCT__
-#define __FUNCT__ "VecAssemblyBegin_MPI"
 PetscErrorCode VecAssemblyBegin_MPI(Vec xin)
 {
   PetscErrorCode ierr;
@@ -1010,9 +1126,10 @@ PetscErrorCode VecAssemblyBegin_MPI(Vec xin)
   ierr = PetscObjectGetComm((PetscObject)xin,&comm);CHKERRQ(ierr);
   if (xin->stash.donotstash) PetscFunctionReturn(0);
 
-  ierr = MPI_Allreduce((PetscEnum*)&xin->stash.insertmode,(PetscEnum*)&addv,1,MPIU_ENUM,MPI_BOR,comm);CHKERRQ(ierr);
+  ierr = MPIU_Allreduce((PetscEnum*)&xin->stash.insertmode,(PetscEnum*)&addv,1,MPIU_ENUM,MPI_BOR,comm);CHKERRQ(ierr);
   if (addv == (ADD_VALUES|INSERT_VALUES)) SETERRQ(comm,PETSC_ERR_ARG_NOTSAMETYPE,"Some processors inserted values while others added");
   xin->stash.insertmode = addv; /* in case this processor had no cache */
+  xin->bstash.insertmode = addv; /* Block stash implicitly tracks InsertMode of scalar stash */
 
   ierr = VecGetBlockSize(xin,&bs);CHKERRQ(ierr);
   ierr = MPI_Comm_size(PetscObjectComm((PetscObject)xin),&size);CHKERRQ(ierr);
@@ -1031,8 +1148,6 @@ PetscErrorCode VecAssemblyBegin_MPI(Vec xin)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecAssemblyEnd_MPI"
 PetscErrorCode VecAssemblyEnd_MPI(Vec vec)
 {
   PetscErrorCode ierr;
