@@ -20,19 +20,20 @@ PETSC_EXTERN void PETSC_STDCALL petscsectionview_(PetscSection *s, PetscViewer *
   *ierr = PetscSectionView(*s, v);
 }
 
-PETSC_EXTERN void PETSC_STDCALL petscsectiongetfieldname_(PetscSection *s, PetscInt *field, CHAR name PETSC_MIXED_LEN(len), PetscErrorCode *ierr PETSC_END_LEN(len))
+PETSC_EXTERN void PETSC_STDCALL petscsectiongetfieldname_(PetscSection *s, PetscInt *field, char* name PETSC_MIXED_LEN(len), PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   const char *fname;
 
   *ierr = PetscSectionGetFieldName(*s, *field, &fname);if (*ierr) return;
   *ierr = PetscStrncpy(name, fname, len);
+  FIXRETURNCHAR(PETSC_TRUE,name,len);
 }
 
-PETSC_EXTERN void PETSC_STDCALL petscsectionsetfieldname_(PetscSection *s, PetscInt *field, CHAR name PETSC_MIXED_LEN(len), PetscErrorCode *ierr PETSC_END_LEN(len))
+PETSC_EXTERN void PETSC_STDCALL petscsectionsetfieldname_(PetscSection *s, PetscInt *field, char* name PETSC_MIXED_LEN(len), PetscErrorCode *ierr PETSC_END_LEN(len))
 {
   char *f;
 
   FIXCHAR(name, len, f);
-  *ierr = PetscSectionSetFieldName(*s, *field, f);
+  *ierr = PetscSectionSetFieldName(*s, *field, f);if (*ierr) return;
   FREECHAR(name, f);
 }
