@@ -7,18 +7,16 @@
 
 /* Logging support */
 PetscClassId  VEC_CLASSID;
-PetscLogEvent VEC_View, VEC_Max, VEC_Min, VEC_DotBarrier, VEC_Dot, VEC_MDotBarrier, VEC_MDot, VEC_TDot;
+PetscLogEvent VEC_View, VEC_Max, VEC_Min, VEC_Dot, VEC_MDot, VEC_TDot;
 PetscLogEvent VEC_Norm, VEC_Normalize, VEC_Scale, VEC_Copy, VEC_Set, VEC_AXPY, VEC_AYPX, VEC_WAXPY;
-PetscLogEvent VEC_MTDot, VEC_NormBarrier, VEC_MAXPY, VEC_Swap, VEC_AssemblyBegin, VEC_ScatterBegin, VEC_ScatterEnd;
-PetscLogEvent VEC_AssemblyEnd, VEC_PointwiseMult, VEC_SetValues, VEC_Load, VEC_ScatterBarrier;
-PetscLogEvent VEC_SetRandom, VEC_ReduceArithmetic, VEC_ReduceBarrier, VEC_ReduceCommunication,VEC_ReduceBegin,VEC_ReduceEnd,VEC_Ops;
-PetscLogEvent VEC_DotNormBarrier, VEC_DotNorm, VEC_AXPBYPCZ, VEC_CUSPCopyFromGPU, VEC_CUSPCopyToGPU;
-PetscLogEvent VEC_CUSPCopyFromGPUSome, VEC_CUSPCopyToGPUSome;
+PetscLogEvent VEC_MTDot, VEC_MAXPY, VEC_Swap, VEC_AssemblyBegin, VEC_ScatterBegin, VEC_ScatterEnd;
+PetscLogEvent VEC_AssemblyEnd, VEC_PointwiseMult, VEC_SetValues, VEC_Load;
+PetscLogEvent VEC_SetRandom, VEC_ReduceArithmetic, VEC_ReduceCommunication,VEC_ReduceBegin,VEC_ReduceEnd,VEC_Ops;
+PetscLogEvent VEC_DotNorm2, VEC_AXPBYPCZ;
 PetscLogEvent VEC_ViennaCLCopyFromGPU, VEC_ViennaCLCopyToGPU;
+PetscLogEvent VEC_CUDACopyFromGPU, VEC_CUDACopyToGPU;
+PetscLogEvent VEC_CUDACopyFromGPUSome, VEC_CUDACopyToGPUSome;
 
-extern PetscErrorCode VecStashGetInfo_Private(VecStash*,PetscInt*,PetscInt*);
-#undef __FUNCT__
-#define __FUNCT__ "VecStashGetInfo"
 /*@
    VecStashGetInfo - Gets how many values are currently in the vector stash, i.e. need
        to be communicated to other processors during the VecAssemblyBegin/End() process
@@ -49,8 +47,6 @@ PetscErrorCode  VecStashGetInfo(Vec vec,PetscInt *nstash,PetscInt *reallocs,Pets
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetLocalToGlobalMapping"
 /*@
    VecSetLocalToGlobalMapping - Sets a local numbering to global numbering used
    by the routine VecSetValuesLocal() to allow users to insert vector entries
@@ -88,8 +84,6 @@ PetscErrorCode  VecSetLocalToGlobalMapping(Vec x,ISLocalToGlobalMapping mapping)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecGetLocalToGlobalMapping"
 /*@
    VecGetLocalToGlobalMapping - Gets the local-to-global numbering set by VecSetLocalToGlobalMapping()
 
@@ -118,8 +112,6 @@ PetscErrorCode VecGetLocalToGlobalMapping(Vec X,ISLocalToGlobalMapping *mapping)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecAssemblyBegin"
 /*@
    VecAssemblyBegin - Begins assembling the vector.  This routine should
    be called after completing all calls to VecSetValues().
@@ -152,8 +144,6 @@ PetscErrorCode  VecAssemblyBegin(Vec vec)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecAssemblyEnd"
 /*@
    VecAssemblyEnd - Completes assembling the vector.  This routine should
    be called after VecAssemblyBegin().
@@ -192,8 +182,6 @@ PetscErrorCode  VecAssemblyEnd(Vec vec)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseMax"
 /*@
    VecPointwiseMax - Computes the componentwise maximum w_i = max(x_i, y_i).
 
@@ -207,7 +195,8 @@ PetscErrorCode  VecAssemblyEnd(Vec vec)
 
    Level: advanced
 
-   Notes: any subset of the x, y, and w may be the same vector.
+   Notes:
+    any subset of the x, y, and w may be the same vector.
           For complex numbers compares only the real part
 
    Concepts: vector^pointwise multiply
@@ -227,17 +216,14 @@ PetscErrorCode  VecPointwiseMax(Vec w,Vec x,Vec y)
   PetscValidType(y,3);
   PetscCheckSameTypeAndComm(x,2,y,3);
   PetscCheckSameTypeAndComm(y,3,w,1);
-  if (x->map->N != y->map->N || x->map->N != w->map->N) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Incompatible vector global lengths");
-  if (x->map->n != y->map->n || x->map->n != w->map->n) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Incompatible vector local lengths");
-
+  VecCheckSameSize(w,1,x,2);
+  VecCheckSameSize(w,1,y,3);
   ierr = (*w->ops->pointwisemax)(w,x,y);CHKERRQ(ierr);
   ierr = PetscObjectStateIncrease((PetscObject)w);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseMin"
 /*@
    VecPointwiseMin - Computes the componentwise minimum w_i = min(x_i, y_i).
 
@@ -251,7 +237,8 @@ PetscErrorCode  VecPointwiseMax(Vec w,Vec x,Vec y)
 
    Level: advanced
 
-   Notes: any subset of the x, y, and w may be the same vector.
+   Notes:
+    any subset of the x, y, and w may be the same vector.
           For complex numbers compares only the real part
 
    Concepts: vector^pointwise multiply
@@ -271,16 +258,13 @@ PetscErrorCode  VecPointwiseMin(Vec w,Vec x,Vec y)
   PetscValidType(y,3);
   PetscCheckSameTypeAndComm(x,2,y,3);
   PetscCheckSameTypeAndComm(y,3,w,1);
-  if (x->map->N != y->map->N || x->map->N != w->map->N) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Incompatible vector global lengths");
-  if (x->map->n != y->map->n || x->map->n != w->map->n) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Incompatible vector local lengths");
-
+  VecCheckSameSize(w,1,x,2);
+  VecCheckSameSize(w,1,y,3);
   ierr = (*w->ops->pointwisemin)(w,x,y);CHKERRQ(ierr);
   ierr = PetscObjectStateIncrease((PetscObject)w);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseMaxAbs"
 /*@
    VecPointwiseMaxAbs - Computes the componentwise maximum of the absolute values w_i = max(abs(x_i), abs(y_i)).
 
@@ -294,7 +278,8 @@ PetscErrorCode  VecPointwiseMin(Vec w,Vec x,Vec y)
 
    Level: advanced
 
-   Notes: any subset of the x, y, and w may be the same vector.
+   Notes:
+    any subset of the x, y, and w may be the same vector.
 
    Concepts: vector^pointwise multiply
 
@@ -313,16 +298,13 @@ PetscErrorCode  VecPointwiseMaxAbs(Vec w,Vec x,Vec y)
   PetscValidType(y,3);
   PetscCheckSameTypeAndComm(x,2,y,3);
   PetscCheckSameTypeAndComm(y,3,w,1);
-  if (x->map->N != y->map->N || x->map->N != w->map->N) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Incompatible vector global lengths");
-  if (x->map->n != y->map->n || x->map->n != w->map->n) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Incompatible vector local lengths");
-
+  VecCheckSameSize(w,1,x,2);
+  VecCheckSameSize(w,1,y,3);
   ierr = (*w->ops->pointwisemaxabs)(w,x,y);CHKERRQ(ierr);
   ierr = PetscObjectStateIncrease((PetscObject)w);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseDivide"
 /*@
    VecPointwiseDivide - Computes the componentwise division w = x/y.
 
@@ -336,7 +318,8 @@ PetscErrorCode  VecPointwiseMaxAbs(Vec w,Vec x,Vec y)
 
    Level: advanced
 
-   Notes: any subset of the x, y, and w may be the same vector.
+   Notes:
+    any subset of the x, y, and w may be the same vector.
 
    Concepts: vector^pointwise divide
 
@@ -355,17 +338,14 @@ PetscErrorCode  VecPointwiseDivide(Vec w,Vec x,Vec y)
   PetscValidType(y,3);
   PetscCheckSameTypeAndComm(x,2,y,3);
   PetscCheckSameTypeAndComm(y,3,w,1);
-  if (x->map->N != y->map->N || x->map->N != w->map->N) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Incompatible vector global lengths");
-  if (x->map->n != y->map->n || x->map->n != w->map->n) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Incompatible vector local lengths");
-
+  VecCheckSameSize(w,1,x,2);
+  VecCheckSameSize(w,1,y,3);
   ierr = (*w->ops->pointwisedivide)(w,x,y);CHKERRQ(ierr);
   ierr = PetscObjectStateIncrease((PetscObject)w);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "VecDuplicate"
 /*@
    VecDuplicate - Creates a new vector of the same type as an existing vector.
 
@@ -401,8 +381,6 @@ PetscErrorCode  VecDuplicate(Vec v,Vec *newv)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecDestroy"
 /*@
    VecDestroy - Destroys a vector.
 
@@ -435,8 +413,6 @@ PetscErrorCode  VecDestroy(Vec *v)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecDuplicateVecs"
 /*@C
    VecDuplicateVecs - Creates several vectors of the same type as an existing vector.
 
@@ -470,12 +446,10 @@ PetscErrorCode  VecDuplicateVecs(Vec v,PetscInt m,Vec *V[])
   PetscValidHeaderSpecific(v,VEC_CLASSID,1);
   PetscValidPointer(V,3);
   PetscValidType(v,1);
-  ierr = (*v->ops->duplicatevecs)(v, m,V);CHKERRQ(ierr);
+  ierr = (*v->ops->duplicatevecs)(v,m,V);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecDestroyVecs"
 /*@C
    VecDestroyVecs - Frees a block of vectors obtained with VecDuplicateVecs().
 
@@ -499,19 +473,15 @@ PetscErrorCode  VecDestroyVecs(PetscInt m,Vec *vv[])
 
   PetscFunctionBegin;
   PetscValidPointer(vv,1);
-  if (!*vv) PetscFunctionReturn(0);
+  if (m < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Trying to destroy negative number of vectors %D",m);
+  if (!m || !*vv) {*vv  = NULL; PetscFunctionReturn(0);}
   PetscValidHeaderSpecific(**vv,VEC_CLASSID,1);
   PetscValidType(**vv,1);
-  if (m < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Trying to destroy negative number of vectors %D",m);
-  if (!m) PetscFunctionReturn(0);
-  if (!*vv) PetscFunctionReturn(0);
   ierr = (*(**vv)->ops->destroyvecs)(m,*vv);CHKERRQ(ierr);
-  *vv  = 0;
+  *vv  = NULL;
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecView"
 /*@C
    VecView - Views a vector object.
 
@@ -528,7 +498,7 @@ PetscErrorCode  VecDestroyVecs(PetscInt m,Vec *vv[])
 -     PETSC_VIEWER_STDOUT_(comm) - for parallel vectors created on MPI communicator comm
 
    You can change the format the vector is printed using the
-   option PetscViewerSetFormat().
+   option PetscViewerPushFormat().
 
    The user can open alternative visualization contexts with
 +    PetscViewerASCIIOpen() - Outputs vector to a specified file
@@ -537,7 +507,7 @@ PetscErrorCode  VecDestroyVecs(PetscInt m,Vec *vv[])
 .    PetscViewerDrawOpen() - Outputs vector to an X window display
 -    PetscViewerSocketOpen() - Outputs vector to Socket viewer
 
-   The user can call PetscViewerSetFormat() to specify the output
+   The user can call PetscViewerPushFormat() to specify the output
    format of ASCII printed objects (when using PETSC_VIEWER_STDOUT_SELF,
    PETSC_VIEWER_STDOUT_WORLD and PetscViewerASCIIOpen).  Available formats include
 +    PETSC_VIEWER_DEFAULT - default, prints vector contents
@@ -546,7 +516,8 @@ PetscErrorCode  VecDestroyVecs(PetscInt m,Vec *vv[])
 -    PETSC_VIEWER_ASCII_COMMON - prints vector contents, using a
          format common among all vector types
 
-   Notes: You can pass any number of vector objects, or other PETSc objects to the same viewer.
+   Notes:
+    You can pass any number of vector objects, or other PETSc objects to the same viewer.
 
    Notes for binary viewer: If you pass multiply vectors to a binary viewer you can read them back in in the same order
 $     with VecLoad().
@@ -579,6 +550,7 @@ PetscErrorCode  VecView(Vec vec,PetscViewer viewer)
   PetscErrorCode    ierr;
   PetscBool         iascii;
   PetscViewerFormat format;
+  PetscMPIInt       size;
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(vec,VEC_CLASSID,1);
@@ -587,11 +559,13 @@ PetscErrorCode  VecView(Vec vec,PetscViewer viewer)
     ierr = PetscViewerASCIIGetStdout(PetscObjectComm((PetscObject)vec),&viewer);CHKERRQ(ierr);
   }
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
-  /* PetscCheckSameComm(vec,1,viewer,2); Viewing a SELF vec on a parallel viewer should be acceptable */
+  ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(PetscObjectComm((PetscObject)vec),&size);CHKERRQ(ierr);
+  if (size == 1 && format == PETSC_VIEWER_LOAD_BALANCE) PetscFunctionReturn(0);
+
   if (vec->stash.n || vec->bstash.n) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Must call VecAssemblyBegin/End() before viewing this vector");
 
   ierr = PetscLogEventBegin(VEC_View,vec,viewer,0,0);CHKERRQ(ierr);
-  ierr = PetscViewerGetFormat(viewer,&format);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&iascii);CHKERRQ(ierr);
   if (iascii) {
     PetscInt rows,bs;
@@ -610,7 +584,7 @@ PetscErrorCode  VecView(Vec vec,PetscViewer viewer)
     }
   }
   ierr = VecLockPush(vec);CHKERRQ(ierr);
-  if (format == PETSC_VIEWER_NATIVE && vec->ops->viewnative) {
+  if ((format == PETSC_VIEWER_NATIVE || format == PETSC_VIEWER_LOAD_BALANCE) && vec->ops->viewnative) {
     ierr = (*vec->ops->viewnative)(vec,viewer);CHKERRQ(ierr);
   } else {
     ierr = (*vec->ops->view)(vec,viewer);CHKERRQ(ierr);
@@ -640,8 +614,6 @@ PETSC_UNUSED static int TV_display_type(const struct _p_Vec *v)
 }
 #endif
 
-#undef __FUNCT__
-#define __FUNCT__ "VecGetSize"
 /*@
    VecGetSize - Returns the global number of elements of the vector.
 
@@ -671,8 +643,6 @@ PetscErrorCode  VecGetSize(Vec x,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecGetLocalSize"
 /*@
    VecGetLocalSize - Returns the number of elements of the vector stored
    in local memory. This routine may be implementation dependent, so use
@@ -704,8 +674,6 @@ PetscErrorCode  VecGetLocalSize(Vec x,PetscInt *size)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecGetOwnershipRange"
 /*@C
    VecGetOwnershipRange - Returns the range of indices owned by
    this processor, assuming that the vectors are laid out with the
@@ -746,8 +714,6 @@ PetscErrorCode  VecGetOwnershipRange(Vec x,PetscInt *low,PetscInt *high)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecGetOwnershipRanges"
 /*@C
    VecGetOwnershipRanges - Returns the range of indices owned by EACH processor,
    assuming that the vectors are laid out with the
@@ -786,8 +752,6 @@ PetscErrorCode  VecGetOwnershipRanges(Vec x,const PetscInt *ranges[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetOption"
 /*@
    VecSetOption - Sets an option for controling a vector's behavior.
 
@@ -806,6 +770,14 @@ PetscErrorCode  VecGetOwnershipRanges(Vec x,const PetscInt *ranges[])
 .     VEC_IGNORE_NEGATIVE_INDICES, which means you can pass negative indices
           in ix in calls to VecSetValues() or VecGetValues(). These rows are simply
           ignored.
+-     VEC_SUBSET_OFF_PROC_ENTRIES, which causes VecAssemblyBegin() to assume that the off-process
+          entries will always be a subset (possibly equal) of the off-process entries set on the
+          first assembly.  This reuses the communication pattern, thus avoiding a global reduction.
+          Subsequent assemblies setting off-process values should use the same InsertMode as the
+          first assembly.
+
+   Developer Note:
+   The InsertMode restriction could be removed by packing the stash messages out of place.
 
    Level: intermediate
 
@@ -823,8 +795,6 @@ PetscErrorCode  VecSetOption(Vec x,VecOption op,PetscBool flag)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecDuplicateVecs_Default"
 /* Default routines for obtaining and releasing; */
 /* may be used by any implementation */
 PetscErrorCode VecDuplicateVecs_Default(Vec w,PetscInt m,Vec *V[])
@@ -841,8 +811,6 @@ PetscErrorCode VecDuplicateVecs_Default(Vec w,PetscInt m,Vec *V[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecDestroyVecs_Default"
 PetscErrorCode VecDestroyVecs_Default(PetscInt m,Vec v[])
 {
   PetscErrorCode ierr;
@@ -855,8 +823,6 @@ PetscErrorCode VecDestroyVecs_Default(PetscInt m,Vec v[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecResetArray"
 /*@
    VecResetArray - Resets a vector to use its default memory. Call this
    after the use of VecPlaceArray().
@@ -885,8 +851,6 @@ PetscErrorCode  VecResetArray(Vec vec)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecLoad"
 /*@C
   VecLoad - Loads a vector that has been stored in binary or HDF5 format
   with VecView().
@@ -925,7 +889,7 @@ PetscErrorCode  VecResetArray(Vec vec)
   Notes for advanced users:
   Most users should not need to know the details of the binary storage
   format, since VecLoad() and VecView() completely hide these details.
-  But for anyone who's interested, the standard binary matrix storage
+  But for anyone who's interested, the standard binary vector storage
   format is
 .vb
      int    VEC_FILE_CLASSID
@@ -945,8 +909,8 @@ and PetscBinaryWrite() to see how this may be done.
 @*/
 PetscErrorCode  VecLoad(Vec newvec, PetscViewer viewer)
 {
-  PetscErrorCode ierr;
-  PetscBool      isbinary,ishdf5;
+  PetscErrorCode    ierr;
+  PetscBool         isbinary,ishdf5,isadios,isadios2;
   PetscViewerFormat format;
 
   PetscFunctionBegin;
@@ -954,7 +918,9 @@ PetscErrorCode  VecLoad(Vec newvec, PetscViewer viewer)
   PetscValidHeaderSpecific(viewer,PETSC_VIEWER_CLASSID,2);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERBINARY,&isbinary);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERHDF5,&ishdf5);CHKERRQ(ierr);
-  if (!isbinary && !ishdf5) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Invalid viewer; open viewer with PetscViewerBinaryOpen()");
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERADIOS,&isadios);CHKERRQ(ierr);
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERADIOS,&isadios2);CHKERRQ(ierr);    
+  if (!isbinary && !ishdf5 && !isadios && !isadios2) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"Invalid viewer; open viewer with PetscViewerBinaryOpen()");
 
   ierr = PetscLogEventBegin(VEC_Load,viewer,0,0,0);CHKERRQ(ierr);
   if (!((PetscObject)newvec)->type_name && !newvec->ops->create) {
@@ -971,8 +937,6 @@ PetscErrorCode  VecLoad(Vec newvec, PetscViewer viewer)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "VecReciprocal"
 /*@
    VecReciprocal - Replaces each component of a vector by its reciprocal.
 
@@ -1005,8 +969,6 @@ PetscErrorCode  VecReciprocal(Vec vec)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetOperation"
 /*@C
     VecSetOperation - Allows user to set a vector operation.
 
@@ -1050,8 +1012,6 @@ PetscErrorCode VecSetOperation(Vec vec,VecOperation op, void (*f)(void))
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "VecStashSetInitialSize"
 /*@
    VecStashSetInitialSize - sets the sizes of the vec-stash, that is
    used during the assembly process to store values that belong to
@@ -1097,8 +1057,6 @@ PetscErrorCode  VecStashSetInitialSize(Vec vec,PetscInt size,PetscInt bsize)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecConjugate"
 /*@
    VecConjugate - Conjugates a vector.
 
@@ -1130,8 +1088,6 @@ PetscErrorCode  VecConjugate(Vec x)
 #endif
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecPointwiseMult"
 /*@
    VecPointwiseMult - Computes the componentwise multiplication w = x*y.
 
@@ -1145,7 +1101,8 @@ PetscErrorCode  VecConjugate(Vec x)
 
    Level: advanced
 
-   Notes: any subset of the x, y, and w may be the same vector.
+   Notes:
+    any subset of the x, y, and w may be the same vector.
 
    Concepts: vector^pointwise multiply
 
@@ -1164,8 +1121,8 @@ PetscErrorCode  VecPointwiseMult(Vec w, Vec x,Vec y)
   PetscValidType(y,3);
   PetscCheckSameTypeAndComm(x,2,y,3);
   PetscCheckSameTypeAndComm(y,3,w,1);
-  if (x->map->n != y->map->n || x->map->n != w->map->n) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Incompatible vector local lengths");
-
+  VecCheckSameSize(w,1,x,2);
+  VecCheckSameSize(w,2,y,3);
   ierr = PetscLogEventBegin(VEC_PointwiseMult,x,y,w,0);CHKERRQ(ierr);
   ierr = (*w->ops->pointwisemult)(w,x,y);CHKERRQ(ierr);
   ierr = PetscLogEventEnd(VEC_PointwiseMult,x,y,w,0);CHKERRQ(ierr);
@@ -1173,8 +1130,6 @@ PetscErrorCode  VecPointwiseMult(Vec w, Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetRandom"
 /*@
    VecSetRandom - Sets all components of a vector to random numbers.
 
@@ -1230,8 +1185,6 @@ PetscErrorCode  VecSetRandom(Vec x,PetscRandom rctx)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecZeroEntries"
 /*@
   VecZeroEntries - puts a 0.0 in each element of a vector
 
@@ -1259,8 +1212,6 @@ PetscErrorCode  VecZeroEntries(Vec vec)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetTypeFromOptions_Private"
 /*
   VecSetTypeFromOptions_Private - Sets the type of vector from user options. Defaults to a PETSc sequential vector on one
   processor and a PETSc MPI vector on more than one processor.
@@ -1275,7 +1226,7 @@ PetscErrorCode  VecZeroEntries(Vec vec)
 .keywords: Vec, set, options, database, type
 .seealso: VecSetFromOptions(), VecSetType()
 */
-static PetscErrorCode VecSetTypeFromOptions_Private(PetscOptions *PetscOptionsObject,Vec vec)
+static PetscErrorCode VecSetTypeFromOptions_Private(PetscOptionItems *PetscOptionsObject,Vec vec)
 {
   PetscBool      opt;
   VecType        defaultType;
@@ -1301,8 +1252,6 @@ static PetscErrorCode VecSetTypeFromOptions_Private(PetscOptions *PetscOptionsOb
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetFromOptions"
 /*@
   VecSetFromOptions - Configures the vector from the options database.
 
@@ -1311,7 +1260,8 @@ static PetscErrorCode VecSetTypeFromOptions_Private(PetscOptions *PetscOptionsOb
   Input Parameter:
 . vec - The vector
 
-  Notes:  To see all options, run your program with the -help option, or consult the users manual.
+  Notes:
+    To see all options, run your program with the -help option, or consult the users manual.
           Must be called after VecCreate() but before the vector is used.
 
   Level: beginner
@@ -1339,13 +1289,11 @@ PetscErrorCode  VecSetFromOptions(Vec vec)
   }
 
   /* process any options handlers added with PetscObjectAddOptionsHandler() */
-  ierr = PetscObjectProcessOptionsHandlers((PetscObject)vec);CHKERRQ(ierr);
+  ierr = PetscObjectProcessOptionsHandlers(PetscOptionsObject,(PetscObject)vec);CHKERRQ(ierr);
   ierr = PetscOptionsEnd();CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetSizes"
 /*@
   VecSetSizes - Sets the local and global sizes, and checks to determine compatibility
 
@@ -1382,8 +1330,6 @@ PetscErrorCode  VecSetSizes(Vec v, PetscInt n, PetscInt N)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetBlockSize"
 /*@
    VecSetBlockSize - Sets the blocksize for future calls to VecSetValuesBlocked()
    and VecSetValuesBlockedLocal().
@@ -1416,8 +1362,6 @@ PetscErrorCode  VecSetBlockSize(Vec v,PetscInt bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecGetBlockSize"
 /*@
    VecGetBlockSize - Gets the blocksize for the vector, i.e. what is used for VecSetValuesBlocked()
    and VecSetValuesBlockedLocal().
@@ -1452,8 +1396,6 @@ PetscErrorCode  VecGetBlockSize(Vec v,PetscInt *bs)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetOptionsPrefix"
 /*@C
    VecSetOptionsPrefix - Sets the prefix used for searching for all
    Vec options in the database.
@@ -1484,8 +1426,6 @@ PetscErrorCode  VecSetOptionsPrefix(Vec v,const char prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecAppendOptionsPrefix"
 /*@C
    VecAppendOptionsPrefix - Appends to the prefix used for searching for all
    Vec options in the database.
@@ -1516,8 +1456,6 @@ PetscErrorCode  VecAppendOptionsPrefix(Vec v,const char prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecGetOptionsPrefix"
 /*@C
    VecGetOptionsPrefix - Sets the prefix used for searching for all
    Vec options in the database.
@@ -1530,7 +1468,8 @@ PetscErrorCode  VecAppendOptionsPrefix(Vec v,const char prefix[])
    Output Parameter:
 .  prefix - pointer to the prefix string used
 
-   Notes: On the fortran side, the user should pass in a string 'prefix' of
+   Notes:
+    On the fortran side, the user should pass in a string 'prefix' of
    sufficient length to hold the prefix.
 
    Level: advanced
@@ -1549,8 +1488,6 @@ PetscErrorCode  VecGetOptionsPrefix(Vec v,const char *prefix[])
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetUp"
 /*@
    VecSetUp - Sets up the internal vector data structures for the later use.
 
@@ -1593,8 +1530,6 @@ PetscErrorCode  VecSetUp(Vec v)
     will become independent of PetscScalar/PetscReal
 */
 
-#undef __FUNCT__
-#define __FUNCT__ "VecCopy"
 /*@
    VecCopy - Copies a vector. y <- x
 
@@ -1610,6 +1545,11 @@ PetscErrorCode  VecSetUp(Vec v)
    For default parallel PETSc vectors, both x and y must be distributed in
    the same manner; local copies are done.
 
+   Developer Notes:
+   PetscCheckSameTypeAndComm(x,1,y,2) is not used on these vectors because we allow one
+   of the vectors to be sequential and one to be parallel so long as both have the same
+   local sizes. This is used in some internal functions in PETSc.
+
    Level: beginner
 
 .seealso: VecDuplicate()
@@ -1627,8 +1567,8 @@ PetscErrorCode  VecCopy(Vec x,Vec y)
   PetscValidType(x,1);
   PetscValidType(y,2);
   if (x == y) PetscFunctionReturn(0);
+  VecCheckSameLocalSize(x,1,y,2);
   if (x->stash.insertmode != NOT_SET_VALUES) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Not for unassembled vector");
-  if (x->map->n != y->map->n) SETERRQ2(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Incompatible vector local lengths %d != %d", x->map->n, y->map->n);
   VecLocked(y,2);
 
 #if !defined(PETSC_USE_MIXED_PRECISION)
@@ -1687,8 +1627,6 @@ PetscErrorCode  VecCopy(Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSwap"
 /*@
    VecSwap - Swaps the vectors x and y.
 
@@ -1715,10 +1653,9 @@ PetscErrorCode  VecSwap(Vec x,Vec y)
   PetscValidType(x,1);
   PetscValidType(y,2);
   PetscCheckSameTypeAndComm(x,1,y,2);
+  VecCheckSameSize(x,1,y,2);
   if (x->stash.insertmode != NOT_SET_VALUES) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Not for unassembled vector");
   if (y->stash.insertmode != NOT_SET_VALUES) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONGSTATE,"Not for unassembled vector");
-  if (x->map->N != y->map->N) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Incompatible vector global lengths");
-  if (x->map->n != y->map->n) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_INCOMP,"Incompatible vector local lengths");
 
   ierr = PetscLogEventBegin(VEC_Swap,x,y,0,0);CHKERRQ(ierr);
   for (i=0; i<4; i++) {
@@ -1740,10 +1677,8 @@ PetscErrorCode  VecSwap(Vec x,Vec y)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecStashViewFromOptions"
 /*
-  VecStashViewFromOptions - Processes command line options to determine if/how an VecStash object is to be viewed. 
+  VecStashViewFromOptions - Processes command line options to determine if/how an VecStash object is to be viewed.
 
   Collective on VecStash
 
@@ -1777,8 +1712,6 @@ PetscErrorCode VecStashViewFromOptions(Vec obj,PetscObject bobj,const char optio
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecStashView"
 /*@
    VecStashView - Prints the entries in the vector stash and block stash.
 
@@ -1817,7 +1750,7 @@ PetscErrorCode  VecStashView(Vec v,PetscViewer viewer)
   s    = &v->bstash;
 
   /* print block stash */
-  ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_TRUE);CHKERRQ(ierr);
+  ierr = PetscViewerASCIIPushSynchronized(viewer);CHKERRQ(ierr);
   ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d]Vector Block stash size %D block size %D\n",rank,s->n,s->bs);CHKERRQ(ierr);
   for (i=0; i<s->n; i++) {
     ierr = PetscViewerASCIISynchronizedPrintf(viewer,"[%d] Element %D ",rank,s->idx[i]);CHKERRQ(ierr);
@@ -1846,15 +1779,12 @@ PetscErrorCode  VecStashView(Vec v,PetscViewer viewer)
 #endif
   }
   ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
-  ierr = PetscViewerASCIISynchronizedAllow(viewer,PETSC_FALSE);CHKERRQ(ierr);
-
+  ierr = PetscViewerASCIIPopSynchronized(viewer);CHKERRQ(ierr);
   ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "PetscOptionsGetVec"
-PetscErrorCode PetscOptionsGetVec(const char prefix[],const char key[],Vec v,PetscBool *set)
+PetscErrorCode PetscOptionsGetVec(PetscOptions options,const char prefix[],const char key[],Vec v,PetscBool *set)
 {
   PetscInt       i,N,rstart,rend;
   PetscErrorCode ierr;
@@ -1866,7 +1796,7 @@ PetscErrorCode PetscOptionsGetVec(const char prefix[],const char key[],Vec v,Pet
   ierr = VecGetOwnershipRange(v,&rstart,&rend);CHKERRQ(ierr);
   ierr = VecGetSize(v,&N);CHKERRQ(ierr);
   ierr = PetscCalloc1(N,&xreal);CHKERRQ(ierr);
-  ierr = PetscOptionsGetRealArray(prefix,key,xreal,&N,&iset);CHKERRQ(ierr);
+  ierr = PetscOptionsGetRealArray(options,prefix,key,xreal,&N,&iset);CHKERRQ(ierr);
   if (iset) {
     ierr = VecGetArray(v,&xx);CHKERRQ(ierr);
     for (i=rstart; i<rend; i++) xx[i-rstart] = xreal[i];
@@ -1877,8 +1807,6 @@ PetscErrorCode PetscOptionsGetVec(const char prefix[],const char key[],Vec v,Pet
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecGetLayout"
 /*@
    VecGetLayout - get PetscLayout describing vector layout
 
@@ -1903,8 +1831,6 @@ PetscErrorCode VecGetLayout(Vec x,PetscLayout *map)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetLayout"
 /*@
    VecSetLayout - set PetscLayout describing vector layout
 
@@ -1931,8 +1857,6 @@ PetscErrorCode VecSetLayout(Vec x,PetscLayout map)
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__
-#define __FUNCT__ "VecSetInf"
 PetscErrorCode VecSetInf(Vec xin)
 {
   PetscInt       i,n = xin->map->n;
