@@ -1,5 +1,5 @@
 #include <petscvec.h>
-#include <../src/sys/f90-src/f90impl.h>
+#include <petsc/private/f90impl.h>
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
 #define vecsetvaluessectionf90_ VECSETVALUESSECTIONF90
@@ -13,6 +13,6 @@ PETSC_EXTERN void PETSC_STDCALL vecsetvaluessectionf90_(Vec *v, PetscSection *se
 {
   PetscScalar *array;
 
-  *__ierr = F90Array1dAccess(ptr, PETSC_SCALAR, (void**) &array PETSC_F90_2PTR_PARAM(ptrd));if (*__ierr) return;
+  *__ierr = F90Array1dAccess(ptr, MPIU_SCALAR, (void**) &array PETSC_F90_2PTR_PARAM(ptrd));if (*__ierr) return;
   *__ierr = VecSetValuesSection(*v, *section, *point, array, *mode);
 }
