@@ -1,9 +1,7 @@
 
 #include <petsc/private/vecimpl.h>    /*I   "petscvec.h"    I*/
 
-#undef __FUNCT__
-#define __FUNCT__ "VecScatterCreateToAll"
-/*@
+/*@C
       VecScatterCreateToAll - Creates a vector and a scatter context that copies all
           vector values to each processor
 
@@ -18,7 +16,7 @@
 
   Level: intermediate
 
-   Note: vout may be NULL [NULL_OBJECT from fortran] if you do not
+   Note: vout may be NULL [PETSC_NULL_VEC from fortran] if you do not
    need to have it created
 
    Usage:
@@ -72,9 +70,7 @@ PetscErrorCode  VecScatterCreateToAll(Vec vin,VecScatter *ctx,Vec *vout)
 }
 
 
-#undef __FUNCT__
-#define __FUNCT__ "VecScatterCreateToZero"
-/*@
+/*@C
       VecScatterCreateToZero - Creates an output vector and a scatter context used to
               copy all vector values into the output vector on the zeroth processor
 
@@ -90,7 +86,7 @@ PetscErrorCode  VecScatterCreateToAll(Vec vin,VecScatter *ctx,Vec *vout)
 
   Level: intermediate
 
-   Note: vout may be NULL [NULL_OBJECT from fortran] if you do not
+   Note: vout may be NULL [PETSC_NULL_VEC from fortran] if you do not
    need to have it created
 
    Usage:
@@ -146,3 +142,44 @@ PetscErrorCode  VecScatterCreateToZero(Vec vin,VecScatter *ctx,Vec *vout)
   PetscFunctionReturn(0);
 }
 
+#include <petscsf.h>
+
+/*@
+  PetscSFCreateFromZero - Create a PetscSF that maps a Vec from sequential to distributed
+
+  Input Parameters:
+. gv - A distributed Vec
+
+  Output Parameters:
+. sf - The SF created mapping a sequential Vec to gv
+
+  Level: developer
+
+.seealso: DMPlexDistributedToSequential()
+@*/
+PetscErrorCode PetscSFCreateFromZero(MPI_Comm comm, Vec gv, PetscSF *sf)
+{
+  PetscSFNode   *remotenodes;
+  PetscInt      *localnodes;
+  PetscInt       N, n, start, numroots, l;
+  PetscMPIInt    rank;
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  ierr = PetscSFCreate(comm, sf);CHKERRQ(ierr);
+  ierr = VecGetSize(gv, &N);CHKERRQ(ierr);
+  ierr = VecGetLocalSize(gv, &n);CHKERRQ(ierr);
+  ierr = VecGetOwnershipRange(gv, &start, NULL);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(comm, &rank);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &localnodes);CHKERRQ(ierr);
+  ierr = PetscMalloc1(n, &remotenodes);CHKERRQ(ierr);
+  if (!rank) numroots = N;
+  else       numroots = 0;
+  for (l = 0; l < n; ++l) {
+    localnodes[l]        = l;
+    remotenodes[l].rank  = 0;
+    remotenodes[l].index = l+start;
+  }
+  ierr = PetscSFSetGraph(*sf, numroots, n, localnodes, PETSC_OWN_POINTER, remotenodes, PETSC_OWN_POINTER);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
