@@ -17,8 +17,6 @@
   Output Parameters:
   stash    - the newly created stash
 */
-#undef __FUNCT__
-#define __FUNCT__ "VecStashCreate_Private"
 PetscErrorCode VecStashCreate_Private(MPI_Comm comm,PetscInt bs,VecStash *stash)
 {
   PetscErrorCode ierr;
@@ -35,7 +33,7 @@ PetscErrorCode VecStashCreate_Private(MPI_Comm comm,PetscInt bs,VecStash *stash)
 
   nopt = stash->size;
   ierr = PetscMalloc1(nopt,&opt);CHKERRQ(ierr);
-  ierr = PetscOptionsGetIntArray(NULL,"-vecstash_initial_size",opt,&nopt,&flg);CHKERRQ(ierr);
+  ierr = PetscOptionsGetIntArray(NULL,NULL,"-vecstash_initial_size",opt,&nopt,&flg);CHKERRQ(ierr);
   if (flg) {
     if (nopt == 1)                max = opt[0];
     else if (nopt == stash->size) max = opt[stash->rank];
@@ -75,8 +73,6 @@ PetscErrorCode VecStashCreate_Private(MPI_Comm comm,PetscInt bs,VecStash *stash)
 /*
    VecStashDestroy_Private - Destroy the stash
 */
-#undef __FUNCT__
-#define __FUNCT__ "VecStashDestroy_Private"
 PetscErrorCode VecStashDestroy_Private(VecStash *stash)
 {
   PetscErrorCode ierr;
@@ -95,8 +91,6 @@ PetscErrorCode VecStashDestroy_Private(VecStash *stash)
    for the stash. It also keeps track of the current memory usage
    so that the same value can be used the next time through.
 */
-#undef __FUNCT__
-#define __FUNCT__ "VecStashScatterEnd_Private"
 PetscErrorCode VecStashScatterEnd_Private(VecStash *stash)
 {
   PetscErrorCode ierr;
@@ -145,8 +139,6 @@ PetscErrorCode VecStashScatterEnd_Private(VecStash *stash)
    reallocs - the number of additional mallocs incurred.
 
 */
-#undef __FUNCT__
-#define __FUNCT__ "VecStashGetInfo_Private"
 PetscErrorCode VecStashGetInfo_Private(VecStash *stash,PetscInt *nstash,PetscInt *reallocs)
 {
   PetscFunctionBegin;
@@ -168,8 +160,6 @@ PetscErrorCode VecStashGetInfo_Private(VecStash *stash,PetscInt *nstash,PetscInt
             this value is used while allocating memory. It specifies
             the number of vals stored, even with the block-stash
 */
-#undef __FUNCT__
-#define __FUNCT__ "VecStashSetInitialSize_Private"
 PetscErrorCode VecStashSetInitialSize_Private(VecStash *stash,PetscInt max)
 {
   PetscFunctionBegin;
@@ -188,8 +178,6 @@ PetscErrorCode VecStashSetInitialSize_Private(VecStash *stash,PetscInt max)
    Notes:
    This routine doubles the currently used memory.
 */
-#undef __FUNCT__
-#define __FUNCT__ "VecStashExpand_Private"
 PetscErrorCode VecStashExpand_Private(VecStash *stash,PetscInt incr)
 {
   PetscErrorCode ierr;
@@ -230,12 +218,11 @@ PetscErrorCode VecStashExpand_Private(VecStash *stash,PetscInt incr)
   owners - an array of size 'no-of-procs' which gives the ownership range
            for each node.
 
-  Notes: The 'owners' array in the cased of the blocked-stash has the
+  Notes:
+    The 'owners' array in the cased of the blocked-stash has the
   ranges specified blocked global indices, and for the regular stash in
   the proper global indices.
 */
-#undef __FUNCT__
-#define __FUNCT__ "VecStashScatterBegin_Private"
 PetscErrorCode VecStashScatterBegin_Private(VecStash *stash,PetscInt *owners)
 {
   PetscErrorCode ierr;
@@ -346,8 +333,6 @@ PetscErrorCode VecStashScatterBegin_Private(VecStash *stash,PetscInt *owners)
            1 indicates that the current call successfully received a message, and the
              other output parameters nvals,rows,cols,vals are set appropriately.
 */
-#undef __FUNCT__
-#define __FUNCT__ "VecStashScatterGetMesg_Private"
 PetscErrorCode VecStashScatterGetMesg_Private(VecStash *stash,PetscMPIInt *nvals,PetscInt **rows,PetscScalar **vals,PetscInt *flg)
 {
   PetscErrorCode ierr;
@@ -363,7 +348,7 @@ PetscErrorCode VecStashScatterGetMesg_Private(VecStash *stash,PetscMPIInt *nvals
   if (stash->nprocessed == stash->nrecvs) PetscFunctionReturn(0);
 
   flg_v = stash->nprocs;
-  /* If a matching pair of receieves are found, process them, and return the data to
+  /* If a matching pair of receives are found, process them, and return the data to
      the calling function. Until then keep receiving messages */
   while (!match_found) {
     ierr = MPI_Waitany(2*stash->nrecvs,stash->recv_waits,&i,&recv_status);CHKERRQ(ierr);
@@ -390,3 +375,91 @@ PetscErrorCode VecStashScatterGetMesg_Private(VecStash *stash,PetscMPIInt *nvals
   }
   PetscFunctionReturn(0);
 }
+
+/*
+ * Sort the stash, removing duplicates (combining as appropriate).
+ */
+PetscErrorCode VecStashSortCompress_Private(VecStash *stash)
+{
+  PetscErrorCode ierr;
+  PetscInt i,j,bs = stash->bs;
+
+  PetscFunctionBegin;
+  if (!stash->n) PetscFunctionReturn(0);
+  if (bs == 1) {
+    ierr = PetscSortIntWithScalarArray(stash->n,stash->idx,stash->array);CHKERRQ(ierr);
+    for (i=1,j=0; i<stash->n; i++) {
+      if (stash->idx[i] == stash->idx[j]) {
+        switch (stash->insertmode) {
+        case ADD_VALUES:
+          stash->array[j] += stash->array[i];
+          break;
+        case INSERT_VALUES:
+          stash->array[j] = stash->array[i];
+          break;
+        default: SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Insert mode not supported 0x%x",stash->insertmode);
+        }
+      } else {
+        j++;
+        stash->idx[j]   = stash->idx[i];
+        stash->array[j] = stash->array[i];
+      }
+    }
+    stash->n = j + 1;
+  } else {                      /* block stash */
+    PetscInt *perm = NULL;
+    PetscScalar *arr;
+    ierr = PetscMalloc2(stash->n,&perm,stash->n*bs,&arr);CHKERRQ(ierr);
+    for (i=0; i<stash->n; i++) perm[i] = i;
+    ierr = PetscSortIntWithArray(stash->n,stash->idx,perm);CHKERRQ(ierr);
+
+    /* Out-of-place copy of arr */
+    ierr = PetscMemcpy(arr,stash->array,bs*sizeof(PetscScalar));CHKERRQ(ierr);
+    for (i=1,j=0; i<stash->n; i++) {
+      PetscInt k;
+      if (stash->idx[i] == stash->idx[j]) {
+        switch (stash->insertmode) {
+        case ADD_VALUES:
+          for (k=0; k<bs; k++) arr[j*bs+k] += stash->array[perm[i]*bs+k];
+          break;
+        case INSERT_VALUES:
+          for (k=0; k<bs; k++) arr[j*bs+k] = stash->array[perm[i]*bs+k];
+          break;
+        default: SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_SUP,"Insert mode not supported 0x%x",stash->insertmode);
+        }
+      } else {
+        j++;
+        stash->idx[j] = stash->idx[i];
+        for (k=0; k<bs; k++) arr[j*bs+k] = stash->array[perm[i]*bs+k];
+      }
+    }
+    stash->n = j + 1;
+    ierr = PetscMemcpy(stash->array,arr,stash->n*bs*sizeof(PetscScalar));CHKERRQ(ierr);
+    ierr = PetscFree2(perm,arr);CHKERRQ(ierr);
+  }
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode VecStashGetOwnerList_Private(VecStash *stash,PetscLayout map,PetscMPIInt *nowners,PetscMPIInt **owners)
+{
+  PetscErrorCode ierr;
+  PetscInt i,bs = stash->bs,r;
+  PetscSegBuffer seg;
+
+  PetscFunctionBegin;
+  if (bs != 1 && bs != map->bs) SETERRQ2(map->comm,PETSC_ERR_PLIB,"Stash block size %D does not match layout block size %D",bs,map->bs);
+  ierr = PetscSegBufferCreate(sizeof(PetscMPIInt),50,&seg);CHKERRQ(ierr);
+  *nowners = 0;
+  for (i=0,r=-1; i<stash->n; i++) {
+    if (stash->idx[i]*bs >= map->range[r+1]) {
+      PetscMPIInt *rank;
+      ierr = PetscSegBufferGet(seg,1,&rank);CHKERRQ(ierr);
+      ierr = PetscLayoutFindOwner(map,stash->idx[i]*bs,&r);CHKERRQ(ierr);
+      *rank = r;
+      (*nowners)++;
+    }
+  }
+  ierr = PetscSegBufferExtractAlloc(seg,owners);CHKERRQ(ierr);
+  ierr = PetscSegBufferDestroy(&seg);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
